(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    684258,      16469]
NotebookOptionsPosition[    666518,      15883]
NotebookOutlinePosition[    667119,      15907]
CellTagsIndexPosition[    667076,      15904]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Abstract Algebra: An Interactive Approach", "Special2",
 Evaluatable->False,
 CellChangeTimes->{{3.436525223984375*^9, 3.436525239609375*^9}},
 TextAlignment->Center,
 FontFamily->"Arial",
 FontSize->36,
 FontWeight->"Bold",
 FontSlant->"Italic",
 FontColor->RGBColor[1, 0, 1],
 Background->RGBColor[0.500008, 1, 1]],

Cell[CellGroupData[{

Cell["\[Copyright]2009   William Paulsen", "Name",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43652524240625*^9, 3.4365252448125*^9}},
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]],

Cell["\<\
All rights reserved:  No part of these materials may be reproduced, either \
electronically, mechanically, or by any other means or in any form, without \
the written permission from the author. All software is provided \"as is\" \
without warranty of any kind, either expressed or implied, including, but not \
limited to, the implied warranties of merchantability and fitness for a \
particular purpose.\
\>", "Name",
 Editable->False,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]]
}, Closed]],

Cell["\<\
Chapter 1
Understanding the Group Concept\
\>", "Title",
 CellFrame->True,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 TextAlignment->Center,
 FontFamily->"New York",
 FontSize->24,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell["Initialization", "Special2",
 Evaluatable->False,
 TextAlignment->Center,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["<<c:\\math\\group.m", "Input",
 PageWidth->Infinity,
 InitializationCell->True,
 CellChangeTimes->{{3.42754509609375*^9, 3.427545098765625*^9}, {
  3.4362707143206253`*^9, 3.4362707152425003`*^9}},
 AspectRatioFixed->True],

Cell[BoxData["\<\"Initialization Done\"\>"], "Print",
 CellChangeTimes->{3.4362707187893753`*^9, 3.4425876693521376`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Introduction to Groups", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4274919894375*^9, 3.427491997453125*^9}}],

Cell[TextData[{
 "To help introduce us to the concept of groups, let us meet a triangle \
which, through ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", will dance before your eyes! To see Terry the Triangle, execute the \
following ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command. This is done by clicking the mouse on the following boldface line \
so that a thick vertical line appears, and then pressing \[ShiftKey] and the \
Enter key \[ReturnIndicator]  at the same time. Try it!"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ShowTerry", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4362709351643753`*^9, 3.4425876734771376`*^9}]
}, Open  ]],

Cell[TextData[{
 "Terry may not look very impressive, but with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " animation, Terry can do some tricks. For example, Terry can rotate with \
the following command: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Terry[RotRt]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], 
    PolygonBox[{{0, 0}, {1.286, 1.532}, {0.684, -1.879}}]}, 
   {RGBColor[0, 1, 0], 
    PolygonBox[{{0, 0}, {1.286, 1.532}, {-1.97, 0.347}}]}, 
   {RGBColor[0, 1, 1], 
    PolygonBox[{{0, 0}, {0.684, -1.879}, {-1.97, 0.347}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876762115126`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], 
    PolygonBox[{{0, 0}, {1.97, 0.347}, {-0.684, -1.879}}]}, 
   {RGBColor[0, 1, 0], 
    PolygonBox[{{0, 0}, {1.97, 0.347}, {-1.286, 1.532}}]}, 
   {RGBColor[0, 1, 1], 
    PolygonBox[{{0, 0}, {-0.684, -1.879}, {-1.286, 1.532}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876762271376`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876762427626`*^9}]
}, Open  ]],

Cell[TextData[{
 "As you can see, ",
 StyleBox["RotRt",
  FontWeight->"Bold"],
 " is the abbreviation for rotation right, or clockwise. Terry can rotate \
counterclockwise with the command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Terry[RotLft]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], 
    PolygonBox[{{0, 0}, {-1.286, 1.532}, {1.97, 0.347}}]}, 
   {RGBColor[0, 1, 1], 
    PolygonBox[{{0, 0}, {-1.286, 1.532}, {-0.684, -1.879}}]}, 
   {RGBColor[1, 0, 0], 
    PolygonBox[{{0, 0}, {1.97, 0.347}, {-0.684, -1.879}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876802583876`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], 
    PolygonBox[{{0, 0}, {-1.97, 0.347}, {1.286, 1.532}}]}, 
   {RGBColor[0, 1, 1], 
    PolygonBox[{{0, 0}, {-1.97, 0.347}, {0.684, -1.879}}]}, 
   {RGBColor[1, 0, 0], 
    PolygonBox[{{0, 0}, {1.286, 1.532}, {0.684, -1.879}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876802896376`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876802896376`*^9}]
}, Open  ]],

Cell["\<\
Terry can do more than just rotate. For example, Terry can spin in three \
dimensions.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Terry[Spin]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {0.577, -0.5}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {-0.577, -1.5}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0.577, -0.5}, {-0.577, -1.5}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876818365126`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {-0.577, -0.5}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {0.577, -1.5}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {-0.577, -0.5}, {0.577, -1.5}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876818521376`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876818677626`*^9}]
}, Open  ]],

Cell["\<\
Terry can also do some gymnastic tricks, such as flipping over the right and \
left shoulder.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Terry[FlipRt]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0.794, 1.125}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 0], 
    PolygonBox[{{0, 0}, {0.794, 1.125}, {0.938, -0.125}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {-1.732, -1}, {0.938, -0.125}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876869302626`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {1.443, 0}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {1.443, 0}, {0.289, 1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {-1.732, -1}, {0.289, 1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876869458876`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876869615126`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Terry[FlipLft]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {-0.794, 1.125}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 1], 
    PolygonBox[{{0, 0}, {-0.794, 1.125}, {-0.938, -0.125}}]}, 
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {1.732, -1}, {-0.938, -0.125}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876899458876`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {-1.443, 0}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {-1.443, 0}, {-0.289, 1}}]}, 
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {1.732, -1}, {-0.289, 1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876899615126`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876899771376`*^9}]
}, Open  ]],

Cell["\<\
This makes 5 different moves that Terry can do. There is one more trick that \
Terry can perform:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Terry[Stay]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876916802626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
Good triangle!

These six commands are the six dance steps that Terry can do. Terry can \
combine these dance steps to form a dance routine. We separate the dance \
steps by periods. Here is a simple routine that Terry can do:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Terry[FlipRt.Spin.RotRt]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0.794, 1.125}, {-1.732, -1}}]}, 
   {RGBColor[1, 0, 0], 
    PolygonBox[{{0, 0}, {0.794, 1.125}, {0.938, -0.125}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {-1.732, -1}, {0.938, -0.125}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876937740126`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {1.443, 0}, {-1.732, -1}}]}, 
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {1.443, 0}, {0.289, 1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {-1.732, -1}, {0.289, 1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876937896376`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876938052626`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {0.577, -0.5}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {-0.577, -1.5}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0.577, -0.5}, {-0.577, -1.5}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876938208876`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {-0.577, -0.5}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {0.577, -1.5}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {-0.577, -0.5}, {0.577, -1.5}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876938365126`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876938521376`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], 
    PolygonBox[{{0, 0}, {1.286, 1.532}, {0.684, -1.879}}]}, 
   {RGBColor[1, 0, 0], 
    PolygonBox[{{0, 0}, {1.286, 1.532}, {-1.97, 0.347}}]}, 
   {RGBColor[0, 1, 1], 
    PolygonBox[{{0, 0}, {0.684, -1.879}, {-1.97, 0.347}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876938677626`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], 
    PolygonBox[{{0, 0}, {1.97, 0.347}, {-0.684, -1.879}}]}, 
   {RGBColor[1, 0, 0], 
    PolygonBox[{{0, 0}, {1.97, 0.347}, {-1.286, 1.532}}]}, 
   {RGBColor[0, 1, 1], 
    PolygonBox[{{0, 0}, {-0.684, -1.879}, {-1.286, 1.532}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876938833876`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4425876938990126`*^9}]
}, Open  ]],

Cell[TextData[{
 "This is as good a time as any to explain how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can make an animated film using this sequence of images. Notice of the \
right hand side of the screen there are a several brackets (]). If one uses \
the mouse to click on one of these, it will turn black. After reading this \
paragraph, position one of the images so that it is entirely on the screen. \
Then click on the bracket that is second from the left (the large bracket \
that encompasses all of the images of the sequence), and press \[ControlKey] \
and Y together to begin the animation. A control panel appears that allows \
one to stop, back up, speed up or slow down the animation. At first the \
motion goes both backwards and forwards, so click on the the forward button \
\[RightSkeleton] to get the motion going forward. Neat, huh? Click anywhere \
on the animation to stop.\n\nWhile the large bracket is blackened, if one \
presses \[ControlKey] and X together, the entire sequence is deleted. This is \
useful for computers with limited memory. Deleting these animations after \
they are viewed elimates the pesky \"Low on memory\" messages. However, but \
careful not to blacken the far right bracket while pressing \[ControlKey] and \
X, for that would erase this whole section!\n\nEXPERIMENT:\nYou may have \
noticed in the previous animation that Terry ends the dance routine in the \
original orientation. What if the ",
 StyleBox["order",
  FontSlant->"Italic"],
 " of the dance steps were changed? Edit the last command to change the order \
of Terry's steps, and see what happens. \n\nHere is a shorter dance sequence \
for Terry. Pay special attention to how Terry is oriented after the dance \
routine. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Terry[FlipRt.Spin]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4362711220862503`*^9, 3.4362711270550003`*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0.794, 1.125}, {-1.732, -1}}]}, 
   {RGBColor[1, 0, 0], 
    PolygonBox[{{0, 0}, {0.794, 1.125}, {0.938, -0.125}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {-1.732, -1}, {0.938, -0.125}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4362711286175003`*^9, 3.4425877427115126`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {1.443, 0}, {-1.732, -1}}]}, 
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {1.443, 0}, {0.289, 1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {-1.732, -1}, {0.289, 1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4362711286175003`*^9, 3.4425877427271376`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4362711286175003`*^9, 3.4425877427427626`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {0.577, -0.5}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {-0.577, -1.5}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0.577, -0.5}, {-0.577, -1.5}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4362711286175003`*^9, 3.4425877427583876`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {-0.577, -0.5}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {0.577, -1.5}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {-0.577, -0.5}, {0.577, -1.5}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4362711286175003`*^9, 3.4425877427740126`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4362711286175003`*^9, 3.4425877427896376`*^9}]
}, Open  ]],

Cell[TextData[{
 "Sometimes Terry gets tired of doing all of these long routines. When this \
happens, Terry takes a short cut to get to the final orientation. For \
example, the above routine leaves Terry in the same orientation as the ",
 StyleBox["RotLft",
  FontWeight->"Bold"],
 ", and so Terry would simply rotate instead of doing the extra gymnastics. \
Executing the follow command allows Terry to take short cuts in all dance \
routines."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4362712083206253`*^9, 3.4362712300550003`*^9}}],

Cell[CellGroupData[{

Cell["InitTerry", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "Stay", ",", "FlipRt", ",", "RotRt", ",", "FlipLft", ",", "RotLft", ",", 
   "Spin"}], "}"}]], "Output",
 CellChangeTimes->{3.4362712383675003`*^9, 3.4425877471333876`*^9}]
}, Open  ]],

Cell["Now let's ask Terry to do the same dance routine.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Terry[FlipRt.Spin]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.4362712992112503`*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], 
    PolygonBox[{{0, 0}, {-1.286, 1.532}, {1.97, 0.347}}]}, 
   {RGBColor[1, 0, 0], 
    PolygonBox[{{0, 0}, {-1.286, 1.532}, {-0.684, -1.879}}]}, 
   {RGBColor[0, 1, 1], 
    PolygonBox[{{0, 0}, {1.97, 0.347}, {-0.684, -1.879}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4362713074612503`*^9, 3.4425877490708876`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 0], 
    PolygonBox[{{0, 0}, {-1.97, 0.347}, {1.286, 1.532}}]}, 
   {RGBColor[1, 0, 0], 
    PolygonBox[{{0, 0}, {-1.97, 0.347}, {0.684, -1.879}}]}, 
   {RGBColor[0, 1, 1], 
    PolygonBox[{{0, 0}, {1.286, 1.532}, {0.684, -1.879}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4362713074612503`*^9, 3.4425877491021376`*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], PolygonBox[{{0, 0}, {0, 2}, {1.732, -1}}]}, 
   {RGBColor[0, 1, 0], PolygonBox[{{0, 0}, {0, 2}, {-1.732, -1}}]}, 
   {RGBColor[1, 0, 0], PolygonBox[{{0, 0}, {1.732, -1}, {-1.732, -1}}]}},
  AspectRatio->Automatic,
  PlotRange->{{-2, 2}, {-2, 2}}]], "Print",
 CellChangeTimes->{3.4362713074612503`*^9, 3.4425877491177626`*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that Terry simply rotates to get to the correct orientation. Rather \
than calling Terry a bad triangle for being lazy, we must commend Terry for \
being smart enough to know that a flip to the right followed by a spin is \
equivalent to a counter-clockwise rotation. In essence Terry has learned how \
to \"multiply\" two dance steps together.\n\nWe can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to multiply dance steps together, using the period to indicate Terry's \
multiplication. Thus, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " simplifies ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    RowBox[{"RotRt", " ", "\[CenterDot]", " ", "FlipRt"}],
    FontWeight->"Bold"], TraditionalForm]]],
 " as follows: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4362713041956253`*^9, 3.4362713054300003`*^9}, {
  3.4362713375393753`*^9, 3.4362713737893753`*^9}}],

Cell[CellGroupData[{

Cell["FlipRt . Spin", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4362714192581253`*^9, 3.4362714254612503`*^9}},
 AspectRatioFixed->True],

Cell[BoxData["RotLft"], "Output",
 CellChangeTimes->{{3.4362714035706253`*^9, 3.4362714262737503`*^9}, 
   3.4425877523833876`*^9}]
}, Open  ]],

Cell["\<\
We can display a multiplication table of all of Terry's dance steps by the \
following command:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[{Stay,FlipRt,RotRt,FlipLft,RotLft,Spin}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}, {0, 1, 
      0}, {1, 0, 0}, {1, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {1, 0, 0}, {1, 1, 
      0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}}, {{1, 0, 0}, {1, 0, 0}, {0, 
      1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {
      0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 1}, {1, 1, 0}, {1, 0, 
      0}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 
      0.8}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 
      1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"Stay\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"Spin\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"Stay\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"Spin\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.4362714299612503`*^9, 3.4425877553990126`*^9}]
}, Open  ]],

Cell["\<\
This table can easily by enlarged to make it more readable. Simply click on \
the table, and a black frame appears. By clicking and dragging the small \
black squares on the borders of this frame, one can resize the table.

This table illuminates some properties of the dance steps. Each of the six \
dance steps is in a different color, to highlight these properties. One \
property this table demonstrates that the combination of any two dance steps \
is equivalent to one of the six dance steps.

Another of the easiest properties to observe is in the \"Stay\" row and \
column. Notice that whenever the command \"Stay\" is combined with any other \
dance step, it leaves that dance step unchanged. Try this for yourself:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4425877701177626`*^9, 3.4425877713990126`*^9}}],

Cell[CellGroupData[{

Cell["Stay . RotRt", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["RotRt"], "Output",
 CellChangeTimes->{3.4425877742583876`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["RotRt . Stay", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["RotRt"], "Output",
 CellChangeTimes->{3.4425877762271376`*^9}]
}, Open  ]],

Cell[TextData[{
 "\nEXPERIMENT:\nReplace ",
 StyleBox["RotRt",
  FontWeight->"Bold"],
 " in the above two ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " commands with a different dance step and re-execute them. Notice what \
happens.\n\nYou may observe that the Stay command acts like the number 1 in \
our standard multiplication. In mathematical notation, we say that \n\n       \
             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox["Stay",
      FontWeight->"Bold"], " ", "\[CenterDot]", " ", "x"}], " ", "=", " ", 
    "x"}], TraditionalForm]]],
 "\nand \n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"x", " ", "\[CenterDot]", " ", 
      StyleBox["Stay",
       FontWeight->"Bold"]}], " ", "=", " ", "x"}]}], TraditionalForm]]],
 "\n\nfor all dance steps ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". We call Stay the ",
 StyleBox["identity",
  FontSlant->"Italic"],
 " of the dance steps.\n\nBut there is a property of these dance steps that \
is unlike our standard multiplication. In a previous experiment, you were \
asked to change the order of the dance steps in a routine to see if that \
would affect the final orientation. In fact, it did! Check out the following \
two products:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427485831375*^9, 3.42748583253125*^9}, 
   3.428443750890625*^9}],

Cell[CellGroupData[{

Cell["Spin . FlipRt", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4362715477112503`*^9, 3.4362715490550003`*^9}},
 AspectRatioFixed->True],

Cell[BoxData["RotRt"], "Output",
 CellChangeTimes->{3.4362715496018753`*^9, 3.4425877844927626`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["FlipRt . Spin", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4362715538987503`*^9, 3.4362715542737503`*^9}},
 AspectRatioFixed->True],

Cell[BoxData["RotLft"], "Output",
 CellChangeTimes->{3.4362715550862503`*^9, 3.4425877865396376`*^9}]
}, Open  ]],

Cell[TextData[{
 "As we see, the product depends on the order in which the dance steps are \
multiplied. In other words,\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}]}], TraditionalForm]]],
 "   is sometimes not equal to  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"y", " ", "\[CenterDot]", " ", "x"}]}], TraditionalForm]]],
 ".\n\nThis is perhaps the biggest difference between Terry's algebra and the \
algebra that we are familiar with. We are used to what is called the \
\"commutative property\" of numbers:\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "y"}], " ", "=", " ", 
    RowBox[{"y", " ", "x"}]}], TraditionalForm]]],
 "   for all numbers ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ".\n\nSince Terry's dance steps do not obey this law, we say that the dance \
steps are ",
 StyleBox["non-commutative",
  FontSlant->"Italic"],
 ".\n\nThere is another property that we are familiar with in standard \
algebra:\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", 
     RowBox[{"(", 
      RowBox[{"y", " ", "z"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "y"}], ")"}], " ", "z"}]}], TraditionalForm]]],
 "   for all numbers ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ".\n\nThis is called the ",
 StyleBox["associative law",
  FontSlant->"Italic"],
 " of multiplication. Do Terry's dance steps obey this law?\n\nEXPERIMENT:\n\
Compare the two products:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["(Spin . RotRt) . FlipLft", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["RotRt"], "Output",
 CellChangeTimes->{3.4425877910240126`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Spin . (RotRt . FlipLft)", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["RotRt"], "Output",
 CellChangeTimes->{3.4425877927740126`*^9}]
}, Open  ]],

Cell["\<\
Try changing the dance steps to see if we always get the same product, \
regardless of where the parenthesis are.

There is another property that we can observe from Terry's multiplication \
table. Here it is again.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[{Stay,FlipRt,RotRt,FlipLft,RotLft,Spin}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}, {0, 1, 
      0}, {1, 0, 0}, {1, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {1, 0, 0}, {1, 1, 
      0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}}, {{1, 0, 0}, {1, 0, 0}, {0, 
      1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {
      0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 1}, {1, 1, 0}, {1, 0, 
      0}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 
      0.8}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 
      1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"Stay\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"Spin\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"Stay\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"Spin\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.4425877973208876`*^9}]
}, Open  ]],

Cell["\<\
Notice that the identity dance step, \"Stay\", is in every row and every \
column in the table. (Notice the yellow squares.) This indicates that every \
dance step has some dance step which \"undoes\" it. For example, RotLft will \
undo a RotRt command, since \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotLft . RotRt", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["Stay"], "Output",
 CellChangeTimes->{3.4425878000865126`*^9}]
}, Open  ]],

Cell["\<\
gives us back to the original orientation. Likewise, the Spin command undoes \
itself, for two spins in secession \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Spin . Spin", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["Stay"], "Output",
 CellChangeTimes->{3.4425878022740126`*^9}]
}, Open  ]],

Cell[TextData[{
 "returns to the original position.\n\nUsing mathematical notation again, we \
say that for every dance step ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", there is some dance step ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " such that\n \n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", "Stay"}], 
   TraditionalForm]]],
 ".\n\nThe dance step ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is called the ",
 StyleBox["inverse",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". For example, RotLft is the inverse of RotRt, and Spin is the inverse of \
Spin. We can denote the inverse of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 ". This is entered into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " as "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4274859105625*^9, 3.42748591159375*^9}}],

Cell[CellGroupData[{

Cell["RotRt^(-1)", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["RotLft"], "Output",
 CellChangeTimes->{3.4425878049458876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Terry claims that if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", "Stay"}], 
   TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "\[CenterDot]", " ", "x"}], TraditionalForm]]],
 " will also equal Stay. Although this can be seen to be true by looking at \
the multiplication table, can you prove that this is so using only the \
properties of Terry's dance steps that we have discovered? Here is a summary \
of what we have found so far:\n\n    1)   The ",
 StyleBox["order",
  FontSlant->"Italic"],
 " in which the dance steps are performed are important.  \n           For \
example, ",
 StyleBox["Spin",
  FontWeight->"Bold"],
 " \[CenterDot] ",
 StyleBox["FlipRt",
  FontWeight->"Bold"],
 " \[NotEqual] ",
 StyleBox["FlipRt",
  FontWeight->"Bold"],
 " \[CenterDot] ",
 StyleBox["Spin",
  FontWeight->"Bold"],
 ".\n         \n    2)   The combination of any two dance steps is   \n       \
   equivalent to one of the six dance steps.\n\n    3)   The order in which a \
dance routine is simplified does not matter.\n          That is,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], ")"}], " ", 
     "\[CenterDot]", " ", "z"}], "  ", "=", "  ", 
    RowBox[{"x", " ", "\[CenterDot]", " ", 
     RowBox[{"(", 
      RowBox[{"y", " ", "\[CenterDot]", " ", "z"}], ")"}]}]}], 
   TraditionalForm]]],
 "  for any dance steps ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ".\n\n    4)   Any dance step combined with Stay yields the same dance step.\
\n          That is,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", 
     StyleBox["Stay",
      FontWeight->"Bold"]}], " ", "=", " ", "x"}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox["Stay",
      FontWeight->"Bold"], " ", "\[CenterDot]", " ", "x"}], " ", "=", " ", 
    "x"}], TraditionalForm]]],
 "  for any dance step ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".\n\n    5)   Every dance step has another dance step that \"undoes\" it.  \
That is, \n          for any dance step ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", there is a dance step ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", 
    StyleBox["Stay",
     FontWeight->"Bold"]}], TraditionalForm]]],
 ".\n          For example, the step that undoes ",
 StyleBox["RotRt",
  FontWeight->"Bold"],
 " is ",
 StyleBox["RotLft",
  FontWeight->"Bold"],
 ".\n\nWe will use the following mathematical terminology to expree each of \
these properties.\n\n    1)  The dance steps are not ",
 StyleBox["commutative",
  FontSlant->"Italic"],
 ". \n    \n    2)  The dance steps are ",
 StyleBox["closed",
  FontSlant->"Italic"],
 " under multiplication.\n    \n    3)  The dance steps are ",
 StyleBox["associative",
  FontSlant->"Italic"],
 ".\n    \n    4)  There is an ",
 StyleBox["identity",
  FontSlant->"Italic"],
 " dance step.\n    \n    5)  Every dance step has an ",
 StyleBox["inverse",
  FontSlant->"Italic"],
 ".\n    \nWith just these properties, can use prove the following \
proposition on your own without looking at the proof?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42748602096875*^9, 3.42748626015625*^9}, {
  3.427486296484375*^9, 3.427486389703125*^9}, {3.42748652509375*^9, 
  3.427486746859375*^9}, {3.428443822296875*^9, 3.428443842828125*^9}}],

Cell[TextData[{
 "PROPOSITION 1.1\nIf ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is the inverse of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is the only inverse of ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ".\n\nHint: Because we don't have the commutative property, we must use some \
other strategy. Recall that ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " must also have an inverse!  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4274867725625*^9, 3.427486809328125*^9}, {
   3.427487227796875*^9, 3.4274872279375*^9}, 3.4425878625552626`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " be any inverse of ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ". Consider the product  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y", "\[CenterDot]", "z"}], 
   TraditionalForm]]],
 ".  According to the associative property, \n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"y", "\[CenterDot]", "z"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "\[CenterDot]", 
     RowBox[{"z", "."}]}]}], TraditionalForm]]],
 "\n\nOn the left side, we see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "z"}], TraditionalForm]]],
 " is an identity element, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"y", "\[CenterDot]", "z"}], ")"}]}], "=", "x"}], 
   TraditionalForm]]],
 ".  But on the right side, we find that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is an identity element, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "\[CenterDot]", "z"}], "=", 
    RowBox[{"z", "."}]}], TraditionalForm]]],
 "  Therefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", "z"}], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is the only inverse of ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ".  "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427486842875*^9, 3.427487061921875*^9}, {
   3.4414589532764997`*^9, 3.4414589678077497`*^9}, {3.4414590018702497`*^9, 
   3.4414591594952497`*^9}, 3.4425878606021376`*^9}]
}, Open  ]],

Cell["\<\
All of the proofs in these notebooks are in a subsection that can opened and \
closed by double clicking the the second bracket from the left. This allows \
one to hide the proof until after it has been attempted.

Notice that we did not yet assume that the inverse are unique, or even that \
there is only one identity element.  However, these facts immediately follow \
from proposition 1.1.  (See problems 1.8 and 1.9.) \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427487379390625*^9, 3.4274874555*^9}, {
   3.427487526734375*^9, 3.427487527625*^9}, {3.427488243375*^9, 
   3.42748830078125*^9}, 3.42749191890625*^9, {3.4362720302112503`*^9, 
   3.4362720356487503`*^9}, 3.4425878585396376`*^9}],

Cell[TextData[{
 "DEFINITION 1.1   We use the notation ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " for the unique inverse of the element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427487379390625*^9, 3.4274874555*^9}, {
  3.427487526734375*^9, 3.427487527625*^9}, {3.427488243375*^9, 
  3.42748830078125*^9}, {3.42749191890625*^9, 3.42749192471875*^9}}],

Cell[TextData[{
 "Proposition 1.1 can now be expressed simply as  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]], ")"}], 
     RowBox[{
      RowBox[{"-", "1"}], " "}]], "=", " ", "x"}], TraditionalForm]]],
 ". Is it true that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], 
     RowBox[{"-", "1"}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ", as we are accustomed too?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427487379390625*^9, 3.4274874555*^9}, {
  3.427487526734375*^9, 3.427487527625*^9}, {3.427488243375*^9, 
  3.42748830078125*^9}, {3.42749191890625*^9, 3.427491932265625*^9}}],

Cell["\<\
EXPERIMENT:
Try the following commands to see if they give the same result. \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427487379390625*^9, 3.4274874555*^9}, {
   3.427487526734375*^9, 3.427487527625*^9}, {3.427488243375*^9, 
   3.42748830078125*^9}, {3.42749191890625*^9, 3.427491933890625*^9}, 
   3.4425878561177626`*^9}],

Cell[CellGroupData[{

Cell["(FlipRt . Spin)^(-1) ", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.427488335671875*^9, 3.42748834178125*^9}},
 AspectRatioFixed->True],

Cell[BoxData["RotRt"], "Output",
 CellChangeTimes->{3.4425878122115126`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["(FlipRt^(-1)) . (Spin^(-1))", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.42748834484375*^9, 3.427488349875*^9}},
 AspectRatioFixed->True],

Cell[BoxData["RotLft"], "Output",
 CellChangeTimes->{3.4425878141333876`*^9}]
}, Open  ]],

Cell["\<\
Try using different dance steps in place of Spin and RotRt. What do you \
observe?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.42749194559375*^9}],

Cell[TextData[{
 "Apparently, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "\[CenterDot]", "y"}], ")"}], 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " is not always equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 ". But Terry would tell you that in order to undo a sequence of dance steps, \
one must begin by undoing the last dance step. Thus, the way to undo the \
routine (Spin \[CenterDot] RotRt) is by first rotating left, and spinning \
back to the original position. This suggests that \n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], 
     RowBox[{"-", "1"}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ".     \n\nCan you prove this? "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42749194559375*^9, 3.427491947125*^9}, 
   3.4425878539615126`*^9}],

Cell[TextData[{
 "PROPOSITION 1.2\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], 
     RowBox[{"-", "1"}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ".  \n                   "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427488832078125*^9, 3.42748884659375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since the inverse ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "\[CenterDot]", "y"}], ")"}], 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 "is the dance step ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " such that  \n           \n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "\[CenterDot]", "z"}], " ", 
     "=", " ", "Stay"}]}], TraditionalForm]]],
 ", \n\nit suffices to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " has this property. We see that\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["y", 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]]}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"x", "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"y", "\[CenterDot]", 
        SuperscriptBox["y", 
         RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "Stay", "\[CenterDot]", 
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]]}], "=", " ", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", 
        SuperscriptBox["x", 
         RowBox[{"-", "1"}]]}], " ", "=", " ", "Stay"}]}]}]}], 
   TraditionalForm]]],
 ".\n\nSo   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], 
     RowBox[{"-", "1"}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427488850078125*^9, 3.427488921890625*^9}, {
  3.4362721624300003`*^9, 3.4362721634768753`*^9}}]
}, Open  ]],

Cell["\<\

Let us see if we can find any more patterns in Terry's multiplication table. \
Here is the table again:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[{Stay,FlipRt,RotRt,FlipLft,RotLft,Spin}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}, {0, 1, 
      0}, {1, 0, 0}, {1, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {1, 0, 0}, {1, 1, 
      0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}}, {{1, 0, 0}, {1, 0, 0}, {0, 
      1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {
      0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 1}, {1, 1, 0}, {1, 0, 
      0}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 
      0.8}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 
      1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"Stay\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"Spin\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"Stay\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"Spin\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.4425878244771376`*^9}]
}, Open  ]],

Cell[TextData[{
 "Before we noticed that every row and every column contained the identity \
element, Stay. Which other elements have this property?\n\nIn answering this, \
you probably discovered that every row and every column in this table \
contains all six elements. If we disregard the heading row and column in the \
table, each element appears exactly once in each row, and once in each \
column. This formation is refered to as a \"Latan Square.\" How could we show \
that this would happen using only the four properties we have been using?\n\n\
First we must translate this pattern into a mathematical statement before it \
can be proven. To say that the element ",
 StyleBox["RotRt",
  FontWeight->"Bold"],
 " appears only once in the row beginning with ",
 StyleBox["Spin",
  FontWeight->"Bold"],
 " is to say that there is only one dance step ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " such that \n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox["Spin",
      FontWeight->"Bold"], " ", "\[CenterDot]", " ", "x"}], " ", "=", " ", 
    StyleBox["RotRt",
     FontWeight->"Bold"]}], TraditionalForm]]],
 ". \n\nSo we can say that each row contains exactly one of every element by \
the following statement:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425878501021376`*^9}],

Cell[TextData[{
 "PROPOSITION 1.3\n \nIf ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are given, then there exists a unique ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " such that \n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "x"}], " ", "=", " ", "b"}], 
   TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42748893825*^9, 3.427488938421875*^9}, 
   3.4425878481333876`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that there is an ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " such that  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", " ", "\[CenterDot]", " ", "x"}], " ", "=", " ", "b"}], 
   TraditionalForm]]],
 ".  We can multiply both sides of the equation on the ",
 StyleBox["left",
  FontSlant->"Italic"],
 " by ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["a", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " to give us\n\n                                           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["a", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     RowBox[{"(", " ", 
      RowBox[{"a", " ", "\[CenterDot]", " ", "x"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["a", 
      RowBox[{"-", "1"}]], "\[CenterDot]", " ", "b"}]}], TraditionalForm]]],
 ".         \nThen                 \n                                         \
 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["a", 
           RowBox[{"-", "1"}]], "\[CenterDot]", " ", "a"}], ")"}], " ", 
        "\[CenterDot]", " ", "x"}], "=", " ", 
       RowBox[{
        SuperscriptBox["a", 
         RowBox[{"-", "1"}]], "\[CenterDot]", " ", "b"}]}],
      TraditionalForm], ","}], "  "}], TraditionalForm]]],
 "\n                                               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Stay", " ", "\[CenterDot]", " ", "x"}], "=", " ", 
    RowBox[{
     SuperscriptBox["a", 
      RowBox[{"-", "1"}]], "\[CenterDot]", " ", "b"}]}], TraditionalForm]]],
 ".     \nSo\n                                                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["a", 
      RowBox[{"-", "1"}]], "\[CenterDot]", " ", "b"}]}], TraditionalForm]]],
 ".\nThus, if there is a solution, this must be the unique solution ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["a", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "b"}]}], TraditionalForm]]],
 ". Let us check that this is indeed a solution. \n\n                         \
         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "x"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"a", "\[CenterDot]", 
      SuperscriptBox["a", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "b"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"Stay", "\[CenterDot]", "b"}], " ", "=", " ", "b"}]}]}], 
   TraditionalForm]]],
 ".\n                                  \nThus, there is only one solution to \
the equation, namely  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", 
     RowBox[{"-", "1"}]], "\[CenterDot]", " ", "b"}], TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425878447896376`*^9}]
}, Open  ]],

Cell[TextData[{
 "Can you use this proposition to show that any ",
 StyleBox["column",
  FontSlant->"Italic"],
 " of the table contains only one of each element?   This is essentually \
problem 1.10.\n\nTerry's dance steps provides us with a new and rather \
different type of multiplication. In the next section we will consider \
another operation that has many of the same patterns. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427489002421875*^9, 3.427489015546875*^9}, {
   3.42748906171875*^9, 3.427489077015625*^9}, {3.4362722500393753`*^9, 
   3.4362722504768753`*^9}, 3.4425878428677626`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Modular Arithmetic", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Although we have seen an example of a totally different type of \
\"multiplication\", in this section we will explore different ways of \
combining two integers that has many of the same properties. These other \
types of \"products\" have some important applications. These multiplications \
stem from a property of adding and multiplying standard integers.\n\nThe \
property of integers we which to explore is as follows: for both sums and \
products, the final digit of the answer only depends on the final digits of \
the numbers being added or multiplied. For example, a positive number ending \
in a 4 added to a positive number ending in a 7 will always yield a number \
ending in a 1. On the other hand, if we multiplied these two numbers, we \
would get a number which ends in 8. This last digit is called the modulus of \
the number in base 10. Thus, we do ",
 StyleBox["modulo arithmetic",
  FontSlant->"Italic"],
 " by just considering the last digit of the number. We can write modulo \
arithmetic this way:\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", " ", "+", " ", "7"}], " ", "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "10"}], ")"}]}]}], TraditionalForm]]],
 ",   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", " ", "\[Times]", " ", "7"}], " ", "\[Congruent]", " ", 
    RowBox[{"8", "  ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"Mod", " ", "10"}], ")"}], "."}]}]}], TraditionalForm]]],
 " "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425878384771376`*^9}],

Cell[TextData[{
 "The reason for writing the (Mod 10) after each statement is because we are \
doing this \"last digit arithmetic\" in base 10. Of course, other bases could \
be used. The easiest way to read these two statements is \"4 plus 7 is 1 plus \
a multiple of 10\" and \"4 times 7 is 8 plus a multiple of 10.\" We can also \
consider the modulus of a number in base ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " to be the remainder when that number is divided by ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Two numbers are considered ",
 StyleBox["equivalent modulo",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " if the modulus of the numbers are the same. The official definition of \
modulo arithmetic is as follows:\n\nDEFINITION 1.2:\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "\[Congruent]", " ", 
    RowBox[{"y", "   ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 " "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428443921875*^9, 3.42844392378125*^9}, 
   3.4425878401646376`*^9}],

Cell[TextData[{
 "if, and only if, there is an integer ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " such that \n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "-", "y"}], ")"}], " ", "=", " ", 
    RowBox[{"k", " ", "n"}]}], TraditionalForm]]],
 ".\n\nSince we have two different ways of combining two numbers from the set \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
    "0", ",", " ", "1", ",", " ", "2", ",", " ", "3", ",", " ", "4", ",", " ",
      "5", ",", " ", "6", ",", " ", "7", ",", " ", "8", ",", " ", "9"}], 
    "}"}], TraditionalForm]]],
 ", we will work with addition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"Mod", " ", "10"}], ")"}], TraditionalForm]]],
 " first and consider multiplication (Mod 10) later. The command "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428443951609375*^9, 3.428443952265625*^9}, 
   3.4425878817115126`*^9}],

Cell["DefSumMod[10]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4284439605625*^9, 3.42844396515625*^9}},
 AspectRatioFixed->True],

Cell[TextData[{
 "loads this new type of arithmetic into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".  We have already used the period ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", ".", ")"}], TraditionalForm]]],
 " to combine two of Terry's dance steps. We will continue to use the period \
as the way to combine two numbers modulo 10. Although it seems strange to use \
the period instead of the plus sign, for consistency we will always use the \
period for the operator, whatever operation that operator performs. Thus, to \
add 4 and 7 modulo 10, we enter "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428443968109375*^9, 3.428443988953125*^9}, 
   3.4425878851646376`*^9}],

Cell[CellGroupData[{

Cell["4 . 7", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.4425878872115126`*^9}]
}, Open  ]],

Cell[TextData[{
 "It is important to put spaces between the number and the period, to keep ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " from interpreting the period as a decimal point.\n\nWe can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " give us a \"multiplication\" table as we did for Terry's dance steps. It \
may seem a bit of a misnomer to call this a multiplication table when we are \
adding the numbers. Yet this command creates a table based upon whenever the \
dot is currently defined to be. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[{0,1,2,3,4,5,6,7,8,9}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6,
       0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 
      0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 
      1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}, {{1, 1, 
      1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.4362724156175003`*^9, 3.4425878903208876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Can we find any patterns in this table, as we did in Terry's multiplication \
table? Yes, and even more. We can see that this table is a Latin square\
\[LongDash]every row and every column contains one of each number. Notice the \
diagonal streaks of color in the table. This multiplication table has \
symmetry\[LongDash]imagine a mirror placed along the northwest to southeast \
corners. What is causing this symmetry?\n\nEXPERIMENT:\nWe saw that Terry's \
multiplication was not commutative. That is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " did not always equal ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "x"}], TraditionalForm]]],
 ". What about adding modulo 10? Try the following commands "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425878927740126`*^9}],

Cell[CellGroupData[{

Cell["4 . 7", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.4425878941177626`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["7 . 4", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.4425878958052626`*^9}]
}, Open  ]],

Cell[TextData[{
 "Are these the same? Try this with different numbers.\n\nThe symmetry in the \
table comes from the fact that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is always equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "x"}], TraditionalForm]]],
 ". We say that addition modulo 10 is ",
 StyleBox["communitive",
  FontSlant->"Italic"],
 ". Thus, we can see 5 properties from the table:\n\n    1)   For any two \
numbers ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is in that set. \n          (Recall that we are using the dot to \
indication the operation, regardless of what\n          that operation is.  \
In this example, the operation is addition modulo 10.)\n\n    2)   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], ")"}], " ", 
     "\[CenterDot]", " ", "z"}], "  ", "=", "  ", 
    RowBox[{"x", " ", "\[CenterDot]", " ", 
     RowBox[{"(", 
      RowBox[{"y", " ", "\[CenterDot]", " ", "z"}], ")"}]}]}], 
   TraditionalForm]]],
 "  for any  ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ",  and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ".\n\n    3)   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "0"}], " ", "=", " ", "x"}], 
   TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"0", " ", "\[CenterDot]", " ", "x"}], " ", "=", " ", "x"}], 
   TraditionalForm]]],
 "  for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".\n\n    4)   For any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", there is a ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " such that  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 ".\n\n    5)   For any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", 
    RowBox[{"y", " ", "\[CenterDot]", " ", "x"}]}], TraditionalForm]]],
 ".\n\nNotice that 0 now acts like Terry's command Stay. This is the ",
 StyleBox["identity",
  FontSlant->"Italic"],
 ".\n\nBesides the table, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has another way of picturing a \"product\" by means of circular graphs. \
Enter in the following ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4274893618125*^9, 3.4274894518125*^9}}],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6,7,8,9}, Add[1] ]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924731, 0.8090169943749475}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, 0.30901699437494745`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924732, -0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, -0.8090169943749472}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951536, 0.30901699437494723`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, 0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"5\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217205, 0.8899186938124423}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, 0.3399186938124422}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, -0.33991869381244233`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217207, -0.8899186938124422}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, -0.889918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0461621679246689`, -0.33991869381244233`}, {1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.046162167924669, 0.339918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, 0.8899186938124422}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{0., 1.}, {0.4926796006629578, 0.8399186938124422}}], 
     PolygonBox[{{0.5877852522924731, 0.8090169943749475}, {
      0.5081304503817051, 0.8874715196271998}, {0.4772287509442104, 
      0.7923658679976845}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{0.5877852522924731, 0.8090169943749475}, {0.8922779910659062, 
      0.3899186938124422}}], 
     PolygonBox[{{0.9510565162951535, 0.30901699437494745`}, {
      0.9327288407846536, 0.41930795642706586`}, {0.8518271413471589, 
      0.36052943119781855`}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{0.9510565162951535, 0.30901699437494745`}, {
      0.9510565162951535, -0.20901699437494756`}}], 
     PolygonBox[{{0.9510565162951535, -0.30901699437494756`}, {
      1.0010565162951535`, -0.20901699437494756`}, {
      0.9010565162951535, -0.20901699437494756`}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{0.9510565162951535, -0.30901699437494756`}, {
      0.6465637775217206, -0.7281152949374526}}], 
     PolygonBox[{{0.5877852522924732, -0.8090169943749473}, {
      0.6870146272404679, -0.7575045575520762}, {
      0.6061129278029732, -0.6987260323228289}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{0.5877852522924732, -0.8090169943749473}, {
      0.09510565162951547, -0.9690983005625052}}], 
     PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
      0.11055650134826286`, -1.0166511263772628`}, {
      0.0796548019107681, -0.9215454747477476}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{
      1.2246467991473532`*^-16, -1.}, {-0.492679600662958, \
-0.839918693812442}}], 
     PolygonBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.5081304503817053, -0.8874715196271996}, {-0.47722875094421063`, \
-0.7923658679976843}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.8922779910659062, -0.3899186938124423}}], 
     PolygonBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.9327288407846536, -0.419307956427066}, {-0.8518271413471589, \
-0.36052943119781866`}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.9510565162951536, 0.20901699437494725`}}], 
     PolygonBox[{{-0.9510565162951536, 
      0.30901699437494723`}, {-1.0010565162951537`, 
      0.20901699437494722`}, {-0.9010565162951536, 0.20901699437494725`}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.9510565162951536, 
      0.30901699437494723`}, {-0.6465637775217207, 0.7281152949374525}}], 
     PolygonBox[{{-0.5877852522924734, 
      0.8090169943749473}, {-0.687014627240468, 
      0.7575045575520762}, {-0.6061129278029732, 0.6987260323228289}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.5877852522924734, 
      0.8090169943749473}, {-0.09510565162951536, 0.9690983005625052}}], 
     PolygonBox[{{0., 1.}, {-0.11055650134826274`, 
      1.0166511263772628`}, {-0.07965480191076799, 0.9215454747477476}}]}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425879002427626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "As you can see, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " labels 10 equi-distant points on the circle with the digits 0 through 9, \
like a clock. It then draws an arrow from each point to the point produces by \
\"adding 1 modulo 10\".\n\nEXPERIMENT:\nCreate eight more circle graphs by \
changing the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Add", "[", "1", "]"}], TraditionalForm]]],
 " in the last command to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Add", "[", "2", "]"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Add", "[", "3", "]"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Add", "[", "4", "]"}], ","}], TraditionalForm]]],
 " ... ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Add", "[", "9", "]"}], TraditionalForm]]],
 ". Which circular graph is the prettiest?\n\nIf none of those graphs were \
pretty enough, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can graph several of them at the same time. For example, the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
CircleGraph[{0,1,2,3,4,5,6,7,8,9}, 
   Add[1], Add[2], Add[3], Add[4], Add[5] ]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924731, 0.8090169943749475}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, 0.30901699437494745`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924732, -0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, -0.8090169943749472}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951536, 0.30901699437494723`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, 0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"5\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217205, 0.8899186938124423}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, 0.3399186938124422}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, -0.33991869381244233`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217207, -0.8899186938124422}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, -0.889918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0461621679246689`, -0.33991869381244233`}, {1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.046162167924669, 0.339918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, 0.8899186938124422}, {1, 0}]}, {{
     {RGBColor[1, 0, 0], 
      LineBox[{{0., 1.}, {0.4926796006629578, 0.8399186938124422}}], 
      PolygonBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.5081304503817051, 0.8874715196271998}, {0.4772287509442104, 
       0.7923658679976845}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{0.5877852522924731, 0.8090169943749475}, {0.8922779910659062, 
       0.3899186938124422}}], 
      PolygonBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.9327288407846536, 0.41930795642706586`}, {0.8518271413471589, 
       0.36052943119781855`}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.9510565162951535, -0.20901699437494756`}}], 
      PolygonBox[{{0.9510565162951535, -0.30901699437494756`}, {
       1.0010565162951535`, -0.20901699437494756`}, {
       0.9010565162951535, -0.20901699437494756`}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{0.9510565162951535, -0.30901699437494756`}, {
       0.6465637775217206, -0.7281152949374526}}], 
      PolygonBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.6870146272404679, -0.7575045575520762}, {
       0.6061129278029732, -0.6987260323228289}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.09510565162951547, -0.9690983005625052}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.11055650134826286`, -1.0166511263772628`}, {
       0.0796548019107681, -0.9215454747477476}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.492679600662958, \
-0.839918693812442}}], 
      PolygonBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.5081304503817053, -0.8874715196271996}, {-0.47722875094421063`, \
-0.7923658679976843}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.8922779910659062, -0.3899186938124423}}], 
      PolygonBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.9327288407846536, -0.419307956427066}, {-0.8518271413471589, \
-0.36052943119781866`}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.9510565162951536, 0.20901699437494725`}}], 
      PolygonBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-1.0010565162951537`, 
       0.20901699437494722`}, {-0.9010565162951536, 
       0.20901699437494725`}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.6465637775217207, 0.7281152949374525}}], 
      PolygonBox[{{-0.5877852522924734, 
       0.8090169943749473}, {-0.687014627240468, 
       0.7575045575520762}, {-0.6061129278029732, 0.6987260323228289}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.5877852522924734, 
       0.8090169943749473}, {-0.09510565162951536, 0.9690983005625052}}], 
      PolygonBox[{{0., 1.}, {-0.11055650134826274`, 
       1.0166511263772628`}, {-0.07965480191076799, 
       0.9215454747477476}}]}}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0., 1.}, {0.8701548168576588, 0.36779551960419476`}}], 
      PolygonBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.8995440794722824, 0.4082463693229421}, {0.8407655542430351, 
       0.3273446698854474}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.9201548168576588, -0.21391134274543222`}}], 
      PolygonBox[{{0.9510565162951535, -0.30901699437494756`}, {
       0.9677076426724165, -0.19846049302668484`}, {
       0.8726019910429011, -0.22936219246417955`}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.618686951729968, -0.713911342745432}}], 
      PolygonBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.6662397775447256, -0.7293621924641793}, {
       0.5711341259152103, -0.6984604930266847}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.9510565162951535, -0.30901699437494756`}, {
       0.08090169943749487, -0.9412214747707527}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.11029096205211854`, -0.9816723244895}, {
       0.05151243682287121, -0.9007706250520053}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{
       0.5877852522924732, -0.8090169943749473}, {-0.4877852522924734, \
-0.8090169943749472}}], 
      PolygonBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.4877852522924734, -0.8590169943749473}, {-0.4877852522924734, \
-0.7590169943749472}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.8701548168576588, \
-0.3677955196041949}}], 
      PolygonBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.8995440794722824, -0.4082463693229422}, {-0.8407655542430351, \
-0.32734466988544747`}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.9201548168576589, 0.2139113427454319}}], 
      PolygonBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.9677076426724166, 
       0.1984604930266845}, {-0.8726019910429013, 0.22936219246417924`}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.6186869517299681, 0.713911342745432}}], 
      PolygonBox[{{-0.5877852522924734, 
       0.8090169943749473}, {-0.6662397775447257, 
       0.7293621924641793}, {-0.5711341259152104, 0.6984604930266847}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.08090169943749474, 0.9412214747707527}}], 
      PolygonBox[{{0., 1.}, {-0.11029096205211839`, 
       0.9816723244895}, {-0.05151243682287108, 0.9007706250520053}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.5877852522924734, 0.8090169943749473}, {
       0.48778525229247316`, 0.8090169943749475}}], 
      PolygonBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.48778525229247316`, 0.8590169943749475}, {0.48778525229247316`, 
       0.7590169943749474}}]}}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0., 1.}, {0.8922779910659062, -0.2281152949374528}}], 
      PolygonBox[{{0.9510565162951535, -0.30901699437494756`}, {
       0.9327288407846536, -0.19872603232282918`}, {
       0.8518271413471589, -0.25750455755207646`}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.5877852522924732, -0.7090169943749474}}], 
      PolygonBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.6377852522924733, -0.7090169943749474}, {
       0.5377852522924732, -0.7090169943749474}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.05877852522924743, -0.9190983005625053}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.0992293749479948, -0.9484875631771289}, {
       0.018327675510500054`, -0.8897090379478816}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{
       0.9510565162951535, -0.30901699437494756`}, {-0.492679600662958, \
-0.7781152949374525}}], 
      PolygonBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.47722875094421063`, -0.8256681207522102}, {-0.5081304503817053, \
-0.7305624691226948}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{
       0.5877852522924732, -0.8090169943749473}, {-0.8559508646656382, \
-0.3399186938124423}}], 
      PolygonBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.8714017143843855, -0.38747151962719995`}, {-0.8405000149468909, \
-0.2923658679976846}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {-0.8922779910659063, 
       0.22811529493745253`}}], 
      PolygonBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.9327288407846537, 
       0.19872603232282884`}, {-0.851827141347159, 0.2575045575520762}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.5877852522924734, 0.7090169943749474}}], 
      PolygonBox[{{-0.5877852522924734, 
       0.8090169943749473}, {-0.6377852522924734, 
       0.7090169943749474}, {-0.5377852522924733, 0.7090169943749474}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.05877852522924731, 0.9190983005625053}}], 
      PolygonBox[{{0., 1.}, {-0.09922937494799466, 
       0.9484875631771289}, {-0.01832767551049994, 0.8897090379478816}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.9510565162951536, 0.30901699437494723`}, {
       0.4926796006629578, 0.7781152949374527}}], 
      PolygonBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.4772287509442104, 0.8256681207522104}, {0.5081304503817051, 
       0.7305624691226951}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.5877852522924734, 0.8090169943749473}, {0.8559508646656382,
        0.33991869381244216`}}], 
      PolygonBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.8714017143843855, 0.3874715196271999}, {0.8405000149468907, 
       0.2923658679976845}}]}}, {
     {RGBColor[1, 0.6, 0], 
      LineBox[{{0., 1.}, {0.5568835528549785, -0.713911342745432}}], 
      PolygonBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.6044363786697362, -0.6984604930266846}, {
       0.5093307270402208, -0.7293621924641793}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.03090169943749486, -0.9048943483704847}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.07845452525225254, -0.920345198089232}, {-0.016651126377262824`, \
-0.8894434986517372}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{0.9510565162951535, 
       0.30901699437494745`}, {-0.5068835528549787, -0.7502384691456999}}], 
      PolygonBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.47749429024035495`, -0.7906893188644473}, {-0.5362728154696023, \
-0.7097876194269526}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{
       0.9510565162951535, -0.30901699437494756`}, {-0.8510565162951536, \
-0.30901699437494756`}}], 
      PolygonBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.8510565162951536, -0.35901699437494755`}, {-0.8510565162951536, \
-0.2590169943749476}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{
       0.5877852522924732, -0.8090169943749473}, {-0.8701548168576589, 
       0.2502384691456999}}], 
      PolygonBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.8995440794722825, 
       0.20978761942695257`}, {-0.8407655542430352, 0.2906893188644473}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {-0.5568835528549786, 
       0.713911342745432}}], 
      PolygonBox[{{-0.5877852522924734, 
       0.8090169943749473}, {-0.6044363786697363, 
       0.6984604930266846}, {-0.5093307270402209, 0.7293621924641793}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.030901699437494757`, 0.9048943483704847}}], 
      PolygonBox[{{0., 1.}, {-0.07845452525225244, 0.920345198089232}, {
       0.016651126377262918`, 0.8894434986517372}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{-0.9510565162951535, -0.30901699437494756`}, {
       0.5068835528549784, 0.7502384691457001}}], 
      PolygonBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.4774942902403547, 0.7906893188644475}, {0.536272815469602, 
       0.7097876194269528}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{-0.9510565162951536, 0.30901699437494723`}, {
       0.8510565162951536, 0.30901699437494745`}}], 
      PolygonBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.8510565162951536, 0.35901699437494744`}, {0.8510565162951536, 
       0.2590169943749474}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{-0.5877852522924734, 0.8090169943749473}, {
       0.8701548168576587, -0.25023846914570025`}}], 
      PolygonBox[{{0.9510565162951535, -0.30901699437494756`}, {
       0.8995440794722824, -0.20978761942695287`}, {
       0.8407655542430351, -0.2906893188644476}}]}}, {
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{0., 1.}, {1.1634144591899854`*^-16, -0.9}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.05000000000000012, -0.9}, {-0.049999999999999885`, -0.9}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{0.5877852522924731, 
       0.8090169943749475}, {-0.529006727063226, -0.7281152949374525}}], 
      PolygonBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.4885558773444787, -0.7575045575520761}, {-0.5694575767819734, \
-0.6987260323228288}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{0.9510565162951535, 
       0.30901699437494745`}, {-0.8559508646656382, -0.2781152949374528}}], 
      PolygonBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.8405000149468909, -0.32566812075221047`}, {-0.8714017143843855, \
-0.23056246912269512`}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{
       0.9510565162951535, -0.30901699437494756`}, {-0.8559508646656383, 
       0.27811529493745246`}}], 
      PolygonBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.8714017143843856, 
       0.2305624691226948}, {-0.8405000149468909, 0.3256681207522102}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{0.5877852522924732, -0.8090169943749473}, {-0.529006727063226,
        0.7281152949374526}}], 
      PolygonBox[{{-0.5877852522924734, 
       0.8090169943749473}, {-0.5694575767819734, 
       0.6987260323228289}, {-0.48855587734447864`, 0.7575045575520762}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {6.123233995736766*^-18, 
       0.9}}], PolygonBox[{{0., 1.}, {-0.049999999999999996`, 0.9}, {
       0.05000000000000001, 0.9}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{-0.5877852522924734, -0.8090169943749472}, {
       0.5290067270632258, 0.7281152949374528}}], 
      PolygonBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.4885558773444785, 0.7575045575520764}, {0.5694575767819732, 
       0.698726032322829}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{-0.9510565162951535, -0.30901699437494756`}, {
       0.8559508646656382, 0.2781152949374527}}], 
      PolygonBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.8405000149468909, 0.32566812075221035`}, {0.8714017143843855, 
       0.230562469122695}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{-0.9510565162951536, 0.30901699437494723`}, {
       0.8559508646656382, -0.2781152949374528}}], 
      PolygonBox[{{0.9510565162951535, -0.30901699437494756`}, {
       0.8714017143843855, -0.23056246912269512`}, {
       0.8405000149468907, -0.3256681207522105}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{-0.5877852522924734, 0.8090169943749473}, {
       0.5290067270632259, -0.7281152949374526}}], 
      PolygonBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.5694575767819733, -0.6987260323228289}, {
       0.48855587734447853`, -0.7575045575520762}}]}}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425879036333876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
draws a circular graph with five different colored arrows. One may have to \
enlarge the graph to see the colors clearly. The red arrows represent adding \
1 modulo 10, the green arrows represent adding 2, the purple arrows adding 3, \
the orange arrows adding 4, and the grey arrows represent adding 5 modulo 10.

We can also draw a circular graph of the inverses of the elements. The \
command\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6,7,8,9}, Inv]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924731, 0.8090169943749475}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, 0.30901699437494745`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924732, -0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, -0.8090169943749472}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951536, 0.30901699437494723`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, 0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"5\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217205, 0.8899186938124423}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, 0.3399186938124422}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, -0.33991869381244233`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217207, -0.8899186938124422}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, -0.889918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0461621679246689`, -0.33991869381244233`}, {1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.046162167924669, 0.339918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, 0.8899186938124422}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.5877852522924731, 0.8090169943749475}, {-0.4877852522924734,
        0.8090169943749473}}], 
      PolygonBox[{{-0.5877852522924734, 
       0.8090169943749473}, {-0.4877852522924734, 
       0.7590169943749474}, {-0.4877852522924734, 0.8590169943749474}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.5877852522924734, 0.8090169943749473}, {
       0.48778525229247316`, 0.8090169943749475}}], 
      PolygonBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.48778525229247316`, 0.8590169943749475}, {0.48778525229247316`, 
       0.7590169943749474}}]}}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0.9510565162951535, 
       0.30901699437494745`}, {-0.8510565162951537, 0.30901699437494723`}}], 
      PolygonBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.8510565162951537, 
       0.25901699437494724`}, {-0.8510565162951537, 0.3590169943749473}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.9510565162951536, 0.30901699437494723`}, {
       0.8510565162951536, 0.30901699437494745`}}], 
      PolygonBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.8510565162951536, 0.35901699437494744`}, {0.8510565162951536, 
       0.2590169943749474}}]}}, {
     {RGBColor[1, 0.6, 0], 
      LineBox[{{
       0.9510565162951535, -0.30901699437494756`}, {-0.8510565162951536, \
-0.30901699437494756`}}], 
      PolygonBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.8510565162951536, -0.35901699437494755`}, {-0.8510565162951536, \
-0.2590169943749476}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{-0.9510565162951535, -0.30901699437494756`}, {
       0.8510565162951536, -0.30901699437494756`}}], 
      PolygonBox[{{0.9510565162951535, -0.30901699437494756`}, {
       0.8510565162951536, -0.2590169943749476}, {
       0.8510565162951536, -0.35901699437494755`}}]}}, {
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{
       0.5877852522924732, -0.8090169943749473}, {-0.4877852522924734, \
-0.8090169943749472}}], 
      PolygonBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.4877852522924734, -0.8590169943749473}, {-0.4877852522924734, \
-0.7590169943749472}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{-0.5877852522924734, -0.8090169943749472}, {
       0.48778525229247327`, -0.8090169943749473}}], 
      PolygonBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.48778525229247327`, -0.7590169943749473}, {
       0.48778525229247327`, -0.8590169943749473}}]}}, 
    {RGBColor[0.8, 0.6, 0.5], 
     LineBox[{{0.07071067811865486, -0.9292893218813453}, {
      0.09241885630108188, -0.8968331206248827}, {
      0.09998489885977792, -0.8585786437626906}, {
      0.09241885630108186, -0.8203241669004984}, {
      0.07071067811865485, -0.7878679656440357}, {
      0.03825447686219231, -0.7661597874616087}, {
      9.290959971541795*^-17, -0.7585937449029128}, {-0.03825447686219212, \
-0.7661597874616087}, {-0.07071067811865465, -0.7878679656440357}, \
{-0.09241885630108167, -0.8203241669004984}, {-0.0999848988597777, \
-0.8585786437626906}, {-0.09241885630108165, -0.8968331206248827}, \
{-0.07071067811865464, -0.9292893218813453}, {
      1.2246467991473532`*^-16, -1.}}], 
     PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
      0.10606601717798224`, -0.9646446609406726}, {
      0.03535533905932749, -0.8939339828220179}}]}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425879064146376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "draws an arrow from each point to its additive inverse. Notice that every \
straight line has arrowheads on both ends, which indicates that\n             \
       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]], ")"}], 
     RowBox[{"-", "1"}]], " ", "=", " ", "x"}], TraditionalForm]]],
 ",\n\nas we found in Proposition 1.1. Also notice that 0 and 5 are there own \
inverses, as indicated by the two looped arrows.\n\nWe are more familiar with \
base 10 arithmetic since most of us have ten fingers. However, this modular \
arithmetic can be done in other bases as well. The beings on the planet Xantz \
have seven fingers (three on each hand, and one coming out of the nose) so \
they only use the digits 0, 1, 2, 3, 4, 5, and 6. Their modular arithmetic \
would look like this:\n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"4", " ", "+", " ", "5"}], " ", "\[Congruent]", " ", 
     RowBox[{"2", "  ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", "7"}], ")"}]}]}], ","}], TraditionalForm]]],
 "    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", " ", "\[Times]", " ", "5"}], " ", "\[Congruent]", " ", 
    RowBox[{"6", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "7"}], ")"}]}]}], TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879090240126`*^9}],

Cell[TextData[{
 "(Actually, the symbols used on Xantz don't look anything like the symbols \
we use, so this is really a translation of what they would write.)\n\nWe can \
have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " switch to base 7 with one easy command: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879103208876`*^9}],

Cell["DefSumMod[7]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["\<\
We now can look at the multiplication tables and circle graphs of the set \
{0,1,2,3,4,5,6}: \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879133833876`*^9}],

Cell[CellGroupData[{

Cell["MultTable[{0,1,2,3,4,5,6}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0.6, 0}, {1, 0.6, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 
      0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}}, {{0,
       1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {1, 1, 0}, {1, 0, 1}, {
      0, 1, 1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {1, 1, 0}, {1, 
      0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {
      0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 1, 
      1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 7}, {8, 7}}]}]], "Output",
 CellChangeTimes->{3.4425879148833876`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6},Add[1],Add[2],Add[3]]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7818314824680298, 0.6234898018587336}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9749279121818236, -0.22252093395631434`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.43388373911755823`, -0.900968867902419}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.433883739117558, -0.9009688679024191}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9749279121818234, -0.22252093395631545`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7818314824680299, 0.6234898018587334}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], {}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8600146307148329, 0.685838782044607}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0724207034000062`, -0.24477302735194578`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.4772721130293141, -0.991065754692661}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.4772721130293138, -0.9910657546926611}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0724207034000057`, -0.244773027351947}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.860014630714833, 0.6858387820446068}, {1, 0}]}, {{
     {RGBColor[1, 0, 0], 
      LineBox[{{0., 1.}, {0.6917345956777878, 0.6668781757704894}}], 
      PolygonBox[{{0.7818314824680298, 0.6234898018587336}, {
       0.7134287826336658, 0.7119266191656104}, {0.67004040872191, 
       0.6218297323753684}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{0.7818314824680298, 0.6234898018587336}, {
       0.9526758187861921, -0.12502814273813195`}}], 
      PolygonBox[{{0.9749279121818236, -0.22252093395631434`}, {
       1.0014222143952833`, -0.11390209604031624`}, {
       0.903929423177101, -0.1361541894359477}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{0.9749279121818236, -0.22252093395631434`}, {
       0.4962327193034316, -0.8227857196556161}}], 
      PolygonBox[{{0.43388373911755823`, -0.900968867902419}, {
       0.5353242934268331, -0.8539602097485527}, {
       0.4571411451800301, -0.7916112295626794}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{
       0.43388373911755823`, -0.900968867902419}, {-0.333883739117558, \
-0.9009688679024191}}], 
      PolygonBox[{{-0.433883739117558, -0.9009688679024191}, \
{-0.333883739117558, -0.9509688679024192}, {-0.333883739117558, \
-0.8509688679024191}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.433883739117558, -0.9009688679024191}, {-0.91257893199595, \
-0.3007040822031184}}], 
      PolygonBox[{{-0.9749279121818234, -0.22252093395631545`}, \
{-0.9516705061193514, -0.3318785722960551}, {-0.8734873578725485, \
-0.2695295921101817}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.9749279121818234, -0.22252093395631545`}, \
{-0.8040835758636613, 0.525997010640551}}], 
      PolygonBox[{{-0.7818314824680299, 
       0.6234898018587334}, {-0.8528299714727525, 
       0.5371230573383667}, {-0.7553371802545701, 0.5148709639427353}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.7818314824680299, 
       0.6234898018587334}, {-0.0900968867902419, 0.9566116260882442}}], 
      PolygonBox[{{0., 1.}, {-0.11179107374611981`, 
       1.001660069483365}, {-0.068402699834364, 0.9115631826931232}}]}}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0., 1.}, {0.9125789319959503, -0.14433778570951136`}}], 
      PolygonBox[{{0.9749279121818236, -0.22252093395631434`}, {
       0.9516705061193518, -0.11316329561657468`}, {
       0.8734873578725487, -0.17551227580244805`}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7818314824680298, 0.6234898018587336}, {
       0.4561358325131897, -0.8034760766842367}}], 
      PolygonBox[{{0.43388373911755823`, -0.900968867902419}, {
       0.5048822281222809, -0.8146021233820524}, {
       0.40738943690409846`, -0.7923500299864209}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{
       0.9749279121818236, -0.22252093395631434`}, {-0.3437868523273161, \
-0.8575804939906633}}], 
      PolygonBox[{{-0.433883739117558, -0.9009688679024191}, \
{-0.3220926653714382, -0.9026289373857843}, {-0.365481039283194, \
-0.8125320505955423}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{
       0.43388373911755823`, -0.900968867902419}, {-0.8848310253915814, \
-0.2659093078680712}}], 
      PolygonBox[{{-0.9749279121818234, -0.22252093395631545`}, \
{-0.9065252123474593, -0.3109577512631922}, {-0.8631368384357035, \
-0.22086086447295022`}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.433883739117558, -0.9009688679024191}, \
{-0.7595793890723984, 0.525997010640551}}], 
      PolygonBox[{{-0.7818314824680299, 
       0.6234898018587334}, {-0.8083257846814896, 
       0.5148709639427352}, {-0.7108329934633073, 0.5371230573383667}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.9749279121818234, -0.22252093395631545`}, \
{-0.06234898018587331, 0.9218168517531969}}], 
      PolygonBox[{{0., 1.}, {-0.10144055430927482`, 
       0.9529913418461337}, {-0.023257406062471807`, 0.8906423616602603}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.7818314824680299, 0.6234898018587334}, {0.6818314824680298,
        0.6234898018587336}}], 
      PolygonBox[{{0.7818314824680298, 0.6234898018587336}, {
       0.6818314824680298, 0.6734898018587336}, {0.6818314824680298, 
       0.5734898018587335}}]}}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0., 1.}, {0.4116316457219268, -0.8034760766842367}}], 
      PolygonBox[{{0.43388373911755823`, -0.900968867902419}, {
       0.46037804133101795`, -0.7923500299864209}, {
       0.36288525011283557`, -0.8146021233820524}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0.7818314824680298, 
       0.6234898018587336}, {-0.37153475893168464`, -0.8227857196556162}}], 
      PolygonBox[{{-0.433883739117558, -0.9009688679024191}, \
{-0.33244318480828317`, -0.8539602097485528}, {-0.41062633305508617`, \
-0.7916112295626795}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{
       0.9749279121818236, -0.22252093395631434`}, {-0.8749279121818234, \
-0.2225209339563154}}], 
      PolygonBox[{{-0.9749279121818234, -0.22252093395631545`}, \
{-0.8749279121818234, -0.2725209339563154}, {-0.8749279121818234, \
-0.1725209339563154}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{
       0.43388373911755823`, -0.900968867902419}, {-0.7194825022821565, 
       0.5453066536119304}}], 
      PolygonBox[{{-0.7818314824680299, 
       0.6234898018587334}, {-0.7585740764055581, 
       0.5141321635189937}, {-0.680390928158755, 0.576481143704867}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.433883739117558, -0.9009688679024191}, \
{-0.022252093395631438`, 0.9025072087818177}}], 
      PolygonBox[{{0., 1.}, {-0.07099848900472262, 0.9136332554796334}, {
       0.026494302213459747`, 0.8913811620840019}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.9749279121818234, -0.22252093395631545`}, {
       0.691734595677788, 0.5801014279469777}}], 
      PolygonBox[{{0.7818314824680298, 0.6234898018587336}, {0.67004040872191,
        0.6251498713420987}, {0.7134287826336658, 0.5350529845518568}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.7818314824680299, 0.6234898018587334}, {
       0.8848310253915816, -0.17913256004455852`}}], 
      PolygonBox[{{0.9749279121818236, -0.22252093395631434`}, {
       0.9065252123474596, -0.1340841166494376}, {
       0.8631368384357038, -0.2241810034396795}}]}}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425879170240126`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "Notice that the patterns that we observed in modular 10 arithmetic are \
showing up hear as well. \n\nEXPERIMENT:\nTry creating tables and circle \
graphs for modulos other than 7 and 10. Do they all form the same basic \
pattern?\n\nWe can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " define the period to be multiplication modulo 7 instead of addition. This \
is done by the command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["DefMultMod[7]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["Now the name \"multiplication table\" seems more appropriate: ", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879230708876`*^9}],

Cell[CellGroupData[{

Cell["MultTable[{0,1,2,3,4,5,6}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0.6, 0}, {1, 1, 0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0, 1, 
      0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {0.6,
       0.4, 0.8}, {1, 0, 0}, {1, 0, 1}, {1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}}, {{
      0, 1, 0}, {1, 1, 0}, {0, 1, 0}, {1, 0, 1}, {0.6, 0.4, 0.8}, {0, 1, 1}, {
      1, 0.6, 0}, {1, 0, 0}}, {{1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {
      0, 1, 1}, {0.6, 0.4, 0.8}, {1, 0, 1}, {0, 1, 0}}, {{0, 1, 1}, {1, 1, 
      0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {1, 0, 1}, {1, 0, 0}, {0.6, 0.4, 
      0.8}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 7}, {8, 7}}]}]], "Output",
 CellChangeTimes->{3.4362728312112503`*^9, 3.4425879246490126`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
Does this multiplication table have the same features that we have been \
seeing? No it doesn't! Notice that the row starting with 0 contains nothing \
but 0's. However, if we remove the 0 from the table, \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879266958876`*^9}],

Cell[CellGroupData[{

Cell["MultTable[{1,2,3,4,5,6}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 0}, {0, 1, 
      1}, {1, 0, 1}, {1, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0, 1, 1}, {1, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0, 0}, {1, 0, 1}}, {{1, 0, 0}, {1, 0, 0}, {1, 
      1, 0}, {0, 1, 0}, {1, 0, 1}, {0.6, 0.4, 0.8}, {0, 1, 1}}, {{0, 1, 1}, {
      0, 1, 1}, {0.6, 0.4, 0.8}, {1, 0, 1}, {0, 1, 0}, {1, 1, 0}, {1, 0, 
      0}}, {{1, 0, 1}, {1, 0, 1}, {1, 0, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0, 
      1, 1}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 
      1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.4362728336175003`*^9, 3.4425879280396376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztnb9vJEkVx+v8Y1cIAuLDnu756b0ELbqEhJNFwO2ZszjpTgQIJFYLyCeE
OG5PYiVMgoQuwBJkaJ2Q4YjEIkJaLRGbEEy2cujQ0fo/aPq9ququ7nrv9avZ
DcamZtfjme737anvq+qq6v647MOHXxz9/FcPv/j00cPy/c8ffnb06aPH5fd+
/Xm9afMtYzZ26q9vlAZeV8b4p6r+b59G8ASb13TDJ/ht897Lly+/fPLkCb7D
/f+x+78K3z6onv7iv2azevzh02rx9rv47gE8H0DwAcTBqwpewVNlZVmd1Vmd
1Vmd1bdAPcb4O+b3H//dHHzrkfEPeP2b7/91Qhy8kCVT2D2CzzZ3jbmodxyb
9nFYf53WX0tj5hCzgJJswJ4JvAJdPXIb8+yZMUdHKMG9sO24PtD5uTFXV2YG
cSMc1jfr47efcNcFH5kjOyWoIwp4LuH1Hdg56+egtCbNz777R1TXJsyG2/an
n/zL/Pg7v0NzYyIZI6V20mRlw5ezdmKuDCbKXNuUoJFLY6ZsarZsGuBxWudx
udzwGauzgpvqDIG8KoTsXJiLJjuYxTFmY8tcXrZVVb+e91NVOGu/fP/PGANt
YNttc1aDNtJN1a5SW7YNCGxhdvDdtt92hNnCd2MiUwuwtgHZmMCr0iXi0lw2
prfabZigs/ofxk1gPx5q0fc+CqoVHp98+zNzp7fttz/4G+l9R6lF7wvv89Kd
MNeCT4i7vrY1tlxiHNRm5HjbbXtmnnWrXuHYN+4//PAf5q7b5s748CQgHQ9p
MVt73jE8zmzNloTjPe8YHsfH+A7iwETkeK509947H8N+fIclxHCpFmlFGTk5
DJxArs3XbBxVd8bWcTV1Tl6YF42TmcYJ7rrba1OiE1bRbYVtAW0rRCf1Dui/
TvG60GDrolomdOOhu5lzd27OG3fTZHe+B4GOVecuVHTdnbviHTl3hXMHneYL
526r627ue+KLizY1x8docO4MnprTxuBE2RC/WbzXNCvbMW51qqU+haRWqZCj
dRx3ofjLtvR1t9Bxf2UzgnXfcz/1dQvuT05Q3QzRy6UzDYcK0gH97Ik5SUjH
No4Jvs5grPjLT//thtYtfO/8Qt2WbFqSDoMD/Mynp226MC7DUGJPXTw5a+uu
cWx204Ot4I7Px9lZkw/spxYuH3UufE9sxqp82PkWPKBHhddgrC68zz28/fJH
/2yHUDIfSYfBfJQ+H9AWYEQ6x7bRzsqwk2sH47HL0WlwAm10c4TDcl01ON04
O6s7ykOb5noAm4ZpujD2DLuu/w2lqbAnfW3lHjf5wL3vwKsinEq4R+PoEM8H
jJj6BgsZgO1X1lHXzyzszGF6Vn+hjz3nAx71FKPybWjQR63x9QRd154wnQrj
7rXettvzO5hzL0I/l9iwCT9z78dPB+tufNL342bYg376s2so70Ixk4a4vbYB
bvoOCd/t+YHDD0TPfEtrTDRjEBb2CN+Nw8uGsIv2s6Mr7PnsY6jBucsBnMv5
As/Z66Vu3KJjbGmaWc+eH++hgo5tg/OP7hUSzPWvruxQ1K8v484xbxWmt25C
0ViFLsjV46DViesvfd1AJ1H3o1PCbUmHzmDzmKjJRWgYzJ6QhrfsBQ50FDCn
uL7e8JvrPqPpP05OAttb2G0YbPynULMb1j/2KifB59SZGczA1PWeMI1107+m
B60HWarDHcuScScjwVi8hc+b/uIQIwo/ofWjTydFha//5TI+CuTGtRNs/v66
cBl85rYLdn1Ue14XmIFNHOeho4ZLcjpBruWjW3AJwy07kZ5ywZjHCdE5b/rn
k15SFsFwY1wngEMQjrdwxQsnxelpeyhICHipO2vspO314V2co0BLOTS2NcF3
eA/bp20ytrGXh8kP3ItwE6A4Hzf3llBWZ3VWZ3VW3xY1DqeVeV5/37dD5wMr
tBp4hQfD49Qbszqrszqrszqrb4taQ6RnxCfsKHR4Rz0JXZdQsJZGD1B4vFyO
cDB8qBJ+R4nQQOgJlw01wU5F1hKF1kL6CZGrQshVhMLjRjNMoUu24agJtgmR
de9GlJa6l4T30nlX0e/QO0WhKZ8zIq71ROHpnjstV6fcJZHu0B1FnAvC3ZyI
wyxIKJpyp+LgPXcs1WadHCqdhGSaRdGME4F+9yA3y7LJdhZwZrGd9Xk0C5up
8svMW4G2WYJNegrosuipT6ElxDxoK4LdhbMlMm1rK+LWoS2GHFPOpnTo2KbL
VZiGIDONUAu7vXUBaLOMOrTOUOExYX1Ch+JmmQ6TXtMJduk8C5Sa5dCh59Ib
GSa/VBMY6eVjm3MoqkCFyfSkA20FoGb5c5ieIhzM3YNiyNQ8alepndouM+ig
Q7xMTRNaxj2IrVk6HbukafKcdSbQZxYuM5OeiG8P4muWUseNm2CRZCsm6DPL
l3s+KP6NXFvG10mUmjQWUGXRWJ8+JxHmnlmKiSPrlpG2taWi1KFViidTbbIg
4hat1SS23DPMkPFZ6Jmm2vD56YiatD8Aj0dSSjTgufC1r+TLFY+MBzg6th+B
c9tGoULXYaa0RJnKVKnUFvaTIQ8K6GyzJLBklqe3l0E09YZiKGD2utyXyeqs
zuqszuqsBjWOa9Xzekzbd+PaAxdpD2E1B55+Z3VWZ3VWZ3VW3xp1i4HGxHFm
zV4P+EKUrF55nQSQNYA6CYCvbFdixGnrqwshAREVTsLPqdx75WREd8Rx7qVD
zqWzrwLDiQTahDD7Tbljl0UnQeBE2uxqrI+w35QnduEzi36TeDILqcPyt9C2
IMq/0+wth8ofQmUW/SooMguko1KzqJaqiR1RIba4PnJmIbCCMbO4OnTnaoIh
tpS7XVEhuovhs3UXsWAFapbANWWQ5rKSQYHk9gxKC51Z6JsKmJNgdpiB0vmh
KS2VgZGooDIgrWVmEfCq2Flm26F1mcBS1gtRQVkvvXUlfWaR76qYeRBnhxmR
WC2Vj1KIp7JRhLMA92B5M8uFe4CZ5dKUM4rWSs5Yuks6E3gzS4NZwMwS6tAX
S28pU2MumG+5BHFOQsIsdma5deiOwbWUtwkdKjqLkbN1oSLALGNOAteU2x6o
ldxSTDdi4XE9Llq3qyxP5lnzCiQ7zACFXCn7UyLOo9mKp75Jy5z9dZmK8mp4
cuHbgAZfRzP7HmUtiazMiLhxmxWB8iYtdbZXbAq2K7NjHI41jHpdbodkdVZn
dVZndVZbwFwhYH5emf19fAeA2Yej0B7CHsdkdVZndVZndVbfFrUWtk6JT9lR
avH+no78VkaNktWAugRRC7Z7ayPjZKh464TNhx7XJrHgQshNRJnTALaEwoVs
tXh2TCRj1uzFVCWB39J5VSHlJFgdoSvZXQ/ZlpxPCu0mAeIk9MwCbL07CugW
hLs5EVeG7lT4mIXQErZetZ6M0dWTMd16EkAyC6FZbB39QEUVwm+y/AHGFcvf
x70KVMxiZhZMU+WXkXfoiYG3lK0pHco4IzCxdRYhZhZKR9jHOhNwd+hMS2lH
hNWxUluErVL/y7jb7rHDoSV4zeRCIONhLlhOS3UkBRcc9CbpqJhlzxKwjmCS
dS3Q8NC1hGOpSUEpxE9b7+lcmAXNpfeuXi9ddXGTzYeAyMN8UPCWysOEiJs2
M4MWBgvQOBjT3YNFz9HqSjyvecwd915dbEvNcKZEXGeuEwFhASHjrEX8VduU
HxZukyMMfuxR0CqZESaIQ98sAk4EyUH9hTi6501ezkxeuwxwWqrydpRarFCB
FFuzKt7cJKBPraMftrGVS4Fw8mpFhWr5JCQuW5bYMbhJ59Es4GYSQzHzMDGF
r8cBTkudBLtKrfsbKxpobJuDikKn8u2KR+Xs4ucwUzYDMsKlOvORQofd4xBB
Bh8KEJ2Et21eBFjOLnpel5syWZ3VWZ3VWX1b1LAPQPXzat/s47sHVlMBrbaH
sMjaHierszqrszqrszqr4ZuWDY+IT9lRagt4jn4z8DC3LkHeXTqt5uJqjN29
w8EbCmHwLpGMXaV2RCQjiVgPrbBOQeVpQLt7i8PaHWLJBZGqkVJbEqlKAt7E
isqAnScR7tC7lipTzaRQaqlmkoS+Ke8UT2epd+iYIs0zwh3VKuaCEy3m7q9p
Jtk5S71DJxRznhJOxkScWyBdhcxZgbmHasK4W+ks9Q7LTzHnCVH+CRE3bcov
M2cF+uY89Sk6S8JDTxRtHhOepkQc+iycJ4E2K6A3sTyZQ+gsBqdshexYshUx
Zm9LAMepPDuCQq7edH8s2lQc8g59U/S4JHzPiDhHdKxvAR2vSraJNYA8Ppdo
N2U4RMaS4QgtK9jw6sucqy7hhKML2Lz0npWUO8wDxZip8X9OxGG+Bnly/Cu0
+8vpGRYejOruwTJuzpEny0OO+n942YhEWfo12nFvRFJxnLfwjJua7Q+RZar1
7ii12KJl2sySb25Ywbx20HlQoSH0pswOUWTJrIpAS2SZZdzRj4I5qxpw3tjv
I+/YfhI25jORTp8lqizRbjIvSSyd5d7Upd4QP6bO9l2lFtPJAmMl5a54Fl34
9qGB6kkMPL7Sk2kyNQMdKXTupyXb0c+E3Fim3DY3Ao/GMVxD05Mw+LrcuMnq
rM7qrM7qW6OucNX1c1Pt79sdD2x4ZSm4P46p7MGyOquzOquzOquzGr6V8BJv
506J44yavQ4n43CtXxWtZs8R7IKLRwWK1jDx0C5DgmeE94IORWhcCIkgl0Cn
EGYqF2rgnMTFw8y0N5HmRDLKZm+zUi0BR5uQIZNVrUTKSSh7ZXfJS6AZStzx
yULjJES9sicKRzcLn0k23Cs/i4qTwPTK5WcXPrNsuEedWVSswNErl1pc7kzS
XwVrZgGxAkKv7IRd3iwx38KZESAzi4UV9Pm1zUT4OAnuenfiH2I2Fcd/Uzn0
ym7Zpckyyi2dPQEpS7R3Vdz82jYjVDwIcBUsufROlYx3Vfa8snt2sbL0p5bN
ADkORmH3YBFwDzO/ER8dLCz9kWUzwItxTsHjXhYor+wiaTkyi2yHiHBQPSHQ
ZVHxa9tRod0kLCtx4MZgH9mygPg1DaYT2xVQqwR5heXHPP0NTdtrFgJ+UjnY
5YKbX8GhJraFr18NVmXxbhJv1bDgODU0++SzI7JSKIYC1uJoqoGqLNVNoq0y
Al6X+w5ZndVZndVZvY7q+/cNfK/V9fMB4NkDWHAMr0AN2x5/+LSvBpL76tWr
6qOP8PNrdf18AGD3AJYqwytQ75sPqsXb79bB98v9r9cCPMxXgoMB860O1CXP
4Tk8h+fwHD4QXsL724Tz0JW0urMQLN5QUIeeez/yXTqbN4bHUSZuCHajir6G
dI0q5tpANKpwa83KqAKvOxKTynxjyRdl6uYCLsnNreFYlMk1xlVDxV0rKkUV
dv3hk1Tqm8SYeB83HyWhtwLf/78gosDyreY+63dhmsNzeA7P4cpwHFjwaXTT
Npi3/gcBlNWn\
\>"]]
}, Open  ]],

Cell["\<\
we see that the rest of the table has the same properties that we observed in \
Terry's multiplication table. The identity is now the number 1. This table is \
in fact symmetrical, indicating that multiplication modulo 7 is commutative.

Here is a circular graph of multiplication modulo 7: \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879299927626`*^9}],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6},Mult[2],Mult[3]]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7818314824680298, 0.6234898018587336}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9749279121818236, -0.22252093395631434`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.43388373911755823`, -0.900968867902419}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.433883739117558, -0.9009688679024191}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9749279121818234, -0.22252093395631545`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7818314824680299, 0.6234898018587334}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], {}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8600146307148329, 0.685838782044607}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0724207034000062`, -0.24477302735194578`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.4772721130293141, -0.991065754692661}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.4772721130293138, -0.9910657546926611}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0724207034000057`, -0.244773027351947}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.860014630714833, 0.6858387820446068}, {1, 0}]}, {{
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.07071067811865475, 
       0.9292893218813453}, {-0.09241885630108176, 
       0.8968331206248827}, {-0.09998489885977781, 
       0.8585786437626906}, {-0.09241885630108176, 
       0.8203241669004984}, {-0.07071067811865475, 
       0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0.,
        0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
       0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
       0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
       0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
       0.9292893218813453}, {0., 1.}}], 
      PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
       0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{0.7818314824680298, 0.6234898018587336}, {
       0.9526758187861921, -0.12502814273813195`}}], 
      PolygonBox[{{0.9749279121818236, -0.22252093395631434`}, {
       1.0014222143952833`, -0.11390209604031624`}, {
       0.903929423177101, -0.1361541894359477}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{
       0.9749279121818236, -0.22252093395631434`}, {-0.3437868523273161, \
-0.8575804939906633}}], 
      PolygonBox[{{-0.433883739117558, -0.9009688679024191}, \
{-0.3220926653714382, -0.9026289373857843}, {-0.365481039283194, \
-0.8125320505955423}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{
       0.43388373911755823`, -0.900968867902419}, {-0.7194825022821565, 
       0.5453066536119304}}], 
      PolygonBox[{{-0.7818314824680299, 
       0.6234898018587334}, {-0.7585740764055581, 
       0.5141321635189937}, {-0.680390928158755, 0.576481143704867}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.433883739117558, -0.9009688679024191}, {0.7194825022821565,
        0.5453066536119306}}], 
      PolygonBox[{{0.7818314824680298, 0.6234898018587336}, {
       0.680390928158755, 0.5764811437048672}, {0.758574076405558, 
       0.5141321635189939}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.9749279121818234, -0.22252093395631545`}, {
       0.3437868523273163, -0.8575804939906633}}], 
      PolygonBox[{{0.43388373911755823`, -0.900968867902419}, {
       0.36548103928319414`, -0.8125320505955423}, {
       0.32209266537143844`, -0.9026289373857842}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.7818314824680299, 
       0.6234898018587334}, {-0.952675818786192, -0.12502814273813306`}}], 
      PolygonBox[{{-0.9749279121818234, -0.22252093395631545`}, \
{-0.9039294231771008, -0.13615418943594876`}, {-1.0014222143952831`, \
-0.11390209604031738`}}]}}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.07071067811865475, 
       0.9292893218813453}, {-0.09241885630108176, 
       0.8968331206248827}, {-0.09998489885977781, 
       0.8585786437626906}, {-0.09241885630108176, 
       0.8203241669004984}, {-0.07071067811865475, 
       0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0.,
        0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
       0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
       0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
       0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
       0.9292893218813453}, {0., 1.}}], 
      PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
       0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7818314824680298, 0.6234898018587336}, {
       0.4561358325131897, -0.8034760766842367}}], 
      PolygonBox[{{0.43388373911755823`, -0.900968867902419}, {
       0.5048822281222809, -0.8146021233820524}, {
       0.40738943690409846`, -0.7923500299864209}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{
       0.9749279121818236, -0.22252093395631434`}, {-0.691734595677788, 
       0.5801014279469776}}], 
      PolygonBox[{{-0.7818314824680299, 
       0.6234898018587334}, {-0.7134287826336658, 
       0.5350529845518566}, {-0.6700404087219101, 0.6251498713420985}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.43388373911755823`, -0.900968867902419}, {
       0.9125789319959503, -0.3007040822031173}}], 
      PolygonBox[{{0.9749279121818236, -0.22252093395631434`}, {
       0.8734873578725488, -0.2695295921101806}, {
       0.9516705061193518, -0.331878572296054}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.433883739117558, -0.9009688679024191}, {-0.91257893199595, \
-0.3007040822031184}}], 
      PolygonBox[{{-0.9749279121818234, -0.22252093395631545`}, \
{-0.9516705061193514, -0.3318785722960551}, {-0.8734873578725485, \
-0.2695295921101817}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.9749279121818234, -0.22252093395631545`}, {
       0.691734595677788, 0.5801014279469777}}], 
      PolygonBox[{{0.7818314824680298, 0.6234898018587336}, {0.67004040872191,
        0.6251498713420987}, {0.7134287826336658, 0.5350529845518568}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.7818314824680299, 
       0.6234898018587334}, {-0.45613583251318945`, -0.8034760766842368}}], 
      PolygonBox[{{-0.433883739117558, -0.9009688679024191}, \
{-0.4073894369040983, -0.792350029986421}, {-0.5048822281222807, \
-0.8146021233820525}}]}}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{{3.4362728164768753`*^9, 3.4362728365550003`*^9}, 
   3.4425879316646376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
Here, the red arrow indicates multiplication by 2, and the green arrow \
represents multiplication by 3 modulo 7. There is no red arrow coming from 0 \
simply because the green loop completely covers up the red loop.

Here is a circular graph of the inverse elements: \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879339458876`*^9}],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6},Inv]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7818314824680298, 0.6234898018587336}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9749279121818236, -0.22252093395631434`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.43388373911755823`, -0.900968867902419}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.433883739117558, -0.9009688679024191}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9749279121818234, -0.22252093395631545`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7818314824680299, 0.6234898018587334}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], {}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8600146307148329, 0.685838782044607}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0724207034000062`, -0.24477302735194578`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.4772721130293141, -0.991065754692661}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.4772721130293138, -0.9910657546926611}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0724207034000057`, -0.244773027351947}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.860014630714833, 0.6858387820446068}, {1, 0}]}, {{
     InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[1, 0, 0], LineBox[{{0., 1.}, {0., 0.2}}], 
      PolygonBox[{{0., 0.1}, {0.05000000000000001, 
       0.2}, {-0.05000000000000001, 0.2}}]}}, 
    {RGBColor[0, 1, 0], 
     LineBox[{{0.6824602614787055, 0.6346862494690644}, {0.6435501538214092, 
      0.6314222761086324}, {0.6089242490894264, 0.6134863701796962}, {
      0.5837330451089958, 0.583719723710597}, {0.5718925928790505, 
      0.5465114760900709}, {0.5751565662394824, 0.5076013684327746}, {
      0.5931477560027185, 0.47297546370079174`}, {0.622859118637518, 
      0.4477842597203612}, {0.660067366258044, 0.43594380749041584`}, {
      0.6989774739153404, 0.43920778085084783`}, {0.7336033786473232, 
      0.457143686779784}, {0.7587945826277537, 0.4869103332488833}, {
      0.770635034857699, 0.5241185808694093}, {0.7818314824680298, 
      0.6234898018587336}}], 
     PolygonBox[{{0.7818314824680298, 0.6234898018587336}, {
      0.6880584852838709, 0.6843718599637265}, {0.6768620376735401, 
      0.5850006389744022}}]}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{
       0.9749279121818236, -0.22252093395631434`}, {-0.3437868523273161, \
-0.8575804939906633}}], 
      PolygonBox[{{-0.433883739117558, -0.9009688679024191}, \
{-0.3220926653714382, -0.9026289373857843}, {-0.365481039283194, \
-0.8125320505955423}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.433883739117558, -0.9009688679024191}, {
       0.8848310253915816, -0.26590930786807015`}}], 
      PolygonBox[{{0.9749279121818236, -0.22252093395631434`}, {
       0.8631368384357038, -0.2208608644729492}, {
       0.9065252123474596, -0.31095775126319114`}}]}}, {
     {RGBColor[1, 0.6, 0], 
      LineBox[{{
       0.43388373911755823`, -0.900968867902419}, {-0.8848310253915814, \
-0.2659093078680712}}], 
      PolygonBox[{{-0.9749279121818234, -0.22252093395631545`}, \
{-0.9065252123474593, -0.3109577512631922}, {-0.8631368384357035, \
-0.22086086447295022`}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{-0.9749279121818234, -0.22252093395631545`}, {
       0.3437868523273163, -0.8575804939906633}}], 
      PolygonBox[{{0.43388373911755823`, -0.900968867902419}, {
       0.36548103928319414`, -0.8125320505955423}, {
       0.32209266537143844`, -0.9026289373857842}}]}}, 
    {RGBColor[0.7, 0.8, 0.7], 
     LineBox[{{-0.7706350348576991, 0.5241185808694091}, {-0.7587945826277538,
       0.48691033324888305`}, {-0.7336033786473233, 
      0.4571436867797838}, {-0.6989774739153404, 
      0.4392077808508476}, {-0.6600673662580441, 
      0.4359438074904157}, {-0.622859118637518, 
      0.447784259720361}, {-0.5931477560027185, 
      0.4729754637007916}, {-0.5751565662394825, 
      0.5076013684327744}, {-0.5718925928790506, 
      0.5465114760900708}, {-0.5837330451089959, 
      0.5837197237105968}, {-0.6089242490894264, 
      0.6134863701796961}, {-0.6435501538214093, 
      0.6314222761086322}, {-0.6824602614787056, 
      0.6346862494690642}, {-0.7818314824680299, 0.6234898018587334}}], 
     PolygonBox[{{-0.7818314824680299, 
      0.6234898018587334}, {-0.8203206453523613, 
      0.5185203570642437}, {-0.720949424363037, 0.5297168046745745}}]}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4362728501956253`*^9, 3.4425879353052626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "What happens to 0? Recall that the inverse of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is defined to be the number ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", "1"}], 
   TraditionalForm]]],
 ". But the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"0", "\[CenterDot]", "y"}], " ", "=", " ", 
    RowBox[{"1", " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "7"}], ")"}]}]}], TraditionalForm]]],
 " has no solution, so 0 does not have an inverse (as we saw in the \
multiplication table.) Thus, when ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " graphs the inverse function, 0 is sent to a big question mark to indicate \
that there is no inverse of 0. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879380396376`*^9}],

Cell[TextData[{
 "EXPERIMENT:\nTry having ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " work with multiplication with a modulo other than 7, and form a circle \
graph of the inverse function. Is 0 the only number that does not have an \
inverse? If not, can you find a pattern to the numbers which do not have an \
inverse? "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879403521376`*^9}],

Cell["\<\
In the last experiment, you probably discovered that there may be many \
numbers without a multiplicative inverse. For example, by looking at the \
multiplication table modulo 10, \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879433990126`*^9}],

Cell["DefMultMod[10]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["MultTable[{0,1,2,3,4,5,6,7,8,9}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {1, 1, 0}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {
      0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 0}, {0, 
      1, 1}, {1, 0, 1}}, {{0.8, 0.6, 0.5}, {1, 1, 0}, {0.8, 0.6, 0.5}, {
      1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {
      1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0.7, 
      0.8, 0.7}, {0, 1, 0}, {1, 0, 1}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {0, 
      1, 1}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {1, 0, 0}}, {{1, 0.6, 0}, {1, 1, 
      0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0, 1, 0}, {1, 1, 0}, {
      1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0, 1, 0}}, {{0.6, 0.4, 0.8}, {
      1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}}, {{0,
       1, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {1, 0.6, 0}, {
      1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {1, 0.6, 0}}, {{1, 0, 
      0}, {1, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0.6,
       0.4, 0.8}, {0.8, 0.6, 0.5}, {1, 0, 1}, {0, 1, 0}, {0.7, 0.8, 0.7}}, {{
      0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 0.6, 
      0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 0.6, 0.5}}, {{
      1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.4362728736331253`*^9, 3.4425879467115126`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
we find several rows that do not contain any 1's. These rows indicate the \
numbers without inverses modulo 10. We can see these more clearly by looking \
at the circle graph of the inverse function. \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879488208876`*^9}],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6,7,8,9},Inv]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924731, 0.8090169943749475}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, 0.30901699437494745`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924732, -0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, -0.8090169943749472}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951536, 0.30901699437494723`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, 0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"5\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217205, 0.8899186938124423}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, 0.3399186938124422}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, -0.33991869381244233`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217207, -0.8899186938124422}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, -0.889918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0461621679246689`, -0.33991869381244233`}, {1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.046162167924669, 0.339918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, 0.8899186938124422}, {1, 0}]}, {{
     InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[1, 0, 0], LineBox[{{0., 1.}, {0., 0.2}}], 
      PolygonBox[{{0., 0.1}, {0.05000000000000001, 
       0.2}, {-0.05000000000000001, 0.2}}]}}, 
    {RGBColor[0, 1, 0], 
     LineBox[{{0.4890164182329594, 0.7933735478709244}, {0.45237685672247135`,
       0.7798756764713605}, {0.4237703823785619, 0.7533743628131425}, {
      0.4074060220549509, 0.717978632686487}, {0.4058910306774687, 
      0.6789612673073875}, {0.4193889020770326, 0.6423217057968995}, {
      0.4459317784290283, 0.6137152314529901}, {0.481285945861906, 
      0.597350871129379}, {0.5203033112410056, 0.5958358797518968}, {
      0.5569428727514936, 0.6093337511514607}, {0.585549347095403, 
      0.6358350648096787}, {0.6019137074190141, 0.6712307949363341}, {
      0.6034286987964962, 0.7102481603154337}, {0.5877852522924731, 
      0.8090169943749475}}], 
     PolygonBox[{{0.5877852522924731, 0.8090169943749475}, {
      0.4811946949809478, 0.8427579649006812}, {0.4968381414849709, 
      0.7439891308411675}}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.19021130325903074`, 0.06180339887498949}}], 
      PolygonBox[{{0.09510565162951536, 0.030901699437494747`}, {
       0.2056621529777781, 0.014250573060231805`}, {0.17476045354028336`, 
       0.10935622468974718`}}]}}, {
     {RGBColor[1, 0.6, 0], 
      LineBox[{{
       0.9510565162951535, -0.30901699437494756`}, {-0.8510565162951536, \
-0.30901699437494756`}}], 
      PolygonBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.8510565162951536, -0.35901699437494755`}, {-0.8510565162951536, \
-0.2590169943749476}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{-0.9510565162951535, -0.30901699437494756`}, {
       0.8510565162951536, -0.30901699437494756`}}], 
      PolygonBox[{{0.9510565162951535, -0.30901699437494756`}, {
       0.8510565162951536, -0.2590169943749476}, {
       0.8510565162951536, -0.35901699437494755`}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.11755705045849466`, -0.16180339887498948`}}], 
      PolygonBox[{{0.05877852522924733, -0.08090169943749474}, {
       0.07710620073974728, -0.19119266148961317`}, {
       0.15800790017724203`, -0.13241413626036583`}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.8, 0.6, 0.5], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {
       2.4492935982947065`*^-17, -0.2}}], 
      PolygonBox[{{
       1.2246467991473533`*^-17, -0.1}, {-0.04999999999999998, -0.2}, {
       0.05000000000000004, -0.2}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.5, 0.6, 0.8], 
      LineBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.11755705045849468`, -0.16180339887498946`}}], 
      PolygonBox[{{-0.05877852522924734, -0.08090169943749473}, \
{-0.15800790017724203`, -0.1324141362603658}, {-0.07710620073974732, \
-0.1911926614896131}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.8, 0.2, 0.5], 
      LineBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.19021130325903074`, 0.06180339887498945}}], 
      PolygonBox[{{-0.09510565162951537, 
       0.030901699437494726`}, {-0.17476045354028336`, 
       0.10935622468974714`}, {-0.20566215297777812`, 
       0.01425057306023177}}]}}, 
    {RGBColor[0.8, 0.6, 0.2], 
     LineBox[{{-0.6034286987964964, 0.7102481603154336}, {-0.6019137074190142,
       0.6712307949363341}, {-0.5855493470954032, 
      0.6358350648096786}, {-0.5569428727514938, 
      0.6093337511514606}, {-0.5203033112410057, 
      0.5958358797518968}, {-0.4812859458619062, 
      0.5973508711293789}, {-0.4459317784290284, 
      0.61371523145299}, {-0.4193889020770327, 
      0.6423217057968995}, {-0.4058910306774689, 
      0.6789612673073875}, {-0.4074060220549511, 
      0.717978632686487}, {-0.4237703823785621, 
      0.7533743628131424}, {-0.4523768567224715, 
      0.7798756764713604}, {-0.4890164182329596, 
      0.7933735478709243}, {-0.5877852522924734, 0.8090169943749473}}], 
     PolygonBox[{{-0.5877852522924734, 
      0.8090169943749473}, {-0.6528131158262533, 
      0.718069883567445}, {-0.5540442817667395, 0.702426437063422}}]}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425879506958876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "So 0, 2, 4, 5, 6, and 8 do not have inverses. Do you see a pattern in these \
numbers? Hint: 2 does not have an inverse because there is no integer ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " such that \n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", " ", "\[CenterDot]", " ", "x"}], " ", "=", " ", 
    RowBox[{"1", " ", "+", " ", 
     RowBox[{"10", " ", "n"}]}]}], TraditionalForm]]],
 "\n\nfor some ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".\n\nTo help see the pattern, look at multiplication modulo 15:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["DefMultMod[15]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6,7,8,9,10,11,12,13,14},Inv]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.40673664307580015`, 0.9135454576426009}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7431448254773941, 0.6691306063588582}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, 0.30901699437494745`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9945218953682734, -0.10452846326765333`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8660254037844387, -0.4999999999999998}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924732, -0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.20791169081775931`, -0.9781476007338057}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.20791169081775907`, -0.9781476007338057}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.587785252292473, -0.8090169943749476}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.8660254037844384, -0.5000000000000004}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9945218953682733, -0.10452846326765423`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951536, 0.30901699437494723`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.743144825477394, 0.6691306063588585}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.40673664307580015`, 0.913545457642601}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], {}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.4474103073833802, 1.0049000034068611`}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8174593080251336, 0.7360436669947441}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, 0.3399186938124422}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0939740849051007`, -0.11498130959441867`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9526279441628827, -0.5499999999999998}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217207, -0.8899186938124422}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.22870285989953526`, -1.0759623608071864`}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.22870285989953498`, -1.0759623608071864`}, {1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217203, -0.8899186938124424}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9526279441628823, -0.5500000000000005}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0939740849051007`, -0.11498130959441967`}, {1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"12\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.046162167924669, 0.339918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.8174593080251334, 0.7360436669947443}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.4474103073833802, 1.0049000034068611`}, {1, 0}]}, {{
     InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[1, 0, 0], LineBox[{{0., 1.}, {0., 0.2}}], 
      PolygonBox[{{0., 0.1}, {0.05000000000000001, 
       0.2}, {-0.05000000000000001, 0.2}}]}}, 
    {RGBColor[0, 1, 0], 
     LineBox[{{0.31337860042607996`, 0.8777086626880708}, {0.2803460665077814,
       0.8568879589791071}, {0.2578746451944051, 0.82501814215885}, {
      0.24922707150468354`, 0.7869935518352078}, {0.25585735273088983`, 
      0.7485138250838206}, {0.27667805643985355`, 0.7154812911655221}, {
      0.3085766338839582, 0.6930098698521457}, {0.34657246358375293`, 
      0.6843622961624242}, {0.38505219033514004`, 0.6909925773886305}, {
      0.41808472425343857`, 0.7118132810975942}, {0.4405561455668149, 
      0.7436830979178513}, {0.44920371925653646`, 0.7817076882414935}, {
      0.44257343803033017`, 0.8201874149928807}, {0.40673664307580015`, 
      0.9135454576426009}}], 
     PolygonBox[{{0.40673664307580015`, 0.9135454576426009}, {
      0.2954602029488149, 0.9243876840129309}, {0.331296997903345, 
      0.8310296413632108}}]}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0.7431448254773941, 
       0.6691306063588582}, {-0.15791169081775908`, -0.8915450603553619}}], 
      PolygonBox[{{-0.20791169081775907`, -0.9781476007338057}, \
{-0.11461042062853712`, -0.9165450603553618}, {-0.20121296100698102`, \
-0.8665450603553618}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.20791169081775907`, -0.9781476007338057}, {
       0.6931448254773941, 0.5825280659804144}}], 
      PolygonBox[{{0.7431448254773941, 0.6691306063588582}, {
       0.6498435552881722, 0.6075280659804143}, {0.7364460956666161, 
       0.5575280659804144}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.19021130325903074`, 0.06180339887498949}}], 
      PolygonBox[{{0.09510565162951536, 0.030901699437494747`}, {
       0.2056621529777781, 0.014250573060231805`}, {0.17476045354028336`, 
       0.10935622468974718`}}]}}, 
    {RGBColor[0.7, 0.8, 0.7], 
     LineBox[{{0.9315898562632896, -0.026813867121956242`}, {
      0.9015805759790079, -0.0018320117701330213`}, {0.8643265279453769, 
      0.009691264895291346}, {0.8254907463083896, 0.006165351588892878}, {
      0.7909432210126088, -0.012031310081242919`}, {
      0.7659613656607855, -0.04204059036552469}, {
      0.7545084123129866, -0.0792946383991557}, {
      0.7579640023017598, -0.11813042003614302`}, {
      0.7761606639718955, -0.15267794533192378`}, {
      0.8061699442561773, -0.17765980068374698`}, {
      0.8434239922898082, -0.18918307734917136`}, {
      0.8822597739267956, -0.18565716404277288`}, {
      0.9168072992225763, -0.1674605023726371}, {
      0.9945218953682734, -0.10452846326765333`}}], 
     PolygonBox[{{0.9945218953682734, -0.10452846326765333`}, {
      0.9704471543361382, 0.004652152430535614}, {
      0.8927325581904411, -0.058279886674448125`}}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.8, 0.6, 0.5], 
      LineBox[{{0.8660254037844387, -0.4999999999999998}, {
       0.17320508075688776`, -0.09999999999999996}}], 
      PolygonBox[{{0.08660254037844388, -0.04999999999999998}, {
       0.14820508075688776`, -0.1433012701892219}, {
       0.19820508075688775`, -0.056698729810778024`}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.5, 0.6, 0.8], 
      LineBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.11755705045849466`, -0.16180339887498948`}}], 
      PolygonBox[{{0.05877852522924733, -0.08090169943749474}, {
       0.07710620073974728, -0.19119266148961317`}, {
       0.15800790017724203`, -0.13241413626036583`}}]}}, {
     {RGBColor[0.8, 0.2, 0.5], 
      LineBox[{{
       0.20791169081775931`, -0.9781476007338057}, {-0.693144825477394, 
       0.5825280659804146}}], 
      PolygonBox[{{-0.743144825477394, 
       0.6691306063588585}, {-0.736446095666616, 
       0.5575280659804146}, {-0.6498435552881721, 0.6075280659804145}}]}, 
     {RGBColor[0.8, 0.2, 0.5], 
      LineBox[{{-0.743144825477394, 0.6691306063588585}, {
       0.15791169081775933`, -0.8915450603553619}}], 
      PolygonBox[{{0.20791169081775931`, -0.9781476007338057}, {
       0.20121296100698127`, -0.8665450603553618}, {
       0.1146104206285374, -0.9165450603553618}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.8, 0.6, 0.2], 
      LineBox[{{-0.587785252292473, -0.8090169943749476}, \
{-0.11755705045849461`, -0.1618033988749895}}], 
      PolygonBox[{{-0.05877852522924731, -0.08090169943749476}, \
{-0.158007900177242, -0.13241413626036586`}, {-0.07710620073974724, \
-0.19119266148961317`}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.6, 0.6, 0.2], 
      LineBox[{{-0.8660254037844384, -0.5000000000000004}, \
{-0.17320508075688767`, -0.10000000000000009`}}], 
      PolygonBox[{{-0.08660254037844384, -0.050000000000000044`}, \
{-0.1982050807568877, -0.05669872981077816}, {-0.14820508075688765`, \
-0.143301270189222}}]}}, 
    {RGBColor[1, 0.4, 0.4], 
     LineBox[{{-0.9168072992225762, -0.16746050237263793`}, \
{-0.8822597739267954, -0.18565716404277371`}, {-0.843423992289808, \
-0.1891830773491721}, {-0.806169944256177, -0.1776598006837477}, \
{-0.7761606639718953, -0.15267794533192447`}, {-0.7579640023017595, \
-0.11813042003614371`}, {-0.7545084123129865, -0.07929463839915638}, \
{-0.7659613656607855, -0.04204059036552539}, {-0.7909432210126088, \
-0.012031310081243654`}, {-0.8254907463083896, 
      0.006165351588892129}, {-0.8643265279453769, 
      0.009691264895290555}, {-0.9015805759790079, -0.001832011770133854}, \
{-0.9315898562632896, -0.02681386712195709}, {-0.9945218953682733, \
-0.10452846326765423`}}], 
     PolygonBox[{{-0.9945218953682733, -0.10452846326765423`}, \
{-0.948273318775068, -0.20631780044548648`}, {-0.8853412796700844, \
-0.12860320429978936`}}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.2, 0.8, 0.7], 
      LineBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.19021130325903074`, 0.06180339887498945}}], 
      PolygonBox[{{-0.09510565162951537, 
       0.030901699437494726`}, {-0.17476045354028336`, 
       0.10935622468974714`}, {-0.20566215297777812`, 
       0.01425057306023177}}]}}, 
    {RGBColor[1, 0.6, 1], 
     LineBox[{{-0.44257343803033017`, 
      0.8201874149928808}, {-0.44920371925653646`, 
      0.7817076882414937}, {-0.4405561455668149, 
      0.7436830979178514}, {-0.4180847242534386, 
      0.7118132810975943}, {-0.38505219033514004`, 
      0.6909925773886306}, {-0.34657246358375293`, 
      0.6843622961624243}, {-0.3085766338839582, 
      0.6930098698521459}, {-0.27667805643985355`, 
      0.7154812911655222}, {-0.25585735273088983`, 
      0.7485138250838207}, {-0.24922707150468354`, 
      0.7869935518352078}, {-0.2578746451944051, 
      0.8250181421588502}, {-0.28034606650778143`, 
      0.8568879589791072}, {-0.31337860042608, 
      0.8777086626880709}, {-0.40673664307580015`, 0.913545457642601}}], 
     PolygonBox[{{-0.40673664307580015`, 
      0.913545457642601}, {-0.4892524593551903, 
      0.8381058124701458}, {-0.3958944167054701, 0.8022690175156157}}]}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425879562115126`*^9}]
}, Open  ]],

Cell["\<\
We see that 0, 3, 5, 6, 9, 10, and 12 do not have inverses. There are eight \
numbers which do have inverses modulo 15. What if we just considered those \
eight elements?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[{1,2,4,7,8,11,13,14},Inv]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"1\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"8\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.03826834323650909, -0.9076120467488713}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.08446231986207345, -0.9267462183671258}, {-0.007925633389055248, \
-0.8884778751306168}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {0.6688384379500385, 
       0.6147188279354189}}], 
      PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.6226444613244742, 0.6338529995536734}, {0.7150324145756028, 
       0.5955846563171644}}]}}, 
    {RGBColor[0.6, 0.4, 0.8], 
     LineBox[{{0.9292893218813453, 0.07071067811865481}, {0.8968331206248827, 
      0.09241885630108182}, {0.8585786437626906, 0.09998489885977785}, {
      0.8203241669004984, 0.0924188563010818}, {0.7878679656440357, 
      0.0707106781186548}, {0.7661597874616087, 0.038254476862192265`}, {
      0.7586644555810313, 4.645047007742779*^-17}, {
      0.7661597874616087, -0.03825447686219217}, {
      0.7878679656440357, -0.07071067811865471}, {
      0.8203241669004984, -0.09241885630108172}, {
      0.8585786437626906, -0.09998489885977777}, {
      0.8968331206248827, -0.09241885630108171}, {
      0.9292893218813453, -0.0707106781186547}, {1., 
      6.123233995736766*^-17}}], 
     PolygonBox[{{1., 6.123233995736766*^-17}, {0.9646446609406726, 
      0.10606601717798217`}, {0.8939339828220179, 0.03535533905932743}}]}, {
     {RGBColor[1, 0.6, 0], 
      LineBox[{{
       0.7071067811865476, -0.7071067811865475}, {-0.9076120467488713, \
-0.03826834323650914}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9267462183671258, \
-0.08446231986207349}, {-0.8884778751306168, 0.007925633389055194}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {
       0.6147188279354189, -0.6688384379500385}}], 
      PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
       0.6338529995536734, -0.6226444613244742}, {
       0.5955846563171644, -0.7150324145756028}}]}}, 
    {RGBColor[0.7, 0.8, 0.7], 
     LineBox[{{-0.6071067811865475, -0.7071067811865477}, \
{-0.5688067811865475, -0.6995067811865476}, {-0.5364067811865475, \
-0.6778067811865476}, {-0.5147067811865474, -0.6454067811865476}, \
{-0.5071067811865475, -0.6071067811865476}, {-0.5147067811865474, \
-0.5688067811865476}, {-0.5364567811865475, -0.5364067811865476}, \
{-0.5688067811865475, -0.5147067811865477}, {-0.6071067811865475, \
-0.5071067811865477}, {-0.6454067811865475, -0.5147067811865477}, \
{-0.6778067811865475, -0.5364067811865476}, {-0.6995067811865475, \
-0.5688067811865477}, {-0.7071067811865475, -0.6071067811865477}, \
{-0.7071067811865475, -0.7071067811865477}}], 
     PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6071067811865475, -0.7571067811865476}, {-0.6071067811865475, \
-0.6571067811865476}}]}, 
    {RGBColor[0.8, 0.6, 0.5], 
     LineBox[{{-0.7071067811865477, 0.6071067811865474}, {-0.6995067811865476,
       0.5688067811865474}, {-0.6778067811865476, 
      0.5364067811865474}, {-0.6454067811865476, 
      0.5147067811865473}, {-0.6071067811865476, 
      0.5071067811865474}, {-0.5688067811865476, 
      0.5147067811865473}, {-0.5364567811865477, 
      0.5364067811865474}, {-0.5147067811865477, 
      0.5688067811865474}, {-0.5071067811865477, 
      0.6071067811865474}, {-0.5147067811865477, 
      0.6454067811865474}, {-0.5364067811865476, 
      0.6778067811865474}, {-0.5688067811865477, 
      0.6995067811865474}, {-0.6071067811865477, 
      0.7071067811865474}, {-0.7071067811865477, 0.7071067811865474}}], 
     PolygonBox[{{-0.7071067811865477, 
      0.7071067811865474}, {-0.7571067811865476, 
      0.6071067811865474}, {-0.6571067811865476, 0.6071067811865474}}]}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425879599302626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
By using this smaller set of numbers, every element now has an inverse. What \
if we looked at the multiplication table of this smaller set?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[{1,2,4,7,8,11,13,14}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 
      0.8}, {0, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}}, {{
      1, 0.6, 0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {1, 0, 0}, {1, 1, 0}, {0.7, 
      0.8, 0.7}, {0, 1, 0}, {0, 1, 1}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 
      0.4, 0.8}, {1, 0, 0}, {0.7, 0.8, 0.7}, {1, 0, 1}, {1, 0.6, 0}, {1, 1, 
      0}, {0, 1, 0}, {0, 1, 1}}, {{0, 1, 0}, {0, 1, 0}, {1, 1, 0}, {1, 0, 
      1}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 0, 
      0}}, {{1, 0, 0}, {1, 0, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0, 1, 1}, {
      0.6, 0.4, 0.8}, {1, 0, 1}, {1, 1, 0}, {0, 1, 0}}, {{0, 1, 1}, {0, 1, 
      1}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {1, 0, 1}, {0.7, 0.8, 0.7}, {1, 
      0, 0}, {0.6, 0.4, 0.8}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {0, 1, 0}, {
      0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{
      1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 
      1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7,
       0.8, 0.7}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.4425879629927626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
One again, we can see some of the patterns in this table that we have seen \
before. Notice that this is a Latin square\[LongDash]every row and column \
contains each of the eight numbers. In fact, we can see the symmetry in this \
table which indicates that the multiplication is communative. Thus, we see \
that \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.42749208834375*^9}],

Cell[TextData[{
 "    1)   For any two numbers ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in \n         {1, 2, 4, 7, 8, 11, 13, 14}, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is in \n         that set. \n\n    2)   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "\[CenterDot]", "z"}], " ", 
    "=", " ", 
    RowBox[{"x", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"y", "\[CenterDot]", "z"}], ")"}]}]}], TraditionalForm]]],
 " for any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ".\n\n    3)   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "1"}], " ", "=", " ", "x"}], 
   TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", " ", "\[CenterDot]", " ", "x"}], " ", "=", " ", "x"}], 
   TraditionalForm]]],
 "  for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".\n\n    4)   For any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", there is a ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " such that \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", "1"}], 
   TraditionalForm]]],
 ".\n\n    5)   For any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", 
    RowBox[{"y", " ", "\[CenterDot]", " ", "x"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42749208834375*^9, 3.4274920901875*^9}}],

Cell[TextData[{
 "EXPERIMENT:\nTry this procedure working with different bases, such as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "8"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "9"}], TraditionalForm]]],
 ": First type in ",
 StyleBox["DefMultMod[",
  FontWeight->"Bold"],
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 StyleBox["]",
  FontWeight->"Bold"],
 " which defines the period as the operator. Then use make a circle graph of \
the ",
 StyleBox["Inv",
  FontWeight->"Bold"],
 " function to see which numbers have inverses modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Finally, make a multiplication table of those numbers. Will this table \
always form a Latin square? "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879663052626`*^9}],

Cell["\<\
If the Latin square pattern always seems to appear for the invertible \
numbers, can we prove that this pattern will always appear?  \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4362731117737503`*^9, 3.4425879675240126`*^9}],

Cell[TextData[{
 "PROPOSITION 1.4\nFor ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " a positive integer greater than 1, let the dot ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "\[CenterDot]", ")"}], TraditionalForm]]],
 " denote multiplication modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be the set of all non-negative numbers less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " which have inverses modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Then the set ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has the following properties: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427489694015625*^9, 3.427489695234375*^9}, 
   3.427492073703125*^9, 3.4425879689615126`*^9}],

Cell[TextData[{
 "   1) For any two numbers ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\n   2) ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], ")"}], " ", 
     "\[CenterDot]", " ", "z"}], " ", "=", " ", 
    RowBox[{"x", " ", "\[CenterDot]", " ", 
     RowBox[{"(", 
      RowBox[{"y", " ", "\[CenterDot]", " ", "z"}], ")"}]}]}], 
   TraditionalForm]]],
 " for any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ".\n\n   3) ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "1"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"1", " ", "\[CenterDot]", " ", "x"}], " ", "=", " ", "x"}]}], 
   TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".\n\n   4) For any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " which is in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", there is a ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", "1"}], 
   TraditionalForm]]],
 ".\n\n   5) For any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", 
    RowBox[{"y", " ", "\[CenterDot]", " ", "x"}]}], TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427489694015625*^9, 3.427489695234375*^9}, {
   3.427492073703125*^9, 3.4274920756875*^9}, {3.42844412484375*^9, 
   3.42844414828125*^9}, 3.4425879706177626`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Properties 2, 3, and 5 come from the properties of standard multiplication.\
\n\nProperty 1 comes from proposition 1.2. If ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are both invertible, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is an inverse of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is ",
 "invertible",
 " modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".\n\nProperty 4 seems obvious, since if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is invertible modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "=", " ", 
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " making ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", "1"}], 
   TraditionalForm]]],
 ". But we must check that ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is also invertible, which it is since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "=", " ", "x"}], TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427489713453125*^9, 3.427489713625*^9}, {
   3.4362730330550003`*^9, 3.4362730728987503`*^9}, {3.4362731139456253`*^9, 
   3.4362731160862503`*^9}, 3.4425879727896376`*^9}]
}, Open  ]],

Cell[TextData[{
 "Although we have found that the ",
 "invertible",
 " elements modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " will always have a multiplication table which forms a Latin square, we \
have only hinted at which numbers are invertible modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".\n\nWe saw that the numbers 0, 2, 4, 5, 6, and 8 are not invertible modulo \
10. Note that these numbers all have a prime factor in common with 10:  0, 2, \
4, 6, 8, and 10 are multiples of 2, whereas 0, 5, and 10 are multiples of 5. \
The remaining numbers, 1 ,3 ,7, and 9, have no prime factors in common with \
10. Apparently, the prime factors determine whether a number in invertible. \
In the next section we will review some basic properties of prime \
factorizations, which will be needed thoughout the course. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4362731026800003`*^9, 3.4362731179143753`*^9}, 
   3.4425879744302626`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Prime Factorizations ", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Many properties of modular arithmetic come from the factorization of the \
positive integers. In this section we will look at some of the properties of \
integers that are needed for modular arithmetic.\n\nWe will denote the set of \
all integers,\n\n        { \[Ellipsis], \[Dash]3, \[Dash]2, \[Dash]1, 0, 1, \
2, 3, \[Ellipsis] }\n\nby the stylized letter \[DoubleStruckCapitalZ], which \
stands for the German \"Zahl\" meaning \"number.\" Rather than saying \"",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is an integer,\" we can succintly say \"",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is in \[DoubleStruckCapitalZ].\" Many of the properties of factorizations \
refer only to positive integers, which are denoted ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalZ]", "+"], TraditionalForm]]],
 ".\n\nWe define a ",
 StyleBox["prime",
  FontSlant->"Italic"],
 " as an integer that has only two positive factors: one and itself. This \
definition actually allows negative numbers, such as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "5"}], TraditionalForm]]],
 ", to be prime.  Although this may seem to be a non-standard definition, it \
agrees with the generalized definition of primes defined in chapters 10 and \
12.  The numbers 1 and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "1"}], TraditionalForm]]],
 " are ",
 StyleBox["not",
  FontSlant->"Italic"],
 " considered to be prime. The fundamental property of prime numbers is that \
any integer greater than 1 can be uniquely factored into a product of \
positive primes.  The uniqueness aspect of this statement will be proven in \
chapter 12, in a much more generalized context.  We will begin by proving \
that every large number has at least one prime factor. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427490051703125*^9, 3.427490071296875*^9}, {
   3.4362731661331253`*^9, 3.4362731695550003`*^9}, 3.4425879766490126`*^9}],

Cell["\<\
LEMMA 1.1:   

Every number greater than 1 has a prime factor.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42749007603125*^9, 3.427490077796875*^9}, 
   3.4425879811646376`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that some number greater than 1 does not have a prime factor.  Then \
there is a smallest such number, called ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".  Then ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is not prime, otherwise ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " would have a prime factor.  Then by definition, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " must have a positive divisor besides 1 and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", say ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ".  Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "<", " ", "m", " ", "<", " ", "n"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " was the ",
 StyleBox["smallest",
  FontSlant->"Italic"],
 " number greater than 1 without a prime factor,  ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " must have a prime factor, say ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ".  Then ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is also a prime factor of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", so we have a contradiction.  Therefore, every number greater than 1 has a \
prime factor. "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425879834302626`*^9}]
}, Open  ]],

Cell[TextData[{
 "The proof of lemma 1.1 introduces an important strategy for proving \
theorems. Notice that to prove that every number greater than 1 had a prime \
factor, we assumed just the opposite. It was as if we admitted defeat from \
the very beginning! Yet from this we were able to reach a conclusion that was \
absurd\[LongDash]a number without a prime factor which did have a prime \
factor. This strategy is known as ",
 StyleBox["reductio ad absurdum",
  FontSlant->"Italic"],
 ", which is Latin for \"reduce to the absurd.\" We assume what we are trying \
to prove is actually false, and proceed logically until we reach a \
contradiction. The only explanation would be that the assumption was wrong, \
which proves the original statement.  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42749017209375*^9, 3.427490172234375*^9}, {
   3.428444179921875*^9, 3.428444196703125*^9}, 3.4425879846490126`*^9}],

Cell[TextData[{
 "In problem 1.33, you will be asked to use lemma 1.1 to prove that every \
positive integer can be written as a product of primes.  For example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"420", " ", "=", " ", 
    RowBox[{
     RowBox[{"21", "\[CenterDot]", "20"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
      "3", "\[CenterDot]", "7", "\[CenterDot]", "2", "\[CenterDot]", "2", 
       "\[CenterDot]", "5"}], " ", "=", " ", 
      RowBox[{
       SuperscriptBox["2", "2"], "\[CenterDot]", "3", "\[CenterDot]", "5", 
       "\[CenterDot]", "7"}]}]}]}], TraditionalForm]]],
 ". We can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " compute to prime factorization for us, using the command FactorInteger: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4274901338125*^9, 3.427490156078125*^9}, {
   3.428444214109375*^9, 3.42844426428125*^9}, {3.4362734461331253`*^9, 
   3.4362734466643753`*^9}, 3.4425879861021376`*^9}],

Cell[CellGroupData[{

Cell["FactorInteger[420]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"7", ",", "1"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4362734639300003`*^9, 3.4425879883521376`*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s output is rather cryptic, so it takes a bit of interpretation to \
understand the answer. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " returns a sequence of ordered pairs, like ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"2", ",", "2"}], "}"}], TraditionalForm]]],
 ". The first number in the pair is the prime, and the second number \
represents the number of times that prime occurred. \n\nInteger factorization \
is a fundamentally difficult problem even with modern technology. As a \
result, one can easily type in a number that neither ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", nor any other computer program, will be able to factor in anything short \
of an astronomical length of time. As long as the integers are less than \
about 40 digits long, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " should not have any trouble factoring them. Ironically, the difficulty in \
factorization will later turn to our advantage!"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42749017209375*^9, 3.427490172234375*^9}, 
   3.428444179921875*^9, 3.4425879898521376`*^9}],

Cell[TextData[{
 "The prime factorizations leads to an important question. Is there a largest \
prime number? The Greek mathmatican Euclid answered this question using ",
 StyleBox["reductio ad absurdum",
  FontSlant->"Italic"],
 " in the third century B.C. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42749017209375*^9, 3.427490172234375*^9}, {
   3.428444179921875*^9, 3.42844420334375*^9}, 3.4425879914146376`*^9}],

Cell["\<\
THEOREM 1.1:  EUCLID'S PRIME NUMBER THEOREM 

There are an infinite number of primes. \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4274902505*^9, 3.427490251765625*^9}, {
  3.4425879940865126`*^9, 3.4425879955552626`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that there are only a finite number of prime numbers. Label these \
prime numbers\n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["p", "1"], " ", "=", " ", "2"}], ",", " ", 
    RowBox[{
     SubscriptBox["p", "2"], " ", "=", " ", "3"}], ",", " ", 
    RowBox[{
     SubscriptBox["p", "3"], " ", "=", " ", "5"}], ",", " ", "\[Ellipsis]", 
    ",", " ", 
    SubscriptBox["p", "n"]}], TraditionalForm]]],
 "\n\nNow consider the number \n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
      "2", "\[CenterDot]", "3", "\[CenterDot]", "5", "\[CenterDot]", "7", 
       "\[CenterDot]", "11", "\[CenterDot]", "13", "\[CenterDot]", 
       "\[CenterDot]", "\[CenterDot]", " ", 
       SubscriptBox["p", "n"]}], " ", ")"}], " ", "+", " ", "1."}]}], 
   TraditionalForm]]],
 "\n\nThis number is odd, so it cannot be divisible by 2. Likewise, ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " is one more than a multiple of 3, so it is not divisible by 3. In this way \
we see that ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " is not divisible by any of the prime numbers. But this is ridiculous, \
since ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " must have a prime factor by lemma 1.1. Thus, the original assumption that \
there is a largest prime number is false, so there are an infinite number of \
prime numbers. "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42844427846875*^9, 3.42844427859375*^9}, {
   3.4362737178050003`*^9, 3.4362737178050003`*^9}, 3.4425879978833876`*^9}]
}, Open  ]],

Cell[TextData[{
 "We define the ",
 StyleBox["greatest common divisor",
  FontSlant->"Italic"],
 " (GCD) of two numbers to be the largest\ninteger which divides both of the \
numbers.  If the greatest common divisor is 1, \nthis means that there are no \
prime factors in common.  We say the numbers are \n",
 StyleBox["coprime",
  FontSlant->"Italic"],
 " in this case.  We can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to test if two numbers are coprime without having to factor them: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427490572015625*^9, 3.42749063146875*^9}, 
   3.4425879988365126`*^9}],

Cell[CellGroupData[{

Cell["\<\
GCD[138153809229555633320990299469,
    145730407810127891189961221324529]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.4425880011021376`*^9}]
}, Open  ]],

Cell[TextData[{
 "So ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can quickly see that these two numbers are coprime, even though the \
numbers are too large to find the prime factorization.\n\nThe greatest common \
divisor can be found in another way, as seen in the following theorem. The \
proof of a theorem is usually much more difficult than propositions, \
illustrating new techniques. Go ahead and read the proof of this theorem \
without trying to prove it on your own. It is still important that you \
understand the techniques involved."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "THEOREM 1.2:  THE GREATEST COMMON DIVISOR THEOREM\n\n   Given two positive \
integers ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", the greatest common divisor of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is the smallest positive integer which can be expressed in the form\n\n    \
            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", " ", "x"}], " ", "+", " ", 
    RowBox[{"v", " ", "y"}]}], TraditionalForm]]],
 "\n\nwith ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " being integers. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42749067484375*^9, 3.42749067525*^9}, 
   3.4425880052740126`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " denote the set of all positive numbers that can be expressed in the form  \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "\[CenterDot]", "x"}], " ", "+", " ", 
    RowBox[{"v", "\[CenterDot]", "y"}]}], TraditionalForm]]],
 ".\nNote that both ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " can be written in the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "\[CenterDot]", "x"}], " ", "+", " ", 
    RowBox[{"v", "\[CenterDot]", "y"}]}], TraditionalForm]]],
 ", so we can consider the smallest positive number ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " that can be written in the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "\[CenterDot]", "x"}], " ", "+", " ", 
    RowBox[{"v", "\[CenterDot]", 
     RowBox[{"y", "."}]}]}], TraditionalForm]]],
 " Note that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"GCD", "(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 " is a factor of both ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"GCD", "(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 " must be a factor of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".\n\nNext, consider the number\n                                           \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Congruent]", " ", 
    RowBox[{"x", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 "   with   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", " ", "\[LessEqual]", " ", "k", " ", "<", " ", "n"}], 
   TraditionalForm]]],
 ".\n                                          \nThen ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "=", " ", 
    RowBox[{"x", " ", "+", " ", 
     RowBox[{"n", " ", "r"}]}]}], TraditionalForm]]],
 " for some number ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", 
    RowBox[{
     RowBox[{"u", " ", "x"}], " ", "+", " ", 
     RowBox[{"v", " ", "y"}]}]}], TraditionalForm]]],
 " for some numbers ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "."}], TraditionalForm]]],
 " Thus, \n                                                   \n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"k", " ", "=", " ", 
      RowBox[{
       RowBox[{"x", " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"u", " ", "x"}], " ", "+", " ", 
           RowBox[{"v", " ", "y"}]}], ")"}], " ", "r"}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "+", " ", 
           RowBox[{"r", " ", "u"}]}], ")"}], " ", "x"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"r", " ", "v"}], ")"}], " ", "y"}]}]}]}], ","}]}], 
   TraditionalForm]]],
 "\n\nso ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". But since ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the smallest positive integer in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " cannot be equivalent ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"Mod", " ", "n"}], ")"}], " "}], TraditionalForm]]],
 "to any number less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", other than ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 ".  Thus,\n\n                                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "\[Congruent]", " ", 
    RowBox[{"0", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 ". \n            \nTherefore, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a divisor of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".  By a similar reasoning, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is also a divisor of ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a common divisor of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and since the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"GCD", "(", 
    RowBox[{"x", ",", " ", "y"}], ")"}], TraditionalForm]]],
 " is in turn a divisor of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " must be equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"GCD", "(", 
    RowBox[{"x", ",", " ", "y"}], ")"}], TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427490703265625*^9, 3.427490719734375*^9}, {
   3.4274907569375*^9, 3.427490783671875*^9}, 3.4425880077896376`*^9}]
}, Open  ]],

Cell[TextData[{
 "We can find the numbers ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " from the greatest common divisor theorom (1.2) as follows:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427490835484375*^9, 3.42749087421875*^9}, {
  3.4414648164171247`*^9, 3.4414648178233747`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExtendedGCD", "[", 
  RowBox[{"105", ",", "196"}], "]"}]], "Input",
 CellChangeTimes->{{3.4274908778125*^9, 3.427490888578125*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"7", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "13"}], ",", "7"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.427490892453125*^9, 3.427490910875*^9}, 
   3.4362738860081253`*^9, 3.4425880101490126`*^9}]
}, Open  ]],

Cell[TextData[{
 "This says that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"GCD", 
     RowBox[{"(", 
      RowBox[{"105", ",", "196"}], ")"}]}], " ", "=", " ", "7"}], 
   TraditionalForm]]],
 ", but also that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "=", " ", 
    RowBox[{"-", "13"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", " ", "=", " ", "7"}], TraditionalForm]]],
 " will satisfy theorem 1.2, so that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "13"}], ")"}], "\[CenterDot]", "105"}], " ", "+", " ", 
     RowBox[{"7", "\[CenterDot]", "196"}]}], " ", "=", " ", "7"}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427490998609375*^9, 3.427491094328125*^9}}],

Cell[TextData[{
 "In the last section we found that the invertible elements modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " had many of the properties of Terry's dance steps.  We now can determine \
which numbers less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " have a multiplicative inverse modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4362739093675003`*^9, 3.4362739473362503`*^9}, {
   3.4362740301487503`*^9, 3.4362740431800003`*^9}, 3.4425880185396376`*^9}],

Cell[TextData[{
 "PROPOSITION 1.5\nLet ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " be in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalZ]", "+"], TraditionalForm]]],
 ". Then for ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " between 0 and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " has a multiplicatative inverse modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " if, and only if, ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.427491556171875*^9, 3.4425880210552626`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are not coprime, then there is a common prime factor ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". In order for ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " to have a multiplicative inverse, there must be a ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " such that\n\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "\[Congruent]", " ", 
    RowBox[{"1", "   ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"Mod", " ", "n"}], ")"}], "."}]}]}], TraditionalForm]]],
 "\n                       \nBut this means that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "y"}], " ", "=", " ", 
    RowBox[{"1", " ", "+", " ", 
     RowBox[{"w", " ", "n"}]}]}], TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 ". But this is impossible, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "y"}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", but ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "+", " ", 
    RowBox[{"w", " ", "n"}]}], TraditionalForm]]],
 " is one more than a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ".\n\nNow suppose that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are coprime. By the greatest common divisor theorem (1.3.3), there are ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " such that\n\n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"u", " ", "x"}], " ", "+", " ", 
     RowBox[{"v", " ", "n"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"GCD", "(", 
      RowBox[{"x", ",", "n"}], ")"}], " ", "=", " ", "1"}]}], 
   TraditionalForm]]],
 ".\n       \nBut then \n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", " ", "x"}], " ", "=", " ", 
    RowBox[{"1", " ", "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "v"}], ")"}], " ", "n"}]}]}], TraditionalForm]]],
 ",\n                         \nand so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "\[CenterDot]", "x"}], " ", "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"Mod", " ", "n"}], ")"}], "."}]}]}], TraditionalForm]]],
 " Hence, ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is a multiplicative inverse of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428444298953125*^9, 3.4284443024375*^9}, 
   3.4425880226802626`*^9}]
}, Open  ]],

Cell["\<\
There is another property of modular arithmitic involving coprime numbers 
that will be used often throughout this course, known to the ancient Chinese. \
 \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427491670203125*^9, 3.427491689765625*^9}, 
   3.427491880109375*^9, 3.4425880248521376`*^9}],

Cell[TextData[{
 "THEOREM 1.3:  THE CHINESE REMAINDER THEOREM \nIf ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalZ]", "+"], TraditionalForm]]],
 "  are coprime, then given any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in \[DoubleStruckCapitalZ], there is a unique ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " in \[DoubleStruckCapitalZ] such that\n\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"0", " ", "\[LessEqual]", " ", "k", " ", "<", " ", 
     RowBox[{"u", "\[CenterDot]", "v"}]}], ","}], TraditionalForm]]],
 "\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", " ", "\[Congruent]", " ", 
     RowBox[{"x", "  ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", "u"}], ")"}]}]}], ",", " "}], TraditionalForm]]],
 "\n  and             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Congruent]", " ", 
    RowBox[{"y", "  ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"Mod", " ", "v"}], ")"}], "."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427491670203125*^9, 3.427491689765625*^9}, {
   3.427491880109375*^9, 3.42749188434375*^9}, {3.4362741814456253`*^9, 
   3.4362741815706253`*^9}, {3.441470486953125*^9, 3.441470493296875*^9}, 
   3.4425880260708876`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Ironically, the way that we will show that there is such a number is to \
show that there cannot be more than one such number!  \n\nSuppose we have two \
different numbers, ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ", which satisfy the above conditions.  Then \n       \n                     \
  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", " ", "-", "q"}], " ", "\[Congruent]", " ", 
    RowBox[{"0", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "u"}], ")"}]}]}], TraditionalForm]]],
 "    and     \n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", "-", "q"}], " ", "\[Congruent]", " ", 
    RowBox[{"0", "  ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"Mod", " ", "v"}], ")"}], "."}]}]}], TraditionalForm]]],
 "\n                       \nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "-", " ", "q"}], TraditionalForm]]],
 " must be a multiple of both ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 ".  But since ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " are coprime, the least common multiple of ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "v"}], TraditionalForm]]],
 ".  \nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "-", " ", "q", " "}], TraditionalForm]]],
 "is a multiple of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "v"}], TraditionalForm]]],
 ".\n\nHowever, both ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " are less then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "v"}], TraditionalForm]]],
 ".  So the only way this is possible is for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", " ", "-", "q"}], " ", "=", " ", "0"}], TraditionalForm]]],
 ", which contradicts our assumption that ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " were distinct solutions.\n\nThus, we have shown that there cannot be more \
than one value for ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ".  But how does that help us prove that there must be such a ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 "?  Notice that for any number ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ",\n\n                                              ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "  ", 
    RowBox[{"(", 
     RowBox[{"Mod", " ", "u"}], ")"}]}], TraditionalForm]]],
 "\n                                              \ncan have ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " possible values, from 0 to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"u", "-", "1"}], ")"}], TraditionalForm]]],
 ".  Also,\n\n                                             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "  ", 
    RowBox[{"(", 
     RowBox[{"Mod", " ", "v"}], ")"}]}], TraditionalForm]]],
 "\n                                             \ncan have ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " possible values, from 0 to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"v", "-", "1"}], ")"}], TraditionalForm]]],
 ".  Thus, for any ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", there are only ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "v"}], TraditionalForm]]],
 " possible values for the ordered pair\n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{"k", "  ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", "u"}], ")"}]}], ",", "   ", 
     RowBox[{"k", "  ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", "v"}], ")"}]}]}], "  ", ")"}], TraditionalForm]]],
 ".\n                        \nWhat is shown above is that no two values of \
",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " between 0 and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "\[CenterDot]", "v"}], "-", "1"}], TraditionalForm]]],
 " can give the same ordered pair. But there are exactly ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "v"}], TraditionalForm]]],
 " such values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ".\n\nImagine having ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "v"}], TraditionalForm]]],
 " \"pigeonholes\" labeled by these ordered pairs. If one has ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "v"}], TraditionalForm]]],
 " pigeons and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "v"}], TraditionalForm]]],
 " pigeonholes, and each pigeon goes into a pigeonhole with no two pigeons \
going into the same hole, then every hole must be filled!  \n \nIn the same \
way, since each of the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "v"}], TraditionalForm]]],
 " possible values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " produces one of the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "v"}], TraditionalForm]]],
 " possible ordered pairs, and no two ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 "'s can produce the same ordered pair, each ordered pair must be produced by \
some (unique) value of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ".  And this is what we wanted to prove."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427491783828125*^9, 3.427491804*^9}, {
   3.428444323484375*^9, 3.428444342*^9}, {3.4362742756487503`*^9, 
   3.4362743348050003`*^9}, {3.4362744345550003`*^9, 
   3.4362744436175003`*^9}, {3.44147050453125*^9, 3.441470588578125*^9}, {
   3.44147099190625*^9, 3.44147099428125*^9}, 3.4425880325552626`*^9}]
}, Open  ]],

Cell[TextData[{
 "This proof introduced a second technique to prove theorems, called the ",
 StyleBox["pigeonhole principle",
  FontSlant->"Italic"],
 ". Whenever we have a mapping from ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects into ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " other objects, and there are no duplications, then there must be a \
one-to-one correspondence between the two sets of objects. Note that a \
formula for this one-to-one correspondence was not derived in the process. We \
have proven that the correspondence exists, but the actual formula may in \
fact be overwhelmingly complex.\n\nEven though you were not asked to try to \
prove the theorems in this section, you should open the subsection and study \
the proofs. Soon you will be writing proofs like these yourself!  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425880360396376`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Definition of a Group", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
In this notebook, we have seen several different ways of combining numbers or \
dance steps. Yet, all of the different \"products\" had many properties in \
common. We are now ready to try to generalize these examples. Our strategy is \
to define a \"group\" abstractly by requiring the same patterns we observed \
to continue. Thus, we make the following definition based upon the first four \
properties we saw in all of our examples. \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4425880383208876`*^9}],

Cell[TextData[{
 "DEFINITION\nA ",
 StyleBox["group",
  FontSlant->"Italic"],
 " is a set ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " together with an operation ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "\[CenterDot]", ")"}], TraditionalForm]]],
 " such that the following four properties hold:\n\n   1)  (closure)  For any \
",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], TraditionalForm]]],
 " is \n                   in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\n   2)  (identity) There exists a member ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "  \n                   which has the property that\n                   \n   \
                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"e", " ", "\[CenterDot]", " ", "x"}], "  ", "=", "  ", 
    RowBox[{
     RowBox[{"x", " ", "\[CenterDot]", " ", "e"}], "  ", "=", "  ", "x"}]}], 
   TraditionalForm]]],
 "\n                   \n                   for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\n   3)  (inverse)  For every ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", there exists a \n                   ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", called the ",
 StyleBox["inverse",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", \n                   such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], "  ", "=", "  ", "e"}], 
   TraditionalForm]]],
 ".\n        \n   4)  (associative law)  For any ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", \n        \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", "\[CenterDot]", " ", "b"}], ")"}], " ", 
     "\[CenterDot]", " ", "c"}], "  ", "=", " ", 
    RowBox[{"a", " ", "\[CenterDot]", " ", 
     RowBox[{"(", 
      RowBox[{"b", " ", "\[CenterDot]", " ", "c"}], ")"}]}]}], 
   TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427492782828125*^9, 3.427492863765625*^9}, {
   3.427492898671875*^9, 3.427492910171875*^9}, 3.4425880417271376`*^9}],

Cell[TextData[{
 "These are the same four properties that we observed from Terry's dance \
steps, so Terry's dance steps gives us the first example of a group. This \
group is more commonly refered to as the group of symmetries of a triangle, \
abbreviated by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "3"], TraditionalForm]]],
 ".\n\nThe members of the group, whether they are numbers, dance steps, or \
even ordered pairs, are called the ",
 StyleBox["elements",
  FontSlant->"Italic"],
 " of the group. The element ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " that satisfies property 2 is commonly called the ",
 StyleBox["identity element",
  FontSlant->"Italic"],
 " of the group. \n\nThe mathematical notation for an element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " to be in a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is \n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "\[Element]", " ", "G"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.42750375021875*^9, {3.4362747034925003`*^9, 3.4362747060706253`*^9}, 
   3.4425880434302626`*^9}],

Cell[TextData[{
 "Thus, the symbol \[Element] means \"is an element of.\"\n\nSince \
propositions 1.1, 1.2, and 1.3 only used these four properties, the proofs \
are valid for all groups, using the identity element ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " in place of the dance step ",
 StyleBox["Stay",
  FontWeight->"Bold"],
 ".\n                                                           \nOther \
examples of groups come from modular arithmetic. For ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalZ]", "+"], TraditionalForm]]],
 ", we considered the elements\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", " ", "1", ",", " ", "2", ",", " ", "...", ",", " ", 
     RowBox[{"n", "-", "1"}]}], "}"}], TraditionalForm]]],
 ",\n\nwith the opperator ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "\[CenterDot]", ")"}], TraditionalForm]]],
 " being the sum modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". This group will be denoted ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". For example, the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "10"], TraditionalForm]]],
 " is given by:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4275037560625*^9, 3.427503759140625*^9}, {
  3.4362747115550003`*^9, 3.4362747131487503`*^9}, {3.4362747542581253`*^9, 
  3.4362747545081253`*^9}, {3.441471137734375*^9, 3.44147116078125*^9}, {
  3.4425880459927626`*^9, 3.4425880468833876`*^9}}],

Cell[CellGroupData[{

Cell["\<\
DefSumMod[10]
MultTable[{0,1,2,3,4,5,6,7,8,9}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6,
       0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 
      0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 
      1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}, {{1, 1, 
      1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.4425880497271376`*^9}]
}, Open  ]],

Cell[TextData[{
 "We also considered having the opperator ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "\[CenterDot]", ")"}], TraditionalForm]]],
 " denoted the product modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and considered only the set of numbers less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " which are coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Proposition 1.2.1 shows that this set also has the 4 properties of \
groups. We will refer to this group by ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "n", "*"], TraditionalForm]]],
 ". For example, the group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 " has 8 elements, with the following multiplication table."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
DefMultMod[15] 
MultTable[{1,2,4,7,8,11,13,14}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.4425880538833876`*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 
      0.8}, {0, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}}, {{
      1, 0.6, 0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {1, 0, 0}, {1, 1, 0}, {0.7, 
      0.8, 0.7}, {0, 1, 0}, {0, 1, 1}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 
      0.4, 0.8}, {1, 0, 0}, {0.7, 0.8, 0.7}, {1, 0, 1}, {1, 0.6, 0}, {1, 1, 
      0}, {0, 1, 0}, {0, 1, 1}}, {{0, 1, 0}, {0, 1, 0}, {1, 1, 0}, {1, 0, 
      1}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 0, 
      0}}, {{1, 0, 0}, {1, 0, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0, 1, 1}, {
      0.6, 0.4, 0.8}, {1, 0, 1}, {1, 1, 0}, {0, 1, 0}}, {{0, 1, 1}, {0, 1, 
      1}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {1, 0, 1}, {0.7, 0.8, 0.7}, {1, 
      0, 0}, {0.6, 0.4, 0.8}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {0, 1, 0}, {
      0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{
      1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 
      1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7,
       0.8, 0.7}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.4425880552740126`*^9}]
}, Open  ]],

Cell[TextData[{
 "The groups ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "n", "*"], TraditionalForm]]],
 " had an additional property\[LongDash]the multiplication table were \
symmetric about the northwest to southeast diagonal. Not all groups have this \
property, but those that do are important enough to give such groups a \
special name.\n                                              \nDEFINITION:\nA \
group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is ",
 StyleBox["abelian",
  FontSlant->"Italic"],
 " (or ",
 StyleBox["commutative",
  FontSlant->"Italic"],
 ") if  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "x"}]}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", ",", " ", 
    RowBox[{"y", "  ", "\[Element]", "  ", "G"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4425880639146376`*^9, 3.4425880639146376`*^9}}],

Cell[TextData[{
 "Although these definitions appear ad hoc, the four properties of groups \
appear in many different aspects of mathematics. Here are some examples of \
groups that appear on other contexts besides group theory:\n\nExample 1.1:\n\
Consider the group consisting of all of the integers, \
\[DoubleStruckCapitalZ]. Our binary operation would be the sum of the two \
numbers. It is trivial to see that this satisfies all 4 properties, with 0 as \
the identity, and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "x"}], TraditionalForm]]],
 " as the inverse of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". This is obviously an abelian group, since \n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", " ", "+", " ", "b"}], " ", "=", " ", 
    RowBox[{"b", " ", "+", " ", "a"}]}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " in \[DoubleStruckCapitalZ].\n\nExample 1.2:  \nNext, we could consider the \
group of all rational numbers, which is denoted by \[DoubleStruckCapitalQ]. \
(This is easy to remember, since \[DoubleStruckCapitalQ] stands for \
Quotient.) Our binary operation would still be addition, so the identity \
element would still be 0, and this would also be an abelian group. \n\n\
Example 1.3:\n Consider the group of all rational numbers except for 0. We \
will refer to this group as ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalQ]", "*"], TraditionalForm]]],
 ". Our group operation would be multiplication instead of addition. The \
identity element is now 1, and the inverse of an element is the reciprocal. \
Again, this is an abelian group.\n\nExample 1.4:  \nConsider the set of all \
",
 StyleBox["linear",
  FontSlant->"Italic"],
 " functions of the form \n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"m", " ", "x"}], " ", "+", " ", "b"}]}], TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", ",", " ", 
    RowBox[{"b", " ", "\[Epsilon]", " ", "\[DoubleStruckCapitalR]"}], ",", 
    "  ", 
    RowBox[{"m", " ", "\[NotEqual]", " ", "0"}]}], TraditionalForm]]],
 ". \n\n(The fancy \[DoubleStruckCapitalR] here represents the real numbers.) \
We multiply two linear functions together by function composition.  That is, \
if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"m", " ", "x"}], " ", "+", " ", "b"}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"n", " ", "x"}], " ", "+", " ", "c"}]}], TraditionalForm]]],
 ", then\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "\[CenterDot]", "g"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"g", "(", 
      RowBox[{"f", "(", "x", ")"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"n", "(", 
        RowBox[{
         RowBox[{"m", " ", "x"}], " ", "+", " ", "b"}], ")"}], " ", "+", " ", 
       "c"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"m", " ", "n"}], ")"}], " ", "x"}], " ", "+", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"n", " ", "b"}], " ", "+", " ", "c"}], ")"}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nwhich is again a linear function. Note that in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[CenterDot]", "g"}], TraditionalForm]]],
 ", we do ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " first, then ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ", so it appears reversed in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", 
    RowBox[{"f", "(", "x", ")"}], ")"}], TraditionalForm]]],
 ".",
 Cell[BoxData[
  FormBox[
   RowBox[{Cell[""], Cell[""]}], TraditionalForm]]],
 " We can find the inverse of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " as well:\n  \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["f", 
      RowBox[{"-", "1"}]], "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      FractionBox["1", "m"], "x"}], " ", "-", " ", 
     FractionBox["b", "m"]}]}], TraditionalForm]]],
 "\n\nwhich is also a linear function. The identity function is just ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"e", "(", "x", ")"}], " ", "=", " ", "x"}], TraditionalForm]]],
 ". The associativity property of function composition completes the \
requirements for the set of all linear functions to constitute a group.  \n\n\
Is this group commutative?  Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"2", "x"}], "+", "3"}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"3", "x"}], "+", "2"}]}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", 
    RowBox[{"g", "(", "x", ")"}], ")"}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"2", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"3", "x"}], "+", "2"}], ")"}]}], "+", "3"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"6", "x"}], " ", "+", " ", "7"}]}], TraditionalForm]]],
 ", whereas ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", 
     RowBox[{"f", "(", "x", ")"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"3", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", "x"}], "+", "3"}], ")"}]}], "+", "2"}], "=", 
     RowBox[{
      RowBox[{"6", "x"}], " ", "+", " ", "11"}]}]}], TraditionalForm]]],
 ". So this is not a commutative group.\n\nThese last four groups have an \
infinite number of elements, so we unfortunately cannot construct \
multiplication tables for them. To distinquish the groups which have \
multiplication tables, we make the following definition:\n\nDEFINITION 1.5:\n\
The number of elements in a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is called the ",
 StyleBox["order",
  FontSlant->"Italic"],
 " of the group, and is denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "G", "|"}], TraditionalForm]]],
 ". \nIf ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is has an infinite number of elements, we say ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "G", "|"}], " ", "=", " ", "\[Infinity]"}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427503881953125*^9, 3.42750402375*^9}, {
  3.4284443868125*^9, 3.428444465890625*^9}, {3.42844451359375*^9, 
  3.42844452584375*^9}, {3.441471303296875*^9, 3.441471338375*^9}, {
  3.4414713696875*^9, 3.441471455171875*^9}}],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is has an infinite number of elements, we say \n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "G", "|"}], " ", "=", " ", "\[Infinity]"}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.428444572359375*^9}],

Cell[TextData[{
 "Examples 1.1 through 1.4 have infinite order, and hence we cannot form \
multiplication tables for these groups. On the opposite end of the spectrum, \
what is the ",
 StyleBox["smallest",
  FontSlant->"Italic"],
 " possible group? The third group property states that there must be an \
identity element in the group. So the smallest group is the following:\n\n\
Example 1.5:  \nConsider the group containing just the identity element, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "e", "}"}], TraditionalForm]]],
 ". Why is this a group?  \n\nWe know that since ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " is an identity, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"e", " ", "\[CenterDot]", " ", "e"}], " ", "=", " ", "e"}], 
   TraditionalForm]]],
 ". It does contain an identity element, and ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " has an inverse, namely ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ". We can check associativity by noting that\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", " ", 
      RowBox[{"e", " ", "\[CenterDot]", " ", "e"}], " ", ")"}], " ", 
     "\[CenterDot]", " ", "e"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"e", " ", "\[CenterDot]", " ", 
      RowBox[{"(", " ", 
       RowBox[{"e", " ", "\[CenterDot]", " ", "e"}], " ", ")"}]}], " ", "=", 
     " ", "e"}]}], TraditionalForm]]],
 ".\n\nWe can give a multiplication table of this group by executing the \
following: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427504008703125*^9, 3.4275040096875*^9}, {
   3.441471663078125*^9, 3.441471667421875*^9}, 3.4425880786958876`*^9}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
MultTable[{e}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{RasterBox[{{{1, 1, 0}, {1, 1, 0}}, {{1, 1, 1}, {1, 1, 0}}}], 
   InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}], InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], 
   LineBox[{{1, 0}, {1, 1}, {2, 1}}]},
  ImageSize->{67.333333333333, Automatic}]], "Output",
 CellChangeTimes->{3.4362750266643753`*^9, 3.4425880809771376`*^9}]
}, Open  ]],

Cell[TextData[{
 "This group seems too simple to be of practical value, so we call this the \
",
 StyleBox["trivial group",
  FontSlant->"Italic"],
 ". Most of our groups will be much more interesting!\n\nThe last ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command introduces a new command\[LongDash]",
 StyleBox["InitGroup[ ]",
  FontWeight->"Bold"],
 ". This command erases all previous groups, and designates the new identity \
element. The command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["ClearDefs", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "erases all groups, and returns the dot to its normal definition.\n\nAs we \
see from these examples, sometimes the operator ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "\[CenterDot]", ")"}], TraditionalForm]]],
 " means addition, sometimes it means multiplication, and sometimes it means \
neither. Nonetheless, we can consider operation an element by itself a number \
of times. This produces combinations like\n\n",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ",\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "\[CenterDot]", " ", "x"}], TraditionalForm]]],
 ",\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "x", " ", "\[CenterDot]", " ", "x", " ", "\[CenterDot]", " ", "x"}], 
   TraditionalForm]]],
 ",\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "x", " ", "\[CenterDot]", " ", "x", " ", "\[CenterDot]", " ", "x", " ", 
    "\[CenterDot]", " ", "x"}], TraditionalForm]]],
 ",            and\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "x", " ", "\[CenterDot]", " ", "x", " ", "\[CenterDot]", " ", "x", " ", 
    "\[CenterDot]", " ", "x", " ", "\[CenterDot]", " ", "x"}], 
   TraditionalForm]]],
 ".\n\nEven though the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "\[CenterDot]", ")"}], TraditionalForm]]],
 " does not always mean multiplication, we can define ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "n"], TraditionalForm]]],
 " to mean ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " opperated on itself ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " times. Thus, \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    SuperscriptBox["x", "1"]}], TraditionalForm]]],
 ",\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "x"}], " ", "=", " ", 
    SuperscriptBox["x", "2"]}], TraditionalForm]]],
 ",\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "x", " ", "\[CenterDot]", " ", "x", " ", "\[CenterDot]", " ", "x"}], " ", 
    "=", " ", 
    SuperscriptBox["x", "3"]}], TraditionalForm]]],
 ", etc.\n                    \nWe formally define ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "n"], TraditionalForm]]],
 " for any integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". We let \n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "0"], " ", "=", " ", "e"}], TraditionalForm]]],
 ", the identity element, then we define, for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", ">", "0"}], TraditionalForm]]],
 ",\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "n"], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"n", "-", "1"}]], "\[CenterDot]", "x"}]}], TraditionalForm]]],
 ".\n\nBy defining the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-th power in terms of the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"n", "-", "1"}], ")"}], TraditionalForm]]],
 "-th power, we ",
 StyleBox["inductively",
  FontSlant->"Italic"],
 " have defined ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "n"], TraditionalForm]]],
 " whenever ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a positive integer.  \n\nFinally, we can define negative powers by \
letting ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"-", "n"}]], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox["x", "n"], ")"}], 
       RowBox[{"-", "1"}]], "   ", "if", "    ", "n"}], ">", "0"}]}], 
   TraditionalForm]]],
 ".\n\nNote that if the operator ",
 Cell[BoxData[
  FormBox["\[CenterDot]", TraditionalForm]]],
 " represents addition, the raising ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " to a power is repeated addition, which is multiplication.  \n\nWe can now \
prove some theorems which involve this exponential notation.\n\nPROPOSITION \
1.6\nIf ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is an element in a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are integers, then\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"m", "+", "n"}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", "m"], "\[CenterDot]", 
     SuperscriptBox["x", "n"]}]}], TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427504069765625*^9, 3.4275042218125*^9}, 
   3.4425880865083876`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 PageBreakAbove->True],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are 0, this proposition is very easy to verify:\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"m", "+", "0"}]], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["x", "m"], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", "m"], "\[CenterDot]", "e"}], " ", "=", " ", 
       RowBox[{
        SuperscriptBox["x", "m"], "\[CenterDot]", " ", 
        SuperscriptBox["x", "0"]}]}]}]}], ",", "   ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"0", "+", "n"}]], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["x", "n"], "=", " ", 
      RowBox[{
       RowBox[{"e", "\[CenterDot]", 
        SuperscriptBox["x", "n"]}], "=", 
       RowBox[{
        SuperscriptBox["x", "0"], "\[CenterDot]", 
        SuperscriptBox["x", "n"]}]}]}]}]}], TraditionalForm]]],
 ".\n\nWe will now prove the statement when ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are positive integers.  If ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is 1, then we have\n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"m", "+", "1"}]], "=", 
    RowBox[{
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "-", "1"}]], "\[CenterDot]", "x"}], 
     "=", " ", 
     RowBox[{
      SuperscriptBox["x", "m"], " ", "\[CenterDot]", 
      SuperscriptBox["x", "1"]}]}]}], TraditionalForm]]],
 ",\nusing the inductive definition of the power of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".\n\nWe will now proceed by means of ",
 StyleBox["induction",
  FontSlant->"Italic"],
 ".  That is, we will assume that the statement is true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", 
    RowBox[{"k", "-", "1"}]}], TraditionalForm]]],
 ", and then prove that it is then true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "k"}], TraditionalForm]]],
 ". Then we will have that, since the statement is true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ", and it is true for each number that follows, it must be true for all \
positive ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".\n\nThus, we will assume that  \n                                          \
    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"m", "+", 
      RowBox[{"(", 
       RowBox[{"k", "-", "1"}], ")"}]}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", "m"], "\[CenterDot]", 
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"(", 
        RowBox[{"k", "-", "1"}], ")"}]], "."}]}]}], TraditionalForm]]],
 "\n But then\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"m", "+", "k"}]], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"m", "+", "k", "-", "1"}]], "\[CenterDot]", "x"}], " ", "=", 
     " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "m"], "\[CenterDot]", 
       SuperscriptBox["x", 
        RowBox[{"k", "-", "1"}]], "\[CenterDot]", "x"}], " ", "=", " ", 
      RowBox[{
       SuperscriptBox["x", "m"], "\[CenterDot]", 
       SuperscriptBox["x", "k"]}]}]}]}], TraditionalForm]]],
 ".\n                  \nThus, by assuming the statement is true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", 
    RowBox[{"k", "-", "1"}]}], TraditionalForm]]],
 ", we found that it was also true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "k"}], TraditionalForm]]],
 ".  By induction, this proves that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"m", "+", "n"}]], "=", 
    RowBox[{
     SuperscriptBox["x", "m"], "\[CenterDot]", 
     SuperscriptBox["x", "n"]}]}], TraditionalForm]]],
 " for all positive ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".\n\nOnce we have the statement true for positive ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", we can take the inverse of both sides to give us\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["x", 
       RowBox[{"m", "+", "n"}]], ")"}], 
     RowBox[{"-", "1"}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["x", "n"], ")"}], 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["x", "m"], ")"}], 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ".\n\nBut by the definition of negative exponents, this is\n\n               \
 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "n"}], ")"}], "+", 
      RowBox[{"(", 
       RowBox[{"-", "m"}], ")"}]}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", "n"}]], "\[CenterDot]", 
     SuperscriptBox["x", 
      RowBox[{
       RowBox[{"-", "m"}], " "}]]}]}], TraditionalForm]]],
 "\n\nwhich, by letting ",
 Cell[BoxData[
  FormBox[
   RowBox[{"M", "=", " ", 
    RowBox[{"-", "n"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "=", " ", 
    RowBox[{"-", "m"}]}], TraditionalForm]]],
 ", proves the proposition for the case of both exponents being negative.\n\n\
Finally, if ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " have different signs, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"m", "+", "n"}], ")"}], TraditionalForm]]],
 " will either have the same sign as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "n"}], TraditionalForm]]],
 ", or the same sign as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "m"}], TraditionalForm]]],
 ".  If",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"(", 
     RowBox[{"m", "+", "n"}], ")"}]}], TraditionalForm]]],
 " has the same sign as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "n"}], TraditionalForm]]],
 ", then we have already shown that\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "m"], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"m", "+", "n"}], ")"}], "+", 
       RowBox[{"(", 
        RowBox[{"-", "n"}], ")"}]}]], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"m", "+", "n"}]], "\[CenterDot]", 
      SuperscriptBox["x", 
       RowBox[{"-", "n"}]]}]}]}], TraditionalForm]]],
 ".\n\n So we have   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "m"], "\[CenterDot]", 
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["x", 
        RowBox[{"-", "n"}]], ")"}], 
      RowBox[{"-", "1"}]]}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"m", "+", "n"}]], "\[CenterDot]", " ", 
     SuperscriptBox["x", 
      RowBox[{"-", "n"}]], "\[CenterDot]", 
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["x", 
        RowBox[{"-", "n"}]], ")"}], 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ",  and hence  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"m", "+", "n"}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", "m"], "\[CenterDot]", 
     SuperscriptBox["x", "n"]}]}], TraditionalForm]]],
 ".  \n \nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"m", "+", "n"}], ")"}], " "}], TraditionalForm]]],
 "has the same sign as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "m"}], TraditionalForm]]],
 ", then we have already shown that\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "n"], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", "m"}], ")"}], "+", 
       RowBox[{"(", 
        RowBox[{"m", "+", "n"}], ")"}]}]], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"-", "m"}]], "\[CenterDot]", 
      SuperscriptBox["x", 
       RowBox[{"m", "+", "n"}]]}]}]}], TraditionalForm]]],
 ".\n\n So we have  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["x", 
        RowBox[{"-", "m"}]], ")"}], 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SuperscriptBox["x", "n"]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      SuperscriptBox["x", 
       RowBox[{"-", "m"}]], ")"}], "\[CenterDot]", 
     SuperscriptBox["x", 
      RowBox[{"-", "m"}]], "\[CenterDot]", " ", 
     SuperscriptBox["x", 
      RowBox[{"m", "+", "n"}]]}]}], TraditionalForm]]],
 ",  and hence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"m", "+", "n"}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", "m"], "\[CenterDot]", 
     SuperscriptBox["x", "n"]}]}], TraditionalForm]]],
 " .\n\nThus, we have proven the proposition for all integers ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42750424290625*^9, 3.427504748421875*^9}, {
  3.4284445626875*^9, 3.428444714578125*^9}, {3.4362751885237503`*^9, 
  3.4362751911956253`*^9}, {3.4362752370706253`*^9, 3.4362752685081253`*^9}, {
  3.4362753872425003`*^9, 3.4362754035862503`*^9}, {3.441471769953125*^9, 
  3.44147177409375*^9}},
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]]
}, Open  ]],

Cell[TextData[{
 "This last proof introduces an important method of proving theorems called \
",
 StyleBox["induction",
  FontSlant->"Italic"],
 ". Induction is based on the simple fact that if a set of positive integers \
contains the number 1, and has the property that ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is in the set whenever ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "-", "1"}], TraditionalForm]]],
 " is, then the set must be all positive integers.\n\nIt is not hard to see \
why this must be true.  If there were some positive integer not in the set, \
then there must be a ",
 StyleBox["smallest",
  FontSlant->"Italic"],
 " positive integer ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " that is not in the set.  Since ",
 Cell[BoxData[
  FormBox["1", TraditionalForm]]],
 " is in the set, we see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", ">", "1"}], TraditionalForm]]],
 ", and since ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is the smallest number not in the set, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "-", "1"}], TraditionalForm]]],
 " must be in the set.  But the property of the set is that if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "-", "1"}], TraditionalForm]]],
 " is in the set, then ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " also is.  So we have a contradition, so there is no such ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ", meaning the the set is indeed all positive integers.\n\nThis gives us a \
powerful tool for proofs.  In fact, we really do not need to introduce the \
variable ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ".  To prove a statement for all positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", we can first prove the statement is true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ", and then we can assume that the statement is true for the previous case \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ".  This extra information often gives us the leverage we need to be able to \
prove the statement is true for ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".  Here is another example of the use of induction. \n"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42750498659375*^9, 3.4275052660625*^9}, {
   3.427506252484375*^9, 3.4275063171875*^9}, {3.428444741515625*^9, 
   3.428444761796875*^9}, 3.4425880907740126`*^9}],

Cell[TextData[{
 "PROPOSITION 1.7           \nIf ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is an element in a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are in \[DoubleStruckCapitalZ], then\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["x", "m"], ")"}], "n"], " ", "=", " ", 
    SuperscriptBox["x", 
     RowBox[{"(", 
      RowBox[{"m", " ", "n"}], ")"}]]}], TraditionalForm]]],
 ".                 "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.427505275875*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Notice that this statement is trivial if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "0"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ":\n          \n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["x", "m"], ")"}], "0"], " ", "=", " ", 
    RowBox[{"e", " ", "=", " ", 
     SuperscriptBox["x", 
      RowBox[{"m", "\[CenterDot]", "0"}]]}]}], TraditionalForm]]],
 ",           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["x", "m"], ")"}], "1"], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"m", " "}]], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"(", 
        RowBox[{"m", "\[CenterDot]", "1"}], ")"}]], "."}]}]}], 
   TraditionalForm]]],
 "\n \nWe will again proceed by means of induction, which means we can assume \
that the statement is true for the previous case, with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " replaced by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ".  That is, we can assume that \n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["x", "m"], ")"}], 
     RowBox[{"n", "-", "1"}]], " ", "=", " ", 
    SuperscriptBox["x", 
     RowBox[{"m", "(", 
      RowBox[{"n", "-", "1"}], ")"}]]}], TraditionalForm]]],
 ".\n\nNote that\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["x", "m"], ")"}], "n"], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox["x", "m"], ")"}], 
       RowBox[{"n", "-", "1"}]], "\[CenterDot]", 
      SuperscriptBox["x", "m"]}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"m", " ", 
        RowBox[{"(", 
         RowBox[{"n", "-", "1"}], ")"}]}]], "\[CenterDot]", 
      SuperscriptBox["x", "m"]}]}]}], TraditionalForm]]],
 ".\n\nBy proposition 1.6, this is equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"m", " ", 
        RowBox[{"(", 
         RowBox[{"n", "-", "1"}], ")"}]}], " ", "+", " ", "m"}], ")"}]], " ", 
    "=", " ", 
    SuperscriptBox["x", 
     RowBox[{"m", " ", "n"}]]}], TraditionalForm]]],
 ".\n\nSo by induction, the proposition holds for positive ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".  To see that it holds \nfor negative ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " as well, simply note that\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["x", "m"], ")"}], "n"], " ", "=", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox["x", "m"], ")"}], 
        RowBox[{"-", "n"}]], ")"}], 
      RowBox[{"-", "1"}]], " ", "=", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox["x", 
         RowBox[{
          RowBox[{"-", "m"}], " ", "n"}]], ")"}], 
       RowBox[{"-", "1"}]], " ", "=", " ", 
      SuperscriptBox["x", 
       RowBox[{"(", 
        RowBox[{"m", " ", "n"}], ")"}]]}]}], " "}], TraditionalForm]]],
 ".\n\nIf ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is negative, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "n"}], TraditionalForm]]],
 " is positive, so the second step is valid.   "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427505311828125*^9, 3.42750534484375*^9}, {
   3.42750635521875*^9, 3.427506549640625*^9}, {3.4362754996800003`*^9, 
   3.4362755039612503`*^9}, {3.4362755460393753`*^9, 3.4362755472112503`*^9}, 
   3.4425880937271376`*^9},
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]]
}, Open  ]],

Cell[TextData[{
 "The principle of induction can easily be generalized. In proving a \
statement is true for the case ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", we can not only assume that it is true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ", but we can also assume that the statement is true for all values smaller \
than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " as well. Also, there is no reason why we must start with the number 1. Any \
other integer can be used for the starting point. \nThat is, we first prove \
the statement is true for the case ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ".  If we can then prove that the statement is true for ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", utilizing the assumption that the statement is true for all numbers \
between ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ", then we have successfully proven that the statement is true for all \
integers greater than ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ".  Problems 1.27 through 1.33 give some practice for using the principle of \
induction. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42750668296875*^9, 3.42750677059375*^9}, {
   3.4275068088125*^9, 3.427506836046875*^9}, {3.427507857515625*^9, 
   3.42750786065625*^9}, {3.428444782546875*^9, 3.42844478903125*^9}, {
   3.4362756380862503`*^9, 3.4362756411018753`*^9}, 3.4425880956490126`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problems", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["", "Input",
 PageWidth->Infinity,
 FontFamily->"System",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->RGBColor[1, 0, 0],
 Background->GrayLevel[1]],

Cell["                  MATHEMATICA PROBLEMS:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.427506865578125*^9, {3.42844479925*^9, 3.428444812296875*^9}, 
   3.4362758040862503`*^9}],

Cell[TextData[{
 "1.1 ) If Terry was only allowed to do the steps \n   ",
 StyleBox["FlipRt",
  FontWeight->"Bold"],
 " or ",
 StyleBox["FlipLft",
  FontWeight->"Bold"],
 ", could it get itself into all \n   6 possible positions? If possible, \
express the \n   other 4 dance steps in terms of thes two. What \n   if Terry \
was only allowed to do the dance steps\n   RotRt and RotLft? The following ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " \n   command reloads Terry's group. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.427506865578125*^9, {3.42844479925*^9, 3.428444812296875*^9}, {
   3.4362758040862503`*^9, 3.4362758072112503`*^9}, 3.4425881001333876`*^9}],

Cell[CellGroupData[{

Cell["InitTerry", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "Stay", ",", "FlipRt", ",", "RotRt", ",", "FlipLft", ",", "RotLft", ",", 
   "Spin"}], "}"}]], "Output",
 CellChangeTimes->{3.4425881018677626`*^9}]
}, Open  ]],

Cell["      ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "1.2 ) Repeat problem 1.1, only allow Terry to do \n   only the steps ",
 StyleBox["RotRt",
  FontWeight->"Bold"],
 " or ",
 StyleBox["RotLft",
  FontWeight->"Bold"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.427506865578125*^9, {3.42844479925*^9, 3.428444812296875*^9}, {
   3.4362758040862503`*^9, 3.4362758973362503`*^9}, {3.4362759655237503`*^9, 
   3.4362759664456253`*^9}}],

Cell["      ", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4362758331175003`*^9, 3.4362758338675003`*^9}},
 AspectRatioFixed->True],

Cell[TextData[{
 "\n1.3 ) We saw that there were exactly 4 numbers\n   less than 10 which \
were invertible modulo 10. \n   For what other values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are there exactly 4 \n   numbers less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " which are invertible modulo \n   ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "? Use ",
 StyleBox["Mathematica'",
  FontSlant->"Italic"],
 "s circle graph to graph the \n   inverse functions."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.427506867953125*^9, {3.4362759039925003`*^9, 3.4362759722425003`*^9}}],

Cell["         ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "\n1.4 ) According to the Chinese Remainder \n    Theorem (1.3), there is a \
number less than 77 \n    that is congruent to 4 (Mod 11), and congruent \n   \
 to 6 (Mod 7). Find this number, using \n    ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to help."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.427506869390625*^9, {3.427506935421875*^9, 3.427506935578125*^9}, {
   3.4362759901175003`*^9, 3.4362760118675003`*^9}, {3.4362760561643753`*^9, 
   3.4362760564300003`*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "\n1.5 ) The following ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command creates a \n    multiplication table of the five elements \n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"e", ",", " ", "a", ",", " ", "b", ",", " ", "c", ",", " ", "d"}],
     "}"}], TraditionalForm]]],
 ". First execute this command."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.42750687075*^9, {3.4362760589925003`*^9, 3.4362760591956253`*^9}, {
   3.4362761204925003`*^9, 3.4362761226643753`*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a.a, e]; Define[a.b, c]; Define[a.c, d]; Define[a.d, b];
Define[b.a, d]; Define[b.b, e]; Define[b.c, a]; Define[b.d, c];
Define[c.a, b]; Define[c.b, d]; Define[c.c, e]; Define[c.d, a];
Define[d.a, c]; Define[d.b, a]; Define[d.c, b]; Define[d.d, e];
MultTable[{e, a, b, c, d}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0, 1, 0}, {0, 1, 0}, {1, 0, 0}, {1, 0, 1}, {0, 1, 1}, {1, 1, 
    0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 1}, {0, 1, 0}, {1, 1, 0}, {1, 0, 
    1}}, {{0, 1, 1}, {0, 1, 1}, {0, 1, 0}, {1, 1, 0}, {1, 0, 1}, {1, 0, 
    0}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 0}, {0, 1, 
    1}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
    0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
    0}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"c\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"d\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"c\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"d\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"d\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"c\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"c\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"d\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"d\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"c\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 5}, {6, 5}}]}]], "Output",
 CellChangeTimes->{3.4425881100865126`*^9}]
}, Open  ]],

Cell["\<\
    Notice that this multiplication table satisfies 
    the \"Latin square\" property, hence this
    multiplication satisfies proposition 1.3. Does
    this set form a group? Why or why not?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427506958671875*^9, 3.427506958828125*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.5,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[568, 21, 322, 9, 285, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[915, 34, 285, 8, 29, "Name",
 Evaluatable->False],
Cell[1203, 44, 619, 15, 70, "Name",
 Evaluatable->False]
}, Closed]],
Cell[1837, 62, 276, 12, 199, "Title",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2138, 78, 104, 3, 32, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2267, 85, 228, 5, 43, "Input",
 InitializationCell->True],
Cell[2498, 92, 121, 1, 32, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2668, 99, 159, 3, 105, "Section",
 Evaluatable->False],
Cell[2830, 104, 574, 14, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[3429, 122, 73, 2, 43, "Input"],
Cell[3505, 126, 377, 7, 390, "Print"]
}, Open  ]],
Cell[3897, 136, 282, 8, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[4204, 148, 76, 2, 43, "Input"],
Cell[4283, 152, 396, 10, 390, "Print"],
Cell[4682, 164, 398, 10, 390, "Print"],
Cell[5083, 176, 353, 7, 390, "Print"]
}, Open  ]],
Cell[5451, 186, 256, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[5732, 198, 77, 2, 43, "Input"],
Cell[5812, 202, 398, 10, 390, "Print"],
Cell[6213, 214, 396, 10, 390, "Print"],
Cell[6612, 226, 353, 7, 390, "Print"]
}, Open  ]],
Cell[6980, 236, 164, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[7169, 245, 75, 2, 43, "Input"],
Cell[7247, 249, 361, 7, 390, "Print"],
Cell[7611, 258, 361, 7, 390, "Print"],
Cell[7975, 267, 353, 7, 390, "Print"]
}, Open  ]],
Cell[8343, 277, 171, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[8539, 286, 77, 2, 43, "Input"],
Cell[8619, 290, 382, 8, 390, "Print"],
Cell[9004, 300, 359, 7, 390, "Print"],
Cell[9366, 309, 353, 7, 390, "Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9756, 321, 78, 2, 43, "Input"],
Cell[9837, 325, 384, 8, 390, "Print"],
Cell[10224, 335, 361, 7, 390, "Print"],
Cell[10588, 344, 353, 7, 390, "Print"]
}, Open  ]],
Cell[10956, 354, 175, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[11156, 363, 75, 2, 43, "Input"],
Cell[11234, 367, 6422, 108, 390, 309, 6, "CachedBoxData", "BoxData", "Print"]
}, Open  ]],
Cell[17671, 478, 304, 8, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[18000, 490, 88, 2, 43, "Input"],
Cell[18091, 494, 382, 8, 390, "Print"],
Cell[18476, 504, 359, 7, 390, "Print"],
Cell[18838, 513, 353, 7, 386, "Print"],
Cell[19194, 522, 361, 7, 382, "Print"],
Cell[19558, 531, 361, 7, 378, "Print"],
Cell[19922, 540, 353, 7, 374, "Print"],
Cell[20278, 549, 396, 10, 390, "Print"],
Cell[20677, 561, 398, 10, 390, "Print"],
Cell[21078, 573, 353, 7, 390, "Print"]
}, Open  ]],
Cell[21446, 583, 1819, 30, 923, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[23290, 617, 152, 3, 43, "Input"],
Cell[23445, 622, 406, 8, 390, "Print"],
Cell[23854, 632, 383, 7, 390, "Print"],
Cell[24240, 641, 377, 7, 390, "Print"],
Cell[24620, 650, 385, 7, 390, "Print"],
Cell[25008, 659, 385, 7, 390, "Print"],
Cell[25396, 668, 377, 7, 390, "Print"]
}, Open  ]],
Cell[25788, 678, 583, 12, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[26396, 694, 73, 2, 43, "Input"],
Cell[26472, 698, 214, 5, 68, "Output"]
}, Open  ]],
Cell[26701, 706, 119, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[26845, 712, 126, 3, 43, "Input"],
Cell[26974, 717, 422, 10, 390, "Print"],
Cell[27399, 729, 420, 10, 390, "Print"],
Cell[27822, 741, 377, 7, 552, "Print"]
}, Open  ]],
Cell[28214, 751, 951, 23, 311, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[29190, 778, 147, 3, 43, "Input"],
Cell[29340, 783, 131, 2, 42, "Output"]
}, Open  ]],
Cell[29486, 788, 173, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[29684, 797, 114, 2, 43, "Input"],
Cell[29801, 801, 7191, 204, 561, "Output"]
}, Open  ]],
Cell[37007, 1008, 872, 16, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[37904, 1028, 76, 2, 43, "Input"],
Cell[37983, 1032, 76, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[38096, 1038, 76, 2, 43, "Input"],
Cell[38175, 1042, 76, 1, 42, "Output"]
}, Open  ]],
Cell[38266, 1046, 1442, 41, 433, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[39733, 1091, 147, 3, 43, "Input"],
Cell[39883, 1096, 100, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[40020, 1102, 147, 3, 43, "Input"],
Cell[40170, 1107, 101, 1, 42, "Output"]
}, Open  ]],
Cell[40286, 1111, 1790, 57, 482, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[42101, 1172, 88, 2, 43, "Input"],
Cell[42192, 1176, 76, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42305, 1182, 88, 2, 43, "Input"],
Cell[42396, 1186, 76, 1, 42, "Output"]
}, Open  ]],
Cell[42487, 1190, 293, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[42805, 1202, 114, 2, 43, "Input"],
Cell[42922, 1206, 7167, 204, 561, "Output"]
}, Open  ]],
Cell[50104, 1413, 338, 7, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[50467, 1424, 78, 2, 43, "Input"],
Cell[50548, 1428, 75, 1, 42, "Output"]
}, Open  ]],
Cell[50638, 1432, 192, 5, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[50855, 1441, 75, 2, 43, "Input"],
Cell[50933, 1445, 75, 1, 42, "Output"]
}, Open  ]],
Cell[51023, 1449, 1118, 39, 215, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[52166, 1492, 74, 2, 43, "Input"],
Cell[52243, 1496, 77, 1, 42, "Output"]
}, Open  ]],
Cell[52335, 1500, 3774, 115, 825, "Text",
 Evaluatable->False],
Cell[56112, 1617, 701, 22, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[56838, 1643, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[56926, 1647, 1868, 63, 137, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[58809, 1713, 754, 14, 139, "Text",
 Evaluatable->False],
Cell[59566, 1729, 513, 15, 68, "Text",
 Evaluatable->False],
Cell[60082, 1746, 939, 30, 97, "Text",
 Evaluatable->False],
Cell[61024, 1778, 381, 9, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[61430, 1791, 150, 3, 43, "Input"],
Cell[61583, 1796, 76, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61696, 1802, 153, 3, 43, "Input"],
Cell[61852, 1807, 77, 1, 42, "Output"]
}, Open  ]],
Cell[61944, 1811, 201, 6, 41, "Text",
 Evaluatable->False],
Cell[62148, 1819, 1221, 37, 192, "Text",
 Evaluatable->False],
Cell[63372, 1858, 541, 18, 92, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[63938, 1880, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[64026, 1884, 2237, 76, 224, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[66278, 1963, 182, 6, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[66485, 1973, 114, 2, 43, "Input"],
Cell[66602, 1977, 7167, 204, 561, "Output"]
}, Open  ]],
Cell[73784, 2184, 1379, 32, 360, "Text",
 Evaluatable->False],
Cell[75166, 2218, 572, 21, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[75763, 2243, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[75851, 2247, 2944, 94, 326, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[78810, 2344, 646, 13, 139, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[79493, 2362, 91, 2, 105, "Section",
 Evaluatable->False],
Cell[79587, 2366, 1630, 37, 311, "Text",
 Evaluatable->False],
Cell[81220, 2405, 1171, 30, 213, "Text",
 Evaluatable->False],
Cell[82394, 2437, 994, 30, 164, "Text",
 Evaluatable->False],
Cell[83391, 2469, 140, 3, 43, "Input"],
Cell[83534, 2474, 732, 17, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[84291, 2495, 69, 2, 43, "Input"],
Cell[84363, 2499, 72, 1, 42, "Output"]
}, Open  ]],
Cell[84450, 2503, 597, 13, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[85072, 2520, 96, 2, 43, "Input"],
Cell[85171, 2524, 17447, 508, 561, "Output"]
}, Open  ]],
Cell[102633, 3035, 893, 19, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[103551, 3058, 69, 2, 43, "Input"],
Cell[103623, 3062, 72, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[103732, 3068, 69, 2, 43, "Input"],
Cell[103804, 3072, 72, 1, 42, "Output"]
}, Open  ]],
Cell[103891, 3076, 2945, 100, 531, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[106861, 3180, 107, 2, 43, "Input"],
Cell[106971, 3184, 24677, 468, 622, 6649, 171, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[131663, 3655, 1097, 35, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[132785, 3694, 151, 5, 66, "Input"],
Cell[132939, 3701, 98602, 1723, 622, 18598, 410, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[231556, 5427, 473, 10, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[232054, 5441, 103, 2, 43, "Input"],
Cell[232160, 5445, 23645, 452, 622, 7922, 192, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[255820, 5900, 1478, 39, 293, "Text",
 Evaluatable->False],
Cell[257301, 5941, 384, 10, 115, "Text",
 Evaluatable->False],
Cell[257688, 5953, 76, 2, 43, "Input"],
Cell[257767, 5957, 215, 6, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[258007, 5967, 90, 2, 43, "Input"],
Cell[258100, 5971, 9100, 267, 561, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[267237, 6243, 113, 2, 43, "Input"],
Cell[267353, 6247, 52090, 915, 568, 8860, 204, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[319458, 7165, 485, 11, 188, "Text",
 Evaluatable->False],
Cell[319946, 7178, 77, 2, 43, "Input"],
Cell[320026, 7182, 176, 3, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[320227, 7189, 90, 2, 43, "Input"],
Cell[320320, 7193, 16240, 385, 561, 9046, 266, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[336575, 7581, 327, 7, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[336927, 7592, 88, 2, 43, "Input"],
Cell[337018, 7596, 12544, 297, 561, 6915, 203, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[349577, 7896, 413, 9, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[350015, 7909, 108, 2, 43, "Input"],
Cell[350126, 7913, 40208, 712, 568, 7970, 182, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[390349, 8628, 390, 9, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[390764, 8641, 96, 2, 43, "Input"],
Cell[390863, 8645, 24982, 458, 568, 6023, 146, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[415860, 9106, 942, 30, 90, "Text",
 Evaluatable->False],
Cell[416805, 9138, 444, 11, 115, "Text",
 Evaluatable->False],
Cell[417252, 9151, 302, 7, 66, "Text",
 Evaluatable->False],
Cell[417557, 9160, 78, 2, 43, "Input"],
Cell[CellGroupData[{
Cell[417660, 9166, 96, 2, 43, "Input"],
Cell[417759, 9170, 30118, 717, 561, 17308, 506, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[447892, 9890, 322, 7, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[448239, 9901, 102, 2, 43, "Input"],
Cell[448344, 9905, 30690, 580, 622, 8884, 221, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[479049, 10488, 647, 18, 213, "Text",
 Evaluatable->False],
Cell[479699, 10508, 78, 2, 43, "Input"],
Cell[CellGroupData[{
Cell[479802, 10514, 117, 2, 43, "Input"],
Cell[479922, 10518, 13888, 335, 564, "Output"]
}, Open  ]],
Cell[493825, 10856, 248, 6, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[494098, 10866, 101, 2, 43, "Input"],
Cell[494202, 10870, 28034, 508, 582, 7517, 170, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[522251, 11381, 218, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[522494, 11390, 95, 2, 43, "Input"],
Cell[522592, 11394, 19729, 474, 561, 11558, 338, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[542336, 11871, 433, 9, 90, "Text",
 Evaluatable->False],
Cell[542772, 11882, 1934, 74, 311, "Text",
 Evaluatable->False],
Cell[544709, 11958, 854, 28, 115, "Text",
 Evaluatable->False],
Cell[545566, 11988, 279, 6, 66, "Text",
 Evaluatable->False],
Cell[545848, 11996, 850, 28, 90, "Text",
 Evaluatable->False],
Cell[546701, 12026, 2191, 85, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[548917, 12115, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[549005, 12119, 1744, 62, 152, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[550764, 12184, 1012, 21, 213, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[551813, 12210, 94, 2, 105, "Section",
 Evaluatable->False],
Cell[551910, 12214, 2000, 43, 433, "Text",
 Evaluatable->False],
Cell[553913, 12259, 233, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[554171, 12271, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[554259, 12275, 1379, 47, 96, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[555653, 12325, 963, 17, 164, "Text",
 Evaluatable->False],
Cell[556619, 12344, 995, 24, 92, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[557639, 12372, 82, 2, 43, "Input"],
Cell[557724, 12376, 357, 11, 42, "Output"]
}, Open  ]],
Cell[558096, 12390, 1224, 29, 213, "Text",
 Evaluatable->False],
Cell[559323, 12421, 463, 10, 66, "Text",
 Evaluatable->False],
Cell[559789, 12433, 278, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[560092, 12445, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[560180, 12449, 1681, 44, 243, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[561876, 12496, 664, 17, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[562565, 12517, 146, 5, 66, "Input"],
Cell[562714, 12524, 72, 1, 42, "Output"]
}, Open  ]],
Cell[562801, 12528, 624, 13, 164, "Text",
 Evaluatable->False],
Cell[563428, 12543, 893, 32, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[564346, 12579, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[564434, 12583, 5108, 186, 383, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[569557, 12772, 401, 12, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[569983, 12788, 155, 3, 43, "Input"],
Cell[570141, 12793, 259, 7, 42, "Output"]
}, Open  ]],
Cell[570415, 12803, 851, 32, 66, "Text",
 Evaluatable->False],
Cell[571269, 12837, 603, 16, 66, "Text",
 Evaluatable->False],
Cell[571875, 12855, 792, 31, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[572692, 12890, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[572780, 12894, 2962, 105, 342, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[575757, 13002, 350, 8, 66, "Text",
 Evaluatable->False],
Cell[576110, 13012, 1533, 48, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[577668, 13064, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[577756, 13068, 6002, 194, 711, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[583773, 13265, 943, 20, 213, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[584753, 13290, 98, 2, 105, "Section",
 Evaluatable->False],
Cell[584854, 13294, 559, 10, 115, "Text",
 Evaluatable->False],
Cell[585416, 13306, 2795, 102, 507, "Text",
 Evaluatable->False],
Cell[588214, 13410, 1193, 34, 213, "Text",
 Evaluatable->False],
Cell[589410, 13446, 1611, 45, 262, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[591046, 13495, 93, 4, 66, "Input"],
Cell[591142, 13501, 17423, 508, 561, "Output"]
}, Open  ]],
Cell[608580, 14012, 850, 26, 93, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[609455, 14042, 138, 5, 66, "Input"],
Cell[609596, 14049, 11602, 339, 561, "Output"]
}, Open  ]],
Cell[621213, 14391, 1122, 37, 167, "Text",
 Evaluatable->False],
Cell[622338, 14430, 6922, 209, 1180, "Text",
 Evaluatable->False],
Cell[629263, 14641, 386, 14, 66, "Text",
 Evaluatable->False],
Cell[629652, 14657, 1715, 46, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[631392, 14707, 100, 5, 66, "Input"],
Cell[631495, 14714, 618, 17, 123, "Output"]
}, Open  ]],
Cell[632128, 14734, 530, 15, 139, "Text",
 Evaluatable->False],
Cell[632661, 14751, 73, 2, 43, "Input"],
Cell[632737, 14755, 4563, 157, 886, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[637325, 14916, 108, 3, 42, "Subsubsection",
 Evaluatable->False,
 PageBreakAbove->True],
Cell[637436, 14921, 9909, 341, 981, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[647360, 15265, 2512, 72, 360, "Text",
 Evaluatable->False],
Cell[649875, 15339, 736, 27, 141, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[650636, 15370, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[650724, 15374, 4141, 139, 481, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[654880, 15516, 1598, 41, 188, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[656515, 15562, 81, 2, 105, "Section",
 Evaluatable->False],
Cell[656599, 15566, 156, 6, 43, "Input"],
Cell[656758, 15574, 224, 5, 41, "Text",
 Evaluatable->False],
Cell[656985, 15581, 723, 18, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[657733, 15603, 73, 2, 43, "Input"],
Cell[657809, 15607, 190, 5, 42, "Output"]
}, Open  ]],
Cell[658014, 15615, 70, 2, 43, "Input"],
Cell[658087, 15619, 449, 14, 66, "Text",
 Evaluatable->False],
Cell[658539, 15635, 140, 3, 43, "Input"],
Cell[658682, 15640, 666, 19, 188, "Text",
 Evaluatable->False],
Cell[659351, 15661, 73, 2, 43, "Input"],
Cell[659427, 15665, 549, 13, 164, "Text",
 Evaluatable->False],
Cell[659979, 15680, 67, 2, 43, "Input"],
Cell[660049, 15684, 567, 16, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[660641, 15704, 368, 9, 156, "Input"],
Cell[661012, 15715, 5069, 150, 561, "Output"]
}, Open  ]],
Cell[666096, 15868, 336, 8, 115, "Text",
 Evaluatable->False],
Cell[666435, 15878, 67, 2, 43, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
