(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    643876,      14910]
NotebookOptionsPosition[    621332,      14185]
NotebookOutlinePosition[    622046,      14212]
CellTagsIndexPosition[    622003,      14209]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Abstract Algebra: An Interactive Approach", "Special2",
 Evaluatable->False,
 CellChangeTimes->{{3.436285960703125*^9, 3.43628598803125*^9}},
 TextAlignment->Center,
 FontFamily->"Arial",
 FontSize->36,
 FontWeight->"Bold",
 FontSlant->"Italic",
 FontColor->RGBColor[1, 0, 1],
 Background->RGBColor[0.500008, 1, 1]],

Cell[CellGroupData[{

Cell["\[Copyright]2009   William Paulsen", "Name",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436285998296875*^9, 3.436285998765625*^9}},
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]],

Cell["\<\
All rights reserved:  No part of these materials may be reproduced, either \
electronically, mechanically, or by any other means or in any form, without \
the written permission from the author. All software is provided \"as is\" \
without warranty of any kind, either expressed or implied, including, but not \
limited to, the implied warranties of merchantability and fitness for a \
particular purpose.\
\>", "Name",
 Editable->False,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]]
}, Closed]],

Cell["\<\
Chapter 2
The Structure Within a Group\
\>", "Title",
 CellFrame->True,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 TextAlignment->Center,
 FontFamily->"New York",
 FontSize->24,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell["Initialization", "Special2",
 Evaluatable->False,
 TextAlignment->Center,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["<<c:\\math\\group.m", "Input",
 PageWidth->Infinity,
 InitializationCell->True,
 CellChangeTimes->{{3.42754511496875*^9, 3.427545119484375*^9}, {
  3.42755467134375*^9, 3.4275546775625*^9}, {3.4362771608050003`*^9, 
  3.4362771618675003`*^9}},
 AspectRatioFixed->True],

Cell[BoxData["\<\"Initialization Done\"\>"], "Print",
 CellChangeTimes->{3.427554672140625*^9, 3.4362771650862503`*^9, 
  3.4414774086875*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generators of Groups", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In the last chapter, we introduced the definition of a group, giving some \
simple examples of groups. There are, of course, many other examples of \
groups that we will want to work with. We want to learn how to efficiently \
program ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to work with any specific group. Many of these groups will be fairly \
large, and so rather than giving ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " the entire group, we will define a group using a very small number of \
elements. From these few elements, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will be able to reconstruct the entire group. These few elements will be \
called the generators of the group. So before we can learn how to program a \
group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we first need to understand generators.\n\nIn this section we will deal \
with ",
 StyleBox["finite",
  FontSlant->"Italic"],
 " groups, that is groups which only have a finite number of elements. We \
have seen several examples of finite groups, such as Terry's group, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "n", "*"], TraditionalForm]]],
 ". Let us return to one of the groups that we studied in the last section, \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "10"], TraditionalForm]]],
 ". The following command loads this group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.427546016421875*^9}],

Cell["DefSumMod[10]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "One of the experiments from the last section was to make circle graphs of \
the opperation \"Add ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " to each number.\" For example, the circle graph of Add[3], which maps each \
element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " to the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "3"}], TraditionalForm]]],
 ", is given by "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.42754601209375*^9}],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6,7,8,9}, Add[3] ]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924731, 0.8090169943749475}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, 0.30901699437494745`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924732, -0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, -0.8090169943749472}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951536, 0.30901699437494723`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, 0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"5\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217205, 0.8899186938124423}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, 0.3399186938124422}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, -0.33991869381244233`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217207, -0.8899186938124422}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, -0.889918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0461621679246689`, -0.33991869381244233`}, {1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.046162167924669, 0.339918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, 0.8899186938124422}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{0., 1.}, {0.8922779910659062, -0.2281152949374528}}], 
     PolygonBox[{{0.9510565162951535, -0.30901699437494756`}, {
      0.9327288407846536, -0.19872603232282918`}, {
      0.8518271413471589, -0.25750455755207646`}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{
      0.9510565162951535, -0.30901699437494756`}, {-0.492679600662958, \
-0.7781152949374525}}], 
     PolygonBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.47722875094421063`, -0.8256681207522102}, {-0.5081304503817053, \
-0.7305624691226948}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.5877852522924734, 0.7090169943749474}}], 
     PolygonBox[{{-0.5877852522924734, 
      0.8090169943749473}, {-0.6377852522924734, 
      0.7090169943749474}, {-0.5377852522924733, 0.7090169943749474}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.5877852522924734, 0.8090169943749473}, {0.8559508646656382, 
      0.33991869381244216`}}], 
     PolygonBox[{{0.9510565162951535, 0.30901699437494745`}, {
      0.8714017143843855, 0.3874715196271999}, {0.8405000149468907, 
      0.2923658679976845}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{0.9510565162951535, 0.30901699437494745`}, {
      0.05877852522924743, -0.9190983005625053}}], 
     PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
      0.0992293749479948, -0.9484875631771289}, {
      0.018327675510500054`, -0.8897090379478816}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{1.2246467991473532`*^-16, -1.}, {-0.8922779910659063, 
      0.22811529493745253`}}], 
     PolygonBox[{{-0.9510565162951536, 
      0.30901699437494723`}, {-0.9327288407846537, 
      0.19872603232282884`}, {-0.851827141347159, 0.2575045575520762}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.9510565162951536, 0.30901699437494723`}, {
      0.4926796006629578, 0.7781152949374527}}], 
     PolygonBox[{{0.5877852522924731, 0.8090169943749475}, {
      0.4772287509442104, 0.8256681207522104}, {0.5081304503817051, 
      0.7305624691226951}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{0.5877852522924731, 0.8090169943749475}, {
      0.5877852522924732, -0.7090169943749474}}], 
     PolygonBox[{{0.5877852522924732, -0.8090169943749473}, {
      0.6377852522924733, -0.7090169943749474}, {
      0.5377852522924732, -0.7090169943749474}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{
      0.5877852522924732, -0.8090169943749473}, {-0.8559508646656382, \
-0.3399186938124423}}], 
     PolygonBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.8714017143843855, -0.38747151962719995`}, {-0.8405000149468909, \
-0.2923658679976846}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.05877852522924731, 0.9190983005625053}}], 
     PolygonBox[{{0., 1.}, {-0.09922937494799466, 
      0.9484875631771289}, {-0.01832767551049994, 0.8897090379478816}}]}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425882305083876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "This graph allows us to visualize powers of 3 in the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "10"], TraditionalForm]]],
 ". If we follow the red arrows, starting with 0, we have the sequence        \
     \n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
    "0", ",", " ", "3", ",", " ", "6", ",", " ", "9", ",", " ", "2", ",", " ",
      "5", ",", " ", "8", ",", " ", "1", ",", " ", "4", ",", " ", "7", ",", 
     " ", 
     RowBox[{"0", " ", "..."}]}], "}"}], TraditionalForm]]],
 ". \nThis tells us that, in this group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "10"], TraditionalForm]]],
 ",\n \n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["3", "0"],
     TraditionalForm], " ", "=", " ", "0"}], TraditionalForm]]],
 ",\n \n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["3", "1"], " ", "=", " ", "3"}], TraditionalForm]]],
 ",\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["3", "2"], " ", "=", " ", 
    RowBox[{
     RowBox[{"3", "\[CenterDot]", "3"}], " ", "=", " ", "6"}]}], 
   TraditionalForm]]],
 ",\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["3", "3"], " ", "=", " ", 
    RowBox[{
     RowBox[{"3", "\[CenterDot]", "3", "\[CenterDot]", "3"}], " ", "=", " ", 
     "9"}]}], TraditionalForm]]],
 ",\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["3", "4"], " ", "=", " ", 
    RowBox[{
     RowBox[{
     "3", "\[CenterDot]", "3", "\[CenterDot]", "3", "\[CenterDot]", "3"}], 
     " ", "=", " ", "2"}]}], TraditionalForm]]],
 ", etc.\n\nRecall that in this group, the dot represents addition, so an \
exponient represents repeated addition. It is easy to see from the circle \
graph that every element in the group can be expressed as a power of 3, since \
one can travel the red arrows to get from 0 to any element of the group. This \
will not always be the case, as the circle graph of \n\"Add[4]\" indicates. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.427546006546875*^9}],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6,7,8,9}, Add[4]]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924731, 0.8090169943749475}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, 0.30901699437494745`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924732, -0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, -0.8090169943749472}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951535, -0.30901699437494756`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951536, 0.30901699437494723`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5877852522924734, 0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"5\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217205, 0.8899186938124423}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, 0.3399186938124422}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, -0.33991869381244233`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217207, -0.8899186938124422}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, -0.889918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0461621679246689`, -0.33991869381244233`}, {1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.046162167924669, 0.339918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217208, 0.8899186938124422}, {1, 0}]}, {{
     {RGBColor[1, 0, 0], 
      LineBox[{{0., 1.}, {0.5568835528549785, -0.713911342745432}}], 
      PolygonBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.6044363786697362, -0.6984604930266846}, {
       0.5093307270402208, -0.7293621924641793}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{
       0.5877852522924732, -0.8090169943749473}, {-0.8701548168576589, 
       0.2502384691456999}}], 
      PolygonBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.8995440794722825, 
       0.20978761942695257`}, {-0.8407655542430352, 0.2906893188644473}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.9510565162951536, 0.30901699437494723`}, {
       0.8510565162951536, 0.30901699437494745`}}], 
      PolygonBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.8510565162951536, 0.35901699437494744`}, {0.8510565162951536, 
       0.2590169943749474}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{0.9510565162951535, 
       0.30901699437494745`}, {-0.5068835528549787, -0.7502384691456999}}], 
      PolygonBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.47749429024035495`, -0.7906893188644473}, {-0.5362728154696023, \
-0.7097876194269526}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.5877852522924734, -0.8090169943749472}, \
{-0.030901699437494757`, 0.9048943483704847}}], 
      PolygonBox[{{0., 1.}, {-0.07845452525225244, 0.920345198089232}, {
       0.016651126377262918`, 0.8894434986517372}}]}}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.03090169943749486, -0.9048943483704847}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.07845452525225254, -0.920345198089232}, {-0.016651126377262824`, \
-0.8894434986517372}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {-0.5568835528549786, 
       0.713911342745432}}], 
      PolygonBox[{{-0.5877852522924734, 
       0.8090169943749473}, {-0.6044363786697363, 
       0.6984604930266846}, {-0.5093307270402209, 0.7293621924641793}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.5877852522924734, 0.8090169943749473}, {
       0.8701548168576587, -0.25023846914570025`}}], 
      PolygonBox[{{0.9510565162951535, -0.30901699437494756`}, {
       0.8995440794722824, -0.20978761942695287`}, {
       0.8407655542430351, -0.2906893188644476}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{
       0.9510565162951535, -0.30901699437494756`}, {-0.8510565162951536, \
-0.30901699437494756`}}], 
      PolygonBox[{{-0.9510565162951535, -0.30901699437494756`}, \
{-0.8510565162951536, -0.35901699437494755`}, {-0.8510565162951536, \
-0.2590169943749476}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.9510565162951535, -0.30901699437494756`}, {
       0.5068835528549784, 0.7502384691457001}}], 
      PolygonBox[{{0.5877852522924731, 0.8090169943749475}, {
       0.4774942902403547, 0.7906893188644475}, {0.536272815469602, 
       0.7097876194269528}}]}}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425882358365126`*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " draws this graph with arrows of two different colors. This highlights the \
fact that one cannot travel an arrow to get from a red point to a green \
point. Thus, not every element can be written as a power of 4.\n\nDEFINITION \
2.1:\nWill say that the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", " ", "\[Element]", " ", "G"}], TraditionalForm]]],
 " is a ",
 StyleBox["generator ",
  FontSlant->"Italic"],
 "of the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " if every element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " can be expressed as a power of ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ".  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427545643625*^9, 3.427545656921875*^9}, {
  3.42754569078125*^9, 3.42754569834375*^9}, {3.42754572909375*^9, 
  3.427545738359375*^9}, {3.4275460016875*^9, 3.427546002953125*^9}}],

Cell[TextData[{
 "EXPERIMENT:\nUsing the circle graph, determine which elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "10"], TraditionalForm]]],
 " are generators. Is this set of numbers familar?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427545996640625*^9, 3.42754599740625*^9}}],

Cell[TextData[{
 "From this experiment one might notice a pattern.  Notice that 1 will always \
be a generator of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ", for Add[1] takes every point on the circle to the next point. See if you \
can use this fact to prove the next proposition."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427545992421875*^9, 3.427545993140625*^9}}],

Cell[TextData[{
 "PROPOSITION 2.1\nThe generators of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " are precisely the integers between 0 and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " that are coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427545780390625*^9, 3.427545780546875*^9}, {
  3.427545989109375*^9, 3.427545989625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is a generator of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". Then 1 is able to be expressed as a power of ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ", so we have that \n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", "v"], " ", "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 "\nfor some ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 ". Since the group action of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " is addition, raising to a power is equivalent to repeated addition, or \
standard multiplication.\nThus, we have that \n                               \
          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", " ", "v"}], " ", "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 ".\n                                         \nBy proposition 1.5, there is \
such a ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " if, and only if, ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".\n\nNow suppose that ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". By proposition 1.5, there is a ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " such that\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"v", " "}]], "=", " ", 
    RowBox[{
     RowBox[{"g", " ", "v"}], " ", "\[Congruent]", " ", 
     RowBox[{"1", "  ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", "n"}], ")"}]}]}]}], TraditionalForm]]],
 ".\nSo 1 can be expressed as a power of ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ". But 1 is a generator of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ", and so every element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " can be expressed as a power of 1, say ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["1", "w"], TraditionalForm]]],
 ". Then that element can be written as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"(", 
      RowBox[{"v", " ", "w"}], ")"}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["g", "v"], ")"}], "w"], " ", "=", " ", 
     SuperscriptBox["1", "w"]}]}], TraditionalForm]]],
 ".  So every element can be expressed as a power of ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ", hence ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is a generator of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427545813390625*^9, 3.42754587215625*^9}, {
  3.42754598409375*^9, 3.427545984875*^9}, {3.4362773798987503`*^9, 
  3.4362773927581253`*^9}}]
}, Open  ]],

Cell[TextData[{
 "The count of numbers less than n which are coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is called the ",
 StyleBox["Euler totient function",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and is denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "n", ")"}], TraditionalForm]]],
 ". Thus, the number of generators of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " is precisely \n",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "n", ")"}], TraditionalForm]]],
 ". Here is a small table of this function up to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "36"}], TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427545894453125*^9, 3.42754592484375*^9}, {
  3.42754597978125*^9, 3.427545980484375*^9}, {3.42754603959375*^9, 
  3.4275460426875*^9}}],

Cell[TextData[{
 "             Table of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "n", ")"}], TraditionalForm]]],
 ":\n\n",
 Cell[BoxData[
  FormBox[GridBox[{
     {"n", "|", 
      RowBox[{"\[Phi]", "(", "n", ")"}]},
     {"-", "+", 
      RowBox[{"--", "--"}]},
     {"1", "|", "1"},
     {"2", "|", "1"},
     {"3", "|", "2"},
     {"4", "|", "2"},
     {"5", "|", "4"},
     {"6", "|", "2"},
     {"7", "|", "6"},
     {"8", "|", "4"},
     {"9", "|", "6"},
     {"10", "|", "4"},
     {"11", "|", "10"},
     {"12", "|", "4"}
    }], TraditionalForm]]],
 "       ",
 Cell[BoxData[
  FormBox[GridBox[{
     {"n", "|", 
      RowBox[{"\[Phi]", "(", "n", ")"}]},
     {"-", "+", 
      RowBox[{"--", "--"}]},
     {"13", "|", "12"},
     {"14", "|", "6"},
     {"15", "|", "8"},
     {"16", "|", "8"},
     {"17", "|", "16"},
     {"18", "|", "6"},
     {"19", "|", "18"},
     {"20", "|", "8"},
     {"21", "|", "12"},
     {"22", "|", "10"},
     {"23", "|", "22"},
     {"24", "|", "8"}
    }], TraditionalForm]]],
 "         ",
 Cell[BoxData[
  FormBox[GridBox[{
     {"n", "|", 
      RowBox[{"\[Phi]", "(", "n", ")"}]},
     {"-", "+", 
      RowBox[{"--", "--"}]},
     {"25", "|", "20"},
     {"26", "|", "12"},
     {"27", "|", "18"},
     {"28", "|", "12"},
     {"29", "|", "28"},
     {"30", "|", "8"},
     {"31", "|", "30"},
     {"32", "|", "16"},
     {"33", "|", "20"},
     {"34", "|", "16"},
     {"35", "|", "24"},
     {"36", "|", "12"}
    }], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428445402546875*^9, 3.428445733359375*^9}},
 FontFamily->"Courier New"],

Cell[TextData[{
 "For larger values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", we can use the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command ",
 StyleBox["EulerPhi",
  FontWeight->"Bold"],
 ". For example, to find the number of generators of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "60"], TraditionalForm]]],
 ", type"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4275459561875*^9, 3.427545958109375*^9}}],

Cell[CellGroupData[{

Cell["EulerPhi[60]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["16"], "Output",
 CellChangeTimes->{3.4425882462583876`*^9}]
}, Open  ]],

Cell[TextData[{
 "How does ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " come up with this anwser? It uses a fairly simple formula based on the \
prime factorization of the number. Here is the formula, along with a proof of \
why this formula works:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427545973875*^9, 3.427545974421875*^9}}],

Cell[TextData[{
 "THEOREM 2.1:  THE TOTIENT FUNCTION THEOREM\nIf the prime factorization of \
",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is given by\n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", 
    RowBox[{
     SuperscriptBox[
      SubscriptBox["p", "1"], 
      SubscriptBox["r", "1"]], "\[CenterDot]", " ", 
     SuperscriptBox[
      SubscriptBox["p", "2"], 
      SubscriptBox["r", "2"]], "\[CenterDot]", 
     SuperscriptBox[
      SubscriptBox["p", "3"], 
      SubscriptBox["r", "3"]], "\[CenterDot]", " ", "\[CenterEllipsis]", " ", 
     "\[CenterDot]", " ", 
     SuperscriptBox[
      SubscriptBox["p", "k"], 
      SubscriptBox["r", "k"]]}]}], TraditionalForm]]],
 ",\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["p", "1"], ",", " ", 
    SubscriptBox["p", "2"], ",", " ", 
    SubscriptBox["p", "3"], ",", 
    RowBox[{"\[Ellipsis]", ".", " ", 
     SubscriptBox["p", "k"]}]}], TraditionalForm]]],
 " are distinct primes, and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      SubscriptBox["r", "1"], ",", " ", 
      SubscriptBox["r", "2"], ",", " ", 
      SubscriptBox["r", "3"], ",", "\[Ellipsis]"}],
     TraditionalForm], ",", 
    SubscriptBox["r", "k"]}], TraditionalForm]]],
 " are positive integers, then the count of numbers less then ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " which are coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "n", ")"}], "="}], TraditionalForm]]],
 "\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["p", "1"], "-", "1"}], ")"}], "\[CenterDot]", 
    SuperscriptBox[
     SubscriptBox["p", "1"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["r", "1"], "-", "1"}], ")"}]], "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["p", "2"], "-", "1"}], ")"}], "\[CenterDot]", 
    SuperscriptBox[
     SubscriptBox["p", "2"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["r", "2"], "-", "1"}], ")"}]], "\[CenterDot]", " ", 
    "\[CenterEllipsis]", " ", "\[CenterDot]", " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["p", "k"], "-", "1"}], ")"}], "\[CenterDot]", 
    SuperscriptBox[
     SubscriptBox["p", "k"], 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["r", "k"], "-", "1"}], ")"}]]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42754609440625*^9, 3.42754609478125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "To begin, let us show that if ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", 
     SuperscriptBox["p", "r"], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"p", " ", "-", " ", "1"}], ")"}], " ", 
     SuperscriptBox["p", 
      RowBox[{"r", "-", "1"}]]}]}], TraditionalForm]]],
 ".\n                                                    \nNote that the only \
numbers that are ",
 StyleBox["not",
  FontSlant->"Italic"],
 " coprime to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "r"], TraditionalForm]]],
 " will be multiples of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ".  So of the numbers between 1 and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "r"], TraditionalForm]]],
 ", exactly ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", "p"}], TraditionalForm]]],
 " of them will be multiples of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ".  The remaining ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", " ", "\[Dash]", " ", 
      RowBox[{"1", "/", "p"}]}], ")"}], "\[CenterDot]", 
    SuperscriptBox["p", "r"]}], TraditionalForm]]],
 " will be coprime, and this can be simplified to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "-", "1"}], ")"}], 
    SuperscriptBox["p", 
     RowBox[{"r", "-", "1"}]]}], TraditionalForm]]],
 ".\n\nNext we want to show that if ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " are coprime, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", 
     RowBox[{"n", " ", "m"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Phi]", "(", "n", ")"}], " ", 
     RowBox[{"\[Phi]", "(", "m", ")"}]}]}], TraditionalForm]]],
 ".  Let ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " denote the set of numbers which are less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", but coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".  Let ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " denote the set of numbers which are less than ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", but coprime to ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ".\n\nThen for any number coprime to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[CenterDot]", "m"}], TraditionalForm]]],
 ", that number, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"modulo", " ", "n"}], TraditionalForm]]],
 ", must be in the set ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", while that number, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"module", " ", "m"}], TraditionalForm]]],
 ", must be in ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ".  Yet for every ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", there is, by the Chinese remainder theorem, a unique number less than ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[CenterDot]", "m"}], TraditionalForm]]],
 " which is equalivalent to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", "n"}], ")"}], " "}], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", "m"}], ")"}]}], TraditionalForm]]],
 ".  This number will be coprime to both ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", and hence will be coprime to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[CenterDot]", "m"}], TraditionalForm]]],
 ".\n\nTherefore, we have a one-to-one correspondence between ordered pairs \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", and numbers coprime to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[CenterDot]", "m"}], TraditionalForm]]],
 ".  Thus, we have \n\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", 
     RowBox[{"n", " ", "m"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Phi]", "(", "n", ")"}], "\[CenterDot]", 
     RowBox[{"\[Phi]", "(", "m", ")"}]}]}], TraditionalForm]]],
 ".\n\nFinally, we can combine these results together.  By simply noting that \
if\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", 
    RowBox[{
     SuperscriptBox[
      SubscriptBox["p", "1"], 
      SubscriptBox["r", "1"]], "\[CenterDot]", " ", 
     SuperscriptBox[
      SubscriptBox["p", "2"], 
      SubscriptBox["r", "2"]], "\[CenterDot]", 
     SuperscriptBox[
      SubscriptBox["p", "3"], 
      SubscriptBox["r", "3"]], "\[CenterDot]", " ", "\[CenterEllipsis]", " ", 
     "\[CenterDot]", " ", 
     SuperscriptBox[
      SubscriptBox["p", "k"], 
      SubscriptBox["r", "k"]]}]}], TraditionalForm]]],
 ",\n          \n then  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox[
      SubscriptBox["p", "1"], 
      SubscriptBox["r", "1"]], ",", "  ", 
     SuperscriptBox[
      SubscriptBox["p", "2"], 
      SubscriptBox["r", "2"]], ",", " ", 
     SuperscriptBox[
      SubscriptBox["p", "3"], 
      SubscriptBox["r", "3"]], ",", "  ", "\[CenterEllipsis]", " ", ",", "  ",
      
     SuperscriptBox[
      SubscriptBox["p", "k"], 
      SubscriptBox["r", "k"]]}]}], TraditionalForm]]],
 " will all be coprime.  Hence, we can find \[Phi] for each of these terms, \
and multiply them together, giving us our formula."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4362774823831253`*^9, 3.4362774826800003`*^9}, {
  3.4362775251331253`*^9, 3.4362775341643753`*^9}},
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]]
}, Open  ]],

Cell[TextData[{
 "Now that we know how to find generators of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ", let us move on to other groups. Consider for example ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["Z", "10", "*"], "."}], TraditionalForm]]],
 " First we load this into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["DefMultMod[10]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["\<\
The elements are {1, 3, 7, 9}. Is 1 a generator of this group? Let's check:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[{1,3,7,9}, Mult[1]]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"1\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"7\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], InsetBox[
    TagBox[
     StyleBox["\<\"3\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}],
    InsetBox[
    TagBox[
     StyleBox["\<\"9\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 
      18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, 
    {RGBColor[0, 1, 0], 
     LineBox[{{0.9292893218813453, 0.07071067811865481}, {0.8968331206248827, 
      0.09241885630108182}, {0.8585786437626906, 0.09998489885977785}, {
      0.8203241669004984, 0.0924188563010818}, {0.7878679656440357, 
      0.0707106781186548}, {0.7661597874616087, 0.038254476862192265`}, {
      0.7586644555810313, 4.645047007742779*^-17}, {
      0.7661597874616087, -0.03825447686219217}, {
      0.7878679656440357, -0.07071067811865471}, {
      0.8203241669004984, -0.09241885630108172}, {
      0.8585786437626906, -0.09998489885977777}, {
      0.8968331206248827, -0.09241885630108171}, {
      0.9292893218813453, -0.0707106781186547}, {1., 
      6.123233995736766*^-17}}], 
     PolygonBox[{{1., 6.123233995736766*^-17}, {0.9646446609406726, 
      0.10606601717798217`}, {0.8939339828220179, 0.03535533905932743}}]}, 
    {RGBColor[0.6, 0.4, 0.8], 
     LineBox[{{0.07071067811865486, -0.9292893218813453}, {
      0.09241885630108188, -0.8968331206248827}, {
      0.09998489885977792, -0.8585786437626906}, {
      0.09241885630108186, -0.8203241669004984}, {
      0.07071067811865485, -0.7878679656440357}, {
      0.03825447686219231, -0.7661597874616087}, {
      9.290959971541795*^-17, -0.7585937449029128}, {-0.03825447686219212, \
-0.7661597874616087}, {-0.07071067811865465, -0.7878679656440357}, \
{-0.09241885630108167, -0.8203241669004984}, {-0.0999848988597777, \
-0.8585786437626906}, {-0.09241885630108165, -0.8968331206248827}, \
{-0.07071067811865464, -0.9292893218813453}, {
      1.2246467991473532`*^-16, -1.}}], 
     PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
      0.10606601717798224`, -0.9646446609406726}, {
      0.03535533905932749, -0.8939339828220179}}]}, 
    {RGBColor[1, 0.6, 0], 
     LineBox[{{-0.9292893218813453, -0.07071067811865492}, \
{-0.8968331206248827, -0.09241885630108193}, {-0.8585786437626904, \
-0.09998489885977796}, {-0.8203241669004983, -0.09241885630108192}, \
{-0.7878679656440357, -0.07071067811865489}, {-0.7661597874616087, \
-0.038254476862192355`}, {-0.7586644555810313, -1.3935141023228338`*^-16}, \
{-0.7661597874616087, 0.03825447686219208}, {-0.7878679656440357, 
      0.07071067811865461}, {-0.8203241669004984, 
      0.09241885630108161}, {-0.8585786437626906, 
      0.09998489885977765}, {-0.8968331206248827, 
      0.0924188563010816}, {-0.9292893218813453, 
      0.07071067811865459}, {-1., -1.8369701987210297`*^-16}}], 
     PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.035355339059327535`}}]}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425882536021376`*^9}]
}, Open  ]],

Cell[TextData[{
 "We see that 1 is not a generator because 1 is the identity element.\n\n\
EXPERIMENT:\nReplace Mult[1] in the last command with Mult[3], Mult[7], and \
Mult[9] to see which elements, if any, are generators. Recall that the test \
for a generator is that any element can be reached by any other element in \
the circle graph by means of the arrows. How many generators are there?\n\n\
According to the table for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", 
    RowBox[{"(", "n", ")"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " also has four elements. Let us try the same experiment with ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ". First we load the group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4362778126331253`*^9}],

Cell["DefMultMod[8]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["\<\
The elements of this group are {1, 3, 5, 7}. Let us check to see if 3 is a \
generator:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[{1,3,5,7},Mult[3]]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"1\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"5\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], InsetBox[
    TagBox[
     StyleBox["\<\"3\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}],
    InsetBox[
    TagBox[
     StyleBox["\<\"7\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 
      18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], {{
     {RGBColor[1, 0, 0], 
      LineBox[{{0., 1.}, {0.9292893218813453, 0.07071067811865481}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {0.9646446609406726, 
       0.10606601717798218`}, {0.8939339828220179, 0.03535533905932743}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{1., 6.123233995736766*^-17}, {0.07071067811865477, 
       0.9292893218813453}}], 
      PolygonBox[{{0., 1.}, {0.03535533905932738, 0.8939339828220179}, {
       0.10606601717798213`, 0.9646446609406726}}]}}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.9292893218813453, \
-0.0707106781186549}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.03535533905932753}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {-0.07071067811865464, \
-0.9292893218813453}}], 
      PolygonBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.03535533905932728, \
-0.8939339828220179}, {-0.10606601717798202`, -0.9646446609406727}}]}}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425882611177626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "\nEXPERIMENT:\nReplace Mult[3] in the above command with Mult[5], Mult[7], \
and Mult[1]. How many generators are there to this group?\n \nBy doing this \
experiment, you will discover that ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " has a totally different structure than ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "10", "*"], TraditionalForm]]],
 ", in spite of the similarity in how they are defined. As you tried to \
determine why no element of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " acts as a generator, you might notice that every element squared is equal \
to 1. This is seen in the multiplication table"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[{1,3,5,7}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}}, {{0, 1,
     1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {
    1, 1, 0}, {1, 0, 0}, {0, 1, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}, {1, 0, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
    0}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.4425882641333876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "Because of this property of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ", no element can generate the whole group. \n\nFrom these examples, we see \
that some groups have generators, while others do not. This leads us to the \
following definition."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427546181609375*^9, 3.42754618740625*^9}}],

Cell[TextData[{
 "DEFINITION 2.2:\nWe say a group is ",
 StyleBox["cyclic",
  FontSlant->"Italic"],
 " if there is one element which can generate the entire group. \n\nAlthough \
we have seen an example of a finite group that is not cyclic, we will later, \
we will see that the structure of ",
 StyleBox["any",
  FontSlant->"Italic"],
 " finite abelian group can be expressed in terms of the cyclic groups.\n\n\
Even when a group is not cyclic, we sometimes can find ",
 StyleBox["two",
  FontSlant->"Italic"],
 " elements by which every element of the group can be expressed. For \
example, consider the group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ", and the two elements 3 and 5. The following command maps both Mult[3] and \
Mult[5] on the same circle graph."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42754617425*^9, 3.427546176046875*^9}, {
  3.427554070828125*^9, 3.427554072390625*^9}}],

Cell[CellGroupData[{

Cell["CircleGraph[{1,3,5,7},Mult[3],Mult[5]]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"1\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"5\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], InsetBox[
    TagBox[
     StyleBox["\<\"3\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}],
    InsetBox[
    TagBox[
     StyleBox["\<\"7\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 
      18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], {{
     {RGBColor[1, 0, 0], 
      LineBox[{{0., 1.}, {0.9292893218813453, 0.07071067811865481}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {0.9646446609406726, 
       0.10606601717798218`}, {0.8939339828220179, 0.03535533905932743}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{1., 6.123233995736766*^-17}, {0.07071067811865477, 
       0.9292893218813453}}], 
      PolygonBox[{{0., 1.}, {0.03535533905932738, 0.8939339828220179}, {
       0.10606601717798213`, 0.9646446609406726}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.9292893218813453, \
-0.0707106781186549}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.03535533905932753}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {-0.07071067811865464, \
-0.9292893218813453}}], 
      PolygonBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.03535533905932728, \
-0.8939339828220179}, {-0.10606601717798202`, -0.9646446609406727}}]}}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0., 1.}, {1.1634144591899854`*^-16, -0.9}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.05000000000000012, -0.9}, {-0.049999999999999885`, -0.9}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{1., 
       6.123233995736766*^-17}, {-0.9, -1.7145055188062944`*^-16}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9, \
-0.05000000000000017}, {-0.9, 0.049999999999999836`}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {6.123233995736766*^-18, 
       0.9}}], PolygonBox[{{0., 1.}, {-0.049999999999999996`, 0.9}, {
       0.05000000000000001, 0.9}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {0.9, 
       4.898587196589413*^-17}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {0.9, 0.05000000000000005}, {
       0.9, -0.049999999999999954`}}]}}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{
  3.427554663828125*^9, {3.4275548686875*^9, 3.427554890890625*^9}, 
   3.4425882681958876`*^9}]
}, Open  ]],

Cell[TextData[{
 "The red arrows represent multiplying by 3, while the green arrows represent \
multiplication by 5. Notice that by a combination of red and green arrows, \
any element of the group can reach any other element through a sequence of \
arrows, even though neither the red arrows alone nor the green arrows can do \
this job. We say that the ",
 StyleBox["set",
  FontSlant->"Italic"],
 " {3,5} generates the group.\n\nLet us look at one more example. Consider \
Terry's group, which is loaded into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = InitTerry", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "Stay", ",", "FlipRt", ",", "RotRt", ",", "FlipLft", ",", "RotLft", ",", 
   "Spin"}], "}"}]], "Output",
 CellChangeTimes->{3.4425882708521376`*^9}]
}, Open  ]],

Cell[TextData[{
 "By setting ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " equal to the output, we can now abbreviate that list of elements by just \
",
 StyleBox["G",
  FontWeight->"Bold"],
 ". Let us see if one of the commands will generate the group."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[G, Mult[Spin]]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8660254037844386, 0.5000000000000001}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8660254037844387, -0.4999999999999998}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.8660254037844384, -0.5000000000000004}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.866025403784439, 0.49999999999999933`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"Stay\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"FlipLft\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"FlipRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9526279441628825, 0.5500000000000002}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"RotRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9526279441628827, -0.5499999999999998}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"RotLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9526279441628823, -0.5500000000000005}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"Spin\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.952627944162883, 0.5499999999999993}, {1, 0}]}, {{
     {RGBColor[1, 0, 0], 
      LineBox[{{0., 1.}, {-0.7794228634059952, 0.5499999999999994}}], 
      PolygonBox[{{-0.866025403784439, 
       0.49999999999999933`}, {-0.7544228634059952, 
       0.5066987298107775}, {-0.8044228634059952, 0.5933012701892213}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-0.866025403784439, 
       0.49999999999999933`}, {-0.08660254037844385, 0.95}}], 
      PolygonBox[{{0., 1.}, {-0.11160254037844387`, 
       0.9933012701892219}, {-0.06160254037844383, 0.906698729810778}}]}}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.8660254037844386, 0.5000000000000001}, {
       0.8660254037844387, -0.3999999999999998}}], 
      PolygonBox[{{0.8660254037844387, -0.4999999999999998}, {
       0.9160254037844386, -0.3999999999999998}, {
       0.8160254037844387, -0.3999999999999998}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.8660254037844387, -0.4999999999999998}, {0.8660254037844386,
        0.40000000000000013`}}], 
      PolygonBox[{{0.8660254037844386, 0.5000000000000001}, {
       0.8160254037844387, 0.40000000000000013`}, {0.9160254037844386, 
       0.40000000000000013`}}]}}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.7794228634059945, \
-0.5500000000000004}}], 
      PolygonBox[{{-0.8660254037844384, -0.5000000000000004}, \
{-0.8044228634059944, -0.5933012701892224}, {-0.7544228634059945, \
-0.5066987298107785}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.8660254037844384, -0.5000000000000004}, \
{-0.08660254037844377, -0.9500000000000001}}], 
      PolygonBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.061602540378443774`, \
-0.906698729810778}, {-0.11160254037844376`, -0.993301270189222}}]}}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425882727740126`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "\nEXPERIMENT:\nBy replacing the \"Spin\" in the last command with the other \
elements of Terry's group, see if any of the elements generate the group. If \
not, see if you can find a combination of ",
 StyleBox["two",
  FontSlant->"Italic"],
 " elements which generate the group."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "One of the keys for entering a group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is knowing one or two elements (or sometimes even three) which will \
generate the entire group. This information begins to reveal the structure of \
the group itself."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Defining Finite Groups in ",
 StyleBox["Mathematica ",
  FontSlant->"Italic"],
 "and GAP"
}], "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427555165765625*^9, 3.427555188515625*^9}, {
  3.42755527209375*^9, 3.42755527440625*^9}}],

Cell[TextData[{
 "This part of the notebook shows how to define finite groups in only ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". The corresponding section in the textbook explains entering groups into \
GAP.\n\nWe saw in the last section that for some groups there is a single \
element that generates the entire group, whereas in other groups two or more \
elements are required. In this section we will show how a finite group can be \
entered into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using a set of elements which generates the group. Suppose that a cyclic \
group of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " elements has a generator which we will call ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "."}], TraditionalForm]]],
 " From the circle graphs of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " that we saw in the last secton, we could see that the sequence \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"e", "  ", "=", " ", 
    SuperscriptBox["x", "0"]}], TraditionalForm]]],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "  ", "=", " ", 
    SuperscriptBox["x", "1"]}], TraditionalForm]]],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "x"}], " ", "=", " ", 
    SuperscriptBox["x", "2"]}], TraditionalForm]]],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "x", "\[CenterDot]", "x"}], " ", "=", " ", 
    SuperscriptBox["x", "3"]}], TraditionalForm]]],
 "\n\[CenterEllipsis]\[CenterEllipsis]\[CenterEllipsis]\[CenterEllipsis]\
\[CenterEllipsis]\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "x", "\[CenterDot]", "x", "\[CenterDot]", "x", "\[CenterDot]", " ", 
     "\[CenterEllipsis]", " ", "\[CenterDot]", "x"}], " ", "=", " ", 
    SuperscriptBox["x", 
     RowBox[{"n", "-", "1"}]]}], TraditionalForm]]],
 "\n\nmust mention every element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " exactly once. In fact, this gives us a way to label the elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " by relating each element to the power of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " gives that element. We also find that \n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "n"], " ", "=", " ", "e"}], TraditionalForm]]],
 ".\n\nThus, we can define the group merely by saying \"",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is a generator of the group, and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the lowest number such that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "n"], TraditionalForm]]],
 " is the identity.\" \n                           \nThere are ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " routines included which allow us to quickly make these definitions. \
Execute the following two statements:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427555299171875*^9, 3.427555299390625*^9}, {
  3.42755543934375*^9, 3.427555483671875*^9}, {3.427555520234375*^9, 
  3.4275555925625*^9}}],

Cell[CellGroupData[{

Cell["InitGroup[e]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["e"], "Output",
 CellChangeTimes->{3.4425882797271376`*^9}]
}, Open  ]],

Cell["Define[x^5, e]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "What does this do? The first command we have seen once before: it defines \
the identity element to be ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ". The second command defines ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "5"], TraditionalForm]]],
 " to be the identity. This is all we need to define the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 ".\n\nOf course we will want to see the group. This can be done by typing \
the following command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Z5 = Group[{x}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "x", ",", 
   InterpretationBox["\<\"\\!\\(x\\) \[CenterDot] \\!\\(x\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`x, $CellContext`x],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(x\\\\) \\\\[CenterDot] \
\\\\!\\\\(x\\\\)\\\"\\) \[CenterDot] \\!\\(x\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`x, $CellContext`x], $CellContext`x],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(x\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(x\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(x\\\\)\\\"\\) \[CenterDot] \\!\\(x\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`x, $CellContext`x], $CellContext`x], $CellContext`x],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425882861490126`*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives a list of all of the elements in the group.  Note that the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in curly braces. (We will see why later.) Thus, the elements of this \
group are:\n\n",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 "\n",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "x"}], TraditionalForm]]],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"x", "\[CenterDot]", "x"}], "\[CenterDot]", "x"}], " "}], 
   TraditionalForm]]],
 "           and\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "x", "\[CenterDot]", "x", "\[CenterDot]", "x", "\[CenterDot]", "x"}], 
   TraditionalForm]]],
 "        \n\nA multiplication table can be given by:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[Z5]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0, 1, 0}, {0, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
    0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {1, 1, 0}, {1, 0, 
    1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {1, 1, 
    0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
    0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
    0}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"x\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"x\[CenterDot]x\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\[CenterDot]x\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"x\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"x\[CenterDot]x\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\[CenterDot]x\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"x\[CenterDot]x\[CenterDot]x\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 5}, {6, 5}}]}]], "Output",
 CellChangeTimes->{3.4425882892583876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Although the {0, 1, 2, 3, 4} notation is more concise for this particular \
example, the use of generators is more versitile, since almost all finite \
groups can be exprerssed in an easy way using generators.\n\nFor example, \
consider the group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ".  We saw that every element squared is the identity so this group is not \
cyclic. However, we found that every element can be expressed in terms of two \
of the elements, 3 and 5. Hence the ",
 StyleBox["set",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"3", ",", " ", "5"}], "}"}], TraditionalForm]]],
 " generates the group.\n\nBy calling ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "=", "3"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", "5"}], TraditionalForm]]],
 ", we can define the group this way:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["InitGroup[e]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["e"], "Output",
 CellChangeTimes->{3.4425882987896376`*^9}]
}, Open  ]],

Cell["Define[a.a,e]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["Define[b.b,e]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "Is this enough to define the group? Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"7", " ", "=", " ", 
     RowBox[{"3", "\[CenterDot]", "5"}]}], ","}], TraditionalForm]]],
 " the last element can be written ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 ". The list of four elements would be "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = {e,a,b,a.b} ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425883048677626`*^9}]
}, Open  ]],

Cell["Let us see the multiplication table:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 0, 0}, {0, 0, 0}, {1, 0, 1}, {0, 0, 0}}, {{0, 1,
     1}, {0, 1, 1}, {0, 0, 0}, {1, 1, 0}, {0, 0, 0}}, {{1, 0, 1}, {1, 0, 1}, {
    1, 1, 0}, {1, 0, 0}, {0, 1, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}, {1, 0, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
    0}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], 
     InsetBox["\<\" \"\>", {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], 
     InsetBox["\<\" \"\>", {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], 
     InsetBox["\<\" \"\>", {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], 
     InsetBox["\<\" \"\>", {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.4425883071802626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "Notice the four black squares, such as one corresponding to b times a. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " needs to be told that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 " is the same thing as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 ", so we need a third define statement:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Define[b.a,a.b]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "Will this be enough for ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find all products? Check out the multiplication table again."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}}, {{0, 1,
     1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {
    1, 1, 0}, {1, 0, 0}, {0, 1, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}, {1, 0, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
    0}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.4425883117583876`*^9}]
}, Open  ]],

Cell[TextData[{
 "This time, there are no black squares, so the group has been completely \
defined in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". We can also find out if we have finished defining the group by entering \
the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = Group[{a, b}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425883150083876`*^9}]
}, Open  ]],

Cell[TextData[{
 "As long as Mathematica returns with a list of elements, the group has been \
sucessfully been defined. Notice that since we needed two generators for this \
group, we needed to include both generators in the Group command, listed as a \
set. \n\nThe ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command ",
 StyleBox["Group",
  FontWeight->"Bold"],
 " always expects a list of generators, not just one generator. This is why \
when we had a single generator ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", we enclosed the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in curly brackets.\n\nWe can define several groups in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " at the same time, and by listing the generators, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " knows which group we are refering to. As long as the identity element is \
the same for all of the groups and we use different letters for the \
generators, there is no problem. However, the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["InitGroup[e]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["e"], "Output",
 CellChangeTimes->{3.4425883189927626`*^9}]
}, Open  ]],

Cell["\<\
clears out all previously defined groups. (In fact, all commands begining \
with Init will clear all previous definitions.)

Here is another example: Suppose we have three different colored books on a \
shelf, as illustrated by executing the following command:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["InitBooks[3]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0, 1, 1], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[1, 0, 0], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883213833876`*^9}]
}, Open  ]],

Cell["\<\
Suppose we want to rearrange the three books. One way we can rearrange them \
is to swap the first two books. This can be done with the command\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MoveBooks[First]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[1, 0, 0], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[0, 1, 1], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883249615126`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "Another way we can rearrange the books is to take the first book out, slide \
the other two to the left, and replace the book ",
 StyleBox["after",
  FontSlant->"Italic"],
 " the other two. This can be done by the command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MoveBooks[Left]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0, 1, 1], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[1, 0, 0], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883270083876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztfQt0HFWaXln9fqhVetltyZZbGNvCGNM2xjRg5Db4IcsPhJ/CxkYYzEg8
G+wZiWfztAxi6Fl2wRuys9phd4/2MIcoG85GuyebdSbJiXaYbBR2sxEzyRmz
J7NRzswwsjFYM+MZKve/f1V3lf1XdZWq22r1qWOrW+qu+r/H/eve/966Le24
73j30UfvO95z/32xLU/el+ruuf9YbPPjT7KXXHMEYc73BaHiZEyA7yVBUB4k
9l/6CB7glSJ97+Pfu8XJycnk6dOnla/TqkMq+PfelvHx8Y7h4eEX0un0aFsi
8bNYXd0vRZ/PNZkURcEPnC+IgcBYMh7/i872djisa3BwEKJFJyYmRjDib/hT
MDE6Otp96tSpd1I9PXDGRb/bfa4lEjnbUVUlpRnwCPs6IwgeOEeISJPsx9Ps
a4h9sbfPd4XDAPtlNBiEA8YTLS0QBkD7+vv7ATQ+Njb2rxH0An8KKSLhKKD4
43hzM5w9mWCRulmkjIwyxV6skZFH5dcY6sU+txtQz8YZSfYmOACo3+k+fBhi
gjsQ3z81NfVniHyeP/nbRkZG/qZjw4Zz0UgEPIMoXCboGROERRzMC4o52CkE
BDfg0AqwR3AB1lOZEyd4zD9BgHP8yQPQIGeyjQUeZmdPsHjX8qgukAPS4HTh
T/G0SWx2aCBQwW1nPK6XzzgtH60C+jkCgbbjmYEBUAKtAJSFVfqn/RRPA+tP
pnt7P49WVn6RCoU4vxv0T/t/eBokzrupo0cnY6I4lfb7IQ+EG/VP+794WuzM
mTPf7dqz51wsErmYcbv5aTflTvvH5PLl6tN+gqdBxkBenI8xXYNyHtzMT3Pz
ZkkKwrDU0RGXxsb+GM/8P/zJBa3Cs4K1p7BObkz5DH4dSbkzv4Nn/hOeCX7+
ZeeWLedSkYjQmqOonMl+4o/K2fysP8QQn2IIUPujxJIlvx7yeIT1RAgPEeIP
MMQZDAFZ+9N4fT3kgJAkQviIEO9iiB/nLOBpDV5vIEIE8LW/G1y9+lvpxx/n
Z/0+hvjfhBd36Ifg12LM7eZh5BD/i/Bio16ICogguCbTopil8g7G+RFhyKY8
cfgFpcSS4/yQcGWzYZwKfFdD6vcw2DjhT5thMBcR7Hcx2P/EYHBFZp3aahjM
TQR7SxPs+21r1/5qxOvlBxoH8xDBfgeD/SMGe6evp+fz/spKfmC7YTAvEexb
GOx/YLC9w++//1lHDfbm2wyD+YhgGQz2DxgMhsDP4iwYdAzbDYP5iWBvYrC/
zzXAT+KNjbwH3WEYLEAE+6YmmKYBjIMFiWBvYLCPiQbYaRgsRAQbwGD/nWiA
Ow2DhYlgr2OwMaIBOgyDVRLBXsNg/41ogLsMg0WIYCc1wTQNYBysigjWj8H+
jmiAXYbBRCIYFgjCfyUaYLdhsGoi2KsY7Ae5LuizGAsG4/cew2A1RLBXNMGg
aOGVh5Q3WC0R7GVNMF7MjcknGgerI4K9hME+IpjtNQxWTwR7URNMw8w42Fwi
2AsY7PsYDApnqFf5gfsMg80jgqU1waAk5dVi/mBRItjzGOxvMRhUT7wGggP3
GwabTwR7ThNMw8w4WAMR7FkMNporCnhNBwd2GgZrJII9owkG5SivKfMHW0AE
exqD4fSA17cwN0hlMhkoOYCpMB+B4D0oYHlfd7ch0EIC6CkEwjq1GhoHiv2B
433Hx+LJ+IWAGPgyGA1Oim3iL30p36SYFM+HY2GI888NLQ1wzDs9qR6Y1gBB
SDuhhnDAmFgTQawPiX2bP4WgQwHl4MB393TtGW9JtMA5Z6viVefDXWHeNQps
OsqmLGy4h6lxRIKCiL/GhmeW0iBiUkyIcB4IA/J/0d6pmYJCZx/I9ftfRNkc
BIw9YMh/EcG/F/lj3RmEFgOD+Kw40Zb4WV2s7qLb7wZDwVhJGGQcR1kHJlOf
lKkPceogEQ8VfQAF8oG+eiILgx7OwV0w8+LTJykv9RhB/RtI/W3+hJPSDR0b
gC60PdjIewiBzdXZzCDGKQeBPVJOCxfdfVzZuUhLBML/vHZR7dRcNi9jw8FB
QzrNBJ2vIx2sS3EK2xxvRhrDDI+1zwrOwSUJ/0r4qr5iDg9iDHQVAXQcgbDM
xEkva6azVR1sGj/GJr0yxn8yi7GYwDiGGFjj4TR5IDMAOJjGZ2CabBXoagLo
SQTCkg0n1r3p3s9qYjVfhFIhDnSDZaAlBNATCPR6rqvKpPpSk2JMnPKn/ZIw
AVNxq0BLCaAUAmFBxSfv7x5NHT0XiUUuuvvd/Mq/yTLQMgIIH+X6CFcJWJcD
uX/RnXHzhLs5ByTVVQgmgFoIoMcQCGsn16H3ht6bFFtE6AuEdZYBriEAHkUA
rKdc3z6SOvKL6s5qruA2ywDLCYBHEOClXJeJw0YmIKy3DHAtAfAwAryYKx5+
0hhvhI5GSFoGWEEAPIQAL+Ta4Oe1iVp+fWywDHAdAdCDAFgFueGKH+rs7lTa
4XbLGCsJjG7EeC5XEvzDiuSKC4FTAWGjZYDrCYCvIcCzuepyIhqPQp8vbLIM
ECcAHkQALKJcT7x56s2f1SXreO+x2TLAKgLgKAI8pW2GSbFbhGbYYhljNYHx
AGJg6cInQR+vTK7kzbDVMsANBMD9GoC/3NK55deeIQ8PbR1gDQFwBAGweHE9
dSJz4lwkFeGh2y0D3EgA3IcAWGLw9TMoFHjobZYB1hIAXQiARQOv6X9aH6/n
o8N2ywA3EQD3IsDxXCP/aEliCVQIwg7LAAkC4LAGQNPI1gFuJgAOIcAxopF3
Wga4hQC4BwGeJBr5TssAtxIABxHgCaKROywDrCMADiBAimjkuywD3EYA3K0B
0DSydYBWAqATAR4nGnmXZYD1BMB+BHiMaOTdlgGSBMA+DQDU/OD/NAE2EAB7
EQCLJFzM4bW+BCtDVgFuJwD2aABgEoUzlmkB3EEA7EaAR3LVEUyIeei9lgE2
EgC7NAAaBdYBNhEAdyHAw4SCfZYBNhMAHRqA7xzuPiwJp4RpAmwhAO5EgIdy
tRdMSXiNvd8yQBsBsFMDAHMenIZI0wHYSgDsQIAeQkGnZYB2AmC7BuCVZ9LP
XHSnUYF1gG0EwDYEaOdPmi0I6pvt0VxvDvNgXlzebRl/O4GPwMIm/lQDeHzX
Qk8qxaZDSTj+XKSlhfUubHRKp+GZ9ZXJi24/3w8Bx4wm2tqAL5ynLPFVI19Y
IMA5uzQdvjsIvluR73r+FIQWz254aO/s/OeGFr64Bhy/DHZ3S0Imw2Zcp/2w
z0OohG8lYWQEpMDbcBibHMfgFDgV5IB0CAdh+aJggPD+gGUtOwktbajlFtQC
PUjn0NAQgAORc5Fo9MtgNAokgbAkDA1Jwvh4HddSDd+iHpCYTsNhk2IiATAX
AqIIIcASCAdhlTVCAIOmYvqDjLNVHXcSOragjpv4k7Lq1gHHsR4xfi7S2Yn8
gezERIzzD7KSZxJfAlnpNBwGGn7pE/k655Q/GoBR0zrFDoLiZqR4I39yyytx
8UmxrY2NCsMrOSUXnCbJ/6bRxHcRuHhdCTcgrrwwF8PLaWwsXhDcXQTuRsTF
vSPKYl0GUup8uKtLEs6cYdPAQoDvJsDvQPCVCC4v4KXZxRNjnUEKcoDN4AoB
vocAvx3BVyC4vKjXNynGYlN+SEIGvrYg4HsJ8A0Ijntv+PobG7a72ZgUu+ju
74eUZ/OmQoDvI8CTCN6SA//unq4u7N9OnWKD4hSb8hQCfD8Bjh2ysAw7y73v
Dw+zeQ3L78FBvl5nH7STAG1F0KUI+sYxSDK4spjNrQUBvZsAvQ1Br0ZQyO7v
tXZ0nA+n03xdzz7oAQJ0HYIuzhWbP25OJH7tGR7ma332QQ8SoLci6FUIeui9
oaGf18IAMz7OF//sg95DgOJoKDQj6LePpFK/qIYBhGXvHQUBPUSA3oygi3Jt
CsPnhUAmwxcJ7YMeJkATCNqUa9OfNMIlc/o0Xzi0D3ovAYqDtLDw0jZlA8KW
goB2EaBrEXQB0aZtBQG9jwDFsV5oyJVv31/b1sbbtL0goEcI0DUaUFAKgByu
MKD3E6BYXSi31Y++Ozh4tgoGefbTtoKAPkCArkZQeWYClfLP6qBQZW26vSCg
RwnQVQg6T8jO9/6pSc7eHQUBfZAAxSJKmJtLpB+sgbqRXac7CwL6NQL0eg2o
JpEKA9pNgGLFJtQTiXRnQUB7CNDrELSOSKSOgoA+RIBieSjUEol0V0FAHyZA
r0XQGiKRdhUE9BECdLkGVJNIhQF9lAC9BkGriUTaXRDQxwhQLHwFMTee4tDG
EmlPQUAfJ0CXaUBx3js8zOEKA5oiQLHgFapySmGizeH2FgT0CQJ0iQZUo7Qw
oE8SoFhlC5HcdQqTKd6m+woCeowAXawBffdoKnXRLV8yhQE9ToBilS1UEkr3
FwT06wRoswZUo7QwoN8gQGMIGs51g59XRqN81tZZENBeAnSRBhQWQ3Atgv1U
GNA+AhRLeyGUUwqLIHwN4u6CgD5FgC7UgGqUFgb0aQJ0gQYU1rhgLC0g6DME
aCOCBhFUvguGNcOBgoA+S4A2aEBhFRFW8goI+hwBiqW9shStUXqwIKDPE6BR
Deh4C4ylo6MFBE0ToPM0oJpEKgzoCwTo3GKDvkiA1hcb9CUCtK7YoC8ToLXF
Bn2FAK0pNuirBGh1sUFPEKBisUH7CdCqYoOeJEAjxQZ9jQCtLDbo6wRouNig
AwRoqNigbxCgwWKDfpMADRQb9E0C1F9s0IwMmhBFfucacHxXCjQpiknl1yo4
oA6oA+qAOqAOqAPqgDqgDqgD6oA6oA6oA+qAOqAOqAPqgDqgDqgD6oA6oA6o
A+qAOqAOqAPqgDqgDqgD6oA6oA6oA+qAOqAOqAPqgDqgDqgD6oA6oA6oA+qA
OqAOqAPqgDqgDqgD6oA6oA6oA+qAOqAOqAPqgDqgDqgD6oA6oA6oA1pCoPh3
JYPSGXbcMPtiSFKbKEoMRuoKh/nPQ+wLSE0KQt1McMQ/cDpXGmXHnWJfKZ+P
8/O73VKsrk7q2LBBSqfT0sjIiMRgpcHBQf5zZ3u7lIzHJTEQ4BhwDmgDTRlZ
07ggVM6EJvzTvyHwlAP3s6/OSESKV1XxcxMtLVL34cNSJpPhmqamptjRgFfJ
fwatoDHV08M1tjQ08PNaWAzQCR6BzhGML3hnQuR1cnKNq5ILyEWDQd4oQLzv
+HFpaGhIGhsbi3J9EWl8fJxrBO2gsS2R4IcCnCi3PXgF4QbltkwCTpHl4N8W
93EPIY/AcyX3gO/ExIRwNdfgkyYnJ/lrIA3e79qzR1qzbJnU4vcXn+g1/NHN
PU/AxczMYwmzknNzcV7g4RXJgWUyl0E5OcGw0dHR+ExwuZo/eqQpSBroPFgH
Bw3DEg4uyCtP6CqZEPQCab9fioZCUuroUenMmTPwp7WvPKGYTGgCullGJlpZ
KaV7eyGd4S9lZwndwrqpK0KoSSYE4xOMR1GWQpmBAd4fJjght/Thhx9KQZdL
6mLHsOP8xea0QE5paLXO6mo+BrEx51ZOx5ttMMj4GNBGWt5i02qUu3jobRPz
5klD770Hwzt71a9OIv51CTd3sbk1yNyGPR4p0dzMr7hWIsFdqt5Cxa+i2Pzm
y+hplmIdra0839cT/Dx6/IpHbZ5MDbKto6ZGSh87Bn+J+jJqvkuoxXDALRKr
uZck2/D778Ofr+bnwtiXZPXLmNJBsNcG5bqDqcj+St4CU6oncmyjAaUxFaVv
FoeSUi6r02qzSUpvFIdSDZFOW01SGigOpWoil7aZpPR6cSgpvZE6l3aYpPRa
cShVyVAjXq/UtmYNz6U7TVI6WRxKERkKBud4YyOndJdJSv3FoaRM5qDEi7Nx
GRput0lKJ4pLCa42ltqczB6TlF4tDqWwDNXPyru+Rx/lZPaZpPRKcSiF1Omd
wNKu0ySll4tDKahO76Ymnt4HTFJ6qTiUEApLzxZYvriNVXhTUwdNsnqxmKw8
0mp23O1yhSm1tQEzwSy1F4pDza9kOjtuGxZsWE1CqYkUOUGz7ZouDk2leIPl
mja5Pu9SyuHpMX2+OEy9clvziwKhFLY4jZgm3eeKQ9cjk56S6Y6rJkBdytvT
Y/xscRl3C7jaKWkZe20wfqY4jJVmn5BnHVOqa0xmrUqd6VF/urjUUwIuI0o6
1H02qD9VHOouwnUXQd1vg3pfcan3CbjwLelQD9ig3lsc6kpyTMmuT6iSSE09
aIP6N4pLHfI8JWjzX009ZIP61wtF3aemzr/UjnsI2mEbtI8Xirb3Mtqy216C
caUNxseKxxiGyTGaccQG4ycLxdijYQz3ODpUPbSabpUNuk8Uh65sLklXtEE3
ZZ/uOqXPGGPPQ4Lk7nOzn8SkyHoJ9r0k9LPXWWT2fh0/1J1VJjeCnxBVbUPU
4/ZFreWPAUmYkslnBCncFZbEhMiPa443S+2d7VJ/f7/A11NZP8K+l473HZfi
yTh/H44LRoNgBT9XSLM4w3K8KXaC1gyo30doM2psmPGYfTNW8ccgO4Y9jwhc
SFVHlRSOhSW3380F96R6+A300dHRMGcWgfuamvvqGzo28EPhFAgPvohtIvpy
SvblDCR+dgzyYOprloMVU2ptmPKofVNWKKaMs+dBnvZcEairi9VJibYEVz08
PAz35ebLpqhvwh/uPswNgcMhdKQlwkP4Uj405S2Bv/4ypMlpLuKvBLzvf5oP
zPhaL/v5gGxKnQ1THrFvynJlEGZnQ9M2tDRIe7r28EtD2Y6whNPxXna7fnli
ueSrmMNcjAOzqpWC5HULUivnmeSPaxnUUpltvQ2lD9tX2qJcveyChk4B2lF1
r5y1OL885IN4QoeOCtKAW1u2qlnZUfSQfUVLFbKDmIlwuaruuFOKhAlBqong
HMJdYEU99hUtVgZS1om7M25OH7JRfd9eLctDNFShZXXbl9WskJ0UJH/aL0Vi
EeloSnv3n5Klbi1PgWV9zb6sRQpZRjOUCkliTJR60718e8xalazQgpBSgV7W
WoWW9aB9WQsVWWewgqiJ1UgDmQG+KqzsRABZ7iC7qpJ8FKQazFtgZUftK2tU
XV6V3ZV8GIOxHjr5Wy7Nw9MwXjF9EON0LkYF0YiFlvqAfakNSouwRqxN1Epv
nXpL3iPhUV9q+C8r1auWqm5NX4EV3m9f4XxFITu7Md4I++qyOy3UnYlLNcLz
dE0KWZW8EV25ErZQ8o7YlxeVyQcyAT5UqzdqXN5XerXyPmANF+ANV2hl99lX
NldpFnYVVndWS0dSR7L7PNTKfMrILSsLBVljVZCs7Cjqsq+o/pKL7b2h94Q7
ZPidXTul4GCQH+FXjvtQkKJBQZeSHTn32pdTR1xZm4zkfFo8OYfty6klrqQt
MyTnkH051cTl0z5Dcu6xL0ckrp3tMyTnoH05VcS1s3OG5BywLyei0Bxl89jE
Un7tdMyQnLvty6lUaLLKZn58Pq/Ud82QnE77crIFDusK6pJ1MPnNbri50nL2
F06O2C1CvcqB986QnH325YSUcedUQDrYjdtD9s+QnL325QRVXQEMozBpunuG
5OyxLyeg6gqi8SjvCg7OkJzd9uUgTTdfYhFWs95tdL50Rjpzzwwp2lUoRWx6
s5M9L8VXY+wfUyXMlKy77MvyKTTh/gFIS+bWFGR5nMxM5WKHfYleJRdHBZyO
/hV/dpWQyjvtq/Qoj6wDEeICPp/GBnWXkNSd9qW6lWadkht0VH4H5XpKSO2O
AqrtE/BWsKRR6y0htdvtq1WuSt62ogDP2f4ou6YkCSUle1sBZWfYcwpfo2T7
Skh2u33ZFerWjvFOK2eFSra/hGRvLaBsVWtTsgNXVvZvktdeq3tuWwFlR+Xh
SUd28MrKPpu86Sbdc7dMR7ZXLRv/wbaVjtyNKZXaUAmp3VwgtVB/jJFqwyWk
dtN01Hq0arFds8ORWmplCUndWACp2Kik1EgJSb1jOlLdOaljXKqPUFlVQipv
t6myg+cupVIsIZUbrKlcxx9Zh3r6NJvQnpJ8qZQkJpOS2+/nz5HOTklIpyVh
cBCPmZjgd5eyAxW2fU6aypbqErIlac2WtfwxCnJRNrMArIiwKgdkx5k1h7u7
1b/zRvM7VPZ0dfFjItEoP74KfrcP+5lb2d+PMcfGKtVOYn6RTtaUkJPrrTm5
ij8GQa0kMHfAAXAiyJwBd8AlcAycGx8fr+QsInDXhRsKbx3v6+OHNcu/WAdO
5SFYA3BDh4fR0Kkpj9rQqKA1RWVobQkZ2mrN0BWKoePjrBrOcBd4agkCd6id
panq1/wom0CVnbGn2EWOO2M7uKlwocOpEKKKvcYNZcdwBx/Gm/cC3vDmBp7m
RenpbC1+CxpaV0KG3mbN0OVKScbcERlcA7vGe1g3CE6x05dxVPxdKbCxFnfR
sszzzZGE1pWwgyoOgoIeNCOp2nUEPzcLcGc0u8OgFAxaZ82gFmUkZOrFRIKn
zcjIyPUyGg6QknLl4WNmgE3TlDpdfjwjX38l5sat1txYqrjBRkQYDuBCGh0d
XWXkBmMkxGr4XN1d4m7cYs2NxUrZy2DcrDcKs/dg8IP9rjcQlriJBCl1S262
ZkmzYsnkpOSHuoG9d5T1J7BX9kYjS1RZ4ilxSxLWLFmkWMIGoRBUmOy9XmYN
3LK7ibDEQ2RJqVtykzVLFipCYb8cu2Bq2HsD7AKC/Rk3qy2BcwYHc5aossRb
4pastWZJo6ovibAsgfeUzb34G8vk6TUczyxTrKH6lFK35kZr1jQoEtkFVMeG
3xOZjLxV1qe+cC6zpoLIGl+JW7PGmjXzFYmjo1KU1b+whWY90aegFb5LLcL3
IXMe9efklag1N1izJipbE2CV7AqWNdDdJnVHIK/WmnffZVdkJWRMqbuy2por
cxXNDEZkhXwn+2JdzAbCGK9yqGJMJdPT7QqUuCGrrBlSf0nn8uapU9nNz4/D
dB2mhJLEqfPjPvlEEpaJkvBv3UKpWxG3ZkV2aUvVmWwysoLVeEJLzayw4npr
VtQSnceWMrFipTUrqukeY2uZuHGdNTdEorvYXiZWrLBmRRXRXewsEyuutWZF
RJE4Pi4tYFZAd9FRJlYst2ZFpUpiPbMCJv67ysSKa6ZnRQ2D+vOREQ6yu0ys
aLFmRViWWNXXJz3d389B9paJFcusWRGSJXqGh6XWDuwl9peJFUutWRFUdZtN
ce2e/NluxRJrVgRUEqvlbvNgmVhx9TStWLVK8rJuk3Wc2R3zs92KxdasQIke
SejpYYU4r8QlP/sqJ0uusmaJT5EI9703b4blmuxCn2yNRs5st6fZmj3ZRRu4
QQ5bBmC5IhZzlbFDMWsOeZRHkAgOwevwxRLJXcY2LZqmTQxKYL0wP0S2yVPG
NjVZsym7epzJSEIqhTblrPKWsVMLrTml3Q8Q45vpsndo0C1VD1aeli2YpmVw
pwZW4CWJtMxXxpY1TtMy6Nlh152OZf4ytqzBmmVZe6Cigk1wahtVlgWunGW/
7ejquFA7WGvbMpxNusGyX/b29k6J4TCcQ50+f5quqRKNci145Vw7e6jrUEFc
wz8xwv844lQ6nZ6MxWIn2fOiTz/9dNzoo07RfBZ61RbiF5QWbW0ewrnQLHQO
l/c8lzonwsaWCswi+N7QxXnTcBEu2+FhysXwLHSxw8BFj0kX5+Zz0aN1ES7h
eDxbfagtrJyFFu4wsNBv0sJ6ixZiEpIWRmahhe0GFoZMWliXz0J3zkLMQB/h
XtUsdG+LgXsRk+7VWnAP5l6ZDOWeOAvd22jgXrVJ92pMujc6ilZNTVHuVc9C
9zYYuFdn0r3qfO7J5SB0eplMTr7KuppZaF2rgXXzTFonmrQOPh42NUVaVzsL
rbvVwLoGk9ZV5bXuF9ULFrCp4AH4xA4QxccPPsAPgbGfKpTX4PNmjbituG4W
2pkwsHOhSTsjee38Nzv27v3769atG4snk4ILH6f8EVavrFnD5Cb5x5mSIKG1
VRLA+pGR7J7B2WTnjQZ2xkzaWalvZxWG6BgeHj5bdekiAqz4wWuz1LrVBtYt
NmldWN86EUP8uJlaSZjl1l1vYN1Sk9aF8lqnyTp3mVi3wsC6a0xaF9S3rprI
unKx7hoD61aYtC6Q1zpN1nnKxLqlBtZdb9I6v751NUTWlYt1iw2sW23SOl9e
6zRZ5y0T62IG1t1o0jqvvnW1RNaVi3ULDaxLmLTOk9c6Tdb5ysS6BgPrbjVp
nVvfujoi68rFunkG1rWatM6V1zpN1vnLxLq6nHW/PdjZeXbRokVZ6zaYtK5C
37p6DPEtJn/yMvmz3LrqnHVSurdXionir0Sfb+XHH38sbNS1zquOC68otuF9
EGFuGTsW4Y8+tWNS2u+H48/HwuEYbDPepOucRx0/m2yybfPK2LbQ5Ymm2KZc
e1n74Eh9C90K1vl4OKxKumgZu+c3cM9NuLfZoNf7a0H4rb+iovV73/ueknjz
y9g6j4F1XsK6LYR1XjnxJgThi1Qo9Fmspubp1159FRc8lZXosnSvwsA9P+Fe
G+GeS+75VO4dzwwM+OGzKUG0sLGsLPRlLaRsCxK2beWPfrVtlF1hdGtBWbnl
lkYTCbVDYcKgdv7oUQzS5BDe+1buYMx6V5S/fzyVK6+U2wQaS7blLjVuA/yE
9wyEpjKxYjEh+xBKXFQmEq/SlxgrE4nN+hKby0RiTF/iVWUicZG+xMVlIrFJ
X+LVZSJxob7EJWUicYG+xKVlIrFRX+KyMpHYoC+xpUwkzteXeE2ZSIzqS1xe
JhLn6Uu8tkwkztWXuMK+xK9S6fS5pgJIVH0yCe6g/CoUDl0IvB0ImFRZr6/y
Ovsqzz1XIJU6nySCaeCf7+za+WVwMCiYlVynL3llCUk2+giQC3kq0i3lda2+
/OtLSL7RZ3e8NuTX6MuPl5B8o8/dBGzIr9aXv6qE5Bt9ZiZsQ76oL391Cck3
+tBLlQ35Vfrybygh+UafWqmxIT+iL39NCck3+thJvQ351HqqLP/GEpJv9NGR
qA351Aq7LH9tCcm/1UB+ow35IX35N5WQfKOPazTZkE/dgJLlJ0pIvtHHK5pt
yA/oy7+5hOQbfUTiahvyqbu2svxbSki+0cccltmQ79OXf2sJyTf6qMJyG/Kp
HQ+y/HUlJN/o4wbX2ZDv0Zd/WwnJN/rIQNyGfGqrkCy/tYTkG237v8GGfJe+
/PUlJD9mIH+tDfnUNjtZfrKE5Bttv7/ZkvzLtqkq0g+j6g0lpNpo5/w6S6o1
W0yzjS1Lvr2EJBvteF9vSXJ2S+gXTaGQqoHvKCG1qk3qX+3ev/tsU1NTVu3t
Fq9q2MLpqahY9sknnygNu7GEpKo3lfemeyUxJk4FY8HtH374obKvN69UYstl
+pUXXrgX1W4qIbXqDeGyWsmfZhEnhUkxiRu8N5tUbbRVsgulby4h6aHLG1qR
zn+bGzs+awEcqW+DX20DJf8Iqt9SQur9BurdhPotuuo9kvAD4av5rjlfRoPB
bIM/gJLbSkiyx0Cyl5DcRkj2KI8TwhehVOizmljNq08//zSX/CBK3lpCkisM
JPsJyVsJyRWXSz4+kMFW7kbJ7TMq2ZeVTMkMEjKRry8nk5D2EErbNqPS3NLC
iYVSVaxKkRMm1CBFN6hRK3gEBWyf4XQUpgRJ/keyl/lldwHDl0x9xwxRX0zQ
/HdIaecMUbpKn9KdM0SpWZ9SxwxRiulTumuGKC3Sp7Rrhig16VPaPUOUFupT
2jNDlBboU9o7Q5Qa9SntmyFKDfqU9s8Qpfn6lDpniFJUn9LdM0Rpnj6lA4Wh
9PnCZ56xRGmuPqWDNihdsrNuqkoUz0WOHQuYZFWvz+oeG6wMdsIp8QSzFOv0
KR6yQdHEzjU6bh66tfp0D9uga2Kn2bTo1ujTvdcGXRM7w6ZFt1qfbpcNuiZ2
ck2LrqhP9z4bdE3svJoW3Sp9ukds0DWxU2padCP6dO+3QdfEzqZp0a3Up/uA
DbomdiJNiy41nZXpHrVB18TOoWnRDenTfdAGXRM7faZFl1q4kel+zQZdEztz
pkU3oE+32wZdEztppkWXWv2T6fbYoGti58u06Pr06T5kg66JnSrTokstJ8t0
H7ZB18TOkmnR9ejTfcQGXRM7QaZFl7o/IdN91AZdEzs3pkXXpU/3MRt0YwZ0
DTfC5qFL3fuS6T5ug66JnREm6XrVdOEVhepfI8uUDZYmdjKYZOlRs8yaKVN8
wgZFEzsPTFJ0KxTPRRIRlYFP2mCn3imw7+DBzxsbGy/dKWA+Kz8SfuMKunZ9
8MEHinHHbFDT3tlPS2Is9qtAPP7Q22+/bXxPXkXNqxiXu2P04vMvPf/vkd1x
G+y0d+I5O8kPMSYnz4e7uroGBweNb5+rWLqUDCTua/0NUv26Daqhy41UqCp/
By1LGY7Up+3P0f5wDkX3PyDbb9hg6zdg6ybY6t+lZ4b+8IdfzWnyTfnn+rOG
/kek2GuDoseAopegSN1Vz/6B1okJZmKKmRh77qX+fk7xPyPFPhsUKwwo+gmK
1F3wisspMhcznOJ/QYpPWaLoy1KkaAUJWvKd31x8gsrfIpWnLVFxSz5GoTIH
HybQ5Tu1EPJcJBpVED9CwGdMAl5NRIaXJPxdbsKzJuMszhPnOZNxrsoT53mT
cZrzxEmbjBPLE+cFk3EW5Ynzosk4TXnivGQyzsI8cV42GWdBnjivmIzTmCfO
qybjNOSJc8JknPl54vSbjBPNE+ekyTjz8sR5zWScuXnivG6po9KPM2AyTl2e
OG+YjFObJ843TcapyRPnTZNxqvPEyZiMI+aJ8y2TcaryxPkdk3EieeK8ZTJO
ZZ44v2syDjUyquP8nsk4oTxx3jYZh6oT1HHeMRknkCfOKZNxqHJKHef3Tcbx
5YnzL0zGoSpQdZx3Tcbx5InzL03GoYp2dZw/MBnHlSfOt03GoaY86jh/qBvH
q44Drygx+Om41i8M6p7uUZ+eg1ed+0e657qVcy8ENm26FPI7Rso//fQ3LlHs
PXnypAbqPeIcjwKlnpOcOMFPw98DK/wxcZrRPIGfi78tVfgT/mRczPPjF+Hx
f8qfLiu/+SG49I+Hf3Rlvhfm/H9HgJ5B\
\>"]]
}, Open  ]],

Cell[TextData[{
 "\nEXPERIMENT:\nBy using only the commands First and Left, how many ways can \
you rearrange the three books? There are a total of 6 ways to arrange 3 \
books. Can you obtain all 6 arrangements using just these two commands?\n\nIn \
doing this experiment, you probably found that there are 4 other ways we \
could have rearranged the books, namely swapping the last two books (Last), \
moving that last book to the front (Right), reversing the order of the books \
(Rev), and finally leaving the books as they are (Stay). These four \
additional motions also work in the MoveBooks command, yet the last \
experiment showed that all arrangements can be obtained using just the \
commands First and Left. Therefore, the set {First, Left} generates the group \
of all permutations of the three books.\n\nLet us define this group in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Rather than using the names, we will use the abbreviations: ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " = the identity element, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", "First"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " = Left.\n\nWe begin by defining the identity element to be ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["InitGroup[e]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["e"], "Output",
 CellChangeTimes->{3.4425883305552626`*^9}]
}, Open  ]],

Cell[TextData[{
 "Now, if we swap the first two books twice, we get back where we started. We \
thus see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", "2"], "=", " ", "e"}], TraditionalForm]]],
 ". So we define: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Define[a^2,e]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "If we perform the Left action three times, we also will be back where we \
started. Thus, we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "3"], " ", "=", " ", "e"}], ","}], 
   TraditionalForm]]],
 " so we can define"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Define[b^3,e]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "In defining ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ", we needed to also define ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 " in terms of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 ". Let us do that for this group also. Notice what happens if we first move \
the books to the left, and then swap the first two books:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitBooks[3];
MoveBooks[Left . First]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0, 1, 1], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[1, 0, 0], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883368052626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[1, 0, 0], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[0, 1, 1], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883368521376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0, 1, 0], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[1, 0, 0], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[0, 1, 1], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883368990126`*^9}]
}, Open  ]],

Cell["\<\
Is this the same result if we do those actions in the opposite order? Let's \
check:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitBooks[3];
MoveBooks[First . Left]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0, 1, 1], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[1, 0, 0], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883394615126`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztfQt0G1Wa5o31flgqy3ai2HnIBBITQlAgJAJCkIEkzhORBzEJCSYQ2uYp
SGibp3jGAdOohxnwLDs9XpiZ4zn0Yb29nDmeOTs73t7ds56md9bLzM6a7t3T
Yc72rPd0N+2EAO7udHP3/vevkkrJrVKVS4plnTqJJEuq+3+P+99X1ZW0857j
XUcfvud49733xLY8fk+6q/veY7HNjz7OXnLMI2TeDwipORkj8DclRLmj7D/9
CO7glTL97eF/O6Wpqank6OiochtVHVLD/3a3TkxMpIaHh5/PZDJjifbEzxti
Db/ySB7HlJSUiBc4f+WTfOPxZPwvtnVsg8M6BwcHIVp0cnJyBCP+lj/4E2Nj
Y10DAwNvd6e7ocQ5p9d5JtQaOh1OhSnJMGB2PDlFXBTMCFEyxR4ZKTJE4O2z
wc4gwH7pj/oBeKI10QphALS3r68PQOPj4+P/BkG/4g8BRSQcBRR/0hJvgdJT
UoJF6mKRsjLKNCERBXlMfi1Dzjl7nYB6OhwPQzlwAFDfPdx1GGKCOxDfOz09
/eeIfJY/eNtHRkb+pi3VdiYUDYFnEAVlgp5xspSDuUExgg1wQHADDq0Be4gD
sJ48kT3BY/4pApzhDy6ABjlTUjsLPMxKTxJyBY/qADkgDYqTP8NiU1jtUEGg
Am0fJ1cpJUbxaBXQLxAItB3vz/aDEqgFoEzWaBf7GRYD60/2ZHo+r43WfhFI
Bzi/a7SL/T8sBonzztH00SkpJk17M17IA3KtdrH/i8Vip06d+u7ezr1nQrHQ
OWfWyYutzxf7x5XJlepiP8VikDGQF2eDMaZrEPPgOl7MidWSJMMpmoqP0/E/
wZL/hz84oFYwK4YI2aBUJpag8j+l5LtY8p+wJPj5l1s6tpwJpUNkY56iUhKe
wb1Smpf6YwzxKYYAtT++LHHZb1xDLnKTIIRLEOKPMMQpDAFZ+7PGeCPkAEkK
QngEId7BED/JW4BpzbxuE4Tw4Wt/d/Xg1d9+NPMoL/WHGOJ/C7y4RTsEtsWY
k4eRQ/wvgRe3aoWogQiEdVwZKUflbYzzY4Ehm4rEgT9zseQ4PxK4slk3Tg2+
W0DqDzDYhMCfdt1gDkGw38dg/xODQYvMObVVN5hTEOzNgmA/WNe+7tfuETc/
UD+YSxDs9zDYP2Kwt7t7uz+v7avlB27TDeYWBPs2BvsfGGzf+8PvfxZJYW++
XTeYRxAsi8H+AYPBEPhZJB7hHcMO3WBeQbA3MNjf5yvgp83xZt6D7tQN5hME
+1ZBsIIK0A/mFwR7HYN9LKiAXbrBAoJg/Rjsvwsq4DbdYEFBsNcw2LigAlK6
wWoFwV7FYP9NUAG36wYLCYKdLAhWUAH6wcKCYH0Y7O8EFbBbN5gkCIYTBPJf
BRWwRzdYnSDYKxjsh/ku6LNILMLH7726wSKCYC8XBINJC848aLFg9YJgLxUE
w8ncODEQrEEQ7EUM9pGA2T7dYI2CYC8UBCtgph9sviDY8xjsBxgMJs4wX+UH
3qEbbIEgWKYgGExJcbZYNFhUEOw5DPa3GAxmTzgHYq/t1w22UBDs2YJgBcz0
gzUJgj2DwcbykwKc07HXOnSDNQuCPV0QDKajOKcsGmyRINhTGOzP8/NbWBuk
s9ksTDmAKVmIQPAeTGB5X3enLtBiAdCTCITz1DqoHJjs9/cePz6ejMe/kny+
L6N+/1S7JP0q7fFMJSXpbCzILCLkn1ubmuCYt9Pd3bCsAYKQdiQicECf2BIB
sV4k9h3+EIAOBZSDA9/t3Lt3ItHaCiROx8Phs52MUB9bhY6yG5s/seEe3grR
Cfm1LLtl2GKGiZhKSBK8CcKA/F90bCtYgkJn78v3+18EogFu7AFd/ksF/HuQ
P847/VBjYBBfFbcnEj+PNTSc8zqdYCgYSwcZxzHCOjCkPiVTH0LqIJEfKrFD
YSHL5AN99UIWBj1cgztg5YXLJ1qMekxA/ZtI/S3+gIvSVFsb0IW6BxuBEx1h
NzYkxjhlP7DnlNlb53pR2ZnWUAje/MXS+vpp73wvDAcHdem0COg8gXRwXopL
2HhLC6cxzPBY7ZBVnIOD/mtCvq5pnMeD6ANdIgA6jkA4zcRFL6um06lwmC1l
2KIXMf6TYYxlAoxjiIFzPFwmZ/v7AYenMfOTLZPNAl0qAHocgXDKhgvrTE/P
Z7FI5It0IMCBrjENdJkA6DEEei3fVWV70+mpmCRNZ7xeyiYAbCluFmi5ACiN
QDih4ov3d9JHj56JhULn+pxO3vLXmwZaIQDCe3l+hGcJWJcDuX8uy4Ag4a7L
A9GaBmIAqFUA9AgC4dzJcWjovfemWllWs76AbDANcLkA4GEEwPmU4zvpI0d+
2VFXxxXcaBpgpQDgIQR4Md9l8mEj6/ORm0wDXCEAeBABXshPHn4ab26GjoYk
TQOsEgA8gADP5+vgF4n6et4+2kwDXCkA6EYAnAU5ocUPdXV0KPVws2mM1QKM
LsR4Nj8l+IfkqlVfDbBquNU0wFUCgG8gwDP52eVkPBqFPp9sMg0QFwDcjwA4
iXI8NvDGGz9PNjTw3mOzaYA1AoCjCPBkYTVMdbH2xqphi2mMqwUY9yFGb34R
9HFy9WpeDVtNA1wjALi3AOAvO7Zs+c2Qy8VDmwdYKwA4ggA4eXE8mT1x4kya
jeEQeptpgGsFAPcgAE4x+PkzmCjw0NtNA6wTAHQiwBP5Of3P4o2NfHTYYRpg
vQDgbgQ4nq/kHycuuwxmCGSnaYCEAOBwAUBBJZsHuE4AcAgBjgkqeZdpgOsF
AHchwOOCSr7NNMANAoCDCPCYoJJTpgE2CAAOIEBaUMm3mwa4UQBwZwFAQSWb
B9goAOhAgEcFlbzbNMBNAoD9CPCIoJL3mAZICgDuKACAOT/4P0OANgHAPgTA
SRKezIG5PoTeaxrgZgHA3gIAWETxFcvMAG4RAOxBgIfysyNYEPPQ+0wD3CoA
2F0AUKDAPMAmAcDtCPCgQMEdpgE2CwBSBQDvdh0+TAdkBeYBtggAbkOAB/Jz
L1iS8Dn2ftMA7QKAXQUAsObhyxA6I4CtAoCdCNAtUNBhGmCbAGBHAcDLmaef
PpeRFZgH2C4A2I4A2/hDwRYE9cX2aL43h3Uwn1zeaRp/hwAfgckm/hABPHnX
Qno8nkwCwplQa+vpcCrFImTgcUpKJs85vXw/BBwzlmhvB75QTjnFV4d84QQB
X7PTGfHdKeC7FfnexB/8UOOqDQ8d/9yEJ9eA45f+ri5KslkWfBQuVlG4YjI6
SsnICEiBt+Gws8FYDIpAUZAD0iEchOUnBX0C7w+Y1rJLoKUdtVyPWqAH6Rga
GgJwIHImFI1+6Y9GgSQQpmRoiJKJiQaupQ7+RD0gMZOBw6akRILiNhUJQoAl
EA7CKucIAQyq6ssuv59xNqvjNoGOLahjPX9QtoKkIPLpcDx+JtTRgfyB7ORk
jPP3UzI1hS+BrEwGDgMNv/Lgec7pBT4fjJrmKaYEFDcjxWtxDSVvJolPSe2s
2xoeXs0pOShuSsIu1nwV3y7AxXZFrkFceTdKDJvT+Hi8JLi7Bbi3Ii7uHVH2
tGQhpc4GOzspOXWKLwOtg+8RgN+C4KsRXN4Zk/ksEot9EUinIQf4Cs46+F4B
+M0IvgrB+Um9dG/vlBSLTXshCRn4upKA7xOAtyE47r3h59/ePdzVdSYUi51z
9vVByvN1k3XwOwTgSQRvzYN/d29nJ/ZvAwOUTE/zJY918P0CcOyQyQrsLPe9
Pzz8s8Y4y+/BQb6IsA7aIQDdiKDLEfT1Y5Bk0LKYzRtLAnqnAPRGBL0UQSG7
v78xlTobzOBawDroAQHoBgRdlp9s/qQlkfiNa3iYrw+sgx4UgN6AoJcg6KH3
hoZ+UQ8DzMQEXzNYB71LAIqjIWlB0O8cSad/WQcDCMveW0oCekgAeh2CLs3X
KQyfX/myWT71tw56WACaQNAl+Tr9aTM0mdFRvhywDnq3ABQHabL4/DplA8KW
koB2CkDXIegiQZ22lwT0HgEojvWkKT99+8G69nZep9tKAnpEALq2ABSUAiCH
Kw3ovQJQnF0ol9WPvjM4eDoMgzx7tr0koPcJQK9GUHllAjPlnzfARJXV6Y6S
gB4VgK5B0AUkt977pyVy9u4sCej9AlCcRJH5+UT64VqYN7J2uqskoN8QgF5V
AFqQSKUB7RKA4oyNNAoS6baSgHYLQK9E0AZBIqVKAvqAABSnh6RekEi3lwT0
QQHoFQgaESTS7pKAPiQAXVkAWpBIpQF9WAB6OYLWCRJpT0lAHxGA4sSXSPnx
FIc2lkh7SwL6qAB0RQEornuHhzlcaUDTAlCc8JJwXikstDncvpKAPiYAvawA
tEBpaUAfF4DiLJuE8u0UFlO8Tu8oCegxAeiyAtB3jqbT55xykykN6HEBKM6y
Sa1A6f6SgD4hAG0pAC1QWhrQbwpAYwgazHeDn9dGo3zV1lES0B4B6NICUDgZ
guci2LPSgPYKQHFqTwJ5pXAShJ+DuLMkoE8KQBcXgBYoLQ3oUwLQRQWgcI4L
xtISgj4tAG1GUD+Cyluacc5woCSgzwhAmwpA4SwinMkrIeizAlCc2iunoguU
HiwJ6HMC0GgB6EQrjKVjYyUEzQhAFxSAFiRSaUCfF4DOLzfoCwLQxnKDvigA
bSg36EsC0Ppyg74sAI2UG/QVAWhduUFPCEClcoP2CUDD5QY9KQANlRv0VQFo
bblBXxOABssN2i8ADZQb9HUBqL/coN8SgPrKDfqGANRbbtCsApqQ+JVrwPFc
NNCklFS+VsEGtUFtUBvUBrVBbVAb1Aa1QW1QG9QGtUFtUBvUBrVBbVAb1Aa1
QW1QG9QGtUFtUBvUBrVBbVAb1Aa1QW1QG9QGtUFtUBvUBrVBbVAb1Aa1QW1Q
G9QGtUFtUBvUBrVBbVAb1Aa1QW1QG9QGtUFtUBvUBrVBbVAb1Aa1QW1QG9QG
tUFt0AoCvZHf+1k59jhMAIlK7RJlMDTYGeTPyRBBUlOkYTY44g+czqdkjD0O
EOpJezg/p9dJG2INtC3VRjOZDB0ZGaEMlg4ODvLn2zq20XgyTn2Sj2NAGdDG
NWVlTRPwY2QXXxP+9G8APEXgPkJDHSEajod52dZEKz3cdZhms1muaXp6mh0N
eLX8OWgFjd3pbq6xqbWJlwu1hrhO8IjrHOHxiXs2RF6pJNeEKrkYOX/UzysF
iB/vPU6Hhobo+Ph4lOsL0YmJCa4RtIPGRHuCHwpwHgnrHrzi+gblukwCTnnl
4G+Le7iHkEfguZJ7wHdycpJcyjV46NTUFH8NpMH7ezv30hVrV1Bvq7f8RC/n
907uuZSQuHksYVZzbg7OCzy8KDmwQuEyiMkJho2NjcVng8ul/N5FyTShzqyT
BmNBXjEs4ViDnAVClyiEWC/gzXhpIBqgR9NH6alTp8ja2SAUUwhNEhpIB2ht
tJb2ZHognck6FaHw9eGLQ2iJQoiNTzAehaIh2p/t5/1hghNy0g8//JA6/A7+
u93sOG+5OS1SUprVWl1HHR+D2JhzA6fjzlcYdE0xotByl5tWs9LFs952QWIB
fW/oPTa8QzyvOonwXyE3Z7m5NcncXMMu2pJo4S1uoyDBHareQsWvptz8Fsro
wUyQbkxt5Pl+k4CfS5Nf2agtUIxh2RZJReixzDHSJqDmOZ9ajJSR1fzzku39
4ffJzTIrGPtak62UjMsdBBw3KM87pkj+K3lLS6lRkGO36lEaz1P6VnkoKdNl
dVptNkjp9fJQigjSaatBSv3loVQnyKXtBim9Vh5KkiCXdhqk9Gp5KIVlKPeI
m65tX8tz6TaDlE6Wh1JIgWKDc3O8mVO63SClvvJQyi3m2BSvId7AK26PQUon
yksJWhtLbU5mr0FKr5SHUlCGqu2rpQ/3PszJ3GGQ0svloRRQpTcs+CBSh0FK
L5WHkl+V3kviS3h6HzBI6cXyUPIpgzycNmgl9MaJG+k0nT5okNUL5WTl4j9F
CT/7DP/a2T/GjBil9nx5qHkVqD72uJ3AhA1nk+w1mSInaLReM+WhmZu8wema
doLTpU55Ojwzps+Vh6lbqWvWKOAHOXkmIltcRsyQ7rPloetSSE/LdCcwPWXK
LguMnykz4y7Cz3bm/iFjtwXGT5eHsVLtcNKArzqm821MZq1KnZlRf6rM1NME
TyNSMXWPBepPloe6Q+C6Q0Dda4F6b5mp9xLsmKmYus8C9Z7yUM8lx7Ts+qQq
iVTU/Raof7PM1CHP0+flv4p6wAL1J0pF3aOmjv9UjrsEtIMWaB8vFW33hbTR
bbeAca0FxsfKyBiGyXEh45AFxo+XirGrkDFcV0rle2g13bAFuo+ViS6aK6Qr
WaCbtk53g9xnMAQ6xG69Tid7lpQk1kuwv2kfe22U3cbhvBYc6swrw0rwCkTV
WRD1qHVR6/i9jy1EkHyW3TqDQZqQJH5cvKWFdmzbRvv6+gg/n8r6EfY37T1+
nCbjcf4+HBf1+8EKXjbDng/L8aah4znPDJi/jwjNiFgw4xHrZqzh9356ij0f
YTcQkgqHaYyJ8rIaBsHp7m5+AX1sbCzImYXgumbBdfVUWxs/FIpAePClnd0g
3IDsyylI/PwYRFw89QtPB8um1Fsw5WHrpqySTZlgzwcx7bkiUBdraKDtiQRX
PTw8DNflFsqmqC/Cdx0+zA2BwyF0ayjEQ6Q9Hm7Km+AA3F6CAXeUi/grgtfi
R/nAjK/1sOcH0JQGC6Y8ZN2UlfIgDFUJVdva1EQ79+7lTUPZjnAZp+O+8HL9
ypW0xjPPSeMSMFsdJtTpZow2cp5Jfr+O8N9OB7aNFpQ+aF1pq9x6oUFDpwD1
qLpWzmqcNw88CBP6aIAp6icKnwtYWVH0gHVFy2Wyg3ImQnNVXXEXKWJTKhqK
EL6GcJZYUbd1RcvkgRQ68Sw0S0YfslF93V4tyyWoqFLL6rIuq0UmOwVdsddL
Y6yy0kcLr/6LZKlry1ViWd+wLmupTBZopgMBGoOhoaeHb49Zp5K1iL1H+Qz0
wtoqtaz7rctaLMs6Jc8gYpEIzfb387PCyk4EkOWH8QNHQVGFuUus7Kh1Zc2q
5tVVW8uHMRjroZO//rw8ZKrYeJXkskZVMWoElVhqqfdZl9ok04RKTNTX04E3
35T3SLjUTY3f8lLdaqnq2vSUWOG91hUulBUC43hzM+yry+20UHcmDpof4ZPy
TVEJlejoV508L5G8I9blRWXyWZ+PD9XqjRoX9pXuAnkfsJsPK67Uyu6xrmy+
rAxaYUddHU0fOZLb56FW5qE4civK/KyyavqFrKwo6rSuqPG8xjb03nvkFhm+
c9cuOsjWVHCEVz7uQ1ATJZqUrMi527qcBkHL2qQj59MyyjlsXU69oCVtmSU5
h6zLqRM0n22zJOcu63IkQdvZMUtyDlqXExa0nV2zJOeAdTkhmeYY1M7y5bzt
pGZJzp3W5dTKNGFmE1+4kM/Ud8+SnA7rcpQJDnQFSTaXZYvf3Iabiy1nf+nk
dLF1FJuvcuB9syTnDutyAjLNATbudB3E7SH7Z0nOPuty/KquAIZRWDTdOUty
9lqX41N3BdEo7woOzpKcPdblIE0nP8VyNbuNsd6NzabvmiVFu0ulyEV3sefL
lddZGaaKzJas263L8ii9Anu+C1c2uXMKsjxOZrZyMWVdolvOxTF54fZX+Oio
IJW3WVfpku95B0LwbMmoXKHOCpK6y7pUp1yt03KFjsnvyXJdFaR2Z+nU9rLn
ffnXFbXuClK7w7papVVC3UqEX+fN9UeyYlWjrgzZ20snO8uep+XXRLI9FSR7
m3XZikSo7Rh2Wjkr1LK9FSR7a+lkq2tbJNt3cWX/9orkFZpl20snOyoPT1qy
/RdX9un1yfWaZbfMRLZbLZvfhtktlR+a1GoDFaR2c2nUwvxjXKw2WEFqN81E
ratArVyvueFILbW2gqTeal2qXKlCqaEKknrLTKQ6c1LHUapHoDJcQSpvtqYy
hbkrUilVkMo2cyo38PsAJaOjlAwMUE86TaVkkjq9Xv4Y6uigJJOhZHAQj5mc
5Bu+lIFKrvucNLUtdRVkS9KcLev4fRTkomxmAVgRam3lsuPMmsNdXervvDnv
O1Q6+TGhaJQfH47HuZ3cyr4+jDk+XqtyUs4voZORCnLyJnNOruH3flBLCXMH
HAAn/MwZcAdcAsfAuYmJiVrOIgRXXbih8Nbx3l5+WEscv1gHivIQrAK4ocPD
aOj0tEtlaPS8mbDa0PoKMnSjOUNXKYZOTFDC0g9c4KnFKIJD21iaqr7mR9kE
quyMHWCNHN5uS6W4qdDQoSiECLPXuKHsGHjtQfniPcEL3qNE2WkympuLXy8b
2lBBht5oztCVypQMvqGivZ02sTbezbpBcIoVX8FR8btSYGMtmAdN3zNvHpVW
r3ZSKR5nBFwuFzcjSfK7juB5C+E7jHM7DCrBoA3mDGpVRkKmXkokeNqMjIxc
JaNBpvBS+IzfO/v76dFAbp7O70/J7a/S3LjBnBvLFTfYiAjDATSksbGxNTpu
sI6JhiIRvlZ3Vrgb15tzY5ky7WUSnaw3CrL3YPCD/a7XCCxxChKk0i25zpwl
LYolU1PUC/MG9t5R1p/AXtlrdSxRZ4mrwi1JmLNkqWIJG4QCMMNk7/Uwa+CS
3XqBJS5BllS6JevNWbJYEcqeB1mDibD3+lkDgv0Z16ktgTKDgzlL1FnirnBL
1pmzpFnVl4RYlsB7yuZe/MYyXF7z45llijWiPqXSrbnWnDVNSu2zBtTAht8T
2ay8VdajbjgXWFMjyBpPhVuz1pw1CxWJY2M0yua/sIXmJkGfglZ4zreIv88z
x+vNyatUa64xZ01UtsbHZrKrWNZAd5vUHIHchda88w4N1dZCxlS6K1ebc2W+
opm1CYlN5DvYjXUxbQJj3MqhsjGwWDrqcPgq3JA15gxpPK9zeWNgILf5+VFY
rsOSkFK+E50f98kndIEk0e85naTSrYibsyJ3akvVmWzSs4LN8RawfnUuWHGV
OSvqBZ3HliqxYrU5K+rEPcbWKnHjSnNuSILuYkeVWLHKnBVhQXexq0qsuMKc
FSFF4sQEXcSsgO4iVSVWrDRnRa1KYiOzAhb+u6vEistnZkWkvZ1+b2SEg+yp
EitazVkRlCWGe3vpU319HGRflVixwpwVAVmia3iYbkxhL7G/SqxYbs4Kv6rb
XHLenvy5bsVl5qzwqSTWyd3mwSqx4tIZWrFmDXWzbpN1nLkd83PdimXmrECJ
Lkq6u9lEnM/EKaNZVZZcYs4SjyIRrntv3gyna3In+mRrCuTMdXtazNmTO2kD
F8hhy8Ann4BFjip2KGbOIZdyzyRyh+B1uLFEclaxTUtnaBObvBLWC/NDZJtc
VWzTEnM25c4eZ7OUpNNoU94qdxU7tdicU+r9APwcO1sj567QoFuqHqw6LVs0
Q8vgSg1cmqBUaJmnii1rnqFl0LPDrjsNy7xVbFmTOcty9sCMCjbBqW1UWea7
eJb9rjOV+mqwvt6yZbiadIJlv+rp6ZkOSkEoIyq+cIauqRJN5Jr/4rl2uvPQ
oZK4hj8xwn8ccTqTyUzFYrGT7HHpp59+OqH3UadoMQvdagvxBlOL9naXwLnA
HHQOT++5zndOgo0tNZhF8Leuiwtm4CI02+FhkYvBOehiSsdFl0EX5xdz0VXo
IjTheDw3+1BbWDsHLdypY6HXoIWNJi3EJBRaGJqDFm7TsTBg0MKGYhY68xZi
BnoE7oXnoHtbdNwLGXSv3oR7sPbKZkXuSXPQvVt13Ksz6F7EoHtjY2jV9LTI
vbo56F6bjnsNBt2rMzgdhE4vm83LV1kXmYPWbdSxboFB6ySD1sHHw6anhdbV
z0HrbtCxrsmgdeGi1v2ybtEithQ8AJ/YgcJ4/8EH+CEw9qxGeQ0+b9aM24ob
5qCdCR07Fxu0M1TUzn+7c9++v79yw4bxeDJJHHg/7Q2xgmvXsjBJsBNeYy1j
IyVg/chIbs/gXLLzWh07YwbtrNW2M4whUsPDw6fD559EgDN+8Nocte5qHeuW
GbQuqG2dhCF+0iI6kzDHrbtKx7rlBq0LFLWuIOucVWLdKh3rLjdonV/bujpB
1lWLdZfrWLfKoHW+otYVZJ2rSqxbrmPdVQat82pbFxFkXbVYt0zHuqsNWucp
al1B1rmrxLqYjnXXGrTOrW1dvSDrqsW6xTrWJQxa5ypqXUHWearEuiYd624w
aJ1T27oGQdZVi3ULdKzbaNA6R1HrCrLOWyXWNeSt+13HwY7TS5cuzVnXZtC6
Gm3rGjHEt5n8qQvkz3Hr6vLW0Z5MD5Vi0q89kmf1xx9/TG7VtM6tjguvKLbh
dRAyv4odC/F7j9ox6s144fizwVgwBtuMN2k651LHzyWbbNuCKrYtcGGiKbYp
bS9nHxypbaFTwTobjAdVSRetYve8Ou45Be5t1uv1/pr8rsZbs/H73/++kngL
q9g6l451boF1WwTWuZXEmyRfBNKBzyKxyFOvvPoKnvBUzkRXpXs1Ou55Be61
C9xzKD1f3r3j/dl+L3w2xY8WNleVhZ6chSLb/ALbtvJ7r9o2kV1BdGtRVbnl
pImxhNqhoMCgbfzepRhUkEN47Vu5gjHnXVF+/5hM56ZXymWCAku255satwGe
4TUDsqRKrFgmkH0IJS6tEomXaEuMVYnEFm2JLVUiMaYt8ZIqkbhUW+KyKpG4
RFvipVUicbG2xMuqROIibYnLq0Ris7bEFVUisUlbYmuVSFyoLfHyKpEY1Za4
skokLtCWeEWVSJyvLXGVdYlfpzOZM0tKIFH1ySS4gvLrYCDw1Vs+n8+gykZt
lVdaV3nm2RKp1PgkESwDv9e5a9eXg34/MSq5QVvy6gqSrPcRIAfyVKSbyut6
bflXVZB8vc/uuC3Ij2jLj1eQfL3P3fgsyK/Tlr+mguTrfWYmaEG+pC3/6gqS
r/ehl7AF+WFt+ddUkHy9T61ELMgPactfW0Hy9T520mhBvuh8qiz/2gqSr/fR
kagF+aIz7LL8dRUk/wYd+c0W5Ae05a+vIPl6H9dYYkG+6AKULD9RQfL1Pl7R
YkG+T1v+dRUkX+8jEpdakC+6aivLv76C5Ot9zGGFBfkebfk3VJB8vY8qrLQg
X7TjQZa/oYLk633c4EoL8l3a8m+sIPl6HxmIW5Av2ioky99YQfL1tv1fY0G+
Q1v+TRUkP6Yjf50F+aJtdrL8ZAXJ19t+f50p+RdsU1WkH0bVbRWkWm/n/AZT
qgu2mOYqW5Z8cwVJ1tvxfpMpybktoV8ElgRUFXxLBalVbVL/ev+ePaeXLFmS
U3uz2VYNWzhdNSs++eQTpWJvrSCp6k3lmZ4eGpOk6Zjfv+PDDz9U9vUWlSrY
cpl5/uXn70a1mypIrXpDuKyWZrxeynrcqaSEG7w3G1Stt1WyE6VvriDpgQsr
WpEOO0fh+JwFcKS2DV61DSL5R1D9lgpS79VR7xSo36Kp3kV/SMjXjoXzvvRH
/bkKvw8lt1eQZJeOZLdAcrtAsku+nyTki3Qg8FksEnnluaee4pLvR8lbK0hy
jY5kr0DyVoHkmgslH8/2Yy13oeRtsyrZk5MskukXyES+npxMkbQHUNr2WZXm
pJOLF9NYOKzICQrUIEUnqFEreAgF7JjldJwmcDy/CdnL/HK7gOEmU985S9SX
CWj+O6S0a5YoXaJN6bZZotSiTSk1S5Ri2pRunyVKS7Up7Z4lSku0Ke2ZJUqL
tSntnSVKi7Qp7ZslSs3alO6YJUpN2pT2zxKlhdqUOmaJUlSb0p2zRGmBNqUD
paH0+eKnnzZFab42pYMWKJ23s246LElnQseO+QyyatRmdZcFVjo74ZR4xCjF
Bm2KhyxQNLBzTRy3CN16bbqHLdA1sNNsRnQj2nTvtkDXwM6wGdGt06bbaYGu
gZ1cM6IradO9xwJdAzuvZkQ3rE33iAW6BnZKzYhuSJvuvRboGtjZNCO6tdp0
77NA18BOpBnRFS1nZbpHLdA1sHNoRnQD2nTvt0DXwE6fGdEVnbiR6X7DAl0D
O3NmRNenTbfLAl0DO2lmRFd09k+m222BroGdLzOi69Gm+4AFugZ2qsyIruh0
skz3QQt0DewsmRFdlzbdhyzQNbATZEZ0RdcnZLoPW6BrYOfGjOg6tOk+YoFu
TIeu7kbYInRF175kuo9aoGtgZ4RBum41XXhFofrXyDJtgaWBnQwGWbrULHNm
yhQfs0DRwM4DgxSdCsUziVBIZeDjFtipdwrccfDg583NzefvFDCclR8R8lu/
w7H7gw8+UIw7ZoFa4dfFZagUi/3aF48/8NZbb+lfk1dRc8vGqa4YvfDic8/9
e2R33AK7wq9m4+yoF2JMTZ0NdnZ2Dg4O6l8+V7F0yBkouq71N0j1CQtUAxca
qVBVfgctRxmO1KbtzdOeN09E9z8g229aYOvVYesUsNW+Su+i5Ec/+nqBxzM9
3+vNGfofkWKPBYouHYpuAUXRVfXcD7ROTn4RSKc/i8Riz77Y18cp/mek2GuB
Yo0ORa+AougqeM2FFI/3Z7Oc4n9Bik+aoujJURTR8gtoyVd+8/EFVP4WqTxl
ioqTehiF2jx8UIAuX6mFkGdC0aiC+BECPm0Q8FJBZHiJ4ne5kWcMxllWJM6z
BuNcUiTOcwbjtBSJkzEYJ1YkzvMG4ywtEucFg3GWFInzosE4i4vEeclgnEVF
4rxsME5zkTivGIzTVCTOCYNxFhaJ02cwTrRInJMG4ywoEudVg3HmF4nzmqmO
SjtOv8E4DUXivG4wTn2RON8yGCdSJM4bBuPUFYmTNRhHKhLn2wbjhIvE+T2D
cUJF4rxpME5tkTi/bzCOaGRUx/kDg3ECReK8ZTCOaJ6gjvO2wTi+InEGDMYR
TafUcf7QYBxPkTj/wmAc0QxUHecdg3FcReL8S4NxRJN2dZw/MhjHUSTOdwzG
ES151HH+WDOOWx0HXlFi8OJ4rp8MahZ3qYvn4VVl/5VmWadS9ivfpk3nQ76r
p/zTT3/rkKSekydPFkC9JyjjUqDUa5ITJ3gx/B5Y8ieCYnrrBF4Wvy2V/Cl/
0J/M8+OX4vF/xh8umH7zQ/DUPx7+0cX5m8z7//jtnj8=\
\>"]],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[1, 0, 0], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[0, 1, 1], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883394927626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztfQt0HFV6ZiH1u1ut1stqS7bcssEWxthtY+zGGNEGbMvyg8ZPYYMRBjES
LzfYjMSzwTxkENATEkYJyUSBJEc55BAty9mjzNlsvJPkrDLMZrUkmxWZ3TMi
ZyerPTPDyMaAZoYZau9//6ruKvuv6ipVt9XqU8fud93/e9y/7qtuq3fffbK7
66G7T/bcc3dk+6N3J7t77jkR2Xb8UfZW+WWCcNn3BaHsdESA56IgyHci+y9+
CHfwToGeu/lzR2h6ejp+5swZ+XZGcUgZf+5qmZiYSIyMjDybSqXGYm2xn9ZG
an/hDrnLp0PxkOABzl96Q97xaDz6H9o72uGwzqGhIYgWnpqaGsWIv+YPvtjY
2Fj34ODgt3uSPVDiK4fHcS7YEjxbmagUhRQDZscLk4JTBDOCojDNHhkpYViA
j88HOgMA+4Uv7APgiZZYC4QB0L7+/n4AjY6Pj/87BP2SP/hlkXAUUPxRc7QZ
Sk+HYixSN4uUllBmBKFaRh6T3ksJXzn6HIB6tjJaCeXAAUB9+2j3UYgJ7kB8
z8zMzJ8h8nn+4GkbHR396y2JLeeC4SB4BlFQJugZF5ZwMBcoRrBBDghuwKFl
YI9QDliPv5R+icf8EwQ4xx+cAA1ypkNtLPAIKz0lCFfxqOUgB6RBceFPsdg0
VjtUEKhA28eFNXKJM3i0AuhnCATaTg6kB0AJ1AJQFtZqF/sJFgPrT/emej+r
CFd87k/6Ob9rtIv9PywGifNWV7JrOhQJzXhSHsgD4VrtYv8Xi0UmJyf/fH/n
/nPBSPArR9rBi23MFvvnlfGVymI/xmKQMZAX5wMRpmsI8+A6XsyB1RIXRhJi
Ijoujv8xlvw//KEcagWzYlgQNsuViSVE6Z9c8m0s+a9YEvz8y+0d288Fk0Gh
NUtRLgmv4F4uzUv9IYb4BEOA2h9eEbviV85hp3AjEcJJhPgDDDGJISBrf1IX
rYMcEOJECDcR4i0M8aOsBZjWzOstRAgvvvcP64bWfet46jgv9bsY4n8TXtys
HQLPxYiDh5FC/C/Ci1u0QpRBBIE1XKlQhsq3Mc4PCUO25ogDTzOxpDj/Qriy
TTdOGX6qIvU7GGyC8KdNN1g5Eey3Mdj/xGBwRmac2qEbzEEEe0MV7Psb2jb8
0jXq4gfqB3MSwX4Lg/0zBvt2T1/PZxX9FfzAdt1gLiLYtzDY/8BgB94deffT
6gS25jt1g7mJYGkM9k8YDLrAT6uj1bxh2KUbzEMEex2D/WO2An7cGG3kLehu
3WBeIthrqmCqCtAP5iOCvYrBPiIqYI9uMD8RbACD/XeiAm7VDRYggr2CwcaJ
CkjoBqsggr2Mwf4bUQG36QYLEsFOq4KpKkA/WCURrB+D/QNRAXt1g4WIYDhA
EP4rUQH7dINVEcFexGA/yDZBn1ZHqnn/vV83WDUR7AVVMBi04MhDzBWshgj2
vCoYDubGBQPBaolgpzDYhwSzA7rB6ohgz6mCqZjpB1tABHsWg30fg8HAGcar
/MCDusHqiWApVTAYkuJoMWewMBHsGQz29xgMRk84BmLvHdINtpAI9rQqmIqZ
frAGIthTGGwsOyjAMR17r0M3WCMR7ElVMBiO4pgyZ7BFRLAnMNifZce3MDdI
ptNpGHIAU2EhAsFnMIDlbd3tukCLCaDHEQjHqVVQOTDYHzjZ18cGzXE2U4PJ
U5jNF9p+4U4m2RglziyKsBLCvzW0wGQqzuZmSZjWAEFIO6GacECfWBNBrA+J
fYc/+KFBAeXgABuyd7KZXAxIsEyKsvlFJ2sa+1nwM2wANc27exEa44kJfC+d
ZigpEMHOCV4OhAF5No7vUE5BobH3Ztv9z/1hPzf2sC7/JQT/XuSP404f1BgY
JM2K29hsKsJmax4wFIxl84ghNoccq5GpT08j9eFhoA4S8dBQSMSJbAzoKyey
0OnhHLwcZl44fRJzUY8Q1L+J1N/kD/KkNAF0oe7BRuDE5t6jrE+cjHDKPmCP
lFMpNgPuA7psqt4CdH9Ws2TJjGeBB7qDI7p0mgk6jyEdHJfKU9go0hgZYdUz
I6ziHFjQv/iLry+rwyGyPtBSAugkAuEwU570RlgDz4YQ4+NrZIy//VuDGMsI
jBOIgWM8eZqcBhxMY3ZirzUNdDkB9CgC4ZBNnlinWKccYZmR5EDXmAa6ggB6
BIFeyTZV6SRUfyTCpuIsS6am+FTcHNByAiiJQDig4pN31sQkWdvHziQ4+9mZ
v9E00AoCCO+l8ZG8StAJuc/aszRPuOuyQKJgCKiFAHoYgXDsVH7nO8PD0yE4
Y4aG+OqAOYArCYCHEADHU+XfOZZM/ryqo4MruME0wEoC4EEEOJVtMrHbSKf5
EoM5gKsIgAcQ4Lns4IGNxaPQ0PAFCHMAqwiA+xHg2Wwd/KwG+gh2fmwxDXA1
AdCDADgKcsAZP9zR3S3Xw02mMVYTGN2I8XR2SPBPq6AaBgf56oY5gDUEwDcQ
4Kns6HIqDNXAuvutpgGiBMB9CICDqPJHXh8c/GltPM5bj22mAdYSAF0I8Li6
GqZD3d1QDdtNY6wjMO5FjL7sJOij1VI17DANcA0BcI8K4C+3d3T8ygkjBPbK
PMB6AuAYAuDgBZal0ueC0FuwV+2mAa4lAO5GABxi8PUzGCjw0DtNA2wgADoR
4LHsmP4ndZCprHfYZRpgIwFwFwKczFbyD6+ABmN8nK+omAOIEQBHVQCqSjYP
cB0BcCcCnCAqeY9pgE0EwB0I8ChRybeaBrieADiCAI8QlZwwDbCZADiMAEmi
km8zDXADAXC7CkBVyeYBWgmADgQ4TlTyXtMANxIAhxDgYaKS95kGiBMAB1UA
MOYH/2cJsIUAOIAAOEiSF3Mwf/abBriJANivAoBJFM5YZgVwMwGwDwEezI6O
YELMQx8wDXALAbBXBaBSYB5gKwFwGwI8QCg4aBpgGwGQUAG8fRT6+8HBWQJs
JwBuRYD7s2MvmJLwMfYh0wBtBMAeFQDMeXAaAuegaYAdBMBuBOghFHSYBmgn
AHapAF54ElYIYGoIzZRpgJ0EwE4EaOcPqi0Iyovt4WxrDvNgPri83TT+LgIf
gYWt/KEa8PiuhWRPz3gcslkQzrUEg2cTlZViShDgcToeCn3lcTjgMzhmrC0W
A75QTl7iq0K+sECAc3boN0zz3U3w3YF8b+QPPqjxzIaHjvb2f2tpaABewPGL
bp9PTAuCeEYQPFBGqICn4ii7MSnwMRx2PhIIwIdQFOSAdAgHYfmioJfw/rBp
LXsILW2oZRNqgRakY3h4GMCByLlwMPhFGEkCYXGY3SYEoZZrqYKnXE8a9cBh
0zFcZPsy5PVCCLAEwkFYeY0QwKCqvvB1d3PO5nTcSujYjjo28gdcdUts2QJE
zkYrK891BIOcP5CdEoQI5+8Tp6W3hpE/HAYafhFyu+GAGW99PfSa5ikmCIrb
kOK1/MHBV+Kizc3TbcyvEYFNTAGxXBSkDUGAaL6KbyNw8bwSrkFcvjAXqa3l
p9O4wKaT+cDdS+Degri4dwQX69IDA5BS5ztZvk8yxHV5Ad9HgN+M4KsRnC/g
pXp7P41UV3+e9PshB9gMLh/g+wnwmxB8FYLzRb2+ZHI6EgrNpDweDr4hL+AH
CPAtCI57b/j629vdR4+eiwSDX/WzFpOlPJs35QP8IAEeR/CWLPifd+7fz9u3
QQY2wxA35QX8EAGODbKwAhvLAyPvvvuTaF2dOMSQbsgLaAcB2oqgyxH01dSJ
E58mqqu5za15Ab2dAL0BQS9HUMju7yVaW8+nAgE2F8gH6GECdDOCLssONn8U
a27+1YjTyeYH+QA9QoBej6BLEfTO4Xfe+VmspgY6HjZnyAfoHQQo9oZCM4J+
J3ns2M87qqp49t6cF9A7CdDrEHRJtk6h+/wy7fWyoX8+QI8SoDEEbcrW6Y+j
jY3QJ7LpQD5A7yJAsZMWFl9Qp9AhbM8LaCcBugFBFxF12pYX0LsJUOzrhYbs
8O37bRs28DptzwvoMQJ0vQoUlAIgh8sP6D0EKI4u5MvqXUNvvXW2kw0wAGln
XkDvJUDXIag0M4GR8k/jtbW8TnflBbSLAF2LoPVCZr73r7GmJp69u/MCeh8B
ioMoYUE2kX7Qtn49P0/35AX0GwToGhWoKpHyA9pNgOKITagjEunWvID2EKBX
I2gtkUiJvIDeT4Di8FCoIRLptryAPkCAXoWg1UQi7c0L6IME6EoVqCqR8gP6
EAF6JYJWEYm0Ly+gDxOgOPAVQtn+lHdtkEj78wJ6nABdoQLl8142oeRw+QFN
EqA44BUqs0phos3hDuQF9BEC9AoVqEppfkAfJUBxlC0Es+cpTKZ4nR7MC+gJ
AnSZCvStZFfXV2k2exPzBnqSAMVRtlBBKD2UF9DHCNBmFahKaX5Av0mARhA0
kG0GPwtXVPBZW0deQHsJ0CUqUFgM4WsRYt5A+whQHNoL/qxSWAThaxC35wX0
cQJ0sQpUpTQ/oE8QoItUoLDGBX1pHkGfJEAbEdSHoPwqWEJqBg/nBfQpArRB
BQqriLCSl0fQpwlQHNrLS9EqpUfyAvoMARpWgU7EWlrEMUlpfkBTBGi9ClSV
SPkBfZYAXVBo0OcI0LpCg54iQGsLDfo8AVpTaNAXCNDqQoO+SIBWFRr0JQI0
VGjQfgK0stCgpwnQYKFBXyZAKwoN+goBGig06AAB6i806KsEqK/QoK8RoN5C
g75OgHoKDZqWQWMhfuUaArkvGWg8FJf/rIINaoPaoDaoDWqD2qA2qA1qg9qg
NqgNaoPaoDaoDWqD2qA2qA1qg9qgNqgNaoPaoDaoDWqD2qA2qA1qg9qgNqgN
aoPaoDaoDWqD2qA2qA1qg9qgNqgNaoPaoDaoDWqD2qA2qA1qg9qgNqgNaoPa
oDaoDWqD2qA2qA1qg9qgNqgNaoPaoEUEegO/94nCJHscEQBJDLWFRAYjBjoD
/LUwLCCpaaF2LjjiD5wuEIUx9jgoiO6km/NzeBxibaRW3JLYIqZSKXF0dFRk
sOLQ0BB/3d7RLkbjUdEb8nIMKAPauKa0pGlCqJgLTfjTv37wFIH7BTHYERQr
o5W8bEusRTzafVRMp9Nc08zMDDsaAlbw16AVNPYke7jGhpYGXi7YEuQ6wSOu
c5THF1xzIfJqObkmFMnFyPnCPl4pQPxk30lxeHhYHB8fD3N9QXFiYoJrBO2g
MdYW44cCnDuEdQ9ecX1DUl3GAaewcvC3xd3cQ8gj8FzOPeA7NTUlXM41uMXp
6Wn+HkiDz/d37hdXrF8helo8hSd6Jb93cM9DsRA3jyXMas6tnPMCDy9JDqyQ
uQxhcoJhY2Nj0bngcjm/d4rCjCA60g4xEAnwimEJBz/CdukJLZUJsVbAk/KI
/rBf7Ep2iZOTk/DDbJeeUEQmNCWI/qRfrAhXiL2pXkhn+LG2DKHKTZWXhlCT
TIj1T9AfBcNBcSA9wNvDGCfkED/44AOx3FfOf7ebHecpNKdFckqzWqvqqOJ9
EOtzrud0XNkKg6YpIsi0XIWm1Sg38ay1rY/Vi+8MvwO/+MaO8yiTCP+puTkK
za1B4uYccYrNsWZ+xrUSCV6uaC0U/MoKzW+hhB5IBcTWRCvP9xsJfk5NfgWj
Vi8bw7KtOlEtnkidgF+au4ia+0JqEaGArBZckGzvjrwLP0XHj4O+ryXeIgrj
UgMBxw1J445pIfsnefNLqY7IsVv0KI1nKb1WGErycFmZVtsMUnq1MJSqiXTa
YZDSQGEoVRG5tNMgpVcKQylE5NJug5ReLgylSgnKNeoS17et57l0q0FKpwtD
KShDsc65MdrIKd1mkFJ/YShlJnNsiFcbreUVt88gpZcKSwnONpbanMx+g5Re
LAylgARV0V8hPtT3ECdz0CClFwpDya9Ib5jwwaEdBik9XxhKPkV6N0WbeHof
NkjpVGEoeeVOHpYNWgTxhokbxBlx5ohBVs8VkpWT/xQl/Owz/Gtj/xgzwSi1
ZwtDzSND9bPHnXzAhqNJ9p5EkRM0Wq+pwtDMDN5guaZNwOFSpzQcnh3TZwrD
1CXXNTsp4Ac5eSYiW5xGzJLu04Wh65RJz0h0JzA9JcpOC4yfKjDjboGvdmb+
IWOXBcZPFoaxXO2waMBnHTPZc0xirUid2VF/osDUkwIuI4o0dbcF6o8Xhno5
4Xo5Qd1jgXpfgan3CdgwizR1rwXqvYWhnhEwI7k+pUgiBXWfBerfLDB1yPOk
oM5/BXW/BeqP5Yu6W0kd/ykcdxK0AxZon8wXbdfFtNFtF8G4wgLjEwVkDN3k
OMk4aIHxo/li7FQzhutKiWwLraRbaYHuIwWii+aSdEMW6Cat090stxnj46Iw
PCw6+vrYq1A8zloJ9py11GzOfuYMfF7LD3VklWEleAhRVRZEHbcuagO/Z8Fn
ZpB8Oi0GOjvFUCzGj2uORsX2jg6xv79f4OuprB1hz8WTTHA0Huefw3G+cBis
4GWFVIopHsF4MzOBC82A8fsoaUa1BTMetm7GWn4PF/snGcNRLqQykRADkYjo
8Hi44J5kkl9AHxsbC3BmQbiuqbquvoUVgUOhCIQHX0JtbejL4CD6MjnpVPZB
Tp766uVgyZQaC6Y8ZN2UVbIpExOM1hBP9ZCkrpYZE2PKQPUIq/HJycmFkinK
i/BHu7u5IXA4hA62tPAQbuYlN+WNNxDyeUiTM5zBdwW8Fn+Gd8z4Xi97fRhN
qbVgyoPWTVkpd8JwgYEZ0MAU7WepD6eGvB3hCk7HddHl+pXs1LrMW8aUxqMQ
tnU1s5e9auUgcX6/QeC/nQ5s6ywofcC60hb57GXVC40C1KPiWjmrcX56SAdh
Qie7WDuSGTpdxMqKovutK1ouk2XJDJkIp6viijupiJ3hQiTI5xCOPCvqsa5o
mdyRskbcAQ04ow/ZqLxur5TlJCoq37K6rctqlskyKA87cYKMfhdrMpRX/0lZ
itpy5lnWN6zLWiKTZTT9TE6I0e9l8mB7zAaFLP+iRaI0Ar2otvIt6z7rshbL
sqBGWPJVM1kDLBlhVVjeiQCyHD7Wl7BGhDWdVIW58qysy7qyRsXpVcG6MujG
oK+HRn7ThXnIIMpAH8SAfl6KUUZUYr6l3mtdaoNMk8mpYa39G4OD0h4Jp/JU
w1tGqkspVVmb7jwrvMe6woWyQsa4kQ1ex8fHMzstlI1JuaKH5+mKKYtxoBJf
K88OYfMk75h1eWGJvJededBVKzdqXNxWutTy3nuPJTvMBfKu7G7ryhbI1cLO
wio2IzmWTGb2eSiVueWeW1ZWy9qbV8soVlYUdVpXVHfByfbO8LBwswS/h7Wg
PtbGwBEe+bgPPhCFpT5NSlbk3GVdTi1xZm3Vk/PJJwWTc9S6nBriTNo+R3Lu
tC6nijh92udIzh3W5YSIc2fXHMk5Yl1OJXHu7JkjOYetywnKNMfGxOWsduDc
ScyRnNuty6mQabKRzUJWOzBS3ztHcjqsywkomoJa1rKxyW9mw82llnMof3JC
bGjOxqsc+MAcyTloXY5f7neYlCNMErx9aI7kHLAux6doCqAbhUnT7XMkZ791
OV5FUxCWmoIjcyRnn3U5SNPBl1iEdevEhayWJkXxjjlStDdfitj0Zs8eUVjO
1/rECLsxVcJcybrNuiy3TBOuH4A0Nu2W1xQkeZzMXOViwrpEl5yLLAf5xO27
34XH8iJSeat1lU75HlZJ4OIZPOJUNbPcXAxS91iX6pCrFa4tQoVCxcJnKNdZ
RGp351GtfCkY35fVuopI7S7rauWzktdtKASP2TVOeeVFzC5eF4PsnXmUnU6L
Aly9FEVStruIZLdbl12mrG1YZ2OjnnJCtqeIZO/Io2xFbVOyvZdW9q+vil+l
WbYtj7LDYeyeNGT7Lq3ssxvjGzXLbp+NbJdSNt5g20oi4STU+otI7bY8qYXx
x/g4pTZQRGq3zkatU60W6zV7LVUhtaKIpN6SB6lYqaTUYBFJvXk2Uh1ZqbAL
Lxp1Eyori0jlTRZVJhKQu5TKUBGp3GJO5WZ+7xfPsNeD7JZ0u8U4G0R6HA7+
2BEMiin2/hC7wTFTAmz4UnRUWPdZaQpbqorIlrg5Wzbw+zDI5bLBArCihd1A
dpyd2d1HVX/zRvU3VDr37+fHhKXjo5WV3E6I0y/FHIdt1QonMb9IJ6uLyMkb
zTm5lt/7QK04LDkJToR9Pu4OuASOgXMTExMVnEUQrrpwQ+GjvpMn+WHR5mbu
FBSFEN3sEcKNSIbyPUYKQ8NhtSkKQ2uKyNBWc4aukgydYK/T7NYtucHTjDnU
0d6u/DM/8iZQeWfs4OAg/zixZQs3FU50QaqTBMvSlNQQcAcfeIBfvAfCcMEb
DMSdJvgejMU3beIW1RaRoTeYM3SliEMySKE25kJLQ4OY7OnhTrHiKzgq/q0U
2FgL5sGpfxlrKVeHQg4xGgoxAk6nE83A1UDcdQSvIWknJzM7DIrBoM3mDGqR
ekI4z2LMIEib0dHRNRKagIdJZx7eD7Cs8nd1yVrwHrZyw/lXZG5cb86N5ZIb
0CNCdwAn0tjY2FodN1jDJFZDT8Dm6o4id2OTOTeWiTjsBYlpVumRQIB3frDf
9RrCEgeRIMVuyXXmLGmWLJmG3s7jESOs5pNd+JeyrtWxRJklziK3JGbOkiWS
JTCcSvr9YgSGQr29/JLdRsISuRtXZkmxW7LRnCWLJaHwupOdNJHqajE9MMD3
Z1ynsCSCg++MJcoscRW5JRvMWdKoaEuSTCJ8Jm/uvZ5D4vSaW6awhmpTit2a
a81Z0yBJhBMozt5Pv/SStFXWrTxxLrKmjMgad5Fbs96cNQsliWMw/mVDfthC
cyPRpqAV7gst4p9D5ni6urLyitSaa8xZE5asGfR6xfiqVby5jWv2QC6VNW+x
WwVmTLG7ss6cKwskzXBOdMM0sqMDmpgthDEu6VDZGJgslXd1eYvckLXmDKm7
oHEZfP31zObn1PHjYoqZBEd4pOM+ZseF6utFx/vvC8VuRdScFfLSlrIx2apj
BYzxqueJFWvMWVFDNB7bS8SK1easqKJbjB0l4sbV5twIEc3FrhKxYpU5KyqJ
5mJPiVhxlTkrgpJEWJaMLlrEm4tEiVix0pwVFQqJ0bo6PvHfWyJWXDk7K9rY
7Hb0/fc5yL4SsaLFnBUBSWJfZaXY/8QTHORAiVixwpwVfkniiNMpJlpxHnuo
RKxYbs4Kn7LZbGpS7cmf71ZcYc4Kr7LZrKrizeaRErHi8tlZsZa9HnWxWejo
aGbH/Hy3Ypk5K1CiU+xhr6vkqbnHU1KWLDVniVtuPtnrbbhck1nok6xRyZnv
9jSbs0detOFNKrt9jBaVl7BDEXMOOaX7acmhSekWUSyml6JNS2ZnUxu0wlLT
I9vkLGGbmszZJK8ep9nrZHbxVLbKVcJOLTbnlHI/AKyxTykabsktRQtWmpYt
mp1lQ9IKvKhhmbuELWucnWXQso/rWOYpYcsazFkm2wMjqsQFNiot8146y36T
6Oz8siYPluFs0gGW/aK3t3cmEApAGar4wtm5pkw0yjXfpXPt7J15cg1/YoT/
OOJMKpWajkQip9njkk8++WRC76tO4VwWupQW8hsMLdqyIwqlc/556Bwu7zkv
dC4EG1vKMIvgua6L9eZdhNN2hHYxMA9dTOi46DTo4oJcLjpVLo7jbCAz+lBa
WDEPLdytY6HHoIV15iyUkpC0MDgPLWzXsdBv0MLaXBY6MhZKGegm3Kuch+5t
13EvaNC9GuPuwdwrTbsXmofu3aLjXpVB96qNuTcmWTVDu1c1D93bouNerUH3
qowNBxOYdxn5Suuq56F1rTrW1Ru0LmTMujAmHWldzTy07nod6xoMWleZ07qf
L6qqEg8L/EtfrDDev4ev+asy6T34vlmjZGftPLQzpmPnYoN2BnPa+e8P7N79
j5uvvno8Ho0K5Xg/E2Q2rWcWxLmdcW5JK3u9CCcsmT2D88nOa3XsjBi0s0Lb
zkoMkRgZGTkL36QTL1jxS8xj69bpWLfMoHUBbetCGOJH8P3FC1cS5rt1a3Ss
W27QOn9O61RZ5ygR61bpWHelQet82tZVEVlXKtZdqWPdKoPWeXNap8o6Z4lY
t1zHujUGrfNoW1dNZF2pWLdMx7p1Bq1z57ROlXWuErEuomPdtQatc2lbV0Nk
XalYt1jHuphB65w5rVNlnbtErGvQse56g9Y5tK2rJbKuVKyr17Gu1aB15Tmt
U2Wdp0Ssq81a95uOIx1nlyxZkrFui0HryrStq8MQ32Lypy+UP9+tq8paJ/am
esVQJPRLd8i9+qOPPhJu0bTOpYwL78i24XUQYUEJOxbk926lY6In5YHjzwci
gQhsM96q6ZxTGT+TbJJt9SVsm//iRJNtk8+9jH1wpLaFDhnrfCAaUCRduITd
8+i45yDc26bX6v2V8JsyT1nr9773PTnxFpawdU4d61yEddsJ61xy4k0Jn/uT
/k+rI9VPvPjyi7jgKa9El6R7ZTrueQj32gj3yuWWL+veyYH0gAe+m+JDCxtL
ykJ3xkLKNh9h2w5+71HaRtkVQLcWlZRbDjE2FlM6FCAMauf3TtkgVQ7htW/5
Csa8d0X+/WNhJjO8ki8TqCzZmT3VuA3wCq8ZCE0lYsUyQvadKHFJiUhcqi0x
UiISm7UlNpeIxIi2xKUlInGJtsRlJSKxSVvi5SUicbG2xCtKROIibYnLS0Ri
o7bEFSUisUFbYkuJSFyoLfHKEpEY1pa4skQk1mtLvKpEJC7QlrjKusSvU8nk
uVRTk2WJim8mwRWUX/oDgS+9b77pNaiyTlvl1dZVnks9/XReVGp8kwimge/v
6ez8wjc0JBiVXKsteXURSdb7ClA58pSlm8rrGm35a4pIvt53d1wW5Fdry48W
kXy97914Lciv0pa/tojk631nJmBBfkhb/roikq/3pZdKC/IrteVfU0Ty9b61
Um1BflBb/voikq/3tZM6C/Kp9VRJ/rVFJF/vqyNhC/KpFXZJ/oYikq/39Y9G
C/L92vI3FpF8va9rNFmQT12AkuTHiki+3tcrmi3I92rLv66I5Ot9ReJyC/Kp
q7aS/E1FJF/vaw4rLMh3a8u/vojk631VYaUF+dSOB0n+5iKSr/d1g6styHdq
y7+hiOTrfWUgakE+tVVIkt9aRPL1tv1fY0F+ubb8G4tIfkRH/gYL8qltdpL8
eBHJ19t+f50p+RdtU5WlH0XVW4pItd7O+c2mVKu2mGYqW5J8UxFJ1tvxfqMp
yZktoZ/7m/yKCr65iNQqNql/ve/QobNNTU0ZtTeZPathC6ezbMXHH38sV+wt
RSRVvak8JYYikRlfJLLrgw8+kPf15pRKbLlMPfvCs3eh2q1FpFa9IZyrFT3s
UWBvhOJxvsF7m0HVelslO1H6tiKS7r+4omXpvLcR+Rcq4pkvVGjb4FHaQMk/
huq3F5F6j456B6F+u6Z6VuE/+MHXlzWWf+EL+zIVfi9KbisiyU4dyS5Cchsh
2SnfT02xSk6ySo68+MQzz3DJ96HkHUUkuUxHsoeQvIOQXHaxZFbLaS65GyW3
z6lkd0YyJdNHyES+7qxMQtr9KG3nnEpziIunpsTKrJwAoQYpOkCNUsGDKGDX
HKejMDMjcgrsP8Ve4pfZBQw3ifruOaK+jKD5H5HSnjmitFSb0q1zRKlZm1Ji
jihFtCndNkeUlmhT2jtHlJq0Ke2bI0qLtSntnyNKi7QpHZgjSo3alA7OEaUG
bUqH5ojSQm1KHXNEKaxN6fY5olSvTelwfih99uTixaYoLdCmdMQCpQt21s2E
KivPnQgGvQZZ1WmzusMCK52dcHI8wSjFWm2Kd1qgaGDnGhk3F90abbpHLdA1
sNNsVnSrteneZYGugZ1hs6JbpU230wJdAzu5ZkU3pE33bgt0Dey8mhXdSm26
xyzQNbBTalZ0g9p077FA18DOplnRrdCme68FugZ2Is2KLjWdleh2WaBrYOfQ
rOj6teneZ4GugZ0+s6JLLdxIdL9hga6BnTmzouvVptttga6BnTSzokut/kl0
eyzQNbDzZVZ03dp077dA18BOlVnRpZaTJboPWKBrYGfJrOg6tek+aIGugZ0g
s6JLXZ+Q6D5kga6BnRuzoluuTfdhC3QjOnT1NsLmoktd+5LoHrdA18DOCIN0
XUq68I5M9a+QZdICSwM7GQyydCpZZsyUKD5igaKBnQcGKTpkiueCsZjCwEct
sFPuFDhy8OBnjY2NF+4UMJ6VH37463Kfb+97770nG3fCAjXllf1Ub68YCYV+
GfV673/zzTd1r8krqblk47JXjJ575tSp/4TsTlpgp7wSL7ETU+xzhn6+MxDo
HBoa0r18rmRZrnPJ7q+R6mMWqPovNlKmKv8OWoYyHKlN25Ohfdn771N0/zOy
/aYFth4dtg6CrfZVeqf4L4Lwtbu+fsazYEHG0L9Bir0WKDp1KLoIitRVdfkH
WqcE4fOk3/9ppLr66f5TpzjFv0OKfRYolulQ9BAUqavgZRdTPJkewP0C/wUp
Pm6KojtDkaLlI2hJV34z8Skqf49UnjBFxSFOu91ipKJChg8Q6NKVWgh5LhwM
yogfIuCTBgEvJyLz66b4t9yEpwzGWZYjztMG4yzNEecZg3Gac8RJGYwTyRHn
WYNxluSI85zBOE054pwyGGdxjjjPG4yzKEecFwzGacwR50WDcRpyxHnJYJyF
OeL0G4wTzhHntME49TnivGwwzoIccV4x16xrxhkwGKc2R5xXDcapyRHnNYNx
qnPEed1gnKoccdIG44RyxPmWwTiVOeL8lsE4wRxx3jAYpyJHnN82GIfqGZVx
fsdgHH+OOG8ajEONE5Rxvm0wjjdHnEGDcajhlDLO7xqM484R5/cMxqFGoMo4
bxmM48wR5/cNxqEG7co4f2AwTnmOON8xGIea8ijj/KFmHJcyDrwjx+DFca1f
GNIs7lQWz8Iryv6RZlmHXPbLrV7vhZBv6yj/RBB+HSov7z19+rQK6h2ijFOC
Us5JXjp1ihfDvwMr/DFRTG+ewMviX0sV/oQ/6A/m+fFL8Pg/5Q8XDb/5Ibj0
j4d/eGmeC5f9f6HaJkg=\
\>"]],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0, 1, 1], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[1, 0, 0], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883395396376`*^9}]
}, Open  ]],

Cell["\<\
Apparently, we need to move the books once more to the left to get the same \
effect. Let's try it.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitBooks[3];
MoveBooks[First . Left . Left]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0, 1, 1], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[1, 0, 0], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883422115126`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[1, 0, 0], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[0, 1, 1], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883422583876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0, 1, 1], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[0, 1, 0], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[1, 0, 0], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883423208876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0, 1, 0], 
     PolygonBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, -0.875}, {2.25, -0.75}, {5.333, 
       2.333}, {5.333, 9.333}, {5.167, 9.417}, {3.5, 7.75}, {3.5, 10.25}, {
       3.333, 10.333}, {0.25, 7.25}, {0.125, 7}, {0, 7}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{4.5, 8.75}, {2, 6.25}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.271, 6.521}, {3.167, 9.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{0, 0}, {0.16, -0.34}, {0.382, -0.618}, {0.66, -0.84}, {
       1, -1}, {1.421, -1.079}, {2, -1}, {2, 6}, {1.421, 5.921}, {1, 6}, {
       0.66, 6.16}, {0.382, 6.382}, {0.16, 6.66}, {0, 7}, {0, 0}}], 
     LineBox[{{3.167, 9.417}, {4.5, 8.75}}], 
     LineBox[{{2.25, 6.25}, {2.25, -0.75}, {5.333, 2.333}, {5.333, 9.333}, {
       2.25, 6.25}, {2, 6.125}}], LineBox[{{0, 7}, {0.25, 7}, {3.5, 10.25}}], 
     LineBox[{{2, 6}, {2, 6.25}, {5.167, 9.417}, {5.333, 9.333}}], 
     LineBox[{{0.25, 6.547}, {0.25, 7}}], 
     LineBox[{{0.125, 7}, {0.25, 7.25}, {3.333, 10.333}, {3.5, 10.25}, {3.5, 
       9.25}}]}}, {
    {RGBColor[1, 0, 0], 
     PolygonBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, -1.875}, {
       4.25, -1.75}, {7.333, 1.3330000000000002`}, {7.333, 8.333}, {7.167, 
       8.417}, {5.5, 6.75}, {5.5, 9.25}, {5.333, 9.333}, {2.25, 6.25}, {
       2.125, 6}, {2, 6}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{6.5, 7.75}, {4, 5.25}, {4, 5}, {3.4210000000000003`, 
       4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {2.271, 5.521}, {5.167, 
       8.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{2, -1}, {2.16, -1.34}, {2.382, -1.6179999999999999`}, {
       2.66, -1.8399999999999999`}, {3, -2}, {
       3.4210000000000003`, -2.0789999999999997`}, {4, -2}, {4, 5}, {
       3.4210000000000003`, 4.921}, {3, 5}, {2.66, 5.16}, {2.382, 5.382}, {
       2.16, 5.66}, {2, 6}, {2, -1}}], LineBox[{{5.167, 8.417}, {6.5, 7.75}}],
      LineBox[{{4.25, 5.25}, {4.25, -1.75}, {7.333, 1.3330000000000002`}, {
       7.333, 8.333}, {4.25, 5.25}, {4, 5.125}}], 
     LineBox[{{2, 6}, {2.25, 6}, {5.5, 9.25}}], 
     LineBox[{{4, 5}, {4, 5.25}, {7.167, 8.417}, {7.333, 8.333}}], 
     LineBox[{{2.25, 5.547}, {2.25, 6}}], 
     LineBox[{{2.125, 6}, {2.25, 6.25}, {5.333, 9.333}, {5.5, 9.25}, {5.5, 
       8.25}}]}}, {
    {RGBColor[0, 1, 1], 
     PolygonBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, -2.875}, {
       6.25, -2.75}, {9.333, 0.3330000000000002}, {9.333, 7.333}, {9.167, 
       7.417}, {7.5, 5.75}, {7.5, 8.25}, {7.333, 8.333}, {4.25, 5.25}, {
       4.125, 5}, {4, 5}}]}, 
    {RGBColor[1, 1, 1], 
     PolygonBox[{{8.5, 6.75}, {6, 4.25}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.271, 
       4.521}, {7.167, 7.417}}]}, 
    {RGBColor[0, 0, 0], 
     LineBox[{{4, -2}, {4.16, -2.34}, {4.382, -2.618}, {4.66, -2.84}, {
       5, -3}, {5.421, -3.0789999999999997`}, {6, -3}, {6, 4}, {5.421, 
       3.9210000000000003`}, {5, 4}, {4.66, 4.16}, {4.382, 4.382}, {4.16, 
       4.66}, {4, 5}, {4, -2}}], LineBox[{{7.167, 7.417}, {8.5, 6.75}}], 
     LineBox[{{6.25, 4.25}, {6.25, -2.75}, {9.333, 0.3330000000000002}, {
       9.333, 7.333}, {6.25, 4.25}, {6, 4.125}}], 
     LineBox[{{4, 5}, {4.25, 5}, {7.5, 8.25}}], 
     LineBox[{{6, 4}, {6, 4.25}, {9.167, 7.417}, {9.333, 7.333}}], 
     LineBox[{{4.25, 4.547}, {4.25, 5}}], 
     LineBox[{{4.125, 5}, {4.25, 5.25}, {7.333, 8.333}, {7.5, 8.25}, {7.5, 
       7.25}}]}}},
  AspectRatio->Automatic]], "Print",
 CellChangeTimes->{3.4425883423521376`*^9}]
}, Open  ]],

Cell[TextData[{
 "This tells us that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{"a", "\[CenterDot]", "b", "\[CenterDot]", "b"}]}], 
   TraditionalForm]]],
 ". This group is apparently not commutative. We enter this final definition \
into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Define[b.a, a.b.b]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["We can see the whole group:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = Group[{a, b}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425883471958876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " expresses all 6 elements in terms of the generators ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ". Here is a multiplication table: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}, {0, 1, 
      0}, {1, 0, 0}, {1, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {1, 0, 0}, {1, 1, 
      0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}}, {{1, 0, 0}, {1, 0, 0}, {0, 
      1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {
      0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 1}, {1, 1, 0}, {1, 0, 
      0}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 
      0.8}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 
      1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.4425883497271376`*^9}]
}, Open  ]],

Cell[TextData[{
 "Is this really a group? We can tell from the multiplication table that ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is closed with respect to multiplication, and that there is an identity \
element, ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ". We also recognize the familiar Latin square property that we have seen in \
all of the other multiplication tables. Since every row and every column \
contains exactly one ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ", every element has a unique inverse. The only property that we cannot \
check using the multiplication table is the associativity property. Rather \
than manually checking all ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["6", "3"], " ", "=", " ", "216"}], TraditionalForm]]],
 " possibilities, we can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " do the work with the command "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckGroup[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["\<\"This set of elements is a group.\"\>"], "Print",
 CellChangeTimes->{3.4425883523052626`*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " shows that this set a group. We call this group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ", the permutation group on 3 objects.\n\nCan ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " determine the inverse of an element?  Let's try it:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["(a.b)^(-1)", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(1\\/b\\) \[CenterDot] \\!\\(1\\/a\\)\"\>",
  StringForm["`` \[CenterDot] ``", $CellContext`b^(-1), $CellContext`a^(-1)],
  Editable->False]], "Output",
 CellChangeTimes->{3.4425883545708876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Apparently, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is using proposition 1.2, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "\[CenterDot]", "v"}], ")"}], 
     RowBox[{"-", "1"}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["v", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SuperscriptBox["u", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ". However, it does not know how to take the inverse of the two generators. \
So we need to help ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " out by telling it the inverse of a and b using the commands"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42755868571875*^9, 3.42755868590625*^9}}],

Cell["Define[a^(-1),a]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["Define[b^(-1),b.b]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "Once ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " knows how to take the inverse of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can find the inverse of any element."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["(a.b)^(-1)", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
  StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
  Editable->False]], "Output",
 CellChangeTimes->{3.4425883611646376`*^9}]
}, Open  ]],

Cell[TextData[{
 "In fact, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can abbreviate the inverse of an element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", "x"}], TraditionalForm]]],
 ". So the last example could be entered as"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["1/ a.b", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
  StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
  Editable->False]], "Output",
 CellChangeTimes->{3.4425883633521376`*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " performs the product first before taking the inverse, so no parenthesis \
are needed. Using this abbreviation, see if you can determine which elements \
of this group are their own inverses."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "EXPERIMENT:\nYou may have noticed some simularities between the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " and Terry's group. How are these two groups related? \n\nHint: Terry is \
colored with the same three colors as the books. Can you find a \
correspondence between an orientation of Terry and an arrangement of the \
three books? "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The relationship between Terry's group and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 "  can become crystal clear by looking the multiplication tables of these \
two. The following ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command reloads ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ", and displays a multiplication table."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2,e];
Define[b^3,e];
Define[b.a,a.b.b];
G = Group[{a,b}];
MultTable[G]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}, {0, 1, 
      0}, {1, 0, 0}, {1, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {1, 0, 0}, {1, 1, 
      0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}}, {{1, 0, 0}, {1, 0, 0}, {0, 
      1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {
      0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 1}, {1, 1, 0}, {1, 0, 
      0}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 
      0.8}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 
      1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.4425883666802626`*^9}]
}, Open  ]],

Cell["The following command does the same thing for Terry's group.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
G = InitTerry;
MultTable[G]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}, {0, 1, 
      0}, {1, 0, 0}, {1, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {1, 0, 0}, {1, 1, 
      0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 1}}, {{1, 0, 0}, {1, 0, 0}, {0, 
      1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {
      0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 1}, {1, 1, 0}, {1, 0, 
      0}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 
      0.8}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 
      1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"Stay\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"Spin\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"Stay\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotRt\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"FlipLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"RotLft\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"Spin\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"Spin\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipRt\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"RotLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"FlipLft\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"Stay\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.4425883688208876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that even though the names of the elements are different, the ",
 StyleBox["color patterns",
  FontSlant->"Italic"],
 " in the two tables are identical. Since all of the group properties come \
from the multiplication table, these two groups must act in exactly the same \
way. We say that Terry's group and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " are ",
 StyleBox["isomorphic",
  FontSlant->"Italic"],
 ". We will cover isomophic groups later in Chapter 4.\n\nLet's see an \
example of a larger group. The command below produces a picture of an \
octahedron. (Think of this as two square pyramids fused together at the \
bases.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ShowOctahedron           ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425883752427626`*^9}]
}, Open  ]],

Cell["\<\
This happens to be the shape of an uncut diamond. (Many other gemstones are \
this shape.) There are eight triangles, which are colored with eight \
different colors. To see the other colors we can flip over this octahedron, \
using the following command: \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotateOctahedron[a] ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425883809146376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
Notice that in the flipping, the orange and blue faces switched places, while \
the red and fuchsia faces went to the other side. We can see even more of the \
octahedron by rotating the face closest to us, that is, the face which is now \
black. We can rotate this one third of a turn counterclockwise with the \
command \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotateOctahedron[b] ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425883833208876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
Between these two actions we can see all of the colors.  But there is one \
more thing that we can do:  rotate about a vertex. We can rotate the \
octahedron one quarter of a turn clockwise about the vertex closest to us \
with the command \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotateOctahedron[c] ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425883855708876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "With these three actions we can rotate the octahedron in many different \
ways. The set of all actions formed by combining actions ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " form a group. This group might be of interest to a diamond cutter, for \
instance, since it would give him all of the possibilities of orientating his \
diamond before he begins to cut it.\n\nEXPERIMENT:\nIn the previous command, \
change the letter ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " to either ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and re-execute the command. Repeat this for different letters, and \
observe what happens. Can you get the octahedron back to the way it began? \
(The face closest to us was red, and the fuchsia face was immediately below \
it.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "In experimenting with this group you probably noticed some truisms for this \
group. For example, the action ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", when performed twice, returns the octahedren to its starting position. \
The action ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", performed three times, also gives us the identity. Finally, the action ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " must be performed four times before we get to where we started. We can \
explain these statements to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
InitGroup[e];
Define[a^2,e];
Define[b^3,e];
Define[c^4,e];\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "With these defined it is easy to find the inverses of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ". We can enter these into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Define[1/a,a];
Define[1/b,b^2];
Define[1/c,c^3];\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "You may also have noticed that this group is not commutative. That is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 " does not do the same thing as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 ". However, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 " is another kind of flip, so that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "b", "\[CenterDot]", "a", "\[CenterDot]", "b", "\[CenterDot]", "a"}], " ",
     "=", " ", "e"}], TraditionalForm]]],
 ". To see this clearly, try the following command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ShowOctahedron;
RotateOctahedron[b.a.b.a]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4414774165*^9, 3.4425883945552626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4414774165*^9, 3.4425883946021376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4414774165*^9, 3.4425883946646376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJzt3HeYVNUZx/Fh6dJ7FVExxoYFBAWRJiBIESmuILLLupQVWFiqskgXFKlS
RIoCFkCKAoooRQFFbFhQwVgitqiJmhhLEpMNO+97X+dcvndy9p88yfOsPq7D
Mu/nd+bMLTP3nnO6pI8enDksffSQjPT6HXLSRwwekjGqfvvsnBO/KlokFivy
eSyWUqZ+LP9xXiwW/Ij/80X+D/1D4ePCx4WPCx8XPv7vPC6S/yMl/2D8acJv
i+f/4rj84l/2jGLxPxfNswP3iT9+JE/6Z/x/8lfxf0/8Uxye/mHC0+NmSXjS
B/KkX8Aslf+reKE+6R/wpDJgvidP/zs8vRw8/Zg8/W/x/xX79elSUh4qjkrF
z1xRESrelYqf4v8r7lZo0yrZy9Un/5jkyVUh44iU/ZCkrDqUvSVlfw331olH
KfGfwdPj/8WqAfGmEN97EjXslWrhX6CwKBTWguzDQvzZk6gDxGtCfAdEMSDq
AvFqNEGtqGc9oIXfehaeDtkvCfGNZ/PPAOKQEH/yJBoA8aIQfwSiOBBnAXFQ
iK89ibOtE7XwK8/CcyH7gBBfAlECiPOA2C/EHzyJC4DYJ8QXQJQEoqH1gBZ+
7ll4MWTvFeIzIEoBcQkQe4T41JNoDMRuIT7xJJoAsatgRFPrRC087lnYDLJ3
CvExEKWBaA7EU0L83pNoAcQOIT7yJFpaD2jhh1B4ChS2huztQnzgSbQFYpsQ
7wNRBoirgNgqxO88ifZAPC7Ee0CUBaKDdaIWHvMs7ATZm4U46kl0BmKTEO8C
UQ6ILkBsFOIdT6Kb9YAWvg2F5aGwO2SvjyYouwcQ64Q44tmKnkA8IsRbnkRv
IB4W4k1PItU6UQvfgMIKUNgHstcK8boncSMQa4Q4DERFIPoBsVqI1zyJ/tYD
WviqZ+EAyF4lxCtAVAIiA4iVQrzsSWQCsUKIl4CoDMRAIJYLcciTGGydqIUv
QmEVKMyC7HujCcoeCsRSIQ56EsOBWCLEC54vJNt6QAuf9yzMgex7hDgARFUg
RgGxUIj9nsQYIBYIsc+TGGc9oIXPQWE1KLwVsucK8awnMQGIOULsBaI6ELlA
3C3EHk/idiBmC7EbiBpATLJO1MJdnoVTIXuWEM94EtOBmFkwYgYQdwjxtCcx
03pAC3dCYU0ovBOypwnxlCcxG4ipQuzwJOYAMUWIJ4GoBcRcICYL8YQnMd86
UQu3Q2FtKFwI2ROF2OZJLAIiV4itQNQBYjEQE4R43JNYaj2ghY95Ft4H2eOF
2AJEXSCWAzFOiM2exEogxgqxCYhTgVgFxJhoglrxgHWiFm70LFwL2TlCPOrZ
/AeBGCnEBk/iYSBGCLEeiHpAPGI9oIXrPAs3QPawPDORiP9qIxQOzbPXw9ly
UTgv//fx/+Qq+zqgbhHqIQ9KuRLm0VuSlWfvFr4l8b4rmic/tXFxuLSxv25l
iq39T++v/IdseWNpFxokAWs8A4pDQGULWAYBAyVgddR+En+ZpYCtYSwd/DKF
fSCKDbWbAurm/4y/uQutv5OwdYAtDWx9a/c8aPcACbgfAmpbd5QDtoGxdE5O
F3ZVFBtqNwWcYwEzISBNAlZ6BpSHgIYWMA0C+kvACs+AChDQyAIm2Ruq7HJg
a1l/VwGsqWH0OfhGYe+LYkOtpYArbPOjj/t9JWCZZ0BVCGhlr4C+y/SRgHsh
oKZ1TA1g2xk70npZsaVRWKi1NYHtaCx997xeApZ4BtSCgK4WQF+xe0vAYgio
Yd1RF9gextJVg17CLopiQ+2mgFQLGGD9rew9nuypwPazzS8N2n1ddEA1647T
gU2z1tLFrO7CLoxiQ+2mgEwLSIWAayVggWfAGRCQZQF0QbKbBMyHgKrWMWcB
m23stfY2KjYvCgu19jfAjjKWLiB3loC5ngFnQ8B4207oOvk1EjAHAqpYd5wL
bK61my79dxL27ig21G4KmGIBbay/lZ3tyZ4P7AxjW0K7O0jAXRBQ2brjQmDv
MpbuSLUX9s4oNtRuCphnAU0hoJ0EzPIMuAgCFllAIwi4SgJmQkAl65hGwC6z
ze8iexsVuyMKC7W2MbArrLV0F7h1dEBFCLgUAlZbwDkQ0EoCZkBABeuOy4B9
2Fi6f99S2OlRbKjdFPCoBZxu/a3sNE+2GbBbjK0H7b5CAqZCQHnrjhbAbje2
NrDNhZ0SxYbaTQE7bfOrAQHNJGCyZ8CVELDbXkEVCLhcAiZBQDnrmDbAPmds
BXsbFbs9Cgu1ltgXjC0LrW0iARM9A9pCwMsWUAoCLpWAXAgoa93RHtjXjaVh
dY2FnRDFhtrdAQKO5P8smSdHm/xf3eaJXQ3Ysfyf8V81EuxWT6wjYO/Ff7ov
+BJhxwNbxvqxc1TLQlijaOwUaCOxR4HVfhznyXYF9h1gdRMaC2xpe+ndAXsb
MN3gx0RhoTYSewTYpsKO9mSvA/YtYC8TdhSwpeyl9wbsDcD0YJIThYXaWEB2
pCd7fdS+zgfqEcCWtJfeB7DDgOlpJTsKC7WxL7CvAqsnweGe7I3AvgJsC2GH
AVvCXnr/qENxCLtSsKFRWKiNxL4EbEthb/Fk04A9BGyraLaYvfSbATsImH4m
zIrCQm0k9gVg2wg7xJPNBPZ5YNsKOxjYovbSBwN2ADD9vD0oCgu1cQiw+6PZ
gZ5sFrD7gNXvMpnApthLHwbYc4DpN6+bo7BQG4l9Flj9npjhyWYDuwfYq4WV
a8yhYfH6wnOA2g1UR6HkujKPZR8N1C6g9Ft3WhJqDFDPAKWXG/pzv8U/LZzk
7ARHr4vcRB0V/7h1kvIUKF0SlYTWxOIf/04idgDRVYj8UVvyiW8CFD4JhXrp
KH4dOv7SJxassK8VToLC7VCol9b6WOFkKNwGhXrR74aEbgo2bCK2AqGXI1Mt
eyoUPg6FPRKyIwu3QGHPhFfr0ejNQPRK6OkQQX2+CYjeCe9yiLgdiI1AXJ+w
hXkQG4BIPXk7Dwja6tYDIW++s+MGRC4Q64DQewxpQNBOk4RIB4J22EeA0Psy
AzyJh4DQe0cZtk1S4YNQKO+hc2YLsunAtxYIeQ+dc25AjAViDRDyHjqfBpIR
DwAh76Hz6SQg6ExwPxB6w3MIEHReWgWE3orNAmIUECuBkBO48wkzGbEcCPlo
4XykDgg6V98HhN60HgbESCCWAaF36YcDMaJgRLYnsRQIHYwwwvYMKlwChbIl
Od9Gg2z6SLYYCNmSnG/HATEUiEVA6GCN0UDcAsQ9QOhglDGexAIgdGjMWCDo
I/R8IHRYzjgg6MP9PCBkS3KuCgUEfe2YC4RsSc7Fr2TE3UDoEKXbgBgExGwg
dITVBCAGFozIBYK+z90FhA4Vm2h7BmXPgkLZnp0ryEF2BhAzgZDt2bnKHRAD
gLgDCB0sNwWIdCBmAKFj/aYCkQbEdCB0xOE0T2IqEDrucToQ/YGYAoRsz869
n4C4CYjJQMj27NyfCoh+QEwCQseAzvQkJgKhQ1hnAUEXp5IQd9pmTYW5UKgj
cGdD9g1ATABC7oM495iTEbcCMUmIOUCkAjEeCNmrnDv0AUEXN8cBoYOp5wFB
l13HAqFDuud7EqOB0IHlC4DoBcQoIGSvckaHBERPIHKAkL3KGbcSED2AGAmE
DrJfBARdaR8BhM4RWOxJZEcTS2zPoMLhUKhTHO6F7G5ADANCxhk4Y88Cgm6z
DAVCRkA44+OSEVlA6HSP5UB0AWIIEDpbZQUQdBtqMBA6Z2YlENcAMQgInbmz
ypPIBEL2bWfUaEB0AuJmIGTfdka2BgTdNswAQmcxrQaCbmMOAEInYa0Bgm6r
JiHW2p5B2WlQqHPIHoLsdkD0B0IOCs6w84C4CoibgJCDgjPkPSDohng/IHQ+
3TpPoi8QOh1wPRB0378PEDopcQMQrYG4AQg5wjgTIAKiFRCpQMgRxpmGkYzo
DYRO0NwEREsgegGh80s3A0GDPnoCobNctwBBA1OSEI/ZnkHZ10GhTtLdCtnN
gegOxP1CbAOCxgNdC4QcmpyJWQFxORDdgNAJy08AQYOdugKh862f9CQ6A6Gz
vncA0RSIa4CQ2SPONL2AaAJEJyDkOOdMFgwIGrDWEQidAf+0J9EBCJ3A/wwQ
NC6vPRC6jMAu26wLWLgHsi8Goh0QugrCXk+iLRBygHQm7gYEjaNsA4QcIJ3p
wwFBYz1bA6ErQuwDoiEQrYDQBS32A3EBEC2B0GU1DngSLYCQA6QzmTwgaLzu
FUDIxEdnIntAnAdEcyB0iZGDQNBg5GZA6AopL3oSlwGh67Qcsj2DCptCoa4R
8zJk0xjwJMQrQNA49SZAyGHaWV4iGdEYCDlMO0tbBAQNwm8EhC65cxiIBkBc
AoSuGPQ6EGcCcTEQum7RG57EhUDIYdpZ6CQgaJ5DQyDkMO0stxIQNBfjAiB0
DacjQNQH4nwgdAmqt4E4DYjzgNCFsN6xPYOyz4HCvVJ4FLJpFk0S4hgQNL/n
t0DIycJZ+igg6gBxNhBysnAWYEpGnAWErmn2PhC1gWgAhC7J9gEQNEfrTCB0
YbgPgaB5ZGcAIScLZzmuZER9IORk4SwKFhA0Se40IHSRvI+BqA5EPSB0jb/j
QFQD4lQgdKXBT2zPoMK6UKirHH4G2TS9sU408TkQlYGoDYScspwF+gKiEhC1
gJBTlrNMYEBUBKImELpo5JeeRHUgdM3Lr4Cgua3VgNCVN78GgubfVgVCTlnO
opEBQXOEqwAhpyxn6cpkRCUgdBXSb4AoC0RFIHQR1W+BKANEBSB0+dbvbM+g
wvJQKCcqZxHVIJvmnSchvvckygIhq8Q5y8gGREkgygCh697+AEQJIE4BQlfm
/REIWjugNBC6HPBPnkRJIHQB4p+BKAZECSDkxKmLErsErbJQHAg5cTrLIAdE
ChDFgICFl5MRKUDoKtC/2Gadol2ZUBiYenKSZagT8mKxGNpyOpQ/q538SfEV
sON//YlTmhCnVccT/v5/Ye3uwseFj/8PHseK/Bsbrc8l\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4414774165*^9, 3.4425883946958876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4414774165*^9, 3.4425883947427626`*^9}]
}, Open  ]],

Cell[TextData[{
 "This animation told us that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 " is its own inverse, and so\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"b", "\[CenterDot]", "a"}], ")"}], 
      RowBox[{"-", "1"}]], "=", 
     RowBox[{
      RowBox[{
       SuperscriptBox["a", 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       SuperscriptBox["b", 
        RowBox[{"-", "1"}]]}], "=", 
      RowBox[{"a", "\[CenterDot]", 
       SuperscriptBox["b", "2"]}]}]}]}], TraditionalForm]]],
 ".\n\nWe can enter this into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Define[b.a, a.b.b];", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "To find some other identities, we can try to find other combinations of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " that produce the identity element. Try this out:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ShowOctahedron;
RotateOctahedron[c.b.c.c.a]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884002896376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884003365126`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884003833876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884004302626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJzt3HeUlNUZx/Fh6dJ7FVExBgsWEBREmoAgRaoriOyyLmUFFpYqLB2kSZUi
0kFpUhRQRCkKKCIqFlQwlogtaqImxpLERJl5nvdx7/h9x7v/5CTnrOe4DMs8
n9+dO2+Zed97b/vU4f3TB6UOH5CWWrN1VuqQ/gPShtVslZl19lf580Ui+T6J
RJKK1YxEH/8UiQQ/Yv99Gv2hf8l7nPc473He47zH/53H+aI/kqIH449y/LZg
9Bdn5Bf/sWcUOPvXnyL5f7ID99m/vy9P+nfsD/2nZXJwLwhPfy/H02NmYXjS
u/KkH8EsEn16rFCf9C94UjEw35an/xOeXgKeflqe/o/YHwXs6VpSEipOScUP
XFEaKt6Siu9jfxR0KoKmlbGXq0/+LsGTy0PGSSn7NkFZRSh7Xcr+HtdbZ/+P
JMV+6u+iFWeLKwDxmhDfeBKV7JVq4d+gMD8UVoHsE0L81ZOoBsTLQnwNRAEg
qgPxUjhBrahhPaCFX3kWng/ZLwjxpWfzLwDimBB/8SRqAfG8EH8GoiAQFwFx
VIgvPImLrRO18HPPwksg+4gQnwFRCIhLgTgsxJ88icuBOCTEp0AUBqKO9YAW
fuJZeBVkHxTiYyCKAHE1EAeE+MiTqAfEfiE+9CTqA7Evd0QD60QtPONZ2BCy
9wrxARBFgWgExBNC/NGTaAzEHiHe9ySaWA9o4XtQeA4UNoPs3UK860m0AGKX
EO8AUQyIG4HYKcQfPIlWQDwqxNtAFAeitXWiFp72LGwL2duFOOVJtANimxBv
AVECiPZAbBXiTU+io/WAFr4BhSWhsBNkbw4nKLsLEJuEOOnZiq5AbBTidU+i
OxAbhHjNk0i2TtTCV6GwFBT2gOz1QrziSdwOxDohTgBRGoheQKwV4mVPorf1
gBa+5FnYB7JXCfEiEGWASANipRDHPYl0IFYI8QIQZYHoC8RyIY55Ev2tE7Xw
eSgsB4UZkH1/OEHZA4FYKsRRT2IwEEuEeM7zhWRaD2jhs56FWZB9nxBHgCgP
xDAgFgpx2JMYAcQCIQ55EqOsB7TwGSisAIV3Q/ZcIZ72JMYCMUeIg0BUBCIb
iHuFOOBJjAdithD7gagExATrRC3c51k4GbJnCPGUJzEViOm5I6YBcY8QT3oS
060HtHAvFFaGwpmQPUWIJzyJ2UBMFmKPJzEHiElCPA5EFSDmAjFRiMc8ifnW
iVq4GwqrQuFCyB4nxC5PYhEQ2ULsBKIaEIuBGCvEo57EUusBLXzEs/AByB4t
xA4gqgOxHIhRQmz3JFYCMVKIbUCcC8QqIEaEE9SKNdaJWrjVs3A9ZGcJ8bBn
8x8EYqgQWzyJDUAMEWIzEDWA2Gg9oIWbPAu3QPYgITaGEbFfbYXCgVK4ITRb
LgrH4uVH7HrsJqDuEuohLyrGFTKP3pIM8R4Me0tifRf9vf2rskWN/WUrU2z9
b7y/qiBb0ljahfpJwDrPgIIQUNYClkFAXwlYG7afxF5mEWArGUsHv3Rh1/zG
7he0mwKil5HlzV1o/Z2ArQZsUWBrWrvnQbv7SMBqCKhq3VEC2FrG0jk5VdhV
YWxcuymgtgVMh4AUCVjpGVASAupYwBQI6C0BKzwDSkFAXQuYYG+ossuBrWL9
XQ6wBobR5+DbhX0gjI1rLQVcb5sffdzvKQHLPAPKQ0BTewX0XaaHBNwPAZWt
YyoB29LYodbLii0Nw+JaWxnYNsbSd89bJWCJZ0AVCOhgAfQVu7sELIaAStYd
1YHtYixdNegm7KIwNq7dFJBsAX2sv5W9z5M9F9hetvmlQLs7hwdUsO44H9gU
ay1dzOok7MIwNq7dFJBuAckQcIsELPAMuAACMiyALkh2lID5EFDeOuYiYDON
vcXeRsXmhWFxrf0dsMOMpQvI7SRgrmfAxRAw2rYTuk5+swTMgYBy1h2XAJtt
7aZL/22FvTeMjWs3BUyygObW38rO9mQvA3aasU2g3a0lYBYElLXuuALYWcbS
HalWws4MY+PaTQHzLKABBLSUgBmeAVdCwCILqAsBN0rAdAgoYx1TF9hltvld
aW+jYveEYXGtrQfsCmst3QVuFh5QGgKugYC1FlAbAppKwDQIKGXdcS2wG4yl
+/dNhJ0axsa1mwIetoDzrb+VneLJNgR2h7E1oN3XS8BkCChp3dEY2N3GVgW2
kbCTwti4dlPAXtv8KkFAQwmY6BlwAwTst1dQDgKuk4AJEFDCOqY5sM8YW8re
RsXGh2FxrSX2OWOLQ2vrS8A4z4AWEHDcAopAwDUSkA0Bxa07WgH7irE0rK6e
sGPD2Lh2t4aAk9GfhaOPFBvjid0E2Onoz9iv6gp2tyfWBrC3Yz/dF3y1sKOB
LWb92C6sZXFY3XDsHGgjsaeA1X4c5cl2APZNYHUTGglsUXvpnQB7AzDd4EeE
YXFtJPYksA2EHe7Jdgb2dWCvFXYYsEXspXcH7FXA9GCSFYbFtTGX7FBP9taw
fZ0P1EOALWwvvQdgJwDT00pmGBbXxp7AvgSsngQHe7K3A/sisI2FHQRsIXvp
vQE7DtgNgg0Mw+LaSOwLwDYR9i5PNgXYY8A2DWcL2Eu/E7CjgOlnwowwLK6N
xD4HbHNhB3iy6cA+C2wLYfsDm99een/AjgCmn7f7hWFxbRwA7OFwtq8nmwHs
IWD1u0w6sEn20gcB9gxg+s3rzjAsro3EPg2sfk9M82QzgT0A7E3CyjVmd1h8
8MKzgNoPVBuh5Loyj2UfDtQ+oPRbd0oCagRQTwGllxt6Y79Fop8Wfu3sBUev
i9wBHRWJftz6tfIEKO1zKr+0JvqCxgCxB4gOQvSK/hH7l7FQ+DgU6qWj2HXo
2Esfl7vCnlY4AQp3Q6FeWuthhROhcBcU6kW/23J0U7BhE7ETCL0cmWzZk6Hw
USjskiM7tHAHFHbN8Wo9Gr0diG45ejqOoD7fBkT3HO9yHDEeiK1A3JpjC/Mg
tgCR/OvtPCBoq9sMhLz5zo4bENlAbAJC7zGkAEE7TQIiFQjaYTcCofdl+ngS
DwGh947SbJukwgehUN5D58wWZNOBbz0Q8h4659yAGAnEOiDkPXQ+DSQi1gAh
76Hz6SQg6EywGgi94TkACDovrQJCb8VmADEMiJVAyAnc+YSZiFgOhHy0cD5S
BwSdqx8AQm9aDwJiKBDLgNC79IOBGJI7ItOTWAqEDkYYYnsGFS6BQtmSnG+j
QTZ9JFsMhGxJzrfjgBgIxCIgdLDGcCDuAuI+IHQwyghPYgEQOjRmJBD0EXo+
EDosZxQQ9OF+HhCyJTlXhQKCvnbMBUK2JOfiVyLiXiB0iNIYIPoBMRsIHWE1
Foi+uSOygaDvc7OA0KFi42zPoOwZUCjbs3MFOchOA2I6ELI9O1e5A6IPEPcA
oYPlJgGRCsQ0IHSs32QgUoCYCoSOOJziSUwGQsc9TgWiNxCTgJDt2bn3ExB3
ADERCNmenftTAdELiAlA6BjQ6Z7EOCB0COsMIOjiVAJipm3WVJgNhToCdzZk
3wbEWCDkPohzjzkRcTcQE4SYA0QyEKOBkL3KuUMfEHRxcxQQOph6HhB02XUk
EDqke74nMRwIHVi+AIhuQAwDQvYqZ3RIQHQFIgsI2auccSsB0QWIoUDoIPtF
QNCV9iFA6ByBxZ5EZjixxPYMKhwMhTrF4X7I7gjEICBknIEz9iwg6DbLQCBk
BIQzPi4RkQGETvdYDkR7IAYAobNVVgBBt6H6A6FzZlYCcTMQ/YDQmTurPIl0
IGTfdkaNBkRbIO4EQvZtZ2RrQNBtwzQgdBbTWiDoNmYfIHQS1jog6LZqAmK9
7RmUnQKFOofsIchuCURvIOSg4Aw7D4gbgbgDCDkoOEPeA6IFEL2A0Pl0mzyJ
nkDodMDNQNB9/x5A6KTELUA0A+I2IOQI40yACIimQCQDIUcYZxpGIqI7EDpB
cxsQTYDoBoTOL90OBA366AqEznLdAQQNTElAPGJ7BmV3hkKdpLsTshsB0QmI
1ULsAoLGA90ChByanIlZAXEdEB2B0AnLjwFBg506AKHzrR/3JNoBobO+9wDR
AIibgZDZI840vYCoD0RbIOQ450wWDAgasNYGCJ0B/6Qn0RoIncD/FBA0Lq8V
ELqMwD7brHNZeACyrwKiJRC6CsJBT6IFEHKAdCbuBgSNo2wOhBwgnenDAUFj
PZsBoStCHAKiDhBNgdAFLQ4DcTkQTYDQZTWOeBKNgZADpDOZPCBovO71QMjE
R2cie0BcCkQjIHSJkaNA0GDkhkDoCinPexLXAqHrtByzPYMKG0ChrhFzHLJp
DHgC4kUgaJx6fSDkMO0sL5GIqAeEHKadpS0Cggbh1wVCl9w5AUQtIK4GQlcM
egWIC4G4Cghdt+hVT+IKIOQw7Sx0EhA0z6EOEHKYdpZbCQiai3E5ELqG00kg
agJxGRC6BNUbQJwHxKVA6EJYb9qeQdm1ofCgFJ6CbJpFk4A4DQTN7/k9EHKy
cJY+CohqQFwMhJwsnAWYEhEXAaFrmr0DRFUgagGhS7K9CwTN0boQCF0Y7j0g
aB7ZBUDIycJZjisRURMIOVk4i4IFBE2SOw8IXSTvAyAqAlEDCF3j7wwQFYA4
FwhdafBD2zOosDoU6iqHH0M2TW+sFk58AkRZIKoCIacsZ4G+gCgDRBUg5JTl
LBMYEKWBqAyELhr5mSdREQhd8/JzIGhuawUgdOXNL4Cg+bflgZBTlrNoZEDQ
HOFyQMgpy1m6MhFRBghdhfRLIIoDURoIXUT1KyCKAVEKCF2+9WvbM6iwJBTK
icpZRDXIpnnnCYhvPIniQMgqcc4ysgFRGIhiQOi6t98CUQiIc4DQlXm/A4LW
DigKhC4H/L0nURgIXYD4ByAKAFEICDlx6qLELpEfiIJAyInTWQY5IJKAKAAE
LLyciEgCQleB/tE26yTrSisUMRKcnGQZaiePbTkdyt+TwhqQ80mxFbBj//yh
U/pLXFB1Jse//y+s3Z33OO/x/8HjSL6fASvE+zo=\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884004771376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884005240126`*^9}]
}, Open  ]],

Cell["Finally, try this:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ShowOctahedron;
RotateOctahedron[c.a.c.c.c.a.b]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884034146376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884034615126`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884035083876`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884035552626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884036021376`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884036490126`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884036802626`*^9}],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
   {RGBColor[1, 0.6, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Print",
 CellChangeTimes->{3.4425884037271376`*^9}]
}, Open  ]],

Cell[TextData[{
 "These animations show us that \n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "c", "\[CenterDot]", "b", "\[CenterDot]", "c", "\[CenterDot]", "c", 
     "\[CenterDot]", "a"}], " ", "=", " ", "e"}], TraditionalForm]]],
 ",  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "c", "\[CenterDot]", "a", "\[CenterDot]", "c", "\[CenterDot]", "c", 
     "\[CenterDot]", "c", "\[CenterDot]", "a", "\[CenterDot]", "b"}], " ", 
    "=", " ", "e"}], TraditionalForm]]],
 ".\n\nEven though you probably wouldn't have discovered these on your own, \
you might have discovered formulas similar to these in your experimentation. \
Let's see what we can do with these two.  \n\nFrom the first one, we have \
that\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"c", "\[CenterDot]", "b"}], "=", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"c", "\[CenterDot]", "c", "\[CenterDot]", "a"}], ")"}], 
      RowBox[{"-", "1"}]], "=", 
     RowBox[{
      RowBox[{
       SuperscriptBox["a", 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       SuperscriptBox["c", 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       SuperscriptBox["c", 
        RowBox[{"-", "1"}]]}], "=", 
      RowBox[{
       RowBox[{"a", "\[CenterDot]", 
        SuperscriptBox["c", "3"], "\[CenterDot]", 
        SuperscriptBox["c", "3"]}], "=", 
       RowBox[{"a", "\[CenterDot]", 
        SuperscriptBox["c", "2"]}]}]}]}]}], TraditionalForm]]],
 ".\n\nFrom the second, we can deduce that\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"c", "\[CenterDot]", "a"}], " ", "=", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
        "c", "\[CenterDot]", "c", "\[CenterDot]", "c", "\[CenterDot]", "a", 
         "\[CenterDot]", "b"}], ")"}], 
       RowBox[{"-", "1"}]]}], "\n", "          ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["b", 
         RowBox[{"-", "1"}]], "\[CenterDot]", 
        SuperscriptBox["a", 
         RowBox[{"-", "1"}]], "\[CenterDot]", 
        SuperscriptBox["c", 
         RowBox[{"-", "1"}]], "\[CenterDot]", 
        SuperscriptBox["c", 
         RowBox[{"-", "1"}]], "\[CenterDot]", 
        SuperscriptBox["c", 
         RowBox[{"-", "1"}]]}], "=", 
       RowBox[{
        SuperscriptBox["b", "2"], "\[CenterDot]", "a", "\[CenterDot]", 
        SuperscriptBox["c", "3"], "\[CenterDot]", 
        SuperscriptBox["c", "3"], "\[CenterDot]", 
        SuperscriptBox["c", "3"]}]}]}], "\n", "          ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["b", "2"], "\[CenterDot]", "a", "\[CenterDot]", "c"}], 
       " ", "=", " ", 
       RowBox[{
        RowBox[{"b", "\[CenterDot]", "a", "\[CenterDot]", 
         SuperscriptBox["b", "2"], "\[CenterDot]", "c"}], " ", "=", " ", 
        RowBox[{
         RowBox[{"a", "\[CenterDot]", 
          SuperscriptBox["b", "2"], "\[CenterDot]", 
          SuperscriptBox["b", "2"], "\[CenterDot]", "c"}], " ", "=", " ", 
         RowBox[{"a", "\[CenterDot]", "b", "\[CenterDot]", 
          RowBox[{"c", "."}]}]}]}]}]}]}]}], TraditionalForm]]],
 " \n\nNotice that we had to use the identity ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{"a", "\[CenterDot]", 
     SuperscriptBox["b", "2"]}]}], TraditionalForm]]],
 " twice in the final computation. We can enter these properties into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " as well:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Define[c.a, a.b.c];
Define[c.b, a.c.c];\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "Notice that we have defined ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "\[CenterDot]", "a"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "\[CenterDot]", "b"}], TraditionalForm]]],
 " in terms of operations that are performed in \"alphabetical order.\" This \
is a very good strategy for defining any group. If ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can sort ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "\[CenterDot]", "a"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "\[CenterDot]", "b"}], TraditionalForm]]],
 " into alphabetical order, then it can play with any combination of \
operations until it produces a form in alphabetical order. This is not the \
only way that we can define a group in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", but it is the one of the most efficient. Thus, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " should have all of the information it needs to determine the group.\n\n\
Before executing the following command, you should make sure that ",
 StyleBox["all",
  FontSlant->"Italic"],
 " of the \"Define\" statements in this section have been executed ",
 StyleBox["first",
  FontSlant->"Italic"],
 ". Otherwise, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will go into an infinite loop. But just in case that happens, here's how \
to get out: Use the mouse and click on the word \"Kernal\" at the top of the \
screen. This should open a menu. Select the option \"Abort Evaluation.\" A \
non-mouse alternative is to hold down the \[AltKey] key while typing a \
period.\n\nThat being said, let's plunge ahead and find out what this group \
looks like."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = Group[{a,b,c}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", "c", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(c\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`c, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`b], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`b, $CellContext`b], $CellContext`c], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.441477430984375*^9, 3.4425884140552626`*^9}]
}, Open  ]],

Cell["\<\
We call this the octahedral group. How large is it?  Well, we can either \
squint and count commas, or we can use the command\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4362789296956253`*^9}],

Cell[CellGroupData[{

Cell["Length[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["24"], "Output",
 CellChangeTimes->{3.4425884167427626`*^9}]
}, Open  ]],

Cell[TextData[{
 "and have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " count for us. Either way, we find we have 24 elements in our group. (We \
could also have found this number using geometry on the octahedron. See \
problem 2.25.) \n\nIs the set of rotations a group? The hard property to \
check is associativity, which ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can check with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.42755891875*^9, {3.427558961671875*^9, 3.42755896228125*^9}, {
   3.42844585790625*^9, 3.428445858375*^9}, {3.4362789774300003`*^9, 
   3.4362789779143753`*^9}}],

Cell[CellGroupData[{

Cell["CheckGroup[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["\<\"This set of elements is a group.\"\>"], "Print",
 CellChangeTimes->{3.4425884237740126`*^9}]
}, Open  ]],

Cell[TextData[{
 "This may take quite awhile, since there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["24", "3"], "=", " ", "13824"}], TraditionalForm]]],
 " combinations to check! If you get impatient, you can abort this command as \
described above.\n\nWhat happens if we try to make a table of this group? Of \
course, this group is too large to write out all of the elements in the \
table. But watch what happens:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42755911015625*^9, 3.42755911203125*^9}}],

Cell[CellGroupData[{

Cell["MultTable[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.2, 0.6, 0.4}, {0.2, 0.6, 0.4}, {0.8, 0.6, 0.2}, {0.8, 0.2, 
      0.5}, {0.5, 0.6, 0.8}, {0, 0.6, 1}, {0, 0, 1}, {0.6, 0.4, 0.4}, {
      0.7, 1, 0}, {1, 0.6, 1}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {
      1, 0.8, 0.4}, {0.4, 0.2, 1}, {1, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.7, 0.8, 
      0.7}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0, 1, 0}, {0.2, 0.8, 0.7}, {
      1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {1, 1, 0}}, {{1, 0.8, 0.4}, {
      1, 0.8, 0.4}, {0.8, 0.6, 0.5}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {
      0.4, 0.2, 1}, {0.2, 0.8, 0.7}, {1, 0.8, 0.7}, {1, 0.4, 0.4}, {0.6, 0.6, 
      0.2}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 1, 0}, {0.2, 0.6, 0.4}, {
      0, 0.6, 1}, {0.6, 0.4, 0.4}, {0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}, {1, 0, 
      0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0, 0, 1}, {0.7, 1, 0}, {1, 0.6, 1}, {1,
       0, 1}}, {{0.4, 0.2, 1}, {0.4, 0.2, 1}, {1, 0.8, 0.7}, {0.6, 0.4, 
      0.8}, {0, 1, 0}, {1, 0.8, 0.4}, {0.2, 0.6, 0.4}, {0.8, 0.6, 0.5}, {0.8, 
      0.6, 0.2}, {0.8, 0.2, 0.5}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.5, 0.6, 
      0.8}, {0.2, 0.8, 0.7}, {0, 0, 1}, {0.7, 1, 0}, {1, 0.4, 0.4}, {0.6, 0.6,
       0.2}, {1, 0.6, 1}, {1, 1, 0}, {1, 0, 1}, {0, 0.6, 1}, {0.6, 0.4, 
      0.4}, {1, 0, 0}, {0, 1, 1}}, {{1, 0.8, 0.7}, {1, 0.8, 0.7}, {
      0.4, 0.2, 1}, {1, 0.8, 0.4}, {0.2, 0.6, 0.4}, {0.6, 0.4, 0.8}, {0, 1, 
      0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {
      0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {
      1, 0.6, 1}, {0.2, 0.8, 0.7}, {0, 0, 1}, {0.7, 1, 0}, {0, 0.6, 1}, {0.6, 
      0.4, 0.4}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{0, 0.6, 1}, {
      0, 0.6, 1}, {0.6, 0.4, 0.4}, {1, 0, 0}, {0, 1, 1}, {0.2, 0.6, 0.4}, {
      1, 0.8, 0.4}, {0.8, 0.6, 0.2}, {0.8, 0.6, 0.5}, {0.7, 0.8, 0.7}, {0.8, 
      0.2, 0.5}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {0, 0, 1}, {0.2, 0.8, 0.7}, {
      1, 0.4, 0.4}, {0.7, 1, 0}, {1, 0.6, 1}, {0.6, 0.6, 0.2}, {1, 0, 1}, {1, 
      1, 0}, {0.4, 0.2, 1}, {1, 0.8, 0.7}, {0.6, 0.4, 0.8}, {0, 1, 0}}, {{0.6,
       0.4, 0.4}, {0.6, 0.4, 0.4}, {0, 0.6, 1}, {0.2, 0.6, 0.4}, {
      1, 0.8, 0.4}, {1, 0, 0}, {0, 1, 1}, {0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {
      1, 0.6, 0}, {0.8, 0.6, 0.2}, {0.8, 0.6, 0.5}, {0.7, 0.8, 0.7}, {
      0.7, 1, 0}, {1, 0.6, 1}, {0.6, 0.6, 0.2}, {0, 0, 1}, {0.2, 0.8, 0.7}, {
      1, 0.4, 0.4}, {0.4, 0.2, 1}, {1, 0.8, 0.7}, {1, 0, 1}, {1, 1, 0}, {0, 1,
       0}, {0.6, 0.4, 0.8}}, {{0, 0, 1}, {0, 0, 1}, {0.7, 1, 0}, {
      1, 0.6, 1}, {1, 0, 1}, {0.2, 0.8, 0.7}, {0.4, 0.2, 1}, {1, 0.4, 0.4}, {
      1, 0.8, 0.7}, {0.6, 0.4, 0.8}, {0.6, 0.6, 0.2}, {1, 1, 0}, {0, 1, 0}, {
      0, 0.6, 1}, {0.2, 0.6, 0.4}, {0.8, 0.6, 0.2}, {0.6, 0.4, 0.4}, {1, 0, 
      0}, {0.8, 0.2, 0.5}, {0, 1, 1}, {0.5, 0.6, 0.8}, {1, 0.8, 0.4}, {0.8, 
      0.6, 0.5}, {0.7, 0.8, 0.7}, {1, 0.6, 0}}, {{0.7, 1, 0}, {0.7, 1, 0}, {0,
       0, 1}, {0.2, 0.8, 0.7}, {0.4, 0.2, 1}, {1, 0.6, 1}, {1, 0, 1}, {0.6, 
      0.6, 0.2}, {1, 1, 0}, {0, 1, 0}, {1, 0.4, 0.4}, {1, 0.8, 0.7}, {0.6, 
      0.4, 0.8}, {0.6, 0.4, 0.4}, {1, 0, 0}, {0.8, 0.2, 0.5}, {0, 0.6, 1}, {
      0.2, 0.6, 0.4}, {0.8, 0.6, 0.2}, {1, 0.8, 0.4}, {0.8, 0.6, 0.5}, {0, 1, 
      1}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 0.6, 1}, {
      1, 0.6, 1}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {1, 0.8, 0.7}, {
      0.7, 1, 0}, {0.6, 0.4, 0.4}, {0, 0, 1}, {0, 0.6, 1}, {0.2, 0.6, 0.4}, {
      0.2, 0.8, 0.7}, {0.4, 0.2, 1}, {1, 0.8, 0.4}, {1, 0, 1}, {0, 1, 1}, {
      0.5, 0.6, 0.8}, {1, 1, 0}, {0, 1, 0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {
      0.7, 0.8, 0.7}, {1, 0, 0}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.8, 0.6, 
      0.5}}, {{0.2, 0.8, 0.7}, {0.2, 0.8, 0.7}, {1, 0.4, 0.4}, {0.6, 0.6, 
      0.2}, {1, 1, 0}, {0, 0, 1}, {0, 0.6, 1}, {0.7, 1, 0}, {0.6, 0.4, 0.4}, {
      1, 0, 0}, {1, 0.6, 1}, {1, 0, 1}, {0, 1, 1}, {0.4, 0.2, 1}, {
      1, 0.8, 0.4}, {0.8, 0.6, 0.5}, {1, 0.8, 0.7}, {0.6, 0.4, 0.8}, {0.7, 
      0.8, 0.7}, {0, 1, 0}, {1, 0.6, 0}, {0.2, 0.6, 0.4}, {0.8, 0.6, 0.2}, {
      0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}}, {{1, 0.4, 0.4}, {1, 0.4, 0.4}, {0.2, 
      0.8, 0.7}, {0, 0, 1}, {0, 0.6, 1}, {0.6, 0.6, 0.2}, {1, 1, 0}, {
      1, 0.6, 1}, {1, 0, 1}, {0, 1, 1}, {0.7, 1, 0}, {0.6, 0.4, 0.4}, {1, 0, 
      0}, {1, 0.8, 0.7}, {0.6, 0.4, 0.8}, {0.7, 0.8, 0.7}, {0.4, 0.2, 1}, {
      1, 0.8, 0.4}, {0.8, 0.6, 0.5}, {0.2, 0.6, 0.4}, {0.8, 0.6, 0.2}, {0, 1, 
      0}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}}, {{0.6, 0.6, 0.2}, {
      0.6, 0.6, 0.2}, {1, 0.6, 1}, {0.7, 1, 0}, {0.6, 0.4, 0.4}, {
      1, 0.4, 0.4}, {1, 0.8, 0.7}, {0.2, 0.8, 0.7}, {0.4, 0.2, 1}, {
      1, 0.8, 0.4}, {0, 0, 1}, {0, 0.6, 1}, {0.2, 0.6, 0.4}, {1, 1, 0}, {0, 1,
       0}, {1, 0.6, 0}, {1, 0, 1}, {0, 1, 1}, {0.5, 0.6, 0.8}, {1, 0, 0}, {
      0.8, 0.2, 0.5}, {0.6, 0.4, 0.8}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.8,
       0.6, 0.2}}, {{0.8, 0.6, 0.2}, {0.8, 0.6, 0.2}, {0.2, 0.6, 0.4}, {
      0, 0.6, 1}, {0, 0, 1}, {0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {1, 0, 0}, {0, 
      1, 1}, {1, 0, 1}, {0.6, 0.4, 0.4}, {0.7, 1, 0}, {1, 0.6, 1}, {0.8, 0.6, 
      0.5}, {0.7, 0.8, 0.7}, {0.6, 0.4, 0.8}, {1, 0.8, 0.4}, {0.4, 0.2, 1}, {
      1, 0.8, 0.7}, {0.2, 0.8, 0.7}, {1, 0.4, 0.4}, {1, 0.6, 0}, {0, 1, 0}, {
      1, 1, 0}, {0.6, 0.6, 0.2}}, {{0.8, 0.2, 0.5}, {0.8, 0.2, 0.5}, {1, 0, 
      0}, {0.6, 0.4, 0.4}, {0.7, 1, 0}, {0.8, 0.6, 0.2}, {0.8, 0.6, 0.5}, {
      0.2, 0.6, 0.4}, {1, 0.8, 0.4}, {0.4, 0.2, 1}, {0, 0.6, 1}, {0, 0, 1}, {
      0.2, 0.8, 0.7}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}, {1, 
      0, 1}, {1, 1, 0}, {1, 0.6, 1}, {0.6, 0.6, 0.2}, {0.7, 0.8, 0.7}, {0.6, 
      0.4, 0.8}, {1, 0.8, 0.7}, {1, 0.4, 0.4}}, {{0.5, 0.6, 0.8}, {0.5, 0.6, 
      0.8}, {0, 1, 1}, {1, 0, 1}, {1, 0.6, 1}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {
      0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.8, 0.7}, {1, 1, 0}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.2, 0.6, 0.4}, {1, 0, 
      0}, {0.6, 0.4, 0.4}, {0, 0.6, 1}, {0.7, 1, 0}, {0, 0, 1}, {0.8, 0.6, 
      0.5}, {1, 0.8, 0.4}, {0.4, 0.2, 1}, {0.2, 0.8, 0.7}}, {{0.8, 0.6, 
      0.5}, {0.8, 0.6, 0.5}, {1, 0.8, 0.4}, {0.4, 0.2, 1}, {0.2, 0.8, 0.7}, {
      0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 1, 0}, {
      1, 0.8, 0.7}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {0.8, 0.6, 0.2}, {0.8, 
      0.2, 0.5}, {1, 0, 0}, {0.2, 0.6, 0.4}, {0, 0.6, 1}, {0.6, 0.4, 0.4}, {0,
       0, 1}, {0.7, 1, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {1, 0, 1}, {
      1, 0.6, 1}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {0.6, 0.4, 0.8}, {
      1, 0.8, 0.7}, {1, 0.4, 0.4}, {0.8, 0.6, 0.5}, {0.8, 0.6, 0.2}, {
      1, 0.8, 0.4}, {0.2, 0.6, 0.4}, {0, 0.6, 1}, {0.4, 0.2, 1}, {0.2, 0.8, 
      0.7}, {0, 0, 1}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0, 1, 0}, {1,
       1, 0}, {1, 0, 1}, {0.6, 0.6, 0.2}, {1, 0.6, 1}, {0.8, 0.2, 0.5}, {1, 0,
       0}, {0.6, 0.4, 0.4}, {0.7, 1, 0}}, {{1, 0.6, 0}, {1, 0.6, 0}, {0, 1, 
      0}, {1, 1, 0}, {0.6, 0.6, 0.2}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0, 1,
       1}, {1, 0, 0}, {0.6, 0.4, 0.4}, {1, 0, 1}, {1, 0.6, 1}, {0.7, 1, 0}, {
      0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {1, 0.8, 0.4}, {0.6, 0.4, 0.8}, {
      1, 0.8, 0.7}, {0.4, 0.2, 1}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {0.8, 0.6, 
      0.2}, {0.2, 0.6, 0.4}, {0, 0.6, 1}, {0, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 
      0.4, 0.8}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.8, 0.6, 0.2}, {
      1, 0.8, 0.7}, {1, 0.4, 0.4}, {0.4, 0.2, 1}, {0.2, 0.8, 0.7}, {0, 0, 
      1}, {1, 0.8, 0.4}, {0.2, 0.6, 0.4}, {0, 0.6, 1}, {0, 1, 0}, {1, 1, 0}, {
      1, 0, 1}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0.8, 0.2, 0.5}, {1, 
      0, 0}, {0.6, 0.6, 0.2}, {1, 0.6, 1}, {0.7, 1, 0}, {0.6, 0.4, 0.4}}, {{0,
       1, 0}, {0, 1, 0}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {1, 1,
       0}, {0.6, 0.6, 0.2}, {1, 0, 1}, {1, 0.6, 1}, {0.7, 1, 0}, {0, 1, 1}, {
      1, 0, 0}, {0.6, 0.4, 0.4}, {0.6, 0.4, 0.8}, {1, 0.8, 0.7}, {
      0.4, 0.2, 1}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {1, 0.8, 0.4}, {0.8, 
      0.6, 0.2}, {0.2, 0.6, 0.4}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {0, 0, 1}, {
      0, 0.6, 1}}, {{1, 0, 0}, {1, 0, 0}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {
      0.8, 0.6, 0.5}, {0.6, 0.4, 0.4}, {0.7, 1, 0}, {0, 0.6, 1}, {0, 0, 1}, {
      0.2, 0.8, 0.7}, {0.2, 0.6, 0.4}, {1, 0.8, 0.4}, {0.4, 0.2, 1}, {0, 1, 
      1}, {1, 0, 1}, {1, 1, 0}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {0, 1, 0}, {0.7,
       0.8, 0.7}, {0.6, 0.4, 0.8}, {1, 0.6, 1}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {1, 0.8, 0.7}}, {{0, 1, 1}, {0, 1, 1}, {0.5, 0.6, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 0, 1}, {1, 0.6, 1}, {1, 1, 0}, {0.6, 
      0.6, 0.2}, {1, 0.4, 0.4}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.8, 0.7}, {1,
       0, 0}, {0.6, 0.4, 0.4}, {0, 0.6, 1}, {0.8, 0.2, 0.5}, {0.8, 0.6, 
      0.2}, {0.2, 0.6, 0.4}, {0.8, 0.6, 0.5}, {1, 0.8, 0.4}, {0.7, 1, 0}, {0, 
      0, 1}, {0.2, 0.8, 0.7}, {0.4, 0.2, 1}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0, 0}, {0.5, 0.6, 0.8}, {
      0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 
      0.6, 0.5}, {1, 0.6, 1}, {0.7, 1, 0}, {0, 0, 1}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.8, 0.7}, {0.4, 0.2, 1}, {0.6, 0.4, 
      0.4}, {0, 0.6, 1}, {0.2, 0.6, 0.4}, {1, 0.8, 0.4}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 
      0.7}, {1, 0.6, 1}, {0.7, 1, 0}, {0, 0, 1}, {0.6, 0.4, 0.4}, {
      0, 0.6, 1}, {1, 0.8, 0.7}, {0.4, 0.2, 1}, {1, 0.8, 0.4}, {0.2, 0.6, 
      0.4}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}, {0.7, 1, 0}, {0, 0, 1}, {
      0.6, 0.4, 0.4}, {0, 0.6, 1}, {1, 0.8, 0.7}, {0.4, 0.2, 1}, {
      1, 0.8, 0.4}, {0.2, 0.6, 0.4}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 23.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 22.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 21.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 20.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 19.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 18.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 17.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 16.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"c\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 15.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 14.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 13.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]c\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 12.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 11.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]c\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 10.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"c\[CenterDot]c\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 9.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 8.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]c\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 7.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]c\[CenterDot]c\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 6.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]c\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 5.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]c\[CenterDot]c\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 4.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]c\[CenterDot]c\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 3.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]c\[CenterDot]c\[CenterDot]c\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 2.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]c\[CenterDot]c\[CenterDot]c\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 1.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]c\[CenterDot]c\
\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 0.5}, {1, 0}]}, 
   LineBox[{{1, 0}, {1, 24}, {25, 24}}]},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4425884289302626`*^9}]
}, Open  ]],

Cell["\<\
This table may have to be enlarged before it can be read. Each element is \
color coded in one of 24 colors, and this code is given by the list on the \
left. Can you use this table to find the elements which are their own \
inverses?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Subgroups", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Whenever we have a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", we say that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a ",
 StyleBox["subset",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", denoted\n                           ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[SubsetEqual]", " ", "G"}], TraditionalForm]]],
 ","
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "if ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " consists only of the elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". The empty set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", " ", "}"}], TraditionalForm]]],
 " is always considered to be a subset, but we will restrict our attention to \
non-empty subsets. We are particularly interested if the set ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " forms a group since we then would have a \"group within a group.\"\n\n\
DEFINITION  2.3\nWe say that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a ",
 StyleBox["subgroup",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " if ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a non-empty subset of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a group with respect to the multiplication ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "\[CenterDot]", ")"}], TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nFor ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to be a group, it must satisfy the 4 properties of groups. But the \
associative property of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is guaranteed because ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is associative. Thus, we need these three properties to be true:\n\n1:  ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is closed under multiplication.  That is,\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", ",", 
    RowBox[{"v", " ", "\[Element]", " ", 
     RowBox[{"H", "  ", "\[DoubleRightArrow]", " ", 
      RowBox[{"u", "\[CenterDot]", "v"}]}], " ", "\[Element]", " ", "H"}]}], 
   TraditionalForm]]],
 ".\n\n2:  The identity element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ".\n\n3:  Every element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " has its inverse in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ".  That is,\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "\[Element]", " ", 
    RowBox[{"H", "  ", "\[DoubleRightArrow]", "  ", 
     SuperscriptBox["u", 
      RowBox[{"-", "1"}]]}], " ", "\[Element]", " ", "H"}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427559357515625*^9, 3.42755937821875*^9}, {
  3.427559434*^9, 3.427559456390625*^9}, {3.4362827501800003`*^9, 
  3.4362827561800003`*^9}, {3.43628618246875*^9, 3.43628622946875*^9}, {
  3.4425884416802626`*^9, 3.4425884559927626`*^9}}],

Cell[TextData[{
 "\nThese three properties can be combined into one simple test.\n\n\
PROPOSITION 2.2\nLet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[SubsetEqual]", " ", "G"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[NotEqual]", " ", 
    RowBox[{"{", " ", "}"}]}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " if, and only if, we have\n               \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{"x", "\[CenterDot]", 
      SuperscriptBox["y", 
       RowBox[{"-", "1"}]]}], " ", "\[Element]", " ", "H"}]}], 
   TraditionalForm]]],
 "    for all     ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "\[Element]", "H"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427559466828125*^9, 3.427559485453125*^9}, {
  3.436286250359375*^9, 3.436286251875*^9}, {3.4362863735*^9, 
  3.43628641725*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First of all, we need to see that if ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subgroup, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " whenever ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ".  By property (3), ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and so by property (1), ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ".\n\nConversely, let us suppose that  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[SubsetEqual]", " ", "G"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[NotEqual]", " ", 
    RowBox[{"{", " ", "}"}]}], TraditionalForm]]],
 ", and whenever ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", ",", 
    RowBox[{"y", " ", "\[Element]", " ", "H"}]}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", 
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]]}], " ", "\[Element]", " ", 
    RowBox[{"H", "."}]}], TraditionalForm]]],
 " We need to see that properties (1) through (3) are satisfied.  \n\nSince \
",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is not the empty set, there is an element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", 
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]]}], " ", "=", " ", "e"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". Thus, property (2) holds.  \n\nNext, we have that if ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"e", "\[CenterDot]", 
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]]}], "=", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and so property (3) holds.\n\n Finally, if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", 
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["y", 
        RowBox[{"-", "1"}]], ")"}], 
      RowBox[{"-", "1"}]]}], "=", " ", 
    RowBox[{"x", "\[CenterDot]", "y"}]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ".  Thus, property (1) also holds."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427559653328125*^9, 3.42755969890625*^9}, {
  3.427559938234375*^9, 3.427559989125*^9}, {3.43628643434375*^9, 
  3.436286481609375*^9}, {3.436286564625*^9, 3.436286636453125*^9}, {
  3.4362867308125*^9, 3.436286922*^9}}]
}, Open  ]],

Cell[TextData[{
 "Let us look at an example. The following commands reload the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ", the permutation group on three objects, into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2,e]
Define[b^3,e]
Define[b.a,a.b.b]
Define[1/a,a]
Define[1/b,b^2]
Group[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4362869465*^9, 3.436286948890625*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425884730865126`*^9}]
}, Open  ]],

Cell[TextData[{
 "Even though this is a fairly small group (6 elements) we can still find \
smaller groups within this one.  For example, consider the subset\n\n         \
     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{"e", ",", "  ", "b", ",", "  ", 
      SuperscriptBox["b", "2"]}], "}"}]}], TraditionalForm]]],
 ".\n\nIt is easy to see that if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". (There are nine multiplications to check.)  Therefore, this is a \
subgroup.\n\nHow can we verify this is a subgroup using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "? Let us look at the multiplication table for ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.436286666515625*^9, 3.436286701796875*^9, {3.436286968046875*^9, 
   3.436286979984375*^9}}],

Cell[CellGroupData[{

Cell["MultTable[{e, b, b.b}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0, 1, 1}, {0, 1, 1}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0,
     1}, {0, 1, 1}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 3}, {4, 3}}]}]], "Output",
 CellChangeTimes->{3.4425884770708876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Since there are no black squares in this table, it is closed under \
multiplication. Also, every row and column contains one ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ", so inverses exist for every element. Thus, ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ".\n\nTry the following subset instead:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[{e, a, b}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0, 1, 1}, {0, 1, 1}, {0, 0, 0}, {0, 0, 0}}, {{1, 0, 1}, {1, 0,
     1}, {1, 1, 0}, {0, 0, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], 
     InsetBox["\<\" \"\>", {3.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], 
     InsetBox["\<\" \"\>", {2.5, 0.5}], InsetBox["\<\" \"\>", {3.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 3}, {4, 3}}]}]], "Output",
 CellChangeTimes->{3.4425884795083876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice the black squares in the table. This indicates that the subset is \
not closed under multiplication, and hence is not a subgroup.\n\nEXPERIMENT:\n\
Try changing the subset in the above command to see if you can find other \
subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ". How many are there? Don't forget the group itself can be considered to be \
a subgroup. What about the set containing just the identity?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Here is another example. Recall that ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is the group of integers using addition as the operation. If we let ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " be any integer then we can let\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", " ", "\[DoubleStruckCapitalZ]"}], "  ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"k", "\[CenterDot]", "x"}], "  ", "|", "  ", 
      RowBox[{"x", " ", "\[Element]", "\[DoubleStruckCapitalZ]"}]}], "}"}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436287275046875*^9, 3.436287278890625*^9}}],

Cell[TextData[{
 "The vertical line is read \"for which.\" Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[DoubleStruckCapitalZ]"}], TraditionalForm]]],
 " is the set of \"all numbers which can be expressed as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[CenterDot]", "x"}], TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is an integer.\" In other words, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[DoubleStruckCapitalZ]"}], TraditionalForm]]],
 " is the set of all multiples of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ". It is clear that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[DoubleStruckCapitalZ]"}], TraditionalForm]]],
 " is a group under addition, since the difference of two multiples of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " gives another multiple of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ". \n\nOne of the properties of subgroups is that if we take the \
intersection of 2 subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", we will have another subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Recall that the intersection of two sets, denoted\n\n                     \
   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Intersection]", " ", "K"}], TraditionalForm]]],
 "\n                        \nrepresents all elements that are in both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". For example, if we are given two sets"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436287083375*^9, 3.43628709278125*^9}, {
  3.441478396109375*^9, 3.44147839828125*^9}}],

Cell[CellGroupData[{

Cell["\<\
H = {e, b, b^2}
K = {e, a}\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425884848990126`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a"}], "}"}]], "Output",
 CellChangeTimes->{3.4425884848990126`*^9}]
}, Open  ]],

Cell[TextData[{
 "then the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Intersection[H, K]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 RowBox[{"{", "e", "}"}]], "Output",
 CellChangeTimes->{3.4425884870240126`*^9}]
}, Open  ]],

Cell[TextData[{
 "finds the set of all elements in common with ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Moreover, we can consider taking the intersection of a collection of many \
sets. If we let"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["L = {{e, a, b},{e, a.b, b},{e, a, b, b^2}}", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.427560357484375*^9, 3.427560357890625*^9}},
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"e", ",", "a", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"e", ",", 
     InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
      Editable->False], ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"e", ",", "a", ",", "b", ",", 
     InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
      Editable->False]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4425884892271376`*^9}]
}, Open  ]],

Cell[TextData[{
 "then ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " represents a \"set of sets.\" We can take the intersection of all of the \
sets in this collection with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42756037015625*^9, 3.427560374140625*^9}}],

Cell[CellGroupData[{

Cell["Intersection[L]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.42756037978125*^9, 3.427560380125*^9}},
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"b", ",", "e"}], "}"}]], "Output",
 CellChangeTimes->{3.4425884914146376`*^9}]
}, Open  ]],

Cell[TextData[{
 "The mathematical notation for this intersection is \n\n                     \
 ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     StyleBox[
      UnderscriptBox[
       RowBox[{"  ", 
        SubscriptBox[
         StyleBox["\[Intersection]",
          FontSize->24], " "]}], 
       RowBox[{" ", 
        RowBox[{"H", " ", "\[Element]", " ", "L"}]}]],
      FontSize->18], "H"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42756038321875*^9, 3.4275603835*^9}, {
  3.43628726784375*^9, 3.436287269375*^9}}],

Cell[TextData[{
 "PROPOSITION 2.3\nGiven a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " and a non-empty collection of subgroups, denoted by ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 ",  then the intersection of all of the subgroups in the collection "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427560407359375*^9, 3.427560437296875*^9}, {
  3.4362872150625*^9, 3.4362872483125*^9}}],

Cell[TextData[{
 "\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["H", 
     RowBox[{"*", " "}]], "=", " ", 
    RowBox[{
     StyleBox[
      UnderscriptBox[
       RowBox[{"  ", 
        SubscriptBox[
         StyleBox["\[Intersection]",
          FontSize->24], " "]}], 
       RowBox[{" ", 
        RowBox[{"H", " ", "\[Element]", " ", "L"}]}]],
      FontSize->18], "H"}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42756038803125*^9, 3.427560388546875*^9}, {
  3.43628726359375*^9, 3.43628726446875*^9}}],

Cell[TextData[{
 "is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "                         \nFirst of all, note that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["H", "*"], TraditionalForm]]],
 " is not the empty set, since the identity element is in each ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " in the collection.  We now can apply proposition 2.2. Let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " be two elements in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["H", "*"], TraditionalForm]]],
 ".  Then, for every  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Epsilon]", " ", "L"}], TraditionalForm]]],
 " we have  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", ",", 
    RowBox[{"y", " ", "\[Epsilon]", " ", "H"}]}], TraditionalForm]]],
 ". Since each ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", we have\n\n                                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", 
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]]}], " ", "\[Element]", "H"}], TraditionalForm]]],
 ".\n                                          \nTherefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["H", "*"], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["H", "*"], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427562998796875*^9, 3.42756301028125*^9}, {
  3.436287296234375*^9, 3.436287300265625*^9}, {3.43628733503125*^9, 
  3.4362873578125*^9}}]
}, Open  ]],

Cell[TextData[{
 "This proposition provides us with a general method of producing the \
subgroups of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " be any subset of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". We can consider the collection of all subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contain the set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", denoted by ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427563030265625*^9, 3.427563037890625*^9}, {
  3.427563090984375*^9, 3.427563093734375*^9}}],

Cell[TextData[{
 "DEFINITION 2.4:\nGiven a subset ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", we define the ",
 StyleBox["subgroup generated by S ",
  FontSlant->"Italic"],
 " to be\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"[", "S", "]"}], " "], "=", " ", 
    RowBox[{
     StyleBox[
      UnderscriptBox[
       RowBox[{"  ", 
        SubscriptBox[
         StyleBox["\[Intersection]",
          FontSize->24], " "]}], 
       RowBox[{" ", 
        RowBox[{"H", " ", "\[Element]", " ", "L"}]}]],
      FontSize->18], "H"}]}], TraditionalForm]]],
 "\nwhere ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " denotes the collection of subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contains the set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ".  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427563030265625*^9, 3.427563037890625*^9}, {
  3.427563090984375*^9, 3.427563093734375*^9}, {3.43628768321875*^9, 
  3.43628786325*^9}, {3.436287895328125*^9, 3.436287895828125*^9}}],

Cell[TextData[{
 "By proposition 2.3, this a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". (The collection ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is non-empty since it contains ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".)  By the way that the collection was defined, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " contains ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ".  \n\nActually, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " is the ",
 StyleBox["smallest",
  FontSlant->"Italic"],
 " subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contains ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". For if ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " containing ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Element]", "L"}], TraditionalForm]]],
 ", so that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"[", "S", "]"}], " ", "\[SubsetEqual]", " ", "H"}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42844592440625*^9, 3.428445961984375*^9}, {
  3.43628787728125*^9, 3.43628796090625*^9}}],

Cell[TextData[{
 "We can define ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " in another way. It is clear that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " contains all of the products of the form\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"(*", ")"}], TraditionalForm]]],
 "              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "1"], "\[CenterDot]", 
    SubscriptBox["x", "2"], "\[CenterDot]", 
    SubscriptBox["x", "3"], "\[CenterDot]", " ", "\[CenterEllipsis]", " ", 
    "\[CenterDot]", " ", 
    SubscriptBox["x", "n"]}], TraditionalForm]]],
 ",\n\nwhere either ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "k"], " ", "\[Epsilon]", " ", "S"}], 
   TraditionalForm]]],
 "  or  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubsuperscriptBox["x", "k", 
      RowBox[{"-", "1"}]], " ", "\[Epsilon]", " ", "S"}], ",", "   ", 
    RowBox[{"(", 
     RowBox[{
     "1", " ", "\[LessEqual]", " ", "k", " ", "\[LessEqual]", " ", "n"}], 
     ")"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42844606815625*^9, 3.42844608546875*^9}, {
  3.43628793490625*^9, 3.436287937328125*^9}, {3.43628799575*^9, 
  3.43628799884375*^9}}],

Cell[TextData[{
 "Notice that the set of all these products forms a subgroup ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and this subgroup contains ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"[", "S", "]"}], " ", "\[SubsetEqual]", " ", "H"}], 
   TraditionalForm]]],
 ". But any element of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(*", ")"}], TraditionalForm]]],
 " must be in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 ". Thus we also have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[SubsetEqual]", " ", 
    RowBox[{"[", "S", "]"}]}], TraditionalForm]]],
 ".  Therefore ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    RowBox[{"[", "S", "]"}]}], TraditionalForm]]],
 ". In other words, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " is the subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " consisting of all products of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(*", ")"}], TraditionalForm]]],
 ".\n\nIn fact, the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command ",
 StyleBox["Group",
  FontWeight->"Bold"],
 " finds ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " for any set ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a set of generators of the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"[", "S", "]"}], " ", "=", " ", "G"}], TraditionalForm]]],
 ", so we produced the entire group with the command ",
 StyleBox["Group[S]. ",
  FontWeight->"Bold"],
 "Yet for other sets, we can use the ",
 StyleBox["Group",
  FontWeight->"Bold"],
 " command to find the subgroups of a given group.\n\nFor example, consider \
the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427564553359375*^9, 3.427564554390625*^9}, {
  3.4284460915*^9, 3.428446128828125*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2,e];
Define[b^3,e];
Define[b.a,a.b.b];
Define[1/a,a];
Define[1/b,b^2];
Group[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425884996333876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Now we can try out the group command to find ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " for different sets ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". For example, what if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", " ", "=", " ", 
    RowBox[{
     RowBox[{"{", "b", "}"}], "?"}]}], TraditionalForm]]],
 "  Try it out for yourself:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428446142859375*^9, 3.428446147171875*^9}}],

Cell[CellGroupData[{

Cell["Group[{b}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425885019458876`*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives us the subgroup of order 3 that we saw before. What if ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"b", ",", " ", 
     RowBox[{"a", "\[CenterDot]", "b"}]}], "}"}], TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42844615115625*^9, 3.42844615115625*^9}}],

Cell[CellGroupData[{

Cell["Group[{b, a.b}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425885037583876`*^9}]
}, Open  ]],

Cell[TextData[{
 "This apparently gives us the whole group. So ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"b", ",", 
     RowBox[{"a", "\[CenterDot]", "b"}]}], "}"}], TraditionalForm]]],
 " is also a set of generators for the group.\n\nBy using the ",
 StyleBox["Group",
  FontWeight->"Bold"],
 " command, find all of the subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". How many are there? \n\nLet's look at a larger group. We found the ",
 "octahedral",
 " group had order 24. Recall that the group was entered with the following \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " commands.  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4362789109612503`*^9}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e]
Define[a^2,e]
Define[b^3,e]
Define[c^4,e]
Define[1/a,a]
Define[1/b,b^2]
Define[1/c,c^3]
Define[b.a,a.b.b]
Define[c.a,a.b.c]
Define[c.b,a.c.c]
Group[{a,b,c}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["e"], "Output",
 CellChangeTimes->{3.4425885065396376`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", "c", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(c\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`c, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`b], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`b, $CellContext`b], $CellContext`c], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425885065552626`*^9}]
}, Open  ]],

Cell[TextData[{
 "\nFor any set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " we can find the subgroup ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " by typing\n\nG = Group[S]\n\nFor example, suppose that the ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is the single element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "c", "}"}], TraditionalForm]]],
 ".  We would type"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4284461595625*^9, 3.428446169671875*^9}}],

Cell[CellGroupData[{

Cell["Group[{c}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "c", ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(c\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`c, $CellContext`c], $CellContext`c],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425885097583876`*^9}]
}, Open  ]],

Cell[TextData[{
 "and find we have a subgroup of order 4.  \n\nWe can try other combinations \
of elements. For example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{"b", ",", " ", "c"}], "}"}]}], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428446172734375*^9, 3.428446172734375*^9}}],

Cell[CellGroupData[{

Cell["Group[{b, c}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", "c", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(c\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`c, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`b], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`b, $CellContext`b], $CellContext`c], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425885119146376`*^9}]
}, Open  ]],

Cell["This appears as big as the whole group. Let's double check:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[%]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["24"], "Output",
 CellChangeTimes->{3.4425885143521376`*^9}]
}, Open  ]],

Cell[TextData[{
 "Since the size of this subgroup is 24 it must be the entire group. This \
means that the group can be generated with just two of the elements.\n\nWhy \
then did we use three elements to define the group originally? It turns out \
to be ",
 StyleBox["more efficient",
  FontSlant->"Italic"],
 " to define the group in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using three elements rather than two! Besides, it is much easier to put \
the octahedron back into its original position when one can flip the whole \
thing over. \n\nHere is another subgroup:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Group[{a, b}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425885173208876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Wait!  This looks familiar. Where have we seen this before? \n \nLook back \
at the definition of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ". We had ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " \nas the generators, and we even had ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["a", "2"],
     TraditionalForm], " ", "=", " ", "e"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["b", "3"], "=", " ", "e"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{"a", "\[CenterDot]", "b", "\[CenterDot]", "b"}]}], 
   TraditionalForm]]],
 ". But these rules are all true of this subgroup of the octahedral group. \
Therefore, these are the same groups. We have found a copy of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " inside of the ",
 "octahedral",
 " group.    "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4362789147581253`*^9, 3.4362789169612503`*^9}}],

Cell[TextData[{
 "Recall that we defined a group to be ",
 StyleBox["cyclic",
  FontSlant->"Italic"],
 " if there was one element in the group that generates the entire group. We \
also found that the element which generated the group was not unique. We \
called all such elements ",
 StyleBox["generators",
  FontSlant->"Italic"],
 " of the group.\n\nLet us now consider the cyclic subgroups of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Notice that if we pick any element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", ","}], TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"{", "x", "}"}], "]"}], TraditionalForm]]],
 " is a cyclic subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "."}], TraditionalForm]]],
 "  This subgroup is usually denoted by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"[", "x", "]"}], "."}], TraditionalForm]]],
 "\n\nDEFINITION 2.5:\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group and let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be an element in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "."}], TraditionalForm]]],
 "  We define the ",
 StyleBox["order",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " to be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", 
    RowBox[{"[", "x", "]"}], "|", "."}], TraditionalForm]]],
 "  That is, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "x", "]"}], TraditionalForm]]],
 " is finite the order of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is the number of elements in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "x", "]"}], TraditionalForm]]],
 ".  If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "x", "]"}], TraditionalForm]]],
 " is an infinite group we define the order of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " to be infinity.\n\nPROPOSITION 2.4\nSuppose that the element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " has finite order ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the      \nsmallest positive integer such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "n"], "=", " ", "e"}], TraditionalForm]]],
 ".  Furthermore,\n       \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"[", "x", "]"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"e", ",", " ", "x", ",", " ", 
        SuperscriptBox["x", "2"], ",", " ", 
        SuperscriptBox["x", "3"], ",", " ", "\[Ellipsis]", " ", ",", " ", 
        SuperscriptBox["x", 
         RowBox[{"n", "-", "1"}]]}], "}"}], "."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427564733171875*^9, 3.4275647453125*^9}, {
  3.42844619121875*^9, 3.42844620515625*^9}, {3.43628813478125*^9, 
  3.436288135203125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "x", "]"}], TraditionalForm]]],
 " is finite, not all of the elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["x", "0"], ",", " ", 
     SuperscriptBox["x", "1"], ",", " ", 
     SuperscriptBox["x", "2"], ",", " ", 
     SuperscriptBox["x", "3"], ",", " ", 
     SuperscriptBox["x", "4"], ",", " ", "\[Ellipsis]"}], " ", "}"}], 
   TraditionalForm]]],
 " can be distinct.  Suppose that  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "p"], " ", "=", " ", 
    SuperscriptBox["x", "q"]}], TraditionalForm]]],
 " for two integers ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", " ", ">", " ", "q"}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"(", 
       RowBox[{"p", "-", "q"}], ")"}]], " ", "=", " ", "e"}], ","}], 
   TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "-", "q"}], ")"}], " ", ">", " ", "0"}], 
   TraditionalForm]]],
 ". So there exists a positive integer ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "r"], "=", " ", 
    RowBox[{"e", "."}]}], TraditionalForm]]],
 "  We can let ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " be the smallest such integer. We want to prove that\n\n                    \
    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"[", "x", "]"}], " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"e", " ", "=", " ", 
       SuperscriptBox["x", "0"]}], ",", " ", 
      SuperscriptBox["x", "1"], ",", " ", 
      SuperscriptBox["x", "2"], ",", " ", 
      SuperscriptBox["x", "3"], ",", " ", 
      RowBox[{"\[Ellipsis]", " ", 
       SuperscriptBox["x", 
        RowBox[{"n", "-", "1"}]]}]}], " ", "}"}]}], TraditionalForm]]],
 "\n\nwith these elements distinct.  Indeed, if \n\n                          \
        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "p"], " ", "=", " ", 
     SuperscriptBox["x", "q"]}], " ", ",", "   ", 
    RowBox[{
     RowBox[{"with", "  ", "0"}], " ", "\[LessEqual]", " ", "q", " ", "<", 
     " ", "p", " ", "\[LessEqual]", " ", 
     RowBox[{"n", "-", "1"}]}], ",", " "}], TraditionalForm]]],
 "\n                                  \nthen ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"p", "-", "q"}]], " ", "=", " ", "e"}], ","}], 
   TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", " ", "<", " ", 
    RowBox[{"p", "-", "q"}], " ", "<", " ", "n"}], TraditionalForm]]],
 ",  which contradicts the definition of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".  Therefore, the elements in \n                                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"e", " ", "=", " ", 
      SuperscriptBox["x", "0"]}], ",", " ", 
     SuperscriptBox["x", "1"], ",", " ", 
     SuperscriptBox["x", "2"], ",", " ", 
     SuperscriptBox["x", "3"], ",", " ", 
     RowBox[{"\[Ellipsis]", " ", 
      SuperscriptBox["x", 
       RowBox[{"n", "-", "1"}]]}]}], " ", "}"}], TraditionalForm]]],
 "\nare all distinct.\n\nFinally, we need to show that if ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "x", "]"}], TraditionalForm]]],
 ", then there exists a ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " such that\n\n                                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "q"], " ", "=", " ", "y"}], ",", "  ", 
    RowBox[{
     RowBox[{"with", "    ", "0"}], " ", "\[LessEqual]", " ", "q", " ", 
     "\[LessEqual]", " ", 
     RowBox[{"n", "-", "1."}]}]}], TraditionalForm]]],
 "\n                                       \nBut ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "=", " ", 
    SuperscriptBox["x", "k"]}], TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Epsilon]", " ", 
    RowBox[{"\[DoubleStruckCapitalZ]", "."}]}], TraditionalForm]]],
 " We can define ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", " ", "=", " ", 
    RowBox[{"k", " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 ".  Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"0", " ", "\[LessEqual]", " ", "q", " ", "\[LessEqual]", " ", 
     RowBox[{"n", "-", "1"}]}], ",", " "}], TraditionalForm]]],
 "and furthermore, there is an integer ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", "-", "q"}], " ", "=", " ", 
    RowBox[{"n", "\[CenterDot]", 
     RowBox[{"r", "."}]}]}], TraditionalForm]]],
 " Thus,\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", "k"], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"n", " ", "r"}], " ", "+", " ", "q"}], ")"}]], " ", "=", " ",
       
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox["x", "n"], ")"}], "r"], "\[CenterDot]", 
        SuperscriptBox["x", "q"]}], " ", "=", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["e", "r"], "\[CenterDot]", " ", 
         SuperscriptBox["x", "q"]}], " ", "=", " ", 
        RowBox[{
         SuperscriptBox["x", "q"], "."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nSo every element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "x", "]"}], TraditionalForm]]],
 " is of the form ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "q"], TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", " ", "\[LessEqual]", " ", "q", " ", "\[LessEqual]", " ", 
    RowBox[{"n", "-", "1"}]}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427565162109375*^9, 3.427565218734375*^9}, {
  3.42756525078125*^9, 3.427565441515625*^9}, {3.428446228046875*^9, 
  3.4284462575*^9}}]
}, Open  ]],

Cell[TextData[{
 "PROPOSITION 2.5\nSuppose that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " has infinite order. Then ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "n"], TraditionalForm]]],
 " is not the identity element for all non-zero integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Furthermore, \n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"[", "x", "]"}], " ", "=", " ", 
     RowBox[{"{", " ", 
      RowBox[{"\[Ellipsis]", ",", " ", 
       SuperscriptBox["x", 
        RowBox[{"-", "3"}]], ",", " ", 
       SuperscriptBox["x", 
        RowBox[{"-", "2"}]], ",", " ", 
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]], ",", " ", 
       RowBox[{
        SuperscriptBox["x", "0"], "=", "e"}], ",", " ", 
       SuperscriptBox["x", "1"], ",", " ", 
       SuperscriptBox["x", "2"], ",", " ", 
       SuperscriptBox["x", "3"], ",", " ", "\[Ellipsis]"}], " ", "}"}]}], 
    ","}], TraditionalForm]]],
 "\n\nwhere the powers of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " are all distinct."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.427564786015625*^9, {3.4284462748125*^9, 3.428446278625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "n"], "=", " ", "e"}], TraditionalForm]]],
 "  for some non-zero ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". It suffices to consider the case ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "0"}], TraditionalForm]]],
 ", for if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "n"], "=", " ", "e"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"-", "n"}]], "=", " ", "e"}], TraditionalForm]]],
 ".\n\nBy exactly the same reasoning as was used to prove proposition 2.4, we \
see that \n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"[", "x", "]"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"e", "=", 
        SuperscriptBox["x", "0"]}], ",", " ", 
       SuperscriptBox["x", "1"], ",", " ", 
       SuperscriptBox["x", "2"], ",", " ", 
       SuperscriptBox["x", "3"], ",", " ", "\[Ellipsis]", " ", ",", " ", 
       SuperscriptBox["x", 
        RowBox[{"n", "-", "1"}]]}], "}"}], "."}]}], TraditionalForm]]],
 "\n\nBut this contradicts the fact that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "x", "]"}], TraditionalForm]]],
 " was infinite.  Therefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "n"], " ", "=", " ", "e"}], TraditionalForm]]],
 " only if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "0"}], TraditionalForm]]],
 ".\n\nMoreover, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "p"], " ", "=", " ", 
    SuperscriptBox["x", "q"]}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"p", "-", "q"}]], " ", "=", " ", "e"}], ","}], 
   TraditionalForm]]],
 " and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", " ", "-", "q"}], " ", "=", " ", "0"}], TraditionalForm]]],
 " by what we have just proved. Thus, the powers of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " are all distinct.  "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42756483984375*^9, 3.4275648765*^9}, {
  3.4284462674375*^9, 3.428446309953125*^9}}]
}, Open  ]],

Cell[TextData[{
 "Because of propositions 2.4 and 2.5, we know something about all cyclic \
groups. Any cyclic group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is either a finite group \n \n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"G", " ", "=", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"e", "=", 
        SuperscriptBox["x", "0"]}], ",", 
       SuperscriptBox["x", "1"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["x", "3"], ",", " ", "\[Ellipsis]", " ", ",", 
       SuperscriptBox["x", 
        RowBox[{"n", "-", "1"}]]}], "}"}]}]}], TraditionalForm]]],
 "\n   \nof order ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", or is an infinite group\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"G", " ", "=", " ", 
     RowBox[{"{", " ", 
      RowBox[{"\[Ellipsis]", ",", " ", 
       SuperscriptBox["x", 
        RowBox[{"-", "3"}]], ",", " ", 
       SuperscriptBox["x", 
        RowBox[{"-", "2"}]], ",", " ", 
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]], ",", " ", 
       RowBox[{
        SuperscriptBox["x", "0"], "=", "e"}], ",", 
       SuperscriptBox["x", "1"], ",", 
       SuperscriptBox["x", "2"], ",", 
       SuperscriptBox["x", "3"], ",", " ", "\[Ellipsis]"}], " ", "}"}]}]}], 
   TraditionalForm]]],
 "\n\nwhere all powers of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " are distinct.\n\nNote that in the first case, ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " closely resembles ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". In the second case, ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " closely resembles ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". \n   \nWe can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to quickly find the order of any element in the group. For example, the ",
 "octahedral",
 " group should already be loaded into memory. To find the order of the \
element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "c"}], TraditionalForm]]],
 ", we can type"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427565557578125*^9, 3.4275655931875*^9}, {
   3.427565681890625*^9, 3.427565682203125*^9}, 3.4284463245*^9, 
   3.4362789189612503`*^9}],

Cell[CellGroupData[{

Cell["Group[{b.c}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4425885315865126`*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives us the subgroup generated by the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "c"}], TraditionalForm]]],
 ". We can observe the length of this to be 4 directly, or we can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox["Length",
  FontWeight->"Bold"],
 " command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[%]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["4"], "Output",
 CellChangeTimes->{3.4425885335083876`*^9}]
}, Open  ]],

Cell[TextData[{
 "We can combine these last two ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " commands into one. For example, to find the order of  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "c"}], TraditionalForm]]],
 ", we type"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[Group[{a.c}] ]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{3.4425885378208876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Try replacing the \"",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", ".", "c"}], TraditionalForm]]],
 "\" with other elements of the group.  What do you observe about the orders \
of the elements?\n\nEvery element of this group besides the identity has \
order 2, 3, or 4. There is a geometrical reason for this: every element \
represents a rotation of either \[PlusMinus]90\.ba, \[PlusMinus]120\.ba, or \
180\.ba. (See problem 2.25.)\n\nLet us consider the orders of the elements of \
a cyclic group. Consider, for example, the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "12"], ":"}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427565720140625*^9, 3.427565720609375*^9}, {
  3.428446338546875*^9, 3.42844633909375*^9}, {3.43628839128125*^9, 
  3.436288391796875*^9}}],

Cell[CellGroupData[{

Cell["\<\
DefSumMod[12]
G = Group[{1}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", 
   ",", "8", ",", "9", ",", "10", ",", "11"}], "}"}]], "Output",
 CellChangeTimes->{3.44147870259375*^9, 3.4425885414771376`*^9}]
}, Open  ]],

Cell[TextData[{
 "How many elements are there of order 2? We can answer this question by \
taking the square of all of the elements, and see which elements satisfy the \
equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], "=", " ", 
    RowBox[{"e", " ", "=", " ", "0."}]}], TraditionalForm]]],
 " We can take the square of all of the elements at once with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G^2", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "2", ",", "4", ",", "6", ",", "8", ",", "10", ",", "0", ",", "2", 
   ",", "4", ",", "6", ",", "8", ",", "10"}], "}"}]], "Output",
 CellChangeTimes->{3.441478706953125*^9, 3.4425885437896376`*^9}]
}, Open  ]],

Cell[TextData[{
 "and see that only 0 and 6 satisfy the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], "=", " ", "0"}], TraditionalForm]]],
 ". Of course, the identity element 0 is of order 1 instead of 2, so there is \
only one element of order 2, namely 6.\n\nEXPERIMENT: \nBy changing the ",
 StyleBox["G^2",
  FontWeight->"Bold"],
 " to ",
 StyleBox["G^3",
  FontWeight->"Bold"],
 " in the above command, find all of the elements of order 3.\n\nNext, try to \
determine how many elements there are of order 6. Remember that you cannot \
count the elements of order 2 or 3 that have already been found!\n\nIt is \
apparent from the experiment that finding the number of elements of order ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " involves finding the number of solutions to the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "k"], "=", " ", "e"}], TraditionalForm]]],
 ". To help us find the number of solutions for a cyclic group, let us first \
prove the following proposition about modular multiplication.\n\nPROPOSITION \
2.6\n\nLet ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " be two positive integers.  Then \n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", " ", "k"}], " ", "\[Congruent]", " ", 
    RowBox[{"0", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 "\nif and only if\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    StyleBox[
     FractionBox[
      RowBox[{"a", "\[CenterDot]", "n"}], 
      RowBox[{"GCD", "(", 
       RowBox[{"n", ",", "k"}], ")"}]],
     FontSize->16]}], TraditionalForm]]],
 "\nfor some integer ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.42756576159375*^9, {3.441478910140625*^9, 3.4414789153125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First of all, notice that if \n\n                                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    StyleBox[
     FractionBox[
      RowBox[{"a", "\[CenterDot]", "n"}], 
      RowBox[{"GCD", "(", 
       RowBox[{"n", ",", "k"}], ")"}]],
     FontSize->16]}], TraditionalForm]]],
 ",\nthen \n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     FormBox[
      RowBox[{
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "k"}], " ", "=", " ", 
        RowBox[{
         StyleBox[
          FractionBox[
           RowBox[{"a", "\[CenterDot]", "n", "\[CenterDot]", "k"}], 
           RowBox[{"GCD", "(", 
            RowBox[{"n", ",", "k"}], ")"}]],
          FontSize->16], 
         StyleBox["=",
          FontSize->10]}]}], 
       StyleBox[" ",
        FontSize->16]}],
      TraditionalForm], 
     RowBox[{"a", "\[CenterDot]", "n", "\[CenterDot]", 
      FormBox[
       StyleBox[
        FractionBox["k", 
         RowBox[{"GCD", "(", 
          RowBox[{"n", ",", "k"}], ")"}]],
        FontSize->16],
       TraditionalForm]}]}]}], TraditionalForm]]],
 ".\n\nand since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"GCD", "(", 
    RowBox[{"n", ",", "k"}], ")"}], TraditionalForm]]],
 " is a divisor of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ", we see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "k"}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".  Thus, \n\n                                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "k"}], " ", "\[Congruent]", " ", 
    RowBox[{"0", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 ".\n                      \nNow suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "k"}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". We want to show that\n\n                            ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", 
    StyleBox[
     FractionBox[
      RowBox[{"x", "\[CenterDot]", 
       RowBox[{"GCD", "(", 
        RowBox[{"n", ",", "k"}], ")"}]}], "n"],
     FontSize->16]}], TraditionalForm]]],
 "\n \nis in fact an integer. By the greatest common divisor theorem (1.2), \
there exist integers ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"GCD", "(", 
     RowBox[{"n", ",", "k"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"u", "\[CenterDot]", "n"}], " ", "+", " ", 
     RowBox[{"v", "\[CenterDot]", "k"}]}]}], TraditionalForm]]],
 ". Then \n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", 
    StyleBox[" ",
     FontSize->16], 
    RowBox[{
     StyleBox[
      FractionBox[
       RowBox[{"x", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"u", "\[CenterDot]", "n"}], " ", "+", " ", 
          RowBox[{"v", "\[CenterDot]", "k"}]}], ")"}]}], "n"],
      FontSize->16], " ", "=", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "u"}], " ", "+", " ", 
      RowBox[{
       StyleBox[
        FractionBox[
         RowBox[{"x", "\[CenterDot]", "k", "\[CenterDot]", "v"}], "n"],
        FontSize->16], " ", "."}]}]}]}], TraditionalForm]]],
 "\n\nSince",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"x", "\[CenterDot]", "k"}]}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", we see that ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is an integer. Thus,\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    StyleBox[
     FractionBox[
      RowBox[{"a", "\[CenterDot]", "n"}], 
      RowBox[{"GCD", "(", 
       RowBox[{"n", ",", "k"}], ")"}]],
     FontSize->16]}], TraditionalForm]]],
 "\nfor some integer a."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.427565812953125*^9, 3.427565813078125*^9}, {
  3.4284463653125*^9, 3.4284463699375*^9}}]
}, Open  ]],

Cell[TextData[{
 "We can now find the number of elements in a cyclic group which satisfy the \
equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "k"], "=", " ", "e"}], TraditionalForm]]],
 ".\n\nCOROLLARY 2.1\nThere are precisely GCD(",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", ",", "k"}], TraditionalForm]]],
 ") elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "k"], " ", "=", " ", "e"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.427566092671875*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " be a generator of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    SuperscriptBox["z", 
     RowBox[{" ", "y"}]]}], TraditionalForm]]],
 " be an element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "k"], "=", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["z", 
        RowBox[{" ", "y"}]], ")"}], "k"], "=", 
     SuperscriptBox["z", 
      RowBox[{" ", 
       RowBox[{"y", "\[CenterDot]", "k"}]}]], " "}]}], TraditionalForm]]],
 ", which is equal to the identity if and only if \n                          \
  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", "\[CenterDot]", "k"}], " ", "\[Congruent]", " ", 
    RowBox[{"0", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"Mod", " ", "n"}], ")"}], "."}]}]}], TraditionalForm]]],
 "\n                            \nBy proposition 2.6, this is true if and \
only if\n\n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "=", " ", 
    StyleBox[
     FractionBox[
      RowBox[{"a", "\[CenterDot]", "n"}], 
      RowBox[{"GCD", "(", 
       RowBox[{"n", ",", "k"}], ")"}]],
     FontSize->16]}], TraditionalForm]]],
 "\n for some integer ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ".  Hence, the number of possible values of ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " between 0 and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{" ", 
      RowBox[{"y", "\[CenterDot]", "k"}]}]], "=", "e"}], TraditionalForm]]],
 " is\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox[
     FractionBox["n", 
      RowBox[{"n", "/", " ", 
       RowBox[{"GCD", "(", 
        RowBox[{"n", ",", "k"}], ")"}]}]],
     FontSize->16], " ", "=", " ", 
    RowBox[{"GCD", "(", 
     RowBox[{"n", ",", "k"}], ")"}]}], TraditionalForm]]],
 ".\n\nEach such value of ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " between 0 and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " produces a different solution  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    SuperscriptBox["z", 
     RowBox[{" ", "y"}]]}], TraditionalForm]]],
 ", so there are exactly ",
 Cell[BoxData[
  FormBox[
   RowBox[{"GCD", "(", 
    RowBox[{"n", ",", "k"}], ")"}], TraditionalForm]]],
 " solutions."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.427566100125*^9, {3.427566147609375*^9, 3.42756615196875*^9}, {
   3.428446379359375*^9, 3.428446492109375*^9}, {3.4362885245*^9, 
   3.436288527640625*^9}, {3.436288569875*^9, 3.4362885725625*^9}}]
}, Open  ]],

Cell[TextData[{
 "EXPERIMENT:\nUse ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the number of solutions to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "8"], "=", " ", "0"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "12"], TraditionalForm]]],
 ". Does this result agree with corollary 2.1?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.42756623353125*^9}],

Cell[BoxData["    "], "Input"],

Cell["\<\
Now let's look at a more complicated group. You probably have heard of the \
infamous Rubik's cube\[Trademark]. Rubik also came out with a number of other \
puzzles, one of which is called the Pyraminx\[Trademark]. The Pyraminx\
\[Trademark] consists of a triangular pyramid, with each of the four \
triangular sides partitioned into 9 smaller triangles. The four \"tips\" can \
rotate, but this does not affect the puzzle. To simplify the picture we can \
chop off the tips of the pyramid. Execute the following command to show a \
picture of the puzzle with the tips cut off:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ShowPuzzle", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4425885542115126`*^9}]
}, Open  ]],

Cell["\<\
Now the four corners of this puzzle can rotate, moving some of the middle \
pieces in the process. For example, the front corner can rotate clockwise 120\
\.ba by the command:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotatePuzzle[f]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4425885567271376`*^9}]
}, Open  ]],

Cell["We can rotate the back corner clockwise with the command:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotatePuzzle[b]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4425885591646376`*^9}]
}, Open  ]],

Cell["\<\
At this point, we see the advantage of chopping off the four tips of the \
pyramid: this allows us to peer inside of the puzzle to see what is on the \
back face!

The left and right corners can be rotated using the commands\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotatePuzzle[l]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4425885619302626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[CellGroupData[{

Cell["RotatePuzzle[r]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4425885644615126`*^9}]
}, Open  ]],

Cell["\<\
Now we have really scrambled the puzzle! Not to fear\[Dash]\[Dash]we can put \
the puzzle back into its original form with the command\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ResetPuzzle", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4425885669927626`*^9}]
}, Open  ]],

Cell[TextData[{
 "That is sure much easier than taking the puzzle apart and putting it back \
together again!\n\nThe set of all actions on the puzzle forms a group, called \
the Pyraminx\[Trademark] group. This group is generated by the elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"t", ",", " ", "b", ",", " ", "r", ",", " ", "l"}], "}"}], 
   TraditionalForm]]],
 ". However, this group has over 900,000 elements!  Thus, it is not such a \
good idea to have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " list all elements of the group. It really wouldn't tell us much, and would \
probably run us out of memory.  \n\nWe can determine information about the \
Pyraminx\[Trademark] group by working with the puzzle. For example, we know \
that the elements ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["l", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["t", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " each have order 3. But what about the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "f"}], TraditionalForm]]],
 "? Try this out yourself:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotatePuzzle[b.f]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4425885719927626`*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztnXmQHUUdxx+7+/a+j+y9+zabbK5NNtlAMF6IKFoKioAoCiRLDCQhgRgi
RwQCBkSUS0BAkUtERFHLW0tLS0st76u01NLS0lLRUqsstfTPlZlfz69n5n36
bY/zNlDlJrXv9fb09O/7/f16prt//eveU+cO7tq5b+7g7h1zpZccmNu/a/eO
S0snX3LgqazaYwqFY/5YKNSMlwpBer5QiD7Cf08GH+aXpfRSeim9lF5KL6X/
j9JPBF9N81GXOCu5jwdfDUFucf6pDrNDr0/K9ceCr5ogtyG4Hqa6g1S7luyT
ko9qyZbgeqE2LGD+F0aTtdfLPY/oPe1a+7SW/LcUelgLdWuhrQrhT1LoQS20
TAu9WGv6hRS6XwsNaaHTtdC3pdB9WmhMC21TcV+UQvcGXyHD5VroIq3pI1Lo
bq1pFWjk8uCzPsh6TIrfpcXXap1XBKm6IBXcHd4nw5pH5Z479J4Nes9BTVmT
PSLFb9fis3ppv6ZaNPWwFL9Vi2/RS/uASptefVBuvFlvfLZe2p1sP2Hqfil+
k2rz+Xppp6asQe+T4jdq7ScBmO1a3Br53XLjDXrjyXrpPE1Zc98jxY9o8Zfp
pddpyhr+XVL8Oi3+Cr10lqamNHWnFD+sxU9PNsZUE3inFL9ai9s6rRxr/duk
+CFVqUVsWVjr3yLFr9Dat4FKT4a8Y7WKd0gVl2kV5+ulFyUf2DB1kxQ/qMUv
0Esv0NTzNHWjFD+gxffopedq6sQYQPOvcL3cuF9vvFiLPwsAmuIXa/EDeslS
famm3iLF92rxy/TSRk2doqlrpfhutcohUOk6LX6apq6RGy9UOdfopTWaOlNT
V0nxnSrnumQDTEm0TelQ7Mbu5FvENvTXx95LtniY8VTF89HrKXojhO/CC+H5
2qapy9L11AS/h6lhrcK2YPtE74i97mwV2gQKFlNYaLVWdiq8VyzGA1LZG8Kv
Oq3MVFgML8Jz0KGtN3W1K9k7mBduTEYxISMCXlTgJ2n19pnt0NQlmrp44Uob
tNLTtNKV8PK/VFN745XWxisVO6Xy6lXAuSpgQCuzndHlmtqTTUAj6H+3irIM
6sASV+rVXW6htS6htjPuSj6EF2SrrFkrO6KVWX2/wV1ZHVTWopUdrki9WVVk
X7XnZxPVBpo/BKYdVFFna952t6iiS5TVd6pxTqkA+3I9TwTsAAH1IKBDBdh3
nH2kNquAEzTvHLeABhDQCcqyT2qPpsIXRrsKNeOLTKK6lMs+raJfU6coFzv6
eK1bQCMI6FYBe6BlDWq1Z6som3dWNlG9Kso20zEQulNFtWq5M92imkBUH1ho
TiubiPUtT6U6g9Tp2QQsUy7btDL7rrWd8qvc1TZDtf1are2K7TDg6iBVuwBa
qnYQ1GGHBus1Zd+hwctNZiLhb0be+SCv1SUvNXLZpKlwJCUzo1BIJMh8Z5Q1
rLLsoOo4TdlBXkgoundhWm0gakRF2XfTVmjA9lGtA6FnZBM6BrZ7KQi1E7G9
satGv4Z9XVrnFbC0u7CEMuz8yr5Fd2mqvirSSirthVrzSbG+1PZOKO1Mt7QO
kDah0uwc9SWamgOdVkfuJNjXTqZerqlzNdUUyi0m5EaNq4LwTpfwsNLjtfpX
xvqqctKNqRYdJGriefIverTmAEgXAFmpQGxXeQa08lfr1ZYKWni1W3g3CJ8C
E9gJ91lJQGGq1VMLZ2QDskq1MJ00gn2/pYS2AJDabEB6AMhqBbIGjHCOQrLv
wfbFAbJWgdje1Xbkdrre4Sn+TLf4XhC/DlpGSYVaF9aLwTSkkbr8kKZVI3a+
bV1vJ2qqK7/4PhC/XsXbuTq5DK3vphuAFPMDmVEgdp5vXZ3PAYOQRgiIeXts
ByD9AGQjtBHrFrBTEPuG7V0cIJtUI3a2Y6dY9r3aB+Lrs4kfAPGzKt7O5q7S
1EYwCOmBgJyVDcixYJAGaKJ2km/ftv35IQ26INWInSNRRzS1WlMDIL4hv/gt
Kj64MC/D7eC3G0EtK8BSpJbGbLiGANfxwWdrkHVYVbBcU0OeynjN/yA04mzG
DMGy2Xw45bQTBvt6HfZkXwHIMAA5LpUXGGRQgeyMiS83COmmyQ1pG0Aa8YQ0
Gmooqa/G4DOcN78m9uhEqdH84Eadz3WZ03YE8A2oGu2aSE/Msn74XlsVfKS/
KcVnF0I6wcykyeZsSMcA6ebwk73IpM6NkFejTWAGXiK2CygdfQqk8Q2Q16NW
sB6hJk1NeAI/2w18HIDPQl7cFx6BGwLA6yFvUkn0KvR6aEpkh5b8dDZBXi3Q
GQbo05A3q82qXunUaGoyP4kSAN4IeXVAgmyyDvJOCj6bg6wVALj16AEmra+B
vFNieRHM5Z7QX+eGPgEwZyCvCNBJ1wT95cFnTRZdG8DnAeDlOQCTrle5dO1s
zW3ZAE8CuA2QVw+AScNTHo2j4Ggc7fmhr4e8BoBOul7pahwpmL5af70b+gqA
OQ15jQCdtL6iylpfJOik9eVO6GWOUepQO7KhXwlI10FeE6AnxRN6aTPsWHya
KZABSh5tZ97Rdgj6OW7oUwBzLeQ1A3TS/rgTOmu/BBQ6jx4F0v6YswElYdYA
dJqGVKBzLtBZBdDXQF4r0CGL0MiZGhPRobkK0TnXTWc1QKc8okPWoTyik14b
DBIjkNe1OHTagA5Zh+hQYyM6ZJ2MdKhhUQMkOgR9sMrW6V4cOu1Ah6wz4Ekn
vSYcJMjDRHTOc9Ohtxi97YgOWaffs7EVjx4d6jo7gQ5ZZ1mVrdOTjQ718DQU
IzpknV5POmQd8h32Lg6dLqBD1iE61NjqPemQdbYJnXOADk07aHpCdMg63Z7W
ITqDntapAp1uoEPW6aoynb5sdGg+SxN1okPW6fRsbA2edMg627PRIUdJL9Ah
63R4Wofo0HoTWacCHXKPlDzpkHXaPOlQIAotUy1bHDp9QIes40vH1zoZ6ZCj
cNyTDlmn5ShYZy4/nWWe1mn2pNO0EHSiuDAdcp7TYgfRIes0HT06oYcjvEIL
GkSiH0iQTRqrTGLATSK0iYmOKAdMC6QDQIIsUe9JotmTBFnifGhYoTi2CdEZ
BDpkkzx0KNyEbGLoyHDTOg9FJD4qtKhOjMhAdZ6MKMCO4lYG/RmZn7DdUXdD
tIaAFhmqNgctMlQFWgn3mimejIEU6DQapbiMo0HR13I73BRr1Xa+xIadxIrx
vEhAsOuzjAwFnPZA3pCbzDYgkwwIFLg0t6MwqxEnrboULWTUtjiMimoeXx5h
KEe4S6d8JO0PnaIcCbrZArYdoNcrdHJ+UMQfRaGEm8a6smidoA9ng56MhhIg
5EsjEjRWCN/iYRTACUCCImszkphzkTABeeWQlmWBHoVEGxIzniQoQnUkG4lk
hIpAIr8Z0Rl3WgyCmubNHq1x4EbR2J2e3GSfb3IHQ5xbqFpybFKQMDES5hw5
NK0ma/ek5WuyCrSSMQoCktw1FA1OjgNSBO223ao2JBJkr1E3sR0uYlFoRpl4
CvdfyA8S5dHW25PjeRVIkBXHshFrV2LkhyJiC/mrojzaMPsiIEYWI2JkMdrS
TFVU8H4Q9GpTzGO7ChQ71Xa+xOhZpM1UtH/3hZ62o95gTJGaHe07gU5yDU+E
UrdEm66IGG1Ro828JwIx6oOdxFIWu9BNsVv1QMTIEtTh0Q5D2tn7Ak9iNJIa
1Vt3C50LgE5ybUKE0tycLEHEaNMobQI+IS+xQnJkvMdNsVct5kuMOnTa7dsC
ec8DYjSTIGLDijS8MB+1yYjdhcCuT++hmTq1M2JHTIixLzuaIcoAmGeZF7kJ
Jp1NAoNmTkSVRv30mNG9zwGqNGOnmf2QGiW0Y+rtYqy5y0U2vJF2ztDDlYfi
sz0pkjXjjgqTLVH4KbIXuckmfR0CiWYCRJsmRQudjhHlbc1h2QE1UNFNdreL
bLSrqwwSPX8UEkInjFBn+iygSB41cpHG3aGRZeMOyojsXjfZ5FxXINHImWiT
u4FGaTQ42gK0yYHtdHRHGxbKyO4TsntcZKO9jGWQqPMjinQODo3diaKvZeOq
iCybkeyokiVwvmRpxkTKOxbIkj1p9cV2jPE1nIiiOcHqIqCYHBC6nzF6y5Kb
j2a7RHYzkKVVSiIbf31Flm0A2pe4aY+rvrKRZd8FdV+zngRpubZH0cVXSVO0
9gKtEliTxnzUeaR96gEx8qlvykssZbn4ymlEcb+b4oTqJg8xGj7OADGKAyBi
XYoqvkwf0Xmj0NkHdBKx4RWGANQXUjgLWcyXGAVCxCe5kcUyUpxU3eQhRhZb
D8QoTIiiozpjqMrpmMMKLwY6K/RGmjNQ500rjWSnaaBDIXZEJ+6eieyUkdgU
NEV6E/pSJIutA4q+FrOuv/hqUUTsUiF2iYuYeZ2WQ6LumZa8yWJrgQ5ZjGJB
4+6ZyGIJ17chdtBNbDVYjN6AvhTJYquBIkXlEkXrhY7HMaaI7XcRM8OZckg0
xqCwC7KYLx0KRo7PDCOLxSMaI2JvchNbCxaj58mXIllsKgfFFlV8PJA2ImaO
lH2ji1h0Atm8a6CUPuTAZaWVQIH2HRCF+FwgslJGMtNKpi4jGbLHCk8ytNmi
SZH0AoXLhcIBoLAeGpo0qrIFxS2AmYwyCTxobwvtUIoP4yOj9ACjK9yMNqgq
0g9HYIzjATAZY8KTBBmjQREsc0O/FKDPoDEKKc9FBdX7oibVx0d1ker7AP+V
bvwbtd0QYFLzOACWqSbvL7QBAP0AzZygfRCgbdIbtwIM0uUYQBuvAI30lxHk
bEWDk/5GAWQl/dkAlkGA9maB9iaX/kLT0hNE+hsBaKK/sm3H8VFclDsA+K5y
49uYatGFdCsn50kFxQ4B+hKjr1OdDrsxX+Z83Mvqq9H6fLtBN9ikQuLdYJQ9
BJivzoyZotR8xyEDnuhtWN8IYDan9F8OmDdUeCCs8Si+mJTdD3DTm87TrS/K
riJw+yT7jtCXuYBHi2um6vC3MUB6WJBeAUjXpxQwX+BOJnmGnuDynfb2Af70
lvlCIRXuZKBfmx+6VbjvJLYXAE9oVskN80qAuQ4g0bA2efyaAPENg+0BwHEN
R0omDV+XH7rt3AkwabgLAFfWsPmjIYcA5lqARLPV5DlkAoRc3KRhAlwyCrEa
LiwiehuhR5hJyR2AeVxrCS8oaER9RFC/GVCvcT1oqTw736SVE9J0uwt1Weyz
bC+hNn19fuDJY2sEGq1Ok+LbgIJdaqqNATYXK5G4Ckiscj2ERMIZdkvab3VB
T71JZHG8LBqdep8b8vOg2DmK1yFjNAOjEdVLkXnED+uIeLxVeFwNPKYAMy1s
dahcClshexD6uK8nskedvz2qwIMi4yjijezRCIxshIks4HJYDRnF/Imra4DM
SgBO68h2aYdCackoDS4KKaNUIkMjyrflJ2NXFilmlOxRD2RsXAjtGKyBPJqN
GDqHgQ6dJ7fgpiIDuORppaKLWAp6fGU5K8Wb3BTpLDQKPbIr+0SMLFYLxGyo
Fe27JTrkR3i70LkW6NAZVxT1l9x1LIAptpQsRsQoyoP24pb3qotG0eqaiA2b
xhfLC38KiTtpszftmyRvivmjgdcBBTpEjcKEk5tEBTjFk4qVyjzK8XCACCkx
IqMQo5vzM7IRaRl42GePzhLwtYdB/xZAXwKkFGhvO0CnT6vsfPkIUivALEIe
+TRvcUMfB5gLbzQSwBTRSr4uG35OJ20QCXLM35qNhHNbkTNSdRCgU9xEOwD2
tYQhcQRI0GF2RIJCvSgClfx2NlqCjnChPepkidvykxhXIBRZSq6weKOMwPmS
oHWS24XE9UCClmQozr+kJCg0jkjY2Ac64IiGKIsEneKVKDiSfGQU8UAHHPnS
MX++9wagQ+urFLq/XPVKJMhvZufMdHIWQaeV2zvc0GmpngK2E2NkckOZvG4g
QZEMPQCdhvcV6LwV6FA4DIXw2rAqIkH+Nes76gOYNAikFec780OnwCkKTiSH
W3xWQ209qyUMnRuBDsVZUXyxDZfyJWGjFOjoQl9L3JUfuo0boh0g5HejwAQ6
sJCmTRRVYv5g+duABIVcUuBswgVpoC+4Zcnca13tdMRnFUgQYCJm44EW3Etm
7qB1djrYkyYSFF52t5C4CUhQ06EmlljgMNApIpK8cHZZiU5S8SVxT34SNpiJ
oh/J3UbL9lUg8XYgQa9Teu1SOBNFOhIduxRMf2CFpkcU1HdvfhI2gomiGsnR
Rgv7vif3UZDsu4XEO4AEDTFoKDKjJChym5xqdn2fjuz21X8VoG+CRkQ7keqA
BK3605HdvpZ4j9C5GejQsJuG5zb0iUjUGiqxO7T8OICkySmFI9+XH/hmsEON
aT6xvPCnEDWgFLQqULgFKCw4b45TiAUj1hNwLTORA+x784M9LvxsTECM0lG5
QgH/IgP5XWhbwf2C8lZASU6hcRfKUI+TTy+Q49Vu9MdByHXQCkAeECC3ARDy
T5Ifcwu0ezKSL6QHn7GQbgdI5FonF/xxAIkeOPI00E6mhxYHUgnEky+Hdvw9
LJDeCZBoyYiWlo71hOSrpSpA2lxlLb1PIN0BkGgplJZMZwHSOIgnXwntY3zk
GQvpToBES/m05L8JINHQzhfS+xcHEg3PyGdHO6YfFUh3ASQKQKFAlRlPLT3N
kEhL5ICiDdYfEEjvAki0l3Q15G3IYTiC9JgbEoVxUbgXQaKZo6+WDKS7ARIF
8lHA33qARPNA8lzRmQcfzA9p2lNLBIlOE3lcIN0DkCimlGJP13lqiZyURxES
uVJ8IX1IIN0LkGjH0jTkrc0BiU5G+bAbEoVbU1j2Gk9I5A6sAElcBhxmn95X
Mp96P0VFyY1KebUA44nFhUHaIBgfERjicSjb4bEJEKwCBCTNVxHVQUANkloJ
Hcz0UUFwX6qRBiaYBfErPRVQ6Szpj+cXmYfxx0R86D6I9sGUCV0BQumFPVyB
5ydigsK7iF02QWVxqp8UGQ/AO4ZazyRIo956pKK0B0EaPa+0kyKDtE+JtIfU
UMRoIpOMeF6h8GkR8D6gs8FTFI2+ykdpKuoR5TIDAkoggKYK4WA5rOUzUu2j
wIB6OF8BxMCI+oAyoP5qLDODz0q1HwQGNLgiATQNHo/lRQ++YfC4MqBxCcXs
kwAKEDdcPgxcaIRPosjLQKI+J6KeUC40Xh8BAeR/GlVrfEGq/SgwoMklBdQT
A4rZNqI+pgxoXkbB4cRgWBl8Sar9ODCg2GkSQB5V2uhqRH3CUxQdlUiiKFz6
yyLqk6osEkAx2iRgUJX1Van208CAvHcUvUreegofNqI+owzyCOhXBl+Xaj8H
DEoggIKiaWGKAg2NqM8rA18BtJBHAr4hAr4AXGjxgg57Iy59qqxviYAvKgPf
aokBxZ0ZAV8CBgvFnUXV0tI5ifq2iPqycskjoEdV9D2p9ivAgBauu0EAxZJQ
VI8R9VVlQAIoWtYZrBLW8kOp9mvAgMJRKAiUBFD4ghH1dRBF0TAkikKgSNSP
RNQ3VFnEhY76ovCwTlXWT6TabwIDil3wFUDr2kbUt5QBham0eQpoVwY/k2q/
AwwWOm86qoxCM2nB1Yj6rjJY6JDgSgJoifLnIuD7wIVWdEkUxSq3qrJ+KQJ+
oAxoYZ4CDykYnVbtjIAfAQPavUAnMhGDZmXwaxHwY2VAa7cUukcMaK3ICPgJ
MKBFPhJF2z9I1G9E1E+VC621NnoKaFQV/U6q/RkwoJ1yFF3nFJBiYET93FMU
hY+RqHrl8gcR8AtVkW+1ta5qUwyMgF8CA1phKoIo2iNHov4oon6lXPIIqFMV
/Vmq/TUwoKUWCplxCkgxMKJ+owzcAtgjVFTUf5WqfguoaZ2hpkKlhNRU/ztF
6q60zO1C9f1N6vs9wCVXdnlMmPwmWP4ulf1BwdXGgKgBCq6t8v+Q258Mv/gs
obKzJUTOP+XOP4VfxfSd85FMgzVSQOFfclto/Vqj8uge8/NvKfKX4CuU/R/J
+GtcYUaX5lL4y5NL6aX0Uvr/LV045r8hVGyK\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4425885720708876`*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that we have the animation feature here, as we did for the \
octahedron. How many times must this statement be executed to get back to the \
original state? This will give us the order of the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "f"}], TraditionalForm]]],
 ". So we have a subgroup, namely ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"b", "\[CenterDot]", "f"}], "]"}], TraditionalForm]]],
 " of that order. \n\nEXPERIMENT:\nUsing this method of repeating the \
operation, find the orders of the following elements:\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "b", "\[CenterDot]", "f", "\[CenterDot]", "r", "\[CenterDot]", "f", 
    "\[CenterDot]", "f"}], TraditionalForm]]],
 " \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[CenterDot]", "b", "\[CenterDot]", "r"}], 
   TraditionalForm]]],
 "\n\nOne can see quite a variety of possibilities for the order of an \
element in this group.  Can you find an element with an order larger that the \
last example? Perhaps we need to study the puzzle further to answer that \
question. When we know more about groups in general we will not only be able \
to answer this question, but also build up some tools that will help us to \
solve the puzzle."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Problems"
}], "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4362892188125*^9, 3.436289221234375*^9}}],

Cell["     ", "Input",
 PageWidth->Infinity,
 FontFamily->"System",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->RGBColor[1, 0, 0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "\n2.1 ) Use ",
 StyleBox["Mathematica'",
  FontSlant->"Italic"],
 "s circle graph to \n    find all of the generators of the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "21"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436289009921875*^9, 3.436289048328125*^9}, {
  3.436289225703125*^9, 3.436289227765625*^9}}],

Cell["          ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "\n2.2 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to see if there an \n    element of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "25", "*"], TraditionalForm]]],
 " which generates ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "25", "*"], TraditionalForm]]],
 ".\n    If so, how many such elements are there?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4362890303125*^9, 3.436289060578125*^9}}],

Cell["      ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "\n2.3 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to define a group\n     which has two elements, ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", for \n     which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["a", "5"], "=", " ", 
     RowBox[{
      SuperscriptBox["b", "4"], "=", "e"}]}], ","}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["a", "2"], "\[CenterDot]", "b"}]}], TraditionalForm]]],
 ". How\n     many elements does this group have?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428446685828125*^9, 3.42844668746875*^9}, {
  3.436289073125*^9, 3.436289093296875*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "   \n2.4 ) Use problem 2.3 to find the subgroup \n     generated by the \
elements ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", "2"], TraditionalForm]]],
 ".  How \n     many elements does this have?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["    ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "\n2.5 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the order \n     of the elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "f"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "b", "\[CenterDot]", "f", "\[CenterDot]", "r", "\[CenterDot]", "f", 
    "\[CenterDot]", "f"}], TraditionalForm]]],
 ", \n     and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[CenterDot]", "b", "\[CenterDot]", "r"}], 
   TraditionalForm]]],
 " in the Pyramix\[Trademark] group."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428446934703125*^9, 3.42844693509375*^9}, {
  3.4362891281875*^9, 3.436289154203125*^9}}],

Cell["    ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "\n2.6) Can you use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find \n    an element of the Pyramix\[Trademark] group \n    that has \
order 30?  \n    (Hint: exactly five of the six edges must be \n    moved out \
of place. The sixth edge must flip\n    as well.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428446937515625*^9, 3.428446937953125*^9}, {
  3.436289163015625*^9, 3.4362891869375*^9}}],

Cell["    ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "\n2.7 ) Find all of the generators of the group \n     ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "24."], TraditionalForm]]],
 " Then have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " construct a \n     multiplication table for the group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.428446940015625*^9, 3.4284469404375*^9}, {
  3.43628919390625*^9, 3.436289194265625*^9}}],

Cell["    ", "Input",
 PageWidth->Infinity]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{626, 566},
WindowMargins->{{Automatic, 223}, {Automatic, 28}},
DockedCells->(FrontEndExecute[{
   FrontEnd`NotebookApply[
    FrontEnd`InputNotebook[], #, Placeholder]}]& ),
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.5,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[568, 21, 321, 9, 160, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[914, 34, 288, 8, 29, "Name",
 Evaluatable->False],
Cell[1205, 44, 619, 15, 70, "Name",
 Evaluatable->False]
}, Closed]],
Cell[1839, 62, 273, 12, 142, "Title",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2137, 78, 104, 3, 32, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2266, 85, 274, 6, 43, "Input",
 InitializationCell->True],
Cell[2543, 93, 142, 2, 32, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2734, 101, 93, 2, 105, "Section",
 Evaluatable->False],
Cell[2830, 105, 1657, 45, 436, "Text",
 Evaluatable->False],
Cell[4490, 152, 77, 2, 43, "Input"],
Cell[4570, 156, 539, 17, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[5134, 177, 107, 2, 43, "Input"],
Cell[5244, 181, 30678, 568, 546, 6643, 172, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[35937, 752, 2093, 64, 566, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[38055, 820, 106, 2, 43, "Input"],
Cell[38164, 824, 6753, 173, 546, "Output"]
}, Open  ]],
Cell[44932, 1000, 1006, 29, 213, "Text",
 Evaluatable->False],
Cell[45941, 1031, 342, 9, 90, "Text",
 Evaluatable->False],
Cell[46286, 1042, 450, 11, 115, "Text",
 Evaluatable->False],
Cell[46739, 1055, 484, 16, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[47248, 1075, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[47336, 1079, 3191, 109, 389, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[50542, 1191, 946, 32, 115, "Text",
 Evaluatable->False],
Cell[51491, 1225, 1659, 69, 333, "Text",
 Evaluatable->False],
Cell[53153, 1296, 491, 18, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[53669, 1318, 76, 2, 43, "Input"],
Cell[53748, 1322, 73, 1, 42, "Output"]
}, Open  ]],
Cell[53836, 1326, 388, 10, 90, "Text",
 Evaluatable->False],
Cell[54227, 1338, 2582, 87, 264, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[56834, 1429, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[56922, 1433, 6331, 224, 661, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[63268, 1660, 450, 16, 93, "Text",
 Evaluatable->False],
Cell[63721, 1678, 78, 2, 43, "Input"],
Cell[63802, 1682, 153, 4, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[63980, 1690, 95, 2, 43, "Input"],
Cell[64078, 1694, 4869, 102, 523, "Output"]
}, Open  ]],
Cell[68962, 1799, 968, 26, 317, "Text",
 Evaluatable->False],
Cell[69933, 1827, 77, 2, 43, "Input"],
Cell[70013, 1831, 165, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[70203, 1840, 94, 2, 43, "Input"],
Cell[70300, 1844, 12779, 234, 523, 2489, 63, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[83094, 2081, 803, 20, 271, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[83922, 2105, 84, 2, 43, "Input"],
Cell[84009, 2109, 6558, 155, 488, 3479, 103, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[90582, 2267, 442, 11, 142, "Text",
 Evaluatable->False],
Cell[91027, 2280, 989, 24, 338, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[92041, 2308, 102, 2, 43, "Input"],
Cell[92146, 2312, 3498, 84, 523, "Output"]
}, Open  ]],
Cell[95659, 2399, 648, 15, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[96332, 2418, 77, 2, 43, "Input"],
Cell[96412, 2422, 190, 5, 42, "Output"]
}, Open  ]],
Cell[96617, 2430, 330, 11, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[96972, 2445, 90, 2, 43, "Input"],
Cell[97065, 2449, 16927, 315, 459, 3994, 101, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[114007, 2767, 365, 9, 164, "Text",
 Evaluatable->False],
Cell[114375, 2778, 349, 9, 115, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[114761, 2792, 289, 9, 143, "Section",
 Evaluatable->False],
Cell[115053, 2803, 3164, 91, 813, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[118242, 2898, 76, 2, 43, "Input"],
Cell[118321, 2902, 72, 1, 42, "Output"]
}, Open  ]],
Cell[118408, 2906, 78, 2, 43, "Input"],
Cell[118489, 2910, 588, 17, 190, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[119102, 2931, 79, 2, 43, "Input"],
Cell[119184, 2935, 904, 18, 42, "Output"]
}, Open  ]],
Cell[120103, 2956, 863, 31, 273, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[120991, 2991, 77, 2, 43, "Input"],
Cell[121071, 2995, 5615, 150, 488, "Output"]
}, Open  ]],
Cell[126701, 3148, 1002, 29, 289, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[127728, 3181, 76, 2, 43, "Input"],
Cell[127807, 3185, 72, 1, 42, "Output"]
}, Open  ]],
Cell[127894, 3189, 77, 2, 43, "Input"],
Cell[127974, 3193, 77, 2, 43, "Input"],
Cell[128054, 3197, 433, 14, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[128512, 3215, 80, 2, 43, "Input"],
Cell[128595, 3219, 292, 6, 42, "Output"]
}, Open  ]],
Cell[128902, 3228, 106, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[129033, 3234, 76, 2, 43, "Input"],
Cell[129112, 3238, 6153, 141, 488, 3192, 91, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[135280, 3382, 473, 15, 90, "Text",
 Evaluatable->False],
Cell[135756, 3399, 79, 2, 43, "Input"],
Cell[135838, 3403, 227, 7, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[136090, 3414, 76, 2, 43, "Input"],
Cell[136169, 3418, 3601, 104, 488, "Output"]
}, Open  ]],
Cell[139785, 3525, 311, 9, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[140121, 3538, 81, 2, 43, "Input"],
Cell[140205, 3542, 292, 6, 42, "Output"]
}, Open  ]],
Cell[140512, 3551, 1092, 28, 360, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[141629, 3583, 76, 2, 43, "Input"],
Cell[141708, 3587, 72, 1, 42, "Output"]
}, Open  ]],
Cell[141795, 3591, 338, 8, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[142158, 3603, 76, 2, 43, "Input"],
Cell[142237, 3607, 3832, 68, 660, "Print"]
}, Open  ]],
Cell[146084, 3678, 221, 5, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[146330, 3687, 80, 2, 43, "Input"],
Cell[146413, 3691, 17087, 287, 660, 3788, 67, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[163515, 3981, 307, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[163847, 3993, 79, 2, 43, "Input"],
Cell[163929, 3997, 17099, 287, 660, 3788, 67, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[181043, 4287, 1379, 30, 556, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[182447, 4321, 76, 2, 43, "Input"],
Cell[182526, 4325, 72, 1, 42, "Output"]
}, Open  ]],
Cell[182613, 4329, 303, 10, 68, "Text",
 Evaluatable->False],
Cell[182919, 4341, 77, 2, 43, "Input"],
Cell[182999, 4345, 340, 12, 68, "Text",
 Evaluatable->False],
Cell[183342, 4359, 77, 2, 43, "Input"],
Cell[183422, 4363, 539, 17, 93, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[183986, 4384, 109, 5, 66, "Input"],
Cell[184098, 4391, 17111, 287, 660, 3788, 67, "CachedBoxData", "BoxData", \
"Print"],
Cell[201212, 4680, 17083, 287, 660, 3788, 67, "CachedBoxData", "BoxData", \
"Print"],
Cell[218298, 4969, 3832, 68, 660, "Print"]
}, Open  ]],
Cell[222145, 5040, 162, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[222332, 5049, 109, 5, 66, "Input"],
Cell[222444, 5056, 17111, 287, 660, 3788, 67, "CachedBoxData", "BoxData", \
"Print"],
Cell[239558, 5345, 17087, 287, 660, 3788, 67, "CachedBoxData", "BoxData", \
"Print"],
Cell[256648, 5634, 3832, 68, 660, "Print"]
}, Open  ]],
Cell[260495, 5705, 177, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[260697, 5714, 116, 5, 66, "Input"],
Cell[260816, 5721, 17111, 287, 660, 3788, 67, "CachedBoxData", "BoxData", \
"Print"],
Cell[277930, 6010, 17087, 287, 660, 3788, 67, "CachedBoxData", "BoxData", \
"Print"],
Cell[295020, 6299, 17099, 287, 660, 3788, 67, "CachedBoxData", "BoxData", \
"Print"],
Cell[312122, 6588, 3832, 68, 660, "Print"]
}, Open  ]],
Cell[315969, 6659, 427, 15, 66, "Text",
 Evaluatable->False],
Cell[316399, 6676, 82, 2, 43, "Input"],
Cell[316484, 6680, 97, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[316606, 6686, 81, 2, 43, "Input"],
Cell[316690, 6690, 707, 14, 42, "Output"]
}, Open  ]],
Cell[317412, 6707, 353, 13, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[317790, 6724, 76, 2, 43, "Input"],
Cell[317869, 6728, 7375, 204, 488, "Output"]
}, Open  ]],
Cell[325259, 6935, 985, 26, 239, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[326269, 6965, 77, 2, 43, "Input"],
Cell[326349, 6969, 110, 1, 32, "Print"]
}, Open  ]],
Cell[326474, 6973, 412, 13, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[326911, 6990, 74, 2, 43, "Input"],
Cell[326988, 6994, 239, 4, 60, "Output"]
}, Open  ]],
Cell[327242, 7001, 786, 25, 117, "Text",
 Evaluatable->False],
Cell[328031, 7028, 80, 2, 43, "Input"],
Cell[328114, 7032, 82, 2, 43, "Input"],
Cell[328199, 7036, 386, 16, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[328610, 7056, 74, 2, 43, "Input"],
Cell[328687, 7060, 221, 4, 42, "Output"]
}, Open  ]],
Cell[328923, 7067, 369, 14, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[329317, 7085, 70, 2, 43, "Input"],
Cell[329390, 7089, 221, 4, 42, "Output"]
}, Open  ]],
Cell[329626, 7096, 325, 8, 90, "Text",
 Evaluatable->False],
Cell[329954, 7106, 462, 11, 188, "Text",
 Evaluatable->False],
Cell[330419, 7119, 487, 16, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[330931, 7139, 165, 9, 156, "Input"],
Cell[331099, 7150, 7375, 204, 488, "Output"]
}, Open  ]],
Cell[338489, 7357, 130, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[338644, 7363, 99, 5, 66, "Input"],
Cell[338746, 7370, 7167, 204, 488, "Output"]
}, Open  ]],
Cell[345928, 7577, 760, 19, 262, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[346713, 7600, 89, 2, 43, "Input"],
Cell[346805, 7604, 780, 14, 535, "Print"]
}, Open  ]],
Cell[347600, 7621, 334, 7, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[347959, 7632, 84, 2, 43, "Input"],
Cell[348046, 7636, 5832, 98, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[353893, 7737, 400, 8, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[354318, 7749, 84, 2, 43, "Input"],
Cell[354405, 7753, 5812, 98, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[360232, 7854, 318, 7, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[360575, 7865, 84, 2, 43, "Input"],
Cell[360662, 7869, 5828, 98, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[366505, 7970, 1049, 29, 311, "Text",
 Evaluatable->False],
Cell[367557, 8001, 177, 7, 43, "Input"],
Cell[367737, 8010, 675, 19, 164, "Text",
 Evaluatable->False],
Cell[368415, 8031, 130, 7, 111, "Input"],
Cell[368548, 8040, 393, 16, 66, "Text",
 Evaluatable->False],
Cell[368944, 8058, 120, 6, 88, "Input"],
Cell[369067, 8066, 728, 23, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[369820, 8093, 113, 5, 66, "Input"],
Cell[369936, 8100, 5845, 98, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[375784, 8200, 5927, 100, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[381714, 8302, 5902, 99, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[387619, 8403, 5878, 99, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[393500, 8504, 797, 14, 535, "Print"]
}, Open  ]],
Cell[394312, 8521, 840, 29, 141, "Text",
 Evaluatable->False],
Cell[395155, 8552, 83, 2, 43, "Input"],
Cell[395241, 8556, 379, 13, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[395645, 8573, 115, 5, 66, "Input"],
Cell[395763, 8580, 5828, 98, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[401594, 8680, 5727, 97, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[407324, 8779, 5906, 100, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[413233, 8881, 5881, 99, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[419117, 8982, 5861, 99, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[424981, 9083, 780, 14, 535, "Print"]
}, Open  ]],
Cell[425776, 9100, 88, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[425889, 9106, 119, 5, 66, "Input"],
Cell[426011, 9113, 5828, 98, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[431842, 9213, 5727, 97, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[437572, 9312, 5723, 97, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[443298, 9411, 5832, 98, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[449133, 9511, 5812, 98, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[454948, 9611, 5735, 97, 535, 736, 13, "CachedBoxData", "BoxData", \
"Print"],
Cell[460686, 9710, 780, 14, 535, "Print"],
Cell[461469, 9726, 780, 14, 535, "Print"]
}, Open  ]],
Cell[462264, 9743, 3605, 101, 541, "Text",
 Evaluatable->False],
Cell[465872, 9846, 111, 5, 66, "Input"],
Cell[465986, 9853, 1963, 55, 531, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[467974, 9912, 82, 2, 43, "Input"],
Cell[468059, 9916, 6982, 136, 218, "Output"]
}, Open  ]],
Cell[475056, 10055, 247, 6, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[475328, 10065, 73, 2, 43, "Input"],
Cell[475404, 10069, 73, 1, 42, "Output"]
}, Open  ]],
Cell[475492, 10073, 669, 17, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[476186, 10094, 77, 2, 43, "Input"],
Cell[476266, 10098, 110, 1, 32, "Print"]
}, Open  ]],
Cell[476391, 10102, 569, 13, 190, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[476985, 10119, 76, 2, 43, "Input"],
Cell[477064, 10123, 14097, 238, 384, "Output"]
}, Open  ]],
Cell[491176, 10364, 312, 7, 115, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[491525, 10376, 82, 2, 105, "Section",
 Evaluatable->False],
Cell[491610, 10380, 489, 20, 90, "Text",
 Evaluatable->False],
Cell[492102, 10402, 2756, 89, 509, "Text",
 Evaluatable->False],
Cell[494861, 10493, 1130, 39, 215, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[496016, 10536, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[496104, 10540, 3620, 137, 384, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[499739, 10680, 332, 11, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[500096, 10695, 235, 11, 178, "Input"],
Cell[500334, 10708, 707, 14, 42, "Output"]
}, Open  ]],
Cell[501056, 10725, 1248, 41, 319, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[502329, 10770, 86, 2, 43, "Input"],
Cell[502418, 10774, 2322, 66, 488, "Output"]
}, Open  ]],
Cell[504755, 10843, 499, 15, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[505279, 10862, 84, 2, 43, "Input"],
Cell[505366, 10866, 1987, 56, 488, "Output"]
}, Open  ]],
Cell[507368, 10925, 545, 12, 213, "Text",
 Evaluatable->False],
Cell[507916, 10939, 742, 21, 115, "Text",
 Evaluatable->False],
Cell[508661, 10962, 1751, 52, 360, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[510437, 11018, 211, 10, 66, "Input"],
Cell[510651, 11030, 282, 6, 42, "Output"],
Cell[510936, 11038, 117, 3, 42, "Output"]
}, Open  ]],
Cell[511068, 11044, 156, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[511249, 11055, 195, 7, 43, "Input"],
Cell[511447, 11064, 94, 2, 42, "Output"]
}, Open  ]],
Cell[511556, 11069, 338, 11, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[511919, 11084, 285, 8, 43, "Input"],
Cell[512207, 11094, 636, 15, 42, "Output"]
}, Open  ]],
Cell[512858, 11112, 326, 9, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[513209, 11125, 254, 8, 43, "Input"],
Cell[513466, 11135, 117, 3, 42, "Output"]
}, Open  ]],
Cell[513598, 11141, 609, 21, 125, "Text",
 Evaluatable->False],
Cell[514210, 11164, 457, 12, 115, "Text",
 Evaluatable->False],
Cell[514670, 11178, 607, 21, 101, "Text",
 Evaluatable->False],
Cell[515280, 11201, 158, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[515463, 11212, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[515551, 11216, 1841, 63, 208, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[517407, 11282, 735, 25, 115, "Text",
 Evaluatable->False],
Cell[518145, 11309, 1200, 41, 223, "Text",
 Evaluatable->False],
Cell[519348, 11352, 1383, 54, 188, "Text",
 Evaluatable->False],
Cell[520734, 11408, 1319, 45, 169, "Text",
 Evaluatable->False],
Cell[522056, 11455, 2253, 85, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[524334, 11544, 179, 10, 178, "Input"],
Cell[524516, 11556, 707, 14, 42, "Output"]
}, Open  ]],
Cell[525238, 11573, 534, 18, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[525797, 11595, 74, 2, 43, "Input"],
Cell[525874, 11599, 282, 6, 42, "Output"]
}, Open  ]],
Cell[526171, 11608, 429, 14, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[526625, 11626, 79, 2, 43, "Input"],
Cell[526707, 11630, 707, 14, 42, "Output"]
}, Open  ]],
Cell[527429, 11647, 747, 23, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[528201, 11674, 241, 14, 268, "Input"],
Cell[528445, 11690, 72, 1, 42, "Output"],
Cell[528520, 11693, 6960, 136, 218, "Output"]
}, Open  ]],
Cell[535495, 11832, 556, 19, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[536076, 11855, 74, 2, 43, "Input"],
Cell[536153, 11859, 532, 11, 42, "Output"]
}, Open  ]],
Cell[536700, 11873, 400, 12, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[537125, 11889, 77, 2, 43, "Input"],
Cell[537205, 11893, 6960, 136, 218, "Output"]
}, Open  ]],
Cell[544180, 12032, 129, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[544334, 12038, 73, 2, 43, "Input"],
Cell[544410, 12042, 73, 1, 42, "Output"]
}, Open  ]],
Cell[544498, 12046, 649, 15, 286, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[545172, 12065, 77, 2, 43, "Input"],
Cell[545252, 12069, 707, 14, 42, "Output"]
}, Open  ]],
Cell[545974, 12086, 1210, 42, 190, "Text",
 Evaluatable->False],
Cell[547187, 12130, 3007, 104, 513, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[550219, 12238, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[550307, 12242, 6314, 213, 636, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[556636, 12458, 1228, 39, 194, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[557889, 12501, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[557977, 12505, 2304, 80, 303, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[560296, 12588, 2363, 73, 470, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[562684, 12665, 76, 2, 43, "Input"],
Cell[562763, 12669, 1563, 30, 68, "Output"]
}, Open  ]],
Cell[564341, 12702, 409, 14, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[564775, 12720, 73, 2, 43, "Input"],
Cell[564851, 12724, 72, 1, 42, "Output"]
}, Open  ]],
Cell[564938, 12728, 323, 11, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[565286, 12743, 85, 2, 43, "Input"],
Cell[565374, 12747, 72, 1, 42, "Output"]
}, Open  ]],
Cell[565461, 12751, 873, 20, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[566359, 12775, 100, 5, 66, "Input"],
Cell[566462, 12782, 247, 5, 42, "Output"]
}, Open  ]],
Cell[566724, 12790, 463, 12, 117, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[567212, 12806, 67, 2, 43, "Input"],
Cell[567282, 12810, 248, 5, 42, "Output"]
}, Open  ]],
Cell[567545, 12818, 2018, 58, 671, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[569588, 12880, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[569676, 12884, 4183, 144, 606, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[573874, 13031, 665, 24, 143, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[574564, 13059, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[574652, 13063, 2977, 106, 360, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[577644, 13172, 470, 17, 92, "Text",
 Evaluatable->False],
Cell[578117, 13191, 30, 0, 43, "Input"],
Cell[578150, 13193, 655, 11, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[578830, 13208, 74, 2, 43, "Input"],
Cell[578907, 13212, 2133, 45, 555, "Print"]
}, Open  ]],
Cell[581055, 13260, 253, 6, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[581333, 13270, 79, 2, 43, "Input"],
Cell[581415, 13274, 2133, 45, 555, "Print"]
}, Open  ]],
Cell[583563, 13322, 127, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[583715, 13328, 79, 2, 43, "Input"],
Cell[583797, 13332, 2133, 45, 555, "Print"]
}, Open  ]],
Cell[585945, 13380, 302, 8, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[586272, 13392, 79, 2, 43, "Input"],
Cell[586354, 13396, 10435, 183, 555, 2089, 44, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[596826, 13584, 79, 2, 43, "Input"],
Cell[596908, 13588, 2133, 45, 555, "Print"]
}, Open  ]],
Cell[599056, 13636, 212, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[599293, 13645, 75, 2, 43, "Input"],
Cell[599371, 13649, 2133, 45, 555, "Print"]
}, Open  ]],
Cell[601519, 13697, 1244, 35, 360, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[602788, 13736, 81, 2, 43, "Input"],
Cell[602872, 13740, 10280, 180, 555, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[613155, 13922, 2133, 45, 555, "Print"]
}, Open  ]],
Cell[615303, 13970, 1337, 32, 478, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[616677, 14007, 210, 7, 105, "Section",
 Evaluatable->False],
Cell[616890, 14016, 161, 6, 43, "Input"],
Cell[617054, 14024, 411, 13, 90, "Text",
 Evaluatable->False],
Cell[617468, 14039, 74, 2, 43, "Input"],
Cell[617545, 14043, 494, 16, 118, "Text",
 Evaluatable->False],
Cell[618042, 14061, 70, 2, 43, "Input"],
Cell[618115, 14065, 870, 30, 141, "Text",
 Evaluatable->False],
Cell[618988, 14097, 42, 1, 43, "Input"],
Cell[619033, 14100, 351, 12, 117, "Text",
 Evaluatable->False],
Cell[619387, 14114, 43, 1, 43, "Input"],
Cell[619433, 14117, 721, 24, 115, "Text",
 Evaluatable->False],
Cell[620157, 14143, 43, 1, 43, "Input"],
Cell[620203, 14146, 476, 11, 188, "Text",
 Evaluatable->False],
Cell[620682, 14159, 43, 1, 43, "Input"],
Cell[620728, 14162, 542, 17, 118, "Text",
 Evaluatable->False],
Cell[621273, 14181, 43, 1, 43, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
