(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    741106,      21689]
NotebookOptionsPosition[    710686,      20686]
NotebookOutlinePosition[    711349,      20712]
CellTagsIndexPosition[    711306,      20709]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Abstract Algebra: An Interactive Approach", "Special2",
 Evaluatable->False,
 CellChangeTimes->{{3.43663419171875*^9, 3.4366342023125*^9}},
 TextAlignment->Center,
 FontFamily->"Arial",
 FontSize->36,
 FontWeight->"Bold",
 FontSlant->"Italic",
 FontColor->RGBColor[1, 0, 1],
 Background->RGBColor[0.500008, 1, 1]],

Cell[CellGroupData[{

Cell["\[Copyright]2009   William Paulsen", "Name",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436634206484375*^9, 3.436634206953125*^9}},
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]],

Cell["\<\
All rights reserved:  No part of these materials may be reproduced, either \
electronically, mechanically, or by any other means or in any form, without \
the written permission from the author. All software is provided \"as is\" \
without warranty of any kind, either expressed or implied, including, but not \
limited to, the implied warranties of merchantability and fitness for a \
particular purpose.\
\>", "Name",
 Editable->False,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]]
}, Closed]],

Cell["\<\
Chapter 6
Building Larger Groups from Smaller Groups \
\>", "Title",
 CellFrame->True,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 TextAlignment->Center,
 FontFamily->"New York",
 FontSize->24,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell["Initialization", "Special2",
 Evaluatable->False,
 TextAlignment->Center,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["<<c:\\math\\group.m", "Input",
 PageWidth->Infinity,
 InitializationCell->True,
 CellChangeTimes->{{3.43663421040625*^9, 3.436634212765625*^9}},
 AspectRatioFixed->True],

Cell[BoxData["\<\"Initialization Done\"\>"], "Print",
 CellChangeTimes->{3.43663421640625*^9, 3.436692484828125*^9, 
  3.43680807184375*^9, 3.43684212290625*^9, 3.44094081482075*^9, 
  3.4415441475577497`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Direct Product", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In this chapter, we will focus on forming larger groups using the smaller \
groups previously studied as building blocks. In this way, many important \
groups can be constructed. In fact, we will be able to construct ",
 StyleBox["all",
  FontSlant->"Italic"],
 " finite abelian groups using only the groups that we have already studied.\n\
\nThe first step in building larger groups is to understand how two groups \
can be combined. Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " "}], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be two groups, and let ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be the set of all ordered pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", "k"}], ")"}], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "\[Element]", " ", "H", " "}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Element]", " ", "K"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.4226474685625*^9, {3.4366342273125*^9, 3.436634230203125*^9}}],

Cell[TextData[{
 "We need to describe how to multiply two ordered pairs together. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["h", "1"], ",", 
     SubscriptBox["k", "1"]}], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["h", "2"], ",", 
     SubscriptBox["k", "2"]}], ")"}], TraditionalForm]]],
 " are two ordered pairs in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", the product could be defined in the natural way:\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["h", "1"], ",", 
       SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["h", "2"], ",", 
       SubscriptBox["k", "2"]}], ")"}]}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["h", "1"], "\[CenterDot]", 
       SubscriptBox["h", "2"]}], ",", 
      RowBox[{
       SubscriptBox["k", "1"], "\[CenterDot]", 
       SubscriptBox["k", "2"]}]}], ")"}]}], TraditionalForm]]],
 ".\n \nIt is clear that the associate law would hold. If ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "1"], TraditionalForm]]],
 " is the identity element for ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "2"], TraditionalForm]]],
 " the identity element for ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["e", "1"], ",", 
     SubscriptBox["e", "2"]}], ")"}], TraditionalForm]]],
 " would be the identity element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nFinally we can define the inverse of an ordered pair \n",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", "k"}], ")"}], TraditionalForm]]],
 " to be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["h", 
      RowBox[{"-", "1"}]], ",", 
     SuperscriptBox["k", 
      RowBox[{"-", "1"}]]}], ")"}], TraditionalForm]]],
 ". This is so\n \n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"h", ",", "k"}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["h", 
        RowBox[{"-", "1"}]], ",", 
       SuperscriptBox["k", 
        RowBox[{"-", "1"}]]}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["h", 
         RowBox[{"-", "1"}]], ",", 
        SuperscriptBox["k", 
         RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"h", ",", "k"}], ")"}]}], " ", "=", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["e", "1"], ",", 
       SubscriptBox["e", "2"]}], ")"}]}]}], TraditionalForm]]],
 ".\n    \nSince the four properties of groups hold for the set ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of ordered pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", " ", "k"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " forms a group.  \n\nDEFINITION 6.1:\nGiven two groups ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", the ",
 StyleBox["direct product",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Times]", " ", "K"}], TraditionalForm]]],
 ", is the group of ordered pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", " ", "k"}], ")"}], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "\[Epsilon]", " ", "H", " "}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Epsilon]", " ", "K"}], TraditionalForm]]],
 ", with multiplication defined by\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["h", "1"], ",", 
       SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["h", "2"], ",", 
       SubscriptBox["k", "2"]}], ")"}]}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["h", "1"], "\[CenterDot]", 
       SubscriptBox["h", "2"]}], ",", 
      RowBox[{
       SubscriptBox["k", "1"], "\[CenterDot]", 
       SubscriptBox["k", "2"]}]}], ")"}]}], TraditionalForm]]],
 ".\n\nExample 6.1:\nLet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    FormBox[
     SubscriptBox["Z", "4"],
     TraditionalForm]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", " ", "=", " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 ". Consider the direct product ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     SubscriptBox["Z", "4"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"]}]}], TraditionalForm]]],
 ".  Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "4"], TraditionalForm]]],
 " consists of the elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "2", ",", "3"}], "}"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 " consists of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", " ", "1"}], "}"}], TraditionalForm]]],
 ", the set of all ordered pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", "k"}], ")"}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "4"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 " is\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0", ",", "0"}], ")"}], ",", " ", 
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}], ",", " ", 
       RowBox[{"(", 
        RowBox[{"1", ",", "0"}], ")"}], ",", " ", 
       RowBox[{"(", 
        RowBox[{"1", ",", "1"}], ")"}], ",", " ", 
       RowBox[{"(", 
        RowBox[{"2", ",", "0"}], ")"}], ",", " ", 
       RowBox[{"(", 
        RowBox[{"2", ",", "1"}], ")"}], ",", " ", 
       RowBox[{"(", 
        RowBox[{"3", ",", "0"}], ")"}], ",", " ", 
       RowBox[{"(", 
        RowBox[{"3", ",", "1"}], ")"}]}], "}"}], "."}]}], TraditionalForm]]],
 "\n\nThus, we will have a group of order 8. Multiplication is performed \
component-wise in the two groups. The multiplication table for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], " ", "\[Times]", " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 " is given by executing the following group of commands:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436636447390625*^9, 3.43663647459375*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[0];
{x_,y_}.{z_,w_} := {Mod[x+z,4],Mod[y+w,2]}
MultTable[
{{0,0},{0,1},{1,0},{1,1},{2,0},{2,1},{3,0},{3,1}}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.44267347038325*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0, 1}, {1, 
      1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}}, {{
      1, 0.6, 0}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6, 0.4, 0.8}, {0.6, 0.4,
       0.8}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0, 1}, {1, 1, 0}, {
      1, 0, 0}, {0, 1, 1}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7,
       0.8, 0.7}, {1, 0.6, 0}, {1, 0, 1}, {1, 1, 0}}, {{0, 1, 1}, {0, 1, 1}, {
      1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 
      1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 
      1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}}, {{1, 1, 
      0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {
      0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8,
       0.7}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"{0,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{0,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{1,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{1,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{2,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{2,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{3,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{3,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"{0,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{0,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{1,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{1,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{2,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{2,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{3,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{3,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.442673475727*^9}]
}, Open  ]],

Cell[TextData[{
 "(In order to get ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to give a multiplication table, we have to use curly braces instead of \
parathesis.)\n\nRecall that in chapter 4, we found that there were only 5 \
non-isomorphic groups of order 8: ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "4"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ".  Hence, this group must be isomorphic to one of these groups.  Notice \
that this group is commutative, it is neither ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "4"], TraditionalForm]]],
 ". There is an element of order 4, namely ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"0", ",", "1"}], ")"}], TraditionalForm]]],
 ", but there are no elements of order 8. Hence this group cannot be either \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 " is the only remaining possibility, this group must be isomorphic to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubsuperscriptBox["Z", "15", "*"],
     TraditionalForm], " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
     "1", ",", " ", "2", ",", " ", "4", ",", " ", "7", ",", " ", "8", ",", 
      " ", "11", ",", " ", "13", ",", " ", "14"}], "}"}]}], 
   TraditionalForm]]],
 ".\n\nEXPERIMENT:\nCompare the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], "\[Times]", " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 " given by the table"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[0];
{x_,y_}.{z_,w_} := {Mod[x+z,4],Mod[y+w,2]}
MultTable[
{{0,0},{0,1},{1,0},{1,1},{2,0},{2,1},{3,0},{3,1}}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.442673491148875*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0, 1}, {1, 
      1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}}, {{
      1, 0.6, 0}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6, 0.4, 0.8}, {0.6, 0.4,
       0.8}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0, 1}, {1, 1, 0}, {
      1, 0, 0}, {0, 1, 1}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7,
       0.8, 0.7}, {1, 0.6, 0}, {1, 0, 1}, {1, 1, 0}}, {{0, 1, 1}, {0, 1, 1}, {
      1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 
      1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 
      1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}}, {{1, 1, 
      0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {
      0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8,
       0.7}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"{0,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{0,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{1,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{1,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{2,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{2,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{3,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{3,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"{0,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{0,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{1,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{1,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{2,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{2,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{3,0}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{3,1}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{3,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{3,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{1,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,1}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2,0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.442673492367625*^9}]
}, Open  ]],

Cell[TextData[{
 "with the table for ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 ", generated by the commands"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
DefMultMod[15];
MultTable[{1,2,4,7,8,11,13,14}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.442673499023875*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 
      0.8}, {0, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}}, {{
      1, 0.6, 0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {1, 0, 0}, {1, 1, 0}, {0.7, 
      0.8, 0.7}, {0, 1, 0}, {0, 1, 1}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 
      0.4, 0.8}, {1, 0, 0}, {0.7, 0.8, 0.7}, {1, 0, 1}, {1, 0.6, 0}, {1, 1, 
      0}, {0, 1, 0}, {0, 1, 1}}, {{0, 1, 0}, {0, 1, 0}, {1, 1, 0}, {1, 0, 
      1}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 0, 
      0}}, {{1, 0, 0}, {1, 0, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0, 1, 1}, {
      0.6, 0.4, 0.8}, {1, 0, 1}, {1, 1, 0}, {0, 1, 0}}, {{0, 1, 1}, {0, 1, 
      1}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {1, 0, 1}, {0.7, 0.8, 0.7}, {1, 
      0, 0}, {0.6, 0.4, 0.8}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {0, 1, 0}, {
      0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{
      1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 
      1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7,
       0.8, 0.7}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.44267349994575*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "Can you, by rearranging the elements in the last MultTable command, get the \
color pattern of this table to resemble the pattern for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], "\[Times]", " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 "?  Hint: Which elements of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 " corresponds to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"2", ",", "0"}], ")"}], TraditionalForm]]],
 "? What are the other elements of order 2?\n\nIn this example, we saw that \
the direct product of two commutative groups turned out to be commutative. \
This is, in fact, what generally happens.\n\nPROPOSITION 6.1\nLet ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be two groups. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Times]", " ", "K"}], TraditionalForm]]],
 " is commutative if, and only if, both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are commutative."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436636527671875*^9, 3.43663652775*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First, suppose that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are both commutative. Then for two elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["h", "1"], ",", 
      SubscriptBox["k", "1"]}], ")"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["h", "2"], ",", 
     SubscriptBox["k", "2"]}], ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Times]", " ", "K"}], TraditionalForm]]],
 ", we have\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["h", "1"], ",", 
       SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["h", "2"], ",", 
       SubscriptBox["k", "2"]}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["h", "1"], "\[CenterDot]", 
        SubscriptBox["h", "2"]}], ",", 
       RowBox[{
        SubscriptBox["k", "1"], "\[CenterDot]", 
        SubscriptBox["k", "2"]}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["h", "2"], "\[CenterDot]", 
         SubscriptBox["h", "1"]}], ",", 
        RowBox[{
         SubscriptBox["k", "2"], "\[CenterDot]", 
         SubscriptBox["k", "1"]}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "2"], ",", 
         SubscriptBox["k", "2"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "1"], ",", 
         SubscriptBox["k", "1"]}], ")"}]}]}]}]}], TraditionalForm]]],
 ".\n\nSo the two elements in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Times]", " ", "K"}], TraditionalForm]]],
 " commute. Hence, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Times]", " ", "K"}], TraditionalForm]]],
 " is commutative.\n\nNow suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Times]", " ", "K"}], TraditionalForm]]],
 " is commutative. If we let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "2"], TraditionalForm]]],
 " be the identity elements of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", respectively, then we have\n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["h", "1"], "\[CenterDot]", 
        SubscriptBox["h", "2"]}], ",", 
       SubscriptBox["e", "2"]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "1"], ",", 
         SubscriptBox["e", "2"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "2"], ",", 
         SubscriptBox["e", "2"]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["h", "2"], ",", 
          SubscriptBox["e", "2"]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["h", "1"], ",", 
          SubscriptBox["e", "2"]}], ")"}]}], " ", "=", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "2"], "\[CenterDot]", 
          SubscriptBox["h", "1"]}], ",", 
         SubscriptBox["e", "2"]}], ")"}]}]}]}]}], TraditionalForm]]],
 "\nand\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["e", "1"], ",", 
       RowBox[{
        SubscriptBox["k", "1"], "\[CenterDot]", 
        SubscriptBox["k", "2"]}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["e", "1"], ",", 
         SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["e", "1"], ",", 
         SubscriptBox["k", "2"]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["e", "1"], ",", 
          SubscriptBox["k", "2"]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["e", "1"], ",", 
          SubscriptBox["k", "2"]}], ")"}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["e", "1"], ",", 
          RowBox[{
           SubscriptBox["k", "2"], "\[CenterDot]", 
           SubscriptBox["k", "1"]}]}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1"], "\[CenterDot]", 
     SubscriptBox["h", "2"]}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["h", "2"], "\[CenterDot]", 
     SubscriptBox["h", "1"]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["k", "1"], "\[CenterDot]", 
     SubscriptBox["k", "2"]}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["k", "2"], "\[CenterDot]", 
     SubscriptBox["k", "1"]}]}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["h", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["h", "2"], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and all ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "2"], TraditionalForm]]],
 "  in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".  Hence, both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are commutative."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "It is easy to find the number of elements in a direct product. If ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " has order ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " has order ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", then the number of ordered pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", "k"}], ")"}], TraditionalForm]]],
 " would be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[CenterDot]", "m"}], TraditionalForm]]],
 ".  \n\nWe can generalize the direct product to a set of more than two \
groups. Let\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "1"], ",", 
    SubscriptBox["G", "2"], ",", 
    SubscriptBox["G", "3"], ",", "\[Ellipsis]", ",", " ", 
    SubscriptBox["G", "n"]}], TraditionalForm]]],
 "\n\nbe a collection of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " groups. Then we define\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "1"], "\[Times]", " ", 
    SubscriptBox["G", "2"], "\[Times]", " ", 
    SubscriptBox["G", "3"], "\[Times]", " ", "\[Ellipsis]", " ", "\[Times]", 
    " ", 
    SubscriptBox["G", "n"]}], TraditionalForm]]],
 "\n \nto be the set of ordered ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-tuples ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["g", "1"], ",", 
     SubscriptBox["g", "2"], ",", 
     SubscriptBox["g", "3"], ",", "\[Ellipsis]", " ", ",", 
     SubscriptBox["g", "n"]}], ")"}], TraditionalForm]]],
 " with multiplication defined by\n\n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["g", "1"], ",", 
       SubscriptBox["g", "2"], ",", "\[Ellipsis]", " ", ",", 
       SubscriptBox["g", "n"]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["h", "1"], ",", 
       SubscriptBox["h", "2"], ",", "\[Ellipsis]", " ", ",", 
       SubscriptBox["h", "n"]}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["g", "1"], "\[CenterDot]", 
        SubscriptBox["h", "1"]}], ",", 
       RowBox[{
        SubscriptBox["g", "2"], "\[CenterDot]", 
        SubscriptBox["h", "2"]}], ",", "\[Ellipsis]", " ", ",", 
       RowBox[{
        SubscriptBox["g", "n"], "\[CenterDot]", 
        SubscriptBox["h", "n"]}]}], ")"}], "."}]}], TraditionalForm]]],
 "\n\nThe direct product of more than two groups can be defined by taking the \
direct product of direct products. That is, given three groups ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", we can define\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"G", " ", "\[Times]", " ", "H"}], ")"}], " ", "\[Times]", " ", 
    "K"}], TraditionalForm]]],
 "   and   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "\[Times]", " ", 
    RowBox[{"(", 
     RowBox[{"H", " ", "\[Times]", " ", "K"}], ")"}]}], TraditionalForm]]],
 "\n\nusing only the definition for the direct product of two groups. But it \
is trivial to see that the mappings\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", " ", ":", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"G", " ", "\[Times]", " ", "H"}], ")"}], " ", "\[Times]", " ", 
      "K"}], " ", "\[RightArrow]", " ", 
     RowBox[{"G", " ", "\[Times]", " ", "H", " ", "\[Times]", " ", "K"}]}]}], 
   TraditionalForm]]],
 "\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", " ", ":", " ", 
    RowBox[{
     RowBox[{"G", " ", "\[Times]", " ", 
      RowBox[{"(", 
       RowBox[{"H", " ", "\[Times]", " ", "K"}], ")"}]}], " ", 
     "\[RightArrow]", " ", 
     RowBox[{"G", " ", "\[Times]", " ", "H", " ", "\[Times]", " ", "K"}]}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "given by\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"g", ",", "h"}], ")"}], ",", "k"}], ")"}], ")"}], " ", "=", 
    " ", 
    RowBox[{"(", 
     RowBox[{"g", ",", "h", ",", "k"}], ")"}]}], TraditionalForm]]],
 "\nand                          \n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"(", 
      RowBox[{"g", ",", 
       RowBox[{"(", 
        RowBox[{"h", ",", "k"}], ")"}]}], ")"}], ")"}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"g", ",", "h", ",", "k"}], ")"}]}], TraditionalForm]]],
 "\n\nare both surjective isomorphisms. Thus, \n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"G", " ", "\[Times]", " ", "H"}], ")"}], " ", "\[Times]", " ", 
     "K"}], "  ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], "  ", 
    RowBox[{"G", " ", "\[Times]", " ", "H", " ", "\[Times]", " ", "K"}], "  ",
     
    StyleBox["\[TildeTilde]",
     FontSize->16], "  ", 
    RowBox[{"G", " ", "\[Times]", " ", 
     RowBox[{"(", 
      RowBox[{"H", " ", "\[Times]", " ", "K"}], ")"}]}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "It also should be noted that we have the natural mapping\n                \n\
                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", " ", ":", " ", 
    RowBox[{
     RowBox[{"H", " ", "\[Times]", " ", "K"}], "  ", "\[RightArrow]", "  ", 
     RowBox[{"K", " ", "\[Times]", " ", "H"}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "given by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", 
     RowBox[{"(", 
      RowBox[{"h", ",", "k"}], ")"}], ")"}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"k", ",", "h"}], ")"}]}], TraditionalForm]]],
 ". This is clearly one-to one and onto, and is a homomorphism. Thus,\n \n    \
                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", " ", "\[Times]", " ", "K"}], "  ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], "  ", 
    RowBox[{"K", " ", "\[Times]", " ", "H"}]}], TraditionalForm]]],
 ".\n                      \nThis shows that the direct product between \
groups is a commutative operation, as well as associative. This suggests that \
some groups may be able to be expressed as a direct product of two or more \
smaller groups. If this is the case, then the order in which the smaller \
groups are combined would be irrelevant.\n\nDEFINITION 6.2:\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group. We say that ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has a ",
 StyleBox["decomposition",
  FontSlant->"Italic"],
 " if\n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "  ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], "  ", 
    RowBox[{"H", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 ",\n                        \nwhere neither ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is the trivial group. \n\nFor example, we saw above that ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 " had a decomposition, namely ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], "\[Times]", " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 ".   \n\nEXPERIMENT:\nDoes the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " have a decomposition into a direct product? Why or why not?\n\nWe would \
like to find a way of testing whether a general group can be decomposed into \
smaller groups. In the case of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ", we could use the fact that all smaller groups are abelian, along with \
proposition 6.1 to show that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " cannot have a decomposition. But for other groups, we\nproblem is more \
difficult. The following theorem gives us a way to test to see whether a \
given group has a decomposition:\n\nTHEOREM 6.1:  THE DIRECT PRODUCT THEOREM\n\
Let ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group with identity ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be two subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Suppose the following two statements are true:\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"1", ")"}], "  ", "H"}], " ", "\[Intersection]", " ", "K"}], 
    " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 ".\n     2)  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"h", "\[CenterDot]", "k"}], " ", "=", " ", 
    RowBox[{"k", "\[CenterDot]", "h"}]}], TraditionalForm]]],
 "   for all  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "\[Element]", " ", "H", " "}], TraditionalForm]]],
 " and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "\[Element]", " ", "K"}], TraditionalForm]]],
 ".\n     \nThen ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", " ", "\[CenterDot]", " ", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"H", " ", "\[Times]", " ", 
     RowBox[{"K", "."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436636581859375*^9, 3.43663660003125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First, let us show that every element in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " can be ",
 StyleBox["uniquely",
  FontSlant->"Italic"],
 " written in the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "k"}], TraditionalForm]]],
 ", where  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "\[Epsilon]", " ", "H"}], TraditionalForm]]],
 "   and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Epsilon]", " ", "K"}], TraditionalForm]]],
 ". Suppose that \n                                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1"], "\[CenterDot]", 
     SubscriptBox["k", "1"]}], "=", 
    RowBox[{
     SubscriptBox["h", "2"], "\[CenterDot]", 
     SubscriptBox["k", "2."]}]}], TraditionalForm]]],
 "\n\nThen ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubsuperscriptBox["h", "2", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SubscriptBox["h", "1"]}], "=", " ", 
    RowBox[{
     SubscriptBox["k", "2"], "\[CenterDot]", 
     SubsuperscriptBox["k", "1", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ".  Since this element must be in both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", and the intersection of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is the identity element, we have that \n\n                           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubsuperscriptBox["h", "2", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SubscriptBox["h", "1"]}], "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["k", "2"], "\[CenterDot]", 
      SubsuperscriptBox["k", "1", 
       RowBox[{"-", "1"}]]}], " ", "=", " ", 
     RowBox[{"e", "."}]}]}], TraditionalForm]]],
 "\n         \nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["h", "1"], "=", 
    SubscriptBox["h", "2"]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["k", "1"], "=", 
    SubscriptBox["k", "2"]}], TraditionalForm]]],
 ".  Therefore, every element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " can be written uniquely as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "k"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".\n\nNext, we need to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " is a group.  Since\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"h", "\[CenterDot]", "k"}], " ", "=", " ", 
    RowBox[{"k", "\[CenterDot]", "h"}]}], TraditionalForm]]],
 "   for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "\[Epsilon]", " ", "H"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Epsilon]", " ", "K"}], TraditionalForm]]],
 ",\n             \nwe have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", "=", " ", 
    RowBox[{"K", "\[CenterDot]", "H"}]}], TraditionalForm]]],
 ".  Thus, by lemma 4.4, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".  \n\nWe can now define a mapping\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", " ", ":", " ", 
    RowBox[{
     RowBox[{"H", "\[CenterDot]", "K"}], " ", "\[RightArrow]", " ", 
     RowBox[{"H", " ", "\[Times]", " ", "K"}]}]}], TraditionalForm]]],
 "\n                  \nby ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"h", ",", "k"}], ")"}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " are the unique elements such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "\[Epsilon]", " ", "H"}], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Epsilon]", " ", "K"}], TraditionalForm]]],
 ",  and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", " ", "=", " ", 
    RowBox[{"h", "\[CenterDot]", "k"}]}], TraditionalForm]]],
 ". It is clear that ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is one-to-one, since the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", "k"}], ")"}], TraditionalForm]]],
 " can only have come from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "k"}], TraditionalForm]]],
 ".  Also, ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is onto, for the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "k"}], TraditionalForm]]],
 " maps to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", "k"}], ")"}], TraditionalForm]]],
 ". All that remains to show that \[Phi] is an isomorphism is that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", 
     RowBox[{"x", "\[CenterDot]", "y"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Phi]", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"\[Phi]", "(", "y", ")"}]}]}], TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "=", " ", 
     RowBox[{
      SubscriptBox["h", "1"], "\[CenterDot]", 
      SubscriptBox["k", "1"]}]}], ",", " ", 
    RowBox[{
     RowBox[{"and", " ", "y"}], " ", "=", " ", 
     RowBox[{
      SubscriptBox["h", "2"], "\[CenterDot]", 
      SubscriptBox["k", "2"]}]}]}], TraditionalForm]]],
 ".  Then\n\n   ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"\[Phi]", "(", 
       RowBox[{
        SubscriptBox["h", "1"], "\[CenterDot]", 
        SubscriptBox["k", "1"], "\[CenterDot]", 
        SubscriptBox["h", "2"], "\[CenterDot]", 
        SubscriptBox["k", "2"]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"\[Phi]", "(", 
        RowBox[{
         SubscriptBox["h", "1"], "\[CenterDot]", 
         SubscriptBox["h", "2"], "\[CenterDot]", 
         SubscriptBox["k", "2"], "\[CenterDot]", 
         SubscriptBox["k", "2"]}], ")"}], " ", "=", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"]}], ",", 
         RowBox[{
          SubscriptBox["k", "1"], "\[CenterDot]", 
          SubscriptBox["k", "2"]}]}], ")"}]}]}]}], " "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "1"], ",", 
         SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "2"], ",", 
         SubscriptBox["k", "2"]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"\[Phi]", "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{
        RowBox[{"\[Phi]", "(", "y", ")"}], ".", "   "}]}]}]}]}], 
   TraditionalForm]}]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is an isomorphism, and so  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"H", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43663667990625*^9, 3.436636681140625*^9}, {
  3.4415440435733747`*^9, 3.4415440905889997`*^9}}]
}, Open  ]],

Cell[TextData[{
 "We can use this theorem to find the direct product of two groups in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Suppose we want to generate the direct product ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "3"], "\[Times]", " ", 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 ".  We first must define the two groups in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using the same identity and different letters for the generators. This \
way, the identity will be the only element in common between the two groups.\n\
\nLet us define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^3, e]; Define[b^2, e]
Define[b.a, a.a.b]
Define[1/a, a^3]; Define[1/b, b]
H = Group[{a, b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43663670846875*^9, 3.436636722859375*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.42264760384375*^9, 3.4415441476514997`*^9, 
  3.44267351107075*^9}]
}, Open  ]],

Cell[TextData[{
 "Now let us define ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ", using ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " for the two generators."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
Define[c^2, e]; Define[d^2, e]
Define[d.c, c.d]
Define[1/c, c]; Define[1/d, d]
K = Group[{c, d}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43663680778125*^9, 3.436636812015625*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "c", ",", "d", ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`d],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4226476079375*^9, 3.4415441504327497`*^9, 
  3.442673512492625*^9}]
}, Open  ]],

Cell[TextData[{
 "Of course we did not use the ",
 StyleBox["InitGroup",
  FontWeight->"Bold"],
 " command before defining the second group because this would clear the \
first group!\n\nBy using e as the identity for both groups, we have that the \
intersection of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Intersection[H, K]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", "e", "}"}]], "Output",
 CellChangeTimes->{3.422647612453125*^9, 3.4415441647921247`*^9, 
  3.4426735139145*^9}]
}, Open  ]],

Cell[TextData[{
 "so the first condition of the direct product theorem is satisfied.\n\nIn \
order for the second condition of the direct product theorem to be satisfied, \
every element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " must commute with every element of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". This will be true as long as all of the ",
 StyleBox["generators",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " commute with all of the ",
 StyleBox["generators ",
  FontSlant->"Italic"],
 "of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Since there are 2 generators of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and 2 of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", we can tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that the generators commute using 2\[CenterDot]2 = 4 definitions:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Define[c.a, a.c]; Define[c.b, b.c]
Define[d.a, a.d]; Define[d.b, b.d]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436636822125*^9, 3.436636825125*^9}},
 AspectRatioFixed->True],

Cell[TextData[{
 "We have to be consistent in the direction of these definitions. That is, we \
must define an element of the  form  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[CenterDot]", "h"}], TraditionalForm]]],
 "  to  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "k"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". This informs ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to express all elements as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "k"}], TraditionalForm]]],
 ".\n\nAccording to the direct product theorem ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " is now the same as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Times]", " ", "K"}], TraditionalForm]]],
 ". Here, then, is the direct product:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["H.K", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`c], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`c], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`b], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`c], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`a], $CellContext`b], $CellContext`c], \
$CellContext`d],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.42264761965625*^9, 3.4415441706827497`*^9, 
  3.442673517852*^9}]
}, Open  ]],

Cell["\<\
Alternatively, we could find the smallest group that contains all of the \
generators:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = Group[{a,b,c,d}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", "c", ",", "d", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`c], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`c], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`b], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`c], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`d],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(d\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`a], $CellContext`b], $CellContext`c], \
$CellContext`d],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.422647623703125*^9, 3.4415441733546247`*^9, 
  3.44267351969575*^9}]
}, Open  ]],

Cell["We can check that the order of this direct product is", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["24"], "Output",
 CellChangeTimes->{3.422647636359375*^9, 3.4415441796358747`*^9, 
  3.4426735214145*^9}]
}, Open  ]],

Cell[TextData[{
 "as expected, since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " is of order 6, and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " is of order 4. The multiplication table of this new group is given as"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.2, 0.6, 0.4}, {0.2, 0.6, 0.4}, {1, 0.8, 0.4}, {
      0.4, 0.2, 1}, {1, 0.8, 0.7}, {0, 0.6, 1}, {0.6, 0.4, 0.4}, {0, 0, 1}, {
      0.7, 1, 0}, {1, 0.6, 1}, {0.2, 0.8, 0.7}, {1, 0.4, 0.4}, {0.6, 0.6, 
      0.2}, {0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.6, 
      0.5}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 
      0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}}, {{1, 0.8, 0.4}, {1, 0.8, 0.4}, {
      0.6, 0.4, 0.4}, {0, 0, 1}, {0.7, 1, 0}, {1, 0.6, 1}, {0.2, 0.6, 0.4}, {
      0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {0.7,
       0.8, 0.7}, {1, 0.6, 0}, {0.4, 0.2, 1}, {1, 0.8, 0.7}, {0, 0.6, 1}, {0, 
      1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {0.2, 0.8, 0.7}, {
      1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {1, 1, 0}, {0.6, 0.4, 0.8}}, {{
      0.4, 0.2, 1}, {0.4, 0.2, 1}, {0.8, 0.6, 0.2}, {0.2, 0.6, 0.4}, {0.2, 
      0.8, 0.7}, {1, 0.4, 0.4}, {0, 0, 1}, {0.6, 0.4, 0.4}, {0, 1, 0}, {1, 0, 
      0}, {1, 0.8, 0.7}, {0, 0.6, 1}, {0.6, 0.4, 0.8}, {1, 0.8, 0.4}, {0.8, 
      0.6, 0.5}, {0.7, 0.8, 0.7}, {0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {0.6, 0.6, 0.2}, {0.7, 1, 0}, {1, 0.6, 1}, {1, 0, 1}, {0, 1, 1}, {
      1, 0.6, 0}}, {{1, 0.8, 0.7}, {1, 0.8, 0.7}, {0.7, 1, 0}, {0.2, 0.8, 
      0.7}, {0.2, 0.6, 0.4}, {0.6, 0.6, 0.2}, {0.8, 0.2, 0.5}, {0.8, 0.6, 
      0.5}, {1, 0.8, 0.4}, {1, 0.6, 0}, {0.4, 0.2, 1}, {0.6, 0.4, 0.8}, {
      0, 0.6, 1}, {0, 1, 0}, {0.6, 0.4, 0.4}, {0, 1, 1}, {0, 0, 1}, {1, 0, 
      1}, {1, 0.6, 1}, {1, 0.4, 0.4}, {0.8, 0.6, 0.2}, {1, 1, 0}, {0.5, 0.6, 
      0.8}, {0.7, 0.8, 0.7}, {1, 0, 0}}, {{0, 0.6, 1}, {0, 0.6, 1}, {
      1, 0.6, 1}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {0.2, 0.6, 0.4}, {0.5, 0.6, 
      0.8}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0.8, 0.4}, {0.6, 0.4, 0.8}, {
      0.4, 0.2, 1}, {1, 0.8, 0.7}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.4}, {1, 
      0, 1}, {0, 0, 1}, {0.7, 1, 0}, {0.2, 0.8, 0.7}, {1, 1, 0}, {0.8, 0.6, 
      0.2}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.5}, {0, 1, 0}}, {{0.6, 0.4, 0.4}, {
      0.6, 0.4, 0.4}, {0.2, 0.6, 0.4}, {0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}, {0.5,
       0.6, 0.8}, {1, 0.8, 0.4}, {0.4, 0.2, 1}, {1, 0.8, 0.7}, {0, 0.6, 1}, {
      0, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0, 0, 1}, {0.7, 1, 0}, {1, 0.6, 1}, {
      0.2, 0.8, 0.7}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {1, 1, 0}, {0.8, 0.6, 
      0.5}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {1, 0, 1}}, {{0, 0,
       1}, {0, 0, 1}, {0.4, 0.2, 1}, {1, 0.8, 0.4}, {0.8, 0.6, 0.5}, {0.7, 
      0.8, 0.7}, {0.8, 0.6, 0.2}, {0.2, 0.6, 0.4}, {0.2, 0.8, 0.7}, {
      1, 0.4, 0.4}, {0.7, 1, 0}, {1, 0.6, 1}, {1, 0, 1}, {0.6, 0.4, 0.4}, {0, 
      1, 0}, {1, 0, 0}, {1, 0.8, 0.7}, {0, 0.6, 1}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {0.6, 0.6, 
      0.2}, {0, 1, 1}}, {{0.7, 1, 0}, {0.7, 1, 0}, {0.8, 0.2, 0.5}, {0.8, 0.6,
       0.5}, {1, 0.8, 0.4}, {1, 0.6, 0}, {1, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 
      0.4}, {0, 1, 1}, {0, 0, 1}, {1, 0, 1}, {1, 0.6, 1}, {0.2, 0.8, 0.7}, {
      0.2, 0.6, 0.4}, {0.6, 0.6, 0.2}, {0.8, 0.6, 0.2}, {1, 1, 0}, {0.5, 0.6, 
      0.8}, {0.7, 0.8, 0.7}, {0.4, 0.2, 1}, {0.6, 0.4, 0.8}, {0, 0.6, 1}, {1, 
      0, 0}, {1, 0.4, 0.4}}, {{1, 0.6, 1}, {1, 0.6, 1}, {0.5, 0.6, 0.8}, {0.7,
       0.8, 0.7}, {1, 0.6, 0}, {1, 0.8, 0.4}, {0, 0.6, 1}, {1, 0, 0}, {0, 1, 
      1}, {0.6, 0.4, 0.4}, {1, 0, 1}, {0, 0, 1}, {0.7, 1, 0}, {1, 0.4, 0.4}, {
      0.6, 0.6, 0.2}, {0.2, 0.6, 0.4}, {1, 1, 0}, {0.8, 0.6, 0.2}, {0.8, 0.2, 
      0.5}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {0.4, 0.2, 1}, {1, 0.8, 0.7}, {
      0, 1, 0}, {0.2, 0.8, 0.7}}, {{0.2, 0.8, 0.7}, {0.2, 0.8, 0.7}, {0, 1, 
      0}, {1, 0.8, 0.7}, {0.4, 0.2, 1}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {
      0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {1, 1, 0}, {0.2, 0.6, 0.4}, {0.6, 0.6, 
      0.2}, {1, 0.4, 0.4}, {0.7, 1, 0}, {0, 0, 1}, {1, 0, 1}, {0.6, 0.4, 
      0.4}, {0, 1, 1}, {1, 0, 0}, {0, 0.6, 1}, {1, 0.8, 0.4}, {1, 0.6, 0}, {
      0.7, 0.8, 0.7}, {0.5, 0.6, 0.8}, {1, 0.6, 1}}, {{1, 0.4, 0.4}, {
      1, 0.4, 0.4}, {1, 0, 0}, {0, 0.6, 1}, {0.6, 0.4, 0.8}, {0.4, 0.2, 1}, {
      0.7, 0.8, 0.7}, {0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 0.2}, {0.6, 0.6, 
      0.2}, {0.2, 0.6, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}, {1, 0, 1}, {0, 0, 
      1}, {0, 1, 1}, {0.6, 0.4, 0.4}, {0, 1, 0}, {1, 0.8, 0.7}, {1, 0.6, 0}, {
      1, 0.8, 0.4}, {0.8, 0.6, 0.5}, {0.8, 0.2, 0.5}, {0.7, 1, 0}}, {{0.6, 
      0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0, 0.6, 1}, {
      1, 0.8, 0.7}, {0, 1, 1}, {1, 0, 1}, {1, 0.6, 1}, {0.7, 1, 0}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {0.2, 0.6, 0.4}, {1, 1, 0}, {0.5, 0.6, 
      0.8}, {0.8, 0.2, 0.5}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      1, 0.8, 0.4}, {0.4, 0.2, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.4}, {0, 
      0, 1}, {0.8, 0.6, 0.2}}, {{0.8, 0.6, 0.2}, {0.8, 0.6, 0.2}, {0, 0, 1}, {
      0.6, 0.4, 0.4}, {0, 1, 0}, {1, 0, 0}, {0.4, 0.2, 1}, {1, 0.8, 0.4}, {
      0.8, 0.6, 0.5}, {0.7, 0.8, 0.7}, {0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {1, 
      1, 0}, {0.2, 0.6, 0.4}, {0.2, 0.8, 0.7}, {1, 0.4, 0.4}, {0.7, 1, 0}, {
      1, 0.6, 1}, {1, 0, 1}, {0, 1, 1}, {1, 0.8, 0.7}, {0, 0.6, 1}, {0.6, 0.4,
       0.8}, {1, 0.6, 0}, {0.6, 0.6, 0.2}}, {{0.8, 0.2, 0.5}, {0.8, 0.2, 
      0.5}, {1, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 0.4}, {0, 1, 1}, {
      0.7, 1, 0}, {0.2, 0.8, 0.7}, {0.2, 0.6, 0.4}, {0.6, 0.6, 0.2}, {0.8, 
      0.6, 0.2}, {1, 1, 0}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {1, 0.8, 0.4}, {
      1, 0.6, 0}, {0.4, 0.2, 1}, {0.6, 0.4, 0.8}, {0, 0.6, 1}, {1, 0, 0}, {0, 
      0, 1}, {1, 0, 1}, {1, 0.6, 1}, {1, 0.4, 0.4}, {0.7, 0.8, 0.7}}, {{0.5, 
      0.6, 0.8}, {0.5, 0.6, 0.8}, {0, 0.6, 1}, {1, 0, 0}, {0, 1, 1}, {0.6, 
      0.4, 0.4}, {1, 0.6, 1}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {0.2, 0.6, 
      0.4}, {1, 1, 0}, {0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}, {0.7, 0.8, 0.7}, {
      1, 0.6, 0}, {1, 0.8, 0.4}, {0.6, 0.4, 0.8}, {0.4, 0.2, 1}, {
      1, 0.8, 0.7}, {0, 1, 0}, {1, 0, 1}, {0, 0, 1}, {0.7, 1, 0}, {0.2, 0.8, 
      0.7}, {0.8, 0.6, 0.5}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.2, 0.8, 
      0.7}, {0.7, 1, 0}, {0, 0, 1}, {1, 0, 1}, {0, 1, 0}, {1, 0.8, 0.7}, {
      0.4, 0.2, 1}, {0.6, 0.4, 0.8}, {1, 0.8, 0.4}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {1, 1, 0}, {0.2, 0.6, 0.4}, {
      0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {1, 0.6, 1}, {0.6, 0.4, 0.4}, {0, 1, 
      1}, {1, 0, 0}, {0, 0.6, 1}, {0.5, 0.6, 0.8}}, {{0.7, 0.8, 0.7}, {0.7, 
      0.8, 0.7}, {1, 0.4, 0.4}, {1, 0.6, 1}, {1, 0, 1}, {0, 0, 1}, {1, 0, 
      0}, {0, 0.6, 1}, {0.6, 0.4, 0.8}, {0.4, 0.2, 1}, {1, 0.6, 0}, {
      1, 0.8, 0.4}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {0.2, 0.6, 0.4}, {0.2, 0.8, 0.7}, {0.7, 1, 0}, {
      0, 1, 1}, {0.6, 0.4, 0.4}, {0, 1, 0}, {1, 0.8, 0.7}, {0.8, 0.2, 
      0.5}}, {{1, 0.6, 0}, {1, 0.6, 0}, {0, 1, 1}, {1, 0, 1}, {1, 0.6, 1}, {
      0.7, 1, 0}, {0.6, 0.6, 0.2}, {1, 1, 0}, {0.5, 0.6, 0.8}, {0.8, 0.2, 
      0.5}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {1, 0.8, 0.4}, {0.6, 0.4, 
      0.8}, {0, 0.6, 1}, {1, 0.8, 0.7}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.4}, {0, 0, 1}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {0.2, 0.6, 0.4}, {0.8, 
      0.6, 0.2}, {0.4, 0.2, 1}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 1, 
      0}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0, 0.6, 1}, {1, 0.8, 0.7}, {0.4, 0.2, 1}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {0.8, 0.6, 0.2}, {0.2, 0.6, 0.4}, {1, 0.6, 1}, {0.7, 1, 0}, {
      0, 0, 1}, {0.6, 0.4, 0.4}, {1, 0.8, 0.4}}, {{0, 1, 0}, {0, 1, 0}, {0.8, 
      0.6, 0.5}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {1, 1, 0}, {0.2, 0.8, 
      0.7}, {0.7, 1, 0}, {0, 0, 1}, {1, 0, 1}, {0.6, 0.4, 0.4}, {0, 1, 1}, {1,
       0, 0}, {1, 0.8, 0.7}, {0.4, 0.2, 1}, {0.6, 0.4, 0.8}, {1, 0.8, 0.4}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.5, 0.6, 0.8}, {0.2, 0.6, 0.4}, {0.6, 
      0.6, 0.2}, {1, 0.4, 0.4}, {1, 0.6, 1}, {0, 0.6, 1}}, {{1, 0, 0}, {1, 0, 
      0}, {0.7, 0.8, 0.7}, {0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 0.2}, {
      1, 0.4, 0.4}, {1, 0.6, 1}, {1, 0, 1}, {0, 0, 1}, {0, 1, 1}, {0.6, 0.4, 
      0.4}, {0, 1, 0}, {0, 0.6, 1}, {0.6, 0.4, 0.8}, {0.4, 0.2, 1}, {
      1, 0.6, 0}, {1, 0.8, 0.4}, {0.8, 0.6, 0.5}, {0.8, 0.2, 0.5}, {0.6, 0.6, 
      0.2}, {0.2, 0.6, 0.4}, {0.2, 0.8, 0.7}, {0.7, 1, 0}, {1, 0.8, 0.7}}, {{
      0, 1, 1}, {0, 1, 1}, {0.6, 0.6, 0.2}, {1, 1, 0}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0, 0.6, 1}, {1, 0.8, 0.7}, {1,
       0, 0}, {0, 1, 0}, {0.6, 0.4, 0.4}, {1, 0, 1}, {1, 0.6, 1}, {
      0.7, 1, 0}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {0.2, 0.6, 0.4}, {0.8, 0.6, 
      0.2}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {1, 0.8, 0.4}, {0.4, 0.2, 1}, {
      0, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 
      0.8, 0.7}, {0.8, 0.6, 0.5}, {1, 1, 0}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {
      0.2, 0.8, 0.7}, {1, 0.6, 1}, {0.7, 1, 0}, {0, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0, 0.6, 1}, {1, 0.8, 0.7}, {0.4, 0.2, 1}, {
      1, 0.8, 0.4}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.2, 
      0.6, 0.4}, {0.6, 0.4, 0.4}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}, {
      0.7, 1, 0}, {0, 0, 1}, {0.6, 0.4, 0.4}, {0, 0.6, 1}, {1, 0.8, 0.7}, {
      0.4, 0.2, 1}, {1, 0.8, 0.4}, {0.2, 0.6, 0.4}}, {{1, 1, 1}, {1, 1, 0}, {
      1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 
      0.7}, {1, 0.6, 1}, {0.7, 1, 0}, {0, 0, 1}, {0.6, 0.4, 0.4}, {
      0, 0.6, 1}, {1, 0.8, 0.7}, {0.4, 0.2, 1}, {1, 0.8, 0.4}, {0.2, 0.6, 
      0.4}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 23.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 22.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 21.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 20.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 19.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 18.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 17.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 16.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 15.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 14.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 13.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"c\[CenterDot]d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 12.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 11.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 10.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 9.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 8.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 7.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]c\[CenterDot]d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 6.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]c\[CenterDot]d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 5.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 4.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]b\[CenterDot]d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 3.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]c\[CenterDot]d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 2.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]c\[CenterDot]d\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 1.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]b\[CenterDot]c\[CenterDot]d\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 0.5}, {1, 0}]}, 
   LineBox[{{1, 0}, {1, 24}, {25, 24}}]},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.422647642265625*^9, 3.442673523367625*^9}]
}, Open  ]],

Cell[TextData[{
 "We saw two other groups of order 24, namely ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 " and the octahedronal group. Yet these turned out to be isomorphic. Is this \
new group of order 24 possibly the same group?\n\nRecall that the main tool \
we used for finding the isomorphism between the octahedronal group and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 " was determining the orders of the elements in the groups. We defined ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 " to be:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["S4 = Group[{P[2,1],P[2,3,1],P[2,3,4,1]}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.44267352625825*^9, 3.442673527352*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "1", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "3", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "2", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "4", ",", "1", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "2", ",", "1", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "4", ",", "1", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "3", ",", "1", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "3", ",", "4", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "2", ",", "4", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "4", ",", "3", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "2", ",", "3", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "4", ",", "2", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "3", ",", "2", ",", "1"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.422647647890625*^9, 3.442673528242625*^9}]
}, Open  ]],

Cell["Determining which elements were of order 2, we took", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["S4^2", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "3", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "1", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "4", ",", "1", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "3", ",", "2", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "2", ",", "4", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "3", ",", "2", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "4", ",", "3", ",", "1"}], "]"}], ",", 
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "4", ",", "1", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "2", ",", "1", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.42264765078125*^9, 3.442673529836375*^9}]
}, Open  ]],

Cell[TextData[{
 "and saw that elements 2, 3, 6, 7, 8, 15, 17, 22, and 24 were of order 2. \
Now let us try this with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     SubscriptBox["S", "3"], " ", "\[Times]", " ", 
     SubsuperscriptBox["Z", "8", "*"]}]}], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G^2", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", "e", ",", "e", ",", "e", ",", "a", ",", "e", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", "e", ",", "e", ",", "e", ",", "e", ",", "a", ",", 
   "a", ",", "e", ",", "e", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", "e", ",", "e", ",", "e", ",", "a", ",", "e", ",", 
   "e"}], "}"}]], "Output",
 CellChangeTimes->{3.4226476705625*^9, 3.4415441893546247`*^9, 
  3.4426735315395*^9}]
}, Open  ]],

Cell[TextData[{
 "So elements 3, 4, 5, 7, 10, 11, 12, 13, 16, 17, 19, 20, 21, 23 and 24 have \
order 2.  (Element 1 was the identity, which we don't count.) So there are 15 \
elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which are of order 2, as opposed to 9 elements in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ". Hence, these two groups are not isomorphic."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4226476795625*^9, 3.42264771684375*^9}}],

Cell[TextData[{
 "This example shows it would be helpful to know the number of elements of \
order ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " of the direct product ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[Times]", "K"}], TraditionalForm]]],
 ". Then we could determine whether or not two groups expressed as direct \
products were isomorphic or not. The following proposition helps us out:\n\n\
PROPOSITION 6.2\nLet ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be finite groups, and let ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " be a positive integer. If there are ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " elements of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["a", "n"], TraditionalForm]]],
 "  is the identity in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " elemetns of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", "n"], TraditionalForm]]],
 " is the identity element of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", then there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[CenterDot]", "s"}], TraditionalForm]]],
 " elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[Times]", "K"}], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["c", "n"], TraditionalForm]]],
 " is the identity element in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[Times]", 
    RowBox[{"K", "."}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436636866171875*^9, 3.436636866765625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "1"], TraditionalForm]]],
 " denote the identity element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "2"], TraditionalForm]]],
 " denote the identity element of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". An element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"h", ",", "k"}], ")"}]}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[Times]", "K"}], TraditionalForm]]],
 " solves the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["c", "n"], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["e", "1"], ",", " ", 
      SubscriptBox["e", "2"]}], ")"}]}], TraditionalForm]]],
 " if and only if  \n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox["h", "n"], "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["e", "1"], "    ", "and", "    ", 
       SuperscriptBox["k", "n"]}], " ", "=", " ", 
      SubscriptBox["e", "2"]}]}]}], TraditionalForm]]],
 ".\n\nSince there are ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " solutions to the former, and ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " solutions to the latter, there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[CenterDot]", "s"}], TraditionalForm]]],
 " ordered pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", "k"}], ")"}], TraditionalForm]]],
 " which solve both of these equations. Thus, there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[CenterDot]", "s"}], TraditionalForm]]],
 " elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[Times]", "K"}], TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["c", "n"], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["e", "1"], ",", " ", 
       SubscriptBox["e", "2"]}], ")"}], "."}]}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "Let us use this proposition on the example ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "3"], "\[Times]", 
    RowBox[{
     SubsuperscriptBox["Z", "8", "*"], "."}]}], TraditionalForm]]],
 " The number of elements of this group solve ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], "=", "e"}], TraditionalForm]]],
 " is the product of the number of elements solving the equation in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " and the number of elements in ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ". Note that four of the six elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 "  satisfy ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], "=", "e"}], TraditionalForm]]],
 ", while all four elements of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " satisfy the equation. Thus, there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", "\[CenterDot]", "4"}], " ", "=", " ", "16"}], 
   TraditionalForm]]],
 " elements of the product ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "3"], "\[Times]", 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 " whose square is the identity. One of these is the identity element, but \
the other 15 elements are of order 2. Thus, we can find the number of \
elements of a direct product of given order without having to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".\n\nEXPERIMENT:\nUse proposition 6.1.3 to predict the number of elements \
of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "3"], "\[Times]", 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 " which are of order 3 and of order 6. Then use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to verify your predictions."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["          ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "We will end this section by showing another way to test whether a group is \
a direct product of two of its subgroups. \n\nCOROLLARY 6.1\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group with identity ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be two ",
 StyleBox["normal",
  FontSlant->"Italic"],
 " subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then \nif ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", " ", "\[Intersection]", " ", "K"}], " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", "\[TildeTilde]", " ", 
    RowBox[{"H", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436636902859375*^9, 3.436636902984375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The first condition of the direct product theorem (6.1) is given, so we \
only need to show that the second condition holds. That is, we need to show \
that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"h", "\[CenterDot]", "k"}], " ", "=", " ", 
    RowBox[{"k", "\[CenterDot]", "h"}]}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Let  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "\[Element]", " ", "H", " "}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Element]", "K"}], TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "k", "\[CenterDot]", 
    SuperscriptBox["h", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "k", "\[CenterDot]", 
    SuperscriptBox["h", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["k", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".  \n\nBut ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is also a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[CenterDot]", 
    SuperscriptBox["h", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["k", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ".  Hence,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "k", "\[CenterDot]", 
    SuperscriptBox["h", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["k", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ".\n\nWe now use the fact that the only element in both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ".  Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"h", "\[CenterDot]", "k", "\[CenterDot]", 
     SuperscriptBox["h", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SuperscriptBox["k", 
      RowBox[{"-", "1"}]]}], "=", " ", "e"}], TraditionalForm]]],
 ", which implies ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"h", "\[CenterDot]", "k"}], " ", "=", " ", 
    RowBox[{"k", "\[CenterDot]", "h"}]}], TraditionalForm]]],
 ". \n\nTherefore, the second condition of the direct product theorem (6.1) \
holds, and so by this theorem,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"H", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436636914953125*^9, 3.43663696340625*^9}}]
}, Open  ]],

Cell[TextData[{
 "This corollary is sometimes more useful than the direct product theorem, \
even though for abelian groups the two are equivalent, since all subgroups of \
abelian groups are normal. In the next section we will continue to study the \
decomposition of abelian groups, and find that ",
 StyleBox["all",
  FontSlant->"Italic"],
 " finite abelian groups can be decomposed uniquely into a certain form.     \
"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fundamental Theorem of Finite Abelian Groups", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In the last section we mentioned that we will be able to construct any \
finite commutative group using as building blocks the groups that we have \
already learned. In this section, we will show how that can be done by \
considering the direct products of the cyclic groups ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". We will\neven be able to find the number of abelian groups of a given \
order. Let us begin with a simple example.\n\nConsider the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "6"], " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
     "0", ",", " ", "1", ",", " ", "2", ",", " ", "3", ",", " ", "4", ",", 
      " ", "5"}], "}"}]}], TraditionalForm]]],
 ". Can we express this as the direct product of two smaller groups? \n\nNote \
that for abelian groups, the second condition of the direct product theorem \
is automatically satisfied, so we merely have to find two subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " whose intersection\nis just the identity ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "0", "}"}], TraditionalForm]]],
 ", and whose product gives us the whole group. With a group this small, it \
is easy to find ",
 StyleBox["all",
  FontSlant->"Italic"],
 " of the subgroups:\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      RowBox[{"{", "0", "}"}], ",", "                  ", "\n", "  ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "3"}], "}"}], ",", "                  ", "\n", 
      "  ", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", "2", ",", " ", "4"}], "}"}], ",", " ", "and"}], 
     "                ", "\n", "  ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "0", ",", " ", "1", ",", " ", "2", ",", " ", "3", ",", " ", "4", ",", 
        " ", "5"}], "}"}], "."}]}]}], TraditionalForm]]],
 "\n\nThus, the only two candidates are the subgroups \n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "3"}], "}"}], " ", 
     StyleBox["\[TildeTilde]",
      FontSize->16], " ", 
     SubscriptBox["Z", "2"]}]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"K", " ", "=", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "2", ",", "4"}], "}"}], " ", 
      StyleBox["\[TildeTilde]",
       FontSize->16], " ", 
      SubscriptBox["Z", "3."]}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since the intersection of these two subgroups is the identity, we have by \
the direct product theorem that\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"H", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[Times]", "K"}], TraditionalForm]]],
 " is of order 6, we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " is of order 6, and hence is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 ".  Therefore,\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "6"], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["Z", "2"], "\[Times]", " ", 
     SubscriptBox["Z", "3"]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let us check this using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ": ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 "  are easy to define using ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " as the two generators."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
InitGroup[e];
Define[a^2, e]
Define[1/a, a]
Define[b^3, e]
Define[1/b, b^2]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "We now have the groups ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["H = Group[{a}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a"}], "}"}]], "Output",
 CellChangeTimes->{3.442673539852*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["K = Group[{b}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.442673542117625*^9}]
}, Open  ]],

Cell[TextData[{
 "To take the direct product we make the generator of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " commute with the generator of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "."}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Define[b.a, a.b]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "We now can find the direct product by finding ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", 
    RowBox[{"K", ":"}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = H.K", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"a", ",", "b", ",", "e", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4426735447895*^9}]
}, Open  ]],

Cell["Here is the multiplication table:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {0, 1, 0}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 
      1}, {1, 0, 0}, {1, 0, 1}}, {{0, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {0, 
      1, 0}, {1, 1, 0}, {1, 0, 1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 1}, {0.6, 
      0.4, 0.8}, {1, 0, 0}, {0, 1, 0}, {1, 1, 0}, {0, 1, 1}}, {{0, 1, 1}, {1, 
      1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{
      1, 0, 1}, {1, 0, 0}, {0, 1, 0}, {1, 0, 1}, {0.6, 0.4, 0.8}, {0, 1, 1}, {
      1, 1, 0}}, {{1, 1, 0}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0.6,
       0.4, 0.8}, {0, 1, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1,
       0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\[CenterDot]b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.44267354613325*^9}]
}, Open  ]],

Cell[TextData[{
 "We want to see if this group is isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 ". Thus, we want to find an element of order 6. By comparing ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " with"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G^2", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", "e", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", "b", ",", "b"}], "}"}]], "Output",
 CellChangeTimes->{3.44267354788325*^9}]
}, Open  ]],

Cell["and", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G^3", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"a", ",", "e", ",", "e", ",", "a", ",", "e", ",", "a"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.442673548930125*^9}]
}, Open  ]],

Cell[TextData[{
 "we find that the only elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which are not of order 1, 2 or 3 are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", 
    SuperscriptBox["b", "2"]}], TraditionalForm]]],
 ". From this information we can create an isomorphism. Let us define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " without erasing the previous group. We can do this by using ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " as the generator:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
Define[c^6, e]
Define[1/c, c^5]
Z6 = Group[{c}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "c", ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(c\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`c, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(c\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`c, $CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(c\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(c\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`c, $CellContext`c], $CellContext`c], $CellContext`c], \
$CellContext`c],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.442673550398875*^9}]
}, Open  ]],

Cell[TextData[{
 "We now let ",
 StyleBox["F",
  FontWeight->"Bold"],
 " be the isomorphism between ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Times]", 
    RowBox[{
     SubscriptBox["Z", "3"], ":"}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Homomorph[F]
Define[F[c], a.b]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["Does this work?  Let's check:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[F, Z6] ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.44267355519575*^9}]
}, Open  ]],

Cell["So this is a homomorphism. The image is", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["F[{Z6}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"a", ",", "b", ",", "e", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.442673556586375*^9}]
}, Open  ]],

Cell[TextData[{
 "So ",
 StyleBox["F",
  FontWeight->"Bold"],
 " is surjective. The kernel of ",
 StyleBox["F",
  FontWeight->"Bold"],
 " is "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Kernel[F, Z6]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", "e", "}"}]], "Output",
 CellChangeTimes->{3.4426735579145*^9}]
}, Open  ]],

Cell[TextData[{
 "So ",
 StyleBox["F",
  FontWeight->"Bold"],
 " is also injective. Therefore, we have found the isomorphism from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " onto ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ".\n\nEXPERIMENT:\nUsing the same technique as above, see if you can find a \
decomposition of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "10"], TraditionalForm]]],
 " into a direct product of smaller groups. Define the homomorphism between \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "10"], TraditionalForm]]],
 " and the direct product, and show that the homomorphism is surjective and \
injective."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["      ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "By observing the patterns in the decomposition of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "10"], TraditionalForm]]],
 ", let us see if we can find a decomposition of any commutative group.\n\n\
Observe the groups ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{"0", ",", " ", "3"}], "}"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{"0", ",", " ", "2", ",", " ", "4"}], "}"}]}], 
   TraditionalForm]]],
 " in the example of the decomposition of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 ". Notice that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " consists of all of the elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["h", "2"], "=", " ", "0"}], TraditionalForm]]],
 ".  Also, ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " consists of all the elements such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["k", "3"], " ", "=", " ", "0"}], TraditionalForm]]],
 ". The fact that these two subgroups have only the identity in common is not \
a coincidence, as the next lemma shows:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\nLEMMA 6.1\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be an abelian group of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "v"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " are relatively prime numbers. Then \n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"h", " ", "\[Element]", " ", "G"}], " ", "|", " ", 
       SuperscriptBox["h", "m"]}], "=", " ", "e"}], "}"}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436637040421875*^9, 3.436637081140625*^9}}],

Cell[TextData[{
 "and \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"k", " ", "\[Element]", " ", "G"}], " ", "|", " ", 
       SuperscriptBox["k", "n"]}], "=", " ", "e"}], "}"}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436637061515625*^9, 3.436637083375*^9}}],

Cell[TextData[{
 "are both subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"H", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "To check that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are indeed subgroups simply observe that since ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is  commutative the functions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "x", ")"}], " ", "=", " ", 
    SuperscriptBox["x", "m"]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "x", ")"}], " ", "=", " ", 
    SuperscriptBox["x", "n"]}], TraditionalForm]]],
 " are both homomorphisms of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are the kernels of the mappings ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ".\n\nTo show that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " have only the identity element in common, we consider an element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in the intersection.  By the Chinese remainder theorem (1.3), there exists \
a non-negative number ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", " ", "<", " ", 
    RowBox[{"m", "\[CenterDot]", "n"}]}], TraditionalForm]]],
 " such that\n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Congruent]", " ", 
    RowBox[{"1", " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "m"}], ")"}]}]}], TraditionalForm]]],
 "  and    ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", " ", "\[Congruent]", " ", 
    RowBox[{"0", " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 ".\n                        \nThen ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"1", " ", "+", " ", 
      RowBox[{"m", " ", "b"}]}], ")"}]}], TraditionalForm]]],
 "  for some number ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ".  Thus,\n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox["x", "k"], "=", " ", 
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"m", " ", "b"}]}], ")"}]], "=", " ", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", 
        SuperscriptBox[
         RowBox[{"(", 
          SuperscriptBox["x", "m"], ")"}], "b"]}], "  ", "=", " ", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", 
         SuperscriptBox["e", "b"]}], "=", " ", "x"}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nsince ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". Yet  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "=", " ", 
    RowBox[{"n", " ", "c"}]}], TraditionalForm]]],
 "  for some number ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", so\n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "k"], "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"n", " ", "c"}]], "=", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox["x", "n"], " ", ")"}], "c"], "=", " ", 
      RowBox[{
       SuperscriptBox["e", "c"], "=", " ", "e"}]}]}]}], TraditionalForm]]],
 "\n\nsince ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "e"}], TraditionalForm]]],
 ", and the so  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", " ", "\[Intersection]", " ", "K"}], " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is abelian, the direct product theorem (6.1) proves that\n\n               \
        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"H", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 ".\n                       \nAll that is left to prove is that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{"H", "\[CenterDot]", "K"}]}], TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " be an element in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " are coprime, by the greatest common divisor theorem (1.2) there exists ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " such that\n\n                                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "m"}], " ", "+", " ", 
     RowBox[{"b", " ", "n"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"GCD", "[", 
      RowBox[{"m", ",", "n"}], "]"}], " ", "=", " ", "1"}]}], 
   TraditionalForm]]],
 ".\nThen \n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"g", " ", "=", " ", 
     RowBox[{
      SuperscriptBox["g", "1"], "=", " ", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"a", " ", "n"}], " ", "+", " ", 
          RowBox[{"b", " ", "m"}]}], ")"}]], "=", " ", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"a", " ", "n"}]], "\[CenterDot]", " ", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"b", " ", "m"}]], "."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nNow, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["g", 
       RowBox[{"a", " ", "n"}]], " ", ")"}], "m"], "=", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["g", "a"], ")"}], 
      RowBox[{"n", " ", "m"}]], " ", "=", " ", "e"}]}], TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["g", 
    RowBox[{"a", " ", "n"}]], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ".  Likewise, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["g", 
    RowBox[{"b", " ", "m"}]], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Thus, every element of ",
 StyleBox["G",
  FontSlant->"Italic"],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 ", and so \n\n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"H", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436637088625*^9, 3.4366372203125*^9}, {
  3.4415447789014997`*^9, 3.4415448062452497`*^9}}]
}, Open  ]],

Cell[TextData[{
 "This lemma tells us that if an abelian group has an order that is a product \
of two coprime numbers, this group can be writen as a direct product of two \
groups. Unfortunately, the lemma does not tell us that neither ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are trivial groups. As a result, we still have to ask whether this is a \
decomposition.\n\nHere is an example to illustrate the possible problem that \
could occur. Suppose we know ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is an abelian group of order 24. Since 24 = 8\[CenterDot]3, we could let \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"m", " ", "=", " ", "8"}], ","}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "3"}], TraditionalForm]]],
 " in lemma 6.1.  Then ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " would consist of all elements of order 1, 2, 4, or 8, while ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " would consist of the elements of order 1 or 3. Then we would have\n\n      \
            ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"H", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43663726675*^9, 3.436637269046875*^9}}],

Cell[TextData[{
 "What if ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " had no elements of order 3?  Then ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " would be just the identity element, and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " would have to be all of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Lemma 6.1 would hold, but since ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are not proper subgroups, this would not give a decomposition of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". The next lemma will show that, in fact, ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " must have an element of order 3.\n\nLEMMA 6.2\n\nIf ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a finite abelian group and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime which divides the order of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has an element of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43663729228125*^9, 3.4366372924375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We will proceed using induction on the order of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".  If ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "G", "|"}], " ", "=", " ", "2"}], TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " must be 2 and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " must isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ", and so there is an element of order 2 in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nIn fact, whenever ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "G", "|"}], TraditionalForm]]],
 " is a prime number, then ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " must be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "G", "|"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " must be isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "p"], TraditionalForm]]],
 ". So again, there would be an element of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nSuppose that the assumption is true for all groups of order less than \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "G", "|"}], TraditionalForm]]],
 ". If  ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " does not have any proper subgroups, then ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " would be a cyclic group of prime order (which we have already covered.)  \
Thus, we may assume that ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has a subgroup ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " that is neither ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "e", "}"}], TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is abelian all subgroups are normal. Thus we could consider the quotient \
group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "N"}], TraditionalForm]]],
 ".  Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "G", "|"}], " ", "=", " ", 
    RowBox[{"|", "N", "|", 
     RowBox[{"\[CenterDot]", 
      RowBox[{"|", 
       RowBox[{"G", "/", "N"}], "|"}]}]}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " must divide either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "N", "|"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", 
    RowBox[{"G", "/", "N"}], "|"}], TraditionalForm]]],
 ".  If ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ", then because ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a smaller group than ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", by induction ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " must have an element of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", which would be in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nIf ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " does not divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "N", "|"}], TraditionalForm]]],
 " it must divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", 
    RowBox[{"G", "/", "N"}], "|"}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "N"}], TraditionalForm]]],
 " is a smaller group than ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", by induction ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "N"}], TraditionalForm]]],
 " must have an element of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". This element can be written ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "N"}], TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "N"}], TraditionalForm]]],
 " is of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " cannot be in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ", yet ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["a", "p"], TraditionalForm]]],
 " must be in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". If the order of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ", we would have by corollary 3.1.5 that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox[
      RowBox[{
       FormBox[
        RowBox[{"(", 
         SuperscriptBox["a", "p"]}],
        TraditionalForm], ")"}], "q"],
     TraditionalForm], " ", "=", " ", "e"}], TraditionalForm]]],
 ".\n\nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", 
    SuperscriptBox["a", "q"]}], TraditionalForm]]],
 " is not the identity, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["b", "p"], "=", " ", "e"}], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " would be the required element. But if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", "e"}], TraditionalForm]]],
 ", then",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"a", "\[CenterDot]", "N"}], ")"}], "q"], " ", "=", " ", 
     "N"}]}], TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "N"}], TraditionalForm]]],
 " was of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " must divide ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ". But we assumed that ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " did not divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", " ", "=", " ", 
    RowBox[{"|", "N", "|"}]}], TraditionalForm]]],
 ". Hence, ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is not the identity, and so ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has an element of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "In fact we will see that lemma 6.2 is true for ",
 StyleBox["all",
  FontSlant->"Italic"],
 " groups, not just abelian groups. However, the result for abelian groups is \
good enough for us to proceed. \n\nEXPERIMENT:\nSince the Euler totient \
function of 35 is"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436637339640625*^9, 3.43663733978125*^9}}],

Cell[CellGroupData[{

Cell["EulerPhi[35]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["24"], "Output",
 CellChangeTimes->{3.442673570461375*^9}]
}, Open  ]],

Cell[TextData[{
 "the group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "35", "*"], TraditionalForm]]],
 " is an abelian group with 24 elements. Lemma 6.2.2 then tells us that there \
are elements of order 2 and of order 3. Using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", find the set of elements ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "3"], "=", " ", "1"}], TraditionalForm]]],
 ", and the set of elements ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["k", "8"], "=", " ", "1"}], TraditionalForm]]],
 ". Show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", "=", " ", 
    SubsuperscriptBox["Z", "35", "*"]}], TraditionalForm]]],
 ", and hence ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "35", "*"], TraditionalForm]]],
 " has the decomposition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[Times]", "K"}], TraditionalForm]]],
 ".\n\nThis experiment demonstrates how lemma 6.2 guarantees that the \
subgroups ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " generated by lemma 6.1 must be proper subgroups. Notice that in this \
experiment, ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " contained 3 elements while ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " contained 8 elements. There seems to be a pattern: the number of elements \
of ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " satisfying\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"(", 
      StyleBox[
       SuperscriptBox["p", 
        StyleBox["a",
         FontSize->10]],
       FontSize->10], 
      StyleBox[")",
       FontSize->10]}]], "=", " ", "e"}], TraditionalForm]]],
 ",\n\nis ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "a"], TraditionalForm]]],
 " whenever ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " divides the order of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " exactly ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " times. This apparent pattern can be shown to exist for all abelian groups. \
We thus have the following lemma:\n\nLEMMA 6.3                            \n\
Let ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be an abelian group of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "n"], "\[CenterDot]", "k"}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is prime, ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is not divisible by ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", ">", "0"}], TraditionalForm]]],
 ". Then there are subgroups ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " such that \n \n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"P", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 ",\n                  \nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "P", "|"}], " ", "=", " ", 
    SuperscriptBox["p", "n"]}], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "K", "|"}], " ", "=", " ", "k"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4366373458125*^9, 3.436637410109375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " are coprime, we can use lemma 6.1 to form the subgroups\n\n                \
                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{"P", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"x", " ", "\[Element]", " ", "G"}], "  ", "|", " ", 
         SuperscriptBox["x", 
          RowBox[{"(", 
           SuperscriptBox["p", "n"], ")"}]]}]}]}],
      TraditionalForm], " ", "=", " ", "e"}], "}"}], TraditionalForm]]],
 "\nand \n                                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{"K", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"x", " ", "\[Element]", " ", "G"}], " ", "|", " ", 
         SuperscriptBox["x", "k"]}]}]}],
      TraditionalForm], " ", "=", " ", "e"}], "}"}], TraditionalForm]]],
 ".\n                                 \nBy lemma 6.1 these two subgroups have \
only the identity in common, and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"P", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " divided ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "K", "|"}], TraditionalForm]]],
 ", then by lemma 6.2, ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " would contain an element of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". But this element would then be in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " as well, which contradicts the fact that only the identity element is in \
common between ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". So ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " does not divide the order of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".\n\nAlso note that the order of every element of ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a power of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Thus, lemma 6.2 tells us that no other prime other than ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "P", "|"}], TraditionalForm]]],
 ". \n\nFinally, note that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "G", "|"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["p", "n"], "\[CenterDot]", "k"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"|", "P", "|", "\[CenterDot]"}], "|", "K", "|"}]}]}], 
   TraditionalForm]]],
 ".  Since ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " does not divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "K", "|"}], TraditionalForm]]],
 ", we have that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " must divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "P", "|"}], TraditionalForm]]],
 ". But no other primes can divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "P", "|"}], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "P", "|"}], " ", "=", " ", 
    SuperscriptBox["p", "n"]}], TraditionalForm]]],
 ".  Hence, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "K", "|"}], " ", "=", " ", "k"}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436637418265625*^9, 3.436637464953125*^9}}]
}, Open  ]],

Cell[TextData[{
 "Lemma 6.3 is a tremendous help in finding the structure of abelian groups. \
For example, suppose we have an abelian group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of order 24. Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"24", "=", 
    RowBox[{
     SuperscriptBox["2", "3"], "\[CenterDot]", "3"}]}], TraditionalForm]]],
 ", lemma 6.3 states that ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is isomorphic to a direct product of a group of order 8 and a group of \
order 3. Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " is the only group of order 3, and there are 3 abelian groups of order 8, \
namely \n\n              ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ",\n              \nwe have that ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " must be isomorphic to one of the three groups \n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["Z", "8"], "\[Times]", 
     SubscriptBox["Z", "3"]}], ",", "  ", 
    RowBox[{
     SubsuperscriptBox["Z", "15", "*"], "\[Times]", 
     SubscriptBox["Z", "3"]}], ",", "  ", 
    RowBox[{"or", "   ", 
     SubsuperscriptBox["Z", "24", "*"], "\[Times]", 
     SubscriptBox["Z", "3."]}]}], TraditionalForm]]],
 "\n\nEXPERIMENT: \nSince ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "35", "*"], TraditionalForm]]],
 " is an abelian group with 24 elements, this must be isomorphic to one of \
the three groups mentioned above. Which of the three is it? (Hint: first use \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "35", "*"], TraditionalForm]]],
 ", and to determine the number of elements of order 2. Then use proposition \
6.2 to find the number of elements of order 2 for the three groups listed \
above.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4366374719375*^9, 3.436637476609375*^9}}],

Cell["      ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We have seen that if we can find all abelian groups of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " a prime number, then we will be able to find all finite abelian groups. \
The next lemma is the key to finding all abelian groups whose order is a \
power of a prime."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "LEMMA 6.4\nSuppose ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is an abelian group of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime. Let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be an element in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " that has the maximal order of all of the elements of ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Then\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"X", " ", "\[Times]", " ", "T"}]}], TraditionalForm]]],
 ",\n                 \nwhere ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " is the cyclic group generated by ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.436637485828125*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We will use induction on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a cyclic group of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", and hence is generated by non-identity element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ".  We then have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", " ", "=", " ", "P"}], TraditionalForm]]],
 ", so we can let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 ", and hence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", " ", "\[TildeTilde]", " ", 
    RowBox[{"X", " ", "\[Times]", " ", "T"}]}], TraditionalForm]]],
 ".\n\nNow suppose that the assertion is true for all powers of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".  Notice that the order of every element of ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a power of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Thus, if we let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be an element with the ",
 StyleBox["largest",
  FontSlant->"Italic"],
 " order, say ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", then the order of all elements in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " must divide ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ". Hence, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["g", "m"],
     TraditionalForm], " ", "=", " ", "e"}], TraditionalForm]]],
 " for all elements ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ".\n\nWe now let ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " be the subgroup generated by ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", " ", "=", " ", "P"}], TraditionalForm]]],
 ", then we can again let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 " and we are done. If ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " is not ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", we let ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " be an element of ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " not in ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " which has the ",
 StyleBox["smallest",
  FontSlant->"Italic"],
 " possible order. Then since the order of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", "p"], TraditionalForm]]],
 " is less than the order of ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", "p"], TraditionalForm]]],
 " must be in ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ". This means that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", "p"], "=", " ", 
    SuperscriptBox["x", "q"]}], TraditionalForm]]],
 "  for some  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", " ", "\[LessEqual]", " ", "q", " ", "<", " ", "m"}], 
   TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", "m"], "=", " ", "e"}], TraditionalForm]]],
 ". But \n\n                             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", "m"], "=", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["y", "p"], ")"}], 
      RowBox[{"(", 
       RowBox[{"m", "/", "p"}], ")"}]], "=", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox["x", "q"], ")"}], 
       RowBox[{"(", 
        RowBox[{"m", "/", "p"}], ")"}]], "  ", "=", " ", 
      SuperscriptBox["x", 
       RowBox[{"(", 
        RowBox[{"m", " ", 
         RowBox[{"q", "/", "p"}]}], ")"}]]}]}]}], TraditionalForm]]],
 ".\n\nBecause ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is of order ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", this can be the identity only if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", 
    RowBox[{"q", "/", "p"}]}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ".  Hence, ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". \n                                      \nIf we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"q", "/", "p"}], ")"}]}]], "\[CenterDot]", "y"}]}], 
   TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is not in ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " because ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " isn't, and \n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox["k", "p"], "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         SuperscriptBox["x", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{"q", "/", "p"}], ")"}]}]], " ", ")"}], "p"], 
       "\[CenterDot]", 
       SuperscriptBox["y", 
        RowBox[{" ", "p"}]]}], "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{"-", "q"}]], "\[CenterDot]", 
        SuperscriptBox["y", 
         RowBox[{" ", "p"}]]}], "=", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["x", 
          RowBox[{"-", "q"}]], "\[CenterDot]", 
         SuperscriptBox["x", "q"]}], "=", " ", 
        RowBox[{"e", "."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nTherefore, we have found an element ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " which is not in ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ".\n\nIf we let ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be the group generated by the element ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", " ", "\[Intersection]", " ", "K"}], " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 ".\n\nConsider the quotient group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "/", "K"}], TraditionalForm]]],
 ".  What is the order of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "K"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "/", "K"}], TraditionalForm]]],
 "? We see that\n       \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", " ", "K"}], ")"}], "n"], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"K", "   ", "\[DoubleLongLeftRightArrow]", "  ", 
       SuperscriptBox["x", 
        RowBox[{"n", " "}]]}], "\[Element]", " ", 
      RowBox[{"K", "  ", "\[DoubleLongLeftRightArrow]", "  ", 
       SuperscriptBox["x", "n"]}], " ", "\[Element]", "  ", 
      RowBox[{"X", "\[Intersection]", 
       RowBox[{"K", "  ", "\[DoubleLongLeftRightArrow]", " ", 
        SuperscriptBox["x", "n"]}]}]}], " ", "=", " ", "e"}]}], 
   TraditionalForm]]],
 ".\n         \nTherefore, the order of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "K"}], TraditionalForm]]],
 " is the same as the order of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", which is ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ". Also note that no element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "/", "K"}], TraditionalForm]]],
 " can have an element of higher order since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", "m"], " ", "=", " ", "1"}], TraditionalForm]]],
 " for all elements  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "  "}], TraditionalForm]]],
 "in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ".\n\nNow we use the induction!  Since the order of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "/", "K"}], TraditionalForm]]],
 " is less than the order of ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "K"}], TraditionalForm]]],
 " is an element of maximal order, we have by induction that\n\n              \
                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"P", "/", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"Y", " ", "\[Times]", " ", "B"}]}], TraditionalForm]]],
 ",\n                                 \nwhere ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " is the subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "/", "K"}], TraditionalForm]]],
 " generated by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "K"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "/", "K"}], TraditionalForm]]],
 " such that only the identity element ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is in the intersection of ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ".\n\nLet \[Phi] be the canonical homomorphism from ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "/", "K"}], TraditionalForm]]],
 " given by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "g", ")"}], " ", "=", " ", 
    RowBox[{"g", " ", 
     RowBox[{"K", "."}]}]}], TraditionalForm]]],
 "  Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Phi]", 
       RowBox[{"-", "1"}]], "(", "B", ")"}], "."}]}], TraditionalForm]]],
 "  Then ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". \n\nIf ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is in both ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "g", ")"}], TraditionalForm]]],
 " is in both ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". Since the intersection of ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " is the identity element, we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "g", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"g", " ", "\[CenterDot]", "K"}], " ", "=", " ", "K"}]}], 
   TraditionalForm]]],
 ".  Thus, ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is in the subgroup ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". But  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", " ", "\[Intersection]", " ", "K"}], " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 ", so we have \n\n                                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", " ", "\[Intersection]", " ", "T"}], " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 ".\n                                 \nThus, by the direct product theorem \
(6.1), we find that  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", "\[CenterDot]", "T"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"X", " ", "\[Times]", " ", "T"}]}], TraditionalForm]]],
 ".\n\nWe finally need to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", " ", "=", " ", 
    RowBox[{"X", "\[CenterDot]", "T"}]}], TraditionalForm]]],
 ".  Let ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " be an element in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", and since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"P", "/", "K"}], " ", "\[TildeTilde]", " ", 
    RowBox[{"Y", " ", "\[Times]", " ", "B"}]}], TraditionalForm]]],
 ", we can write ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "u", ")"}], TraditionalForm]]],
 " as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["x", "b"], "\[CenterDot]", "K"}], ")"}], "\[CenterDot]", 
    
    RowBox[{"(", 
     RowBox[{"k", " ", "K"}], ")"}]}], TraditionalForm]]],
 " for some number ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and some ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "K"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ".  Then\n       \n                                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", " ", "\[Epsilon]", " ", 
     RowBox[{
      SuperscriptBox["x", "b"], "\[CenterDot]", "k", "\[CenterDot]", "K"}]}], 
    " ", "\[SubsetEqual]", " ", 
    RowBox[{"X", "\[CenterDot]", "T"}]}], TraditionalForm]]],
 ".\n                                     \nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", " ", "=", " ", 
    RowBox[{"X", "\[CenterDot]", "T"}]}], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"X", " ", "\[Times]", " ", "T"}]}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436637502515625*^9, 3.436637599375*^9}, {
  3.436637664171875*^9, 3.43663766428125*^9}, {3.4415449573389997`*^9, 
  3.4415450499639997`*^9}, {3.4415452005421247`*^9, 3.4415452228077497`*^9}}]
}, Open  ]],

Cell[TextData[{
 "To give an illustration of how lemma 6.4 is applied, consider the groups of \
order 8. Lemma 6.4 states that these can be decomposed into cyclic groups. We \
have already seen that ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 " was isomorphic to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], "\[Times]", " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 " is already a cyclic group.  Let's use lemma 6.4 on the group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ".\n\nAll non-identity elements of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 " are of order 2, so this is the maximal order. Thus, lemma 6.4 states that \
",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 " can be decomposed into ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 " and a group of order 4. Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "4"]}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 ", the only other choice is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Times]", " ", 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 ".\n\nNow we can apply lemma 6.4 again to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ".  This is of order 4, and all elements besides the identity are of order \
2, so ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " can be decomposed into ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 " and a group of order 2, which\nmust be ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ".  Thus,\n\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["Z", "8", "*"], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"]}]}], TraditionalForm]]],
 ",\nand so\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["Z", "24", "*"], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["Z", 
      RowBox[{"2", " "}]], "\[Times]", " ", 
     SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43663768946875*^9, 3.436637707859375*^9}}],

Cell[TextData[{
 "It seems we have found a way to decompose any abelian group. But now we \
want to address the issue as to whether a decomposition is ",
 StyleBox["unique",
  FontSlant->"Italic"],
 ". Is it possible for a group to be represented as a direct product of \
cyclic groups in two different ways?\n\nIn one sence, the answer is yes, for \
\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["Z", "12"], " ", "\[Times]", " ", 
      SubscriptBox["Z", "2"]}], " ", 
     StyleBox["\[TildeTilde]",
      FontSize->16], " ", 
     RowBox[{
      SubscriptBox["Z", "6"], " ", "\[Times]", " ", 
      SubscriptBox["Z", "4"]}]}], " "}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Yet this doesn't give the whole story, since both of these can futher \
decomposed:\n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["Z", "12"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["Z", "6"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "4"]}], "  ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "3"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "4"]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
So the question that we should ask is whether an abelian group can be \
decomposed in two different ways into groups which can not be decomposed \
further. A way to reword this question is to ask whether two different \
decompositions are isomorphic. 

Since the main tool we have been using to test whether two groups are \
isomorphic is to count elements of a given order, it is natural to ask how \
many elements there are of a given order for a decomposition of cyclic \
groups. The following lemma gives us the answer.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\nLEMMA 6.5\nLet ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " be a prime number, and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be the direct product of cyclic groups\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", 
     RowBox[{"(", 
      SuperscriptBox["p", 
       SubscriptBox["m", "1"]], ")"}]], "\[Times]", 
    SubscriptBox["Z", 
     RowBox[{"(", 
      SuperscriptBox["p", 
       SubscriptBox["m", "2"]], ")"}]], "\[Times]", 
    SubscriptBox["Z", 
     RowBox[{"(", 
      SuperscriptBox["p", 
       SubscriptBox["m", "3"]], ")"}]], "\[Times]", " ", "\[CenterEllipsis]", 
    " ", "\[Times]", " ", 
    SubscriptBox["Z", 
     RowBox[{"(", 
      SuperscriptBox["p", 
       SubscriptBox["m", "j"]], ")"}]], "\[Times]", " ", 
    SubscriptBox["Z", 
     SubscriptBox["r", "1"]], "\[Times]", 
    SubscriptBox["Z", 
     SubscriptBox["r", "2"]], "\[Times]", " ", "\[CenterEllipsis]", " ", 
    SubscriptBox["Z", 
     SubscriptBox["r", "s"]]}], TraditionalForm]]],
 "\n\n  \nwhere ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "2"], TraditionalForm]]],
 ", \[Ellipsis] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "j"], TraditionalForm]]],
 " are positive integers, and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "2"], TraditionalForm]]],
 ", \[Ellipsis] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "s"], TraditionalForm]]],
 " are coprime to ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Then the number of elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " is given by\n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     StyleBox["p",
      FontSize->18], 
     RowBox[{
      StyleBox["(",
       FontSize->16], 
      RowBox[{
       StyleBox[
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"k", "=", "1"}], "j"],
        FontSize->16], 
       RowBox[{
        StyleBox["Min",
         FontSize->12], 
        StyleBox["(",
         FontSize->16], 
        RowBox[{
         SubscriptBox[
          StyleBox["m",
           FontSize->12], "k"], 
         StyleBox[",",
          FontSize->12], 
         StyleBox[" ",
          FontSize->12], 
         StyleBox["n",
          FontSize->12]}], 
        StyleBox[")",
         FontSize->16]}]}], 
      StyleBox[")",
       FontSize->16]}]], " ", 
    StyleBox["-",
     FontSize->18], 
    StyleBox[" ",
     FontSize->18], 
    SuperscriptBox[
     StyleBox["p",
      FontSize->18], 
     RowBox[{
      StyleBox["(",
       FontSize->16], 
      RowBox[{
       StyleBox[
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"k", "=", "1"}], "j"],
        FontSize->16], 
       RowBox[{
        StyleBox["Min",
         FontSize->12], 
        StyleBox["(",
         FontSize->16], 
        RowBox[{
         SubscriptBox[
          StyleBox["m",
           FontSize->12], "k"], 
         StyleBox[",",
          FontSize->9], 
         StyleBox["  ",
          FontSize->9], 
         StyleBox[
          RowBox[{"n", "-", "1"}],
          FontSize->12]}], 
        StyleBox[")",
         FontSize->16]}]}], 
      StyleBox[")",
       FontSize->16]}]]}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Min", "(", 
    RowBox[{
     SubscriptBox["m", "k"], ",", "n"}], ")"}], TraditionalForm]]],
 " denotes the minimum of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "k"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436637715390625*^9, 3.436637746140625*^9}, {
  3.4415492237608747`*^9, 3.4415492598077497`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We begin by noticing that the number of elements of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " are characterized by the elements ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " such that \n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"(", 
      SuperscriptBox["p", "n"], ")"}]], "=", " ", "e"}], TraditionalForm]]],
 ",   but   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"(", 
      SuperscriptBox["p", 
       RowBox[{"n", "-", "1"}]], ")"}]], "\[NotEqual]", "e"}], 
   TraditionalForm]]],
 ",\n\nwhere ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " is the identity of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Thus, if we can find the number of solutions to the first equation,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"(", 
       SuperscriptBox["p", "n"], ")"}]], "=", " ", "e"}], ","}], 
   TraditionalForm]]],
 " we are on our way to finding the number of elements of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is expressed as a direct product we can use proposition 6.2 and find the \
number of solutions to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"(", 
      SuperscriptBox["p", "n"], ")"}]], "=", " ", "e"}], TraditionalForm]]],
 " for each factor in the product, and multiply these numbers together. Since \
each factor is cyclic, we can use corollary 2.1. For all of the factors ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    SubscriptBox["r", "1"]], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    SubscriptBox["r", "2"]], TraditionalForm]]],
 ", \[Ellipsis] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    SubscriptBox["r", "s"]], TraditionalForm]]],
 ", there is only one solution to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"(", 
      SuperscriptBox["p", "n"], ")"}]], "=", " ", "e"}], TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"GCD", "(", 
     RowBox[{
      SubscriptBox[
       RowBox[{"r", " "}], "k"], " ", ",", 
      SuperscriptBox[
       RowBox[{"p", " "}], "n"]}], ")"}], " ", "=", " ", "1"}], 
   TraditionalForm]]],
 ". On the other hand, the number of solutions to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"(", 
      SuperscriptBox["p", "n"], ")"}]], "=", " ", "e"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    RowBox[{"(", 
     SuperscriptBox["p", 
      SubscriptBox["m", "k"]], ")"}]], TraditionalForm]]],
 " is \n\n                               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"GCD", "(", 
     RowBox[{
      SuperscriptBox["p", 
       SubscriptBox["m", "k"]], ",", " ", 
      SuperscriptBox["p", "n"]}], " ", ")"}], " ", "=", " ", 
    SuperscriptBox["p", 
     RowBox[{"Min", "(", 
      RowBox[{
       SubscriptBox["m", "k"], ",", "n"}], ")"}]]}], TraditionalForm]]],
 ".\n                               \nThus, the number of solutions to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"(", 
      SuperscriptBox["p", "n"], ")"}]], "=", " ", "e"}], TraditionalForm]]],
 " for the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is the product of  the above for factors 1 through ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", which gives us a grand total of"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436637782328125*^9, 3.43663783365625*^9}, {
  3.4415492625577497`*^9, 3.4415493619483747`*^9}, {3.4415494514639997`*^9, 
  3.4415494514639997`*^9}}],

Cell[TextData[{
 "                               ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["p",
     FontSize->18], 
    RowBox[{
     StyleBox["(",
      FontSize->16], 
     RowBox[{
      StyleBox[
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"k", "=", "1"}], "j"],
       FontSize->16], 
      RowBox[{
       StyleBox["Min",
        FontSize->12], 
       StyleBox["(",
        FontSize->16], 
       RowBox[{
        SubscriptBox[
         StyleBox["m",
          FontSize->12], "k"], 
        StyleBox[",",
         FontSize->12], 
        StyleBox[" ",
         FontSize->12], 
        StyleBox["n",
         FontSize->12]}], 
       StyleBox[")",
        FontSize->16]}]}], 
     StyleBox[")",
      FontSize->16]}]], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436637873*^9, 3.436637876453125*^9}, {
  3.4415494425733747`*^9, 3.4415494443858747`*^9}}],

Cell[TextData[{
 "solutions. However, not all of these solutions will be elements of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ". We have to subtract the number of solutions to the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["y", 
       RowBox[{"(", 
        SuperscriptBox["p", 
         RowBox[{"n", "-", "1"}]], ")"}]], "=", " ", "e"}], ","}], " "}], 
   TraditionalForm]]],
 " giving us"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4415494586671247`*^9, 3.4415494599327497`*^9}}],

Cell[TextData[{
 "              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     StyleBox["p",
      FontSize->18], 
     RowBox[{
      StyleBox["(",
       FontSize->16], 
      RowBox[{
       StyleBox[
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"k", "=", "1"}], "j"],
        FontSize->16], 
       RowBox[{
        StyleBox["Min",
         FontSize->12], 
        StyleBox["(",
         FontSize->16], 
        RowBox[{
         SubscriptBox[
          StyleBox["m",
           FontSize->12], "k"], 
         StyleBox[",",
          FontSize->12], 
         StyleBox[" ",
          FontSize->12], 
         StyleBox["n",
          FontSize->12]}], 
        StyleBox[")",
         FontSize->16]}]}], 
      StyleBox[")",
       FontSize->16]}]], " ", 
    StyleBox["-",
     FontSize->18], 
    StyleBox[" ",
     FontSize->18], 
    SuperscriptBox[
     StyleBox["p",
      FontSize->18], 
     RowBox[{
      StyleBox["(",
       FontSize->16], 
      RowBox[{
       StyleBox[
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"k", "=", "1"}], "j"],
        FontSize->16], 
       RowBox[{
        StyleBox["Min",
         FontSize->12], 
        StyleBox["(",
         FontSize->16], 
        RowBox[{
         SubscriptBox[
          StyleBox["m",
           FontSize->12], "k"], 
         StyleBox[",",
          FontSize->9], 
         StyleBox["  ",
          FontSize->9], 
         StyleBox[
          RowBox[{"n", "-", "1"}],
          FontSize->12]}], 
        StyleBox[")",
         FontSize->16]}]}], 
      StyleBox[")",
       FontSize->16]}]]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.436637888328125*^9, {3.4415494627764997`*^9, 3.4415494666983747`*^9}}],

Cell[TextData[{
 "elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4415494731046247`*^9, 3.4415494731046247`*^9}}]
}, Open  ]],

Cell[TextData[{
 "An example here would help to illustrate the power of lemma 6.5. Suppose we \
are given the group\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     SubscriptBox["Z", 
      RowBox[{"3", " "}]], "\[Times]", " ", 
     SubscriptBox["Z", "9"], " ", "\[Times]", " ", 
     SubscriptBox["Z", 
      RowBox[{"9", " "}]], "\[Times]", 
     RowBox[{
      SubscriptBox["Z", "27"], "."}]}]}], TraditionalForm]]],
 "\n             \n How many elements are there of order 9?  Note that we can \
rewrite\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " "}], TraditionalForm]]],
 " as \n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", 
     SuperscriptBox["3", "1"]], " ", "\[Times]", 
    SubscriptBox["Z", 
     SuperscriptBox["3", "2"]], " ", "\[Times]", 
    SubscriptBox["Z", 
     SuperscriptBox["3", "2"]], "\[Times]", " ", 
    RowBox[{
     SubscriptBox["Z", 
      SuperscriptBox["3", "3"]], "."}]}], TraditionalForm]]],
 "\n              \n Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["m", "1"], " ", "=", " ", "1"}], ",", " ", 
    RowBox[{
     SubscriptBox["m", "2"], "=", "2"}], ",", " ", 
    RowBox[{
     SubscriptBox["m", "3"], "=", "2"}], ","}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["m", "4"], "=", "3"}], TraditionalForm]]],
 ". Since we want the elements of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["3", "2"], ","}], TraditionalForm]]],
 " we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "2"}], TraditionalForm]]],
 ". Then\n \n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox[
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"k", "=", "1"}], "4"],
     FontSize->16], 
    RowBox[{
     StyleBox["Min",
      FontSize->12], 
     StyleBox["(",
      FontSize->16], 
     RowBox[{
      SubscriptBox[
       StyleBox["m",
        FontSize->12], "k"], 
      StyleBox[",",
       FontSize->12], 
      StyleBox[" ",
       FontSize->12], 
      StyleBox["n",
       FontSize->12]}], 
     StyleBox[")",
      FontSize->16]}]}], TraditionalForm]]],
 " = 1 + 2 + 2 + 2 = 7, \nand \n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox[
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"k", "=", "1"}], "4"],
     FontSize->16], 
    RowBox[{
     StyleBox["Min",
      FontSize->12], 
     StyleBox["(",
      FontSize->16], 
     RowBox[{
      SubscriptBox[
       StyleBox["m",
        FontSize->12], "k"], 
      StyleBox[",",
       FontSize->12], 
      StyleBox[" ",
       FontSize->12], 
      RowBox[{
       StyleBox["n",
        FontSize->12], "-", "1"}]}], 
     StyleBox[")",
      FontSize->16]}]}], TraditionalForm]]],
 " = 1 + 1 + 1 + 1 = 4.\n          \nso the total number of elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of order 9 is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["3", "7"], "-", 
     SuperscriptBox["3", "4"]}], "=", " ", "2106."}], TraditionalForm]]],
 "\n\nEXPERIMENT:\n\nHow many elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " have order 27?  (Use ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "3"}], TraditionalForm]]],
 " instead of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "2"}], TraditionalForm]]],
 ")."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.4366379031875*^9, {3.4415494947139997`*^9, 3.4415495231827497`*^9}}],

Cell["      ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 Cell["", "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False],
 "We are now ready to show that all finite abelian groups can be represented \
as the direct product of cyclic groups. However, we would like to show at the \
same time that such a representation is unique. To this end we will use the \
previous lemma in conjunction with the following:"
}], "Text"],

Cell[TextData[{
 "LEMMA 6.6\n   Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["m", "1"], ",", " ", 
    SubscriptBox["m", "2"], ",", " ", 
    SubscriptBox["m", "3"], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     SubscriptBox["m", "j"]}]}], TraditionalForm]]],
 " be a set of positive integers, and define ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "n", ")"}], TraditionalForm]]],
 " as\n   \n",
 Cell[BoxData[
  FormBox[
   RowBox[{"                                   ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "n", ")"}], " ", "=", " ", 
     RowBox[{
      StyleBox[
       UnderoverscriptBox[
        SubscriptBox[
         StyleBox["\[Sum]",
          FontSize->24], " "], 
        RowBox[{"k", "=", "1"}], 
        RowBox[{"j", " "}]],
       FontSize->16], 
      RowBox[{"Min", "(", 
       RowBox[{
        SubscriptBox["m", "k"], 
        StyleBox[",",
         FontSize->12], 
        StyleBox["n",
         FontSize->12]}], ")"}]}]}]}], TraditionalForm]], "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Min", "(", 
    RowBox[{
     SubscriptBox["m", "k"], ",", " ", "n"}], ")"}], TraditionalForm]]],
 " denotes the minimum of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "k"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Then the number of times that the integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " appears in the set of integers\n ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "2"], TraditionalForm]]],
 ", \[Ellipsis], ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "j"], TraditionalForm]]],
 " is given by              \n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", " ", 
     RowBox[{"\[Florin]", "(", "n", ")"}]}], " ", "-", " ", 
    RowBox[{"\[Florin]", "(", 
     RowBox[{"n", "-", "1"}], ")"}], " ", "-", " ", 
    RowBox[{"\[Florin]", "(", 
     RowBox[{"n", "+", "1"}], ")"}]}], TraditionalForm]]],
 "."
}], "Text",
 CellChangeTimes->{{3.4366422920625*^9, 3.436642292171875*^9}, {
  3.4415495278702497`*^9, 3.4415495613233747`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let us begin by observing the value of the expression\n\n                   \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", " ", 
     RowBox[{"Min", "(", 
      RowBox[{
       SubscriptBox["m", "k"], ",", " ", "n"}], ")"}]}], " ", "-", " ", 
    RowBox[{"Min", "(", 
     RowBox[{
      SubscriptBox["m", "k"], ",", " ", 
      RowBox[{"n", "-", "1"}]}], ")"}], " ", "-", " ", 
    RowBox[{
     RowBox[{"Min", "(", 
      RowBox[{
       SubscriptBox["m", "k"], ",", 
       RowBox[{"n", "+", "1"}]}], ")"}], "."}]}], TraditionalForm]]],
 "\n                   \nWhen ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["m", "k"], " ", "<", " ", "n"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      RowBox[{
       FormBox[
        RowBox[{"Min", "(", 
         SubscriptBox["m", "k"]}],
        TraditionalForm], ",", " ", "n"}], ")"}],
     TraditionalForm], " ", "=", " ", 
    RowBox[{
     RowBox[{"Min", "(", 
      RowBox[{
       SubscriptBox["m", "k"], ",", " ", 
       RowBox[{"n", "-", "1"}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"Min", "(", 
       RowBox[{
        SubscriptBox["m", "k"], ",", " ", 
        RowBox[{"n", "+", "1"}]}], ")"}], " ", "=", " ", 
      SubscriptBox["m", "k"]}]}]}], TraditionalForm]]],
 " , and so the above evaluates to ",
 Cell[BoxData[
  FormBox["0.", TraditionalForm]]],
 " On the other hand, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["m", "k"], " ", ">", " ", "n"}], TraditionalForm]]],
 ", then the above expression simplifies to be\n  \n                          \
    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"(", "n", ")"}]}], " ", "-", " ", 
     RowBox[{"(", 
      RowBox[{"n", "-", "1"}], ")"}], " ", "-", " ", 
     RowBox[{"(", 
      RowBox[{"n", "+", "1"}], ")"}]}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 ".\n\nHowever, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["m", "k"], " ", "=", " ", "n"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"Min", "(", 
        SubscriptBox["m", "k"]}],
       TraditionalForm], ",", " ", "n"}], ")"}], " ", "=", "n"}], 
   TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Min", "(", 
     RowBox[{
      SubscriptBox["m", "k"], ",", " ", 
      RowBox[{"n", "-", "1"}]}], ")"}], " ", "=", " ", 
    RowBox[{"n", "-", "1"}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Min", "(", 
     RowBox[{
      SubscriptBox["m", "k"], ",", " ", 
      RowBox[{"n", "+", "1"}]}], ")"}], " ", "=", " ", "n"}], 
   TraditionalForm]]],
 ". Hence, we have \n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", 
       RowBox[{"Min", "(", 
        RowBox[{
         SubscriptBox["m", "k"], ",", " ", "n"}], ")"}]}], " ", "-", " ", 
      RowBox[{"Min", "(", 
       RowBox[{
        SubscriptBox["m", "k"], ",", " ", 
        RowBox[{"n", "-", "1"}]}], ")"}], " ", "-", " ", 
      RowBox[{"Min", "(", 
       RowBox[{
        SubscriptBox["m", "k"], ",", " ", 
        RowBox[{"n", "+", "1"}]}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"2", " ", "n"}], " ", "-", " ", 
       RowBox[{"(", 
        RowBox[{"n", "-", "1"}], ")"}], " ", "-", " ", "n"}], " ", "=", " ", 
      "1."}]}]}], TraditionalForm]]],
 "\n\nThus, we see that \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"Min", "(", 
       RowBox[{
        SubscriptBox["m", "k"], ",", " ", "n"}], ")"}]}], " ", "-", " ", 
     RowBox[{"Min", "(", 
      RowBox[{
       SubscriptBox["m", "k"], ",", " ", 
       RowBox[{"n", "-", "1"}]}], ")"}], " ", "-", " ", 
     RowBox[{"Min", "(", 
      RowBox[{
       RowBox[{
        SubscriptBox["m", 
         RowBox[{
          RowBox[{"k", ","}], " "}]], "n"}], "+", "1"}], ")"}]}], " ", "=", 
    "   ", 
    RowBox[{
     StyleBox["{",
      FontSize->14], GridBox[{
       {"1", 
        RowBox[{
         RowBox[{"if", "   ", 
          SubscriptBox["m", "k"]}], "=", " ", "n"}]},
       {"0", 
        RowBox[{
         RowBox[{"if", "   ", 
          SubscriptBox["m", "k"]}], "\[NotEqual]", " ", "n"}]}
      }], " "}]}], TraditionalForm]]],
 ".\n\nThus, if we sum the above expression for ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " going from 1 to ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", we will count the number of terms ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "k"], TraditionalForm]]],
 " which are equal to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Hence this count will be\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       StyleBox[
        UnderoverscriptBox[
         SubscriptBox[
          StyleBox["\[Sum]",
           FontSize->24], " "], 
         RowBox[{"k", "=", "1"}], 
         RowBox[{"j", " "}]],
        FontSize->16], "  ", 
       RowBox[{
        StyleBox["2",
         FontSize->12], 
        StyleBox[" ",
         FontSize->12], 
        RowBox[{
         StyleBox["Min",
          FontSize->12], 
         StyleBox["(",
          FontSize->12], 
         RowBox[{
          SubscriptBox["m", "k"], 
          StyleBox[",",
           FontSize->12], 
          StyleBox["n",
           FontSize->12]}], 
         StyleBox[")",
          FontSize->12]}]}]}], 
      StyleBox["-",
       FontSize->12], 
      RowBox[{
       StyleBox["Min",
        FontSize->12], 
       StyleBox["(",
        FontSize->12], 
       RowBox[{
        SubscriptBox["m", "k"], 
        StyleBox[",",
         FontSize->12], 
        StyleBox[
         RowBox[{"n", "-", "1"}],
         FontSize->12]}], 
       StyleBox[")",
        FontSize->12]}], 
      StyleBox["-",
       FontSize->12], 
      RowBox[{
       StyleBox["Min",
        FontSize->12], 
       StyleBox["(",
        FontSize->12], 
       RowBox[{
        SubscriptBox["m", "k"], 
        StyleBox[",",
         FontSize->12], 
        StyleBox[
         RowBox[{"n", "+", "1"}],
         FontSize->12]}], 
       StyleBox[")",
        FontSize->12]}]}], " ", "=", "\n", "\t                           ", 
     RowBox[{
      StyleBox[
       RowBox[{"2", " ", 
        RowBox[{"\[Florin]", "(", "n", ")"}]}],
       FontSize->12], 
      StyleBox[" ",
       FontSize->12], 
      StyleBox["-",
       FontSize->12], 
      StyleBox[" ",
       FontSize->12], 
      StyleBox[
       RowBox[{"\[Florin]", "(", 
        RowBox[{"n", "-", "1"}], ")"}],
       FontSize->12], 
      StyleBox[" ",
       FontSize->12], 
      StyleBox["-",
       FontSize->12], 
      StyleBox[" ",
       FontSize->12], 
      RowBox[{
       StyleBox[
        RowBox[{"\[Florin]", "(", 
         RowBox[{"n", "+", "1"}], ")"}],
        FontSize->12], 
       StyleBox[".",
        FontSize->12], "   "}]}]}]}], TraditionalForm]], "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4415495784639997`*^9, 3.4415497097608747`*^9}}]
}, Open  ]],

Cell["\<\
We can now use lemmas 6.3 through 6.6 to show that not only can a finite \
abelian group be decomposed into a direct product of cyclic groups, but that \
it can be decomposed into a standard form in a unique way, much as a positive \
integer can be uniquely decomposed into a product of prime numbers.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436642356359375*^9, 3.436642359390625*^9}}],

Cell[TextData[{
 "\nTHEOREM 6.2:  THE FUNDAMENTAL THEOREM OF FINITE ABELIAN GROUPS\nA \
nontrivial finite abelian group is isomorphic to\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["Z", 
      RowBox[{"(", 
       SuperscriptBox[
        SubscriptBox["p", "1"], 
        SubscriptBox["m", "1"]], ")"}]], " ", "\[Times]"}], " "}], 
   TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["Z", 
      RowBox[{"(", 
       SuperscriptBox[
        SubscriptBox["p", "2"], 
        SubscriptBox["m", "2"]], ")"}]], " ", "\[Times]"}], " "}], 
   TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["Z", 
      RowBox[{"(", 
       SuperscriptBox[
        SubscriptBox["p", "3"], 
        SubscriptBox["m", "3"]], ")"}]], " ", "\[Times]"}], " "}], 
   TraditionalForm]]],
 "... \[Times]  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    RowBox[{"(", 
     SuperscriptBox[
      SubscriptBox["p", "s"], 
      SubscriptBox["m", "s"]], ")"}]], TraditionalForm]]],
 " ,\n        \nwhere ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "3"], TraditionalForm]]],
 ", \[Ellipsis] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "s"], TraditionalForm]]],
 " are primes numbers (not necessarily distinct). Furthermore, this \
decomposition is unique up to the rearrangement of the factors.           "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436642368234375*^9, 3.436642368359375*^9}, {
  3.4415497465264997`*^9, 3.4415497499171247`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We will proceed on induction on the order of the group. If the order of the \
group is 2 then the theorem is true since the group would be isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a finite abelian group and suppose the theorem is true for all groups \
of order less than ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " be a prime that divides the order of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". By lemma 6.3,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"P", " ", "\[Times]", " ", "K"}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", " ", "is", " ", "the"}], TraditionalForm]]],
 " subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " containing the elements of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".\n\nFurthermore, if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is an element of maximal order in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " is the group generated by ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", then by lemma 6.4  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"X", " ", "\[Times]", " ", "T", " ", "\[Times]", " ", "K"}]}], 
   TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " will be a nontrivial cyclic group the orders of ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " will be less than ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Thus, by induction, ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " can be written as a direct product of cyclic groups whose orders are \
powers of primes. Since ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " is also a cyclic group of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "r"], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " can be written as a direct product of cyclic groups whose orders are \
powers of primes.\n\nWe next have to show that this decomposition is ",
 StyleBox["unique. ",
  FontSlant->"Italic"],
 " We will do this by showing that the number of times ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    RowBox[{"(", 
     SuperscriptBox["p", "n"], ")"}]], TraditionalForm]]],
 "  appears in the decomposition, where ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime, is completely determined by the order of the elements in the \
group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". From lemma 6.5, The number of elements of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " is given by"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436642401046875*^9, 3.436642413234375*^9}, {
  3.43664245234375*^9, 3.43664245246875*^9}, {3.4415497833858747`*^9, 
  3.4415497885108747`*^9}}],

Cell[TextData[{
 "             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     StyleBox["p",
      FontSize->18], 
     RowBox[{"\[Sum]", "  ", 
      RowBox[{
       StyleBox["Min",
        FontSize->12], 
       StyleBox["(",
        FontSize->16], 
       RowBox[{
        SubscriptBox[
         StyleBox["m",
          FontSize->12], "k"], 
        StyleBox[",",
         FontSize->12], 
        StyleBox[" ",
         FontSize->12], 
        StyleBox["n",
         FontSize->12]}], 
       StyleBox[")",
        FontSize->16]}]}]], " ", 
    StyleBox["-",
     FontSize->18], 
    StyleBox[" ",
     FontSize->18], 
    SuperscriptBox[
     StyleBox["p",
      FontSize->18], 
     RowBox[{"\[Sum]", "  ", 
      RowBox[{
       StyleBox["Min",
        FontSize->12], 
       StyleBox["(",
        FontSize->16], 
       RowBox[{
        SubscriptBox[
         StyleBox["m",
          FontSize->12], "k"], 
        StyleBox[",",
         FontSize->9], 
        StyleBox["  ",
         FontSize->9], 
        StyleBox[
         RowBox[{"n", "-", "1"}],
         FontSize->12]}], 
       StyleBox[")",
        FontSize->16]}]}]]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4415497923702497`*^9, 3.4415498690889997`*^9}}],

Cell[TextData[{
 "where the sum is taken over all ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["p", "k"],
     TraditionalForm], " ", "=", " ", "p"}], TraditionalForm]]],
 ". Thus, we see that \n\n                                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Florin]", "p"], "(", "n", ")"}], " ", "=", " ", 
    RowBox[{
     UnderscriptBox[
      RowBox[{"  ", 
       StyleBox["\[Sum]",
        FontSize->18]}], 
      RowBox[{
       SubscriptBox["p", "k"], "=", " ", "p"}]], "  ", 
     RowBox[{"Min", "(", 
      RowBox[{
       SubscriptBox["m", "k"], ",", "n"}], ")"}]}]}], TraditionalForm]]],
 "\n\nwill be completely determined by the order of the elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". But then by lemma 6.6 the number of times that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    RowBox[{"(", 
     SuperscriptBox["p", "n"], ")"}]], TraditionalForm]]],
 " appears in the decomposition is given by\n                                 \
   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"   ", 
    RowBox[{
     RowBox[{"2", "  ", 
      RowBox[{
       SubscriptBox["\[Florin]", "p"], "(", "n", ")"}]}], " ", "-", " ", 
     RowBox[{
      SubscriptBox["\[Florin]", "p"], "(", 
      RowBox[{"n", "-", "1"}], ")"}], " ", "-", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "p"], "(", 
       RowBox[{"n", "+", "1"}], ")"}], "."}]}]}], TraditionalForm]]],
 "\n\nHence, the decomposition of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " as a direct product of cyclic groups of the form ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    RowBox[{"(", 
     SuperscriptBox["p", "n"], ")"}]], TraditionalForm]]],
 " is unique."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436642483296875*^9, 3.436642487703125*^9}, {
  3.4415499051358747`*^9, 3.4415499347921247`*^9}}]
}, Open  ]],

Cell[TextData[{
 "From this theorem, we can easily find all non-isomorphic abelian groups of \
a given order. For example, to find all non-isomorphic abelian groups of \
order 16, we note that all such groups are direct products of the cyclic \
groups of orders 2, 4, 8, or 16. We want to find all possible combinations of \
2, 4, 8, and 16 which will multiply to give 16. With a little experimenting, \
we find that there are 5 such combinations: \n\n                          2\
\[CenterDot]2\[CenterDot]2\[CenterDot]2,\n                            2\
\[CenterDot]2\[CenterDot]4,\n                              4\[CenterDot]4,\n  \
                            2\[CenterDot]8,\n                     and     16.\
\n\nThus, there are 5 possible abelian groups of order 16:\n                  \
                   \n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "4"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], "\[Times]", 
    SubscriptBox["Z", "4"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "8"]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "16."], TraditionalForm]]],
 "\n\nNotice that there is a correlation between these five groups, and the \
five ways we can\nexpress the number 4 as a sum of positive integers:\n\n \
1+1+1+1 = 4\n 1+1+2 = 4\n 2+2 = 4\n 1+3 = 4\n 4 = 4\n \nSince the fundamental \
theorem (6.2) also states that the representation is unique, these five \
groups must be non-isomorphic to each other. We can state this result in \
general as follows:\n\nCOROLLARY 6.2\nLet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "n", ")"}], TraditionalForm]]],
 " denote the number of ways in which ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be expressed as a sum of positive integers, without regard to order. \
Then if ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime number, there are exactly ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "n", ")"}], TraditionalForm]]],
 " non-isomorphic abelian groups of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436642507859375*^9, 3.436642529953125*^9}, {
  3.4415509205733747`*^9, 3.4415509294014997`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "By the fundamental theorem of abelian groups (6.2), every abelian group of \
order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " must be isomorphic to\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", 
     RowBox[{"(", 
      SuperscriptBox["p", 
       SubscriptBox["m", "1"]], ")"}]], " ", "\[Times]", "  ", 
    SubscriptBox["Z", 
     RowBox[{"(", 
      SuperscriptBox["p", 
       SubscriptBox["m", "2"]], ")"}]], " ", "\[Times]", " ", 
    SubscriptBox["Z", 
     RowBox[{"(", 
      SuperscriptBox["p", 
       SubscriptBox["m", "3"]], ")"}]], " ", "\[Times]", " ", 
    "\[CenterEllipsis]", " ", "\[Times]", " ", 
    SubscriptBox["Z", 
     RowBox[{"(", 
      SuperscriptBox["p", 
       SubscriptBox["m", "s"]], ")"}]]}], TraditionalForm]]],
 ".\nAlso,\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", 
     SubscriptBox["m", "1"]], "\[CenterDot]"}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    SubscriptBox["m", "2"]], TraditionalForm]]],
 "\[CenterDot]",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    SubscriptBox["m", "3"]], TraditionalForm]]],
 "\[CenterDot] \[CenterEllipsis] \[CenterDot] ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", 
     SubscriptBox["m", "s"]], " ", "=", " ", 
    SuperscriptBox["q", "n"]}], TraditionalForm]]],
 ".\n                 \nHence",
 Cell[BoxData[
  FormBox[
   RowBox[{"   ", 
    RowBox[{
     RowBox[{
      SubscriptBox["m", "1"], "+", " ", 
      SubscriptBox["m", "2"], "+", " ", 
      SubscriptBox["m", "3"], "+", " ", "\[Ellipsis]", " ", "+", " ", 
      SubscriptBox["m", "s"]}], " ", "=", " ", "n"}]}], TraditionalForm]]],
 ".  Furthermore, the decomposition of the abelian group is unique up to \
rearrangement of the factors. Thus, there is a one-to-one correspondence \
between non-isomorphic abelian groups of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " and ways ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be written as a sum of positive integers without regard to order."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436642537421875*^9, 3.4366425533125*^9}, {
  3.4415509339639997`*^9, 3.4415509682764997`*^9}}]
}, Open  ]],

Cell[TextData[{
 "We call ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "n", ")"}], TraditionalForm]]],
 " the number of ",
 StyleBox["partitions ",
  FontSlant->"Italic"],
 "of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". We can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " count the number of partitions for us, using the command \n\n",
 StyleBox["PartitionsP",
  FontWeight->"Bold"],
 "[ n ]\n\nThus, to find the number of partitions of the number 4, we can \
enter"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436642572484375*^9, 3.436642572484375*^9}, {
  3.4415509887452497`*^9, 3.4415509887452497`*^9}}],

Cell[CellGroupData[{

Cell["PartitionsP[4]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["5"], "Output",
 CellChangeTimes->{3.442673596523875*^9}]
}, Open  ]],

Cell[TextData[{
 "According to corollary 6.2, the number of non-isomorphic abelian groups of \
order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["3", "100"], TraditionalForm]]],
 " is "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436642593015625*^9, 3.436642593171875*^9}}],

Cell[CellGroupData[{

Cell["PartitionsP[100]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["190569292"], "Output",
 CellChangeTimes->{3.442673598180125*^9}]
}, Open  ]],

Cell[TextData[{
 "We won't list them all here!  As we can see, the number of partitions of n \
grows very rapidly as ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " gets large. How rapidly?  We can make a logarithmic plot of the first 100 \
values of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "n", ")"}], TraditionalForm]]],
 " with the following command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ListPlot[Table[
  N[Log[PartitionsP[n]]],{n,1,100}],PlotJoined -> True]  \
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxVy39M1HUcx/GvnsQvQRIzEBSk44cIx48D7kDQFxw/DjjgDu44IAkzymgY
eqBA03VtOtBFUfxymMuNhsmKjPghx69rJCCgiSQGaDVDGJASFDAQoRrvzx99
tu8+e3yfn9fuw1nxb27kOC793++/e/38eYD7/8GM//JqiGCc/hvBlPf1e1Ml
s2QLCIueLAbrl8jboE7xrtU4rJGtIBO93/S7gId128JOGuO/vcaAbI+eG0G5
OTojsgMuDpkuVfuakvlwDxIoO8bNyE6QZ7r2zVtbkF0Q6C0rMD/0ItkVXjob
50cntpLdUOb4PJn73JIswKXrga+W1G4je2JVs+hofOMlsheepn7f9HrXdrI3
zpVFGNjdeZksROP9v3y2dluRfbDme/R0Qac12ReP574T6+t2kP2QlobU8iob
sggeBncG7c/bksVo+GV4rE6zk+yPkaMxCRWKXeQAjKU/zAr1siPvQ9mouHdh
kz05EJWit4V2x5mDoJv9KFs4wrwfnu6qFC/JbvIB7LrWhTeuMgO8q+dqYrc4
rFsLKEO/urmgIXPBMJKc5e0YYj0YNhN5p6SiV6iHIHvqwZmVCrI2BPvcVr95
uMC6BIrCEHs/FZ+6BFGy5Y5NdWQuFE2zG06IzB2ph+LaIYsJXgaZC8ORx34b
gzpZD0ObTuroZutEPRx2yk9Lz+SQteH4rbvHpLmP9Qh4a0rG4xycqUegIuuT
e6O5ZE6K6dn9+Xf7WZeCM9lQd9vehXokbor/3izKIWsjsWKjy2vtYj0K1Q15
brVWe6hH4Z2W4OL+DDIXjbQ1g2OpOtaj0fmCR6uziSt1GZx/Heg+kkzWynA4
fqp95grrMbDly+7PzLMeg7ARYxuJZC/1WJjl/7hsWUzWxuLySd0gHrAeh7cO
Ns88dXajHgcuqzSFl03m5Lhgk+7yWRsZclR2WIqqDN3pvRzDQ+Z+QgVZL8dc
587XxJVkToGDoiX73kdkKNBWky792VVAewVCh8PCTmrIegXMejPay5vJXDze
Dax/HsV50D4egmPPDAvCydp4eJgE7En5kKyPR0PkvUu6ATKXgNuK6soUU0/a
J6Doy73Tl93J2gRcWJ5caYgj6xNgHJ3lV36czClRnKn6NryE7ZVY7rmS80M9
2ysR+uTuqOUQ2yvRKG3O9l5kexUKJxP5fCsv2qvwwUCVflxM1qogLGvpPp1M
1qvQ2Niimcgnc4nou27V7VTJ9onoz1DLAnRsn4ii9sY2/gjbJ8L1IjcwtsT2
aowZBT47Ze1NezX6pws8/hCTtWqsbBkv9Uki69WI9BuRq3LJXBLmDTer5eVs
nwT3wvpJfgPbJ+GLqulbPw2yfRLO3vo4M22O7ZPR2ruY2GUhxD8fRp6U
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  PlotRange->{{0., 100.}, {0., 19.06552642392738}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.442673599648875*^9}]
}, Open  ]],

Cell[TextData[{
 "This looks like a sideways parabola, indicating that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", "n", ")"}], TraditionalForm]]],
 " grows in about the same rate as ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", 
    StyleBox[
     SqrtBox[
      StyleBox["n",
       FontSize->12]],
     FontSize->9]], TraditionalForm]]],
 ".\n\nWe can now find the number of non-isomorphic abelian groups of any \
order, as the following corollary shows.\n\nCOROLLARY 6.3\nLet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "1"}], TraditionalForm]]],
 " be an integer with prime factorization\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      SuperscriptBox[
       SubscriptBox["p", "1"], 
       SubscriptBox["m", "1"]], "\[CenterDot]"}],
     TraditionalForm], 
    FormBox[
     RowBox[{
      SuperscriptBox[
       SubscriptBox["p", "2"], 
       SubscriptBox["m", "2"]], "\[CenterDot]", 
      SuperscriptBox[
       SubscriptBox["p", "3"], 
       SubscriptBox["m", "3"]], "\[CenterDot]", " ", "\[CenterEllipsis]", " ",
       "\[CenterDot]", " ", 
      SuperscriptBox[
       SubscriptBox["p", "k"], 
       SubscriptBox["m", "k"]]}],
     TraditionalForm]}], TraditionalForm]]],
 "\n            \nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["p", "1"], ",", 
    SubscriptBox["p", "2"], ",", 
    SubscriptBox["p", "3"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["p", "k"]}], TraditionalForm]]],
 " are distinct primes.  Then the number of non-isomorphic abelian groups of \
order ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is given by\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"P", "(", 
     SubscriptBox["m", "1"], ")"}], " ", "\[CenterDot]", " ", 
    RowBox[{"P", "(", 
     SubscriptBox["m", "2"], ")"}], " ", "\[CenterDot]", " ", 
    RowBox[{"P", "(", 
     SubscriptBox["m", "3"], ")"}], " ", "\[CenterDot]", " ", 
    "\[Ellipsis]\[Ellipsis]", " ", "\[CenterDot]", " ", 
    RowBox[{
     RowBox[{"P", "(", 
      SubscriptBox["m", "k"], ")"}], "."}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436642615203125*^9, 3.436642637703125*^9}, {
  3.4415511192921247`*^9, 3.4415512077139997`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We know from the fundamental theorem of abelian groups (6.2) that each such \
group is isomorphic to a direct product of cyclic groups whose order is a \
power of a prime. If we collect all factors involving the same primes \
together, we find that such a group is isomorphic to a \ndirect product of a \
series of groups of orders\n\n                            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      SubscriptBox["q", "1"], 
      SubscriptBox["r", "1"]], ","}], " "}], TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      SubscriptBox["q", "2"], 
      SubscriptBox["r", "2"]], ","}], " "}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     SuperscriptBox[
      SubscriptBox["q", "3"], 
      SubscriptBox["r", "3"]], ",", "    ", "\[CenterEllipsis]", " ", ",", 
     " "}]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["q", "n"], 
    SubscriptBox["r", 
     RowBox[{"n", " "}]]], TraditionalForm]]],
 ".\n\nWe know from corollary 6.2 that there are exactly ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "(", "r", ")"}], TraditionalForm]]],
 " non-isomorphic abelian groups of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["q", "r"], TraditionalForm]]],
 ". Thus, there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "(", 
    SubscriptBox["r", "i"], ")"}], TraditionalForm]]],
 " possible groups for\nthe ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "-th factor in this decomposition. Therefore, there are\n\n                  \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "(", 
     SubscriptBox["r", "1"], ")"}], " ", "\[CenterDot]", " ", 
    RowBox[{"p", "(", 
     SubscriptBox["r", "2"], ")"}], " ", "\[CenterDot]", " ", 
    RowBox[{"p", "(", 
     SubscriptBox["r", "3"], ")"}], " ", "\[CenterDot]", " ", 
    "\[Ellipsis]\[Ellipsis]", " ", "\[CenterDot]", " ", 
    RowBox[{"p", "(", 
     SubscriptBox["r", "n"], ")"}]}], TraditionalForm]]],
 "\n\npossible ways of forming a product of groups with orders \n\n           \
        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      SubscriptBox["q", "1"], 
      SubscriptBox["r", "1"]], ","}], " "}], TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      SubscriptBox["q", "2"], 
      SubscriptBox["r", "2"]], ","}], " "}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     SuperscriptBox[
      SubscriptBox["q", "3"], 
      SubscriptBox["r", "3"]], ",", "    ", "\[CenterEllipsis]", " ", ",", 
     " "}]}], TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["q", "n"], 
    SubscriptBox["r", 
     RowBox[{"n", " "}]]], TraditionalForm]]],
 ".\n\nSince the fundamental theorem of abelian groups (6.2) also states that \
the decomposition is unique up to the rearrangement of the factors. Thus, \
every group thus formed is isomorphically different. So we have exactly    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "(", 
     SubscriptBox["r", "1"], ")"}], " ", "\[CenterDot]", " ", 
    RowBox[{"p", "(", 
     SubscriptBox["r", "2"], ")"}], " ", "\[CenterDot]", " ", 
    RowBox[{"p", "(", 
     SubscriptBox["r", "3"], ")"}], " ", "\[CenterDot]", " ", 
    "\[Ellipsis]\[Ellipsis]", " ", "\[CenterDot]", " ", 
    RowBox[{"p", "(", 
     SubscriptBox["r", "n"], ")"}]}], TraditionalForm]]],
 " non-isomorphic abelian groups of order ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43664264275*^9, 3.436642642890625*^9}, {
  3.436642728796875*^9, 3.436642794015625*^9}}]
}, Open  ]],

Cell[TextData[{
 "This can be demonstrated. Suppose we want to find the number of \
non-isomorphic abelian groups of order 180 billion.  Since 180,000,000,000 = \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "11"], "\[CenterDot]", 
    SuperscriptBox["3", "2"], "\[CenterDot]", 
    SuperscriptBox["5", "10"]}], TraditionalForm]]],
 ", we have that the number of groups is"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["PartitionsP[11] * PartitionsP[2] * PartitionsP[10]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["4704"], "Output",
 CellChangeTimes->{3.4426736082895*^9}]
}, Open  ]],

Cell[TextData[{
 "So there are 4704 abelian groups of order 180 billion.  \n\nFrom these two \
corollaries, we see that all finite abelian groups have been classified. One \
of the outstanding problems in group theory is to classify ",
 StyleBox["all",
  FontSlant->"Italic"],
 " finite groups. This is as yet an unsolved problem although much progress \
has been made through the use of computers. In the next two sections we will \
show some other ways of generating larger groups which have become a key to \
some of the recent work that has been done in group theory. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Automorphisms", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We have already studied several examples of homomorphisms and isomorphisms \
",
 StyleBox["between",
  FontSlant->"Italic"],
 " two groups, but suppose we considered a mapping from a group ",
 StyleBox["to itself",
  FontSlant->"Italic"],
 ". For example, we could consider the following mapping from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 " onto itself:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.436692473875*^9}],

Cell[CellGroupData[{

Cell["\<\
DefMultMod[8]
CircleGraph[{0,1,2,3,4,5,6,7},Mult[3]]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"4\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.7071067811865476, -0.6071067811865475}}], 
      PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
       0.7571067811865476, -0.6071067811865475}, {
       0.6571067811865475, -0.6071067811865475}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865476, -0.7071067811865475}, {0.7071067811865475,
        0.6071067811865476}}], 
      PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.6571067811865474, 0.6071067811865476}, {0.7571067811865475, 
       0.6071067811865476}}]}}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{1., 
       6.123233995736766*^-17}, {-0.9, -1.7145055188062944`*^-16}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9, \
-0.05000000000000017}, {-0.9, 0.049999999999999836`}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {0.9, 
       4.898587196589413*^-17}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {0.9, 0.05000000000000005}, {
       0.9, -0.049999999999999954`}}]}}, 
    {RGBColor[1, 0.6, 0], 
     LineBox[{{0.07071067811865486, -0.9292893218813453}, {
      0.09241885630108188, -0.8968331206248827}, {
      0.09998489885977792, -0.8585786437626906}, {
      0.09241885630108186, -0.8203241669004984}, {
      0.07071067811865485, -0.7878679656440357}, {
      0.03825447686219231, -0.7661597874616087}, {
      9.290959971541795*^-17, -0.7585937449029128}, {-0.03825447686219212, \
-0.7661597874616087}, {-0.07071067811865465, -0.7878679656440357}, \
{-0.09241885630108167, -0.8203241669004984}, {-0.0999848988597777, \
-0.8585786437626906}, {-0.09241885630108165, -0.8968331206248827}, \
{-0.07071067811865464, -0.9292893218813453}, {
      1.2246467991473532`*^-16, -1.}}], 
     PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
      0.10606601717798224`, -0.9646446609406726}, {
      0.03535533905932749, -0.8939339828220179}}]}, {
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.7071067811865477, 0.6071067811865474}}], 
      PolygonBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.7571067811865476, 
       0.6071067811865474}, {-0.6571067811865476, 0.6071067811865474}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.7071067811865475, -0.6071067811865477}}], 
      PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6571067811865475, -0.6071067811865477}, {-0.7571067811865475, \
-0.6071067811865477}}]}}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.422647809671875*^9, 3.4366924881875*^9, 
  3.442673613148875*^9}]
}, Open  ]],

Cell["\<\
Notice that this mapping is one-to-one and onto, so in fact we could consider \
this as a permutation of the numbers 0 through 7, namely (13)(26)(57). In \
fact, since this mapping leaves 0 fixed, we can define this as the \
permutation\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["F = P[3, 6, 1, 4, 7, 2, 5]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"P", "[", 
  RowBox[{"3", ",", "6", ",", "1", ",", "4", ",", "7", ",", "2", ",", "5"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.4226478170625*^9, 3.436692492921875*^9, 
  3.44267361507075*^9}]
}, Open  ]],

Cell[TextData[{
 "Then we can treat ",
 StyleBox["F",
  FontWeight->"Bold"],
 " as a function! That is, we can plug values into ",
 StyleBox["F",
  FontWeight->"Bold"],
 " as we would a standard function. For example, we can evaluate"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["F[2]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["6"], "Output",
 CellChangeTimes->{3.422647822984375*^9, 3.4366924956875*^9, 
  3.442673616430125*^9}]
}, Open  ]],

Cell["and", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["F[0]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.422647825109375*^9, 3.442673617398875*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that if we plug in a value not mentioned in the permutation, ",
 StyleBox["F",
  FontWeight->"Bold"],
 " returns the same value.  \n\nWe now can ask whether ",
 StyleBox["F",
  FontWeight->"Bold"],
 " is a homomorphism. We first must define the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
DefSumMod[8]
Z8 = Group[{1}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7"}],
   "}"}]], "Output",
 CellChangeTimes->{3.422647831328125*^9, 3.4426736186645*^9}]
}, Open  ]],

Cell[TextData[{
 "We can now have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " check to see if ",
 StyleBox["F",
  FontWeight->"Bold"],
 ", as defined on ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 ", is a homomorphism."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[F,Z8]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.422647833984375*^9, 3.442673620055125*^9}]
}, Open  ]],

Cell[TextData[{
 "So the permutation ",
 StyleBox["F",
  FontWeight->"Bold"],
 " is a homomorphism from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 "  onto itself. We give such a homomorphism a special name.\n\nDEFINITION \
6.3:\nAn ",
 StyleBox["automorphism",
  FontSlant->"Italic"],
 " of the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a homomorphism from ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which is one-to-one and onto.                 \n\nIf we study the above \
automorphism ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " on ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 ", we discover why this works. We defined ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " so that each element of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " maps to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", "x", " ", 
    RowBox[{"(", 
     RowBox[{"Mod", " ", "8"}], ")"}]}], TraditionalForm]]],
 ". Therefore,\n\n   ",
 Cell[BoxData[{
  RowBox[{
   FormBox[
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{"x", "+", "y"}], ")"}], " ", "\[Congruent]", " ", 
     RowBox[{"3", 
      RowBox[{"(", 
       RowBox[{"x", "+", "y"}], ")"}], "  ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", "8"}], ")"}]}], "  ", "\[Congruent]", " ", 
     RowBox[{
      RowBox[{"3", "x"}], "+", 
      RowBox[{"3", "y", "  ", 
       RowBox[{"(", 
        RowBox[{"Mod", " ", "8"}], ")"}]}]}]}],
    TraditionalForm], "  "}], "\n", 
  RowBox[{"                     ", 
   RowBox[{"\[Congruent]", " ", 
    RowBox[{
     RowBox[{"\[Florin]", 
      RowBox[{"(", "x", ")"}]}], " ", "+", " ", 
     RowBox[{
      RowBox[{"\[Florin]", 
       RowBox[{"(", "y", ")"}]}], "."}]}]}]}]}]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43669256953125*^9, 3.436692570796875*^9}}],

Cell[TextData[{
 "By observing this pattern, we find another automorphism ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 " by letting ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " send ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " into ",
 Cell[BoxData[
  FormBox[
   RowBox[{"5", "x", " ", 
    RowBox[{"(", 
     RowBox[{"Mod", " ", "8"}], ")"}]}], TraditionalForm]]],
 ". The graph of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " would be given by"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
DefMultMod[8]
CircleGraph[{0,1,2,3,4,5,6,7},Mult[5]]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"4\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865475, 
       0.7071067811865476}, {-0.6363961030678927, -0.636396103067893}}], 
      PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6010407640085653, -0.6717514421272203}, {-0.67175144212722, \
-0.6010407640085655}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.7071067811865475, -0.7071067811865477}, {
       0.6363961030678927, 0.6363961030678928}}], 
      PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.6010407640085653, 0.6717514421272202}, {0.67175144212722, 
       0.6010407640085654}}]}}, 
    {RGBColor[0.6, 0.4, 0.8], 
     LineBox[{{0.9292893218813453, 0.07071067811865481}, {0.8968331206248827, 
      0.09241885630108182}, {0.8585786437626906, 0.09998489885977785}, {
      0.8203241669004984, 0.0924188563010818}, {0.7878679656440357, 
      0.0707106781186548}, {0.7661597874616087, 0.038254476862192265`}, {
      0.7586644555810313, 4.645047007742779*^-17}, {
      0.7661597874616087, -0.03825447686219217}, {
      0.7878679656440357, -0.07071067811865471}, {
      0.8203241669004984, -0.09241885630108172}, {
      0.8585786437626906, -0.09998489885977777}, {
      0.8968331206248827, -0.09241885630108171}, {
      0.9292893218813453, -0.0707106781186547}, {1., 
      6.123233995736766*^-17}}], 
     PolygonBox[{{1., 6.123233995736766*^-17}, {0.9646446609406726, 
      0.10606601717798217`}, {0.8939339828220179, 0.03535533905932743}}]}, {
     {RGBColor[1, 0.6, 0], 
      LineBox[{{0.7071067811865476, -0.7071067811865475}, {-0.636396103067893,
        0.6363961030678926}}], 
      PolygonBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.6717514421272203, 
       0.6010407640085652}, {-0.6010407640085655, 0.6717514421272199}}]}, 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{-0.7071067811865477, 0.7071067811865474}, {
       0.6363961030678928, -0.6363961030678927}}], 
      PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
       0.6717514421272202, -0.6010407640085653}, {
       0.6010407640085654, -0.67175144212722}}]}}, 
    {RGBColor[0.7, 0.8, 0.7], 
     LineBox[{{0.07071067811865486, -0.9292893218813453}, {
      0.09241885630108188, -0.8968331206248827}, {
      0.09998489885977792, -0.8585786437626906}, {
      0.09241885630108186, -0.8203241669004984}, {
      0.07071067811865485, -0.7878679656440357}, {
      0.03825447686219231, -0.7661597874616087}, {
      9.290959971541795*^-17, -0.7585937449029128}, {-0.03825447686219212, \
-0.7661597874616087}, {-0.07071067811865465, -0.7878679656440357}, \
{-0.09241885630108167, -0.8203241669004984}, {-0.0999848988597777, \
-0.8585786437626906}, {-0.09241885630108165, -0.8968331206248827}, \
{-0.07071067811865464, -0.9292893218813453}, {
      1.2246467991473532`*^-16, -1.}}], 
     PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
      0.10606601717798224`, -0.9646446609406726}, {
      0.03535533905932749, -0.8939339828220179}}]}, 
    {RGBColor[0.8, 0.6, 0.5], 
     LineBox[{{-0.9292893218813453, -0.07071067811865492}, \
{-0.8968331206248827, -0.09241885630108193}, {-0.8585786437626904, \
-0.09998489885977796}, {-0.8203241669004983, -0.09241885630108192}, \
{-0.7878679656440357, -0.07071067811865489}, {-0.7661597874616087, \
-0.038254476862192355`}, {-0.7586644555810313, -1.3935141023228338`*^-16}, \
{-0.7661597874616087, 0.03825447686219208}, {-0.7878679656440357, 
      0.07071067811865461}, {-0.8203241669004984, 
      0.09241885630108161}, {-0.8585786437626906, 
      0.09998489885977765}, {-0.8968331206248827, 
      0.0924188563010816}, {-0.9292893218813453, 
      0.07071067811865459}, {-1., -1.8369701987210297`*^-16}}], 
     PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.035355339059327535`}}]}}},
  AspectRatio->Automatic,
  ImageSize->{203.99999999999977`, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4226478490625*^9, 3.44267362257075*^9}]
}, Open  ]],

Cell["which can be represented by the permutation", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = P[5,2,7,4,1,6,3]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"P", "[", 
  RowBox[{"5", ",", "2", ",", "7", ",", "4", ",", "1", ",", "6", ",", "3"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.4226479251875*^9, 3.442673624477*^9}]
}, Open  ]],

Cell["We can check that this is a homomorphism:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
DefSumMod[8]
Z8 = Group[{1}]
CheckHomo[G,Z8]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.422647900859375*^9, 3.42264791065625*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7"}],
   "}"}]], "Output",
 CellChangeTimes->{
  3.422647875453125*^9, {3.422647911390625*^9, 3.42264792846875*^9}, 
   3.44267362744575*^9}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{
  3.422647875453125*^9, {3.422647911390625*^9, 3.42264792846875*^9}, 
   3.44267362744575*^9}]
}, Open  ]],

Cell[TextData[{
 "So we have found 2 automorphisms on ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 ". But these are also permutations, so we could multiply them together.\n\n\
EXPERIMENT:\nIs the permutation"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["H = F.G", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"P", "[", 
  RowBox[{"7", ",", "6", ",", "5", ",", "4", ",", "3", ",", "2", ",", "1"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.422647939875*^9, 3.4426736295395*^9}]
}, Open  ]],

Cell[TextData[{
 "also a automorphism of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["      ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "This experiment suggests that the product of two automorphisms is again an \
automorphism. But there are some other patterns of the automorphisms that we \
can observe. Because an automorphism ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "x", ")"}], TraditionalForm]]],
 " on ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a one-to-one and onto mapping, we can consider the inverse mapping ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], TraditionalForm]]],
 ".  For example, the inverse to the automorphism ",
 StyleBox["F",
  FontWeight->"Bold"],
 " is given by"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["F^(-1)", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"P", "[", 
  RowBox[{"3", ",", "6", ",", "1", ",", "4", ",", "7", ",", "2", ",", "5"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.4426736331645*^9}]
}, Open  ]],

Cell[TextData[{
 "In this case, the inverse of ",
 StyleBox["F",
  FontWeight->"Bold"],
 " is again ",
 StyleBox["F",
  FontWeight->"Bold"],
 ", which is an automorphism.\n\nEXPERIMENT:\nAre the inverses of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " again automorphisms? Which automorphisms are they?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["      ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "Since we seem to be able to multiply two automorphisms together and take \
inverses, the natural question comes up, which is answered in the next \
proposition.\n\nPROPOSITION 6.3\nGiven a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", the set of all automorphisms on ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " forms a group, denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "G", ")"}], TraditionalForm]]],
 ". In fact, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "G", ")"}], TraditionalForm]]],
 " is a subgroup of the group of permutations on the elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436692549421875*^9, 3.436692549515625*^9}}],

Cell[CellGroupData[{

Cell["Proof: ", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The mapping ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"i", "(", "x", ")"}], " ", "=", " ", "x"}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is obviously an automorphism on ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", so the set of all automorphisms on ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is non-empty. Also, each automorphism is a permutation on the elements of \
",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Suppose ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " are two automorphisms on ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", 
    RowBox[{"f", "(", "x", ")"}], ")"}], TraditionalForm]]],
 " is a one-to-one and onto mapping from ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". \n\nFurthermore, \n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", 
     RowBox[{"\[Florin]", "(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Phi]", "(", 
      RowBox[{
       RowBox[{"\[Florin]", "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{"\[Florin]", "(", "y", ")"}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"\[Phi]", "(", 
       RowBox[{"\[Florin]", "(", "x", ")"}], ")"}], "\[CenterDot]", 
      RowBox[{
       RowBox[{"\[Phi]", "(", 
        RowBox[{"\[Florin]", "(", "y", ")"}], ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", 
    RowBox[{"\[Florin]", "(", "x", ")"}], ")"}], TraditionalForm]]],
 " is a homomorphism on ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "\[CenterDot]", "\[Florin]"}], TraditionalForm]]],
 " is an automorphism of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". \n\nAlso, since ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is one-to-one and onto, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Florin]", 
    RowBox[{
     RowBox[{"-", "1"}], " "}]], TraditionalForm]]],
 "exists on ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and \n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Florin]", 
        RowBox[{"-", "1"}]], "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{
       SuperscriptBox["\[Florin]", 
        RowBox[{"-", "1"}]], "(", "y", ")"}]}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", 
       RowBox[{
        SuperscriptBox["\[Florin]", 
         RowBox[{"-", "1"}]], "(", "x", ")"}], ")"}], "\[CenterDot]", 
      RowBox[{"\[Florin]", "(", 
       RowBox[{
        SuperscriptBox["\[Florin]", 
         RowBox[{"-", "1"}]], "(", "y", ")"}], ")"}]}], " ", "=", " ", 
     RowBox[{"x", "\[CenterDot]", 
      RowBox[{"y", "."}]}]}]}], TraditionalForm]]],
 "\n\nTaking ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Florin]", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " of both sides of the equation gives us\n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Florin]", 
         RowBox[{"-", "1"}]], "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{
        SuperscriptBox["\[Florin]", 
         RowBox[{"-", "1"}]], "(", "y", ")"}]}], " ", "=", " ", 
      RowBox[{
       SuperscriptBox["\[Florin]", 
        RowBox[{"-", "1"}]], "(", 
       RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], ","}]}], 
   TraditionalForm]]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Florin]", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " is a homomorphism.  So ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Florin]", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 ", and thus ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "\[CenterDot]", 
     SuperscriptBox["\[Florin]", 
      RowBox[{"-", "1"}]]}], ","}], TraditionalForm]]],
 " are an automorphisms of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Therefore by proposition 2.2, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "G", ")"}], TraditionalForm]]],
 " is a subgroup of the group of permutations on the elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42264811503125*^9, 3.422648139109375*^9}, {
  3.436692653484375*^9, 3.436692654859375*^9}}]
}, Open  ]],

Cell[TextData[{
 "Let's see if we can find the automorphism group for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 ". We have two elements already, namely ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Let's find the group generated by these two elements:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["A = Group[{F,G}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "6", ",", "1", ",", "4", ",", "7", ",", "2", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "2", ",", "7", ",", "4", ",", "1", ",", "6", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "6", ",", "5", ",", "4", ",", "3", ",", "2", ",", "1"}],
     "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.422648314640625*^9, 3.44267363913325*^9}]
}, Open  ]],

Cell[TextData[{
 "So ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"Aut", "(", 
      SubscriptBox["Z", "8"]}],
     TraditionalForm], ")"}], TraditionalForm]]],
 " has at least 4 elements. Is this the maximum number of automorphisms? \
Notice that 1 is of order 8 in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 ", so it must be mapped to an element of order 8, namely 1, 3, 5, or 7.  But \
since 1 is a generator of the group, any automorphism is determined by where \
the generator is mapped to.  Hence, there are at most 4 automorphisms of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 ". Since we have already found 4, we must have found all of them!\n\nSince \
we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["Z", "8"]}], TraditionalForm]]],
 ") is a group of order 4, the natural question is which group is this \
isomorphic to? Here is a multiplication table for A:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[A]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}}, {{0, 1,
     1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {
    1, 1, 0}, {1, 0, 0}, {0, 1, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}, {1, 0, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
    0}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"P[]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"P[3,6,1,4,7,2,5]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"P[5,2,7,4,1,6,3]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"P[7,6,5,4,3,2,1]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"P[]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"P[3,6,1,4,7,2,5]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"P[5,2,7,4,1,6,3]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"P[7,6,5,4,3,2,1]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"P[]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[3,6,1,4,7,2,5]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[5,2,7,4,1,6,3]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[7,6,5,4,3,2,1]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"P[3,6,1,4,7,2,5]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[7,6,5,4,3,2,1]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[5,2,7,4,1,6,3]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"P[5,2,7,4,1,6,3]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[7,6,5,4,3,2,1]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[3,6,1,4,7,2,5]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"P[7,6,5,4,3,2,1]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[5,2,7,4,1,6,3]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[3,6,1,4,7,2,5]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"P[]\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]},
  ImageSize->{607.9999999999974, Automatic}]], "Output",
 CellChangeTimes->{3.42264833540625*^9, 3.4426736422895*^9}]
}, Open  ]],

Cell[TextData[{
 "From this, we see that all elements of A besides the identity are of order \
2.  So\n                  \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", 
     SubscriptBox["Z", "8"], ")"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->14], " ", 
    RowBox[{
     SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We can use a similar argument to find the automorphism group for any cyclic \
group.\n\nPROPOSITION 6.4\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", 
     SubscriptBox["Z", "n"], ")"}], " ", "\[TildeTilde]", " ", 
    SubsuperscriptBox["Z", "n", "*"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43669285165625*^9}],

Cell[CellGroupData[{

Cell["Proof: ", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42264876621875*^9, 3.422648775515625*^9}, {
  3.4226489425625*^9, 3.42264900178125*^9}}],

Cell[TextData[{
 "Consider the mapping        \n                                          \
\[Phi]: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubsuperscriptBox["Z", "n", "*"], " ", "\[RightArrow]"}], " "}], 
   TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", 
     SubscriptBox["Z", "n"], ")"}], " "}], TraditionalForm]]],
 "\n\ngiven by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "j", ")"}], " ", "=", " ", 
    SubscriptBox["\[Florin]", "j"]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Florin]", "j"], "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"j", "\[CenterDot]", "x"}], " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 ". Then given two elements ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["j", "1"], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["j", "2"], TraditionalForm]]],
 "  in ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "n", "*"], TraditionalForm]]],
 ", we have that \n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[Florin]", 
       SubscriptBox["j", "1"]],
      TraditionalForm], "(", 
     RowBox[{
      SubscriptBox["\[Florin]", 
       SubscriptBox["j", "2"]], "(", "x", ")"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["j", "1"], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["j", "2"], "\[CenterDot]", "x"}], ")"}]}], "  ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", "n"}], ")"}]}], " ", "=", "  ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["j", "2"], "\[CenterDot]", 
          SubscriptBox["j", "1"]}], ")"}], "\[CenterDot]", "x"}], " ", 
       RowBox[{"(", 
        RowBox[{"Mod", " ", "n"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Florin]", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["j", "2"], "\[CenterDot]", 
           SubscriptBox["j", "1"]}], ")"}]], "(", "x", ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\n      \nSo \n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "(", 
      SubscriptBox["j", "2"], ")"}], "\[CenterDot]", 
     RowBox[{"\[Phi]", "(", 
      SubscriptBox["j", "1"], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Florin]", 
       SubscriptBox["j", "1"]], "(", " ", 
      SubscriptBox["\[Florin]", 
       SubscriptBox["j", "2"]], ")"}], "=", " ", 
     RowBox[{
      SubscriptBox["\[Florin]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["j", "2"], "\[CenterDot]", 
         SubscriptBox["j", "1"]}], ")"}]], "=", " ", 
      RowBox[{
       RowBox[{"\[Phi]", "(", 
        RowBox[{
         SubscriptBox["j", "2"], "\[CenterDot]", 
         SubscriptBox["j", "1"]}], ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nHence, ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is a homomorphism from ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "n", "*"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", 
     SubscriptBox["Z", "n"], ")"}], "."}], TraditionalForm]]],
 " To see that ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is one-to-one, note that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Florin]", "j"], "(", "1", ")"}], " ", "=", " ", "j"}], 
   TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Florin]", 
     SubscriptBox["j", "1"]], "=", " ", 
    SubscriptBox["\[Florin]", 
     SubscriptBox["j", "2"]]}], TraditionalForm]]],
 "  only if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["j", "1"], "=", " ", 
    SubscriptBox["j", "2"]}], TraditionalForm]]],
 ".  To see that ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is onto, we consider a general automorphism ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", " ", "of", " ", 
    SubscriptBox["Z", "n"]}], TraditionalForm]]],
 ". Since 1 is a generator of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ", \[Florin](1) must also be a generator of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". But \[Florin] will be completely determined by knowing \[Florin](1). \
Thus, the number of automorphisms is at most the number of elements in ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "n", "*"], TraditionalForm]]],
 ".  But we have an automorphism for each such element, accounting for all \
automorphisms of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.422649057765625*^9, 3.422649081796875*^9}, {
  3.42264921996875*^9, 3.422649294109375*^9}, {3.42264932971875*^9, 
  3.4226493393125*^9}, {3.422649389828125*^9, 3.422649406703125*^9}}]
}, Open  ]],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "n", "*"], TraditionalForm]]],
 " is smaller then ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "1"}], TraditionalForm]]],
 ", we find that the automorphism group of a cyclic group gives a smaller \
group than the original group. So why is this section in a chapter called \
\"Building Larger Groups from Smaller Groups\"? Let us look at a non-cyclic \
group. Consider the set of automorphisms on the group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ". First define this group in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2,e]; Define[b^2,e]
Define[b.a,a.b]
Define[1/a,a]; Define[1/b,b]
G = Group[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43669297790625*^9, 3.436692982546875*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.42264942025*^9, 3.442673668086375*^9}]
}, Open  ]],

Cell[TextData[{
 "Suppose ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an automorphism. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "e", ")"}], "=", "e"}], TraditionalForm]]],
 ", but all other elements are of order 2. Hence, any of the other elements \
might map to each other in any way. For example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "a", ")"}], TraditionalForm]]],
 " might be ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 ". Once we know where ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is mapped, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "b", ")"}], TraditionalForm]]],
 " might be either of the other 2 elements. However, once we know ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "a", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "b", ")"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", 
    RowBox[{"a", "\[CenterDot]", "b"}], ")"}], TraditionalForm]]],
 " must be ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "a", ")"}], "\[CenterDot]", 
    RowBox[{"\[Florin]", "(", "b", ")"}]}], TraditionalForm]]],
 ". Thus, there \nare at most ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"3", "\[CenterDot]", "2"}], " ", "=", " ", "6"}], 
   TraditionalForm]]],
 " elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "8", "*"], ")"}], TraditionalForm]]],
 ". If there are this many automorphisms, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "8", "*"], ")"}], TraditionalForm]]],
 " would actually be larger than ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ".\n\nHere would be a possible automorphism:\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"           ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", "e", ")"}], " ", "=", " ", "e"}], 
     "                       ", "\n", "           ", 
     RowBox[{
      RowBox[{"\[Florin]", "(", "a", ")"}], " ", "=", " ", "b"}], 
     "                      ", "\n", "           ", 
     RowBox[{
      RowBox[{"\[Florin]", "(", "b", ")"}], " ", "=", " ", "a"}], 
     "                        ", "\n", "     ", 
     RowBox[{
      RowBox[{"\[Florin]", "(", 
       RowBox[{"a", "\[CenterDot]", "b"}], ")"}], " ", "=", " ", 
      RowBox[{"a", "\[CenterDot]", "b"}]}]}]}], TraditionalForm]]],
 "\n\nLet us represent this as a permutation on the \nelements of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ". The elements are not numbered, so we cannot use the ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    RowBox[{"P", "[", " ", "]"}],
    FontWeight->"Bold"], TraditionalForm]]],
 " notation. However, we still can use the notation of cycles, and say that \
",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is the 2-cycle ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", " ", "b"}], ")"}], TraditionalForm]]],
 ". Note that we are using the cycle notation with ",
 StyleBox["elements",
  FontSlant->"Italic"],
 " instead of numbers.  We are allowed to do this in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", and so we can enter this function as follows:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4226494418125*^9, 3.42264944453125*^9}, {
  3.436693260234375*^9, 3.4366933191875*^9}}],

Cell[CellGroupData[{

Cell["F = C[a, b]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"C", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.42264945459375*^9, 3.44267367094575*^9}]
}, Open  ]],

Cell["\<\
In fact, we can still use the CircleGraph command to graph this function:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[G, F]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"b\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], InsetBox[
    TagBox[
     StyleBox["\<\"a\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}],
    InsetBox[
    TagBox[
     StyleBox["\<\"a\[CenterDot]b\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 
      18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{1., 6.123233995736766*^-17}, {
       0.07071067811865488, -0.9292893218813453}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.10606601717798225`, -0.9646446609406726}, {
       0.035355339059327494`, -0.8939339828220179}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {
       0.9292893218813453, -0.07071067811865471}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {
       0.8939339828220179, -0.03535533905932732}, {
       0.9646446609406726, -0.10606601717798207`}}]}}, 
    {RGBColor[0.6, 0.4, 0.8], 
     LineBox[{{-0.9292893218813453, -0.07071067811865492}, \
{-0.8968331206248827, -0.09241885630108193}, {-0.8585786437626904, \
-0.09998489885977796}, {-0.8203241669004983, -0.09241885630108192}, \
{-0.7878679656440357, -0.07071067811865489}, {-0.7661597874616087, \
-0.038254476862192355`}, {-0.7586644555810313, -1.3935141023228338`*^-16}, \
{-0.7661597874616087, 0.03825447686219208}, {-0.7878679656440357, 
      0.07071067811865461}, {-0.8203241669004984, 
      0.09241885630108161}, {-0.8585786437626906, 
      0.09998489885977765}, {-0.8968331206248827, 
      0.0924188563010816}, {-0.9292893218813453, 
      0.07071067811865459}, {-1., -1.8369701987210297`*^-16}}], 
     PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.035355339059327535`}}]}}},
  AspectRatio->Automatic,
  ImageSize->{179.33333333333334`, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.422649456796875*^9, 3.442673672398875*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can check if this is an automorphism of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ".   "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[F, G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.422649463296875*^9, 3.44267367388325*^9}]
}, Open  ]],

Cell[TextData[{
 "Let us consider having ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "a", ")"}], " ", "=", " ", "a"}], 
   TraditionalForm]]],
 ", but having ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "b", ")"}], " ", "=", " ", 
    RowBox[{"a", "\[CenterDot]", "b"}]}], TraditionalForm]]],
 ".  Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", 
    RowBox[{"a", "\[CenterDot]", "b"}], ")"}], TraditionalForm]]],
 " must be ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and we have the permutation"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
H = C[b, a.b]
CircleGraph[G, H]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.42264948040625*^9, 3.422649480875*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"C", "[", 
  RowBox[{"b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False]}], "]"}]], "Output",
 CellChangeTimes->{{3.42264946703125*^9, 3.4226494816875*^9}, 
   3.4426736751645*^9}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"b\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], InsetBox[
    TagBox[
     StyleBox["\<\"a\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}],
    InsetBox[
    TagBox[
     StyleBox["\<\"a\[CenterDot]b\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 
      18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, 
    {RGBColor[0, 1, 0], 
     LineBox[{{0.9292893218813453, 0.07071067811865481}, {0.8968331206248827, 
      0.09241885630108182}, {0.8585786437626906, 0.09998489885977785}, {
      0.8203241669004984, 0.0924188563010818}, {0.7878679656440357, 
      0.0707106781186548}, {0.7661597874616087, 0.038254476862192265`}, {
      0.7586644555810313, 4.645047007742779*^-17}, {
      0.7661597874616087, -0.03825447686219217}, {
      0.7878679656440357, -0.07071067811865471}, {
      0.8203241669004984, -0.09241885630108172}, {
      0.8585786437626906, -0.09998489885977777}, {
      0.8968331206248827, -0.09241885630108171}, {
      0.9292893218813453, -0.0707106781186547}, {1., 
      6.123233995736766*^-17}}], 
     PolygonBox[{{1., 6.123233995736766*^-17}, {0.9646446609406726, 
      0.10606601717798217`}, {0.8939339828220179, 0.03535533905932743}}]}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.9292893218813453, \
-0.0707106781186549}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.03535533905932753}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {-0.07071067811865464, \
-0.9292893218813453}}], 
      PolygonBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.03535533905932728, \
-0.8939339828220179}, {-0.10606601717798202`, -0.9646446609406727}}]}}}},
  AspectRatio->Automatic,
  ImageSize->{194., Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{{3.42264946703125*^9, 3.4226494816875*^9}, 
   3.44267367519575*^9}]
}, Open  ]],

Cell["This also checks out.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[H, G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.42264948828125*^9, 3.442673677430125*^9}]
}, Open  ]],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " are both automorphisms, we can consider the subgroup generated by ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["A = Group[{F, H}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"a", ",", "b"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"a", ",", 
     InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
      Editable->False]}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"b", ",", 
     InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
      Editable->False]}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"a", ",", "b", ",", 
     InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
      Editable->False]}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"a", ",", 
     InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
      Editable->False], ",", "b"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4226494913125*^9, 3.44267367869575*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives us 6 elements, which is the maximum number of automorphisms we \
can have. Thus, ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "8", "*"], ")"}], TraditionalForm]]],
 ". Therefore, we have found that the automorphism group is sometimes larger \
than the original group. Which group is this? It is not hard to recognize A \
as the permutation group on the elements ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 ".  Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 ") is isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let us look at the automorphisms of a non-commutative group, such the \
quaternion group ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". First, we load ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436693354984375*^9, 3.436693355125*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[i^4, e]; Define[j^2, i^2]
Define[j.i, i.i.i.j]
Define[1/i, i^3]; Define[1/j, i.i.j]
Q = Group[{i, j}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436693350421875*^9, 3.436693358875*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "i", ",", "j", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.42264950725*^9, {3.44267368057075*^9, 3.442673707477*^9}}
  ]
}, Open  ]],

Cell[TextData[{
 "Suppose ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an automorphism of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "e", ")"}], " ", "=", " ", "e"}], 
   TraditionalForm]]],
 ", as always, but ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", 
    SuperscriptBox["i", "2"], ")"}], TraditionalForm]]],
 " must be ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 " is the only element of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " of order 2. All of the other elements are of order 4, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "i", ")"}], TraditionalForm]]],
 " could be any one of the remaining 6 elements. Once ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "i", ")"}], TraditionalForm]]],
 " is determined, though, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     SuperscriptBox["i", "3"], ")"}], " ", "=", " ", 
    SuperscriptBox[
     RowBox[{"\[Florin]", "(", "i", ")"}], "3"]}], TraditionalForm]]],
 ", so this will be determined. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "j", ")"}], TraditionalForm]]],
 " could be any one of the remaining 4 elements. Since ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " generate ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", once ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "i", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "j", ")"}], TraditionalForm]]],
 " is determined, ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " will be determined for all of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". Thus, there is a maximum of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"6", "\[CenterDot]", "4"}], " ", "=", " ", "24"}], 
   TraditionalForm]]],
 " possible automorphisms.\n\nCan we find some of these automorphisms? For \
non-commutative groups, there is an easy way to find many of the \
automorphisms.\n\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a non-commutative group, and let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be any element in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".  Consider the mapping \n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Florin]", "x"], ":", " ", 
    RowBox[{"G", " ", "\[RightArrow]", " ", "G"}]}], TraditionalForm]]],
 "\ndefined by \n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Florin]", "x"], "(", "y", ")"}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "y", "\[CenterDot]", "x"}]}], 
   TraditionalForm]]],
 ".    \nThen \n    ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["\[Florin]", "x"], "(", 
      RowBox[{"y", "\[CenterDot]", "z"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]], "\[CenterDot]", "y", "\[CenterDot]", "z", 
       "\[CenterDot]", "x"}], "=", " ", 
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]], "\[CenterDot]", "y", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"x", "\[CenterDot]", 
         SuperscriptBox["x", 
          RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", "z", "\[CenterDot]", 
       "x"}]}]}], " "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                   ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", 
          RowBox[{"-", "1"}]], "\[CenterDot]", "y", "\[CenterDot]", "x"}], 
        ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", 
          RowBox[{"-", "1"}]], "\[CenterDot]", "z", "\[CenterDot]", "x"}], 
        ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Florin]", "x"], "(", "y", ")"}], "\[CenterDot]", 
       RowBox[{
        RowBox[{
         SubscriptBox["\[Florin]", "x"], "(", "z", ")"}], "."}]}]}]}]}], 
   TraditionalForm]}]],
 "                                        \n\nSo ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["\[Florin]", "x"],
     TraditionalForm], "(", "y", ")"}], TraditionalForm]]],
 " is a homomorphism. Since the inverse homomorphism can easily be found,\n\n \
",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{"y", " ", "\[Element]", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubsuperscriptBox["\[Florin]", "x", 
         RowBox[{"-", "1"}]], "(", "v", ")"}], " ", 
       "\[DoubleLongLeftRightArrow]", " ", 
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]]}], "\[CenterDot]", "y", "\[CenterDot]", "x"}]}], 
    "=", "v"}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t                   ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[DoubleLongLeftRightArrow]", "  ", "y"}], "=", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", "v", "\[CenterDot]", 
        RowBox[{
         SuperscriptBox["x", 
          RowBox[{"-", "1"}]], "  ", "\[DoubleLongLeftRightArrow]", " ", 
         "y"}]}], "=", 
       RowBox[{
        SubscriptBox["\[Florin]", 
         RowBox[{"(", 
          SuperscriptBox["x", 
           RowBox[{"-", "1"}]], ")"}]], "(", "v", ")"}]}]}], ","}]}], 
   TraditionalForm]}]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436694134890625*^9, 3.43669423984375*^9}}],

Cell[TextData[{
 "we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["\[Florin]", "x"],
     TraditionalForm], "(", "y", ")"}], TraditionalForm]]],
 " is one-to-one and onto, therefore ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Florin]", "x"], "(", "y", ")"}], TraditionalForm]]],
 " is an automorphism. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.422649569625*^9, 3.422649586328125*^9}, 
   3.436693559046875*^9}],

Cell[TextData[{
 " \nLet us try this technique to find some automorphisms on ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". If we choose ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "i"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Florin]", "i"], "(", "y", ")"}], TraditionalForm]]],
 " is the following mapping:\n                 \n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"                         ", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "i"], "(", "e", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "3"], " ", "\[CenterDot]", " ", "e", " ", 
        "\[CenterDot]", " ", "i"}], "              ", "=", " ", "e"}]}], 
     "                  ", "\n", "\t      ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "i"], "(", "i", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "3"], " ", "\[CenterDot]", " ", "i", " ", 
        "\[CenterDot]", " ", "i"}], "              ", "=", " ", "i"}]}], 
     "                  ", "\n", "\t      ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "i"], "(", "j", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "3"], " ", "\[CenterDot]", " ", "j", " ", 
        "\[CenterDot]", " ", "i"}], "              ", "=", " ", 
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j"}]}]}], 
     "                 ", "\n", "                       ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "i"], "(", 
       SuperscriptBox["i", "2"], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "3"], " ", "\[CenterDot]", " ", 
        SuperscriptBox["i", "2"], " ", "\[CenterDot]", " ", "i"}], 
       "            ", "=", " ", 
       SuperscriptBox["i", "2"]}]}], "                ", "\n", 
     "                       ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "i"], "(", 
       SuperscriptBox["i", "3"], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "3"], " ", "\[CenterDot]", " ", 
        SuperscriptBox["i", "3"], " ", "\[CenterDot]", " ", "i"}], 
       "            ", "=", " ", 
       SuperscriptBox["i", "3"]}]}], "                ", "\n", 
     "                    ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "i"], "(", 
       RowBox[{"i", "\[CenterDot]", "j"}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "3"], " ", "\[CenterDot]", " ", 
        RowBox[{"(", 
         RowBox[{"i", "\[CenterDot]", "j"}], ")"}], " ", "\[CenterDot]", " ", 
        "i"}], "   ", "=", " ", 
       RowBox[{
        SuperscriptBox["i", "3"], "\[CenterDot]", "j"}]}]}], 
     "                ", "\n", "                 ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "i"], "(", 
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j"}], ")"}], " ", "=", " ",
       
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "3"], " ", "\[CenterDot]", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["i", "2"], "\[CenterDot]", "j"}], ")"}], " ", 
        "\[CenterDot]", " ", "i"}], " ", "=", " ", "j"}]}], "               ",
      "\n", "                ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "i"], "(", 
       RowBox[{
        SuperscriptBox["i", "3"], "\[CenterDot]", "j"}], ")"}], " ", "=", " ",
       
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "3"], " ", "\[CenterDot]", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["i", "3"], "\[CenterDot]", "j"}], ")"}], " ", 
        "\[CenterDot]", " ", "i"}], " ", "=", " ", 
       RowBox[{"i", "\[CenterDot]", "j", " "}]}]}]}]}], TraditionalForm]]],
 "\n\nSo \[Florin] exchanges ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["i", "2"], "\[CenterDot]", "j"}], TraditionalForm]]],
 ", and exchanges ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"i", "\[CenterDot]", "j"}], " "}], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["i", "3"], "\[CenterDot]", "j"}], TraditionalForm]]],
 ", leaving all other elements alone. Thus, we can express this automorphism \
as"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.422649569625*^9, 3.422649586328125*^9}, 
   3.436693559046875*^9, 3.436693596171875*^9, {3.436694271125*^9, 
   3.436694324*^9}}],

Cell[CellGroupData[{

Cell["F = C[j, i.i.j] . C[i.j, i.i.i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(C[\\(\\(j, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
  StringForm["`` \[CenterDot] ``", 
   C[$CellContext`j, 
    Dot[
     Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
   C[
    Dot[$CellContext`i, $CellContext`j], 
    Dot[
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j]]],
  
  Editable->False]], "Output",
 CellChangeTimes->{3.42264959409375*^9, 3.442673724852*^9}]
}, Open  ]],

Cell["Here is the graph of this automorphism:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[Q,F]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"i\[CenterDot]j\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, 
    {RGBColor[0, 1, 0], 
     LineBox[{{0.6071067811865475, 0.7071067811865476}, {0.5688067811865475, 
      0.6995067811865475}, {0.5364067811865475, 0.6778067811865476}, {
      0.5147067811865476, 0.6454067811865476}, {0.5071067811865475, 
      0.6071067811865476}, {0.5147067811865476, 0.5688067811865476}, {
      0.5364567811865475, 0.5364067811865476}, {0.5688067811865475, 
      0.5147067811865476}, {0.6071067811865475, 0.5071067811865476}, {
      0.6454067811865475, 0.5147067811865476}, {0.6778067811865475, 
      0.5364067811865476}, {0.6995067811865475, 0.5688067811865476}, {
      0.7071067811865475, 0.6071067811865476}, {0.7071067811865475, 
      0.7071067811865476}}], 
     PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
      0.6071067811865475, 0.7571067811865475}, {0.6071067811865475, 
      0.6571067811865475}}]}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{1., 
       6.123233995736766*^-17}, {-0.9, -1.7145055188062944`*^-16}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9, \
-0.05000000000000017}, {-0.9, 0.049999999999999836`}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {0.9, 
       4.898587196589413*^-17}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {0.9, 0.05000000000000005}, {
       0.9, -0.049999999999999954`}}]}}, 
    {RGBColor[1, 0.6, 0], 
     LineBox[{{0.7071067811865476, -0.6071067811865475}, {
      0.6995067811865475, -0.5688067811865475}, {
      0.6778067811865476, -0.5364067811865475}, {
      0.6454067811865476, -0.5147067811865476}, {
      0.6071067811865476, -0.5071067811865475}, {
      0.5688067811865476, -0.5147067811865476}, {
      0.5364567811865476, -0.5364067811865475}, {
      0.5147067811865476, -0.5688067811865475}, {
      0.5071067811865476, -0.6071067811865475}, {
      0.5147067811865476, -0.6454067811865475}, {
      0.5364067811865476, -0.6778067811865475}, {
      0.5688067811865476, -0.6995067811865475}, {
      0.6071067811865476, -0.7071067811865475}, {
      0.7071067811865476, -0.7071067811865475}}], 
     PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
      0.7571067811865475, -0.6071067811865475}, {
      0.6571067811865475, -0.6071067811865475}}]}, {
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {-0.6688384379500387, 
       0.6147188279354187}}], 
      PolygonBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.7150324145756031, 
       0.5955846563171642}, {-0.6226444613244744, 0.6338529995536731}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.038268343236508864`, -0.9076120467488713}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.00792563338905547, -0.8884778751306168}, {-0.0844623198620732, \
-0.9267462183671258}}]}}, 
    {RGBColor[0.8, 0.6, 0.5], 
     LineBox[{{-0.6071067811865475, -0.7071067811865477}, \
{-0.5688067811865475, -0.6995067811865476}, {-0.5364067811865475, \
-0.6778067811865476}, {-0.5147067811865474, -0.6454067811865476}, \
{-0.5071067811865475, -0.6071067811865476}, {-0.5147067811865474, \
-0.5688067811865476}, {-0.5364567811865475, -0.5364067811865476}, \
{-0.5688067811865475, -0.5147067811865477}, {-0.6071067811865475, \
-0.5071067811865477}, {-0.6454067811865475, -0.5147067811865477}, \
{-0.6778067811865475, -0.5364067811865476}, {-0.6995067811865475, \
-0.5688067811865477}, {-0.7071067811865475, -0.6071067811865477}, \
{-0.7071067811865475, -0.7071067811865477}}], 
     PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6071067811865475, -0.7571067811865476}, {-0.6071067811865475, \
-0.6571067811865476}}]}}},
  AspectRatio->Automatic,
  ImageSize->{156.0000000000001, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4226495968125*^9, 3.4426737265395*^9}]
}, Open  ]],

Cell["We can double check that this is an automorphism:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[F,Q]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.42264960403125*^9, 3.442673728148875*^9}]
}, Open  ]],

Cell[TextData[{
 "If we use ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "j"}], TraditionalForm]]],
 " instead of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "i"}], TraditionalForm]]],
 ", we get a different automorphism:  \n\n   ",
 Cell[BoxData[{
  RowBox[{
   FormBox[
    RowBox[{"                             ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "j"], "(", "e", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j", " ", "\[CenterDot]", 
        " ", "e", " ", "\[CenterDot]", "j"}], "              ", "=", " ", 
       "e"}]}]}],
    TraditionalForm], "                "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"\t\t\t   ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "j"], "(", "i", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j", " ", "\[CenterDot]", 
        " ", "i", " ", "\[CenterDot]", " ", "j"}], "              ", "=", " ",
        
       SuperscriptBox["i", "3"]}]}]}],
    TraditionalForm], "              "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"\t\t\t   ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "j"], "(", "j", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j", " ", "\[CenterDot]", 
        " ", "j", " ", "\[CenterDot]", " ", "j"}], "              ", "=", " ",
        "j"}]}]}],
    TraditionalForm], "               "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"\t\t\t ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "j"], "(", 
       SuperscriptBox["i", "2"], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j", " ", "\[CenterDot]", 
        " ", 
        SuperscriptBox["i", "2"], " ", "\[CenterDot]", " ", "j"}], 
       "            ", "=", " ", 
       SuperscriptBox["i", "2"]}]}]}],
    TraditionalForm], "              "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"\t\t          ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "j"], "(", 
       SuperscriptBox["i", "3"], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j", " ", "\[CenterDot]", 
        " ", 
        SuperscriptBox["i", "3"], " ", "\[CenterDot]", " ", "j"}], 
       "            ", "=", " ", "i"}]}]}],
    TraditionalForm], "             "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"\t              ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "j"], "(", 
       RowBox[{"i", "\[CenterDot]", "j"}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j", " ", "\[CenterDot]", 
        " ", 
        RowBox[{"(", 
         RowBox[{"i", "\[CenterDot]", "j"}], ")"}], " ", "\[CenterDot]", " ", 
        "j"}], "   ", "=", " ", 
       RowBox[{
        SuperscriptBox["i", "3"], "\[CenterDot]", "j"}]}]}]}],
    TraditionalForm], "      "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"\t            ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "j"], "(", 
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j"}], ")"}], " ", "=", " ",
       
      RowBox[{
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j", " ", "\[CenterDot]", 
        " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["i", "2"], "\[CenterDot]", "j"}], ")"}], " ", 
        "\[CenterDot]", " ", "j"}], " ", "=", " ", 
       RowBox[{
        SuperscriptBox["i", "2"], "\[CenterDot]", "j"}]}]}]}],
    TraditionalForm], "             "}], "\n", 
  RowBox[{"\t            ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[Florin]", "j"], 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["i", "3"], "\[CenterDot]", "j"}], ")"}]}], " ", "=", 
    " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["i", "2"], "\[CenterDot]", "j", " ", "\[CenterDot]", " ",
       
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["i", "3"], "\[CenterDot]", "j"}], ")"}], " ", 
      "\[CenterDot]", "j"}], " ", "=", " ", 
     RowBox[{"i", "\[CenterDot]", "j"}]}]}]}]}]],
 "\n\nWe can represent this automorphism in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.422649608078125*^9, 3.4226496316875*^9}, {
  3.43669434996875*^9, 3.436694387546875*^9}}],

Cell[CellGroupData[{

Cell["G = C[i, i.i.i] . C[i.j, i.i.i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
  StringForm["`` \[CenterDot] ``", 
   C[$CellContext`i, 
    Dot[
     Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
   C[
    Dot[$CellContext`i, $CellContext`j], 
    Dot[
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j]]],
  
  Editable->False]], "Output",
 CellChangeTimes->{3.422649636171875*^9, 3.44267373038325*^9}]
}, Open  ]],

Cell["Here is the graph:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[Q,G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"i\[CenterDot]j\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865475, 
       0.7071067811865476}, {-0.6363961030678927, -0.636396103067893}}], 
      PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6010407640085653, -0.6717514421272203}, {-0.67175144212722, \
-0.6010407640085655}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.7071067811865475, -0.7071067811865477}, {
       0.6363961030678927, 0.6363961030678928}}], 
      PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.6010407640085653, 0.6717514421272202}, {0.67175144212722, 
       0.6010407640085654}}]}}, 
    {RGBColor[0.6, 0.4, 0.8], 
     LineBox[{{0.9292893218813453, 0.07071067811865481}, {0.8968331206248827, 
      0.09241885630108182}, {0.8585786437626906, 0.09998489885977785}, {
      0.8203241669004984, 0.0924188563010818}, {0.7878679656440357, 
      0.0707106781186548}, {0.7661597874616087, 0.038254476862192265`}, {
      0.7586644555810313, 4.645047007742779*^-17}, {
      0.7661597874616087, -0.03825447686219217}, {
      0.7878679656440357, -0.07071067811865471}, {
      0.8203241669004984, -0.09241885630108172}, {
      0.8585786437626906, -0.09998489885977777}, {
      0.8968331206248827, -0.09241885630108171}, {
      0.9292893218813453, -0.0707106781186547}, {1., 
      6.123233995736766*^-17}}], 
     PolygonBox[{{1., 6.123233995736766*^-17}, {0.9646446609406726, 
      0.10606601717798217`}, {0.8939339828220179, 0.03535533905932743}}]}, 
    {RGBColor[1, 0.6, 0], 
     LineBox[{{0.7071067811865476, -0.6071067811865475}, {
      0.6995067811865475, -0.5688067811865475}, {
      0.6778067811865476, -0.5364067811865475}, {
      0.6454067811865476, -0.5147067811865476}, {
      0.6071067811865476, -0.5071067811865475}, {
      0.5688067811865476, -0.5147067811865476}, {
      0.5364567811865476, -0.5364067811865475}, {
      0.5147067811865476, -0.5688067811865475}, {
      0.5071067811865476, -0.6071067811865475}, {
      0.5147067811865476, -0.6454067811865475}, {
      0.5364067811865476, -0.6778067811865475}, {
      0.5688067811865476, -0.6995067811865475}, {
      0.6071067811865476, -0.7071067811865475}, {
      0.7071067811865476, -0.7071067811865475}}], 
     PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
      0.7571067811865475, -0.6071067811865475}, {
      0.6571067811865475, -0.6071067811865475}}]}, {
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {-0.6688384379500387, 
       0.6147188279354187}}], 
      PolygonBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.7150324145756031, 
       0.5955846563171642}, {-0.6226444613244744, 0.6338529995536731}}]}, 
     {RGBColor[0.7, 0.8, 0.7], 
      LineBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.038268343236508864`, -0.9076120467488713}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.00792563338905547, -0.8884778751306168}, {-0.0844623198620732, \
-0.9267462183671258}}]}}, 
    {RGBColor[0.8, 0.6, 0.5], 
     LineBox[{{-0.9292893218813453, -0.07071067811865492}, \
{-0.8968331206248827, -0.09241885630108193}, {-0.8585786437626904, \
-0.09998489885977796}, {-0.8203241669004983, -0.09241885630108192}, \
{-0.7878679656440357, -0.07071067811865489}, {-0.7661597874616087, \
-0.038254476862192355`}, {-0.7586644555810313, -1.3935141023228338`*^-16}, \
{-0.7661597874616087, 0.03825447686219208}, {-0.7878679656440357, 
      0.07071067811865461}, {-0.8203241669004984, 
      0.09241885630108161}, {-0.8585786437626906, 
      0.09998489885977765}, {-0.8968331206248827, 
      0.0924188563010816}, {-0.9292893218813453, 
      0.07071067811865459}, {-1., -1.8369701987210297`*^-16}}], 
     PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.035355339059327535`}}]}}},
  AspectRatio->Automatic,
  ImageSize->{157.33333333333283`, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.42264963853125*^9, 3.442673731555125*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can double check that this is an automorphism, too."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[G,Q]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.4226496428125*^9, 3.442673733555125*^9}]
}, Open  ]],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " are both automorphisms, we can find the group generated by these two:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Group[{F,G}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \\\"\\\\!\\\\(\\\
\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
     C[$CellContext`j, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(j, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`j, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4226496448125*^9, 3.44267373482075*^9}]
}, Open  ]],

Cell[TextData[{
 "Therefore, we have found 4 automorphisms on ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". But is this all of the automorphisms? \n\nEXPERIMENT:\nFind the \
automorphism ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Florin]", "x"], "(", "y", ")"}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{"i", "\[CenterDot]", "j"}]}], TraditionalForm]]],
 ". Find the corresponding permutation on ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", and have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " draw a circle graph of this automorphism. Is this automorphism one that we \
have seen before?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43669374278125*^9, 3.43669375171875*^9}, {
  3.436693877421875*^9, 3.43669387846875*^9}}],

Cell["       ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "If we try the above procedure with ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " equaling any of the eight elements of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", we will always get one of the four automorphisms. But there certainly \
could be more. Suppose ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "i", ")"}], " ", "=", " ", "i"}], 
   TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "j", ")"}], " ", "=", " ", 
    RowBox[{"i", "\[CenterDot]", "j"}]}], TraditionalForm]]],
 "? Since ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "\[CenterDot]", "j"}], TraditionalForm]]],
 " are all of order 4, it is conceivable that this would produce an \
automorphism. This time, we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s Homomorph function to define the mapping."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Homomorph[X]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["Define[X[i], i]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["Define[X[j], i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "Is this a homomorphism? We can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " check for us:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[X, Q]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.4226496731875*^9, 3.442673741305125*^9}]
}, Open  ]],

Cell[TextData[{
 "So ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " is a homomorphism, but is ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " an automorphism? By finding"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["X[{Q}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "i", ",", "j", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.42264967628125*^9, 3.442673742398875*^9}]
}, Open  ]],

Cell[TextData[{
 "we see that ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " is onto, and by finding"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Kernel[X, Q]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", "e", "}"}]], "Output",
 CellChangeTimes->{3.422649678953125*^9, 3.44267374375825*^9}]
}, Open  ]],

Cell[TextData[{
 "we see that ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " is one-to-one. Therefore, ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " is an automorphism. Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", "(", "i", ")"}], " ", "=", " ", "i"}], TraditionalForm]]],
 ", we must have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", "(", 
     SuperscriptBox["i", "2"], ")"}], "=", " ", 
    SuperscriptBox["i", "2"]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", "(", 
    SuperscriptBox["i", "3"]}], TraditionalForm]]],
 ") = ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "3"], TraditionalForm]]],
 ". Here is ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " for the other three values of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["X[i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
  StringForm["`` \[CenterDot] ``", 
   Dot[$CellContext`i, $CellContext`i], $CellContext`j],
  Editable->False]], "Output",
 CellChangeTimes->{3.422649682296875*^9, 3.442673745461375*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["X[i.i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\
\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
  StringForm["`` \[CenterDot] ``", 
   Dot[
    Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
  Editable->False]], "Output",
 CellChangeTimes->{3.422649683265625*^9, 3.442673746367625*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["X[i.i.i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["j"], "Output",
 CellChangeTimes->{3.422649684171875*^9, 3.44267374794575*^9}]
}, Open  ]],

Cell["So the automorphism can be represented by the 4-cycle", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["H = C[j, i.j, i.i.j, i.i.i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"C", "[", 
  RowBox[{"j", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
    Editable->False]}], "]"}]], "Output",
 CellChangeTimes->{3.422649690265625*^9, 3.442673749336375*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["CircleGraph[Q, H]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"i\[CenterDot]j\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, 
    {RGBColor[0, 1, 0], 
     LineBox[{{0.6071067811865475, 0.7071067811865476}, {0.5688067811865475, 
      0.6995067811865475}, {0.5364067811865475, 0.6778067811865476}, {
      0.5147067811865476, 0.6454067811865476}, {0.5071067811865475, 
      0.6071067811865476}, {0.5147067811865476, 0.5688067811865476}, {
      0.5364567811865475, 0.5364067811865476}, {0.5688067811865475, 
      0.5147067811865476}, {0.6071067811865475, 0.5071067811865476}, {
      0.6454067811865475, 0.5147067811865476}, {0.6778067811865475, 
      0.5364067811865476}, {0.6995067811865475, 0.5688067811865476}, {
      0.7071067811865475, 0.6071067811865476}, {0.7071067811865475, 
      0.7071067811865476}}], 
     PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
      0.6071067811865475, 0.7571067811865475}, {0.6071067811865475, 
      0.6571067811865475}}]}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{1., 6.123233995736766*^-17}, {
       0.07071067811865488, -0.9292893218813453}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.10606601717798225`, -0.9646446609406726}, {
       0.035355339059327494`, -0.8939339828220179}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.9292893218813453, \
-0.0707106781186549}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.03535533905932753}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {-0.7453751244230566, 
       0.6147188279354187}}], 
      PolygonBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.791569101048621, 
       0.6338529995536731}, {-0.6991811477974923, 0.5955846563171642}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.7071067811865477, 0.7071067811865474}, {0.9076120467488713,
        0.03826834323650902}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {0.9267462183671258, 
       0.08446231986207337}, {
       0.8884778751306168, -0.007925633389055322}}]}}, 
    {RGBColor[1, 0.6, 0], 
     LineBox[{{0.7071067811865476, -0.6071067811865475}, {
      0.6995067811865475, -0.5688067811865475}, {
      0.6778067811865476, -0.5364067811865475}, {
      0.6454067811865476, -0.5147067811865476}, {
      0.6071067811865476, -0.5071067811865475}, {
      0.5688067811865476, -0.5147067811865476}, {
      0.5364567811865476, -0.5364067811865475}, {
      0.5147067811865476, -0.5688067811865475}, {
      0.5071067811865476, -0.6071067811865475}, {
      0.5147067811865476, -0.6454067811865475}, {
      0.5364067811865476, -0.6778067811865475}, {
      0.5688067811865476, -0.6995067811865475}, {
      0.6071067811865476, -0.7071067811865475}, {
      0.7071067811865476, -0.7071067811865475}}], 
     PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
      0.7571067811865475, -0.6071067811865475}, {
      0.6571067811865475, -0.6071067811865475}}]}, 
    {RGBColor[0.7, 0.8, 0.7], 
     LineBox[{{-0.6071067811865475, -0.7071067811865477}, \
{-0.5688067811865475, -0.6995067811865476}, {-0.5364067811865475, \
-0.6778067811865476}, {-0.5147067811865474, -0.6454067811865476}, \
{-0.5071067811865475, -0.6071067811865476}, {-0.5147067811865474, \
-0.5688067811865476}, {-0.5364567811865475, -0.5364067811865476}, \
{-0.5688067811865475, -0.5147067811865477}, {-0.6071067811865475, \
-0.5071067811865477}, {-0.6454067811865475, -0.5147067811865477}, \
{-0.6778067811865475, -0.5364067811865476}, {-0.6995067811865475, \
-0.5688067811865477}, {-0.7071067811865475, -0.6071067811865477}, \
{-0.7071067811865475, -0.7071067811865477}}], 
     PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6071067811865475, -0.7571067811865476}, {-0.6071067811865475, \
-0.6571067811865476}}]}}},
  AspectRatio->Automatic,
  ImageSize->{164.66666666666612`, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.422649691390625*^9, 3.4426737506645*^9}]
}, Open  ]],

Cell["Let us add this to our group of automorphisms:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Group[{F, G, H}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"j", ",", 
     InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`j],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
      
      Editable->False]}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"j", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
      
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`j],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
      Editable->False]}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \\\"\\\\!\\\\(\\\
\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
     C[$CellContext`j, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(j, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`j, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \
\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\
\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\
\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \\\\[CenterDot] \
\\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      C[$CellContext`i, 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
      C[$CellContext`j, 
       Dot[$CellContext`i, $CellContext`j]]], 
     C[
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \
\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\
\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\
\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \\\\[CenterDot] \
\\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\
\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\
 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      C[$CellContext`i, 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
      C[$CellContext`j, 
       Dot[
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.42264970265625*^9, 3.442673752727*^9}]
}, Open  ]],

Cell[TextData[{
 "So far, we have 8 automorphisms. Could there be any others?\n\nNotice that \
all 8 of the automorphisms that we have come up with so far send ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " either to ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "3"], TraditionalForm]]],
 ". What if we sent ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " to, say, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", ".", "j"}], TraditionalForm]]],
 "? We could let ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " be sent to ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " for simplicity. Let's try it."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Homomorph[J]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["Define[J[i], i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["Define[J[j], j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["Let's see if this is a homomorphism:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[J, Q]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.42264971953125*^9, 3.442673758055125*^9}]
}, Open  ]],

Cell[TextData[{
 "So ",
 Cell[BoxData[
  FormBox["J", TraditionalForm]]],
 " is a homomorphism. To check that ",
 Cell[BoxData[
  FormBox["J", TraditionalForm]]],
 " is one-to-one and onto, we look at"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Kernel[J, Q]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", "e", "}"}]], "Output",
 CellChangeTimes->{3.42264972065625*^9, 3.442673759805125*^9}]
}, Open  ]],

Cell["and", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["J[{Q}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "i", ",", "j", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.42264972221875*^9, 3.442673761336375*^9}]
}, Open  ]],

Cell[TextData[{
 "So ",
 Cell[BoxData[
  FormBox["J", TraditionalForm]]],
 " is an automorphism of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". ",
 Cell[BoxData[
  FormBox["J", TraditionalForm]]],
 " fixes ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", and so fixes ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["i", "3"], "\[CenterDot]", "j"}], TraditionalForm]]],
 ". Here are the values of ",
 Cell[BoxData[
  FormBox["J", TraditionalForm]]],
 " for the last three elements of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["J[i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
  StringForm["`` \[CenterDot] ``", 
   Dot[$CellContext`i, $CellContext`i], $CellContext`i],
  Editable->False]], "Output",
 CellChangeTimes->{3.422649724203125*^9, 3.44267376332075*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["J[i.i.i]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\
\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
  StringForm["`` \[CenterDot] ``", 
   Dot[
    Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
  Editable->False]], "Output",
 CellChangeTimes->{3.42264972528125*^9, 3.44267376432075*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["J[i.i.i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["i"], "Output",
 CellChangeTimes->{3.422649726171875*^9, 3.442673765727*^9}]
}, Open  ]],

Cell[TextData[{
 "Hence, ",
 Cell[BoxData[
  FormBox["J", TraditionalForm]]],
 " is also represented by a 4-cycle:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["J = C[i, i.j, i.i.i, i.i.i.j]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"C", "[", 
  RowBox[{"i", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
    Editable->False]}], "]"}]], "Output",
 CellChangeTimes->{3.42264972775*^9, 3.4426737674145*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["CircleGraph[Q, J]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"i\[CenterDot]j\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.03826834323650909, -0.9076120467488713}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.08446231986207345, -0.9267462183671258}, {-0.007925633389055248, \
-0.8884778751306168}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.6147188279354188, \
-0.7453751244230566}}], 
      PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6338529995536732, -0.791569101048621}, {-0.5955846563171643, \
-0.6991811477974923}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.7071067811865477, 0.6071067811865474}}], 
      PolygonBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.7571067811865476, 
       0.6071067811865474}, {-0.6571067811865476, 0.6071067811865474}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.7071067811865477, 0.7071067811865474}, {0.6071067811865475,
        0.7071067811865476}}], 
      PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.6071067811865475, 0.7571067811865475}, {0.6071067811865475, 
       0.6571067811865475}}]}}, 
    {RGBColor[0.6, 0.4, 0.8], 
     LineBox[{{0.9292893218813453, 0.07071067811865481}, {0.8968331206248827, 
      0.09241885630108182}, {0.8585786437626906, 0.09998489885977785}, {
      0.8203241669004984, 0.0924188563010818}, {0.7878679656440357, 
      0.0707106781186548}, {0.7661597874616087, 0.038254476862192265`}, {
      0.7586644555810313, 4.645047007742779*^-17}, {
      0.7661597874616087, -0.03825447686219217}, {
      0.7878679656440357, -0.07071067811865471}, {
      0.8203241669004984, -0.09241885630108172}, {
      0.8585786437626906, -0.09998489885977777}, {
      0.8968331206248827, -0.09241885630108171}, {
      0.9292893218813453, -0.0707106781186547}, {1., 
      6.123233995736766*^-17}}], 
     PolygonBox[{{1., 6.123233995736766*^-17}, {0.9646446609406726, 
      0.10606601717798217`}, {0.8939339828220179, 0.03535533905932743}}]}, 
    {RGBColor[1, 0.6, 0], 
     LineBox[{{0.7071067811865476, -0.6071067811865475}, {
      0.6995067811865475, -0.5688067811865475}, {
      0.6778067811865476, -0.5364067811865475}, {
      0.6454067811865476, -0.5147067811865476}, {
      0.6071067811865476, -0.5071067811865475}, {
      0.5688067811865476, -0.5147067811865476}, {
      0.5364567811865476, -0.5364067811865475}, {
      0.5147067811865476, -0.5688067811865475}, {
      0.5071067811865476, -0.6071067811865475}, {
      0.5147067811865476, -0.6454067811865475}, {
      0.5364067811865476, -0.6778067811865475}, {
      0.5688067811865476, -0.6995067811865475}, {
      0.6071067811865476, -0.7071067811865475}, {
      0.7071067811865476, -0.7071067811865475}}], 
     PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
      0.7571067811865475, -0.6071067811865475}, {
      0.6571067811865475, -0.6071067811865475}}]}, 
    {RGBColor[0.7, 0.8, 0.7], 
     LineBox[{{-0.9292893218813453, -0.07071067811865492}, \
{-0.8968331206248827, -0.09241885630108193}, {-0.8585786437626904, \
-0.09998489885977796}, {-0.8203241669004983, -0.09241885630108192}, \
{-0.7878679656440357, -0.07071067811865489}, {-0.7661597874616087, \
-0.038254476862192355`}, {-0.7586644555810313, -1.3935141023228338`*^-16}, \
{-0.7661597874616087, 0.03825447686219208}, {-0.7878679656440357, 
      0.07071067811865461}, {-0.8203241669004984, 
      0.09241885630108161}, {-0.8585786437626906, 
      0.09998489885977765}, {-0.8968331206248827, 
      0.0924188563010816}, {-0.9292893218813453, 
      0.07071067811865459}, {-1., -1.8369701987210297`*^-16}}], 
     PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.035355339059327535`}}]}}},
  AspectRatio->Automatic,
  ImageSize->{149.33333333333312`, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.42264973590625*^9, 3.442673768992625*^9}]
}, Open  ]],

Cell["So here is an updated list of automorphisms:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["A = Group[{F,G,H,J}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"i", ",", "j", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`i],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`j],
      Editable->False]}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"i", ",", 
     InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`i],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
      
      Editable->False]}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"i", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`j],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`i],
      Editable->False], ",", "j"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"i", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
      
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`i],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
      Editable->False]}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"j", ",", 
     InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`j],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
      
      Editable->False]}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"j", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
      
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`j],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
      Editable->False]}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \\\"\\\\!\\\\(\\\
\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
     C[$CellContext`j, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(j, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`j, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, j, \\\"\\\\!\\\\(i\\\\) \
\\\\[CenterDot] \\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, $CellContext`j, 
      Dot[$CellContext`i, $CellContext`j]], 
     C[
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, j, \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\
\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\
\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\
\\\\\\\"\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\
\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \\\\!\\\\(i\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, $CellContext`j, 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \\\"\\\\!\\\\(i\\\\) \
\\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", j\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(\
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \\\\!\\\\(i\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[$CellContext`i, $CellContext`j], $CellContext`j], 
     C[
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
       
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \\\"\\\\!\\\\(i\\\\) \
\\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \\\"\\\\!\\\\(\\\
\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
     C[$CellContext`j, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \
\[CenterDot] \\!\\(C[\\(\\(j, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
      Dot[$CellContext`i, $CellContext`j]], 
     C[$CellContext`j, 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
       
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \
\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\\\\!\\\\(\\\\\
\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]], 
     C[$CellContext`j, 
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", j\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j], $CellContext`j], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \\\"\\\\!\\\\(\\\
\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\
\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
       
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
     C[$CellContext`j, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
      Dot[$CellContext`i, $CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \
j\\\\)\\\\)]\\\\) \\\\[CenterDot] \\\\!\\\\(C[\\\\(\\\\(\\\\\\\"\\\\\\\\!\\\\\
\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\", \
\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      C[$CellContext`i, $CellContext`j], 
      C[
       Dot[$CellContext`i, $CellContext`j], 
       Dot[
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]], 
     C[
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \
\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \\\\[CenterDot] \
\\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\
\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\
\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      C[$CellContext`i, 
       Dot[$CellContext`i, $CellContext`j]], 
      C[$CellContext`j, 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`j]]], 
     C[
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \
\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\
\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\
\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \\\\[CenterDot] \
\\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      C[$CellContext`i, 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
      C[$CellContext`j, 
       Dot[$CellContext`i, $CellContext`j]]], 
     C[
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \
\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\
\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\
\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \\\\[CenterDot] \
\\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\
\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\
 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      C[$CellContext`i, 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
      C[$CellContext`j, 
       Dot[
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \
\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\
\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\
\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \\\\[CenterDot] \
\\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\
\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\
\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      C[$CellContext`i, 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
      C[$CellContext`j, 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i]]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \
\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \
\\\\[CenterDot] \\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\
\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\
\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(j\\\\\\\\)\
\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      C[$CellContext`i, 
       Dot[
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]], 
      C[$CellContext`j, 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`j]]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.422649742296875*^9, 3.442673771586375*^9}]
}, Open  ]],

Cell["\<\
This is a long list. Let's see how many automorphisms we have\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[A]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["24"], "Output",
 CellChangeTimes->{3.4226497451875*^9, 3.442673774336375*^9}]
}, Open  ]],

Cell[TextData[{
 "So we have 24 automorphisms of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". Since we began by observing that there could be at most 24 automorphisms, \
we finally must have them all. Is this group isomorphic to one of the groups \
of order 24 that we have studied before?\n\nRather than proving an \
isomorphism directly as we have done in the past, in this case there is a \
geometrical correlation between this automorphism group and the octahedronal \
group. Note that any automorphism of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " must fix ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 ", but the other 6 elements can go to any other of the 6 elements. Suppose \
we were to label the 6 vertices of the octahedron with the 6 remaining \
elements of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", as follows:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ShowOctahedronWithQuaternions", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 Graphics3DBox[{{
    {RGBColor[1, 0, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {1, 0, 0}}]}, 
    {RGBColor[0, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}}]}, 
    {RGBColor[0, 0, 1], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {1, 0, 0}}]}, 
    {RGBColor[1, 1, 0], Polygon3DBox[{{0, 0, 1}, {0, -1, 0}, {-1, 0, 0}}]}, 
    {RGBColor[1, 0, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {1, 0, 0}}]}, 
    {RGBColor[0, 1, 1], Polygon3DBox[{{0, 0, -1}, {0, 1, 0}, {-1, 0, 0}}]}, 
    {RGBColor[1, 0.6, 0], 
     Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {1, 0, 0}}]}, 
    {RGBColor[0, 0, 0], Polygon3DBox[{{0, 0, -1}, {0, -1, 0}, {-1, 0, 0}}]}}, 
   Text3DBox[
    FormBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], TraditionalForm], {0, 0, 1.1}], 
   Text3DBox[
    FormBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], TraditionalForm], {0, 1.1, 0}], 
   Text3DBox[
    FormBox[
     TagBox[
      StyleBox["\<\"i\.b3\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], TraditionalForm], {0, 0, -1.1}], 
   Text3DBox[
    FormBox[
     TagBox[
      StyleBox["\<\"i\.b2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], TraditionalForm], {0, -1.1, 0}], 
   Text3DBox[
    FormBox[
     TagBox[
      StyleBox["\<\"<\[Dash]\[Dash]\[Dash]\[Dash]\[Dash]\[Dash]i\.b2\
\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], TraditionalForm], {-1, 0, 0}, {-1, 0}],
    Text3DBox[
    FormBox[
     TagBox[
      StyleBox["\<\"j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], TraditionalForm], {0.7, 0, 0}]},
  Boxed->False,
  ImageSize->{215.9999999999996, Automatic},
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 1.5, 2}]], "Output",
 CellChangeTimes->{3.442673814773875*^9}]
}, Open  ]],

Cell[TextData[{
 "The vertex in the back is labeled ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["i", "2"], "\[CenterDot]", "j"}], TraditionalForm]]],
 ". Now notice that each rotation of the octahedron gives an automorphism of \
",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", and vice versa. For example, rotating the red face 120\[Degree] clockwise \
corresponds to the automorphism\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"i", "    ", "j", "    ", 
      RowBox[{"i", "\[CenterDot]", "j"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["i", "3"], "    ", 
      RowBox[{
       SuperscriptBox["i", "3"], "\[CenterDot]", "j"}], "    ", 
      RowBox[{
       SuperscriptBox["i", "3"], "\[CenterDot]", "j"}]}], ")"}]}], 
   TraditionalForm]]],
 ".\n\nSo the automorphism group is isomorphic to the octahedronal group, \
which we saw was isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "Q", ")"}], TraditionalForm]]],
 " is isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ".\n         \nThis example introduced a way of finding automorphisms for \
non-abelian groups.  We found that some of the automorphisms of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " could be expressed as\n            \n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[Florin]", "x"],
      TraditionalForm], "(", "y", ")"}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "y", "\[CenterDot]", "x"}]}], 
   TraditionalForm]]],
 "\n \nfor some ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". We give these automorphisms a special name.\n\nDEFINITION 6.4:\nAn \
automorphism ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "y", ")"}], TraditionalForm]]],
 " of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is called an ",
 StyleBox["inner automorphism",
  FontSlant->"Italic"],
 " if there is an element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " such that \n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "y", ")"}], "=", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "y", "\[CenterDot]", "x"}]}], 
   TraditionalForm]]],
 "   for all   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "\[Element]", " ", "G"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4366938276875*^9, 3.4366938276875*^9}, {
  3.436693897890625*^9, 3.436693899390625*^9}, {3.436694080703125*^9, 
  3.436694111609375*^9}}],

Cell[TextData[{
 "The set of inner automorphisms of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Inn", "(", "G", ")"}], TraditionalForm]]],
 ".\n\nWe found the homomorphisms ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " by looking at the inner automorphisms. But we also found that the product \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "\[CenterDot]", "G"}], TraditionalForm]]],
 " was an inner automorphism. This suggests that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Inn", "(", "G", ")"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "G", ")"}], TraditionalForm]]],
 ". In fact we can even say more!\n\nPROPOSITION 6.5\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Inn", "(", "G", ")"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "G", ")"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43669379890625*^9, 3.436693819765625*^9}, 
   3.436694621765625*^9}],

Cell[CellGroupData[{

Cell["Proof: ", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First we need to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Inn", "(", "G", ")"}], TraditionalForm]]],
 " is a subgroup. Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Florin]", "x"], "(", "y", ")"}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "y", "\[CenterDot]", "x"}]}], 
   TraditionalForm]]],
 "\nbe an inner automorphism.  The inverse can be easily found by observing\n\
\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", " ", "\[Element]", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubsuperscriptBox["\[Florin]", "x", 
         RowBox[{"-", "1"}]], "(", "v", ")"}], " ", 
       "\[DoubleLongLeftRightArrow]", " ", 
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]]}], "\[CenterDot]", "y", "\[CenterDot]", "x"}]}], 
    "=", 
    RowBox[{
     RowBox[{"v", "  ", "\[DoubleLongLeftRightArrow]", "  ", "y"}], "=", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "v", "\[CenterDot]", 
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{"-", "1"}]], "  ", "\[DoubleLongLeftRightArrow]", " ", 
        "y"}]}], "=", 
      RowBox[{
       SubscriptBox["\[Florin]", 
        RowBox[{"(", 
         SuperscriptBox["x", 
          RowBox[{"-", "1"}]], ")"}]], "(", "v", ")"}]}]}]}], 
   TraditionalForm]]],
 ",\nso the inverse of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Florin]", "x"], TraditionalForm]]],
 " is also an inner automorphism.\n\nIf we consider two inner automorphisms \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Florin]", "x"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Florin]", "y"], TraditionalForm]]],
 ", then \n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["f", "x"], "\[CenterDot]", 
        SubscriptBox["f", "y"]}], ")"}], 
      RowBox[{"(", "v", ")"}]}], "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Florin]", "y"], "(", 
       RowBox[{
        SubscriptBox["\[Florin]", "x"], "(", "v", ")"}], ")"}], " ", "=", " ",
       
      RowBox[{
       RowBox[{
        SuperscriptBox["y", 
         RowBox[{"-", "1"}]], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", 
           RowBox[{"-", "1"}]], "\[CenterDot]", "v", "\[CenterDot]", "x"}], 
         ")"}], "\[CenterDot]", "y"}], "=", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "\[CenterDot]", "y"}], ")"}], 
          RowBox[{"-", "1"}]], "\[CenterDot]", "v", "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], " ", "=", " ", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Florin]", 
           RowBox[{"(", 
            RowBox[{"x", "\[CenterDot]", "y"}], ")"}]], "(", "v", ")"}], 
         "."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nThus the product of two inner automorphisms is also an inner \
automorphism. So by proposition 2.2, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Inn", "(", "G", ")"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", "G", ")"}], "."}], TraditionalForm]]],
 "\n\nFinally, we need to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Inn", "(", "G", ")"}], TraditionalForm]]],
 " is normal in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", "G", ")"}], "."}], TraditionalForm]]],
 "  Let ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " be any automorphism and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Florin]", "x"], "(", "y", ")"}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "y", "\[CenterDot]", "x"}]}], 
   TraditionalForm]]],
 " be an inner automorphism.  Then\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Phi]", "\[CenterDot]", 
       SubscriptBox["\[Florin]", "x"], "\[CenterDot]", 
       SuperscriptBox["\[Phi]", 
        RowBox[{"-", "1"}]]}], ")"}], 
     RowBox[{"(", "v", ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Phi]", 
       RowBox[{"-", "1"}]], "(", 
      RowBox[{
       SubscriptBox["\[Florin]", "x"], " ", 
       RowBox[{"(", 
        RowBox[{"\[Phi]", "(", "v", ")"}], ")"}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Phi]", 
        RowBox[{"-", "1"}]], "(", 
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{"-", "1"}]], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"\[Phi]", "(", "v", ")"}], ")"}], "\[CenterDot]", "x"}], 
       ")"}], ".", "  "}]}]}], TraditionalForm]]],
 "\n\nSince ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Phi]", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " is a homomorphism, this simplifies!\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Phi]", 
      RowBox[{"-", "1"}]], "(", 
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"\[Phi]", "(", "v", ")"}], ")"}], "\[CenterDot]", "x"}], ")"}],
     "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["\[Phi]", 
          RowBox[{"-", "1"}]], "(", "x", ")"}], ")"}], 
       RowBox[{"-", "1"}]], " ", "\[CenterDot]", " ", 
      RowBox[{
       SuperscriptBox["\[Phi]", 
        RowBox[{"-", "1"}]], "(", 
       RowBox[{"\[Phi]", "(", "v", ")"}], ")"}], "\[CenterDot]", " ", 
      RowBox[{
       SuperscriptBox["\[Phi]", 
        RowBox[{"-", "1"}]], "(", "x", ")"}]}], "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["\[Phi]", 
           RowBox[{"-", "1"}]], "(", "x", ")"}], ")"}], 
        RowBox[{"-", "1"}]], " ", "\[CenterDot]", "v", " ", "\[CenterDot]", 
       " ", 
       RowBox[{
        SuperscriptBox["\[Phi]", 
         RowBox[{"-", "1"}]], "(", "x", ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Florin]", 
         RowBox[{
          SuperscriptBox["\[Phi]", 
           RowBox[{"-", "1"}]], "(", "x", ")"}]], "(", "v", ")"}], 
       "."}]}]}]}], TraditionalForm]]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "\[CenterDot]", 
    SubscriptBox["\[Florin]", "x"], "\[CenterDot]", 
    SuperscriptBox["\[Phi]", 
     RowBox[{
      RowBox[{"-", "1"}], " "}]]}], TraditionalForm]]],
 "is an inner automorphism of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Therefore, by proposition 3.4, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Inn", "(", "G", ")"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", "G", ")"}], "."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436694642875*^9, 3.4366946848125*^9}, {
  3.436694773046875*^9, 3.436695017515625*^9}, {3.43669504975*^9, 
  3.4366950918125*^9}}]
}, Open  ]],

Cell[TextData[{
 "We found two inner-automorphisms of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", namely ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". We found one more inner automorphism by considering"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Inn = Group[{F,G}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \\\"\\\\!\\\\(\\\
\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
     C[$CellContext`j, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`i, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(j, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[$CellContext`j, 
      Dot[
       Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
     C[
      Dot[$CellContext`i, $CellContext`j], 
      Dot[
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{
  3.4226499135*^9, {3.44267380525825*^9, 3.44267383044575*^9}}]
}, Open  ]],

Cell[TextData[{
 "If we examine the automorphism ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Florin]", "x"], "(", "y", ")"}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "y", "\[CenterDot]", "x"}]}], 
   TraditionalForm]]],
 " for different elements ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", we will find that we will always get one of these four automorphisms. \
Thus, this group is the group of inner automorphisms on ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". Here is the multiplication table:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43669511034375*^9, 3.436695114046875*^9}}],

Cell[CellGroupData[{

Cell["MultTable[Inn]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}}, {{0, 1,
     1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {
    1, 1, 0}, {1, 0, 0}, {0, 1, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}, {1, 0, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
    0}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"C[]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 3.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"C[i,i\[CenterDot]i\[CenterDot]i]\[CenterDot]C[j,i\
\[CenterDot]i\[CenterDot]j]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 2.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"C[i,i\[CenterDot]i\[CenterDot]i]\[CenterDot]C[i\
\[CenterDot]j,i\[CenterDot]i\[CenterDot]i\[CenterDot]j]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 1.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"C[j,i\[CenterDot]i\[CenterDot]j]\[CenterDot]C[i\
\[CenterDot]j,i\[CenterDot]i\[CenterDot]i\[CenterDot]j]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 0.5}, {1, 0}]}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{
  3.4226499155*^9, {3.442673808336375*^9, 3.4426738321645*^9}},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "Since the square of every element is the identity, we recognize ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Inn", "(", "Q", ")"}], TraditionalForm]]],
 " as isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ".\n\nBecause the inner automorphism group is always a normal subgroup, we \
could consider the quotient group.\n\nDEFINITION 6.5:\nWe define the ",
 StyleBox["outer automorphism group",
  FontSlant->"Italic"],
 " to be the group  \n     \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Out", "(", "G", ")"}], "=", " ", 
    RowBox[{
     RowBox[{"Aut", "(", "G", ")"}], "/", 
     RowBox[{"Inn", "(", "G", ")"}]}]}], TraditionalForm]]],
 ".\n\nFor example, the outer automorphism group of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " is given by"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43669513278125*^9, 3.436695133765625*^9}}],

Cell[CellGroupData[{

Cell["Z = LftCoset[A, Inn]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.442673839930125*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", "]"}], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \\\"\\\\!\\\\(\\\
\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`i, 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
       C[$CellContext`j, 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`i, 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
       C[
        Dot[$CellContext`i, $CellContext`j], 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(j, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`j, 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
       C[
        Dot[$CellContext`i, $CellContext`j], 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"i", ",", "j", ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[$CellContext`i, $CellContext`i], $CellContext`i],
        Editable->False], ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[$CellContext`i, $CellContext`i], $CellContext`j],
        Editable->False]}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"i", ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[$CellContext`i, $CellContext`i], $CellContext`j],
        Editable->False], ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[$CellContext`i, $CellContext`i], $CellContext`i],
        Editable->False], ",", "j"}], "]"}], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, j\\\\)\\\\)]\\\
\\) \\\\[CenterDot] \
\\\\!\\\\(C[\\\\(\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\", \\\\\\\"\\\\\\\\!\\\
\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(\
i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        C[$CellContext`i, $CellContext`j], 
        C[
         Dot[$CellContext`i, $CellContext`j], 
         Dot[
          Dot[
           
           Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]], 
       C[
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \\\\\\\"\\\\\\\
\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\
\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\
\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \\\\[CenterDot] \
\\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\
\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\
\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\
\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        C[$CellContext`i, 
         Dot[
          Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
        C[$CellContext`j, 
         Dot[
          Dot[$CellContext`i, $CellContext`i], $CellContext`i]]], 
       C[
        Dot[$CellContext`i, $CellContext`j], 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"i", ",", 
       InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
        Editable->False], ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[$CellContext`i, $CellContext`i], $CellContext`i],
        Editable->False], ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\
\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j],
        Editable->False]}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"i", ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\
\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j],
        Editable->False], ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[$CellContext`i, $CellContext`i], $CellContext`i],
        Editable->False], ",", 
       InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
        Editable->False]}], "]"}], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \\\\\\\"\\\\\\\
\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\
\"\\\\)\\\\)]\\\\) \\\\[CenterDot] \\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\
\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\
\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(j\\\\\\\\)\
\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \\!\\(C[\\(\\(\\\"\\\\!\\\\(\\\\\
\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        C[$CellContext`i, 
         Dot[$CellContext`i, $CellContext`j]], 
        C[$CellContext`j, 
         Dot[
          Dot[$CellContext`i, $CellContext`i], $CellContext`j]]], 
       C[
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \\\\\\\"\\\\\\\
\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\(i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \
\\\\[CenterDot] \\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\
\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\
\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(j\\\\\\\\)\
\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        C[$CellContext`i, 
         Dot[
          Dot[
           
           Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]], 
        C[$CellContext`j, 
         Dot[
          Dot[$CellContext`i, $CellContext`i], $CellContext`j]]], 
       C[
        Dot[$CellContext`i, $CellContext`j], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i]]],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"j", ",", 
       InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
        Editable->False], ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[$CellContext`i, $CellContext`i], $CellContext`j],
        Editable->False], ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\
\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j],
        Editable->False]}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"j", ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\
\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j],
        Editable->False], ",", 
       InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", 
         Dot[$CellContext`i, $CellContext`i], $CellContext`j],
        Editable->False], ",", 
       InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
        StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
        Editable->False]}], "]"}], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \\\\\\\"\\\\\\\
\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\
\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\
\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \\\\[CenterDot] \
\\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        C[$CellContext`i, 
         Dot[
          Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
        C[$CellContext`j, 
         Dot[$CellContext`i, $CellContext`j]]], 
       C[
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(C[\\\\(\\\\(i, \\\\\\\"\\\\\\\
\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\
\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\
\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\) \\\\[CenterDot] \
\\\\!\\\\(C[\\\\(\\\\(j, \\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\
\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\
 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(i\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(j\\\\\\\\)\\\\\\\"\\\\)\\\\)]\\\\)\\\"\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        C[$CellContext`i, 
         Dot[
          Dot[$CellContext`i, $CellContext`i], $CellContext`i]], 
        C[$CellContext`j, 
         Dot[
          Dot[
           
           Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]], 
       C[
        Dot[$CellContext`i, $CellContext`j], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(C[\\(\\(i, j, \\\"\\\\!\\\\(i\\\\) \
\\\\[CenterDot] \\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`i, $CellContext`j, 
        Dot[$CellContext`i, $CellContext`j]], 
       C[
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(i, j, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`i, $CellContext`j, 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]], 
       C[
        Dot[$CellContext`i, $CellContext`j], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \
\[CenterDot] \\!\\(C[\\(\\(j, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`i, 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
        Dot[$CellContext`i, $CellContext`j]], 
       C[$CellContext`j, 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \
\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\\\\!\\\\(\\\\\
\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`i, 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]], 
       C[$CellContext`j, 
        Dot[$CellContext`i, $CellContext`j], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i]]],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(C[\\(\\(i, \\\"\\\\!\\\\(i\\\\) \
\\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", j\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(\
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \\\\!\\\\(i\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`i, 
        Dot[$CellContext`i, $CellContext`j], $CellContext`j], 
       C[
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(i, \\\"\\\\!\\\\(i\\\\) \
\\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \\\\\\\
\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \\\"\\\\!\\\\(\\\
\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`i, 
        Dot[$CellContext`i, $CellContext`j], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
       C[$CellContext`j, 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", j\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`i, 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j], $CellContext`j], 
       C[
        Dot[$CellContext`i, $CellContext`j], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i]]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(i, \
\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\
\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\(i\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\", \\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\
\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(j\\\\)\\\"\\)\\)]\\) \[CenterDot] \\!\\(C[\\(\\(j, \\\"\\\\!\\\\(\\\
\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\", \\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \\\\!\\\\(j\\\\)\\\
\"\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[$CellContext`i, 
        Dot[
         Dot[
          
          Dot[$CellContext`i, $CellContext`i], $CellContext`i], \
$CellContext`j], 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`j]], 
       C[$CellContext`j, 
        Dot[
         Dot[$CellContext`i, $CellContext`i], $CellContext`i], 
        Dot[$CellContext`i, $CellContext`j]]],
      Editable->False]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.422649923109375*^9, {3.442673834398875*^9, 3.442673841180125*^9}}]
}, Open  ]],

Cell["This gives a group of order 6. ", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "EXPERIMENT:\nThe elements of the group ",
 StyleBox["Z",
  FontWeight->"Bold"],
 " are too complicated to have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " complete a multiplication table for this group. Can you determine without \
asking for the multiplication table whether ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Out", "(", "Q", ")"}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "3"], "?"}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is an abelian group, then the only inner automorphism is the identity \
automorphism. Thus, for abelian groups, \n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Inn", "(", "G", ")"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Out", "(", "G", ")"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"Aut", "(", "G", ")"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\nEXPERIMENT:\nLoad ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with the following commands:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2,e]
Define[b^3,e]
Define[b.a,a.b.b]
Define[1/a,a]
Define[1/b,b.b]
G = Group[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.422649943640625*^9, 3.442673854242625*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that for any automorphism, the element ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " must map to one of the 3 elements of order 2, and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " must map to one of the 2 elements of order 3. Since the elements ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " generate the group, there can be at most 6 automorphisms. By find a few of \
the inner automorphisms, and having ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " create\na multiplication table, identify ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Inn", "(", 
    SubscriptBox["S", "3"], ")"}], TraditionalForm]]],
 ". What is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["S", "3"], ")"}], TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.436695189375*^9}],

Cell[TextData[{
 "Let us look at one last example where the automorphism group turns out to \
be much larger than the original group. Consider the automorphism of the \
group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ". The following loads the group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436695189375*^9, 3.436695194015625*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2,e]; Define[b^2,e]; Define[c^2,e]
Define[b.a,a.b]; Define[c.a,a.c]; Define[c.b,b.c]
Define[1/a,a]; Define[1/b,b]; Define[1/c,c]
Y = Group[{a,b,c}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436807931421875*^9, 3.4368079481875*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", "c", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`c],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.422649964265625*^9, 3.44267385694575*^9}]
}, Open  ]],

Cell[TextData[{
 "Suppose ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "x", ")"}], TraditionalForm]]],
 " is an automorphism of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ". Naturally, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "e", ")"}], "=", "e"}], TraditionalForm]]],
 ", but the other 7 elements are of order 2. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "a", ")"}], TraditionalForm]]],
 " could be any of the 7 remaining elements. Now ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "a", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "b", ")"}], TraditionalForm]]],
 " are unrelated so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "b", ")"}], TraditionalForm]]],
 " could be any of the 6 elements besides ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "a", ")"}], TraditionalForm]]],
 ". We then would have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", 
     RowBox[{"a", "\[CenterDot]", "b"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Phi]", "(", "a", ")"}], "\[CenterDot]", 
     RowBox[{"\[Phi]", "(", "b", ")"}]}]}], TraditionalForm]]],
 ", so this would be determined. But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "c", ")"}], TraditionalForm]]],
 " could be any of the 4 elements left over. Once ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "a", ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "b", ")"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "c", ")"}], TraditionalForm]]],
 " are known the entire automorphism is determined. Therefore, there are at \
most ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"7", "\[CenterDot]", "6", "\[CenterDot]", "4"}], " ", "=", " ", 
    "168"}], TraditionalForm]]],
 " possible automorphisms.\n\nNow let us try to find an automorphism. Since \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", ","}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " are interchangeable in the definition of the group, we could consider a \
mapping ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " that maps ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " back to ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 " would map to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "c"}], TraditionalForm]]],
 ", which would map to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "\[CenterDot]", "a"}], TraditionalForm]]],
 ", which would map back to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 ". The element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b", "\[CenterDot]", "c"}], 
   TraditionalForm]]],
 " would be fixed by this mapping. Thus, this mapping would be the \
permutation"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["F = C[a, b, c] . C[a.b, b.c, a.c]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(C[\\(\\(a, b, c\\)\\)]\\) \[CenterDot] \\!\\(C[\
\\(\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \\\\!\\\\(b\\\\)\\\", \
\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \\\\!\\\\(c\\\\)\\\", \
\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\)\\)]\\)\"\>",
  StringForm["`` \[CenterDot] ``", 
   C[$CellContext`a, $CellContext`b, $CellContext`c], 
   C[
    Dot[$CellContext`a, $CellContext`b], 
    Dot[$CellContext`b, $CellContext`c], 
    Dot[$CellContext`a, $CellContext`c]]],
  Editable->False]], "Output",
 CellChangeTimes->{3.42264997021875*^9, 3.44267385900825*^9}]
}, Open  ]],

Cell["Here is the graph of this permutation:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[Y, F]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"a\[CenterDot]b\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865475, 0.7071067811865476}, {0.961731656763491, 
       0.09238795325112874}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {1.0079256333890554`, 
       0.11152212486938323`}, {0.9155376801379267, 0.07325378163287424}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{1., 6.123233995736766*^-17}, {
       0.7453751244230565, -0.6147188279354188}}], 
      PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
       0.7915691010486209, -0.6338529995536732}, {
       0.6991811477974922, -0.5955846563171643}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865476, -0.7071067811865475}, {0.7071067811865475,
        0.6071067811865476}}], 
      PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.6571067811865474, 0.6071067811865476}, {0.7571067811865475, 
       0.6071067811865476}}]}}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.9292893218813453, \
-0.0707106781186549}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.03535533905932753}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {-0.7453751244230564, \
-0.614718827935419}}], 
      PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6991811477974921, -0.5955846563171645}, {-0.7915691010486208, \
-0.6338529995536735}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.09238795325112857, -0.961731656763491}}], 
      PolygonBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.07325378163287409, \
-0.9155376801379267}, {-0.11152212486938305`, -1.0079256333890554`}}]}}, 
    {RGBColor[1, 0.6, 0], 
     LineBox[{{-0.7071067811865477, 0.6071067811865474}, {-0.6995067811865476,
       0.5688067811865474}, {-0.6778067811865476, 
      0.5364067811865474}, {-0.6454067811865476, 
      0.5147067811865473}, {-0.6071067811865476, 
      0.5071067811865474}, {-0.5688067811865476, 
      0.5147067811865473}, {-0.5364567811865477, 
      0.5364067811865474}, {-0.5147067811865477, 
      0.5688067811865474}, {-0.5071067811865477, 
      0.6071067811865474}, {-0.5147067811865477, 
      0.6454067811865474}, {-0.5364067811865476, 
      0.6778067811865474}, {-0.5688067811865477, 
      0.6995067811865474}, {-0.6071067811865477, 
      0.7071067811865474}, {-0.7071067811865477, 0.7071067811865474}}], 
     PolygonBox[{{-0.7071067811865477, 
      0.7071067811865474}, {-0.7571067811865476, 
      0.6071067811865474}, {-0.6571067811865476, 0.6071067811865474}}]}}},
  AspectRatio->Automatic,
  ImageSize->{198.66666666666657`, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.422649976875*^9, 3.4426738602895*^9}]
}, Open  ]],

Cell[TextData[{
 "We can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " check that this is an automorphism by the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[F, Y]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.42264998353125*^9, 3.442673861727*^9}]
}, Open  ]],

Cell[TextData[{
 "Therefore ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " is an automorphism of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ". \n\nCan we find other automorphisms? Our success should indicate that \
there are many of them! Suppose we consider ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "(", "a", ")"}], " ", "=", " ", "a"}], ","}], 
   TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "(", "c", ")"}], " ", "=", " ", "c"}], ","}], 
   TraditionalForm]]],
 " but let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "b", ")"}], " ", "=", " ", 
    RowBox[{"a", "\[CenterDot]", 
     RowBox[{"b", "?"}]}]}], TraditionalForm]]],
 " Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", 
    RowBox[{"a", "\[CenterDot]", "c"}], ")"}], TraditionalForm]]],
 " would be ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "c"}], ","}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", 
    RowBox[{"a", "\[CenterDot]", "b"}], ")"}], TraditionalForm]]],
 " would be ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"a", "\[CenterDot]", "a", "\[CenterDot]", "b"}], " ", "=", " ", 
     "b"}], ","}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", 
    RowBox[{"b", "\[CenterDot]", "c"}], ")"}], TraditionalForm]]],
 " would be ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "b", "\[CenterDot]", "c"}], ","}], 
   TraditionalForm]]],
 " and finally ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", 
    RowBox[{"a", "\[CenterDot]", "b", "\[CenterDot]", "c"}], ")"}], 
   TraditionalForm]]],
 " would be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", 
    RowBox[{"c", "."}]}], TraditionalForm]]],
 " Hence, this mapping would be described by"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = C[b, a.b] . C[b.c, a.b.c]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(C[\\(\\(b, \\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \\\\!\\\\(c\\\\)\\\", \\\"\
\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\\\\) \\\\\\\\[CenterDot] \
\\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\)\\)]\\)\"\>",
  StringForm["`` \[CenterDot] ``", 
   C[$CellContext`b, 
    Dot[$CellContext`a, $CellContext`b]], 
   C[
    Dot[$CellContext`b, $CellContext`c], 
    Dot[
     Dot[$CellContext`a, $CellContext`b], $CellContext`c]]],
  Editable->False]], "Output",
 CellChangeTimes->{3.422649987015625*^9, 3.442673863305125*^9}]
}, Open  ]],

Cell["The graph is as follows:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[Y, G]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.442673864586375*^9, 3.442673865852*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"e\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"a\[CenterDot]b\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]c\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, 
    {RGBColor[0, 1, 0], 
     LineBox[{{0.6071067811865475, 0.7071067811865476}, {0.5688067811865475, 
      0.6995067811865475}, {0.5364067811865475, 0.6778067811865476}, {
      0.5147067811865476, 0.6454067811865476}, {0.5071067811865475, 
      0.6071067811865476}, {0.5147067811865476, 0.5688067811865476}, {
      0.5364567811865475, 0.5364067811865476}, {0.5688067811865475, 
      0.5147067811865476}, {0.6071067811865475, 0.5071067811865476}, {
      0.6454067811865475, 0.5147067811865476}, {0.6778067811865475, 
      0.5364067811865476}, {0.6995067811865475, 0.5688067811865476}, {
      0.7071067811865475, 0.6071067811865476}, {0.7071067811865475, 
      0.7071067811865476}}], 
     PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
      0.6071067811865475, 0.7571067811865475}, {0.6071067811865475, 
      0.6571067811865475}}]}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{1., 6.123233995736766*^-17}, {
       0.07071067811865488, -0.9292893218813453}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.10606601717798225`, -0.9646446609406726}, {
       0.035355339059327494`, -0.8939339828220179}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {
       0.9292893218813453, -0.07071067811865471}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {
       0.8939339828220179, -0.03535533905932732}, {
       0.9646446609406726, -0.10606601717798207`}}]}}, 
    {RGBColor[1, 0.6, 0], 
     LineBox[{{0.7071067811865476, -0.6071067811865475}, {
      0.6995067811865475, -0.5688067811865475}, {
      0.6778067811865476, -0.5364067811865475}, {
      0.6454067811865476, -0.5147067811865476}, {
      0.6071067811865476, -0.5071067811865475}, {
      0.5688067811865476, -0.5147067811865476}, {
      0.5364567811865476, -0.5364067811865475}, {
      0.5147067811865476, -0.5688067811865475}, {
      0.5071067811865476, -0.6071067811865475}, {
      0.5147067811865476, -0.6454067811865475}, {
      0.5364067811865476, -0.6778067811865475}, {
      0.5688067811865476, -0.6995067811865475}, {
      0.6071067811865476, -0.7071067811865475}, {
      0.7071067811865476, -0.7071067811865475}}], 
     PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
      0.7571067811865475, -0.6071067811865475}, {
      0.6571067811865475, -0.6071067811865475}}]}, 
    {RGBColor[0.7, 0.8, 0.7], 
     LineBox[{{-0.6071067811865475, -0.7071067811865477}, \
{-0.5688067811865475, -0.6995067811865476}, {-0.5364067811865475, \
-0.6778067811865476}, {-0.5147067811865474, -0.6454067811865476}, \
{-0.5071067811865475, -0.6071067811865476}, {-0.5147067811865474, \
-0.5688067811865476}, {-0.5364567811865475, -0.5364067811865476}, \
{-0.5688067811865475, -0.5147067811865477}, {-0.6071067811865475, \
-0.5071067811865477}, {-0.6454067811865475, -0.5147067811865477}, \
{-0.6778067811865475, -0.5364067811865476}, {-0.6995067811865475, \
-0.5688067811865477}, {-0.7071067811865475, -0.6071067811865477}, \
{-0.7071067811865475, -0.7071067811865477}}], 
     PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6071067811865475, -0.7571067811865476}, {-0.6071067811865475, \
-0.6571067811865476}}]}, {
     {RGBColor[0.8, 0.6, 0.5], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {-0.7453751244230566, 
       0.6147188279354187}}], 
      PolygonBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.791569101048621, 
       0.6338529995536731}, {-0.6991811477974923, 0.5955846563171642}}]}, 
     {RGBColor[0.8, 0.6, 0.5], 
      LineBox[{{-0.7071067811865477, 0.7071067811865474}, {-0.961731656763491,
        0.09238795325112852}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9155376801379267, 
       0.07325378163287403}, {-1.0079256333890554`, 0.111522124869383}}]}}}},
  AspectRatio->Automatic,
  ImageSize->{211.99999999999997`, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4226499891875*^9, 3.44267386644575*^9}]
}, Open  ]],

Cell[TextData[{
 "Would this be an automorphism? Let's check using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[G, Y]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.422649994546875*^9, 3.4426738682895*^9}]
}, Open  ]],

Cell[TextData[{
 "Therefore this is indeed another automorphism. We can now use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find other automorphisms generated by these two.\n\nBecause of the \
large number of possible elements in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "24", "*"]}], TraditionalForm]]],
 "), we will be better off if we number the elements in the group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ". This will allow us to use the ",
 StyleBox["P[ ]",
  FontWeight->"Bold"],
 " notation. (",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " works much faster using the ",
 StyleBox["P[ ]",
  FontWeight->"Bold"],
 " notation than using cycles.) Since all automorphisms fix the element ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " this element need not be numbered. The other 7 elements can be numbered as \
follows:\n \n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"                  ", "1", ")"}], "  ", "a"}], 
                    "                             ", "\n", "\t", "2"}], ")"}],
                    "  ", "b"}], "                             ", "\n", "\t", 
                  "3"}], ")"}], "  ", "c"}], "                             ", 
               "\n", "\t", "4"}], ")"}], "  ", 
             RowBox[{"a", "\[CenterDot]", "b"}]}], 
            "                             ", "\n", "\t", "5"}], ")"}], "  ", 
          RowBox[{"a", "\[CenterDot]", "c"}]}], 
         "                             ", "\n", "\t", "6"}], ")"}], "  ", 
       RowBox[{"b", "\[CenterDot]", "c"}]}], "                             ", 
      "\n", "\t", "7"}], ")"}], "  ", 
    RowBox[{"a", "\[CenterDot]", "b", "\[CenterDot]", 
     RowBox[{"c", "."}]}]}], TraditionalForm]]],
 "\n\nLet us convert ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " to a permutation using the numbering scheme we gave to the elements."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.422649999*^9, 3.4226500006875*^9}}],

Cell[CellGroupData[{

Cell["F = CycleToPerm[C[1,2,3] . C[4,6,5]]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"P", "[", 
  RowBox[{"2", ",", "3", ",", "1", ",", "6", ",", "4", ",", "5"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.422650004109375*^9, 3.442673870430125*^9}]
}, Open  ]],

Cell[TextData[{
 "The graph of ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " still looks the same:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6,7}, F]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"4\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865475, 0.7071067811865476}, {0.961731656763491, 
       0.09238795325112874}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {1.0079256333890554`, 
       0.11152212486938323`}, {0.9155376801379267, 0.07325378163287424}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{1., 6.123233995736766*^-17}, {
       0.7453751244230565, -0.6147188279354188}}], 
      PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
       0.7915691010486209, -0.6338529995536732}, {
       0.6991811477974922, -0.5955846563171643}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865476, -0.7071067811865475}, {0.7071067811865475,
        0.6071067811865476}}], 
      PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.6571067811865474, 0.6071067811865476}, {0.7571067811865475, 
       0.6071067811865476}}]}}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.9292893218813453, \
-0.0707106781186549}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9646446609406726, \
-0.1060660171779823}, {-0.8939339828220179, -0.03535533905932753}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {-0.7453751244230564, \
-0.614718827935419}}], 
      PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6991811477974921, -0.5955846563171645}, {-0.7915691010486208, \
-0.6338529995536735}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.09238795325112857, -0.961731656763491}}], 
      PolygonBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.07325378163287409, \
-0.9155376801379267}, {-0.11152212486938305`, -1.0079256333890554`}}]}}, 
    {RGBColor[1, 0.6, 0], 
     LineBox[{{-0.7071067811865477, 0.6071067811865474}, {-0.6995067811865476,
       0.5688067811865474}, {-0.6778067811865476, 
      0.5364067811865474}, {-0.6454067811865476, 
      0.5147067811865473}, {-0.6071067811865476, 
      0.5071067811865474}, {-0.5688067811865476, 
      0.5147067811865473}, {-0.5364567811865477, 
      0.5364067811865474}, {-0.5147067811865477, 
      0.5688067811865474}, {-0.5071067811865477, 
      0.6071067811865474}, {-0.5147067811865477, 
      0.6454067811865474}, {-0.5364067811865476, 
      0.6778067811865474}, {-0.5688067811865477, 
      0.6995067811865474}, {-0.6071067811865477, 
      0.7071067811865474}, {-0.7071067811865477, 0.7071067811865474}}], 
     PolygonBox[{{-0.7071067811865477, 
      0.7071067811865474}, {-0.7571067811865476, 
      0.6071067811865474}, {-0.6571067811865476, 0.6071067811865474}}]}}},
  AspectRatio->Automatic,
  ImageSize->{160.00000000000023`, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4226500065625*^9, 3.44267387200825*^9}]
}, Open  ]],

Cell[TextData[{
 "That is, element 1 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "a", ")"}], TraditionalForm]]],
 " is mapped to element 2 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "b", ")"}], TraditionalForm]]],
 ", which is mapped to element 4 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "c", ")"}], TraditionalForm]]],
 ", etc.\n\nLikewise, ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " exchanges the 2nd and 3rd elements, and exchanges the 6th and 7th elements \
of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ", we can convert ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " to a permutation as follows:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = CycleToPerm[C[2,4] . C[6,7]]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"P", "[", 
  RowBox[{"1", ",", "4", ",", "3", ",", "2", ",", "5", ",", "7", ",", "6"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.422650011171875*^9, 3.4426738737895*^9}]
}, Open  ]],

Cell[TextData[{
 "The graph of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " now becomes"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[{0,1,2,3,4,5,6,7}, G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"4\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.07071067811865475, 
      0.9292893218813453}, {-0.09241885630108176, 
      0.8968331206248827}, {-0.09998489885977781, 
      0.8585786437626906}, {-0.09241885630108176, 
      0.8203241669004984}, {-0.07071067811865475, 
      0.7878679656440357}, {-0.038254476862192216`, 0.7661597874616087}, {0., 
      0.7585937449029128}, {0.038254476862192216`, 0.7661597874616087}, {
      0.07071067811865475, 0.7878679656440357}, {0.09241885630108176, 
      0.8203241669004984}, {0.09998489885977781, 0.8585786437626906}, {
      0.09241885630108176, 0.8968331206248827}, {0.07071067811865475, 
      0.9292893218813453}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {-0.10606601717798211`, 
      0.9646446609406726}, {-0.035355339059327376`, 0.8939339828220179}}]}, 
    {RGBColor[0, 1, 0], 
     LineBox[{{0.6071067811865475, 0.7071067811865476}, {0.5688067811865475, 
      0.6995067811865475}, {0.5364067811865475, 0.6778067811865476}, {
      0.5147067811865476, 0.6454067811865476}, {0.5071067811865475, 
      0.6071067811865476}, {0.5147067811865476, 0.5688067811865476}, {
      0.5364567811865475, 0.5364067811865476}, {0.5688067811865475, 
      0.5147067811865476}, {0.6071067811865475, 0.5071067811865476}, {
      0.6454067811865475, 0.5147067811865476}, {0.6778067811865475, 
      0.5364067811865476}, {0.6995067811865475, 0.5688067811865476}, {
      0.7071067811865475, 0.6071067811865476}, {0.7071067811865475, 
      0.7071067811865476}}], 
     PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
      0.6071067811865475, 0.7571067811865475}, {0.6071067811865475, 
      0.6571067811865475}}]}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{1., 6.123233995736766*^-17}, {
       0.07071067811865488, -0.9292893218813453}}], 
      PolygonBox[{{1.2246467991473532`*^-16, -1.}, {
       0.10606601717798225`, -0.9646446609406726}, {
       0.035355339059327494`, -0.8939339828220179}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {
       0.9292893218813453, -0.07071067811865471}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {
       0.8939339828220179, -0.03535533905932732}, {
       0.9646446609406726, -0.10606601717798207`}}]}}, 
    {RGBColor[1, 0.6, 0], 
     LineBox[{{0.7071067811865476, -0.6071067811865475}, {
      0.6995067811865475, -0.5688067811865475}, {
      0.6778067811865476, -0.5364067811865475}, {
      0.6454067811865476, -0.5147067811865476}, {
      0.6071067811865476, -0.5071067811865475}, {
      0.5688067811865476, -0.5147067811865476}, {
      0.5364567811865476, -0.5364067811865475}, {
      0.5147067811865476, -0.5688067811865475}, {
      0.5071067811865476, -0.6071067811865475}, {
      0.5147067811865476, -0.6454067811865475}, {
      0.5364067811865476, -0.6778067811865475}, {
      0.5688067811865476, -0.6995067811865475}, {
      0.6071067811865476, -0.7071067811865475}, {
      0.7071067811865476, -0.7071067811865475}}], 
     PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
      0.7571067811865475, -0.6071067811865475}, {
      0.6571067811865475, -0.6071067811865475}}]}, 
    {RGBColor[0.7, 0.8, 0.7], 
     LineBox[{{-0.6071067811865475, -0.7071067811865477}, \
{-0.5688067811865475, -0.6995067811865476}, {-0.5364067811865475, \
-0.6778067811865476}, {-0.5147067811865474, -0.6454067811865476}, \
{-0.5071067811865475, -0.6071067811865476}, {-0.5147067811865474, \
-0.5688067811865476}, {-0.5364567811865475, -0.5364067811865476}, \
{-0.5688067811865475, -0.5147067811865477}, {-0.6071067811865475, \
-0.5071067811865477}, {-0.6454067811865475, -0.5147067811865477}, \
{-0.6778067811865475, -0.5364067811865476}, {-0.6995067811865475, \
-0.5688067811865477}, {-0.7071067811865475, -0.6071067811865477}, \
{-0.7071067811865475, -0.7071067811865477}}], 
     PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6071067811865475, -0.7571067811865476}, {-0.6071067811865475, \
-0.6571067811865476}}]}, {
     {RGBColor[0.8, 0.6, 0.5], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {-0.7453751244230566, 
       0.6147188279354187}}], 
      PolygonBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.791569101048621, 
       0.6338529995536731}, {-0.6991811477974923, 0.5955846563171642}}]}, 
     {RGBColor[0.8, 0.6, 0.5], 
      LineBox[{{-0.7071067811865477, 0.7071067811865474}, {-0.961731656763491,
        0.09238795325112852}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9155376801379267, 
       0.07325378163287403}, {-1.0079256333890554`, 0.111522124869383}}]}}}},
  AspectRatio->Automatic,
  ImageSize->{131.33333333333286`, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.42265001296875*^9, 3.442673875305125*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "which is of course the same picture.\n\nSince ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " both represent automorphisms of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ", we can find other automorphisms be considering the group generated by ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". (Even with the improved notation, this may take a few seconds.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["A = Group[{F, G}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "1", ",", "3", ",", "4", ",", "6", ",", "5"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "3", ",", "1", ",", "6", ",", "4", ",", "5"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "4", ",", "6", ",", "1", ",", "3", ",", "5"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "6", ",", "4", ",", "3", ",", "1", ",", "5"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "1", ",", "2", ",", "5", ",", "6", ",", "4"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "2", ",", "1", ",", "6", ",", "5", ",", "4"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "5", ",", "6", ",", "1", ",", "2", ",", "4"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "6", ",", "5", ",", "2", ",", "1", ",", "4"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "1", ",", "5", ",", "2", ",", "6", ",", "3"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "2", ",", "6", ",", "1", ",", "5", ",", "3"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "5", ",", "1", ",", "6", ",", "2", ",", "3"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "6", ",", "2", ",", "5", ",", "1", ",", "3"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "1", ",", "4", ",", "3", ",", "6", ",", "2"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "3", ",", "6", ",", "1", ",", "4", ",", "2"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "4", ",", "1", ",", "6", ",", "3", ",", "2"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "6", ",", "3", ",", "4", ",", "1", ",", "2"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "2", ",", "4", ",", "3", ",", "5", ",", "1"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "3", ",", "5", ",", "2", ",", "4", ",", "1"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "4", ",", "2", ",", "5", ",", "3", ",", "1"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "5", ",", "3", ",", "4", ",", "2", ",", "1"}], "]"}], 
   ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2", ",", "5", ",", "7", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "3", ",", "4", ",", "7", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "4", ",", "3", ",", "7", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "5", ",", "2", ",", "7", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "1", ",", "3", ",", "2", ",", "7", ",", "5", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "2", ",", "7", ",", "1", ",", "3", ",", "5", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "3", ",", "1", ",", "7", ",", "2", ",", "5", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "7", ",", "2", ",", "3", ",", "1", ",", "5", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "1", ",", "2", ",", "3", ",", "7", ",", "4", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "2", ",", "1", ",", "7", ",", "3", ",", "4", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "3", ",", "7", ",", "1", ",", "2", ",", "4", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "7", ",", "3", ",", "2", ",", "1", ",", "4", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "1", ",", "5", ",", "4", ",", "7", ",", "3", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "4", ",", "7", ",", "1", ",", "5", ",", "3", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "5", ",", "1", ",", "7", ",", "4", ",", "3", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "7", ",", "4", ",", "5", ",", "1", ",", "3", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "1", ",", "4", ",", "5", ",", "7", ",", "2", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "4", ",", "1", ",", "7", ",", "5", ",", "2", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "5", ",", "7", ",", "1", ",", "4", ",", "2", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "7", ",", "5", ",", "4", ",", "1", ",", "2", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "4", ",", "2", ",", "3", ",", "5", ",", "1", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "5", ",", "3", ",", "2", ",", "4", ",", "1", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "2", ",", "4", ",", "5", ",", "3", ",", "1", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "3", ",", "5", ",", "4", ",", "2", ",", "1", ",", "6"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "4", ",", "3", ",", "1", ",", "6", ",", "7", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "6", ",", "1", ",", "3", ",", "4", ",", "7", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "1", ",", "6", ",", "4", ",", "3", ",", "7", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "3", ",", "4", ",", "6", ",", "1", ",", "7", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "2", ",", "3", ",", "1", ",", "7", ",", "6", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "1", ",", "7", ",", "2", ",", "3", ",", "6", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "7", ",", "1", ",", "3", ",", "2", ",", "6", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "3", ",", "2", ",", "7", ",", "1", ",", "6", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "2", ",", "1", ",", "3", ",", "7", ",", "4", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "1", ",", "2", ",", "7", ",", "3", ",", "4", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "7", ",", "3", ",", "1", ",", "2", ",", "4", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "3", ",", "7", ",", "2", ",", "1", ",", "4", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "2", ",", "6", ",", "4", ",", "7", ",", "3", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "4", ",", "7", ",", "2", ",", "6", ",", "3", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "6", ",", "2", ",", "7", ",", "4", ",", "3", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "7", ",", "4", ",", "6", ",", "2", ",", "3", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "4", ",", "1", ",", "3", ",", "6", ",", "2", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "6", ",", "3", ",", "1", ",", "4", ",", "2", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "1", ",", "4", ",", "6", ",", "3", ",", "2", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "3", ",", "6", ",", "4", ",", "1", ",", "2", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "2", ",", "4", ",", "6", ",", "7", ",", "1", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "4", ",", "2", ",", "7", ",", "6", ",", "1", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "6", ",", "7", ",", "2", ",", "4", ",", "1", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "7", ",", "6", ",", "4", ",", "2", ",", "1", ",", "5"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "5", ",", "2", ",", "1", ",", "6", ",", "7", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "6", ",", "1", ",", "2", ",", "5", ",", "7", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "1", ",", "6", ",", "5", ",", "2", ",", "7", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "2", ",", "5", ",", "6", ",", "1", ",", "7", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "3", ",", "2", ",", "1", ",", "7", ",", "6", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "7", ",", "1", ",", "2", ",", "3", ",", "6", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "1", ",", "7", ",", "3", ",", "2", ",", "6", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "2", ",", "3", ",", "7", ",", "1", ",", "6", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "3", ",", "1", ",", "2", ",", "7", ",", "5", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "7", ",", "2", ",", "1", ",", "3", ",", "5", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "1", ",", "3", ",", "7", ",", "2", ",", "5", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "2", ",", "7", ",", "3", ",", "1", ",", "5", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "5", ",", "1", ",", "2", ",", "6", ",", "3", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "6", ",", "2", ",", "1", ",", "5", ",", "3", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "1", ",", "5", ",", "6", ",", "2", ",", "3", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "2", ",", "6", ",", "5", ",", "1", ",", "3", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "3", ",", "6", ",", "5", ",", "7", ",", "2", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "5", ",", "7", ",", "3", ",", "6", ",", "2", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "6", ",", "3", ",", "7", ",", "5", ",", "2", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "7", ",", "5", ",", "6", ",", "3", ",", "2", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "3", ",", "5", ",", "6", ",", "7", ",", "1", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "5", ",", "3", ",", "7", ",", "6", ",", "1", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "6", ",", "7", ",", "3", ",", "5", ",", "1", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "7", ",", "6", ",", "5", ",", "3", ",", "1", ",", "4"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "2", ",", "5", ",", "1", ",", "6", ",", "7", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "1", ",", "6", ",", "2", ",", "5", ",", "7", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "6", ",", "1", ",", "5", ",", "2", ",", "7", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "5", ",", "2", ",", "6", ",", "1", ",", "7", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "4", ",", "5", ",", "1", ",", "7", ",", "6", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "1", ",", "7", ",", "4", ",", "5", ",", "6", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "7", ",", "1", ",", "5", ",", "4", ",", "6", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "5", ",", "4", ",", "7", ",", "1", ",", "6", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "4", ",", "6", ",", "2", ",", "7", ",", "5", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "2", ",", "7", ",", "4", ",", "6", ",", "5", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "7", ",", "2", ",", "6", ",", "4", ",", "5", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "6", ",", "4", ",", "7", ",", "2", ",", "5", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "2", ",", "1", ",", "5", ",", "6", ",", "4", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "1", ",", "2", ",", "6", ",", "5", ",", "4", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "6", ",", "5", ",", "1", ",", "2", ",", "4", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "5", ",", "6", ",", "2", ",", "1", ",", "4", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "4", ",", "1", ",", "5", ",", "7", ",", "2", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "1", ",", "4", ",", "7", ",", "5", ",", "2", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "7", ",", "5", ",", "1", ",", "4", ",", "2", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "5", ",", "7", ",", "4", ",", "1", ",", "2", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "4", ",", "2", ",", "6", ",", "7", ",", "1", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "2", ",", "4", ",", "7", ",", "6", ",", "1", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "7", ",", "6", ",", "2", ",", "4", ",", "1", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "6", ",", "7", ",", "4", ",", "2", ",", "1", ",", "3"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "3", ",", "4", ",", "1", ",", "6", ",", "7", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "1", ",", "6", ",", "3", ",", "4", ",", "7", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "6", ",", "1", ",", "4", ",", "3", ",", "7", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "4", ",", "3", ",", "6", ",", "1", ",", "7", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "5", ",", "4", ",", "1", ",", "7", ",", "6", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "7", ",", "1", ",", "4", ",", "5", ",", "6", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "1", ",", "7", ",", "5", ",", "4", ",", "6", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "4", ",", "5", ",", "7", ",", "1", ",", "6", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "3", ",", "1", ",", "4", ",", "6", ",", "5", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "1", ",", "3", ",", "6", ",", "4", ",", "5", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "6", ",", "4", ",", "1", ",", "3", ",", "5", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "4", ",", "6", ",", "3", ",", "1", ",", "5", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "5", ",", "6", ",", "3", ",", "7", ",", "4", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "3", ",", "7", ",", "5", ",", "6", ",", "4", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "7", ",", "3", ",", "6", ",", "5", ",", "4", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"1", ",", "6", ",", "5", ",", "7", ",", "3", ",", "4", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "5", ",", "1", ",", "4", ",", "7", ",", "3", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "7", ",", "4", ",", "1", ",", "5", ",", "3", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "1", ",", "5", ",", "7", ",", "4", ",", "3", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "4", ",", "7", ",", "5", ",", "1", ",", "3", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "5", ",", "3", ",", "6", ",", "7", ",", "1", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "3", ",", "5", ",", "7", ",", "6", ",", "1", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "7", ",", "6", ",", "3", ",", "5", ",", "1", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "6", ",", "7", ",", "5", ",", "3", ",", "1", ",", "2"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "3", ",", "4", ",", "2", ",", "5", ",", "7", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "2", ",", "5", ",", "3", ",", "4", ",", "7", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "5", ",", "2", ",", "4", ",", "3", ",", "7", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"6", ",", "4", ",", "3", ",", "5", ",", "2", ",", "7", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "3", ",", "2", ",", "4", ",", "5", ",", "6", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "2", ",", "3", ",", "5", ",", "4", ",", "6", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "5", ",", "4", ",", "2", ",", "3", ",", "6", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"7", ",", "4", ",", "5", ",", "3", ",", "2", ",", "6", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "6", ",", "4", ",", "2", ",", "7", ",", "5", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "7", ",", "2", ",", "4", ",", "6", ",", "5", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "2", ",", "7", ",", "6", ",", "4", ",", "5", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "4", ",", "6", ",", "7", ",", "2", ",", "5", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "6", ",", "5", ",", "3", ",", "7", ",", "4", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "7", ",", "3", ",", "5", ",", "6", ",", "4", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "3", ",", "7", ",", "6", ",", "5", ",", "4", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "5", ",", "6", ",", "7", ",", "3", ",", "4", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "6", ",", "2", ",", "4", ",", "7", ",", "3", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "7", ",", "4", ",", "2", ",", "6", ",", "3", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "2", ",", "6", ",", "7", ",", "4", ",", "3", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"5", ",", "4", ",", "7", ",", "6", ",", "2", ",", "3", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "6", ",", "3", ",", "5", ",", "7", ",", "2", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "7", ",", "5", ",", "3", ",", "6", ",", "2", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "3", ",", "6", ",", "7", ",", "5", ",", "2", ",", "1"}],
     "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "5", ",", "7", ",", "6", ",", "3", ",", "2", ",", "1"}],
     "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4226500259375*^9, 3.442673878086375*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " came up with quite a few automorphisms. Let's see how many we have."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[A]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["168"], "Output",
 CellChangeTimes->{3.422650029640625*^9, 3.44267388025825*^9}]
}, Open  ]],

Cell[TextData[{
 "Therefore, we have found 168 different automorphisms of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ". We saw that this was the maximum number of automorphisms we could have, \
so we must have all of them. However, this is a difficult way to see the \
elements of the group! It is a challenge just to see if a given permutation \
is somewhere in this list. Is there an easier way of viewing this group?\n\n\
One way to tidy up this long list is to convert the permutations to numbers. \
Recall that in Chapter 5 we ordered all of the permutations, thereby \
converting every permutation to a number. For example, we can convert ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " to numbers by the commands"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["PermToInt[F]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["149"], "Output",
 CellChangeTimes->{3.422650040203125*^9, 3.44267388232075*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["PermToInt[G]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["735"], "Output",
 CellChangeTimes->{3.422650041828125*^9, 3.442673883867625*^9}]
}, Open  ]],

Cell[TextData[{
 "Thus, ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " represent the 149th and the 735rd permutations. We can define the product \
on two numbers by the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["InitPermMultiplication", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["\<\
Therefore the entire group can be generated by 149 and 735, as follows:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["A = Group[{149,735}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "27", ",", "61", ",", "87", ",", "122", ",", "149", ",", "187", 
   ",", "231", ",", "244", ",", "270", ",", "331", ",", "357", ",", "374", 
   ",", "404", ",", "437", ",", "467", ",", "496", ",", "548", ",", "558", 
   ",", "593", ",", "640", ",", "670", ",", "684", ",", "714", ",", "735", 
   ",", "753", ",", "775", ",", "793", ",", "856", ",", "908", ",", "918", 
   ",", "953", ",", "970", ",", "1014", ",", "1052", ",", "1079", ",", "1088",
    ",", "1123", ",", "1133", ",", "1185", ",", "1202", ",", "1229", ",", 
   "1267", ",", "1311", ",", "1362", ",", "1392", ",", "1402", ",", "1432", 
   ",", "1461", ",", "1475", ",", "1496", ",", "1537", ",", "1582", ",", 
   "1622", ",", "1649", ",", "1662", ",", "1692", ",", "1732", ",", "1775", 
   ",", "1796", ",", "1807", ",", "1837", ",", "1851", ",", "1881", ",", 
   "1962", ",", "1992", ",", "2002", ",", "2032", ",", "2042", ",", "2069", 
   ",", "2107", ",", "2151", ",", "2183", ",", "2201", ",", "2240", ",", 
   "2258", ",", "2304", ",", "2345", ",", "2366", ",", "2380", ",", "2418", 
   ",", "2471", ",", "2476", ",", "2510", ",", "2562", ",", "2592", ",", 
   "2602", ",", "2632", ",", "2647", ",", "2677", ",", "2691", ",", "2721", 
   ",", "2761", ",", "2787", ",", "2821", ",", "2847", ",", "2900", ",", 
   "2918", ",", "2963", ",", "2981", ",", "3019", ",", "3032", ",", "3059", 
   ",", "3099", ",", "3133", ",", "3151", ",", "3177", ",", "3195", ",", 
   "3276", ",", "3298", ",", "3336", ",", "3358", ",", "3372", ",", "3412", 
   ",", "3455", ",", "3476", ",", "3486", ",", "3508", ",", "3573", ",", 
   "3595", ",", "3622", ",", "3662", ",", "3689", ",", "3702", ",", "3741", 
   ",", "3755", ",", "3776", ",", "3817", ",", "3876", ",", "3898", ",", 
   "3936", ",", "3958", ",", "3973", ",", "3991", ",", "4017", ",", "4035", 
   ",", "4098", ",", "4151", ",", "4156", ",", "4190", ",", "4205", ",", 
   "4226", ",", "4269", ",", "4309", ",", "4366", ",", "4384", ",", "4410", 
   ",", "4428", ",", "4476", ",", "4498", ",", "4536", ",", "4558", ",", 
   "4581", ",", "4595", ",", "4616", ",", "4657", ",", "4695", ",", "4713", 
   ",", "4735", ",", "4753", ",", "4817", ",", "4847", ",", "4874", ",", 
   "4904", ",", "4931", ",", "4965", ",", "4970", ",", "5023"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.4226500743125*^9, 3.4368080774375*^9, 3.4426738886645*^9}
  ]
}, Open  ]],

Cell[TextData[{
 "This may take some time, but it is worth the wait to see the group \
represented simply as a list of numbers. \n\nThis is the largest group with \
which we will work using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". In fact we will discover in the next chapter that this group has some ",
 StyleBox["very",
  FontSlant->"Italic"],
 " special properties. We will explore this group in detail using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in the next chapter."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Semi-direct Products", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "So far in this chapter we have seen how we can combine two groups ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " together to form a larger group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Times]", " ", "K"}], TraditionalForm]]],
 " using the direct product. We then saw that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Times]", " ", "K"}], TraditionalForm]]],
 " had isomorphic copies of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " as normal subgroups. \n\nIn this section we will see another way to \
combine to groups ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Once again the larger group will have isomorphic copies of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " as subgroups, but only ",
 StyleBox["one",
  FontSlant->"Italic"],
 " of the two subgroups will be a normal subgroup. For this reason, the \
combination of groups is called a ",
 StyleBox["semi-direct product",
  FontSlant->"Italic"],
 ".\n\nSuppose that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are any two groups, and suppose that we have a homomorphism \n\n           \
      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", ":", " ", 
    RowBox[{"H", " ", "\[RightArrow]", " ", 
     RowBox[{"Aut", "(", "K", ")"}]}]}], TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.442673895227*^9}],

Cell[TextData[{
 "That is, for every ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "h", ")"}], TraditionalForm]]],
 " will be a mapping from ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " into ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Because the output of the function ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is again a function, we will write ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "h"], TraditionalForm]]],
 " instead of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "h", ")"}], TraditionalForm]]],
 ". This way we can write ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Phi]", "h"], "(", "k", ")"}], TraditionalForm]]],
 " to express the function ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "h"], TraditionalForm]]],
 " evaluated at ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ".   That is, if ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["h", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["h", "2"], TraditionalForm]]],
 " are two elements of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Phi]", 
     SubscriptBox["h", "1"]], "(", "k", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Phi]", 
     SubscriptBox["h", "2"]], "(", "k", ")"}], TraditionalForm]]],
 " will be two automorphisms of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", and also ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", 
      RowBox[{
       SubscriptBox["h", "1"], "\[CenterDot]", 
       SubscriptBox["h", "2"]}]], "(", "k", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Phi]", 
         SubscriptBox["h", "1"]], "\[CenterDot]", 
        SubscriptBox["\[Phi]", 
         SubscriptBox["h", "2"]]}], ")"}], 
      RowBox[{"(", "k", ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Phi]", 
        SubscriptBox["h", "2"]], "(", 
       RowBox[{
        SubscriptBox["\[Phi]", 
         SubscriptBox["h", "1"]], "(", "k", ")"}], ")"}], "."}]}]}], 
   TraditionalForm]]],
 "  (Recall that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Phi]", 
     SubscriptBox["h", "1"]], "\[CenterDot]", 
    SubscriptBox["\[Phi]", 
     SubscriptBox["h", "2"]]}], TraditionalForm]]],
 " means we do ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", 
    SubscriptBox["h", "1"]], TraditionalForm]]],
 " first, then do ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", 
    SubscriptBox["h", "2"]], TraditionalForm]]],
 ".)\n\nThere will always be at least one possible homomorphism from ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "K", ")"}], TraditionalForm]]],
 " since we could let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "h"], TraditionalForm]]],
 " be the identity automorphism for all ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 ". However, there will usually be several other possible homomorphisms from \
",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "K", ")"}], TraditionalForm]]],
 ". For each such homomorphism, we can define a product of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". \n\nDEFINITION 6.6:\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " to be the set of all ordered pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", "k"}], ")"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " be a non-trivial homomorphism from ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", "K", ")"}], "."}], TraditionalForm]]],
 " Then the ",
 StyleBox["semi-direct product of",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " ",
 StyleBox["with",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " ",
 StyleBox["through",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", ","}], TraditionalForm]]],
 " denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", 
    SubscriptBox["\[Proportional]", "\[Phi]"], "K"}], TraditionalForm]]],
 ", is the set ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " with multiplication defined by\n    \n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       FormBox[
        RowBox[{"(", 
         SubscriptBox["h", "1"]}],
        TraditionalForm], ",", 
       SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["h", "2"], ",", 
       SubscriptBox[
        RowBox[{"k", " "}], "2"]}], ")"}]}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["h", "1"], "\[CenterDot]", 
       SubscriptBox["h", "2"]}], ",", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Phi]", 
         SubscriptBox["h", "2"]], "(", 
        SubscriptBox["k", "1"], ")"}], "\[CenterDot]", 
       SubscriptBox["k", "2"]}]}], ")"}]}], TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43680813634375*^9, 3.436808137390625*^9}, {
   3.4368081758125*^9, 3.436808196265625*^9}, {3.436808229984375*^9, 
   3.4368082620625*^9}, {3.436808547578125*^9, 3.43680859959375*^9}, {
   3.436808655640625*^9, 3.436808736734375*^9}, 3.43680881509375*^9, {
   3.4368088915*^9, 3.436809030421875*^9}, 3.436809089046875*^9, 
   3.436839833921875*^9, {3.44093712000825*^9, 3.440937218461375*^9}, {
   3.440938272680125*^9, 3.4409384497895*^9}, 3.44267389925825*^9}],

Cell[TextData[{
 "Note:  The real notation for the semi-direct product does not use a \
\[Proportional] symbol, but rather looks like a \[Cross]\nwith a vertical \
line on the left to make a triangle.  Unfortunately, this symbol is not in \
the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " palette, so the ",
 "\[Proportional]",
 " symbol is as close as one can get. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43680813634375*^9, 3.436808137390625*^9}, {
   3.4368081758125*^9, 3.436808196265625*^9}, {3.436808229984375*^9, 
   3.4368082620625*^9}, {3.436808547578125*^9, 3.43680859959375*^9}, {
   3.436808655640625*^9, 3.436808736734375*^9}, 3.43680881509375*^9, {
   3.4368088915*^9, 3.436809030421875*^9}, 3.436809089046875*^9, {
   3.436839833921875*^9, 3.436840005875*^9}, {3.44093368238325*^9, 
   3.440933707930125*^9}, 3.44267390138325*^9}],

Cell[TextData[{
 "PROPOSITION 6.6\nThe semi-direct product of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " through ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is a group."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43680813634375*^9, 3.436808137390625*^9}, {
   3.4368081758125*^9, 3.436808196265625*^9}, {3.436808229984375*^9, 
   3.4368082620625*^9}, {3.436808547578125*^9, 3.43680859959375*^9}, {
   3.436808655640625*^9, 3.436808736734375*^9}, 3.43680881509375*^9, {
   3.4368088915*^9, 3.436809030421875*^9}, 3.436809089046875*^9, {
   3.436839833921875*^9, 3.43684000771875*^9}, {3.440937243227*^9, 
   3.440937243961375*^9}}],

Cell[CellGroupData[{

Cell["Proof: ", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "It is clear that the product of two ordered pairs in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is an ordered pair in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". If we let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "1"], TraditionalForm]]],
 " denote the identity element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "2"], TraditionalForm]]],
 " denote the indentity element of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   RowBox[{Cell[""], Cell[""]}], TraditionalForm]]],
 "then \n                                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[Phi]", 
         SubscriptBox["e", "1"]],
        TraditionalForm], 
       FormBox[
        RowBox[{"(", 
         SubscriptBox["k", "1"]}],
        TraditionalForm]}], ")"}], " ", "=", " ", 
     FormBox[
      SubscriptBox["k", "1"],
      TraditionalForm]}], ","}], TraditionalForm]]],
 "\n\nsince ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " must map ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "1"], TraditionalForm]]],
 " to the identity automorphism of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Thus\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "1"], ",", 
        SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["e", "1"], ",", 
        SubscriptBox["e", "2"]}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["h", "1"], "\[CenterDot]", 
         SubscriptBox["e", "1"]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Phi]", 
           SubscriptBox["e", "1"]], "(", 
          SubscriptBox["k", "1"], ")"}], "\[CenterDot]", 
         SubscriptBox["e", "2"]}]}], " ", ")"}], " ", "=", " ", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "1"], ",", 
        SubscriptBox["k", "1"]}], ")"}]}]}], ",", "  "}], TraditionalForm]]],
 "\n                                  \nand\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["e", "1"], ",", 
        SubscriptBox["e", "2"]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "2"], ",", 
        SubscriptBox["k", "2"]}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["e", "1"], "\[CenterDot]", 
         SubscriptBox["h", "2"]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Phi]", 
           SubscriptBox["h", "2"]], "(", 
          SubscriptBox["e", "2"], ")"}], "\[CenterDot]", 
         SubscriptBox["k", "2"]}]}], " ", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "2"], ",", 
         SubscriptBox["k", "2"]}], ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["e", "1"], ",", 
      SubscriptBox["e", "2"]}], ")"}], " "}], TraditionalForm]]],
 "acts as the identity element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".  \n\nNext we note that the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", "k"}], ")"}], TraditionalForm]]],
 " has an inverse ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["h", 
       RowBox[{"-", "1"}]], ",", 
      RowBox[{
       SubscriptBox["\[Phi]", 
        SuperscriptBox["h", 
         RowBox[{"-", "1"}]]], "(", 
       SuperscriptBox["k", 
        RowBox[{"-", "1"}]], ")"}]}], " ", ")"}], ","}], TraditionalForm]]],
 " since\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"   ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["h", 
          RowBox[{"-", "1"}]], ",", 
         RowBox[{
          SubscriptBox["\[Phi]", 
           SuperscriptBox["h", 
            RowBox[{"-", "1"}]]], "(", 
          SuperscriptBox["k", 
           RowBox[{"-", "1"}]], ")"}]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"h", ",", "k"}], ")"}]}], " ", "=", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["h", 
          RowBox[{"-", "1"}]], "\[CenterDot]", "h"}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Phi]", "h"], "(", 
          RowBox[{
           SubscriptBox["\[Phi]", 
            SuperscriptBox["h", 
             RowBox[{"-", "1"}]]], "(", 
           SuperscriptBox["k", 
            RowBox[{"-", "1"}]], ")"}], ")"}], "\[CenterDot]", "k"}]}], 
       ")"}]}], " ", "\n", 
     "                                                          ", 
     RowBox[{
      RowBox[{"=", "  ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["e", 
           RowBox[{"1", ","}]], 
          RowBox[{
           RowBox[{
            SubscriptBox["\[Phi]", 
             SubscriptBox["e", "1"]], "(", 
            SuperscriptBox["k", 
             RowBox[{"-", "1"}]], ")"}], "\[CenterDot]", "k"}]}], ")"}], " ", 
        "=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["e", 
            RowBox[{"1", ","}]], 
           RowBox[{
            SuperscriptBox["k", 
             RowBox[{"-", "1"}]], "\[CenterDot]", "k"}]}], ")"}], " ", "=", 
         " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["e", "1"], ",", 
           SubscriptBox["e", "2"]}], ")"}]}]}]}], ","}]}]}], 
   TraditionalForm]]],
 "\n    \nand                                                      \n\n      \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"h", ",", "k"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["h", 
          RowBox[{"-", "1"}]], ",", 
         RowBox[{
          SubscriptBox["\[Phi]", 
           SuperscriptBox["h", 
            RowBox[{"-", "1"}]]], "(", 
          SuperscriptBox["k", 
           RowBox[{"-", "1"}]], ")"}]}], " ", ")"}]}], " ", "=", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"h", "\[CenterDot]", 
         SuperscriptBox["h", 
          RowBox[{"-", "1"}]]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Phi]", 
           SuperscriptBox["h", 
            RowBox[{"-", "1"}]]], "(", "k", ")"}], "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["\[Phi]", 
            SuperscriptBox["h", 
             RowBox[{"-", "1"}]]], "(", 
           SuperscriptBox["k", 
            RowBox[{"-", "1"}]], ")"}], ")"}]}]}], ")"}]}], " ", "\n", 
     "                                                         ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["e", 
          RowBox[{"1", ","}]], 
         SubscriptBox["\[Phi]", 
          SuperscriptBox["h", 
           RowBox[{"-", "1"}]]], " ", 
         RowBox[{"(", 
          RowBox[{"k", "\[CenterDot]", 
           SuperscriptBox["k", 
            RowBox[{"-", "1"}]]}], ")"}]}], ")"}], " ", "=", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["e", "1"], ",", 
          RowBox[{
           SubscriptBox["\[Phi]", 
            SuperscriptBox["h", 
             RowBox[{"-", "1"}]]], "(", 
           SubscriptBox["e", "2"], ")"}]}], ")"}], "=", " ", 
        RowBox[{
         RowBox[{"(", " ", 
          RowBox[{
           SubscriptBox["e", "1"], ",", 
           SubscriptBox["e", "2"]}], ")"}], "."}]}]}]}]}]}], 
   TraditionalForm]]],
 "   \n                                                         \nThe final \
thing we need to check is that the multiplication on ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is associative. Note that\n  \n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "1"], ",", 
         SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["h", "2"], ",", 
           SubscriptBox["k", "2"]}], ")"}], "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["h", "3"], ",", 
           SubscriptBox["k", "3"]}], ")"}]}], "]"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "1"], ",", 
         SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "2"], "\[CenterDot]", 
          SubscriptBox["h", "3"]}], ",", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Phi]", 
            SubscriptBox["h", "3"]], "(", 
           SubscriptBox["k", "2"], ")"}], "\[CenterDot]", 
          SubscriptBox["k", "3"]}]}], " ", ")"}]}]}], "\n", 
     "                                                        \t", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"], "\[CenterDot]", 
          SubscriptBox["h", "3"]}], ",", " ", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Phi]", 
            RowBox[{
             SubscriptBox["h", "2"], "\[CenterDot]", 
             SubscriptBox["h", "3"]}]], "(", 
           SubscriptBox["k", "1"], ")"}], "\[CenterDot]", 
          RowBox[{
           SubscriptBox["\[Phi]", 
            SubscriptBox["h", "3"]], "(", 
           SubscriptBox["k", "2"], ")"}], "\[CenterDot]", 
          SubscriptBox["k", "3"]}]}], " ", ")"}], " ", "."}]}]}]}], 
   TraditionalForm]]],
 "\nwhile\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["h", "1"], ",", 
           SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["h", "2"], ",", 
           SubscriptBox["k", "2"]}], ")"}]}], "]"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "3"], ",", 
         SubscriptBox["k", "3"]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"]}], ",", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Phi]", 
            SubscriptBox["h", "2"]], "(", 
           SubscriptBox["k", "1"], ")"}], "\[CenterDot]", 
          SubscriptBox["k", "2"]}]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "3"], ",", 
         SubscriptBox["k", "3"]}], ")"}]}]}], "\n", 
     "\t                                                        ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"], "\[CenterDot]", 
          SubscriptBox["h", "3"]}], ",", " ", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Phi]", 
            SubscriptBox["h", "3"]], "(", 
           RowBox[{
            RowBox[{
             SubscriptBox["\[Phi]", 
              SubscriptBox["h", "2"]], "(", 
             SubscriptBox["k", "1"], ")"}], "\[CenterDot]", 
            SubscriptBox["k", "2"]}], ")"}], "\[CenterDot]", 
          SubscriptBox["k", "3"]}]}], ")"}], " ", "=", "\[IndentingNewLine]", 
       
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["h", "1"], "\[CenterDot]", 
           SubscriptBox["h", "2"], "\[CenterDot]", 
           SubscriptBox["h", "3"]}], ",", " ", 
          RowBox[{
           RowBox[{
            SubscriptBox["\[Phi]", 
             SubscriptBox["h", "3"]], "(", 
            RowBox[{
             SubscriptBox["\[Phi]", 
              SubscriptBox["h", "2"]], "(", 
             SubscriptBox["k", "1"], ")"}], ")"}], "\[CenterDot]", 
           RowBox[{
            SubscriptBox["\[Phi]", 
             SubscriptBox["h", "3"]], "(", 
            SubscriptBox["k", "2"], ")"}], "\[CenterDot]", 
           SubscriptBox["k", "3"]}]}], ")"}], "=", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["h", "1"], "\[CenterDot]", 
            SubscriptBox["h", "2"], "\[CenterDot]", 
            SubscriptBox["h", "3"]}], ",", "  ", 
           RowBox[{
            RowBox[{
             SubscriptBox["\[Phi]", 
              RowBox[{
               SubscriptBox["h", "2"], "\[CenterDot]", 
               SubscriptBox["h", "3"]}]], "(", 
             SubscriptBox["k", "1"], ")"}], "\[CenterDot]", 
            RowBox[{
             SubscriptBox["\[Phi]", 
              SubscriptBox["h", "3"]], "(", 
             SubscriptBox["k", "2"], ")"}], "\[CenterDot]", 
            SubscriptBox["k", "3"]}]}], " ", ")"}], " ", ".", " "}]}]}]}]}]}],
    TraditionalForm]]],
 "\n\nHence the multiplication on ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is associative and so ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " forms a group. "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43680910465625*^9, 3.436809466296875*^9}, {
  3.436809556890625*^9, 3.43680964678125*^9}, {3.43680980890625*^9, 
  3.436809911078125*^9}, {3.440937268305125*^9, 3.440937804836375*^9}, {
  3.440937845523875*^9, 3.440938089305125*^9}, {3.4409381199145*^9, 
  3.440938213742625*^9}, {3.4415521680577497`*^9, 3.4415521889483747`*^9}, {
  3.4415522213233747`*^9, 3.4415522269796247`*^9}}]
}, Open  ]],

Cell[TextData[{
 "A semi-product of two groups acts in many ways like the direct product. One \
property that is in common is that there are copies of the two original \
groups within the product. In fact, we have the following:\n\nLEMMA 6.7\nLet \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", 
    RowBox[{"H", " ", 
     SubscriptBox["\[Proportional]", "\[Phi]"], "K"}]}], TraditionalForm]]],
 " be the semi-direct product of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " through the homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 ". Suppose that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "1"], TraditionalForm]]],
 " is the identity element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "2"], TraditionalForm]]],
 " is the identity element of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".  Then \n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["H", "_"], "  ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"h", ",", 
        SubscriptBox["e", "2"]}], ")"}], " ", "|", " ", 
      RowBox[{"h", " ", "\[Element]", " ", "H"}]}], " ", "}"}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436809984015625*^9, 3.436810030859375*^9}, {
   3.43681009346875*^9, 3.436810095328125*^9}, {3.43681027559375*^9, 
   3.436810282796875*^9}, 3.43684002471875*^9, {3.440938522477*^9, 
   3.440938529211375*^9}, {3.440938666211375*^9, 3.44093869213325*^9}, {
   3.4415523043389997`*^9, 3.4415523068077497`*^9}}],

Cell[TextData[{
 "is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["K", "__"], "  ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"(", " ", 
       RowBox[{
        SubscriptBox["e", "1"], ",", "k"}], ")"}], " ", "|", " ", 
      RowBox[{"k", " ", "\[Element]", " ", "K"}]}], " ", "}"}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436810034109375*^9, 3.436810041265625*^9}, {
  3.440938678523875*^9, 3.440938695117625*^9}}],

Cell[TextData[{
 "is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Furthermore, \n\n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["H", "_"], " ", "\[TildeTilde]", " ", "H"}], 
   TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox[
     RowBox[{"K", " "}], "_"], "\[TildeTilde]", " ", "K"}], 
   TraditionalForm]]],
 ", \n    \nand ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["H", "_"], " ", "\[Intersection]", " ", 
    OverscriptBox[
     RowBox[{"K", " "}], "_"], " "}], TraditionalForm]]],
 " is the identity element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Proof: ", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We will use proposition 2.2 and observe that \n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"h", ",", 
         SubscriptBox["e", "2"]}], ")"}], 
       RowBox[{"-", "1"}]], "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["h", 
          RowBox[{"-", "1"}]], ",", 
         RowBox[{
          SubscriptBox["\[Phi]", 
           SuperscriptBox["h", 
            RowBox[{"-", "1"}]]], "(", 
          SubscriptBox["e", "2"], ")"}]}], ")"}], " ", "=", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["h", 
          RowBox[{"-", "1"}]], ",", 
         SubscriptBox["e", "2"]}], ")"}]}]}], ","}]}], TraditionalForm]]],
 " \nso \n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "1"], ",", 
        SubscriptBox["e", "2"]}], ")"}], "\[CenterDot]", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "2"], ",", 
         SubscriptBox["e", "2"]}], ")"}], 
       RowBox[{"-", "1"}]]}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "1"], ",", 
         SubscriptBox["e", "2"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["h", "2", 
          RowBox[{"-", "1"}]], ",", 
         SubscriptBox["e", "2"]}], ")"}]}], " ", "=", 
      RowBox[{
       RowBox[{"(", " ", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubsuperscriptBox["h", "2", 
           RowBox[{"-", "1"}]]}], ",", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Phi]", 
            SubsuperscriptBox["h", "2", 
             RowBox[{"-", "1"}]]], "(", 
           SubscriptBox["e", "2"], ")"}], "\[CenterDot]", 
          SubscriptBox["e", "2"]}]}], ")"}], " ", "=", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["h", "1"], "\[CenterDot]", 
           SubsuperscriptBox["h", "2", 
            RowBox[{"-", "1"}]]}], ",", 
          SubscriptBox["e", "2"]}], ")"}], "."}]}]}]}]}], TraditionalForm]]],
 "\n\nThus, whenever ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["H", "_"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["H", "_"], TraditionalForm]]],
 ". So ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["H", "_"], TraditionalForm]]],
 " is a subgroup. \n\nAlso, \n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"    ", 
       RowBox[{
        FormBox[
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["e", "1"], ",", "k"}], ")"}], 
          RowBox[{"-", "1"}]],
         TraditionalForm], " ", "=", " ", 
        FormBox[
         RowBox[{
          FormBox[
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["e", "1"], ",", 
             SubscriptBox["\[Phi]", 
              SubscriptBox["e", "1"]]}]}],
           TraditionalForm], 
          RowBox[{"(", 
           SuperscriptBox["k", 
            RowBox[{"-", "1"}]]}]}],
         TraditionalForm]}], ")"}], ")"}], " ", "=", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["e", "1"], ",", 
       SuperscriptBox["k", 
        RowBox[{"-", "1"}]]}], ")"}]}], ","}], TraditionalForm]]],
 "\nso \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["e", "1"], ",", 
        SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["e", "1"], ",", 
         SubscriptBox["k", "2"]}], ")"}], 
       RowBox[{"-", "1"}]]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["e", "1"], ",", 
         SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["e", "1"], ",", 
         SubsuperscriptBox["k", "2", 
          RowBox[{"-", "1"}]]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["e", "1"], ",", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Phi]", 
            SubscriptBox["e", "1"]], "(", 
           SubscriptBox["k", "1"], ")"}], "\[CenterDot]", 
          SubsuperscriptBox["k", "2", 
           RowBox[{"-", "1"}]]}]}], " ", ")"}], "=", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["e", "1"], ",", 
          RowBox[{
           SubscriptBox["k", "1"], "\[CenterDot]", 
           SubsuperscriptBox["k", "2", 
            RowBox[{"-", "1"}]]}]}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["K", "_"], TraditionalForm]]],
 " whenever ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["K", "_"], TraditionalForm]]],
 ". So ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["K", "_"], TraditionalForm]]],
 " is also a subgroup by proposition 2.2. To show that this group is also a \
normal subgroup we look at\n\n     ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"h", ",", 
          SubscriptBox["k", "1"]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["e", "1"], ",", 
          SubscriptBox["k", "2"]}], ")"}]}], "]"}], "\[CenterDot]", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"h", ",", 
         SubscriptBox["k", "1"]}], ")"}], 
       RowBox[{"-", "1"}]]}], "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"h", ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Phi]", 
           SubscriptBox["e", "1"]], "(", 
          SubscriptBox["k", "1"], ")"}], "\[CenterDot]", 
         SubscriptBox["k", "2"]}]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["h", 
         RowBox[{"-", "1"}]], ",", 
        RowBox[{
         SubscriptBox["\[Phi]", 
          SuperscriptBox["h", 
           RowBox[{"-", "1"}]]], "(", 
         SubsuperscriptBox["k", "1", 
          RowBox[{"-", "1"}]], ")"}]}], ")"}]}]}], " "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                                                        ", 
    RowBox[{"=", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["e", 
        RowBox[{"1", ","}]], 
       RowBox[{
        SubscriptBox["k", "1"], "\[CenterDot]", 
        RowBox[{
         SubscriptBox["\[Phi]", 
          SuperscriptBox["h", 
           RowBox[{"-", "1"}]]], "(", 
         RowBox[{
          SubscriptBox["k", "1"], "\[CenterDot]", 
          SubscriptBox["k", "2"]}], ")"}], "\[CenterDot]", 
        RowBox[{
         SubscriptBox["\[Phi]", 
          SuperscriptBox["h", 
           RowBox[{"-", "1"}]]], "(", 
         SubsuperscriptBox["k", "1", 
          RowBox[{"-", "1"}]], ")"}]}]}], ")"}]}]}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t                                                    ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"(", " ", 
       RowBox[{
        SubscriptBox["e", "1"], "\[CenterDot]", 
        RowBox[{
         SubscriptBox["\[Phi]", 
          SuperscriptBox["h", 
           RowBox[{"-", "1"}]]], "(", 
         RowBox[{
          SubscriptBox["k", "1"], "\[CenterDot]", 
          SubscriptBox["k", "2"], "\[CenterDot]", 
          SubsuperscriptBox["k", "1", 
           RowBox[{"-", "1"}]]}], ")"}]}], ")"}], "."}]}]}], 
   TraditionalForm]}]],
 "\n\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "\[CenterDot]", "k", "\[CenterDot]", 
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    FormBox[
     OverscriptBox["K", "_"],
     TraditionalForm]}], TraditionalForm]]],
 " whenever ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    FormBox[
     OverscriptBox["K", "_"],
     TraditionalForm]}], TraditionalForm]]],
 ", by proposition 3.4 ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    FormBox[
     OverscriptBox["K", "_"],
     TraditionalForm]}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". \n\nFinally, the two mappings\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Florin]", "1"], "(", "h", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"h", ",", 
        SubscriptBox["e", "2"]}], ")"}], "   ", "and", "   ", 
      RowBox[{
       SubscriptBox["\[Florin]", "2"], "(", "k", ")"}]}], " ", "=", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["e", "1"], ",", "k"}], ")"}]}]}], TraditionalForm]]],
 "             \n\nare isomorphisms from ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " onto ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["H", "_"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " onto ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["K", "_"], TraditionalForm]]],
 " respectively, as seen by the above computations. Also, it is clear that \
the intersections of the two groups gives just ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["e", "1"], ",", " ", 
      SubscriptBox["e", "2"]}], ")"}], "}"}], TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4368101196875*^9, 3.4368101744375*^9}, {
  3.436839177390625*^9, 3.436839364640625*^9}, {3.436839449703125*^9, 
  3.436839482890625*^9}, {3.436839514609375*^9, 3.436839643328125*^9}, {
  3.436839680703125*^9, 3.43683970465625*^9}, {3.440938722477*^9, 
  3.440938883242625*^9}, {3.4409389155395*^9, 3.440939095492625*^9}, {
  3.440939126930125*^9, 3.44093913025825*^9}, {3.44093924807075*^9, 
  3.44093946200825*^9}}]
}, Open  ]],

Cell[TextData[{
 "Since the semi-direct product contains copies of the two smaller groups \
within itself, the natural question is whether an arbitrary group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " can be expressed as a semi-direct product of two of its subgroups. The \
conditions for which this will happen is set forth the following theorem.\n\n\
THEOREM 6.3:  THE SEMI-DIRECT PRODUCT THEOREM\nSuppose that a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has two subgroups ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " whose intersection is the identity element. If ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is not a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "N"}], TraditionalForm]]],
 ", then there exists a non-trivial homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "N", ")"}], TraditionalForm]]],
 " such that  \n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "N"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", "H", " ", 
    SubscriptBox["\[Proportional]", "\[Phi]"], "N"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43683977575*^9, 3.436839810375*^9}, {
  3.436840034484375*^9, 3.436840034609375*^9}, {3.43684023946875*^9, 
  3.43684023946875*^9}, {3.440939480836375*^9, 3.440939500711375*^9}}],

Cell[CellGroupData[{

Cell["Proof: ", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Note that since ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a normal subgroup we have by lemma 4.5 that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "N"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". We next want to define the homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 ". For each ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", we define  \n                             ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Phi]", "h"], "(", "n", ")"}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["h", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "n", "\[CenterDot]", "h"}]}]}], 
   TraditionalForm]]],
 "\nfor all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[Element]", "N"}], TraditionalForm]]],
 ".  We first need to show that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "h"], TraditionalForm]]],
 " is an automorphism on ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " for each ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and then we need to show that ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " itself is a non-trivial homomorphism. Note that \n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Phi]", "h"], "(", 
      RowBox[{
       SubscriptBox["n", "1"], "\[CenterDot]", 
       SubscriptBox["n", "2"]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["h", 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       SubscriptBox["n", "1"], "\[CenterDot]", 
       SubscriptBox["n", "2"], "\[CenterDot]", "h"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["h", 
           RowBox[{"-", "1"}]], "\[CenterDot]", 
          SubscriptBox["n", "1"], "\[CenterDot]", "h"}], ")"}], 
        "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["h", 
           RowBox[{"-", "1"}]], "\[CenterDot]", 
          SubscriptBox["n", "2"], "\[CenterDot]", "h"}], ")"}]}], " ", "=", 
       " ", 
       RowBox[{
        RowBox[{
         RowBox[{
          SubscriptBox["\[Phi]", "h"], "(", 
          SubscriptBox["n", "1"], ")"}], "\[CenterDot]", 
         SubscriptBox["\[Phi]", "h"]}], " ", 
        RowBox[{
         RowBox[{"(", 
          SubscriptBox["n", "2"], ")"}], "."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "h"], TraditionalForm]]],
 " is a homomorphism from ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". Since \n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", " ", "\[Element]", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubsuperscriptBox["\[Phi]", "h", 
         RowBox[{"-", "1"}]], "(", "n", ")"}], "    ", 
       "\[DoubleLongLeftRightArrow]", "     ", 
       SuperscriptBox["h", 
        RowBox[{"-", "1"}]]}], "\[CenterDot]", "y", "\[CenterDot]", "h"}]}], 
    "=", 
    RowBox[{
     RowBox[{"n", "      ", "\[DoubleLongLeftRightArrow]", "      ", "y"}], 
     "=", 
     RowBox[{"h", "\[CenterDot]", "n", "\[CenterDot]", 
      SuperscriptBox["h", 
       RowBox[{"-", "1"}]]}]}]}], TraditionalForm]]],
 "\n         \nwe see that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "h"], TraditionalForm]]],
 " is a one-to-one and onto function.  Thus, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "h"], TraditionalForm]]],
 " is an automorphism of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". \n\nNext, we need to see that ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " itself is a homomorphism from ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "N", ")"}], TraditionalForm]]],
 ". Note that\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"   ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Phi]", 
          SubscriptBox["h", "1"]], "\[CenterDot]", 
         SubscriptBox["\[Phi]", 
          SubscriptBox["h", "2"]]}], ")"}], 
       RowBox[{"(", "n", ")"}]}], " ", "=", "  ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Phi]", 
         SubscriptBox["h", "2"]], "(", 
        RowBox[{
         SubscriptBox["\[Phi]", 
          SubscriptBox["h", "1"]], "(", "n", ")"}], ")"}], " ", "=", " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["\[Phi]", 
          SubscriptBox["h", "2"]], "(", 
         RowBox[{
          SubsuperscriptBox["h", "1", 
           RowBox[{"-", "1"}]], "\[CenterDot]", "n", "\[CenterDot]", 
          SubscriptBox["h", "1"]}], ")"}], " ", "=", " ", 
        RowBox[{
         SubsuperscriptBox["h", "2", 
          RowBox[{"-", "1"}]], "\[CenterDot]", 
         SubsuperscriptBox["h", "1", 
          RowBox[{"-", "1"}]], "\[CenterDot]", "n", "\[CenterDot]", 
         SubscriptBox["h", "1"], "\[CenterDot]", 
         SubscriptBox["h", "2"]}]}]}]}], "\n", 
     "                               ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["h", "1"], "\[CenterDot]", 
           SubscriptBox["h", "2"]}], ")"}], 
         RowBox[{"-", "1"}]], "\[CenterDot]", "n", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"]}], ")"}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["\[Phi]", 
          RowBox[{
           SubscriptBox["h", "1"], "\[CenterDot]", 
           SubscriptBox["h", "2"]}]], "(", "n", ")"}], ".", "  "}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", 
      SubscriptBox["h", "1"]], "\[CenterDot]", 
     SubscriptBox["\[Phi]", 
      SubscriptBox["h", "2"]]}], " ", "=", " ", 
    SubscriptBox["\[Phi]", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "1"], "\[CenterDot]", 
        SubscriptBox["h", "2"]}], ")"}], " "}]]}], TraditionalForm]]],
 "and we see that \[Phi] is a homomorphism. In fact, the homomorphism must be \
non-trivial, because if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "h"], "(", "n", ")"}], " ", "=", " ", "n"}], 
   TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", then since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["\[Phi]", "h"], "(", "n", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["h", 
        RowBox[{"-", "1"}]], "\[CenterDot]", "n", "\[CenterDot]", "h"}], " ", 
      "=", " ", "n"}]}], ","}], TraditionalForm]]],
 " we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"n", "\[CenterDot]", "h"}], " ", "=", " ", 
     RowBox[{"h", "\[CenterDot]", "n"}]}]}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". This would indicate that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a \nnormal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "N"}], TraditionalForm]]],
 ", which contradicts our original assumption. Thus,  ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is a non-trivial homomorphism. \n\nWe can now proceed in a similar way \
that we proved the direct product theorem (6.1). However, it will be easier \
to show that every element in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "N"}], TraditionalForm]]],
 " can be ",
 StyleBox["uniquely",
  FontSlant->"Italic"],
 " written in the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "n"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "\[Epsilon]", " ", "H"}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "\[Epsilon]", " ", "N"}], TraditionalForm]]],
 ".\nSuppose that we have\n                                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["h", "1"], "\[CenterDot]", 
     SubscriptBox["n", "1"]}], "=", " ", 
    RowBox[{
     SubscriptBox["h", "2"], "\[CenterDot]", 
     SubscriptBox["n", "2"]}]}], TraditionalForm]]],
 ".\n\nThen ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubsuperscriptBox["h", "2", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SubscriptBox["h", "1"]}], "=", " ", 
    RowBox[{
     SubscriptBox["n", "2"], " ", "\[CenterDot]", 
     SubsuperscriptBox["n", "1", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ".  Since this element is in both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ", which has just the identity element in the intersection, we must have \n\n\
                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubsuperscriptBox["h", "2", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SubscriptBox["h", "1"]}], "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["n", "2"], " ", "\[CenterDot]", 
      SubsuperscriptBox["n", "1", 
       RowBox[{"-", "1"}]]}], " ", "=", " ", 
     RowBox[{"e", "."}]}]}], TraditionalForm]]],
 "\n \nTherefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["h", "1"], "=", 
    SubscriptBox["h", "2"]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], "=", 
    SubscriptBox["n", "2"]}], TraditionalForm]]],
 ".  Thus, we have shown that every element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "N"}], TraditionalForm]]],
 " is written uniquely as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "n"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ".\n\nWe now want to create a mapping\n\n                                    \
      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", ":", " ", 
    RowBox[{
     RowBox[{"H", "\[CenterDot]", "N"}], " ", "\[RightArrow]", " ", 
     RowBox[{"H", " ", 
      SubscriptBox["\[Proportional]", "\[Phi]"], "N"}]}]}], 
   TraditionalForm]]],
 "\ndefined by\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Florin]", "(", "v", ")"}], " ", "=", " ", 
     RowBox[{"(", 
      RowBox[{"h", ",", "n"}], ")"}]}], ","}], TraditionalForm]]],
 "\n                 \nwhere ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are the unique elements such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "\[Epsilon]", " ", "H"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "\[Epsilon]", " ", "N"}], TraditionalForm]]],
 ",  and   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "=", 
    RowBox[{"h", "\[CenterDot]", "n"}]}], TraditionalForm]]],
 ".  The function ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is one-to-one since the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"h", ",", "n"}], ")"}], TraditionalForm]]],
 " can only come from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "n"}], TraditionalForm]]],
 ".  Also, the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "n"}], TraditionalForm]]],
 " maps to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"h", ",", "n"}], ")"}], ","}], TraditionalForm]]],
 " so ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is onto. \n\nThe final step is to show that ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an homomorphism. Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", " ", "=", " ", 
    RowBox[{
     SubscriptBox["h", "1"], "\[CenterDot]", 
     SubscriptBox["n", "1"]}]}], TraditionalForm]]],
 " , and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"w", " ", "=", " ", 
    RowBox[{
     SubscriptBox["h", "2"], "\[CenterDot]", 
     SubscriptBox["n", "2"]}]}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"v", "\[CenterDot]", "w"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["h", "1"], "\[CenterDot]", 
      SubscriptBox["n", "1"], "\[CenterDot]", 
      SubscriptBox["h", "2"], "\[CenterDot]", 
      SubscriptBox["n", "2"]}], " ", "=", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "1"], "\[CenterDot]", 
        SubscriptBox["h", "2"]}], ")"}], "\[CenterDot]", " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["h", "2", 
         RowBox[{"-", "1"}]], "\[CenterDot]", 
        SubscriptBox["n", "1"], "\[CenterDot]", 
        SubscriptBox["h", "2"], "\[CenterDot]", 
        SubscriptBox["n", "2"]}], ")"}]}]}]}], TraditionalForm]]],
 " . \n\nSince ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a normal subgroup, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["h", "2", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SubscriptBox["n", "1"], "\[CenterDot]", 
    SubscriptBox["h", "2"]}], TraditionalForm]]],
 "  is in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["h", "2", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SubscriptBox["n", "1"], "\[CenterDot]", 
    SubscriptBox["h", "2"], "\[CenterDot]", 
    SubscriptBox["n", "2"]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " while ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["h", "1"], "\[CenterDot]", 
    SubscriptBox["h", "2"]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". Thus,     \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{"v", "\[CenterDot]", "w"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"f", "(", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"]}], ")"}], "\[CenterDot]", " ", 
        RowBox[{"(", 
         RowBox[{
          SubsuperscriptBox["h", "2", 
           RowBox[{"-", "1"}]], "\[CenterDot]", 
          SubscriptBox["n", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"], "\[CenterDot]", 
          SubscriptBox["n", "2"]}], ")"}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", " ", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"]}], ",", " ", 
         RowBox[{
          SubsuperscriptBox["h", "2", 
           RowBox[{"-", "1"}]], "\[CenterDot]", 
          SubscriptBox["n", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"], "\[CenterDot]", 
          SubscriptBox["n", "2"]}]}], ")"}], "  ", "=", " ", 
       RowBox[{
        RowBox[{"(", " ", 
         RowBox[{
          RowBox[{
           SubscriptBox["h", "1"], "\[CenterDot]", 
           SubscriptBox["h", "2"]}], ",", " ", 
          RowBox[{
           RowBox[{
            SubscriptBox["\[Phi]", 
             SubscriptBox["h", "2"]], "(", 
            SubscriptBox["n", "1"], ")"}], "\[CenterDot]", 
           SubscriptBox["n", "2"]}]}], ")"}], " ", "=", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["h", "1"], ",", 
            SubscriptBox["n", "1"]}], ")"}], "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["h", "2"], ",", 
            SubscriptBox["n", "2"]}], ")"}]}], " ", "=", " ", 
         RowBox[{
          RowBox[{"f", "(", "v", ")"}], "\[CenterDot]", 
          RowBox[{
           RowBox[{"f", "(", "w", ")"}], "."}]}]}]}]}]}]}]}], 
   TraditionalForm]]],
 "           \n\n So ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an isomorphism, and we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "N"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", "H", "  ", 
    SubscriptBox["\[Proportional]", "\[Phi]"], "N"}], TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4368400606875*^9, 3.436840292703125*^9}, {
  3.4368403666875*^9, 3.43684078815625*^9}, {3.436841489984375*^9, 
  3.436841489984375*^9}, {3.43684155140625*^9, 3.4368415724375*^9}, {
  3.440939521898875*^9, 3.440939529961375*^9}, {3.440939610242625*^9, 
  3.440940087930125*^9}, {3.440940172617625*^9, 3.44094039113325*^9}, {
  3.4415524491358747`*^9, 3.4415524626202497`*^9}, {3.4415525141202497`*^9, 
  3.4415525169171247`*^9}, {3.4415526258546247`*^9, 3.4415526258546247`*^9}}]
}, Open  ]],

Cell[TextData[{
 "Enough of the technical proofs, let us learn how to define a semi-direct \
product with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". We will use this result to define the semi-direct product of two groups \
in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". We first must define the two groups in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using the same identity as we did for the direct product, using different \
letters for the generators. We must also know the homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "N", ")"}], TraditionalForm]]],
 ".\n\nWe will want to express every element of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "n"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". Thus, we need to tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " how to deal with an expression of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "n"}], TraditionalForm]]],
 ".\n\nFrom the definition, we see that \n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"h", ",", 
       SubscriptBox["e", "2"]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["e", "1"], ",", "n"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox[
        RowBox[{
         RowBox[{"h", "\[CenterDot]", 
          SubscriptBox["e", "1"]}], ",", 
         SubscriptBox["\[Phi]", 
          SubscriptBox["e", "1"]]}],
        TraditionalForm], 
       RowBox[{
        RowBox[{"(", 
         SubscriptBox["e", "2"], ")"}], "\[CenterDot]", "n"}]}], ")"}], " ", 
     "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"h", ",", " ", "n"}], ")"}], "."}]}]}], TraditionalForm]]],
 "\n \n So for each generator ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and each generator ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ", we can calculate how ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 " should be defined by evaluating\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           FormBox[
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["e", "1"], ",", "b"}], ")"}], "\[CenterDot]", 
             "("}],
            TraditionalForm], "a"}], ",", 
          SubscriptBox["e", "2"]}], ")"}], "=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["e", "1"], "\[CenterDot]", "a"}], ",", 
           RowBox[{
            FormBox[
             RowBox[{
              FormBox[
               SubscriptBox["\[Phi]", "a"],
               TraditionalForm], "("}],
             TraditionalForm], "b"}]}], ")"}], "\[CenterDot]", 
         SubscriptBox["e", "2"]}]}], ")"}], "=", 
      RowBox[{"(", 
       RowBox[{"a", ",", 
        RowBox[{
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["\[Phi]", "b"],
            TraditionalForm], "("}],
          TraditionalForm], "b"}]}], ")"}]}], ")"}], "."}], 
   TraditionalForm]]],
 "\n\nSo for each generator ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and each generator ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ", we make a definition\n\nDefine[b . a, a .  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "a"], "(", "b", ")"}], Cell[""]}], 
   TraditionalForm]]],
 "]\n\nwhere we replace the expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Phi]", "a"], "(", "b", ")"}], TraditionalForm]]],
 " with its element of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ".   \n\nFor example, suppose we want to find a semi-direct product of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ". We first must define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 " into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using the same identity but different generators."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.4028343701053753`*^9, {3.436840801046875*^9, 3.4368409329375*^9}, {
   3.43684096340625*^9, 3.43684106634375*^9}, {3.43684110859375*^9, 
   3.43684110859375*^9}, {3.436841169015625*^9, 3.4368412770625*^9}, {
   3.44094041413325*^9, 3.440940638977*^9}, {3.440940721742625*^9, 
   3.440940769930125*^9}, {3.4415526897452497`*^9, 3.4415527817139997`*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2,e]
Define[1/a,a]
Z2 = Group[{a}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436841304359375*^9, 3.436841313828125*^9}, {
  3.44094079019575*^9, 3.440940797492625*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a"}], "}"}]], "Output",
 CellChangeTimes->{3.44094081488325*^9, 3.442673917898875*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Define[b^5,e]
Define[1/b,b^4]
Z5 = Group[{b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43684131803125*^9, 3.436841324921875*^9}, {
  3.440940800586375*^9, 3.44094081075825*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4409408169145*^9, 3.442673919336375*^9}]
}, Open  ]],

Cell[TextData[{
 "Next, we must find a non-trivial homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["Z", "5"], ")"}], TraditionalForm]]],
 ". But\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", 
     SubscriptBox["Z", "5"], ")"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubsuperscriptBox["Z", "5", "*"], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "4"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since the element ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is of order 2, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"\[Phi]", "(", "a", ")"}], "2"], " ", "=", " ", 
     RowBox[{
      RowBox[{"\[Phi]", "(", 
       SuperscriptBox["a", "2"], ")"}], " ", "=", " ", "e"}]}], ","}], 
   TraditionalForm]]],
 " so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "a", ")"}], TraditionalForm]]],
 " must be of order 2 to keep the homomorphism from being trivial. We need ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "a", ")"}], TraditionalForm]]],
 " to map to the only element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["Z", "5"], ")"}], TraditionalForm]]],
 " which is of order 2.\n\nWhich element is this? With a moment's reflection, \
we can see that\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", ":", " ", 
    RowBox[{"N", " ", "\[RightArrow]", " ", "N"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "n", ")"}], " ", "=", " ", 
    SuperscriptBox["n", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4368413649375*^9, 3.43684136959375*^9}}],

Cell[TextData[{
 "will always be an automorphism of whenever ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is an abelian group. As long as ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " has an element which is not its own inverse, this automorphism will be of \
order 2. Therefore, to find a semi-direct product of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ", we must let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "a"], TraditionalForm]]],
 " be the automorphism ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "n", ")"}], " ", "=", " ", 
    SuperscriptBox["n", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "a"], "(", "b", ")"}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["b", 
      RowBox[{"-", "1"}]], "=", " ", 
     SuperscriptBox["b", "4"]}]}], TraditionalForm]]],
 ". So the definition"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436841376515625*^9, 3.436841433546875*^9}, {
  3.440940843555125*^9, 3.44094084788325*^9}}],

Cell["Define[b.a, a.b.b.b.b]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436841437375*^9, 3.4368414395*^9}, {3.440940856852*^9, 
  3.44094085925825*^9}},
 AspectRatioFixed->True],

Cell["\<\
completes the definition of the semi-direct product. We now have the group \
along with the multiplication table:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
G = Group[{a, b}]
MultTable[G]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.440940870227*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{{3.440940864352*^9, 3.440940870836375*^9}, 
   3.44267392557075*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {0.5, 0.6, 0.8}, {0, 1, 1}, {1, 0, 1}, {0, 1, 
      0}, {1, 0, 0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {0.7, 0.8,
       0.7}, {1, 1, 0}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {1, 0, 0}, {1, 1, 
      0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.5, 0.6, 
      0.8}, {1, 0.6, 0}, {1, 0, 1}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {0, 1,
       0}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {1, 0, 1}, {0.8, 0.6, 0.5}, {1, 0, 
      0}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 1}}, {{1, 0.6, 0}, {1, 0.6, 0}, {
      0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {0.7, 0.8, 0.7}, {1, 1, 0}, {0.5, 0.6, 
      0.8}, {0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {1, 0, 0}}, {{0.6, 0.4, 0.8}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}}, {{0, 
      1, 0}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0.8, 
      0.6, 0.5}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 
      0.8}}, {{1, 0, 0}, {1, 0, 0}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {1, 1, 
      0}, {0.7, 0.8, 0.7}, {0, 1, 1}, {0.5, 0.6, 0.8}, {0, 1, 0}, {1, 0, 1}, {
      1, 0.6, 0}}, {{0, 1, 1}, {0, 1, 1}, {0.5, 0.6, 0.8}, {0, 1, 0}, {1, 0, 
      1}, {1, 0.6, 0}, {1, 0, 0}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {1, 1, 
      0}, {0.7, 0.8, 0.7}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 
      1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.5, 
      0.6, 0.8}, {0.8, 0.6, 0.5}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 
      1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7,
       0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 9.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 8.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 7.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 6.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 5.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 4.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 3.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 2.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 1.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 0.5}, {1, 0}]}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{{3.440940864352*^9, 3.440940870836375*^9}, 
   3.442673925586375*^9}]
}, Open  ]],

Cell[TextData[{
 "This is the semi-direct product ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Proportional]", " ", 
    SubscriptBox["Z", "5"]}], TraditionalForm]]],
 ", a non-abelian group of order 10.  Note that when there is only one \
possible semi-direct product between two groups, we can leave out the \[Phi] \
in the notation.  \n\nEXPERIMENT:\nFind a semi-direct product ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Proportional]", " ", 
    RowBox[{
     SubscriptBox["Z", "6"], "."}]}], TraditionalForm]]],
 "\n\nWe can easily generalize these examples to form an important class of \
groups.\n\nDEFINITION 6.7:\nLet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "2"}], TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " be the homomorphism from ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "=", " ", 
    RowBox[{"{", 
     RowBox[{"e", ",", "a"}], "}"}]}], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["Z", "n"], ")"}], TraditionalForm]]],
 " given by   \n                         \n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["\[Phi]", "e"], "(", "k", ")"}], " ", "=", " ", "k"}], ",",
     "   ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Phi]", "a"], "(", "k", ")"}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["k", 
       RowBox[{"-", "1"}]], "."}]}]}], TraditionalForm]]],
 "\n\nThen the semi-direct product ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], 
    SubscriptBox["\[Proportional]", "\[Phi]"], 
    SubscriptBox["Z", "n"]}], TraditionalForm]]],
 " is called the ",
 StyleBox["dihedral group of order",
  FontSlant->"Italic"],
 " 2",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". It is denoted ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "n"], TraditionalForm]]],
 ", and is a non-abelian group of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "n"}], TraditionalForm]]],
 ".       \n\nFollowing this example, we see that the dihedral group can be \
defined in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by the commands"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43684159096875*^9, 3.436841609375*^9}, {
  3.4368416705625*^9, 3.436841858890625*^9}, {3.4409408914145*^9, 
  3.44094091882075*^9}, {3.440944458336375*^9, 3.44094448675825*^9}}],

Cell[TextData[{
 "InitGroup[e];\nDefine[a^",
 Cell[BoxData[
  FormBox["2", TraditionalForm]]],
 ", e]\nDefine[b^",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", "n"}], TraditionalForm]]],
 ", e]\nDefine[1/a, a]\[LineSeparator]Define[1/b, b^(",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ")]\nDefine[b.a, a.(1/b)]\nDn = Group[{a, b}]"
}], "Text",
 CellChangeTimes->{{3.422651080296875*^9, 3.4226511513125*^9}, {
  3.436841954953125*^9, 3.436842025296875*^9}, {3.440944513180125*^9, 
  3.440944553273875*^9}}],

Cell[TextData[{
 "Note that ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " must be defined before executing these commands. When ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "3"}], TraditionalForm]]],
 ", we get a non-abelian group of order 6, so we must have that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "3"], TraditionalForm]]],
 " is isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ". We also have already seen ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "4"], TraditionalForm]]],
 ", since this was one of the 5 groups of order 8 that we found in chapter 4. \
\n\nIt should be noted that the semi-direct product may greatly depend on the \
choice of the homomorphism \[Phi]. For example, consider finding the \
semi-direct products of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ". The dihedral group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "8"], TraditionalForm]]],
 " is certainly one such product, but there could be others. The homomorphism \
\[Phi] would have to map the element ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " of order 2 in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 " to an element of order 2 in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["Z", "8"], ")"}], TraditionalForm]]],
 ". But since    \n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", 
     SubscriptBox["Z", "8"], ")"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubsuperscriptBox["Z", "8", "*"], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"]}]}], TraditionalForm]]],
 "\n          \nhas 3 elements of order 2, we have 3 choices of what ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "a"], TraditionalForm]]],
 " could be. Do these produce different groups?      \n\nTo explore this \
possibility, let us first load in the dihedral group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "8"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.440944595398875*^9, 3.44094460119575*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2,e]; Define[b^8,e]
Define[1/a,a]; Define[1/b,b^7]
Define[b.a,a.(b^7)]
D8 = Group[{a,b}]
MultTable[D8]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436842088171875*^9, 3.436842110328125*^9}, {
  3.4368421558125*^9, 3.436842172171875*^9}, {3.44094465038325*^9, 
  3.440944672086375*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`b, $CellContext`b], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`b, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[
         Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[
         Dot[$CellContext`b, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[
         Dot[
          
          Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{
  3.436842123125*^9, {3.4368421689375*^9, 3.436842173765625*^9}, 
   3.4409446761645*^9, 3.44267393300825*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0.6, 1}, {1, 0.6, 1}, {0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {1, 
      0, 0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {0.7, 0.8, 0.7}, {
      0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {0.6, 0.6, 0.2}, {0.8, 0.6, 0.2}, {0.2,
       0.8, 0.7}, {1, 0.4, 0.4}, {1, 1, 0}}, {{0.2, 0.8, 0.7}, {0.2, 0.8, 
      0.7}, {1, 0, 0}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {0.7, 0.8, 
      0.7}, {0, 1, 0}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {0.8, 0.6, 0.2}, {0.8, 
      0.6, 0.5}, {1, 0.4, 0.4}, {0.8, 0.2, 0.5}, {1, 0.6, 1}, {0.6, 0.6, 
      0.2}, {1, 0, 1}}, {{1, 0.4, 0.4}, {1, 0.4, 0.4}, {0, 1, 0}, {
      1, 0.6, 1}, {1, 0.6, 0}, {1, 0, 1}, {0.8, 0.6, 0.5}, {1, 0, 0}, {0.8, 
      0.2, 0.5}, {0.6, 0.4, 0.8}, {0.6, 0.6, 0.2}, {0.7, 0.8, 0.7}, {0.2, 0.8,
       0.7}, {0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 0.2}, {0, 1, 1}}, {{0.6, 
      0.6, 0.2}, {0.6, 0.6, 0.2}, {0.6, 0.4, 0.8}, {0.2, 0.8, 0.7}, {0.7, 0.8,
       0.7}, {1, 1, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0.8, 0.6, 0.2}, {0, 1, 
      0}, {1, 0.4, 0.4}, {1, 0.6, 0}, {1, 0.6, 1}, {0.8, 0.6, 0.5}, {1, 0, 
      1}, {0.8, 0.2, 0.5}, {1, 0, 0}}, {{0.8, 0.6, 0.2}, {0.8, 0.6, 0.2}, {
      1, 0.6, 0}, {1, 0.4, 0.4}, {0.8, 0.6, 0.5}, {1, 0.6, 1}, {0.8, 0.2, 
      0.5}, {1, 0, 1}, {0.6, 0.6, 0.2}, {1, 0, 0}, {0.2, 0.8, 0.7}, {0.6, 0.4,
       0.8}, {1, 1, 0}, {0.7, 0.8, 0.7}, {0, 1, 1}, {0.5, 0.6, 0.8}, {0, 1, 
      0}}, {{0.8, 0.2, 0.5}, {0.8, 0.2, 0.5}, {0.7, 0.8, 0.7}, {0.6, 0.6, 
      0.2}, {0.5, 0.6, 0.8}, {0.2, 0.8, 0.7}, {0.8, 0.6, 0.2}, {1, 1, 0}, {
      1, 0.4, 0.4}, {0, 1, 1}, {1, 0.6, 1}, {0, 1, 0}, {1, 0, 1}, {
      1, 0.6, 0}, {1, 0, 0}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}}, {{0.5, 0.6, 
      0.8}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {0.8, 0.6, 0.2}, {0.8, 0.2, 
      0.5}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {1, 0.6, 1}, {0.2, 0.8, 0.7}, {1, 
      0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {
      0.7, 0.8, 0.7}, {1, 0.6, 0}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1,
       1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {0.8, 0.2, 0.5}, {0.5, 0.6, 
      0.8}, {0.6, 0.6, 0.2}, {0.8, 0.6, 0.2}, {0.2, 0.8, 0.7}, {
      1, 0.4, 0.4}, {1, 1, 0}, {1, 0.6, 1}, {0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {
      1, 0, 0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}}, {{
      1, 0.6, 0}, {1, 0.6, 0}, {0.8, 0.6, 0.2}, {0.8, 0.6, 0.5}, {
      1, 0.4, 0.4}, {0.8, 0.2, 0.5}, {1, 0.6, 1}, {0.6, 0.6, 0.2}, {1, 0, 
      1}, {0.2, 0.8, 0.7}, {1, 0, 0}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {
      0.7, 0.8, 0.7}, {0, 1, 0}, {0.5, 0.6, 0.8}}, {{0.6, 0.4, 0.8}, {0.6, 
      0.4, 0.8}, {0.6, 0.6, 0.2}, {0.7, 0.8, 0.7}, {0.2, 0.8, 0.7}, {0.5, 0.6,
       0.8}, {1, 1, 0}, {0.8, 0.6, 0.2}, {0, 1, 1}, {1, 0.4, 0.4}, {0, 1, 
      0}, {1, 0.6, 1}, {1, 0.6, 0}, {1, 0, 1}, {0.8, 0.6, 0.5}, {1, 0, 0}, {
      0.8, 0.2, 0.5}}, {{0, 1, 0}, {0, 1, 0}, {1, 0.4, 0.4}, {1, 0.6, 0}, {
      1, 0.6, 1}, {0.8, 0.6, 0.5}, {1, 0, 1}, {0.8, 0.2, 0.5}, {1, 0, 0}, {
      0.6, 0.6, 0.2}, {0.6, 0.4, 0.8}, {0.2, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 
      1, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0.8, 0.6, 0.2}}, {{1, 0, 0}, {1, 0, 
      0}, {0.2, 0.8, 0.7}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.7, 0.8, 0.7}, {0, 1,
       1}, {0.5, 0.6, 0.8}, {0, 1, 0}, {0.8, 0.6, 0.2}, {1, 0.6, 0}, {
      1, 0.4, 0.4}, {0.8, 0.6, 0.5}, {1, 0.6, 1}, {0.8, 0.2, 0.5}, {1, 0, 
      1}, {0.6, 0.6, 0.2}}, {{0, 1, 1}, {0, 1, 1}, {1, 0.6, 1}, {0, 1, 0}, {1,
       0, 1}, {1, 0.6, 0}, {1, 0, 0}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {0.8, 
      0.2, 0.5}, {0.7, 0.8, 0.7}, {0.6, 0.6, 0.2}, {0.5, 0.6, 0.8}, {0.2, 0.8,
       0.7}, {0.8, 0.6, 0.2}, {1, 1, 0}, {1, 0.4, 0.4}}, {{1, 0, 1}, {1, 0, 
      1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7, 
      0.8, 0.7}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {0.8, 0.6, 
      0.2}, {0.8, 0.2, 0.5}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {1, 0.6, 1}, {
      0.2, 0.8, 0.7}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6,
       0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 
      0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}}, {{1, 1, 1}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 
      0.7}, {1, 0.6, 1}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 15.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 14.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 13.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 12.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 11.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 10.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 9.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 8.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 7.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 6.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 5.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 4.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 3.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 2.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 1.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 0.5}, {1, 0}]}, 
   LineBox[{{1, 0}, {1, 16}, {17, 16}}]},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{
  3.436842123125*^9, {3.4368421689375*^9, 3.436842173765625*^9}, 
   3.4409446761645*^9, 3.442673933117625*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that the main diagonal contains 10 ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 "'s. This means that there are 9 elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "8"], TraditionalForm]]],
 " which are of order 2. This will be useful for later comparisons. \n\nWhat \
are the other automorphisms on ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 "?  Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubsuperscriptBox["Z", "8", "*"],
     TraditionalForm], " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{"1", ",", " ", "3", ",", " ", "5", ",", " ", "7"}], "}"}]}], 
   TraditionalForm]]],
 ", we have the automorphisms   \n  \n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "(", "b", ")"}], " ", "=", " ", "b"}], ",", "    ", 
    RowBox[{
     RowBox[{"\[Phi]", "(", "b", ")"}], " ", "=", " ", 
     SuperscriptBox["b", "7"]}], ",", "    ", 
    RowBox[{
     RowBox[{"\[Phi]", "(", "b", ")"}], " ", "=", " ", 
     SuperscriptBox["b", "5"]}], ",", "\n", 
    RowBox[{
     RowBox[{"and", "      ", 
      RowBox[{"\[Phi]", "(", "b", ")"}]}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["b", "7"], "=", " ", 
      RowBox[{
       SuperscriptBox["b", 
        RowBox[{"-", "1"}]], "."}]}]}]}], TraditionalForm]]],
 "\n\nThe first automorphism is the trivial automorphism, and the last is the \
one we just considered. Suppose we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "a"], "(", "b", ")"}], " ", "=", " ", 
    SuperscriptBox["b", "3"]}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{"a", "\[CenterDot]", 
     SuperscriptBox["b", "3"]}]}], TraditionalForm]]],
 ",  so we can enter this into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with the commands"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42265116671875*^9, 3.4226511701875*^9}, {
  3.440944774273875*^9, 3.440944791930125*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2,e]; Define[b^8,e]
Define[1/a,a]; Define[1/b,b^7]
Define[b.a,a.(b^3)]
G = Group[{a,b}]
MultTable[G]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436842147703125*^9, 3.436842149734375*^9}, {
  3.436842192453125*^9, 3.436842221*^9}, {3.4409447012895*^9, 
  3.4409447194145*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`b, $CellContext`b], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`b, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[
         Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[
         Dot[$CellContext`b, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[
         Dot[
          
          Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.43684222428125*^9, 3.44094472200825*^9, 
  3.442673940523875*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0.6, 1}, {1, 0.6, 1}, {0.8, 0.2, 0.5}, {1, 0, 1}, {0.6, 
      0.6, 0.2}, {1, 0, 0}, {0.2, 0.8, 0.7}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.7,
       0.8, 0.7}, {0, 1, 1}, {0.5, 0.6, 0.8}, {0, 1, 0}, {0.8, 0.6, 0.2}, {
      1, 0.6, 0}, {1, 0.4, 0.4}, {0.8, 0.6, 0.5}}, {{0.2, 0.8, 0.7}, {0.2, 
      0.8, 0.7}, {0.8, 0.6, 0.2}, {1, 1, 0}, {1, 0.4, 0.4}, {0, 1, 1}, {
      1, 0.6, 1}, {0, 1, 0}, {1, 0, 1}, {1, 0.6, 0}, {1, 0, 0}, {0.8, 0.6, 
      0.5}, {0.6, 0.4, 0.8}, {0.8, 0.2, 0.5}, {0.7, 0.8, 0.7}, {0.6, 0.6, 
      0.2}, {0.5, 0.6, 0.8}}, {{1, 0.4, 0.4}, {1, 0.4, 0.4}, {0, 1, 0}, {
      1, 0.6, 1}, {1, 0.6, 0}, {1, 0, 1}, {0.8, 0.6, 0.5}, {1, 0, 0}, {0.8, 
      0.2, 0.5}, {0.6, 0.4, 0.8}, {0.6, 0.6, 0.2}, {0.7, 0.8, 0.7}, {0.2, 0.8,
       0.7}, {0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 0.2}, {0, 1, 1}}, {{0.6, 
      0.6, 0.2}, {0.6, 0.6, 0.2}, {0.6, 0.4, 0.8}, {0.2, 0.8, 0.7}, {0.7, 0.8,
       0.7}, {1, 1, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0.8, 0.6, 0.2}, {0, 1, 
      0}, {1, 0.4, 0.4}, {1, 0.6, 0}, {1, 0.6, 1}, {0.8, 0.6, 0.5}, {1, 0, 
      1}, {0.8, 0.2, 0.5}, {1, 0, 0}}, {{0.8, 0.6, 0.2}, {0.8, 0.6, 0.2}, {
      0.2, 0.8, 0.7}, {1, 0.4, 0.4}, {1, 1, 0}, {1, 0.6, 1}, {0, 1, 1}, {1, 0,
       1}, {0, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0.8, 0.6, 
      0.5}, {0.7, 0.8, 0.7}, {0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {0.6, 0.6, 
      0.2}}, {{0.8, 0.2, 0.5}, {0.8, 0.2, 0.5}, {1, 0.6, 1}, {0.6, 0.6, 
      0.2}, {1, 0, 1}, {0.2, 0.8, 0.7}, {1, 0, 0}, {1, 1, 0}, {0.6, 0.4, 
      0.8}, {0, 1, 1}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.5, 0.6, 0.8}, {
      1, 0.6, 0}, {0.8, 0.6, 0.2}, {0.8, 0.6, 0.5}, {1, 0.4, 0.4}}, {{0.5, 
      0.6, 0.8}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {0.8, 0.6, 0.2}, {0.8, 0.2,
       0.5}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {1, 0.6, 1}, {0.2, 0.8, 0.7}, {1,
       0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {
      0.7, 0.8, 0.7}, {1, 0.6, 0}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1,
       1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {0, 1, 1}, {0.5, 0.6, 0.8}, {
      0, 1, 0}, {0.8, 0.6, 0.2}, {1, 0.6, 0}, {1, 0.4, 0.4}, {0.8, 0.6, 
      0.5}, {1, 0.6, 1}, {0.8, 0.2, 0.5}, {1, 0, 1}, {0.6, 0.6, 0.2}, {1, 0, 
      0}, {0.2, 0.8, 0.7}, {0.6, 0.4, 0.8}, {1, 1, 0}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {1, 0, 0}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {0.8, 0.2, 
      0.5}, {0.7, 0.8, 0.7}, {0.6, 0.6, 0.2}, {0.5, 0.6, 0.8}, {0.2, 0.8, 
      0.7}, {0.8, 0.6, 0.2}, {1, 1, 0}, {1, 0.4, 0.4}, {0, 1, 1}, {
      1, 0.6, 1}, {0, 1, 0}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {
      0.6, 0.6, 0.2}, {0.7, 0.8, 0.7}, {0.2, 0.8, 0.7}, {0.5, 0.6, 0.8}, {1, 
      1, 0}, {0.8, 0.6, 0.2}, {0, 1, 1}, {1, 0.4, 0.4}, {0, 1, 0}, {
      1, 0.6, 1}, {1, 0.6, 0}, {1, 0, 1}, {0.8, 0.6, 0.5}, {1, 0, 0}, {0.8, 
      0.2, 0.5}}, {{0, 1, 0}, {0, 1, 0}, {1, 0.4, 0.4}, {1, 0.6, 0}, {
      1, 0.6, 1}, {0.8, 0.6, 0.5}, {1, 0, 1}, {0.8, 0.2, 0.5}, {1, 0, 0}, {
      0.6, 0.6, 0.2}, {0.6, 0.4, 0.8}, {0.2, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 
      1, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0.8, 0.6, 0.2}}, {{1, 0, 0}, {1, 0, 
      0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {0.7, 0.8, 0.7}, {
      0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {0.6, 0.6, 0.2}, {0.8, 0.6, 0.2}, {0.2,
       0.8, 0.7}, {1, 0.4, 0.4}, {1, 1, 0}, {1, 0.6, 1}, {0, 1, 1}, {1, 0, 
      1}, {0, 1, 0}}, {{0, 1, 1}, {0, 1, 1}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.5,
       0.6, 0.8}, {1, 0.6, 0}, {0.8, 0.6, 0.2}, {0.8, 0.6, 0.5}, {
      1, 0.4, 0.4}, {0.8, 0.2, 0.5}, {1, 0.6, 1}, {0.6, 0.6, 0.2}, {1, 0, 
      1}, {0.2, 0.8, 0.7}, {1, 0, 0}, {1, 1, 0}, {0.6, 0.4, 0.8}}, {{1, 0, 
      1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 
      0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {
      0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {
      1, 0.6, 1}, {0.2, 0.8, 0.7}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}}, {{
      1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 15.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 14.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 13.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 12.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 11.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 10.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 9.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 8.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 7.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 6.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 5.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 4.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 3.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 2.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 1.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 0.5}, {1, 0}]}, 
   LineBox[{{1, 0}, {1, 16}, {17, 16}}]},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.43684222428125*^9, 3.44094472200825*^9, 
  3.442673940617625*^9}]
}, Open  ]],

Cell[TextData[{
 "Is this the same group as the dihedral group? Look along the main diagonal. \
We can count 6 occurances of the identity elenent, so only 5 elements of this \
group are of order 2. Therefore this cannot be isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "8"], TraditionalForm]]],
 ".\n                       \nWhat if we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "a"], "(", "b", ")"}], " ", "=", " ", 
    SuperscriptBox["b", 
     RowBox[{"5", "?"}]]}], TraditionalForm]]],
 " We would have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{"a", "\[CenterDot]", 
     SuperscriptBox["b", "5"]}]}], TraditionalForm]]],
 ",  and so we would use the following commands to enter this group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.440944797211375*^9, 3.440944814961375*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2,e]; Define[b^8,e]
Define[1/a,a]; Define[1/b,b^7]
Define[b.a,a.(b^5)]
M = Group[{a,b}]
MultTable[M]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436842237234375*^9, 3.43684224953125*^9}, {
  3.44094483669575*^9, 3.440944854977*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`b, $CellContext`b], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`b, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[
         Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[
         Dot[$CellContext`b, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\) \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[
         Dot[
          
          Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`b], $CellContext`b], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436842251140625*^9, 3.440944861711375*^9, 
  3.44267394682075*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0.6, 1}, {1, 0.6, 1}, {1, 0.6, 0}, {1, 0, 1}, {0.8, 0.6, 
      0.5}, {1, 0, 0}, {0.8, 0.2, 0.5}, {0.6, 0.4, 0.8}, {0.6, 0.6, 0.2}, {
      0.7, 0.8, 0.7}, {0.2, 0.8, 0.7}, {0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 
      0.2}, {0, 1, 1}, {1, 0.4, 0.4}, {0, 1, 0}}, {{0.2, 0.8, 0.7}, {0.2, 0.8,
       0.7}, {0.7, 0.8, 0.7}, {1, 1, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0.8, 
      0.6, 0.2}, {0, 1, 0}, {1, 0.4, 0.4}, {1, 0.6, 0}, {1, 0.6, 1}, {0.8, 
      0.6, 0.5}, {1, 0, 1}, {0.8, 0.2, 0.5}, {1, 0, 0}, {0.6, 0.6, 0.2}, {0.6,
       0.4, 0.8}}, {{1, 0.4, 0.4}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {
      1, 0.6, 1}, {0.2, 0.8, 0.7}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 
      1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.5, 0.6,
       0.8}, {0.8, 0.6, 0.5}, {0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}}, {{0.6, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}, {1, 
      1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {0.8, 0.6, 0.2}}, {{0.8, 0.6, 0.2}, {0.8, 0.6, 0.2}, {0, 1, 
      1}, {1, 0.4, 0.4}, {0, 1, 0}, {1, 0.6, 1}, {1, 0.6, 0}, {1, 0, 1}, {0.8,
       0.6, 0.5}, {1, 0, 0}, {0.8, 0.2, 0.5}, {0.6, 0.4, 0.8}, {0.6, 0.6, 
      0.2}, {0.7, 0.8, 0.7}, {0.2, 0.8, 0.7}, {0.5, 0.6, 0.8}, {1, 1, 0}}, {{
      0.8, 0.2, 0.5}, {0.8, 0.2, 0.5}, {1, 0, 0}, {0.6, 0.6, 0.2}, {0.6, 0.4, 
      0.8}, {0.2, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 1, 0}, {0.5, 0.6, 0.8}, {0, 
      1, 1}, {0.8, 0.6, 0.2}, {0, 1, 0}, {1, 0.4, 0.4}, {1, 0.6, 0}, {
      1, 0.6, 1}, {0.8, 0.6, 0.5}, {1, 0, 1}}, {{0.5, 0.6, 0.8}, {0.5, 0.6, 
      0.8}, {0.8, 0.6, 0.5}, {0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}, {
      1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {1, 0.6, 1}, {0.2, 0.8, 0.7}, {1, 0, 
      1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7, 
      0.8, 0.7}, {1, 0.6, 0}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1,
       1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {0.2, 0.8, 0.7}, {0.5, 0.6, 
      0.8}, {1, 1, 0}, {0.8, 0.6, 0.2}, {0, 1, 1}, {1, 0.4, 0.4}, {0, 1, 0}, {
      1, 0.6, 1}, {1, 0.6, 0}, {1, 0, 1}, {0.8, 0.6, 0.5}, {1, 0, 0}, {0.8, 
      0.2, 0.5}, {0.6, 0.4, 0.8}, {0.6, 0.6, 0.2}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {1, 0.6, 1}, {0.8, 0.6, 0.5}, {1, 0, 1}, {0.8, 0.2, 0.5}, {
      1, 0, 0}, {0.6, 0.6, 0.2}, {0.6, 0.4, 0.8}, {0.2, 0.8, 0.7}, {0.7, 0.8, 
      0.7}, {1, 1, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0.8, 0.6, 0.2}, {0, 1, 
      0}, {1, 0.4, 0.4}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7, 
      0.8, 0.7}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {0.8, 0.6, 
      0.2}, {0.8, 0.2, 0.5}, {1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {1, 0.6, 1}, {
      0.2, 0.8, 0.7}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}}, {{0, 1, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6,
       0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 
      0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}, {1, 1, 0}, {1, 0, 
      1}, {0, 1, 1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 0}, {0.8, 0.2, 0.5}, {0.6,
       0.4, 0.8}, {0.6, 0.6, 0.2}, {0.7, 0.8, 0.7}, {0.2, 0.8, 0.7}, {0.5, 
      0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 0.2}, {0, 1, 1}, {1, 0.4, 0.4}, {0, 1, 
      0}, {1, 0.6, 1}, {1, 0.6, 0}, {1, 0, 1}, {0.8, 0.6, 0.5}}, {{0, 1, 1}, {
      0, 1, 1}, {0.8, 0.6, 0.2}, {0, 1, 0}, {1, 0.4, 0.4}, {1, 0.6, 0}, {
      1, 0.6, 1}, {0.8, 0.6, 0.5}, {1, 0, 1}, {0.8, 0.2, 0.5}, {1, 0, 0}, {
      0.6, 0.6, 0.2}, {0.6, 0.4, 0.8}, {0.2, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 
      1, 0}, {0.5, 0.6, 0.8}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {
      0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {
      0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}, {
      1, 0.4, 0.4}, {0.6, 0.6, 0.2}, {1, 0.6, 1}, {0.2, 0.8, 0.7}}, {{1, 1, 
      0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {
      0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 
      0.8, 0.7}, {1, 0.6, 1}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {
      1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {
      0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6,
       0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 15.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 14.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 13.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 12.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 11.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 10.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 9.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 8.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 7.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 6.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 5.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 4.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 3.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 2.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 1.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 0.5}, {1, 0}]}, 
   LineBox[{{1, 0}, {1, 16}, {17, 16}}]},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.436842251140625*^9, 3.440944861711375*^9, 
  3.442673946898875*^9}]
}, Open  ]],

Cell[TextData[{
 "How many elements of this group are of order 2? This time there are only 4 \
occurances of the identity along the main diagonal, so there are only 3 \
elements of order 2. Thus we have yet another non-isomorphic group.\n\nWe see \
in these 3 examples that the semi-direct product ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"H", " ", 
     SubscriptBox["\[Proportional]", "\[Phi]"], "N"}]}], TraditionalForm]]],
 " depends on the choice of the homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " form ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " into ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "N", ")"}], TraditionalForm]]],
 ". In fact, even though the 3 non-identity elements of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " are essentially equivalent (The automorphism group of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " included all permutations of these 3 elements), we see that the 3 elements \
produced three different semi-direct products of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ".\n\nThis example is really more of an exception rather than a rule. Part \
of what makes this example unusual is that the automorphism group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " is abelian, and hence does not have any non-trivial ",
 StyleBox["inner",
  FontSlant->"Italic"],
 " automorphisms. We will see in the next proposition that if two \
homomorphisms ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "N", ")"}], TraditionalForm]]],
 " are related through an ",
 Cell[BoxData[
  FormBox["inner", TraditionalForm]],
  FontSlant->"Italic"],
 " automorphism of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", "N", ")"}], ","}], TraditionalForm]]],
 " then the corresponding semi-direct products will be isomorphic.\n\n\
PROPOSITION 6.7\nLet ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " be a homomorphism from a group ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "N", ")"}], TraditionalForm]]],
 ". Suppose that ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is another homomorphism such that \n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Florin]", "h"], "(", "n", ")"}], " ", "=", " ", 
     RowBox[{"w", "(", 
      RowBox[{
       SubscriptBox["\[Phi]", "h"], "(", 
       RowBox[{
        SuperscriptBox["w", 
         RowBox[{"-", "1"}]], "(", "n", ")"}], ")"}], ")"}]}]}], 
   TraditionalForm]]],
 ", \n                \nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"w", "(", "n", ")"}], TraditionalForm]]],
 " is an automorphism of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". Then \n\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", 
    SubscriptBox["\[Proportional]", "f"], "N", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", "H", " ", 
    SubscriptBox["\[Proportional]", "\[Phi]"], "N"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43684230325*^9, 3.43684230325*^9}, {3.43684236084375*^9,
   3.436842443515625*^9}, {3.440944873586375*^9, 3.440944893586375*^9}, {
  3.4409449721645*^9, 3.440944986305125*^9}}],

Cell[CellGroupData[{

Cell["Proof: ", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let us write ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{"H", " ", 
     SubscriptBox["\[Proportional]", "\[Phi]"], "N"}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"M", " ", "=", " ", 
    RowBox[{"H", " ", 
     SubscriptBox["\[Proportional]", "f"], "N"}]}], TraditionalForm]]],
 ". These are two different groups, even though they are both written using \
ordered pairs. Let us define a mapping\n\n                                    \
  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", ":", " ", 
    RowBox[{"G", " ", "\[RightArrow]", " ", "M"}]}], TraditionalForm]]],
 "\ndefined by\n                               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"v", "(", 
     RowBox[{"(", 
      RowBox[{"h", ",", "n"}], ")"}], ")"}], " ", "=", "  ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"h", ",", 
       RowBox[{"w", "(", "n", ")"}]}], ")"}], "."}]}], TraditionalForm]]],
 "\n                               \nBecause ",
 Cell[BoxData[
  FormBox[
   RowBox[{"w", "(", "n", ")"}], TraditionalForm]]],
 " is one-to-one and onto, certainly ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " is one-to-one and onto. All we would have to check is that \n\n            \
    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"v", "(", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "1"], ",", 
        SubscriptBox["n", "1"]}], ")"}], ")"}], "\[CenterDot]", 
     RowBox[{"v", "(", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "2"], ",", 
        SubscriptBox["n", "2"]}], ")"}], ")"}]}], " ", "=", " ", 
    RowBox[{"v", "(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "1"], ",", 
        SubscriptBox["n", "1"]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["h", "2"], ",", 
        SubscriptBox["n", "2"]}], ")"}]}], ")"}]}], TraditionalForm]]],
 ". \n\nWe have that \n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{"     ", 
    RowBox[{
     FormBox[
      RowBox[{
       RowBox[{
        RowBox[{"v", "(", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["h", "1"], ",", 
           SubscriptBox["n", "1"]}], ")"}], ")"}], "\[CenterDot]", 
        RowBox[{"v", "(", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["h", "2"], ",", 
           SubscriptBox["n", "2"]}], ")"}], ")"}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["h", "1"], ",", 
          RowBox[{"w", "(", 
           SubscriptBox["n", "1"], ")"}]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["h", "2"], ",", " ", 
          RowBox[{"w", "(", 
           SubscriptBox["n", "2"], ")"}]}], ")"}], " "}]}],
      TraditionalForm], "      ", "\n", 
     "                                               ", 
     RowBox[{"=", " ", 
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["h", "1"], "\[CenterDot]", 
         SubscriptBox["h", "2"]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[Florin]", 
           SubscriptBox["h", "2"]], "(", 
          RowBox[{"w", "(", 
           SubscriptBox["n", "1"], ")"}], ")"}], "\[CenterDot]", 
         RowBox[{"w", "(", 
          SubscriptBox["n", "2"], ")"}]}]}], ")"}]}], "\n", 
     "                                               ", 
     RowBox[{"=", "  ", 
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["h", "1"], "\[CenterDot]", 
         SubscriptBox["h", "2"]}], ",", 
        RowBox[{
         RowBox[{"w", "(", 
          RowBox[{
           SubscriptBox["\[Phi]", 
            SubscriptBox["h", "2"]], "(", 
           RowBox[{
            SuperscriptBox["w", 
             RowBox[{"-", "1"}]], "(", 
            RowBox[{"w", "(", 
             SubscriptBox["n", "1"], ")"}], ")"}], ")"}], ")"}], 
         "\[CenterDot]", 
         RowBox[{"w", "(", 
          SubscriptBox["n", "2"], ")"}]}]}], ")"}]}], "\n", 
     "                                               ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"(", " ", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"]}], ",", 
         RowBox[{
          RowBox[{"w", "(", 
           RowBox[{
            SubscriptBox["\[Phi]", 
             SubscriptBox["h", "2"]], "(", 
            SubscriptBox["n", "1"], ")"}], ")"}], "\[CenterDot]", 
          RowBox[{"w", "(", 
           SubscriptBox["n", "2"], ")"}]}]}], ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "           \nOn the other hand,\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"v", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["h", "1"], ",", 
          SubscriptBox["n", "1"]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["h", "2"], "\[CenterDot]", 
          SubscriptBox["n", "2"]}], ")"}]}], ")"}], " ", "=", " ", 
      RowBox[{"v", "(", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"]}], ",", " ", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Phi]", 
            SubscriptBox["h", "2"]], "(", 
           SubscriptBox["n", "1"], ")"}], "\[CenterDot]", 
          SubscriptBox["n", "2"]}]}], ")"}], "  ", ")"}]}], "        ", "\n", 
     "\t\t                                   ", 
     RowBox[{"=", "   ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["h", "1"], "\[CenterDot]", 
         SubscriptBox["h", "2"]}], ",", 
        RowBox[{"w", "(", " ", 
         RowBox[{
          RowBox[{
           SubscriptBox["\[Phi]", 
            SubscriptBox["h", "2"]], "(", 
           SubscriptBox["n", "1"], ")"}], "\[CenterDot]", 
          SubscriptBox["n", "2"]}], ")"}]}], ")"}]}], "\n", 
     "                                           ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["h", "1"], "\[CenterDot]", 
          SubscriptBox["h", "2"]}], ",", 
         RowBox[{
          RowBox[{"w", "(", " ", 
           RowBox[{
            SubscriptBox["\[Phi]", 
             SubscriptBox["h", "2"]], "(", 
            SubscriptBox["n", "1"], ")"}], ")"}], "\[CenterDot]", 
          RowBox[{"w", "(", 
           SubscriptBox["n", "2"], ")"}]}]}], ")"}], ".", "     "}]}]}]}], 
   TraditionalForm]]],
 "\n      \nSince these are equal, we have an isomorphism."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436842455328125*^9, 3.436842565953125*^9}, {
  3.436842624171875*^9, 3.436842805640625*^9}, {3.440945006492625*^9, 
  3.440945374727*^9}}]
}, Open  ]],

Cell[TextData[{
 "It is also clear that two homomorphisms ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " are related through an automorphism of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", the semi-direct products must be isomorphic since we are merely \
relabeling the elements of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". As a result there will be many instances in which there will be only one \
non-isomorphic semi-direct product of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". Whenever this happens, we can denote ",
 Cell[BoxData[
  FormBox["the", TraditionalForm]]],
 " semi-direct product as\n\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", " ", "\[Proportional]", " ", "N"}], ","}], 
   TraditionalForm]]],
 "\n\nwithout having to specify the homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 ". \n\nWe will find that we can describe many groups in terms of semi-direct \
products that would be hard to describe in any other way. With ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", the structure of these semi-direct products can easily be studied."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436842839125*^9, 3.436842843984375*^9}, {
  3.44094540738325*^9, 3.440945411086375*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Problems"
}], "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43684286196875*^9, 3.436842865078125*^9}}],

Cell["", "Input",
 PageWidth->Infinity,
 FontFamily->"System",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->RGBColor[1, 0, 0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "6.1 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to define the group \n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
    SubscriptBox["Z", "6"]}], TraditionalForm]]],
 ". Show that this group is not \n     isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "12"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43684286953125*^9}],

Cell["       ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "6.2 ) Define the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["S", "3"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". \n     Show that this group is not isomorphic to \n     ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 ".\n     (Hint: Count the elements of order 2.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.436842910265625*^9}],

Cell["       ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "6.3 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox["PartitionsP",
  FontWeight->"Bold"],
 " command\n     to find the number of abelian groups of \n     order \
120,000."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4368429086875*^9}],

Cell["      ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["\<\
For problems 6.4 through 6.7:  Find all of the 
automorphisms of the following groups.
(Hint: for the non-abelian groups, find the 
inner automorphisms first.)  \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43684290678125*^9, 3.43684298496875*^9}}],

Cell[TextData[{
 "6.4 ) ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ".  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43684290678125*^9, 3.436842907046875*^9}, {
  3.43684299025*^9, 3.436842996*^9}}],

Cell["         ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "   \n6.5 )  ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.436843004578125*^9}],

Cell["       ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "\n6.6 )  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "4"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43684301575*^9, 3.436843019375*^9}}],

Cell["       ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "\n6.7 )  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "5"], TraditionalForm]]],
 ".  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.436843026671875*^9}],

Cell["         ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "6.8 ) Show that there is only one semi-direct \n     product ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Proportional]", 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 ". Which of the 5 groups \n     of order 8 is this group isomorphic to?\n    \
 (Hint: Use proposition 6.7.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43684305215625*^9, 3.43684308171875*^9}, {
  3.4409455381645*^9, 3.440945544242625*^9}}],

Cell["        ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "6.9 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the only semi-direct \n     product ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["Z", "8", "*"], Cell["\[Proportional]", "Text",
     Evaluatable->False,
     CellHorizontalScrolling->False], 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 ". Is this group \n     isomorphic to any of the three groups of \n     \
order 16 found by considering ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], Cell["\[Proportional]", "Text",
     Evaluatable->False,
     CellHorizontalScrolling->False], 
    SubscriptBox["Z", "8"]}], TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436843079265625*^9, 3.436843148015625*^9}, {
  3.436843182109375*^9, 3.436843193734375*^9}, {3.44094556263325*^9, 
  3.440945574227*^9}}],

Cell["       ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "6.10) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to define the only \n     semi-direct product ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], Cell["\[Proportional]", "Text",
     Evaluatable->False,
     CellHorizontalScrolling->False], 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ". Show that \n     this group is different than both ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " and\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["S", "3"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4368432075*^9, 3.4368432261875*^9}, {
  3.440945592461375*^9, 3.440945596336375*^9}}],

Cell["    ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "6.11) From problems 6.1, 6.2, 6.10 and \n     section 6.4, we have found 6 \
groups of \n     order 12: ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "12"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["Z", "2"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["Z", "6"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "6"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["S", "3"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 ", and \n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], Cell["\[Proportional]", "Text",
     Evaluatable->False,
     CellHorizontalScrolling->False], 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ". Yet the table on page 85 \n     indicates that there are only 5 non-  \n  \
   isomorphic groups of order 12. Which two\n     of these groups are \
isomorphic?\n     Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to show the isomorphism."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.43684323990625*^9, {3.4368432711875*^9, 3.4368432876875*^9}, {
   3.440945618930125*^9, 3.440945622352*^9}}],

Cell["       ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{1016, 666},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.5,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[568, 21, 319, 9, 222, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[912, 34, 288, 8, 29, "Name",
 Evaluatable->False],
Cell[1203, 44, 619, 15, 70, "Name",
 Evaluatable->False]
}, Closed]],
Cell[1837, 62, 288, 12, 199, "Title",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2150, 78, 104, 3, 32, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2279, 85, 175, 4, 43, "Input",
 InitializationCell->True],
Cell[2457, 91, 209, 3, 32, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2715, 100, 91, 2, 105, "Section",
 Evaluatable->False],
Cell[2809, 104, 1178, 36, 286, "Text",
 Evaluatable->False],
Cell[3990, 142, 7159, 258, 955, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[11174, 404, 231, 8, 111, "Input"],
Cell[11408, 414, 11887, 339, 561, "Output"]
}, Open  ]],
Cell[23310, 756, 2078, 72, 344, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[25413, 832, 232, 8, 111, "Input"],
Cell[25648, 842, 11890, 339, 561, "Output"]
}, Open  ]],
Cell[37553, 1184, 220, 8, 44, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[37798, 1196, 161, 6, 66, "Input"],
Cell[37962, 1204, 19726, 474, 561, 11558, 338, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[57703, 1681, 1277, 39, 265, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[59005, 1724, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[59093, 1728, 6004, 219, 356, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[65112, 1950, 4104, 130, 654, "Text",
 Evaluatable->False],
Cell[69219, 2082, 1340, 45, 213, "Text",
 Evaluatable->False],
Cell[70562, 2129, 410, 11, 90, "Text",
 Evaluatable->False],
Cell[70975, 2142, 4018, 123, 975, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[75018, 2269, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[75106, 2273, 7931, 274, 677, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[83052, 2550, 808, 25, 191, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[83885, 2579, 251, 9, 133, "Input"],
Cell[84139, 2590, 752, 15, 42, "Output"]
}, Open  ]],
Cell[84906, 2608, 338, 14, 44, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[85269, 2626, 233, 8, 111, "Input"],
Cell[85505, 2636, 337, 7, 42, "Output"]
}, Open  ]],
Cell[85857, 2646, 452, 15, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[86334, 2665, 82, 2, 43, "Input"],
Cell[86419, 2669, 139, 3, 42, "Output"]
}, Open  ]],
Cell[86573, 2675, 982, 33, 188, "Text",
 Evaluatable->False],
Cell[87558, 2710, 201, 6, 66, "Input"],
Cell[87762, 2718, 1108, 40, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[88895, 2762, 67, 2, 43, "Input"],
Cell[88965, 2766, 5156, 100, 143, "Output"]
}, Open  ]],
Cell[94136, 2869, 164, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[94325, 2878, 84, 2, 43, "Input"],
Cell[94412, 2882, 5159, 100, 143, "Output"]
}, Open  ]],
Cell[99586, 2985, 123, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[99734, 2991, 73, 2, 43, "Input"],
Cell[99810, 2995, 118, 2, 42, "Output"]
}, Open  ]],
Cell[99943, 3000, 360, 12, 69, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[100328, 3016, 76, 2, 43, "Input"],
Cell[100407, 3020, 13955, 235, 474, "Output"]
}, Open  ]],
Cell[114377, 3258, 667, 18, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[115069, 3280, 166, 3, 43, "Input"],
Cell[115238, 3285, 2023, 50, 218, "Output"]
}, Open  ]],
Cell[117276, 3338, 121, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[117422, 3344, 68, 2, 43, "Input"],
Cell[117493, 3348, 1612, 41, 143, "Output"]
}, Open  ]],
Cell[119120, 3392, 377, 12, 69, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[119522, 3408, 67, 2, 43, "Input"],
Cell[119592, 3412, 1008, 18, 68, "Output"]
}, Open  ]],
Cell[120615, 3433, 546, 14, 115, "Text",
 Evaluatable->False],
Cell[121164, 3449, 1822, 66, 262, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[123011, 3519, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[123099, 3523, 2156, 80, 195, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[125270, 3606, 2012, 64, 330, "Text",
 Evaluatable->False],
Cell[127285, 3672, 74, 2, 43, "Input"],
Cell[127362, 3676, 1042, 36, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[128429, 3716, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[128517, 3720, 3307, 120, 321, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[131839, 3843, 489, 11, 139, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[132365, 3859, 117, 2, 143, "Section",
 Evaluatable->False],
Cell[132485, 3863, 2599, 73, 607, "Text",
 Evaluatable->False],
Cell[135087, 3938, 425, 13, 90, "Text",
 Evaluatable->False],
Cell[135515, 3953, 686, 26, 115, "Text",
 Evaluatable->False],
Cell[136204, 3981, 491, 21, 66, "Text",
 Evaluatable->False],
Cell[136698, 4004, 147, 8, 133, "Input"],
Cell[136848, 4014, 224, 10, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[137097, 4028, 78, 2, 43, "Input"],
Cell[137178, 4032, 112, 3, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[137327, 4040, 78, 2, 43, "Input"],
Cell[137408, 4044, 280, 6, 42, "Output"]
}, Open  ]],
Cell[137703, 4053, 292, 10, 66, "Text",
 Evaluatable->False],
Cell[137998, 4065, 80, 2, 43, "Input"],
Cell[138081, 4069, 236, 8, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[138342, 4081, 71, 2, 43, "Input"],
Cell[138416, 4085, 703, 14, 42, "Output"]
}, Open  ]],
Cell[139134, 4102, 103, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[139262, 4108, 76, 2, 43, "Input"],
Cell[139341, 4112, 7372, 204, 561, "Output"]
}, Open  ]],
Cell[146728, 4319, 329, 11, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[147082, 4334, 67, 2, 43, "Input"],
Cell[147152, 4338, 464, 9, 42, "Output"]
}, Open  ]],
Cell[147631, 4350, 73, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[147729, 4356, 67, 2, 43, "Input"],
Cell[147799, 4360, 158, 4, 42, "Output"]
}, Open  ]],
Cell[147972, 4367, 761, 26, 117, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[148758, 4397, 119, 6, 88, "Input"],
Cell[148880, 4405, 1475, 30, 68, "Output"]
}, Open  ]],
Cell[150370, 4438, 400, 17, 41, "Text",
 Evaluatable->False],
Cell[150773, 4457, 102, 5, 66, "Input"],
Cell[150878, 4464, 99, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[151002, 4470, 81, 2, 43, "Input"],
Cell[151086, 4474, 72, 1, 42, "Output"]
}, Open  ]],
Cell[151173, 4478, 109, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[151307, 4484, 71, 2, 43, "Input"],
Cell[151381, 4488, 705, 14, 42, "Output"]
}, Open  ]],
Cell[152101, 4505, 208, 10, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[152334, 4519, 77, 2, 43, "Input"],
Cell[152414, 4523, 90, 2, 42, "Output"]
}, Open  ]],
Cell[152519, 4528, 829, 28, 213, "Text",
 Evaluatable->False],
Cell[153351, 4558, 70, 2, 43, "Input"],
Cell[153424, 4562, 1463, 51, 215, "Text",
 Evaluatable->False],
Cell[154890, 4615, 767, 27, 164, "Text",
 Evaluatable->False],
Cell[155660, 4644, 426, 14, 66, "Text",
 Evaluatable->False],
Cell[156089, 4660, 346, 14, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[156460, 4678, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[156548, 4682, 7009, 253, 743, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[163572, 4938, 1458, 42, 311, "Text",
 Evaluatable->False],
Cell[165033, 4982, 1275, 44, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[166333, 5030, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[166421, 5034, 6328, 254, 589, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[172764, 5291, 412, 10, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[173201, 5305, 76, 2, 43, "Input"],
Cell[173280, 5309, 71, 1, 42, "Output"]
}, Open  ]],
Cell[173366, 5313, 3728, 137, 595, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[177119, 5454, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[177207, 5458, 3626, 130, 382, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[180848, 5591, 2212, 67, 472, "Text",
 Evaluatable->False],
Cell[183063, 5660, 45, 1, 43, "Input"],
Cell[183111, 5663, 453, 13, 90, "Text",
 Evaluatable->False],
Cell[183567, 5678, 1139, 43, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[184731, 5725, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[184819, 5729, 13831, 512, 1368, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[198665, 6244, 2811, 93, 460, "Text",
 Evaluatable->False],
Cell[201479, 6339, 787, 23, 213, "Text",
 Evaluatable->False],
Cell[202269, 6364, 713, 23, 115, "Text",
 Evaluatable->False],
Cell[202985, 6389, 601, 12, 237, "Text",
 Evaluatable->False],
Cell[203589, 6403, 3940, 155, 376, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[207554, 6562, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[207642, 6566, 4032, 140, 482, "SmallText",
 Evaluatable->False],
Cell[211677, 6708, 951, 38, 64, "Text",
 Evaluatable->False],
Cell[212631, 6748, 630, 20, 73, "SmallText",
 Evaluatable->False],
Cell[213264, 6770, 1780, 74, 64, "Text",
 Evaluatable->False],
Cell[215047, 6846, 318, 12, 37, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[215380, 6861, 3534, 133, 562, "Text",
 Evaluatable->False],
Cell[218917, 6996, 45, 1, 43, "Input"],
Cell[218965, 6999, 390, 8, 115, "Text"],
Cell[219358, 7009, 2249, 79, 317, "Text"],
Cell[CellGroupData[{
Cell[221632, 7092, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[221720, 7096, 7278, 269, 602, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[229013, 7368, 445, 8, 115, "Text",
 Evaluatable->False],
Cell[229461, 7378, 1762, 65, 271, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[231248, 7447, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[231336, 7451, 3438, 116, 384, "SmallText",
 Evaluatable->False],
Cell[234777, 7569, 1307, 55, 55, "Text",
 Evaluatable->False],
Cell[236087, 7626, 2031, 66, 277, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[238133, 7695, 2732, 71, 874, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[240890, 7770, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[240978, 7774, 2362, 73, 261, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[243355, 7850, 685, 23, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[244065, 7877, 78, 2, 43, "Input"],
Cell[244146, 7881, 70, 1, 42, "Output"]
}, Open  ]],
Cell[244231, 7885, 320, 10, 68, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[244576, 7899, 80, 2, 43, "Input"],
Cell[244659, 7903, 78, 1, 42, "Output"]
}, Open  ]],
Cell[244752, 7907, 438, 13, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[245215, 7924, 145, 5, 66, "Input"],
Cell[245363, 7931, 1789, 35, 366, "Output"]
}, Open  ]],
Cell[247167, 7969, 2297, 72, 390, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[249489, 8045, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[249577, 8049, 3838, 121, 484, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[253430, 8173, 459, 13, 92, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[253914, 8190, 114, 2, 43, "Input"],
Cell[254031, 8194, 71, 1, 42, "Output"]
}, Open  ]],
Cell[254117, 8198, 639, 12, 213, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[254793, 8215, 86, 2, 105, "Section",
 Evaluatable->False],
Cell[254882, 8219, 516, 16, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[255423, 8239, 124, 5, 66, "Input"],
Cell[255550, 8246, 6394, 152, 596, "Output"]
}, Open  ]],
Cell[261959, 8401, 314, 7, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[262298, 8412, 90, 2, 43, "Input"],
Cell[262391, 8416, 217, 5, 42, "Output"]
}, Open  ]],
Cell[262623, 8424, 300, 10, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[262948, 8438, 68, 2, 43, "Input"],
Cell[263019, 8442, 115, 2, 42, "Output"]
}, Open  ]],
Cell[263149, 8447, 73, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[263247, 8453, 68, 2, 43, "Input"],
Cell[263318, 8457, 92, 1, 42, "Output"]
}, Open  ]],
Cell[263425, 8461, 478, 17, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[263928, 8482, 100, 5, 66, "Input"],
Cell[264031, 8489, 201, 5, 42, "Output"]
}, Open  ]],
Cell[264247, 8497, 329, 14, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[264601, 8515, 79, 2, 43, "Input"],
Cell[264683, 8519, 95, 1, 42, "Output"]
}, Open  ]],
Cell[264793, 8523, 2000, 70, 313, "Text",
 Evaluatable->False],
Cell[266796, 8595, 637, 26, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[267458, 8625, 124, 5, 66, "Input"],
Cell[267585, 8632, 7650, 171, 362, "Output"]
}, Open  ]],
Cell[275250, 8806, 113, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[275388, 8812, 84, 2, 43, "Input"],
Cell[275475, 8816, 190, 4, 42, "Output"]
}, Open  ]],
Cell[275680, 8823, 111, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[275816, 8829, 181, 7, 88, "Input"],
Cell[276000, 8838, 254, 7, 42, "Output"],
Cell[276257, 8847, 146, 3, 42, "Output"]
}, Open  ]],
Cell[276418, 8853, 306, 9, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[276749, 8866, 71, 2, 43, "Input"],
Cell[276823, 8870, 190, 4, 42, "Output"]
}, Open  ]],
Cell[277028, 8877, 187, 8, 41, "Text",
 Evaluatable->False],
Cell[277218, 8887, 70, 2, 43, "Input"],
Cell[277291, 8891, 742, 22, 141, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[278058, 8917, 70, 2, 43, "Input"],
Cell[278131, 8921, 171, 4, 42, "Output"]
}, Open  ]],
Cell[278317, 8928, 440, 16, 139, "Text",
 Evaluatable->False],
Cell[278760, 8946, 70, 2, 43, "Input"],
Cell[278833, 8950, 816, 24, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[279674, 8978, 86, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[279763, 8982, 4828, 167, 475, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[284606, 9152, 424, 14, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[285055, 9170, 80, 2, 43, "Input"],
Cell[285138, 9174, 507, 13, 68, "Output"]
}, Open  ]],
Cell[285660, 9190, 1054, 30, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[286739, 9224, 76, 2, 43, "Input"],
Cell[286818, 9228, 3867, 105, 933, "Output"]
}, Open  ]],
Cell[290700, 9336, 584, 19, 93, "Text",
 Evaluatable->False],
Cell[291287, 9357, 441, 13, 118, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[291753, 9374, 197, 4, 42, "Subsubsection",
 Evaluatable->False],
Cell[291953, 9380, 5133, 176, 407, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[297101, 9559, 822, 26, 96, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[297948, 9589, 241, 9, 133, "Input"],
Cell[298192, 9600, 308, 6, 42, "Output"]
}, Open  ]],
Cell[298515, 9609, 3711, 118, 469, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[302251, 9731, 75, 2, 43, "Input"],
Cell[302329, 9735, 140, 3, 42, "Output"]
}, Open  ]],
Cell[302484, 9741, 151, 4, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[302660, 9749, 81, 2, 43, "Input"],
Cell[302744, 9753, 3793, 84, 309, "Output"]
}, Open  ]],
Cell[306552, 9840, 267, 10, 44, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[306844, 9854, 79, 2, 43, "Input"],
Cell[306926, 9858, 94, 1, 42, "Output"]
}, Open  ]],
Cell[307035, 9862, 644, 24, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[307704, 9890, 165, 6, 66, "Input"],
Cell[307872, 9898, 320, 7, 42, "Output"],
Cell[308195, 9907, 3800, 86, 331, "Output"]
}, Open  ]],
Cell[312010, 9996, 91, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[312126, 10002, 79, 2, 43, "Input"],
Cell[312208, 10006, 94, 1, 42, "Output"]
}, Open  ]],
Cell[312317, 10010, 389, 16, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[312731, 10030, 81, 2, 43, "Input"],
Cell[312815, 10034, 1132, 26, 42, "Output"]
}, Open  ]],
Cell[313962, 10063, 982, 34, 96, "Text",
 Evaluatable->False],
Cell[314947, 10099, 425, 15, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[315397, 10118, 257, 9, 133, "Input"],
Cell[315657, 10129, 1368, 27, 42, "Output"]
}, Open  ]],
Cell[317040, 10159, 5772, 190, 578, "Text",
 Evaluatable->False],
Cell[322815, 10351, 512, 18, 41, "Text",
 Evaluatable->False],
Cell[323330, 10371, 4633, 132, 389, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[327988, 10507, 97, 2, 43, "Input"],
Cell[328088, 10511, 1016, 22, 42, "Output"]
}, Open  ]],
Cell[329119, 10536, 109, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[329253, 10542, 80, 2, 43, "Input"],
Cell[329336, 10546, 7638, 170, 274, "Output"]
}, Open  ]],
Cell[336989, 10719, 119, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[337133, 10725, 78, 2, 43, "Input"],
Cell[337214, 10729, 94, 1, 42, "Output"]
}, Open  ]],
Cell[337323, 10733, 4520, 139, 340, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[341868, 10876, 97, 2, 43, "Input"],
Cell[341968, 10880, 1019, 22, 42, "Output"]
}, Open  ]],
Cell[343002, 10905, 88, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[343115, 10911, 80, 2, 43, "Input"],
Cell[343198, 10915, 7786, 173, 276, "Output"]
}, Open  ]],
Cell[350999, 11091, 186, 6, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[351210, 11101, 78, 2, 43, "Input"],
Cell[351291, 11105, 93, 1, 42, "Output"]
}, Open  ]],
Cell[351399, 11109, 276, 10, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[351700, 11123, 76, 2, 43, "Input"],
Cell[351779, 11127, 2739, 61, 93, "Output"]
}, Open  ]],
Cell[354533, 11191, 879, 27, 139, "Text",
 Evaluatable->False],
Cell[355415, 11220, 71, 2, 43, "Input"],
Cell[355489, 11224, 1085, 37, 115, "Text",
 Evaluatable->False],
Cell[356577, 11263, 76, 2, 43, "Input"],
Cell[356656, 11267, 79, 2, 43, "Input"],
Cell[356738, 11271, 81, 2, 43, "Input"],
Cell[356822, 11275, 189, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[357036, 11286, 79, 2, 43, "Input"],
Cell[357118, 11290, 93, 1, 42, "Output"]
}, Open  ]],
Cell[357226, 11294, 253, 10, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[357504, 11308, 70, 2, 43, "Input"],
Cell[357577, 11312, 1348, 26, 42, "Output"]
}, Open  ]],
Cell[358940, 11341, 176, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[359141, 11352, 76, 2, 43, "Input"],
Cell[359220, 11356, 113, 2, 42, "Output"]
}, Open  ]],
Cell[359348, 11361, 908, 37, 74, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[360281, 11402, 70, 2, 43, "Input"],
Cell[360354, 11406, 324, 6, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[360715, 11417, 72, 2, 43, "Input"],
Cell[360790, 11421, 444, 8, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[361271, 11434, 74, 2, 43, "Input"],
Cell[361348, 11438, 91, 1, 42, "Output"]
}, Open  ]],
Cell[361454, 11442, 123, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[361602, 11448, 93, 2, 43, "Input"],
Cell[361698, 11452, 919, 18, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[362654, 11475, 81, 2, 43, "Input"],
Cell[362738, 11479, 7733, 172, 287, "Output"]
}, Open  ]],
Cell[370486, 11654, 116, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[370627, 11660, 80, 2, 43, "Input"],
Cell[370710, 11664, 7640, 162, 168, "Output"]
}, Open  ]],
Cell[378365, 11829, 761, 28, 117, "Text",
 Evaluatable->False],
Cell[379129, 11859, 76, 2, 43, "Input"],
Cell[379208, 11863, 81, 2, 43, "Input"],
Cell[379292, 11867, 79, 2, 43, "Input"],
Cell[379374, 11871, 106, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[379505, 11877, 79, 2, 43, "Input"],
Cell[379587, 11881, 94, 1, 42, "Output"]
}, Open  ]],
Cell[379696, 11885, 267, 10, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[379988, 11899, 76, 2, 43, "Input"],
Cell[380067, 11903, 113, 2, 42, "Output"]
}, Open  ]],
Cell[380195, 11908, 73, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[380293, 11914, 70, 2, 43, "Input"],
Cell[380366, 11918, 1348, 26, 42, "Output"]
}, Open  ]],
Cell[381729, 11947, 724, 31, 68, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[382478, 11982, 70, 2, 43, "Input"],
Cell[382551, 11986, 323, 6, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[382911, 11997, 72, 2, 43, "Input"],
Cell[382986, 12001, 442, 8, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[383465, 12014, 74, 2, 43, "Input"],
Cell[383542, 12018, 89, 1, 42, "Output"]
}, Open  ]],
Cell[383646, 12022, 181, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[383852, 12033, 93, 2, 43, "Input"],
Cell[383948, 12037, 913, 18, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[384898, 12060, 81, 2, 43, "Input"],
Cell[384982, 12064, 7771, 174, 264, "Output"]
}, Open  ]],
Cell[392768, 12241, 114, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[392907, 12247, 84, 2, 43, "Input"],
Cell[392994, 12251, 26519, 548, 493, "Output"]
}, Open  ]],
Cell[419528, 12802, 139, 4, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[419692, 12810, 73, 2, 43, "Input"],
Cell[419768, 12814, 91, 1, 42, "Output"]
}, Open  ]],
Cell[419874, 12818, 1023, 27, 190, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[420922, 12849, 93, 2, 43, "Input"],
Cell[421018, 12853, 1918, 53, 369, "Output"]
}, Open  ]],
Cell[422951, 12909, 2925, 97, 494, "Text",
 Evaluatable->False],
Cell[425879, 13008, 1300, 46, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[427204, 13058, 86, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[427293, 13062, 7205, 235, 558, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[434513, 13300, 353, 13, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[434891, 13317, 82, 2, 43, "Input"],
Cell[434976, 13321, 2762, 62, 93, "Output"]
}, Open  ]],
Cell[437753, 13386, 742, 22, 68, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[438520, 13412, 78, 2, 43, "Input"],
Cell[438601, 13416, 3842, 72, 333, 1268, 28, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[442458, 13491, 984, 28, 265, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[443467, 13523, 126, 3, 43, "Input"],
Cell[443596, 13528, 27574, 582, 518, "Output"]
}, Open  ]],
Cell[471185, 14113, 101, 2, 41, "Text",
 Evaluatable->False],
Cell[471289, 14117, 636, 23, 90, "Text",
 Evaluatable->False],
Cell[471928, 14142, 664, 24, 90, "Text",
 Evaluatable->False],
Cell[472595, 14168, 272, 11, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[472892, 14183, 178, 10, 178, "Input"],
Cell[473073, 14195, 727, 14, 42, "Output"]
}, Open  ]],
Cell[473815, 14212, 951, 31, 115, "Text",
 Evaluatable->False],
Cell[474769, 14245, 486, 14, 69, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[475280, 14263, 306, 9, 133, "Input"],
Cell[475589, 14274, 901, 17, 42, "Output"]
}, Open  ]],
Cell[476505, 14294, 3467, 127, 265, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[479997, 14425, 97, 2, 43, "Input"],
Cell[480097, 14429, 611, 12, 42, "Output"]
}, Open  ]],
Cell[480723, 14444, 108, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[480856, 14450, 81, 2, 43, "Input"],
Cell[480940, 14454, 6546, 154, 336, "Output"]
}, Open  ]],
Cell[487501, 14611, 201, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[487727, 14622, 79, 2, 43, "Input"],
Cell[487809, 14626, 91, 1, 42, "Output"]
}, Open  ]],
Cell[487915, 14630, 2029, 77, 142, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[489969, 14711, 93, 2, 43, "Input"],
Cell[490065, 14715, 698, 15, 42, "Output"]
}, Open  ]],
Cell[490778, 14733, 94, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[490897, 14739, 144, 3, 43, "Input"],
Cell[491044, 14744, 7685, 171, 356, "Output"]
}, Open  ]],
Cell[498744, 14918, 189, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[498958, 14929, 79, 2, 43, "Input"],
Cell[499040, 14933, 93, 1, 42, "Output"]
}, Open  ]],
Cell[499148, 14937, 2499, 69, 443, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[501672, 15010, 100, 2, 43, "Input"],
Cell[501775, 15014, 185, 4, 42, "Output"]
}, Open  ]],
Cell[501975, 15021, 175, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[502175, 15032, 97, 2, 43, "Input"],
Cell[502275, 15036, 6483, 154, 296, "Output"]
}, Open  ]],
Cell[508773, 15193, 742, 27, 118, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[509540, 15224, 96, 2, 43, "Input"],
Cell[509639, 15228, 193, 4, 42, "Output"]
}, Open  ]],
Cell[509847, 15235, 165, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[510037, 15246, 97, 2, 43, "Input"],
Cell[510137, 15250, 15680, 305, 253, 7559, 170, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[525832, 15558, 620, 20, 118, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[526477, 15582, 81, 2, 43, "Input"],
Cell[526561, 15586, 19727, 502, 1368, "Output"]
}, Open  ]],
Cell[546303, 16091, 202, 6, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[546530, 16101, 73, 2, 43, "Input"],
Cell[546606, 16105, 93, 1, 42, "Output"]
}, Open  ]],
Cell[546714, 16109, 899, 20, 216, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[547638, 16133, 76, 2, 43, "Input"],
Cell[547717, 16137, 93, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[547847, 16143, 76, 2, 43, "Input"],
Cell[547926, 16147, 94, 1, 42, "Output"]
}, Open  ]],
Cell[548035, 16151, 312, 11, 66, "Text",
 Evaluatable->False],
Cell[548350, 16164, 86, 2, 43, "Input"],
Cell[548439, 16168, 149, 4, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[548613, 16176, 84, 2, 43, "Input"],
Cell[548700, 16180, 2404, 35, 343, "Output"]
}, Open  ]],
Cell[551119, 16218, 569, 15, 188, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[551725, 16238, 93, 2, 105, "Section",
 Evaluatable->False],
Cell[551821, 16242, 1721, 58, 262, "Text",
 Evaluatable->False],
Cell[553545, 16302, 6292, 234, 406, "Text",
 Evaluatable->False],
Cell[559840, 16538, 905, 19, 90, "Text",
 Evaluatable->False],
Cell[560748, 16559, 766, 20, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[561539, 16583, 86, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[561628, 16587, 14156, 449, 874, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[575799, 17039, 1714, 52, 217, "Text",
 Evaluatable->False],
Cell[577516, 17093, 608, 20, 92, "Text",
 Evaluatable->False],
Cell[578127, 17115, 742, 30, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[578894, 17149, 86, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[578983, 17153, 10705, 366, 726, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[589703, 17522, 1776, 53, 241, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[591504, 17579, 86, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[591593, 17583, 17590, 588, 1208, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[609198, 18174, 5374, 187, 585, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[614597, 18365, 243, 9, 111, "Input"],
Cell[614843, 18376, 136, 3, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[615016, 18384, 230, 8, 88, "Input"],
Cell[615249, 18394, 922, 18, 42, "Output"]
}, Open  ]],
Cell[616186, 18415, 729, 28, 93, "Text",
 Evaluatable->False],
Cell[616918, 18445, 1330, 44, 172, "Text",
 Evaluatable->False],
Cell[618251, 18491, 1248, 42, 92, "Text",
 Evaluatable->False],
Cell[619502, 18535, 189, 4, 43, "Input"],
Cell[619694, 18541, 191, 5, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[619910, 18550, 141, 6, 66, "Input"],
Cell[620054, 18558, 2318, 46, 68, "Output"],
Cell[622375, 18606, 3610, 71, 459, "Output"]
}, Open  ]],
Cell[626000, 18680, 2518, 81, 439, "Text",
 Evaluatable->False],
Cell[628521, 18763, 535, 16, 188, "Text"],
Cell[629059, 18781, 2385, 75, 363, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[631469, 18860, 358, 12, 156, "Input"],
Cell[631830, 18874, 10705, 189, 118, "Output"],
Cell[642538, 19065, 7726, 140, 406, "Output"]
}, Open  ]],
Cell[650279, 19208, 2114, 66, 271, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[652418, 19278, 349, 12, 156, "Input"],
Cell[652770, 19292, 10661, 188, 118, "Output"],
Cell[663434, 19482, 7681, 139, 406, "Output"]
}, Open  ]],
Cell[671130, 19624, 1016, 29, 141, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[672171, 19657, 305, 11, 156, "Input"],
Cell[672479, 19670, 10662, 188, 118, "Output"],
Cell[683144, 19860, 7676, 138, 406, "Output"]
}, Open  ]],
Cell[690835, 20001, 3756, 117, 554, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[694616, 20122, 86, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[694705, 20126, 6926, 211, 547, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[701646, 20340, 1481, 42, 286, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[703164, 20387, 211, 7, 105, "Section",
 Evaluatable->False],
Cell[703378, 20396, 156, 6, 43, "Input"],
Cell[703537, 20404, 493, 18, 90, "Text",
 Evaluatable->False],
Cell[704033, 20424, 71, 2, 43, "Input"],
Cell[704107, 20428, 561, 20, 115, "Text",
 Evaluatable->False],
Cell[704671, 20450, 71, 2, 43, "Input"],
Cell[704745, 20454, 323, 12, 90, "Text",
 Evaluatable->False],
Cell[705071, 20468, 70, 2, 43, "Input"],
Cell[705144, 20472, 303, 8, 115, "Text",
 Evaluatable->False],
Cell[705450, 20482, 276, 10, 41, "Text",
 Evaluatable->False],
Cell[705729, 20494, 73, 2, 43, "Input"],
Cell[705805, 20498, 230, 9, 69, "Text",
 Evaluatable->False],
Cell[706038, 20509, 71, 2, 43, "Input"],
Cell[706112, 20513, 232, 9, 66, "Text",
 Evaluatable->False],
Cell[706347, 20524, 71, 2, 43, "Input"],
Cell[706421, 20528, 217, 9, 66, "Text",
 Evaluatable->False],
Cell[706641, 20539, 73, 2, 43, "Input"],
Cell[706717, 20543, 515, 13, 118, "Text",
 Evaluatable->False],
Cell[707235, 20558, 72, 2, 43, "Input"],
Cell[707310, 20562, 912, 27, 118, "Text",
 Evaluatable->False],
Cell[708225, 20591, 71, 2, 43, "Input"],
Cell[708299, 20595, 835, 29, 115, "Text",
 Evaluatable->False],
Cell[709137, 20626, 68, 2, 43, "Input"],
Cell[709208, 20630, 1388, 49, 213, "Text",
 Evaluatable->False],
Cell[710599, 20681, 71, 2, 43, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
