(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    383143,      11989]
NotebookOptionsPosition[    364020,      11345]
NotebookOutlinePosition[    364684,      11371]
CellTagsIndexPosition[    364641,      11368]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Abstract Algebra: An Interactive Approach", "Special2",
 Evaluatable->False,
 CellChangeTimes->{{3.43686720265625*^9, 3.436867216140625*^9}},
 TextAlignment->Center,
 FontFamily->"Arial",
 FontSize->36,
 FontWeight->"Bold",
 FontSlant->"Italic",
 FontColor->RGBColor[1, 0, 1],
 Background->RGBColor[0.500008, 1, 1]],

Cell[CellGroupData[{

Cell["\[Copyright]2009   William Paulsen", "Name",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436867220703125*^9, 3.43686722153125*^9}},
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]],

Cell["\<\
All rights reserved:  No part of these materials may be reproduced, either \
electronically, mechanically, or by any other means or in any form, without \
the written permission from the author. All software is provided \"as is\" \
without warranty of any kind, either expressed or implied, including, but not \
limited to, the implied warranties of merchantability and fitness for a \
particular purpose.\
\>", "Name",
 Editable->False,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]]
}, Closed]],

Cell["\<\
Chapter 7
The Search for Normal Subgroups\
\>", "Title",
 CellFrame->True,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 TextAlignment->Center,
 FontFamily->"New York",
 FontSize->24,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell["Initialization", "Special2",
 Evaluatable->False,
 TextAlignment->Center,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["<<c:\\math\\group.m", "Input",
 PageWidth->Infinity,
 InitializationCell->True,
 CellChangeTimes->{{3.422652393078125*^9, 3.4226523958125*^9}, {
  3.43686722646875*^9, 3.436867230046875*^9}},
 AspectRatioFixed->True],

Cell[BoxData["\<\"Initialization Done\"\>"], "Print",
 CellChangeTimes->{3.422652400703125*^9, 3.4368672340625*^9, 
  3.43692546096875*^9, 3.44094731613325*^9, 3.44267409882075*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Center of a Group", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We seen several instances in the last chapter in which show that the \
structure of a group hinges on its normal subgroups. For example, in the last \
section we saw that if a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has a normal subgroup ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", and if there is one other group ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{"X", "\[CenterDot]", "K"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", " ", "\[Intersection]", " ", "K"}], " ", "=", " ", 
    RowBox[{"{", "1", "}"}]}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is either a direct product ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", " ", "\[Times]", " ", "K"}], TraditionalForm]]],
 ", or a semi-direct product of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ". \n\nBecause of the special importance of normal subgroups, we will want \
to develop techniques for finding ",
 StyleBox["all",
  FontSlant->"Italic"],
 " of the normal subgroups of a given group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". We will discover in the process that some of the normal groups have \
additional properties.\n\nSince every subgroup of an abelian group is \
automatically a normal subgroup we will concentrate our attention in this \
chapter to non-abelian groups. Consider, for example, the quaternionic group \
",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", along with its multiplication table:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[i^4,e]; Define[j^2,i^2]
Define[j.i,i.i.i.j]
Define[1/i,i^3]; Define[1/j,i.i.j]
Q = Group[{i,j}]
MultTable[Q]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436867512046875*^9, 3.43686751709375*^9}, 
   3.442674045367625*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "i", ",", "j", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{{3.442674042148875*^9, 3.442674046148875*^9}, 
   3.4426741022895*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0, 1}, {0, 
      1, 0}, {1, 1, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {1, 0, 0}}, {{
      1, 0.6, 0}, {1, 0.6, 0}, {0, 1, 0}, {1, 1, 0}, {0, 1, 1}, {0.6, 0.4, 
      0.8}, {0.7, 0.8, 0.7}, {1, 0, 0}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 
      0.4, 0.8}, {1, 1, 0}, {0.7, 0.8, 0.7}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {1, 0.6, 0}}, {{0, 1, 0}, {0, 1, 0}, {0, 1, 1}, {0.6, 
      0.4, 0.8}, {0.7, 0.8, 0.7}, {1, 0, 0}, {1, 0.6, 0}, {1, 0, 1}, {1, 1, 
      0}}, {{1, 0, 0}, {1, 0, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {1, 1, 0}, {
      0.7, 0.8, 0.7}, {1, 0, 1}, {0, 1, 1}, {0, 1, 0}}, {{0, 1, 1}, {0, 1, 
      1}, {0.7, 0.8, 0.7}, {1, 0, 0}, {1, 0.6, 0}, {1, 0, 1}, {0, 1, 0}, {1, 
      1, 0}, {0.6, 0.4, 0.8}}, {{1, 0, 1}, {1, 0, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {1, 1, 0}, {0.7, 0.8, 0.7}, {0, 1, 1}}, {{
      1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 
      1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7,
       0.8, 0.7}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"j\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\[CenterDot]i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{{3.442674042148875*^9, 3.442674046148875*^9}, 
   3.44267410238325*^9}]
}, Open  ]],

Cell[TextData[{
 "Recall that there is only one element of order 2 in this group, namely ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 ". This can be seen by noticing that there are only 2 ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 "'s appearing in the main diagonal of the multiplication table.\n\nThe \
element ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 " has another property in the multiplication table. Look at the pattern of \
all of the yellow and red squares in the table. (Yellow and red squares \
correspond to ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 ".) These two colors, considered alone, are symetric by a mirror image alone \
the main diagonal even though the entire table is not symetric. We could \
interpret this by saying that whenever ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "b"}], " ", "=", " ", 
    SuperscriptBox["i", "2"]}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 " is also ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 ".  Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["a", 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      SuperscriptBox["i", "2"]}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["i", "2"], "\[CenterDot]", 
      SuperscriptBox["a", 
       RowBox[{"-", "1"}]]}]}]}], TraditionalForm]]],
 " for all elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "."}], TraditionalForm]]],
 " In order for this to happen, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 " must commute with all of the elements in ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". We can see this using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". For example, compute"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["j.i.i", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
  StringForm["`` \[CenterDot] ``", 
   Dot[$CellContext`i, $CellContext`i], $CellContext`j],
  Editable->False]], "Output",
 CellChangeTimes->{3.442674050805125*^9, 3.442674105148875*^9}]
}, Open  ]],

Cell[TextData[{
 "Whenever we have a non-commutative group, we want to identify all such \
elements. Thus, we make the following definition.\n\nDEFINITION 7.1:\nGiven a \
group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", the ",
 StyleBox["center",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is defined to be the set of elements ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "x"}]}], TraditionalForm]]],
 " for all elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[Element]", 
    RowBox[{"G", "."}]}], TraditionalForm]]],
 " The center of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is customary denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " because of the German spelling."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43686759553125*^9, 3.4368676636875*^9}}],

Cell[TextData[{
 "We then have the following easy proposition:\n\nPROPOSITION 7.1\nGiven a \
group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "."}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43686767021875*^9, 3.436867676703125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First, we need to show that ",
 StyleBox["Z(",
  FontSlant->"Italic"],
 "G",
 StyleBox[")",
  FontSlant->"Italic"],
 " is a subgroup of G. If ",
 StyleBox["x",
  FontSlant->"Italic"],
 " and ",
 StyleBox["y",
  FontSlant->"Italic"],
 " are in ",
 StyleBox["Z(",
  FontSlant->"Italic"],
 "G",
 StyleBox[")",
  FontSlant->"Italic"],
 ", and ",
 StyleBox["a",
  FontSlant->"Italic"],
 " is any element in G, then \n\n                            ",
 StyleBox["x\[CenterDot]y\[CenterDot]a = x\[CenterDot]a\[CenterDot]y = a\
\[CenterDot]x\[CenterDot]y",
  FontSlant->"Italic"],
 ".\n\nSo ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " commutes with all of the elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 ".\n\nAlso, we have\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "a"}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["a", 
          RowBox[{"-", "1"}]], "\[CenterDot]", "x"}], ")"}], 
       RowBox[{"-", "1"}]], " ", "=", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "\[CenterDot]", 
          SuperscriptBox["a", 
           RowBox[{"-", "1"}]]}], ")"}], 
        RowBox[{"-", "1"}]], " ", "=", " ", 
       RowBox[{"a", "\[CenterDot]", 
        SuperscriptBox["x", 
         RowBox[{"-", "1"}]]}]}]}]}], ","}], TraditionalForm]]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " must also be in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 ". Thus, by proposition 2.2, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nNext, we can see that\n   \n                              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "x", "\[CenterDot]", 
     SuperscriptBox["a", 
      RowBox[{"-", "1"}]]}], "=", " ", 
    RowBox[{
     RowBox[{"x", "\[CenterDot]", "a", "\[CenterDot]", 
      SuperscriptBox["a", 
       RowBox[{"-", "1"}]]}], "=", " ", "x"}]}], TraditionalForm]]],
 ",\n\nSo ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "x", "\[CenterDot]", 
    SuperscriptBox["a", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " whenever ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Thus, by proposition 3.4, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436867712734375*^9, 3.43686773221875*^9}}]
}, Open  ]],

Cell[TextData[{
 "We use the command ",
 StyleBox["GroupCenter[",
  FontWeight->"Bold"],
 "G",
 StyleBox["] ",
  FontWeight->"Bold"],
 "to find the center of a group in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". For example, the center of the quaternionic group is "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Z = GroupCenter[Q]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436867749375*^9, 3.436867749875*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.44267405475825*^9, 3.442674108961375*^9}]
}, Open  ]],

Cell[TextData[{
 "So ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "Q", ")"}], TraditionalForm]]],
 " consists just of the identity and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 ". To verify that this is indeed a normal subgroup, let us look at the left \
and right cosets."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["LftCoset[Q, Z]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.44267405875825*^9, 3.44267413950825*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"e", ",", 
     InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`i],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`j],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
      
      Editable->False]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.442674056055125*^9, 3.44267414044575*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["RtCoset[Q, Z]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.442674063148875*^9, 3.44267406344575*^9}, {
  3.442674142648875*^9, 3.442674143367625*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"e", ",", 
     InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`i],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`i, $CellContext`i], $CellContext`j],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
      
      Editable->False]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442674064227*^9, 3.442674144336375*^9}]
}, Open  ]],

Cell[TextData[{
 "So these are the same, which demonstrates the fact that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "Q", ")"}], TraditionalForm]]],
 " is indeed normal.\n\nSometimes the center of a group consists of just the \
\nidentity element. For example, consider the symmetric group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2, e]
Define[b^3, e]
Define[b.a, a.b.b]
Define[1/a, a]
Define[1/b, b^2]
S3 = Group[{a, b}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.442674156586375*^9}]
}, Open  ]],

Cell["The center of this group is", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["GroupCenter[S3]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", "e", "}"}]], "Output",
 CellChangeTimes->{3.442674158930125*^9}]
}, Open  ]],

Cell[TextData[{
 "So the center of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " is just ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "e", "}"}], TraditionalForm]]],
 ".  Whenever this happens, we say that the group is ",
 StyleBox["centerless",
  FontSlant->"Italic"],
 ". Of course this is a normal subgroup, but it hampers our efforts in \
finding proper normal subgroups. In fact, this will happen for all of the \
permutation groups larger than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ". Since the proof involves an even permutation, we end up finding the \
center of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " at the same time.                                     \n            \n\
PROPOSITION 7.2\nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "3"}], TraditionalForm]]],
 ", then the groups ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " are centerless. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436867787203125*^9, 3.43686782746875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is an element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " which is not the identity. We need to show that ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " cannot be in the center of either ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ", which amounts to finding an element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 "  which does not commute with ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is not the identity, there is some number ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " which is not fixed by ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 ", say ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is mapped to ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "3"}], TraditionalForm]]],
 ", there is at least one number not is the list ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"x", ",", " ", "y", ",", " ", 
     RowBox[{"\[Phi]", "(", "y", ")"}]}], "}"}], TraditionalForm]]],
 ".  Let ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " be one of these remaining numbers. Finally, we let ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " be the 3-cycle ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", " ", "y", " ", "z"}], ")"}], TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an even permutation ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ".   Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "\[CenterDot]", "f"}], TraditionalForm]]],
 " sends ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " to z, but ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[CenterDot]", "\[Phi]"}], TraditionalForm]]],
 " sends ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " to",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Phi]", "(", "y", ")"}], "\[NotEqual]", "z"}]}], 
   TraditionalForm]]],
 ".  Thus,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Florin]", "\[CenterDot]", "\[Phi]"}], " ", "\[NotEqual]", " ",
      
     RowBox[{"\[Phi]", "\[CenterDot]", "\[Florin]"}]}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is not in the center of either ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42265143425*^9, 3.422651530125*^9}, {
  3.436867953140625*^9, 3.43686798446875*^9}, {3.4368680241875*^9, 
  3.43686803125*^9}}]
}, Open  ]],

Cell[TextData[{
 "The other extreme is if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " is the entire group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". In this case every element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " must commute with all other elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", so this happens if, and only if, the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is abelian.\n\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", what is the quotient group? The answer is rather interesting.\n\n\
PROPOSITION 7.3\nIf ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a group, then\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"G", "/", 
     RowBox[{"Z", "(", "G", ")"}]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"Inn", "(", "G", ")"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436868045375*^9, 3.436868045515625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We begin by observing that the mapping \n\n                               \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", ":", 
    RowBox[{"G", " ", "\[RightArrow]", " ", 
     RowBox[{"Inn", "(", "G", ")"}]}]}], TraditionalForm]]],
 "\ngiven by\n                                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[Phi]", "x"],
      TraditionalForm], " ", 
     RowBox[{"(", "y", ")"}]}], " ", "=", " ", 
    RowBox[{"x", "\[CenterDot]", "y", "\[CenterDot]", 
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 "\n\nis a homomorphism, as we saw in the proof of the semi-direct product \
theorem (6.3). By the definition of the inner automorphisms, this mapping is \
surjective. However, this mapping is not necessarily injective. Let us \
determine the \nkernel of ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 ".\n\nSuppose that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "x"], TraditionalForm]]],
 "  is the identity homomorphism. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "x"], "(", "y", ")"}], " ", "=", " ", "y"}], 
   TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". This means that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y", "\[CenterDot]", 
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]]}], " ", "=", " ", "y"}], TraditionalForm]]],
 ", or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "x"}]}], TraditionalForm]]],
 ", for all ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in the center of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nNow, suppose ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "x"], "(", "y", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"x", "\[CenterDot]", "y", "\[CenterDot]", 
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"y", "\[CenterDot]", "x", "\[CenterDot]", 
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]]}], " ", "=", " ", "y"}]}]}], TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 "  is the identity homomorphism. Thus the kernel of ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is precisely the center of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Therefore, by the first isomorphism theorem (4.1), we have\n\n            \
                             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"G", "/", 
     RowBox[{"Z", "(", "G", ")"}]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"Inn", "(", "G", ")"}]}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436868064375*^9, 3.436868064453125*^9}, {
  3.43686818684375*^9, 3.436868187*^9}}]
}, Open  ]],

Cell[TextData[{
 "The center of a group possesses a characteristic that is even stronger than \
that of a normal subgroup. To illustrate this characteristic, consider the \
next proposition.\n\nPROPOSITION 7.4\nLet ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " be a normal subgroup of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "N", ")"}], TraditionalForm]]],
 " is a normal subgroup not only of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ", but of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43686819521875*^9, 3.43686819534375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " be an element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " an element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "N", ")"}], TraditionalForm]]],
 ". We need to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "\[CenterDot]", "z", "\[CenterDot]", 
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "N", ")"}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", we certainly know that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "\[CenterDot]", "z", "\[CenterDot]", 
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ", so the way to test that it is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "N", ")"}], TraditionalForm]]],
 " is to show that it commutes with every element of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ".\n\nLet ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " be an element of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". We want to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "\[CenterDot]", "z", "\[CenterDot]", 
     SuperscriptBox["g", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "n"}], " ", "=", " ", 
    RowBox[{"n", "\[CenterDot]", "g", "\[CenterDot]", "z", "\[CenterDot]", 
     SuperscriptBox["g", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["g", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "n", "\[CenterDot]", "g"}]}], 
   TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is normal in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Also, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", 
    RowBox[{"g", "\[CenterDot]", "h", "\[CenterDot]", 
     SuperscriptBox["g", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ", so\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       FormBox[
        RowBox[{"g", "\[CenterDot]", "z", "\[CenterDot]", 
         SuperscriptBox["g", 
          RowBox[{"-", "1"}]]}],
        TraditionalForm], "\[CenterDot]", "n"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"g", "\[CenterDot]", "z", "\[CenterDot]", 
          SuperscriptBox["g", 
           RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"g", "\[CenterDot]", "h", "\[CenterDot]", 
          SuperscriptBox["g", 
           RowBox[{"-", "1"}]]}], ")"}]}], "=", " ", 
       RowBox[{
        RowBox[{
        "g", "\[CenterDot]", "z", "\[CenterDot]", "h", "\[CenterDot]", 
         SuperscriptBox["g", 
          RowBox[{"-", "1"}]]}], "=", " ", 
        RowBox[{
        "g", "\[CenterDot]", "h", "\[CenterDot]", "z", "\[CenterDot]", 
         SuperscriptBox["g", 
          RowBox[{"-", "1"}]]}]}]}]}], "\n", "                          ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"g", "\[CenterDot]", "h", "\[CenterDot]", 
          SuperscriptBox["g", 
           RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"g", "\[CenterDot]", "z", "\[CenterDot]", 
          SuperscriptBox["g", 
           RowBox[{"-", "1"}]]}], ")"}]}], " ", "=", "  ", 
       RowBox[{
       "n", "\[CenterDot]", "g", "\[CenterDot]", "z", "\[CenterDot]", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"-", "1"}]], "."}]}]}]}]}]}], TraditionalForm]]],
 "\n           \nHence, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "\[CenterDot]", "z", "\[CenterDot]", 
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " commutes with every element ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "\[CenterDot]", "z", "\[CenterDot]", 
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "N", ")"}], TraditionalForm]]],
 ". By proposition 3.4, we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "N", ")"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436868279125*^9, 3.436868279875*^9}}]
}, Open  ]],

Cell["\<\
This proposition demonstrates a very unusual property of a center of a group \
since, in general, the normal subgroup of a normal subgroup is not a normal \
subgroup. To give an example, consider the octahedronal group.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2, e]
Define[b^3, e]
Define[c^4, e]
Define[b.a, a.b.b]
Define[c.a, a.b.c]
Define[c.b, a.c.c]
Define[1/a, a]
Define[1/b, b^2]
Define[1/c, c^3]
G = Group[{a, b, c}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", "c", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(c\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`c, $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`b], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`b, $CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`b, $CellContext`b], $CellContext`c], $CellContext`c], \
$CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4426741685395*^9}]
}, Open  ]],

Cell["We have found a normal subgroup of order 4, namely", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["M = Group[{a.b.b.c, c.c}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`c], $CellContext`c], $CellContext`c],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.442674171461375*^9}]
}, Open  ]],

Cell[TextData[{
 "Now we can find a normal subgroup of ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 ". (In fact, since ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " is abelian, any subgroup of ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " is a normal subgroup.) For example, let us pick the subgroup"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["H = Group[{c.c}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", 
   InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(c\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`c],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.44267417307075*^9}]
}, Open  ]],

Cell[TextData[{
 "Now ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". But when we compare"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["LftCoset[G, H]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.44267417557075*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`a, $CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"b", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`b, $CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"c", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(c\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`c, $CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"e", ",", 
     InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c],
      
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`c],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`a, $CellContext`c], $CellContext`c], $CellContext`c],
      
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`b, $CellContext`b], $CellContext`c], $CellContext`c],
      
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`c],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`b, $CellContext`c], $CellContext`c], $CellContext`c],
      
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`a, $CellContext`b], $CellContext`b],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\
\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\
\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[
         Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`a, $CellContext`b], $CellContext`c],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\
\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\
\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[
         Dot[$CellContext`a, $CellContext`b], $CellContext`c], \
$CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`b, $CellContext`b], $CellContext`c],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\
\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\
\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[
         Dot[$CellContext`b, $CellContext`b], $CellContext`c], \
$CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c],
      
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\
\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[
         Dot[
          
          Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`c], $CellContext`c], $CellContext`c],
      Editable->False]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.44267417613325*^9}]
}, Open  ]],

Cell["with", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RtCoset[G, H]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.442674178477*^9, 3.44267417875825*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\
\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\
\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[
         Dot[$CellContext`b, $CellContext`b], $CellContext`c], \
$CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"b", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\
\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\
\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[
         Dot[$CellContext`a, $CellContext`b], $CellContext`c], \
$CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"c", ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(c\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`c, $CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"e", ",", 
     InterpretationBox["\<\"\\!\\(c\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`c, $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`c],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`a, $CellContext`b], $CellContext`b],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\
\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\
\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[
         Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`a, $CellContext`b], $CellContext`c],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`b, $CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`a, $CellContext`c], $CellContext`c],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[$CellContext`b, $CellContext`b], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`c],
      
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\
\\\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(b\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(b\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[
         Dot[
          
          Dot[$CellContext`a, $CellContext`b], $CellContext`b], \
$CellContext`c], $CellContext`c], $CellContext`c],
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`a, $CellContext`b], $CellContext`c], $CellContext`c],
      
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`b, $CellContext`c], $CellContext`c], $CellContext`c],
      
      Editable->False]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(c\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`a, $CellContext`c], $CellContext`c], $CellContext`c],
      
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(b\\\
\\\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(c\\\\)\\\"\\) \[CenterDot] \\!\\(c\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       Dot[
        Dot[$CellContext`b, $CellContext`b], $CellContext`c], $CellContext`c],
      
      Editable->False]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442674179617625*^9}]
}, Open  ]],

Cell[TextData[{
 "we see that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is not a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Thus, a normal subgroup of a normal subgroup is not necessarily normal.  \
\n\nContrast this situation to the center of a group. Here, we found that the \
center ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "N", ")"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", even though ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "N", ")"}], TraditionalForm]]],
 " contains no information about ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Proposition 7.4 tells us that for any group which contains ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " as a normal subgroup, such as a semi-direct product of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " by another group, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "N", ")"}], TraditionalForm]]],
 " will be a normal subgroup. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43686829684375*^9, 3.436868296953125*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Normalizer and Generated Normal Subgroups", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In the last section, we found a subgroup of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " that was not only normal, but also was normal in any group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " was a normal subgroup. In this section, we will essentially turn the \
question around: Given a subgroup ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", can we find a subgroup ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " lies inside of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " as a normal subgroup?\n\nWe answer this question by defining the ",
 StyleBox["normalizer",
  FontSlant->"Italic"],
 " of a group ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nDEFINITION 7.2\nLet ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " be a ",
 StyleBox["subset",
  FontSlant->"Italic"],
 " of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". We define the ",
 StyleBox["normalizer of",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " ",
 StyleBox["by",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", "S", ")"}], TraditionalForm]]],
 ", to be the set\n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["N", "G"], "(", "S", ")"}], " ", "=", " ", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"g", " ", "\[Element]", " ", "G"}], "  ", "|", 
       RowBox[{"g", "\[CenterDot]", "S", "\[CenterDot]", 
        SuperscriptBox["g", 
         RowBox[{"-", "1"}]]}]}], " ", "=", " ", "S"}], " ", "}"}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43686831440625*^9, 3.4368683325*^9}}],

Cell[TextData[{
 "Notice that this definition allows for ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " to be merely a ",
 StyleBox["subset",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", not necessarily a subgroup. We will later find uses for having a more \
generalized definition. For now, let us show that the normalizer has some of \
the properties that we are looking for.\n\nPROPOSITION 7.5\nLet ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " be a subset of the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["N", "G"],
     TraditionalForm], "(", "S", ")"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43686834521875*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", "S", ")"}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "S", "\[CenterDot]", 
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]]}], "=", " ", "S"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"y", "\[CenterDot]", "S", "\[CenterDot]", 
      SuperscriptBox["y", 
       RowBox[{"-", "1"}]]}],
     TraditionalForm], " ", "=", " ", "S"}], TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "S", "\[CenterDot]", "y"}]}], 
   TraditionalForm]]],
 ", and so\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{"   ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", "\[CenterDot]", 
        SuperscriptBox["y", 
         RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", "S", "\[CenterDot]", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "\[CenterDot]", 
         SuperscriptBox["y", 
          RowBox[{"-", "1"}]]}], ")"}], 
       RowBox[{"-", "1"}]]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["y", 
          RowBox[{"-", "1"}]], "\[CenterDot]", "S", "\[CenterDot]", "y"}], 
        ")"}], "\[CenterDot]", 
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", "S", "\[CenterDot]", 
        SuperscriptBox["x", 
         RowBox[{"-", "1"}]]}], " ", "=", " ", 
       RowBox[{"S", "."}]}]}]}]}], TraditionalForm]]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", "S", ")"}], TraditionalForm]]],
 ", and so by proposition 2.2, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", "S", ")"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436868375484375*^9, 3.436868376921875*^9}}]
}, Open  ]],

Cell[TextData[{
 "If, in addition, ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then the normalizer lives up to its name.\n\nPROPOSITION 7.6\nLet ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " be a subgroup of the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", "H", ")"}], TraditionalForm]]],
 " is the largest subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contains ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " as a normal subgroup."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436868383640625*^9, 3.436868384578125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First, we must check to see that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", "H", ")"}], TraditionalForm]]],
 ". But this is obvious, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "\[CenterDot]", "H", "\[CenterDot]", 
     SuperscriptBox["g", 
      RowBox[{"-", "1"}]]}], " ", "=", " ", "H"}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", "H", ")"}], TraditionalForm]]],
 ".\n\nNext, we must see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", "H", ")"}], TraditionalForm]]],
 " is the largest such group. Suppose that ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " is another subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contained ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " as a normal subgroup. Then \n\n                               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", "\[CenterDot]", "H", "\[CenterDot]", 
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]]}], " ", "=", " ", "H", " "}], TraditionalForm]]],
 "  for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "\[Epsilon]", " ", "Y"}], TraditionalForm]]],
 ".\nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Y", " ", "\[SubsetEqual]", " ", 
    RowBox[{
     SubscriptBox["N", "G"], "(", "H", ")"}]}], TraditionalForm]]],
 ".\n\nSince any subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contains ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " as a normal subgroup is itself contained in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["N", "G"], "(", "H", ")"}], ","}], TraditionalForm]]],
 " we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", "H", ")"}], TraditionalForm]]],
 " is the largest such group."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command for finding the normalizer ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", "H", ")"}], TraditionalForm]]],
 " of the set ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is given by:                         \n\n",
 StyleBox["Normalizer[G, H]",
  FontWeight->"Bold"],
 "\n\nLet us look at a few examples of the normalizer subgroup. Suppose we \
consider the quaternionic group ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[i^4,e]; Define[j^2,i^2]
Define[j.i,i.i.i.j]
Define[1/i,i^3]; Define[1/j,i.i.j]
Q = Group[{i,j}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436868434640625*^9, 3.436868438859375*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "i", ",", "j", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.43686885840625*^9, 3.44267419057075*^9}]
}, Open  ]],

Cell[TextData[{
 "Let begin by finding the normalizer of a single element of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", say the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "i", "}"}], TraditionalForm]]],
 ". In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " we would enter:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["H = Normalizer[Q,{i}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436868455734375*^9, 3.436868456328125*^9}, {
  3.436868553671875*^9, 3.43686856446875*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "i", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`i],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436868864015625*^9, 3.442674192367625*^9}]
}, Open  ]],

Cell[TextData[{
 "This seems to give us the group generated by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "i", "}"}], TraditionalForm]]],
 ". Since this is a subgroup of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", we could consider the normalizer of this subgroup by ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". We would get"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Normalizer[Q,H]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43686845884375*^9, 3.436868459296875*^9}, {
  3.436868556328125*^9, 3.436868562*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "i", ",", "j", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{{3.43686886165625*^9, 3.436868866125*^9}, 
   3.442674193602*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives us the entire group ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". Actually this fact could be reasoned without ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", for ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", so the largest subgroup of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " which contains ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " as a normal subgroup would be ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " itself.\n\nIn fact, whenever ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", it is clear from the definition that the normalizer of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " will be all of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". \n\nLet us try the normalizer of another element"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Normalizer[Q,{j}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{
  3.43686850334375*^9, {3.436868567296875*^9, 3.4368685981875*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "j", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`j],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4426741954145*^9}]
}, Open  ]],

Cell[TextData[{
 "This time we get the subgroup generated by ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ". What if we tried the subset containing both ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Normalizer[Q,{i,j}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4368685059375*^9, 3.43686850659375*^9}, {
  3.436868569671875*^9, 3.4368685964375*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.442674196992625*^9}]
}, Open  ]],

Cell[TextData[{
 "What happened this time? The normalizer doesn't contain either ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 "! When ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a ",
 StyleBox["subgroup",
  FontSlant->"Italic"],
 " we can say that the normalizer of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " will contain ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". In general though, all we can say is that the normalizer will be a \
subgroup, which this example illustrates.\n\nThere is one other case in which \
we can say that the normalizer will contain ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". Notice that in the two examples we did where ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " was a single element, the normalizer contained that element. In fact, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    RowBox[{"{", "a", "}"}], ")"}], TraditionalForm]]],
 " will consist of all elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which commute with ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ". It should be noted that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    RowBox[{"{", "a", "}"}], ")"}], TraditionalForm]]],
 " is not the same thing as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    RowBox[{"[", "a", "]"}], ")"}], TraditionalForm]]],
 ", the normalizer of the group generated by ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ". The former is the set of elements which commute with ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", and the latter is the largest subgroup which contains ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "a", "]"}], TraditionalForm]]],
 " as a normal subgroup.\n\nWe have seen that the normalizer of a subgroup ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " finds the largest subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contains ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " as a normal subgroup.  We could turn the question around and ask for the \
",
 StyleBox["smallest",
  FontSlant->"Italic"],
 " subgroup containing ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " which is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". In this case, it does not matter whether ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subgroup or just a subset, the answer is given in the following \
proposition.\n\nPROPOSITION 7.7\nLet ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " be a ",
 StyleBox["subset",
  FontSlant->"Italic"],
 " of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then the smallest group containing ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " which is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is given by \n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["N", 
     RowBox[{"*", " "}]], "=", " ", 
    RowBox[{
     StyleBox[
      UnderscriptBox[
       RowBox[{"  ", 
        SubscriptBox[
         StyleBox["\[Intersection]",
          FontSize->24], " "]}], 
       RowBox[{" ", 
        RowBox[{"N", " ", "\[Element]", " ", "L"}]}]],
      FontSize->18], 
     StyleBox[" ",
      FontSize->18], 
     StyleBox["N",
      FontSize->12]}]}], TraditionalForm]]],
 "\nwhere ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " denotes the collection of normal subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contain ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43686861525*^9, 3.436868648734375*^9}, {
  3.441585676890625*^9, 3.4415856956875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " itself is in the collection ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 ", so this collection is not empty. Thus, by proposition 2.3, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["N", "*"], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". \n\nAlso, since each ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " in the collection contained the set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", the intersection will also contain ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". All that needs to be shown is that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["N", "*"], TraditionalForm]]],
 " is normal.\n\nIf ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is an element of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["N", "*"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " in an element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then since each ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " would be in all of the groups ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ",\n\n                             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "\[CenterDot]", "n", "\[CenterDot]", 
     SuperscriptBox["g", 
      RowBox[{"-", "1"}]]}], " ", "\[Epsilon]", " ", "N"}], 
   TraditionalForm]]],
 "  for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", " ", "\[Element]", " ", "L"}], TraditionalForm]]],
 ".\n                  \nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "\[CenterDot]", "n", "\[CenterDot]", 
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in the intersection of all of the ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 "'s, which is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["N", "*"], TraditionalForm]]],
 ". Hence, by proposition 3.4, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["N", "*"], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43686868775*^9, 3.436868782875*^9}}]
}, Open  ]],

Cell[TextData[{
 "We will call this subgroup the ",
 StyleBox["normal closure of",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ".\n We can use the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command\n\n",
 StyleBox["NormalClosure[G, S]",
  FontWeight->"Bold"],
 "\n\nto compute this subgroup, thereby systematically find ",
 StyleBox["all",
  FontSlant->"Italic"],
 " of the normal subgroups of a given group.\n\nFor example, suppose we want \
to find all of the normal subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ". First, we load the group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436868793328125*^9, 3.436868833734375*^9}, {
  3.436868877640625*^9, 3.436868894375*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[e];
Define[a^2, e]
Define[b^3, e]
Define[b.a, a.b.b]
Define[1/a, a]
Define[1/b, b^2]
S3 = Group[{a, b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.442674205023875*^9, 3.44267420675825*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.43686891325*^9, 3.44267420763325*^9}]
}, Open  ]],

Cell[TextData[{
 "We will always have two trivial normal subgroups, namely the entire group \
and the group containing just the identity element. We would like to see if \
there are any other normal subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ". Since a proper subgroup must contain one of the elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"a", "\[CenterDot]", "b"}], ",", 
     SuperscriptBox["b", "2"], ",", 
     RowBox[{"a", "\[CenterDot]", 
      SuperscriptBox["b", "2"]}]}], "}"}], TraditionalForm]]],
 ", we have 5 groups to try. Let us see if we can find the smallest normal \
subgroup which contains the element ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["NormalClosure[S3,{a}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4368689075*^9, 3.436868933203125*^9}, {
  3.442674209867625*^9, 3.442674210742625*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{{3.436868915515625*^9, 3.4368689336875*^9}, 
   3.442674211648875*^9}]
}, Open  ]],

Cell[TextData[{
 "Since the whole group is the smallest normal subgroup containing ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", it is apparent that there are no non-trivial normal subgroups containing \
",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ". Let us move on to the next element."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["NormalClosure[S3,{b}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4368689776875*^9, 3.43686897959375*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436868980109375*^9, 3.44267421325825*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives us a non-trivial subgroup. In fact, this subgroup of order 3 is \
equivalent to the subgroup ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "3"], TraditionalForm]]],
 "  which we saw was a normal subgroup in corollary 5.1. Let us keep trying \
to find other normal subgroups."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436869042421875*^9, 3.436869043015625*^9}}],

Cell[CellGroupData[{

Cell["NormalClosure[S3,{a.b}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.4368691196875*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436869124015625*^9, 3.442674215086375*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["NormalClosure[S3,{b^2}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.436869121265625*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436869125734375*^9, 3.442674216102*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["NormalClosure[S3,{a.b.b}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.43686912265625*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436869127828125*^9, 3.442674217805125*^9}]
}, Open  ]],

Cell[TextData[{
 "We didn't find any new normal subgroups. In fact, we discovered that a \
non-trivial normal subgroup cannot contain the elements ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 ", or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", 
    SuperscriptBox["b", "2"]}], TraditionalForm]]],
 ". Of course the element ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " would have to be included in the subgroup, but if either of theother two \
elements ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", "2"], TraditionalForm]]],
 " were in the subgroup, both of them had to be in the subgroup. Therefore, \
the only non-trivial normal subgroup is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"e", ",", " ", "b", ",", " ", 
     SuperscriptBox["b", "2"]}], "}"}], TraditionalForm]]],
 ".\n\nWe have just used ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to ",
 StyleBox["prove ",
  FontSlant->"Italic"],
 "that the only non-trivial normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "3"], TraditionalForm]]],
 ". This method of exhausting all possibilities works well for small groups, \
but one can imagine that this method would be time consuming for larger \
groups. In the next section, we will find a shortcut so that we will not have \
to try every element of the group, but rather just a handful of elements. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.441585949046875*^9, 3.441585954234375*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Conjugacy Classes and Simple Groups", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In the last section, we introduced a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command ",
 StyleBox["NormalClosure[G, S]",
  FontWeight->"Bold"],
 " which found the smallest group containing the subset ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " which was a normal group of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Although we proved that such a group existed, we made no mention as to \
how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " found this group. \n\nThe idea behind the function ",
 StyleBox["NormalClosure",
  FontWeight->"Bold"],
 " is really quite simple. We know that if ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is in this normal group, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "a", "\[CenterDot]", "g"}], 
   TraditionalForm]]],
 " must also be in the group for all ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Thus, if we can find the elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which can be expressed as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "a", "\[CenterDot]", "g"}], 
   TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ", we know many of the elements that must be in the normal group. Let us \
make the following definition:\n\nDEFINITION 7.3:\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group. We say that the element ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is ",
 StyleBox["conjugate",
  FontSlant->"Italic"],
 " to the element ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " if there exist an element ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " such that\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"u", " ", "=", " ", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "v", "\[CenterDot]", 
      RowBox[{"g", "."}]}]}]}], TraditionalForm]]],
 "\n\nNote that every element is conjugate to itself, for we can let ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " be the identity element. Also note that if ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is conjugate to ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " is also conjugate to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", ","}], TraditionalForm]]],
 " since \n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", " ", "=", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["g", 
        RowBox[{"-", "1"}]], ")"}], 
      RowBox[{"-", "1"}]], "\[CenterDot]", "u", "\[CenterDot]", 
     RowBox[{
      RowBox[{"(", 
       SuperscriptBox["g", 
        RowBox[{"-", "1"}]], ")"}], "."}]}]}], TraditionalForm]]],
 "\n\nFinally, let us suppose that ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is conjugate to ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 ", that is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["g", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "v", "\[CenterDot]", "g"}]}], 
   TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " in turn is conjugate to ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 ", that is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["h", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "w", "\[CenterDot]", "h"}]}], 
   TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Is ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " conjugate to ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 "? We have \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "v", "\[CenterDot]", "g"}], " ", 
     "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["h", 
          RowBox[{"-", "1"}]], "\[CenterDot]", "w", "\[CenterDot]", "h"}], 
        ")"}], "\[CenterDot]", "g"}], " ", "=", " ", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"h", "\[CenterDot]", "g"}], ")"}], 
        RowBox[{"-", "1"}]], "\[CenterDot]", "w", "\[CenterDot]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"h", "\[CenterDot]", "g"}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is then conjugate to ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 ".\n\nRecall that in section 3.4, we defined an equivalence relationship as \
a relationship having 3 properties:\n\n1. Evert element ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is equivalent to itself.\n\n2. If ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is equivalent to ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " is equivalent to ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 ".\n\n3. If ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is equivalent to ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " in turn is equivalent to ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is equivalent to ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436869156484375*^9, 3.43686950815625*^9}}],

Cell[TextData[{
 "These were called the reflexive, symmetric, and transitive properties. In \
section 3.4, we applied these properties to modular arithmetic, but we have \
just shown that the conjugate relationship between two elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " also has all three of these properties. Hence the conjugate relationship \
is an equivalence relationship.\n\nWhat does this mean?  It means that we can \
divide the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " into different ",
 StyleBox["conjugacy classes.",
  FontSlant->"Italic"],
 " Two elements are in the same conjugacy class if, and only if, they are \
conjugate to one another. In fact, if ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is an element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then the conjugacy class which contains ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is given by\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "u", "\[CenterDot]", "g"}], "  ", 
     "|", "  ", 
     RowBox[{"g", " ", "\[Element]", " ", "G"}]}], "}"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43686951584375*^9, 3.4368695433125*^9}}],

Cell[TextData[{
 "Moreover, no two of these conjugacy classes have any elements in common, \
and every element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is in one of the conjugacy classes.\n\nBefore we proceed with proving some \
properties of the conjugacy classes, let us explore some examples with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". This will allow us to see the usefulness of the conjugacy classes, as \
well as show us some patterns.\n\nThe ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command for finding all of the conjugacy classes of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is \n\n",
 StyleBox["ConjugacyClasses[G]",
  FontWeight->"Bold"],
 "\n\nFor example, let us find the conjugacy classes of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4."], TraditionalForm]]],
 "\nWe will use the cycle notation, noting that the group is generated by the \
cycles (12) and (234)."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4368695715*^9, 3.436869571671875*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitGroup[ C[] ];
S4 = Group[{C[1,2], C[2,3,4]}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.43686958746875*^9, 3.44267422644575*^9}]
}, Open  ]],

Cell["We can find the conjugacy classes by the command", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ConjugacyClasses[S4]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436869584390625*^9, 3.436869596265625*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"C", "[", "]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"3", ",", "4"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "4", ",", "3"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "3", ",", "2"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 2], 
       C[3, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 3], 
       C[2, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 4], 
       C[2, 3]],
      Editable->False]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.43686959040625*^9, 3.436869597296875*^9}, 
   3.4426742280395*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that the identity element is in a class by itself since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "e", "\[CenterDot]", "g"}], 
   TraditionalForm]]],
 " will always by ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ". But notice something interesting about the other four classes: one \
contains all of the transpositions, one contains all of the 3-cycles, one \
contains all of the 4-cycles, and the last conjugacy class contains the \
products of two disjoint transpositions. Let us give names to these 5 \
conjagacy classes:\n\n      ",
 Cell[BoxData[{
  FormBox[
   RowBox[{"A", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "  ", "2"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "3"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "4"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"2", "  ", "3"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"2", "  ", "4"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"3", "  ", "4"}], ")"}]}], "}"}]}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"B", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "  ", "2"}], ")"}], 
       RowBox[{"(", 
        RowBox[{"3", "  ", "4"}], ")"}]}], ",", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "  ", "3"}], ")"}], 
       RowBox[{"(", 
        RowBox[{"2", "  ", "4"}], ")"}]}], ",", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "  ", "4"}], ")"}], 
       RowBox[{"(", 
        RowBox[{"2", "  ", "3"}], ")"}]}]}], "}"}]}], 
   TraditionalForm], "\[IndentingNewLine]", 
  FormBox[
   RowBox[{"C", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "  ", "2", "  ", "3"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "2", "  ", "4"}], ")"}], ",", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "  ", "3", "  ", "2"}], ")"}], 
       RowBox[{"(", 
        RowBox[{"1", "  ", "3", "  ", "4"}], ")"}]}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "4", "  ", "2"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "4", "  ", "3"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"2", "  ", "3", "  ", "4"}], " ", ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"2", "  ", "4", "  ", "3"}], ")"}]}], "}"}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"D", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "  ", "2", "  ", "3", "  ", "4"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "2", "  ", "4", "  ", "3"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "3", "  ", "2", "  ", "4"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "3", "  ", "4", "  ", "2"}], ")"}], ",", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "4", "  ", "2", "  ", "3"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "4", "  ", "3", "  ", "2"}], ")"}]}], " ", "}"}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"E", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{"(", " ", ")"}], "}"}]}], TraditionalForm]}]],
 "\n\nWhat is the usefulness of the conjugacy classes? We know that whenever \
one element of a conjugacy class is in a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", the entire conjugacy class must be in the normal subgroup. Thus, in order \
to find ",
 StyleBox["all",
  FontSlant->"Italic"],
 " normal subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 " we only have to try the different combinations of the conjugacy classes. \
Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{"(", " ", ")"}], "}"}]}], TraditionalForm]]],
 " is guaranteed to be in every subgroup, this means that any non-trivial \
normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 " must be one of the 14 possible groups:\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"  ", "1", ")"}], "  ", "E"}], " ", "\[Union]", 
                    " ", "A"}], " ", "\n", "  ", "2"}], ")"}], "  ", "E"}], 
                    " ", "\[Union]", " ", "B"}], "\n", "  ", "3"}], ")"}], "  ",
                     "E"}], " ", "\[Union]", " ", "C"}], "\n", "  ", "4"}], 
                    ")"}], "  ", "E"}], " ", "\[Union]", " ", "D"}], "\n", 
                    "  ", "5"}], ")"}], "  ", "E"}], " ", "\[Union]", " ", 
                    "A", " ", "\[Union]", " ", "B"}], "\n", "  ", "6"}], 
                    ")"}], "  ", "E"}], " ", "\[Union]", " ", "A", " ", 
                    "\[Union]", " ", "C"}], "\n", "  ", "7"}], ")"}], "  ", 
                    "E"}], " ", "\[Union]", " ", "A", " ", "\[Union]", " ", 
                    "D"}], "\n", "  ", "8"}], ")"}], "  ", "E"}], " ", 
                    "\[Union]", " ", "B", " ", "\[Union]", " ", "C"}], "\n", 
                    "  ", "9"}], ")"}], "  ", "E"}], " ", "\[Union]", " ", 
                    "B", " ", "\[Union]", " ", "D"}], "\n", "10"}], ")"}], "  ",
                     "E"}], " ", "\[Union]", " ", "C", " ", "\[Union]", " ", 
                    "D"}], "\n", "11"}], ")"}], "  ", "E"}], " ", "\[Union]", 
                " ", "A", " ", "\[Union]", " ", "B", " ", "\[Union]", " ", 
                "C"}], "\n", "12"}], ")"}], "  ", "E"}], " ", "\[Union]", " ",
             "A", " ", "\[Union]", " ", "B", " ", "\[Union]", " ", "D"}], 
           "\n", "13"}], ")"}], "  ", "E"}], " ", "\[Union]", " ", "A", " ", 
        "\[Union]", " ", "C", " ", "\[Union]", " ", "D"}], "\n", "14"}], 
      ")"}], "  ", "E"}], " ", "\[Union]", " ", "B", " ", "\[Union]", " ", 
    "C", " ", "\[Union]", " ", "D"}], TraditionalForm]]],
 "\n\nThe 15th combination, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "E", " ", "\[Union]", " ", "A", " ", "\[Union]", " ", "B", " ", 
     "\[Union]", " ", "C", " ", "\[Union]", " ", "D"}], ","}], 
   TraditionalForm]]],
 " obviously would give us the whole group. As we try some of these \
combinations to find proper normal subgroups, we will be able to eliminate \
other combinations. \n\nWith ",
 StyleBox["Mathematica'",
  FontSlant->"Italic"],
 "s ",
 StyleBox["NormalClosure",
  FontWeight->"Bold"],
 " command, we can eliminate many of these combinations at once. For example, \
let us see if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "\[Union]", " ", "A"}], TraditionalForm]]],
 " is a normal subgroup, we can actually test at the same time whether ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " can be contained in ",
 StyleBox["any ",
  FontSlant->"Italic"],
 "non-trivial normal subgroup with the ",
 Cell[BoxData[
  FormBox["command", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4032697724518747`*^9, 3.4032698719987497`*^9}, {
  3.43686961584375*^9, 3.436869621640625*^9}, {3.436869702625*^9, 
  3.43686970509375*^9}, {3.441586692*^9, 3.441586753625*^9}}],

Cell[CellGroupData[{

Cell["NormalClosure[S4, {C[1,2]}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436869727859375*^9, 3.43686972971875*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.4368697303125*^9, 3.442674232555125*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives us the whole group. Thus, the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2"}], "]"}], TraditionalForm]]],
 " cannot be in any nontrivial normal subgroup. That eliminates not just the \
first combination, but 6 other combinations as well! Let us try using the \
combination ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "\[Union]", " ", "B"}], TraditionalForm]]],
 " instead:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["NormalClosure[S4, {C[1,2].C[3,4]}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436869759421875*^9, 3.436869760984375*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436869761609375*^9, 3.442674234352*^9}]
}, Open  ]],

Cell[TextData[{
 "This is a subgroup of order 4, which we have seen many times before. It is \
isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ".  Let us consider ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "\[Union]", " ", "C"}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.441586840921875*^9, 3.441586875421875*^9}}],

Cell[CellGroupData[{

Cell["NormalClosure[S4, {C[1,2,3]}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436869736515625*^9, 3.4368697383125*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436869739234375*^9, 3.442674236055125*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives us a proper subgroup, but in fact this is the subgroup ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "\[Union]", " ", "B", " ", "\[Union]", " ", "C"}], 
   TraditionalForm]]],
 ". If we look at the length:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4415868898125*^9, 3.441586899578125*^9}}],

Cell[CellGroupData[{

Cell["Length[%]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["12"], "Output",
 CellChangeTimes->{3.436869745125*^9, 3.44267423825825*^9}]
}, Open  ]],

Cell[TextData[{
 "We see that this group is of order 12. In fact, since all of the elements \
are even permutations, this must be the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " which we knew was normal. In fact, proposition 5.3.5 would tell us that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " is the subgroup generated by the 3-cycles.  Let us go on and try the last \
combination ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "\[Union]", " ", "D"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.44158691965625*^9, 3.441586933359375*^9}}],

Cell[CellGroupData[{

Cell["NormalClosure[S4, {C[1,2,3,4]}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436869748140625*^9, 3.436869751375*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436869751921875*^9, 3.44267423994575*^9}]
}, Open  ]],

Cell["\<\
This is the whole group again, so no non-trivial normal group can contain the \
element C[1,2,3,4]. \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.44158694640625*^9}],

Cell[TextData[{
 "Since we know that a proper normal subgroup cannot contain either ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["D", TraditionalForm]]],
 ", and also ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "\[Union]", " ", "C"}], TraditionalForm]]],
 " is not a normal subgroup, there can only be 2 non-trivial\nnormal \
subgroups: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"E", " ", "\[Union]", " ", "B", " ", "\[Union]", " ", "C"}], " ", 
    "=", " ", 
    SubscriptBox["A", "4"]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"E", " ", "\[Union]", " ", "B"}], " ", "\[TildeTilde]", " ", 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 ".\n\nWhat if were not able to have ",
 StyleBox["Mathematica ",
  FontSlant->"Italic"],
 "help us determine which combinations were normal subgroups? Note that \
almost all of the 14 combinations listed could be eliminated simply by ",
 StyleBox["counting",
  FontSlant->"Italic"],
 ". For example, here are the number of elements in these 14 sets.\n\n 1)  E \
\[Union] A          : 7 elements\n 2)  E \[Union] B          : 9 elements\n \
3)  E \[Union] C          : 7 elements\n 4)  E \[Union] D          : 4 \
elements\n 5)  E \[Union] A \[Union] B      : 15 elements\n 6)  E \[Union] A \
\[Union] C      : 13 elements\n 7)  E \[Union] A \[Union] D      : 10 \
elements\n 8)  E \[Union] B \[Union] C      : 15 elements\n 9)  E \[Union] B \
\[Union] D      : 12 elements\n10)  E \[Union] C \[Union] D      : 10 \
elements\n11)  E \[Union] A \[Union] B \[Union] C  : 21 elements\n12)  E \
\[Union] A \[Union] B \[Union] D  : 18 elements\n13)  E \[Union] A \[Union] C \
\[Union] D  : 16 elements\n14)  E \[Union] B \[Union] C \[Union] D  : 18 \
elements\n\nHowever, by Lagrange's theorem, the number of elements is a \
subgroup must divide the order of the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "4"], ","}], TraditionalForm]]],
 " which is 24. Only the fourth and ninth combinations has the number of \
elements divide 24. Thus, there can be at most 2 non-trivial normal subgroups \
of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ": ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "\[Union]", " ", "D"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "\[Union]", " ", "B", " ", "\[Union]", " ", "D"}], 
   TraditionalForm]]],
 ". Of course we would still need to verify that these are indeed normal \
subgroups."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.44158695709375*^9, 3.441586984734375*^9}}],

Cell[TextData[{
 "Since we have found all normal subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ", let us move on to even larger groups. The next group that comes to mind \
is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "5"], TraditionalForm]]],
 ", which is a group of order 120. However, we will find it easier to find \
all of the normal subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 ". After all, any normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "5"], TraditionalForm]]],
 ", when intersected with ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 ", gives a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 ". Thus, if we can find all of the normal subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 ", we will be on our way to finding all normal subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "5"], TraditionalForm]]],
 ".                                                   \n\nSince the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " is generated by the two cycles ",
 StyleBox["C[1,2,3]",
  FontWeight->"Bold"],
 " and ",
 StyleBox["C[3,4,5]",
  FontWeight->"Bold"],
 ", we have"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitGroup[ C[] ];
A5 = Group[{C[1,2,3], C[3,4,5]}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[2, 3]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 3], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 4], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 5], 
     C[3, 4]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436869807484375*^9, 3.44267424557075*^9}]
}, Open  ]],

Cell["The length of this group is", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[A5]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["60"], "Output",
 CellChangeTimes->{3.43686980959375*^9, 3.442674251102*^9}]
}, Open  ]],

Cell["\<\
which is half of 5! = 120. The conjugacy classes are as follows:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ConjugacyClasses[A5]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43686981315625*^9, 3.43686981334375*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"C", "[", "]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "3", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "4", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "5", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "5", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"3", ",", "4", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"3", ",", "5", ",", "4"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "4", ",", "5", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "5", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "2", ",", "5", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "4", ",", "2", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "5", ",", "4", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "2", ",", "3", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "3", ",", "5", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "5", ",", "2", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "4", ",", "3", ",", "2"}], "]"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "3", ",", "5", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "4", ",", "3", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "5", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "2", ",", "4", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "4", ",", "5", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "5", ",", "2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "2", ",", "5", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "3", ",", "2", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "5", ",", "3", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "4", ",", "2", ",", "3"}], "]"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 2], 
       C[3, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 2], 
       C[3, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 2], 
       C[4, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 3], 
       C[2, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 3], 
       C[2, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 3], 
       C[4, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 4], 
       C[2, 3]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 4], 
       C[2, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 4], 
       C[3, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 5], 
       C[2, 3]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 5], 
       C[2, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 5], 
       C[3, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(2, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[2, 3], 
       C[4, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(2, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[2, 4], 
       C[3, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(2, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[2, 5], 
       C[3, 4]],
      Editable->False]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4368698156875*^9, 3.442674253586375*^9}]
}, Open  ]],

Cell[TextData[{
 "Looking at the list we see only 5 conjugacy classes with one of the 5 \
containing just the identity. Thus, finding all of the normal subgroups is no \
more difficult for this group than for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ". We can pick the representatives\n\n",
 StyleBox["C[1,2,3],",
  FontWeight->"Bold"],
 "\n",
 StyleBox["C[1,2,3,4,5],",
  FontWeight->"Bold"],
 "\n",
 StyleBox["C[1,2,3,5,4]",
  FontWeight->"Bold"],
 ", and\n",
 StyleBox["C[1,2].C[3,4]",
  FontWeight->"Bold"],
 ".\n\nLet us find the normal group generated by the 3-cycles."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["NormalClosure[A5,{C[1,2,3]}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436869822625*^9, 3.436869825359375*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[2, 3]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 3], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 4], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 5], 
     C[3, 4]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436869826171875*^9, 3.442674256898875*^9}]
}, Open  ]],

Cell["The length of this is", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[%]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["60"], "Output",
 CellChangeTimes->{3.436869827875*^9, 3.442674258398875*^9}]
}, Open  ]],

Cell[TextData[{
 "So this gives the whole group. This should come as no surprise since we \
have shown in proposition 5.3.5 that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " was generated by the set of 3-cycles. Next let us try one of the \
5-cycles."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["NormalClosure[A5,{C[1,2,3,4,5]}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436869831890625*^9, 3.4368698334375*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[2, 3]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 3], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 4], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 5], 
     C[3, 4]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.43686983434375*^9, 3.442674260023875*^9}]
}, Open  ]],

Cell["The length of this is", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[%]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["60"], "Output",
 CellChangeTimes->{3.43686983603125*^9, 3.4426742614145*^9}]
}, Open  ]],

Cell["\<\
So again we get the whole group. If we used the other 5-cycle we get\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["NormalClosure[A5,{C[1,2,3,5,4]}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43686983996875*^9, 3.43686984121875*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[2, 3]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 3], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 4], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 5], 
     C[3, 4]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.43686984209375*^9, 3.442674263086375*^9}]
}, Open  ]],

Cell["The length of this one is", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[%]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["60"], "Output",
 CellChangeTimes->{3.436869843859375*^9, 3.4426742645395*^9}]
}, Open  ]],

Cell[TextData[{
 "So this produces the same thing. So we have found that a non-trivial normal \
subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " cannot contain a 3-cycle or a 5-cycle. There's only one more conjugacy \
class to try.\n\nEXPERIMENT:\nFind the normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " generated by the element ",
 StyleBox["C[1,2]\[CenterDot]C[3,4]",
  FontWeight->"Bold"],
 ". Does this also give the whole group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["      ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "This experiment demonstrates that the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " does  \nnot contain ",
 StyleBox["any",
  FontSlant->"Italic"],
 " proper normal subgroups. (See problem 7.19 for a non-computerized way to \
prove this.)  We will see that this is a rather unusual property for a group \
to have, so we will give this a special name.\n\nDEFINITION 7.4:\nA group is \
said to be ",
 StyleBox["simple",
  FontSlant->"Italic"],
 " if it contains no normal subgroups besides itself, and the identity \
subgroup.\n\nWhat are some examples of simple groups? Suppose that the group \
is of prime order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Then by Lagrange's theorem, the order of any subgroup must divide ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", so the only two subgroups are the group itself and the trivial subgroup. \
We noticed in corollary 3.3 that this meant that the group was the cyclic \
group, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "p"], TraditionalForm]]],
 ".\n\nWe now have seen an example of a non-cyclic simple group, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 ". In fact this is the ",
 StyleBox["smallest",
  FontSlant->"Italic"],
 " non-cyclic simple group!\n\nLet us see if there are any other simple \
groups. The natural place to look is higher order alternating groups, so let \
us begin there. We will begin by showing that one of the conjugacy classes of \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " consists of all 3-cycles.\n\nLEMMA 7.1\nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4"}], TraditionalForm]]],
 ", Any two 3-cycles are conjugate in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ". Furthermore, the conjugate of a 3-cycle is again a 3-cycle. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43686987325*^9, 3.43686991503125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We begin by showing that the conjugate of a 3-cycle is again a 3-cycle. Let \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", " ", "b", " ", "c"}], ")"}], TraditionalForm]]],
 " be a 3-cycle, and let ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " be any permutation in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ". Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{"\[Phi]", "(", "a", ")"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      RowBox[{"y", " ", "=", " ", 
       RowBox[{"\[Phi]", "(", "b", ")"}]}],
      TraditionalForm], ","}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    FormBox[
     RowBox[{"z", " ", "=", " ", 
      RowBox[{"\[Phi]", "(", "c", ")"}]}],
     TraditionalForm]}], TraditionalForm]]],
 ". Then we can compute\n                  \n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Phi]", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"a", " ", "b", " ", "c"}], ")"}], "\[CenterDot]", "\[Phi]"}], 
    " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "y", " ", "z"}], ")"}], "."}]}], TraditionalForm]]],
 "\n\nThus the conjugate of a 3-cycle is another 3-cycle.\n\nNext we will \
show that any 3-cycle is conjugate to the element (1 2 3)  in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ". Let",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"(", 
     RowBox[{"u", " ", "v", " ", "w"}], ")"}]}], TraditionalForm]]],
 " be a 3-cycle. Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4"}], TraditionalForm]]],
 " there must be at least 2 numbers not mentioned in this 3-cycle, so we will \
call two of them ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ". Consider the permutation\n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", " ", "=", " ", 
    RowBox[{"(", GridBox[{
       {"1", "2", "3", "4", "5", "\[CenterEllipsis]"},
       {"u", "v", "w", "x", "y", "\[CenterEllipsis]"}
      }], ")"}]}], TraditionalForm]]],
 ".\n                          \t\nHere, the dots indicate that when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "5"}], TraditionalForm]]],
 ", we can complete the permutation in any way so that the numbers on the \
bottom row will be a permutation of the numbers 1 through ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".\n\nNow ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " will either be an even permutation or an odd permutation. If ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is an odd permutation, we can consider instead the permutation\n\n         \
                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", " ", "=", " ", 
    RowBox[{"(", GridBox[{
       {"1", "2", "3", "4", "5", "\[CenterEllipsis]"},
       {"u", "v", "w", "y", "x", "\[CenterEllipsis]"}
      }], ")"}]}], TraditionalForm]]],
 ".\t\n                           \nSo we may assume that ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is an even permutation. Thus ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ", and we can compute\n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Phi]", 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"1", "  ", "2", "  ", "3"}], ")"}], "\[CenterDot]", "\[Phi]"}],
      " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"u", " ", "v", " ", "w"}], ")"}], "."}]}]}], 
   TraditionalForm]]],
 "\n\nTherefore, any 3-cycle is conjugate to (1 2 3) , and so any two \
3-cycles are \nconjugate to each other in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " whenever ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.42265171328125*^9, 3.422651734265625*^9}, {
  3.422651771109375*^9, 3.422651814578125*^9}, {3.4226518465*^9, 
  3.42265188728125*^9}, {3.43686997371875*^9, 3.43687003875*^9}, {
  3.436870295109375*^9, 3.4368703504375*^9}}]
}, Open  ]],

Cell[TextData[{
 "With this lemma, we can show that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " will be a simple group whenever ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4"}], TraditionalForm]]],
 ". This was originally proved by Abel using a long case-by-case argument. \
Since ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has already shown that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " is simple, most of the cases can be taken care of at once.\n\nTHEOREM 7.1: \
 ABEL'S THEOREM\nThe alternating group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " is simple for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436870387484375*^9, 3.43687038871875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a proper normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " be an element of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " besides the identity. By proposition 7.2, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " is centerless. Since proposition 5.1 tells us that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " is generated by 3-cycles, there must be at least one 3-cycle that does not \
commute with ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 ", say ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", " ", "b", " ", "c"}], ")"}], TraditionalForm]]],
 ". Thus,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{"a", " ", "b", " ", "c"}], ")"}]}], TraditionalForm]]],
 " is not equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", "b", " ", "c"}], ")"}], "\[CenterDot]", "\[Phi]"}], 
   TraditionalForm]]],
 ", or equivalently, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", "b", " ", "c"}], ")"}], "\[CenterDot]", "\[Phi]", 
    "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{"a", " ", "c", " ", "b"}], ")"}], "\[CenterDot]", 
    SuperscriptBox["\[Phi]", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is not the identity element.\n\nSince ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a normal subgroup, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", "b", " ", "c"}], ")"}], "\[CenterDot]", "\[Phi]", 
    "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{"a", " ", "c", " ", "b"}], ")"}]}], TraditionalForm]]],
 " must be in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", "b", " ", "c"}], ")"}], "\[CenterDot]", "\[Phi]", 
    "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{"a", " ", "c", " ", "b"}], ")"}], "\[CenterDot]", 
    SuperscriptBox["\[Phi]", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " must also be in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{"a", " ", "c", " ", "b"}], ")"}], "\[CenterDot]", 
    SuperscriptBox["\[Phi]", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is the conjugate of a 3-cycle, so by lemma 7.1 this is also a 3-cycle, say \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", " ", "y", " ", "z"}], ")"}], TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " contains a product of two 3-cycles, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", "b", " ", "c"}], ")"}], "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{"x", " ", "y", " ", "z"}], ")"}]}], TraditionalForm]]],
 " which is not the identity.\n\nSuppose that the cycles ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", " ", "b", " ", "c"}], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", " ", "y", " ", "z"}], ")"}], TraditionalForm]]],
 " are ",
 StyleBox["disjoint",
  FontSlant->"Italic"],
 " 3-cycles. Then we can conjugate the product by the 3-cycle ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"c", " ", "z", " ", "y"}], ")"}], TraditionalForm]]],
 " to get another element in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ":\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"c", " ", "y", " ", "z"}], ")"}], "\[CenterDot]", 
      RowBox[{"[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"a", " ", "b", " ", "c"}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"x", " ", "y", " ", "z"}], ")"}]}], "]"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"c", " ", "z", " ", "y"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", " ", "b", " ", "z"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"c", " ", "y", " ", "x"}], ")"}]}]}]}], TraditionalForm]]],
 "\n\nWe now have two elements of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " that consist of two disjoint 3-cycles. If we multiply these two elements \
together we get\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", " ", "b", " ", "z"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"c", " ", "y", " ", "x"}], ")"}]}], "]"}], " ", 
     "\[CenterDot]", " ", 
     RowBox[{"[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", " ", "b", " ", "c"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"x", " ", "y", " ", "z"}], ")"}]}], "]"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", "c", " ", "z", " ", "b", " ", "x"}], ")"}], " ", "=", 
     " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", " ", "c", " ", "x"}], ")"}], "\[CenterDot]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "z", " ", "b"}], ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\n\nThis must also be in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". Thus ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " contains a product of two 3-cycles which are ",
 StyleBox["not ",
  FontSlant->"Italic"],
 "disjoint. In essence we can say that there is a non-identity element of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " which moves at most 5 numbers, labeled ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ".\n\nNow, consider the subgroup ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " consisting of all even permutations of the 5 numbers ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ". We have just showed that there is a nontrivial intersection of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". Let this intersection be ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 ". Whenever ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " \nis in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", then\n                                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"h", "\[CenterDot]", "x", "\[CenterDot]", 
     SuperscriptBox["h", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 "\n\nis in both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". Thus ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "x", "\[CenterDot]", 
    SuperscriptBox["h", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 ". Hence ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " is a nontrivial normal subgroup of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ".\n\nBut ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " which we have proven using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " or GAP to be a simple group. Thus ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " must be all of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". In particular ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " contains a 3-cycle, and so ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " contains a 3-cycle. By lemma 7.1 all 3-cycles of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " are conjugate, so ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " contains all 3-cycles of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ". Finally, by proposition 5.1 the 3-cycles generate ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " must be all of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ".  Therefore, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " is simple whenever ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4"}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.422652038953125*^9, 3.42265208421875*^9}, {
  3.42265212021875*^9, 3.4226522641875*^9}, {3.422652295703125*^9, 
  3.422652297859375*^9}, {3.436870739546875*^9, 3.43687075959375*^9}, {
  3.43687081040625*^9, 3.4368708105625*^9}, {3.436924844375*^9, 
  3.436924846734375*^9}, {3.43692528634375*^9, 3.4369253123125*^9}, {
  3.441587765125*^9, 3.44158778278125*^9}}]
}, Open  ]],

Cell[TextData[{
 "This theorem has an immediate application to the permutation groups ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 ".\n\nCOROLLARY 7.1\nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4"}], TraditionalForm]]],
 " then the only proper normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43692532353125*^9, 3.43692533403125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that there were another normal subgroup, ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". Then the intersection of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " would be another normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 ", and so would be a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " is simple for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4"}], TraditionalForm]]],
 ", this intersection must \neither by the identity or all of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ".\n\nSuppose that the intersection is all of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " contains ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ", and if ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is not equal to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " would contain more than half of the elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 ". But this would contradict Lagrange's theorem (3.1) unless ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", " ", "=", " ", 
    FormBox[
     SubscriptBox["S", "n"],
     TraditionalForm]}], TraditionalForm]]],
 ".\n\nSuppose that the intersection of ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " is just the identity element. Then since both ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " are normal subgroups, we have by corollary 6.1,\n\n                        \
                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "\[CenterDot]", 
     SubscriptBox["A", "n"]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"N", " ", "\[Times]", " ", 
     SubscriptBox["A", "n"]}]}], TraditionalForm]]],
 ".\n                                         \nIf ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is not just the identity element, this quickly leads to a contradiction, \
for ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " could have order of at most 2, telling us that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 " was isomorphic to ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubscriptBox["Z", "2"], " ", "\[Times]", 
     SubscriptBox["A", "n"]}]}], TraditionalForm]]],
 ". But this is ridiculous, for we saw in proposition 7.2 that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 " had a trivial center, whereas ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["A", "n"]}], TraditionalForm]]],
 "  has both ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"0", ",", 
      RowBox[{"(", " ", ")"}]}], ")"}], " "}], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"1", ",", 
     RowBox[{"(", " ", ")"}]}], ")"}], TraditionalForm]]],
 " in its center. Therefore, the only normal subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4"}], TraditionalForm]]],
 " are ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 " itself, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ", and the identity element."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43692536125*^9, 3.4369254169375*^9}}]
}, Open  ]],

Cell[TextData[{
 "We now have found 2 sequences of simple groups, namely ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "p"], ","}], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " being a prime number, and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4"}], TraditionalForm]]],
 ".  Are any of the other groups that we have looked at simple groups? \
Surprisingly, there is another group that we have seen which is simple, and \
we will now prove this. Remember the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"Aut", "(", 
      SubsuperscriptBox["Z", "24", "*"]}],
     TraditionalForm], ")"}], TraditionalForm]]],
 "? It was a group of order 168 generated by the 149th and 735rd permutation \
elements. Here it is again:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitPermMultiplication
A = Group[{149,735}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "27", ",", "61", ",", "87", ",", "122", ",", "149", ",", "187", 
   ",", "231", ",", "244", ",", "270", ",", "331", ",", "357", ",", "374", 
   ",", "404", ",", "437", ",", "467", ",", "496", ",", "548", ",", "558", 
   ",", "593", ",", "640", ",", "670", ",", "684", ",", "714", ",", "735", 
   ",", "753", ",", "775", ",", "793", ",", "856", ",", "908", ",", "918", 
   ",", "953", ",", "970", ",", "1014", ",", "1052", ",", "1079", ",", "1088",
    ",", "1123", ",", "1133", ",", "1185", ",", "1202", ",", "1229", ",", 
   "1267", ",", "1311", ",", "1362", ",", "1392", ",", "1402", ",", "1432", 
   ",", "1461", ",", "1475", ",", "1496", ",", "1537", ",", "1582", ",", 
   "1622", ",", "1649", ",", "1662", ",", "1692", ",", "1732", ",", "1775", 
   ",", "1796", ",", "1807", ",", "1837", ",", "1851", ",", "1881", ",", 
   "1962", ",", "1992", ",", "2002", ",", "2032", ",", "2042", ",", "2069", 
   ",", "2107", ",", "2151", ",", "2183", ",", "2201", ",", "2240", ",", 
   "2258", ",", "2304", ",", "2345", ",", "2366", ",", "2380", ",", "2418", 
   ",", "2471", ",", "2476", ",", "2510", ",", "2562", ",", "2592", ",", 
   "2602", ",", "2632", ",", "2647", ",", "2677", ",", "2691", ",", "2721", 
   ",", "2761", ",", "2787", ",", "2821", ",", "2847", ",", "2900", ",", 
   "2918", ",", "2963", ",", "2981", ",", "3019", ",", "3032", ",", "3059", 
   ",", "3099", ",", "3133", ",", "3151", ",", "3177", ",", "3195", ",", 
   "3276", ",", "3298", ",", "3336", ",", "3358", ",", "3372", ",", "3412", 
   ",", "3455", ",", "3476", ",", "3486", ",", "3508", ",", "3573", ",", 
   "3595", ",", "3622", ",", "3662", ",", "3689", ",", "3702", ",", "3741", 
   ",", "3755", ",", "3776", ",", "3817", ",", "3876", ",", "3898", ",", 
   "3936", ",", "3958", ",", "3973", ",", "3991", ",", "4017", ",", "4035", 
   ",", "4098", ",", "4151", ",", "4156", ",", "4190", ",", "4205", ",", 
   "4226", ",", "4269", ",", "4309", ",", "4366", ",", "4384", ",", "4410", 
   ",", "4428", ",", "4476", ",", "4498", ",", "4536", ",", "4558", ",", 
   "4581", ",", "4595", ",", "4616", ",", "4657", ",", "4695", ",", "4713", 
   ",", "4735", ",", "4753", ",", "4817", ",", "4847", ",", "4874", ",", 
   "4904", ",", "4931", ",", "4965", ",", "4970", ",", "5023"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.436925464953125*^9, 3.4426742794145*^9}]
}, Open  ]],

Cell["\<\
As large as this group is, we can still break this up into conjugacy classes \
in a fairly reasonable amount of time.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ConjugacyClasses[A]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.44267428988325*^9, 3.4426742900395*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "27", ",", "61", ",", "87", ",", "122", ",", "270", ",", "404", ",", 
     "593", ",", "640", ",", "714", ",", "735", ",", "775", ",", "1582", ",", 
     "1807", ",", "2380", ",", "2691", ",", "3032", ",", "3151", ",", "3755", 
     ",", "4017", ",", "4476", ",", "4498"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "149", ",", "187", ",", "244", ",", "357", ",", "374", ",", "467", ",", 
     "548", ",", "558", ",", "856", ",", "1014", ",", "1123", ",", "1311", 
     ",", "1362", ",", "1392", ",", "1402", ",", "1432", ",", "1461", ",", 
     "1622", ",", "1649", ",", "1775", ",", "1851", ",", "1881", ",", "2032", 
     ",", "2151", ",", "2258", ",", "2345", ",", "2366", ",", "2510", ",", 
     "2592", ",", "2647", ",", "2677", ",", "2821", ",", "2918", ",", "3019", 
     ",", "3099", ",", "3177", ",", "3195", ",", "3276", ",", "3412", ",", 
     "3508", ",", "3689", ",", "3741", ",", "3817", ",", "3898", ",", "3973", 
     ",", "3991", ",", "4098", ",", "4205", ",", "4366", ",", "4384", ",", 
     "4410", ",", "4428", ",", "4616", ",", "4713", ",", "4817", ",", 
     "4970"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "231", ",", "331", ",", "437", ",", "496", ",", "670", ",", "684", ",", 
     "753", ",", "793", ",", "908", ",", "1079", ",", "1088", ",", "1229", 
     ",", "1496", ",", "1662", ",", "1692", ",", "1837", ",", "1992", ",", 
     "2042", ",", "2201", ",", "2304", ",", "2476", ",", "2632", ",", "2721", 
     ",", "2787", ",", "2900", ",", "3059", ",", "3133", ",", "3298", ",", 
     "3476", ",", "3595", ",", "3702", ",", "3776", ",", "3876", ",", "4035", 
     ",", "4151", ",", "4269", ",", "4536", ",", "4558", ",", "4595", ",", 
     "4735", ",", "4874", ",", "4931"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "918", ",", "970", ",", "1185", ",", "1267", ",", "1475", ",", "1796", 
     ",", "2002", ",", "2069", ",", "2240", ",", "2471", ",", "2562", ",", 
     "2761", ",", "2981", ",", "3336", ",", "3372", ",", "3573", ",", "3622", 
     ",", "3958", ",", "4156", ",", "4309", ",", "4581", ",", "4753", ",", 
     "4904", ",", "4965"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "953", ",", "1052", ",", "1133", ",", "1202", ",", "1537", ",", "1732", 
     ",", "1962", ",", "2107", ",", "2183", ",", "2418", ",", "2602", ",", 
     "2847", ",", "2963", ",", "3358", ",", "3455", ",", "3486", ",", "3662", 
     ",", "3936", ",", "4190", ",", "4226", ",", "4657", ",", "4695", ",", 
     "4847", ",", "5023"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.436925469640625*^9, {3.442674280961375*^9, 3.442674292852*^9}}]
}, Open  ]],

Cell["\<\
So we have 6 conjugacy classes of this group, one of which is just the \
identity. The other 5 classes are represented by the 27th, 149th, 231st, \
918st, and 953th permutations. Thus, if we can show that a proper normal \
subgroup cannot contain any of these five elements, we have shown that there \
are no proper normal subgroups. (By the way, this next operation will be \
faster than the last two.)

Let us try to find a normal subgroup containing the 27th permutation.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["NormalClosure[A,{27}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43692549675*^9, 3.436925499390625*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "27", ",", "61", ",", "87", ",", "122", ",", "149", ",", "187", 
   ",", "231", ",", "244", ",", "270", ",", "331", ",", "357", ",", "374", 
   ",", "404", ",", "437", ",", "467", ",", "496", ",", "548", ",", "558", 
   ",", "593", ",", "640", ",", "670", ",", "684", ",", "714", ",", "735", 
   ",", "753", ",", "775", ",", "793", ",", "856", ",", "908", ",", "918", 
   ",", "953", ",", "970", ",", "1014", ",", "1052", ",", "1079", ",", "1088",
    ",", "1123", ",", "1133", ",", "1185", ",", "1202", ",", "1229", ",", 
   "1267", ",", "1311", ",", "1362", ",", "1392", ",", "1402", ",", "1432", 
   ",", "1461", ",", "1475", ",", "1496", ",", "1537", ",", "1582", ",", 
   "1622", ",", "1649", ",", "1662", ",", "1692", ",", "1732", ",", "1775", 
   ",", "1796", ",", "1807", ",", "1837", ",", "1851", ",", "1881", ",", 
   "1962", ",", "1992", ",", "2002", ",", "2032", ",", "2042", ",", "2069", 
   ",", "2107", ",", "2151", ",", "2183", ",", "2201", ",", "2240", ",", 
   "2258", ",", "2304", ",", "2345", ",", "2366", ",", "2380", ",", "2418", 
   ",", "2471", ",", "2476", ",", "2510", ",", "2562", ",", "2592", ",", 
   "2602", ",", "2632", ",", "2647", ",", "2677", ",", "2691", ",", "2721", 
   ",", "2761", ",", "2787", ",", "2821", ",", "2847", ",", "2900", ",", 
   "2918", ",", "2963", ",", "2981", ",", "3019", ",", "3032", ",", "3059", 
   ",", "3099", ",", "3133", ",", "3151", ",", "3177", ",", "3195", ",", 
   "3276", ",", "3298", ",", "3336", ",", "3358", ",", "3372", ",", "3412", 
   ",", "3455", ",", "3476", ",", "3486", ",", "3508", ",", "3573", ",", 
   "3595", ",", "3622", ",", "3662", ",", "3689", ",", "3702", ",", "3741", 
   ",", "3755", ",", "3776", ",", "3817", ",", "3876", ",", "3898", ",", 
   "3936", ",", "3958", ",", "3973", ",", "3991", ",", "4017", ",", "4035", 
   ",", "4098", ",", "4151", ",", "4156", ",", "4190", ",", "4205", ",", 
   "4226", ",", "4269", ",", "4309", ",", "4366", ",", "4384", ",", "4410", 
   ",", "4428", ",", "4476", ",", "4498", ",", "4536", ",", "4558", ",", 
   "4581", ",", "4595", ",", "4616", ",", "4657", ",", "4695", ",", "4713", 
   ",", "4735", ",", "4753", ",", "4817", ",", "4847", ",", "4874", ",", 
   "4904", ",", "4931", ",", "4965", ",", "4970", ",", "5023"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.436925501984375*^9, 3.44267431844575*^9}]
}, Open  ]],

Cell["The length of this is", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Length[%]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData["168"], "Output",
 CellChangeTimes->{3.436925506828125*^9, 3.442674320102*^9}]
}, Open  ]],

Cell["so this is the whole group.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
EXPERIMENT:

Replace the 27 above with 149, 231, 918, and 953, re-executing the command \
each time. Is the length of the normal subgroup generated 168 each time?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "This experiment shows that no proper normal subgroup can contain any of \
these 5 elements, and so there are no proper normal subgroups. Therefore, \
this group is simple.\n\nIt should be pointed out that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "24", "*"], ")"}], TraditionalForm]]],
 " is the second smallest non-cyclic simple group. ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    SubscriptBox["A", "5"]}], TraditionalForm]]],
 " is the smallest and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " is the third smallest.) In fact, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "24", "*"], ")"}], TraditionalForm]]],
 " is the beginning of yet another infinite family of simple groups, called \
the Chevalley groups. We will not go into all of the ways this group can be \
generalized to produce these other groups, but we will mention an important \
result that has taken place during the 20th century. It was once thought that \
",
 StyleBox["all",
  FontSlant->"Italic"],
 " finite simple groups were either the cyclic groups of prime order, the \
alternating groups, or one of the Chevalley or twisted Chevalley groups. (One \
of these groups turned out to be not quite simple.  Yet taking half of the \
elements forms a new simple group, just as we took half of the elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "n"], TraditionalForm]]],
 " to form the simple groups ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 ".)  But there were several other simple groups that were discovered, called \
",
 StyleBox["sporadic",
  FontSlant->"Italic"],
 " groups. In the 1960's and 1970's it was proved that there are exactly 26 \
sporadic groups, ranging in size from a mere 7,920 elements to the monstrous\n\
\n808,017,424,794,512,875,886,459,904,961,710,757,005,754,368,000,000,000\n\n\
elements! These 26 sporatic groups are listed in the Atlas of Finite Groups, \
edited by J. Conway et al. (Oxford University Press, 1984) Because these are \
the only sporatic groups, all finite simple groups have been found and \
classified. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.441587961484375*^9, 3.441587994671875*^9}, {
  3.441588628625*^9, 3.441588722640625*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Class Equation and Sylow's Theorems", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In the last section, we noticed some uses for the conjugacy classes of a \
group, namely finding all normal subgroups, or proving that a given group was \
simple. However, one may have noticed a pattern in the conjugacy classes. In \
particular, let us look at the ",
 StyleBox["size",
  FontSlant->"Italic"],
 " of each of the classes. Let us reload ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["S4 = Group[{C[1,2], C[2,3,4]}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.43692555865625*^9, 3.44267432700825*^9}]
}, Open  ]],

Cell["We found the conjugacy classes to be", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ConjugacyClasses[S4]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.442674330617625*^9, 3.442674330805125*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"C", "[", "]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"3", ",", "4"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "4", ",", "3"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "3", ",", "2"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 2], 
       C[3, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 3], 
       C[2, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 4], 
       C[2, 3]],
      Editable->False]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4369255605625*^9, 3.442674331930125*^9}]
}, Open  ]],

Cell[TextData[{
 "Now let us count the number of elements in each class. The first class has \
only the identity element, so there is only 1 element there. The class with \
the transpositions has exactly 6 elements, while the class with the 3-cycles \
has 8 elements. So immediately we see that the number of elements in the \
classes may be different. There are 6 elements in the class of 4-cycles, and \
finally there are 3 elements in the remaining conjugacy class.\n\nSo what is \
the pattern here? One thing we can notice is that \n\n              1 + 6 + 8 \
+ 6 + 3 = 24,\n\nwhich is the order of the group. But this is obviously going \
to be the case, since every element in the group belongs to one and only one \
conjugacy class. Is there another pattern? Notice that the number of elements \
in each class is always a ",
 StyleBox["divisor of the order of the group",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "EXPERIMENT:\n\nRecompute the conjugacy classes of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " with the following commands:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
A5 = Group[{C[1,2,3], C[3,4,5]}]
ConjugacyClasses[A5]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4426743396645*^9, 3.442674339836375*^9}},
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "5", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "5", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "5", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "5", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2", ",", "5"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "5", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "5", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "5", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[2, 3]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 5], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 3], 
     C[4, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 4], 
     C[3, 5]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(2, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[2, 5], 
     C[3, 4]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{{3.44267433613325*^9, 3.442674341023875*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"C", "[", "]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "3", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "4", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "5", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"2", ",", "5", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"3", ",", "4", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"3", ",", "5", ",", "4"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "4", ",", "5", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "5", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "2", ",", "5", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "4", ",", "2", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "5", ",", "4", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "2", ",", "3", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "3", ",", "5", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "5", ",", "2", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "4", ",", "3", ",", "2"}], "]"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "3", ",", "5", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "4", ",", "3", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "2", ",", "5", ",", "4", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "2", ",", "4", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "4", ",", "5", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "3", ",", "5", ",", "2", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "2", ",", "5", ",", "3"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "3", ",", "2", ",", "5"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "4", ",", "5", ",", "3", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
     RowBox[{"C", "[", 
      RowBox[{"1", ",", "5", ",", "4", ",", "2", ",", "3"}], "]"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 2], 
       C[3, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 2], 
       C[3, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 2], 
       C[4, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 3], 
       C[2, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 3], 
       C[2, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 3], 
       C[4, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 4], 
       C[2, 3]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 4], 
       C[2, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 4], 
       C[3, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 5], 
       C[2, 3]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 5], 
       C[2, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(1, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[1, 5], 
       C[3, 4]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(2, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(4, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[2, 3], 
       C[4, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(2, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 5\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[2, 4], 
       C[3, 5]],
      Editable->False], ",", 
     InterpretationBox["\<\"\\!\\(C[\\(\\(2, 5\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
      StringForm["`` \[CenterDot] ``", 
       C[2, 5], 
       C[3, 4]],
      Editable->False]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.44267433613325*^9, 3.442674341523875*^9}}]
}, Open  ]],

Cell["\<\
Is the size of each conjugacy class a divisor of 120? Is this a coincidence?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let us look back at the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "24", "*"]}], TraditionalForm]]],
 "). (We won't bother to recompute them here.) We had 6 conjugacy classes of \
size 1, 21, 56, 42, 24, and 24. We can double check that\n\n           1 + 21 \
+ 56 + 42 + 24 + 24 = 168.\n\nOnce again, we find that 21, 56, 42, and 24 are \
all divisors of 168.\n\nThis is no longer looking like a coincidence. Let's \
see if we can discover why this pattern exists.\n\nLEMMA 7.2\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a finite group, and let ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " be an element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then the number of elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which are conjugate to ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is given by\n                    ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    FractionBox[
     RowBox[{"|", "G", "|"}], 
     RowBox[{"|", 
      RowBox[{
       SubscriptBox["N", "G"], "(", 
       RowBox[{"{", "g", "}"}], ")"}], "|"}]],
    FontSize->16], TraditionalForm]]],
 ",\n                    \nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    RowBox[{"{", "g", "}"}], ")"}], TraditionalForm]]],
 " denotes the normalizer of the single element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "g", "}"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.436925575890625*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We saw in proposition 7.5 that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    RowBox[{"{", "g", "}"}], ")"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". We want to determine all possible conjugates of the element ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ". Note that if ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " are two elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "g", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "\[CenterDot]", "g", "\[CenterDot]", 
    SuperscriptBox["v", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " will represent the same element if, and only if,"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436925602828125*^9, 3.436925603203125*^9}}],

Cell[TextData[{
 "         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{
       RowBox[{"u", "\[CenterDot]", "g", "\[CenterDot]", 
        SuperscriptBox["u", 
         RowBox[{"-", "1"}]]}], " ", "=", " ", 
       RowBox[{"v", "\[CenterDot]", "g", "\[CenterDot]", 
        RowBox[{
         SuperscriptBox["v", 
          RowBox[{"-", "1"}]], "  ", "\[DoubleLongLeftRightArrow]", 
         "  "}]}]}],
      TraditionalForm], 
     RowBox[{
      SuperscriptBox["v", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "u", "\[CenterDot]", "g", 
      "\[CenterDot]", 
      SuperscriptBox["u", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "v"}]}], "=", "g"}], 
   TraditionalForm]]],
 "\n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[DoubleLongLeftRightArrow]", "     ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["v", 
         RowBox[{"-", "1"}]], "\[CenterDot]", "u"}], ")"}]}], "\[CenterDot]", 
     "g", "\[CenterDot]", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["v", 
         RowBox[{"-", "1"}]], "\[CenterDot]", "u"}], ")"}], 
      RowBox[{"-", "1"}]]}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "    ", "\[DoubleLongLeftRightArrow]", "    ", 
       SuperscriptBox["v", 
        RowBox[{"-", "1"}]]}], "\[CenterDot]", "u"}], " ", "\[Element]", " ", 
     
     RowBox[{
      SubscriptBox["N", "G"], "(", 
      RowBox[{"{", "g", "}"}], ")"}]}]}], TraditionalForm]]],
 "\n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[DoubleLongLeftRightArrow]", "     ", "u"}], " ", "\[Epsilon]",
      " ", 
     RowBox[{"v", "\[CenterDot]", 
      RowBox[{
       RowBox[{
        SubscriptBox["N", "G"], "(", 
        RowBox[{"{", "g", "}"}], ")"}], "     ", 
       "\[DoubleLongLeftRightArrow]", "     ", "u"}], "\[CenterDot]", 
      RowBox[{
       SubscriptBox["N", "G"], "(", 
       RowBox[{"{", "g", "}"}], ")"}]}]}], " ", "=", " ", 
    RowBox[{"v", "\[CenterDot]", 
     RowBox[{
      SubscriptBox["N", "G"], "(", 
      RowBox[{"{", "g", "}"}], ")"}]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43692575246875*^9, 3.436925752484375*^9}},
 FontFamily->"Courier New",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "Thus ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "g", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "\[CenterDot]", "g", "\[CenterDot]", 
    SuperscriptBox["v", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " represent the same element if, and only if, ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 "\nbelong to the same left coset of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    RowBox[{"{", "g", "}"}], ")"}], TraditionalForm]]],
 ".  Therefore, to count all of the possible\nconjugates of ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ", we merely count the number of left cosets of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    RowBox[{"{", "g", "}"}], ")"}], TraditionalForm]]],
 ", which is\n\n                                                  ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    FractionBox[
     RowBox[{"|", "G", "|"}], 
     RowBox[{"|", 
      RowBox[{
       SubscriptBox["N", "G"], "(", 
       RowBox[{"{", "g", "}"}], ")"}], "|"}]],
    FontSize->16], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "We have already observed that the sum of the number of elements in each of \
the conjugacy classes must give the number of elements in the group. Since we \
now know how many elements are in each conjugacy class, we can derive what is \
called the ",
 StyleBox["class equation",
  FontSlant->"Italic"],
 ".\n\nTHEOREM 7.2:  THE CLASS EQUATION THEOREM\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a finite group. Then\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"                                         ", 
    RowBox[{
     RowBox[{"|", "G", "|"}], "  ", "=", " ", 
     RowBox[{
      UnderscriptBox[
       SubscriptBox[
        StyleBox["\[Sum]",
         FontSize->24], " "], 
       RowBox[{"g", " "}]], 
      StyleBox[
       FractionBox[
        RowBox[{"|", "G", "|"}], 
        RowBox[{"|", 
         RowBox[{
          SubscriptBox["N", "G"], "(", 
          RowBox[{"{", "g", "}"}], ")"}], "|"}]],
       FontSize->16]}]}]}], TraditionalForm]], "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436925782078125*^9, 3.4369257821875*^9}}],

Cell[TextData[{
 "where the sum runs over one ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " from each conjugacy class."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We simply observe that every element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " appears in exactly one of the conjugacy classes. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "G", "|"}], TraditionalForm]]],
 " is the sum of the sizes of all of the conjugacy classes. We have by lemma \
7.2 that the size of each conjugacy class is \n                               \
                   ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    FractionBox[
     RowBox[{"|", "G", "|"}], 
     RowBox[{"|", 
      RowBox[{
       SubscriptBox["N", "G"], "(", 
       RowBox[{"{", "g", "}"}], ")"}], "|"}]],
    FontSize->16], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is a representative element of the conjugacy class. Thus we get the class \
equation."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436925798390625*^9, 3.436925799703125*^9}}]
}, Open  ]],

Cell[TextData[{
 "It is helpful to give an example of the class equation, to understand the \
notation. The group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "24", "*"], ")"}], TraditionalForm]]],
 " had 168 elements and 6 conjugacy classes."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitPermMultiplication
A = Group[{149,735}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "27", ",", "61", ",", "87", ",", "122", ",", "149", ",", "187", 
   ",", "231", ",", "244", ",", "270", ",", "331", ",", "357", ",", "374", 
   ",", "404", ",", "437", ",", "467", ",", "496", ",", "548", ",", "558", 
   ",", "593", ",", "640", ",", "670", ",", "684", ",", "714", ",", "735", 
   ",", "753", ",", "775", ",", "793", ",", "856", ",", "908", ",", "918", 
   ",", "953", ",", "970", ",", "1014", ",", "1052", ",", "1079", ",", "1088",
    ",", "1123", ",", "1133", ",", "1185", ",", "1202", ",", "1229", ",", 
   "1267", ",", "1311", ",", "1362", ",", "1392", ",", "1402", ",", "1432", 
   ",", "1461", ",", "1475", ",", "1496", ",", "1537", ",", "1582", ",", 
   "1622", ",", "1649", ",", "1662", ",", "1692", ",", "1732", ",", "1775", 
   ",", "1796", ",", "1807", ",", "1837", ",", "1851", ",", "1881", ",", 
   "1962", ",", "1992", ",", "2002", ",", "2032", ",", "2042", ",", "2069", 
   ",", "2107", ",", "2151", ",", "2183", ",", "2201", ",", "2240", ",", 
   "2258", ",", "2304", ",", "2345", ",", "2366", ",", "2380", ",", "2418", 
   ",", "2471", ",", "2476", ",", "2510", ",", "2562", ",", "2592", ",", 
   "2602", ",", "2632", ",", "2647", ",", "2677", ",", "2691", ",", "2721", 
   ",", "2761", ",", "2787", ",", "2821", ",", "2847", ",", "2900", ",", 
   "2918", ",", "2963", ",", "2981", ",", "3019", ",", "3032", ",", "3059", 
   ",", "3099", ",", "3133", ",", "3151", ",", "3177", ",", "3195", ",", 
   "3276", ",", "3298", ",", "3336", ",", "3358", ",", "3372", ",", "3412", 
   ",", "3455", ",", "3476", ",", "3486", ",", "3508", ",", "3573", ",", 
   "3595", ",", "3622", ",", "3662", ",", "3689", ",", "3702", ",", "3741", 
   ",", "3755", ",", "3776", ",", "3817", ",", "3876", ",", "3898", ",", 
   "3936", ",", "3958", ",", "3973", ",", "3991", ",", "4017", ",", "4035", 
   ",", "4098", ",", "4151", ",", "4156", ",", "4190", ",", "4205", ",", 
   "4226", ",", "4269", ",", "4309", ",", "4366", ",", "4384", ",", "4410", 
   ",", "4428", ",", "4476", ",", "4498", ",", "4536", ",", "4558", ",", 
   "4581", ",", "4595", ",", "4616", ",", "4657", ",", "4695", ",", "4713", 
   ",", "4735", ",", "4753", ",", "4817", ",", "4847", ",", "4874", ",", 
   "4904", ",", "4931", ",", "4965", ",", "4970", ",", "5023"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.44267435450825*^9}]
}, Open  ]],

Cell["\<\
The 6 conjugacy classes were represented by the elements 1, 27, 149, 231, \
918, and 953. The
normalizers of these 6 elements are as follows:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Normalizer", "[", 
  RowBox[{"A", ",", 
   RowBox[{"{", "1", "}"}]}], "]"}], "\n", 
 RowBox[{"Length", "[", "%", "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "27", ",", "61", ",", "87", ",", "122", ",", "149", ",", "187", 
   ",", "231", ",", "244", ",", "270", ",", "331", ",", "357", ",", "374", 
   ",", "404", ",", "437", ",", "467", ",", "496", ",", "548", ",", "558", 
   ",", "593", ",", "640", ",", "670", ",", "684", ",", "714", ",", "735", 
   ",", "753", ",", "775", ",", "793", ",", "856", ",", "908", ",", "918", 
   ",", "953", ",", "970", ",", "1014", ",", "1052", ",", "1079", ",", "1088",
    ",", "1123", ",", "1133", ",", "1185", ",", "1202", ",", "1229", ",", 
   "1267", ",", "1311", ",", "1362", ",", "1392", ",", "1402", ",", "1432", 
   ",", "1461", ",", "1475", ",", "1496", ",", "1537", ",", "1582", ",", 
   "1622", ",", "1649", ",", "1662", ",", "1692", ",", "1732", ",", "1775", 
   ",", "1796", ",", "1807", ",", "1837", ",", "1851", ",", "1881", ",", 
   "1962", ",", "1992", ",", "2002", ",", "2032", ",", "2042", ",", "2069", 
   ",", "2107", ",", "2151", ",", "2183", ",", "2201", ",", "2240", ",", 
   "2258", ",", "2304", ",", "2345", ",", "2366", ",", "2380", ",", "2418", 
   ",", "2471", ",", "2476", ",", "2510", ",", "2562", ",", "2592", ",", 
   "2602", ",", "2632", ",", "2647", ",", "2677", ",", "2691", ",", "2721", 
   ",", "2761", ",", "2787", ",", "2821", ",", "2847", ",", "2900", ",", 
   "2918", ",", "2963", ",", "2981", ",", "3019", ",", "3032", ",", "3059", 
   ",", "3099", ",", "3133", ",", "3151", ",", "3177", ",", "3195", ",", 
   "3276", ",", "3298", ",", "3336", ",", "3358", ",", "3372", ",", "3412", 
   ",", "3455", ",", "3476", ",", "3486", ",", "3508", ",", "3573", ",", 
   "3595", ",", "3622", ",", "3662", ",", "3689", ",", "3702", ",", "3741", 
   ",", "3755", ",", "3776", ",", "3817", ",", "3876", ",", "3898", ",", 
   "3936", ",", "3958", ",", "3973", ",", "3991", ",", "4017", ",", "4035", 
   ",", "4098", ",", "4151", ",", "4156", ",", "4190", ",", "4205", ",", 
   "4226", ",", "4269", ",", "4309", ",", "4366", ",", "4384", ",", "4410", 
   ",", "4428", ",", "4476", ",", "4498", ",", "4536", ",", "4558", ",", 
   "4581", ",", "4595", ",", "4616", ",", "4657", ",", "4695", ",", "4713", 
   ",", "4735", ",", "4753", ",", "4817", ",", "4847", ",", "4874", ",", 
   "4904", ",", "4931", ",", "4965", ",", "4970", ",", "5023"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.442674357477*^9}],

Cell[BoxData["168"], "Output",
 CellChangeTimes->{3.442674357477*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Normalizer", "[", 
  RowBox[{"A", ",", 
   RowBox[{"{", "27", "}"}]}], "]"}], "\n", 
 RowBox[{"Length", "[", "%", "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "27", ",", "61", ",", "87", ",", "4476", ",", "4498", ",", "4536",
    ",", "4558"}], "}"}]], "Output",
 CellChangeTimes->{3.4426743606645*^9}],

Cell[BoxData["8"], "Output",
 CellChangeTimes->{3.442674360680125*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Normalizer", "[", 
  RowBox[{"A", ",", 
   RowBox[{"{", "149", "}"}]}], "]"}], "\n", 
 RowBox[{"Length", "[", "%", "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "149", ",", "244"}], "}"}]], "Output",
 CellChangeTimes->{3.442674363148875*^9}],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{3.442674363148875*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Normalizer", "[", 
  RowBox[{"A", ",", 
   RowBox[{"{", "231", "}"}]}], "]"}], "\n", 
 RowBox[{"Length", "[", "%", "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "231", ",", "496", ",", "714"}], "}"}]], "Output",
 CellChangeTimes->{3.442674365492625*^9}],

Cell[BoxData["4"], "Output",
 CellChangeTimes->{3.442674365492625*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Normalizer", "[", 
  RowBox[{"A", ",", 
   RowBox[{"{", "918", "}"}]}], "]"}], "\n", 
 RowBox[{"Length", "[", "%", "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "918", ",", "2002", ",", "2183", ",", "3573", ",", "4190", ",", 
   "4695"}], "}"}]], "Output",
 CellChangeTimes->{3.442674367336375*^9}],

Cell[BoxData["7"], "Output",
 CellChangeTimes->{3.442674367352*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Normalizer", "[", 
  RowBox[{"A", ",", 
   RowBox[{"{", "953", "}"}]}], "]"}], "\n", 
 RowBox[{"Length", "[", "%", "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "953", ",", "2107", ",", "2602", ",", "3372", ",", "3622", ",", 
   "4753"}], "}"}]], "Output",
 CellChangeTimes->{3.442674369117625*^9}],

Cell[BoxData["7"], "Output",
 CellChangeTimes->{3.442674369117625*^9}]
}, Open  ]],

Cell[TextData[{
 "Finally, we form the sum\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderscriptBox[
     SubscriptBox[
      StyleBox["\[Sum]",
       FontSize->24], " "], 
     RowBox[{"g", " "}]], 
    StyleBox[
     FractionBox[
      RowBox[{"|", "G", "|"}], 
      RowBox[{"|", 
       RowBox[{
        SubscriptBox["N", "G"], "(", 
        RowBox[{"{", "g", "}"}], ")"}], "|"}]],
     FontSize->16]}], TraditionalForm]]],
 " = ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["168", "168"], " ", "+", " ", 
     FractionBox["168", "8"], " ", "+", " ", 
     FractionBox["168", "3"], "+", 
     FractionBox["168", "4"], "+", 
     FractionBox["168", "7"], "+", 
     FractionBox["168", "7"]}], " ", "=", " ", "168."}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We will see many very important applications of this equation, but let us \
begin by learning what this has to say about groups whose order is a power of \
a prime.\n\nCOROLLARY 7.2\nIf ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a group of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a positive integer, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 ", the center of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", is not just the identity element."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43692581671875*^9, 3.436925817484375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First we observe that an element ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is in the center of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " if, and only if,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", "\[CenterDot]", "g", "\[CenterDot]", 
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]]}], " ", "=", " ", "g"}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", which would happen if, and only if, the conjugacy class of ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " consists of just ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " by itself.\n\nNow suppose ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has a trivial center. Then the only conjugacy class which contains just \
one element would be the class ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", "e", "}"}], "."}], TraditionalForm]]],
 " All other conjugacy classes would have a size which is a divisor of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ", so the number of elements in the other conjugacy classes would be a power \
of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". But this is impossible since the sum on the right hand side of the class \
equation (7.2) would be congruent to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "p"}], ")"}]}], ","}], TraditionalForm]]],
 " while the left hand side of the class equation would be ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " which is congruent to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", " ", 
    RowBox[{"(", 
     RowBox[{"Mod", " ", "p"}], ")"}]}], TraditionalForm]]],
 ". Therefore, ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has a nontrivial center."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436925851109375*^9, 3.43692585178125*^9}}]
}, Open  ]],

Cell[TextData[{
 "This corollary is useful in finding all non-isomorphic groups of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime. For example, we can easily find all non-isomorphic groups of \
order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 ".\n\nCOROLLARY 7.3\nIf ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime then there are exactly two non-isomorphic groups of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 ", namely, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    SuperscriptBox["p", "2"]], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["Z", "p"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["Z", "p"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436925861609375*^9, 3.436925862828125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a group of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 ", then by corollary 7.2, ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has a nontrivial center. Since the number of elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " must divide ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", 
    RowBox[{"Z", "(", "G", ")"}], "|"}], TraditionalForm]]],
 " is either equal to ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 ".\n\nSuppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", 
     RowBox[{"Z", "(", "G", ")"}], "|"}], " ", "=", " ", "p"}], 
   TraditionalForm]]],
 ". Then there exists an element ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " not in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], " ", 
    RowBox[{"(", 
     RowBox[{"{", "g", "}"}], ")"}]}], TraditionalForm]]],
 " denotes the set of elements which commute with ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ". Certainly\n\n                                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Z", "(", "G", ")"}], " ", "\[SubsetEqual]", " ", 
    RowBox[{
     SubscriptBox["N", "G"], "(", 
     RowBox[{"{", "g", "}"}], ")"}]}], TraditionalForm]]],
 ",\nand also \n                                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     FormBox[
      RowBox[{"g", " ", "\[Epsilon]", " ", 
       SubscriptBox["N", "G"]}],
      TraditionalForm], 
     RowBox[{"(", 
      RowBox[{"{", "g", "}"}], ")"}]}]}], TraditionalForm]]],
 ",\n                                    \nso ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    RowBox[{"{", "g", "}"}], ")"}], TraditionalForm]]],
 " contains at least ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "+", "1"}], TraditionalForm]]],
 " elements. But this is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", so the number of elements must divide ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 ". Hence, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    RowBox[{"{", "g", "}"}], ")"}], TraditionalForm]]],
 " contains all of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", but this would say that ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is in the center ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 ", which contradicts our assumption. Thus, there are ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 " elements in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Z", "(", "G", ")"}], ","}], TraditionalForm]]],
 " and hence ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is an abelian group. \n\nFinally, we can use the fundamental theorem of \
finite abelian groups (6.2) to say that ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " must be isomorphic to the direct product of cyclic groups. It is easy to \
see that there are exactly two possibilities for such a product to have ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 " elements, namely ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    SuperscriptBox["p", "2"]], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "p"], "\[Times]", 
    SubscriptBox["Z", "p"]}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436925879984375*^9, 3.436925880765625*^9}, {
  3.43692592459375*^9, 3.43692592475*^9}}]
}, Open  ]],

Cell[TextData[{
 "In particular we can use corollary 7.3 to see that there are only 2 \
non-isomorphic groups of order 9:  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "9"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "3"], "\[Times]", 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ". \n\nOne of the keys for finding all groups of a certain order is knowing \
whether there is a normal subgroup of a certain order. The next proposition \
will allow us to know that there will be a normal subgroup ",
 StyleBox["without knowing the structure of the group",
  FontSlant->"Italic"],
 ".\n\nPROPOSITION 7.8\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " contains a normal subgroup of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"n", "-", "1"}]], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436925936515625*^9, 3.436925947375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We will proceed by using induction on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Note that if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ", then there is obviously a normal subgroup of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", 
     RowBox[{"1", "-", "1"}]], "=", " ", 
    RowBox[{
     SuperscriptBox["p", "0"], " ", "=", " ", "1"}]}], TraditionalForm]]],
 ", namely the trivial subgroup ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "e", "}"}], TraditionalForm]]],
 ". \n\nSuppose that we know that every group of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"n", "-", "1"}]], TraditionalForm]]],
 " has a normal subgroup of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"n", "-", "2"}]], TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ". Then by corollary 7.2, the center of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is not just the identity element. Since ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " would then divide the order of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 ", by lemma 6.2 there is an element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", say ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". Then the group generated by ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " would be of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", and since ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in the center, all elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " would commute with ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", "=", 
    RowBox[{"[", "x", "]"}]}], TraditionalForm]]],
 " would be a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". \n\nWe then can consider the quotient group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "X"}], TraditionalForm]]],
 ". This would have order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"n", "-", "1"}]], TraditionalForm]]],
 ", and we would have the canonical homomorphism \n\n                         \
                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", ":", " ", 
    RowBox[{"G", " ", "\[RightArrow]", " ", 
     RowBox[{"G", "/", "X"}]}]}], TraditionalForm]]],
 "\n                                                 \nwhose kernel is the \
subgroup ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ". By the induction hypothesis, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "X"}], TraditionalForm]]],
 " is a group of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"n", "-", "1"}]], TraditionalForm]]],
 ", and so has a normal subgroup of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"n", "-", "2"}]], TraditionalForm]]],
 ", say ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ". \n    We will now \"lift\" the subgroup ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " back to the original group.  Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Phi]", 
     RowBox[{"-", "1"}]], "(", "Y", ")"}], TraditionalForm]]],
 " is the inverse image of a normal subgroup, by corollary 4.2, \nthis is a \
normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Note ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " is a set of cosets, and that  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", " ", "\[Element]", " ", 
    RowBox[{
     SuperscriptBox["\[Phi]", 
      RowBox[{"-", "1"}]], "(", "Y", ")"}]}], TraditionalForm]]],
 " if and only if ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is contained in one of the cosets of ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ".  Since each of the cosets of ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " contains ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " elements, it is clear that the size of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Phi]", 
     RowBox[{"-", "1"}]], "(", "Y", ")"}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[CenterDot]", 
    SuperscriptBox["p", 
     RowBox[{"n", "-", "2"}]]}], TraditionalForm]]],
 " = ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"n", "-", "1"}]], TraditionalForm]]],
 ". Therefore, we have proved by induction that there is a normal subgroup of \
",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"n", "-", "1"}]], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43692597715625*^9, 3.43692598584375*^9}, {
  3.436926023984375*^9, 3.436926266140625*^9}}]
}, Open  ]],

Cell[TextData[{
 "We now are ready to start finding normal subgroups of a more general group, \
knowing only the group's order. The most important set of theorems which \
tackle this problem are by a Norwegian high school teacher named Ludwig Sylow \
(1832-1918). Before we work on finding normal subgroups let us see if we can \
find a subgroup of a given order within a group.\n\nTHEOREM 7.3:  THE FIRST \
SYLOW THEOREM\nSuppose that ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a group of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "n"], "\[CenterDot]", "m"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime, and ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has a subgroup of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369262863125*^9, 3.43692632575*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We will proceed by using induction on the size of the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". That is, we will assume that the theorem is true for all groups smaller \
than ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". \n\nIf ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " divided ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "H", "|"}], TraditionalForm]]],
 " for some proper subgroup ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then by our induction hypothesis, ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " would have a subgroup of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ", which would be a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " for which we are searching. So we may assume that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " does not divide the order of any proper subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". \n\nIn particular if ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is not in the center of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    RowBox[{"{", "g", "}"}], ")"}], TraditionalForm]]],
 " will not be all of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Hence, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " does not divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", " ", 
    RowBox[{
     SubscriptBox["N", "G"], "(", " ", 
     RowBox[{"{", "g", "}"}], ")"}], "|"}], TraditionalForm]]],
 ". But since ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " does divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "G", "|"}], TraditionalForm]]],
 ", we have from lemma 7.2 that the number of conjugates of ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "G", "|", 
    RowBox[{"/", 
     RowBox[{"|", 
      RowBox[{
       SubscriptBox["N", "G"], "(", 
       RowBox[{"{", "g", "}"}], ")"}], "|"}]}]}], TraditionalForm]]],
 ", which must be a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". \n\nNow we can use the argument that we used in corollary 7.2. The class \
equation theorem (7.2) states that\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"                                         ", 
    RowBox[{
     RowBox[{"|", "G", "|"}], "  ", "=", " ", 
     RowBox[{
      UnderscriptBox[
       SubscriptBox[
        StyleBox["\[Sum]",
         FontSize->24], " "], 
       RowBox[{"g", " "}]], "   ", 
      StyleBox[
       FractionBox[
        RowBox[{"|", "G", "|"}], 
        RowBox[{"|", 
         RowBox[{
          SubscriptBox["N", "G"], "(", 
          RowBox[{"{", "g", "}"}], ")"}], "|"}]],
       FontSize->16]}]}]}], TraditionalForm]], "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False],
 " ,\n\nwhere the sum runs over one ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " from each conjugacy class. For those ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " in the center of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "G", "|", " ", 
    RowBox[{"/", " ", 
     RowBox[{"|", 
      RowBox[{
       SubscriptBox["N", "G"], "(", 
       RowBox[{"{", "g", "}"}], ")"}], "|", " "}]}]}], TraditionalForm]]],
 " will be 1, while for all other terms, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "G", "|", " ", 
    RowBox[{"/", " ", 
     RowBox[{"|", 
      RowBox[{
       SubscriptBox["N", "G"], "(", 
       RowBox[{"{", "g", "}"}], ")"}], "|"}]}]}], TraditionalForm]]],
 " will be a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Since the sum is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "n"], "\[CenterDot]", "m"}], TraditionalForm]]],
 " which is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", the number of elements in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " must be a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " is an abelian group and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", 
    RowBox[{"Z", "(", "G", ")"}], "|"}], TraditionalForm]]],
 ", we have by lemma 6.2 that there is an element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "(", "G", ")"}], TraditionalForm]]],
 " of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", say ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". We now can proceed in the same way as we did in proposition 7.8. Since ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in the center, all elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " would commute with ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", " ", "=", " ", 
    RowBox[{"[", "x", "]"}]}], TraditionalForm]]],
 " would be a normal subgroup of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". \n\nThe quotient group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "X"}], TraditionalForm]]],
 " would then have order ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", 
     RowBox[{"n", "-", "1"}]], "\[CenterDot]", "m"}], TraditionalForm]]],
 ", and we would have the canonical homomorphism \n\n                         \
                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", ":", 
    RowBox[{"G", " ", "\[RightArrow]", " ", 
     RowBox[{"G", "/", "X"}]}]}], TraditionalForm]]],
 "\n                                                   \nwhose kernel is the \
subgroup ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ". By the induction hypothesis, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "X"}], TraditionalForm]]],
 " is smaller then ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and so has a subgroup of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", 
    RowBox[{"n", "-", "1"}]], TraditionalForm]]],
 ", say ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ". We can then lift ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " back to the original group.  Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Phi]", 
     RowBox[{"-", "1"}]], "(", "Y", ")"}], TraditionalForm]]],
 " is the inverse image of a subgroup,  by corollary 4.2, this is a subgroup \
of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". But the kernel of the homomorphism is of order ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", so the size of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Phi]", 
     RowBox[{"-", "1"}]], "(", "Y", ")"}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "\[CenterDot]", 
     SuperscriptBox["p", 
      RowBox[{"n", "-", "1"}]]}], "=", " ", 
    RowBox[{
     SuperscriptBox["p", "n"], "."}]}], TraditionalForm]]],
 " Therefore, we have proved by \ninduction that there is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43692637075*^9, 3.436926392984375*^9}, {
  3.436926452703125*^9, 3.436926462234375*^9}, {3.43692650384375*^9, 
  3.43692653578125*^9}}]
}, Open  ]],

Cell[TextData[{
 "Because these subgroups of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " are so important, we will give them a special name\n\nDEFINITION 7.5:\nIf \
",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a group of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "n"], "\[CenterDot]", "m"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " is coprime to the prime ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", then a subgroup of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " is called a ",
 StyleBox["p-Sylow",
  FontSlant->"Italic"],
 " ",
 StyleBox["subgroup",
  FontSlant->"Italic"],
 ".\n\nLet us give a quick application of the first Sylow theorem (7.3). \
Suppose we have a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of order 10. There is guaranteed to be a 2-Sylow subgroup, say ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and a 5-Sylow subgroup, say ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Obviously, \n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[TildeTilde]", " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", " ", "\[TildeTilde]", " ", 
    SubscriptBox["Z", "5"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369265498125*^9, 3.436926566609375*^9}}],

Cell[TextData[{
 "Furthermore, the intersection of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " must just be the identity, since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 " does not have any elements of order 2. Also, ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " with index 2, so by proposition 3.5, ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is also normal, we have by the direct product theorem (6.1) that \n        \
 ",
 Cell[BoxData[
  FormBox[Cell[TextData[Cell[BoxData[
    FormBox[
     RowBox[{Cell[TextData[Cell[BoxData[
       FormBox[
        RowBox[{
         RowBox[{"H", "\[CenterDot]", "K"}], "\[TildeTilde]", 
         RowBox[{"H", "\[Times]", "K"}], "\[TildeTilde]", 
         RowBox[{
          SubscriptBox["Z", "2"], "\[Times]", 
          SubscriptBox["Z", "5"]}], "\[TildeTilde]", 
         SubscriptBox["Z", "10"]}], TraditionalForm]]]]], Cell[""]}], 
     TraditionalForm]]]]], TraditionalForm]]],
 ".\n On the other hand, if ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is not a normal subgroup, then by the semi-direct product theorem (6.3), \n\
\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", "H", " ", 
    SubscriptBox["\[Proportional]", "\[Phi]"], "K"}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436926583265625*^9, 3.4369266765*^9}, {
  3.4369267571875*^9, 3.43692697615625*^9}, {3.43692708765625*^9, 
  3.436927101578125*^9}, {3.441588927*^9, 3.44158895859375*^9}}],

Cell[TextData[{
 "for some non-trivial homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "K", ")"}], TraditionalForm]]],
 ". But in chapter 6, we found that there were only one non-trivial \
homomorphism, yielding ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "5"], TraditionalForm]]],
 ".  In either case, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " must be isomorphic to one of these two groups. Since both of these are of \
order 10, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " must be all of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and so we have found exactly two non-isomorphic groups of order 10. \n\n\
Even though Sylow's first theorem (7.3) guarantees that there will be at \
least one ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup, there may be more than one. The next of Sylow's theorems \
shows that any two ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups are related.\n\nTHEOREM 7.4:  THE SECOND SYLOW THEOREM\nIf \
",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are two  ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then there exists an element ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    RowBox[{"u", "\[CenterDot]", "K", "\[CenterDot]", 
     RowBox[{
      SuperscriptBox["u", 
       RowBox[{"-", "1"}]], "."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43692711146875*^9, 3.43692723578125*^9}, {
  3.441589214015625*^9, 3.441589223703125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "n"], "\[CenterDot]", "m"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " is coprime to the prime ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". We begin by showing that whenever ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is a ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow",
 Cell[BoxData[
  FormBox[" ", TraditionalForm]]],
 "subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "K", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 "will also be a ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup for all ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Note that the number of \nelements in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "K", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is also ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ", and if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", 
    SubscriptBox["k", "1"], "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", 
    SubscriptBox["k", "2"], "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " are two elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "K", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 ", then \n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"u", "\[CenterDot]", 
         SubscriptBox["k", "1"], "\[CenterDot]", 
         SuperscriptBox["u", 
          RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"u", "\[CenterDot]", 
          SubscriptBox["k", "2"], "\[CenterDot]", 
          SuperscriptBox["u", 
           RowBox[{"-", "1"}]]}], ")"}], 
        RowBox[{"-", "1"}]]}], "=", " ", 
      RowBox[{
       RowBox[{"u", "\[CenterDot]", 
        SubscriptBox["k", "1"], "\[CenterDot]", 
        SuperscriptBox["u", 
         RowBox[{"-", "1"}]], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"u", "\[CenterDot]", 
          SubsuperscriptBox["k", "2", 
           RowBox[{"-", "1"}]], "\[CenterDot]", 
          SuperscriptBox["u", 
           RowBox[{"-", "1"}]]}], ")"}]}], " ", "=", " ", 
       RowBox[{"u", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["k", "1"], "\[CenterDot]", 
          SubsuperscriptBox["k", "2", 
           RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
        SuperscriptBox["u", 
         RowBox[{"-", "1"}]]}]}]}], ","}]}], TraditionalForm]]],
 "\n\nwhich is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "K", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 ". So by proposition 2.2, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "K", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is a ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nIf there is only one ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " there is nothing to prove. Suppose ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are two subgroups of order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ". Let us call two elements ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " to be \"related\" if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "=", " ", 
    RowBox[{"h", "\[CenterDot]", "v", "\[CenterDot]", "k"}]}], 
   TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Note that every element is related to itself, for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "=", " ", 
    RowBox[{"e", "\[CenterDot]", "u", "\[CenterDot]", "e"}]}], 
   TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " is in both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Also, if ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is related to ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " is related to ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 ", for\n\n                           ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "=", 
    RowBox[{
     RowBox[{"h", "\[CenterDot]", "v", "\[CenterDot]", 
      RowBox[{"k", "    ", "\[DoubleLongLeftRightArrow]", "    ", "v"}]}], 
     "=", 
     RowBox[{
      SuperscriptBox["h", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "u", "\[CenterDot]", 
      SuperscriptBox["k", 
       RowBox[{"-", "1"}]]}]}]}], TraditionalForm]]],
 ".\n                           \nFinally, if ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is related to ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " is related to ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 ", then \n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["h", "1"], "\[CenterDot]", "v", "\[CenterDot]", 
        SubscriptBox["k", "1"]}], "  ", "and", "  ", "v"}], " ", "=", " ", 
      RowBox[{
       SubscriptBox["h", "2"], "\[CenterDot]", "w", "\[CenterDot]", 
       SubscriptBox["k", "2"]}]}]}], ",", " ", 
    RowBox[{
     RowBox[{"and", " ", "so", "  ", "u"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["h", "1"], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "2"], "\[CenterDot]", "w", "\[CenterDot]", 
         SubscriptBox["k", "2"]}], ")"}], "\[CenterDot]", 
       SubscriptBox["k", "1"]}], "=", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["h", "1"], "\[CenterDot]", 
         SubscriptBox["h", "2"]}], ")"}], "\[CenterDot]", "w", "\[CenterDot]",
        
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["k", "2"], "\[CenterDot]", 
         SubscriptBox["k", "1"]}], ")"}]}]}]}], ","}], TraditionalForm]]],
 "\n\nso ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 " are related. Therefore, we can partition the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " into \"families\", where each family consists of all elements related to \
one element. \n\nNow suppose that there are ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " families, and we select one element ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "i"], TraditionalForm]]],
 " from each family. Each of the families can be described as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", 
    SubscriptBox["u", "i"], "\[CenterDot]", "K"}], TraditionalForm]]],
 ". Hence, we can write\n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"H", "\[CenterDot]", 
       SubscriptBox["u", "1"], "\[CenterDot]", "K"}], ")"}], " ", "\[Union]", 
     " ", 
     RowBox[{"(", 
      RowBox[{"H", "\[CenterDot]", 
       SubscriptBox["u", "2"], "\[CenterDot]", "K"}], ")"}], " ", "\[Union]", 
     "  ", "\[CenterEllipsis]", " ", "\[Union]", " ", 
     RowBox[{"(", 
      RowBox[{"H", "\[CenterDot]", 
       SubscriptBox["u", "j"], "\[CenterDot]", "K"}], ")"}]}]}], 
   TraditionalForm]]],
 ".\n                        \nSince each of the families have no elements in \
common, we have\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "G", "|"}], "=", " ", 
    RowBox[{"|", 
     RowBox[{"H", "\[CenterDot]", 
      SubscriptBox["u", "1"], "\[CenterDot]", "K"}], "|", " ", 
     RowBox[{"+", " ", 
      RowBox[{"|", 
       RowBox[{"H", "\[CenterDot]", 
        SubscriptBox["u", "2"], "\[CenterDot]", "K"}], "|", " ", 
       RowBox[{
        RowBox[{"+", "  ", "\[CenterEllipsis]"}], " ", "+"}], " ", "|", 
       RowBox[{"H", "\[CenterDot]", 
        SubscriptBox["u", "j"], "\[CenterDot]", "K"}], "|"}]}]}]}], 
   TraditionalForm]]],
 ".\n                     \nHow many elements are in each family? We note \
that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"H", "\[CenterDot]", 
      SubscriptBox["u", "i"]}],
     TraditionalForm], "\[CenterDot]", "K"}], TraditionalForm]]],
 " has the same number of elements as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", 
    SubscriptBox["u", "i"], "\[CenterDot]", "K", "\[CenterDot]", 
    SuperscriptBox[
     SubscriptBox["u", "i"], 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 ". We saw that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["u", "i"], "\[CenterDot]", "K", "\[CenterDot]", 
    SuperscriptBox[
     SubscriptBox["u", "i"], 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is a group, and so even though the product of two groups was not always a \
group, proposition 4.9 gave us the number of elements in the set to be \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{" ", 
        RowBox[{"|", 
         RowBox[{"H", "\[CenterDot]", 
          SubscriptBox["u", "i"]}]}]}],
       TraditionalForm], "\[CenterDot]", "K"}], "|"}], " ", "=", " "}], 
   TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", 
     RowBox[{"H", "\[CenterDot]", 
      SubscriptBox["u", "i"], "\[CenterDot]", "K", "\[CenterDot]", 
      SubsuperscriptBox["u", "i", 
       RowBox[{"-", "1"}]]}], "|"}], " ", "=", " ", 
    RowBox[{
     StyleBox[
      FractionBox[
       RowBox[{"|", "H", "|", 
        RowBox[{"\[CenterDot]", 
         RowBox[{"|", 
          RowBox[{
           SubscriptBox["u", "i"], "\[CenterDot]", "K", "\[CenterDot]", 
           SubsuperscriptBox["u", "i", 
            RowBox[{"-", "1"}]]}], "|"}]}]}], 
       RowBox[{"|", 
        RowBox[{"H", " ", "\[Intersection]", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["u", "i"], "\[CenterDot]", "K", "\[CenterDot]", 
           SubsuperscriptBox["u", "i", 
            RowBox[{"-", "1"}]]}], ")"}]}], "|"}]],
      FontSize->16], " ", "=", 
     StyleBox[" ",
      FontSize->16], 
     StyleBox[
      FractionBox[
       RowBox[{
        SuperscriptBox["p", "n"], "\[CenterDot]", 
        SuperscriptBox["p", "n"]}], 
       RowBox[{"|", 
        RowBox[{"H", " ", "\[Intersection]", " ", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["u", "i"], "\[CenterDot]", "K", "\[CenterDot]", 
           SubsuperscriptBox["u", "i", 
            RowBox[{"-", "1"}]]}], ")"}]}], "|"}]],
      FontSize->16]}], " "}], TraditionalForm]]],
 ".\n\nIf we plug this formula into the equation above it, we have that \n\n\
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "G", "|"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["p", "n"], "\[CenterDot]", "m"}], " ", "=", "\n", 
     StyleBox[" ",
      FontSize->16], "\n", 
     RowBox[{
      StyleBox[
       FractionBox[
        RowBox[{
         SuperscriptBox["p", "n"], "\[CenterDot]", 
         SuperscriptBox["p", "n"]}], 
        RowBox[{"|", 
         RowBox[{"H", "\[Intersection]", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["u", "1"], "\[CenterDot]", "K", "\[CenterDot]", 
            SubsuperscriptBox["u", "1", 
             RowBox[{"-", "1"}]]}], ")"}]}], "|"}]],
       FontSize->16], " ", "+", 
      StyleBox[" ",
       FontSize->16], 
      StyleBox[
       FractionBox[
        RowBox[{
         SuperscriptBox["p", "n"], "\[CenterDot]", 
         SuperscriptBox["p", "n"]}], 
        RowBox[{"|", 
         RowBox[{"H", "\[Intersection]", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["u", "2"], "\[CenterDot]", "K", "\[CenterDot]", 
            SubsuperscriptBox["u", "2", 
             RowBox[{"-", "1"}]]}], ")"}]}], "|"}]],
       FontSize->16], "+", "\[CenterEllipsis]", " ", "+", 
      StyleBox[
       RowBox[{" ", 
        StyleBox[" ",
         FontSize->16]}]], 
      StyleBox[
       FractionBox[
        RowBox[{
         SuperscriptBox["p", "n"], "\[CenterDot]", 
         SuperscriptBox["p", "n"]}], 
        RowBox[{"|", 
         RowBox[{"H", "\[Intersection]", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["u", "j"], "\[CenterDot]", "K", "\[CenterDot]", 
            SubsuperscriptBox["u", "j", 
             RowBox[{"-", "1"}]]}], ")"}]}], "|", "."}]],
       FontSize->16]}]}]}], TraditionalForm]]],
 "\n\nNote that the intersection of two groups is a subgroup of both the \
groups, and so the denominators will all be powers of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Dividing both sides of the equation by ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ", we have\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox[" ",
     FontSize->16], 
    RowBox[{
     StyleBox["m",
      FontSize->12], 
     StyleBox[" ",
      FontSize->12], 
     StyleBox["=",
      FontSize->12], 
     StyleBox[" ",
      FontSize->16], 
     RowBox[{
      StyleBox[
       FractionBox[
        SuperscriptBox["p", "n"], 
        RowBox[{"|", 
         RowBox[{"H", "\[Intersection]", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["u", "1"], "\[CenterDot]", "K", "\[CenterDot]", 
            SubsuperscriptBox["u", "1", 
             RowBox[{"-", "1"}]]}], ")"}]}], "|"}]],
       FontSize->16], " ", "+", 
      StyleBox[" ",
       FontSize->16], 
      StyleBox[
       FractionBox[
        SuperscriptBox["p", "n"], 
        RowBox[{"|", 
         RowBox[{"H", "\[Intersection]", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["u", "2"], "\[CenterDot]", "K", "\[CenterDot]", 
            SubsuperscriptBox["u", "2", 
             RowBox[{"-", "1"}]]}], ")"}]}], "|"}]],
       FontSize->16], 
      StyleBox[" ",
       FontSize->16], "+", "\[CenterEllipsis]", " ", "+", 
      StyleBox[
       RowBox[{" ", 
        StyleBox[" ",
         FontSize->16]}]], 
      StyleBox[
       RowBox[{
        FractionBox[
         SuperscriptBox["p", "n"], 
         RowBox[{"|", 
          RowBox[{"H", "\[Intersection]", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["u", "j"], "\[CenterDot]", "K", "\[CenterDot]", 
             SubsuperscriptBox["u", "j", 
              RowBox[{"-", "1"}]]}], ")"}]}], "|"}]], "."}],
       FontSize->16]}]}]}], TraditionalForm]]],
 "\n\nSince ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " is not a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", there must be some term on the right hand side of this equation which is \
not a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". But this can happen only if one of the denominators is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ", that is,\n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", 
     RowBox[{"H", 
      StyleBox["\[Intersection]",
       FontSize->9], " ", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["u", "i"], "\[CenterDot]", "K", "\[CenterDot]", 
        SubsuperscriptBox["u", "i", 
         RowBox[{"-", "1"}]]}], ")"}]}], "|"}], " ", "=", " ", 
    RowBox[{"|", "H", "|"}]}], TraditionalForm]]],
 "\n               \nfor some ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 ".  Since ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["u", "i"], "\[CenterDot]", "K", "\[CenterDot]", 
    SubsuperscriptBox["u", "i", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " both have ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 " elements, we must have H = ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["u", "i"], "\[CenterDot]", "K", "\[CenterDot]", 
    SubsuperscriptBox["u", "i", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 ". Therefore, for any two ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", there is a ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    RowBox[{"u", "\[CenterDot]", "K", "\[CenterDot]", 
     RowBox[{
      SuperscriptBox["u", 
       RowBox[{"-", "1"}]], "."}]}]}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436927275*^9, 3.436927276421875*^9}, {
   3.43692732809375*^9, 3.43692737128125*^9}, 3.43692742203125*^9, {
   3.4415892349375*^9, 3.441589275625*^9}, {3.44158930709375*^9, 
   3.441589578578125*^9}, {3.441589842578125*^9, 3.44158990415625*^9}}]
}, Open  ]],

Cell[TextData[{
 "The second Sylow theorem (7.4) allows us to know exactly when a ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup is normal.\n\nCOROLLARY 7.4\nThe group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has only one ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup for a given prime ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " if, and only if, ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " has a  ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup that is normal."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436927489875*^9, 3.43692749353125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is the only ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then for any element ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "H", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " will be a ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". But since there is only one ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow  subgroup, we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    RowBox[{"u", "\[CenterDot]", "H", "\[CenterDot]", 
     SuperscriptBox["u", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Hence, ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a normal subgroup.\n\nNow suppose that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a normal ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". By the second Sylow theorem (7.4) every other ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup is of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "H", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 ".   But since ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is normal, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "\[CenterDot]", "H", "\[CenterDot]", 
     SuperscriptBox["u", 
      RowBox[{"-", "1"}]]}], " ", "=", " ", 
    RowBox[{"H", "."}]}], TraditionalForm]]],
 " Therefore, ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is the only ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436927537640625*^9, 3.436927539078125*^9}}]
}, Open  ]],

Cell[TextData[{
 "Of course, the natural question that corollary 7.4 raises is, \"How do we \
know if there is only one  ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup?\" This lemma allows us to find the number of ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups in terms of the size of the normalizer. In fact it allows \
us to find the number of  ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups of a certain type.\n\nLEMMA 7.3\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "n"], "\[CenterDot]", "m"}], TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " be a ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " be any other subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then the number of ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups which can be written as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "P", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " an element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is given by\n\n                   ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{"|", "H", "|"}], 
    RowBox[{"|", 
     RowBox[{
      RowBox[{
       SubscriptBox["N", "G"], "(", "P", ")"}], "\[Intersection]", "H"}], 
     "|"}]], TraditionalForm]],
  FontSize->18],
 ".\t\t\t"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369275511875*^9, 3.436927581421875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["N", "G"],
     TraditionalForm], "(", "P", ")"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", so the intersection of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["N", "G"],
     TraditionalForm], "(", "P", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " will be a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". We can use the same argument as lemma 7.2, and note that if ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " are two elements of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "P", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "\[CenterDot]", "P", "\[CenterDot]", 
    SuperscriptBox["v", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " will represent the same ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup if, and only if,\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{
        RowBox[{
         RowBox[{"u", "\[CenterDot]", "P", "\[CenterDot]", 
          SuperscriptBox["u", 
           RowBox[{"-", "1"}]]}], " ", "=", " ", 
         RowBox[{
          RowBox[{"v", "\[CenterDot]", "P", "\[CenterDot]", 
           RowBox[{
            SuperscriptBox["v", 
             RowBox[{"-", "1"}]], "   ", "\[DoubleLongLeftRightArrow]", "   ",
             
            SuperscriptBox["v", 
             RowBox[{"-", "1"}]]}], "\[CenterDot]", "u", "\[CenterDot]", "P", 
           "\[CenterDot]", 
           SuperscriptBox["u", 
            RowBox[{"-", "1"}]], "\[CenterDot]", "v"}], " ", "=", " ", 
          "P"}]}], " ", "\n", 
        "                                               "}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[DoubleLongLeftRightArrow]", "    ", 
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["v", 
              RowBox[{"-", "1"}]], "\[CenterDot]", "u"}], ")"}]}], 
          "\[CenterDot]", "P", "\[CenterDot]", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["v", 
              RowBox[{"-", "1"}]], "\[CenterDot]", "u"}], ")"}], 
           RowBox[{"-", "1"}]]}], " ", "=", " ", "P"}], "\n", 
        "                                                ", 
        RowBox[{
         RowBox[{
          RowBox[{"\[DoubleLongLeftRightArrow]", "   ", 
           SuperscriptBox["v", 
            RowBox[{"-", "1"}]]}], "\[CenterDot]", "u"}], " ", "\[Element]", 
         " ", 
         RowBox[{
          RowBox[{
           SubscriptBox["N", "G"], "(", "P", ")"}], 
          StyleBox["\[Intersection]",
           FontSize->8,
           FontWeight->"Plain",
           FontSlant->"Plain",
           FontTracking->"Plain",
           FontVariations->{"Outline"->False,
           "Shadow"->False,
           "Underline"->False}], "H"}]}]}],
       TraditionalForm]}], " ", "\n", 
     "\t\t                                  ", 
     RowBox[{
      FormBox[
       RowBox[{
        RowBox[{
         RowBox[{"\[DoubleLongLeftRightArrow]", "     ", "u"}], " ", 
         "\[Element]", " ", 
         RowBox[{"v", "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SubscriptBox["N", "G"], "(", "P", ")"}], "\[Intersection]", 
            "H"}], ")"}]}]}], "    ", "\n", 
        "                                                ", 
        RowBox[{
         RowBox[{
          RowBox[{"\[DoubleLongLeftRightArrow]", "     ", "u"}], 
          "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SubscriptBox["N", "G"], "(", "P", ")"}], "\[Intersection]", 
            "H"}], ")"}]}], " ", "=", " ", 
         RowBox[{"v", "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SubscriptBox["N", "G"], "(", "P", ")"}], "\[Intersection]", 
            "H"}], ")"}]}]}]}],
       TraditionalForm], "."}]}]}], TraditionalForm]]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", "P", "\[CenterDot]", 
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "\[CenterDot]", "P", "\[CenterDot]", 
    SuperscriptBox["v", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " represent the same ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup if, and only if, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["N", "G"], "(", "P", ")"}], "\[Intersection]", "H"}], 
      ")"}]}], " "}], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["N", "G"], "(", "P", ")"}], "\[Intersection]", "H"}], 
     ")"}]}], TraditionalForm]]],
 " are the same left cosets of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["N", "G"], "(", "P", ")"}], " ", "\[Intersection]", " ", 
    "H"}], TraditionalForm]]],
 ". Therefore, the number of ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups which can be express as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "\[CenterDot]", "P", "\[CenterDot]", 
     SuperscriptBox["u", 
      RowBox[{"-", "1"}]]}], ","}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " an element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", is\n\n                                              ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{"|", "H", "|"}], 
    RowBox[{"|", 
     RowBox[{
      RowBox[{
       SubscriptBox["N", "G"], "(", "P", ")"}], "\[Intersection]", "H"}], 
     "|"}]], TraditionalForm]],
  FontSize->18],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436927596375*^9, 3.4369276454375*^9}}]
}, Open  ]],

Cell[TextData[{
 "We now are ready to prove the last of Sylow's theorem, which in many cases \
will tell us the number of ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups of a group."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "THEOREM 7.5:  THE THIRD SYLOW THEOREM\nSuppose that the number of ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "G", "|"}], TraditionalForm]]],
 ", and\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "p"}], ")"}]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436927677953125*^9, 3.436927698125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that we label the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "-", "Sylow"}], TraditionalForm]]],
 " subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["P", "0"], ",", " ", 
    SubscriptBox["P", "1"], ",", " ", 
    SubscriptBox["P", "2"], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     SubscriptBox["P", 
      RowBox[{"k", "-", "1"}]]}]}], TraditionalForm]]],
 ". Let us partition all of the ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow  subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " into different categories where two ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "i"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "j"], TraditionalForm]]],
 " are in the same category if there is an element ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "0"], TraditionalForm]]],
 " such that \n                         \n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["P", "j"],
     TraditionalForm], " ", "=", " ", 
    RowBox[{
     FormBox[
      RowBox[{"u", "\[CenterDot]", 
       SubscriptBox["P", "i"]}],
      TraditionalForm], "\[CenterDot]", 
     SuperscriptBox["u", 
      RowBox[{"-", "1"}]]}]}], TraditionalForm]]],
 ".\n\nNote that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "0"], TraditionalForm]]],
 " would be in its own category while the number of ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups in the other categories would be, according to lemma 7.3,  \
\n\n                                    ",
 StyleBox[" ",
  FontSize->18],
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{
     SubscriptBox[
      RowBox[{"|", "P"}], "0"], "|"}], 
    RowBox[{"|", 
     RowBox[{
      RowBox[{
       SubscriptBox["N", "G"], "(", 
       SubscriptBox["P", "i"], ")"}], "\[Intersection]", " ", 
      SubscriptBox["P", "0"]}], "|"}]], TraditionalForm]],
  FontSize->18],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "i"], TraditionalForm]]],
 " is one ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow  subgroup in the category. \n\nRecall that the normalizer of each ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "i"], TraditionalForm]]],
 " contains ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "i"], TraditionalForm]]],
 "as a normal subgroup, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      FormBox[
       SubscriptBox["N", "G"],
       TraditionalForm], "(", 
      SubscriptBox["P", "i"]}],
     TraditionalForm], ")"}], TraditionalForm]]],
 " is divisible by ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ", and hence by corollary 7.4 the only ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["N", "G"],
     TraditionalForm], "(", 
    SubscriptBox["P", "i"], ")"}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "i"], TraditionalForm]]],
 ". Thus, the intersection of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "G"], "(", 
    SubscriptBox["P", "i"], ")"}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "0"], TraditionalForm]]],
 " is smaller than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "0"], TraditionalForm]]],
 "  when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", " ", ">", " ", "0"}], TraditionalForm]]],
 ". Since the order of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "0"], TraditionalForm]]],
 "  is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "n"], TraditionalForm]]],
 ", we have that the number of ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups in each category, besides the category containing just ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "0"], TraditionalForm]]],
 ", is a power of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", and hence is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ".\n\nTherefore, the total number of ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow subgroups is one more than a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "p"}], ")"}]}]}], TraditionalForm]]],
 ".\n\nFinally, if we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", "G"}], TraditionalForm]]],
 " in lemma 7.3, we find that the number of conjugates of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "0"], TraditionalForm]]],
 " is \n\n                                    ",
 StyleBox[" ",
  FontSize->18],
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{"|", "G", "|"}], 
    RowBox[{"|", 
     RowBox[{
      SubscriptBox["N", "G"], "(", 
      SubscriptBox["P", "0"], ")"}], "|"}]], TraditionalForm]],
  FontSize->18],
 ".\n                                     \nBy the second Sylow theorem \
(7.4), this would give us all of the ",
 StyleBox["p-",
  FontSlant->"Italic"],
 "Sylow  subgroups. Therefore, ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is also a divisor of the order of the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436927709984375*^9, 3.436927820953125*^9}, {
  3.436927937515625*^9, 3.4369279638125*^9}, {3.441590372984375*^9, 
  3.44159039090625*^9}}]
}, Open  ]],

Cell[TextData[{
 "These three theorems of Sylow provide a means of finding normal subgroups \
of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " even if all we know is the order of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". For example suppose that all that we know about a group is that the order \
is 45. How can we find some normal subgroups? Since ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["3", "2"], TraditionalForm]]],
 " divides 45, there is a 3-Sylow subgroup of order 9. How many of them will \
there be? We know that the number of 3-Sylow subgroups divides 45, so this \
number must be 1, 3, 5, 9, 15, or 45. However, the number must be congruent \
to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", 
    RowBox[{"(", 
     RowBox[{"Mod", " ", "3"}], ")"}]}], TraditionalForm]]],
 ". Thus, the only possibility is that there is only one subgroup of order 9, \
say ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". But then this subgroup is normal.\n\nWe can use the same argument to find \
a subgroup of order 5. Again, the number of 5-Sylow subgroups must be 1, 3, \
5, 9, 15, or 45. But this number must also be congruent to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", 
    RowBox[{"(", 
     RowBox[{"Mod", " ", "5"}], ")"}]}], TraditionalForm]]],
 ", and so there is one 1 subgroup of order 5. This group must also be \
normal.\n\nAlthough the Sylow theorems are a powerful tools, when combined \
with the tools of semi-direct products and the computational power of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we can determine most of the finite groups. For example, let us see if we \
can find all of the groups of order 12.\n\nSuppose that ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a group of order 12. The divisors of 12 are 1, 2, 3, 4, 6, and 12, and \
so by the third Sylow theorem there are either 1 or 4 3-Sylow subgroups and \
there are either 1 or 3 2-Sylow subgroups. Let ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " be a 3-Sylow subgroup, and let ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be a 2-Sylow subgroup (which will be of order 4.)  Certainly the \
intersection of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is just the identity element since ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " cannot contain an element of order 3. \n\nEven though we do not know \
whether ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is normal, we would like to show that at least one of them is. Suppose \
that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is not normal. Then there must be 4 3-Sylow subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Each of these 3-Sylow groups contain 2 different elements of order 3, so \
",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " must have 8 elements of order 3. But then there are only 4 elements left \
over, and so ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " must be composed of all of those 4 elements. Then there would be only one \
2-Sylow subgroup, which would be normal.\n\nSince ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " must be isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " must be isomorphic to either ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "4"], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ", we have by the direct product theorem (6.1.2) and the semi-direct product \
theorem (6.4.3) that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " is one of these six forms:\n\n1)         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["Z", "3"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "4"]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "12"]}], TraditionalForm]]],
 ",\n\n2)         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["Z", "3"], " ", "\[Times]", " ", 
     SubsuperscriptBox["Z", "8", "*"]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], 
    StyleBox[" ",
     FontSize->16], 
    RowBox[{
     SubscriptBox["Z", "3"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"]}]}], TraditionalForm]]],
 ",\n\n3)         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "3"], 
    SubscriptBox["\[Proportional]", "\[Phi]"], 
    SubscriptBox["Z", "4"]}], TraditionalForm]]],
 ",\n\n4)         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "3"], 
    SubscriptBox["\[Proportional]", "\[Phi]"], 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 ",\n\n5)         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], "  ", 
    SubscriptBox["Z", "4"], 
    SubscriptBox["\[Proportional]", "\[Phi]"], 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ",\n\n6)         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], "  ", 
    SubsuperscriptBox["Z", "8", "*"], 
    SubscriptBox["\[Proportional]", "\[Phi]"], 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436928045421875*^9, 3.43692816503125*^9}, {
  3.440947099242625*^9, 3.44094715557075*^9}}],

Cell[TextData[{
 "In all 6 cases ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " contains 12 elements, and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{"H", "\[CenterDot]", 
     RowBox[{"K", "."}]}]}], TraditionalForm]]],
 " Let us work these 6 cases separately. The first two give the two possible \
abelian groups of order 12. \n\nCase 3) \n\nThe non-trivial homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " maps ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"Aut", "(", 
      SubscriptBox["Z", "4"]}],
     TraditionalForm], ")"}], TraditionalForm]]],
 ". An element of order 3 must map to an element of order 3. But ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"Aut", "(", 
      SubscriptBox["Z", "4"]}],
     TraditionalForm], ")"}], TraditionalForm]]],
 " has only 2 elements, and therefore has no elements of order 3. So there \
are no groups of this form."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\nCase 4)\n\nThe non-trivial homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " maps ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "8", "*"], ")"}], TraditionalForm]]],
 ". Again, an element of order 3 must map to an element of order 3. This time \
we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"Aut", "(", 
      SubsuperscriptBox["Z", "8", "*"]}],
     TraditionalForm], ")"}], TraditionalForm]]],
 " isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ", which has exactly 2 elements of order 3. Thus, ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " maps the 2 non-identity elements in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " to the two elements of order 3 in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 "). By proposition 6.7, it does not matter which element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " maps to which elements in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "8", "*"]}], TraditionalForm]]],
 "). Thus, there is only one non-isomorphic group of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubscriptBox["Z", "3"], "\[Proportional]", 
     SubsuperscriptBox["Z", "8", "*"]}]}], TraditionalForm]]],
 ". Rather than having ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " produce the group, let us see if we can find it by process of elimination. \
We know that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " is a group of order 12, has a normal subgroup isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ", and does not have a normal subgroup of order 3. This cannot be covered by \
cases 5 or 6, and this certainly is not the abelian groups covered in cases 1 \
and 2. Thus, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " must be the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubscriptBox["Z", "3"], "\[Proportional]", 
     SubsuperscriptBox["Z", "8", "*"]}]}], TraditionalForm]]],
 ".\n\nCase 5)\nThe homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " maps ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "4"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["Z", "3"], ")"}], TraditionalForm]]],
 ". The homomorphism is completely determined by where it sends a generator. \
But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["Z", "3"], ")"}], TraditionalForm]]],
 " has only 2 elements, so there is only one non-trivial homomorphism. Let us \
use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to explore the homomorphism. First, we define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "4"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43692821721875*^9, 3.43692821721875*^9}, {
  3.44094719307075*^9, 3.440947245102*^9}, {3.44159047834375*^9, 
  3.441590506640625*^9}}],

Cell["\<\
InitGroup[e];
Define[a^4, e]
Define[1/a, a^3]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.44094732000825*^9},
 AspectRatioFixed->True],

Cell["\<\
Define[b^3, e]
Define[1/b, b^2]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "Now we know that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 " equals ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", 
    RowBox[{
     SubscriptBox["\[Phi]", "a"], "(", "b", ")"}]}], TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "a"], TraditionalForm]]],
 "    must be non-trivial, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Phi]", "a"], "(", "b", ")"}], TraditionalForm]]],
 " would be ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", "2"], TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    FormBox[
     RowBox[{"a", "\[CenterDot]", 
      SuperscriptBox["b", "2"]}],
     TraditionalForm]}], TraditionalForm]]],
 ", and we can define"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.440947287477*^9, 3.440947308711375*^9}}],

Cell["Define[b.a, a.b.b]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["Let us see if this forms a group.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
M = Group[{a, b}]
MultTable[M];
CheckGroup[M]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`a], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`a], $CellContext`a], $CellContext`b], \
$CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.44094732657075*^9, 3.442674403773875*^9}],

Cell[BoxData["\<\"This set of elements is a group.\"\>"], "Print",
 CellChangeTimes->{3.44094732682075*^9, 3.442674404055125*^9}]
}, Open  ]],

Cell[TextData[{
 "We can find the orders of the elements by computing the powers of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["M^2", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", "e", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.442674407242625*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["M^3", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`a],
    Editable->False], ",", "e", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`a], $CellContext`b],
    Editable->False], ",", "e", ",", "a", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`a], $CellContext`a], $CellContext`b], \
$CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.442674408305125*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["M^4", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "e", ",", "b", ",", "e", ",", "e", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", "e", ",", "b", ",", "e", ",", "e", ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", "e"}], "}"}]], "Output",
 CellChangeTimes->{3.442674409305125*^9}]
}, Open  ]],

Cell[TextData[{
 "So there is only one element of order 2, two elements of order 3, and six \
elements of order 4. The remaining two elements must be of order 6, since \
there cannot be any elements of order 12. This group is not isomorphic to any \
group we have studied before. We can describe this group simply as the \
semi-direct product\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], "\[Proportional]", " ", 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ".\n\nCase 6)\nThe homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " maps ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["Z", "3"], ")"}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["Z", "3"], ")"}], TraditionalForm]]],
 " \ncontains only 2 elements, the homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is completely determined by its kernel. The kernel of ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " cannot be just the identity, for this would indicate that \[Phi] is an \
isomorphism, and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubscriptBox["Z", "3"], ")"}], TraditionalForm]]],
 " does not have enough elements. On the other hand, the kernel of a \
non-trivial homomorphism connot be all of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ". So the kernal must be a subgroup of order 2. Since there is an \
automorphism of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 " mapping one subgroup of order 2 to any other, it will not matter which \
subgroup of order 2 we pick. Thus, there is only one semi-direct product ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["Z", "8", "*"], "\[Proportional]", 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ". This group contains ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " as a normal subgroup. Can we find such a group? Consider the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["Z", "2"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["S", "3"]}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 " contains a normal subgroup of order 3, so will ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["Z", "2"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["S", "3."]}], TraditionalForm]]],
 " Also, this group contains 2 elements of order 3, and no elements of order \
6. Hence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["Z", "2"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["S", "3"]}], TraditionalForm]]],
 " is not isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], "\[Proportional]", " ", 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ". So ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["Z", "2"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["S", "3"]}], TraditionalForm]]],
 " must be the remaining group covered by case 6.  So by process of \
elimination,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["Z", "2"],
      TraditionalForm], "\[Times]", 
     SubscriptBox["S", "3"]}], "\[TildeTilde]", 
    SubsuperscriptBox["Z", "8", "*"], "\[Proportional]", 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ".  Therefore, we have found 5 possible groups of order 12:\n\n     ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "12"], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      FormBox[
       SubscriptBox["Z", "3"],
       TraditionalForm], "\[Times]", 
      SubscriptBox["Z", "2"]}],
     TraditionalForm], "\[Times]", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["Z", "2"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["S", "3"]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], "\[Proportional]", " ", 
    RowBox[{
     SubscriptBox["Z", "3"], "."}]}], TraditionalForm]]],
 " \n\nWe can summarize all of our findings plus a few more in the following \
proposition.\n\nPROPOSITION 7.9\nThere are exactly 28 non-isomorphic groups \
of order less than 16."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436928268546875*^9, 3.436928274921875*^9}, {
  3.436928347703125*^9, 3.436928372015625*^9}, {3.436928420484375*^9, 
  3.436928479640625*^9}, {3.440947340086375*^9, 3.440947451305125*^9}, {
  3.441590666796875*^9, 3.4415906694375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The trivial group is the only group of order 1, and since 2, 3, 5, 7, 11, \
and 13 are prime, we have only one non-isomorphic group of each of these \
orders.\n\nIn chapter 4 we found that the only non-isomorphic groups of order \
4 were \n\n                               ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "4"], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ",\n                               \nthe only non-isomorphic groups of order \
6 were\n\n                               ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " and  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ",\n\nand the only non-isomorphic groups of order 8 were\n\n                 \
         ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 " , ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 " , ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "4"], TraditionalForm]]],
 ".\n\nBy corollary 7.3 the only two non-isomorphic groups of order 9 are\n\n \
                             ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "9"], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["Z", "3"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 " .\n\nWe have already used the first Sylow theorem (7.3) to find all of the \
non-isomorphic groups of order 10:\n\n                                ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "10"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "5"], TraditionalForm]]],
 ".\n\nWe just found all of the groups of order 12:\n\n               ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "12"], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      FormBox[
       SubscriptBox["Z", "2"],
       TraditionalForm], "\[Times]", 
      SubscriptBox["Z", "2"]}],
     TraditionalForm], "\[Times]", 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["Z", "2"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["S", "3"]}], TraditionalForm]]],
 ",  and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], "\[Proportional]", " ", 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ".\n\nWe can use the same argument to find all of the non-isomorphic groups \
of order 14. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "G", "|"}], "=", "14"}], TraditionalForm]]],
 ", there must be a 7-Sylow subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", say ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".  Since ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " contains half of the elements, by proposition 3.5, ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is normal.  We also must have a 2-Sylow subgroup, ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " cannot have an element of order 2, ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " have only the identity element in common. \nIf ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is normal, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", "\[TildeTilde]", 
    RowBox[{"H", "\[Times]", "K"}], "\[TildeTilde]", 
    RowBox[{
     SubscriptBox["Z", "2"], "\[Times]", 
     SubscriptBox["Z", "7"]}], "\[TildeTilde]", 
    SubscriptBox["Z", "14"]}], TraditionalForm]]],
 ".  If H is not normal, by the semi-direct product theorem (6.4.3), \n\n     \
                                             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", "H", " ", 
    SubscriptBox["\[Proportional]", "\[Phi]"], "K"}], TraditionalForm]]],
 "\n                                                  \nfor some homomorphism \
",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "K", ")"}], TraditionalForm]]],
 ". In either case ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " has 14 elements, and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{"H", "\[CenterDot]", "K"}]}], TraditionalForm]]],
 ". Also, ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is determined by where the non-identity element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is mapped. Since this must be an element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "K", ")"}], TraditionalForm]]],
 " of order  2, and since \n\n                               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", "K", ")"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"Aut", "(", 
     SubscriptBox["Z", "7"], ")"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubsuperscriptBox["Z", "7", "*"], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "6"]}], TraditionalForm]]],
 "\n                               \nhas only one element of order 2, there \
can only be one such homomorphism. Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "7"], TraditionalForm]]],
 " is a non-abelian group of order 14, this must be the one semi-direct \
product that we found. Thus, the only two groups of order 14 are\n\n          \
                    ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "14"], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "7"], TraditionalForm]]],
 ".\n\nLet us move on to find all groups of order 15. Suppose ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "G", "|"}], " ", "=", " ", "15"}], TraditionalForm]]],
 ". Then the number of 3-Sylow subgroups and the number of 5-Sylow subgroups \
must both divide 15, so both of these numbers must be one of 1, 3, 5, or 15. \
But 1\nis the only number in this set which is congruent to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"Mod", " ", "5"}], ")"}], "."}]}], TraditionalForm]]],
 " So there is only one 5-Sylow subgroup, ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Likewise, 1 is the only number in the set which is congruent to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", 
    RowBox[{"(", 
     RowBox[{"Mod", " ", "3"}], ")"}]}], TraditionalForm]]],
 ". So there is only one 3-Sylow subgroup, ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". By\ncorollary 7.4 both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are normal subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and the intersection must be just the identity element. Thus, by \
corollary 6.1, \n\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "\[CenterDot]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"H", "\[Times]", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["Z", "3"], "\[Times]", 
     SubscriptBox["Z", "5"]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "15"]}], TraditionalForm]]],
 ".\n                       \nSince this has all 15 elements, this must be \
all of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and so there is only one non-isomorphic group of order 15, namely, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "15"], TraditionalForm]]],
 ". \n\nTherefore, counting all of the groups of order less than 16, we find \
that there are exactly 28 of them."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436928506203125*^9, 3.436928618265625*^9}, {
   3.4369286494375*^9, 3.436928667234375*^9}, {3.43692876215625*^9, 
   3.436928778703125*^9}, {3.4369288411875*^9, 3.4369288494375*^9}, {
   3.4409474620395*^9, 3.440947485961375*^9}, {3.44159090225*^9, 
   3.441591029421875*^9}, {3.44159107734375*^9, 3.44159126746875*^9}, {
   3.44159130078125*^9, 3.441591302671875*^9}, 3.44159166253125*^9}]
}, Open  ]],

Cell[TextData[{
 "Unfortunately, finding all the groups of order 16 is a difficult problem. \
Even though proposition 7.8 tells us that there must be a normal subgroup ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " of order 8, there is no guarantee that there would be a subgroup ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " of order 2 such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "K"}], TraditionalForm]]],
 " gives the whole group. Thus, we would not be able to use the semi-direct \
product theorem (6.3) to find ",
 StyleBox["all",
  FontSlant->"Italic"],
 " of the groups of order 16, (although we can find many of them, as we did \
in the last chapter.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436928872953125*^9, 3.4369288850625*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Problems"
}], "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43692889053125*^9, 3.436928892375*^9}}],

Cell["", "Input",
 PageWidth->Infinity,
 FontFamily->"System",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->RGBColor[1, 0, 0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "7.1 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the center of the \n     group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "6"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436928896546875*^9, 3.436928904296875*^9}}],

Cell["\<\
InitGroup[e];
Define[a^2, e]; Define[b^6, e]
Define[b.a, a.b.b.b.b.b]
Define[1/a, a]; Define[1/b, b^5]
D6 = Group[{a, b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4369289110625*^9, 3.43692893440625*^9}, {
  3.441591734359375*^9, 3.44159174846875*^9}},
 AspectRatioFixed->True],

Cell[TextData[{
 "    What familar group is the quotient group \n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["D", "6"],
     TraditionalForm], "/", 
    RowBox[{"Z", "(", 
     SubscriptBox["D", "6"], ")"}]}], TraditionalForm]]],
 " isomorphic to?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436928943390625*^9, 3.43692895003125*^9}}],

Cell["       ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "7.2 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the normalizer \n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", 
     SubscriptBox["D", "6"]], "(", 
    RowBox[{"{", "x", "}"}], ")"}], TraditionalForm]]],
 " for each of the twelve elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "6"], TraditionalForm]]],
 " \n     listed in problem 7.1. For which elements is the \n     normalizer \
the same subgroup?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436928988390625*^9, 3.4369290190625*^9}, 
   3.4369290860625*^9}],

Cell["", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "7.3 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox["NormalClosure",
  FontWeight->"Bold"],
 " \n     command to find all of the normal \n     subgroups of the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "6"], TraditionalForm]]],
 " given in problem 7.1."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436929087484375*^9, 3.436929120734375*^9}}],

Cell["        ", "Input",
 PageWidth->Infinity],

Cell["    List of normal subgroups:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "7.4 ) The following commands loads a group of\n     order 20 into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436929313125*^9, 3.436929315859375*^9}}],

Cell["\<\
InitGroup[e];
Define[a^4, e]; Define[b^5, e]
Define[1/a, a^3]; Define[1/b, b^4]
Define[b.a, a.b.b]
M = Group[{a, b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43692913546875*^9, 3.436929171171875*^9}, {
  3.436929223625*^9, 3.436929257296875*^9}}],

Cell[TextData[{
 "    Find the conjugacy classes of this group, and \n    use this to find \
all of the normal subgroups of \n    ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["    ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "7.5 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find all of the 2-Sylow \n     and 5-Sylow subgroups of the group ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " defined \n     in problem 7.4. How many of the subgroups are\n     there? \
Does this agree with the prediction \n     given by the third Sylow theorem?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436929311296875*^9, 3.43692931159375*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "7.6 ) Using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", find all non-isomorphic \n     groups of order 21. \n     (Hint: What can \
you determine from Sylow's \n     theorems? Which semi-direct products are \n \
    possible?)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4369293396875*^9}],

Cell["        ", "Input",
 PageWidth->Infinity]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{732, 680},
WindowMargins->{{83, Automatic}, {11, Automatic}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.5,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[568, 21, 321, 9, 160, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[914, 34, 287, 8, 29, "Name",
 Evaluatable->False],
Cell[1204, 44, 619, 15, 70, "Name",
 Evaluatable->False]
}, Closed]],
Cell[1838, 62, 276, 12, 142, "Title",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2139, 78, 104, 3, 32, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2268, 85, 222, 5, 43, "Input",
 InitializationCell->True],
Cell[2493, 92, 181, 2, 32, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2723, 100, 94, 2, 105, "Section",
 Evaluatable->False],
Cell[2820, 104, 1768, 53, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[4613, 161, 292, 11, 156, "Input"],
Cell[4908, 174, 1375, 27, 68, "Output"],
Cell[6286, 203, 12789, 340, 572, "Output"]
}, Open  ]],
Cell[19090, 546, 2160, 71, 323, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[21275, 621, 69, 2, 43, "Input"],
Cell[21347, 625, 324, 6, 42, "Output"]
}, Open  ]],
Cell[21686, 634, 1083, 37, 188, "Text",
 Evaluatable->False],
Cell[22772, 673, 471, 16, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[23268, 693, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[23356, 697, 3383, 127, 410, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[26754, 827, 345, 13, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[27124, 844, 142, 3, 43, "Input"],
Cell[27269, 849, 291, 6, 42, "Output"]
}, Open  ]],
Cell[27575, 858, 381, 13, 68, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[27981, 875, 142, 3, 43, "Input"],
Cell[28126, 880, 1551, 35, 68, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[29714, 920, 191, 4, 43, "Input"],
Cell[29908, 926, 1547, 35, 68, "Output"]
}, Open  ]],
Cell[31470, 964, 443, 13, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[31938, 981, 185, 10, 178, "Input"],
Cell[32126, 993, 705, 14, 42, "Output"]
}, Open  ]],
Cell[32846, 1010, 97, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[32968, 1016, 79, 2, 43, "Input"],
Cell[33050, 1020, 92, 2, 42, "Output"]
}, Open  ]],
Cell[33157, 1025, 1231, 40, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[34413, 1069, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[34501, 1073, 3224, 122, 227, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[37740, 1198, 1211, 41, 262, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[38976, 1243, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[39064, 1247, 3369, 113, 424, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[42448, 1363, 734, 23, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[43207, 1390, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[43295, 1394, 4927, 169, 306, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[48237, 1566, 297, 6, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[48559, 1576, 257, 14, 268, "Input"],
Cell[48819, 1592, 6956, 136, 168, "Output"]
}, Open  ]],
Cell[55790, 1731, 120, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[55935, 1737, 89, 2, 43, "Input"],
Cell[56027, 1741, 1561, 30, 42, "Output"]
}, Open  ]],
Cell[57603, 1774, 395, 13, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[58023, 1791, 80, 2, 43, "Input"],
Cell[58106, 1795, 269, 6, 42, "Output"]
}, Open  ]],
Cell[58390, 1804, 386, 16, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[58801, 1824, 119, 3, 43, "Input"],
Cell[58923, 1829, 7649, 166, 218, "Output"]
}, Open  ]],
Cell[66587, 1998, 74, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[66686, 2004, 139, 3, 43, "Input"],
Cell[66828, 2009, 7650, 166, 193, "Output"]
}, Open  ]],
Cell[74493, 2178, 1171, 37, 213, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[75701, 2220, 118, 2, 143, "Section",
 Evaluatable->False],
Cell[75822, 2224, 2159, 82, 317, "Text",
 Evaluatable->False],
Cell[77984, 2308, 925, 32, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[78934, 2344, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[79022, 2348, 2479, 89, 163, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[81516, 2440, 793, 28, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[82334, 2472, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[82422, 2476, 2107, 75, 261, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[84544, 2554, 676, 25, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[85245, 2583, 254, 9, 133, "Input"],
Cell[85502, 2594, 1347, 26, 68, "Output"]
}, Open  ]],
Cell[86864, 2623, 371, 14, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[87260, 2641, 199, 4, 43, "Input"],
Cell[87462, 2647, 552, 11, 42, "Output"]
}, Open  ]],
Cell[88029, 2661, 418, 14, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[88472, 2679, 187, 4, 43, "Input"],
Cell[88662, 2685, 1370, 27, 68, "Output"]
}, Open  ]],
Cell[90047, 2715, 1111, 40, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[91183, 2759, 169, 4, 43, "Input"],
Cell[91355, 2765, 528, 11, 42, "Output"]
}, Open  ]],
Cell[91898, 2779, 345, 13, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[92268, 2796, 193, 4, 43, "Input"],
Cell[92464, 2802, 270, 6, 42, "Output"]
}, Open  ]],
Cell[92749, 2811, 4024, 139, 640, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[96798, 2954, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[96886, 2958, 2409, 88, 290, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[99310, 3049, 875, 29, 262, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[100210, 3082, 250, 11, 178, "Input"],
Cell[100463, 3095, 722, 14, 42, "Output"]
}, Open  ]],
Cell[101200, 3112, 846, 23, 145, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[102071, 3139, 195, 4, 43, "Input"],
Cell[102269, 3145, 753, 15, 42, "Output"]
}, Open  ]],
Cell[103037, 3163, 374, 11, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[103436, 3178, 148, 3, 43, "Input"],
Cell[103587, 3183, 301, 6, 42, "Output"]
}, Open  ]],
Cell[103903, 3192, 440, 11, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[104368, 3207, 127, 3, 43, "Input"],
Cell[104498, 3212, 727, 14, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[105262, 3231, 129, 3, 43, "Input"],
Cell[105394, 3236, 299, 6, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[105730, 3247, 130, 3, 43, "Input"],
Cell[105863, 3252, 727, 14, 42, "Output"]
}, Open  ]],
Cell[106605, 3269, 1758, 53, 296, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[108400, 3327, 108, 2, 105, "Section",
 Evaluatable->False],
Cell[108511, 3331, 6180, 224, 966, "Text",
 Evaluatable->False],
Cell[114694, 3557, 1375, 38, 292, "Text",
 Evaluatable->False],
Cell[116072, 3597, 1084, 28, 384, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[117181, 3629, 120, 5, 66, "Input"],
Cell[117304, 3636, 2232, 62, 193, "Output"]
}, Open  ]],
Cell[119551, 3701, 118, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[119694, 3707, 150, 3, 43, "Input"],
Cell[119847, 3712, 2550, 72, 218, "Output"]
}, Open  ]],
Cell[122412, 3787, 8642, 236, 1058, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[131079, 4027, 156, 3, 43, "Input"],
Cell[131238, 4032, 2232, 62, 193, "Output"]
}, Open  ]],
Cell[133485, 4097, 513, 15, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[134023, 4116, 164, 3, 43, "Input"],
Cell[134190, 4121, 751, 22, 68, "Output"]
}, Open  ]],
Cell[134956, 4146, 440, 13, 69, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[135421, 4163, 157, 3, 43, "Input"],
Cell[135581, 4168, 1346, 38, 118, "Output"]
}, Open  ]],
Cell[136942, 4209, 369, 10, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[137336, 4223, 73, 2, 43, "Input"],
Cell[137412, 4227, 89, 1, 42, "Output"]
}, Open  ]],
Cell[137516, 4231, 687, 19, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[138228, 4254, 158, 3, 43, "Input"],
Cell[138389, 4259, 2233, 62, 193, "Output"]
}, Open  ]],
Cell[140637, 4324, 219, 6, 66, "Text",
 Evaluatable->False],
Cell[140859, 4332, 2694, 68, 730, "Text",
 Evaluatable->False],
Cell[143556, 4402, 1519, 52, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[145100, 4458, 122, 5, 66, "Input"],
Cell[145225, 4465, 6901, 182, 543, "Output"]
}, Open  ]],
Cell[152141, 4650, 97, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[152263, 4656, 74, 2, 43, "Input"],
Cell[152340, 4660, 89, 1, 42, "Output"]
}, Open  ]],
Cell[152444, 4664, 142, 4, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[152611, 4672, 148, 3, 43, "Input"],
Cell[152762, 4677, 7415, 193, 543, "Output"]
}, Open  ]],
Cell[160192, 4873, 677, 22, 262, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[160894, 4899, 155, 3, 43, "Input"],
Cell[161052, 4904, 6902, 182, 543, "Output"]
}, Open  ]],
Cell[167969, 5089, 91, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[168085, 5095, 73, 2, 43, "Input"],
Cell[168161, 5099, 90, 1, 42, "Output"]
}, Open  ]],
Cell[168266, 5103, 351, 10, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[168642, 5117, 160, 3, 43, "Input"],
Cell[168805, 5122, 6901, 182, 543, "Output"]
}, Open  ]],
Cell[175721, 5307, 91, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[175837, 5313, 73, 2, 43, "Input"],
Cell[175913, 5317, 90, 1, 42, "Output"]
}, Open  ]],
Cell[176018, 5321, 146, 4, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[176189, 5329, 160, 3, 43, "Input"],
Cell[176352, 5334, 6901, 182, 543, "Output"]
}, Open  ]],
Cell[183268, 5519, 95, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[183388, 5525, 73, 2, 43, "Input"],
Cell[183464, 5529, 91, 1, 42, "Output"]
}, Open  ]],
Cell[183570, 5533, 672, 21, 188, "Text",
 Evaluatable->False],
Cell[184245, 5556, 70, 2, 43, "Input"],
Cell[184318, 5560, 2045, 54, 609, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[186388, 5618, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[186476, 5622, 4531, 142, 640, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[191022, 5767, 925, 30, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[191972, 5801, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[192060, 5805, 9687, 343, 830, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[201762, 6151, 609, 21, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[202396, 6176, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[202484, 6180, 4096, 149, 429, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[206595, 6332, 972, 31, 171, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[207592, 6367, 115, 5, 66, "Input"],
Cell[207710, 6374, 2383, 34, 518, "Output"]
}, Open  ]],
Cell[210108, 6411, 195, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[210328, 6420, 146, 3, 43, "Input"],
Cell[210477, 6425, 2699, 48, 543, "Output"]
}, Open  ]],
Cell[213191, 6476, 552, 11, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[213768, 6491, 147, 3, 43, "Input"],
Cell[213918, 6496, 2384, 34, 518, "Output"]
}, Open  ]],
Cell[216317, 6533, 91, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[216433, 6539, 73, 2, 43, "Input"],
Cell[216509, 6543, 91, 1, 42, "Output"]
}, Open  ]],
Cell[216615, 6547, 97, 2, 41, "Text",
 Evaluatable->False],
Cell[216715, 6551, 240, 7, 115, "Text",
 Evaluatable->False],
Cell[216958, 6560, 2378, 55, 586, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[219373, 6620, 112, 2, 105, "Section",
 Evaluatable->False],
Cell[219488, 6624, 515, 14, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[220028, 6642, 94, 2, 43, "Input"],
Cell[220125, 6646, 2232, 62, 193, "Output"]
}, Open  ]],
Cell[222372, 6711, 106, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[222503, 6717, 150, 3, 43, "Input"],
Cell[222656, 6722, 2523, 71, 218, "Output"]
}, Open  ]],
Cell[225194, 6796, 974, 17, 384, "Text",
 Evaluatable->False],
Cell[226171, 6815, 242, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[226438, 6827, 302, 11, 66, "Input"],
Cell[226743, 6840, 6903, 182, 543, "Output"],
Cell[233649, 7024, 7418, 193, 543, "Output"]
}, Open  ]],
Cell[241082, 7220, 154, 4, 41, "Text",
 Evaluatable->False],
Cell[241239, 7226, 1605, 50, 455, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[242869, 7280, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[242957, 7284, 1093, 38, 87, "SmallText",
 Evaluatable->False],
Cell[244053, 7324, 2513, 84, 99, "Text",
 Evaluatable->False],
Cell[246569, 7410, 1351, 48, 149, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[247935, 7461, 1187, 35, 266, "Text",
 Evaluatable->False],
Cell[249125, 7498, 195, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[249345, 7509, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[249433, 7513, 962, 29, 161, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[250410, 7545, 344, 10, 69, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[250779, 7559, 115, 5, 66, "Input"],
Cell[250897, 7566, 2362, 34, 518, "Output"]
}, Open  ]],
Cell[253274, 7603, 219, 6, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[253518, 7613, 155, 4, 71, "Input"],
Cell[253676, 7619, 2360, 34, 518, "Output"],
Cell[256039, 7655, 69, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[256145, 7661, 156, 4, 71, "Input"],
Cell[256304, 7667, 195, 5, 42, "Output"],
Cell[256502, 7674, 70, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[256609, 7680, 157, 4, 71, "Input"],
Cell[256769, 7686, 129, 3, 42, "Output"],
Cell[256901, 7691, 70, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[257008, 7697, 157, 4, 71, "Input"],
Cell[257168, 7703, 141, 3, 42, "Output"],
Cell[257312, 7708, 70, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[257419, 7714, 157, 4, 71, "Input"],
Cell[257579, 7720, 189, 5, 42, "Output"],
Cell[257771, 7727, 67, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[257875, 7733, 157, 4, 71, "Input"],
Cell[258035, 7739, 189, 5, 42, "Output"],
Cell[258227, 7746, 70, 1, 42, "Output"]
}, Open  ]],
Cell[258312, 7750, 845, 31, 123, "Text",
 Evaluatable->False],
Cell[259160, 7783, 844, 27, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[260029, 7814, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[260117, 7818, 2070, 68, 209, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[262202, 7889, 1046, 37, 178, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[263273, 7930, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[263361, 7934, 3989, 144, 429, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[267365, 8081, 1156, 36, 264, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[268546, 8121, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[268634, 8125, 5137, 183, 535, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[273786, 8311, 1128, 34, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[274939, 8349, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[275027, 8353, 7752, 275, 879, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[282794, 8631, 1546, 54, 311, "Text",
 Evaluatable->False],
Cell[284343, 8687, 1937, 58, 241, "Text",
 Evaluatable->False],
Cell[286283, 8747, 2004, 68, 337, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[288312, 8819, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[288400, 8823, 17854, 592, 1428, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[306269, 9418, 668, 23, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[306962, 9445, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[307050, 9449, 2209, 86, 177, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[309274, 9538, 1811, 65, 311, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[311110, 9607, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[311198, 9611, 6479, 217, 417, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[317692, 9831, 261, 8, 66, "Text",
 Evaluatable->False],
Cell[317956, 9841, 752, 28, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[318733, 9873, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[318821, 9877, 5717, 213, 684, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[324553, 10093, 6142, 183, 1217, "Text",
 Evaluatable->False],
Cell[330698, 10278, 1157, 39, 213, "Text",
 Evaluatable->False],
Cell[331858, 10319, 3450, 115, 525, "Text",
 Evaluatable->False],
Cell[335311, 10436, 158, 7, 88, "Input"],
Cell[335472, 10445, 103, 5, 66, "Input"],
Cell[335578, 10452, 986, 37, 68, "Text",
 Evaluatable->False],
Cell[336567, 10491, 82, 2, 43, "Input"],
Cell[336652, 10495, 103, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[336780, 10501, 117, 6, 88, "Input"],
Cell[336900, 10509, 2708, 53, 93, "Output"],
Cell[339611, 10564, 129, 1, 32, "Print"]
}, Open  ]],
Cell[339755, 10568, 207, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[339987, 10579, 67, 2, 43, "Input"],
Cell[340057, 10583, 1455, 27, 68, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[341549, 10615, 67, 2, 43, "Input"],
Cell[341619, 10619, 2240, 44, 68, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[343896, 10668, 67, 2, 43, "Input"],
Cell[343966, 10672, 525, 9, 42, "Output"]
}, Open  ]],
Cell[344506, 10684, 5074, 164, 791, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[349605, 10852, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[349693, 10856, 8812, 279, 1381, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[358520, 11138, 836, 21, 164, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[359393, 11164, 208, 7, 105, "Section",
 Evaluatable->False],
Cell[359604, 11173, 156, 6, 43, "Input"],
Cell[359763, 11181, 334, 12, 66, "Text",
 Evaluatable->False],
Cell[360100, 11195, 304, 10, 133, "Input"],
Cell[360407, 11207, 411, 14, 66, "Text",
 Evaluatable->False],
Cell[360821, 11223, 71, 2, 43, "Input"],
Cell[360895, 11227, 635, 21, 121, "Text",
 Evaluatable->False],
Cell[361533, 11250, 64, 2, 43, "Input"],
Cell[361600, 11254, 445, 15, 90, "Text",
 Evaluatable->False],
Cell[362048, 11271, 47, 1, 43, "Input"],
Cell[362098, 11274, 99, 2, 41, "Text",
 Evaluatable->False],
Cell[362200, 11278, 269, 8, 66, "Text",
 Evaluatable->False],
Cell[362472, 11288, 274, 9, 133, "Input"],
Cell[362749, 11299, 253, 8, 90, "Text",
 Evaluatable->False],
Cell[363005, 11309, 43, 1, 43, "Input"],
Cell[363051, 11312, 492, 12, 139, "Text",
 Evaluatable->False],
Cell[363546, 11326, 42, 1, 43, "Input"],
Cell[363591, 11329, 363, 10, 139, "Text",
 Evaluatable->False],
Cell[363957, 11341, 47, 1, 43, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
