(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    679514,      16279]
NotebookOptionsPosition[    661531,      15716]
NotebookOutlinePosition[    662305,      15745]
CellTagsIndexPosition[    662262,      15742]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Abstract Algebra: An Interactive Approach", "Special2",
 Evaluatable->False,
 CellChangeTimes->{{3.436953091578125*^9, 3.436953111421875*^9}},
 TextAlignment->Center,
 FontFamily->"Arial",
 FontSize->36,
 FontWeight->"Bold",
 FontSlant->"Italic",
 FontColor->RGBColor[1, 0, 1],
 Background->RGBColor[0.500008, 1, 1]],

Cell["\[Copyright]2009  William Paulsen", "Name",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436953124953125*^9, 3.4369531255*^9}},
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]],

Cell["\<\
Chapter 8
Solvable and Insoluble Groups\
\>", "Title",
 CellFrame->True,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 TextAlignment->Center,
 FontFamily->"New York",
 FontSize->24,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell["Initialization", "Special2",
 Evaluatable->False,
 TextAlignment->Center,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["<<c:\\math\\group.m", "Input",
 PageWidth->Infinity,
 InitializationCell->True,
 CellChangeTimes->{{3.43695313409375*^9, 3.43695313628125*^9}},
 AspectRatioFixed->True],

Cell[BoxData["\<\"Initialization Done\"\>"], "Print",
 CellChangeTimes->{3.43695314*^9, 3.442675676461375*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Subnormal Series and the Jordan-H\[ODoubleDot]lder Theorem\
\>", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In this chapter we will study the concept of ",
 StyleBox["solvable",
  FontSlant->"Italic"],
 " groups. Every group will be classified either as solvable or insoluble, \
and in fact most of the groups we have looked at so far turn out to be \
solvable. \n\nSolvable groups play a key role in studying polynomial \
equations. Whether or not a given polynomial can be solved in terms of \
radicals (square roots, cube roots, etc.) depends on whether a certain group \
corresponding to that polynomial is a solvable group. In fact this \
application is the origin of the term \"solvable group.\"\n\nBefore we \
introduce the true definition of a solvable group, we must make some \
preliminary notations. We have already encountered situations in which we had \
a normal subgroup of a normal subgroup, such as in the second isomorphism \
theorem. But suppose we have a whole series of subgroups of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", each one fitting inside of the previous one like Russian dolls.\n\n\
DEFINITION 8.1:\nA ",
 StyleBox["subnormal series",
  FontSlant->"Italic"],
 " for a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a sequence \n\n                       ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "0"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{",", " ", 
    SubscriptBox["G", "1"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "2"], TraditionalForm]]],
 ", \[Ellipsis] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "n"], TraditionalForm]]],
 "  \n                       \nof subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " such that \n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["G", 
       RowBox[{"0", " "}]], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "2"], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "n"]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 ".\n              \nwhere each ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "i"], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"i", "-", "1"}]], TraditionalForm]]],
 "  for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"i", " ", "=", " ", "1"}], ",", " ", "2", ",", "\[Ellipsis]", ",",
     " ", "n"}], TraditionalForm]]],
 ".\n\nA subnormal series is called a ",
 StyleBox["normal series",
  FontSlant->"Italic"],
 " if it satisfies the stronger condition that all of the groups ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "i"], TraditionalForm]]],
 " are normal subgroups of the original group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". We will be mainly interested in subnormal series, but there are a few of \
the exercises regarding normal series.\n\nLet us look at an example of a \
subnormal series. Consider the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ", for which we have seen a normal subgroup of order 4, namely"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695320128125*^9, 3.43695320321875*^9}}],

Cell[CellGroupData[{

Cell["K = Group[{P[2,1,4,3], P[4,3,2,1]}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"3", ",", "4", ",", "1", ",", "2"}], "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "3", ",", "2", ",", "1"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442674532867625*^9}]
}, Open  ]],

Cell[TextData[{
 "Certainly the identity element is a normal subgroup of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", so we can write\n\n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", 
     RowBox[{"4", " "}]], " ", "\[SupersetEqual]", " ", "K", " ", 
    "\[SupersetEqual]", " ", 
    RowBox[{"{", 
     RowBox[{"P", "[", " ", "]"}], "}"}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "which would be a subnormal series of length ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "2"}], TraditionalForm]]],
 ". Is there a way that we can make a longer series out of this one?  Because \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " is also a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 ", we can slip this group into our series. Also, the group ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " contains some proper subgroups, which will be normal subgroups of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " since ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is abelian. For an example of a proper subgroup, consider"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["H = Group[{P[2,1,4,3]}]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"2", ",", "1", ",", "4", ",", "3"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442674536805125*^9}]
}, Open  ]],

Cell[TextData[{
 "Therefore, we have a longer subnormal series for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ":\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "4"], " ", "\[SupersetEqual]", " ", 
    SubscriptBox["A", "4"], " ", "\[SupersetEqual]", " ", "K", " ", 
    "\[SupersetEqual]", " ", "H", " ", "\[SupersetEqual]", " ", 
    RowBox[{"{", 
     RowBox[{"P", "[", " ", "]"}], "}"}]}], TraditionalForm]]],
 ".\n              \nWe say that this new subnormal series is a ",
 StyleBox["refinement",
  FontSlant->"Italic"],
 " of the first subnormal series."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436953433390625*^9, 3.43695346121875*^9}}],

Cell[TextData[{
 "DEFINITION 8.2:\nWe say that a subnormal (or normal) series \n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["H", 
       RowBox[{"0", " "}]], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["H", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["H", "2"], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["H", "n"]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 ".\n         \nis a ",
 StyleBox["refinement ",
  FontSlant->"Italic"],
 "of the subnormal (or normal) series\n \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["G", 
       RowBox[{"0", " "}]], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "2"], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "n"]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 ".\n               \nif each subgroup ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "i"], TraditionalForm]]],
 " appears as ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["H", "j"], TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695328653125*^9, 3.436953421296875*^9}}],

Cell[TextData[{
 "Is there a way that we can refine our subnormal series to produce an even \
longer chain? Our definition did not exclude the possibility of two groups in \
the series being the same, so we could consider\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "4"], " ", "\[SupersetEqual]", " ", 
    SubscriptBox["A", "4"], " ", "\[SupersetEqual]", " ", 
    SubscriptBox["A", "4"], " ", "\[SupersetEqual]", " ", "K", " ", 
    "\[SupersetEqual]", " ", "H", " ", "\[SupersetEqual]", " ", "H", " ", 
    "\[SupersetEqual]", " ", "H", " ", "\[SupersetEqual]", " ", 
    RowBox[{"{", 
     RowBox[{"P", "[", " ", "]"}], "}"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695328653125*^9, 3.436953425640625*^9}}],

Cell[TextData[{
 "This is certainly a longer subnormal series, but it is rather pointless to \
repeat the same subgroup in the series. Let us make the following definition:\
\n\nDEFINITION 8.3:\nA",
 StyleBox[" composition series",
  FontSlant->"Italic"],
 " of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a subnormal series \n \n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["G", 
       RowBox[{"0", " "}]], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "2"], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "n"]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 ",\n            \nfor which each subgroup is smaller than the proceeding \
subgroup, and for which there is no refinement that includes additional \
subgroups."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436953253296875*^9, 3.43695327396875*^9}, 
   3.43695348621875*^9}],

Cell[TextData[{
 "Using this definition, we see that \n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "4"], " ", "\[SupersetEqual]", " ", 
    SubscriptBox["A", "4"], " ", "\[SupersetEqual]", " ", "K", " ", 
    "\[SupersetEqual]", " ", "H", " ", "\[SupersetEqual]", " ", 
    RowBox[{"{", 
     RowBox[{"P", "[", " ", "]"}], "}"}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436953253296875*^9, 3.43695327396875*^9}, {
  3.43695348621875*^9, 3.436953489015625*^9}}],

Cell[TextData[{
 "is a composition series for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ", since no subgroups are repeated, and there simply isn't enough room \
between two of these subgroups to slip in another subgroup. For example, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " is half of the size of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ", so any subgroup containing more than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " must be all of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ". In fact we can easily test to see whether a subnormal series is a \
composition series."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\nPROPOSITION 8.1\nThe subnormal series\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["G", 
       RowBox[{"0", " "}]], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "2"], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "n"]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 "\n              \nis a composition series if, and only if, all of the \
quotient groups ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"k", "-", "1"}]], " ", "/", " ", 
    SubscriptBox["G", "k"]}], TraditionalForm]]],
 " are non-trivial simple groups."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695350421875*^9, 3.436953504375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Note that if there are no repeated subgroups in the subnormal series then \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"i", "-", "1"}]], "/", 
    SubscriptBox["G", "i"]}], TraditionalForm]]],
 "  must contain at least two elements. Likewise, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"i", "-", "1"}]], "/", 
    SubscriptBox["G", "i"]}], TraditionalForm]]],
 "  is non-trivial, then ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"i", "-", "1"}]], TraditionalForm]]],
 " is not equal to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "i"], TraditionalForm]]],
 ". So the quotient groups are non-trivial if, and only if, there are no \
repeated subgroups in the subnormal series.\n\nSuppose that the subnormal \
series is not a decomposition series yet does not repeat any subgroups. Then \
there must be an additional group ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 "  that we can add between ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "k"], TraditionalForm]]],
 ", so that\n\n                            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"k", "-", "1"}]], " ", "\[SupersetEqual]", " ", "H", " ", 
    "\[SupersetEqual]", " ", 
    SubscriptBox["G", "k"]}], TraditionalForm]]],
 ",\n                            \nwhere ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "k"], TraditionalForm]]],
 "  is a normal subgroup of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". Then by lemma 4.3, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "/", 
    SubscriptBox["G", "k"]}], TraditionalForm]]],
 "  will be a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"k", "-", "1"}]], "/", " ", 
    SubscriptBox["G", "k"]}], TraditionalForm]]],
 ", and since ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is neither ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "k"], TraditionalForm]]],
 ", we have a proper normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"k", "-", "1"}]], "/", " ", 
    RowBox[{
     SubscriptBox["G", "k"], "."}]}], TraditionalForm]]],
 "\n\nNow suppose that there is a proper normal subgroup ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"k", "-", "1"}]], "/", 
    RowBox[{
     SubscriptBox["G", "k"], "."}]}], TraditionalForm]]],
 "  Can we then lift ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " to find a suitable subgroup ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " to fit between ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "k"], TraditionalForm]]],
 "? If we consider the canonical homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]]],
 " to the quotient group ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["G", 
      RowBox[{"k", "-", "1"}]], "/", 
     SubscriptBox["G", "k"]}], ","}], TraditionalForm]]],
 " we can take ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["\[Phi]", 
      RowBox[{"-", "1"}]], "(", "N", ")"}]}], TraditionalForm]]],
 ". Then since ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"k", "-", "1"}]], "/", 
    SubscriptBox["G", "k"]}], TraditionalForm]]],
 ", by corollary 4.2 ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " will be a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"k", "-", "1"}]], "."}], TraditionalForm]]],
 " Also, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "k"], TraditionalForm]]],
 " will be a normal subgroup of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", for ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]]],
 ". Because ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " has at least two elements, ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " will be strictly larger than the kernel of ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 ", yet since ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is not the entire image of ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " will be strictly smaller than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"k", "-", "1"}]], TraditionalForm]]],
 ". Therefore, the subnormal series is not a composition series.\n\nThus, a \
subnormal series is a composition series if, and only if, the quotient groups \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"k", "-", "1"}]], "/", " ", 
    SubscriptBox["G", "k"]}], TraditionalForm]]],
 "  are non-trivial simple groups."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369535473125*^9, 3.436953617359375*^9}}]
}, Open  ]],

Cell[TextData[{
 "The quotient groups ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"k", "-", "1"}]], " ", "/", " ", 
    SubscriptBox["G", "k"]}], TraditionalForm]]],
 " in a composition series for ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " are called the ",
 StyleBox["composition factors",
  FontSlant->"Italic"],
 " of the composition series.\n\nFor example, let us find the composition \
factors for the composition series for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ":\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "4"], " ", "\[SupersetEqual]", " ", 
    SubscriptBox["A", "4"], " ", "\[SupersetEqual]", " ", "K", " ", 
    "\[SupersetEqual]", " ", "H", " ", "\[SupersetEqual]", " ", 
    RowBox[{"{", 
     RowBox[{"P", "[", "]"}], "}"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43695365984375*^9}],

Cell[TextData[{
 "We have \n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["S", "4"], "/", 
     SubscriptBox["A", "4"]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["A", "4"], "/", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "3"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"K", "/", "H"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 ",  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "/", 
     RowBox[{"{", 
      RowBox[{"P", "[", "]"}], "}"}]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "So the decomposition factors are ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ", which \nare all simple groups, as we would expect. \n\nIt is certainly \
possible for a group to have more than one composition series. For example, \
if we pick the subgroup "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["B = Group[{P[4,3,2,1]} ]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"P", "[", "]"}], ",", 
   RowBox[{"P", "[", 
    RowBox[{"4", ",", "3", ",", "2", ",", "1"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4426745440395*^9}]
}, Open  ]],

Cell[TextData[{
 "then ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Thus we would have the composition series\n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "4"], " ", "\[SupersetEqual]", " ", 
    SubscriptBox["A", "4"], " ", "\[SupersetEqual]", " ", "K", " ", 
    "\[SupersetEqual]", " ", "B", " ", "\[SupersetEqual]", " ", 
    RowBox[{"{", 
     RowBox[{"P", "[", "]"}], "}"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Even though this is a different composition series, the composition factors \
are isomorphically the same. Will this happen all of the time? To show that \
it will, we first need the following lemmas:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\nLemma 8.1\nLet ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 " be three subgroups of the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " being a subgroup of ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Y", "\[CenterDot]", "Z"}], " ", "=", " ", 
    RowBox[{"Z", "\[CenterDot]", 
     RowBox[{"Y", "."}]}]}], TraditionalForm]]],
 " Then\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", "\[Intersection]", 
     RowBox[{"(", 
      RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"Y", "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"X", "\[Intersection]", "Z"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"X", "\[Intersection]", "Z"}], ")"}], "\[CenterDot]", 
      "Y"}]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436953692921875*^9, 3.436953693078125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Note that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"X", " ", "\[Intersection]", " ", "Z"}], ")"}], " ", 
    "\[SubsetEqual]", " ", "X"}], TraditionalForm]]],
 ",   and  since  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Y", " ", "\[SubsetEqual]", " ", "X"}], TraditionalForm]]],
 " ,   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Y", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"X", " ", "\[Intersection]", " ", "Z"}], ")"}]}], " ", 
    "\[SubsetEqual]", " ", "X", " "}], TraditionalForm]]],
 ".\n                    \nAlso,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"X", "\[Intersection]", "Z"}], ")"}], " ", "\[SubsetEqual]", 
     " ", "Z"}], ","}], TraditionalForm]]],
 "   so",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{"Y", "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"X", "\[Intersection]", "Z"}], ")"}]}], " ", "\[SubsetEqual]", 
     " ", 
     RowBox[{"Y", "\[CenterDot]", "Z"}]}]}], TraditionalForm]]],
 ". Hence, \n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{"Y", "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"X", "\[Intersection]", "Z"}], ")"}]}], " ", "\[SubsetEqual]", 
     " ", 
     RowBox[{"X", "\[Intersection]", 
      RowBox[{"(", 
       RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}]}]}], TraditionalForm]]],
 ".\n                     \nAll we need to do is prove the inclusion in the \
other direction. Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"x", " ", "\[Element]", " ", 
     RowBox[{"X", "\[Intersection]", 
      RowBox[{"(", 
       RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}]}]}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ", and can also be written as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "z"}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 ". But then",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"z", " ", "=", " ", 
     RowBox[{
      SuperscriptBox["y", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "x"}]}]}], TraditionalForm]]],
 " would be in both ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 ". Thus, \n\n                                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{"x", " ", "=", 
     RowBox[{
      RowBox[{"y", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["y", 
          RowBox[{"-", "1"}]], "\[CenterDot]", "x"}], ")"}]}], "\[Element]", 
      " ", 
      RowBox[{"Y", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"X", "\[Intersection]", "Z"}], ")"}]}]}]}]}], 
   TraditionalForm]]],
 ".\n                                     \nTherefore, we have inclusions in \
both directions, so\n\n                               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{"Y", "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"X", "\[Intersection]", "Z"}], ")"}]}], " ", "=", " ", 
     RowBox[{"X", "\[Intersection]", 
      RowBox[{"(", 
       RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}]}]}], TraditionalForm]]],
 ".\n                     \nSo far, we haven't used the fact that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Y", "\[CenterDot]", "Z"}], " ", "=", " ", 
    RowBox[{"Z", "\[CenterDot]", "Y"}]}], TraditionalForm]]],
 ". By lemma 4.4, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Y", "\[CenterDot]", "Z"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and so the intersection of ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Y", "\[CenterDot]", "Z"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". So by lemma 4.4 again, we have \n\n                               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{"Y", "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"X", "\[Intersection]", "Z"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"X", "\[Intersection]", "Z"}], ")"}], "\[CenterDot]", 
      "Y"}]}]}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369537476875*^9, 3.43695385159375*^9}, {
  3.43695389409375*^9, 3.436953907578125*^9}, {3.4417101932764997`*^9, 
  3.4417101965264997`*^9}, {3.4417103310733747`*^9, 3.4417103443858747`*^9}}]
}, Open  ]],

Cell[TextData[{
 "\nLemma 8.2\nLet ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 " be three subgroups of the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " being a normal subgroup of ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 " a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".  Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Y", "\[CenterDot]", "Z"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", "\[CenterDot]", "Z"}], TraditionalForm]]],
 ", and \n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"X", "\[CenterDot]", "Z"}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}], " ", "\[TildeTilde]", " ", 
    
    RowBox[{"X", "/", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"X", 
        FormBox[
         RowBox[{"\[Intersection]", 
          RowBox[{"(", 
           RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}],
         TraditionalForm]}], ")"}], "."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695392278125*^9, 3.43695392353125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Y", "\[CenterDot]", "Z"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", "\[CenterDot]", "Z"}], TraditionalForm]]],
 " are subgroups of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", Cell[""]}], TraditionalForm]]],
 " by lemma 4.5.   If we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "z"}], TraditionalForm]]],
 " be in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Y", "\[CenterDot]", "Z"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "w"}], TraditionalForm]]],
 " be in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", "\[CenterDot]", "Z"}], ","}], TraditionalForm]]],
 " then\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", "\[CenterDot]", "w"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"y", "\[CenterDot]", "z"}], ")"}], "\[CenterDot]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "\[CenterDot]", "w"}], ")"}], 
        RowBox[{"-", "1"}]]}], " ", "=", " ", 
      RowBox[{"x", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"y", "\[CenterDot]", 
         SuperscriptBox["x", 
          RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", 
         SuperscriptBox["y", 
          RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", "w", "\[CenterDot]", 
       "y", "\[CenterDot]", "z", "\[CenterDot]", 
       SuperscriptBox["w", 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]]}]}], " ", "\n", 
     "\t\t\t                                    ", 
     RowBox[{"=", "  ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", "\[CenterDot]", "y", "\[CenterDot]", 
         SuperscriptBox["x", 
          RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"x", "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["y", 
             RowBox[{"-", "1"}]], "\[CenterDot]", "w", "\[CenterDot]", "y"}], 
           ")"}], "\[CenterDot]", "z", "\[CenterDot]", 
          SuperscriptBox["w", 
           RowBox[{"-", "1"}]], "\[CenterDot]", 
          SuperscriptBox["x", 
           RowBox[{"-", "1"}]]}], ")"}], "."}]}]}], " "}]}], 
   TraditionalForm]]],
 "\n\nNow, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y", "\[CenterDot]", 
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ". Likewise, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "w", "\[CenterDot]", "y"}], 
   TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["y", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "w", "\[CenterDot]", "y"}], ")"}],
     "\[CenterDot]", "z", "\[CenterDot]", 
    SuperscriptBox["w", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["y", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "w", "\[CenterDot]", "y"}], ")"}],
     "\[CenterDot]", "z", "\[CenterDot]", 
    SuperscriptBox["w", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Therefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "\[CenterDot]", "w"}], ")"}], "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{"y", "\[CenterDot]", "z"}], ")"}], "\[CenterDot]", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "\[CenterDot]", "w"}], ")"}], 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Y", "\[CenterDot]", "Z"}], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Y", "\[CenterDot]", "Z"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", "\[CenterDot]", 
    RowBox[{"Z", "."}]}], TraditionalForm]]],
 "\n\nWe now can use the third isomorphism theorem (4.3.9), using ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", " ", "=", " ", 
    RowBox[{"Y", "\[CenterDot]", "Z"}]}], TraditionalForm]]],
 ". We have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"X", "\[CenterDot]", "K"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"X", "\[CenterDot]", "Y", "\[CenterDot]", "Z"}], " ", "=", " ", 
      
      RowBox[{"X", "\[CenterDot]", "Z"}]}]}], ","}], TraditionalForm]]],
 " since ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ". So\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    FormBox[
     RowBox[{" ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"X", "\[CenterDot]", "Z"}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}], "  ", "=", "  ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"X", "\[CenterDot]", "K"}], ")"}], "/", "K"}], " ", 
         StyleBox["\[TildeTilde]",
          FontSize->16], " ", 
         RowBox[{"X", "/", 
          RowBox[{"(", 
           RowBox[{"X", "\[Intersection]", "K"}], ")"}]}]}], " ", "=", " ", 
        RowBox[{"X", "/", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"X", "\[Intersection]", 
            RowBox[{"(", 
             RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}], ")"}], "."}]}], 
        " "}]}]}],
     TraditionalForm]}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695394440625*^9, 3.43695394496875*^9}}]
}, Open  ]],

Cell["\<\
We are now ready to show that any two composition series for a group are \
related. We do this by proving a theorem that is true for any two subnormal \
series, and then apply that theorem to a composition series.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\nTHEOREM 8.1:  THE REFINEMENT THEOREM\nSuppose that there are two \
subnormal series for a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". That is, there are subgroups ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "i"], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "j"], TraditionalForm]]],
 "  such that \n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["A", 
       RowBox[{"0", " "}]], "\[SupersetEqual]", " ", 
      SubscriptBox["A", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["A", "2"], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["A", "n"]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 ",\nand\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["B", "0"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["B", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["B", "2"], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["B", "m"]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 ",\n                 \nwhere each ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "i"], TraditionalForm]]],
 "  is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", 
    RowBox[{"i", "-", "1"}]], TraditionalForm]]],
 ", and each ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "j"], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", 
    RowBox[{"j", "-", "1"}]], TraditionalForm]]],
 ".  Then it is possible to refine both series by inserting the subgroups\n\n \
  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", 
     RowBox[{"i", "-", "1"}]], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["A", 
       RowBox[{"i", ",", "0"}]], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["A", 
       RowBox[{"i", ",", "1"}]], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["A", 
       RowBox[{"i", ",", "2"}]], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["A", 
       RowBox[{"i", ",", "m"}]]}], " ", "=", " ", 
     SubscriptBox["A", "i"]}]}], TraditionalForm]]],
 ",   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"i", "=", "1"}], ",", "2", ",", "\[Ellipsis]", ",", "n"}], 
   TraditionalForm]]],
 "\n   \n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["B", 
      RowBox[{"j", "-", "1"}]], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["B", 
        RowBox[{"j", ",", "0"}]], " ", "\[SupersetEqual]", " ", 
       SubscriptBox["B", 
        RowBox[{"j", ",", "1"}]], " ", "\[SupersetEqual]", " ", 
       SubscriptBox["B", 
        RowBox[{"j", ",", "2"}]], " ", "\[SupersetEqual]", " ", 
       "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
       SubscriptBox["B", 
        RowBox[{"j", ",", "n"}]]}], " ", "=", " ", 
      SubscriptBox["B", "j"]}]}], ","}], TraditionalForm]]],
 "     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"j", "=", "1"}], ",", "2", ",", "\[Ellipsis]", ",", "m"}], 
   TraditionalForm]]],
 "\n   \nin such a way that\n                                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["A", 
      RowBox[{"i", ",", 
       RowBox[{"j", "-", "1"}]}]], "/", " ", 
     SubscriptBox["A", 
      RowBox[{"i", ",", "j"}]]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["B", 
      RowBox[{"j", ",", 
       RowBox[{"i", "-", "1"}]}]], "/", 
     SubscriptBox["B", 
      RowBox[{"j", ",", "i"}]]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695401059375*^9, 3.436954010765625*^9}, {
  3.43695411375*^9, 3.43695411375*^9}, {3.436954160359375*^9, 
  3.436954160359375*^9}, {3.43695419771875*^9, 3.436954214375*^9}, {
  3.4417104257764997`*^9, 3.4417105161046247`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We let \n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", 
     RowBox[{"i", ",", "j"}]], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["A", 
          RowBox[{"i", "-", "1"}]], " ", "\[Intersection]", " ", 
         SubscriptBox["B", "j"]}], ")"}], " ", "\[CenterDot]", " ", 
       SubscriptBox["A", "i"]}], "     ", "and", "     ", 
      SubscriptBox["B", 
       RowBox[{"j", ",", "i"}]]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["B", 
         RowBox[{"j", "-", "1"}]], " ", "\[Intersection]", " ", 
        SubscriptBox["A", "i"]}], ")"}], "\[CenterDot]", 
      SubscriptBox["B", "j"]}]}]}], TraditionalForm]]],
 ".\n      \nTo see that these fit the conditions we need, we first want to \
show that these are groups. Note that both\n        \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["A", 
         RowBox[{"i", "-", "1"}]], "\[Intersection]", " ", 
        SubscriptBox["B", 
         RowBox[{"j", "-", "1"}]]}], ")"}], "     ", "and", "      ", "Y"}], 
     " ", "=", " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["A", 
        RowBox[{"i", "-", "1"}]], " ", "\[Intersection]", " ", 
       SubscriptBox["B", "j"]}], ")"}]}]}], TraditionalForm]]],
 "\n               \nare subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", 
    RowBox[{"i", "-", "1"}]], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ", and Z = ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "i"], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", 
    RowBox[{"i", "-", "1"}]], TraditionalForm]]],
 ".\n\nSo by lemma 4.3.7, both ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["A", 
      RowBox[{"i", ",", 
       RowBox[{"j", "-", "1"}]}]],
     TraditionalForm], " ", "=", " ", 
    RowBox[{"X", "\[CenterDot]", "Z"}]}], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", 
     RowBox[{"i", ",", "j"}]], " ", "=", " ", 
    RowBox[{"Y", "\[CenterDot]", "Z"}]}], TraditionalForm]]],
 " are subgroups of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", 
     RowBox[{"i", "-", "1"}]], "."}], TraditionalForm]]],
 "  We can now use lemma 8.1.3, using ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", 
    RowBox[{
     SubscriptBox["A", 
      RowBox[{"i", "-", "1"}]], ".", "  "}]}], TraditionalForm]]],
 "Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "j"], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", 
    RowBox[{"j", "-", "1"}]], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ", so by lemma 8.1.3,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Y", "\[CenterDot]", "Z"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", "\[CenterDot]", "Z"}], ","}], TraditionalForm]]],
 " and  \n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    FormBox[
     RowBox[{
      RowBox[{
       SubscriptBox["A", 
        RowBox[{"i", ",", 
         RowBox[{"j", "-", "1"}]}]], "/", 
       SubscriptBox["A", 
        RowBox[{"i", ",", "j"}]]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"X", "\[CenterDot]", "Z"}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}], "  ", 
       StyleBox["\[TildeTilde]",
        FontSize->16], "  ", 
       RowBox[{"X", "/", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"X", "\[Intersection]", 
           RowBox[{"(", 
            RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}], ")"}], "."}]}], 
       " "}]}],
     TraditionalForm]}], TraditionalForm]]],
 "\n   \nNow lemma 8.1.2 comes into use. Since ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ", \n\n",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{
        RowBox[{"X", "\[Intersection]", 
         RowBox[{"(", 
          RowBox[{"Y", "\[CenterDot]", "Z"}], ")"}]}], " ", "=", " ", 
        RowBox[{
         RowBox[{"Y", "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{"X", "\[Intersection]", "Z"}], ")"}]}], " ", "=", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["A", 
             RowBox[{"i", "-", "1"}]], "\[Intersection]", " ", 
            SubscriptBox["B", "j"]}], ")"}], "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["A", 
             RowBox[{"i", "-", "1"}]], "\[Intersection]", 
            SubscriptBox["B", "j"], "\[Intersection]"}]}]}]}]}],
       TraditionalForm], 
      SubscriptBox["A", "i"]}], ")"}], " "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                      ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["A", 
          RowBox[{"i", "-", "1"}]], " ", "\[Intersection]", " ", 
         SubscriptBox["B", "j"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["A", "i"], " ", "\[Intersection]", " ", 
         SubscriptBox["B", 
          RowBox[{"j", "-", "1"}]]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["A", "i"], "\[Intersection]", 
         SubscriptBox["B", 
          RowBox[{"j", "-", "1"}]]}], ")"}], "\[CenterDot]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["A", 
           RowBox[{"i", "-", "1"}]], "\[Intersection]", 
          SubscriptBox["B", "j"]}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]}]],
 "\nThus, \n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["A", 
      RowBox[{"i", ",", 
       RowBox[{"j", "-", "1"}]}]], "/", 
     SubscriptBox["A", 
      RowBox[{"i", ",", "j"}]]}], 
    StyleBox[" ",
     FontSize->16], 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["A", 
        RowBox[{"i", "-", "1"}]], "\[Intersection]", 
       SubscriptBox["B", 
        RowBox[{"j", "-", "1"}]]}], ")"}], "/", 
     RowBox[{"[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["A", 
          RowBox[{"i", "-", "1"}]], "\[Intersection]", 
         SubscriptBox["B", "j"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["A", "i"], " ", "\[Intersection]", " ", 
         SubscriptBox["B", 
          RowBox[{"j", "-", "1"}]]}], ")"}]}]}]}]}], TraditionalForm]]],
 "].\n      \nBy switching the roles of the two series we find by the exact \
same argument that \n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["B", 
      RowBox[{"j", ",", 
       RowBox[{"i", "-", "1"}]}]], "/", 
     SubscriptBox["B", 
      RowBox[{"j", ",", "i"}]]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["B", 
        RowBox[{"j", "-", "1"}]], "\[Intersection]", 
       SubscriptBox["A", 
        RowBox[{"i", "-", "1"}]]}], ")"}], "/", 
     RowBox[{"[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["B", 
          RowBox[{"j", "-", "1"}]], "\[Intersection]", 
         SubscriptBox["A", "i"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["B", "j"], "\[Intersection]", " ", 
         SubscriptBox["A", 
          RowBox[{"i", "-", "1"}]]}], ")"}]}]}]}]}], TraditionalForm]]],
 "].\n      \nNotice that these are exactly the same thing, so \n\n           \
                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["A", 
      RowBox[{"i", ",", 
       RowBox[{"j", "-", "1"}]}]], "/", 
     SubscriptBox["A", 
      RowBox[{"i", ",", "j"}]]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["B", 
      RowBox[{"j", ",", 
       RowBox[{"i", "-", "1"}]}]], "/", 
     SubscriptBox["B", 
      RowBox[{"j", ",", "i"}]]}]}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417105256358747`*^9, 3.4417106876514997`*^9}, {
  3.4417107210108747`*^9, 3.4417107484952497`*^9}}]
}, Open  ]],

Cell["\<\
If we now apply the refinement theorem to two composition series we find that \
the composition factors will be the same.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\nTHEOREM 8.2:  THE JORDAN-H\[CapitalODoubleDot]LDER THEOREM\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a finite group, and let\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["A", 
       RowBox[{"0", " "}]], "\[SupersetEqual]", " ", 
      SubscriptBox["A", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["A", "2"], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["A", "n"]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 ",\nand\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["B", "0"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["B", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["B", "2"], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["B", "m"]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 "\n                 \nbe two composition series for ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "m"}], TraditionalForm]]],
 ", and the composition factors ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", 
     RowBox[{"u", "-", "1"}]], "/", " ", 
    SubscriptBox["A", "u"]}], TraditionalForm]]],
 "  are isomorphic to the composition factors ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", 
     RowBox[{"v", "-", "1"}]], "/", " ", 
    SubscriptBox["B", "v"]}], TraditionalForm]]],
 "  in some order."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695426515625*^9, 3.436954273953125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "By the refinement theorem (8.1), there is a refinement of both composition \
series such that the quotient groups of the two subnormal series are \
isomorphic to each other in some order. In particular, the non-trivial \
quotient groups of one subnormal series are isomorphic to the non-trivial \
quotient groups of the other. But these are composition series, so any \
refinements merely repeat a subgroup a number of times. Thus, by eliminating \
these repetitions, we eliminate the trivial quotient groups and produce the \
original two composition series. Thus, the quotient groups ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", 
     RowBox[{"u", "-", "1"}]], "/", " ", 
    SubscriptBox["A", "u"]}], TraditionalForm]]],
 "  are isomorphic to the quotient groups ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", 
     RowBox[{"v", "-", "1"}]], "/", " ", 
    SubscriptBox["B", "v"]}], TraditionalForm]]],
 "  in some order. The fact that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "m"}], TraditionalForm]]],
 " merely comes from the one-to-one correspondence of the non-trivial \
quotient groups."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436954283359375*^9, 3.436954303890625*^9}}]
}, Open  ]],

Cell[TextData[{
 "The Jordan-H\[ODoubleDot]lder theorem (8.2) shows that the composition \
factors do not depend on the composition series, but rather the finite group \
",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". This is reminiscent of the unique factorization of integers, where every \
positive integer greater than one can be written as a unique product of prime \
numbers. Since the composition factors are always non-trivial simple groups, \
in a sense the simple groups play the same role in group theory that prime \
numbers play in number theory. The correspondence is heightened by the fact \
that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "p"], TraditionalForm]]],
 " is a non-trivial simple group if, and only if, ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime number. However, we have seen that there are other simple \
groups, such as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "24", "*"], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "n"], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "4"}], TraditionalForm]]],
 ".\nSince these non-cyclic simple groups are rather large (at least 60 \
elements), they will only show up as composition factors for very large \
groups.\n\nFor example, a composition series for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "5"], TraditionalForm]]],
 "  is given by\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "5"], " ", "\[Superset]", " ", 
    SubscriptBox["A", "5"], " ", "\[Superset]", " ", 
    RowBox[{"{", 
     RowBox[{"P", "[", " ", "]"}], "}"}]}], TraditionalForm]]],
 ",  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["S", "5"], "/", 
     SubscriptBox["A", "5"]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 ",  and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["A", "5"], "/", 
     RowBox[{"{", 
      RowBox[{"P", "[", " ", "]"}], "}"}]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["A", "5"]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436954338515625*^9, 3.436954339265625*^9}}],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " are both simple groups, this is a composition series, and the composition \
factors of  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "5"], TraditionalForm]]],
 " are ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 ".\n\nThe composition series play a vital role in determining whether groups \
are solvable or not. However, we will hold off on the definition of a \
solvable group until we have defined another tool in group theory, the \
derived group."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Derived Group Series", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In the last section we saw that two different composition series produced \
the same composition factors. In this section we will find a method for \
producing a composition series which is easily implimented using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". This method hinges on the concept of a \"commutator.\"\n\nDEFINITION 8.4:\
\nGiven two elements ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", the ",
 StyleBox["commutator",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], 
   TraditionalForm]]],
 ", and is written ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"x", ",", "y"}], "]"}], TraditionalForm]]],
 ".\n\nNotice that if ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is an abelian group the commutator will always give the identity element. \
\n\nWe can also consider the commutator of two subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " denote two subgroups, then consider the set \n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      SuperscriptBox["y", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], "  ", 
     "|", "  ", 
     RowBox[{"x", " ", "\[Element]", " ", 
      RowBox[{"H", "   ", "and", "   ", "y"}], " ", "\[Element]", " ", 
      "K"}]}], "}"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436954364984375*^9, 3.436954425859375*^9}, {
  3.436954469765625*^9, 3.436954472484375*^9}}],

Cell[TextData[{
 "Will this set form a group? Unfortunately, not always. Consider the \
subgroups ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"(", " ", ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", ",", "2"}], ")"}]}], "}"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"(", " ", ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"2", ",", "3", ",", "4"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"2", ",", "4", ",", "3"}], ")"}]}], "}"}]}], 
   TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["S", "4"], "."}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417174458233747`*^9, 3.4417174845889997`*^9}, {
  3.4417175154171247`*^9, 3.4417175281358747`*^9}}],

Cell[TextData[{
 "Then the set of all elements of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], 
   TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " can be found by making a table\n                   ",
 Cell[BoxData[
  FormBox[GridBox[{
     {
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       SuperscriptBox["y", 
        RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], 
      RowBox[{"(", " ", ")"}], 
      RowBox[{"(", 
       RowBox[{"2", ",", "3", ",", "4"}], ")"}], 
      RowBox[{"(", 
       RowBox[{"2", ",", "4", ",", "3"}], ")"}]},
     {
      RowBox[{"(", " ", ")"}], 
      RowBox[{"(", " ", ")"}], 
      RowBox[{"(", " ", ")"}], 
      RowBox[{"(", " ", ")"}]},
     {
      RowBox[{"(", 
       RowBox[{"1", ",", "2"}], ")"}], 
      RowBox[{"(", " ", ")"}], 
      RowBox[{"(", 
       RowBox[{"1", ",", "2", ",", "3"}], ")"}], 
      RowBox[{"(", 
       RowBox[{"1", ",", "2", ",", "4"}], ")"}]}
    }], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436954442203125*^9, 3.436954450078125*^9}, {
  3.4417172494952497`*^9, 3.4417173707764997`*^9}}],

Cell[TextData[{
 "So the elements of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], 
   TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is the set\n\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"(", " ", ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", ",", "2", ",", "3"}], ")"}], ",", " ", 
      RowBox[{"(", 
       RowBox[{"1", ",", "2", ",", "4"}], ")"}]}], "}"}], "."}], 
   TraditionalForm]]],
 "\n\nThis is not a subgroup.\n\nHowever, all is not lost, for we can \
consider the group generated by all of the commutators. \n\nDEFINITION 8.5:\n\
Given two subgroups ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " of a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", we define the ",
 StyleBox["mutual commutator subgroup of",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " ",
 StyleBox["and",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"H", ",", "K"}], "]"}], TraditionalForm]]],
 ", to be the subgroup generated by the elements\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      SuperscriptBox["y", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], "  ", 
     "|", "  ", 
     RowBox[{"x", " ", "\[Element]", " ", 
      RowBox[{"H", "   ", "and", "   ", "y"}], " ", "\[Element]", " ", 
      "K"}]}], "}"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436954524828125*^9, 3.436954552828125*^9}, {
  3.436954593515625*^9, 3.43695459503125*^9}, {3.4369546756875*^9, 
  3.43695472934375*^9}, {3.4417175079796247`*^9, 3.4417175514171247`*^9}}],

Cell[TextData[{
 "Although this definition circumvents the problem that the set of \
commutators do not always form a group, it creates difficulty in proving some \
of the theorems about the mutual commutator subgroups. This is because \
whenever ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"X", ",", "Y"}], "]"}], TraditionalForm]]],
 " we cannot say that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}]}], 
   TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " and some ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ". Rather, we have to write\n        \n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "=", " ", 
    RowBox[{
     SubscriptBox["u", "1"], "\[CenterDot]", 
     SubscriptBox["u", "2"], "\[CenterDot]", " ", "\[Ellipsis]", " ", 
     "\[CenterDot]", 
     SubscriptBox["u", "a"]}]}], TraditionalForm]]],
 ",\n\nwhere each ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["u", "i"], "=", " ", 
    RowBox[{
     SubsuperscriptBox["x", "i", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SubsuperscriptBox["y", "i", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SubscriptBox["x", "i"], "\[CenterDot]", 
     RowBox[{
      SubscriptBox["y", "i"], "."}]}]}], TraditionalForm]]],
 " Yet in spite of this difficulty, we will be able to discover some \
remarkable properties of the mutual commutator groups.\n\nThe ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command for finding the mutual commutator subgroup of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is \n\n",
 StyleBox["MutualCommutator[H, K]",
  FontWeight->"Bold"],
 "\n\nFor example, we can find the mutual commutator subgroup of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " by the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369547520625*^9, 3.4369548590625*^9}, {
  3.44267484207075*^9, 3.442674848680125*^9}}],

Cell[CellGroupData[{

Cell["\<\
H = Group[{C[1,2]}]
K = Group[{C[2,3,4]}]
MutualCommutator[H,K]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43695486475*^9, 3.43695486646875*^9}, {
  3.442674611398875*^9, 3.442674630992625*^9}, {3.442674710023875*^9, 
  3.442674739680125*^9}, {3.44267479832075*^9, 3.44267481644575*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.442674560477*^9, {3.44267462563325*^9, 3.44267463350825*^9}, {
   3.4426747282895*^9, 3.4426747400395*^9}, 3.442674817617625*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.442674560477*^9, {3.44267462563325*^9, 3.44267463350825*^9}, {
   3.4426747282895*^9, 3.4426747400395*^9}, 3.442674817617625*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{
  3.442674560477*^9, {3.44267462563325*^9, 3.44267463350825*^9}, {
   3.4426747282895*^9, 3.4426747400395*^9}, 3.442674817680125*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that there are elements in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"H", ",", "K"}], "]"}], TraditionalForm]]],
 " that are neither in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " nor in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".\n\nOne of the simple facts that we can prove for mutual commutator \
subgroups is that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"H", ",", "K"}], "]"}], TraditionalForm]]],
 " will be a normal subgroup whenever ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are normal.\n\nPROPOSITION 8.2\nIf ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are normal subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"H", ",", "K"}], "]"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695486975*^9, 3.436954898765625*^9}, {
  3.43695496746875*^9, 3.4369549795*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " be an element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"H", ",", "K"}], "]"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " an element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "=", " ", 
     RowBox[{
      SubscriptBox["u", "1"], "\[CenterDot]", 
      SubscriptBox["u", "2"], "\[CenterDot]", "\[Ellipsis]", "\[CenterDot]", 
      SubscriptBox["u", "n"]}]}], ","}], TraditionalForm]]],
 " where either ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "i"], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["u", "i", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 "is ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubsuperscriptBox["x", "i", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SubsuperscriptBox["y", "i", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SubscriptBox["x", "i"], "\[CenterDot]", 
     RowBox[{
      SubscriptBox["y", "i"], "."}]}]}], TraditionalForm]]],
 " Then \n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      RowBox[{"v", "\[CenterDot]", "u", "\[CenterDot]", 
       SuperscriptBox["v", 
        RowBox[{"-", "1"}]]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"v", "\[CenterDot]", 
         SubscriptBox["u", "1"], "\[CenterDot]", 
         SuperscriptBox["v", 
          RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"v", "\[CenterDot]", 
         SubscriptBox["u", "2"], "\[CenterDot]", 
         SuperscriptBox["v", 
          RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", "\[Ellipsis]", 
       "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"v", "\[CenterDot]", 
         SubscriptBox["u", "n"], "\[CenterDot]", 
         SuperscriptBox["v", 
          RowBox[{"-", "1"}]]}], ")"}]}]}], ",", " ", "and", " "}]}], 
   TraditionalForm]]],
 "\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{"v", "\[CenterDot]", 
      SubsuperscriptBox["x", "i", 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      SubsuperscriptBox["y", "i", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y", 
      "\[CenterDot]", 
      SuperscriptBox["v", 
       RowBox[{"-", "1"}]]}], "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"v", "\[CenterDot]", 
         SubsuperscriptBox["x", "i", 
          RowBox[{"-", "1"}]], "\[CenterDot]", 
         SuperscriptBox["v", 
          RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"v", "\[CenterDot]", 
         SubsuperscriptBox["y", "i", 
          RowBox[{"-", "1"}]], "\[CenterDot]", 
         SuperscriptBox["v", 
          RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"v", "\[CenterDot]", 
         SubscriptBox["x", "i"], "\[CenterDot]", 
         SuperscriptBox["v", 
          RowBox[{"-", "1"}]]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"v", "\[CenterDot]", 
         SubscriptBox["y", "i"], "\[CenterDot]", 
         SuperscriptBox["v", 
          RowBox[{"-", "1"}]]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"[", 
        RowBox[{
         RowBox[{"v", "\[CenterDot]", 
          SubscriptBox["x", "i"], "\[CenterDot]", 
          SuperscriptBox["v", 
           RowBox[{"-", "1"}]]}], ",", 
         RowBox[{"v", "\[CenterDot]", 
          SubscriptBox["y", "i"], "\[CenterDot]", 
          SuperscriptBox["v", 
           RowBox[{"-", "1"}]]}]}], "]"}], "."}]}]}]}], TraditionalForm]]],
 "\nSince ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are both normal subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "\[CenterDot]", 
    SubscriptBox["x", "i"], "\[CenterDot]", 
    SuperscriptBox["v", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "\[CenterDot]", 
    SubscriptBox["y", "i"], "\[CenterDot]", 
    SuperscriptBox["v", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".  Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{
     RowBox[{"v", "\[CenterDot]", 
      SubscriptBox["x", "i"], "\[CenterDot]", 
      SuperscriptBox["v", 
       RowBox[{"-", "1"}]]}], ",", 
     RowBox[{"v", "\[CenterDot]", 
      SubscriptBox["y", "i"], "\[CenterDot]", 
      SuperscriptBox["v", 
       RowBox[{"-", "1"}]]}]}], "]"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"H", ",", "K"}], "]"}], TraditionalForm]]],
 ".\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"v", "\[CenterDot]", 
       SubscriptBox["u", "i"], "\[CenterDot]", 
       SuperscriptBox["v", 
        RowBox[{"-", "1"}]]}], ")"}], 
     RowBox[{"-", "1"}]], "=", " ", 
    RowBox[{"(", 
     RowBox[{"v", "\[CenterDot]", 
      SubsuperscriptBox["u", "i", 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      SuperscriptBox["v", 
       RowBox[{"-", "1"}]]}], ")"}]}], TraditionalForm]]],
 ", if one of these is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"H", ",", "K"}], "]"}], TraditionalForm]]],
 ", they both are.  Hence, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "\[CenterDot]", 
    SubscriptBox["u", "i"], "\[CenterDot]", 
    SuperscriptBox["v", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"H", ",", "K"}], "]"}], TraditionalForm]]],
 " for every ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "i"], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"v", "\[CenterDot]", "u", "\[CenterDot]", 
     SuperscriptBox["v", 
      RowBox[{"-", "1"}]]}], "\[Element]", 
    RowBox[{"[", 
     RowBox[{"H", ",", "K"}], "]"}]}], TraditionalForm]]],
 ".  By proposition 3.4 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"H", ",", "K"}], "]"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436954987359375*^9, 3.436955016703125*^9}, {
  3.436955056765625*^9, 3.43695512278125*^9}, {3.4417175854171247`*^9, 
  3.4417176417139997`*^9}, {3.4417177044639997`*^9, 3.4417177385264997`*^9}, {
  3.4417178049952497`*^9, 3.4417180043233747`*^9}, {3.4417181736827497`*^9, 
  3.4417182636358747`*^9}}]
}, Open  ]],

Cell[TextData[{
 "Many times one of the two groups ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " will be the whole group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". We call the subgroup ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"G", ",", "H"}], "]"}], TraditionalForm]]],
 " the ",
 StyleBox["commutator subgroup of",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " ",
 StyleBox["in",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". In this case ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can find the commutator subgroup faster with the simplified command\n\n",
 StyleBox["Commutator[",
  FontWeight->"Bold"],
 "G",
 StyleBox[", H]",
  FontWeight->"Bold"],
 "\n\nNote that this ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command assumes that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". In fact, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will correctly find the commutator subgroup if only the ",
 StyleBox["generators",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " are specified. For example, suppose we wish to find the commutator ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      FormBox[
       RowBox[{"[", 
        SubscriptBox["S", "4"]}],
       TraditionalForm], ",", 
      SubscriptBox["A", "4"]}],
     TraditionalForm], "]"}], TraditionalForm]]],
 "\n\nWe first find the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436955183859375*^9, 3.436955187828125*^9}, {
  3.43695528196875*^9, 3.436955298578125*^9}}],

Cell[CellGroupData[{

Cell["S4 = Group[{ C[1,2], C[1,2,3,4] }]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.44267486494575*^9}]
}, Open  ]],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " is generated by the 3-cycles ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], TraditionalForm]]],
 ", we can load this group as follows:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["A4 = Group[{ C[1,2,3], C[2,3,4] }]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.442674867586375*^9}]
}, Open  ]],

Cell[TextData[{
 "Rather than using the command\n\nCommutator[S4, A4]\n\nit is faster to use \
only the generators of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Commutator[S4,{ C[1,2,3], C[2,3,4] }]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.44267492282075*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives us the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " again.\n\nWhat would happen if took the commutator subgroup of the entire \
group? That is, what if we considered ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{
       FormBox[
        RowBox[{"[", 
         SubscriptBox["S", "4"]}],
        TraditionalForm], ",", 
       SubscriptBox["S", "4"]}],
      TraditionalForm], "]"}], "?"}], TraditionalForm]]],
 " Let's try it."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Commutator[S4,{ C[1,2], C[1,2,3,4] }]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "2", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "2"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"1", ",", "4", ",", "3"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "3", ",", "4"}], "]"}], ",", 
   RowBox[{"C", "[", 
    RowBox[{"2", ",", "4", ",", "3"}], "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.44267492682075*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives us the subgroup ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 " again. So the commutator subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 " with itself is something other than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ".\n\nDEFINITION 8.6:\nWe define the commutator subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " with itself, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"G", ",", "G"}], "]"}], TraditionalForm]]],
 ", to be the ",
 StyleBox["derived group",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", denoted ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a normal subgroup of itself, proposition 8.2 states that the derived \
group will be a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Since the commutator of any two elements in an abelian group is ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{"G", ",", "G"}], "]"}], TraditionalForm]]],
 " will be the trivial group whenever ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is abelian. In fact, the size of the derived group in a sense measures \
just how non-commutative the group is.\n\nBefore we explore other properties \
of the derived group, let us look at some more examples. We know that the \
derived group of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 ", so what is the derived group of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 "? Let's explore:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436955355625*^9, 3.436955368171875*^9}, {
  3.44267494532075*^9, 3.44267494563325*^9}}],

Cell[CellGroupData[{

Cell["K = Commutator[A4,{ C[1,2,3], C[2,3,4] }]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"C", "[", "]"}], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 2\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(3, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 2], 
     C[3, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 3\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 4\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 3], 
     C[2, 4]],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(C[\\(\\(1, 4\\)\\)]\\) \[CenterDot] \
\\!\\(C[\\(\\(2, 3\\)\\)]\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     C[1, 4], 
     C[2, 3]],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.442674951430125*^9}]
}, Open  ]],

Cell["\<\
This is the normal subgroup of order 4. What if we took the derived group of \
this group?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Commutator[K,K]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"C", "[", "]"}], "}"}]], "Output",
 CellChangeTimes->{3.44267495388325*^9}]
}, Open  ]],

Cell[TextData[{
 "This is to be expected, since the group ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " was abelian. In this case we happened to find all of the normal subgroups \
of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 " by taking successive derived groups. \n\nWe can denote the derived group \
of the derived group ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 " as ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]\[Prime]"], TraditionalForm]]],
 ". Likewise, the derived group of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]\[Prime]"], TraditionalForm]]],
 " will be denoted ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]\[Prime]\[Prime]"], TraditionalForm]]],
 ", and so on. Because each of these groups is a normal subgroup of the \
previous one, we have the series\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "\[SupersetEqual]", " ", 
    SuperscriptBox["G", "\[Prime]"], " ", "\[SupersetEqual]", " ", 
    SuperscriptBox["G", "\[Prime]\[Prime]"], " ", "\[SupersetEqual]", " ", 
    SuperscriptBox["G", "\[Prime]\[Prime]\[Prime]"], " ", "\[SupersetEqual]", 
    " ", "\[CenterEllipsis]"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "This is called the derived series for the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". The derived series is in fact a subnormal series as long as the groups \
keep getting smaller and smaller until they finally get to the trivial \
subgroup ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "e", "}"}], TraditionalForm]]],
 ". Will this always happen? \n\nIt is not hard to think of a group where the \
derived series never gets to the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "e", "}"}], TraditionalForm]]],
 ". Consider the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 ". We have already seen that this is a simple group, and since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{
     SubscriptBox["A", "5"], 
     FormBox[
      RowBox[{",", 
       SubscriptBox["A", "5"]}],
      TraditionalForm]}], "]"}], TraditionalForm]]],
 StyleBox[" ",
  FontSlant->"Italic"],
 "must be a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 ", we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{
     SubscriptBox["A", "5"], ",", 
     SubscriptBox["A", "5"]}], "]"}], TraditionalForm]]],
 " is either ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "e", "}"}], TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " is not abelian, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{
     SubscriptBox["A", "5"], ",", 
     SubscriptBox["A", "5"]}], "]"}], TraditionalForm]]],
 " can't be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "e", "}"}], TraditionalForm]]],
 ". Hence the derived series for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " is\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", "5"], " ", "\[SupersetEqual]", " ", 
    SubscriptBox["A", "5"], " ", "\[SupersetEqual]", " ", 
    SubscriptBox["A", "5"], " ", "\[SupersetEqual]", " ", 
    SubscriptBox["A", "5"], " ", "\[SupersetEqual]", " ", 
    "\[CenterEllipsis]"}], TraditionalForm]]],
 "\n                   \nwhich never gets to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "e", "}"}], TraditionalForm]]],
 ".\n\nWe want to characterize those groups in which the derived series \
eventually becomes the trivial group. \n\nDEFINITION 8.7:\nA group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is called ",
 StyleBox["solvable",
  FontSlant->"Italic"],
 " if the derived series\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "\[SupersetEqual]", " ", 
    SuperscriptBox["G", "\[Prime]"], " ", "\[SupersetEqual]", " ", 
    SuperscriptBox["G", "\[Prime]\[Prime]"], " ", "\[SupersetEqual]", " ", 
    SuperscriptBox["G", "\[Prime]\[Prime]\[Prime]"], " ", "\[SupersetEqual]", 
    " ", "\[CenterEllipsis]"}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695542259375*^9, 3.43695542378125*^9}}],

Cell[TextData[{
 "includes the trivial group in a finite number of steps. If the derived \
series never reaches the trivial group, ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is said to be ",
 StyleBox["insoluble",
  FontSlant->"Italic"],
 ".\n\nBy our experiments, we see that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 " is a solvable group, whereas ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " is not. Whenever we have a solvable group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", the derived series is in fact a subnormal series for ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". So it is natural that the derived series would shed some light as to what \
the composition factors of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " are. First we will need the following lemma, which characterizes the \
derived group.\n\nLEMMA 8.3\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a group. Then the derived group ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 " is the smallest normal subgroup for which the quotient group is abelian. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436955445921875*^9, 3.43695544653125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First we need to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", 
    SuperscriptBox["G", "\[Prime]"]}], TraditionalForm]]],
 " is abelian. Consider the canonical homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " onto ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", 
    SuperscriptBox["G", "\[Prime]"]}], TraditionalForm]]],
 ". Then for ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], 
   TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", 
    RowBox[{
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SuperscriptBox["y", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], ")"}], 
   TraditionalForm]]],
 " is the identity element in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", 
    SuperscriptBox["G", "\[Prime]"]}], TraditionalForm]]],
 ". But then\n \n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "(", 
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       SuperscriptBox["y", 
        RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], 
      ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"\[Phi]", "(", "x", ")"}], 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       SuperscriptBox[
        RowBox[{"\[Phi]", "(", "y", ")"}], 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       RowBox[{"\[Phi]", "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{"\[Phi]", "(", "y", ")"}]}], " ", "=", " ", "e"}]}], ",", 
    " "}], TraditionalForm]]],
 "\n\nso ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"\[Phi]", "(", "y", ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Phi]", "(", "y", ")"}], "\[CenterDot]", 
     RowBox[{
      RowBox[{"\[Phi]", "(", "x", ")"}], "."}]}]}], TraditionalForm]]],
 " Since ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is surjective, we see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", 
    SuperscriptBox["G", "\[Prime]"]}], TraditionalForm]]],
 " is abelian. \n\nNow suppose that ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is another normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "N"}], TraditionalForm]]],
 " is abelian. To show that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 " is a smaller group, we will show that ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " contains ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 ".\n\nFor any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", note that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], 
   TraditionalForm]]],
 " is certainly contained in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "N", "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "N", "\[CenterDot]", "x", 
    "\[CenterDot]", "N", "\[CenterDot]", "y", "\[CenterDot]", "N"}], 
   TraditionalForm]]],
 ". But since the quotient group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "N"}], TraditionalForm]]],
 " is abelian, we have\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "N", "\[CenterDot]", 
      SuperscriptBox["y", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "N", "\[CenterDot]", "x", 
      "\[CenterDot]", "N", "\[CenterDot]", "y", "\[CenterDot]", "N"}], " ", 
     "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]], "\[CenterDot]", "N", "\[CenterDot]", "x", 
       "\[CenterDot]", "N", "\[CenterDot]", 
       SuperscriptBox["y", 
        RowBox[{"-", "1"}]], "\[CenterDot]", "N", "\[CenterDot]", "y", 
       "\[CenterDot]", "N"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"N", "\[CenterDot]", "N"}], " ", "=", " ", 
       RowBox[{"N", "."}]}]}]}]}], TraditionalForm]]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "x", "\[CenterDot]", "y"}], 
   TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 " is generated by all such elements, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 " is contained in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436955484671875*^9, 3.4369556341875*^9}}]
}, Open  ]],

Cell[TextData[{
 "We now can express a relationship between the composition factors of a \
group and the derived series of a group.\n\nTHEOREM 8.3:  THE SOLVABILITY \
THEOREM\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a finite group. Then ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is solvable if, and only if, the composition factors of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " are cyclic groups of prime order."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369556465*^9, 3.436955646625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that the composition factors of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " are all cyclic groups of prime order. Then there exists a composition \
series for ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ":"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "            ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["G", "0"], " ", "\[Superset]", " ", 
      SubscriptBox["G", "1"], " ", "\[Superset]", " ", 
      SubscriptBox["G", "2"], " ", "\[Superset]", "\[CenterEllipsis]", 
      "\[Superset]", 
      SubscriptBox["G", "n"]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "0"], "/", 
    SubscriptBox["G", "1"]}], TraditionalForm]]],
 "  is an abelian group, we have from lemma 8.3 that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 " is contained in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ". But since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "1"], "/", 
    SubscriptBox["G", "2"]}], TraditionalForm]]],
 "  is also abelian, by lemma 8.2 we have ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["G", "1", "\[Prime]"], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "2"], TraditionalForm]]],
 ", and so"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695566946875*^9, 3.4369556778125*^9}}],

Cell[TextData[{
 "                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["G", "\[Prime]\[Prime]"], " ", "\[SubsetEqual]", " ", 
    SubsuperscriptBox["G", "1", "\[Prime]"], " ", "\[SubsetEqual]", " ", 
    SubscriptBox["G", "2"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Proceeding in this way we find that the ",
 StyleBox["n-",
  FontSlant->"Italic"],
 Cell[BoxData[
  FormBox["th", TraditionalForm]]],
 " derived group, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", 
    RowBox[{"(", "n", ")"}]], TraditionalForm]]],
 ", must be contained in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "n"], "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 ". Thus, the derived series produced the trivial group in at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " steps, so ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is solvable.\n\nNow suppose that ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is solvable and finite, and so the derived series can be written"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436955690578125*^9, 3.436955694015625*^9}}],

Cell[TextData[{
 "       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"G", " ", "\[SupersetEqual]", " ", 
     SuperscriptBox["G", "\[Prime]"], " ", "\[SupersetEqual]", " ", 
     SuperscriptBox["G", "\[Prime]\[Prime]"], " ", "\[SupersetEqual]", " ", 
     SuperscriptBox["G", "\[Prime]\[Prime]\[Prime]"], " ", "\[SupersetEqual]",
      " ", "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
     SuperscriptBox["G", 
      RowBox[{"(", "n", ")"}]]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"{", "e", "}"}], "."}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", 
    RowBox[{"(", "n", ")"}]], TraditionalForm]]],
 " is the first term in the derived series equal to",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 ", then this subnormal series can never repeat any two subgroups. Because \
this is a finite group, there are only a finite number of ways this series \
could be refined without repeating subgroups. Thus, by the refinement \
theorem, we can refine this to produce a composition series. Because each of \
the quotient groups of the derived series are abelian, the quotient groups of \
the refinement must also be abelian. But by proposition 8.1, the quotient \
groups of the composition series must be non-trivial simple groups. The only \
non-trivial simple groups which are abelian are the cyclic groups of prime \
order. Thus, the quotient groups for this composition series are cyclic \
groups of prime order. By the Jordan-H\[ODoubleDot]lder theorem (8.2), all \
composition series are the same way. "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436955726984375*^9, 3.436955738109375*^9}}]
}, Open  ]],

Cell[TextData[{
 "Historically, solvability was defined in terms of the composition factors. \
But this definition could only be used on finite groups, since infinite \
groups do not have a composition series. By defining solvability in terms of \
the derived series, we allow for infinite groups to be solvable.\n\nFor \
example, the group of integers ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is abelian, so the derived group is the trivial group. Hence, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is solvable, yet there is no composition series for ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". This is because every proper subgroup of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is isomorphic to ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 "!\n\nFrom the solvability theorem we see that for finite group, solvability \
can be defined in terms of the composition factors. That is, a finite group \
is solvable as long as there are none of those huge non-abelian simple groups \
appearing either as a subgroup or a quotient group. So we can quickly tell \
that a group of order 48 must be solvable, since the smallest non-abelian \
simple group is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 ", which is of order 60.\n\nDoes this hold true for infinite groups as well? \
That is, is an infinite group solvable as long as there is no non-abelian \
simple group (finite or infinite) lurking somewhere within its structure, \
either as a subgroup or as a quotient group? To shed some light on this \
problem, we will first need the following lemma.\n\nLEMMA 8.4\nIf ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", then\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"H", "\[CenterDot]", 
       RowBox[{"N", " ", "/", "N"}]}], ")"}], "\[Prime]"], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["H", "\[Prime]"], "\[CenterDot]", "N"}], ")"}], "/", 
     "N"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436955769015625*^9, 3.436955769140625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We first note that since ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "N"}], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", "N"}], TraditionalForm]]],
 ". Two typical elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[CenterDot]", 
    RowBox[{"N", " ", "/", " ", "N"}]}], TraditionalForm]]],
 " are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "\[CenterDot]", "n", "\[CenterDot]", "N"}], 
   TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[CenterDot]", "m", "\[CenterDot]", "N"}], 
   TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"H", "\[CenterDot]", 
      RowBox[{"N", "/", "N"}]}], ")"}], "\[Prime]"], TraditionalForm]]],
 " is generated from the elements of the form\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"h", "\[CenterDot]", "n", "\[CenterDot]", "N"}], ")"}], 
      "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"k", "\[CenterDot]", "m", "\[CenterDot]", "N"}], ")"}], 
      "\[CenterDot]", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"h", "\[CenterDot]", "n", "\[CenterDot]", "N"}], ")"}], 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "\[CenterDot]", "m", "\[CenterDot]", "N"}], ")"}], 
       RowBox[{"-", "1"}]]}], " ", "=", " ", 
     RowBox[{"h", "\[CenterDot]", "k", "\[CenterDot]", 
      SuperscriptBox["h", 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      SuperscriptBox["k", 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      RowBox[{"N", "."}]}]}]}], TraditionalForm]]],
 "\n\nBut these elements are also in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["H", "\[Prime]"], "\[CenterDot]", "N"}], ")"}], "/", 
    "N"}], TraditionalForm]]],
 ". In fact, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["H", "\[Prime]"], "\[CenterDot]", "N"}], ")"}], "/", 
    "N"}], TraditionalForm]]],
 " is generated by  the elements of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["h", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["k", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "h", "\[CenterDot]", "k", 
    "\[CenterDot]", "N"}], TraditionalForm]]],
 ". Therefore, the groups ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"H", "\[CenterDot]", 
      RowBox[{"N", " ", "/", " ", "N"}]}], ")"}], "\[Prime]"], 
   TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["H", "\[Prime]"], "\[CenterDot]", "N"}], ")"}], "/", 
    "N"}], TraditionalForm]]],
 " are equal."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369558034375*^9, 3.436955828421875*^9}, {
  3.4417195371202497`*^9, 3.4417195675108747`*^9}}]
}, Open  ]],

Cell[TextData[{
 "With this simple lemma we will be able to show the relationship with a \
solvable group to its subgroups and quotient groups.\n\nPROPOSITION 8.3\n\
Suppose that ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a group and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is solvable if, and only if, both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "H"}], TraditionalForm]]],
 " are solvable."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695583696875*^9, 3.4369558375625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We begin by showing that if ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is solvable, and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", normal or not, then ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is solvable. Since ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is contained in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", we have"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["H", "\[Prime]"], " ", "\[SubsetEqual]", " ", 
    RowBox[{
     SuperscriptBox["G", "\[Prime]"], "   ", "\[DoubleLongRightArrow]", "   ",
      
     SuperscriptBox["H", "\[Prime]\[Prime]"]}], " ", "\[SubsetEqual]", " ", 
    RowBox[{
     SuperscriptBox["G", "\[Prime]\[Prime]"], "    ", 
     "\[DoubleLongRightArrow]", "   ", 
     SuperscriptBox["H", "\[Prime]\[Prime]\[Prime]"]}], " ", "\[SubsetEqual]",
     " ", 
    RowBox[{
     SuperscriptBox["G", "\[Prime]\[Prime]\[Prime]"], "  ", "\[Ellipsis]"}]}],
    TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Thus, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["G", 
      RowBox[{"(", "n", ")"}]],
     TraditionalForm], " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["H", 
      RowBox[{"(", "n", ")"}]],
     TraditionalForm], " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is solvable.\n\nNext we want to show that if ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is normal, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "H"}], TraditionalForm]]],
 " is solvable. Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{"G", "\[CenterDot]", "H"}]}], TraditionalForm]]],
 " we can use\nlemma 8.4 to find ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"G", "/", "H"}], ")"}], "\[Prime]"], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["G", "\[Prime]"], "\[CenterDot]", "H"}], ")"}], "/", 
     RowBox[{"H", "."}]}]}], TraditionalForm]]],
 " But since ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 " is a subgroup, we can continue to use lemma 8.4 to find"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436955871609375*^9, 3.436955886890625*^9}}],

Cell[TextData[{
 "  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    FormBox[
     RowBox[{"        ", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"G", "/", "H"}], ")"}], "\[Prime]\[Prime]"], " ", "=", " ", 
        
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["G", "\[Prime]"], "\[CenterDot]", 
            RowBox[{"H", "/", "H"}]}], ")"}], "\[Prime]"], " ", "=", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["G", "\[Prime]\[Prime]"], "\[CenterDot]", "H"}], 
           ")"}], "/", "H"}]}]}], "   ", ",", "\n", "          ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"G", "/", "H"}], ")"}], "\[Prime]\[Prime]\[Prime]"], " ", 
        "=", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["G", "\[Prime]\[Prime]"], "\[CenterDot]", 
            RowBox[{"H", "/", "H"}]}], ")"}], "\[Prime]"], " ", "=", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["G", "\[Prime]\[Prime]\[Prime]"], "\[CenterDot]", 
            "H"}], ")"}], "/", "H"}]}]}], " ", ",", "  ", 
       "\[CenterEllipsis]"}]}],
     TraditionalForm]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a solvable group, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["G", 
      RowBox[{"(", "n", ")"}]],
     TraditionalForm], " ", "=", " ", 
    RowBox[{"{", "e", "}"}]}], TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Thus\n\n                           ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"G", "/", "H"}], ")"}], 
      RowBox[{"(", "n", ")"}]], "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["G", 
         RowBox[{"(", "n", ")"}]], "\[CenterDot]", " ", "H"}], ")"}], "/", 
      "H"}]}]}], TraditionalForm]]],
 "\n\nwould be the identity group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "/", 
    RowBox[{"H", "."}]}], TraditionalForm]]],
 " Therefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "H"}], TraditionalForm]]],
 " is a solvable group.\n\nNow suppose that both ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "/", "H"}], TraditionalForm]]],
 " is solvable. Then ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"G", "/", "H"}], ")"}], 
    RowBox[{"(", "n", ")"}]], TraditionalForm]]],
 "  is the identity for some ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"(", 
        SuperscriptBox["G", 
         RowBox[{"(", "n", ")"}]]}],
       TraditionalForm], "\[CenterDot]", " ", "H"}], ")"}], "/", "H"}], 
   TraditionalForm]]],
 " is the identity. Thus, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", 
    RowBox[{"(", "n", ")"}]], TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and since ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is solvable, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", 
    RowBox[{"(", "n", ")"}]], TraditionalForm]]],
 " must be solvable. Therefore, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", 
    RowBox[{"(", 
     RowBox[{"n", "+", "m"}], ")"}]], TraditionalForm]]],
 " is the identity for some ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a solvable group."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "From this proposition, we see that for an infinite solvable group there \
cannot be any non-abelian simple groups within its structure whether as a \
subgroup, a quotient group, a subgroup of a quotient group, etc.  Thus the \
current definition of solvability for infinite groups agrees with the \
historical notion of a group which does not contain non-abelian simple groups \
in the composition factors.\n\nWhy do we want to know whether a group is \
solvable or not? We already mentioned one important application, that a \
polynomial equation will be solvable by radicals if, and only if, a \
corresponding group is solvable. But there is another application which we \
have been using throughout these notebooks unawares. Most of the groups used \
in these notebooks were entered into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using the ",
 StyleBox["InitGroup",
  FontWeight->"Bold"],
 " command along with a series of ",
 StyleBox["Define",
  FontWeight->"Bold"],
 " commands. For example, all of the groups up to order 16 were defined this \
way, and even ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 " was introduced as the octahedronal group, and defined in terms of 3 \
generators. Occasionally, we defined a group in terms of a subset of a \
symmetric group. For example, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "24", "*"]}], TraditionalForm]]],
 ") were defined as subgroups of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "5"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "7"], TraditionalForm]]],
 " respectively.\n\nBy now, we can see the pattern. The ",
 StyleBox["solvable",
  FontSlant->"Italic"],
 " groups could be entered into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using the ",
 StyleBox["InitGroup",
  FontWeight->"Bold"],
 " and ",
 StyleBox["Define",
  FontWeight->"Bold"],
 " commands, whereas the ",
 StyleBox["insoluble",
  FontSlant->"Italic"],
 " groups had to be considered as a subgroup of a symmetric group. In the \
next section, we will see not only why this is so, but will determine a \
procedure for entering any finite solvable group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polycyclic Groups", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436955945984375*^9, 3.4369559503125*^9}}],

Cell[TextData[{
 "Throughout these notebooks, we have been using the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " commands ",
 StyleBox["InitGroup",
  FontWeight->"Bold"],
 " and ",
 StyleBox["Define",
  FontWeight->"Bold"],
 " to produce many of the groups that we have been studying. Only \
occasionally did we have to use a different method for representing groups, \
such as in the groups ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "5"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Aut", "(", 
     SubsuperscriptBox["Z", "24", "*"], ")"}], "."}], TraditionalForm]]],
 " However, the exact method for converting a finite group into a set of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " commands has never been fully explained. We know that the groups can be \
represented by a small number \nof generators. How do we know how many? Why \
was ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 "  defined in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with 3 generators when only two generators would generate the group?\n\n\
The secret to defining a group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using a set of generators comes from the composition series for ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Whenever ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is solvable the composition factors of the series will all be cyclic \
groups of prime order. With such a composition series, we can define a group \
in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", but this is actually more than we need. Suppose that we insisted that the \
factors of a series are cyclic, but not necessary of prime order. This leads \
to the following definition.\n\nDEFINITION 8.8:\nA subnormal series \n\n      \
    ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["G", "0"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "2"], " ", "\[SupersetEqual]", " ", 
      "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "n"]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"{", "e", "}"}], "."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695602515625*^9, 3.436956026171875*^9}}],

Cell[TextData[{
 "is a ",
 StyleBox["polycyclic series",
  FontSlant->"Italic"],
 " if the quotient groups ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"i", "-", "1"}]], "/", 
    SubscriptBox["G", "i"]}], TraditionalForm]]],
 "  are all cyclic groups. The number ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is called the ",
 StyleBox["length",
  FontSlant->"Italic"],
 " of the polycyclic series.\n\nIt is obvious that a group with a polycyclic \
series must be solvable, since the cyclic quotient groups would be solvable. \
On the other hand, any finite solvable group has a polycyclic series, since a \
composition series would have cyclic factors of prime order. \n\nIt should be \
noted that an ",
 StyleBox["infinite",
  FontSlant->"Italic"],
 " solvable group may not always have a polycyclic series. The groups that \
have a polycyclic series, whether or not they are finite, are called ",
 StyleBox["polycyclic groups",
  FontSlant->"Italic"],
 ". We will mainly be concerned with finite groups in this section, so this \
distinction will not be important.\n\nThe first step in expressing a finite \
group in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is to find a polycyclic series for the group, preferably with the smallest \
possible length. For example, the quaternionic group ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " has a normal  subgroup of order 4: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", 
     RowBox[{"e", ",", " ", "i", ",", " ", 
      SuperscriptBox["i", "2"], ",", " ", 
      SuperscriptBox["i", "3"]}], "}"}], "."}], TraditionalForm]]],
 " This is of course cyclic, and the quotient \ngroup is isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ", which is also cyclic. Thus, there is a polycyclic series of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " of length 2. The length of the polycyclic series will be the number of \
generators required for expressing the group in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", so naturally the shorter the polycyclic series, the less work the \
definition entails.\n\nOur strategy will be to work inductively on the length \
of the series. That is, given a polycyclic series\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"G", " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["G", "0"], " ", "\[SupersetEqual]", " ", 
       SubscriptBox["G", "1"], " ", "\[SupersetEqual]", " ", 
       SubscriptBox["G", "2"], " ", "\[SupersetEqual]", " ", 
       "\[CenterEllipsis]", " ", "\[SupersetEqual]", " ", 
       SubscriptBox["G", "n"]}], " ", "=", " ", 
      RowBox[{"{", "e", "}"}]}]}], ","}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "we will begin by defining ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "n"], TraditionalForm]]],
 ", the trivial group, and then define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"i", "-", "1"}]], TraditionalForm]]],
 " in terms of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "i"], TraditionalForm]]],
 ". Thus, after ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " steps, we will have defined the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " into",
 StyleBox[" Mathematica",
  FontSlant->"Italic"],
 ".\n\nDefining ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "n"], TraditionalForm]]],
 "  is easy, since this is the trivial group. This group is defined by the \
single command\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox["InitGroup",
     FontWeight->"Bold"], "[", 
    SubscriptBox["x", "n"]}], TraditionalForm]]],
 "];\n\nwhere ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "n"], TraditionalForm]]],
 "  is the identity element.\n\nWe will suppose that the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "i"], TraditionalForm]]],
 " is defined inductively by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in terms of the elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", 
     RowBox[{"i", "+", "1"}]], ",", 
    SubscriptBox["g", 
     RowBox[{"i", "+", "2"}]], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     RowBox[{
      SubscriptBox["g", "n"], ".", " "}]}]}], TraditionalForm]]],
 "Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", 
     RowBox[{"i", "-", "1"}]], "/", 
    SubscriptBox["G", "i"]}], TraditionalForm]]],
 "    is cyclic, there is a generator of this quotient group, which we will \
call ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["g", "i"],
     TraditionalForm], "\[CenterDot]", 
    SubscriptBox["G", "i"]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "i"], TraditionalForm]]],
 " is in the subgroup ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"i", "-", "1"}]], TraditionalForm]]],
 ". If this quotient group is of order ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"n", " "}], "i"], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["g", "i"], "\[CenterDot]", 
       SubscriptBox["G", "i"]}], ")"}], 
     SubscriptBox["n", "i"]], "=", " ", 
    SubscriptBox["G", "i"]}], TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["g", "i", 
     SubscriptBox["n", "i"]], "\[Element]", 
    SubscriptBox["G", "i"]}], TraditionalForm]]],
 ". So  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can represent ",
 Cell[BoxData[
  FormBox[Cell[TextData[Cell[BoxData[
    FormBox[Cell[TextData[Cell[BoxData[
      FormBox[
       SubsuperscriptBox["g", "i", 
        SubscriptBox["n", "i"]], TraditionalForm]]]]], TraditionalForm]]]]], 
   TraditionalForm]]],
 " in terms of the elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubscriptBox[
      RowBox[{"g", " "}], 
      RowBox[{"i", "+", "1"}]], ",", " ", 
     SubscriptBox["g", 
      RowBox[{"i", "+", "2"}]], ",", " ", 
     RowBox[{"\[Ellipsis]", " ", 
      RowBox[{
       SubscriptBox["g", "n"], "."}]}]}]}], TraditionalForm]]],
 " Thus, we can make the definition\n\n",
 StyleBox["Define",
  FontWeight->"Bold"],
 "[",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", 
    RowBox[{"i", " "}]], TraditionalForm]]],
 StyleBox["^",
  FontWeight->"Bold"],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["n", "i"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 "]\n\nwhere ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " representation of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["g", "i"], 
    SubscriptBox["n", "i"]], TraditionalForm]]],
 ". This defines ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["g", "i"], 
    SubscriptBox["n", "i"]], TraditionalForm]]],
 " in terms of the elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", 
     RowBox[{"i", "+", "1"}]], ",", 
    SubscriptBox["g", 
     RowBox[{"i", "+", "2", " "}]], ",", " ", 
    RowBox[{"...", " ", 
     RowBox[{
      SubscriptBox["g", "n"], "."}]}]}], TraditionalForm]]],
 "\n\n\nOur goal will be to have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " express every element in terms of generators that are multiplied in \
increasing order. We consider the generators ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "1"], ",", " ", 
    SubscriptBox["g", "2"], ",", 
    SubscriptBox["g", "3"]}], TraditionalForm]]],
 " ... as \"letters\", going in alphabetical order, then we need definitions \
which would find a way of expressing any element of the group as a product of \
generators such that the generators are in alphabetical order.  That is, the \
expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "2"], "\[CenterDot]", 
    SubscriptBox["x", "3"], "\[CenterDot]", 
    SubscriptBox["x", "1"]}], TraditionalForm]]],
 "  will need to be reduced, whereas ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "1"], "\[CenterDot]", 
    SubscriptBox["x", "2"], "\[CenterDot]", 
    SubscriptBox["x", "3"], "\[CenterDot]", 
    SubscriptBox["x", "3"]}], TraditionalForm]]],
 " does not. As such, we have to program ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to unravel expressions that are \"in the wrong order.\" Any expression in \
the wrong order will contain a sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["g", "k"],
     TraditionalForm], "\[CenterDot]", 
    SubscriptBox["g", "i"]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", ">", " ", "i"}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "i"], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", 
    RowBox[{"i", "-", "1"}]], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      SubscriptBox["g", "i"], 
      RowBox[{"-", "1"}]], "\[CenterDot]", 
     SubscriptBox["g", "k"], "\[CenterDot]", 
     SubscriptBox["g", "i"]}], " ", "=", " ", 
    SubscriptBox["y", "k"]}], TraditionalForm]]],
 "  is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "i"], TraditionalForm]]],
 ", and hence can be expressed in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in terms of the elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     RowBox[{"g", " "}], 
     RowBox[{"i", "+", "1"}]], ",", 
    SubscriptBox["g", 
     RowBox[{"i", "+", "2"}]], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     RowBox[{
      SubscriptBox["g", "n"], "."}]}]}], TraditionalForm]]],
 " Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["x", "k"], "\[CenterDot]", 
     SubscriptBox["x", "i"]}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["x", "i"], "\[CenterDot]", 
     SubscriptBox["y", "k"]}]}], TraditionalForm]]],
 ", which has the generators in the proper order. We can perform the \
following sequence of commands to force ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to always put the generators in the proper order.\n\n",
 StyleBox["Define",
  FontWeight->"Bold"],
 "[",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", 
    RowBox[{"i", "+", "1"}]], TraditionalForm]]],
 ". ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "i"], TraditionalForm]]],
 " , ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "i"], TraditionalForm]]],
 " .",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["y", 
     RowBox[{"i", "+", "1"}]]}], TraditionalForm]]],
 "  ]\n",
 StyleBox["Define",
  FontWeight->"Bold"],
 "[",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", 
    RowBox[{"i", "+", "2"}]], TraditionalForm]]],
 ". ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "i"], TraditionalForm]]],
 " , ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "i"], TraditionalForm]]],
 " .",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["y", 
     RowBox[{"i", "+", "2"}]]}], TraditionalForm]]],
 "  ]\n\[Ellipsis]\[Ellipsis]\[Ellipsis]\[Ellipsis]\[Ellipsis]\[Ellipsis]\
\[Ellipsis]\[Ellipsis]\[Ellipsis]\[Ellipsis]\n",
 StyleBox["Define",
  FontWeight->"Bold"],
 "[",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "n"], TraditionalForm]]],
 ". ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "i"], TraditionalForm]]],
 " , ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "i"], TraditionalForm]]],
 " .",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["y", "n"]}], TraditionalForm]]],
 "  ]\n\nWith these definitions, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will continue to process a given element until the generators are arranged \
in order. Although it is not clear right now that a given element ",
 StyleBox["can",
  FontSlant->"Italic"],
 " be expressed as a product of generators arranged in order, we will see \
that the group generated by the elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["g", "i"], " ", ",", 
     SubscriptBox["g", 
      RowBox[{"i", "+", "1"}]], ",", " ", 
     RowBox[{"\[Ellipsis]", " ", 
      SubscriptBox[
       RowBox[{"g", " "}], "n"]}]}], "}"}], TraditionalForm]]],
 " will be isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "i"], TraditionalForm]]],
 ".  Thus, we will be able to construct the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " inductively.\n\nThis process can best be understood by looking at \
examples. Even though we have already loaded the quaternionic group ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", let us run through the procedure with this group. The first step is to \
find a polycyclic series for ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", which we have already found:\n      \n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     FormBox[
      SubscriptBox["G", "0"],
      TraditionalForm], " ", "=", " ", 
     RowBox[{
      RowBox[{"Q", "  ", "\[Superset]", "  ", 
       SubscriptBox["G", "1"]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "i", ",", 
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", "i"}]}], "}"}], "  ", "\[Superset]", "  ", 
        SubscriptBox["G", "2"]}], "=", " ", 
       RowBox[{
        RowBox[{"{", "1", "}"}], "."}]}]}]}]}], TraditionalForm]]],
 "\n\nSince this is a series of length 2, we will need 2 generators.\nWe \
begin by defining ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "2"], TraditionalForm]]],
 ", the trivial group:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.43695641990625*^9, {3.4369565174375*^9, 3.436956577796875*^9}, {
   3.436957297890625*^9, 3.436957298046875*^9}, {3.436957341046875*^9, 
   3.43695740575*^9}, {3.43695745265625*^9, 3.43695765653125*^9}, {
   3.43695773553125*^9, 3.436957927390625*^9}, {3.436957961578125*^9, 
   3.436957964265625*^9}, 3.43695799965625*^9, {3.4369580319375*^9, 
   3.4369581120625*^9}, {3.436958146421875*^9, 3.43695816721875*^9}, {
   3.4369583296875*^9, 3.4369583941875*^9}, {3.436958949140625*^9, 
   3.436958959*^9}, {3.4417200786514997`*^9, 3.4417200891827497`*^9}}],

Cell[CellGroupData[{

Cell["InitGroup[e]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436956375046875*^9, 3.436956375359375*^9}, {
  3.4369584138125*^9, 3.4369584141875*^9}, {3.4369589324375*^9, 
  3.43695893325*^9}},
 AspectRatioFixed->True],

Cell[BoxData["e"], "Output",
 CellChangeTimes->{3.436956375859375*^9, 3.43695841465625*^9, 
  3.436958940328125*^9, 3.436959109234375*^9, 3.44267497438325*^9}]
}, Open  ]],

Cell[TextData[{
 "To find ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "2"], TraditionalForm]]],
 ", we need to find a generator of the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["G", "1"],
     TraditionalForm], "/", 
    SubscriptBox["G", "2"]}], TraditionalForm]]],
 ". Certainly ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "i", "}"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"-", "i"}], "}"}], TraditionalForm]]],
 " would work, so we let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "2"], TraditionalForm]]],
 " be one of these elements. We might as well pick ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "2"], " ", "=", " ", 
    RowBox[{"i", "."}]}], TraditionalForm]]],
 " \n\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "1"], "/", 
    SubscriptBox["G", "2"]}], TraditionalForm]]],
 "  is cyclic of order 4, we have that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "4"], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "2"], TraditionalForm]]],
 ", which is the identity. So we can define"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436958965046875*^9, 3.436958986234375*^9}, {
  3.4369591884375*^9, 3.43695919365625*^9}, {3.4417200974327497`*^9, 
  3.4417200999327497`*^9}}],

Cell["Define[i^4,e]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436958989953125*^9, 3.4369589905*^9}, {
  3.436959105953125*^9, 3.436959107265625*^9}},
 AspectRatioFixed->True],

Cell[TextData[{
 "This defines ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ". We now observe that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["G", "0"],
     TraditionalForm], "/", 
    SubscriptBox["G", "1"]}], TraditionalForm]]],
 "  is of order 2, and a generator is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"j", ",", " ", "k", ",", " ", 
     RowBox[{"-", "j"}], ",", " ", "k"}], "}"}], TraditionalForm]]],
 ". We pick ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 " to be any one of these elements, say ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["j", "2"], TraditionalForm]]],
 " must be in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ", and in fact ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["j", "2"], " ", "=", 
    RowBox[{
     RowBox[{"-", "1"}], "=", " ", 
     SuperscriptBox["i", "2"]}]}], TraditionalForm]]],
 ". So we can define"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436959011859375*^9, 3.436959022234375*^9}, {
  3.436959184015625*^9, 3.436959184015625*^9}, {3.4417201231046247`*^9, 
  3.4417201432608747`*^9}}],

Cell["Define[j^2,i^2]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436959025390625*^9, 3.436959027921875*^9}},
 AspectRatioFixed->True],

Cell[TextData[{
 "Finally, we note that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["j", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "i", "\[CenterDot]", "j"}], 
   TraditionalForm]]],
 " must be in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ". In fact, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["j", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "i", "\[CenterDot]", "j"}], " ", 
    "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"j", "\[CenterDot]", 
        SuperscriptBox["i", "2"]}], ")"}], "\[CenterDot]", "i", 
      "\[CenterDot]", "j"}], " ", "=", " ", 
     SuperscriptBox["i", "3"]}]}], TraditionalForm]]],
 ". So ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"i", "\[CenterDot]", "j"}], " ", "=", " ", 
    RowBox[{"j", "\[CenterDot]", 
     RowBox[{
      SuperscriptBox["i", "3"], "."}]}]}], TraditionalForm]]],
 " We can define this into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436959043390625*^9, 3.436959088015625*^9}}],

Cell["Define[i.j,j.i.i.i]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4369590905*^9, 3.436959097359375*^9}},
 AspectRatioFixed->True],

Cell["and behold, we have our group!", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Group[{j,i}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436959100953125*^9, 3.436959101828125*^9}},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "i", ",", "j", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(j\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`j, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(j\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`j, $CellContext`i], $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(j\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`j, $CellContext`i], $CellContext`i], $CellContext`i],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436959117390625*^9, 3.442674979680125*^9}]
}, Open  ]],

Cell[TextData[{
 "This puts the elements in \"alphabetical\" order because ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", 
     RowBox[{"1", " "}]], "=", "j"}], TraditionalForm]]],
 " is considered to be before ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "2"], "=", "i"}], TraditionalForm]]],
 ".  Of course, it makes more sence to have ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " come before ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ".\nTo get the group definition that we are more familier with, we can take \
the \"mirror image\" of this definition, where all products are done in \
reverse order."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695845328125*^9, 3.4369586351875*^9}, {
  3.436959137453125*^9, 3.436959175984375*^9}, {3.436959214515625*^9, 
  3.43695926984375*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"InitGroup", "[", "e", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Define", "[", 
  RowBox[{
   RowBox[{"i", "^", "4"}], ",", "e"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Define", "[", 
  RowBox[{
   RowBox[{"j", "^", "2"}], ",", 
   RowBox[{"i", "^", "2"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Define", "[", 
  RowBox[{
   RowBox[{"j", ".", "i"}], ",", " ", 
   RowBox[{"i", ".", "i", ".", "i", ".", "j"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Group", "[", 
  RowBox[{"{", 
   RowBox[{"i", ",", "j"}], "}"}], "]"}]}], "Input",
 CellChangeTimes->{{3.43695882109375*^9, 3.436958868140625*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "i", ",", "j", ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(i\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`i, $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(i\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`i],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(i\\\\) \\\\[CenterDot] \
\\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`i, $CellContext`i], $CellContext`j],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(i\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(i\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(i\\\\)\\\"\\) \[CenterDot] \\!\\(j\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`i, $CellContext`i], $CellContext`i], $CellContext`j],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{3.436958868796875*^9, 3.442674981180125*^9}]
}, Open  ]],

Cell[TextData[{
 "\n\nFor many of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " commands, we will need to define the inverses of the generators, which at \
this point is easy."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43695845328125*^9, 3.4369586351875*^9}, {
  3.436958733390625*^9, 3.436958816703125*^9}}],

Cell["\<\
Define[1/a,a.b.b]
Define[1/b,b^3]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "Let us try another example, only this time we will start with a \
multiplication table. Let ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " be a group of order 16 with the elements \n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
    "1", ",", " ", "L", ",", " ", "M", ",", " ", "N", ",", " ", "O", ",", " ",
      "P", ",", " ", "Q", ",", " ", "R", ",", " ", "S", ",", " ", "T", ",", 
     " ", "U", ",", " ", "V", ",", " ", "W", ",", " ", "X", ",", " ", "Y", 
     ",", " ", "Z"}], "}"}], TraditionalForm]]],
 ".\n\nSuppose we are given the following multiplication table:\n\n           \
     The mystery group ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\n       ",
 Cell[BoxData[{
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{
      RowBox[{"*", " ", 
       RowBox[{"|", " ", 
        RowBox[{
        "1", "  ", "Z", "  ", "Y", "  ", "X", "  ", "W", "  ", "V", "  ", "U",
          "  ", "T", "  ", "S", "  ", "R", "  ", "Q", "  ", "P", "  ", "O", "  ",
          "N", "  ", "M", "  ", "L"}]}]}], "    ", "\n", "  ", 
      RowBox[{
       RowBox[{"--", 
        RowBox[{"+", 
         RowBox[{"--", 
          RowBox[{"--", 
           RowBox[{"--", 
            RowBox[{"--", 
             RowBox[{"--", 
              RowBox[{"--", 
               RowBox[{"--", 
                RowBox[{"--", 
                 RowBox[{"-", 
                  RowBox[{"--", 
                   RowBox[{"--", 
                    RowBox[{"--", 
                    RowBox[{"--", 
                    RowBox[{"--", 
                    RowBox[{"--", 
                    RowBox[{
                    "--", "\n", "   ", 
                    "1"}]}]}]}]}]}]}]}]}]}]}]}]}]}]}]}]}]}], " ", "|", " ", 
       RowBox[{
       "1", "  ", "Z", "  ", "Y", "  ", "X", "  ", "W", "  ", "V", "  ", "U", 
        "  ", "T", "  ", "S", "  ", "R", "  ", "Q", "  ", "P", "  ", "O", "  ",
         "N", "  ", "M", "  ", "L"}]}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"Z", " ", "|", " ", 
      RowBox[{
      "Z", "  ", "Y", "  ", "X", "  ", "1", "  ", "T", "  ", "W", "  ", "V", "  ",
        "U", "  ", "R", "  ", "Q", "  ", "P", "  ", "S", "  ", "L", "  ", "O",
        "  ", "N", "  ", "M"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"Y", " ", "|", " ", 
      RowBox[{
      "Y", "  ", "X", "  ", "1", "  ", "Z", "  ", "U", "  ", "T", "  ", "W", "  ",
        "V", "  ", "Q", "  ", "P", "  ", "S", "  ", "R", "  ", "M", "  ", "L",
        "  ", "O", "  ", "N"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"X", " ", "|", " ", 
      RowBox[{
      "X", "  ", "1", "  ", "Z", "  ", "Y", "  ", "V", "  ", "U", "  ", "T", "  ",
        "W", "  ", "P", "  ", "S", "  ", "R", "  ", "Q", "  ", "N", "  ", "M",
        "  ", "L", "  ", "O"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"W", " ", "|", " ", 
      RowBox[{
      "W", "  ", "V", "  ", "U", "  ", "T", "  ", "S", "  ", "R", "  ", "Q", "  ",
        "P", "  ", "O", "  ", "N", "  ", "M", "  ", "L", "  ", "1", "  ", "Z",
        "  ", "Y", "  ", "X"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"V", " ", "|", " ", 
      RowBox[{
      "V", "  ", "U", "  ", "T", "  ", "W", "  ", "P", "  ", "S", "  ", "R", "  ",
        "Q", "  ", "N", "  ", "M", "  ", "L", "  ", "O", "  ", "X", "  ", "1",
        "  ", "Z", "  ", "Y"}]}]}],
    TraditionalForm], "   "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"U", " ", "|", " ", 
      RowBox[{
      "U", "  ", "T", "  ", "W", "  ", "V", "  ", "Q", "  ", "P", "  ", "S", "  ",
        "R", "  ", "M", "  ", "L", "  ", "O", "  ", "N", "  ", "Y", "  ", "X",
        "  ", "1", "  ", "Z"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"T", " ", "|", " ", 
      RowBox[{
      "T", "  ", "W", "  ", "V", "  ", "U", "  ", "R", "  ", "Q", "  ", "P", "  ",
        "S", "  ", "L", "  ", "O", "  ", "N", "  ", "M", "  ", "Z", "  ", "Y",
        "  ", "X", "  ", "1"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{"   ", 
   FormBox[
    RowBox[{"S", " ", "|", " ", 
     RowBox[{
     "S", "  ", "R", "  ", "Q", "  ", "P", "  ", "O", "  ", "N", "  ", "M", "  ",
       "L", "  ", "1", "  ", "Z", "  ", "Y", "  ", "X", "  ", "W", "  ", "V", 
      "  ", "U", "  ", "T"}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{"   ", 
   FormBox[
    RowBox[{"R", " ", "|", " ", 
     RowBox[{
     "R", "  ", "Q", "  ", "P", "  ", "S", "  ", "L", "  ", "O", "  ", "N", "  ",
       "M", "  ", "Z", "  ", "Y", "  ", "X", "  ", "1", "  ", "T", "  ", "W", 
      "  ", "V", "  ", "U"}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"Q", " ", "|", " ", 
      RowBox[{
      "Q", "  ", "P", "  ", "S", "  ", "R", "  ", "M", "  ", "L", "  ", "O", "  ",
        "N", "  ", "Y", "  ", "X", "  ", "1", "  ", "Z", "  ", "U", "  ", "T",
        "  ", "W", "  ", "V"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"P", " ", "|", " ", 
      RowBox[{
      "P", "  ", "S", "  ", "R", "  ", "Q", "  ", "N", "  ", "M", "  ", "L", "  ",
        "O", "  ", "X", "  ", "1", "  ", "Z", "  ", "Y", "  ", "V", "  ", "U",
        "  ", "T", "  ", "W"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"O", " ", "|", " ", 
      RowBox[{
      "O", "  ", "N", "  ", "M", "  ", "L", "  ", "1", "  ", "Z", "  ", "Y", "  ",
        "X", "  ", "W", "  ", "V", "  ", "U", "  ", "T", "  ", "S", "  ", "R",
        "  ", "Q", "  ", "P"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"N", " ", "|", " ", 
      RowBox[{
      "N", "  ", "M", "  ", "L", "  ", "O", "  ", "X", "  ", "1", "  ", "Z", "  ",
        "Y", "  ", "V", "  ", "U", "  ", "T", "  ", "W", "  ", "P", "  ", "S",
        "  ", "R", "  ", "Q"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"M", " ", "|", " ", 
      RowBox[{
      "M", "  ", "L", "  ", "O", "  ", "N", "  ", "Y", "  ", "X", "  ", "1", "  ",
        "Z", "  ", "U", "  ", "T", "  ", "W", "  ", "V", "  ", "Q", "  ", "P",
        "  ", "S", "  ", "R"}]}]}],
    TraditionalForm], "    "}], "\n", 
  RowBox[{
   FormBox[
    RowBox[{"   ", 
     RowBox[{"L", " ", "|", " ", 
      RowBox[{
      "L", "  ", "O", "  ", "N", "  ", "M", "  ", "Z", "  ", "Y", "  ", "X", "  ",
        "1", "  ", "T", "  ", "W", "  ", "V", "  ", "U", "  ", "R", "  ", "Q",
        "  ", "P", "  ", "S"}]}]}],
    TraditionalForm], " "}]}]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369593844375*^9, 3.436959410875*^9}},
 FontFamily->"Courier"],

Cell[TextData[{
 "\nAlthough our tendency is to look for hidden words, we really should be \
looking for hidden subgroups. Is this a group that we have seen before in \
some other form? Notice that as we look down the main diagonal, we see the \
element 1 only 4 times. So only 3 elements are of order 2, namely ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". But the only other elements appearing in the main diagonal are ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". Thus, the forth power of every element must be 1. Since there are no \
elements of order 8, this cannot be any of the groups of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], 
    SubscriptBox["\[Proportional]", "\[Phi]"], " ", 
    SubscriptBox["Z", "8"]}], TraditionalForm]]],
 " that we studied in chapter 6. Also, this group is not abelian, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Z", "\[CenterDot]", "W"}], " ", "=", " ", "T"}], 
   TraditionalForm]]],
 ", while ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"W", "\[CenterDot]", "Z"}], " ", "=", " ", "V"}], 
   TraditionalForm]]],
 ".\n\nTo find some subgroups, we can begin by looking at the center of the \
group. An element will be in the center if the corresponding row and column \
are identical. Looking at the multiplication table, we find that 1, ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " are in the center. Since this is a normal subgroup it is natural to look \
at the quotient group. We can use the table to find the left cosets: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "Y", ",", "S", ",", "Q"}], "}"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"Z", ",", "X", ",", "R", ",", "P"}], "}"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"W", ",", "U", ",", "O", ",", "M"}], "}"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"V", ",", "T", ",", "N", ",", "L"}], "}"}], TraditionalForm]]],
 ". Both the center and its quotient group are isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ", so to produce a polycyclic series we need to add two more subgroups:\n\n\
",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{"A", " ", "\[Superset]", " ", 
     RowBox[{"{", 
      RowBox[{
      "1", ",", "Y", ",", "S", ",", "Q", ",", "Z", ",", "X", ",", "R", ",", 
       "P"}], "}"}], " ", "\[Superset]", "  ", 
     RowBox[{"{", 
      RowBox[{"1", ",", "Y", ",", "S", ",", "Q"}], "}"}], " ", "\[Superset]", 
     " ", 
     RowBox[{"{", 
      RowBox[{"1", ",", "Y"}], "}"}], " ", "\[Superset]", " ", 
     RowBox[{
      RowBox[{"{", "1", "}"}], "."}]}]}], TraditionalForm]]],
 "\n\nEach quotient group is isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ". Certainly this series would do, but it would involve 4 generators. Is \
there a shorter polycyclic series?\n\nLooking at the table, one might have \
noticed the subgroup in the top left hand corner: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "Z", ",", "Y", ",", "X"}], "}"}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 " is a generator of this subgroup, we have a cyclic group of order 4. Using \
this group in the polycyclic series would shorten it by at least one \
subgroup. We can notice from the table that both the left and right cosets of \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "Z", ",", "Y", ",", "X"}], "}"}], TraditionalForm]]],
 " are \n\n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "Z", ",", "Y", ",", "X"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"W", ",", "V", ",", "U", ",", "T"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"S", ",", "R", ",", "Q", ",", "P"}], "}"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"O", ",", "N", ",", "M", ",", "L"}], "}"}]}], "}"}], "."}]}], 
   TraditionalForm]]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "Z", ",", "Y", ",", "X"}], "}"}], TraditionalForm]]],
 " is a normal subgroup of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". Also notice that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"W", ",", "V", ",", "U", ",", "T"}], "}"}], TraditionalForm]]],
 " generates the quotient group. Thus, both the subgroup and the quotient \
group are isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "4"], TraditionalForm]]],
 ". \nTherefore, we have a much shorter polycyclic series:\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "0"], " ", "=", " ", 
    RowBox[{
     RowBox[{"A", " ", "\[Superset]", " ", 
      SubscriptBox["G", "1"]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "Z", ",", "Y", ",", "X"}], "}"}], " ", 
       "\[Superset]", " ", 
       SubscriptBox["G", "2"]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"{", "1", "}"}], "."}]}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43696847171875*^9, 3.43696848165625*^9}, {
  3.440947548492625*^9, 3.440947558477*^9}}],

Cell[TextData[{
 "By using this series, we need only two generators, ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["G", "1"],
     TraditionalForm], "/", 
    SubscriptBox["G", "2"]}], TraditionalForm]]],
 "  has two generators, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "Z", "}"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "X", "}"}], TraditionalForm]]],
 ", we can let ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " represent either element, say ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", "Z"}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["b", "4"], "=", 
    SuperscriptBox["Z", "4"]}], TraditionalForm]]],
 "  is in  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["G", "2"], " ", "=", " ", 
     RowBox[{"{", "1", "}"}]}], ","}], TraditionalForm]]],
 " and so we have the definition"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417203498858747`*^9, 3.4417203595577497`*^9}}],

Cell["\<\
InitGroup[e];
Define[b^4,e]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "Next, we notice that both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"W", ",", "V", ",", "U", ",", "T"}], "}"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"O", ",", "N", ",", "M", ",", "L"}], "}"}], TraditionalForm]]],
 " are generators of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "0"], "/", 
    SubscriptBox["G", "1"]}], TraditionalForm]]],
 ". Thus, we can let ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " be any of these 8 elements. If we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", "W"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", "4"], "=", 
    SuperscriptBox["W", "4"]}], TraditionalForm]]],
 "  is in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " , and in fact ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", "4"], "=", " ", "e"}], TraditionalForm]]],
 ". So we have the definition"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417203697764997`*^9, 3.4417203710421247`*^9}}],

Cell["Define[a^4,e]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "We also need to let ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " know how to handle the combination ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 ". Using the multiplication table we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["a", 
      RowBox[{"-", "1"}]], "\[CenterDot]", "b", "\[CenterDot]", "a"}], " ", 
    "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["W", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "Z", "\[CenterDot]", "W"}], " ", 
     "=", " ", 
     RowBox[{"X", " ", "=", " ", 
      SuperscriptBox["b", "3"]}]}]}], TraditionalForm]]],
 ". So ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{"a", "\[CenterDot]", 
     SuperscriptBox["b", "3"]}]}], TraditionalForm]]],
 ". Let us add this fact into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Define[b.a,a.b.b.b]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "We now have the group entered into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
A = Group[{a,b}]
MultTable[A]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.44267499482075*^9},
 AspectRatioFixed->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"e", ",", "a", ",", "b", ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(b\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", $CellContext`b, $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(a\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`a],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`a], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(b\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`b, $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(a\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`a], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`a], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\
\\\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[$CellContext`a, $CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(a\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`a], $CellContext`a], $CellContext`b], \
$CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[$CellContext`a, $CellContext`a], $CellContext`b], $CellContext`b], \
$CellContext`b],
    Editable->False], ",", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\(a\\\\\\\\\\\
\\\\\\\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"\\\\\\\\\\\\\\\\) \\\\\
\\\\\\\\\\\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(b\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(b\\\\)\\\"\\) \[CenterDot] \\!\\(b\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[
      Dot[
       Dot[
        Dot[$CellContext`a, $CellContext`a], $CellContext`a], \
$CellContext`b], $CellContext`b], $CellContext`b],
    Editable->False]}], "}"}]], "Output",
 CellChangeTimes->{{3.44267499269575*^9, 3.442674995727*^9}}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0.6, 1}, {1, 0.6, 1}, {0, 1, 1}, {1, 0.6, 0}, {0.6, 0.6, 
      0.2}, {0.6, 0.4, 0.8}, {0.8, 0.2, 0.5}, {0.7, 0.8, 0.7}, {1, 0, 1}, {
      0.5, 0.6, 0.8}, {1, 0.4, 0.4}, {0.8, 0.6, 0.2}, {0, 1, 0}, {1, 1, 0}, {
      0.2, 0.8, 0.7}, {0.8, 0.6, 0.5}, {1, 0, 0}}, {{0.2, 0.8, 0.7}, {0.2, 
      0.8, 0.7}, {0.8, 0.2, 0.5}, {1, 0, 0}, {0.5, 0.6, 0.8}, {1, 0.4, 0.4}, {
      0.7, 0.8, 0.7}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 1}, {0.8, 0.6, 0.2}, {
      0.8, 0.6, 0.5}, {0, 1, 1}, {1, 0.6, 0}, {0.6, 0.6, 0.2}, {0.6, 0.4, 
      0.8}, {1, 0, 1}}, {{1, 0.4, 0.4}, {1, 0.4, 0.4}, {0.6, 0.4, 0.8}, {
      1, 0.6, 1}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {0.8, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {1, 1, 0}, {0.8, 0.2, 0.5}, {0.2, 0.8, 0.7}, {1, 
      0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.7, 0.8, 0.7}}, {{0.6, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {0.7, 0.8, 0.7}, {1, 0, 1}, {1, 0.6, 1}, {0.8, 
      0.6, 0.2}, {0, 1, 0}, {0, 1, 1}, {1, 0.6, 0}, {0.2, 0.8, 0.7}, {0.8, 
      0.6, 0.5}, {0.6, 0.4, 0.8}, {0.8, 0.2, 0.5}, {1, 0, 0}, {0.5, 0.6, 
      0.8}, {1, 0.4, 0.4}, {1, 1, 0}}, {{0.8, 0.6, 0.2}, {0.8, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {0.6, 0.4, 0.8}, {1, 0.6, 1}, {1, 0, 
      0}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {
      0.6, 0.6, 0.2}, {1, 1, 0}, {0.8, 0.2, 0.5}, {1, 0, 1}, {0, 1, 1}, {0, 1,
       0}}, {{0.8, 0.2, 0.5}, {0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {
      1, 0.4, 0.4}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 1}, {0.2, 0.8, 0.7}, {0.8, 
      0.6, 0.5}, {0, 1, 1}, {1, 0.6, 0}, {1, 0, 0}, {0.6, 0.6, 0.2}, {0.6, 
      0.4, 0.8}, {0.7, 0.8, 0.7}, {1, 0, 1}, {0.8, 0.6, 0.2}}, {{0.5, 0.6, 
      0.8}, {0.5, 0.6, 0.8}, {0, 1, 0}, {1, 1, 0}, {0.2, 0.8, 0.7}, {0.8, 0.6,
       0.5}, {0, 1, 1}, {0.8, 0.2, 0.5}, {1, 0, 0}, {0.6, 0.6, 0.2}, {0.6, 
      0.4, 0.8}, {1, 0.4, 0.4}, {0.7, 0.8, 0.7}, {1, 0, 1}, {1, 0.6, 1}, {0.8,
       0.6, 0.2}, {1, 0.6, 0}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.8, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0, 1}, {0.6, 
      0.4, 0.8}, {1, 0.6, 1}, {1, 0, 0}, {0, 1, 0}, {0.5, 0.6, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {0.8, 0.2, 0.5}, {0, 1, 1}}, {{
      0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 0.6, 1}, {0.8, 0.6, 0.2}, {0, 1, 
      1}, {1, 0.6, 0}, {0.2, 0.8, 0.7}, {0.6, 0.6, 0.2}, {0.6, 0.4, 0.8}, {
      0.8, 0.2, 0.5}, {1, 0, 0}, {1, 0, 1}, {0.5, 0.6, 0.8}, {1, 0.4, 0.4}, {
      0, 1, 0}, {1, 1, 0}, {0.8, 0.6, 0.5}}, {{1, 0.6, 0}, {1, 0.6, 0}, {1, 1,
       0}, {0.8, 0.2, 0.5}, {1, 0, 1}, {0, 1, 1}, {1, 0.4, 0.4}, {1, 0, 0}, {
      0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 1}, {0.7, 0.8, 0.7}, {0.8, 0.6, 
      0.5}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {0.2, 0.8, 
      0.7}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 1, 0}, {1, 0.4, 0.4}, {0.2, 
      0.8, 0.7}, {1, 0, 1}, {0, 1, 1}, {1, 0.6, 1}, {1, 0, 0}, {0, 1, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.2, 0.5}}, {{0, 1, 0}, {0, 1, 0}, {
      0.2, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.8, 0.2, 0.5}, {1, 0, 0}, {0.6, 0.6, 
      0.2}, {0.5, 0.6, 0.8}, {1, 0.4, 0.4}, {0.7, 0.8, 0.7}, {1, 0, 1}, {1, 1,
       0}, {1, 0.6, 1}, {0.8, 0.6, 0.2}, {0, 1, 1}, {1, 0.6, 0}, {0.6, 0.4, 
      0.8}}, {{1, 0, 0}, {1, 0, 0}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {
      0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {1, 0, 1}, {0, 1, 1}, {1, 0.4, 0.4}, {
      0.2, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 1}, {0.8, 0.6, 
      0.5}, {0.5, 0.6, 0.8}, {0.6, 0.6, 0.2}}, {{0, 1, 1}, {0, 1, 1}, {0.6, 
      0.6, 0.2}, {0.6, 0.4, 0.8}, {0.7, 0.8, 0.7}, {1, 0, 1}, {0.5, 0.6, 
      0.8}, {1, 0.6, 1}, {0.8, 0.6, 0.2}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {
      0.2, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.8, 0.2, 0.5}, {1, 0, 0}, {
      1, 0.4, 0.4}}, {{1, 0, 1}, {1, 0, 1}, {1, 0, 0}, {0, 1, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {1, 1, 0}, {0.8, 0.2, 
      0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {0, 1, 1}, {1, 0.4, 0.4}, {0.2, 
      0.8, 0.7}, {0.6, 0.4, 0.8}, {1, 0.6, 1}, {0.5, 0.6, 0.8}}, {{1, 1, 0}, {
      1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 
      0.7}, {1, 0.6, 1}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6,
       0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 
      0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 0.6, 1}}}], {InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 15.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 14.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 13.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 12.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 11.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 10.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 9.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 8.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 7.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 6.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]a\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 5.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 4.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 3.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]a\[CenterDot]b\[CenterDot]b\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 2.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]b\[CenterDot]b\[CenterDot]b\"\>\
",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 1.5}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"a\[CenterDot]a\[CenterDot]a\[CenterDot]b\[CenterDot]b\
\[CenterDot]b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.5, 0.5}, {1, 0}]}, 
   LineBox[{{1, 0}, {1, 16}, {17, 16}}]},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{{3.44267499269575*^9, 3.44267499582075*^9}}]
}, Open  ]],

Cell["\<\
Finally, we can define the inverses, should we want to do more operations \
with this group.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Define[1/a,a^3]
Define[1/b,b^3]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "Even though we have entered this group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " we still have not identified this group in terms of the groups that we are \
familiar with. This group is a semi-direct product of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "4"], TraditionalForm]]],
 " with itself. In fact, it is the only such semi-direct product, so we can \
refer to this group as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "4"], "\[Proportional]", " ", 
    RowBox[{
     SubscriptBox["Z", "4"], "."}]}], TraditionalForm]]],
 "\n\nAlthough we have seen plenty of examples of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ability to handle finite solvable groups, there is still one question \
that we have not addressed. How do we know for certain that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will not get hung in a loop?  To illustrate the potential problem, suppose \
we tried to enter a group using the following commands (don't try it yet):\n\n\
",
 StyleBox["InitGroup[e];",
  FontWeight->"Bold"],
 "\n",
 StyleBox["Define[x^3, e]",
  FontWeight->"Bold"],
 "\n",
 StyleBox["Define[y^6, e]",
  FontWeight->"Bold"],
 "\n",
 StyleBox["Define[y.x, x.x.y.y]",
  FontWeight->"Bold"],
 "\n",
 StyleBox["y.y.x",
  FontWeight->"Bold"],
 "\n\nHow would ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " respond? ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will blindly make the following \"simplifications\" the combination",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"y", "\[CenterDot]", "y", "\[CenterDot]", 
     RowBox[{"x", ":"}]}]}], TraditionalForm]]],
 "\n\n ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"y", "\[CenterDot]", "y", "\[CenterDot]", "x"}], " ", 
     "\[RightArrow]", " ", 
     RowBox[{
     "y", "\[CenterDot]", "x", "\[CenterDot]", "x", "\[CenterDot]", "y", 
      "\[CenterDot]", "y"}], " ", "\[RightArrow]", " ", 
     RowBox[{
     "x", "\[CenterDot]", "x", "\[CenterDot]", "y", "\[CenterDot]", "y", 
      "\[CenterDot]", "x", "\[CenterDot]", "y", "\[CenterDot]", "y"}]}], 
    " "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                 ", 
    RowBox[{"\[RightArrow]", " ", 
     RowBox[{
     "x", "\[CenterDot]", "x", "\[CenterDot]", "y", "\[CenterDot]", "x", 
      "\[CenterDot]", "x", "\[CenterDot]", "y", "\[CenterDot]", "y", 
      "\[CenterDot]", "y", "\[CenterDot]", "y"}]}], " "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                 ", 
    RowBox[{"\[RightArrow]", " ", 
     RowBox[{
      RowBox[{
      "x", "\[CenterDot]", "x", "\[CenterDot]", "x", "\[CenterDot]", "x", 
       "\[CenterDot]", "y", "\[CenterDot]", "y", "\[CenterDot]", "x", 
       "\[CenterDot]", "y", "\[CenterDot]", "y", "\[CenterDot]", "y", 
       "\[CenterDot]", "y"}], " ", "\[RightArrow]", " ", 
      "\[CenterEllipsis]"}]}]}], TraditionalForm]}]],
 "\n"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436968563828125*^9, 3.436968573046875*^9}, {
  3.4409475909145*^9, 3.4409475909145*^9}}],

Cell[TextData[{
 "indicated by the ",
 StyleBox["Define",
  FontWeight->"Bold"],
 " statements, creating longer and longer expressions and never stopping. The \
result is that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will go into an \"infinite loop.\" The problem is not that the group does \
not exist, in fact exercise 8.28 asks you to find a group of order 24 for \
which there are elements ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "3"], "=", " ", "e"}], ","}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["y", "6"], "  ", "=", " ", "e"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", "\[CenterDot]", "x"}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["x", "2"], "\[CenterDot]", 
     RowBox[{
      SuperscriptBox["y", "2"], "."}]}]}], TraditionalForm]]],
 " The above infinite loop stems from trying to define this group in terms of \
subgroups which are not normal subgroups.\n\nHow can we be assured that this \
will not happen to us? The following proposition shows that if we use the \
polycyclic series and the definitions described above, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will always be able to simplify any combination of generators to a point \
where no further reductions are possible. In short, ",
 StyleBox["Mathematica ",
  FontSlant->"Italic"],
 "will always return with the correct element of the group.\n\nPROPOSITION \
8.4\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be a finite solvable group, and let\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["G", "0"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", "1"], " ", "\[SupersetEqual]", " ", 
      SubscriptBox["G", 
       RowBox[{"2", " "}]], "\[SupersetEqual]", " ", 
      RowBox[{"\[CenterEllipsis]", " ", 
       SubscriptBox["G", "n"]}]}], " ", "=", " ", 
     RowBox[{"{", "e", "}"}]}]}], TraditionalForm]]],
 "\n               \nbe a polycyclic series for ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". If the group is defined in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " or Gap using ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " generators and the procedure described above, then ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " or GAP will simplify any combination of generators to a point where no \
further reductions are possible."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436968596171875*^9, 3.436968637109375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "This is not really a proof about ",
 StyleBox["Mathematica ",
  FontSlant->"Italic"],
 "or GAP, but about the structure of polycyclic groups. However, the \
proposition can best be stated in terms of how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " handles the elements of the group.\n\nFirst consider the case where ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "1"}], TraditionalForm]]],
 ". The group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is will then be a cyclic group, say of order ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", ">", " ", "1"}], TraditionalForm]]],
 ". The only ",
 StyleBox["Define",
  FontWeight->"Bold"],
 " statement would replace ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["g", "1", "m"], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ", so each substitution would reduce the number of ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 "'s in the expression, and hence would eventually come to the point where no \
more substitutions are possible.\n\nWe can now proceed by induction on the \
length of the polycyclic series of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". That is, we will assume that the proposition is true for all groups with \
shorter polycyclic series, in particular, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "0"], "/", 
    SubscriptBox["G", "1"]}], TraditionalForm]]],
 "  is cyclic, we will let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", 
    SubscriptBox["G", "1"]}], TraditionalForm]]],
 "  be a generator, and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "=", " ", 
    RowBox[{"|", 
     RowBox[{
      SubscriptBox["G", "0"], "/", 
      SubscriptBox["G", "1"]}], "|", "."}]}], TraditionalForm]]],
 " We will then let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 "  be one element from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[CenterDot]", 
    SubscriptBox["G", "1"]}], TraditionalForm]]],
 ".  Since ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["g", "1", "m"], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ", by induction we can let ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubsuperscriptBox["g", "1", "m"], "=", " ", "b"}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is defined",
 StyleBox[" ",
  FontSlant->"Italic"],
 "in terms of the generators ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["g", "2"], ",", 
     SubscriptBox["g", "3"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["g", "n"]}], "}"}], TraditionalForm]]],
 ". Also, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["g", "1", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SubscriptBox["g", "i"], "\[CenterDot]", 
    SubscriptBox["g", "1"]}], TraditionalForm]]],
 "  is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 "  for each of these generators, and so we can define",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubscriptBox["k", "i"], " ", "=", " ", 
     RowBox[{
      SubsuperscriptBox["g", "1", 
       RowBox[{"-", "1"}]], "\[CenterDot]", 
      SubscriptBox["g", "i"], "\[CenterDot]", 
      SubscriptBox["g", "1"]}]}]}], TraditionalForm]]],
 "  in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"i", " ", "=", " ", "1"}], ",", "2", ",", 
    RowBox[{
     RowBox[{"\[Ellipsis]", " ", "n"}], "-", "1"}]}], TraditionalForm]]],
 " in terms of the generators ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["g", "2"], ",", 
     SubscriptBox[
      RowBox[{"g", " "}], "3"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["g", "n"]}], "}"}], TraditionalForm]]],
 ". We then have the additional ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " ",
 StyleBox["Define",
  FontWeight->"Bold"],
 " commands:\n\n",
 StyleBox["Define",
  FontWeight->"Bold"],
 "[",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["g", "1"],
     TraditionalForm], "^", "m"}], TraditionalForm]]],
 " , b]\n\n",
 StyleBox["Define[",
  FontWeight->"Bold"],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["g", "2"],
      TraditionalForm], "\[CenterDot]", 
     SubscriptBox["g", "1"]}], " ", ",", 
    RowBox[{
     SubscriptBox["g", "1"], "\[CenterDot]", 
     SubscriptBox["k", "2"]}]}], TraditionalForm]]],
 " ]\n\n",
 StyleBox["Define[",
  FontWeight->"Bold"],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["g", "3"],
      TraditionalForm], "\[CenterDot]", 
     SubscriptBox["g", "1"]}], " ", ",", 
    RowBox[{
     SubscriptBox["g", "1"], "\[CenterDot]", 
     SubscriptBox["k", "3"]}]}], TraditionalForm]]],
 " ]\n\[Ellipsis]\[Ellipsis]\[Ellipsis]\[Ellipsis]\[Ellipsis]\n\n",
 StyleBox["Define[",
  FontWeight->"Bold"],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["g", "n"],
      TraditionalForm], "\[CenterDot]", 
     SubscriptBox["g", "1"]}], " ", ",", 
    RowBox[{
     SubscriptBox["g", "1"], "\[CenterDot]", 
     SubscriptBox["k", "n"]}]}], TraditionalForm]]],
 " ]\n\nWe will call these ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " new ",
 StyleBox["Define",
  FontWeight->"Bold"],
 " commands \"first category substitutions\", and all previously defined \
definitions as \"second category substitutions.\" Certainly these definitions \
are compatible with the group structure of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", so if ",
 StyleBox["we",
  FontSlant->"Italic"],
 " can simplify every combination to a unique form, this form will be the \
correct representation of the element. \n\nThe only thing that would go wrong \
is if there was some expression for which there existed an infinite sequence \
of substitutions from either category. Suppose that this was the case. That \
is, suppose that there is a infinite sequence of expressions\n\n              \
         ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubscriptBox["u", "1"], ",", " ", 
     SubscriptBox["u", "2"], ",", " ", 
     SubscriptBox["u", "3"], ",", " ", "\[Ellipsis]\[Ellipsis]"}]}], 
   TraditionalForm]]],
 "\n                       \nwhere each expression ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "i"], TraditionalForm]]],
 "  is formed from a substitution of either of the two categories applied to \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", 
    RowBox[{"i", "-", "1"}]], TraditionalForm]]],
 ". Note that the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "i"], TraditionalForm]]],
 "'s do not represent elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", but rather expressions which are products of the generators ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["g", "1"], ",", 
     SubscriptBox["g", "2"], ",", " ", "\[Ellipsis]", ",", " ", 
     SubscriptBox["g", "n"]}], " ", "}"}], TraditionalForm]]],
 ". In fact, all of the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "i"], TraditionalForm]]],
 "'s are different ways of expressing the same element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". If such an infinite sequence of expressions existed, the computer would \
have the potential of running into an infinite loop.\n\nLet ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " represent the number of times that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 "  appears in the expression ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "1"], TraditionalForm]]],
 ". Note that if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "=", "0"}], TraditionalForm]]],
 ", then the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "1"], TraditionalForm]]],
 " is expressed in terms of the generators ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["g", "2"], ",", " ", 
     SubscriptBox["g", "3"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["g", "n"]}], "}"}], TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ". But by induction, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 " does not form any such infinite sequences. Thus, we may assume that there \
is at least one occurrence of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 " in the expression ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "1"], TraditionalForm]]],
 ". By the same argument, we can suppose that there is at least one \
occurrence of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 " in all of the expressions ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "i"], TraditionalForm]]],
 ".\n\nConsider the first appearance of the generator ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 " in each expression ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "i"], TraditionalForm]]],
 ". If we let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "i"], TraditionalForm]]],
 " be the part of the expression occurring before this first ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["w", "i"], TraditionalForm]]],
 " represent the part of the expression occurring after it, we can express ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "i"], TraditionalForm]]],
 " as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      FormBox[
       SubscriptBox["v", "i"],
       TraditionalForm], "\[CenterDot]", 
      SubscriptBox["g", "1"]}],
     TraditionalForm], "\[CenterDot]", 
    SubscriptBox["w", "i"]}], TraditionalForm]]],
 ". Note that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "i"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["w", "i"], TraditionalForm]]],
 " may be empty expressions.\n\nSince ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "1"], TraditionalForm]]],
 " contains no ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 "'s, it is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 " and so by our induction hypothesis, there is only a finite number of \
expressions which could be produced using substitutions from the second \
catagory. Let ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " denote the number of generators in the longest such expression.\n\nWe now \
will show, using induction on the number ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 ", that an infinite sequence of substitutions is impossible. That is, we \
will assume that an expression with only ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "-", "1"}], TraditionalForm]]],
 " occurrences of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 " could not appear in an infinite loop. Note that we are already using an \
induction hypothesis, so this is an \"induction inside of an induction.\" We \
will keep the two induction arguments straight by referring them as the \
\"inner induction\" and the \"outer induction.\"\n\nNotice that the first \
substitution of the first category:\n\n",
 StyleBox["Define",
  FontWeight->"Bold"],
 "[ ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 "^",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " ,",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 "]\n\nreduces the number of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 "'s by ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ". All other substitutions of the first category preserve the number of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 "'s while all substitutions of the second category do not effect any of the \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 "'s. Thus, if ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["g", "1", "m"], TraditionalForm]]],
 " is ever replaced by ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", the resulting  expression would have only ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "-", "m"}], TraditionalForm]]],
 " occurrences of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 ", and by the inner induction hypothesis would not\nget into an infinite \
loop. Hence we can suppose that the number of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 "'s that appear in any of the expressions ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["u", "i"], TraditionalForm]]],
 "  is the same, which is ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 ".\n\nFor each expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["v", "i"], "\[CenterDot]", 
    SubscriptBox["g", "1"], "\[CenterDot]", 
    SubscriptBox["w", "i"]}], TraditionalForm]]],
 ", ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can do one of these three types of substitutions:\n\n  1) A substitution \
of the second category applied to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "i"], TraditionalForm]]],
 " .\n\n  2) A substitution of either category applied to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["w", "i"], " ", "."}], TraditionalForm]]],
 "\n\n  3) A substitution of the first category applied to the last generator \
of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "i"], TraditionalForm]]],
 "  and the first occurrence of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 ". The resulting ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", 
    RowBox[{"i", "+", "1"}]], TraditionalForm]]],
 " will be shorter than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "i"], TraditionalForm]]],
 "  by one generator.   \n\nBy the outer induction hypothesis, since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "i"], TraditionalForm]]],
 "  is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ",  only a finite number of substitutions of the first type can be before \
doing one of the third. Likewise, by the inside induction hypothesis, since \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["w", "i"], TraditionalForm]]],
 " contains only ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"d", "-", "1"}], ")"}], TraditionalForm]]],
 " occurrences of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 ",  only a finite number of  substitutions of the second type can be done \
before performing one of type 3. But the size of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["v", "i"], " "}], TraditionalForm]]],
 " goes down by one each time the third type of substitution occurs, which \
could happen only ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " times.  Thus, the computer will not go into an infinite loop when the \
generator ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["g", "1"], TraditionalForm]]],
 " appears ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " times. Thus, by the inner induction, the computer will not go into an \
infinite loop making substitutions on any combination of generators in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["g", "1"], ",", " ", 
     SubscriptBox["g", "2"], ",", " ", 
     SubscriptBox["g", "3"], ",", " ", "\[Ellipsis]", ",", " ", 
     SubscriptBox["g", "n"]}], "}"}], TraditionalForm]]],
 ".\n\nWe now can close the outer induction argument. Since we have shown \
that there cannot be an infinite number of substitutions on a combination of \
generators in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "0"], TraditionalForm]]],
 " provided that the same was true for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ", and that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "0"], "/", 
    SubscriptBox["G", "1"]}], TraditionalForm]]],
 " was cyclic, we can see by induction that no such infinite number of \
substitutions are possible on the original group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436968644765625*^9, 3.43696865696875*^9}, {
  3.43696869275*^9, 3.4369687738125*^9}, {3.436968811171875*^9, 
  3.43696899225*^9}, {3.4369690671875*^9, 3.436969501828125*^9}, {
  3.436969537265625*^9, 3.436970130515625*^9}, {3.4417204798077497`*^9, 
  3.4417204981202497`*^9}}]
}, Open  ]],

Cell["\<\
This proof is a bit longer and more technical than most of the proofs that we \
have seen. Yet because this result is the foundation which allows this set of \
notebooks to exist, it is included. It gives a good example of how the tools \
that we have learned throughout the course, such as induction and reductio ad \
absurdum, can be applied consecutively to solve harder problems.

We will close this sequence of notebooks by returning to a problem introduced \
in section 2.3--the Rubik's Pyramix\[Trademark]. The understanding that we \
have learned since chapter 2 can be applied to help us to solve the puzzle. \
The techniques that we will show could be applied to any of the Rubik's \
puzzles.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436970147984375*^9, 3.436970152390625*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solving the Pyraminx\[Trademark]", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
In section 2.3, we introduced a very large group called the Pyraminx\
\[Trademark] group. This was the group of all permutations of the faces of \
the Pyraminx\[Trademark] puzzle obtained by rotating the four corners. The \
Pyraminx\[Trademark] puzzle is shown by executing the following command.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369701705625*^9, 3.436970174265625*^9}}],

Cell[CellGroupData[{

Cell["ShowPuzzle", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675010102*^9, 3.44267568763325*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "This group was described by four generators, ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["l", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 ". These four elements represent the 120\[Degree] clockwise rotation of the \
right, left, back, and front corners respectively. Recall that we could \
perform multiple operations in Mathematica using the command RotatePuzzle. \
Thus, the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotatePuzzle[r.l]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750116645*^9, 3.4426756902895*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750116645*^9, 3.442675690367625*^9}]
}, Open  ]],

Cell["\<\
first rotates the right corner, and then the left corner. We can reset the \
puzzle with the command\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ResetPuzzle", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.44267501350825*^9, 3.4426756917895*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "What we want to do in this section is to study this group using the tools \
that we have learned throughout the course. From the result of this study we \
will be able to learn enough to solve the puzzle. The techniques used in this \
section can not only be used on other Rubik's puzzles, but on many other \
puzzles that are marketed today.\n\nBecause this group is so large, (933120 \
elements), we well not be able to have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " list all of the elements as we have done for the other groups we have \
studied. However, we will still be able to gleam information about the group \
by studying the puzzle.\n\nWe will begin by asking whether the group has a \
non-trivial center. That is, are there any elements which commute with all \
other elements?  In terms of the puzzle, we are asking whether there are any \
sequence of moves such that the sequence, when performed in conjunction with \
any other sequence of moves, yields the same effect regardless of the order \
these two sequences of moves are performed. \n\nTo answer this question, we \
notice that the four corner pieces, although they can rotate, will never move \
about in the puzzle. Suppose that a sequence of moves rotates one of these \
corner pieces, returning all other pieces to their original positions. The \
result would look like the following:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["PuzzlePosition1", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.44267501638325*^9, 3.442675693992625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
If another sequence of moves was performed on this position rather than the \
starting position, the only difference would be that the front corner would \
be turned another 120\[Degree] clockwise. Thus, the action of rotating one \
corner 120\[Degree] commutes with all other actions taken on the puzzle. So \
this action would be in the center of the group.

What sequence of moves produces this action? With a little bit of \
experimenting, we find one set of moves that work:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ResetPuzzle
RotatePuzzle[f.r.f.r.r.f.r.f.r.r]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.442675695461375*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.442675695555125*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.442675695617625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.44267569569575*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.44267569575825*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.442675695836375*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.4426756959145*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.442675695992625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.44267569607075*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.44267569613325*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426750192895*^9, 3.442675696211375*^9}]
}, Open  ]],

Cell[TextData[{
 "There are many other sequences of moves that work, all of them are at least \
as long. \n\nNote that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has a nice animation feature for repeated rotations. To form a loop out of \
these sequences, try this: on the right hand side of the pictures there are a \
sequence of three brackets. (Two of them are so large that only the bottom of \
the brackets can be seen.) With the mouse, click once on the middle of these \
three brackets, turning it black. Now hold down the \[ControlKey] key while \
pressing the ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " key. Cool! There are some controls at the bottom of the screen which allow \
one to speed up, slow down, reverse, or pause the animation. To get out of \
the animation, click anywhere on the picture with the mouse pointer.\n\nThis \
element obviously has a order of 3. In fact, we could consider rotating one \
of the other three corners as well. Since the four corners act independently, \
we would have 3\[CenterDot]3\[CenterDot]3\[CenterDot]3 = 81 elements in the \
center of the group. Let us call this subgroup ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Using the above sequence as a model, can you find sequences of moves \
which rotate the other 3 corners?\n\nAre there elements in the center besides \
those in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 "?  Actually, there are. Suppose there was a sequence of moves that produced \
the following pattern:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["PuzzlePosition2", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675025648875*^9, 3.442675698617625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
Notice that in this position all corner pieces are in their proper place. The \
edge pieces are also in the right position, but they are all reversed. So \
such a sequence of moves would effectively reverse all 6 of the edges. If \
another sequence of moves was performed on this position rather than the \
starting position, the difference in the end positions would be that all 6 \
edges would be reversed. Thus, the action of reversing all six of the edges \
will commute with all other actions taken on the puzzle. Therefore, the \
action of reversing all six edges would also be in the center of the group. 

What sequence of moves reverses all six edges? The shortest solution is as \
follows, which happens to be easy to remember.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ResetPuzzle
RotatePuzzle[l.l.b.f.l.l.b.f.l.l.b.f]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.442675700242625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.44267570032075*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.44267570038325*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.442675700461375*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.442675700523875*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.442675700602*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.442675700680125*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.442675700742625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.44267570082075*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.442675700898875*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztnflzpMV5x4c5NCONNJJGtzSSRiutpJW02oNlbYMxBQ4YYxtzX7vsIsPC
Lnt6d4FlwRhzGDBgMCEYE4yxCU7i2PGdy0l8pJyyK47jMrmd2ImrXOVKufIH
5Bdl37f7ffqddz496ndfSWCX7FrpZdT9fL+fp/u9+u235z0Lx/fuObhwfN/N
C9V3HF04snffzceqFx0+euqjzBmp1Bk/TqXShWrK215MpYIf/v9e837o/1jb
Xtte217bXtte236Dbr/q/covBqevKfXpK96vnPdp1ivfLn/vV3//tPcr7X3q
FVJbnd5WSUq2qJIveb8y3qfNUnJYYv6fKvSihCtJoRkJ9ytV6AUpVJZC2yXS
f6pCz0uhXin0din0Q1XoOSk0KIUuE7lvqULPSqER708pn8FPnvdrh8T8sir+
jBQfk5i3S8xXVKGnJBnrpdCd3pb/2cuq0JMSaUYKHZWtUyUXU/7WS6r441J8
Xgodkq28FH9RFX9Mim+t9elvFaX4C6r4I1J8uxS6VbbapfjzqvjDUvxsSNqC
VOySis+pig9KxXOl0G7Z6pPiz6ri90siz5dCO2RrSIo/o4rfJ9EvkkLXytao
FH9aFb9Xil8iha6UrXEp/lFV/KQUv7S2OwX46v9em+uKT6iKJ6TiFVLx3eEd
QBf/iCp+pxS/BpL7Dqk4LxUfVRWPS8XrpdCFsrVVin9YFT8qxW+UQibN26X4
Q6r4YWkL076mEc+W4g+o4gcluulHb5Gtt0nx+1Xx/VLc9NKzZOsCKX6fKr5P
ipt9YItsXSTF71XFb5PiRyGlG6ERL5YQJ1WIPRLiDtGZla33SPETqvjNUvxu
KTQlW5dL8TtV8QXvV17VD3VAvXW1FD8eKu4bznp/8g9Q+8MdPSX9QFc8Gqro
f+DzLi4GNzWdEmdXeP9KSQfRcY5E42QgecMS7HIJ0SdbCxLskAp2k/8rGw4h
kf1ycxLvt6D5zGlij0Q+0DCyOZm9RSKb416HbO2TeLeH4+Vq46mYqSZIxIUS
foMEbYNy+0Vo79JCeQl1lYSvyGfm9HtEgt4aDlqrnPZ/1ia0qbapfIF2+cwk
77gI3BJPoCAhDolAWj6L9MwTovI+u0oGVJoh0Se9n/6p+piEvSle2BYxeo+E
yPgRauvmBO1WKbfLLpUFqaJI3SUhmryq/la3CJjdfKcS2A0CORBoEwGTjhYR
GBOBd8tfb7ALNIFACdrgkAQridS8SJ0nf70unpTpoQdDfz31Nz9Kh0idI1Lm
jHiNXSoPUnSE6BGBi0XAXAlcFU+gDGm7RYINiNTVItUvf73CLlWwSendKwgx
DN3ZHArMxeNl8aS6RWqXhKgKyxERuNQethnC9kjYGyTshIQ1O2nMsH3QBuZE
PC0C/q6ZDQnsAoGiTSC46NTB5iTs8eCooDuxPhy+N57GgGi8VzQ216bc30rL
Zw0EWkFgELL0Ltj/torAATxqLuqjfUx5c4nyTgn7Jui9e8GSzxxk2Fho4KAN
HJgz74Xi4GyBNYf+HMBeFk9qRKTOl7BvEymz+zYxWAO1EqiNQsu+VTQuEN3d
kNrlcVAVXnN3cZHo7pDPCn7UXI2aQ4rbQdLc1Jv7n0tE8lr5rLmB5OXxJM1l
vjkfXSqSV0F2HcRvBPFOEJ+ARt5YcyqJ7knmOFKM9OhFdRxJGbv+/4IdytWS
GTKZA/YrJTPmlqvV0chldiNlMDIpRqZF6jqRvwTMrZCRaWgkMzqxUyyZ294S
GMnEM9JlM2IuDeXqOTgkBJ+1g3w2ubwZJxsWqVtE/jxoEDJCeWiw13SDEXPf
PyBG9oqRc+SzTsc8xJSn8YpuET0AO+2bIDcdYC4Xz1yPzZxvpCyWDktuzpLP
uuLJ7wT5XpDfJPIlkTom8lvks25H+Svs8n0gvxmapgWSf4dYmoe/Um6akpsz
A2QFycNJMWLGIXuTy/eDvBmIzorUfSI/KZ/1gXw+ufw276d/SfIA7CHroBko
D2TkSruRAZsR/0bnLqGvCv3ACor6Am1eBV/+dpGviPwgyBfiyQ+C/JkiX/Eq
eDfQoRPXACSf8kBGrjoNI5HL8CGQalOnj5S6Hwhu8VOSr1jmdoC5im0PiZgj
qSExZy7DO8XcMNRoTm7OP3pYrjopgzPeT7+ht4nJEjQ0AbbEsztsPRKzXZKc
C0kGRf2hPj/PU3DIaJaMjzpm/OrTQWh0OWnfVQhnQHDMCJJ5cFh1bIcGECO2
E7IDBB14ZgDCH0j3u1VRILLQrahNiiuDkwEcapMNgOM/P/UeqZ86AyU2PAqG
5x0NU/6nwfDF4Xi67pij9WviWd8I1nNgnXI9BdbfCdap17fGs14F63OO1inr
623W07Zu0sDwDWB4zNFwEximXJPhcDdRqtxN2uJZXwfWZx2tU67Hl+gmqQbd
hKxfm9x6HqxT1tc5WI+T9QbWx8H6jKN1yvqYtcPUPaqlxJdWxn0B3FPiq9bE
17kfdXR/nd39BLifBvfN4J5yP2J1z0+bVxGhBRCoAQjh4pgI7auHQK1QidkK
IxCjIx7CekCYAoQiIFArDLkcfxZrJwAE8egCnFrkeoVzPeBMJsChFhm0dqpa
3Qzg0O0PtU4DnCnAmQScVsCh1hlI0DqE05kcZz3gtAEOtU6fI45r68TEmQac
8KOTIGwJcKh1CIc6G93YUHoI54bVw+l1bB3CoXjleDgbAGcccNodcboT4FDr
vAFxqLPlErTODjvODOCsA5wOR5wy4FDrEE4/xOt6fXE6HXGakuNcBzhzgFMF
nLIjTgfgUGdzxel+fXHaAce1deg5BuHstONsBJxRwOlyxGlzxMn/+uJQZysA
Dj3c6YmHMw844ZdIOEt2nCLguLYO4fS+8XCWu3VutONsApwK4PQ64jQ74jQD
To9j66wiTgFwqLMRDj0871sCxzx/j4gOAUSfI0QeIJK0yVIQoeKnfnsPaJYV
p8kRh2YUuLaJnoKuLm3MDD0lib0sPMUziNzvSJQFIuplRESzIPrdifQ/v9/R
iWe5saihismxwkM5QfHw87NAahYQBwBxwBExvQqIu+2IGWk7AusHsEErGD8E
ftfKwOwAmJruoqXoLi885TyoO2TFqpteewkQtQJRGYgG4hHlpHlicfgPOqPz
RlMW621gnWbUDdqt7wTrNQdZLUXDVb0AUamRUpbMfBo6sLlCUP5vskPkJf9J
rPMN3Kkfb17m5tAkNwJJQUhoRLcHSIaBJHpz4vUp/866Q+eoDoemotCUyKF4
OOEHAIHUOIB1A9gIgPXyru4/QQte/FkJtl02Nv16TT1RFxCNAlFPg2PyJmmy
PGC1AxbN8CWsBTtWeEg9kKJH5K6AdCVWM+NFf3au99N/57fTEaxiB9ttA9OT
Bupxwu+YBTWqgENniyzgXABpdG2xmGAlkBpdBcTzQbcDEGn6/bAd8SYbopka
GgHrBLAxAKOdvgnAzgMw6pRtjmDwpm5QvEPAaHqVKxh1orwjmGuLVcSpfr15
AXA6QYBmT7YD2DiAkZECgJ2boMUM2B47WFkKJcEh+WbAeSvglAGH3rCpROsa
sPcBWBdI0Uzh8KvLQd0JQKTXoIqAeE4CxEFpDO8PitFHa8DZLXVWg+7spHT1
k2duU2w3A1uPsNFc9/AKC0GN9cDWDF7agO3NwNYFbJSrAXGaqamxGG4+QuwD
UXrhxRW2AOZKywzbD3XTgK3Xo7jFhq1fA6mHLQLsFMDmwVw7wG4Hw90A2wLx
jNMsIN5mRxxYQjQubBOY63CEpZa1wkbqNsDeY8M271FEYFsAdhpgs2CuDLDb
ErSsObo0AeJeO+IQiNILncsNeybo9gAsHfB6oG4OsPcp7Ftt2PqlvnrY8DIs
QY0NAJsGc90Au9URlg545pxYiIc4LBXplWU7Yi5syf+XiuxNQfktCdqQLmny
AKhXMboNAEcgBL2tXgBU9ZpGNmzT2+4Hyk0g0+vYeOaKtBnY9tvZRqUiEYVH
GoIas5H+mUpFRuhi4tD5qAx1G4DtBbAqhKCFI8JLUwV15wCHEDeCRp8jornX
KwLYATvYmFRcbpw5wOkHnKVOqkHdFgA7qMD2Adg6CEHrtuQAcaMj4ixoUIsR
Yjsm3gFsHJt6OXBmAMe1xWigqBXA9EJ0twPYhIC1AFgWwOYdwabBHL0JG105
RrVO4KoEOIftOJMgSitQZQBs0zKD0eVNG9RtgLjfhlh3kg/A0gC22RGMkufa
YkVx1QE4R+w4lE1ayy0J2HrQGHRssSLUbQfE9yvEAzbE0AV3LgwWFKq58As+
HITPJhxhostPLabCjzjK8RDoKJVpAHMmGKeWGYe4Q44wzVC3E7D02qAHbVih
m9xsBAY5qFHGEnCYR4Td8dzPScXwKETQBGc5NoGrdbqBKUDdLoA4piAOAQRd
gOm1VNxSPwoBKo7+mySDPeD6uN31/GLQW7aDJUoz3YEMh/YgbdP/l7KktoHJ
w2Byk9CRScrlMIhWGpg0z+n7wNoddmubG1qj/NUNgy+RP5oH0Qsm9QrFR2z5
y8Tpi4MgOhI6soQMmikmA6fhKhgs18H8/9oGhiiTdpNmh0mlal/oDsr1g9e7
lNf3W3fuuqOqWY7R9aRHQ6+WxJqpVYNg9kRss3RR4HrhQeOKo5DqDJRr4P6o
7WRgORtnJd9knPLdazOuTzdBiitg8u7TM0k3OK73GjTKVxW7Kr3BH9GzXgX9
GHiuud9UH9dejIYvHtM215RkGtfyXft9cGRlbJoFll3vUK029eIw9TbvUTaP
g01aJCQLNmm8ZQbMUV7poXVjw/cmN1yQvM6CJcorjbf4NjNL2LwDbNJaJjmw
SeOr0wkMj0h/JcMfSG7YrNe9wdEmjYrQZIy6WRqBa/0lDHeCa1p3pQlc0+PD
SUf/NEZgVoSM7tOplYNoldRPOVqnuWbh68foMT98TR6UHwKYDyqYuwBmEmBq
xrw1TJvArHeEoTt/850s6hhQd/VBJ5f77e5pbZkCuKcJQeOOHHSfPAT7Q4aJ
hu1EJ4BoAohqHiCFidJJOcwCperYwfcC1CgfSo5A04PHHGHyNhg8JDIW7Sn6
62nuBixaDKjmUW0YKx0Hhu61zIr9tDfSjk/3QcuAQy84jDqC0S0QTdzIx2yl
BxXWScCidY9qpkqEsdJxYOg1jV5pJerg1Ep0B/iQHYfWQmoFKfP9EiOOOPSW
EM0gosMpfaFJA7B7AKwKYG0gRa96DiVANCshU68gMBqaeTg5WJ+0WMWKUzex
nxqIdlpXDv1dX/cCxyhwlECKXsunxoh+5u3bZslU6tPZ1YMwK17SDZGZm0QT
sVyt0+jZI8r6B8A6LWPVDlKDYp1mVNBsMrMWLKUiB9Zp4PTR5NZpnadegKBh
IJpoQ7sZ4VBLaJz7AIdW4+oEqYq0hCuE+ebEjgQt8Vhy67SAGM2uooEimjVD
rd0EOPTERn/P4QcBhxYVI5xRaQmC6AYIsxw8xcuvjPUySFWhJWhSH40q0WwY
wnFticcVzv2AQ+uhEc6YtARB0CSnVmmJLohHLUFPzJ6IZ52k1on1TrBOgzn0
FUSUlEI8iA8BRD9AdIPUBHQnmtdtnbwUPN1zgqCn4k/GgyAp83VdNEubxqto
Mg6lxxVCf8frAwBBi7URxCS0RCvg0PCVWUu8FyI3rwwESU1KS9CbEDR8RZNs
XCFoNsJTCuJBgKB15khqGlqiBXBoAMssiN4PkVsAgubzPB0Pos8GkY7aJJzA
CH0rIkUmCGoJDfEQQPQ4QswIBL31RQNWOcn/oKN1yv/H7NZpObwBkJqFTkSv
GNJAFc3PIo0i4NDcKf391g8DTpcjjvlmQVeIjLTEEMRrXRnr1OgboSVyAEGD
URloCVccmnf42wrnw4BDCxMSzry0BEHQIFRaWmI4gfVn41mnLG2ClsgAREa3
R8iSNpaqXVJMx22Lh/AIINA6ioRgvv2SjNOAUvBQ1Mk1TZv+HbtrWi6R2niL
uE7rXhKKoj6u7+eU6RJ4phnRzynPj4JnWhORPJ8Z6SypwL3/UdXRHCU0pjnK
g2/O76DrHI1Qlj6ujDwGRkqORs6SliUj7StjhHrztrCULkeN1AGW6PWO55Wl
j4AlWrcxiSXKEln6hN1SK1gaW6JTN7LumiVt6XGwROsnkqWtjpY6wRK9vvJC
PEvUbbcksETvQP2usvQEWGpJYIn2R9csxbQ0DlKbwRIdSMuOWXpRWXoSLNHi
lcttiV6p/GQ8SxMgNe9oqSuepY+CJVpOcr2jpUoCSy/FszQJUhsTWKJXRT+l
LD0FlmiVSldLdEXWvTKWpkBq1tFSD1iid8FfVpaeBku5ZbZEWSJLn45naRqk
ZsAS3ci4Zklb+hhYyoClDY6W6C63FyzR2+yfiWdpBqQ2JLBESye8oiw9A5Zo
LU6yNO1oqW9lLM2C1BRYooE1V0u/9/payoClV5UlNTLA69rNQ/hJsEFDaP2r
b4PGVAfABq3t8lll49lIA3nymyHsRIIskPzvJ5dPQq/l/Tv+YMXrumDjsUR5
FOVzpytETy7Ul5bULcD0R0rjuYYa60CDHsQMNoDRQh8XIWqqsWQwn1canwh1
jCAK7RZVR6zwlwsFOF9QUi8IjqsAPaSs2AVeBJY5CDHqKEUsf6ykPiksrgL0
WNTMn/iiCvspIKAT27CjwHBU4GXxnSSs8f0lFfYz4JuupCogQM/ra76yWR9K
NMErQuAqQDNNSECzvAosdDk/6MgyJMn6qhL4rBC4hiWCISDQAn8ABHQnOQBS
NI+FpL6mpP4QpMYTSJlpaX+qBD4nySKCfghrne0WIdACnwcCGlXqc5TqF4K/
UAJfEAIahul1DNsHBFrgi0Aw6ihlnacZkfqGkvqSsFShYg8I0FzBXknRX6uw
XwECGtJwFegBAi31VSGgwa4uR4FuIfi2Cvt1IKC7VhKgabndQKCl/kQIaDyj
7CjQBQLfUQJ/Bix010RSNDW3LMn6rhL4cyGgu+hOCEtzzctAoAW+AQR0Ed3h
SNAhBN9TAn/pKNDuyNIBLFrqryRZrgI0f50Evq8EvgksdB1ZcpQqSbJ+oAS+
JQQUti1O2AiBFvgOEND1XCtI0bsmJPX3SupvhCWJgHlP7kcq7HeBgK4liiBg
fREvQqCl/lYIXAXoJcuiELymwn4PCOgE3+xI0AIEWur7QuAqYH1NNCLwj0rg
74CFTpQFkKI3fZslWf+iBH4gBEnCFoBAC/wQCOgskAepjK2cb/jflcA/gAAd
OZviCERYtNSPJFlEkHMUaAKBnyiBHwMLHYFIil6Yz0myfqoEXhMCOvZndaVQ
WP2P06KD/hO4ph2pUXjj9L9V0H8Wp/Z38BqEijjVQf8VnFJ/Tmu/teEx8s9V
5H8Tu43eekyLYs3f/Hq/UIF+AhZVtTyFFOoMeNMh/0O8NdI3C5v8UlVTrdsS
TUJUOfjv/1G1fub/ykZLRBd98TZ0jf/yfnn9JfUr9cHPxe//qg98mtfWtte2
17Z/k7ZTZ/w/TNA/pw==\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.442675700961375*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztnfmTHsV5x1/eY98933ePd+9bqz20u1odgGwLYwoSsIxtwNwgIWkNAgl0
WRKXwBhzGDBgMCEYE4yxCY7t2LETJ76TOE45ZVccx2VyO6erUpVKufIH5Jc3
eqd7np6Z99Pz9jC7AqcWl/d91dPzfL+f7p55Z3q6e967fGL/vkPLJw7cuDzx
zmPLR/cfuPH4xEVHjp1Kyp2RyZzxs0wm2zyRqX2vZjL+H++/12p/9D/Wvq99
X/u+9n3t+9r3N+n3V2sfxar/8zWrUl+pfRRqqfla/rJsH1DbP1P7yNZST2Wq
ZrxvXbWcJcnZqnK+VPvI1VJbJOeoxPxflelFCVeSTPMS7pcq0wuSqVsybZNI
/6IyPS+Z+iTTr0mmn6hMz0mmIcl0mch9T2V6VjKN1TJlPAav8GofOyXm76vs
z0j2SYl5q8R8RWV6SgpjWjLdUcvkpb2sMj0pkeYl07FaJu9bTtJeUtkfl+xL
sumwZC9K2osq+2OSfWvYp/etTdJeUNkfkezbZNPNkr0sac+r7A9L9u1QaMuy
Y4/s+Jza8UHZ8VzZtEey90vasyr7/VKQ58umnZJ9WNKeUdnvk+gXyaZrJPu4
pD2tst8r2S+WTVdI9ilJ+5jKflKyXxJuTj6++l+gzp9QO94lO14um94jOqb2
P6qy3yHZr4bCfafsaNrBo2rHE7LjdbLpQslu2sFHVPZjkv0G2XS+ZDft4CGV
/YjUxbJsOleyb5e0B1T2QxL9Ztn0Nsn+Dkm7X2W/TbLfKpvOluwXSNp9KvsB
yX5YNm2R7Kb271XZb5Hsx6BIN0Il7pAQJ1WIfRLidtm0IIrvlbS7VPYbJfvd
smlWsr8vcFrwsi/XPopqf3/TlGS/StJOBLJ7hvO1f3onqNsk07jsaNrBscCO
XsIp3qpHqm5quiTObtlnWOKYBnI0GscLowtPl+ipE78fzID2SzDTfA6rYHu9
j7wKUc34vrzIXr5FiffrUH3dEnmfRD4YG7kgkd8mkU1775R4BwKnzkC8Qiie
jplpgoK4UMJvkFAd0NpM3e1vLFQU91dK+BEJ0CLujwbO5IGguWDQU9sDrnVa
kwgsi4D5CSiIwAlJuymZQLMIHBaBrATzMlelZZoD4v12lRyotECFnKz99X6q
j0vYvcnCtor5exRf5CDw69S7YipXQyfA3XapPEi1idSdImUqpyIC5jDfpQT2
gEABBDok2HERMHyTIvAeEbjeLtAEAiWog8MiVRKpJZE6T6SuTSZVlmCHqtVM
dGunbD1HpMwv4tV2qSJImWAHhKVX0naIgLkSuDKZQDcU200iNShSV4nUgEhd
bpdqtkn5h1fGv3Cvb87LImUuHi9LJlURqd0iNSFpR0XgEnvYFghrSv56Cbs+
cJD6bhOG7Yc6uEoE5gKHZkb/xGiB3SDQZhPQF51+2EVJ806ugZ8ufTq8NJmG
aSqXisbmcJF737LBfDaBdhAYglJ6Nxx/W0XgoIhGLx0yyeWHJey7hO8t0Hr3
B64b/H3VL1M+aL1BAXSAgxFxcKE42C5p5tRfANjLkkmNSdjzReodgcPXl2pi
sBi1EqiNQ82+XXQvEN09ULQr48CcG7aL7kWStlN41bmmEFJzKOIySE5K+G0i
ebGkXRO4yrJLvi+Z5JSE3yqSl0jalVC6DuI3gHgXiK+HSt4oNi6HI+lSKYM2
2DcbOvS9//wDytXStLAvhoKprVfIVnPL1e5o5DK7kW4wMiNSc1Ii10raxVAx
q2RkDsJOiaVdYsnc9tJlVy6ZkR6bEX1p6MvvlbQLRL4M8vn08vMiNSryN0na
eVAhZITKIeaoqYCRBREdFCP7Je0cKYcukKdySCi/EcJWxMjBwFb/oH0LlE0n
RCkkM9drM+cZ6RZLRyTtbCmbnmTyu0C+D+Q3iVRJ5I9L2haRrzjKX26X7wf5
zRC2NRRW5btdLC1B1VDZNKU3t0VEm6VsTkqa6YfsSy8/APLmEjAv8vdJ2ozI
04V3Mb38WbW/3iXJA3CErINqoHIgI1fYjQzajHg3OncK/YTQD66iqCfg9T95
8rdKmuk+ouv55mTyQyB/ZlAqU611sQV+uAah8KkcyMiVr8NI5DLc6+eMtIcO
uRu9VGz2SimNJDO3E8yNgLmtYG4EzA2LOXMZ3iXmRsFcS3pz3tnDctVJJeid
T7yKNm2vBBVNZdmazO4o2N0cY5fKdDFgzc/aKofKbEDB38M86Bx3LPGrXg+C
/XLStznoiDMoOKZbwDw4nHCshxiIMYDY5AgxZGtCkXwbqv75w/SO5qFZUZ20
rQ5ODnCoTjYAzvba39oj9VO/QKkNj4PhJUfDVP5zYHhHIJ+vMelo/epk1uk6
twDWqaxnwfq7wDq1+vZk1ifA+qKjdSr1aZv1rK2ZxBi+HgxPOhpuAsNU1mR4
RzRfhptJRzLr68D6gqN1KuupBs0kE9NMyPo16a0XwTqV+joH60lKPcb6FFif
d7ROpT5pbTD5YL6q5VeptDrum8E9FfyEteDr3I87ur/W7n49uKcOoRZwT2VP
o5yU+0LYvW4+pxGhFRCoAghhR0KE8ulDoFoYSVgLY6DVmQxhGhBmIWwbIFAt
DLucf6r6wX5El675qUauUzjXAc5MChyqkSGXHzGVo0532LF2YnBmAWcGwrYD
DtXOYIraIZyu9DjTELYDcKh2+h1xXGsnIc4c4NCjkxLgUO0QDjU2urGhHhrC
uf704fQ51g7hUD9PdzKcDYAzBWHLjjiVFDhUO29CHGpshRS1s9OOMw84dBPT
6YjT7Vg7hDMAuj1vLE6XI05TepxrAWcRcCYaVHocTqdjY3PFqbyxOOUUtUPP
MQhnlx1nI+CMN6j0OJwOR5ziry4ONbZmwKGHO73JcJYAhy7UK444bSlqh3D6
3nw4K107N9hxNgEO3Xf0OeK0OOK0AE5vA903AKcZcKixEQ49PO9vgKOfv9dD
0OV5vyNEcYXrpBFEIPupz/ADmhXAaXLEaU1RJ3oIurq0MSP0lCS2MrqkHXAk
yju2MiKiURAD7kT6/167ox+elcaiimpLjxXqytHZw2PKVNgFQKSr90FHxOxp
QNxjR8xJ3REYXSkOWcG4S+3dqwOzE2DCY75UWLrLo0usYStWXW/zxUDUDkQ0
u2AwGVFBqicRh/egsy1q02K9A6zTiLohu/VdYD08kkmFpe4qugQZgcow42no
xOYKQeW/1w5RlPJPYz143Pv5vEfrb13h6tAkNwCJGYpGPbr0Iz0KJP0Rklqb
8jrFOlUZ1ePQUBQaEjmcDIcG+9EDKPq5HgOwvqo51KsGzht/60/8WQ223TY2
M73G4Tw5DkS9VXNOroapapcfusqKgFUGLBrhS1jLdqzw03MVlh6RuwJWIm2x
9iX0i63Tzq399eb8djmCjdjB9tjA6gcNxPwKTABON+DkAeeC4L4JaywhWPj5
owo7fhoQzwfETkCk4fejdsS9NsT6cVkx591JAOsEsCYAOw/AqFF2OILRTN2M
//vig9HwKlewMoAVHcFca2xEnOrpzcuAE37yooLR6Elq9VMAVgKwZgA7N0WN
GbB9djAzej8NTnCyu5+vBXDeDjjdgEMzbEai+xqw9wNYuINchaWRwtQk1gNi
8HLWz9cGiOekQBySyqhtUIweWgynmRFyOui2p6Wrv525RbHdCGy9wkZj3anN
TwNbC7B1ANtbga0H2GhCnJkglAvtUQ1WHyGG+26UAE14cYVtBtjSCsMOwL5Z
wNbrUdxkw66fBhIjOguwwatHP18ZYLeB4QrAtoKucZoHxFvsiOFbPSVA07xc
YZsAttMRlmrWChvZNwZ7nw1bz6OohyXROYDNA2w3wJ6VombN2aUJEPfbEcM3
HkqAJnSuNOyZANsLsLQiSi/sWwDsAwr7Zht2/aS+GNENAJsF2ArAbnWEbQZd
85vYnAzRTDilKct2xEIQ0d8UPrfrxC0p6rAH9i0CoF7F6BYADPY9+FI0W51K
dd77m4+ghjtJddomcNrnWHnmirQF2G6zs43LjkRUBKmFuvYZOW0nxCGNbtg3
Bmw/gE1ApdHCEbQ01SLgEOJGsNnviGju9doA7KAdzEwvX2mcRcAZABzS6IR9
WwHskAI7AGDroMZo3ZboaiHRmohDXACbVGOEWA4UfCIwsx7BSuPMp6ixEuzb
DmB6IbpbAWy9gLUCWB5ElxzB5sAczYSlwusQVyXAOWLHmYEGSCtQ0bJ+m1YY
jAqvA/aNQbzNhhhYGCYsmgXRzY5gM45gVGNt4qoTcI7acYKliQIrADYNYEOO
NdYG+5YB8QMK8aANMbB0Ea97uBWMD0HaekcYauLmEUd3MoTgWSoswDBngnGq
mSmAGXaEaYF9uwBLrw16yIblr9lQDa5HpkGIgyplMgWHeURYSeZ+UXaMrkJZ
q4KzHavA1TodgM2wbw9AHFcQhwEi9DspUhb/VPTj4GHE0X+TqPWC6xN210vS
WraBJSrmMbA56v3lI4iKNsbkETC5SejIJJXlKIiOxJg0z+n7wdrtdmubY61R
+dV1gzcovybI3wcm9QrFR23ll0vSFodAVE3no0V6pY83uSu/s1wH8/51Fhii
krSbDJ9ACpBvALzeqbx+wHpw1906m+UYXX/0qOvVUrBmaNUQmL0rsVm6KHC9
8KB+xXEo6hzki3F/zPZjYOmQyUt5k3Eq7z6bcf/nRh6e1Zu8+/WZpBsc13sN
6uWbELuqeP2N6Fmvgn4cPNd3lkQ684IXj1mbaypk6tfyXHttcGx1bJoFll3v
UK029eIw9TbvUTZPgE3qGs2DTepvmQdzVK5diQ3fm96wWfR7ASxRuVJ/i2cz
18Dm7WCT+tcLYJP6V+dSGB6T9kqGP5jesFmve4OjTeoVoTn1epRGvWv9EoY7
wHXDB1TaNT3cmnH0T30Eo4Fzb1gts3oQ7VL0s47W28F68Poxes4PXpP7+YcB
5kMK5k6AmQGY0PNRDWPWgp92hKE7f/NOFnUOqLv6oB+X++3uGz7K1u4bDUiJ
46D7ZFrdIMdEo3aiu4CIRlmERiIEibJpOQalPtS5g+8FqFI+nB6h0WiuOJii
DQZPiYxFR4p+Pc3dgEUDmFptWNkkMHSvZVbsp6ORDny6D1oBnEbjWuPA6BaI
FjcoJqylBxXWScBaB1ihUUpBrGwSmDzAmNeLUQOnWqI7wIfsODS6sR2kzFsO
xhxxsjYch9MpvdAkBuweAJsAsA6QoskKwykQK1Jj1CoIjLpmHk4PZt7vMGLF
qRvYTxVEB60rh37X173AMQ4cJZCiyVZUGdG02rFtlkylNp0/fRBmxUu6ITJj
k2ixDFfr1Hv2iLL+QbBO0z3KIDUk1mlERWg+nzZi1oKloiiAdeo4fTS99UYT
lP006gaihTHoMCMcqgmNcx/g0NyoLhtONgmEeW1Wp6N1qonH0ltvNJHfT6OO
IloEg2q7CXDoiY1+z+GHAIdmOxLOuNQEQVQAwrxxlOIVV8d6N0hNQE3QoD7q
VaIFLwjHtSYeVzj3Aw5NpyUc88YUgqBBTu1SEz2ONUFPzJ5IZp2k1on1LrBO
nTntUP5UKM3JID4MEDQVvQJSjdbtisNplZrodYSgp+JPJoMgKfO6LhqlTf1V
rVATVDyuEPodrw8ABK3eQBCNluvz06j7yqwl3geRW1YHgqTMS29oJgR1X9FC
Iq4QNBrhKQXxIEDQ2iYk1WhlTj+NOrDMgugDELkVIGg8z9PJIPptENmoTcLx
jdDyIRSZIKgmNMRDANHrCGFe2UOzvqjDyrxofMjROpX/x+3WGy1f5Es1WpuZ
0kIQDhptgENjp/T7rR8GnEZrZflS5s2CrhA5qYlhiNe+Otap0hutA09pIYiI
hisOjTv8DYXzEcChNeUIZ0lqgiCoEyorNTGawvqzyaxTKTV6+0E4zbaUcX3c
jmQIjwBClyOCefslGacOJf+hqJNrGjb9m3bXtEok1bF5dVNWt5JAFJVc386p
pEvgmUZEP6c8Pwqead1R8kyv2slK0oSjOSrQhOaoHDxzXgNdl6KUPqGMPAZG
aO1cMnK21CwZKa+OEWrNZwWldD6qpE6wRNM7nleWPgqWaNXnNJaolMjSJ+2W
aF3tyQaNOs66aylpS4+DJVq5nCxtdbTUBZZo+soLySxRs92SwhLNgfotZekJ
sEQr7rtaouPRtZQSWpoCqc1giU6k3Y6l9KKy9CRYordDrLQlmlL5qWSW1oPU
kqOlnmSWPgaW6P0l046WRlJYeimZpRmQ2pjCEk0V/bSy9BRYonfruFqiK7LK
6liaBakFR0u9YInmgr+sLD0NluhtT2ksUSmRpc8kszQHUvNgiW5kXEtJW/o4
WKI3kW1wtER3uX1giWazfzaZpXmQ2pDCEi2d8Iqy9AxYovflkaU5R0v9q2Np
AaRmwRJ1rLla+u031lIOLL2qLKmeAV7XbgnCz4AN6kIbOP02qE91EGzQ2i6f
UzaejVRQTX4zhF2fohRI/nfSy6eh1/LeHb9e8bo+2FQiUe5F+eLrFaInF2qB
qLoFmH5XaTwXq7EONOhBzFAMjBb6hAhRVU2mg/mS0vhkoGH4UeiwmHDECi73
7eN8WUm9IDiuAvSQcsQu8CKwLEKIcUcpYvk9JfUpYXEVoMeiZvzEV1TYTwMB
/bCNOgqMRgVeFt9pwhrfX1VhPwu+6UpqBAToeX347b/qVKIJXhECVwEaaUIC
muVVYKHL+SFHlmEprK8pgc8JgWtYIhgGAi3weSCgO8lBkKJxLCT1h0rqCyA1
lULKDEv7hhL4ohQWEQxAWOtotwiBFvgSEFCvUr+j1IAQfFsJfFkIqBumzzFs
PxBoga8AwbijlHWcZkTqO0rqq8IyATv2ggCNFeyTIvoTFfYPgIC6NFwFeoFA
S31NCKizq8dRoCIEf6bC/hEQ0F0rCdCw3AoQaKmvCwH1Z3Q7CvSAwPeVwDeB
he6aSIqG5nZLYf1ACXxLCOguugvC0ljzbiDQAt8BArqI7nQk6BSCHyqB7zoK
lB1ZOoFFS/2xFJarAI1fJ4EfKYE/BRa6jiw5SpWksH6sBL4nBBS2I0nYCIEW
+D4Q0PVcO0jRXBOS+isl9efCkkbAzJP7qQr7AyCga4k2ELBOxIsQaKm/EAJX
AZpk2SYEr6mwPwQC+oFvcSRoBQIt9SMhcBWwThONCPyNEvhLYKEfymaQopm+
LVJYf68EfiwEacI2A4EW+AkQ0K9AEaRytnye4X9SAn8NAnTmbEoiEGHRUj+V
wiKCgqNAEwj8XAn8DFjoDERSNGG+IIX1r0rgNSGgc39e7xQIq//PxaKD/i24
pgMpLrxx+h8q6N+JU/scvJhQEac66D+AU2rPWe03HB4j/0JF/kexGzfrMSuK
oW3efv+pAv0cLKrdihRSqHPgTYf8Z/EWp28WNvkvtZuq3dZoIUSV/X//t9rr
37yPfDRHdNGX2he9x7/XPmrtJfNLlfAL8fs/KsGjeW3t+9r3te//n75nzvg/
t24r1w==\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.4426757010395*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675029148875*^9, 3.442675701102*^9}]
}, Open  ]],

Cell[TextData[{
 "This action obviously has order 2, and so this action is different from the \
other elements of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".\n\nAre there any other actions that would be in the center of the group? \
Could there be an element of the center that moves an edge piece from \
location ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to a new location ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 "? Such an element would not commute with an action that reverses location \
",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " but leaves location ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " alone. So all elements of the center must fix the edge pieces.\n\nWhat if \
an element of the center reversed one of the edges at location ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", but not all of them? Say that it left the edge at location ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " fixed. This element would not commute with an action that sent the edge at \
location ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to location ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". So an element from the center of the group must either reverse all six \
edges, or none at all.\n\nThus, there are at most 2\[CenterDot]3\[CenterDot]3\
\[CenterDot]3\[CenterDot]3 = 162 elements in the center of the group, and we \
have discovered the generators which lead to a group of this size. Thus, the \
center of\nthe Pyramix\[Trademark] group is isomorphic to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "3"], "\[Times]", 
    SubscriptBox["Z", "3"], "\[Times]", 
    SubscriptBox["Z", "3"], "\[Times]", 
    RowBox[{
     SubscriptBox["Z", "3"], "."}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The center of the group is certainly a normal subgroup, and includes all \
actions that return the edges to their original positions. What if we \
considered the set of actions that return all of the ",
 StyleBox["corners",
  FontSlant->"Italic"],
 " to their original place? This certainly would be a subgroup, which we will \
call ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 ". Let us show that ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 " is a normal subgroup of the Pyramix\[Trademark] group. If ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is an element of ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is a general element, say ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " rotates the front corner ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " degrees. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "x", "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " rotates the front corner ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", " ", "+", " ", "0", " ", "+", " ", 
     RowBox[{"(", 
      RowBox[{"-", "n"}], ")"}]}], " ", "=", " ", "0", " "}], 
   TraditionalForm]]],
 "degrees. So the front corner returns to its original position. The same is \
true for the other 3 corners, and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "x", "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " fixes all 4 corners. Thus, ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 " is a normal subgroup.\n\nWhat is the intersection of ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 "? Such an element in the intersection would have to leave both the edges \
and the corners fixed, and hence would have to be the identity element. Since \
both ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are normal (since ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is in the center), by the direct product theorem, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", "\[CenterDot]", "K"}], TraditionalForm]]],
 " is isomorphic to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", "\[Times]", "K"}], TraditionalForm]]],
 ". Yet any action on the Pyramix\[Trademark] can be performed by first \
moving all of the edge pieces, and then moving all of the corners. Thus, the \
entire group is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", "\[CenterDot]", "K"}], TraditionalForm]]],
 ", and so the Pyramix\[Trademark] group is isomorphic to\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"E", " ", "\[Times]", " ", "K"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"E", " ", "\[Times]", " ", 
     SubscriptBox["Z", "3"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "3"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "3"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "3"]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Therefore, if we can find the structure of the subgroup ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 ", we will have found the structure of the entire Pyramix\[Trademark] group.\
\n\nThis breakdown of the structure of the Pyramix\[Trademark] group gives us \
a way the solve the puzzle. Suppose that we first ignore the corners of the \
puzzle and manage to get all 6 edge pieces in place. Then by using the \
routine\n\n",
 StyleBox["f.r.f.r.r.f.r.f.r.r",
  FontWeight->"Bold"],
 "\n\nwe could rotate the front corner until it was in position. We could \
then do the same for the other three corners, and the puzzle would be \
solved."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "So how do we put the edge pieces in place? This amounts to analyzing the \
subgroup ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 ". In order to ignore the corner pieces it is helpful to have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " erase the corners with the following command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["HideCorners", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{{3.442675036711375*^9, 3.442675056398875*^9}, 
   3.442675704336375*^9}]
}, Open  ]],

Cell["\<\
This looks like a much easier puzzle to solve! We can rotate the puzzle \
without the corners just as before:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotatePuzzle[f]", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.44267509944575*^9, 3.44267570825825*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "Try changing the \"f\" in the above command to one of the other possible \
rotations and re-evaluate the expression. Since there are only twelve \
triangles in the puzzle now,  it is clear that each action could be described \
as a permutation of the 12 triangles. In fact, notice that turning one corner \
120\[Degree] moves 6 triangles. If we follow carefully where the triangles \
go, we find that three of  the triangles change places with each other, and \
the other three also rotate places. Thus, each turn produces an ",
 StyleBox["even",
  FontSlant->"Italic"],
 " permutation of the 12 triangles. So ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "12"], TraditionalForm]]],
 ".\n\nLet us now try to find a normal subgroup of ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 ". We know that the center of ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 " is the identity together with the element that reverses all 6 edges. What \
if we considered the subgroup of actions which returns the edge pieces to \
their place, may reverse some of them? Let us call this subgroup ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". Is ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " a normal subgroup of ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 "? If ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be an element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " an element of ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 ", then the action ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " may move an edge piece from position ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to position ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". The action ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " will keep the edge piece at position ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", whether or not it is reversed. Then ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " will send the edge piece back to position ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". So ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "x", "\[CenterDot]", 
    SuperscriptBox["y", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " leaves the edge piece at position ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " fixed, even though it may be reversed. The same will be true for all of \
the edge pieces, and therefore ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " will be a normal subgroup of ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 ".\n\nLet us determine the structure of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". At first one might think that each edge piece may be reversed \
independently of all of the others, but this is not true. An action which \
reverses only ",
 StyleBox["one",
  FontSlant->"Italic"],
 " edge piece, leaving all other edge pieces in place, would be an ",
 StyleBox["odd",
  FontSlant->"Italic"],
 " permutation of the triangles! We already observed that ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 " was a subgroup of ",
 StyleBox["even",
  FontSlant->"Italic"],
 " permutations. So every element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " must reverse an even number of edge pieces."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Is there an element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " that reverses only 2 edges? Yes, and in fact the following command gives \
an animation of a sequence of moves producing such an element."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ResetPuzzle
RotatePuzzle[l.f.l.b.l.b.f.b.f]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675713023875*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675713102*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675713180125*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675713242625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.44267571332075*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.44267571338325*^9}],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675713461375*^9}],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426757135395*^9}],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675713602*^9}],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 3, 1}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-1, -1, 3}, {-3, -1, 1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, 1, -1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {1, 1, 3}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {3, -1, -1}, {1, -3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {-1, -3, 1}, {-1, -1, 3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, 1, 1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 3, -1}, {-1, 1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {1, -1, -3}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, -1, 1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -3, -1}, {1, -1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, 1, -3}, {-3, 1, -1}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675713680125*^9}]
}, Open  ]],

Cell[TextData[{
 "Since we can reverse the two front edge pieces, we can reverse any two edge \
pieces which are touching using a similar sequence of moves. In this way we \
can reverse any combination of edges as long as the number of edges reversed \
is even.\n\nHow many elements of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " will there be? If we had considered the edge pieces to be reversed \
independently, there would have been 2\[CenterDot]2\[CenterDot]2\[CenterDot]2\
\[CenterDot]2\[CenterDot]2 = 64 elements. Of these 64 possibilities, half of \
them reverse an even number of edges. Therefore there are 32 elements of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". Note that every element of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " besides the identity is of order 2. Also, ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is an abelian group. By the fundamental theorem of abelian groups every \
finite abelian group has a unique representation as the direct product of \
cyclic groups whose order is a power of a prime. Thus, the only such \
representation of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " would be \n             \n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{
     SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"], " ", "\[Times]", " ", 
     SubscriptBox["Z", "2"]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Now that we know the structure of the normal group ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", can we find the structure of the quotient group? First, suppose we made \
the corners disappear on our puzzle again."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["HideCorners", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675723555125*^9}]
}, Open  ]],

Cell["\<\
The quotient group can now be realized by ignoring whether the edge pieces \
are reversed or not. That is, we will concern ourselves only with the \
position of the six edge pieces. Certainly this would be a subgroup of the \
permutations of the six edges. Notice that when we rotate one of the corners,\
\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotatePuzzle[f]", "Input",
 PageWidth->Infinity,
 PageBreakAbove->True,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675729180125*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "exactly three of the edge pieces move. Thus, this is an even permutation of \
the six edge pieces. Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", "/", "H"}], TraditionalForm]]],
 " is generated by even permutations, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", "/", "H"}], TraditionalForm]]],
 " must be isomorphic to\na subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 ".\n\nUsing the following reasoning we can see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", "/", "H"}], TraditionalForm]]],
 " is isomorphic to all of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 ". We can clearly manipulate the puzzle to put any of the six edge pieces on \
the bottom. We can then, by rotating just the front and back corners, put any \
of the remaining five edge pieces on the far left. We then can put any of the \
four remaining edge pieces into the far right position as follows: If the \
piece is not already in position, rotate the front corner until it is in the \
top position. Then the sequence\n\n",
 StyleBox["f.b.f.f.b.b",
  FontWeight->"Bold"],
 "\n\nputs the top edge piece into the far right position. Finally, we rotate \
the front corner to place any of the remaining three edge pieces into the top \
position. Therefore, there are at least ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "6", "\[CenterDot]", "5", "\[CenterDot]", "4", "\[CenterDot]", "3"}], " ",
     "=", " ", "360"}], TraditionalForm]]],
 " elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", "/", "H"}], TraditionalForm]]],
 ", so \n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"E", "/", "H"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["A", "6"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "This raises a natural question. Is ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 " isomorphic to a semi-direct product of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 "? To see that it is, we need to find a copy of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " inside of ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 " which contains no elements of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " besides the identity. That is, we need to find a subgroup ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " of actions which will allow us to rearrange the six edge pieces into any \
even permutation, but which will not allow us to reverse an edge piece. \
Notice that when we were showing that we could move four of the six pieces to \
prescribed positions, we only rotated of the front and back corners, except \
in placing the bottom edge piece. That is, once the bottom piece was in \
place, the other five could be put into any even permutation using only the \
moves ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ". However, there is no way to reverse one of the edge pieces using only \
these two commands. It is not hard to find a third sequence of moves that \
changes the bottom edge without reversing any of them. The sequence r\
\[CenterDot]f\[CenterDot]f\[CenterDot]r\[CenterDot]r\[CenterDot]f gives one \
example. Try this yourself."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["RotatePuzzle[r.f.f.r.r.f]", "Input",
 PageWidth->Infinity,
 PageBreakAbove->True,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675733492625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[1, 0, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675733555125*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675733617625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.4426757336645*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 0, 1], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675733711375*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 Graphics3DBox[{
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {1, 1, 3}, {-1, -1, 3}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, 1, 1}, {-3, -1, 1}, {-3, 1, -1}}]}, 
   {RGBColor[0, 1, 0], Polygon3DBox[{{-1, 1, 1}, {-1, 3, -1}, {1, 3, 1}}]}, 
   {RGBColor[1, 0, 0], Polygon3DBox[{{1, -1, 1}, {3, 1, 1}, {3, -1, -1}}]}, 
   {RGBColor[1, 1, 0], 
    Polygon3DBox[{{1, -1, 1}, {1, -3, -1}, {-1, -3, 1}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, -1, 1}, {-1, -1, 3}, {1, 1, 3}}]}, 
   {RGBColor[0, 0, 1], Polygon3DBox[{{1, 1, -1}, {1, 3, 1}, {-1, 3, -1}}]}, 
   {RGBColor[0, 1, 0], 
    Polygon3DBox[{{1, 1, -1}, {-1, 1, -3}, {1, -1, -3}}]}, 
   {RGBColor[1, 1, 0], Polygon3DBox[{{1, 1, -1}, {3, -1, -1}, {3, 1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-3, 1, -1}, {-3, -1, 1}}]}, 
   {RGBColor[0, 0, 1], 
    Polygon3DBox[{{-1, -1, -1}, {-1, -3, 1}, {1, -3, -1}}]}, 
   {RGBColor[1, 0, 0], 
    Polygon3DBox[{{-1, -1, -1}, {1, -1, -3}, {-1, 1, -3}}]}},
  Boxed->False,
  Lighting->{{"Ambient", 
     GrayLevel[1]}},
  ViewPoint->{5, 4, 4.5}]], "Print",
 CellChangeTimes->{3.442675733773875*^9}]
}, Open  ]],

Cell[TextData[{
 "This moves a new edge piece to the bottom, yet there is no way to reverse \
an edge piece using any combination of this sequence and rotating the front \
and back corners. Thus, the group generated by\n\n",
 StyleBox["K = { f, b, r.f.f.r.r.f }",
  FontWeight->"Bold"],
 "\n\ngives us a group isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 ". Since it is impossible to reverse any edges with the elements of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", the intersection of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is the identity. Every arrangement of the edges can be obtained by first \
putting all of the edges into position, and then reversing several edges. \
Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "=", " ", 
    RowBox[{"K", "\[CenterDot]", 
     RowBox[{"H", "."}]}]}], TraditionalForm]]],
 " Therefore by the semi-direct product theorem (6.3), ",
 Cell[BoxData[
  FormBox["E", TraditionalForm]]],
 " is isomorphic to a semi-direct product of ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". If we let ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " represent the homomorphism from ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", "H", ")"}], TraditionalForm]]],
 ", we have that\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{"E", " ", "\[TildeTilde]", " ", 
    SubscriptBox["A", "6"], "\[Proportional]", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["Z", "2"], "\[Times]", 
      SubscriptBox["Z", "2"], "\[Times]", 
      SubscriptBox["Z", "2"], "\[Times]", 
      SubscriptBox["Z", "2"], "\[Times]", 
      SubscriptBox["Z", "2"]}], ")"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436970267*^9, 3.43697027846875*^9}, {3.4409476780395*^9,
   3.440947686461375*^9}}],

Cell[TextData[{
 "At first one would think that we would have to define the homomorphism ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " to complete this definition, but it turns out that there is only ",
 StyleBox["one",
  FontSlant->"Italic"],
 " non-trivial semi-direct product! Let's sketch a proof of this remarkable \
statement."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We begin by finding all non-trivial homomorphisms from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " to to the group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", " ", "=", " ", 
    RowBox[{
     RowBox[{"Aut", "(", 
      RowBox[{
       SubscriptBox["Z", "2"], "\[Times]", 
       SubscriptBox["Z", "2"], "\[Times]", 
       SubscriptBox["Z", "2"], "\[Times]", 
       SubscriptBox["Z", "2"], "\[Times]", 
       SubscriptBox["Z", "2"]}], ")"}], "."}]}], TraditionalForm]]],
 " The kernal of such a homomorphism would have to be a normal subgroup of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " is simple, and so the kernal would have to be just the identity element. \
Thus the homomorphism is an isomorphism from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", 
    RowBox[{"6", " "}]], TraditionalForm]]],
 " onto a copy of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Thus, the number of non-trivial homomorphisms depends on the number of \
copies of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " within ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ".\n\nAlthough the group ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is huge (9999360 elements), there are some shortcuts for finding all of \
the copies of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " within this group. Consider the single element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " given by ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ", where "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Florin]", "(", "A", ")"}], " ", "=", " ", "B"}], ",", 
    "                        ", "\n", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "B", ")"}], " ", "=", " ", "C"}], ",", 
    "                    ", "\n", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "C", ")"}], " ", "=", " ", "D"}], ",", 
    "                        ", "\n", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "D", ")"}], " ", "=", " ", "E"}], ",", 
    "                        ", "\n", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "E", ")"}], " ", "=", " ", "A"}], ","}], 
   TraditionalForm]]],
 "\n\nand ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"A", ",", "B", ",", "C", ",", "D", ",", "E"}], "}"}], 
   TraditionalForm]]],
 " are five generators of the group\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "2"], "\[Times]", 
    RowBox[{
     SubscriptBox["Z", "2"], "."}]}], TraditionalForm]]],
 "\n\nThe element ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is of order 5, and using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we can find that there are exactly 15 elements of the automorphism group \
",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which commute with ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ". These 15 elements form a cyclic group which is generated by the element \
",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 ", where\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"g", "(", "A", ")"}], " ", "=", " ", 
     RowBox[{"A", "\[CenterDot]", "C", "\[CenterDot]", "E"}]}], ",", 
    "                        ", "\n", 
    RowBox[{
     RowBox[{"g", "(", "B", ")"}], " ", "=", " ", 
     RowBox[{"A", "\[CenterDot]", "B", "\[CenterDot]", "D"}]}], ",", 
    "                        ", "\n", 
    RowBox[{
     RowBox[{"g", "(", "C", ")"}], " ", "=", " ", 
     RowBox[{"B", "\[CenterDot]", "C", "\[CenterDot]", "E"}]}], ",", 
    "                        ", "\n", 
    RowBox[{
     RowBox[{"g", "(", "D", ")"}], " ", "=", " ", 
     RowBox[{"A", "\[CenterDot]", "C", "\[CenterDot]", "D"}]}], ",", 
    "                        ", "\n", 
    RowBox[{
     RowBox[{"g", "(", "E", ")"}], " ", "=", " ", 
     RowBox[{"B", "\[CenterDot]", "D", "\[CenterDot]", 
      RowBox[{"E", "."}]}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Notice that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", "3"], "=", "\[Florin]"}], TraditionalForm]]],
 ", and hence ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " commutes with ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ". By lemma 7.2, the number of elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which are conjugate to ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is 9999360/15 = 666624. All of these elements would be of order 5, so \
there are at least 666624 elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of order 5. By the second Sylow theorem (7.4), all 5-sylow subgroups of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " are conjugate. Thus each 5-Sylow subgroup would contain 1, 2, or 4 \
elements conjugate to ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ". But the third Sylow theorem (7.5) eliminates the first two possibilities, \
since there cannot be 666624 or 333312 5-Sylow subgroups. Thus, all 666624 \
elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of order 5 are conjugate.\n \nFor each of these elements of order 5, let \
us determine the number of copies of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contain that element. Because the elements of order 5 all conjugate, \
we only need to consider the number of copies of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contain the element ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " is generated by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "12345", ")"}], TraditionalForm]]],
 " \nand ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", "13", ")"}], 
    RowBox[{"(", "46", ")"}]}], TraditionalForm]]],
 ", it is logical to look for elements in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which are of order 2, and which together with ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " generate a copy of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43697025484375*^9, 3.436970259265625*^9}, {
  3.436970338375*^9, 3.436970339640625*^9}}],

Cell[TextData[{
 "With ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " we can find all of the elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " that are of order 2. After several hours of computation, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " found exactly 6975 elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " of order 2. Notice that \n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", "12345", ")"}], "\[CenterDot]", 
      RowBox[{"(", "13", ")"}]}], 
     RowBox[{"(", "46", ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", "1465", ")"}], 
     RowBox[{"(", "23", ")"}]}]}], TraditionalForm]]],
 ", which is of   \n                                  order 4, and\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", "12345", ")"}], "\[CenterDot]", 
       RowBox[{"(", "12345", ")"}], "\[CenterDot]", 
       RowBox[{"(", "13", ")"}]}], 
      RowBox[{"(", "46", ")"}]}], " ", "=", " ", 
     RowBox[{"(", "15246", ")"}]}]}], TraditionalForm]]],
 ", which is\n                                  of order 5."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436970311578125*^9, 3.436970320625*^9}}],

Cell[TextData[{
 "Thus, to determine which of these elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " could correspond to the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", "13", ")"}], 
    RowBox[{"(", "46", ")"}]}], TraditionalForm]]],
 ", we need to find the elements ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " such that \n                    \n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "\[CenterDot]", "\[Mu]"}], TraditionalForm]]],
 " is of order 4, and\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "\[Florin]", "\[CenterDot]", "\[Florin]", "\[CenterDot]", "\[Mu]"}], 
   TraditionalForm]]],
 " is of order 5.\n \nBy searching though the 6975 elements, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " found exactly 90 such elements. Each of these 90 elements, together with \
",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ", generated a copy of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 ". However, each copy of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " contained 10 of the 90 elements. Thus, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " came up with 9 copies of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 "  in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " which contain the element ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ".\n\nEven though there may be many other copies of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 "  in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ", all copies must contain an element of order 5, and we alreay mentioned \
that all such elements are conjugate to ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Proposition 6.7 tells us that two semi-direct products are isomorphic if \
the images of the ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 "'s are conjugate. Thus, we may assume that the image of ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is one of the 9 copies of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " that contains the element ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ", which we will call ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". But notice that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "H", "\[CenterDot]", "g"}], 
   TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"-", "2"}]], "\[CenterDot]", "H", "\[CenterDot]", 
    SuperscriptBox["g", "2"]}], TraditionalForm]]],
 " would also be copies of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " containing the element ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ". Note that ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " cannot be the same subgroup as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "H", "\[CenterDot]", "g"}], 
   TraditionalForm]]],
 ", since this would imply that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " has an automorphism of order 15, which is not true as ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can verify. Thus, the nine copies of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " containing the element ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " appear as three collections of three groups, with the three groups in each \
collection being conjugate to one another. Therefore, by proposition 6.7 \
there are only three semi-direct products we will have to consider.\n\nWith \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", all three of these semi-direct products can be represented. (Matrices \
were used instead of the representations used in this course.) ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " could find that in all three posible semi-direct product, the orders of \
the elements were as follows:\n\n\n            1  element  of order  1,\n     \
     391  elements of order  2,\n          800  elements of order  3,\n       \
  2520  elements of order  4,\n         2304  elements of order  5,\n         \
1760  elements of order  6,\n         1440  elements of order  8,\n       + \
2304  elements of order 10,\n       -------\n        11520  elements total."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4369703279375*^9, {3.436970373390625*^9, 3.436970377*^9}}
  ],

Cell[TextData[{
 "This gives some strong evidence that the three posible semi-direct products \
are in fact isomorphic to each other. However, showing that two groups have \
the same number of elements of each order is not sufficient to prove that the \
groups are isomorphic. Yet the orders of the different elements provides a \
clue as to what the isomorphisms would be, and ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " was able to verify the isomorphisms. Therefore, all three of the posible \
nontrivial semi-direct products are isomorphic, so there is really only one \
non-trivial semi-direct product of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["Z", "2"],
     TraditionalForm], "\[Times]", 
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "2"], "\[Times]", 
    SubscriptBox["Z", "2"]}], TraditionalForm]]],
 " which we can denote\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", "6"], "\[Proportional]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["Z", "2"], "\[Times]", 
       SubscriptBox["Z", "2"], "\[Times]", 
       SubscriptBox["Z", "2"], "\[Times]", 
       SubscriptBox["Z", "2"], "\[Times]", 
       SubscriptBox["Z", "2"]}], ")"}], "."}]}], TraditionalForm]]],
 "\n\nWe then can describe the Pyraminx\[Trademark] group as being the group \
isomorphic to \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["A", "6"], "\[Proportional]", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["Z", "2"], "\[Times]", 
        SubscriptBox["Z", "2"], "\[Times]", 
        SubscriptBox["Z", "2"], "\[Times]", 
        SubscriptBox["Z", "2"], "\[Times]", 
        SubscriptBox["Z", "2"]}], ")"}]}], " ", ")"}], "\[Times]", 
    SubscriptBox["Z", "3"], "\[Times]", 
    SubscriptBox["Z", "3"], "\[Times]", 
    SubscriptBox["Z", "3"], "\[Times]", 
    RowBox[{
     SubscriptBox["Z", "3"], "."}]}], TraditionalForm]]],
 "\n\nOur knowing the structure of the group allows us the solve the puzzle! \
Here is the strategy based on this decomposition of the group.\n\n 1) First \
put all of the edge pieces in place. \n    We can begin with the bottom, then \
rotate \n    the front and back corners until the back \n    two edges are in \
the right place (they may \n    be reversed.) Finally, rotate the front  \n   \
 corner until all 6 edges are in place.\n\n 2) At this point, an even number \
of edges will\n    be reversed. We can use one or two of the \n    following \
sequences of moves to put the \n    edges into position: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43697051646875*^9, 3.43697053715625*^9}, 
   3.436971463625*^9, {3.4409478064145*^9, 3.440947825023875*^9}}],

Cell[TextData[{
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "b", "\[CenterDot]", "f", 
    "\[CenterDot]", 
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "b", "\[CenterDot]", "f", 
    "\[CenterDot]", 
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "b", "\[CenterDot]", "f"}], 
   TraditionalForm]]],
 "  flip all six edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[CenterDot]", "b", "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "l", "\[CenterDot]", "r", 
    "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 "                 flip two front edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "b", "\[CenterDot]", "l", "\[CenterDot]", "b", "\[CenterDot]", "r", 
    "\[CenterDot]", "l", "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "b"}], TraditionalForm]]],
 StyleBox["           ",
  FontWeight->"Bold"],
 " flip top & bottom edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[CenterDot]", "r", "\[CenterDot]", 
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "b", "\[CenterDot]", "l", 
    "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 StyleBox["           ",
  FontWeight->"Bold"],
 "       flip top & front left edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[CenterDot]", 
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "b", "\[CenterDot]", "l", 
    "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "f", " "}], TraditionalForm]]],
 "                 flip top & front right edges \n",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "r", "\[CenterDot]", "b", "\[CenterDot]", "r", "\[CenterDot]", "l", 
    "\[CenterDot]", "b", "\[CenterDot]", 
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "r"}], TraditionalForm]]],
 StyleBox["         ",
  FontWeight->"Bold"],
 "  flip left rear & front right edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "l", "\[CenterDot]", "r", "\[CenterDot]", "l", "\[CenterDot]", "b", 
    "\[CenterDot]", "r", "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]], "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "l"}], TraditionalForm]]],
 StyleBox["         ",
  FontWeight->"Bold"],
 "   flip right rear & left front edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[CenterDot]", "b", "\[CenterDot]", 
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "f", "\[CenterDot]", "l", 
    "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 StyleBox["          ",
  FontWeight->"Bold"],
 "       flip bottom & front right edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"l", "\[CenterDot]", "b", "\[CenterDot]", 
    SuperscriptBox["f", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "r", "\[CenterDot]", "f", 
    "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 StyleBox["         ",
  FontWeight->"Bold"],
 "       flip bottom & front left edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "r", "\[CenterDot]", 
    SuperscriptBox["f", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "l", "\[CenterDot]", "f", 
    "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 StyleBox["         ",
  FontWeight->"Bold"],
 "       flip top & left rear edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "l", "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "f", "\[CenterDot]", "r", 
    "\[CenterDot]", 
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 StyleBox["         ",
  FontWeight->"Bold"],
 "         flip top & right rear edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "f", "\[CenterDot]", 
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "r", "\[CenterDot]", "l", 
    "\[CenterDot]", 
    SuperscriptBox["f", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 StyleBox["       ",
  FontWeight->"Bold"],
 "          flip rear two edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"l", "\[CenterDot]", "f", "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "b", "\[CenterDot]", "r", 
    "\[CenterDot]", 
    SuperscriptBox["f", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 "                flip bottom & left rear edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[CenterDot]", "f", "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "l", "\[CenterDot]", "b", 
    "\[CenterDot]", 
    SuperscriptBox["f", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 StyleBox["     ",
  FontWeight->"Bold"],
 "           flip bottom & right rear edges\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"l", "\[CenterDot]", "r", "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "f", "\[CenterDot]", "b", 
    "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 StyleBox["        ",
  FontWeight->"Bold"],
 "         flip two left hand edges \n",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[CenterDot]", "l", "\[CenterDot]", 
    SuperscriptBox["f", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "b", "\[CenterDot]", "f", 
    "\[CenterDot]", 
    SuperscriptBox["l", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 StyleBox["      ",
  FontWeight->"Bold"],
 "           flip two right hand edges\n\n 3) Now only the 4 corner pieces \
are out of \n    position. We can rotate these into position \n    with the \
following sequence of moves:\n \n",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[CenterDot]", "r", "\[CenterDot]", "f", "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "f", "\[CenterDot]", "r", 
    "\[CenterDot]", "f", "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], " "}], TraditionalForm]]],
 "     rotate front corner 120\[Degree] clockwise\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"l", "\[CenterDot]", "r", "\[CenterDot]", "l", "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "l", "\[CenterDot]", "r", 
    "\[CenterDot]", "l", "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], " "}], TraditionalForm]]],
 "           rotate left corner 120\[Degree] clockwise\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[CenterDot]", "b", "\[CenterDot]", "r", "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "r", "\[CenterDot]", "b", 
    "\[CenterDot]", "r", "\[CenterDot]", 
    SuperscriptBox["b", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 "        rotate right corner 120\[Degree] clockwise\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "r", "\[CenterDot]", "b", "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], "\[CenterDot]", "b", "\[CenterDot]", "r", 
    "\[CenterDot]", "b", "\[CenterDot]", 
    SuperscriptBox["r", 
     RowBox[{"-", "1"}]], " "}], TraditionalForm]]],
 "       rotate back corner 120\[Degree] clockwise\n\n    By applying these \
four routines once or \n    twice, we can get all four corners into \n    \
position, and have solved the puzzle!\n\nNotice that our three steps can be \
expressed in terms of a subnormal series for the Pyraminx\[Trademark] group:\n\
\n     ",
 Cell[BoxData[{
  RowBox[{
   FormBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["A", "6"], "\[Proportional]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["Z", "2"], "\[Times]", 
         SubscriptBox["Z", "2"], "\[Times]", 
         SubscriptBox["Z", "2"], "\[Times]", 
         SubscriptBox["Z", "2"], "\[Times]", 
         SubscriptBox["Z", "2"]}], ")"}]}], ")"}], "\[Times]", 
     SubscriptBox["Z", "3"], "\[Times]", 
     SubscriptBox["Z", "3"], "\[Times]", 
     SubscriptBox["Z", "3"], "\[Times]", 
     SubscriptBox["Z", "3"]}],
    TraditionalForm], "  "}], "\n", 
  RowBox[{"\t           ", 
   RowBox[{"\[Superset]", "   ", 
    RowBox[{
     SubscriptBox["Z", "2"], "\[Times]", 
     SubscriptBox["Z", "2"], "\[Times]", 
     SubscriptBox["Z", "2"], "\[Times]", 
     SubscriptBox["Z", "2"], "\[Times]", 
     SubscriptBox["Z", "2"], "\[Times]", 
     SubscriptBox["Z", "3"], "\[Times]", 
     SubscriptBox["Z", "3"], "\[Times]", 
     SubscriptBox["Z", "3"], "\[Times]", 
     SubscriptBox["Z", "3"]}]}], "  "}], "\n", 
  RowBox[{"\t           ", 
   RowBox[{"\[Superset]", "   ", 
    RowBox[{
     SubscriptBox["Z", "3"], "\[Times]", 
     SubscriptBox["Z", "3"], "\[Times]", 
     SubscriptBox["Z", "3"], "\[Times]", 
     SubscriptBox["Z", "3"]}], "   ", "\[Superset]", "    ", 
    RowBox[{
     RowBox[{"{", "e", "}"}], "."}]}]}]}]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436970443359375*^9, 3.4369704903125*^9}, {
  3.43697058046875*^9, 3.436971094125*^9}, {3.436971149375*^9, 
  3.4369716579375*^9}, {3.4409478344145*^9, 3.4409478442895*^9}}],

Cell[TextData[{
 "This is not a composition series, but it is easy to see how the series can \
be expanded to make a composition series. Notice that our first step in \
solving the puzzle rids us of the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 " factor, bringing the puzzle to a combination that is in a abelian \
subgroup. The second step in solving the puzzle corresponds to bringing the \
puzzle into a combination which is within the subgroup ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "3"], "\[Times]", 
    SubscriptBox["Z", "3"], "\[Times]", 
    SubscriptBox["Z", "3"], "\[Times]", 
    RowBox[{
     SubscriptBox["Z", "3"], "."}]}], TraditionalForm]]],
 " The third step used four phases for the four corners to bring the puzzle \
to the identity element, the solved puzzle.\n\nSince one of the factors in \
this composition series is the simple group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "6"], TraditionalForm]]],
 ", the Pyraminx\[Trademark] group is not a solvable group. This is not to \
say that the puzzle is not solvable, but rather that we cannot enter this \
group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using the ",
 StyleBox["InitGroup",
  FontWeight->"Bold"],
 " and ",
 StyleBox["Define",
  FontWeight->"Bold"],
 " commands. Not that we would want to anyway, since this group has \n  \n    \
      360\[CenterDot]2\[CenterDot]2\[CenterDot]2\[CenterDot]2\[CenterDot]2\
\[CenterDot]3\[CenterDot]3\[CenterDot]3\[CenterDot]3 = 933120 \nelements.\n\n\
This same type of analysis can be used to solve other puzzles, such as the \
Rubik's cube\[Trademark]. Several problems in the homework relate to this \
puzzle. Thus, we can see a practical application of the properties of groups \
that we have studied throughout the course. \n\nBut not all applications of \
groups are fun and games. Group theory has also become the backbone of modern \
mathematics and many important proofs, such as the impossibility of finding \
solutions to fifth degree polynomials, hinge entirely on finite groups. The \
theory of finite groups also has applications in quantum physics and \
inorganic chemistry and crystallography. Therefore, the material presented in \
this course has many applications beyond mathematics. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Problems"
}], "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436971776875*^9, 3.43697178053125*^9}}],

Cell[" ", "Input",
 PageWidth->Infinity,
 FontFamily->"System",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->RGBColor[1, 0, 0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "8.1 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the derived group \n      series of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369717845*^9, 3.436971789921875*^9}, {
  3.43697205103125*^9, 3.436972055171875*^9}}],

Cell["\<\
InitGroup[e];
Define[i^4, e]; Define[j^2, i^2]
Define[j.i, i.i.i.j]
Define[i^(-1), i^3]; Define[j^(-1), i.i.j]
Q = Group[{i, j}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.436971798875*^9, 3.436971808*^9}, {3.44267576382075*^9, 
  3.442675764180125*^9}},
 AspectRatioFixed->True],

Cell["\<\
    Add any subgroups necessary to make this series a 
    composition series.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["        ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "8.2 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox["Commutator",
  FontWeight->"Bold"],
 " command as \n   an altermative way to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Aut", "(", 
    SubsuperscriptBox["Z", "24", "*"], " ", ")"}], TraditionalForm]]],
 " is \n   insoluble. Load this group with the commands"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4369718261875*^9, 3.43697186175*^9}, {
  3.4369719013125*^9, 3.4369719625625*^9}}],

Cell["\<\
InitPermMultiplication
A = Group[{149, 735}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "   and find ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["A", "\[Prime]"], TraditionalForm]]],
 ". Note that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can find the derived\n   group fairly quickly."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["        ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell["\<\
8.3 ) Find the derived group series of the following
    group.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.436971983375*^9}],

Cell["\<\
InitPermMultiplication
G = Group[{6782, 10159}]\
\>", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "   What group is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["G", "\[Prime]"], TraditionalForm]]],
 " isomorphic to? Is ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " a semi-direct\n   product of two familiar groups?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436971998515625*^9, 3.436972013890625*^9}}],

Cell["       ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "8.4 ) Use a polycyclic decompostion of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "4"], TraditionalForm]]],
 "  \n   to enter this group into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436972032953125*^9, 3.436972042890625*^9}}],

Cell["        ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 " 8.5 ) Find a polycyclic decomposition of the group \n  ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " of order 16, and use this to enter the group \n  into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.436972074078125*^9, 3.436972084625*^9}, {
  3.436972119890625*^9, 3.436972127765625*^9}}],

Cell["\<\

                The mystery group B:

   * | 1  I  J  K  L  M  N  O  P  Q  R  S  T  U  V  W
   --+-------------------------------------------------
   1 | 1  I  J  K  L  M  N  O  P  Q  R  S  T  U  V  W
   I | I  L  K  N  M  1  O  J  Q  T  S  V  U  P  W  R
   J | J  O  L  I  N  K  1  M  R  W  T  Q  V  S  P  U
   K | K  J  M  L  O  N  I  1  S  R  U  T  W  V  Q  P
   L | L  M  N  O  1  I  J  K  T  U  V  W  P  Q  R  S
   M | M  1  O  J  I  L  K  N  U  P  W  R  Q  T  S  V
   N | N  K  1  M  J  O  L  I  V  S  P  U  R  W  T  Q
   O | O  N  I  1  K  J  M  L  W  V  Q  P  S  R  U  T
   P | P  Q  R  S  T  U  V  W  L  M  N  O  1  I  J  K
   Q | Q  T  S  V  U  P  W  R  M  1  O  J  I  L  K  N
   R | R  W  T  Q  V  S  P  U  N  K  1  M  J  O  L  I
   S | S  R  U  T  W  V  Q  P  O  N  I  1  K  J  M  L
   T | T  U  V  W  P  Q  R  S  1  I  J  K  L  M  N  O
   U | U  P  W  R  Q  T  S  V  I  L  K  N  M  1  O  J
   V | V  S  P  U  R  W  T  Q  J  O  L  I  N  K  1  M
   W | W  V  Q  P  S  R  U  T  K  J  M  L  O  N  I  1
\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Courier New",
 FontSize->10],

Cell["           ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "8.6 ) Find a polycyclic decomposition of the group ",
 Cell[BoxData[
  FormBox["C", TraditionalForm]]],
 " \n   of order 16, and use this to enter the group into \n   ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43697213965625*^9, 3.436972159546875*^9}}],

Cell["\<\

                The mystery group C:

   * | 1  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T
   --+-------------------------------------------------
   1 | 1  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T
   F | F  1  H  G  J  I  L  K  N  M  P  O  R  Q  T  S
   G | G  H  1  F  K  L  I  J  O  P  M  N  S  T  Q  R
   H | H  G  F  1  L  K  J  I  P  O  N  M  T  S  R  Q
   I | I  K  J  L  M  O  N  P  Q  S  R  T  1  G  F  H
   J | J  L  I  K  N  P  M  O  R  T  Q  S  F  H  1  G
   K | K  I  L  J  O  M  P  N  S  Q  T  R  G  1  H  F
   L | L  J  K  I  P  N  O  M  T  R  S  Q  H  F  G  1
   M | M  N  O  P  Q  R  S  T  1  F  G  H  I  J  K  L
   N | N  M  P  O  R  Q  T  S  F  1  H  G  J  I  L  K
   O | O  P  M  N  S  T  Q  R  G  H  1  F  K  L  I  J
   P | P  O  N  M  T  S  R  Q  H  G  F  1  L  K  J  I
   Q | Q  S  R  T  1  G  F  H  I  K  J  L  M  O  N  P
   R | R  T  Q  S  F  H  1  G  J  L  I  K  N  P  M  O
   S | S  Q  T  R  G  1  H  F  K  I  L  J  O  M  P  N
   T | T  R  S  Q  H  F  G  1  L  J  K  I  P  N  O  M
\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Courier New",
 FontSize->10],

Cell["           ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True],

Cell[TextData[{
 "8.7 ) Find a polycyclic decomposition of the group ",
 Cell[BoxData[
  FormBox["D", TraditionalForm]]],
 " \n   of order 16, and use this to enter the group into \n   ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43697219309375*^9, 3.43697221384375*^9}}],

Cell["\<\

                The mystery group D:

   * | 1  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z
   --+-------------------------------------------------
   1 | 1  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z
   L | L  M  N  O  P  Q  R  1  T  U  V  W  X  Y  Z  S
   M | M  N  O  P  Q  R  1  L  U  V  W  X  Y  Z  S  T
   N | N  O  P  Q  R  1  L  M  V  W  X  Y  Z  S  T  U
   O | O  P  Q  R  1  L  M  N  W  X  Y  Z  S  T  U  V
   P | P  Q  R  1  L  M  N  O  X  Y  Z  S  T  U  V  W
   Q | Q  R  1  L  M  N  O  P  Y  Z  S  T  U  V  W  X
   R | R  1  L  M  N  O  P  Q  Z  S  T  U  V  W  X  Y
   S | S  Z  Y  X  W  V  U  T  O  N  M  L  1  R  Q  P
   T | T  S  Z  Y  X  W  V  U  P  O  N  M  L  1  R  Q
   U | U  T  S  Z  Y  X  W  V  Q  P  O  N  M  L  1  R
   V | V  U  T  S  Z  Y  X  W  R  Q  P  O  N  M  L  1
   W | W  V  U  T  S  Z  Y  X  1  R  Q  P  O  N  M  L
   X | X  W  V  U  T  S  Z  Y  L  1  R  Q  P  O  N  M
   Y | Y  X  W  V  U  T  S  Z  M  L  1  R  Q  P  O  N
   Z | Z  Y  X  W  V  U  T  S  N  M  L  1  R  Q  P  O
\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Courier New",
 FontSize->10],

Cell["           ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{684, 582},
WindowMargins->{{56, Automatic}, {Automatic, 7}},
DockedCells->(FrontEndExecute[{
   FrontEnd`NotebookApply[
    FrontEnd`InputNotebook[], #, Placeholder]}]& ),
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.5,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[568, 21, 322, 9, 222, "Special2",
 Evaluatable->False],
Cell[893, 32, 282, 8, 29, "Name",
 Evaluatable->False],
Cell[1178, 42, 274, 12, 183, "Title",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1477, 58, 104, 3, 32, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1606, 65, 174, 4, 43, "Input",
 InitializationCell->True],
Cell[1783, 71, 110, 1, 32, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1942, 78, 139, 4, 143, "Section",
 Evaluatable->False],
Cell[2084, 84, 3361, 91, 999, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[5470, 179, 99, 2, 43, "Input"],
Cell[5572, 183, 381, 10, 68, "Output"]
}, Open  ]],
Cell[5968, 196, 481, 15, 117, "Text",
 Evaluatable->False],
Cell[6452, 213, 991, 33, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[7468, 250, 87, 2, 43, "Input"],
Cell[7558, 254, 213, 6, 42, "Output"]
}, Open  ]],
Cell[7786, 263, 762, 21, 164, "Text",
 Evaluatable->False],
Cell[8551, 286, 1528, 45, 270, "Text",
 Evaluatable->False],
Cell[10082, 333, 819, 18, 139, "Text",
 Evaluatable->False],
Cell[10904, 353, 1136, 29, 313, "Text",
 Evaluatable->False],
Cell[12043, 384, 567, 14, 90, "Text",
 Evaluatable->False],
Cell[12613, 400, 816, 26, 164, "Text",
 Evaluatable->False],
Cell[13432, 428, 942, 26, 215, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[14399, 458, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[14487, 462, 5781, 214, 668, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[20283, 679, 998, 32, 188, "Text",
 Evaluatable->False],
Cell[21284, 713, 974, 41, 90, "Text",
 Evaluatable->False],
Cell[22261, 756, 617, 22, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[22903, 782, 88, 2, 43, "Input"],
Cell[22994, 786, 211, 6, 42, "Output"]
}, Open  ]],
Cell[23220, 795, 612, 19, 115, "Text",
 Evaluatable->False],
Cell[23835, 816, 278, 6, 115, "Text",
 Evaluatable->False],
Cell[24116, 824, 1225, 45, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[25366, 873, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[25454, 877, 4904, 165, 481, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[30373, 1045, 1458, 55, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[31856, 1104, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[31944, 1108, 6585, 229, 395, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[38544, 1340, 291, 6, 115, "Text",
 Evaluatable->False],
Cell[38838, 1348, 4194, 131, 547, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[43057, 1483, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[43145, 1487, 8787, 297, 744, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[51947, 1787, 199, 5, 66, "Text",
 Evaluatable->False],
Cell[52149, 1794, 1835, 54, 288, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[54009, 1852, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[54097, 1856, 1304, 31, 243, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[55416, 1890, 2356, 70, 489, "Text",
 Evaluatable->False],
Cell[57775, 1962, 822, 28, 188, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[58634, 1995, 93, 2, 105, "Section",
 Evaluatable->False],
Cell[58730, 1999, 2167, 71, 441, "Text",
 Evaluatable->False],
Cell[60900, 2072, 938, 32, 90, "Text",
 Evaluatable->False],
Cell[61841, 2106, 1572, 54, 139, "Text",
 Evaluatable->False],
Cell[63416, 2162, 2333, 82, 417, "Text",
 Evaluatable->False],
Cell[65752, 2246, 2460, 80, 514, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[68237, 2330, 338, 9, 88, "Input"],
Cell[68578, 2341, 305, 8, 42, "Output"],
Cell[68886, 2351, 389, 10, 42, "Output"],
Cell[69278, 2363, 1436, 40, 118, "Output"]
}, Open  ]],
Cell[70729, 2406, 1233, 46, 262, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[71987, 2456, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[72075, 2460, 6823, 230, 339, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[78913, 2693, 1907, 75, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[80845, 2772, 98, 2, 43, "Input"],
Cell[80946, 2776, 2211, 62, 218, "Output"]
}, Open  ]],
Cell[83172, 2841, 478, 18, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[83675, 2863, 98, 2, 43, "Input"],
Cell[83776, 2867, 1324, 38, 118, "Output"]
}, Open  ]],
Cell[85115, 2908, 264, 9, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[85404, 2921, 101, 2, 43, "Input"],
Cell[85508, 2925, 1323, 38, 118, "Output"]
}, Open  ]],
Cell[86846, 2966, 597, 22, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[87468, 2992, 101, 2, 43, "Input"],
Cell[87572, 2996, 1323, 38, 118, "Output"]
}, Open  ]],
Cell[88910, 3037, 2070, 69, 433, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[91005, 3110, 105, 2, 43, "Input"],
Cell[91113, 3114, 732, 22, 68, "Output"]
}, Open  ]],
Cell[91860, 3139, 168, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[92053, 3148, 79, 2, 43, "Input"],
Cell[92135, 3152, 114, 3, 42, "Output"]
}, Open  ]],
Cell[92264, 3158, 1346, 38, 262, "Text",
 Evaluatable->False],
Cell[93613, 3198, 3124, 100, 580, "Text",
 Evaluatable->False],
Cell[96740, 3300, 1333, 38, 360, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[98098, 3342, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[98186, 3346, 5868, 208, 533, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[104069, 3557, 586, 16, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[104680, 3577, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[104768, 3581, 325, 11, 55, "SmallText",
 Evaluatable->False],
Cell[105096, 3594, 496, 16, 41, "Text",
 Evaluatable->False],
Cell[105595, 3612, 886, 33, 87, "SmallText",
 Evaluatable->False],
Cell[106484, 3647, 349, 11, 44, "Text",
 Evaluatable->False],
Cell[106836, 3660, 909, 30, 144, "SmallText",
 Evaluatable->False],
Cell[107748, 3692, 614, 16, 43, "Text",
 Evaluatable->False],
Cell[108365, 3710, 1215, 25, 265, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[109595, 3738, 2550, 62, 727, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[112170, 3804, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[112258, 3808, 3708, 133, 241, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[115981, 3944, 765, 26, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[116771, 3974, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[116859, 3978, 552, 22, 55, "SmallText",
 Evaluatable->False],
Cell[117414, 4002, 691, 21, 41, "Text",
 Evaluatable->False],
Cell[118108, 4025, 1552, 57, 147, "SmallText",
 Evaluatable->False],
Cell[119663, 4084, 1430, 44, 67, "Text",
 Evaluatable->False],
Cell[121096, 4130, 2509, 100, 281, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[123620, 4233, 2442, 67, 706, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[126099, 4305, 154, 3, 105, "Section",
 Evaluatable->False],
Cell[126256, 4310, 2491, 71, 632, "Text",
 Evaluatable->False],
Cell[128750, 4383, 2836, 73, 782, "Text",
 Evaluatable->False],
Cell[131589, 4458, 11692, 412, 1629, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[143306, 4874, 230, 5, 43, "Input"],
Cell[143539, 4881, 159, 2, 42, "Output"]
}, Open  ]],
Cell[143713, 4886, 1401, 52, 166, "Text",
 Evaluatable->False],
Cell[145117, 4940, 187, 4, 43, "Input"],
Cell[145307, 4946, 1304, 48, 119, "Text",
 Evaluatable->False],
Cell[146614, 4996, 145, 3, 43, "Input"],
Cell[146762, 5001, 1154, 42, 98, "Text",
 Evaluatable->False],
Cell[147919, 5045, 144, 3, 43, "Input"],
Cell[148066, 5050, 100, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[148191, 5056, 142, 3, 43, "Input"],
Cell[148336, 5061, 1349, 26, 68, "Output"]
}, Open  ]],
Cell[149700, 5090, 887, 26, 166, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[150612, 5120, 650, 18, 152, "Input"],
Cell[151265, 5140, 1349, 26, 68, "Output"]
}, Open  ]],
Cell[152629, 5169, 367, 10, 115, "Text",
 Evaluatable->False],
Cell[152999, 5181, 105, 5, 66, "Input"],
Cell[153107, 5188, 780, 21, 237, "Text",
 Evaluatable->False],
Cell[153890, 5211, 6353, 176, 512, "Text",
 Evaluatable->False],
Cell[160246, 5389, 5709, 168, 1097, "Text",
 Evaluatable->False],
Cell[165958, 5559, 1203, 47, 117, "Text",
 Evaluatable->False],
Cell[167164, 5608, 99, 5, 66, "Input"],
Cell[167266, 5615, 1135, 42, 119, "Text",
 Evaluatable->False],
Cell[168404, 5659, 77, 2, 43, "Input"],
Cell[168484, 5663, 1021, 36, 117, "Text",
 Evaluatable->False],
Cell[169508, 5701, 83, 2, 43, "Input"],
Cell[169594, 5705, 175, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[169794, 5716, 142, 6, 66, "Input"],
Cell[169939, 5724, 4821, 94, 168, "Output"],
Cell[174763, 5820, 7435, 133, 358, "Output"]
}, Open  ]],
Cell[182213, 5956, 170, 5, 66, "Text",
 Evaluatable->False],
Cell[182386, 5963, 103, 5, 66, "Input"],
Cell[182492, 5970, 3120, 90, 656, "Text",
 Evaluatable->False],
Cell[185615, 6062, 2779, 80, 660, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[188419, 6146, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[188507, 6150, 16890, 566, 2544, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[205412, 6719, 846, 15, 335, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[206295, 6739, 105, 2, 105, "Section",
 Evaluatable->False],
Cell[206403, 6743, 438, 8, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[206866, 6755, 187, 7, 43, "Input"],
Cell[207056, 6764, 9589, 169, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[216660, 6936, 605, 19, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[217290, 6959, 194, 7, 43, "Input"],
Cell[217487, 6968, 9780, 172, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[227270, 7142, 2151, 45, 532, "Print"]
}, Open  ]],
Cell[229436, 7190, 178, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[229639, 7199, 188, 7, 43, "Input"],
Cell[229830, 7208, 9590, 169, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[239435, 7380, 1457, 23, 654, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[240917, 7407, 192, 7, 43, "Input"],
Cell[241112, 7416, 9685, 170, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[250812, 7589, 557, 11, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[251394, 7604, 230, 10, 66, "Input"],
Cell[251627, 7616, 9591, 169, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[261221, 7787, 9672, 170, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[270896, 7959, 9822, 172, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[280721, 8133, 9890, 173, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[290614, 8308, 9996, 175, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[300613, 8485, 9924, 174, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[310540, 8661, 9873, 173, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[320416, 8836, 9936, 174, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[330355, 9012, 9813, 172, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[340171, 9186, 9809, 172, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[349983, 9360, 2151, 45, 532, "Print"]
}, Open  ]],
Cell[352149, 9408, 1592, 31, 629, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[353766, 9443, 192, 7, 43, "Input"],
Cell[353961, 9452, 9893, 173, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[363869, 9628, 815, 14, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[364709, 9646, 234, 10, 66, "Input"],
Cell[364946, 9658, 9593, 169, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[374542, 9829, 9856, 173, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[384401, 10004, 9779, 172, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[394183, 10178, 9918, 174, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[404104, 10354, 9922, 174, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[414029, 10530, 9886, 173, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[423918, 10705, 9983, 175, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[433904, 10882, 9991, 175, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[443898, 11059, 9827, 172, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[453728, 11233, 9796, 172, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[463527, 11407, 9784, 172, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[473314, 11581, 9891, 173, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[483208, 11756, 2150, 45, 532, "Print"]
}, Open  ]],
Cell[485373, 11804, 1894, 50, 531, "Text",
 Evaluatable->False],
Cell[487270, 11856, 3200, 97, 560, "Text",
 Evaluatable->False],
Cell[490473, 11955, 729, 16, 335, "Text",
 Evaluatable->False],
Cell[491205, 11973, 385, 11, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[491615, 11988, 188, 7, 43, "Input"],
Cell[491806, 11997, 1194, 25, 532, "Print"]
}, Open  ]],
Cell[493015, 12025, 187, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[493227, 12034, 192, 7, 43, "Input"],
Cell[493422, 12043, 8063, 139, 532, 1102, 23, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[501500, 12185, 3557, 108, 780, "Text",
 Evaluatable->False],
Cell[505060, 12295, 301, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[505386, 12307, 228, 10, 66, "Input"],
Cell[505617, 12319, 9571, 169, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[515191, 12490, 9832, 173, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[525026, 12665, 9953, 175, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[534982, 12842, 9876, 174, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[544861, 13018, 9968, 175, 532, 2089, 44, "CachedBoxData", "BoxData", \
"Print"],
Cell[554832, 13195, 2130, 45, 532, "Print"],
Cell[556965, 13242, 2131, 45, 532, "Print"],
Cell[559099, 13289, 2129, 45, 532, "Print"],
Cell[561231, 13336, 2128, 45, 532, "Print"],
Cell[563362, 13383, 2131, 45, 532, "Print"]
}, Open  ]],
Cell[565508, 13431, 1665, 40, 458, "Text",
 Evaluatable->False],
Cell[567176, 13473, 309, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[567510, 13485, 188, 7, 43, "Input"],
Cell[567701, 13494, 1144, 24, 532, "Print"]
}, Open  ]],
Cell[568860, 13521, 383, 8, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[569268, 13533, 215, 8, 43, "Input",
 PageBreakAbove->True],
Cell[569486, 13543, 8043, 139, 532, 1102, 23, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]],
Cell[577544, 13685, 1896, 54, 556, "Text",
 Evaluatable->False],
Cell[579443, 13741, 1650, 43, 409, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[581118, 13788, 225, 8, 43, "Input",
 PageBreakAbove->True],
Cell[581346, 13798, 8083, 139, 532, 1102, 23, "CachedBoxData", "BoxData", \
"Print"],
Cell[589432, 13939, 8030, 138, 532, 1102, 23, "CachedBoxData", "BoxData", \
"Print"],
Cell[597465, 14079, 8006, 138, 532, 1102, 23, "CachedBoxData", "BoxData", \
"Print"],
Cell[605474, 14219, 7984, 138, 532, 1102, 23, "CachedBoxData", "BoxData", \
"Print"],
Cell[613461, 14359, 8030, 138, 532, 1102, 23, "CachedBoxData", "BoxData", \
"Print"],
Cell[621494, 14499, 1144, 24, 532, "Print"]
}, Open  ]],
Cell[622653, 14526, 2071, 63, 433, "Text",
 Evaluatable->False],
Cell[624727, 14591, 414, 11, 115, "Text",
 Evaluatable->False],
Cell[625144, 14604, 1863, 63, 313, "Text",
 Evaluatable->False],
Cell[627010, 14669, 2654, 79, 513, "Text",
 Evaluatable->False],
Cell[629667, 14750, 2548, 82, 460, "Text",
 Evaluatable->False],
Cell[632218, 14834, 1299, 41, 213, "Text",
 Evaluatable->False],
Cell[633520, 14877, 4986, 156, 1172, "Text",
 Evaluatable->False],
Cell[638509, 15035, 2926, 70, 821, "Text",
 Evaluatable->False],
Cell[641438, 15107, 9345, 264, 991, "Text",
 Evaluatable->False],
Cell[650786, 15373, 2377, 50, 850, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[653200, 15428, 208, 7, 105, "Section",
 Evaluatable->False],
Cell[653411, 15437, 157, 6, 43, "Input"],
Cell[653571, 15445, 359, 12, 66, "Text",
 Evaluatable->False],
Cell[653933, 15459, 305, 10, 133, "Input"],
Cell[654241, 15471, 156, 5, 66, "Text",
 Evaluatable->False],
Cell[654400, 15478, 72, 2, 43, "Input"],
Cell[654475, 15482, 537, 17, 93, "Text",
 Evaluatable->False],
Cell[655015, 15501, 116, 5, 66, "Input"],
Cell[655134, 15508, 297, 11, 66, "Text",
 Evaluatable->False],
Cell[655434, 15521, 72, 2, 43, "Input"],
Cell[655509, 15525, 180, 6, 66, "Text",
 Evaluatable->False],
Cell[655692, 15533, 119, 5, 66, "Input"],
Cell[655814, 15540, 379, 12, 66, "Text",
 Evaluatable->False],
Cell[656196, 15554, 71, 2, 43, "Input"],
Cell[656270, 15558, 355, 12, 66, "Text",
 Evaluatable->False],
Cell[656628, 15572, 72, 2, 43, "Input"],
Cell[656703, 15576, 418, 12, 90, "Text",
 Evaluatable->False],
Cell[657124, 15590, 1134, 27, 490, "Text",
 Evaluatable->False],
Cell[658261, 15619, 75, 2, 43, "Input"],
Cell[658339, 15623, 372, 11, 90, "Text",
 Evaluatable->False],
Cell[658714, 15636, 1134, 27, 490, "Text",
 Evaluatable->False],
Cell[659851, 15665, 75, 2, 43, "Input"],
Cell[659929, 15669, 371, 11, 90, "Text",
 Evaluatable->False],
Cell[660303, 15682, 1134, 27, 490, "Text",
 Evaluatable->False],
Cell[661440, 15711, 75, 2, 43, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
