(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[   1048510,      26102]
NotebookOptionsPosition[   1026278,      25379]
NotebookOutlinePosition[   1026900,      25404]
CellTagsIndexPosition[   1026857,      25401]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Abstract Algebra: An Interactive Approach", "Special2",
 Evaluatable->False,
 CellChangeTimes->{{3.43715213115625*^9, 3.43715214728125*^9}},
 TextAlignment->Center,
 FontFamily->"Arial",
 FontSize->36,
 FontWeight->"Bold",
 FontSlant->"Italic",
 FontColor->RGBColor[1, 0, 1],
 Background->RGBColor[0.500008, 1, 1]],

Cell[CellGroupData[{

Cell["\[Copyright]2009   William Paulsen", "Name",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437152152*^9, 3.43715215259375*^9}},
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]],

Cell["\<\
All rights reserved:  These materials can be duplicated or transmitted freely \
for educational use on condition that all copyright statements are retained \
unaltered.  No part of these materials may be sold, by any means or in any \
form, without written permission from the editor.  All software is provided \
\"as is\" without warranty of any kind, either expressed or implied, \
including, but not limited to, the implied warranties of merchantability and \
fitness for a particular purpose.\
\>", "Name",
 Editable->False,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]]
}, Closed]],

Cell["\<\
Chapter 9
Introduction to Rings\
\>", "Title",
 CellFrame->True,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 TextAlignment->Center,
 FontFamily->"New York",
 FontSize->24,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell["Initialization", "Special2",
 Evaluatable->False,
 TextAlignment->Center,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["<<c:\\math\\ring.m", "Input",
 PageWidth->Infinity,
 InitializationCell->True,
 CellChangeTimes->{{3.437152156234375*^9, 3.4371521584375*^9}},
 AspectRatioFixed->True],

Cell[BoxData["\<\"Initialization Done\"\>"], "Print",
 CellChangeTimes->{3.437152161859375*^9, 3.43718258740625*^9, 
  3.4371826289375*^9, 3.437182718078125*^9, 3.442747306742625*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Groups with an Additional Operation", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.437152170453125*^9}],

Cell[TextData[{
 "While studying the previous chapters on groups, we discovered different \
patterns in the group's structure by which we could project and prove many \
useful properties. However, many of the examples of groups we studied possess \
some additional structure which we have yet to take advantage of. \n\nIn the \
remaining chapters we will look at groups which have not just one, but two \
operations defined on the set of elements. Not only will we be able to \
multiply elements together as we did for groups, but we also will be able to \
",
 StyleBox["add",
  FontSlant->"Italic"],
 " elements together. \n\nThe simpliest example to consider is the group of \
integers, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". This is a group under the operation of addition, in fact an Abelian group \
with the identity element being 0. However, we can also multiply two integers \
together, always forming another integer. Is ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " a group using multiplication instead of addition? No, because most \
elements do not have an inverse. However, this extra operation gives ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " a much richer structure than standard groups. Can you find some of the \
additional properties?\n\nEXPERIMENT:\nEven though ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is not a group using multiplication, what \"grouplike\" properties does \
multiplication have over the integers? Are there any properties which show a \
relationship between multiplication and addition?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We can also consider subgroups of the group of integers, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". The simpliest subgroup is the set of ",
 StyleBox["even",
  FontSlant->"Italic"],
 " integers. This still is closed under multiplication, since the product of \
two even numbers is even. But what about the other properties of \
multiplication?\n\nEXPERIMENT:\nAre there any of the properties discovered in \
the last experiment which do ",
 StyleBox["not",
  FontSlant->"Italic"],
 " hold for the even integers?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "The next example of a group possessing both addition and multiplication is \
the group of all rational numbers of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "/", "q"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is an integer and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " is a positive integer. This group is denoted by ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 ". The sum of numbers in ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 " is again rational, since \n                                    ",
 StyleBox["  ",
  FontSize->16],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["a", "b"], " ", "+", "  ", 
     FractionBox["c", "d"]}], " ", "=", " ", 
    FractionBox[
     RowBox[{
      RowBox[{"b", "\[CenterDot]", "c"}], " ", "+", " ", 
      RowBox[{"a", "\[CenterDot]", "d"}]}], 
     RowBox[{"b", "\[CenterDot]", "d"}]]}], TraditionalForm]],
  FontSize->16],
 "\n\nIt is easy to see that ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 " forms an abelian group under addition. Is ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 " a group using multiplication instead of addition?  Almost. The \
multiplicative inverse of every rational ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "/", "b"}], TraditionalForm]]],
 " exists, namely ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "/", "a"}], TraditionalForm]]],
 ", except for when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", "0."}], TraditionalForm]]],
 " However, we can throw out that lone exception, giving us the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", " ", "-", " ", 
    RowBox[{"{", "0", "}"}]}], TraditionalForm]]],
 ". Notice that this set is still closed under multiplication, hence this \
forms a group. (Of course, this is no longer closed under addition.) So in \
this example we actually see ",
 StyleBox["two",
  FontSlant->"Italic"],
 " different groups, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 " using addition, and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", " ", "-", " ", 
    RowBox[{"{", "0", "}"}]}], TraditionalForm]]],
 ", commonly refered to as ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalQ]", "*"], TraditionalForm]]],
 ", using multiplication.\n\nAlthough the group ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 " is easy to define, it is often hard to visualize. One way to illustrate \
the rationals graphically can be seen by executing the following command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ShowRationals[-5,5]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{PointBox[{-5, 1}], PointBox[{-4, 1}], PointBox[{-3, 1}], 
   PointBox[{-2, 1}], PointBox[{-1, 1}], PointBox[{0, 1}], PointBox[{1, 1}], 
   PointBox[{2, 1}], PointBox[{3, 1}], PointBox[{4, 1}], PointBox[{5, 1}], 
   PointBox[NCache[{Rational[-9, 2], 0.75}, {-4.5, 0.75}]], 
   PointBox[NCache[{Rational[-7, 2], 0.75}, {-3.5, 0.75}]], 
   PointBox[NCache[{Rational[-5, 2], 0.75}, {-2.5, 0.75}]], 
   PointBox[NCache[{Rational[-3, 2], 0.75}, {-1.5, 0.75}]], 
   PointBox[NCache[{Rational[-1, 2], 0.75}, {-0.5, 0.75}]], 
   PointBox[NCache[{Rational[1, 2], 0.75}, {0.5, 0.75}]], 
   PointBox[NCache[{Rational[3, 2], 0.75}, {1.5, 0.75}]], 
   PointBox[NCache[{Rational[5, 2], 0.75}, {2.5, 0.75}]], 
   PointBox[NCache[{Rational[7, 2], 0.75}, {3.5, 0.75}]], 
   PointBox[NCache[{Rational[9, 2], 0.75}, {4.5, 0.75}]], 
   PointBox[NCache[{Rational[-14, 3], 0.5625}, {-4.666666666666667, 0.5625}]],
    PointBox[
    NCache[{Rational[-13, 3], 0.5625}, {-4.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[-11, 3], 0.5625}, {-3.6666666666666665`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-10, 3], 0.5625}, {-3.3333333333333335`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-8, 3], 0.5625}, {-2.6666666666666665`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-7, 3], 0.5625}, {-2.3333333333333335`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-5, 3], 0.5625}, {-1.6666666666666667`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-4, 3], 0.5625}, {-1.3333333333333333`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-2, 3], 0.5625}, {-0.6666666666666666, 0.5625}]],
    PointBox[
    NCache[{Rational[-1, 3], 0.5625}, {-0.3333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[1, 3], 0.5625}, {0.3333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[2, 3], 0.5625}, {0.6666666666666666, 0.5625}]], 
   PointBox[NCache[{Rational[4, 3], 0.5625}, {1.3333333333333333`, 0.5625}]], 
   PointBox[NCache[{Rational[5, 3], 0.5625}, {1.6666666666666667`, 0.5625}]], 
   PointBox[NCache[{Rational[7, 3], 0.5625}, {2.3333333333333335`, 0.5625}]], 
   PointBox[NCache[{Rational[8, 3], 0.5625}, {2.6666666666666665`, 0.5625}]], 
   PointBox[NCache[{Rational[10, 3], 0.5625}, {3.3333333333333335`, 0.5625}]],
    PointBox[
    NCache[{Rational[11, 3], 0.5625}, {3.6666666666666665`, 0.5625}]], 
   PointBox[NCache[{Rational[13, 3], 0.5625}, {4.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[14, 3], 0.5625}, {4.666666666666667, 0.5625}]], 
   PointBox[NCache[{Rational[-19, 4], 0.421875}, {-4.75, 0.421875}]], 
   PointBox[NCache[{Rational[-17, 4], 0.421875}, {-4.25, 0.421875}]], 
   PointBox[NCache[{Rational[-15, 4], 0.421875}, {-3.75, 0.421875}]], 
   PointBox[NCache[{Rational[-13, 4], 0.421875}, {-3.25, 0.421875}]], 
   PointBox[NCache[{Rational[-11, 4], 0.421875}, {-2.75, 0.421875}]], 
   PointBox[NCache[{Rational[-9, 4], 0.421875}, {-2.25, 0.421875}]], 
   PointBox[NCache[{Rational[-7, 4], 0.421875}, {-1.75, 0.421875}]], 
   PointBox[NCache[{Rational[-5, 4], 0.421875}, {-1.25, 0.421875}]], 
   PointBox[NCache[{Rational[-3, 4], 0.421875}, {-0.75, 0.421875}]], 
   PointBox[NCache[{Rational[-1, 4], 0.421875}, {-0.25, 0.421875}]], 
   PointBox[NCache[{Rational[1, 4], 0.421875}, {0.25, 0.421875}]], 
   PointBox[NCache[{Rational[3, 4], 0.421875}, {0.75, 0.421875}]], 
   PointBox[NCache[{Rational[5, 4], 0.421875}, {1.25, 0.421875}]], 
   PointBox[NCache[{Rational[7, 4], 0.421875}, {1.75, 0.421875}]], 
   PointBox[NCache[{Rational[9, 4], 0.421875}, {2.25, 0.421875}]], 
   PointBox[NCache[{Rational[11, 4], 0.421875}, {2.75, 0.421875}]], 
   PointBox[NCache[{Rational[13, 4], 0.421875}, {3.25, 0.421875}]], 
   PointBox[NCache[{Rational[15, 4], 0.421875}, {3.75, 0.421875}]], 
   PointBox[NCache[{Rational[17, 4], 0.421875}, {4.25, 0.421875}]], 
   PointBox[NCache[{Rational[19, 4], 0.421875}, {4.75, 0.421875}]], 
   PointBox[NCache[{Rational[-24, 5], 0.31640625}, {-4.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-23, 5], 0.31640625}, {-4.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-22, 5], 0.31640625}, {-4.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-21, 5], 0.31640625}, {-4.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-19, 5], 0.31640625}, {-3.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-18, 5], 0.31640625}, {-3.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-17, 5], 0.31640625}, {-3.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-16, 5], 0.31640625}, {-3.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-14, 5], 0.31640625}, {-2.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-13, 5], 0.31640625}, {-2.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-12, 5], 0.31640625}, {-2.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-11, 5], 0.31640625}, {-2.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-9, 5], 0.31640625}, {-1.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-8, 5], 0.31640625}, {-1.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-7, 5], 0.31640625}, {-1.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-6, 5], 0.31640625}, {-1.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-4, 5], 0.31640625}, {-0.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-3, 5], 0.31640625}, {-0.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-2, 5], 0.31640625}, {-0.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-1, 5], 0.31640625}, {-0.2, 0.31640625}]], 
   PointBox[NCache[{Rational[1, 5], 0.31640625}, {0.2, 0.31640625}]], 
   PointBox[NCache[{Rational[2, 5], 0.31640625}, {0.4, 0.31640625}]], 
   PointBox[NCache[{Rational[3, 5], 0.31640625}, {0.6, 0.31640625}]], 
   PointBox[NCache[{Rational[4, 5], 0.31640625}, {0.8, 0.31640625}]], 
   PointBox[NCache[{Rational[6, 5], 0.31640625}, {1.2, 0.31640625}]], 
   PointBox[NCache[{Rational[7, 5], 0.31640625}, {1.4, 0.31640625}]], 
   PointBox[NCache[{Rational[8, 5], 0.31640625}, {1.6, 0.31640625}]], 
   PointBox[NCache[{Rational[9, 5], 0.31640625}, {1.8, 0.31640625}]], 
   PointBox[NCache[{Rational[11, 5], 0.31640625}, {2.2, 0.31640625}]], 
   PointBox[NCache[{Rational[12, 5], 0.31640625}, {2.4, 0.31640625}]], 
   PointBox[NCache[{Rational[13, 5], 0.31640625}, {2.6, 0.31640625}]], 
   PointBox[NCache[{Rational[14, 5], 0.31640625}, {2.8, 0.31640625}]], 
   PointBox[NCache[{Rational[16, 5], 0.31640625}, {3.2, 0.31640625}]], 
   PointBox[NCache[{Rational[17, 5], 0.31640625}, {3.4, 0.31640625}]], 
   PointBox[NCache[{Rational[18, 5], 0.31640625}, {3.6, 0.31640625}]], 
   PointBox[NCache[{Rational[19, 5], 0.31640625}, {3.8, 0.31640625}]], 
   PointBox[NCache[{Rational[21, 5], 0.31640625}, {4.2, 0.31640625}]], 
   PointBox[NCache[{Rational[22, 5], 0.31640625}, {4.4, 0.31640625}]], 
   PointBox[NCache[{Rational[23, 5], 0.31640625}, {4.6, 0.31640625}]], 
   PointBox[NCache[{Rational[24, 5], 0.31640625}, {4.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-29, 6], 0.2373046875}, {-4.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-25, 6], 0.2373046875}, {-4.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-23, 6], 0.2373046875}, {-3.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-19, 6], 0.2373046875}, {-3.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-17, 6], 0.2373046875}, {-2.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-13, 6], 0.2373046875}, {-2.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-11, 6], 0.2373046875}, {-1.8333333333333333`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-7, 6], 0.2373046875}, {-1.1666666666666667`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-5, 6], 0.2373046875}, {-0.8333333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-1, 6], 0.2373046875}, {-0.16666666666666666`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[1, 6], 0.2373046875}, {0.16666666666666666`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[5, 6], 0.2373046875}, {0.8333333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[7, 6], 0.2373046875}, {1.1666666666666667`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[11, 6], 0.2373046875}, {1.8333333333333333`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[13, 6], 0.2373046875}, {2.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[17, 6], 0.2373046875}, {2.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[19, 6], 0.2373046875}, {3.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[23, 6], 0.2373046875}, {3.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[25, 6], 0.2373046875}, {4.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[29, 6], 0.2373046875}, {4.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-34, 7], 0.177978515625}, {-4.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-33, 7], 0.177978515625}, {-4.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-32, 7], 0.177978515625}, {-4.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-31, 7], 0.177978515625}, {-4.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-30, 7], 0.177978515625}, {-4.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-29, 7], 0.177978515625}, {-4.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-27, 7], 0.177978515625}, {-3.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-26, 7], 0.177978515625}, {-3.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-25, 7], 0.177978515625}, {-3.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-24, 7], 0.177978515625}, {-3.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-23, 7], 0.177978515625}, {-3.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-22, 7], 0.177978515625}, {-3.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-20, 7], 0.177978515625}, {-2.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-19, 7], 0.177978515625}, {-2.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-18, 7], 0.177978515625}, {-2.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-17, 7], 0.177978515625}, {-2.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-16, 7], 0.177978515625}, {-2.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-15, 7], 0.177978515625}, {-2.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-13, 7], 0.177978515625}, {-1.8571428571428572`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-12, 7], 0.177978515625}, {-1.7142857142857142`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-11, 7], 0.177978515625}, {-1.5714285714285714`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-10, 7], 0.177978515625}, {-1.4285714285714286`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-9, 7], 0.177978515625}, {-1.2857142857142858`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-8, 7], 0.177978515625}, {-1.1428571428571428`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-6, 7], 0.177978515625}, {-0.8571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-5, 7], 0.177978515625}, {-0.7142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-4, 7], 0.177978515625}, {-0.5714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-3, 7], 0.177978515625}, {-0.42857142857142855`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-2, 7], 0.177978515625}, {-0.2857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-1, 7], 0.177978515625}, {-0.14285714285714285`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[1, 7], 0.177978515625}, {0.14285714285714285`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[2, 7], 0.177978515625}, {0.2857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[3, 7], 0.177978515625}, {0.42857142857142855`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[4, 7], 0.177978515625}, {0.5714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[5, 7], 0.177978515625}, {0.7142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[6, 7], 0.177978515625}, {0.8571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[8, 7], 0.177978515625}, {1.1428571428571428`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[9, 7], 0.177978515625}, {1.2857142857142858`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[10, 7], 0.177978515625}, {1.4285714285714286`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[11, 7], 0.177978515625}, {1.5714285714285714`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[12, 7], 0.177978515625}, {1.7142857142857142`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[13, 7], 0.177978515625}, {1.8571428571428572`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[15, 7], 0.177978515625}, {2.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[16, 7], 0.177978515625}, {2.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[17, 7], 0.177978515625}, {2.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[18, 7], 0.177978515625}, {2.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[19, 7], 0.177978515625}, {2.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[20, 7], 0.177978515625}, {2.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[22, 7], 0.177978515625}, {3.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[23, 7], 0.177978515625}, {3.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[24, 7], 0.177978515625}, {3.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[25, 7], 0.177978515625}, {3.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[26, 7], 0.177978515625}, {3.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[27, 7], 0.177978515625}, {3.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[29, 7], 0.177978515625}, {4.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[30, 7], 0.177978515625}, {4.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[31, 7], 0.177978515625}, {4.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[32, 7], 0.177978515625}, {4.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[33, 7], 0.177978515625}, {4.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[34, 7], 0.177978515625}, {4.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-39, 8], 0.13348388671875}, {-4.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-37, 8], 0.13348388671875}, {-4.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-35, 8], 0.13348388671875}, {-4.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-33, 8], 0.13348388671875}, {-4.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-31, 8], 0.13348388671875}, {-3.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-29, 8], 0.13348388671875}, {-3.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-27, 8], 0.13348388671875}, {-3.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-25, 8], 0.13348388671875}, {-3.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-23, 8], 0.13348388671875}, {-2.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-21, 8], 0.13348388671875}, {-2.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-19, 8], 0.13348388671875}, {-2.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-17, 8], 0.13348388671875}, {-2.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-15, 8], 0.13348388671875}, {-1.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-13, 8], 0.13348388671875}, {-1.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-11, 8], 0.13348388671875}, {-1.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-9, 8], 0.13348388671875}, {-1.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-7, 8], 0.13348388671875}, {-0.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-5, 8], 0.13348388671875}, {-0.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-3, 8], 0.13348388671875}, {-0.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-1, 8], 0.13348388671875}, {-0.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[1, 8], 0.13348388671875}, {0.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[3, 8], 0.13348388671875}, {0.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[5, 8], 0.13348388671875}, {0.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[7, 8], 0.13348388671875}, {0.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[9, 8], 0.13348388671875}, {1.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[11, 8], 0.13348388671875}, {1.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[13, 8], 0.13348388671875}, {1.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[15, 8], 0.13348388671875}, {1.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[17, 8], 0.13348388671875}, {2.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[19, 8], 0.13348388671875}, {2.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[21, 8], 0.13348388671875}, {2.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[23, 8], 0.13348388671875}, {2.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[25, 8], 0.13348388671875}, {3.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[27, 8], 0.13348388671875}, {3.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[29, 8], 0.13348388671875}, {3.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[31, 8], 0.13348388671875}, {3.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[33, 8], 0.13348388671875}, {4.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[35, 8], 0.13348388671875}, {4.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[37, 8], 0.13348388671875}, {4.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[39, 8], 0.13348388671875}, {4.875, 
     0.13348388671875}]], 
   PointBox[NCache[{
     Rational[-44, 9], 0.1001129150390625}, {-4.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-43, 9], 0.1001129150390625}, {-4.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-41, 9], 0.1001129150390625}, {-4.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-40, 9], 0.1001129150390625}, {-4.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-38, 9], 0.1001129150390625}, {-4.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-37, 9], 0.1001129150390625}, {-4.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-35, 9], 0.1001129150390625}, {-3.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-34, 9], 0.1001129150390625}, {-3.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-32, 9], 0.1001129150390625}, {-3.5555555555555554`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-31, 9], 0.1001129150390625}, {-3.4444444444444446`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-29, 9], 0.1001129150390625}, {-3.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-28, 9], 0.1001129150390625}, {-3.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-26, 9], 0.1001129150390625}, {-2.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-25, 9], 0.1001129150390625}, {-2.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-23, 9], 0.1001129150390625}, {-2.5555555555555554`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-22, 9], 0.1001129150390625}, {-2.4444444444444446`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-20, 9], 0.1001129150390625}, {-2.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-19, 9], 0.1001129150390625}, {-2.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-17, 9], 0.1001129150390625}, {-1.8888888888888888`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-16, 9], 0.1001129150390625}, {-1.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-14, 9], 0.1001129150390625}, {-1.5555555555555556`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-13, 9], 0.1001129150390625}, {-1.4444444444444444`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-11, 9], 0.1001129150390625}, {-1.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-10, 9], 0.1001129150390625}, {-1.1111111111111112`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-8, 9], 0.1001129150390625}, {-0.8888888888888888, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-7, 9], 0.1001129150390625}, {-0.7777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-5, 9], 0.1001129150390625}, {-0.5555555555555556, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-4, 9], 0.1001129150390625}, {-0.4444444444444444, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-2, 9], 0.1001129150390625}, {-0.2222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-1, 9], 0.1001129150390625}, {-0.1111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[1, 9], 0.1001129150390625}, {0.1111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[2, 9], 0.1001129150390625}, {0.2222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[4, 9], 0.1001129150390625}, {0.4444444444444444, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[5, 9], 0.1001129150390625}, {0.5555555555555556, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[7, 9], 0.1001129150390625}, {0.7777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[8, 9], 0.1001129150390625}, {0.8888888888888888, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[10, 9], 0.1001129150390625}, {
     1.1111111111111112`, 0.1001129150390625}]], 
   PointBox[
    NCache[{Rational[11, 9], 0.1001129150390625}, {1.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[13, 9], 0.1001129150390625}, {
     1.4444444444444444`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[14, 9], 0.1001129150390625}, {
     1.5555555555555556`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[16, 9], 0.1001129150390625}, {
     1.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[17, 9], 0.1001129150390625}, {
     1.8888888888888888`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[19, 9], 0.1001129150390625}, {2.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[20, 9], 0.1001129150390625}, {
     2.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[22, 9], 0.1001129150390625}, {
     2.4444444444444446`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[23, 9], 0.1001129150390625}, {
     2.5555555555555554`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[25, 9], 0.1001129150390625}, {
     2.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[26, 9], 0.1001129150390625}, {2.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[28, 9], 0.1001129150390625}, {3.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[29, 9], 0.1001129150390625}, {
     3.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[31, 9], 0.1001129150390625}, {
     3.4444444444444446`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[32, 9], 0.1001129150390625}, {
     3.5555555555555554`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[34, 9], 0.1001129150390625}, {
     3.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[35, 9], 0.1001129150390625}, {3.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[37, 9], 0.1001129150390625}, {4.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[38, 9], 0.1001129150390625}, {4.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[40, 9], 0.1001129150390625}, {4.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[41, 9], 0.1001129150390625}, {4.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[43, 9], 0.1001129150390625}, {4.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[44, 9], 0.1001129150390625}, {4.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[-49, 10], 0.07508468627929688}, {-4.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-47, 10], 0.07508468627929688}, {-4.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-43, 10], 0.07508468627929688}, {-4.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-41, 10], 0.07508468627929688}, {-4.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-39, 10], 0.07508468627929688}, {-3.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-37, 10], 0.07508468627929688}, {-3.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-33, 10], 0.07508468627929688}, {-3.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-31, 10], 0.07508468627929688}, {-3.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-29, 10], 0.07508468627929688}, {-2.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-27, 10], 0.07508468627929688}, {-2.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-23, 10], 0.07508468627929688}, {-2.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-21, 10], 0.07508468627929688}, {-2.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-19, 10], 0.07508468627929688}, {-1.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-17, 10], 0.07508468627929688}, {-1.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-13, 10], 0.07508468627929688}, {-1.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-11, 10], 0.07508468627929688}, {-1.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-9, 10], 0.07508468627929688}, {-0.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-7, 10], 0.07508468627929688}, {-0.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-3, 10], 0.07508468627929688}, {-0.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-1, 10], 0.07508468627929688}, {-0.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[1, 10], 0.07508468627929688}, {0.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[3, 10], 0.07508468627929688}, {0.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[7, 10], 0.07508468627929688}, {0.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[9, 10], 0.07508468627929688}, {0.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[11, 10], 0.07508468627929688}, {1.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[13, 10], 0.07508468627929688}, {1.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[17, 10], 0.07508468627929688}, {1.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[19, 10], 0.07508468627929688}, {1.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[21, 10], 0.07508468627929688}, {2.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[23, 10], 0.07508468627929688}, {2.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[27, 10], 0.07508468627929688}, {2.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[29, 10], 0.07508468627929688}, {2.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[31, 10], 0.07508468627929688}, {3.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[33, 10], 0.07508468627929688}, {3.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[37, 10], 0.07508468627929688}, {3.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[39, 10], 0.07508468627929688}, {3.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[41, 10], 0.07508468627929688}, {4.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[43, 10], 0.07508468627929688}, {4.3, 
     0.07508468627929688}]], 
   PointBox[
    NCache[{Rational[47, 10], 0.07508468627929688}, {4.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[49, 10], 0.07508468627929688}, {4.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{
     Rational[-54, 11], 0.056313514709472656`}, {-4.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-53, 11], 0.056313514709472656`}, {-4.818181818181818, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-52, 11], 0.056313514709472656`}, {-4.7272727272727275`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-51, 11], 0.056313514709472656`}, {-4.636363636363637, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-50, 11], 0.056313514709472656`}, {-4.545454545454546, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-49, 11], 0.056313514709472656`}, {-4.454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-48, 11], 0.056313514709472656`}, {-4.363636363636363, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-47, 11], 0.056313514709472656`}, {-4.2727272727272725`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-46, 11], 0.056313514709472656`}, {-4.181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-45, 11], 0.056313514709472656`}, {-4.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-43, 11], 0.056313514709472656`}, {-3.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-42, 11], 0.056313514709472656`}, {-3.8181818181818183`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-41, 11], 0.056313514709472656`}, {-3.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-40, 11], 0.056313514709472656`}, {-3.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-39, 11], 0.056313514709472656`}, {-3.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-38, 11], 0.056313514709472656`}, {-3.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-37, 11], 0.056313514709472656`}, {-3.3636363636363638`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-36, 11], 0.056313514709472656`}, {-3.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-35, 11], 0.056313514709472656`}, {-3.1818181818181817`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-34, 11], 0.056313514709472656`}, {-3.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-32, 11], 0.056313514709472656`}, {-2.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-31, 11], 0.056313514709472656`}, {-2.8181818181818183`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-30, 11], 0.056313514709472656`}, {-2.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-29, 11], 0.056313514709472656`}, {-2.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-28, 11], 0.056313514709472656`}, {-2.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-27, 11], 0.056313514709472656`}, {-2.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-26, 11], 0.056313514709472656`}, {-2.3636363636363638`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-25, 11], 0.056313514709472656`}, {-2.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-24, 11], 0.056313514709472656`}, {-2.1818181818181817`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-23, 11], 0.056313514709472656`}, {-2.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-21, 11], 0.056313514709472656`}, {-1.9090909090909092`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-20, 11], 0.056313514709472656`}, {-1.8181818181818181`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-19, 11], 0.056313514709472656`}, {-1.7272727272727273`, 
     0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[-18, 11], 0.056313514709472656`}, {-1.6363636363636365`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-17, 11], 0.056313514709472656`}, {-1.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-16, 11], 0.056313514709472656`}, {-1.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-15, 11], 0.056313514709472656`}, {-1.3636363636363635`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-14, 11], 0.056313514709472656`}, {-1.2727272727272727`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-13, 11], 0.056313514709472656`}, {-1.1818181818181819`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-12, 11], 0.056313514709472656`}, {-1.0909090909090908`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-10, 11], 0.056313514709472656`}, {-0.9090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-9, 11], 0.056313514709472656`}, {-0.8181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-8, 11], 0.056313514709472656`}, {-0.7272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-7, 11], 0.056313514709472656`}, {-0.6363636363636364, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-6, 11], 0.056313514709472656`}, {-0.5454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-5, 11], 0.056313514709472656`}, {-0.45454545454545453`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-4, 11], 0.056313514709472656`}, {-0.36363636363636365`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-3, 11], 0.056313514709472656`}, {-0.2727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-2, 11], 0.056313514709472656`}, {-0.18181818181818182`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-1, 11], 0.056313514709472656`}, {-0.09090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[1, 11], 0.056313514709472656`}, {
     0.09090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[2, 11], 0.056313514709472656`}, {
     0.18181818181818182`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[3, 11], 0.056313514709472656`}, {
     0.2727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[4, 11], 0.056313514709472656`}, {
     0.36363636363636365`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[5, 11], 0.056313514709472656`}, {
     0.45454545454545453`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[6, 11], 0.056313514709472656`}, {
     0.5454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[7, 11], 0.056313514709472656`}, {
     0.6363636363636364, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[8, 11], 0.056313514709472656`}, {
     0.7272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[9, 11], 0.056313514709472656`}, {
     0.8181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[10, 11], 0.056313514709472656`}, {
     0.9090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[12, 11], 0.056313514709472656`}, {
     1.0909090909090908`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[13, 11], 0.056313514709472656`}, {
     1.1818181818181819`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[14, 11], 0.056313514709472656`}, {
     1.2727272727272727`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[15, 11], 0.056313514709472656`}, {
     1.3636363636363635`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[16, 11], 0.056313514709472656`}, {
     1.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[17, 11], 0.056313514709472656`}, {
     1.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[18, 11], 0.056313514709472656`}, {
     1.6363636363636365`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[19, 11], 0.056313514709472656`}, {
     1.7272727272727273`, 0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[20, 11], 0.056313514709472656`}, {1.8181818181818181`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[21, 11], 0.056313514709472656`}, {
     1.9090909090909092`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[23, 11], 0.056313514709472656`}, {
     2.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[24, 11], 0.056313514709472656`}, {
     2.1818181818181817`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[25, 11], 0.056313514709472656`}, {
     2.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[26, 11], 0.056313514709472656`}, {
     2.3636363636363638`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[27, 11], 0.056313514709472656`}, {
     2.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[28, 11], 0.056313514709472656`}, {
     2.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[29, 11], 0.056313514709472656`}, {
     2.6363636363636362`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[30, 11], 0.056313514709472656`}, {
     2.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[31, 11], 0.056313514709472656`}, {
     2.8181818181818183`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[32, 11], 0.056313514709472656`}, {
     2.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[34, 11], 0.056313514709472656`}, {
     3.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[35, 11], 0.056313514709472656`}, {
     3.1818181818181817`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[36, 11], 0.056313514709472656`}, {
     3.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[37, 11], 0.056313514709472656`}, {
     3.3636363636363638`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[38, 11], 0.056313514709472656`}, {
     3.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[39, 11], 0.056313514709472656`}, {
     3.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[40, 11], 0.056313514709472656`}, {
     3.6363636363636362`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[41, 11], 0.056313514709472656`}, {
     3.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[42, 11], 0.056313514709472656`}, {
     3.8181818181818183`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[43, 11], 0.056313514709472656`}, {
     3.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[45, 11], 0.056313514709472656`}, {
     4.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[46, 11], 0.056313514709472656`}, {
     4.181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[47, 11], 0.056313514709472656`}, {
     4.2727272727272725`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[48, 11], 0.056313514709472656`}, {
     4.363636363636363, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[49, 11], 0.056313514709472656`}, {
     4.454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[50, 11], 0.056313514709472656`}, {
     4.545454545454546, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[51, 11], 0.056313514709472656`}, {
     4.636363636363637, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[52, 11], 0.056313514709472656`}, {
     4.7272727272727275`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[53, 11], 0.056313514709472656`}, {
     4.818181818181818, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[54, 11], 0.056313514709472656`}, {
     4.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-59, 12], 0.04223513603210449}, {-4.916666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-55, 12], 0.04223513603210449}, {-4.583333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-53, 12], 0.04223513603210449}, {-4.416666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-49, 12], 0.04223513603210449}, {-4.083333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-47, 12], 0.04223513603210449}, {-3.9166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-43, 12], 0.04223513603210449}, {-3.5833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-41, 12], 0.04223513603210449}, {-3.4166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-37, 12], 0.04223513603210449}, {-3.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-35, 12], 0.04223513603210449}, {-2.9166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-31, 12], 0.04223513603210449}, {-2.5833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-29, 12], 0.04223513603210449}, {-2.4166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-25, 12], 0.04223513603210449}, {-2.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-23, 12], 0.04223513603210449}, {-1.9166666666666667`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-19, 12], 0.04223513603210449}, {-1.5833333333333333`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-17, 12], 0.04223513603210449}, {-1.4166666666666667`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-13, 12], 0.04223513603210449}, {-1.0833333333333333`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-11, 12], 0.04223513603210449}, {-0.9166666666666666, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-7, 12], 0.04223513603210449}, {-0.5833333333333334, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-5, 12], 0.04223513603210449}, {-0.4166666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-1, 12], 0.04223513603210449}, {-0.08333333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[1, 12], 0.04223513603210449}, {
     0.08333333333333333, 0.04223513603210449}]], 
   PointBox[
    NCache[{Rational[5, 12], 0.04223513603210449}, {0.4166666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[7, 12], 0.04223513603210449}, {
     0.5833333333333334, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[11, 12], 0.04223513603210449}, {
     0.9166666666666666, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[13, 12], 0.04223513603210449}, {
     1.0833333333333333`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[17, 12], 0.04223513603210449}, {
     1.4166666666666667`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[19, 12], 0.04223513603210449}, {
     1.5833333333333333`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[23, 12], 0.04223513603210449}, {
     1.9166666666666667`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[25, 12], 0.04223513603210449}, {
     2.0833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[29, 12], 0.04223513603210449}, {
     2.4166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[31, 12], 0.04223513603210449}, {
     2.5833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[35, 12], 0.04223513603210449}, {
     2.9166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[37, 12], 0.04223513603210449}, {
     3.0833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[41, 12], 0.04223513603210449}, {
     3.4166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[43, 12], 0.04223513603210449}, {
     3.5833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[47, 12], 0.04223513603210449}, {
     3.9166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[49, 12], 0.04223513603210449}, {
     4.083333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[53, 12], 0.04223513603210449}, {
     4.416666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[55, 12], 0.04223513603210449}, {
     4.583333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[59, 12], 0.04223513603210449}, {
     4.916666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-64, 13], 0.03167635202407837}, {-4.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-63, 13], 0.03167635202407837}, {-4.846153846153846, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-62, 13], 0.03167635202407837}, {-4.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-61, 13], 0.03167635202407837}, {-4.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-60, 13], 0.03167635202407837}, {-4.615384615384615, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-59, 13], 0.03167635202407837}, {-4.538461538461538, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-58, 13], 0.03167635202407837}, {-4.461538461538462, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-57, 13], 0.03167635202407837}, {-4.384615384615385, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-56, 13], 0.03167635202407837}, {-4.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-55, 13], 0.03167635202407837}, {-4.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-54, 13], 0.03167635202407837}, {-4.153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-53, 13], 0.03167635202407837}, {-4.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-51, 13], 0.03167635202407837}, {-3.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-50, 13], 0.03167635202407837}, {-3.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-49, 13], 0.03167635202407837}, {-3.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-48, 13], 0.03167635202407837}, {-3.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-47, 13], 0.03167635202407837}, {-3.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-46, 13], 0.03167635202407837}, {-3.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-45, 13], 0.03167635202407837}, {-3.4615384615384617`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-44, 13], 0.03167635202407837}, {-3.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-43, 13], 0.03167635202407837}, {-3.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-42, 13], 0.03167635202407837}, {-3.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-41, 13], 0.03167635202407837}, {-3.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-40, 13], 0.03167635202407837}, {-3.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-38, 13], 0.03167635202407837}, {-2.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-37, 13], 0.03167635202407837}, {-2.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-36, 13], 0.03167635202407837}, {-2.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-35, 13], 0.03167635202407837}, {-2.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-34, 13], 0.03167635202407837}, {-2.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-33, 13], 0.03167635202407837}, {-2.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-32, 13], 0.03167635202407837}, {-2.4615384615384617`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-31, 13], 0.03167635202407837}, {-2.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-30, 13], 0.03167635202407837}, {-2.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-29, 13], 0.03167635202407837}, {-2.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-28, 13], 0.03167635202407837}, {-2.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-27, 13], 0.03167635202407837}, {-2.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-25, 13], 0.03167635202407837}, {-1.9230769230769231`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-24, 13], 0.03167635202407837}, {-1.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-23, 13], 0.03167635202407837}, {-1.7692307692307692`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-22, 13], 0.03167635202407837}, {-1.6923076923076923`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-21, 13], 0.03167635202407837}, {-1.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-20, 13], 0.03167635202407837}, {-1.5384615384615385`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-19, 13], 0.03167635202407837}, {-1.4615384615384615`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-18, 13], 0.03167635202407837}, {-1.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-17, 13], 0.03167635202407837}, {-1.3076923076923077`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-16, 13], 0.03167635202407837}, {-1.2307692307692308`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-15, 13], 0.03167635202407837}, {-1.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-14, 13], 0.03167635202407837}, {-1.0769230769230769`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-12, 13], 0.03167635202407837}, {-0.9230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-11, 13], 0.03167635202407837}, {-0.8461538461538461, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-10, 13], 0.03167635202407837}, {-0.7692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-9, 13], 0.03167635202407837}, {-0.6923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-8, 13], 0.03167635202407837}, {-0.6153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-7, 13], 0.03167635202407837}, {-0.5384615384615384, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-6, 13], 0.03167635202407837}, {-0.46153846153846156`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-5, 13], 0.03167635202407837}, {-0.38461538461538464`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-4, 13], 0.03167635202407837}, {-0.3076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-3, 13], 0.03167635202407837}, {-0.23076923076923078`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-2, 13], 0.03167635202407837}, {-0.15384615384615385`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-1, 13], 0.03167635202407837}, {-0.07692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[1, 13], 0.03167635202407837}, {
     0.07692307692307693, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[2, 13], 0.03167635202407837}, {
     0.15384615384615385`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[3, 13], 0.03167635202407837}, {
     0.23076923076923078`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[4, 13], 0.03167635202407837}, {
     0.3076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[5, 13], 0.03167635202407837}, {
     0.38461538461538464`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[6, 13], 0.03167635202407837}, {
     0.46153846153846156`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[7, 13], 0.03167635202407837}, {
     0.5384615384615384, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[8, 13], 0.03167635202407837}, {
     0.6153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[9, 13], 0.03167635202407837}, {
     0.6923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[10, 13], 0.03167635202407837}, {
     0.7692307692307693, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[11, 13], 0.03167635202407837}, {
     0.8461538461538461, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[12, 13], 0.03167635202407837}, {
     0.9230769230769231, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[14, 13], 0.03167635202407837}, {1.0769230769230769`, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[15, 13], 0.03167635202407837}, {
     1.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[16, 13], 0.03167635202407837}, {
     1.2307692307692308`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[17, 13], 0.03167635202407837}, {
     1.3076923076923077`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[18, 13], 0.03167635202407837}, {
     1.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[19, 13], 0.03167635202407837}, {
     1.4615384615384615`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[20, 13], 0.03167635202407837}, {
     1.5384615384615385`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[21, 13], 0.03167635202407837}, {
     1.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[22, 13], 0.03167635202407837}, {
     1.6923076923076923`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[23, 13], 0.03167635202407837}, {
     1.7692307692307692`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[24, 13], 0.03167635202407837}, {
     1.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[25, 13], 0.03167635202407837}, {
     1.9230769230769231`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[27, 13], 0.03167635202407837}, {
     2.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[28, 13], 0.03167635202407837}, {
     2.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[29, 13], 0.03167635202407837}, {
     2.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[30, 13], 0.03167635202407837}, {
     2.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[31, 13], 0.03167635202407837}, {
     2.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[32, 13], 0.03167635202407837}, {
     2.4615384615384617`, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[33, 13], 0.03167635202407837}, {2.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[34, 13], 0.03167635202407837}, {
     2.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[35, 13], 0.03167635202407837}, {
     2.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[36, 13], 0.03167635202407837}, {
     2.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[37, 13], 0.03167635202407837}, {
     2.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[38, 13], 0.03167635202407837}, {
     2.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[40, 13], 0.03167635202407837}, {
     3.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[41, 13], 0.03167635202407837}, {
     3.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[42, 13], 0.03167635202407837}, {
     3.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[43, 13], 0.03167635202407837}, {
     3.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[44, 13], 0.03167635202407837}, {
     3.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[45, 13], 0.03167635202407837}, {
     3.4615384615384617`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[46, 13], 0.03167635202407837}, {
     3.5384615384615383`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[47, 13], 0.03167635202407837}, {
     3.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[48, 13], 0.03167635202407837}, {
     3.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[49, 13], 0.03167635202407837}, {
     3.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[50, 13], 0.03167635202407837}, {
     3.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[51, 13], 0.03167635202407837}, {
     3.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[53, 13], 0.03167635202407837}, {
     4.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[54, 13], 0.03167635202407837}, {
     4.153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[55, 13], 0.03167635202407837}, {
     4.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[56, 13], 0.03167635202407837}, {
     4.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[57, 13], 0.03167635202407837}, {
     4.384615384615385, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[58, 13], 0.03167635202407837}, {
     4.461538461538462, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[59, 13], 0.03167635202407837}, {
     4.538461538461538, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[60, 13], 0.03167635202407837}, {
     4.615384615384615, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[61, 13], 0.03167635202407837}, {
     4.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[62, 13], 0.03167635202407837}, {
     4.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[63, 13], 0.03167635202407837}, {
     4.846153846153846, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[64, 13], 0.03167635202407837}, {
     4.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-69, 14], 0.023757264018058777`}, {-4.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-67, 14], 0.023757264018058777`}, {-4.785714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-65, 14], 0.023757264018058777`}, {-4.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-61, 14], 0.023757264018058777`}, {-4.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-59, 14], 0.023757264018058777`}, {-4.214285714285714, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-57, 14], 0.023757264018058777`}, {-4.071428571428571, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-55, 14], 0.023757264018058777`}, {-3.9285714285714284`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-53, 14], 0.023757264018058777`}, {-3.7857142857142856`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-51, 14], 0.023757264018058777`}, {-3.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-47, 14], 0.023757264018058777`}, {-3.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-45, 14], 0.023757264018058777`}, {-3.2142857142857144`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-43, 14], 0.023757264018058777`}, {-3.0714285714285716`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-41, 14], 0.023757264018058777`}, {-2.9285714285714284`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-39, 14], 0.023757264018058777`}, {-2.7857142857142856`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-37, 14], 0.023757264018058777`}, {-2.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-33, 14], 0.023757264018058777`}, {-2.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-31, 14], 0.023757264018058777`}, {-2.2142857142857144`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-29, 14], 0.023757264018058777`}, {-2.0714285714285716`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-27, 14], 0.023757264018058777`}, {-1.9285714285714286`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-25, 14], 0.023757264018058777`}, {-1.7857142857142858`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-23, 14], 0.023757264018058777`}, {-1.6428571428571428`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-19, 14], 0.023757264018058777`}, {-1.3571428571428572`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-17, 14], 0.023757264018058777`}, {-1.2142857142857142`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-15, 14], 0.023757264018058777`}, {-1.0714285714285714`, 
     0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[-13, 14], 0.023757264018058777`}, {-0.9285714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-11, 14], 0.023757264018058777`}, {-0.7857142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-9, 14], 0.023757264018058777`}, {-0.6428571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-5, 14], 0.023757264018058777`}, {-0.35714285714285715`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-3, 14], 0.023757264018058777`}, {-0.21428571428571427`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-1, 14], 0.023757264018058777`}, {-0.07142857142857142, 
     0.023757264018058777`}]], 
   PointBox[NCache[{Rational[1, 14], 0.023757264018058777`}, {
     0.07142857142857142, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[3, 14], 0.023757264018058777`}, {
     0.21428571428571427`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[5, 14], 0.023757264018058777`}, {
     0.35714285714285715`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[9, 14], 0.023757264018058777`}, {
     0.6428571428571429, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[11, 14], 0.023757264018058777`}, {
     0.7857142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[13, 14], 0.023757264018058777`}, {
     0.9285714285714286, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[15, 14], 0.023757264018058777`}, {
     1.0714285714285714`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[17, 14], 0.023757264018058777`}, {
     1.2142857142857142`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[19, 14], 0.023757264018058777`}, {
     1.3571428571428572`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[23, 14], 0.023757264018058777`}, {
     1.6428571428571428`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[25, 14], 0.023757264018058777`}, {
     1.7857142857142858`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[27, 14], 0.023757264018058777`}, {
     1.9285714285714286`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[29, 14], 0.023757264018058777`}, {
     2.0714285714285716`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[31, 14], 0.023757264018058777`}, {
     2.2142857142857144`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[33, 14], 0.023757264018058777`}, {
     2.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[37, 14], 0.023757264018058777`}, {
     2.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[39, 14], 0.023757264018058777`}, {
     2.7857142857142856`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[41, 14], 0.023757264018058777`}, {
     2.9285714285714284`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[43, 14], 0.023757264018058777`}, {
     3.0714285714285716`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[45, 14], 0.023757264018058777`}, {
     3.2142857142857144`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[47, 14], 0.023757264018058777`}, {
     3.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[51, 14], 0.023757264018058777`}, {
     3.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[53, 14], 0.023757264018058777`}, {
     3.7857142857142856`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[55, 14], 0.023757264018058777`}, {
     3.9285714285714284`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[57, 14], 0.023757264018058777`}, {
     4.071428571428571, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[59, 14], 0.023757264018058777`}, {
     4.214285714285714, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[61, 14], 0.023757264018058777`}, {
     4.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[65, 14], 0.023757264018058777`}, {
     4.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[67, 14], 0.023757264018058777`}, {
     4.785714285714286, 0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[69, 14], 0.023757264018058777`}, {4.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-74, 15], 0.017817948013544083`}, {-4.933333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-73, 15], 0.017817948013544083`}, {-4.866666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-71, 15], 0.017817948013544083`}, {-4.733333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-68, 15], 0.017817948013544083`}, {-4.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-67, 15], 0.017817948013544083`}, {-4.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-64, 15], 0.017817948013544083`}, {-4.266666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-62, 15], 0.017817948013544083`}, {-4.133333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-61, 15], 0.017817948013544083`}, {-4.066666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-59, 15], 0.017817948013544083`}, {-3.933333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-58, 15], 0.017817948013544083`}, {-3.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-56, 15], 0.017817948013544083`}, {-3.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-53, 15], 0.017817948013544083`}, {-3.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-52, 15], 0.017817948013544083`}, {-3.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-49, 15], 0.017817948013544083`}, {-3.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-47, 15], 0.017817948013544083`}, {-3.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-46, 15], 0.017817948013544083`}, {-3.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-44, 15], 0.017817948013544083`}, {-2.933333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-43, 15], 0.017817948013544083`}, {-2.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-41, 15], 0.017817948013544083`}, {-2.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-38, 15], 0.017817948013544083`}, {-2.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-37, 15], 0.017817948013544083`}, {-2.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-34, 15], 0.017817948013544083`}, {-2.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-32, 15], 0.017817948013544083`}, {-2.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-31, 15], 0.017817948013544083`}, {-2.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-29, 15], 0.017817948013544083`}, {-1.9333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-28, 15], 0.017817948013544083`}, {-1.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-26, 15], 0.017817948013544083`}, {-1.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-23, 15], 0.017817948013544083`}, {-1.5333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-22, 15], 0.017817948013544083`}, {-1.4666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-19, 15], 0.017817948013544083`}, {-1.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-17, 15], 0.017817948013544083`}, {-1.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-16, 15], 0.017817948013544083`}, {-1.0666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-14, 15], 0.017817948013544083`}, {-0.9333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-13, 15], 0.017817948013544083`}, {-0.8666666666666667, 
     0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[-11, 15], 0.017817948013544083`}, {-0.7333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-8, 15], 0.017817948013544083`}, {-0.5333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-7, 15], 0.017817948013544083`}, {-0.4666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-4, 15], 0.017817948013544083`}, {-0.26666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-2, 15], 0.017817948013544083`}, {-0.13333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-1, 15], 0.017817948013544083`}, {-0.06666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{Rational[1, 15], 0.017817948013544083`}, {
     0.06666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[2, 15], 0.017817948013544083`}, {
     0.13333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[4, 15], 0.017817948013544083`}, {
     0.26666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[7, 15], 0.017817948013544083`}, {
     0.4666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[8, 15], 0.017817948013544083`}, {
     0.5333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[11, 15], 0.017817948013544083`}, {
     0.7333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[13, 15], 0.017817948013544083`}, {
     0.8666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[14, 15], 0.017817948013544083`}, {
     0.9333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[16, 15], 0.017817948013544083`}, {
     1.0666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[17, 15], 0.017817948013544083`}, {
     1.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[19, 15], 0.017817948013544083`}, {
     1.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[22, 15], 0.017817948013544083`}, {
     1.4666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[23, 15], 0.017817948013544083`}, {
     1.5333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[26, 15], 0.017817948013544083`}, {
     1.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[28, 15], 0.017817948013544083`}, {
     1.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[29, 15], 0.017817948013544083`}, {
     1.9333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[31, 15], 0.017817948013544083`}, {
     2.066666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[32, 15], 0.017817948013544083`}, {
     2.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[34, 15], 0.017817948013544083`}, {
     2.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[37, 15], 0.017817948013544083`}, {
     2.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[38, 15], 0.017817948013544083`}, {
     2.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[41, 15], 0.017817948013544083`}, {
     2.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[43, 15], 0.017817948013544083`}, {
     2.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[44, 15], 0.017817948013544083`}, {
     2.933333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[46, 15], 0.017817948013544083`}, {
     3.066666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[47, 15], 0.017817948013544083`}, {
     3.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[49, 15], 0.017817948013544083`}, {
     3.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[52, 15], 0.017817948013544083`}, {
     3.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[53, 15], 0.017817948013544083`}, {
     3.533333333333333, 0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[56, 15], 0.017817948013544083`}, {3.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{Rational[58, 15], 0.017817948013544083`}, {
     3.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[59, 15], 0.017817948013544083`}, {
     3.933333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[61, 15], 0.017817948013544083`}, {
     4.066666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[62, 15], 0.017817948013544083`}, {
     4.133333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[64, 15], 0.017817948013544083`}, {
     4.266666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[67, 15], 0.017817948013544083`}, {
     4.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[68, 15], 0.017817948013544083`}, {
     4.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[71, 15], 0.017817948013544083`}, {
     4.733333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[73, 15], 0.017817948013544083`}, {
     4.866666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[74, 15], 0.017817948013544083`}, {
     4.933333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[-79, 16], 0.013363461010158062`}, {-4.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-77, 16], 0.013363461010158062`}, {-4.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-75, 16], 0.013363461010158062`}, {-4.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-73, 16], 0.013363461010158062`}, {-4.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-71, 16], 0.013363461010158062`}, {-4.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-69, 16], 0.013363461010158062`}, {-4.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-67, 16], 0.013363461010158062`}, {-4.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-65, 16], 0.013363461010158062`}, {-4.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-63, 16], 0.013363461010158062`}, {-3.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-61, 16], 0.013363461010158062`}, {-3.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-59, 16], 0.013363461010158062`}, {-3.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-57, 16], 0.013363461010158062`}, {-3.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-55, 16], 0.013363461010158062`}, {-3.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-53, 16], 0.013363461010158062`}, {-3.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-51, 16], 0.013363461010158062`}, {-3.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-49, 16], 0.013363461010158062`}, {-3.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-47, 16], 0.013363461010158062`}, {-2.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-45, 16], 0.013363461010158062`}, {-2.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-43, 16], 0.013363461010158062`}, {-2.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-41, 16], 0.013363461010158062`}, {-2.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-39, 16], 0.013363461010158062`}, {-2.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-37, 16], 0.013363461010158062`}, {-2.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-35, 16], 0.013363461010158062`}, {-2.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-33, 16], 0.013363461010158062`}, {-2.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-31, 16], 0.013363461010158062`}, {-1.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-29, 16], 0.013363461010158062`}, {-1.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-27, 16], 0.013363461010158062`}, {-1.6875, 
     0.013363461010158062`}]], 
   PointBox[
    NCache[{Rational[-25, 16], 0.013363461010158062`}, {-1.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-23, 16], 0.013363461010158062`}, {-1.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-21, 16], 0.013363461010158062`}, {-1.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-19, 16], 0.013363461010158062`}, {-1.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-17, 16], 0.013363461010158062`}, {-1.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-15, 16], 0.013363461010158062`}, {-0.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-13, 16], 0.013363461010158062`}, {-0.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-11, 16], 0.013363461010158062`}, {-0.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-9, 16], 0.013363461010158062`}, {-0.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-7, 16], 0.013363461010158062`}, {-0.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-5, 16], 0.013363461010158062`}, {-0.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-3, 16], 0.013363461010158062`}, {-0.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-1, 16], 0.013363461010158062`}, {-0.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[1, 16], 0.013363461010158062`}, {0.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[3, 16], 0.013363461010158062`}, {0.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[5, 16], 0.013363461010158062`}, {0.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[7, 16], 0.013363461010158062`}, {0.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[9, 16], 0.013363461010158062`}, {0.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[11, 16], 0.013363461010158062`}, {0.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[13, 16], 0.013363461010158062`}, {0.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[15, 16], 0.013363461010158062`}, {0.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[17, 16], 0.013363461010158062`}, {1.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[19, 16], 0.013363461010158062`}, {1.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[21, 16], 0.013363461010158062`}, {1.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[23, 16], 0.013363461010158062`}, {1.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[25, 16], 0.013363461010158062`}, {1.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[27, 16], 0.013363461010158062`}, {1.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[29, 16], 0.013363461010158062`}, {1.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[31, 16], 0.013363461010158062`}, {1.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[33, 16], 0.013363461010158062`}, {2.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[35, 16], 0.013363461010158062`}, {2.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[37, 16], 0.013363461010158062`}, {2.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[39, 16], 0.013363461010158062`}, {2.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[41, 16], 0.013363461010158062`}, {2.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[43, 16], 0.013363461010158062`}, {2.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[45, 16], 0.013363461010158062`}, {2.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[47, 16], 0.013363461010158062`}, {2.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[49, 16], 0.013363461010158062`}, {3.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[51, 16], 0.013363461010158062`}, {3.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[53, 16], 0.013363461010158062`}, {3.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[55, 16], 0.013363461010158062`}, {3.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[57, 16], 0.013363461010158062`}, {3.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[59, 16], 0.013363461010158062`}, {3.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[61, 16], 0.013363461010158062`}, {3.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[63, 16], 0.013363461010158062`}, {3.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[65, 16], 0.013363461010158062`}, {4.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[67, 16], 0.013363461010158062`}, {4.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[69, 16], 0.013363461010158062`}, {4.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[71, 16], 0.013363461010158062`}, {4.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[73, 16], 0.013363461010158062`}, {4.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[75, 16], 0.013363461010158062`}, {4.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[77, 16], 0.013363461010158062`}, {4.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[79, 16], 0.013363461010158062`}, {4.9375, 
     0.013363461010158062`}]]},
  AspectRatio->NCache[
    Rational[1, 2], 0.5],
  Axes->{True, False},
  AxesOrigin->{0, 0},
  PlotRange->All,
  Prolog->{
    AbsolutePointSize[2], 
    AbsoluteThickness[1]}]], "Output",
 CellChangeTimes->{3.43715220809375*^9, 3.44274731282075*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "This plot helps to visualize the rational numbers from -5 to 5 using a \
sequence of rows. The n-th row represents the rational numbers in simpliest \
form whose denominator is equal to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Thus, the first row represents the integers, the second row the half \
integers, and so on. Each row is drawn three-fourths closer to the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-axis than the previous row. In principle there would be an infinite number \
of rows, getting closer and closer to each other as they get close to the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-axis. \n\nEven though this plot does not show all of the rational numbers \
(since there are an infinite number of them), we can zoom in on this graph to \
see more detail. Simply use different upper and lower bounds. For example, we \
can zoom in on the apparent \"gap\" close to the origin:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ShowRationals[0,0.1]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{PointBox[{0, 1}], 
   PointBox[NCache[{Rational[1, 10], 0.75}, {0.1, 0.75}]], 
   PointBox[NCache[{Rational[1, 11], 0.5625}, {0.09090909090909091, 0.5625}]],
    PointBox[
    NCache[{Rational[1, 12], 0.421875}, {0.08333333333333333, 0.421875}]], 
   PointBox[NCache[{Rational[1, 13], 0.31640625}, {0.07692307692307693, 
     0.31640625}]], 
   PointBox[NCache[{Rational[1, 14], 0.2373046875}, {0.07142857142857142, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[1, 15], 0.177978515625}, {0.06666666666666667, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[1, 16], 0.13348388671875}, {0.0625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[1, 17], 0.1001129150390625}, {
     0.058823529411764705`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[1, 18], 0.07508468627929688}, {
     0.05555555555555555, 0.07508468627929688}]], 
   PointBox[NCache[{Rational[1, 19], 0.056313514709472656`}, {
     0.05263157894736842, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[1, 20], 0.04223513603210449}, {0.05, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[1, 21], 0.03167635202407837}, {
     0.047619047619047616`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[2, 21], 0.03167635202407837}, {
     0.09523809523809523, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[1, 22], 0.023757264018058777`}, {
     0.045454545454545456`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[1, 23], 0.017817948013544083`}, {
     0.043478260869565216`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[2, 23], 0.017817948013544083`}, {
     0.08695652173913043, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[1, 24], 0.013363461010158062`}, {
     0.041666666666666664`, 0.013363461010158062`}]]},
  AspectRatio->NCache[
    Rational[1, 2], 0.5],
  Axes->{True, False},
  AxesOrigin->{0, 0},
  PlotRange->All,
  Prolog->{
    AbsolutePointSize[2], 
    AbsoluteThickness[1]}]], "Output",
 CellChangeTimes->{3.44274731607075*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztWDFv00AYvSSOkzYdEEwIJPIHYACxIXCDBFmY6BDYogqpHVCjUlC2MOEx
GwxlJlO7ZmrDgoREkWBDzDCnWxaQyZ0vyTl+/hw7dnBaR0389b53d+9977Nj
51F9b+vZ8/re9ma9/HC33tja3nxRfrCzOxzKZRjLnDDGTsuMx9YwlB/W8M/6
yj/4SPg4yw85vmRODIhQRPw9yWrjaH3GGSKbDzJDB1kjML/KAmbYuMmRTZBT
s8UclneMjeZkidw9IqeHXNMIuWYe6EoSv7BrFr3cJvdJouuob9dTt8+o2xrg
4F9NHeaGR/1/uJE6HP05WPByWBzRNSJJ5/U0P/6irmHn3W2Kn+bI+RTSd7ns
ZGEnybshVw16ATs7VqNb3LBVDHrCBGkb+Qq73Pk9l5NqcJR3iOn3cxzff0JD
dh6Pw90XRquwMFbh74nyaF0cR+iHBpStgCz9A8IULhtkjxWwcm6clQ0qhiP4
TUYQEJX6ZI0LydbsTS2VwqU5xi4uYK7soka73ebjo3fTNE1ZtHxxMBjw/3uG
YZjNZvNCv98fdXJkfBehXyP0rM6xbmnp7e/UajXV/m61Wh3ZD1tjie2HemKy
f2PGMWlDtdvtqtRqnU5nZAO0iOroOzPufZlY/7a33I92ilN+3Wq1BFWNkEFV
n5LxgVjzGjHPzrHPgGbev6KQZgesVSbwksI3/6pAKjqhrkfMk9t+91YOqeiE
kp6vCewn2K5AsET9YxeT/fJnCYvyw57+25sJFCen9QmyTggkJiGnYPMVgvMX
e9ofgp8TAvk5IZCfhPwF/E7slOtmp0TwPnLjIHmAgwoADspQcWtAy7GHliuE
lgM3DmoBOKgF4KAWFXcVaDn00BI0vk5o33fjoHaAg9oBDmpXcTeA9vcRafeK
K0RN3rhxsCYAB2sCcLAmKu4+qIkZc03miZ8Q9dxx42A9AQ7WE+BgPVXcU1DP
RgLqFkf8kvDisRsHvQA46AXAQS9U3CvgxUYC6rYs8VvC35tuHPQX4KC/AAf9
VXHvgL+3ElC3NKbjY6KvVt044kHagYN9BXCwr1RcD/RVKQF1S+OljKnHG4E7
cuBguwMcbHeAg+2u4tDjjXK+xFITlvkH1TW6WA==\
\>"]]
}, Open  ]],

Cell[TextData[{
 "This picture seems to indicate that there are not any rationals between 0 \
and about 0.04. Is this true?\n\nEXPERIMENT:\nChange the 0 and 0.1 in the \
above command to zoom in even closer on this gap. Try to find an interval in \
which there are no points plotted in the graph. Can you find an interval \
without any rational numbers?\n\nIn doing this experiment you not only might \
make a discovery about rational numbers, but you might even be able to see ",
 StyleBox["why",
  FontSlant->"Italic"],
 " this property is true. \n\nPROPOSITION 9.1\nIf ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are any two different real numbers, then there is a rational number \
between ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ".  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43715222015625*^9, 3.437152220296875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{"|", 
     RowBox[{"a", " ", "-", " ", "b"}], "|", "."}]}], TraditionalForm]]],
 " Since ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is not zero, we let ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " be any number which is greater than ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", "x"}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", 
     RowBox[{
      RowBox[{"a", "\[CenterDot]", "q"}], " ", "-", " ", 
      RowBox[{"b", "\[CenterDot]", "q"}]}], "|"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"q", "\[CenterDot]", "x"}], " ", ">", " ", "1"}]}], 
   TraditionalForm]]],
 ", so there must be an integer between ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "q"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "q"}], TraditionalForm]]],
 ", which we will call ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". But then ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"p", "/", "q"}]}], TraditionalForm]]],
 " will be between ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and the proposition is proved."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "It is not hard to extend this proposition to show that between any two \
numbers, there are in fact an infinite number of rational numbers. \n\nThere \
are some other properties of the rational numbers that are suggested by the \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " plot. Here is the plot again, from -5 to 5:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["ShowRationals[-5,5]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{PointBox[{-5, 1}], PointBox[{-4, 1}], PointBox[{-3, 1}], 
   PointBox[{-2, 1}], PointBox[{-1, 1}], PointBox[{0, 1}], PointBox[{1, 1}], 
   PointBox[{2, 1}], PointBox[{3, 1}], PointBox[{4, 1}], PointBox[{5, 1}], 
   PointBox[NCache[{Rational[-9, 2], 0.75}, {-4.5, 0.75}]], 
   PointBox[NCache[{Rational[-7, 2], 0.75}, {-3.5, 0.75}]], 
   PointBox[NCache[{Rational[-5, 2], 0.75}, {-2.5, 0.75}]], 
   PointBox[NCache[{Rational[-3, 2], 0.75}, {-1.5, 0.75}]], 
   PointBox[NCache[{Rational[-1, 2], 0.75}, {-0.5, 0.75}]], 
   PointBox[NCache[{Rational[1, 2], 0.75}, {0.5, 0.75}]], 
   PointBox[NCache[{Rational[3, 2], 0.75}, {1.5, 0.75}]], 
   PointBox[NCache[{Rational[5, 2], 0.75}, {2.5, 0.75}]], 
   PointBox[NCache[{Rational[7, 2], 0.75}, {3.5, 0.75}]], 
   PointBox[NCache[{Rational[9, 2], 0.75}, {4.5, 0.75}]], 
   PointBox[NCache[{Rational[-14, 3], 0.5625}, {-4.666666666666667, 0.5625}]],
    PointBox[
    NCache[{Rational[-13, 3], 0.5625}, {-4.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[-11, 3], 0.5625}, {-3.6666666666666665`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-10, 3], 0.5625}, {-3.3333333333333335`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-8, 3], 0.5625}, {-2.6666666666666665`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-7, 3], 0.5625}, {-2.3333333333333335`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-5, 3], 0.5625}, {-1.6666666666666667`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-4, 3], 0.5625}, {-1.3333333333333333`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-2, 3], 0.5625}, {-0.6666666666666666, 0.5625}]],
    PointBox[
    NCache[{Rational[-1, 3], 0.5625}, {-0.3333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[1, 3], 0.5625}, {0.3333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[2, 3], 0.5625}, {0.6666666666666666, 0.5625}]], 
   PointBox[NCache[{Rational[4, 3], 0.5625}, {1.3333333333333333`, 0.5625}]], 
   PointBox[NCache[{Rational[5, 3], 0.5625}, {1.6666666666666667`, 0.5625}]], 
   PointBox[NCache[{Rational[7, 3], 0.5625}, {2.3333333333333335`, 0.5625}]], 
   PointBox[NCache[{Rational[8, 3], 0.5625}, {2.6666666666666665`, 0.5625}]], 
   PointBox[NCache[{Rational[10, 3], 0.5625}, {3.3333333333333335`, 0.5625}]],
    PointBox[
    NCache[{Rational[11, 3], 0.5625}, {3.6666666666666665`, 0.5625}]], 
   PointBox[NCache[{Rational[13, 3], 0.5625}, {4.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[14, 3], 0.5625}, {4.666666666666667, 0.5625}]], 
   PointBox[NCache[{Rational[-19, 4], 0.421875}, {-4.75, 0.421875}]], 
   PointBox[NCache[{Rational[-17, 4], 0.421875}, {-4.25, 0.421875}]], 
   PointBox[NCache[{Rational[-15, 4], 0.421875}, {-3.75, 0.421875}]], 
   PointBox[NCache[{Rational[-13, 4], 0.421875}, {-3.25, 0.421875}]], 
   PointBox[NCache[{Rational[-11, 4], 0.421875}, {-2.75, 0.421875}]], 
   PointBox[NCache[{Rational[-9, 4], 0.421875}, {-2.25, 0.421875}]], 
   PointBox[NCache[{Rational[-7, 4], 0.421875}, {-1.75, 0.421875}]], 
   PointBox[NCache[{Rational[-5, 4], 0.421875}, {-1.25, 0.421875}]], 
   PointBox[NCache[{Rational[-3, 4], 0.421875}, {-0.75, 0.421875}]], 
   PointBox[NCache[{Rational[-1, 4], 0.421875}, {-0.25, 0.421875}]], 
   PointBox[NCache[{Rational[1, 4], 0.421875}, {0.25, 0.421875}]], 
   PointBox[NCache[{Rational[3, 4], 0.421875}, {0.75, 0.421875}]], 
   PointBox[NCache[{Rational[5, 4], 0.421875}, {1.25, 0.421875}]], 
   PointBox[NCache[{Rational[7, 4], 0.421875}, {1.75, 0.421875}]], 
   PointBox[NCache[{Rational[9, 4], 0.421875}, {2.25, 0.421875}]], 
   PointBox[NCache[{Rational[11, 4], 0.421875}, {2.75, 0.421875}]], 
   PointBox[NCache[{Rational[13, 4], 0.421875}, {3.25, 0.421875}]], 
   PointBox[NCache[{Rational[15, 4], 0.421875}, {3.75, 0.421875}]], 
   PointBox[NCache[{Rational[17, 4], 0.421875}, {4.25, 0.421875}]], 
   PointBox[NCache[{Rational[19, 4], 0.421875}, {4.75, 0.421875}]], 
   PointBox[NCache[{Rational[-24, 5], 0.31640625}, {-4.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-23, 5], 0.31640625}, {-4.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-22, 5], 0.31640625}, {-4.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-21, 5], 0.31640625}, {-4.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-19, 5], 0.31640625}, {-3.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-18, 5], 0.31640625}, {-3.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-17, 5], 0.31640625}, {-3.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-16, 5], 0.31640625}, {-3.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-14, 5], 0.31640625}, {-2.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-13, 5], 0.31640625}, {-2.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-12, 5], 0.31640625}, {-2.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-11, 5], 0.31640625}, {-2.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-9, 5], 0.31640625}, {-1.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-8, 5], 0.31640625}, {-1.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-7, 5], 0.31640625}, {-1.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-6, 5], 0.31640625}, {-1.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-4, 5], 0.31640625}, {-0.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-3, 5], 0.31640625}, {-0.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-2, 5], 0.31640625}, {-0.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-1, 5], 0.31640625}, {-0.2, 0.31640625}]], 
   PointBox[NCache[{Rational[1, 5], 0.31640625}, {0.2, 0.31640625}]], 
   PointBox[NCache[{Rational[2, 5], 0.31640625}, {0.4, 0.31640625}]], 
   PointBox[NCache[{Rational[3, 5], 0.31640625}, {0.6, 0.31640625}]], 
   PointBox[NCache[{Rational[4, 5], 0.31640625}, {0.8, 0.31640625}]], 
   PointBox[NCache[{Rational[6, 5], 0.31640625}, {1.2, 0.31640625}]], 
   PointBox[NCache[{Rational[7, 5], 0.31640625}, {1.4, 0.31640625}]], 
   PointBox[NCache[{Rational[8, 5], 0.31640625}, {1.6, 0.31640625}]], 
   PointBox[NCache[{Rational[9, 5], 0.31640625}, {1.8, 0.31640625}]], 
   PointBox[NCache[{Rational[11, 5], 0.31640625}, {2.2, 0.31640625}]], 
   PointBox[NCache[{Rational[12, 5], 0.31640625}, {2.4, 0.31640625}]], 
   PointBox[NCache[{Rational[13, 5], 0.31640625}, {2.6, 0.31640625}]], 
   PointBox[NCache[{Rational[14, 5], 0.31640625}, {2.8, 0.31640625}]], 
   PointBox[NCache[{Rational[16, 5], 0.31640625}, {3.2, 0.31640625}]], 
   PointBox[NCache[{Rational[17, 5], 0.31640625}, {3.4, 0.31640625}]], 
   PointBox[NCache[{Rational[18, 5], 0.31640625}, {3.6, 0.31640625}]], 
   PointBox[NCache[{Rational[19, 5], 0.31640625}, {3.8, 0.31640625}]], 
   PointBox[NCache[{Rational[21, 5], 0.31640625}, {4.2, 0.31640625}]], 
   PointBox[NCache[{Rational[22, 5], 0.31640625}, {4.4, 0.31640625}]], 
   PointBox[NCache[{Rational[23, 5], 0.31640625}, {4.6, 0.31640625}]], 
   PointBox[NCache[{Rational[24, 5], 0.31640625}, {4.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-29, 6], 0.2373046875}, {-4.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-25, 6], 0.2373046875}, {-4.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-23, 6], 0.2373046875}, {-3.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-19, 6], 0.2373046875}, {-3.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-17, 6], 0.2373046875}, {-2.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-13, 6], 0.2373046875}, {-2.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-11, 6], 0.2373046875}, {-1.8333333333333333`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-7, 6], 0.2373046875}, {-1.1666666666666667`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-5, 6], 0.2373046875}, {-0.8333333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-1, 6], 0.2373046875}, {-0.16666666666666666`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[1, 6], 0.2373046875}, {0.16666666666666666`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[5, 6], 0.2373046875}, {0.8333333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[7, 6], 0.2373046875}, {1.1666666666666667`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[11, 6], 0.2373046875}, {1.8333333333333333`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[13, 6], 0.2373046875}, {2.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[17, 6], 0.2373046875}, {2.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[19, 6], 0.2373046875}, {3.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[23, 6], 0.2373046875}, {3.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[25, 6], 0.2373046875}, {4.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[29, 6], 0.2373046875}, {4.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-34, 7], 0.177978515625}, {-4.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-33, 7], 0.177978515625}, {-4.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-32, 7], 0.177978515625}, {-4.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-31, 7], 0.177978515625}, {-4.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-30, 7], 0.177978515625}, {-4.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-29, 7], 0.177978515625}, {-4.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-27, 7], 0.177978515625}, {-3.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-26, 7], 0.177978515625}, {-3.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-25, 7], 0.177978515625}, {-3.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-24, 7], 0.177978515625}, {-3.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-23, 7], 0.177978515625}, {-3.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-22, 7], 0.177978515625}, {-3.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-20, 7], 0.177978515625}, {-2.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-19, 7], 0.177978515625}, {-2.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-18, 7], 0.177978515625}, {-2.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-17, 7], 0.177978515625}, {-2.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-16, 7], 0.177978515625}, {-2.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-15, 7], 0.177978515625}, {-2.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-13, 7], 0.177978515625}, {-1.8571428571428572`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-12, 7], 0.177978515625}, {-1.7142857142857142`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-11, 7], 0.177978515625}, {-1.5714285714285714`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-10, 7], 0.177978515625}, {-1.4285714285714286`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-9, 7], 0.177978515625}, {-1.2857142857142858`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-8, 7], 0.177978515625}, {-1.1428571428571428`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-6, 7], 0.177978515625}, {-0.8571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-5, 7], 0.177978515625}, {-0.7142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-4, 7], 0.177978515625}, {-0.5714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-3, 7], 0.177978515625}, {-0.42857142857142855`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-2, 7], 0.177978515625}, {-0.2857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-1, 7], 0.177978515625}, {-0.14285714285714285`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[1, 7], 0.177978515625}, {0.14285714285714285`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[2, 7], 0.177978515625}, {0.2857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[3, 7], 0.177978515625}, {0.42857142857142855`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[4, 7], 0.177978515625}, {0.5714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[5, 7], 0.177978515625}, {0.7142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[6, 7], 0.177978515625}, {0.8571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[8, 7], 0.177978515625}, {1.1428571428571428`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[9, 7], 0.177978515625}, {1.2857142857142858`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[10, 7], 0.177978515625}, {1.4285714285714286`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[11, 7], 0.177978515625}, {1.5714285714285714`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[12, 7], 0.177978515625}, {1.7142857142857142`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[13, 7], 0.177978515625}, {1.8571428571428572`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[15, 7], 0.177978515625}, {2.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[16, 7], 0.177978515625}, {2.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[17, 7], 0.177978515625}, {2.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[18, 7], 0.177978515625}, {2.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[19, 7], 0.177978515625}, {2.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[20, 7], 0.177978515625}, {2.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[22, 7], 0.177978515625}, {3.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[23, 7], 0.177978515625}, {3.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[24, 7], 0.177978515625}, {3.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[25, 7], 0.177978515625}, {3.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[26, 7], 0.177978515625}, {3.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[27, 7], 0.177978515625}, {3.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[29, 7], 0.177978515625}, {4.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[30, 7], 0.177978515625}, {4.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[31, 7], 0.177978515625}, {4.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[32, 7], 0.177978515625}, {4.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[33, 7], 0.177978515625}, {4.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[34, 7], 0.177978515625}, {4.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-39, 8], 0.13348388671875}, {-4.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-37, 8], 0.13348388671875}, {-4.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-35, 8], 0.13348388671875}, {-4.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-33, 8], 0.13348388671875}, {-4.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-31, 8], 0.13348388671875}, {-3.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-29, 8], 0.13348388671875}, {-3.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-27, 8], 0.13348388671875}, {-3.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-25, 8], 0.13348388671875}, {-3.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-23, 8], 0.13348388671875}, {-2.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-21, 8], 0.13348388671875}, {-2.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-19, 8], 0.13348388671875}, {-2.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-17, 8], 0.13348388671875}, {-2.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-15, 8], 0.13348388671875}, {-1.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-13, 8], 0.13348388671875}, {-1.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-11, 8], 0.13348388671875}, {-1.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-9, 8], 0.13348388671875}, {-1.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-7, 8], 0.13348388671875}, {-0.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-5, 8], 0.13348388671875}, {-0.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-3, 8], 0.13348388671875}, {-0.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-1, 8], 0.13348388671875}, {-0.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[1, 8], 0.13348388671875}, {0.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[3, 8], 0.13348388671875}, {0.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[5, 8], 0.13348388671875}, {0.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[7, 8], 0.13348388671875}, {0.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[9, 8], 0.13348388671875}, {1.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[11, 8], 0.13348388671875}, {1.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[13, 8], 0.13348388671875}, {1.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[15, 8], 0.13348388671875}, {1.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[17, 8], 0.13348388671875}, {2.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[19, 8], 0.13348388671875}, {2.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[21, 8], 0.13348388671875}, {2.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[23, 8], 0.13348388671875}, {2.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[25, 8], 0.13348388671875}, {3.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[27, 8], 0.13348388671875}, {3.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[29, 8], 0.13348388671875}, {3.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[31, 8], 0.13348388671875}, {3.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[33, 8], 0.13348388671875}, {4.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[35, 8], 0.13348388671875}, {4.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[37, 8], 0.13348388671875}, {4.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[39, 8], 0.13348388671875}, {4.875, 
     0.13348388671875}]], 
   PointBox[NCache[{
     Rational[-44, 9], 0.1001129150390625}, {-4.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-43, 9], 0.1001129150390625}, {-4.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-41, 9], 0.1001129150390625}, {-4.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-40, 9], 0.1001129150390625}, {-4.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-38, 9], 0.1001129150390625}, {-4.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-37, 9], 0.1001129150390625}, {-4.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-35, 9], 0.1001129150390625}, {-3.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-34, 9], 0.1001129150390625}, {-3.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-32, 9], 0.1001129150390625}, {-3.5555555555555554`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-31, 9], 0.1001129150390625}, {-3.4444444444444446`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-29, 9], 0.1001129150390625}, {-3.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-28, 9], 0.1001129150390625}, {-3.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-26, 9], 0.1001129150390625}, {-2.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-25, 9], 0.1001129150390625}, {-2.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-23, 9], 0.1001129150390625}, {-2.5555555555555554`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-22, 9], 0.1001129150390625}, {-2.4444444444444446`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-20, 9], 0.1001129150390625}, {-2.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-19, 9], 0.1001129150390625}, {-2.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-17, 9], 0.1001129150390625}, {-1.8888888888888888`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-16, 9], 0.1001129150390625}, {-1.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-14, 9], 0.1001129150390625}, {-1.5555555555555556`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-13, 9], 0.1001129150390625}, {-1.4444444444444444`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-11, 9], 0.1001129150390625}, {-1.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-10, 9], 0.1001129150390625}, {-1.1111111111111112`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-8, 9], 0.1001129150390625}, {-0.8888888888888888, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-7, 9], 0.1001129150390625}, {-0.7777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-5, 9], 0.1001129150390625}, {-0.5555555555555556, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-4, 9], 0.1001129150390625}, {-0.4444444444444444, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-2, 9], 0.1001129150390625}, {-0.2222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-1, 9], 0.1001129150390625}, {-0.1111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[1, 9], 0.1001129150390625}, {0.1111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[2, 9], 0.1001129150390625}, {0.2222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[4, 9], 0.1001129150390625}, {0.4444444444444444, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[5, 9], 0.1001129150390625}, {0.5555555555555556, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[7, 9], 0.1001129150390625}, {0.7777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[8, 9], 0.1001129150390625}, {0.8888888888888888, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[10, 9], 0.1001129150390625}, {
     1.1111111111111112`, 0.1001129150390625}]], 
   PointBox[
    NCache[{Rational[11, 9], 0.1001129150390625}, {1.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[13, 9], 0.1001129150390625}, {
     1.4444444444444444`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[14, 9], 0.1001129150390625}, {
     1.5555555555555556`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[16, 9], 0.1001129150390625}, {
     1.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[17, 9], 0.1001129150390625}, {
     1.8888888888888888`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[19, 9], 0.1001129150390625}, {2.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[20, 9], 0.1001129150390625}, {
     2.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[22, 9], 0.1001129150390625}, {
     2.4444444444444446`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[23, 9], 0.1001129150390625}, {
     2.5555555555555554`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[25, 9], 0.1001129150390625}, {
     2.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[26, 9], 0.1001129150390625}, {2.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[28, 9], 0.1001129150390625}, {3.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[29, 9], 0.1001129150390625}, {
     3.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[31, 9], 0.1001129150390625}, {
     3.4444444444444446`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[32, 9], 0.1001129150390625}, {
     3.5555555555555554`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[34, 9], 0.1001129150390625}, {
     3.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[35, 9], 0.1001129150390625}, {3.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[37, 9], 0.1001129150390625}, {4.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[38, 9], 0.1001129150390625}, {4.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[40, 9], 0.1001129150390625}, {4.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[41, 9], 0.1001129150390625}, {4.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[43, 9], 0.1001129150390625}, {4.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[44, 9], 0.1001129150390625}, {4.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[-49, 10], 0.07508468627929688}, {-4.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-47, 10], 0.07508468627929688}, {-4.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-43, 10], 0.07508468627929688}, {-4.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-41, 10], 0.07508468627929688}, {-4.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-39, 10], 0.07508468627929688}, {-3.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-37, 10], 0.07508468627929688}, {-3.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-33, 10], 0.07508468627929688}, {-3.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-31, 10], 0.07508468627929688}, {-3.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-29, 10], 0.07508468627929688}, {-2.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-27, 10], 0.07508468627929688}, {-2.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-23, 10], 0.07508468627929688}, {-2.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-21, 10], 0.07508468627929688}, {-2.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-19, 10], 0.07508468627929688}, {-1.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-17, 10], 0.07508468627929688}, {-1.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-13, 10], 0.07508468627929688}, {-1.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-11, 10], 0.07508468627929688}, {-1.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-9, 10], 0.07508468627929688}, {-0.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-7, 10], 0.07508468627929688}, {-0.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-3, 10], 0.07508468627929688}, {-0.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-1, 10], 0.07508468627929688}, {-0.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[1, 10], 0.07508468627929688}, {0.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[3, 10], 0.07508468627929688}, {0.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[7, 10], 0.07508468627929688}, {0.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[9, 10], 0.07508468627929688}, {0.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[11, 10], 0.07508468627929688}, {1.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[13, 10], 0.07508468627929688}, {1.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[17, 10], 0.07508468627929688}, {1.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[19, 10], 0.07508468627929688}, {1.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[21, 10], 0.07508468627929688}, {2.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[23, 10], 0.07508468627929688}, {2.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[27, 10], 0.07508468627929688}, {2.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[29, 10], 0.07508468627929688}, {2.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[31, 10], 0.07508468627929688}, {3.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[33, 10], 0.07508468627929688}, {3.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[37, 10], 0.07508468627929688}, {3.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[39, 10], 0.07508468627929688}, {3.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[41, 10], 0.07508468627929688}, {4.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[43, 10], 0.07508468627929688}, {4.3, 
     0.07508468627929688}]], 
   PointBox[
    NCache[{Rational[47, 10], 0.07508468627929688}, {4.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[49, 10], 0.07508468627929688}, {4.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{
     Rational[-54, 11], 0.056313514709472656`}, {-4.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-53, 11], 0.056313514709472656`}, {-4.818181818181818, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-52, 11], 0.056313514709472656`}, {-4.7272727272727275`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-51, 11], 0.056313514709472656`}, {-4.636363636363637, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-50, 11], 0.056313514709472656`}, {-4.545454545454546, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-49, 11], 0.056313514709472656`}, {-4.454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-48, 11], 0.056313514709472656`}, {-4.363636363636363, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-47, 11], 0.056313514709472656`}, {-4.2727272727272725`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-46, 11], 0.056313514709472656`}, {-4.181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-45, 11], 0.056313514709472656`}, {-4.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-43, 11], 0.056313514709472656`}, {-3.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-42, 11], 0.056313514709472656`}, {-3.8181818181818183`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-41, 11], 0.056313514709472656`}, {-3.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-40, 11], 0.056313514709472656`}, {-3.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-39, 11], 0.056313514709472656`}, {-3.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-38, 11], 0.056313514709472656`}, {-3.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-37, 11], 0.056313514709472656`}, {-3.3636363636363638`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-36, 11], 0.056313514709472656`}, {-3.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-35, 11], 0.056313514709472656`}, {-3.1818181818181817`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-34, 11], 0.056313514709472656`}, {-3.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-32, 11], 0.056313514709472656`}, {-2.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-31, 11], 0.056313514709472656`}, {-2.8181818181818183`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-30, 11], 0.056313514709472656`}, {-2.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-29, 11], 0.056313514709472656`}, {-2.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-28, 11], 0.056313514709472656`}, {-2.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-27, 11], 0.056313514709472656`}, {-2.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-26, 11], 0.056313514709472656`}, {-2.3636363636363638`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-25, 11], 0.056313514709472656`}, {-2.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-24, 11], 0.056313514709472656`}, {-2.1818181818181817`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-23, 11], 0.056313514709472656`}, {-2.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-21, 11], 0.056313514709472656`}, {-1.9090909090909092`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-20, 11], 0.056313514709472656`}, {-1.8181818181818181`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-19, 11], 0.056313514709472656`}, {-1.7272727272727273`, 
     0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[-18, 11], 0.056313514709472656`}, {-1.6363636363636365`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-17, 11], 0.056313514709472656`}, {-1.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-16, 11], 0.056313514709472656`}, {-1.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-15, 11], 0.056313514709472656`}, {-1.3636363636363635`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-14, 11], 0.056313514709472656`}, {-1.2727272727272727`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-13, 11], 0.056313514709472656`}, {-1.1818181818181819`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-12, 11], 0.056313514709472656`}, {-1.0909090909090908`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-10, 11], 0.056313514709472656`}, {-0.9090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-9, 11], 0.056313514709472656`}, {-0.8181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-8, 11], 0.056313514709472656`}, {-0.7272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-7, 11], 0.056313514709472656`}, {-0.6363636363636364, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-6, 11], 0.056313514709472656`}, {-0.5454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-5, 11], 0.056313514709472656`}, {-0.45454545454545453`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-4, 11], 0.056313514709472656`}, {-0.36363636363636365`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-3, 11], 0.056313514709472656`}, {-0.2727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-2, 11], 0.056313514709472656`}, {-0.18181818181818182`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-1, 11], 0.056313514709472656`}, {-0.09090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[1, 11], 0.056313514709472656`}, {
     0.09090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[2, 11], 0.056313514709472656`}, {
     0.18181818181818182`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[3, 11], 0.056313514709472656`}, {
     0.2727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[4, 11], 0.056313514709472656`}, {
     0.36363636363636365`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[5, 11], 0.056313514709472656`}, {
     0.45454545454545453`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[6, 11], 0.056313514709472656`}, {
     0.5454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[7, 11], 0.056313514709472656`}, {
     0.6363636363636364, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[8, 11], 0.056313514709472656`}, {
     0.7272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[9, 11], 0.056313514709472656`}, {
     0.8181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[10, 11], 0.056313514709472656`}, {
     0.9090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[12, 11], 0.056313514709472656`}, {
     1.0909090909090908`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[13, 11], 0.056313514709472656`}, {
     1.1818181818181819`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[14, 11], 0.056313514709472656`}, {
     1.2727272727272727`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[15, 11], 0.056313514709472656`}, {
     1.3636363636363635`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[16, 11], 0.056313514709472656`}, {
     1.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[17, 11], 0.056313514709472656`}, {
     1.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[18, 11], 0.056313514709472656`}, {
     1.6363636363636365`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[19, 11], 0.056313514709472656`}, {
     1.7272727272727273`, 0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[20, 11], 0.056313514709472656`}, {1.8181818181818181`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[21, 11], 0.056313514709472656`}, {
     1.9090909090909092`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[23, 11], 0.056313514709472656`}, {
     2.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[24, 11], 0.056313514709472656`}, {
     2.1818181818181817`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[25, 11], 0.056313514709472656`}, {
     2.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[26, 11], 0.056313514709472656`}, {
     2.3636363636363638`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[27, 11], 0.056313514709472656`}, {
     2.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[28, 11], 0.056313514709472656`}, {
     2.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[29, 11], 0.056313514709472656`}, {
     2.6363636363636362`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[30, 11], 0.056313514709472656`}, {
     2.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[31, 11], 0.056313514709472656`}, {
     2.8181818181818183`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[32, 11], 0.056313514709472656`}, {
     2.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[34, 11], 0.056313514709472656`}, {
     3.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[35, 11], 0.056313514709472656`}, {
     3.1818181818181817`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[36, 11], 0.056313514709472656`}, {
     3.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[37, 11], 0.056313514709472656`}, {
     3.3636363636363638`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[38, 11], 0.056313514709472656`}, {
     3.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[39, 11], 0.056313514709472656`}, {
     3.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[40, 11], 0.056313514709472656`}, {
     3.6363636363636362`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[41, 11], 0.056313514709472656`}, {
     3.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[42, 11], 0.056313514709472656`}, {
     3.8181818181818183`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[43, 11], 0.056313514709472656`}, {
     3.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[45, 11], 0.056313514709472656`}, {
     4.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[46, 11], 0.056313514709472656`}, {
     4.181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[47, 11], 0.056313514709472656`}, {
     4.2727272727272725`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[48, 11], 0.056313514709472656`}, {
     4.363636363636363, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[49, 11], 0.056313514709472656`}, {
     4.454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[50, 11], 0.056313514709472656`}, {
     4.545454545454546, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[51, 11], 0.056313514709472656`}, {
     4.636363636363637, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[52, 11], 0.056313514709472656`}, {
     4.7272727272727275`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[53, 11], 0.056313514709472656`}, {
     4.818181818181818, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[54, 11], 0.056313514709472656`}, {
     4.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-59, 12], 0.04223513603210449}, {-4.916666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-55, 12], 0.04223513603210449}, {-4.583333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-53, 12], 0.04223513603210449}, {-4.416666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-49, 12], 0.04223513603210449}, {-4.083333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-47, 12], 0.04223513603210449}, {-3.9166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-43, 12], 0.04223513603210449}, {-3.5833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-41, 12], 0.04223513603210449}, {-3.4166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-37, 12], 0.04223513603210449}, {-3.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-35, 12], 0.04223513603210449}, {-2.9166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-31, 12], 0.04223513603210449}, {-2.5833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-29, 12], 0.04223513603210449}, {-2.4166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-25, 12], 0.04223513603210449}, {-2.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-23, 12], 0.04223513603210449}, {-1.9166666666666667`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-19, 12], 0.04223513603210449}, {-1.5833333333333333`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-17, 12], 0.04223513603210449}, {-1.4166666666666667`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-13, 12], 0.04223513603210449}, {-1.0833333333333333`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-11, 12], 0.04223513603210449}, {-0.9166666666666666, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-7, 12], 0.04223513603210449}, {-0.5833333333333334, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-5, 12], 0.04223513603210449}, {-0.4166666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-1, 12], 0.04223513603210449}, {-0.08333333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[1, 12], 0.04223513603210449}, {
     0.08333333333333333, 0.04223513603210449}]], 
   PointBox[
    NCache[{Rational[5, 12], 0.04223513603210449}, {0.4166666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[7, 12], 0.04223513603210449}, {
     0.5833333333333334, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[11, 12], 0.04223513603210449}, {
     0.9166666666666666, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[13, 12], 0.04223513603210449}, {
     1.0833333333333333`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[17, 12], 0.04223513603210449}, {
     1.4166666666666667`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[19, 12], 0.04223513603210449}, {
     1.5833333333333333`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[23, 12], 0.04223513603210449}, {
     1.9166666666666667`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[25, 12], 0.04223513603210449}, {
     2.0833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[29, 12], 0.04223513603210449}, {
     2.4166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[31, 12], 0.04223513603210449}, {
     2.5833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[35, 12], 0.04223513603210449}, {
     2.9166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[37, 12], 0.04223513603210449}, {
     3.0833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[41, 12], 0.04223513603210449}, {
     3.4166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[43, 12], 0.04223513603210449}, {
     3.5833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[47, 12], 0.04223513603210449}, {
     3.9166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[49, 12], 0.04223513603210449}, {
     4.083333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[53, 12], 0.04223513603210449}, {
     4.416666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[55, 12], 0.04223513603210449}, {
     4.583333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[59, 12], 0.04223513603210449}, {
     4.916666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-64, 13], 0.03167635202407837}, {-4.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-63, 13], 0.03167635202407837}, {-4.846153846153846, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-62, 13], 0.03167635202407837}, {-4.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-61, 13], 0.03167635202407837}, {-4.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-60, 13], 0.03167635202407837}, {-4.615384615384615, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-59, 13], 0.03167635202407837}, {-4.538461538461538, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-58, 13], 0.03167635202407837}, {-4.461538461538462, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-57, 13], 0.03167635202407837}, {-4.384615384615385, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-56, 13], 0.03167635202407837}, {-4.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-55, 13], 0.03167635202407837}, {-4.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-54, 13], 0.03167635202407837}, {-4.153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-53, 13], 0.03167635202407837}, {-4.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-51, 13], 0.03167635202407837}, {-3.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-50, 13], 0.03167635202407837}, {-3.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-49, 13], 0.03167635202407837}, {-3.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-48, 13], 0.03167635202407837}, {-3.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-47, 13], 0.03167635202407837}, {-3.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-46, 13], 0.03167635202407837}, {-3.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-45, 13], 0.03167635202407837}, {-3.4615384615384617`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-44, 13], 0.03167635202407837}, {-3.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-43, 13], 0.03167635202407837}, {-3.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-42, 13], 0.03167635202407837}, {-3.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-41, 13], 0.03167635202407837}, {-3.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-40, 13], 0.03167635202407837}, {-3.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-38, 13], 0.03167635202407837}, {-2.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-37, 13], 0.03167635202407837}, {-2.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-36, 13], 0.03167635202407837}, {-2.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-35, 13], 0.03167635202407837}, {-2.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-34, 13], 0.03167635202407837}, {-2.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-33, 13], 0.03167635202407837}, {-2.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-32, 13], 0.03167635202407837}, {-2.4615384615384617`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-31, 13], 0.03167635202407837}, {-2.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-30, 13], 0.03167635202407837}, {-2.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-29, 13], 0.03167635202407837}, {-2.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-28, 13], 0.03167635202407837}, {-2.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-27, 13], 0.03167635202407837}, {-2.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-25, 13], 0.03167635202407837}, {-1.9230769230769231`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-24, 13], 0.03167635202407837}, {-1.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-23, 13], 0.03167635202407837}, {-1.7692307692307692`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-22, 13], 0.03167635202407837}, {-1.6923076923076923`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-21, 13], 0.03167635202407837}, {-1.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-20, 13], 0.03167635202407837}, {-1.5384615384615385`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-19, 13], 0.03167635202407837}, {-1.4615384615384615`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-18, 13], 0.03167635202407837}, {-1.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-17, 13], 0.03167635202407837}, {-1.3076923076923077`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-16, 13], 0.03167635202407837}, {-1.2307692307692308`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-15, 13], 0.03167635202407837}, {-1.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-14, 13], 0.03167635202407837}, {-1.0769230769230769`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-12, 13], 0.03167635202407837}, {-0.9230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-11, 13], 0.03167635202407837}, {-0.8461538461538461, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-10, 13], 0.03167635202407837}, {-0.7692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-9, 13], 0.03167635202407837}, {-0.6923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-8, 13], 0.03167635202407837}, {-0.6153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-7, 13], 0.03167635202407837}, {-0.5384615384615384, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-6, 13], 0.03167635202407837}, {-0.46153846153846156`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-5, 13], 0.03167635202407837}, {-0.38461538461538464`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-4, 13], 0.03167635202407837}, {-0.3076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-3, 13], 0.03167635202407837}, {-0.23076923076923078`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-2, 13], 0.03167635202407837}, {-0.15384615384615385`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-1, 13], 0.03167635202407837}, {-0.07692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[1, 13], 0.03167635202407837}, {
     0.07692307692307693, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[2, 13], 0.03167635202407837}, {
     0.15384615384615385`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[3, 13], 0.03167635202407837}, {
     0.23076923076923078`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[4, 13], 0.03167635202407837}, {
     0.3076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[5, 13], 0.03167635202407837}, {
     0.38461538461538464`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[6, 13], 0.03167635202407837}, {
     0.46153846153846156`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[7, 13], 0.03167635202407837}, {
     0.5384615384615384, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[8, 13], 0.03167635202407837}, {
     0.6153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[9, 13], 0.03167635202407837}, {
     0.6923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[10, 13], 0.03167635202407837}, {
     0.7692307692307693, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[11, 13], 0.03167635202407837}, {
     0.8461538461538461, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[12, 13], 0.03167635202407837}, {
     0.9230769230769231, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[14, 13], 0.03167635202407837}, {1.0769230769230769`, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[15, 13], 0.03167635202407837}, {
     1.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[16, 13], 0.03167635202407837}, {
     1.2307692307692308`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[17, 13], 0.03167635202407837}, {
     1.3076923076923077`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[18, 13], 0.03167635202407837}, {
     1.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[19, 13], 0.03167635202407837}, {
     1.4615384615384615`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[20, 13], 0.03167635202407837}, {
     1.5384615384615385`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[21, 13], 0.03167635202407837}, {
     1.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[22, 13], 0.03167635202407837}, {
     1.6923076923076923`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[23, 13], 0.03167635202407837}, {
     1.7692307692307692`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[24, 13], 0.03167635202407837}, {
     1.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[25, 13], 0.03167635202407837}, {
     1.9230769230769231`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[27, 13], 0.03167635202407837}, {
     2.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[28, 13], 0.03167635202407837}, {
     2.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[29, 13], 0.03167635202407837}, {
     2.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[30, 13], 0.03167635202407837}, {
     2.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[31, 13], 0.03167635202407837}, {
     2.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[32, 13], 0.03167635202407837}, {
     2.4615384615384617`, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[33, 13], 0.03167635202407837}, {2.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[34, 13], 0.03167635202407837}, {
     2.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[35, 13], 0.03167635202407837}, {
     2.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[36, 13], 0.03167635202407837}, {
     2.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[37, 13], 0.03167635202407837}, {
     2.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[38, 13], 0.03167635202407837}, {
     2.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[40, 13], 0.03167635202407837}, {
     3.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[41, 13], 0.03167635202407837}, {
     3.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[42, 13], 0.03167635202407837}, {
     3.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[43, 13], 0.03167635202407837}, {
     3.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[44, 13], 0.03167635202407837}, {
     3.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[45, 13], 0.03167635202407837}, {
     3.4615384615384617`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[46, 13], 0.03167635202407837}, {
     3.5384615384615383`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[47, 13], 0.03167635202407837}, {
     3.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[48, 13], 0.03167635202407837}, {
     3.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[49, 13], 0.03167635202407837}, {
     3.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[50, 13], 0.03167635202407837}, {
     3.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[51, 13], 0.03167635202407837}, {
     3.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[53, 13], 0.03167635202407837}, {
     4.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[54, 13], 0.03167635202407837}, {
     4.153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[55, 13], 0.03167635202407837}, {
     4.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[56, 13], 0.03167635202407837}, {
     4.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[57, 13], 0.03167635202407837}, {
     4.384615384615385, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[58, 13], 0.03167635202407837}, {
     4.461538461538462, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[59, 13], 0.03167635202407837}, {
     4.538461538461538, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[60, 13], 0.03167635202407837}, {
     4.615384615384615, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[61, 13], 0.03167635202407837}, {
     4.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[62, 13], 0.03167635202407837}, {
     4.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[63, 13], 0.03167635202407837}, {
     4.846153846153846, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[64, 13], 0.03167635202407837}, {
     4.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-69, 14], 0.023757264018058777`}, {-4.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-67, 14], 0.023757264018058777`}, {-4.785714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-65, 14], 0.023757264018058777`}, {-4.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-61, 14], 0.023757264018058777`}, {-4.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-59, 14], 0.023757264018058777`}, {-4.214285714285714, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-57, 14], 0.023757264018058777`}, {-4.071428571428571, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-55, 14], 0.023757264018058777`}, {-3.9285714285714284`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-53, 14], 0.023757264018058777`}, {-3.7857142857142856`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-51, 14], 0.023757264018058777`}, {-3.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-47, 14], 0.023757264018058777`}, {-3.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-45, 14], 0.023757264018058777`}, {-3.2142857142857144`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-43, 14], 0.023757264018058777`}, {-3.0714285714285716`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-41, 14], 0.023757264018058777`}, {-2.9285714285714284`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-39, 14], 0.023757264018058777`}, {-2.7857142857142856`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-37, 14], 0.023757264018058777`}, {-2.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-33, 14], 0.023757264018058777`}, {-2.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-31, 14], 0.023757264018058777`}, {-2.2142857142857144`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-29, 14], 0.023757264018058777`}, {-2.0714285714285716`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-27, 14], 0.023757264018058777`}, {-1.9285714285714286`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-25, 14], 0.023757264018058777`}, {-1.7857142857142858`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-23, 14], 0.023757264018058777`}, {-1.6428571428571428`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-19, 14], 0.023757264018058777`}, {-1.3571428571428572`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-17, 14], 0.023757264018058777`}, {-1.2142857142857142`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-15, 14], 0.023757264018058777`}, {-1.0714285714285714`, 
     0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[-13, 14], 0.023757264018058777`}, {-0.9285714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-11, 14], 0.023757264018058777`}, {-0.7857142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-9, 14], 0.023757264018058777`}, {-0.6428571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-5, 14], 0.023757264018058777`}, {-0.35714285714285715`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-3, 14], 0.023757264018058777`}, {-0.21428571428571427`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-1, 14], 0.023757264018058777`}, {-0.07142857142857142, 
     0.023757264018058777`}]], 
   PointBox[NCache[{Rational[1, 14], 0.023757264018058777`}, {
     0.07142857142857142, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[3, 14], 0.023757264018058777`}, {
     0.21428571428571427`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[5, 14], 0.023757264018058777`}, {
     0.35714285714285715`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[9, 14], 0.023757264018058777`}, {
     0.6428571428571429, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[11, 14], 0.023757264018058777`}, {
     0.7857142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[13, 14], 0.023757264018058777`}, {
     0.9285714285714286, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[15, 14], 0.023757264018058777`}, {
     1.0714285714285714`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[17, 14], 0.023757264018058777`}, {
     1.2142857142857142`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[19, 14], 0.023757264018058777`}, {
     1.3571428571428572`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[23, 14], 0.023757264018058777`}, {
     1.6428571428571428`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[25, 14], 0.023757264018058777`}, {
     1.7857142857142858`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[27, 14], 0.023757264018058777`}, {
     1.9285714285714286`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[29, 14], 0.023757264018058777`}, {
     2.0714285714285716`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[31, 14], 0.023757264018058777`}, {
     2.2142857142857144`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[33, 14], 0.023757264018058777`}, {
     2.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[37, 14], 0.023757264018058777`}, {
     2.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[39, 14], 0.023757264018058777`}, {
     2.7857142857142856`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[41, 14], 0.023757264018058777`}, {
     2.9285714285714284`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[43, 14], 0.023757264018058777`}, {
     3.0714285714285716`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[45, 14], 0.023757264018058777`}, {
     3.2142857142857144`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[47, 14], 0.023757264018058777`}, {
     3.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[51, 14], 0.023757264018058777`}, {
     3.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[53, 14], 0.023757264018058777`}, {
     3.7857142857142856`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[55, 14], 0.023757264018058777`}, {
     3.9285714285714284`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[57, 14], 0.023757264018058777`}, {
     4.071428571428571, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[59, 14], 0.023757264018058777`}, {
     4.214285714285714, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[61, 14], 0.023757264018058777`}, {
     4.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[65, 14], 0.023757264018058777`}, {
     4.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[67, 14], 0.023757264018058777`}, {
     4.785714285714286, 0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[69, 14], 0.023757264018058777`}, {4.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-74, 15], 0.017817948013544083`}, {-4.933333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-73, 15], 0.017817948013544083`}, {-4.866666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-71, 15], 0.017817948013544083`}, {-4.733333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-68, 15], 0.017817948013544083`}, {-4.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-67, 15], 0.017817948013544083`}, {-4.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-64, 15], 0.017817948013544083`}, {-4.266666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-62, 15], 0.017817948013544083`}, {-4.133333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-61, 15], 0.017817948013544083`}, {-4.066666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-59, 15], 0.017817948013544083`}, {-3.933333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-58, 15], 0.017817948013544083`}, {-3.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-56, 15], 0.017817948013544083`}, {-3.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-53, 15], 0.017817948013544083`}, {-3.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-52, 15], 0.017817948013544083`}, {-3.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-49, 15], 0.017817948013544083`}, {-3.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-47, 15], 0.017817948013544083`}, {-3.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-46, 15], 0.017817948013544083`}, {-3.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-44, 15], 0.017817948013544083`}, {-2.933333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-43, 15], 0.017817948013544083`}, {-2.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-41, 15], 0.017817948013544083`}, {-2.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-38, 15], 0.017817948013544083`}, {-2.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-37, 15], 0.017817948013544083`}, {-2.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-34, 15], 0.017817948013544083`}, {-2.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-32, 15], 0.017817948013544083`}, {-2.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-31, 15], 0.017817948013544083`}, {-2.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-29, 15], 0.017817948013544083`}, {-1.9333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-28, 15], 0.017817948013544083`}, {-1.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-26, 15], 0.017817948013544083`}, {-1.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-23, 15], 0.017817948013544083`}, {-1.5333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-22, 15], 0.017817948013544083`}, {-1.4666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-19, 15], 0.017817948013544083`}, {-1.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-17, 15], 0.017817948013544083`}, {-1.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-16, 15], 0.017817948013544083`}, {-1.0666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-14, 15], 0.017817948013544083`}, {-0.9333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-13, 15], 0.017817948013544083`}, {-0.8666666666666667, 
     0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[-11, 15], 0.017817948013544083`}, {-0.7333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-8, 15], 0.017817948013544083`}, {-0.5333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-7, 15], 0.017817948013544083`}, {-0.4666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-4, 15], 0.017817948013544083`}, {-0.26666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-2, 15], 0.017817948013544083`}, {-0.13333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-1, 15], 0.017817948013544083`}, {-0.06666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{Rational[1, 15], 0.017817948013544083`}, {
     0.06666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[2, 15], 0.017817948013544083`}, {
     0.13333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[4, 15], 0.017817948013544083`}, {
     0.26666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[7, 15], 0.017817948013544083`}, {
     0.4666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[8, 15], 0.017817948013544083`}, {
     0.5333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[11, 15], 0.017817948013544083`}, {
     0.7333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[13, 15], 0.017817948013544083`}, {
     0.8666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[14, 15], 0.017817948013544083`}, {
     0.9333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[16, 15], 0.017817948013544083`}, {
     1.0666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[17, 15], 0.017817948013544083`}, {
     1.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[19, 15], 0.017817948013544083`}, {
     1.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[22, 15], 0.017817948013544083`}, {
     1.4666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[23, 15], 0.017817948013544083`}, {
     1.5333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[26, 15], 0.017817948013544083`}, {
     1.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[28, 15], 0.017817948013544083`}, {
     1.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[29, 15], 0.017817948013544083`}, {
     1.9333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[31, 15], 0.017817948013544083`}, {
     2.066666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[32, 15], 0.017817948013544083`}, {
     2.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[34, 15], 0.017817948013544083`}, {
     2.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[37, 15], 0.017817948013544083`}, {
     2.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[38, 15], 0.017817948013544083`}, {
     2.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[41, 15], 0.017817948013544083`}, {
     2.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[43, 15], 0.017817948013544083`}, {
     2.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[44, 15], 0.017817948013544083`}, {
     2.933333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[46, 15], 0.017817948013544083`}, {
     3.066666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[47, 15], 0.017817948013544083`}, {
     3.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[49, 15], 0.017817948013544083`}, {
     3.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[52, 15], 0.017817948013544083`}, {
     3.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[53, 15], 0.017817948013544083`}, {
     3.533333333333333, 0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[56, 15], 0.017817948013544083`}, {3.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{Rational[58, 15], 0.017817948013544083`}, {
     3.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[59, 15], 0.017817948013544083`}, {
     3.933333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[61, 15], 0.017817948013544083`}, {
     4.066666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[62, 15], 0.017817948013544083`}, {
     4.133333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[64, 15], 0.017817948013544083`}, {
     4.266666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[67, 15], 0.017817948013544083`}, {
     4.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[68, 15], 0.017817948013544083`}, {
     4.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[71, 15], 0.017817948013544083`}, {
     4.733333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[73, 15], 0.017817948013544083`}, {
     4.866666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[74, 15], 0.017817948013544083`}, {
     4.933333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[-79, 16], 0.013363461010158062`}, {-4.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-77, 16], 0.013363461010158062`}, {-4.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-75, 16], 0.013363461010158062`}, {-4.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-73, 16], 0.013363461010158062`}, {-4.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-71, 16], 0.013363461010158062`}, {-4.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-69, 16], 0.013363461010158062`}, {-4.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-67, 16], 0.013363461010158062`}, {-4.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-65, 16], 0.013363461010158062`}, {-4.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-63, 16], 0.013363461010158062`}, {-3.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-61, 16], 0.013363461010158062`}, {-3.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-59, 16], 0.013363461010158062`}, {-3.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-57, 16], 0.013363461010158062`}, {-3.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-55, 16], 0.013363461010158062`}, {-3.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-53, 16], 0.013363461010158062`}, {-3.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-51, 16], 0.013363461010158062`}, {-3.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-49, 16], 0.013363461010158062`}, {-3.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-47, 16], 0.013363461010158062`}, {-2.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-45, 16], 0.013363461010158062`}, {-2.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-43, 16], 0.013363461010158062`}, {-2.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-41, 16], 0.013363461010158062`}, {-2.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-39, 16], 0.013363461010158062`}, {-2.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-37, 16], 0.013363461010158062`}, {-2.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-35, 16], 0.013363461010158062`}, {-2.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-33, 16], 0.013363461010158062`}, {-2.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-31, 16], 0.013363461010158062`}, {-1.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-29, 16], 0.013363461010158062`}, {-1.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-27, 16], 0.013363461010158062`}, {-1.6875, 
     0.013363461010158062`}]], 
   PointBox[
    NCache[{Rational[-25, 16], 0.013363461010158062`}, {-1.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-23, 16], 0.013363461010158062`}, {-1.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-21, 16], 0.013363461010158062`}, {-1.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-19, 16], 0.013363461010158062`}, {-1.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-17, 16], 0.013363461010158062`}, {-1.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-15, 16], 0.013363461010158062`}, {-0.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-13, 16], 0.013363461010158062`}, {-0.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-11, 16], 0.013363461010158062`}, {-0.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-9, 16], 0.013363461010158062`}, {-0.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-7, 16], 0.013363461010158062`}, {-0.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-5, 16], 0.013363461010158062`}, {-0.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-3, 16], 0.013363461010158062`}, {-0.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-1, 16], 0.013363461010158062`}, {-0.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[1, 16], 0.013363461010158062`}, {0.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[3, 16], 0.013363461010158062`}, {0.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[5, 16], 0.013363461010158062`}, {0.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[7, 16], 0.013363461010158062`}, {0.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[9, 16], 0.013363461010158062`}, {0.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[11, 16], 0.013363461010158062`}, {0.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[13, 16], 0.013363461010158062`}, {0.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[15, 16], 0.013363461010158062`}, {0.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[17, 16], 0.013363461010158062`}, {1.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[19, 16], 0.013363461010158062`}, {1.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[21, 16], 0.013363461010158062`}, {1.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[23, 16], 0.013363461010158062`}, {1.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[25, 16], 0.013363461010158062`}, {1.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[27, 16], 0.013363461010158062`}, {1.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[29, 16], 0.013363461010158062`}, {1.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[31, 16], 0.013363461010158062`}, {1.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[33, 16], 0.013363461010158062`}, {2.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[35, 16], 0.013363461010158062`}, {2.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[37, 16], 0.013363461010158062`}, {2.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[39, 16], 0.013363461010158062`}, {2.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[41, 16], 0.013363461010158062`}, {2.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[43, 16], 0.013363461010158062`}, {2.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[45, 16], 0.013363461010158062`}, {2.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[47, 16], 0.013363461010158062`}, {2.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[49, 16], 0.013363461010158062`}, {3.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[51, 16], 0.013363461010158062`}, {3.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[53, 16], 0.013363461010158062`}, {3.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[55, 16], 0.013363461010158062`}, {3.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[57, 16], 0.013363461010158062`}, {3.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[59, 16], 0.013363461010158062`}, {3.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[61, 16], 0.013363461010158062`}, {3.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[63, 16], 0.013363461010158062`}, {3.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[65, 16], 0.013363461010158062`}, {4.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[67, 16], 0.013363461010158062`}, {4.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[69, 16], 0.013363461010158062`}, {4.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[71, 16], 0.013363461010158062`}, {4.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[73, 16], 0.013363461010158062`}, {4.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[75, 16], 0.013363461010158062`}, {4.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[77, 16], 0.013363461010158062`}, {4.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[79, 16], 0.013363461010158062`}, {4.9375, 
     0.013363461010158062`}]]},
  AspectRatio->NCache[
    Rational[1, 2], 0.5],
  Axes->{True, False},
  AxesOrigin->{0, 0},
  PlotRange->All,
  Prolog->{
    AbsolutePointSize[2], 
    AbsoluteThickness[1]}]], "Output",
 CellChangeTimes->{3.4427473252895*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "Having all of these dots on a page may tempt one to play \"dot to dot\" to \
see if a picture forms. In this case there may not be a picture, but perhaps \
some pattern can be formed by connecting these dots. Go ahead and try it!\n\n\
EXPERIMENT:\nMake a printout of the above plot and try to find a way to start \
at one point, then connect all of the dots using a sequence of straight \
lines. Try to keep the path from crossing itself. The aim is not just to \
connect the dots which are printed, but to created a pattern which, when \
continued indefinitely, will eventually hit every rational number no matter \
how close to the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-axis it appears, or how far to the left or right of the origin it is. \
There are, in fact, many ways this can be done."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "What does this experiment show?  When we are connecting the dots together, \
we are, in fact, forming a ",
 StyleBox["sequence",
  FontSlant->"Italic"],
 " of rational numbers. By finding a way to connect the dots so that every \
rational number is eventually hit, we have  found an infinite sequence that \
includes every rational number once and only once. In other words, there are \
just as many rational numbers as there are integers! This is at first \
surprising since we just saw that there are many more rationals, in fact an \
infinite number of them between any two. \n\nDEFINITION 9.1\nA set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is called ",
 StyleBox["countable",
  FontSlant->"Italic"],
 " if there is an infinite sequence of elements from the set which includes \
every member of the set.\n\nPROPOSITION 9.2\nThe set of rationals forms a \
countable set."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437152500140625*^9, 3.437152517265625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
In order to show that the rationals are countable, we need a sequence which \
will eventually contain every rational somewhere in the sequence.  \
Equivalently, we can connect the dots of the figure formed from \
\>", "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43715255596875*^9, 3.437152686796875*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ShowRationals", "[", 
  RowBox[{
   RowBox[{"-", "5"}], ",", "5"}], "]"}]], "Input",
 CellChangeTimes->{{3.437152689515625*^9, 3.43715269575*^9}}],

Cell[BoxData[
 GraphicsBox[{PointBox[{-5, 1}], PointBox[{-4, 1}], PointBox[{-3, 1}], 
   PointBox[{-2, 1}], PointBox[{-1, 1}], PointBox[{0, 1}], PointBox[{1, 1}], 
   PointBox[{2, 1}], PointBox[{3, 1}], PointBox[{4, 1}], PointBox[{5, 1}], 
   PointBox[NCache[{Rational[-9, 2], 0.75}, {-4.5, 0.75}]], 
   PointBox[NCache[{Rational[-7, 2], 0.75}, {-3.5, 0.75}]], 
   PointBox[NCache[{Rational[-5, 2], 0.75}, {-2.5, 0.75}]], 
   PointBox[NCache[{Rational[-3, 2], 0.75}, {-1.5, 0.75}]], 
   PointBox[NCache[{Rational[-1, 2], 0.75}, {-0.5, 0.75}]], 
   PointBox[NCache[{Rational[1, 2], 0.75}, {0.5, 0.75}]], 
   PointBox[NCache[{Rational[3, 2], 0.75}, {1.5, 0.75}]], 
   PointBox[NCache[{Rational[5, 2], 0.75}, {2.5, 0.75}]], 
   PointBox[NCache[{Rational[7, 2], 0.75}, {3.5, 0.75}]], 
   PointBox[NCache[{Rational[9, 2], 0.75}, {4.5, 0.75}]], 
   PointBox[NCache[{Rational[-14, 3], 0.5625}, {-4.666666666666667, 0.5625}]],
    PointBox[
    NCache[{Rational[-13, 3], 0.5625}, {-4.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[-11, 3], 0.5625}, {-3.6666666666666665`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-10, 3], 0.5625}, {-3.3333333333333335`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-8, 3], 0.5625}, {-2.6666666666666665`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-7, 3], 0.5625}, {-2.3333333333333335`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-5, 3], 0.5625}, {-1.6666666666666667`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-4, 3], 0.5625}, {-1.3333333333333333`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-2, 3], 0.5625}, {-0.6666666666666666, 0.5625}]],
    PointBox[
    NCache[{Rational[-1, 3], 0.5625}, {-0.3333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[1, 3], 0.5625}, {0.3333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[2, 3], 0.5625}, {0.6666666666666666, 0.5625}]], 
   PointBox[NCache[{Rational[4, 3], 0.5625}, {1.3333333333333333`, 0.5625}]], 
   PointBox[NCache[{Rational[5, 3], 0.5625}, {1.6666666666666667`, 0.5625}]], 
   PointBox[NCache[{Rational[7, 3], 0.5625}, {2.3333333333333335`, 0.5625}]], 
   PointBox[NCache[{Rational[8, 3], 0.5625}, {2.6666666666666665`, 0.5625}]], 
   PointBox[NCache[{Rational[10, 3], 0.5625}, {3.3333333333333335`, 0.5625}]],
    PointBox[
    NCache[{Rational[11, 3], 0.5625}, {3.6666666666666665`, 0.5625}]], 
   PointBox[NCache[{Rational[13, 3], 0.5625}, {4.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[14, 3], 0.5625}, {4.666666666666667, 0.5625}]], 
   PointBox[NCache[{Rational[-19, 4], 0.421875}, {-4.75, 0.421875}]], 
   PointBox[NCache[{Rational[-17, 4], 0.421875}, {-4.25, 0.421875}]], 
   PointBox[NCache[{Rational[-15, 4], 0.421875}, {-3.75, 0.421875}]], 
   PointBox[NCache[{Rational[-13, 4], 0.421875}, {-3.25, 0.421875}]], 
   PointBox[NCache[{Rational[-11, 4], 0.421875}, {-2.75, 0.421875}]], 
   PointBox[NCache[{Rational[-9, 4], 0.421875}, {-2.25, 0.421875}]], 
   PointBox[NCache[{Rational[-7, 4], 0.421875}, {-1.75, 0.421875}]], 
   PointBox[NCache[{Rational[-5, 4], 0.421875}, {-1.25, 0.421875}]], 
   PointBox[NCache[{Rational[-3, 4], 0.421875}, {-0.75, 0.421875}]], 
   PointBox[NCache[{Rational[-1, 4], 0.421875}, {-0.25, 0.421875}]], 
   PointBox[NCache[{Rational[1, 4], 0.421875}, {0.25, 0.421875}]], 
   PointBox[NCache[{Rational[3, 4], 0.421875}, {0.75, 0.421875}]], 
   PointBox[NCache[{Rational[5, 4], 0.421875}, {1.25, 0.421875}]], 
   PointBox[NCache[{Rational[7, 4], 0.421875}, {1.75, 0.421875}]], 
   PointBox[NCache[{Rational[9, 4], 0.421875}, {2.25, 0.421875}]], 
   PointBox[NCache[{Rational[11, 4], 0.421875}, {2.75, 0.421875}]], 
   PointBox[NCache[{Rational[13, 4], 0.421875}, {3.25, 0.421875}]], 
   PointBox[NCache[{Rational[15, 4], 0.421875}, {3.75, 0.421875}]], 
   PointBox[NCache[{Rational[17, 4], 0.421875}, {4.25, 0.421875}]], 
   PointBox[NCache[{Rational[19, 4], 0.421875}, {4.75, 0.421875}]], 
   PointBox[NCache[{Rational[-24, 5], 0.31640625}, {-4.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-23, 5], 0.31640625}, {-4.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-22, 5], 0.31640625}, {-4.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-21, 5], 0.31640625}, {-4.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-19, 5], 0.31640625}, {-3.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-18, 5], 0.31640625}, {-3.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-17, 5], 0.31640625}, {-3.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-16, 5], 0.31640625}, {-3.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-14, 5], 0.31640625}, {-2.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-13, 5], 0.31640625}, {-2.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-12, 5], 0.31640625}, {-2.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-11, 5], 0.31640625}, {-2.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-9, 5], 0.31640625}, {-1.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-8, 5], 0.31640625}, {-1.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-7, 5], 0.31640625}, {-1.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-6, 5], 0.31640625}, {-1.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-4, 5], 0.31640625}, {-0.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-3, 5], 0.31640625}, {-0.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-2, 5], 0.31640625}, {-0.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-1, 5], 0.31640625}, {-0.2, 0.31640625}]], 
   PointBox[NCache[{Rational[1, 5], 0.31640625}, {0.2, 0.31640625}]], 
   PointBox[NCache[{Rational[2, 5], 0.31640625}, {0.4, 0.31640625}]], 
   PointBox[NCache[{Rational[3, 5], 0.31640625}, {0.6, 0.31640625}]], 
   PointBox[NCache[{Rational[4, 5], 0.31640625}, {0.8, 0.31640625}]], 
   PointBox[NCache[{Rational[6, 5], 0.31640625}, {1.2, 0.31640625}]], 
   PointBox[NCache[{Rational[7, 5], 0.31640625}, {1.4, 0.31640625}]], 
   PointBox[NCache[{Rational[8, 5], 0.31640625}, {1.6, 0.31640625}]], 
   PointBox[NCache[{Rational[9, 5], 0.31640625}, {1.8, 0.31640625}]], 
   PointBox[NCache[{Rational[11, 5], 0.31640625}, {2.2, 0.31640625}]], 
   PointBox[NCache[{Rational[12, 5], 0.31640625}, {2.4, 0.31640625}]], 
   PointBox[NCache[{Rational[13, 5], 0.31640625}, {2.6, 0.31640625}]], 
   PointBox[NCache[{Rational[14, 5], 0.31640625}, {2.8, 0.31640625}]], 
   PointBox[NCache[{Rational[16, 5], 0.31640625}, {3.2, 0.31640625}]], 
   PointBox[NCache[{Rational[17, 5], 0.31640625}, {3.4, 0.31640625}]], 
   PointBox[NCache[{Rational[18, 5], 0.31640625}, {3.6, 0.31640625}]], 
   PointBox[NCache[{Rational[19, 5], 0.31640625}, {3.8, 0.31640625}]], 
   PointBox[NCache[{Rational[21, 5], 0.31640625}, {4.2, 0.31640625}]], 
   PointBox[NCache[{Rational[22, 5], 0.31640625}, {4.4, 0.31640625}]], 
   PointBox[NCache[{Rational[23, 5], 0.31640625}, {4.6, 0.31640625}]], 
   PointBox[NCache[{Rational[24, 5], 0.31640625}, {4.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-29, 6], 0.2373046875}, {-4.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-25, 6], 0.2373046875}, {-4.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-23, 6], 0.2373046875}, {-3.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-19, 6], 0.2373046875}, {-3.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-17, 6], 0.2373046875}, {-2.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-13, 6], 0.2373046875}, {-2.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-11, 6], 0.2373046875}, {-1.8333333333333333`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-7, 6], 0.2373046875}, {-1.1666666666666667`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-5, 6], 0.2373046875}, {-0.8333333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-1, 6], 0.2373046875}, {-0.16666666666666666`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[1, 6], 0.2373046875}, {0.16666666666666666`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[5, 6], 0.2373046875}, {0.8333333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[7, 6], 0.2373046875}, {1.1666666666666667`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[11, 6], 0.2373046875}, {1.8333333333333333`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[13, 6], 0.2373046875}, {2.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[17, 6], 0.2373046875}, {2.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[19, 6], 0.2373046875}, {3.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[23, 6], 0.2373046875}, {3.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[25, 6], 0.2373046875}, {4.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[29, 6], 0.2373046875}, {4.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-34, 7], 0.177978515625}, {-4.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-33, 7], 0.177978515625}, {-4.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-32, 7], 0.177978515625}, {-4.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-31, 7], 0.177978515625}, {-4.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-30, 7], 0.177978515625}, {-4.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-29, 7], 0.177978515625}, {-4.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-27, 7], 0.177978515625}, {-3.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-26, 7], 0.177978515625}, {-3.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-25, 7], 0.177978515625}, {-3.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-24, 7], 0.177978515625}, {-3.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-23, 7], 0.177978515625}, {-3.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-22, 7], 0.177978515625}, {-3.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-20, 7], 0.177978515625}, {-2.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-19, 7], 0.177978515625}, {-2.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-18, 7], 0.177978515625}, {-2.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-17, 7], 0.177978515625}, {-2.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-16, 7], 0.177978515625}, {-2.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-15, 7], 0.177978515625}, {-2.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-13, 7], 0.177978515625}, {-1.8571428571428572`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-12, 7], 0.177978515625}, {-1.7142857142857142`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-11, 7], 0.177978515625}, {-1.5714285714285714`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-10, 7], 0.177978515625}, {-1.4285714285714286`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-9, 7], 0.177978515625}, {-1.2857142857142858`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-8, 7], 0.177978515625}, {-1.1428571428571428`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-6, 7], 0.177978515625}, {-0.8571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-5, 7], 0.177978515625}, {-0.7142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-4, 7], 0.177978515625}, {-0.5714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-3, 7], 0.177978515625}, {-0.42857142857142855`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-2, 7], 0.177978515625}, {-0.2857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-1, 7], 0.177978515625}, {-0.14285714285714285`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[1, 7], 0.177978515625}, {0.14285714285714285`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[2, 7], 0.177978515625}, {0.2857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[3, 7], 0.177978515625}, {0.42857142857142855`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[4, 7], 0.177978515625}, {0.5714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[5, 7], 0.177978515625}, {0.7142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[6, 7], 0.177978515625}, {0.8571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[8, 7], 0.177978515625}, {1.1428571428571428`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[9, 7], 0.177978515625}, {1.2857142857142858`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[10, 7], 0.177978515625}, {1.4285714285714286`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[11, 7], 0.177978515625}, {1.5714285714285714`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[12, 7], 0.177978515625}, {1.7142857142857142`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[13, 7], 0.177978515625}, {1.8571428571428572`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[15, 7], 0.177978515625}, {2.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[16, 7], 0.177978515625}, {2.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[17, 7], 0.177978515625}, {2.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[18, 7], 0.177978515625}, {2.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[19, 7], 0.177978515625}, {2.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[20, 7], 0.177978515625}, {2.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[22, 7], 0.177978515625}, {3.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[23, 7], 0.177978515625}, {3.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[24, 7], 0.177978515625}, {3.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[25, 7], 0.177978515625}, {3.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[26, 7], 0.177978515625}, {3.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[27, 7], 0.177978515625}, {3.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[29, 7], 0.177978515625}, {4.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[30, 7], 0.177978515625}, {4.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[31, 7], 0.177978515625}, {4.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[32, 7], 0.177978515625}, {4.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[33, 7], 0.177978515625}, {4.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[34, 7], 0.177978515625}, {4.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-39, 8], 0.13348388671875}, {-4.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-37, 8], 0.13348388671875}, {-4.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-35, 8], 0.13348388671875}, {-4.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-33, 8], 0.13348388671875}, {-4.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-31, 8], 0.13348388671875}, {-3.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-29, 8], 0.13348388671875}, {-3.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-27, 8], 0.13348388671875}, {-3.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-25, 8], 0.13348388671875}, {-3.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-23, 8], 0.13348388671875}, {-2.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-21, 8], 0.13348388671875}, {-2.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-19, 8], 0.13348388671875}, {-2.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-17, 8], 0.13348388671875}, {-2.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-15, 8], 0.13348388671875}, {-1.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-13, 8], 0.13348388671875}, {-1.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-11, 8], 0.13348388671875}, {-1.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-9, 8], 0.13348388671875}, {-1.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-7, 8], 0.13348388671875}, {-0.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-5, 8], 0.13348388671875}, {-0.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-3, 8], 0.13348388671875}, {-0.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-1, 8], 0.13348388671875}, {-0.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[1, 8], 0.13348388671875}, {0.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[3, 8], 0.13348388671875}, {0.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[5, 8], 0.13348388671875}, {0.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[7, 8], 0.13348388671875}, {0.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[9, 8], 0.13348388671875}, {1.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[11, 8], 0.13348388671875}, {1.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[13, 8], 0.13348388671875}, {1.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[15, 8], 0.13348388671875}, {1.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[17, 8], 0.13348388671875}, {2.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[19, 8], 0.13348388671875}, {2.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[21, 8], 0.13348388671875}, {2.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[23, 8], 0.13348388671875}, {2.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[25, 8], 0.13348388671875}, {3.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[27, 8], 0.13348388671875}, {3.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[29, 8], 0.13348388671875}, {3.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[31, 8], 0.13348388671875}, {3.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[33, 8], 0.13348388671875}, {4.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[35, 8], 0.13348388671875}, {4.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[37, 8], 0.13348388671875}, {4.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[39, 8], 0.13348388671875}, {4.875, 
     0.13348388671875}]], 
   PointBox[NCache[{
     Rational[-44, 9], 0.1001129150390625}, {-4.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-43, 9], 0.1001129150390625}, {-4.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-41, 9], 0.1001129150390625}, {-4.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-40, 9], 0.1001129150390625}, {-4.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-38, 9], 0.1001129150390625}, {-4.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-37, 9], 0.1001129150390625}, {-4.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-35, 9], 0.1001129150390625}, {-3.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-34, 9], 0.1001129150390625}, {-3.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-32, 9], 0.1001129150390625}, {-3.5555555555555554`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-31, 9], 0.1001129150390625}, {-3.4444444444444446`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-29, 9], 0.1001129150390625}, {-3.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-28, 9], 0.1001129150390625}, {-3.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-26, 9], 0.1001129150390625}, {-2.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-25, 9], 0.1001129150390625}, {-2.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-23, 9], 0.1001129150390625}, {-2.5555555555555554`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-22, 9], 0.1001129150390625}, {-2.4444444444444446`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-20, 9], 0.1001129150390625}, {-2.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-19, 9], 0.1001129150390625}, {-2.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-17, 9], 0.1001129150390625}, {-1.8888888888888888`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-16, 9], 0.1001129150390625}, {-1.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-14, 9], 0.1001129150390625}, {-1.5555555555555556`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-13, 9], 0.1001129150390625}, {-1.4444444444444444`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-11, 9], 0.1001129150390625}, {-1.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-10, 9], 0.1001129150390625}, {-1.1111111111111112`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-8, 9], 0.1001129150390625}, {-0.8888888888888888, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-7, 9], 0.1001129150390625}, {-0.7777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-5, 9], 0.1001129150390625}, {-0.5555555555555556, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-4, 9], 0.1001129150390625}, {-0.4444444444444444, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-2, 9], 0.1001129150390625}, {-0.2222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-1, 9], 0.1001129150390625}, {-0.1111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[1, 9], 0.1001129150390625}, {0.1111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[2, 9], 0.1001129150390625}, {0.2222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[4, 9], 0.1001129150390625}, {0.4444444444444444, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[5, 9], 0.1001129150390625}, {0.5555555555555556, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[7, 9], 0.1001129150390625}, {0.7777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[8, 9], 0.1001129150390625}, {0.8888888888888888, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[10, 9], 0.1001129150390625}, {
     1.1111111111111112`, 0.1001129150390625}]], 
   PointBox[
    NCache[{Rational[11, 9], 0.1001129150390625}, {1.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[13, 9], 0.1001129150390625}, {
     1.4444444444444444`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[14, 9], 0.1001129150390625}, {
     1.5555555555555556`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[16, 9], 0.1001129150390625}, {
     1.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[17, 9], 0.1001129150390625}, {
     1.8888888888888888`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[19, 9], 0.1001129150390625}, {2.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[20, 9], 0.1001129150390625}, {
     2.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[22, 9], 0.1001129150390625}, {
     2.4444444444444446`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[23, 9], 0.1001129150390625}, {
     2.5555555555555554`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[25, 9], 0.1001129150390625}, {
     2.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[26, 9], 0.1001129150390625}, {2.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[28, 9], 0.1001129150390625}, {3.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[29, 9], 0.1001129150390625}, {
     3.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[31, 9], 0.1001129150390625}, {
     3.4444444444444446`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[32, 9], 0.1001129150390625}, {
     3.5555555555555554`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[34, 9], 0.1001129150390625}, {
     3.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[35, 9], 0.1001129150390625}, {3.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[37, 9], 0.1001129150390625}, {4.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[38, 9], 0.1001129150390625}, {4.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[40, 9], 0.1001129150390625}, {4.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[41, 9], 0.1001129150390625}, {4.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[43, 9], 0.1001129150390625}, {4.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[44, 9], 0.1001129150390625}, {4.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[-49, 10], 0.07508468627929688}, {-4.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-47, 10], 0.07508468627929688}, {-4.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-43, 10], 0.07508468627929688}, {-4.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-41, 10], 0.07508468627929688}, {-4.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-39, 10], 0.07508468627929688}, {-3.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-37, 10], 0.07508468627929688}, {-3.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-33, 10], 0.07508468627929688}, {-3.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-31, 10], 0.07508468627929688}, {-3.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-29, 10], 0.07508468627929688}, {-2.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-27, 10], 0.07508468627929688}, {-2.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-23, 10], 0.07508468627929688}, {-2.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-21, 10], 0.07508468627929688}, {-2.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-19, 10], 0.07508468627929688}, {-1.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-17, 10], 0.07508468627929688}, {-1.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-13, 10], 0.07508468627929688}, {-1.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-11, 10], 0.07508468627929688}, {-1.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-9, 10], 0.07508468627929688}, {-0.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-7, 10], 0.07508468627929688}, {-0.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-3, 10], 0.07508468627929688}, {-0.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-1, 10], 0.07508468627929688}, {-0.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[1, 10], 0.07508468627929688}, {0.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[3, 10], 0.07508468627929688}, {0.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[7, 10], 0.07508468627929688}, {0.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[9, 10], 0.07508468627929688}, {0.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[11, 10], 0.07508468627929688}, {1.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[13, 10], 0.07508468627929688}, {1.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[17, 10], 0.07508468627929688}, {1.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[19, 10], 0.07508468627929688}, {1.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[21, 10], 0.07508468627929688}, {2.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[23, 10], 0.07508468627929688}, {2.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[27, 10], 0.07508468627929688}, {2.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[29, 10], 0.07508468627929688}, {2.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[31, 10], 0.07508468627929688}, {3.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[33, 10], 0.07508468627929688}, {3.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[37, 10], 0.07508468627929688}, {3.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[39, 10], 0.07508468627929688}, {3.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[41, 10], 0.07508468627929688}, {4.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[43, 10], 0.07508468627929688}, {4.3, 
     0.07508468627929688}]], 
   PointBox[
    NCache[{Rational[47, 10], 0.07508468627929688}, {4.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[49, 10], 0.07508468627929688}, {4.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{
     Rational[-54, 11], 0.056313514709472656`}, {-4.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-53, 11], 0.056313514709472656`}, {-4.818181818181818, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-52, 11], 0.056313514709472656`}, {-4.7272727272727275`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-51, 11], 0.056313514709472656`}, {-4.636363636363637, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-50, 11], 0.056313514709472656`}, {-4.545454545454546, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-49, 11], 0.056313514709472656`}, {-4.454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-48, 11], 0.056313514709472656`}, {-4.363636363636363, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-47, 11], 0.056313514709472656`}, {-4.2727272727272725`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-46, 11], 0.056313514709472656`}, {-4.181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-45, 11], 0.056313514709472656`}, {-4.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-43, 11], 0.056313514709472656`}, {-3.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-42, 11], 0.056313514709472656`}, {-3.8181818181818183`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-41, 11], 0.056313514709472656`}, {-3.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-40, 11], 0.056313514709472656`}, {-3.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-39, 11], 0.056313514709472656`}, {-3.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-38, 11], 0.056313514709472656`}, {-3.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-37, 11], 0.056313514709472656`}, {-3.3636363636363638`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-36, 11], 0.056313514709472656`}, {-3.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-35, 11], 0.056313514709472656`}, {-3.1818181818181817`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-34, 11], 0.056313514709472656`}, {-3.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-32, 11], 0.056313514709472656`}, {-2.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-31, 11], 0.056313514709472656`}, {-2.8181818181818183`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-30, 11], 0.056313514709472656`}, {-2.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-29, 11], 0.056313514709472656`}, {-2.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-28, 11], 0.056313514709472656`}, {-2.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-27, 11], 0.056313514709472656`}, {-2.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-26, 11], 0.056313514709472656`}, {-2.3636363636363638`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-25, 11], 0.056313514709472656`}, {-2.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-24, 11], 0.056313514709472656`}, {-2.1818181818181817`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-23, 11], 0.056313514709472656`}, {-2.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-21, 11], 0.056313514709472656`}, {-1.9090909090909092`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-20, 11], 0.056313514709472656`}, {-1.8181818181818181`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-19, 11], 0.056313514709472656`}, {-1.7272727272727273`, 
     0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[-18, 11], 0.056313514709472656`}, {-1.6363636363636365`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-17, 11], 0.056313514709472656`}, {-1.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-16, 11], 0.056313514709472656`}, {-1.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-15, 11], 0.056313514709472656`}, {-1.3636363636363635`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-14, 11], 0.056313514709472656`}, {-1.2727272727272727`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-13, 11], 0.056313514709472656`}, {-1.1818181818181819`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-12, 11], 0.056313514709472656`}, {-1.0909090909090908`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-10, 11], 0.056313514709472656`}, {-0.9090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-9, 11], 0.056313514709472656`}, {-0.8181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-8, 11], 0.056313514709472656`}, {-0.7272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-7, 11], 0.056313514709472656`}, {-0.6363636363636364, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-6, 11], 0.056313514709472656`}, {-0.5454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-5, 11], 0.056313514709472656`}, {-0.45454545454545453`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-4, 11], 0.056313514709472656`}, {-0.36363636363636365`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-3, 11], 0.056313514709472656`}, {-0.2727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-2, 11], 0.056313514709472656`}, {-0.18181818181818182`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-1, 11], 0.056313514709472656`}, {-0.09090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[1, 11], 0.056313514709472656`}, {
     0.09090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[2, 11], 0.056313514709472656`}, {
     0.18181818181818182`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[3, 11], 0.056313514709472656`}, {
     0.2727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[4, 11], 0.056313514709472656`}, {
     0.36363636363636365`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[5, 11], 0.056313514709472656`}, {
     0.45454545454545453`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[6, 11], 0.056313514709472656`}, {
     0.5454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[7, 11], 0.056313514709472656`}, {
     0.6363636363636364, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[8, 11], 0.056313514709472656`}, {
     0.7272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[9, 11], 0.056313514709472656`}, {
     0.8181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[10, 11], 0.056313514709472656`}, {
     0.9090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[12, 11], 0.056313514709472656`}, {
     1.0909090909090908`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[13, 11], 0.056313514709472656`}, {
     1.1818181818181819`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[14, 11], 0.056313514709472656`}, {
     1.2727272727272727`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[15, 11], 0.056313514709472656`}, {
     1.3636363636363635`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[16, 11], 0.056313514709472656`}, {
     1.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[17, 11], 0.056313514709472656`}, {
     1.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[18, 11], 0.056313514709472656`}, {
     1.6363636363636365`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[19, 11], 0.056313514709472656`}, {
     1.7272727272727273`, 0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[20, 11], 0.056313514709472656`}, {1.8181818181818181`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[21, 11], 0.056313514709472656`}, {
     1.9090909090909092`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[23, 11], 0.056313514709472656`}, {
     2.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[24, 11], 0.056313514709472656`}, {
     2.1818181818181817`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[25, 11], 0.056313514709472656`}, {
     2.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[26, 11], 0.056313514709472656`}, {
     2.3636363636363638`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[27, 11], 0.056313514709472656`}, {
     2.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[28, 11], 0.056313514709472656`}, {
     2.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[29, 11], 0.056313514709472656`}, {
     2.6363636363636362`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[30, 11], 0.056313514709472656`}, {
     2.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[31, 11], 0.056313514709472656`}, {
     2.8181818181818183`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[32, 11], 0.056313514709472656`}, {
     2.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[34, 11], 0.056313514709472656`}, {
     3.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[35, 11], 0.056313514709472656`}, {
     3.1818181818181817`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[36, 11], 0.056313514709472656`}, {
     3.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[37, 11], 0.056313514709472656`}, {
     3.3636363636363638`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[38, 11], 0.056313514709472656`}, {
     3.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[39, 11], 0.056313514709472656`}, {
     3.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[40, 11], 0.056313514709472656`}, {
     3.6363636363636362`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[41, 11], 0.056313514709472656`}, {
     3.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[42, 11], 0.056313514709472656`}, {
     3.8181818181818183`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[43, 11], 0.056313514709472656`}, {
     3.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[45, 11], 0.056313514709472656`}, {
     4.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[46, 11], 0.056313514709472656`}, {
     4.181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[47, 11], 0.056313514709472656`}, {
     4.2727272727272725`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[48, 11], 0.056313514709472656`}, {
     4.363636363636363, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[49, 11], 0.056313514709472656`}, {
     4.454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[50, 11], 0.056313514709472656`}, {
     4.545454545454546, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[51, 11], 0.056313514709472656`}, {
     4.636363636363637, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[52, 11], 0.056313514709472656`}, {
     4.7272727272727275`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[53, 11], 0.056313514709472656`}, {
     4.818181818181818, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[54, 11], 0.056313514709472656`}, {
     4.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-59, 12], 0.04223513603210449}, {-4.916666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-55, 12], 0.04223513603210449}, {-4.583333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-53, 12], 0.04223513603210449}, {-4.416666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-49, 12], 0.04223513603210449}, {-4.083333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-47, 12], 0.04223513603210449}, {-3.9166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-43, 12], 0.04223513603210449}, {-3.5833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-41, 12], 0.04223513603210449}, {-3.4166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-37, 12], 0.04223513603210449}, {-3.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-35, 12], 0.04223513603210449}, {-2.9166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-31, 12], 0.04223513603210449}, {-2.5833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-29, 12], 0.04223513603210449}, {-2.4166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-25, 12], 0.04223513603210449}, {-2.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-23, 12], 0.04223513603210449}, {-1.9166666666666667`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-19, 12], 0.04223513603210449}, {-1.5833333333333333`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-17, 12], 0.04223513603210449}, {-1.4166666666666667`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-13, 12], 0.04223513603210449}, {-1.0833333333333333`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-11, 12], 0.04223513603210449}, {-0.9166666666666666, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-7, 12], 0.04223513603210449}, {-0.5833333333333334, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-5, 12], 0.04223513603210449}, {-0.4166666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-1, 12], 0.04223513603210449}, {-0.08333333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[1, 12], 0.04223513603210449}, {
     0.08333333333333333, 0.04223513603210449}]], 
   PointBox[
    NCache[{Rational[5, 12], 0.04223513603210449}, {0.4166666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[7, 12], 0.04223513603210449}, {
     0.5833333333333334, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[11, 12], 0.04223513603210449}, {
     0.9166666666666666, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[13, 12], 0.04223513603210449}, {
     1.0833333333333333`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[17, 12], 0.04223513603210449}, {
     1.4166666666666667`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[19, 12], 0.04223513603210449}, {
     1.5833333333333333`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[23, 12], 0.04223513603210449}, {
     1.9166666666666667`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[25, 12], 0.04223513603210449}, {
     2.0833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[29, 12], 0.04223513603210449}, {
     2.4166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[31, 12], 0.04223513603210449}, {
     2.5833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[35, 12], 0.04223513603210449}, {
     2.9166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[37, 12], 0.04223513603210449}, {
     3.0833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[41, 12], 0.04223513603210449}, {
     3.4166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[43, 12], 0.04223513603210449}, {
     3.5833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[47, 12], 0.04223513603210449}, {
     3.9166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[49, 12], 0.04223513603210449}, {
     4.083333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[53, 12], 0.04223513603210449}, {
     4.416666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[55, 12], 0.04223513603210449}, {
     4.583333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[59, 12], 0.04223513603210449}, {
     4.916666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-64, 13], 0.03167635202407837}, {-4.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-63, 13], 0.03167635202407837}, {-4.846153846153846, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-62, 13], 0.03167635202407837}, {-4.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-61, 13], 0.03167635202407837}, {-4.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-60, 13], 0.03167635202407837}, {-4.615384615384615, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-59, 13], 0.03167635202407837}, {-4.538461538461538, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-58, 13], 0.03167635202407837}, {-4.461538461538462, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-57, 13], 0.03167635202407837}, {-4.384615384615385, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-56, 13], 0.03167635202407837}, {-4.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-55, 13], 0.03167635202407837}, {-4.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-54, 13], 0.03167635202407837}, {-4.153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-53, 13], 0.03167635202407837}, {-4.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-51, 13], 0.03167635202407837}, {-3.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-50, 13], 0.03167635202407837}, {-3.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-49, 13], 0.03167635202407837}, {-3.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-48, 13], 0.03167635202407837}, {-3.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-47, 13], 0.03167635202407837}, {-3.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-46, 13], 0.03167635202407837}, {-3.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-45, 13], 0.03167635202407837}, {-3.4615384615384617`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-44, 13], 0.03167635202407837}, {-3.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-43, 13], 0.03167635202407837}, {-3.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-42, 13], 0.03167635202407837}, {-3.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-41, 13], 0.03167635202407837}, {-3.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-40, 13], 0.03167635202407837}, {-3.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-38, 13], 0.03167635202407837}, {-2.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-37, 13], 0.03167635202407837}, {-2.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-36, 13], 0.03167635202407837}, {-2.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-35, 13], 0.03167635202407837}, {-2.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-34, 13], 0.03167635202407837}, {-2.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-33, 13], 0.03167635202407837}, {-2.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-32, 13], 0.03167635202407837}, {-2.4615384615384617`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-31, 13], 0.03167635202407837}, {-2.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-30, 13], 0.03167635202407837}, {-2.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-29, 13], 0.03167635202407837}, {-2.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-28, 13], 0.03167635202407837}, {-2.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-27, 13], 0.03167635202407837}, {-2.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-25, 13], 0.03167635202407837}, {-1.9230769230769231`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-24, 13], 0.03167635202407837}, {-1.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-23, 13], 0.03167635202407837}, {-1.7692307692307692`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-22, 13], 0.03167635202407837}, {-1.6923076923076923`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-21, 13], 0.03167635202407837}, {-1.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-20, 13], 0.03167635202407837}, {-1.5384615384615385`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-19, 13], 0.03167635202407837}, {-1.4615384615384615`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-18, 13], 0.03167635202407837}, {-1.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-17, 13], 0.03167635202407837}, {-1.3076923076923077`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-16, 13], 0.03167635202407837}, {-1.2307692307692308`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-15, 13], 0.03167635202407837}, {-1.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-14, 13], 0.03167635202407837}, {-1.0769230769230769`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-12, 13], 0.03167635202407837}, {-0.9230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-11, 13], 0.03167635202407837}, {-0.8461538461538461, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-10, 13], 0.03167635202407837}, {-0.7692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-9, 13], 0.03167635202407837}, {-0.6923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-8, 13], 0.03167635202407837}, {-0.6153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-7, 13], 0.03167635202407837}, {-0.5384615384615384, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-6, 13], 0.03167635202407837}, {-0.46153846153846156`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-5, 13], 0.03167635202407837}, {-0.38461538461538464`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-4, 13], 0.03167635202407837}, {-0.3076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-3, 13], 0.03167635202407837}, {-0.23076923076923078`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-2, 13], 0.03167635202407837}, {-0.15384615384615385`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-1, 13], 0.03167635202407837}, {-0.07692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[1, 13], 0.03167635202407837}, {
     0.07692307692307693, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[2, 13], 0.03167635202407837}, {
     0.15384615384615385`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[3, 13], 0.03167635202407837}, {
     0.23076923076923078`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[4, 13], 0.03167635202407837}, {
     0.3076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[5, 13], 0.03167635202407837}, {
     0.38461538461538464`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[6, 13], 0.03167635202407837}, {
     0.46153846153846156`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[7, 13], 0.03167635202407837}, {
     0.5384615384615384, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[8, 13], 0.03167635202407837}, {
     0.6153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[9, 13], 0.03167635202407837}, {
     0.6923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[10, 13], 0.03167635202407837}, {
     0.7692307692307693, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[11, 13], 0.03167635202407837}, {
     0.8461538461538461, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[12, 13], 0.03167635202407837}, {
     0.9230769230769231, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[14, 13], 0.03167635202407837}, {1.0769230769230769`, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[15, 13], 0.03167635202407837}, {
     1.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[16, 13], 0.03167635202407837}, {
     1.2307692307692308`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[17, 13], 0.03167635202407837}, {
     1.3076923076923077`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[18, 13], 0.03167635202407837}, {
     1.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[19, 13], 0.03167635202407837}, {
     1.4615384615384615`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[20, 13], 0.03167635202407837}, {
     1.5384615384615385`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[21, 13], 0.03167635202407837}, {
     1.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[22, 13], 0.03167635202407837}, {
     1.6923076923076923`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[23, 13], 0.03167635202407837}, {
     1.7692307692307692`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[24, 13], 0.03167635202407837}, {
     1.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[25, 13], 0.03167635202407837}, {
     1.9230769230769231`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[27, 13], 0.03167635202407837}, {
     2.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[28, 13], 0.03167635202407837}, {
     2.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[29, 13], 0.03167635202407837}, {
     2.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[30, 13], 0.03167635202407837}, {
     2.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[31, 13], 0.03167635202407837}, {
     2.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[32, 13], 0.03167635202407837}, {
     2.4615384615384617`, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[33, 13], 0.03167635202407837}, {2.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[34, 13], 0.03167635202407837}, {
     2.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[35, 13], 0.03167635202407837}, {
     2.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[36, 13], 0.03167635202407837}, {
     2.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[37, 13], 0.03167635202407837}, {
     2.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[38, 13], 0.03167635202407837}, {
     2.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[40, 13], 0.03167635202407837}, {
     3.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[41, 13], 0.03167635202407837}, {
     3.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[42, 13], 0.03167635202407837}, {
     3.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[43, 13], 0.03167635202407837}, {
     3.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[44, 13], 0.03167635202407837}, {
     3.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[45, 13], 0.03167635202407837}, {
     3.4615384615384617`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[46, 13], 0.03167635202407837}, {
     3.5384615384615383`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[47, 13], 0.03167635202407837}, {
     3.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[48, 13], 0.03167635202407837}, {
     3.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[49, 13], 0.03167635202407837}, {
     3.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[50, 13], 0.03167635202407837}, {
     3.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[51, 13], 0.03167635202407837}, {
     3.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[53, 13], 0.03167635202407837}, {
     4.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[54, 13], 0.03167635202407837}, {
     4.153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[55, 13], 0.03167635202407837}, {
     4.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[56, 13], 0.03167635202407837}, {
     4.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[57, 13], 0.03167635202407837}, {
     4.384615384615385, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[58, 13], 0.03167635202407837}, {
     4.461538461538462, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[59, 13], 0.03167635202407837}, {
     4.538461538461538, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[60, 13], 0.03167635202407837}, {
     4.615384615384615, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[61, 13], 0.03167635202407837}, {
     4.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[62, 13], 0.03167635202407837}, {
     4.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[63, 13], 0.03167635202407837}, {
     4.846153846153846, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[64, 13], 0.03167635202407837}, {
     4.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-69, 14], 0.023757264018058777`}, {-4.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-67, 14], 0.023757264018058777`}, {-4.785714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-65, 14], 0.023757264018058777`}, {-4.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-61, 14], 0.023757264018058777`}, {-4.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-59, 14], 0.023757264018058777`}, {-4.214285714285714, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-57, 14], 0.023757264018058777`}, {-4.071428571428571, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-55, 14], 0.023757264018058777`}, {-3.9285714285714284`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-53, 14], 0.023757264018058777`}, {-3.7857142857142856`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-51, 14], 0.023757264018058777`}, {-3.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-47, 14], 0.023757264018058777`}, {-3.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-45, 14], 0.023757264018058777`}, {-3.2142857142857144`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-43, 14], 0.023757264018058777`}, {-3.0714285714285716`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-41, 14], 0.023757264018058777`}, {-2.9285714285714284`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-39, 14], 0.023757264018058777`}, {-2.7857142857142856`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-37, 14], 0.023757264018058777`}, {-2.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-33, 14], 0.023757264018058777`}, {-2.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-31, 14], 0.023757264018058777`}, {-2.2142857142857144`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-29, 14], 0.023757264018058777`}, {-2.0714285714285716`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-27, 14], 0.023757264018058777`}, {-1.9285714285714286`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-25, 14], 0.023757264018058777`}, {-1.7857142857142858`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-23, 14], 0.023757264018058777`}, {-1.6428571428571428`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-19, 14], 0.023757264018058777`}, {-1.3571428571428572`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-17, 14], 0.023757264018058777`}, {-1.2142857142857142`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-15, 14], 0.023757264018058777`}, {-1.0714285714285714`, 
     0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[-13, 14], 0.023757264018058777`}, {-0.9285714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-11, 14], 0.023757264018058777`}, {-0.7857142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-9, 14], 0.023757264018058777`}, {-0.6428571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-5, 14], 0.023757264018058777`}, {-0.35714285714285715`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-3, 14], 0.023757264018058777`}, {-0.21428571428571427`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-1, 14], 0.023757264018058777`}, {-0.07142857142857142, 
     0.023757264018058777`}]], 
   PointBox[NCache[{Rational[1, 14], 0.023757264018058777`}, {
     0.07142857142857142, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[3, 14], 0.023757264018058777`}, {
     0.21428571428571427`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[5, 14], 0.023757264018058777`}, {
     0.35714285714285715`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[9, 14], 0.023757264018058777`}, {
     0.6428571428571429, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[11, 14], 0.023757264018058777`}, {
     0.7857142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[13, 14], 0.023757264018058777`}, {
     0.9285714285714286, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[15, 14], 0.023757264018058777`}, {
     1.0714285714285714`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[17, 14], 0.023757264018058777`}, {
     1.2142857142857142`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[19, 14], 0.023757264018058777`}, {
     1.3571428571428572`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[23, 14], 0.023757264018058777`}, {
     1.6428571428571428`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[25, 14], 0.023757264018058777`}, {
     1.7857142857142858`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[27, 14], 0.023757264018058777`}, {
     1.9285714285714286`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[29, 14], 0.023757264018058777`}, {
     2.0714285714285716`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[31, 14], 0.023757264018058777`}, {
     2.2142857142857144`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[33, 14], 0.023757264018058777`}, {
     2.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[37, 14], 0.023757264018058777`}, {
     2.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[39, 14], 0.023757264018058777`}, {
     2.7857142857142856`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[41, 14], 0.023757264018058777`}, {
     2.9285714285714284`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[43, 14], 0.023757264018058777`}, {
     3.0714285714285716`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[45, 14], 0.023757264018058777`}, {
     3.2142857142857144`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[47, 14], 0.023757264018058777`}, {
     3.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[51, 14], 0.023757264018058777`}, {
     3.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[53, 14], 0.023757264018058777`}, {
     3.7857142857142856`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[55, 14], 0.023757264018058777`}, {
     3.9285714285714284`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[57, 14], 0.023757264018058777`}, {
     4.071428571428571, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[59, 14], 0.023757264018058777`}, {
     4.214285714285714, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[61, 14], 0.023757264018058777`}, {
     4.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[65, 14], 0.023757264018058777`}, {
     4.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[67, 14], 0.023757264018058777`}, {
     4.785714285714286, 0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[69, 14], 0.023757264018058777`}, {4.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-74, 15], 0.017817948013544083`}, {-4.933333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-73, 15], 0.017817948013544083`}, {-4.866666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-71, 15], 0.017817948013544083`}, {-4.733333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-68, 15], 0.017817948013544083`}, {-4.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-67, 15], 0.017817948013544083`}, {-4.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-64, 15], 0.017817948013544083`}, {-4.266666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-62, 15], 0.017817948013544083`}, {-4.133333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-61, 15], 0.017817948013544083`}, {-4.066666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-59, 15], 0.017817948013544083`}, {-3.933333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-58, 15], 0.017817948013544083`}, {-3.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-56, 15], 0.017817948013544083`}, {-3.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-53, 15], 0.017817948013544083`}, {-3.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-52, 15], 0.017817948013544083`}, {-3.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-49, 15], 0.017817948013544083`}, {-3.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-47, 15], 0.017817948013544083`}, {-3.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-46, 15], 0.017817948013544083`}, {-3.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-44, 15], 0.017817948013544083`}, {-2.933333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-43, 15], 0.017817948013544083`}, {-2.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-41, 15], 0.017817948013544083`}, {-2.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-38, 15], 0.017817948013544083`}, {-2.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-37, 15], 0.017817948013544083`}, {-2.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-34, 15], 0.017817948013544083`}, {-2.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-32, 15], 0.017817948013544083`}, {-2.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-31, 15], 0.017817948013544083`}, {-2.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-29, 15], 0.017817948013544083`}, {-1.9333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-28, 15], 0.017817948013544083`}, {-1.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-26, 15], 0.017817948013544083`}, {-1.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-23, 15], 0.017817948013544083`}, {-1.5333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-22, 15], 0.017817948013544083`}, {-1.4666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-19, 15], 0.017817948013544083`}, {-1.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-17, 15], 0.017817948013544083`}, {-1.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-16, 15], 0.017817948013544083`}, {-1.0666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-14, 15], 0.017817948013544083`}, {-0.9333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-13, 15], 0.017817948013544083`}, {-0.8666666666666667, 
     0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[-11, 15], 0.017817948013544083`}, {-0.7333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-8, 15], 0.017817948013544083`}, {-0.5333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-7, 15], 0.017817948013544083`}, {-0.4666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-4, 15], 0.017817948013544083`}, {-0.26666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-2, 15], 0.017817948013544083`}, {-0.13333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-1, 15], 0.017817948013544083`}, {-0.06666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{Rational[1, 15], 0.017817948013544083`}, {
     0.06666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[2, 15], 0.017817948013544083`}, {
     0.13333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[4, 15], 0.017817948013544083`}, {
     0.26666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[7, 15], 0.017817948013544083`}, {
     0.4666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[8, 15], 0.017817948013544083`}, {
     0.5333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[11, 15], 0.017817948013544083`}, {
     0.7333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[13, 15], 0.017817948013544083`}, {
     0.8666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[14, 15], 0.017817948013544083`}, {
     0.9333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[16, 15], 0.017817948013544083`}, {
     1.0666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[17, 15], 0.017817948013544083`}, {
     1.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[19, 15], 0.017817948013544083`}, {
     1.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[22, 15], 0.017817948013544083`}, {
     1.4666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[23, 15], 0.017817948013544083`}, {
     1.5333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[26, 15], 0.017817948013544083`}, {
     1.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[28, 15], 0.017817948013544083`}, {
     1.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[29, 15], 0.017817948013544083`}, {
     1.9333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[31, 15], 0.017817948013544083`}, {
     2.066666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[32, 15], 0.017817948013544083`}, {
     2.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[34, 15], 0.017817948013544083`}, {
     2.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[37, 15], 0.017817948013544083`}, {
     2.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[38, 15], 0.017817948013544083`}, {
     2.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[41, 15], 0.017817948013544083`}, {
     2.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[43, 15], 0.017817948013544083`}, {
     2.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[44, 15], 0.017817948013544083`}, {
     2.933333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[46, 15], 0.017817948013544083`}, {
     3.066666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[47, 15], 0.017817948013544083`}, {
     3.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[49, 15], 0.017817948013544083`}, {
     3.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[52, 15], 0.017817948013544083`}, {
     3.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[53, 15], 0.017817948013544083`}, {
     3.533333333333333, 0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[56, 15], 0.017817948013544083`}, {3.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{Rational[58, 15], 0.017817948013544083`}, {
     3.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[59, 15], 0.017817948013544083`}, {
     3.933333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[61, 15], 0.017817948013544083`}, {
     4.066666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[62, 15], 0.017817948013544083`}, {
     4.133333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[64, 15], 0.017817948013544083`}, {
     4.266666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[67, 15], 0.017817948013544083`}, {
     4.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[68, 15], 0.017817948013544083`}, {
     4.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[71, 15], 0.017817948013544083`}, {
     4.733333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[73, 15], 0.017817948013544083`}, {
     4.866666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[74, 15], 0.017817948013544083`}, {
     4.933333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[-79, 16], 0.013363461010158062`}, {-4.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-77, 16], 0.013363461010158062`}, {-4.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-75, 16], 0.013363461010158062`}, {-4.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-73, 16], 0.013363461010158062`}, {-4.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-71, 16], 0.013363461010158062`}, {-4.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-69, 16], 0.013363461010158062`}, {-4.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-67, 16], 0.013363461010158062`}, {-4.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-65, 16], 0.013363461010158062`}, {-4.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-63, 16], 0.013363461010158062`}, {-3.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-61, 16], 0.013363461010158062`}, {-3.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-59, 16], 0.013363461010158062`}, {-3.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-57, 16], 0.013363461010158062`}, {-3.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-55, 16], 0.013363461010158062`}, {-3.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-53, 16], 0.013363461010158062`}, {-3.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-51, 16], 0.013363461010158062`}, {-3.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-49, 16], 0.013363461010158062`}, {-3.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-47, 16], 0.013363461010158062`}, {-2.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-45, 16], 0.013363461010158062`}, {-2.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-43, 16], 0.013363461010158062`}, {-2.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-41, 16], 0.013363461010158062`}, {-2.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-39, 16], 0.013363461010158062`}, {-2.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-37, 16], 0.013363461010158062`}, {-2.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-35, 16], 0.013363461010158062`}, {-2.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-33, 16], 0.013363461010158062`}, {-2.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-31, 16], 0.013363461010158062`}, {-1.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-29, 16], 0.013363461010158062`}, {-1.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-27, 16], 0.013363461010158062`}, {-1.6875, 
     0.013363461010158062`}]], 
   PointBox[
    NCache[{Rational[-25, 16], 0.013363461010158062`}, {-1.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-23, 16], 0.013363461010158062`}, {-1.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-21, 16], 0.013363461010158062`}, {-1.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-19, 16], 0.013363461010158062`}, {-1.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-17, 16], 0.013363461010158062`}, {-1.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-15, 16], 0.013363461010158062`}, {-0.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-13, 16], 0.013363461010158062`}, {-0.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-11, 16], 0.013363461010158062`}, {-0.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-9, 16], 0.013363461010158062`}, {-0.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-7, 16], 0.013363461010158062`}, {-0.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-5, 16], 0.013363461010158062`}, {-0.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-3, 16], 0.013363461010158062`}, {-0.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-1, 16], 0.013363461010158062`}, {-0.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[1, 16], 0.013363461010158062`}, {0.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[3, 16], 0.013363461010158062`}, {0.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[5, 16], 0.013363461010158062`}, {0.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[7, 16], 0.013363461010158062`}, {0.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[9, 16], 0.013363461010158062`}, {0.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[11, 16], 0.013363461010158062`}, {0.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[13, 16], 0.013363461010158062`}, {0.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[15, 16], 0.013363461010158062`}, {0.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[17, 16], 0.013363461010158062`}, {1.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[19, 16], 0.013363461010158062`}, {1.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[21, 16], 0.013363461010158062`}, {1.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[23, 16], 0.013363461010158062`}, {1.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[25, 16], 0.013363461010158062`}, {1.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[27, 16], 0.013363461010158062`}, {1.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[29, 16], 0.013363461010158062`}, {1.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[31, 16], 0.013363461010158062`}, {1.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[33, 16], 0.013363461010158062`}, {2.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[35, 16], 0.013363461010158062`}, {2.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[37, 16], 0.013363461010158062`}, {2.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[39, 16], 0.013363461010158062`}, {2.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[41, 16], 0.013363461010158062`}, {2.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[43, 16], 0.013363461010158062`}, {2.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[45, 16], 0.013363461010158062`}, {2.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[47, 16], 0.013363461010158062`}, {2.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[49, 16], 0.013363461010158062`}, {3.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[51, 16], 0.013363461010158062`}, {3.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[53, 16], 0.013363461010158062`}, {3.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[55, 16], 0.013363461010158062`}, {3.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[57, 16], 0.013363461010158062`}, {3.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[59, 16], 0.013363461010158062`}, {3.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[61, 16], 0.013363461010158062`}, {3.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[63, 16], 0.013363461010158062`}, {3.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[65, 16], 0.013363461010158062`}, {4.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[67, 16], 0.013363461010158062`}, {4.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[69, 16], 0.013363461010158062`}, {4.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[71, 16], 0.013363461010158062`}, {4.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[73, 16], 0.013363461010158062`}, {4.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[75, 16], 0.013363461010158062`}, {4.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[77, 16], 0.013363461010158062`}, {4.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[79, 16], 0.013363461010158062`}, {4.9375, 
     0.013363461010158062`}]]},
  AspectRatio->NCache[
    Rational[1, 2], 0.5],
  Axes->{True, False},
  AxesOrigin->{0, 0},
  PlotRange->All,
  Prolog->{
    AbsolutePointSize[2], 
    AbsoluteThickness[1]}]], "Output",
 CellChangeTimes->{3.442747330055125*^9}]
}, Open  ]],

Cell["\<\

using a pattern that would, in principle, reach every dot of this figure \
extended to infinity.  There are of course many ways to do this, but one way \
is given by the patten in\
\>", "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43715255596875*^9, 3.437152783078125*^9}, {
  3.43715281346875*^9, 3.437152815875*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CountableQ", "[", "5", "]"}]], "Input",
 CellChangeTimes->{{3.437152797125*^9, 3.437152805359375*^9}}],

Cell[BoxData[
 GraphicsBox[{PointBox[{-5, 1}], PointBox[{-4, 1}], PointBox[{-3, 1}], 
   PointBox[{-2, 1}], PointBox[{-1, 1}], PointBox[{0, 1}], PointBox[{1, 1}], 
   PointBox[{2, 1}], PointBox[{3, 1}], PointBox[{4, 1}], PointBox[{5, 1}], 
   PointBox[NCache[{Rational[-9, 2], 0.75}, {-4.5, 0.75}]], 
   PointBox[NCache[{Rational[-7, 2], 0.75}, {-3.5, 0.75}]], 
   PointBox[NCache[{Rational[-5, 2], 0.75}, {-2.5, 0.75}]], 
   PointBox[NCache[{Rational[-3, 2], 0.75}, {-1.5, 0.75}]], 
   PointBox[NCache[{Rational[-1, 2], 0.75}, {-0.5, 0.75}]], 
   PointBox[NCache[{Rational[1, 2], 0.75}, {0.5, 0.75}]], 
   PointBox[NCache[{Rational[3, 2], 0.75}, {1.5, 0.75}]], 
   PointBox[NCache[{Rational[5, 2], 0.75}, {2.5, 0.75}]], 
   PointBox[NCache[{Rational[7, 2], 0.75}, {3.5, 0.75}]], 
   PointBox[NCache[{Rational[9, 2], 0.75}, {4.5, 0.75}]], 
   PointBox[NCache[{Rational[-14, 3], 0.5625}, {-4.666666666666667, 0.5625}]],
    PointBox[
    NCache[{Rational[-13, 3], 0.5625}, {-4.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[-11, 3], 0.5625}, {-3.6666666666666665`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-10, 3], 0.5625}, {-3.3333333333333335`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-8, 3], 0.5625}, {-2.6666666666666665`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-7, 3], 0.5625}, {-2.3333333333333335`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-5, 3], 0.5625}, {-1.6666666666666667`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-4, 3], 0.5625}, {-1.3333333333333333`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-2, 3], 0.5625}, {-0.6666666666666666, 0.5625}]],
    PointBox[
    NCache[{Rational[-1, 3], 0.5625}, {-0.3333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[1, 3], 0.5625}, {0.3333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[2, 3], 0.5625}, {0.6666666666666666, 0.5625}]], 
   PointBox[NCache[{Rational[4, 3], 0.5625}, {1.3333333333333333`, 0.5625}]], 
   PointBox[NCache[{Rational[5, 3], 0.5625}, {1.6666666666666667`, 0.5625}]], 
   PointBox[NCache[{Rational[7, 3], 0.5625}, {2.3333333333333335`, 0.5625}]], 
   PointBox[NCache[{Rational[8, 3], 0.5625}, {2.6666666666666665`, 0.5625}]], 
   PointBox[NCache[{Rational[10, 3], 0.5625}, {3.3333333333333335`, 0.5625}]],
    PointBox[
    NCache[{Rational[11, 3], 0.5625}, {3.6666666666666665`, 0.5625}]], 
   PointBox[NCache[{Rational[13, 3], 0.5625}, {4.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[14, 3], 0.5625}, {4.666666666666667, 0.5625}]], 
   PointBox[NCache[{Rational[-19, 4], 0.421875}, {-4.75, 0.421875}]], 
   PointBox[NCache[{Rational[-17, 4], 0.421875}, {-4.25, 0.421875}]], 
   PointBox[NCache[{Rational[-15, 4], 0.421875}, {-3.75, 0.421875}]], 
   PointBox[NCache[{Rational[-13, 4], 0.421875}, {-3.25, 0.421875}]], 
   PointBox[NCache[{Rational[-11, 4], 0.421875}, {-2.75, 0.421875}]], 
   PointBox[NCache[{Rational[-9, 4], 0.421875}, {-2.25, 0.421875}]], 
   PointBox[NCache[{Rational[-7, 4], 0.421875}, {-1.75, 0.421875}]], 
   PointBox[NCache[{Rational[-5, 4], 0.421875}, {-1.25, 0.421875}]], 
   PointBox[NCache[{Rational[-3, 4], 0.421875}, {-0.75, 0.421875}]], 
   PointBox[NCache[{Rational[-1, 4], 0.421875}, {-0.25, 0.421875}]], 
   PointBox[NCache[{Rational[1, 4], 0.421875}, {0.25, 0.421875}]], 
   PointBox[NCache[{Rational[3, 4], 0.421875}, {0.75, 0.421875}]], 
   PointBox[NCache[{Rational[5, 4], 0.421875}, {1.25, 0.421875}]], 
   PointBox[NCache[{Rational[7, 4], 0.421875}, {1.75, 0.421875}]], 
   PointBox[NCache[{Rational[9, 4], 0.421875}, {2.25, 0.421875}]], 
   PointBox[NCache[{Rational[11, 4], 0.421875}, {2.75, 0.421875}]], 
   PointBox[NCache[{Rational[13, 4], 0.421875}, {3.25, 0.421875}]], 
   PointBox[NCache[{Rational[15, 4], 0.421875}, {3.75, 0.421875}]], 
   PointBox[NCache[{Rational[17, 4], 0.421875}, {4.25, 0.421875}]], 
   PointBox[NCache[{Rational[19, 4], 0.421875}, {4.75, 0.421875}]], 
   PointBox[NCache[{Rational[-24, 5], 0.31640625}, {-4.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-23, 5], 0.31640625}, {-4.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-22, 5], 0.31640625}, {-4.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-21, 5], 0.31640625}, {-4.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-19, 5], 0.31640625}, {-3.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-18, 5], 0.31640625}, {-3.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-17, 5], 0.31640625}, {-3.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-16, 5], 0.31640625}, {-3.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-14, 5], 0.31640625}, {-2.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-13, 5], 0.31640625}, {-2.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-12, 5], 0.31640625}, {-2.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-11, 5], 0.31640625}, {-2.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-9, 5], 0.31640625}, {-1.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-8, 5], 0.31640625}, {-1.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-7, 5], 0.31640625}, {-1.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-6, 5], 0.31640625}, {-1.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-4, 5], 0.31640625}, {-0.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-3, 5], 0.31640625}, {-0.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-2, 5], 0.31640625}, {-0.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-1, 5], 0.31640625}, {-0.2, 0.31640625}]], 
   PointBox[NCache[{Rational[1, 5], 0.31640625}, {0.2, 0.31640625}]], 
   PointBox[NCache[{Rational[2, 5], 0.31640625}, {0.4, 0.31640625}]], 
   PointBox[NCache[{Rational[3, 5], 0.31640625}, {0.6, 0.31640625}]], 
   PointBox[NCache[{Rational[4, 5], 0.31640625}, {0.8, 0.31640625}]], 
   PointBox[NCache[{Rational[6, 5], 0.31640625}, {1.2, 0.31640625}]], 
   PointBox[NCache[{Rational[7, 5], 0.31640625}, {1.4, 0.31640625}]], 
   PointBox[NCache[{Rational[8, 5], 0.31640625}, {1.6, 0.31640625}]], 
   PointBox[NCache[{Rational[9, 5], 0.31640625}, {1.8, 0.31640625}]], 
   PointBox[NCache[{Rational[11, 5], 0.31640625}, {2.2, 0.31640625}]], 
   PointBox[NCache[{Rational[12, 5], 0.31640625}, {2.4, 0.31640625}]], 
   PointBox[NCache[{Rational[13, 5], 0.31640625}, {2.6, 0.31640625}]], 
   PointBox[NCache[{Rational[14, 5], 0.31640625}, {2.8, 0.31640625}]], 
   PointBox[NCache[{Rational[16, 5], 0.31640625}, {3.2, 0.31640625}]], 
   PointBox[NCache[{Rational[17, 5], 0.31640625}, {3.4, 0.31640625}]], 
   PointBox[NCache[{Rational[18, 5], 0.31640625}, {3.6, 0.31640625}]], 
   PointBox[NCache[{Rational[19, 5], 0.31640625}, {3.8, 0.31640625}]], 
   PointBox[NCache[{Rational[21, 5], 0.31640625}, {4.2, 0.31640625}]], 
   PointBox[NCache[{Rational[22, 5], 0.31640625}, {4.4, 0.31640625}]], 
   PointBox[NCache[{Rational[23, 5], 0.31640625}, {4.6, 0.31640625}]], 
   PointBox[NCache[{Rational[24, 5], 0.31640625}, {4.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-29, 6], 0.2373046875}, {-4.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-25, 6], 0.2373046875}, {-4.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-23, 6], 0.2373046875}, {-3.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-19, 6], 0.2373046875}, {-3.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-17, 6], 0.2373046875}, {-2.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-13, 6], 0.2373046875}, {-2.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-11, 6], 0.2373046875}, {-1.8333333333333333`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-7, 6], 0.2373046875}, {-1.1666666666666667`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-5, 6], 0.2373046875}, {-0.8333333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-1, 6], 0.2373046875}, {-0.16666666666666666`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[1, 6], 0.2373046875}, {0.16666666666666666`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[5, 6], 0.2373046875}, {0.8333333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[7, 6], 0.2373046875}, {1.1666666666666667`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[11, 6], 0.2373046875}, {1.8333333333333333`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[13, 6], 0.2373046875}, {2.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[17, 6], 0.2373046875}, {2.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[19, 6], 0.2373046875}, {3.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[23, 6], 0.2373046875}, {3.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[25, 6], 0.2373046875}, {4.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[29, 6], 0.2373046875}, {4.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-34, 7], 0.177978515625}, {-4.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-33, 7], 0.177978515625}, {-4.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-32, 7], 0.177978515625}, {-4.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-31, 7], 0.177978515625}, {-4.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-30, 7], 0.177978515625}, {-4.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-29, 7], 0.177978515625}, {-4.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-27, 7], 0.177978515625}, {-3.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-26, 7], 0.177978515625}, {-3.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-25, 7], 0.177978515625}, {-3.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-24, 7], 0.177978515625}, {-3.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-23, 7], 0.177978515625}, {-3.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-22, 7], 0.177978515625}, {-3.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-20, 7], 0.177978515625}, {-2.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-19, 7], 0.177978515625}, {-2.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-18, 7], 0.177978515625}, {-2.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-17, 7], 0.177978515625}, {-2.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-16, 7], 0.177978515625}, {-2.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-15, 7], 0.177978515625}, {-2.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-13, 7], 0.177978515625}, {-1.8571428571428572`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-12, 7], 0.177978515625}, {-1.7142857142857142`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-11, 7], 0.177978515625}, {-1.5714285714285714`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-10, 7], 0.177978515625}, {-1.4285714285714286`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-9, 7], 0.177978515625}, {-1.2857142857142858`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-8, 7], 0.177978515625}, {-1.1428571428571428`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-6, 7], 0.177978515625}, {-0.8571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-5, 7], 0.177978515625}, {-0.7142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-4, 7], 0.177978515625}, {-0.5714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-3, 7], 0.177978515625}, {-0.42857142857142855`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-2, 7], 0.177978515625}, {-0.2857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-1, 7], 0.177978515625}, {-0.14285714285714285`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[1, 7], 0.177978515625}, {0.14285714285714285`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[2, 7], 0.177978515625}, {0.2857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[3, 7], 0.177978515625}, {0.42857142857142855`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[4, 7], 0.177978515625}, {0.5714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[5, 7], 0.177978515625}, {0.7142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[6, 7], 0.177978515625}, {0.8571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[8, 7], 0.177978515625}, {1.1428571428571428`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[9, 7], 0.177978515625}, {1.2857142857142858`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[10, 7], 0.177978515625}, {1.4285714285714286`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[11, 7], 0.177978515625}, {1.5714285714285714`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[12, 7], 0.177978515625}, {1.7142857142857142`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[13, 7], 0.177978515625}, {1.8571428571428572`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[15, 7], 0.177978515625}, {2.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[16, 7], 0.177978515625}, {2.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[17, 7], 0.177978515625}, {2.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[18, 7], 0.177978515625}, {2.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[19, 7], 0.177978515625}, {2.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[20, 7], 0.177978515625}, {2.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[22, 7], 0.177978515625}, {3.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[23, 7], 0.177978515625}, {3.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[24, 7], 0.177978515625}, {3.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[25, 7], 0.177978515625}, {3.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[26, 7], 0.177978515625}, {3.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[27, 7], 0.177978515625}, {3.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[29, 7], 0.177978515625}, {4.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[30, 7], 0.177978515625}, {4.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[31, 7], 0.177978515625}, {4.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[32, 7], 0.177978515625}, {4.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[33, 7], 0.177978515625}, {4.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[34, 7], 0.177978515625}, {4.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-39, 8], 0.13348388671875}, {-4.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-37, 8], 0.13348388671875}, {-4.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-35, 8], 0.13348388671875}, {-4.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-33, 8], 0.13348388671875}, {-4.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-31, 8], 0.13348388671875}, {-3.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-29, 8], 0.13348388671875}, {-3.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-27, 8], 0.13348388671875}, {-3.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-25, 8], 0.13348388671875}, {-3.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-23, 8], 0.13348388671875}, {-2.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-21, 8], 0.13348388671875}, {-2.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-19, 8], 0.13348388671875}, {-2.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-17, 8], 0.13348388671875}, {-2.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-15, 8], 0.13348388671875}, {-1.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-13, 8], 0.13348388671875}, {-1.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-11, 8], 0.13348388671875}, {-1.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-9, 8], 0.13348388671875}, {-1.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-7, 8], 0.13348388671875}, {-0.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-5, 8], 0.13348388671875}, {-0.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-3, 8], 0.13348388671875}, {-0.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-1, 8], 0.13348388671875}, {-0.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[1, 8], 0.13348388671875}, {0.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[3, 8], 0.13348388671875}, {0.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[5, 8], 0.13348388671875}, {0.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[7, 8], 0.13348388671875}, {0.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[9, 8], 0.13348388671875}, {1.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[11, 8], 0.13348388671875}, {1.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[13, 8], 0.13348388671875}, {1.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[15, 8], 0.13348388671875}, {1.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[17, 8], 0.13348388671875}, {2.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[19, 8], 0.13348388671875}, {2.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[21, 8], 0.13348388671875}, {2.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[23, 8], 0.13348388671875}, {2.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[25, 8], 0.13348388671875}, {3.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[27, 8], 0.13348388671875}, {3.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[29, 8], 0.13348388671875}, {3.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[31, 8], 0.13348388671875}, {3.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[33, 8], 0.13348388671875}, {4.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[35, 8], 0.13348388671875}, {4.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[37, 8], 0.13348388671875}, {4.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[39, 8], 0.13348388671875}, {4.875, 
     0.13348388671875}]], 
   PointBox[NCache[{
     Rational[-44, 9], 0.1001129150390625}, {-4.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-43, 9], 0.1001129150390625}, {-4.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-41, 9], 0.1001129150390625}, {-4.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-40, 9], 0.1001129150390625}, {-4.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-38, 9], 0.1001129150390625}, {-4.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-37, 9], 0.1001129150390625}, {-4.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-35, 9], 0.1001129150390625}, {-3.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-34, 9], 0.1001129150390625}, {-3.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-32, 9], 0.1001129150390625}, {-3.5555555555555554`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-31, 9], 0.1001129150390625}, {-3.4444444444444446`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-29, 9], 0.1001129150390625}, {-3.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-28, 9], 0.1001129150390625}, {-3.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-26, 9], 0.1001129150390625}, {-2.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-25, 9], 0.1001129150390625}, {-2.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-23, 9], 0.1001129150390625}, {-2.5555555555555554`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-22, 9], 0.1001129150390625}, {-2.4444444444444446`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-20, 9], 0.1001129150390625}, {-2.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-19, 9], 0.1001129150390625}, {-2.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-17, 9], 0.1001129150390625}, {-1.8888888888888888`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-16, 9], 0.1001129150390625}, {-1.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-14, 9], 0.1001129150390625}, {-1.5555555555555556`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-13, 9], 0.1001129150390625}, {-1.4444444444444444`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-11, 9], 0.1001129150390625}, {-1.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-10, 9], 0.1001129150390625}, {-1.1111111111111112`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-8, 9], 0.1001129150390625}, {-0.8888888888888888, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-7, 9], 0.1001129150390625}, {-0.7777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-5, 9], 0.1001129150390625}, {-0.5555555555555556, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-4, 9], 0.1001129150390625}, {-0.4444444444444444, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-2, 9], 0.1001129150390625}, {-0.2222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-1, 9], 0.1001129150390625}, {-0.1111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[1, 9], 0.1001129150390625}, {0.1111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[2, 9], 0.1001129150390625}, {0.2222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[4, 9], 0.1001129150390625}, {0.4444444444444444, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[5, 9], 0.1001129150390625}, {0.5555555555555556, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[7, 9], 0.1001129150390625}, {0.7777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[8, 9], 0.1001129150390625}, {0.8888888888888888, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[10, 9], 0.1001129150390625}, {
     1.1111111111111112`, 0.1001129150390625}]], 
   PointBox[
    NCache[{Rational[11, 9], 0.1001129150390625}, {1.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[13, 9], 0.1001129150390625}, {
     1.4444444444444444`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[14, 9], 0.1001129150390625}, {
     1.5555555555555556`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[16, 9], 0.1001129150390625}, {
     1.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[17, 9], 0.1001129150390625}, {
     1.8888888888888888`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[19, 9], 0.1001129150390625}, {2.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[20, 9], 0.1001129150390625}, {
     2.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[22, 9], 0.1001129150390625}, {
     2.4444444444444446`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[23, 9], 0.1001129150390625}, {
     2.5555555555555554`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[25, 9], 0.1001129150390625}, {
     2.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[26, 9], 0.1001129150390625}, {2.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[28, 9], 0.1001129150390625}, {3.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[29, 9], 0.1001129150390625}, {
     3.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[31, 9], 0.1001129150390625}, {
     3.4444444444444446`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[32, 9], 0.1001129150390625}, {
     3.5555555555555554`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[34, 9], 0.1001129150390625}, {
     3.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[35, 9], 0.1001129150390625}, {3.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[37, 9], 0.1001129150390625}, {4.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[38, 9], 0.1001129150390625}, {4.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[40, 9], 0.1001129150390625}, {4.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[41, 9], 0.1001129150390625}, {4.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[43, 9], 0.1001129150390625}, {4.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[44, 9], 0.1001129150390625}, {4.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[-49, 10], 0.07508468627929688}, {-4.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-47, 10], 0.07508468627929688}, {-4.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-43, 10], 0.07508468627929688}, {-4.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-41, 10], 0.07508468627929688}, {-4.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-39, 10], 0.07508468627929688}, {-3.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-37, 10], 0.07508468627929688}, {-3.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-33, 10], 0.07508468627929688}, {-3.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-31, 10], 0.07508468627929688}, {-3.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-29, 10], 0.07508468627929688}, {-2.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-27, 10], 0.07508468627929688}, {-2.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-23, 10], 0.07508468627929688}, {-2.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-21, 10], 0.07508468627929688}, {-2.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-19, 10], 0.07508468627929688}, {-1.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-17, 10], 0.07508468627929688}, {-1.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-13, 10], 0.07508468627929688}, {-1.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-11, 10], 0.07508468627929688}, {-1.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-9, 10], 0.07508468627929688}, {-0.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-7, 10], 0.07508468627929688}, {-0.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-3, 10], 0.07508468627929688}, {-0.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-1, 10], 0.07508468627929688}, {-0.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[1, 10], 0.07508468627929688}, {0.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[3, 10], 0.07508468627929688}, {0.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[7, 10], 0.07508468627929688}, {0.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[9, 10], 0.07508468627929688}, {0.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[11, 10], 0.07508468627929688}, {1.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[13, 10], 0.07508468627929688}, {1.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[17, 10], 0.07508468627929688}, {1.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[19, 10], 0.07508468627929688}, {1.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[21, 10], 0.07508468627929688}, {2.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[23, 10], 0.07508468627929688}, {2.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[27, 10], 0.07508468627929688}, {2.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[29, 10], 0.07508468627929688}, {2.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[31, 10], 0.07508468627929688}, {3.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[33, 10], 0.07508468627929688}, {3.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[37, 10], 0.07508468627929688}, {3.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[39, 10], 0.07508468627929688}, {3.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[41, 10], 0.07508468627929688}, {4.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[43, 10], 0.07508468627929688}, {4.3, 
     0.07508468627929688}]], 
   PointBox[
    NCache[{Rational[47, 10], 0.07508468627929688}, {4.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[49, 10], 0.07508468627929688}, {4.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{
     Rational[-54, 11], 0.056313514709472656`}, {-4.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-53, 11], 0.056313514709472656`}, {-4.818181818181818, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-52, 11], 0.056313514709472656`}, {-4.7272727272727275`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-51, 11], 0.056313514709472656`}, {-4.636363636363637, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-50, 11], 0.056313514709472656`}, {-4.545454545454546, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-49, 11], 0.056313514709472656`}, {-4.454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-48, 11], 0.056313514709472656`}, {-4.363636363636363, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-47, 11], 0.056313514709472656`}, {-4.2727272727272725`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-46, 11], 0.056313514709472656`}, {-4.181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-45, 11], 0.056313514709472656`}, {-4.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-43, 11], 0.056313514709472656`}, {-3.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-42, 11], 0.056313514709472656`}, {-3.8181818181818183`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-41, 11], 0.056313514709472656`}, {-3.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-40, 11], 0.056313514709472656`}, {-3.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-39, 11], 0.056313514709472656`}, {-3.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-38, 11], 0.056313514709472656`}, {-3.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-37, 11], 0.056313514709472656`}, {-3.3636363636363638`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-36, 11], 0.056313514709472656`}, {-3.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-35, 11], 0.056313514709472656`}, {-3.1818181818181817`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-34, 11], 0.056313514709472656`}, {-3.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-32, 11], 0.056313514709472656`}, {-2.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-31, 11], 0.056313514709472656`}, {-2.8181818181818183`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-30, 11], 0.056313514709472656`}, {-2.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-29, 11], 0.056313514709472656`}, {-2.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-28, 11], 0.056313514709472656`}, {-2.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-27, 11], 0.056313514709472656`}, {-2.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-26, 11], 0.056313514709472656`}, {-2.3636363636363638`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-25, 11], 0.056313514709472656`}, {-2.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-24, 11], 0.056313514709472656`}, {-2.1818181818181817`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-23, 11], 0.056313514709472656`}, {-2.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-21, 11], 0.056313514709472656`}, {-1.9090909090909092`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-20, 11], 0.056313514709472656`}, {-1.8181818181818181`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-19, 11], 0.056313514709472656`}, {-1.7272727272727273`, 
     0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[-18, 11], 0.056313514709472656`}, {-1.6363636363636365`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-17, 11], 0.056313514709472656`}, {-1.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-16, 11], 0.056313514709472656`}, {-1.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-15, 11], 0.056313514709472656`}, {-1.3636363636363635`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-14, 11], 0.056313514709472656`}, {-1.2727272727272727`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-13, 11], 0.056313514709472656`}, {-1.1818181818181819`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-12, 11], 0.056313514709472656`}, {-1.0909090909090908`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-10, 11], 0.056313514709472656`}, {-0.9090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-9, 11], 0.056313514709472656`}, {-0.8181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-8, 11], 0.056313514709472656`}, {-0.7272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-7, 11], 0.056313514709472656`}, {-0.6363636363636364, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-6, 11], 0.056313514709472656`}, {-0.5454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-5, 11], 0.056313514709472656`}, {-0.45454545454545453`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-4, 11], 0.056313514709472656`}, {-0.36363636363636365`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-3, 11], 0.056313514709472656`}, {-0.2727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-2, 11], 0.056313514709472656`}, {-0.18181818181818182`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-1, 11], 0.056313514709472656`}, {-0.09090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[1, 11], 0.056313514709472656`}, {
     0.09090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[2, 11], 0.056313514709472656`}, {
     0.18181818181818182`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[3, 11], 0.056313514709472656`}, {
     0.2727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[4, 11], 0.056313514709472656`}, {
     0.36363636363636365`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[5, 11], 0.056313514709472656`}, {
     0.45454545454545453`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[6, 11], 0.056313514709472656`}, {
     0.5454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[7, 11], 0.056313514709472656`}, {
     0.6363636363636364, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[8, 11], 0.056313514709472656`}, {
     0.7272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[9, 11], 0.056313514709472656`}, {
     0.8181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[10, 11], 0.056313514709472656`}, {
     0.9090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[12, 11], 0.056313514709472656`}, {
     1.0909090909090908`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[13, 11], 0.056313514709472656`}, {
     1.1818181818181819`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[14, 11], 0.056313514709472656`}, {
     1.2727272727272727`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[15, 11], 0.056313514709472656`}, {
     1.3636363636363635`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[16, 11], 0.056313514709472656`}, {
     1.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[17, 11], 0.056313514709472656`}, {
     1.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[18, 11], 0.056313514709472656`}, {
     1.6363636363636365`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[19, 11], 0.056313514709472656`}, {
     1.7272727272727273`, 0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[20, 11], 0.056313514709472656`}, {1.8181818181818181`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[21, 11], 0.056313514709472656`}, {
     1.9090909090909092`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[23, 11], 0.056313514709472656`}, {
     2.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[24, 11], 0.056313514709472656`}, {
     2.1818181818181817`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[25, 11], 0.056313514709472656`}, {
     2.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[26, 11], 0.056313514709472656`}, {
     2.3636363636363638`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[27, 11], 0.056313514709472656`}, {
     2.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[28, 11], 0.056313514709472656`}, {
     2.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[29, 11], 0.056313514709472656`}, {
     2.6363636363636362`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[30, 11], 0.056313514709472656`}, {
     2.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[31, 11], 0.056313514709472656`}, {
     2.8181818181818183`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[32, 11], 0.056313514709472656`}, {
     2.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[34, 11], 0.056313514709472656`}, {
     3.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[35, 11], 0.056313514709472656`}, {
     3.1818181818181817`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[36, 11], 0.056313514709472656`}, {
     3.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[37, 11], 0.056313514709472656`}, {
     3.3636363636363638`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[38, 11], 0.056313514709472656`}, {
     3.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[39, 11], 0.056313514709472656`}, {
     3.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[40, 11], 0.056313514709472656`}, {
     3.6363636363636362`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[41, 11], 0.056313514709472656`}, {
     3.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[42, 11], 0.056313514709472656`}, {
     3.8181818181818183`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[43, 11], 0.056313514709472656`}, {
     3.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[45, 11], 0.056313514709472656`}, {
     4.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[46, 11], 0.056313514709472656`}, {
     4.181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[47, 11], 0.056313514709472656`}, {
     4.2727272727272725`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[48, 11], 0.056313514709472656`}, {
     4.363636363636363, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[49, 11], 0.056313514709472656`}, {
     4.454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[50, 11], 0.056313514709472656`}, {
     4.545454545454546, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[51, 11], 0.056313514709472656`}, {
     4.636363636363637, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[52, 11], 0.056313514709472656`}, {
     4.7272727272727275`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[53, 11], 0.056313514709472656`}, {
     4.818181818181818, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[54, 11], 0.056313514709472656`}, {
     4.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-59, 12], 0.04223513603210449}, {-4.916666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-55, 12], 0.04223513603210449}, {-4.583333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-53, 12], 0.04223513603210449}, {-4.416666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-49, 12], 0.04223513603210449}, {-4.083333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-47, 12], 0.04223513603210449}, {-3.9166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-43, 12], 0.04223513603210449}, {-3.5833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-41, 12], 0.04223513603210449}, {-3.4166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-37, 12], 0.04223513603210449}, {-3.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-35, 12], 0.04223513603210449}, {-2.9166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-31, 12], 0.04223513603210449}, {-2.5833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-29, 12], 0.04223513603210449}, {-2.4166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-25, 12], 0.04223513603210449}, {-2.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-23, 12], 0.04223513603210449}, {-1.9166666666666667`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-19, 12], 0.04223513603210449}, {-1.5833333333333333`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-17, 12], 0.04223513603210449}, {-1.4166666666666667`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-13, 12], 0.04223513603210449}, {-1.0833333333333333`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-11, 12], 0.04223513603210449}, {-0.9166666666666666, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-7, 12], 0.04223513603210449}, {-0.5833333333333334, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-5, 12], 0.04223513603210449}, {-0.4166666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-1, 12], 0.04223513603210449}, {-0.08333333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[1, 12], 0.04223513603210449}, {
     0.08333333333333333, 0.04223513603210449}]], 
   PointBox[
    NCache[{Rational[5, 12], 0.04223513603210449}, {0.4166666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[7, 12], 0.04223513603210449}, {
     0.5833333333333334, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[11, 12], 0.04223513603210449}, {
     0.9166666666666666, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[13, 12], 0.04223513603210449}, {
     1.0833333333333333`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[17, 12], 0.04223513603210449}, {
     1.4166666666666667`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[19, 12], 0.04223513603210449}, {
     1.5833333333333333`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[23, 12], 0.04223513603210449}, {
     1.9166666666666667`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[25, 12], 0.04223513603210449}, {
     2.0833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[29, 12], 0.04223513603210449}, {
     2.4166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[31, 12], 0.04223513603210449}, {
     2.5833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[35, 12], 0.04223513603210449}, {
     2.9166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[37, 12], 0.04223513603210449}, {
     3.0833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[41, 12], 0.04223513603210449}, {
     3.4166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[43, 12], 0.04223513603210449}, {
     3.5833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[47, 12], 0.04223513603210449}, {
     3.9166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[49, 12], 0.04223513603210449}, {
     4.083333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[53, 12], 0.04223513603210449}, {
     4.416666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[55, 12], 0.04223513603210449}, {
     4.583333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[59, 12], 0.04223513603210449}, {
     4.916666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-64, 13], 0.03167635202407837}, {-4.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-63, 13], 0.03167635202407837}, {-4.846153846153846, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-62, 13], 0.03167635202407837}, {-4.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-61, 13], 0.03167635202407837}, {-4.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-60, 13], 0.03167635202407837}, {-4.615384615384615, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-59, 13], 0.03167635202407837}, {-4.538461538461538, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-58, 13], 0.03167635202407837}, {-4.461538461538462, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-57, 13], 0.03167635202407837}, {-4.384615384615385, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-56, 13], 0.03167635202407837}, {-4.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-55, 13], 0.03167635202407837}, {-4.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-54, 13], 0.03167635202407837}, {-4.153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-53, 13], 0.03167635202407837}, {-4.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-51, 13], 0.03167635202407837}, {-3.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-50, 13], 0.03167635202407837}, {-3.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-49, 13], 0.03167635202407837}, {-3.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-48, 13], 0.03167635202407837}, {-3.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-47, 13], 0.03167635202407837}, {-3.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-46, 13], 0.03167635202407837}, {-3.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-45, 13], 0.03167635202407837}, {-3.4615384615384617`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-44, 13], 0.03167635202407837}, {-3.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-43, 13], 0.03167635202407837}, {-3.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-42, 13], 0.03167635202407837}, {-3.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-41, 13], 0.03167635202407837}, {-3.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-40, 13], 0.03167635202407837}, {-3.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-38, 13], 0.03167635202407837}, {-2.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-37, 13], 0.03167635202407837}, {-2.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-36, 13], 0.03167635202407837}, {-2.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-35, 13], 0.03167635202407837}, {-2.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-34, 13], 0.03167635202407837}, {-2.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-33, 13], 0.03167635202407837}, {-2.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-32, 13], 0.03167635202407837}, {-2.4615384615384617`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-31, 13], 0.03167635202407837}, {-2.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-30, 13], 0.03167635202407837}, {-2.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-29, 13], 0.03167635202407837}, {-2.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-28, 13], 0.03167635202407837}, {-2.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-27, 13], 0.03167635202407837}, {-2.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-25, 13], 0.03167635202407837}, {-1.9230769230769231`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-24, 13], 0.03167635202407837}, {-1.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-23, 13], 0.03167635202407837}, {-1.7692307692307692`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-22, 13], 0.03167635202407837}, {-1.6923076923076923`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-21, 13], 0.03167635202407837}, {-1.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-20, 13], 0.03167635202407837}, {-1.5384615384615385`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-19, 13], 0.03167635202407837}, {-1.4615384615384615`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-18, 13], 0.03167635202407837}, {-1.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-17, 13], 0.03167635202407837}, {-1.3076923076923077`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-16, 13], 0.03167635202407837}, {-1.2307692307692308`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-15, 13], 0.03167635202407837}, {-1.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-14, 13], 0.03167635202407837}, {-1.0769230769230769`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-12, 13], 0.03167635202407837}, {-0.9230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-11, 13], 0.03167635202407837}, {-0.8461538461538461, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-10, 13], 0.03167635202407837}, {-0.7692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-9, 13], 0.03167635202407837}, {-0.6923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-8, 13], 0.03167635202407837}, {-0.6153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-7, 13], 0.03167635202407837}, {-0.5384615384615384, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-6, 13], 0.03167635202407837}, {-0.46153846153846156`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-5, 13], 0.03167635202407837}, {-0.38461538461538464`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-4, 13], 0.03167635202407837}, {-0.3076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-3, 13], 0.03167635202407837}, {-0.23076923076923078`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-2, 13], 0.03167635202407837}, {-0.15384615384615385`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-1, 13], 0.03167635202407837}, {-0.07692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[1, 13], 0.03167635202407837}, {
     0.07692307692307693, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[2, 13], 0.03167635202407837}, {
     0.15384615384615385`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[3, 13], 0.03167635202407837}, {
     0.23076923076923078`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[4, 13], 0.03167635202407837}, {
     0.3076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[5, 13], 0.03167635202407837}, {
     0.38461538461538464`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[6, 13], 0.03167635202407837}, {
     0.46153846153846156`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[7, 13], 0.03167635202407837}, {
     0.5384615384615384, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[8, 13], 0.03167635202407837}, {
     0.6153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[9, 13], 0.03167635202407837}, {
     0.6923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[10, 13], 0.03167635202407837}, {
     0.7692307692307693, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[11, 13], 0.03167635202407837}, {
     0.8461538461538461, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[12, 13], 0.03167635202407837}, {
     0.9230769230769231, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[14, 13], 0.03167635202407837}, {1.0769230769230769`, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[15, 13], 0.03167635202407837}, {
     1.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[16, 13], 0.03167635202407837}, {
     1.2307692307692308`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[17, 13], 0.03167635202407837}, {
     1.3076923076923077`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[18, 13], 0.03167635202407837}, {
     1.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[19, 13], 0.03167635202407837}, {
     1.4615384615384615`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[20, 13], 0.03167635202407837}, {
     1.5384615384615385`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[21, 13], 0.03167635202407837}, {
     1.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[22, 13], 0.03167635202407837}, {
     1.6923076923076923`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[23, 13], 0.03167635202407837}, {
     1.7692307692307692`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[24, 13], 0.03167635202407837}, {
     1.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[25, 13], 0.03167635202407837}, {
     1.9230769230769231`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[27, 13], 0.03167635202407837}, {
     2.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[28, 13], 0.03167635202407837}, {
     2.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[29, 13], 0.03167635202407837}, {
     2.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[30, 13], 0.03167635202407837}, {
     2.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[31, 13], 0.03167635202407837}, {
     2.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[32, 13], 0.03167635202407837}, {
     2.4615384615384617`, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[33, 13], 0.03167635202407837}, {2.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[34, 13], 0.03167635202407837}, {
     2.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[35, 13], 0.03167635202407837}, {
     2.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[36, 13], 0.03167635202407837}, {
     2.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[37, 13], 0.03167635202407837}, {
     2.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[38, 13], 0.03167635202407837}, {
     2.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[40, 13], 0.03167635202407837}, {
     3.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[41, 13], 0.03167635202407837}, {
     3.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[42, 13], 0.03167635202407837}, {
     3.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[43, 13], 0.03167635202407837}, {
     3.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[44, 13], 0.03167635202407837}, {
     3.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[45, 13], 0.03167635202407837}, {
     3.4615384615384617`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[46, 13], 0.03167635202407837}, {
     3.5384615384615383`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[47, 13], 0.03167635202407837}, {
     3.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[48, 13], 0.03167635202407837}, {
     3.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[49, 13], 0.03167635202407837}, {
     3.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[50, 13], 0.03167635202407837}, {
     3.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[51, 13], 0.03167635202407837}, {
     3.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[53, 13], 0.03167635202407837}, {
     4.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[54, 13], 0.03167635202407837}, {
     4.153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[55, 13], 0.03167635202407837}, {
     4.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[56, 13], 0.03167635202407837}, {
     4.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[57, 13], 0.03167635202407837}, {
     4.384615384615385, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[58, 13], 0.03167635202407837}, {
     4.461538461538462, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[59, 13], 0.03167635202407837}, {
     4.538461538461538, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[60, 13], 0.03167635202407837}, {
     4.615384615384615, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[61, 13], 0.03167635202407837}, {
     4.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[62, 13], 0.03167635202407837}, {
     4.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[63, 13], 0.03167635202407837}, {
     4.846153846153846, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[64, 13], 0.03167635202407837}, {
     4.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-69, 14], 0.023757264018058777`}, {-4.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-67, 14], 0.023757264018058777`}, {-4.785714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-65, 14], 0.023757264018058777`}, {-4.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-61, 14], 0.023757264018058777`}, {-4.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-59, 14], 0.023757264018058777`}, {-4.214285714285714, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-57, 14], 0.023757264018058777`}, {-4.071428571428571, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-55, 14], 0.023757264018058777`}, {-3.9285714285714284`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-53, 14], 0.023757264018058777`}, {-3.7857142857142856`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-51, 14], 0.023757264018058777`}, {-3.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-47, 14], 0.023757264018058777`}, {-3.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-45, 14], 0.023757264018058777`}, {-3.2142857142857144`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-43, 14], 0.023757264018058777`}, {-3.0714285714285716`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-41, 14], 0.023757264018058777`}, {-2.9285714285714284`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-39, 14], 0.023757264018058777`}, {-2.7857142857142856`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-37, 14], 0.023757264018058777`}, {-2.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-33, 14], 0.023757264018058777`}, {-2.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-31, 14], 0.023757264018058777`}, {-2.2142857142857144`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-29, 14], 0.023757264018058777`}, {-2.0714285714285716`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-27, 14], 0.023757264018058777`}, {-1.9285714285714286`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-25, 14], 0.023757264018058777`}, {-1.7857142857142858`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-23, 14], 0.023757264018058777`}, {-1.6428571428571428`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-19, 14], 0.023757264018058777`}, {-1.3571428571428572`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-17, 14], 0.023757264018058777`}, {-1.2142857142857142`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-15, 14], 0.023757264018058777`}, {-1.0714285714285714`, 
     0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[-13, 14], 0.023757264018058777`}, {-0.9285714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-11, 14], 0.023757264018058777`}, {-0.7857142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-9, 14], 0.023757264018058777`}, {-0.6428571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-5, 14], 0.023757264018058777`}, {-0.35714285714285715`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-3, 14], 0.023757264018058777`}, {-0.21428571428571427`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-1, 14], 0.023757264018058777`}, {-0.07142857142857142, 
     0.023757264018058777`}]], 
   PointBox[NCache[{Rational[1, 14], 0.023757264018058777`}, {
     0.07142857142857142, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[3, 14], 0.023757264018058777`}, {
     0.21428571428571427`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[5, 14], 0.023757264018058777`}, {
     0.35714285714285715`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[9, 14], 0.023757264018058777`}, {
     0.6428571428571429, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[11, 14], 0.023757264018058777`}, {
     0.7857142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[13, 14], 0.023757264018058777`}, {
     0.9285714285714286, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[15, 14], 0.023757264018058777`}, {
     1.0714285714285714`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[17, 14], 0.023757264018058777`}, {
     1.2142857142857142`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[19, 14], 0.023757264018058777`}, {
     1.3571428571428572`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[23, 14], 0.023757264018058777`}, {
     1.6428571428571428`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[25, 14], 0.023757264018058777`}, {
     1.7857142857142858`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[27, 14], 0.023757264018058777`}, {
     1.9285714285714286`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[29, 14], 0.023757264018058777`}, {
     2.0714285714285716`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[31, 14], 0.023757264018058777`}, {
     2.2142857142857144`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[33, 14], 0.023757264018058777`}, {
     2.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[37, 14], 0.023757264018058777`}, {
     2.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[39, 14], 0.023757264018058777`}, {
     2.7857142857142856`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[41, 14], 0.023757264018058777`}, {
     2.9285714285714284`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[43, 14], 0.023757264018058777`}, {
     3.0714285714285716`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[45, 14], 0.023757264018058777`}, {
     3.2142857142857144`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[47, 14], 0.023757264018058777`}, {
     3.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[51, 14], 0.023757264018058777`}, {
     3.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[53, 14], 0.023757264018058777`}, {
     3.7857142857142856`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[55, 14], 0.023757264018058777`}, {
     3.9285714285714284`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[57, 14], 0.023757264018058777`}, {
     4.071428571428571, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[59, 14], 0.023757264018058777`}, {
     4.214285714285714, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[61, 14], 0.023757264018058777`}, {
     4.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[65, 14], 0.023757264018058777`}, {
     4.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[67, 14], 0.023757264018058777`}, {
     4.785714285714286, 0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[69, 14], 0.023757264018058777`}, {4.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-74, 15], 0.017817948013544083`}, {-4.933333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-73, 15], 0.017817948013544083`}, {-4.866666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-71, 15], 0.017817948013544083`}, {-4.733333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-68, 15], 0.017817948013544083`}, {-4.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-67, 15], 0.017817948013544083`}, {-4.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-64, 15], 0.017817948013544083`}, {-4.266666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-62, 15], 0.017817948013544083`}, {-4.133333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-61, 15], 0.017817948013544083`}, {-4.066666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-59, 15], 0.017817948013544083`}, {-3.933333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-58, 15], 0.017817948013544083`}, {-3.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-56, 15], 0.017817948013544083`}, {-3.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-53, 15], 0.017817948013544083`}, {-3.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-52, 15], 0.017817948013544083`}, {-3.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-49, 15], 0.017817948013544083`}, {-3.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-47, 15], 0.017817948013544083`}, {-3.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-46, 15], 0.017817948013544083`}, {-3.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-44, 15], 0.017817948013544083`}, {-2.933333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-43, 15], 0.017817948013544083`}, {-2.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-41, 15], 0.017817948013544083`}, {-2.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-38, 15], 0.017817948013544083`}, {-2.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-37, 15], 0.017817948013544083`}, {-2.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-34, 15], 0.017817948013544083`}, {-2.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-32, 15], 0.017817948013544083`}, {-2.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-31, 15], 0.017817948013544083`}, {-2.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-29, 15], 0.017817948013544083`}, {-1.9333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-28, 15], 0.017817948013544083`}, {-1.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-26, 15], 0.017817948013544083`}, {-1.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-23, 15], 0.017817948013544083`}, {-1.5333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-22, 15], 0.017817948013544083`}, {-1.4666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-19, 15], 0.017817948013544083`}, {-1.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-17, 15], 0.017817948013544083`}, {-1.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-16, 15], 0.017817948013544083`}, {-1.0666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-14, 15], 0.017817948013544083`}, {-0.9333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-13, 15], 0.017817948013544083`}, {-0.8666666666666667, 
     0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[-11, 15], 0.017817948013544083`}, {-0.7333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-8, 15], 0.017817948013544083`}, {-0.5333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-7, 15], 0.017817948013544083`}, {-0.4666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-4, 15], 0.017817948013544083`}, {-0.26666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-2, 15], 0.017817948013544083`}, {-0.13333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-1, 15], 0.017817948013544083`}, {-0.06666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{Rational[1, 15], 0.017817948013544083`}, {
     0.06666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[2, 15], 0.017817948013544083`}, {
     0.13333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[4, 15], 0.017817948013544083`}, {
     0.26666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[7, 15], 0.017817948013544083`}, {
     0.4666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[8, 15], 0.017817948013544083`}, {
     0.5333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[11, 15], 0.017817948013544083`}, {
     0.7333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[13, 15], 0.017817948013544083`}, {
     0.8666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[14, 15], 0.017817948013544083`}, {
     0.9333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[16, 15], 0.017817948013544083`}, {
     1.0666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[17, 15], 0.017817948013544083`}, {
     1.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[19, 15], 0.017817948013544083`}, {
     1.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[22, 15], 0.017817948013544083`}, {
     1.4666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[23, 15], 0.017817948013544083`}, {
     1.5333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[26, 15], 0.017817948013544083`}, {
     1.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[28, 15], 0.017817948013544083`}, {
     1.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[29, 15], 0.017817948013544083`}, {
     1.9333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[31, 15], 0.017817948013544083`}, {
     2.066666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[32, 15], 0.017817948013544083`}, {
     2.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[34, 15], 0.017817948013544083`}, {
     2.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[37, 15], 0.017817948013544083`}, {
     2.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[38, 15], 0.017817948013544083`}, {
     2.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[41, 15], 0.017817948013544083`}, {
     2.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[43, 15], 0.017817948013544083`}, {
     2.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[44, 15], 0.017817948013544083`}, {
     2.933333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[46, 15], 0.017817948013544083`}, {
     3.066666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[47, 15], 0.017817948013544083`}, {
     3.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[49, 15], 0.017817948013544083`}, {
     3.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[52, 15], 0.017817948013544083`}, {
     3.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[53, 15], 0.017817948013544083`}, {
     3.533333333333333, 0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[56, 15], 0.017817948013544083`}, {3.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{Rational[58, 15], 0.017817948013544083`}, {
     3.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[59, 15], 0.017817948013544083`}, {
     3.933333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[61, 15], 0.017817948013544083`}, {
     4.066666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[62, 15], 0.017817948013544083`}, {
     4.133333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[64, 15], 0.017817948013544083`}, {
     4.266666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[67, 15], 0.017817948013544083`}, {
     4.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[68, 15], 0.017817948013544083`}, {
     4.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[71, 15], 0.017817948013544083`}, {
     4.733333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[73, 15], 0.017817948013544083`}, {
     4.866666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[74, 15], 0.017817948013544083`}, {
     4.933333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[-79, 16], 0.013363461010158062`}, {-4.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-77, 16], 0.013363461010158062`}, {-4.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-75, 16], 0.013363461010158062`}, {-4.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-73, 16], 0.013363461010158062`}, {-4.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-71, 16], 0.013363461010158062`}, {-4.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-69, 16], 0.013363461010158062`}, {-4.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-67, 16], 0.013363461010158062`}, {-4.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-65, 16], 0.013363461010158062`}, {-4.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-63, 16], 0.013363461010158062`}, {-3.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-61, 16], 0.013363461010158062`}, {-3.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-59, 16], 0.013363461010158062`}, {-3.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-57, 16], 0.013363461010158062`}, {-3.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-55, 16], 0.013363461010158062`}, {-3.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-53, 16], 0.013363461010158062`}, {-3.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-51, 16], 0.013363461010158062`}, {-3.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-49, 16], 0.013363461010158062`}, {-3.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-47, 16], 0.013363461010158062`}, {-2.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-45, 16], 0.013363461010158062`}, {-2.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-43, 16], 0.013363461010158062`}, {-2.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-41, 16], 0.013363461010158062`}, {-2.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-39, 16], 0.013363461010158062`}, {-2.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-37, 16], 0.013363461010158062`}, {-2.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-35, 16], 0.013363461010158062`}, {-2.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-33, 16], 0.013363461010158062`}, {-2.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-31, 16], 0.013363461010158062`}, {-1.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-29, 16], 0.013363461010158062`}, {-1.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-27, 16], 0.013363461010158062`}, {-1.6875, 
     0.013363461010158062`}]], 
   PointBox[
    NCache[{Rational[-25, 16], 0.013363461010158062`}, {-1.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-23, 16], 0.013363461010158062`}, {-1.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-21, 16], 0.013363461010158062`}, {-1.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-19, 16], 0.013363461010158062`}, {-1.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-17, 16], 0.013363461010158062`}, {-1.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-15, 16], 0.013363461010158062`}, {-0.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-13, 16], 0.013363461010158062`}, {-0.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-11, 16], 0.013363461010158062`}, {-0.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-9, 16], 0.013363461010158062`}, {-0.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-7, 16], 0.013363461010158062`}, {-0.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-5, 16], 0.013363461010158062`}, {-0.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-3, 16], 0.013363461010158062`}, {-0.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-1, 16], 0.013363461010158062`}, {-0.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[1, 16], 0.013363461010158062`}, {0.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[3, 16], 0.013363461010158062`}, {0.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[5, 16], 0.013363461010158062`}, {0.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[7, 16], 0.013363461010158062`}, {0.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[9, 16], 0.013363461010158062`}, {0.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[11, 16], 0.013363461010158062`}, {0.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[13, 16], 0.013363461010158062`}, {0.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[15, 16], 0.013363461010158062`}, {0.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[17, 16], 0.013363461010158062`}, {1.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[19, 16], 0.013363461010158062`}, {1.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[21, 16], 0.013363461010158062`}, {1.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[23, 16], 0.013363461010158062`}, {1.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[25, 16], 0.013363461010158062`}, {1.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[27, 16], 0.013363461010158062`}, {1.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[29, 16], 0.013363461010158062`}, {1.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[31, 16], 0.013363461010158062`}, {1.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[33, 16], 0.013363461010158062`}, {2.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[35, 16], 0.013363461010158062`}, {2.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[37, 16], 0.013363461010158062`}, {2.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[39, 16], 0.013363461010158062`}, {2.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[41, 16], 0.013363461010158062`}, {2.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[43, 16], 0.013363461010158062`}, {2.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[45, 16], 0.013363461010158062`}, {2.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[47, 16], 0.013363461010158062`}, {2.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[49, 16], 0.013363461010158062`}, {3.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[51, 16], 0.013363461010158062`}, {3.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[53, 16], 0.013363461010158062`}, {3.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[55, 16], 0.013363461010158062`}, {3.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[57, 16], 0.013363461010158062`}, {3.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[59, 16], 0.013363461010158062`}, {3.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[61, 16], 0.013363461010158062`}, {3.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[63, 16], 0.013363461010158062`}, {3.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[65, 16], 0.013363461010158062`}, {4.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[67, 16], 0.013363461010158062`}, {4.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[69, 16], 0.013363461010158062`}, {4.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[71, 16], 0.013363461010158062`}, {4.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[73, 16], 0.013363461010158062`}, {4.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[75, 16], 0.013363461010158062`}, {4.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[77, 16], 0.013363461010158062`}, {4.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[79, 16], 0.013363461010158062`}, {4.9375, 
     0.013363461010158062`}]], LineBox[{{-4, 1}, {-3, 1}}], 
   LineBox[{{-2, 1}, {-1, 1}}], LineBox[{{0, 1}, {1, 1}}], 
   LineBox[{{2, 1}, {3, 1}}], LineBox[{{4, 1}, {5, 1}}], 
   LineBox[{{0.5, 0.75}, {1, 1}}], LineBox[{{-0.5, 0.75}, {-1, 1}}], 
   LineBox[NCache[{{Rational[-1, 2], 0.75}, {Rational[1, 2], 0.75}}, {{-0.5, 
     0.75}, {0.5, 0.75}}]], LineBox[{{1.5, 0.75}, {2, 1}}], 
   LineBox[{{-1.5, 0.75}, {-2, 1}}], 
   LineBox[NCache[{{Rational[-1, 3], 0.5625}, {
      Rational[1, 3], 0.5625}}, {{-0.3333333333333333, 0.5625}, {
     0.3333333333333333, 0.5625}}]], LineBox[{{2.5, 0.75}, {3, 1}}], 
   LineBox[{{-2.5, 0.75}, {-3, 1}}], 
   LineBox[NCache[{{Rational[-1, 4], 0.421875}, {
      Rational[1, 4], 0.421875}}, {{-0.25, 0.421875}, {0.25, 0.421875}}]], 
   LineBox[{{3.5, 0.75}, {4, 1}}], LineBox[{{-3.5, 0.75}, {-4, 1}}], 
   LineBox[NCache[{{Rational[-1, 5], 0.31640625}, {
      Rational[1, 5], 0.31640625}}, {{-0.2, 0.31640625}, {0.2, 0.31640625}}]],
    LineBox[{{4.5, 0.75}, {5, 1}}], LineBox[{{-4.5, 0.75}, {-5, 1}}], 
   LineBox[NCache[{{Rational[-1, 6], 0.2373046875}, {
      Rational[1, 6], 0.2373046875}}, {{-0.16666666666666666`, 
     0.2373046875}, {0.16666666666666666`, 0.2373046875}}]], 
   LineBox[NCache[{{Rational[1, 3], 0.5625}, {Rational[2, 3], 0.5625}, {
      Rational[3, 2], 0.75}}, {{0.3333333333333333, 0.5625}, {
     0.6666666666666666, 0.5625}, {1.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-1, 3], 0.5625}, {Rational[-2, 3], 0.5625}, {
      Rational[-3, 2], 0.75}}, {{-0.3333333333333333, 
     0.5625}, {-0.6666666666666666, 0.5625}, {-1.5, 0.75}}]], 
   LineBox[NCache[{{Rational[1, 4], 0.421875}, {Rational[3, 4], 0.421875}, {
      Rational[4, 3], 0.5625}}, {{0.25, 0.421875}, {0.75, 0.421875}, {
     1.3333333333333333`, 0.5625}}]], 
   LineBox[NCache[{{Rational[-1, 4], 0.421875}, {Rational[-3, 4], 0.421875}, {
      Rational[-4, 3], 0.5625}}, {{-0.25, 0.421875}, {-0.75, 
     0.421875}, {-1.3333333333333333`, 0.5625}}]], 
   LineBox[NCache[{{Rational[1, 5], 0.31640625}, {
      Rational[4, 5], 0.31640625}, {Rational[5, 4], 0.421875}}, {{0.2, 
     0.31640625}, {0.8, 0.31640625}, {1.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[-1, 5], 0.31640625}, {
      Rational[-4, 5], 0.31640625}, {Rational[-5, 4], 0.421875}}, {{-0.2, 
     0.31640625}, {-0.8, 0.31640625}, {-1.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[1, 6], 0.2373046875}, {
      Rational[5, 6], 0.2373046875}, {Rational[6, 5], 0.31640625}}, {{
     0.16666666666666666`, 0.2373046875}, {0.8333333333333334, 
     0.2373046875}, {1.2, 0.31640625}}]], 
   LineBox[NCache[{{Rational[-1, 6], 0.2373046875}, {
      Rational[-5, 6], 0.2373046875}, {
      Rational[-6, 5], 0.31640625}}, {{-0.16666666666666666`, 
     0.2373046875}, {-0.8333333333333334, 0.2373046875}, {-1.2, 
     0.31640625}}]], 
   LineBox[NCache[{{Rational[4, 3], 0.5625}, {Rational[5, 3], 0.5625}, {
      Rational[5, 2], 0.75}}, {{1.3333333333333333`, 0.5625}, {
     1.6666666666666667`, 0.5625}, {2.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-4, 3], 0.5625}, {Rational[-5, 3], 0.5625}, {
      Rational[-5, 2], 0.75}}, {{-1.3333333333333333`, 
     0.5625}, {-1.6666666666666667`, 0.5625}, {-2.5, 0.75}}]], 
   LineBox[NCache[{{Rational[5, 4], 0.421875}, {Rational[7, 4], 0.421875}, {
      Rational[7, 3], 0.5625}}, {{1.25, 0.421875}, {1.75, 0.421875}, {
     2.3333333333333335`, 0.5625}}]], 
   LineBox[NCache[{{Rational[-5, 4], 0.421875}, {Rational[-7, 4], 0.421875}, {
      Rational[-7, 3], 0.5625}}, {{-1.25, 0.421875}, {-1.75, 
     0.421875}, {-2.3333333333333335`, 0.5625}}]], 
   LineBox[NCache[{{Rational[6, 5], 0.31640625}, {
      Rational[9, 5], 0.31640625}, {Rational[9, 4], 0.421875}}, {{1.2, 
     0.31640625}, {1.8, 0.31640625}, {2.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[-6, 5], 0.31640625}, {
      Rational[-9, 5], 0.31640625}, {Rational[-9, 4], 0.421875}}, {{-1.2, 
     0.31640625}, {-1.8, 0.31640625}, {-2.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[7, 3], 0.5625}, {Rational[8, 3], 0.5625}, {
      Rational[7, 2], 0.75}}, {{2.3333333333333335`, 0.5625}, {
     2.6666666666666665`, 0.5625}, {3.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-7, 3], 0.5625}, {Rational[-8, 3], 0.5625}, {
      Rational[-7, 2], 0.75}}, {{-2.3333333333333335`, 
     0.5625}, {-2.6666666666666665`, 0.5625}, {-3.5, 0.75}}]], 
   LineBox[NCache[{{Rational[9, 4], 0.421875}, {Rational[11, 4], 0.421875}, {
      Rational[10, 3], 0.5625}}, {{2.25, 0.421875}, {2.75, 0.421875}, {
     3.3333333333333335`, 0.5625}}]], 
   LineBox[NCache[{{Rational[-9, 4], 0.421875}, {
      Rational[-11, 4], 0.421875}, {Rational[-10, 3], 0.5625}}, {{-2.25, 
     0.421875}, {-2.75, 0.421875}, {-3.3333333333333335`, 0.5625}}]], 
   LineBox[NCache[{{Rational[10, 3], 0.5625}, {Rational[11, 3], 0.5625}, {
      Rational[9, 2], 0.75}}, {{3.3333333333333335`, 0.5625}, {
     3.6666666666666665`, 0.5625}, {4.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-10, 3], 0.5625}, {Rational[-11, 3], 0.5625}, {
      Rational[-9, 2], 0.75}}, {{-3.3333333333333335`, 
     0.5625}, {-3.6666666666666665`, 0.5625}, {-4.5, 0.75}}]]},
  AspectRatio->NCache[
    Rational[1, 2], 0.5],
  Axes->{True, False},
  AxesOrigin->{0, 0},
  PlotRange->All,
  Prolog->{
    AbsolutePointSize[2], 
    AbsoluteThickness[1]}]], "Output",
 CellChangeTimes->{{3.43715280196875*^9, 3.43715280675*^9}, 
   3.4427473325395*^9}]
}, Open  ]],

Cell["Here is a picture which shows more of this pattern: ", "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CountableQ[9]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{PointBox[{-9, 1}], PointBox[{-8, 1}], PointBox[{-7, 1}], 
   PointBox[{-6, 1}], PointBox[{-5, 1}], PointBox[{-4, 1}], PointBox[{-3, 1}],
    PointBox[{-2, 1}], PointBox[{-1, 1}], PointBox[{0, 1}], PointBox[{1, 1}], 
   PointBox[{2, 1}], PointBox[{3, 1}], PointBox[{4, 1}], PointBox[{5, 1}], 
   PointBox[{6, 1}], PointBox[{7, 1}], PointBox[{8, 1}], PointBox[{9, 1}], 
   PointBox[NCache[{Rational[-17, 2], 0.75}, {-8.5, 0.75}]], 
   PointBox[NCache[{Rational[-15, 2], 0.75}, {-7.5, 0.75}]], 
   PointBox[NCache[{Rational[-13, 2], 0.75}, {-6.5, 0.75}]], 
   PointBox[NCache[{Rational[-11, 2], 0.75}, {-5.5, 0.75}]], 
   PointBox[NCache[{Rational[-9, 2], 0.75}, {-4.5, 0.75}]], 
   PointBox[NCache[{Rational[-7, 2], 0.75}, {-3.5, 0.75}]], 
   PointBox[NCache[{Rational[-5, 2], 0.75}, {-2.5, 0.75}]], 
   PointBox[NCache[{Rational[-3, 2], 0.75}, {-1.5, 0.75}]], 
   PointBox[NCache[{Rational[-1, 2], 0.75}, {-0.5, 0.75}]], 
   PointBox[NCache[{Rational[1, 2], 0.75}, {0.5, 0.75}]], 
   PointBox[NCache[{Rational[3, 2], 0.75}, {1.5, 0.75}]], 
   PointBox[NCache[{Rational[5, 2], 0.75}, {2.5, 0.75}]], 
   PointBox[NCache[{Rational[7, 2], 0.75}, {3.5, 0.75}]], 
   PointBox[NCache[{Rational[9, 2], 0.75}, {4.5, 0.75}]], 
   PointBox[NCache[{Rational[11, 2], 0.75}, {5.5, 0.75}]], 
   PointBox[NCache[{Rational[13, 2], 0.75}, {6.5, 0.75}]], 
   PointBox[NCache[{Rational[15, 2], 0.75}, {7.5, 0.75}]], 
   PointBox[NCache[{Rational[17, 2], 0.75}, {8.5, 0.75}]], 
   PointBox[NCache[{Rational[-26, 3], 0.5625}, {-8.666666666666666, 0.5625}]],
    PointBox[
    NCache[{Rational[-25, 3], 0.5625}, {-8.333333333333334, 0.5625}]], 
   PointBox[NCache[{Rational[-23, 3], 0.5625}, {-7.666666666666667, 0.5625}]],
    PointBox[
    NCache[{Rational[-22, 3], 0.5625}, {-7.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[-20, 3], 0.5625}, {-6.666666666666667, 0.5625}]],
    PointBox[
    NCache[{Rational[-19, 3], 0.5625}, {-6.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[-17, 3], 0.5625}, {-5.666666666666667, 0.5625}]],
    PointBox[
    NCache[{Rational[-16, 3], 0.5625}, {-5.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[-14, 3], 0.5625}, {-4.666666666666667, 0.5625}]],
    PointBox[
    NCache[{Rational[-13, 3], 0.5625}, {-4.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[-11, 3], 0.5625}, {-3.6666666666666665`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-10, 3], 0.5625}, {-3.3333333333333335`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-8, 3], 0.5625}, {-2.6666666666666665`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-7, 3], 0.5625}, {-2.3333333333333335`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-5, 3], 0.5625}, {-1.6666666666666667`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-4, 3], 0.5625}, {-1.3333333333333333`, 
     0.5625}]], 
   PointBox[NCache[{Rational[-2, 3], 0.5625}, {-0.6666666666666666, 0.5625}]],
    PointBox[
    NCache[{Rational[-1, 3], 0.5625}, {-0.3333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[1, 3], 0.5625}, {0.3333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[2, 3], 0.5625}, {0.6666666666666666, 0.5625}]], 
   PointBox[NCache[{Rational[4, 3], 0.5625}, {1.3333333333333333`, 0.5625}]], 
   PointBox[NCache[{Rational[5, 3], 0.5625}, {1.6666666666666667`, 0.5625}]], 
   PointBox[NCache[{Rational[7, 3], 0.5625}, {2.3333333333333335`, 0.5625}]], 
   PointBox[NCache[{Rational[8, 3], 0.5625}, {2.6666666666666665`, 0.5625}]], 
   PointBox[NCache[{Rational[10, 3], 0.5625}, {3.3333333333333335`, 0.5625}]],
    PointBox[
    NCache[{Rational[11, 3], 0.5625}, {3.6666666666666665`, 0.5625}]], 
   PointBox[NCache[{Rational[13, 3], 0.5625}, {4.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[14, 3], 0.5625}, {4.666666666666667, 0.5625}]], 
   PointBox[NCache[{Rational[16, 3], 0.5625}, {5.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[17, 3], 0.5625}, {5.666666666666667, 0.5625}]], 
   PointBox[NCache[{Rational[19, 3], 0.5625}, {6.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[20, 3], 0.5625}, {6.666666666666667, 0.5625}]], 
   PointBox[NCache[{Rational[22, 3], 0.5625}, {7.333333333333333, 0.5625}]], 
   PointBox[NCache[{Rational[23, 3], 0.5625}, {7.666666666666667, 0.5625}]], 
   PointBox[NCache[{Rational[25, 3], 0.5625}, {8.333333333333334, 0.5625}]], 
   PointBox[NCache[{Rational[26, 3], 0.5625}, {8.666666666666666, 0.5625}]], 
   PointBox[NCache[{Rational[-35, 4], 0.421875}, {-8.75, 0.421875}]], 
   PointBox[NCache[{Rational[-33, 4], 0.421875}, {-8.25, 0.421875}]], 
   PointBox[NCache[{Rational[-31, 4], 0.421875}, {-7.75, 0.421875}]], 
   PointBox[NCache[{Rational[-29, 4], 0.421875}, {-7.25, 0.421875}]], 
   PointBox[NCache[{Rational[-27, 4], 0.421875}, {-6.75, 0.421875}]], 
   PointBox[NCache[{Rational[-25, 4], 0.421875}, {-6.25, 0.421875}]], 
   PointBox[NCache[{Rational[-23, 4], 0.421875}, {-5.75, 0.421875}]], 
   PointBox[NCache[{Rational[-21, 4], 0.421875}, {-5.25, 0.421875}]], 
   PointBox[NCache[{Rational[-19, 4], 0.421875}, {-4.75, 0.421875}]], 
   PointBox[NCache[{Rational[-17, 4], 0.421875}, {-4.25, 0.421875}]], 
   PointBox[NCache[{Rational[-15, 4], 0.421875}, {-3.75, 0.421875}]], 
   PointBox[NCache[{Rational[-13, 4], 0.421875}, {-3.25, 0.421875}]], 
   PointBox[NCache[{Rational[-11, 4], 0.421875}, {-2.75, 0.421875}]], 
   PointBox[NCache[{Rational[-9, 4], 0.421875}, {-2.25, 0.421875}]], 
   PointBox[NCache[{Rational[-7, 4], 0.421875}, {-1.75, 0.421875}]], 
   PointBox[NCache[{Rational[-5, 4], 0.421875}, {-1.25, 0.421875}]], 
   PointBox[NCache[{Rational[-3, 4], 0.421875}, {-0.75, 0.421875}]], 
   PointBox[NCache[{Rational[-1, 4], 0.421875}, {-0.25, 0.421875}]], 
   PointBox[NCache[{Rational[1, 4], 0.421875}, {0.25, 0.421875}]], 
   PointBox[NCache[{Rational[3, 4], 0.421875}, {0.75, 0.421875}]], 
   PointBox[NCache[{Rational[5, 4], 0.421875}, {1.25, 0.421875}]], 
   PointBox[NCache[{Rational[7, 4], 0.421875}, {1.75, 0.421875}]], 
   PointBox[NCache[{Rational[9, 4], 0.421875}, {2.25, 0.421875}]], 
   PointBox[NCache[{Rational[11, 4], 0.421875}, {2.75, 0.421875}]], 
   PointBox[NCache[{Rational[13, 4], 0.421875}, {3.25, 0.421875}]], 
   PointBox[NCache[{Rational[15, 4], 0.421875}, {3.75, 0.421875}]], 
   PointBox[NCache[{Rational[17, 4], 0.421875}, {4.25, 0.421875}]], 
   PointBox[NCache[{Rational[19, 4], 0.421875}, {4.75, 0.421875}]], 
   PointBox[NCache[{Rational[21, 4], 0.421875}, {5.25, 0.421875}]], 
   PointBox[NCache[{Rational[23, 4], 0.421875}, {5.75, 0.421875}]], 
   PointBox[NCache[{Rational[25, 4], 0.421875}, {6.25, 0.421875}]], 
   PointBox[NCache[{Rational[27, 4], 0.421875}, {6.75, 0.421875}]], 
   PointBox[NCache[{Rational[29, 4], 0.421875}, {7.25, 0.421875}]], 
   PointBox[NCache[{Rational[31, 4], 0.421875}, {7.75, 0.421875}]], 
   PointBox[NCache[{Rational[33, 4], 0.421875}, {8.25, 0.421875}]], 
   PointBox[NCache[{Rational[35, 4], 0.421875}, {8.75, 0.421875}]], 
   PointBox[NCache[{Rational[-44, 5], 0.31640625}, {-8.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-43, 5], 0.31640625}, {-8.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-42, 5], 0.31640625}, {-8.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-41, 5], 0.31640625}, {-8.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-39, 5], 0.31640625}, {-7.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-38, 5], 0.31640625}, {-7.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-37, 5], 0.31640625}, {-7.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-36, 5], 0.31640625}, {-7.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-34, 5], 0.31640625}, {-6.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-33, 5], 0.31640625}, {-6.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-32, 5], 0.31640625}, {-6.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-31, 5], 0.31640625}, {-6.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-29, 5], 0.31640625}, {-5.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-28, 5], 0.31640625}, {-5.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-27, 5], 0.31640625}, {-5.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-26, 5], 0.31640625}, {-5.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-24, 5], 0.31640625}, {-4.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-23, 5], 0.31640625}, {-4.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-22, 5], 0.31640625}, {-4.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-21, 5], 0.31640625}, {-4.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-19, 5], 0.31640625}, {-3.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-18, 5], 0.31640625}, {-3.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-17, 5], 0.31640625}, {-3.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-16, 5], 0.31640625}, {-3.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-14, 5], 0.31640625}, {-2.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-13, 5], 0.31640625}, {-2.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-12, 5], 0.31640625}, {-2.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-11, 5], 0.31640625}, {-2.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-9, 5], 0.31640625}, {-1.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-8, 5], 0.31640625}, {-1.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-7, 5], 0.31640625}, {-1.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-6, 5], 0.31640625}, {-1.2, 0.31640625}]], 
   PointBox[NCache[{Rational[-4, 5], 0.31640625}, {-0.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-3, 5], 0.31640625}, {-0.6, 0.31640625}]], 
   PointBox[NCache[{Rational[-2, 5], 0.31640625}, {-0.4, 0.31640625}]], 
   PointBox[NCache[{Rational[-1, 5], 0.31640625}, {-0.2, 0.31640625}]], 
   PointBox[NCache[{Rational[1, 5], 0.31640625}, {0.2, 0.31640625}]], 
   PointBox[NCache[{Rational[2, 5], 0.31640625}, {0.4, 0.31640625}]], 
   PointBox[NCache[{Rational[3, 5], 0.31640625}, {0.6, 0.31640625}]], 
   PointBox[NCache[{Rational[4, 5], 0.31640625}, {0.8, 0.31640625}]], 
   PointBox[NCache[{Rational[6, 5], 0.31640625}, {1.2, 0.31640625}]], 
   PointBox[NCache[{Rational[7, 5], 0.31640625}, {1.4, 0.31640625}]], 
   PointBox[NCache[{Rational[8, 5], 0.31640625}, {1.6, 0.31640625}]], 
   PointBox[NCache[{Rational[9, 5], 0.31640625}, {1.8, 0.31640625}]], 
   PointBox[NCache[{Rational[11, 5], 0.31640625}, {2.2, 0.31640625}]], 
   PointBox[NCache[{Rational[12, 5], 0.31640625}, {2.4, 0.31640625}]], 
   PointBox[NCache[{Rational[13, 5], 0.31640625}, {2.6, 0.31640625}]], 
   PointBox[NCache[{Rational[14, 5], 0.31640625}, {2.8, 0.31640625}]], 
   PointBox[NCache[{Rational[16, 5], 0.31640625}, {3.2, 0.31640625}]], 
   PointBox[NCache[{Rational[17, 5], 0.31640625}, {3.4, 0.31640625}]], 
   PointBox[NCache[{Rational[18, 5], 0.31640625}, {3.6, 0.31640625}]], 
   PointBox[NCache[{Rational[19, 5], 0.31640625}, {3.8, 0.31640625}]], 
   PointBox[NCache[{Rational[21, 5], 0.31640625}, {4.2, 0.31640625}]], 
   PointBox[NCache[{Rational[22, 5], 0.31640625}, {4.4, 0.31640625}]], 
   PointBox[NCache[{Rational[23, 5], 0.31640625}, {4.6, 0.31640625}]], 
   PointBox[NCache[{Rational[24, 5], 0.31640625}, {4.8, 0.31640625}]], 
   PointBox[NCache[{Rational[26, 5], 0.31640625}, {5.2, 0.31640625}]], 
   PointBox[NCache[{Rational[27, 5], 0.31640625}, {5.4, 0.31640625}]], 
   PointBox[NCache[{Rational[28, 5], 0.31640625}, {5.6, 0.31640625}]], 
   PointBox[NCache[{Rational[29, 5], 0.31640625}, {5.8, 0.31640625}]], 
   PointBox[NCache[{Rational[31, 5], 0.31640625}, {6.2, 0.31640625}]], 
   PointBox[NCache[{Rational[32, 5], 0.31640625}, {6.4, 0.31640625}]], 
   PointBox[NCache[{Rational[33, 5], 0.31640625}, {6.6, 0.31640625}]], 
   PointBox[NCache[{Rational[34, 5], 0.31640625}, {6.8, 0.31640625}]], 
   PointBox[NCache[{Rational[36, 5], 0.31640625}, {7.2, 0.31640625}]], 
   PointBox[NCache[{Rational[37, 5], 0.31640625}, {7.4, 0.31640625}]], 
   PointBox[NCache[{Rational[38, 5], 0.31640625}, {7.6, 0.31640625}]], 
   PointBox[NCache[{Rational[39, 5], 0.31640625}, {7.8, 0.31640625}]], 
   PointBox[NCache[{Rational[41, 5], 0.31640625}, {8.2, 0.31640625}]], 
   PointBox[NCache[{Rational[42, 5], 0.31640625}, {8.4, 0.31640625}]], 
   PointBox[NCache[{Rational[43, 5], 0.31640625}, {8.6, 0.31640625}]], 
   PointBox[NCache[{Rational[44, 5], 0.31640625}, {8.8, 0.31640625}]], 
   PointBox[NCache[{Rational[-53, 6], 0.2373046875}, {-8.833333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-49, 6], 0.2373046875}, {-8.166666666666666, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-47, 6], 0.2373046875}, {-7.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-43, 6], 0.2373046875}, {-7.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-41, 6], 0.2373046875}, {-6.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-37, 6], 0.2373046875}, {-6.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-35, 6], 0.2373046875}, {-5.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-31, 6], 0.2373046875}, {-5.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-29, 6], 0.2373046875}, {-4.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-25, 6], 0.2373046875}, {-4.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-23, 6], 0.2373046875}, {-3.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-19, 6], 0.2373046875}, {-3.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-17, 6], 0.2373046875}, {-2.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-13, 6], 0.2373046875}, {-2.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-11, 6], 0.2373046875}, {-1.8333333333333333`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-7, 6], 0.2373046875}, {-1.1666666666666667`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-5, 6], 0.2373046875}, {-0.8333333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-1, 6], 0.2373046875}, {-0.16666666666666666`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[1, 6], 0.2373046875}, {0.16666666666666666`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[5, 6], 0.2373046875}, {0.8333333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[7, 6], 0.2373046875}, {1.1666666666666667`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[11, 6], 0.2373046875}, {1.8333333333333333`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[13, 6], 0.2373046875}, {2.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[17, 6], 0.2373046875}, {2.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[19, 6], 0.2373046875}, {3.1666666666666665`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[23, 6], 0.2373046875}, {3.8333333333333335`, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[25, 6], 0.2373046875}, {4.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[29, 6], 0.2373046875}, {4.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[31, 6], 0.2373046875}, {5.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[35, 6], 0.2373046875}, {5.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[37, 6], 0.2373046875}, {6.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[41, 6], 0.2373046875}, {6.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[43, 6], 0.2373046875}, {7.166666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[47, 6], 0.2373046875}, {7.833333333333333, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[49, 6], 0.2373046875}, {8.166666666666666, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[53, 6], 0.2373046875}, {8.833333333333334, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[-62, 7], 0.177978515625}, {-8.857142857142858, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-61, 7], 0.177978515625}, {-8.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-60, 7], 0.177978515625}, {-8.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-59, 7], 0.177978515625}, {-8.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-58, 7], 0.177978515625}, {-8.285714285714286, 
     0.177978515625}]], 
   PointBox[
    NCache[{Rational[-57, 7], 0.177978515625}, {-8.142857142857142, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-55, 7], 0.177978515625}, {-7.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-54, 7], 0.177978515625}, {-7.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-53, 7], 0.177978515625}, {-7.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-52, 7], 0.177978515625}, {-7.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-51, 7], 0.177978515625}, {-7.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-50, 7], 0.177978515625}, {-7.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-48, 7], 0.177978515625}, {-6.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-47, 7], 0.177978515625}, {-6.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-46, 7], 0.177978515625}, {-6.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-45, 7], 0.177978515625}, {-6.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-44, 7], 0.177978515625}, {-6.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-43, 7], 0.177978515625}, {-6.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-41, 7], 0.177978515625}, {-5.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-40, 7], 0.177978515625}, {-5.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-39, 7], 0.177978515625}, {-5.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-38, 7], 0.177978515625}, {-5.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-37, 7], 0.177978515625}, {-5.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-36, 7], 0.177978515625}, {-5.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-34, 7], 0.177978515625}, {-4.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-33, 7], 0.177978515625}, {-4.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-32, 7], 0.177978515625}, {-4.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-31, 7], 0.177978515625}, {-4.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-30, 7], 0.177978515625}, {-4.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-29, 7], 0.177978515625}, {-4.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-27, 7], 0.177978515625}, {-3.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-26, 7], 0.177978515625}, {-3.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-25, 7], 0.177978515625}, {-3.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-24, 7], 0.177978515625}, {-3.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-23, 7], 0.177978515625}, {-3.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-22, 7], 0.177978515625}, {-3.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-20, 7], 0.177978515625}, {-2.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-19, 7], 0.177978515625}, {-2.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-18, 7], 0.177978515625}, {-2.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-17, 7], 0.177978515625}, {-2.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-16, 7], 0.177978515625}, {-2.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-15, 7], 0.177978515625}, {-2.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-13, 7], 0.177978515625}, {-1.8571428571428572`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-12, 7], 0.177978515625}, {-1.7142857142857142`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-11, 7], 0.177978515625}, {-1.5714285714285714`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-10, 7], 0.177978515625}, {-1.4285714285714286`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-9, 7], 0.177978515625}, {-1.2857142857142858`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-8, 7], 0.177978515625}, {-1.1428571428571428`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-6, 7], 0.177978515625}, {-0.8571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-5, 7], 0.177978515625}, {-0.7142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-4, 7], 0.177978515625}, {-0.5714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-3, 7], 0.177978515625}, {-0.42857142857142855`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-2, 7], 0.177978515625}, {-0.2857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-1, 7], 0.177978515625}, {-0.14285714285714285`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[1, 7], 0.177978515625}, {0.14285714285714285`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[2, 7], 0.177978515625}, {0.2857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[3, 7], 0.177978515625}, {0.42857142857142855`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[4, 7], 0.177978515625}, {0.5714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[5, 7], 0.177978515625}, {0.7142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[6, 7], 0.177978515625}, {0.8571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[8, 7], 0.177978515625}, {1.1428571428571428`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[9, 7], 0.177978515625}, {1.2857142857142858`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[10, 7], 0.177978515625}, {1.4285714285714286`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[11, 7], 0.177978515625}, {1.5714285714285714`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[12, 7], 0.177978515625}, {1.7142857142857142`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[13, 7], 0.177978515625}, {1.8571428571428572`, 
     0.177978515625}]], 
   PointBox[
    NCache[{Rational[15, 7], 0.177978515625}, {2.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[16, 7], 0.177978515625}, {2.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[17, 7], 0.177978515625}, {2.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[18, 7], 0.177978515625}, {2.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[19, 7], 0.177978515625}, {2.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[20, 7], 0.177978515625}, {2.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[22, 7], 0.177978515625}, {3.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[23, 7], 0.177978515625}, {3.2857142857142856`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[24, 7], 0.177978515625}, {3.4285714285714284`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[25, 7], 0.177978515625}, {3.5714285714285716`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[26, 7], 0.177978515625}, {3.7142857142857144`, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[27, 7], 0.177978515625}, {3.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[29, 7], 0.177978515625}, {4.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[30, 7], 0.177978515625}, {4.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[31, 7], 0.177978515625}, {4.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[32, 7], 0.177978515625}, {4.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[33, 7], 0.177978515625}, {4.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[34, 7], 0.177978515625}, {4.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[36, 7], 0.177978515625}, {5.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[37, 7], 0.177978515625}, {5.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[38, 7], 0.177978515625}, {5.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[39, 7], 0.177978515625}, {5.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[40, 7], 0.177978515625}, {5.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[41, 7], 0.177978515625}, {5.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[43, 7], 0.177978515625}, {6.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[44, 7], 0.177978515625}, {6.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[45, 7], 0.177978515625}, {6.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[46, 7], 0.177978515625}, {6.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[47, 7], 0.177978515625}, {6.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[48, 7], 0.177978515625}, {6.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[50, 7], 0.177978515625}, {7.142857142857143, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[51, 7], 0.177978515625}, {7.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[52, 7], 0.177978515625}, {7.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[53, 7], 0.177978515625}, {7.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[54, 7], 0.177978515625}, {7.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[55, 7], 0.177978515625}, {7.857142857142857, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[57, 7], 0.177978515625}, {8.142857142857142, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[58, 7], 0.177978515625}, {8.285714285714286, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[59, 7], 0.177978515625}, {8.428571428571429, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[60, 7], 0.177978515625}, {8.571428571428571, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[61, 7], 0.177978515625}, {8.714285714285714, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[62, 7], 0.177978515625}, {8.857142857142858, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[-71, 8], 0.13348388671875}, {-8.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-69, 8], 0.13348388671875}, {-8.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-67, 8], 0.13348388671875}, {-8.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-65, 8], 0.13348388671875}, {-8.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-63, 8], 0.13348388671875}, {-7.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-61, 8], 0.13348388671875}, {-7.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-59, 8], 0.13348388671875}, {-7.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-57, 8], 0.13348388671875}, {-7.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-55, 8], 0.13348388671875}, {-6.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-53, 8], 0.13348388671875}, {-6.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-51, 8], 0.13348388671875}, {-6.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-49, 8], 0.13348388671875}, {-6.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-47, 8], 0.13348388671875}, {-5.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-45, 8], 0.13348388671875}, {-5.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-43, 8], 0.13348388671875}, {-5.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-41, 8], 0.13348388671875}, {-5.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-39, 8], 0.13348388671875}, {-4.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-37, 8], 0.13348388671875}, {-4.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-35, 8], 0.13348388671875}, {-4.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-33, 8], 0.13348388671875}, {-4.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-31, 8], 0.13348388671875}, {-3.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-29, 8], 0.13348388671875}, {-3.625, 
     0.13348388671875}]], 
   PointBox[
    NCache[{Rational[-27, 8], 0.13348388671875}, {-3.375, 0.13348388671875}]],
    PointBox[
    NCache[{Rational[-25, 8], 0.13348388671875}, {-3.125, 0.13348388671875}]],
    PointBox[
    NCache[{Rational[-23, 8], 0.13348388671875}, {-2.875, 0.13348388671875}]],
    PointBox[
    NCache[{Rational[-21, 8], 0.13348388671875}, {-2.625, 0.13348388671875}]],
    PointBox[
    NCache[{Rational[-19, 8], 0.13348388671875}, {-2.375, 0.13348388671875}]],
    PointBox[
    NCache[{Rational[-17, 8], 0.13348388671875}, {-2.125, 0.13348388671875}]],
    PointBox[
    NCache[{Rational[-15, 8], 0.13348388671875}, {-1.875, 0.13348388671875}]],
    PointBox[
    NCache[{Rational[-13, 8], 0.13348388671875}, {-1.625, 0.13348388671875}]],
    PointBox[
    NCache[{Rational[-11, 8], 0.13348388671875}, {-1.375, 0.13348388671875}]],
    PointBox[
    NCache[{Rational[-9, 8], 0.13348388671875}, {-1.125, 0.13348388671875}]], 
   PointBox[NCache[{Rational[-7, 8], 0.13348388671875}, {-0.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-5, 8], 0.13348388671875}, {-0.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-3, 8], 0.13348388671875}, {-0.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-1, 8], 0.13348388671875}, {-0.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[1, 8], 0.13348388671875}, {0.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[3, 8], 0.13348388671875}, {0.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[5, 8], 0.13348388671875}, {0.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[7, 8], 0.13348388671875}, {0.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[9, 8], 0.13348388671875}, {1.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[11, 8], 0.13348388671875}, {1.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[13, 8], 0.13348388671875}, {1.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[15, 8], 0.13348388671875}, {1.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[17, 8], 0.13348388671875}, {2.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[19, 8], 0.13348388671875}, {2.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[21, 8], 0.13348388671875}, {2.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[23, 8], 0.13348388671875}, {2.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[25, 8], 0.13348388671875}, {3.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[27, 8], 0.13348388671875}, {3.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[29, 8], 0.13348388671875}, {3.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[31, 8], 0.13348388671875}, {3.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[33, 8], 0.13348388671875}, {4.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[35, 8], 0.13348388671875}, {4.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[37, 8], 0.13348388671875}, {4.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[39, 8], 0.13348388671875}, {4.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[41, 8], 0.13348388671875}, {5.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[43, 8], 0.13348388671875}, {5.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[45, 8], 0.13348388671875}, {5.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[47, 8], 0.13348388671875}, {5.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[49, 8], 0.13348388671875}, {6.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[51, 8], 0.13348388671875}, {6.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[53, 8], 0.13348388671875}, {6.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[55, 8], 0.13348388671875}, {6.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[57, 8], 0.13348388671875}, {7.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[59, 8], 0.13348388671875}, {7.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[61, 8], 0.13348388671875}, {7.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[63, 8], 0.13348388671875}, {7.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[65, 8], 0.13348388671875}, {8.125, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[67, 8], 0.13348388671875}, {8.375, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[69, 8], 0.13348388671875}, {8.625, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[71, 8], 0.13348388671875}, {8.875, 
     0.13348388671875}]], 
   PointBox[NCache[{Rational[-80, 9], 0.1001129150390625}, {-8.88888888888889,
      0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-79, 9], 0.1001129150390625}, {-8.777777777777779, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-77, 9], 0.1001129150390625}, {-8.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-76, 9], 0.1001129150390625}, {-8.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-74, 9], 0.1001129150390625}, {-8.222222222222221, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[-73, 9], 0.1001129150390625}, {-8.11111111111111,
      0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-71, 9], 0.1001129150390625}, {-7.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-70, 9], 0.1001129150390625}, {-7.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-68, 9], 0.1001129150390625}, {-7.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-67, 9], 0.1001129150390625}, {-7.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-65, 9], 0.1001129150390625}, {-7.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-64, 9], 0.1001129150390625}, {-7.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-62, 9], 0.1001129150390625}, {-6.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-61, 9], 0.1001129150390625}, {-6.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-59, 9], 0.1001129150390625}, {-6.555555555555555, 
     0.1001129150390625}]], 
   PointBox[
    NCache[{Rational[-58, 9], 0.1001129150390625}, {-6.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-56, 9], 0.1001129150390625}, {-6.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-55, 9], 0.1001129150390625}, {-6.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-53, 9], 0.1001129150390625}, {-5.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-52, 9], 0.1001129150390625}, {-5.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-50, 9], 0.1001129150390625}, {-5.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-49, 9], 0.1001129150390625}, {-5.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-47, 9], 0.1001129150390625}, {-5.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-46, 9], 0.1001129150390625}, {-5.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-44, 9], 0.1001129150390625}, {-4.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-43, 9], 0.1001129150390625}, {-4.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-41, 9], 0.1001129150390625}, {-4.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-40, 9], 0.1001129150390625}, {-4.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-38, 9], 0.1001129150390625}, {-4.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-37, 9], 0.1001129150390625}, {-4.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-35, 9], 0.1001129150390625}, {-3.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-34, 9], 0.1001129150390625}, {-3.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-32, 9], 0.1001129150390625}, {-3.5555555555555554`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-31, 9], 0.1001129150390625}, {-3.4444444444444446`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-29, 9], 0.1001129150390625}, {-3.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-28, 9], 0.1001129150390625}, {-3.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-26, 9], 0.1001129150390625}, {-2.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-25, 9], 0.1001129150390625}, {-2.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-23, 9], 0.1001129150390625}, {-2.5555555555555554`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-22, 9], 0.1001129150390625}, {-2.4444444444444446`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-20, 9], 0.1001129150390625}, {-2.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-19, 9], 0.1001129150390625}, {-2.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-17, 9], 0.1001129150390625}, {-1.8888888888888888`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-16, 9], 0.1001129150390625}, {-1.7777777777777777`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-14, 9], 0.1001129150390625}, {-1.5555555555555556`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-13, 9], 0.1001129150390625}, {-1.4444444444444444`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-11, 9], 0.1001129150390625}, {-1.2222222222222223`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-10, 9], 0.1001129150390625}, {-1.1111111111111112`, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-8, 9], 0.1001129150390625}, {-0.8888888888888888, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-7, 9], 0.1001129150390625}, {-0.7777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-5, 9], 0.1001129150390625}, {-0.5555555555555556, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-4, 9], 0.1001129150390625}, {-0.4444444444444444, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-2, 9], 0.1001129150390625}, {-0.2222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{
     Rational[-1, 9], 0.1001129150390625}, {-0.1111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[1, 9], 0.1001129150390625}, {0.1111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[2, 9], 0.1001129150390625}, {0.2222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[4, 9], 0.1001129150390625}, {0.4444444444444444, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[5, 9], 0.1001129150390625}, {0.5555555555555556, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[7, 9], 0.1001129150390625}, {0.7777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[8, 9], 0.1001129150390625}, {0.8888888888888888, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[10, 9], 0.1001129150390625}, {
     1.1111111111111112`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[11, 9], 0.1001129150390625}, {
     1.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[13, 9], 0.1001129150390625}, {
     1.4444444444444444`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[14, 9], 0.1001129150390625}, {
     1.5555555555555556`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[16, 9], 0.1001129150390625}, {
     1.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[17, 9], 0.1001129150390625}, {
     1.8888888888888888`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[19, 9], 0.1001129150390625}, {2.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[20, 9], 0.1001129150390625}, {
     2.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[22, 9], 0.1001129150390625}, {
     2.4444444444444446`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[23, 9], 0.1001129150390625}, {
     2.5555555555555554`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[25, 9], 0.1001129150390625}, {
     2.7777777777777777`, 0.1001129150390625}]], 
   PointBox[
    NCache[{Rational[26, 9], 0.1001129150390625}, {2.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[28, 9], 0.1001129150390625}, {3.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[29, 9], 0.1001129150390625}, {
     3.2222222222222223`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[31, 9], 0.1001129150390625}, {
     3.4444444444444446`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[32, 9], 0.1001129150390625}, {
     3.5555555555555554`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[34, 9], 0.1001129150390625}, {
     3.7777777777777777`, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[35, 9], 0.1001129150390625}, {3.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[37, 9], 0.1001129150390625}, {4.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[38, 9], 0.1001129150390625}, {4.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[40, 9], 0.1001129150390625}, {4.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[41, 9], 0.1001129150390625}, {4.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[43, 9], 0.1001129150390625}, {4.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[44, 9], 0.1001129150390625}, {4.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[46, 9], 0.1001129150390625}, {5.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[47, 9], 0.1001129150390625}, {5.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[49, 9], 0.1001129150390625}, {5.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[50, 9], 0.1001129150390625}, {5.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[52, 9], 0.1001129150390625}, {5.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[53, 9], 0.1001129150390625}, {5.888888888888889, 
     0.1001129150390625}]], 
   PointBox[
    NCache[{Rational[55, 9], 0.1001129150390625}, {6.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[56, 9], 0.1001129150390625}, {6.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[58, 9], 0.1001129150390625}, {6.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[59, 9], 0.1001129150390625}, {6.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[61, 9], 0.1001129150390625}, {6.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[62, 9], 0.1001129150390625}, {6.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[64, 9], 0.1001129150390625}, {7.111111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[65, 9], 0.1001129150390625}, {7.222222222222222, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[67, 9], 0.1001129150390625}, {7.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[68, 9], 0.1001129150390625}, {7.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[70, 9], 0.1001129150390625}, {7.777777777777778, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[71, 9], 0.1001129150390625}, {7.888888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[73, 9], 0.1001129150390625}, {8.11111111111111, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[74, 9], 0.1001129150390625}, {8.222222222222221, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[76, 9], 0.1001129150390625}, {8.444444444444445, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[77, 9], 0.1001129150390625}, {8.555555555555555, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[79, 9], 0.1001129150390625}, {8.777777777777779, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[80, 9], 0.1001129150390625}, {8.88888888888889, 
     0.1001129150390625}]], 
   PointBox[NCache[{Rational[-89, 10], 0.07508468627929688}, {-8.9, 
     0.07508468627929688}]], 
   PointBox[
    NCache[{Rational[-87, 10], 0.07508468627929688}, {-8.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-83, 10], 0.07508468627929688}, {-8.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-81, 10], 0.07508468627929688}, {-8.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-79, 10], 0.07508468627929688}, {-7.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-77, 10], 0.07508468627929688}, {-7.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-73, 10], 0.07508468627929688}, {-7.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-71, 10], 0.07508468627929688}, {-7.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-69, 10], 0.07508468627929688}, {-6.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-67, 10], 0.07508468627929688}, {-6.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-63, 10], 0.07508468627929688}, {-6.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-61, 10], 0.07508468627929688}, {-6.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-59, 10], 0.07508468627929688}, {-5.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-57, 10], 0.07508468627929688}, {-5.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-53, 10], 0.07508468627929688}, {-5.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-51, 10], 0.07508468627929688}, {-5.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-49, 10], 0.07508468627929688}, {-4.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-47, 10], 0.07508468627929688}, {-4.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-43, 10], 0.07508468627929688}, {-4.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-41, 10], 0.07508468627929688}, {-4.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-39, 10], 0.07508468627929688}, {-3.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-37, 10], 0.07508468627929688}, {-3.7, 
     0.07508468627929688}]], 
   PointBox[
    NCache[{Rational[-33, 10], 0.07508468627929688}, {-3.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-31, 10], 0.07508468627929688}, {-3.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-29, 10], 0.07508468627929688}, {-2.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-27, 10], 0.07508468627929688}, {-2.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-23, 10], 0.07508468627929688}, {-2.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-21, 10], 0.07508468627929688}, {-2.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-19, 10], 0.07508468627929688}, {-1.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-17, 10], 0.07508468627929688}, {-1.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-13, 10], 0.07508468627929688}, {-1.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-11, 10], 0.07508468627929688}, {-1.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-9, 10], 0.07508468627929688}, {-0.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-7, 10], 0.07508468627929688}, {-0.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-3, 10], 0.07508468627929688}, {-0.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[-1, 10], 0.07508468627929688}, {-0.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[1, 10], 0.07508468627929688}, {0.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[3, 10], 0.07508468627929688}, {0.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[7, 10], 0.07508468627929688}, {0.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[9, 10], 0.07508468627929688}, {0.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[11, 10], 0.07508468627929688}, {1.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[13, 10], 0.07508468627929688}, {1.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[17, 10], 0.07508468627929688}, {1.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[19, 10], 0.07508468627929688}, {1.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[21, 10], 0.07508468627929688}, {2.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[23, 10], 0.07508468627929688}, {2.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[27, 10], 0.07508468627929688}, {2.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[29, 10], 0.07508468627929688}, {2.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[31, 10], 0.07508468627929688}, {3.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[33, 10], 0.07508468627929688}, {3.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[37, 10], 0.07508468627929688}, {3.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[39, 10], 0.07508468627929688}, {3.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[41, 10], 0.07508468627929688}, {4.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[43, 10], 0.07508468627929688}, {4.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[47, 10], 0.07508468627929688}, {4.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[49, 10], 0.07508468627929688}, {4.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[51, 10], 0.07508468627929688}, {5.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[53, 10], 0.07508468627929688}, {5.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[57, 10], 0.07508468627929688}, {5.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[59, 10], 0.07508468627929688}, {5.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[61, 10], 0.07508468627929688}, {6.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[63, 10], 0.07508468627929688}, {6.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[67, 10], 0.07508468627929688}, {6.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[69, 10], 0.07508468627929688}, {6.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[71, 10], 0.07508468627929688}, {7.1, 
     0.07508468627929688}]], 
   PointBox[
    NCache[{Rational[73, 10], 0.07508468627929688}, {7.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[77, 10], 0.07508468627929688}, {7.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[79, 10], 0.07508468627929688}, {7.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[81, 10], 0.07508468627929688}, {8.1, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[83, 10], 0.07508468627929688}, {8.3, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[87, 10], 0.07508468627929688}, {8.7, 
     0.07508468627929688}]], 
   PointBox[NCache[{Rational[89, 10], 0.07508468627929688}, {8.9, 
     0.07508468627929688}]], 
   PointBox[NCache[{
     Rational[-98, 11], 0.056313514709472656`}, {-8.909090909090908, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-97, 11], 0.056313514709472656`}, {-8.818181818181818, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-96, 11], 0.056313514709472656`}, {-8.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-95, 11], 0.056313514709472656`}, {-8.636363636363637, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-94, 11], 0.056313514709472656`}, {-8.545454545454545, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-93, 11], 0.056313514709472656`}, {-8.454545454545455, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-92, 11], 0.056313514709472656`}, {-8.363636363636363, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-91, 11], 0.056313514709472656`}, {-8.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-90, 11], 0.056313514709472656`}, {-8.181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-89, 11], 0.056313514709472656`}, {-8.090909090909092, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-87, 11], 0.056313514709472656`}, {-7.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-86, 11], 0.056313514709472656`}, {-7.818181818181818, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-85, 11], 0.056313514709472656`}, {-7.7272727272727275`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-84, 11], 0.056313514709472656`}, {-7.636363636363637, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-83, 11], 0.056313514709472656`}, {-7.545454545454546, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-82, 11], 0.056313514709472656`}, {-7.454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-81, 11], 0.056313514709472656`}, {-7.363636363636363, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-80, 11], 0.056313514709472656`}, {-7.2727272727272725`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-79, 11], 0.056313514709472656`}, {-7.181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-78, 11], 0.056313514709472656`}, {-7.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-76, 11], 0.056313514709472656`}, {-6.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-75, 11], 0.056313514709472656`}, {-6.818181818181818, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-74, 11], 0.056313514709472656`}, {-6.7272727272727275`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-73, 11], 0.056313514709472656`}, {-6.636363636363637, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-72, 11], 0.056313514709472656`}, {-6.545454545454546, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-71, 11], 0.056313514709472656`}, {-6.454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-70, 11], 0.056313514709472656`}, {-6.363636363636363, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-69, 11], 0.056313514709472656`}, {-6.2727272727272725`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-68, 11], 0.056313514709472656`}, {-6.181818181818182, 
     0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[-67, 11], 0.056313514709472656`}, {-6.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-65, 11], 0.056313514709472656`}, {-5.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-64, 11], 0.056313514709472656`}, {-5.818181818181818, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-63, 11], 0.056313514709472656`}, {-5.7272727272727275`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-62, 11], 0.056313514709472656`}, {-5.636363636363637, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-61, 11], 0.056313514709472656`}, {-5.545454545454546, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-60, 11], 0.056313514709472656`}, {-5.454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-59, 11], 0.056313514709472656`}, {-5.363636363636363, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-58, 11], 0.056313514709472656`}, {-5.2727272727272725`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-57, 11], 0.056313514709472656`}, {-5.181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-56, 11], 0.056313514709472656`}, {-5.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-54, 11], 0.056313514709472656`}, {-4.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-53, 11], 0.056313514709472656`}, {-4.818181818181818, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-52, 11], 0.056313514709472656`}, {-4.7272727272727275`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-51, 11], 0.056313514709472656`}, {-4.636363636363637, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-50, 11], 0.056313514709472656`}, {-4.545454545454546, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-49, 11], 0.056313514709472656`}, {-4.454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-48, 11], 0.056313514709472656`}, {-4.363636363636363, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-47, 11], 0.056313514709472656`}, {-4.2727272727272725`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-46, 11], 0.056313514709472656`}, {-4.181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-45, 11], 0.056313514709472656`}, {-4.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-43, 11], 0.056313514709472656`}, {-3.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-42, 11], 0.056313514709472656`}, {-3.8181818181818183`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-41, 11], 0.056313514709472656`}, {-3.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-40, 11], 0.056313514709472656`}, {-3.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-39, 11], 0.056313514709472656`}, {-3.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-38, 11], 0.056313514709472656`}, {-3.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-37, 11], 0.056313514709472656`}, {-3.3636363636363638`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-36, 11], 0.056313514709472656`}, {-3.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-35, 11], 0.056313514709472656`}, {-3.1818181818181817`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-34, 11], 0.056313514709472656`}, {-3.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-32, 11], 0.056313514709472656`}, {-2.909090909090909, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-31, 11], 0.056313514709472656`}, {-2.8181818181818183`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-30, 11], 0.056313514709472656`}, {-2.727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-29, 11], 0.056313514709472656`}, {-2.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[-28, 11], 0.056313514709472656`}, {-2.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-27, 11], 0.056313514709472656`}, {-2.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-26, 11], 0.056313514709472656`}, {-2.3636363636363638`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-25, 11], 0.056313514709472656`}, {-2.272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-24, 11], 0.056313514709472656`}, {-2.1818181818181817`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-23, 11], 0.056313514709472656`}, {-2.090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-21, 11], 0.056313514709472656`}, {-1.9090909090909092`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-20, 11], 0.056313514709472656`}, {-1.8181818181818181`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-19, 11], 0.056313514709472656`}, {-1.7272727272727273`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-18, 11], 0.056313514709472656`}, {-1.6363636363636365`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-17, 11], 0.056313514709472656`}, {-1.5454545454545454`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-16, 11], 0.056313514709472656`}, {-1.4545454545454546`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-15, 11], 0.056313514709472656`}, {-1.3636363636363635`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-14, 11], 0.056313514709472656`}, {-1.2727272727272727`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-13, 11], 0.056313514709472656`}, {-1.1818181818181819`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-12, 11], 0.056313514709472656`}, {-1.0909090909090908`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-10, 11], 0.056313514709472656`}, {-0.9090909090909091, 
     0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[-9, 11], 0.056313514709472656`}, {-0.8181818181818182, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-8, 11], 0.056313514709472656`}, {-0.7272727272727273, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-7, 11], 0.056313514709472656`}, {-0.6363636363636364, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-6, 11], 0.056313514709472656`}, {-0.5454545454545454, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-5, 11], 0.056313514709472656`}, {-0.45454545454545453`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-4, 11], 0.056313514709472656`}, {-0.36363636363636365`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-3, 11], 0.056313514709472656`}, {-0.2727272727272727, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-2, 11], 0.056313514709472656`}, {-0.18181818181818182`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-1, 11], 0.056313514709472656`}, {-0.09090909090909091, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[1, 11], 0.056313514709472656`}, {
     0.09090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[2, 11], 0.056313514709472656`}, {
     0.18181818181818182`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[3, 11], 0.056313514709472656`}, {
     0.2727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[4, 11], 0.056313514709472656`}, {
     0.36363636363636365`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[5, 11], 0.056313514709472656`}, {
     0.45454545454545453`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[6, 11], 0.056313514709472656`}, {
     0.5454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[7, 11], 0.056313514709472656`}, {
     0.6363636363636364, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[8, 11], 0.056313514709472656`}, {
     0.7272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[9, 11], 0.056313514709472656`}, {
     0.8181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[10, 11], 0.056313514709472656`}, {
     0.9090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[12, 11], 0.056313514709472656`}, {
     1.0909090909090908`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[13, 11], 0.056313514709472656`}, {
     1.1818181818181819`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[14, 11], 0.056313514709472656`}, {
     1.2727272727272727`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[15, 11], 0.056313514709472656`}, {
     1.3636363636363635`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[16, 11], 0.056313514709472656`}, {
     1.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[17, 11], 0.056313514709472656`}, {
     1.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[18, 11], 0.056313514709472656`}, {
     1.6363636363636365`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[19, 11], 0.056313514709472656`}, {
     1.7272727272727273`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[20, 11], 0.056313514709472656`}, {
     1.8181818181818181`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[21, 11], 0.056313514709472656`}, {
     1.9090909090909092`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[23, 11], 0.056313514709472656`}, {
     2.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[24, 11], 0.056313514709472656`}, {
     2.1818181818181817`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[25, 11], 0.056313514709472656`}, {
     2.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[26, 11], 0.056313514709472656`}, {
     2.3636363636363638`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[27, 11], 0.056313514709472656`}, {
     2.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[28, 11], 0.056313514709472656`}, {
     2.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[
    NCache[{Rational[29, 11], 0.056313514709472656`}, {2.6363636363636362`, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[30, 11], 0.056313514709472656`}, {
     2.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[31, 11], 0.056313514709472656`}, {
     2.8181818181818183`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[32, 11], 0.056313514709472656`}, {
     2.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[34, 11], 0.056313514709472656`}, {
     3.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[35, 11], 0.056313514709472656`}, {
     3.1818181818181817`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[36, 11], 0.056313514709472656`}, {
     3.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[37, 11], 0.056313514709472656`}, {
     3.3636363636363638`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[38, 11], 0.056313514709472656`}, {
     3.4545454545454546`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[39, 11], 0.056313514709472656`}, {
     3.5454545454545454`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[40, 11], 0.056313514709472656`}, {
     3.6363636363636362`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[41, 11], 0.056313514709472656`}, {
     3.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[42, 11], 0.056313514709472656`}, {
     3.8181818181818183`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[43, 11], 0.056313514709472656`}, {
     3.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[45, 11], 0.056313514709472656`}, {
     4.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[46, 11], 0.056313514709472656`}, {
     4.181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[47, 11], 0.056313514709472656`}, {
     4.2727272727272725`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[48, 11], 0.056313514709472656`}, {
     4.363636363636363, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[49, 11], 0.056313514709472656`}, {
     4.454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[50, 11], 0.056313514709472656`}, {
     4.545454545454546, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[51, 11], 0.056313514709472656`}, {
     4.636363636363637, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[52, 11], 0.056313514709472656`}, {
     4.7272727272727275`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[53, 11], 0.056313514709472656`}, {
     4.818181818181818, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[54, 11], 0.056313514709472656`}, {
     4.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[56, 11], 0.056313514709472656`}, {
     5.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[57, 11], 0.056313514709472656`}, {
     5.181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[58, 11], 0.056313514709472656`}, {
     5.2727272727272725`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[59, 11], 0.056313514709472656`}, {
     5.363636363636363, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[60, 11], 0.056313514709472656`}, {
     5.454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[61, 11], 0.056313514709472656`}, {
     5.545454545454546, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[62, 11], 0.056313514709472656`}, {
     5.636363636363637, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[63, 11], 0.056313514709472656`}, {
     5.7272727272727275`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[64, 11], 0.056313514709472656`}, {
     5.818181818181818, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[65, 11], 0.056313514709472656`}, {
     5.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[67, 11], 0.056313514709472656`}, {
     6.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[68, 11], 0.056313514709472656`}, {
     6.181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[69, 11], 0.056313514709472656`}, {
     6.2727272727272725`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[70, 11], 0.056313514709472656`}, {
     6.363636363636363, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[71, 11], 0.056313514709472656`}, {
     6.454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[72, 11], 0.056313514709472656`}, {
     6.545454545454546, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[73, 11], 0.056313514709472656`}, {
     6.636363636363637, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[74, 11], 0.056313514709472656`}, {
     6.7272727272727275`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[75, 11], 0.056313514709472656`}, {
     6.818181818181818, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[76, 11], 0.056313514709472656`}, {
     6.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[78, 11], 0.056313514709472656`}, {
     7.090909090909091, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[79, 11], 0.056313514709472656`}, {
     7.181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[80, 11], 0.056313514709472656`}, {
     7.2727272727272725`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[81, 11], 0.056313514709472656`}, {
     7.363636363636363, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[82, 11], 0.056313514709472656`}, {
     7.454545454545454, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[83, 11], 0.056313514709472656`}, {
     7.545454545454546, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[84, 11], 0.056313514709472656`}, {
     7.636363636363637, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[85, 11], 0.056313514709472656`}, {
     7.7272727272727275`, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[86, 11], 0.056313514709472656`}, {
     7.818181818181818, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[87, 11], 0.056313514709472656`}, {
     7.909090909090909, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[89, 11], 0.056313514709472656`}, {
     8.090909090909092, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[90, 11], 0.056313514709472656`}, {
     8.181818181818182, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[91, 11], 0.056313514709472656`}, {
     8.272727272727273, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[92, 11], 0.056313514709472656`}, {
     8.363636363636363, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[93, 11], 0.056313514709472656`}, {
     8.454545454545455, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[94, 11], 0.056313514709472656`}, {
     8.545454545454545, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[95, 11], 0.056313514709472656`}, {
     8.636363636363637, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[96, 11], 0.056313514709472656`}, {
     8.727272727272727, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[97, 11], 0.056313514709472656`}, {
     8.818181818181818, 0.056313514709472656`}]], 
   PointBox[NCache[{Rational[98, 11], 0.056313514709472656`}, {
     8.909090909090908, 0.056313514709472656`}]], 
   PointBox[NCache[{
     Rational[-107, 12], 0.04223513603210449}, {-8.916666666666666, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-103, 12], 0.04223513603210449}, {-8.583333333333334, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-101, 12], 0.04223513603210449}, {-8.416666666666666, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-97, 12], 0.04223513603210449}, {-8.083333333333334, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-95, 12], 0.04223513603210449}, {-7.916666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-91, 12], 0.04223513603210449}, {-7.583333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-89, 12], 0.04223513603210449}, {-7.416666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-85, 12], 0.04223513603210449}, {-7.083333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-83, 12], 0.04223513603210449}, {-6.916666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-79, 12], 0.04223513603210449}, {-6.583333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-77, 12], 0.04223513603210449}, {-6.416666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-73, 12], 0.04223513603210449}, {-6.083333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-71, 12], 0.04223513603210449}, {-5.916666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-67, 12], 0.04223513603210449}, {-5.583333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-65, 12], 0.04223513603210449}, {-5.416666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-61, 12], 0.04223513603210449}, {-5.083333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-59, 12], 0.04223513603210449}, {-4.916666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-55, 12], 0.04223513603210449}, {-4.583333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-53, 12], 0.04223513603210449}, {-4.416666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-49, 12], 0.04223513603210449}, {-4.083333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-47, 12], 0.04223513603210449}, {-3.9166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-43, 12], 0.04223513603210449}, {-3.5833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-41, 12], 0.04223513603210449}, {-3.4166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-37, 12], 0.04223513603210449}, {-3.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-35, 12], 0.04223513603210449}, {-2.9166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-31, 12], 0.04223513603210449}, {-2.5833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-29, 12], 0.04223513603210449}, {-2.4166666666666665`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-25, 12], 0.04223513603210449}, {-2.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-23, 12], 0.04223513603210449}, {-1.9166666666666667`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-19, 12], 0.04223513603210449}, {-1.5833333333333333`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-17, 12], 0.04223513603210449}, {-1.4166666666666667`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-13, 12], 0.04223513603210449}, {-1.0833333333333333`, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-11, 12], 0.04223513603210449}, {-0.9166666666666666, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-7, 12], 0.04223513603210449}, {-0.5833333333333334, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-5, 12], 0.04223513603210449}, {-0.4166666666666667, 
     0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-1, 12], 0.04223513603210449}, {-0.08333333333333333, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[1, 12], 0.04223513603210449}, {
     0.08333333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[5, 12], 0.04223513603210449}, {
     0.4166666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[7, 12], 0.04223513603210449}, {
     0.5833333333333334, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[11, 12], 0.04223513603210449}, {
     0.9166666666666666, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[13, 12], 0.04223513603210449}, {
     1.0833333333333333`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[17, 12], 0.04223513603210449}, {
     1.4166666666666667`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[19, 12], 0.04223513603210449}, {
     1.5833333333333333`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[23, 12], 0.04223513603210449}, {
     1.9166666666666667`, 0.04223513603210449}]], 
   PointBox[
    NCache[{Rational[25, 12], 0.04223513603210449}, {2.0833333333333335`, 
     0.04223513603210449}]], 
   PointBox[NCache[{Rational[29, 12], 0.04223513603210449}, {
     2.4166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[31, 12], 0.04223513603210449}, {
     2.5833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[35, 12], 0.04223513603210449}, {
     2.9166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[37, 12], 0.04223513603210449}, {
     3.0833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[41, 12], 0.04223513603210449}, {
     3.4166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[43, 12], 0.04223513603210449}, {
     3.5833333333333335`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[47, 12], 0.04223513603210449}, {
     3.9166666666666665`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[49, 12], 0.04223513603210449}, {
     4.083333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[53, 12], 0.04223513603210449}, {
     4.416666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[55, 12], 0.04223513603210449}, {
     4.583333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[59, 12], 0.04223513603210449}, {
     4.916666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[61, 12], 0.04223513603210449}, {
     5.083333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[65, 12], 0.04223513603210449}, {
     5.416666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[67, 12], 0.04223513603210449}, {
     5.583333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[71, 12], 0.04223513603210449}, {
     5.916666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[73, 12], 0.04223513603210449}, {
     6.083333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[77, 12], 0.04223513603210449}, {
     6.416666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[79, 12], 0.04223513603210449}, {
     6.583333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[83, 12], 0.04223513603210449}, {
     6.916666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[85, 12], 0.04223513603210449}, {
     7.083333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[89, 12], 0.04223513603210449}, {
     7.416666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[91, 12], 0.04223513603210449}, {
     7.583333333333333, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[95, 12], 0.04223513603210449}, {
     7.916666666666667, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[97, 12], 0.04223513603210449}, {
     8.083333333333334, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[101, 12], 0.04223513603210449}, {
     8.416666666666666, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[103, 12], 0.04223513603210449}, {
     8.583333333333334, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[107, 12], 0.04223513603210449}, {
     8.916666666666666, 0.04223513603210449}]], 
   PointBox[NCache[{
     Rational[-116, 13], 0.03167635202407837}, {-8.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-115, 13], 0.03167635202407837}, {-8.846153846153847, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-114, 13], 0.03167635202407837}, {-8.76923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-113, 13], 0.03167635202407837}, {-8.692307692307692, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-112, 13], 0.03167635202407837}, {-8.615384615384615, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-111, 13], 0.03167635202407837}, {-8.538461538461538, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-110, 13], 0.03167635202407837}, {-8.461538461538462, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-109, 13], 0.03167635202407837}, {-8.384615384615385, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-108, 13], 0.03167635202407837}, {-8.307692307692308, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-107, 13], 0.03167635202407837}, {-8.23076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-106, 13], 0.03167635202407837}, {-8.153846153846153, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-105, 13], 0.03167635202407837}, {-8.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-103, 13], 0.03167635202407837}, {-7.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-102, 13], 0.03167635202407837}, {-7.846153846153846, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-101, 13], 0.03167635202407837}, {-7.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-100, 13], 0.03167635202407837}, {-7.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-99, 13], 0.03167635202407837}, {-7.615384615384615, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-98, 13], 0.03167635202407837}, {-7.538461538461538, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-97, 13], 0.03167635202407837}, {-7.461538461538462, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-96, 13], 0.03167635202407837}, {-7.384615384615385, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-95, 13], 0.03167635202407837}, {-7.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-94, 13], 0.03167635202407837}, {-7.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-93, 13], 0.03167635202407837}, {-7.153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-92, 13], 0.03167635202407837}, {-7.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-90, 13], 0.03167635202407837}, {-6.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-89, 13], 0.03167635202407837}, {-6.846153846153846, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-88, 13], 0.03167635202407837}, {-6.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-87, 13], 0.03167635202407837}, {-6.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-86, 13], 0.03167635202407837}, {-6.615384615384615, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-85, 13], 0.03167635202407837}, {-6.538461538461538, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-84, 13], 0.03167635202407837}, {-6.461538461538462, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-83, 13], 0.03167635202407837}, {-6.384615384615385, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-82, 13], 0.03167635202407837}, {-6.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-81, 13], 0.03167635202407837}, {-6.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-80, 13], 0.03167635202407837}, {-6.153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-79, 13], 0.03167635202407837}, {-6.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-77, 13], 0.03167635202407837}, {-5.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-76, 13], 0.03167635202407837}, {-5.846153846153846, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-75, 13], 0.03167635202407837}, {-5.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-74, 13], 0.03167635202407837}, {-5.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-73, 13], 0.03167635202407837}, {-5.615384615384615, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-72, 13], 0.03167635202407837}, {-5.538461538461538, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-71, 13], 0.03167635202407837}, {-5.461538461538462, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-70, 13], 0.03167635202407837}, {-5.384615384615385, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-69, 13], 0.03167635202407837}, {-5.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-68, 13], 0.03167635202407837}, {-5.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-67, 13], 0.03167635202407837}, {-5.153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-66, 13], 0.03167635202407837}, {-5.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-64, 13], 0.03167635202407837}, {-4.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-63, 13], 0.03167635202407837}, {-4.846153846153846, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-62, 13], 0.03167635202407837}, {-4.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-61, 13], 0.03167635202407837}, {-4.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-60, 13], 0.03167635202407837}, {-4.615384615384615, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-59, 13], 0.03167635202407837}, {-4.538461538461538, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-58, 13], 0.03167635202407837}, {-4.461538461538462, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-57, 13], 0.03167635202407837}, {-4.384615384615385, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-56, 13], 0.03167635202407837}, {-4.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-55, 13], 0.03167635202407837}, {-4.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-54, 13], 0.03167635202407837}, {-4.153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-53, 13], 0.03167635202407837}, {-4.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-51, 13], 0.03167635202407837}, {-3.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-50, 13], 0.03167635202407837}, {-3.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-49, 13], 0.03167635202407837}, {-3.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-48, 13], 0.03167635202407837}, {-3.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-47, 13], 0.03167635202407837}, {-3.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-46, 13], 0.03167635202407837}, {-3.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-45, 13], 0.03167635202407837}, {-3.4615384615384617`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-44, 13], 0.03167635202407837}, {-3.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-43, 13], 0.03167635202407837}, {-3.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-42, 13], 0.03167635202407837}, {-3.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-41, 13], 0.03167635202407837}, {-3.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-40, 13], 0.03167635202407837}, {-3.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-38, 13], 0.03167635202407837}, {-2.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-37, 13], 0.03167635202407837}, {-2.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-36, 13], 0.03167635202407837}, {-2.769230769230769, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-35, 13], 0.03167635202407837}, {-2.6923076923076925`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-34, 13], 0.03167635202407837}, {-2.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-33, 13], 0.03167635202407837}, {-2.5384615384615383`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-32, 13], 0.03167635202407837}, {-2.4615384615384617`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-31, 13], 0.03167635202407837}, {-2.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-30, 13], 0.03167635202407837}, {-2.3076923076923075`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-29, 13], 0.03167635202407837}, {-2.230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-28, 13], 0.03167635202407837}, {-2.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-27, 13], 0.03167635202407837}, {-2.076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-25, 13], 0.03167635202407837}, {-1.9230769230769231`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-24, 13], 0.03167635202407837}, {-1.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-23, 13], 0.03167635202407837}, {-1.7692307692307692`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-22, 13], 0.03167635202407837}, {-1.6923076923076923`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-21, 13], 0.03167635202407837}, {-1.6153846153846154`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-20, 13], 0.03167635202407837}, {-1.5384615384615385`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-19, 13], 0.03167635202407837}, {-1.4615384615384615`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-18, 13], 0.03167635202407837}, {-1.3846153846153846`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-17, 13], 0.03167635202407837}, {-1.3076923076923077`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-16, 13], 0.03167635202407837}, {-1.2307692307692308`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-15, 13], 0.03167635202407837}, {-1.1538461538461537`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-14, 13], 0.03167635202407837}, {-1.0769230769230769`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-12, 13], 0.03167635202407837}, {-0.9230769230769231, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-11, 13], 0.03167635202407837}, {-0.8461538461538461, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-10, 13], 0.03167635202407837}, {-0.7692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-9, 13], 0.03167635202407837}, {-0.6923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-8, 13], 0.03167635202407837}, {-0.6153846153846154, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-7, 13], 0.03167635202407837}, {-0.5384615384615384, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-6, 13], 0.03167635202407837}, {-0.46153846153846156`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-5, 13], 0.03167635202407837}, {-0.38461538461538464`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-4, 13], 0.03167635202407837}, {-0.3076923076923077, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-3, 13], 0.03167635202407837}, {-0.23076923076923078`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-2, 13], 0.03167635202407837}, {-0.15384615384615385`, 
     0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-1, 13], 0.03167635202407837}, {-0.07692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[1, 13], 0.03167635202407837}, {
     0.07692307692307693, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[2, 13], 0.03167635202407837}, {
     0.15384615384615385`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[3, 13], 0.03167635202407837}, {
     0.23076923076923078`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[4, 13], 0.03167635202407837}, {
     0.3076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[5, 13], 0.03167635202407837}, {
     0.38461538461538464`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[6, 13], 0.03167635202407837}, {
     0.46153846153846156`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[7, 13], 0.03167635202407837}, {
     0.5384615384615384, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[8, 13], 0.03167635202407837}, {
     0.6153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[9, 13], 0.03167635202407837}, {
     0.6923076923076923, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[10, 13], 0.03167635202407837}, {0.7692307692307693, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[11, 13], 0.03167635202407837}, {
     0.8461538461538461, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[12, 13], 0.03167635202407837}, {
     0.9230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[14, 13], 0.03167635202407837}, {
     1.0769230769230769`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[15, 13], 0.03167635202407837}, {
     1.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[16, 13], 0.03167635202407837}, {
     1.2307692307692308`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[17, 13], 0.03167635202407837}, {
     1.3076923076923077`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[18, 13], 0.03167635202407837}, {
     1.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[19, 13], 0.03167635202407837}, {
     1.4615384615384615`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[20, 13], 0.03167635202407837}, {
     1.5384615384615385`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[21, 13], 0.03167635202407837}, {
     1.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[22, 13], 0.03167635202407837}, {
     1.6923076923076923`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[23, 13], 0.03167635202407837}, {
     1.7692307692307692`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[24, 13], 0.03167635202407837}, {
     1.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[25, 13], 0.03167635202407837}, {
     1.9230769230769231`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[27, 13], 0.03167635202407837}, {
     2.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[28, 13], 0.03167635202407837}, {
     2.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[29, 13], 0.03167635202407837}, {
     2.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[30, 13], 0.03167635202407837}, {
     2.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[31, 13], 0.03167635202407837}, {
     2.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[32, 13], 0.03167635202407837}, {
     2.4615384615384617`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[33, 13], 0.03167635202407837}, {
     2.5384615384615383`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[34, 13], 0.03167635202407837}, {
     2.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[35, 13], 0.03167635202407837}, {
     2.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[36, 13], 0.03167635202407837}, {
     2.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[37, 13], 0.03167635202407837}, {
     2.8461538461538463`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[38, 13], 0.03167635202407837}, {
     2.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[40, 13], 0.03167635202407837}, {
     3.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[41, 13], 0.03167635202407837}, {
     3.1538461538461537`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[42, 13], 0.03167635202407837}, {
     3.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[43, 13], 0.03167635202407837}, {
     3.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[44, 13], 0.03167635202407837}, {
     3.3846153846153846`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[45, 13], 0.03167635202407837}, {
     3.4615384615384617`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[46, 13], 0.03167635202407837}, {
     3.5384615384615383`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[47, 13], 0.03167635202407837}, {
     3.6153846153846154`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[48, 13], 0.03167635202407837}, {
     3.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[49, 13], 0.03167635202407837}, {
     3.769230769230769, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[50, 13], 0.03167635202407837}, {3.8461538461538463`, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[51, 13], 0.03167635202407837}, {
     3.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[53, 13], 0.03167635202407837}, {
     4.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[54, 13], 0.03167635202407837}, {
     4.153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[55, 13], 0.03167635202407837}, {
     4.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[56, 13], 0.03167635202407837}, {
     4.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[57, 13], 0.03167635202407837}, {
     4.384615384615385, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[58, 13], 0.03167635202407837}, {
     4.461538461538462, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[59, 13], 0.03167635202407837}, {
     4.538461538461538, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[60, 13], 0.03167635202407837}, {
     4.615384615384615, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[61, 13], 0.03167635202407837}, {
     4.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[62, 13], 0.03167635202407837}, {
     4.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[63, 13], 0.03167635202407837}, {
     4.846153846153846, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[64, 13], 0.03167635202407837}, {
     4.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[66, 13], 0.03167635202407837}, {
     5.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[67, 13], 0.03167635202407837}, {
     5.153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[68, 13], 0.03167635202407837}, {
     5.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[69, 13], 0.03167635202407837}, {
     5.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[70, 13], 0.03167635202407837}, {
     5.384615384615385, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[71, 13], 0.03167635202407837}, {
     5.461538461538462, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[72, 13], 0.03167635202407837}, {
     5.538461538461538, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[73, 13], 0.03167635202407837}, {
     5.615384615384615, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[74, 13], 0.03167635202407837}, {
     5.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[75, 13], 0.03167635202407837}, {
     5.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[76, 13], 0.03167635202407837}, {
     5.846153846153846, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[77, 13], 0.03167635202407837}, {
     5.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[79, 13], 0.03167635202407837}, {
     6.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[80, 13], 0.03167635202407837}, {
     6.153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[81, 13], 0.03167635202407837}, {
     6.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[82, 13], 0.03167635202407837}, {
     6.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[83, 13], 0.03167635202407837}, {
     6.384615384615385, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[84, 13], 0.03167635202407837}, {
     6.461538461538462, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[85, 13], 0.03167635202407837}, {
     6.538461538461538, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[86, 13], 0.03167635202407837}, {
     6.615384615384615, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[87, 13], 0.03167635202407837}, {
     6.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[88, 13], 0.03167635202407837}, {
     6.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[89, 13], 0.03167635202407837}, {
     6.846153846153846, 0.03167635202407837}]], 
   PointBox[
    NCache[{Rational[90, 13], 0.03167635202407837}, {6.923076923076923, 
     0.03167635202407837}]], 
   PointBox[NCache[{Rational[92, 13], 0.03167635202407837}, {
     7.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[93, 13], 0.03167635202407837}, {
     7.153846153846154, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[94, 13], 0.03167635202407837}, {
     7.230769230769231, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[95, 13], 0.03167635202407837}, {
     7.3076923076923075`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[96, 13], 0.03167635202407837}, {
     7.384615384615385, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[97, 13], 0.03167635202407837}, {
     7.461538461538462, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[98, 13], 0.03167635202407837}, {
     7.538461538461538, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[99, 13], 0.03167635202407837}, {
     7.615384615384615, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[100, 13], 0.03167635202407837}, {
     7.6923076923076925`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[101, 13], 0.03167635202407837}, {
     7.769230769230769, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[102, 13], 0.03167635202407837}, {
     7.846153846153846, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[103, 13], 0.03167635202407837}, {
     7.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[105, 13], 0.03167635202407837}, {
     8.076923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[106, 13], 0.03167635202407837}, {
     8.153846153846153, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[107, 13], 0.03167635202407837}, {
     8.23076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[108, 13], 0.03167635202407837}, {
     8.307692307692308, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[109, 13], 0.03167635202407837}, {
     8.384615384615385, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[110, 13], 0.03167635202407837}, {
     8.461538461538462, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[111, 13], 0.03167635202407837}, {
     8.538461538461538, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[112, 13], 0.03167635202407837}, {
     8.615384615384615, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[113, 13], 0.03167635202407837}, {
     8.692307692307692, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[114, 13], 0.03167635202407837}, {
     8.76923076923077, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[115, 13], 0.03167635202407837}, {
     8.846153846153847, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[116, 13], 0.03167635202407837}, {
     8.923076923076923, 0.03167635202407837}]], 
   PointBox[NCache[{
     Rational[-125, 14], 0.023757264018058777`}, {-8.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-123, 14], 0.023757264018058777`}, {-8.785714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-121, 14], 0.023757264018058777`}, {-8.642857142857142, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-117, 14], 0.023757264018058777`}, {-8.357142857142858, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-115, 14], 0.023757264018058777`}, {-8.214285714285714, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-113, 14], 0.023757264018058777`}, {-8.071428571428571, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-111, 14], 0.023757264018058777`}, {-7.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-109, 14], 0.023757264018058777`}, {-7.785714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-107, 14], 0.023757264018058777`}, {-7.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-103, 14], 0.023757264018058777`}, {-7.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-101, 14], 0.023757264018058777`}, {-7.214285714285714, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-99, 14], 0.023757264018058777`}, {-7.071428571428571, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-97, 14], 0.023757264018058777`}, {-6.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-95, 14], 0.023757264018058777`}, {-6.785714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-93, 14], 0.023757264018058777`}, {-6.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-89, 14], 0.023757264018058777`}, {-6.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-87, 14], 0.023757264018058777`}, {-6.214285714285714, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-85, 14], 0.023757264018058777`}, {-6.071428571428571, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-83, 14], 0.023757264018058777`}, {-5.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-81, 14], 0.023757264018058777`}, {-5.785714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-79, 14], 0.023757264018058777`}, {-5.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-75, 14], 0.023757264018058777`}, {-5.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-73, 14], 0.023757264018058777`}, {-5.214285714285714, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-71, 14], 0.023757264018058777`}, {-5.071428571428571, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-69, 14], 0.023757264018058777`}, {-4.928571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-67, 14], 0.023757264018058777`}, {-4.785714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-65, 14], 0.023757264018058777`}, {-4.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-61, 14], 0.023757264018058777`}, {-4.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-59, 14], 0.023757264018058777`}, {-4.214285714285714, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-57, 14], 0.023757264018058777`}, {-4.071428571428571, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-55, 14], 0.023757264018058777`}, {-3.9285714285714284`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-53, 14], 0.023757264018058777`}, {-3.7857142857142856`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-51, 14], 0.023757264018058777`}, {-3.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-47, 14], 0.023757264018058777`}, {-3.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-45, 14], 0.023757264018058777`}, {-3.2142857142857144`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-43, 14], 0.023757264018058777`}, {-3.0714285714285716`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-41, 14], 0.023757264018058777`}, {-2.9285714285714284`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-39, 14], 0.023757264018058777`}, {-2.7857142857142856`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-37, 14], 0.023757264018058777`}, {-2.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-33, 14], 0.023757264018058777`}, {-2.357142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-31, 14], 0.023757264018058777`}, {-2.2142857142857144`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-29, 14], 0.023757264018058777`}, {-2.0714285714285716`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-27, 14], 0.023757264018058777`}, {-1.9285714285714286`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-25, 14], 0.023757264018058777`}, {-1.7857142857142858`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-23, 14], 0.023757264018058777`}, {-1.6428571428571428`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-19, 14], 0.023757264018058777`}, {-1.3571428571428572`, 
     0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[-17, 14], 0.023757264018058777`}, {-1.2142857142857142`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-15, 14], 0.023757264018058777`}, {-1.0714285714285714`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-13, 14], 0.023757264018058777`}, {-0.9285714285714286, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-11, 14], 0.023757264018058777`}, {-0.7857142857142857, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-9, 14], 0.023757264018058777`}, {-0.6428571428571429, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-5, 14], 0.023757264018058777`}, {-0.35714285714285715`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-3, 14], 0.023757264018058777`}, {-0.21428571428571427`, 
     0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-1, 14], 0.023757264018058777`}, {-0.07142857142857142, 
     0.023757264018058777`}]], 
   PointBox[NCache[{Rational[1, 14], 0.023757264018058777`}, {
     0.07142857142857142, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[3, 14], 0.023757264018058777`}, {
     0.21428571428571427`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[5, 14], 0.023757264018058777`}, {
     0.35714285714285715`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[9, 14], 0.023757264018058777`}, {
     0.6428571428571429, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[11, 14], 0.023757264018058777`}, {
     0.7857142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[13, 14], 0.023757264018058777`}, {
     0.9285714285714286, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[15, 14], 0.023757264018058777`}, {
     1.0714285714285714`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[17, 14], 0.023757264018058777`}, {
     1.2142857142857142`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[19, 14], 0.023757264018058777`}, {
     1.3571428571428572`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[23, 14], 0.023757264018058777`}, {
     1.6428571428571428`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[25, 14], 0.023757264018058777`}, {
     1.7857142857142858`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[27, 14], 0.023757264018058777`}, {
     1.9285714285714286`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[29, 14], 0.023757264018058777`}, {
     2.0714285714285716`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[31, 14], 0.023757264018058777`}, {
     2.2142857142857144`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[33, 14], 0.023757264018058777`}, {
     2.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[37, 14], 0.023757264018058777`}, {
     2.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[39, 14], 0.023757264018058777`}, {
     2.7857142857142856`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[41, 14], 0.023757264018058777`}, {
     2.9285714285714284`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[43, 14], 0.023757264018058777`}, {
     3.0714285714285716`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[45, 14], 0.023757264018058777`}, {
     3.2142857142857144`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[47, 14], 0.023757264018058777`}, {
     3.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[51, 14], 0.023757264018058777`}, {
     3.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[53, 14], 0.023757264018058777`}, {
     3.7857142857142856`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[55, 14], 0.023757264018058777`}, {
     3.9285714285714284`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[57, 14], 0.023757264018058777`}, {
     4.071428571428571, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[59, 14], 0.023757264018058777`}, {
     4.214285714285714, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[61, 14], 0.023757264018058777`}, {
     4.357142857142857, 0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[65, 14], 0.023757264018058777`}, {4.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{Rational[67, 14], 0.023757264018058777`}, {
     4.785714285714286, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[69, 14], 0.023757264018058777`}, {
     4.928571428571429, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[71, 14], 0.023757264018058777`}, {
     5.071428571428571, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[73, 14], 0.023757264018058777`}, {
     5.214285714285714, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[75, 14], 0.023757264018058777`}, {
     5.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[79, 14], 0.023757264018058777`}, {
     5.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[81, 14], 0.023757264018058777`}, {
     5.785714285714286, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[83, 14], 0.023757264018058777`}, {
     5.928571428571429, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[85, 14], 0.023757264018058777`}, {
     6.071428571428571, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[87, 14], 0.023757264018058777`}, {
     6.214285714285714, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[89, 14], 0.023757264018058777`}, {
     6.357142857142857, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[93, 14], 0.023757264018058777`}, {
     6.642857142857143, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[95, 14], 0.023757264018058777`}, {
     6.785714285714286, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[97, 14], 0.023757264018058777`}, {
     6.928571428571429, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[99, 14], 0.023757264018058777`}, {
     7.071428571428571, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[101, 14], 0.023757264018058777`}, {
     7.214285714285714, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[103, 14], 0.023757264018058777`}, {
     7.357142857142857, 0.023757264018058777`}]], 
   PointBox[
    NCache[{Rational[107, 14], 0.023757264018058777`}, {7.642857142857143, 
     0.023757264018058777`}]], 
   PointBox[NCache[{Rational[109, 14], 0.023757264018058777`}, {
     7.785714285714286, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[111, 14], 0.023757264018058777`}, {
     7.928571428571429, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[113, 14], 0.023757264018058777`}, {
     8.071428571428571, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[115, 14], 0.023757264018058777`}, {
     8.214285714285714, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[117, 14], 0.023757264018058777`}, {
     8.357142857142858, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[121, 14], 0.023757264018058777`}, {
     8.642857142857142, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[123, 14], 0.023757264018058777`}, {
     8.785714285714286, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[125, 14], 0.023757264018058777`}, {
     8.928571428571429, 0.023757264018058777`}]], 
   PointBox[NCache[{
     Rational[-134, 15], 0.017817948013544083`}, {-8.933333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-133, 15], 0.017817948013544083`}, {-8.866666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-131, 15], 0.017817948013544083`}, {-8.733333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-128, 15], 0.017817948013544083`}, {-8.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-127, 15], 0.017817948013544083`}, {-8.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-124, 15], 0.017817948013544083`}, {-8.266666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-122, 15], 0.017817948013544083`}, {-8.133333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-121, 15], 0.017817948013544083`}, {-8.066666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-119, 15], 0.017817948013544083`}, {-7.933333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-118, 15], 0.017817948013544083`}, {-7.866666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-116, 15], 0.017817948013544083`}, {-7.733333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-113, 15], 0.017817948013544083`}, {-7.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-112, 15], 0.017817948013544083`}, {-7.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-109, 15], 0.017817948013544083`}, {-7.266666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-107, 15], 0.017817948013544083`}, {-7.133333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-106, 15], 0.017817948013544083`}, {-7.066666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-104, 15], 0.017817948013544083`}, {-6.933333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-103, 15], 0.017817948013544083`}, {-6.866666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-101, 15], 0.017817948013544083`}, {-6.733333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-98, 15], 0.017817948013544083`}, {-6.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-97, 15], 0.017817948013544083`}, {-6.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-94, 15], 0.017817948013544083`}, {-6.266666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-92, 15], 0.017817948013544083`}, {-6.133333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-91, 15], 0.017817948013544083`}, {-6.066666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-89, 15], 0.017817948013544083`}, {-5.933333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-88, 15], 0.017817948013544083`}, {-5.866666666666666, 
     0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[-86, 15], 0.017817948013544083`}, {-5.733333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-83, 15], 0.017817948013544083`}, {-5.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-82, 15], 0.017817948013544083`}, {-5.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-79, 15], 0.017817948013544083`}, {-5.266666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-77, 15], 0.017817948013544083`}, {-5.133333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-76, 15], 0.017817948013544083`}, {-5.066666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-74, 15], 0.017817948013544083`}, {-4.933333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-73, 15], 0.017817948013544083`}, {-4.866666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-71, 15], 0.017817948013544083`}, {-4.733333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-68, 15], 0.017817948013544083`}, {-4.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-67, 15], 0.017817948013544083`}, {-4.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-64, 15], 0.017817948013544083`}, {-4.266666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-62, 15], 0.017817948013544083`}, {-4.133333333333334, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-61, 15], 0.017817948013544083`}, {-4.066666666666666, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-59, 15], 0.017817948013544083`}, {-3.933333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-58, 15], 0.017817948013544083`}, {-3.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-56, 15], 0.017817948013544083`}, {-3.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-53, 15], 0.017817948013544083`}, {-3.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-52, 15], 0.017817948013544083`}, {-3.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-49, 15], 0.017817948013544083`}, {-3.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-47, 15], 0.017817948013544083`}, {-3.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-46, 15], 0.017817948013544083`}, {-3.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-44, 15], 0.017817948013544083`}, {-2.933333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-43, 15], 0.017817948013544083`}, {-2.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-41, 15], 0.017817948013544083`}, {-2.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-38, 15], 0.017817948013544083`}, {-2.533333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-37, 15], 0.017817948013544083`}, {-2.466666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-34, 15], 0.017817948013544083`}, {-2.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-32, 15], 0.017817948013544083`}, {-2.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-31, 15], 0.017817948013544083`}, {-2.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-29, 15], 0.017817948013544083`}, {-1.9333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-28, 15], 0.017817948013544083`}, {-1.8666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-26, 15], 0.017817948013544083`}, {-1.7333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-23, 15], 0.017817948013544083`}, {-1.5333333333333334`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-22, 15], 0.017817948013544083`}, {-1.4666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[-19, 15], 0.017817948013544083`}, {-1.2666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-17, 15], 0.017817948013544083`}, {-1.1333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-16, 15], 0.017817948013544083`}, {-1.0666666666666667`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-14, 15], 0.017817948013544083`}, {-0.9333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-13, 15], 0.017817948013544083`}, {-0.8666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-11, 15], 0.017817948013544083`}, {-0.7333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-8, 15], 0.017817948013544083`}, {-0.5333333333333333, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-7, 15], 0.017817948013544083`}, {-0.4666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-4, 15], 0.017817948013544083`}, {-0.26666666666666666`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-2, 15], 0.017817948013544083`}, {-0.13333333333333333`, 
     0.017817948013544083`}]], 
   PointBox[NCache[{
     Rational[-1, 15], 0.017817948013544083`}, {-0.06666666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{Rational[1, 15], 0.017817948013544083`}, {
     0.06666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[2, 15], 0.017817948013544083`}, {
     0.13333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[4, 15], 0.017817948013544083`}, {
     0.26666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[7, 15], 0.017817948013544083`}, {
     0.4666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[8, 15], 0.017817948013544083`}, {
     0.5333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[11, 15], 0.017817948013544083`}, {
     0.7333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[13, 15], 0.017817948013544083`}, {
     0.8666666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[14, 15], 0.017817948013544083`}, {
     0.9333333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[16, 15], 0.017817948013544083`}, {
     1.0666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[17, 15], 0.017817948013544083`}, {
     1.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[19, 15], 0.017817948013544083`}, {
     1.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[22, 15], 0.017817948013544083`}, {
     1.4666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[23, 15], 0.017817948013544083`}, {
     1.5333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[26, 15], 0.017817948013544083`}, {
     1.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[28, 15], 0.017817948013544083`}, {
     1.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[29, 15], 0.017817948013544083`}, {
     1.9333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[31, 15], 0.017817948013544083`}, {
     2.066666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[32, 15], 0.017817948013544083`}, {
     2.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[34, 15], 0.017817948013544083`}, {
     2.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[37, 15], 0.017817948013544083`}, {
     2.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[38, 15], 0.017817948013544083`}, {
     2.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[41, 15], 0.017817948013544083`}, {
     2.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[43, 15], 0.017817948013544083`}, {
     2.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[44, 15], 0.017817948013544083`}, {
     2.933333333333333, 0.017817948013544083`}]], 
   PointBox[
    NCache[{Rational[46, 15], 0.017817948013544083`}, {3.066666666666667, 
     0.017817948013544083`}]], 
   PointBox[NCache[{Rational[47, 15], 0.017817948013544083`}, {
     3.1333333333333333`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[49, 15], 0.017817948013544083`}, {
     3.2666666666666666`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[52, 15], 0.017817948013544083`}, {
     3.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[53, 15], 0.017817948013544083`}, {
     3.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[56, 15], 0.017817948013544083`}, {
     3.7333333333333334`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[58, 15], 0.017817948013544083`}, {
     3.8666666666666667`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[59, 15], 0.017817948013544083`}, {
     3.933333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[61, 15], 0.017817948013544083`}, {
     4.066666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[62, 15], 0.017817948013544083`}, {
     4.133333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[64, 15], 0.017817948013544083`}, {
     4.266666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[67, 15], 0.017817948013544083`}, {
     4.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[68, 15], 0.017817948013544083`}, {
     4.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[71, 15], 0.017817948013544083`}, {
     4.733333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[73, 15], 0.017817948013544083`}, {
     4.866666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[74, 15], 0.017817948013544083`}, {
     4.933333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[76, 15], 0.017817948013544083`}, {
     5.066666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[77, 15], 0.017817948013544083`}, {
     5.133333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[79, 15], 0.017817948013544083`}, {
     5.266666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[82, 15], 0.017817948013544083`}, {
     5.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[83, 15], 0.017817948013544083`}, {
     5.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[86, 15], 0.017817948013544083`}, {
     5.733333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[88, 15], 0.017817948013544083`}, {
     5.866666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[89, 15], 0.017817948013544083`}, {
     5.933333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[91, 15], 0.017817948013544083`}, {
     6.066666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[92, 15], 0.017817948013544083`}, {
     6.133333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[94, 15], 0.017817948013544083`}, {
     6.266666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[97, 15], 0.017817948013544083`}, {
     6.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[98, 15], 0.017817948013544083`}, {
     6.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[101, 15], 0.017817948013544083`}, {
     6.733333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[103, 15], 0.017817948013544083`}, {
     6.866666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[104, 15], 0.017817948013544083`}, {
     6.933333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[106, 15], 0.017817948013544083`}, {
     7.066666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[107, 15], 0.017817948013544083`}, {
     7.133333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[109, 15], 0.017817948013544083`}, {
     7.266666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[112, 15], 0.017817948013544083`}, {
     7.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[113, 15], 0.017817948013544083`}, {
     7.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[116, 15], 0.017817948013544083`}, {
     7.733333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[118, 15], 0.017817948013544083`}, {
     7.866666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[119, 15], 0.017817948013544083`}, {
     7.933333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[121, 15], 0.017817948013544083`}, {
     8.066666666666666, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[122, 15], 0.017817948013544083`}, {
     8.133333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[124, 15], 0.017817948013544083`}, {
     8.266666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[127, 15], 0.017817948013544083`}, {
     8.466666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[128, 15], 0.017817948013544083`}, {
     8.533333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[131, 15], 0.017817948013544083`}, {
     8.733333333333333, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[133, 15], 0.017817948013544083`}, {
     8.866666666666667, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[134, 15], 0.017817948013544083`}, {
     8.933333333333334, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[-143, 16], 0.013363461010158062`}, {-8.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-141, 16], 0.013363461010158062`}, {-8.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-139, 16], 0.013363461010158062`}, {-8.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-137, 16], 0.013363461010158062`}, {-8.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-135, 16], 0.013363461010158062`}, {-8.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-133, 16], 0.013363461010158062`}, {-8.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-131, 16], 0.013363461010158062`}, {-8.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-129, 16], 0.013363461010158062`}, {-8.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-127, 16], 0.013363461010158062`}, {-7.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-125, 16], 0.013363461010158062`}, {-7.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-123, 16], 0.013363461010158062`}, {-7.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-121, 16], 0.013363461010158062`}, {-7.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-119, 16], 0.013363461010158062`}, {-7.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-117, 16], 0.013363461010158062`}, {-7.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-115, 16], 0.013363461010158062`}, {-7.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-113, 16], 0.013363461010158062`}, {-7.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-111, 16], 0.013363461010158062`}, {-6.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-109, 16], 0.013363461010158062`}, {-6.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-107, 16], 0.013363461010158062`}, {-6.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-105, 16], 0.013363461010158062`}, {-6.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-103, 16], 0.013363461010158062`}, {-6.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-101, 16], 0.013363461010158062`}, {-6.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-99, 16], 0.013363461010158062`}, {-6.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-97, 16], 0.013363461010158062`}, {-6.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-95, 16], 0.013363461010158062`}, {-5.9375, 
     0.013363461010158062`}]], 
   PointBox[
    NCache[{Rational[-93, 16], 0.013363461010158062`}, {-5.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-91, 16], 0.013363461010158062`}, {-5.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-89, 16], 0.013363461010158062`}, {-5.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-87, 16], 0.013363461010158062`}, {-5.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-85, 16], 0.013363461010158062`}, {-5.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-83, 16], 0.013363461010158062`}, {-5.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-81, 16], 0.013363461010158062`}, {-5.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-79, 16], 0.013363461010158062`}, {-4.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-77, 16], 0.013363461010158062`}, {-4.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-75, 16], 0.013363461010158062`}, {-4.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-73, 16], 0.013363461010158062`}, {-4.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-71, 16], 0.013363461010158062`}, {-4.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-69, 16], 0.013363461010158062`}, {-4.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-67, 16], 0.013363461010158062`}, {-4.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-65, 16], 0.013363461010158062`}, {-4.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-63, 16], 0.013363461010158062`}, {-3.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-61, 16], 0.013363461010158062`}, {-3.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-59, 16], 0.013363461010158062`}, {-3.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-57, 16], 0.013363461010158062`}, {-3.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-55, 16], 0.013363461010158062`}, {-3.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-53, 16], 0.013363461010158062`}, {-3.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-51, 16], 0.013363461010158062`}, {-3.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-49, 16], 0.013363461010158062`}, {-3.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-47, 16], 0.013363461010158062`}, {-2.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-45, 16], 0.013363461010158062`}, {-2.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-43, 16], 0.013363461010158062`}, {-2.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-41, 16], 0.013363461010158062`}, {-2.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-39, 16], 0.013363461010158062`}, {-2.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-37, 16], 0.013363461010158062`}, {-2.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-35, 16], 0.013363461010158062`}, {-2.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-33, 16], 0.013363461010158062`}, {-2.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-31, 16], 0.013363461010158062`}, {-1.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-29, 16], 0.013363461010158062`}, {-1.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-27, 16], 0.013363461010158062`}, {-1.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-25, 16], 0.013363461010158062`}, {-1.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-23, 16], 0.013363461010158062`}, {-1.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-21, 16], 0.013363461010158062`}, {-1.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-19, 16], 0.013363461010158062`}, {-1.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-17, 16], 0.013363461010158062`}, {-1.0625, 
     0.013363461010158062`}]], 
   PointBox[
    NCache[{Rational[-15, 16], 0.013363461010158062`}, {-0.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-13, 16], 0.013363461010158062`}, {-0.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-11, 16], 0.013363461010158062`}, {-0.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-9, 16], 0.013363461010158062`}, {-0.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-7, 16], 0.013363461010158062`}, {-0.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-5, 16], 0.013363461010158062`}, {-0.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-3, 16], 0.013363461010158062`}, {-0.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[-1, 16], 0.013363461010158062`}, {-0.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[1, 16], 0.013363461010158062`}, {0.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[3, 16], 0.013363461010158062`}, {0.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[5, 16], 0.013363461010158062`}, {0.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[7, 16], 0.013363461010158062`}, {0.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[9, 16], 0.013363461010158062`}, {0.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[11, 16], 0.013363461010158062`}, {0.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[13, 16], 0.013363461010158062`}, {0.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[15, 16], 0.013363461010158062`}, {0.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[17, 16], 0.013363461010158062`}, {1.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[19, 16], 0.013363461010158062`}, {1.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[21, 16], 0.013363461010158062`}, {1.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[23, 16], 0.013363461010158062`}, {1.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[25, 16], 0.013363461010158062`}, {1.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[27, 16], 0.013363461010158062`}, {1.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[29, 16], 0.013363461010158062`}, {1.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[31, 16], 0.013363461010158062`}, {1.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[33, 16], 0.013363461010158062`}, {2.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[35, 16], 0.013363461010158062`}, {2.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[37, 16], 0.013363461010158062`}, {2.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[39, 16], 0.013363461010158062`}, {2.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[41, 16], 0.013363461010158062`}, {2.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[43, 16], 0.013363461010158062`}, {2.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[45, 16], 0.013363461010158062`}, {2.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[47, 16], 0.013363461010158062`}, {2.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[49, 16], 0.013363461010158062`}, {3.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[51, 16], 0.013363461010158062`}, {3.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[53, 16], 0.013363461010158062`}, {3.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[55, 16], 0.013363461010158062`}, {3.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[57, 16], 0.013363461010158062`}, {3.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[59, 16], 0.013363461010158062`}, {3.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[61, 16], 0.013363461010158062`}, {3.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[63, 16], 0.013363461010158062`}, {3.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[65, 16], 0.013363461010158062`}, {4.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[67, 16], 0.013363461010158062`}, {4.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[69, 16], 0.013363461010158062`}, {4.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[71, 16], 0.013363461010158062`}, {4.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[73, 16], 0.013363461010158062`}, {4.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[75, 16], 0.013363461010158062`}, {4.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[77, 16], 0.013363461010158062`}, {4.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[79, 16], 0.013363461010158062`}, {4.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[81, 16], 0.013363461010158062`}, {5.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[83, 16], 0.013363461010158062`}, {5.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[85, 16], 0.013363461010158062`}, {5.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[87, 16], 0.013363461010158062`}, {5.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[89, 16], 0.013363461010158062`}, {5.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[91, 16], 0.013363461010158062`}, {5.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[93, 16], 0.013363461010158062`}, {5.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[95, 16], 0.013363461010158062`}, {5.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[97, 16], 0.013363461010158062`}, {6.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[99, 16], 0.013363461010158062`}, {6.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[101, 16], 0.013363461010158062`}, {6.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[103, 16], 0.013363461010158062`}, {6.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[105, 16], 0.013363461010158062`}, {6.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[107, 16], 0.013363461010158062`}, {6.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[109, 16], 0.013363461010158062`}, {6.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[111, 16], 0.013363461010158062`}, {6.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[113, 16], 0.013363461010158062`}, {7.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[115, 16], 0.013363461010158062`}, {7.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[117, 16], 0.013363461010158062`}, {7.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[119, 16], 0.013363461010158062`}, {7.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[121, 16], 0.013363461010158062`}, {7.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[123, 16], 0.013363461010158062`}, {7.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[125, 16], 0.013363461010158062`}, {7.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[127, 16], 0.013363461010158062`}, {7.9375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[129, 16], 0.013363461010158062`}, {8.0625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[131, 16], 0.013363461010158062`}, {8.1875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[133, 16], 0.013363461010158062`}, {8.3125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[135, 16], 0.013363461010158062`}, {8.4375, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[137, 16], 0.013363461010158062`}, {8.5625, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[139, 16], 0.013363461010158062`}, {8.6875, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[141, 16], 0.013363461010158062`}, {8.8125, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[143, 16], 0.013363461010158062`}, {8.9375, 
     0.013363461010158062`}]], LineBox[{{-8, 1}, {-7, 1}}], 
   LineBox[{{-6, 1}, {-5, 1}}], LineBox[{{-4, 1}, {-3, 1}}], 
   LineBox[{{-2, 1}, {-1, 1}}], LineBox[{{0, 1}, {1, 1}}], 
   LineBox[{{2, 1}, {3, 1}}], LineBox[{{4, 1}, {5, 1}}], 
   LineBox[{{6, 1}, {7, 1}}], LineBox[{{8, 1}, {9, 1}}], 
   LineBox[{{0.5, 0.75}, {1, 1}}], LineBox[{{-0.5, 0.75}, {-1, 1}}], 
   LineBox[NCache[{{Rational[-1, 2], 0.75}, {Rational[1, 2], 0.75}}, {{-0.5, 
     0.75}, {0.5, 0.75}}]], LineBox[{{1.5, 0.75}, {2, 1}}], 
   LineBox[{{-1.5, 0.75}, {-2, 1}}], 
   LineBox[NCache[{{Rational[-1, 3], 0.5625}, {
      Rational[1, 3], 0.5625}}, {{-0.3333333333333333, 0.5625}, {
     0.3333333333333333, 0.5625}}]], LineBox[{{2.5, 0.75}, {3, 1}}], 
   LineBox[{{-2.5, 0.75}, {-3, 1}}], 
   LineBox[NCache[{{Rational[-1, 4], 0.421875}, {
      Rational[1, 4], 0.421875}}, {{-0.25, 0.421875}, {0.25, 0.421875}}]], 
   LineBox[{{3.5, 0.75}, {4, 1}}], LineBox[{{-3.5, 0.75}, {-4, 1}}], 
   LineBox[NCache[{{Rational[-1, 5], 0.31640625}, {
      Rational[1, 5], 0.31640625}}, {{-0.2, 0.31640625}, {0.2, 0.31640625}}]],
    LineBox[{{4.5, 0.75}, {5, 1}}], LineBox[{{-4.5, 0.75}, {-5, 1}}], 
   LineBox[NCache[{{Rational[-1, 6], 0.2373046875}, {
      Rational[1, 6], 0.2373046875}}, {{-0.16666666666666666`, 
     0.2373046875}, {0.16666666666666666`, 0.2373046875}}]], 
   LineBox[{{5.5, 0.75}, {6, 1}}], LineBox[{{-5.5, 0.75}, {-6, 1}}], 
   LineBox[NCache[{{Rational[-1, 7], 0.177978515625}, {
      Rational[1, 7], 0.177978515625}}, {{-0.14285714285714285`, 
     0.177978515625}, {0.14285714285714285`, 0.177978515625}}]], 
   LineBox[{{6.5, 0.75}, {7, 1}}], LineBox[{{-6.5, 0.75}, {-7, 1}}], 
   LineBox[NCache[{{Rational[-1, 8], 0.13348388671875}, {
      Rational[1, 8], 0.13348388671875}}, {{-0.125, 0.13348388671875}, {0.125,
      0.13348388671875}}]], LineBox[{{7.5, 0.75}, {8, 1}}], 
   LineBox[{{-7.5, 0.75}, {-8, 1}}], 
   LineBox[NCache[{{Rational[-1, 9], 0.1001129150390625}, {
      Rational[1, 9], 0.1001129150390625}}, {{-0.1111111111111111, 
     0.1001129150390625}, {0.1111111111111111, 0.1001129150390625}}]], 
   LineBox[{{8.5, 0.75}, {9, 1}}], LineBox[{{-8.5, 0.75}, {-9, 1}}], 
   LineBox[NCache[{{Rational[-1, 10], 0.07508468627929688}, {
      Rational[1, 10], 0.07508468627929688}}, {{-0.1, 0.07508468627929688}, {
     0.1, 0.07508468627929688}}]], 
   LineBox[NCache[{{Rational[1, 3], 0.5625}, {Rational[2, 3], 0.5625}, {
      Rational[3, 2], 0.75}}, {{0.3333333333333333, 0.5625}, {
     0.6666666666666666, 0.5625}, {1.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-1, 3], 0.5625}, {Rational[-2, 3], 0.5625}, {
      Rational[-3, 2], 0.75}}, {{-0.3333333333333333, 
     0.5625}, {-0.6666666666666666, 0.5625}, {-1.5, 0.75}}]], 
   LineBox[NCache[{{Rational[1, 4], 0.421875}, {Rational[3, 4], 0.421875}, {
      Rational[4, 3], 0.5625}}, {{0.25, 0.421875}, {0.75, 0.421875}, {
     1.3333333333333333`, 0.5625}}]], 
   LineBox[NCache[{{Rational[-1, 4], 0.421875}, {Rational[-3, 4], 0.421875}, {
      Rational[-4, 3], 0.5625}}, {{-0.25, 0.421875}, {-0.75, 
     0.421875}, {-1.3333333333333333`, 0.5625}}]], 
   LineBox[NCache[{{Rational[1, 5], 0.31640625}, {
      Rational[4, 5], 0.31640625}, {Rational[5, 4], 0.421875}}, {{0.2, 
     0.31640625}, {0.8, 0.31640625}, {1.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[-1, 5], 0.31640625}, {
      Rational[-4, 5], 0.31640625}, {Rational[-5, 4], 0.421875}}, {{-0.2, 
     0.31640625}, {-0.8, 0.31640625}, {-1.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[1, 6], 0.2373046875}, {
      Rational[5, 6], 0.2373046875}, {Rational[6, 5], 0.31640625}}, {{
     0.16666666666666666`, 0.2373046875}, {0.8333333333333334, 
     0.2373046875}, {1.2, 0.31640625}}]], 
   LineBox[NCache[{{Rational[-1, 6], 0.2373046875}, {
      Rational[-5, 6], 0.2373046875}, {
      Rational[-6, 5], 0.31640625}}, {{-0.16666666666666666`, 
     0.2373046875}, {-0.8333333333333334, 0.2373046875}, {-1.2, 
     0.31640625}}]], 
   LineBox[NCache[{{Rational[1, 7], 0.177978515625}, {
      Rational[6, 7], 0.177978515625}, {Rational[7, 6], 0.2373046875}}, {{
     0.14285714285714285`, 0.177978515625}, {0.8571428571428571, 
     0.177978515625}, {1.1666666666666667`, 0.2373046875}}]], 
   LineBox[NCache[{{Rational[-1, 7], 0.177978515625}, {
      Rational[-6, 7], 0.177978515625}, {
      Rational[-7, 6], 0.2373046875}}, {{-0.14285714285714285`, 
     0.177978515625}, {-0.8571428571428571, 
     0.177978515625}, {-1.1666666666666667`, 0.2373046875}}]], 
   LineBox[NCache[{{Rational[1, 8], 0.13348388671875}, {
      Rational[7, 8], 0.13348388671875}, {Rational[8, 7], 0.177978515625}}, {{
     0.125, 0.13348388671875}, {0.875, 0.13348388671875}, {
     1.1428571428571428`, 0.177978515625}}]], 
   LineBox[NCache[{{Rational[-1, 8], 0.13348388671875}, {
      Rational[-7, 8], 0.13348388671875}, {
      Rational[-8, 7], 0.177978515625}}, {{-0.125, 0.13348388671875}, {-0.875,
      0.13348388671875}, {-1.1428571428571428`, 0.177978515625}}]], 
   LineBox[NCache[{{Rational[1, 9], 0.1001129150390625}, {
      Rational[8, 9], 0.1001129150390625}, {
      Rational[9, 8], 0.13348388671875}}, {{0.1111111111111111, 
     0.1001129150390625}, {0.8888888888888888, 0.1001129150390625}, {1.125, 
     0.13348388671875}}]], 
   LineBox[NCache[{{Rational[-1, 9], 0.1001129150390625}, {
      Rational[-8, 9], 0.1001129150390625}, {
      Rational[-9, 8], 0.13348388671875}}, {{-0.1111111111111111, 
     0.1001129150390625}, {-0.8888888888888888, 0.1001129150390625}, {-1.125, 
     0.13348388671875}}]], 
   LineBox[NCache[{{Rational[1, 10], 0.07508468627929688}, {
      Rational[9, 10], 0.07508468627929688}, {
      Rational[10, 9], 0.1001129150390625}}, {{0.1, 0.07508468627929688}, {
     0.9, 0.07508468627929688}, {1.1111111111111112`, 0.1001129150390625}}]], 
   LineBox[NCache[{{Rational[-1, 10], 0.07508468627929688}, {
      Rational[-9, 10], 0.07508468627929688}, {
      Rational[-10, 9], 0.1001129150390625}}, {{-0.1, 
     0.07508468627929688}, {-0.9, 0.07508468627929688}, {-1.1111111111111112`,
      0.1001129150390625}}]], 
   LineBox[NCache[{{Rational[4, 3], 0.5625}, {Rational[5, 3], 0.5625}, {
      Rational[5, 2], 0.75}}, {{1.3333333333333333`, 0.5625}, {
     1.6666666666666667`, 0.5625}, {2.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-4, 3], 0.5625}, {Rational[-5, 3], 0.5625}, {
      Rational[-5, 2], 0.75}}, {{-1.3333333333333333`, 
     0.5625}, {-1.6666666666666667`, 0.5625}, {-2.5, 0.75}}]], 
   LineBox[NCache[{{Rational[5, 4], 0.421875}, {Rational[7, 4], 0.421875}, {
      Rational[7, 3], 0.5625}}, {{1.25, 0.421875}, {1.75, 0.421875}, {
     2.3333333333333335`, 0.5625}}]], 
   LineBox[NCache[{{Rational[-5, 4], 0.421875}, {Rational[-7, 4], 0.421875}, {
      Rational[-7, 3], 0.5625}}, {{-1.25, 0.421875}, {-1.75, 
     0.421875}, {-2.3333333333333335`, 0.5625}}]], 
   LineBox[NCache[{{Rational[6, 5], 0.31640625}, {
      Rational[9, 5], 0.31640625}, {Rational[9, 4], 0.421875}}, {{1.2, 
     0.31640625}, {1.8, 0.31640625}, {2.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[-6, 5], 0.31640625}, {
      Rational[-9, 5], 0.31640625}, {Rational[-9, 4], 0.421875}}, {{-1.2, 
     0.31640625}, {-1.8, 0.31640625}, {-2.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[7, 6], 0.2373046875}, {
      Rational[11, 6], 0.2373046875}, {Rational[11, 5], 0.31640625}}, {{
     1.1666666666666667`, 0.2373046875}, {1.8333333333333333`, 
     0.2373046875}, {2.2, 0.31640625}}]], 
   LineBox[NCache[{{Rational[-7, 6], 0.2373046875}, {
      Rational[-11, 6], 0.2373046875}, {
      Rational[-11, 5], 0.31640625}}, {{-1.1666666666666667`, 
     0.2373046875}, {-1.8333333333333333`, 0.2373046875}, {-2.2, 
     0.31640625}}]], 
   LineBox[NCache[{{Rational[8, 7], 0.177978515625}, {
      Rational[13, 7], 0.177978515625}, {Rational[13, 6], 0.2373046875}}, {{
     1.1428571428571428`, 0.177978515625}, {1.8571428571428572`, 
     0.177978515625}, {2.1666666666666665`, 0.2373046875}}]], 
   LineBox[NCache[{{Rational[-8, 7], 0.177978515625}, {
      Rational[-13, 7], 0.177978515625}, {
      Rational[-13, 6], 0.2373046875}}, {{-1.1428571428571428`, 
     0.177978515625}, {-1.8571428571428572`, 
     0.177978515625}, {-2.1666666666666665`, 0.2373046875}}]], 
   LineBox[
    NCache[{{Rational[9, 8], 0.13348388671875}, {
      Rational[15, 8], 0.13348388671875}, {
      Rational[15, 7], 0.177978515625}}, {{1.125, 0.13348388671875}, {1.875, 
     0.13348388671875}, {2.142857142857143, 0.177978515625}}]], 
   LineBox[NCache[{{Rational[-9, 8], 0.13348388671875}, {
      Rational[-15, 8], 0.13348388671875}, {
      Rational[-15, 7], 0.177978515625}}, {{-1.125, 
     0.13348388671875}, {-1.875, 0.13348388671875}, {-2.142857142857143, 
     0.177978515625}}]], 
   LineBox[NCache[{{Rational[10, 9], 0.1001129150390625}, {
      Rational[17, 9], 0.1001129150390625}, {
      Rational[17, 8], 0.13348388671875}}, {{1.1111111111111112`, 
     0.1001129150390625}, {1.8888888888888888`, 0.1001129150390625}, {2.125, 
     0.13348388671875}}]], 
   LineBox[NCache[{{Rational[-10, 9], 0.1001129150390625}, {
      Rational[-17, 9], 0.1001129150390625}, {
      Rational[-17, 8], 0.13348388671875}}, {{-1.1111111111111112`, 
     0.1001129150390625}, {-1.8888888888888888`, 0.1001129150390625}, {-2.125,
      0.13348388671875}}]], 
   LineBox[NCache[{{Rational[7, 3], 0.5625}, {Rational[8, 3], 0.5625}, {
      Rational[7, 2], 0.75}}, {{2.3333333333333335`, 0.5625}, {
     2.6666666666666665`, 0.5625}, {3.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-7, 3], 0.5625}, {Rational[-8, 3], 0.5625}, {
      Rational[-7, 2], 0.75}}, {{-2.3333333333333335`, 
     0.5625}, {-2.6666666666666665`, 0.5625}, {-3.5, 0.75}}]], 
   LineBox[NCache[{{Rational[9, 4], 0.421875}, {Rational[11, 4], 0.421875}, {
      Rational[10, 3], 0.5625}}, {{2.25, 0.421875}, {2.75, 0.421875}, {
     3.3333333333333335`, 0.5625}}]], 
   LineBox[NCache[{{Rational[-9, 4], 0.421875}, {
      Rational[-11, 4], 0.421875}, {Rational[-10, 3], 0.5625}}, {{-2.25, 
     0.421875}, {-2.75, 0.421875}, {-3.3333333333333335`, 0.5625}}]], 
   LineBox[NCache[{{Rational[11, 5], 0.31640625}, {
      Rational[14, 5], 0.31640625}, {Rational[13, 4], 0.421875}}, {{2.2, 
     0.31640625}, {2.8, 0.31640625}, {3.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[-11, 5], 0.31640625}, {
      Rational[-14, 5], 0.31640625}, {Rational[-13, 4], 0.421875}}, {{-2.2, 
     0.31640625}, {-2.8, 0.31640625}, {-3.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[13, 6], 0.2373046875}, {
      Rational[17, 6], 0.2373046875}, {Rational[16, 5], 0.31640625}}, {{
     2.1666666666666665`, 0.2373046875}, {2.8333333333333335`, 
     0.2373046875}, {3.2, 0.31640625}}]], 
   LineBox[NCache[{{Rational[-13, 6], 0.2373046875}, {
      Rational[-17, 6], 0.2373046875}, {
      Rational[-16, 5], 0.31640625}}, {{-2.1666666666666665`, 
     0.2373046875}, {-2.8333333333333335`, 0.2373046875}, {-3.2, 
     0.31640625}}]], 
   LineBox[NCache[{{Rational[15, 7], 0.177978515625}, {
      Rational[20, 7], 0.177978515625}, {Rational[19, 6], 0.2373046875}}, {{
     2.142857142857143, 0.177978515625}, {2.857142857142857, 
     0.177978515625}, {3.1666666666666665`, 0.2373046875}}]], 
   LineBox[NCache[{{Rational[-15, 7], 0.177978515625}, {
      Rational[-20, 7], 0.177978515625}, {
      Rational[-19, 6], 0.2373046875}}, {{-2.142857142857143, 
     0.177978515625}, {-2.857142857142857, 
     0.177978515625}, {-3.1666666666666665`, 0.2373046875}}]], 
   LineBox[NCache[{{Rational[17, 8], 0.13348388671875}, {
      Rational[23, 8], 0.13348388671875}, {
      Rational[22, 7], 0.177978515625}}, {{2.125, 0.13348388671875}, {2.875, 
     0.13348388671875}, {3.142857142857143, 0.177978515625}}]], 
   LineBox[NCache[{{Rational[-17, 8], 0.13348388671875}, {
      Rational[-23, 8], 0.13348388671875}, {
      Rational[-22, 7], 0.177978515625}}, {{-2.125, 
     0.13348388671875}, {-2.875, 0.13348388671875}, {-3.142857142857143, 
     0.177978515625}}]], 
   LineBox[NCache[{{Rational[10, 3], 0.5625}, {Rational[11, 3], 0.5625}, {
      Rational[9, 2], 0.75}}, {{3.3333333333333335`, 0.5625}, {
     3.6666666666666665`, 0.5625}, {4.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-10, 3], 0.5625}, {Rational[-11, 3], 0.5625}, {
      Rational[-9, 2], 0.75}}, {{-3.3333333333333335`, 
     0.5625}, {-3.6666666666666665`, 0.5625}, {-4.5, 0.75}}]], 
   LineBox[NCache[{{Rational[13, 4], 0.421875}, {Rational[15, 4], 0.421875}, {
      Rational[13, 3], 0.5625}}, {{3.25, 0.421875}, {3.75, 0.421875}, {
     4.333333333333333, 0.5625}}]], 
   LineBox[NCache[{{Rational[-13, 4], 0.421875}, {
      Rational[-15, 4], 0.421875}, {Rational[-13, 3], 0.5625}}, {{-3.25, 
     0.421875}, {-3.75, 0.421875}, {-4.333333333333333, 0.5625}}]], 
   LineBox[NCache[{{Rational[16, 5], 0.31640625}, {
      Rational[19, 5], 0.31640625}, {Rational[17, 4], 0.421875}}, {{3.2, 
     0.31640625}, {3.8, 0.31640625}, {4.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[-16, 5], 0.31640625}, {
      Rational[-19, 5], 0.31640625}, {Rational[-17, 4], 0.421875}}, {{-3.2, 
     0.31640625}, {-3.8, 0.31640625}, {-4.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[19, 6], 0.2373046875}, {
      Rational[23, 6], 0.2373046875}, {Rational[21, 5], 0.31640625}}, {{
     3.1666666666666665`, 0.2373046875}, {3.8333333333333335`, 
     0.2373046875}, {4.2, 0.31640625}}]], 
   LineBox[NCache[{{Rational[-19, 6], 0.2373046875}, {
      Rational[-23, 6], 0.2373046875}, {
      Rational[-21, 5], 0.31640625}}, {{-3.1666666666666665`, 
     0.2373046875}, {-3.8333333333333335`, 0.2373046875}, {-4.2, 
     0.31640625}}]], 
   LineBox[NCache[{{Rational[22, 7], 0.177978515625}, {
      Rational[27, 7], 0.177978515625}, {Rational[25, 6], 0.2373046875}}, {{
     3.142857142857143, 0.177978515625}, {3.857142857142857, 
     0.177978515625}, {4.166666666666667, 0.2373046875}}]], 
   LineBox[NCache[{{Rational[-22, 7], 0.177978515625}, {
      Rational[-27, 7], 0.177978515625}, {
      Rational[-25, 6], 0.2373046875}}, {{-3.142857142857143, 
     0.177978515625}, {-3.857142857142857, 
     0.177978515625}, {-4.166666666666667, 0.2373046875}}]], 
   LineBox[NCache[{{Rational[13, 3], 0.5625}, {Rational[14, 3], 0.5625}, {
      Rational[11, 2], 0.75}}, {{4.333333333333333, 0.5625}, {
     4.666666666666667, 0.5625}, {5.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-13, 3], 0.5625}, {Rational[-14, 3], 0.5625}, {
      Rational[-11, 2], 0.75}}, {{-4.333333333333333, 
     0.5625}, {-4.666666666666667, 0.5625}, {-5.5, 0.75}}]], 
   LineBox[
    NCache[{{Rational[17, 4], 0.421875}, {Rational[19, 4], 0.421875}, {
      Rational[16, 3], 0.5625}}, {{4.25, 0.421875}, {4.75, 0.421875}, {
     5.333333333333333, 0.5625}}]], 
   LineBox[NCache[{{Rational[-17, 4], 0.421875}, {
      Rational[-19, 4], 0.421875}, {Rational[-16, 3], 0.5625}}, {{-4.25, 
     0.421875}, {-4.75, 0.421875}, {-5.333333333333333, 0.5625}}]], 
   LineBox[NCache[{{Rational[21, 5], 0.31640625}, {
      Rational[24, 5], 0.31640625}, {Rational[21, 4], 0.421875}}, {{4.2, 
     0.31640625}, {4.8, 0.31640625}, {5.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[-21, 5], 0.31640625}, {
      Rational[-24, 5], 0.31640625}, {Rational[-21, 4], 0.421875}}, {{-4.2, 
     0.31640625}, {-4.8, 0.31640625}, {-5.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[25, 6], 0.2373046875}, {
      Rational[29, 6], 0.2373046875}, {Rational[26, 5], 0.31640625}}, {{
     4.166666666666667, 0.2373046875}, {4.833333333333333, 0.2373046875}, {
     5.2, 0.31640625}}]], 
   LineBox[NCache[{{Rational[-25, 6], 0.2373046875}, {
      Rational[-29, 6], 0.2373046875}, {
      Rational[-26, 5], 0.31640625}}, {{-4.166666666666667, 
     0.2373046875}, {-4.833333333333333, 0.2373046875}, {-5.2, 0.31640625}}]],
    LineBox[NCache[{{Rational[16, 3], 0.5625}, {Rational[17, 3], 0.5625}, {
      Rational[13, 2], 0.75}}, {{5.333333333333333, 0.5625}, {
     5.666666666666667, 0.5625}, {6.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-16, 3], 0.5625}, {Rational[-17, 3], 0.5625}, {
      Rational[-13, 2], 0.75}}, {{-5.333333333333333, 
     0.5625}, {-5.666666666666667, 0.5625}, {-6.5, 0.75}}]], 
   LineBox[NCache[{{Rational[21, 4], 0.421875}, {Rational[23, 4], 0.421875}, {
      Rational[19, 3], 0.5625}}, {{5.25, 0.421875}, {5.75, 0.421875}, {
     6.333333333333333, 0.5625}}]], 
   LineBox[NCache[{{Rational[-21, 4], 0.421875}, {
      Rational[-23, 4], 0.421875}, {Rational[-19, 3], 0.5625}}, {{-5.25, 
     0.421875}, {-5.75, 0.421875}, {-6.333333333333333, 0.5625}}]], 
   LineBox[NCache[{{Rational[26, 5], 0.31640625}, {
      Rational[29, 5], 0.31640625}, {Rational[25, 4], 0.421875}}, {{5.2, 
     0.31640625}, {5.8, 0.31640625}, {6.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[-26, 5], 0.31640625}, {
      Rational[-29, 5], 0.31640625}, {Rational[-25, 4], 0.421875}}, {{-5.2, 
     0.31640625}, {-5.8, 0.31640625}, {-6.25, 0.421875}}]], 
   LineBox[NCache[{{Rational[19, 3], 0.5625}, {Rational[20, 3], 0.5625}, {
      Rational[15, 2], 0.75}}, {{6.333333333333333, 0.5625}, {
     6.666666666666667, 0.5625}, {7.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-19, 3], 0.5625}, {Rational[-20, 3], 0.5625}, {
      Rational[-15, 2], 0.75}}, {{-6.333333333333333, 
     0.5625}, {-6.666666666666667, 0.5625}, {-7.5, 0.75}}]], 
   LineBox[NCache[{{Rational[25, 4], 0.421875}, {Rational[27, 4], 0.421875}, {
      Rational[22, 3], 0.5625}}, {{6.25, 0.421875}, {6.75, 0.421875}, {
     7.333333333333333, 0.5625}}]], 
   LineBox[NCache[{{Rational[-25, 4], 0.421875}, {
      Rational[-27, 4], 0.421875}, {Rational[-22, 3], 0.5625}}, {{-6.25, 
     0.421875}, {-6.75, 0.421875}, {-7.333333333333333, 0.5625}}]], 
   LineBox[NCache[{{Rational[22, 3], 0.5625}, {Rational[23, 3], 0.5625}, {
      Rational[17, 2], 0.75}}, {{7.333333333333333, 0.5625}, {
     7.666666666666667, 0.5625}, {8.5, 0.75}}]], 
   LineBox[NCache[{{Rational[-22, 3], 0.5625}, {Rational[-23, 3], 0.5625}, {
      Rational[-17, 2], 0.75}}, {{-7.333333333333333, 
     0.5625}, {-7.666666666666667, 0.5625}, {-8.5, 0.75}}]]},
  AspectRatio->NCache[
    Rational[1, 2], 0.5],
  Axes->{True, False},
  AxesOrigin->{0, 0},
  PlotRange->All,
  Prolog->{
    AbsolutePointSize[2], 
    AbsoluteThickness[1]}]], "Output",
 CellChangeTimes->{3.442747336102*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "This path starts at 0, and swings back and forth, each time hitting the \
rationals on the next row.  Since there are an infinite number of rows, we \
can extend this pattern indefinitely, and every rational number will \
eventually be hit by this path.  This path gives rise the the sequence \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", " ", "1", ",", " ", 
      RowBox[{"1", "/", "2"}], ",", " ", 
      RowBox[{
       RowBox[{"-", "1"}], "/", "2"}], ",", " ", 
      RowBox[{"-", "1"}], ",", " ", 
      RowBox[{"-", "2"}], ",", " ", 
      RowBox[{
       RowBox[{"-", "3"}], "/", "2"}], ",", " ", 
      RowBox[{
       RowBox[{"-", "2"}], "/", "3"}], ",", " ", 
      RowBox[{
       RowBox[{"-", "1"}], "/", "3"}], ",", " ", 
      RowBox[{"1", "/", "3"}], ",", " ", 
      RowBox[{"2", "/", "3"}], ",", " ", 
      RowBox[{"3", "/", "2"}], ",", " ", "2", ",", " ", "3", ",", " ", 
      "\[Ellipsis]"}], "}"}], "  "}], TraditionalForm]]],
 "\n\nwhich contains every rational number, so we have shown that the \
rationals form a countable set."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437152835859375*^9, 3.437152977078125*^9}}]
}, Open  ]],

Cell[TextData[{
 "Even though we have shown that there are an infinite number of rational \
numbers between any two numbers, the natural question to ask is whether there \
are numbers which are not rational. The first discovery of a number which was \
not rational was the square root of two, proven by the Greeks.\n\nPROPOSITION \
9.3\nThere is no rational number ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "/", "q"}], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"p", "/", "q"}], ")"}], "2"], " ", "=", " ", "2"}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4371530264375*^9, 3.437153026578125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that there was such a rational number, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "/", "q"}], TraditionalForm]]],
 ". Let us further suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "/", "q"}], TraditionalForm]]],
 " is in simplest form, so that ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " are integers with no common factors. We could rewrite the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"p", "/", "q"}], ")"}], "2"], " ", "=", " ", "2"}], 
   TraditionalForm]]],
 " as \n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     SuperscriptBox["p", "2"], " ", "=", " ", 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["q", "2"], "."}]}]}]}], TraditionalForm]]],
 "\n\nThis would indicate that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 " is an even number, which implies that ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is even.\n\nNext, we make the substitution ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", " ", "=", " ", 
    RowBox[{"2", "r"}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is an integer. Making this substitution for ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", we get\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", " ", "r"}], ")"}], "2"], " ", "=", " ", 
     RowBox[{"2", " ", 
      SuperscriptBox["q", "2"]}]}], ","}], TraditionalForm]]],
 "   or\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", " ", 
     SuperscriptBox["r", "2"]}], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["q", "2"], "."}]}], TraditionalForm]]],
 "\n                                \nThis would indicate that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["q", "2"], TraditionalForm]]],
 ", and hence ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ", is even. But this contradicts the fact that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "/", "q"}], TraditionalForm]]],
 " was written in simplest form. Thus, there is no rational number whose \
square is 2."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "This proof is an example of a ",
 StyleBox["reductio ad absurdum",
  FontSlant->"Italic"],
 " proof. That is, we assume that the statement we were trying to prove was \
false, and proceeded until we reach a contradiction. \"Reductio ad absurdom\" \
proofs are particularly effective to prove that something is impossible. (In \
this case we were trying to show that solving ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"p", "/", "q"}], ")"}], "2"], " ", "=", " ", "2"}], 
   TraditionalForm]]],
 " was impossible with integers ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ".)\n\nNumbers which are not rational are called ",
 StyleBox["irrational",
  FontSlant->"Italic"],
 " numbers. Irrational numbers are characterized by the fact that their \
decimal representation never repeats. With ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we can look at as many digits of ",
 StyleBox["Sqrt[2]",
  FontWeight->"Bold"],
 " as we want, and see that there is no pattern. Here are the first 1000 \
digits."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["N[Sqrt[2],1000]", "Input",
 PageWidth->Infinity],

Cell[BoxData["1.\
414213562373095048801688724209698078569671875376948073176679737990732478462107\
038850387534327641572735013846230912297024924836055850737212644121497099935831\
413222665927505592755799950501152782060571470109559971605970274534596862014728\
517418640889198609552329230484308714321450839762603627995251407989687253396546\
331808829640620615258352395054745750287759961729835575220337531857011354374603\
408498847160386899970699004815030544027790316454247823068492936918621580578463\
111596668713013015618568987237235288509264861249497715421833420428568606014682\
472077143585487415565706967765372022648544701585880162075847492265722600208558\
446652145839889394437092659180031138824646815708263010059485870400318648034219\
489727829064104507263688131373985525611732204024509122770022694112757362728049\
573810896750401836986836845072579936472906076299694138047565482372899718032680\
247442062926912485905218100445984215059112024944134172853147810580360337107730\
918286931471017111168391658172688941975871658215212822951848847208969463386`\
1000."], "Output",
 CellChangeTimes->{3.442747342586375*^9}]
}, Open  ]],

Cell[TextData[{
 "By combining the rational numbers with the irrational numbers, we obtain \
all numbers with a decimal representation, called the ",
 StyleBox["real numbers",
  FontSlant->"Italic"],
 ". The set of real numbers is usually denoted by ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalR]", TraditionalForm]]],
 ".",
 "\n",
 "The set of real numbers clearly forms an abelian group using addition. \
Multiplication is also defined on the real numbers, but the real numbers do \
not form a group under multiplication since 0 has no inverse. \n\nEXPERIMENT:\
\nIf we throw out the element 0 from the real numbers, like we did for ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 ", do we get a group using multiplication?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["         ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "As a side note, there is a remarkable result about the real numbers--we \
cannot find a sequence of real numbers that contains every real number, the \
way we did with rational numbers. This fact was proved by Georg Cantor \
(1845-1913) using a classic argument that turned mathematics on its ear. In \
fact, he showed that we cannot form a sequence which contains just the \
numbers between 0 and 1. Since we are trying to prove that something is \
impossible, we expect to see a reductio ad absurdum proof.\n\nTHEOREM 9.1:  \
CANTOR'S DIAGONALIZATION THEOREM\nThe set of all real numbers between 0 and 1 \
is ",
 StyleBox["uncountable",
  FontSlant->"Italic"],
 ". That is, there cannot be a sequence of numbers which contains every real \
number between 0 and 1."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437153096859375*^9, 3.437153108046875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We begin by assuming that we can form such a sequence \n\n                  \
              ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["a", "1"], ",", " ", 
     SubscriptBox["a", "2"], ",", " ", 
     SubscriptBox["a", "3"], ",", " ", "\[Ellipsis]"}], "}"}], 
   TraditionalForm]]],
 "\n                                \nand work to find a contradiction.  The \
plan is to find a number ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " that cannot be in this list. We can do this by forcing ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " to have a different first digit than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "1"], TraditionalForm]]],
 ", a different second digit than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "2"], TraditionalForm]]],
 ", a different third digit than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "3"], TraditionalForm]]],
 ", and so on. The only technical problem with this is that some numbers have \
two decimal representations, such as\n\n      0.348600000000000000\[Ellipsis] \
 = 0.3485999999999999999\[Ellipsis].\n      \nFor these numbers, all we need \
to do is require that ",
 StyleBox["both",
  FontSlant->"Italic"],
 " representations are in the list. (That is, some rational numbers will \
appear twice on the list with different decimal representations.)\n\nWe now \
can find a number ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " using any number of procedures, such as letting the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-th digit of ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " be one more than the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-th digit of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "n"], TraditionalForm]]],
 ", modulo 10. For example, if the list of numbers is\n\n                     \
      ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["a", "1"],
      TraditionalForm], "  ", "=", " ", 
     RowBox[{"0.94837490123798570", "\[Ellipsis]"}]}], 
    "                           "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["a", "2"],
      TraditionalForm], "  ", "=", " ", 
     RowBox[{"0.83840000000000000", "\[Ellipsis]"}]}], 
    "                           "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["a", 
       RowBox[{"3", "  "}]],
      TraditionalForm], "=", " ", 
     RowBox[{"0.83839999999999999", "\[Ellipsis]"}]}], 
    "                            "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{
    SubscriptBox["a", "4"], "  ", "=", " ", 
    RowBox[{"0.34281655343424444", "\[Ellipsis]"}]}], TraditionalForm]}]],
 "\n\nthen",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"b", " ", "=", " ", 
     RowBox[{"0.0499", "\[Ellipsis]"}]}]}], TraditionalForm]]],
 " . Certainly ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is missing from the list, since it differs by all of the elements by at \
least one digit. This contradiction proves the theorem."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "We have seen how the groups ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 ", and the real numbers allow for an additional operation to be defined on \
them, thus providing the familiar structure. But there are some groups that \
we studied in chapter 1 which an we also can define a second operation. Take \
for example the groups formed by modular arithmatic, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". To define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 ", we used the command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["DefSumMod[6]", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Here is the multiplication table for the group ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 ", with 0  being the identity element:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[{0,1,2,3,4,5}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 
      1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 
      1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 1}}, {{
      1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 
      1, 0}, {0.6, 0.4, 0.8}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {
      1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.442747354836375*^9}]
}, Open  ]],

Cell["\<\
Can we define another operation on this group? The natural operation would be \
to try multiplication modulo 6. This is defined by the command\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["DefMultMod[6]", "Input",
 PageWidth->Infinity],

Cell["Let us look at the new multiplication table.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[{0,1,2,3,4,5}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 
      0}, {0, 1, 1}, {1, 0, 1}}, {{0, 1, 0}, {1, 1, 0}, {0, 1, 0}, {0, 1, 
     1}, {1, 1, 0}, {0, 1, 0}, {0, 1, 1}}, {{1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {
     1, 1, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 0}}, {{0, 1, 1}, {1, 1, 0}, {0, 1,
      1}, {0, 1, 0}, {1, 1, 0}, {0, 1, 1}, {0, 1, 
     0}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.442747361477*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that this is not a group, since we don't get a Latin square. In \
particular, the element 0 does not have an inverse. But that shouldn't matter \
since 0 didn't have an inverse in any of the other examples we looked at. \
What if we considered the set without the zero element? Would this be a \
group? \n\nEXPERIMENT:\nIn all of the previous groups we looked at, (",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ", Even numbers, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 ", Reals,) the product of two non-zero numbers was always non-zero. This \
allowed us to consider throwing out the zero element, and consider \
multiplication on the remaining elements. But what about multiplication \
modulo 6? Is the product of two non-zero elements always non-zero? Or is \
there an example where\n\n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", "y"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are both non-zero? If so, how many examples can you find?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell["\<\
Here one last example of appending an additional operation on a group. The \
following command produces the quaternion group of order 8 which we studied \
in the volume on groups:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitQuaternions
Q = {1,I,J,K,-1,-I,-J,-K}\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437153246671875*^9, 3.437153249078125*^9}, {
  3.437182762765625*^9, 3.4371827771875*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "\[ImaginaryI]", ",", "J", ",", "K", ",", 
   RowBox[{"-", "1"}], ",", 
   RowBox[{"-", "\[ImaginaryI]"}], ",", 
   RowBox[{"-", "J"}], ",", 
   RowBox[{"-", "K"}]}], "}"}]], "Output",
 CellChangeTimes->{3.43715324953125*^9, 3.437182598984375*^9, 
  3.437182734984375*^9, 3.43718277790625*^9, 3.44274736957075*^9}]
}, Open  ]],

Cell["This group has the following multiplication table:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[Q]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0, 1}, {1, 
      1, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}}, {{
      1, 0.6, 0}, {1, 0.6, 0}, {1, 0, 0}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 
      1}, {0.7, 0.8, 0.7}, {0, 1, 0}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 0.4,
       0.8}, {1, 1, 0}, {0.7, 0.8, 0.7}, {0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {1, 
      0, 0}, {1, 0.6, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7,
       0.8, 0.7}, {1, 0.6, 0}, {1, 0, 1}, {1, 1, 0}}, {{0, 1, 1}, {0, 1, 1}, {
      0.7, 0.8, 0.7}, {0, 1, 0}, {1, 0, 1}, {1, 0.6, 0}, {1, 0, 0}, {1, 1, 
      0}, {0.6, 0.4, 0.8}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 0}, {1, 0, 0}, {
      1, 0.6, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.7, 0.8, 0.7}, {0, 1, 1}}, {{
      1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 
      1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7,
       0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"I\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"J\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"K\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"-1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"-I\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"-J\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"-K\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"I\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"J\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"K\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"-1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"-I\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"-J\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"-K\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"-1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"-I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"-J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"-K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"K\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"J\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-I\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"-1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.437182784859375*^9, 3.442747373117625*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that the multiplication of the elements of the group acts a little \
like the cross product of two vectors in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalR]", "3"], TraditionalForm]]],
 " expressed in terms of ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["J", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". That is, \n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"I", "\[CenterDot]", "J"}], " ", "=", " ", "K"}], ","}], 
   TraditionalForm]]],
 "   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"J", "\[CenterDot]", "K"}], " ", "=", " ", "I"}], ","}], 
   TraditionalForm]]],
 "  and   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"K", "\[CenterDot]", "I"}], " ", "=", " ", 
    RowBox[{"J", "."}]}], TraditionalForm]]],
 " \n\nThis suggests that we can also ",
 StyleBox["add",
  FontSlant->"Italic"],
 " multiples of these elements together like vectors, forming such elements \
as"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["I + 2 J - K", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"\[ImaginaryI]", "+", 
  RowBox[{"2", " ", "J"}], "-", "K"}]], "Output",
 CellChangeTimes->{3.4427473794145*^9}]
}, Open  ]],

Cell[TextData[{
 "which would represents the vector ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[LeftAngleBracket]", " ", 
     RowBox[{"1", ",", " ", "2", ",", 
      RowBox[{"-", "1"}]}], "\[RightAngleBracket]"}], "."}], 
   TraditionalForm]]],
 " Can we define multiplication on all three dimensional vectors by extending \
the group of quaternions? \n\nEXPERIMENT:\nIs it true that whenever we \
multiply two three-dimension vectors using the rules in the above table, we \
get another three-dimension vector? For example, try the product"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell[" (I - 2 J - K) . (4 I + J + 2*K) ", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437182820953125*^9, 3.43718283425*^9}, {
  3.437182951453125*^9, 3.43718295384375*^9}, {3.4371829869375*^9, 
  3.43718299165625*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "3"}], " ", "\[ImaginaryI]"}], "-", 
  RowBox[{"6", " ", "J"}], "+", 
  RowBox[{"9", " ", "K"}]}]], "Output",
 CellChangeTimes->{3.4371828349375*^9, 3.43718295459375*^9, 
  3.437182992171875*^9, 3.442747384227*^9}]
}, Open  ]],

Cell["\<\
Here we get the vector which one might recognize as the cross product. Try \
changing the numbers in the example to see if the answer is always a three \
dimensional vector.

In doing the experiment, you may have seen results that look as though a \
number was added to a vector. This we never allowed in Calculus class! \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718301221875*^9, 3.437183015015625*^9}}],

Cell[CellGroupData[{

Cell[" (I - 2 J - K) . (3 I + J - 2*K) ", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437182820953125*^9, 3.43718283425*^9}, {
  3.437182951453125*^9, 3.43718295384375*^9}, {3.4371829869375*^9, 
  3.43718299165625*^9}, {3.437183026453125*^9, 3.43718303003125*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "3"}], "+", 
    RowBox[{"5", " ", "\[ImaginaryI]"}]}], ")"}], "-", "J", "+", 
  RowBox[{"7", " ", "K"}]}]], "Output",
 CellChangeTimes->{3.4371828349375*^9, 3.43718295459375*^9, 
  3.437182992171875*^9, 3.43718303128125*^9, 3.442747386961375*^9}]
}, Open  ]],

Cell[TextData[{
 "What could such an expression represent? We could consider the number added \
to the vector as a ",
 StyleBox["fourth",
  FontSlant->"Italic"],
 " dimension. This extra dimension would represent ",
 StyleBox["time",
  FontSlant->"Italic"],
 ". Thus, we would say the expression"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718301221875*^9, 3.43718301828125*^9}}],

Cell[CellGroupData[{

Cell[" 2 + I + 2 J - K", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"2", "+", "\[ImaginaryI]"}], ")"}], "+", 
  RowBox[{"2", " ", "J"}], "-", "K"}]], "Output",
 CellChangeTimes->{3.442747389523875*^9}]
}, Open  ]],

Cell[TextData[{
 "would represent the four dimensional vector \n      \n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftAngleBracket]", " ", 
    RowBox[{"2", ",", " ", "1", ",", " ", "2", ",", " ", 
     RowBox[{"-", "1"}]}], " ", "\[RightAngleBracket]"}], TraditionalForm]]],
 ". \n\n(Notice that the time dimension is listed ",
 StyleBox["before",
  FontSlant->"Italic"],
 " the three spacial dimensions.)\n\nEXPERIMENT:\nThe manipulation of four \
dimensional vectors is currently loaded into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " determine whether the set of non-zero four dimensional vectors forms a \
group. For example, we can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to multiply the  vectors ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[LeftAngleBracket]", 
     RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "\[RightAngleBracket]"}], 
    " ", "\[CenterDot]", " ", 
    RowBox[{"\[LeftAngleBracket]", 
     RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "\[RightAngleBracket]"}]}], 
   TraditionalForm]]],
 " as follows:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4371831615625*^9, 3.437183180578125*^9}}],

Cell[CellGroupData[{

Cell["x = a + b I + c J + d K", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437183101203125*^9, 3.437183109109375*^9}, {
  3.4371832485625*^9, 3.437183248953125*^9}}],

Cell[BoxData[
 RowBox[{"a", "+", 
  RowBox[{"\[ImaginaryI]", " ", "b"}], "+", 
  RowBox[{"c", " ", "J"}], "+", 
  RowBox[{"d", " ", "K"}]}]], "Output",
 CellChangeTimes->{3.437183057796875*^9, 3.437183146921875*^9, 
  3.43718326065625*^9, 3.442747404227*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["y = p + q I + r J + s K", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43718311315625*^9, 3.4371831440625*^9}, {
  3.437183252703125*^9, 3.437183253296875*^9}}],

Cell[BoxData[
 RowBox[{"p", "+", 
  RowBox[{"\[ImaginaryI]", " ", "q"}], "+", 
  RowBox[{"J", " ", "r"}], "+", 
  RowBox[{"K", " ", "s"}]}]], "Output",
 CellChangeTimes->{3.437183060640625*^9, 3.437183144578125*^9, 
  3.43718326271875*^9, 3.44274740638325*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["x . y", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437183255734375*^9, 3.43718325846875*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", " ", "p"}], "+", 
  RowBox[{"\[ImaginaryI]", " ", "b", " ", "p"}], "+", 
  RowBox[{"c", " ", "J", " ", "p"}], "+", 
  RowBox[{"d", " ", "K", " ", "p"}], "+", 
  RowBox[{"\[ImaginaryI]", " ", "a", " ", "q"}], "-", 
  RowBox[{"b", " ", "q"}], "+", 
  RowBox[{"d", " ", "J", " ", "q"}], "-", 
  RowBox[{"c", " ", "K", " ", "q"}], "-", 
  RowBox[{"c", " ", "r"}], "-", 
  RowBox[{"\[ImaginaryI]", " ", "d", " ", "r"}], "+", 
  RowBox[{"a", " ", "J", " ", "r"}], "+", 
  RowBox[{"b", " ", "K", " ", "r"}], "+", 
  RowBox[{"\[ImaginaryI]", " ", "c", " ", "s"}], "-", 
  RowBox[{"d", " ", "s"}], "-", 
  RowBox[{"b", " ", "J", " ", "s"}], "+", 
  RowBox[{"a", " ", "K", " ", "s"}]}]], "Output",
 CellChangeTimes->{3.437183062875*^9, 3.437183148890625*^9, 
  3.437183265734375*^9, 3.442747409055125*^9}]
}, Open  ]],

Cell[TextData[{
 "This shows that the product indeed will be a four dimensional vector. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can also suggest what an inverse element should be:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["x^(-1)", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437183373390625*^9, 3.4371833736875*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["a", 
   RowBox[{
    SuperscriptBox["a", "2"], "+", 
    SuperscriptBox["b", "2"], "+", 
    SuperscriptBox["c", "2"], "+", 
    SuperscriptBox["d", "2"]}]], "-", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", "b"}], 
   RowBox[{
    SuperscriptBox["a", "2"], "+", 
    SuperscriptBox["b", "2"], "+", 
    SuperscriptBox["c", "2"], "+", 
    SuperscriptBox["d", "2"]}]], "-", 
  FractionBox[
   RowBox[{"c", " ", "J"}], 
   RowBox[{
    SuperscriptBox["a", "2"], "+", 
    SuperscriptBox["b", "2"], "+", 
    SuperscriptBox["c", "2"], "+", 
    SuperscriptBox["d", "2"]}]], "-", 
  FractionBox[
   RowBox[{"d", " ", "K"}], 
   RowBox[{
    SuperscriptBox["a", "2"], "+", 
    SuperscriptBox["b", "2"], "+", 
    SuperscriptBox["c", "2"], "+", 
    SuperscriptBox["d", "2"]}]]}]], "Output",
 CellChangeTimes->{3.43718307321875*^9, 3.43718315134375*^9, 
  3.437183374140625*^9, 3.442747412086375*^9}]
}, Open  ]],

Cell["From here, you can do the rest of the work.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["    ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "PROPOSITION 9.4\nThe set of non-zero four dimensional vectors forms a \
non-abelian group using the multiplication table for the quaternion group ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718321871875*^9, 3.437183219625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["If ", "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718323875*^9, 3.437183241640625*^9}}],

Cell[CellGroupData[{

Cell["x = a + b I + c J + d K", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437183101203125*^9, 3.437183109109375*^9}, {
  3.4371832485625*^9, 3.437183248953125*^9}}],

Cell[BoxData[
 RowBox[{"a", "+", 
  RowBox[{"\[ImaginaryI]", " ", "b"}], "+", 
  RowBox[{"c", " ", "J"}], "+", 
  RowBox[{"d", " ", "K"}]}]], "Output",
 CellChangeTimes->{3.437183057796875*^9, 3.437183146921875*^9, 
  3.43718326065625*^9, 3.442747417477*^9}]
}, Open  ]],

Cell["is non-zero, then ", "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718323875*^9, 3.437183241640625*^9}, {
   3.437183300046875*^9, 3.437183320296875*^9}, 3.43718340553125*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"y", " ", "=", " ", 
  SuperscriptBox["x", 
   RowBox[{"-", "1"}]]}]], "Input",
 CellChangeTimes->{{3.437183292890625*^9, 3.437183293640625*^9}, {
  3.437183323828125*^9, 3.43718332821875*^9}, {3.437183387046875*^9, 
  3.4371833886875*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["a", 
   RowBox[{
    SuperscriptBox["a", "2"], "+", 
    SuperscriptBox["b", "2"], "+", 
    SuperscriptBox["c", "2"], "+", 
    SuperscriptBox["d", "2"]}]], "-", 
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", "b"}], 
   RowBox[{
    SuperscriptBox["a", "2"], "+", 
    SuperscriptBox["b", "2"], "+", 
    SuperscriptBox["c", "2"], "+", 
    SuperscriptBox["d", "2"]}]], "-", 
  FractionBox[
   RowBox[{"c", " ", "J"}], 
   RowBox[{
    SuperscriptBox["a", "2"], "+", 
    SuperscriptBox["b", "2"], "+", 
    SuperscriptBox["c", "2"], "+", 
    SuperscriptBox["d", "2"]}]], "-", 
  FractionBox[
   RowBox[{"d", " ", "K"}], 
   RowBox[{
    SuperscriptBox["a", "2"], "+", 
    SuperscriptBox["b", "2"], "+", 
    SuperscriptBox["c", "2"], "+", 
    SuperscriptBox["d", "2"]}]]}]], "Output",
 CellChangeTimes->{3.437183328859375*^9, 3.4371833895625*^9, 
  3.442747420117625*^9}]
}, Open  ]],

Cell[TextData[{
 "forms a multiplicative inverse, since it is a simple exerise to show that \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[CenterDot]", " ", 
     SuperscriptBox["x", 
      RowBox[{"-", "1"}]]}], "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]], "\[CenterDot]", "x"}], " ", "=", " ", "1"}]}], 
   TraditionalForm]]],
 ", the multiplicative identity. (See problem 9.15.)"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718323875*^9, 3.437183241640625*^9}, {
  3.437183300046875*^9, 3.437183320296875*^9}, {3.43718341578125*^9, 
  3.437183490765625*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"y", " ", ".", "x"}], "]"}]], "Input",
 CellChangeTimes->{{3.437183496875*^9, 3.43718351178125*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{{3.43718350240625*^9, 3.437183512484375*^9}, 
   3.442747422461375*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"x", ".", "y"}], "]"}]], "Input",
 CellChangeTimes->{{3.4371835141875*^9, 3.43718351725*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.437183517734375*^9, 3.442747424898875*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[NotEqual]", "0"}], ","}], TraditionalForm]]],
 " the common denominator ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["a", "2"], "+", 
     SuperscriptBox["b", "2"], "+", 
     SuperscriptBox["c", "2"], "+", 
     SuperscriptBox["d", "2"]}], ">", " ", "0"}], TraditionalForm]]],
 ".  It is easy to see that multiplication is closed.  The only hard part is \
to show that the associative law holds, which is best done via a program like \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718323875*^9, 3.437183241640625*^9}, {
   3.437183300046875*^9, 3.437183320296875*^9}, {3.43718341578125*^9, 
   3.437183490765625*^9}, {3.437183529765625*^9, 3.43718360825*^9}, {
   3.437183934765625*^9, 3.437184012484375*^9}, 3.437184297796875*^9}],

Cell[CellGroupData[{

Cell["y = p + q I + r J + s K", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43718311315625*^9, 3.4371831440625*^9}, {
  3.437183252703125*^9, 3.437183253296875*^9}}],

Cell[BoxData[
 RowBox[{"p", "+", 
  RowBox[{"\[ImaginaryI]", " ", "q"}], "+", 
  RowBox[{"J", " ", "r"}], "+", 
  RowBox[{"K", " ", "s"}]}]], "Output",
 CellChangeTimes->{3.437183060640625*^9, 3.437183144578125*^9, 
  3.43718326271875*^9, 3.437183621296875*^9, 3.442747427102*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"z", " ", "=", " ", 
  RowBox[{"t", " ", "+", " ", 
   RowBox[{"u", " ", "I"}], " ", "+", " ", 
   RowBox[{"v", " ", "J"}], " ", "+", " ", 
   RowBox[{"w", " ", "K"}]}]}]], "Input",
 CellChangeTimes->{{3.437183622421875*^9, 3.437183632546875*^9}}],

Cell[BoxData[
 RowBox[{"t", "+", 
  RowBox[{"\[ImaginaryI]", " ", "u"}], "+", 
  RowBox[{"J", " ", "v"}], "+", 
  RowBox[{"K", " ", "w"}]}]], "Output",
 CellChangeTimes->{3.4371836345*^9, 3.442747429180125*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"x", ".", "y"}], ")"}], ".", "z"}], " ", "-", " ", 
  RowBox[{"x", ".", 
   RowBox[{"(", 
    RowBox[{"y", ".", "z"}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.437183639859375*^9, 3.437183668453125*^9}}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{{3.437183643984375*^9, 3.437183669328125*^9}, 
   3.442747436867625*^9}]
}, Open  ]],

Cell[TextData[{
 "Thus, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has proven that multiplication is associative.\n\nGiven that the \
associative law holds, it is easy to see that the product of two non-zero \
quaternions must be non-zero.   If ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", "0"}], TraditionalForm]]],
 ", and say ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "\[NotEqual]", "0"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{"-", "1"}]], "\[CenterDot]", "x"}], ")"}], "\[CenterDot]", 
      "y"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{"-", "1"}]], "\[CenterDot]", "0"}], " ", "=", " ", 
       "0."}]}]}]}], TraditionalForm]]],
 " \nThus, if both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[NotEqual]", "0"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[NotEqual]", "0"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], "\[NotEqual]", "0"}], 
   TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437183726390625*^9, 3.43718372721875*^9}, {
  3.437184091859375*^9, 3.437184238421875*^9}, {3.437184365375*^9, 
  3.4371843685625*^9}, {3.437184535609375*^9, 3.437184557890625*^9}, {
  3.437185078484375*^9, 3.4371850798125*^9}, {3.437185112390625*^9, 
  3.4371851488125*^9}}]
}, Open  ]],

Cell[TextData[{
 "We call the group of four dimensional vectors of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "+", " ", 
    RowBox[{"b", " ", "i"}], " ", "+", " ", 
    RowBox[{"c", " ", "j"}], " ", "+", " ", 
    RowBox[{"d", " ", "k"}]}], TraditionalForm]]],
 " the ",
 StyleBox["quaternions, ",
  FontSlant->"Italic"],
 "denoted by \[DoubleStruckCapitalH] after their descoverer, William Rowan \
Hamilton (1805-1865). \nWe have now seen several example of groups which have \
additional structure in the form of a second operation. In the next section \
we will tie all of these examples together, discovering which properties \
these examples have in common."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718460721875*^9, 3.437184683328125*^9}, {
  3.4371847785*^9, 3.43718485721875*^9}, {3.4371848948125*^9, 
  3.43718489621875*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Definition of a Ring", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
In the preceeding section we have seen many examples of groups which exhibit \
not one but two operations defined on them. One of these operations is \
represented with the plus sign, and the other is usually denoted with a dot. \
However, some of the different groups we looked at possessed additional \
properties. To help us organize our findings, let us construct the following \
table to tell us which property holds for which groups.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\n                            Group: \n\n--------------|---|  Even  \
|---|-----|    |-----------|\nProperty:     | ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "  "}], TraditionalForm]]],
 "|Integers| ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "  "}], TraditionalForm]]],
 "|Reals| ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " |Quaternions| \n==============|===|========|===|=====|====|===========| \n\
Closed under  | x |    x   | x |  x  | x  |     x     |\nAddition      |   |  \
      |   |     |    |           |\n\
--------------|---|--------|---|-----|----|-----------|\nClosed under  |   |  \
      |   |     |    |           |\nMultiplication|   |        |   |     |    \
|           |\n--------------|---|--------|---|-----|----|-----------|\n(a + \
b) + c = |   |        |   |     |    |           |\na + (b + c)   |   |       \
 |   |     |    |           |\n\
--------------|---|--------|---|-----|----|-----------|\n(a \[CenterDot] b) \
\[CenterDot] c = |   |        |   |     |    |           |\na \[CenterDot] (b \
\[CenterDot] c)   |   |        |   |     |    |           |\n\
--------------|---|--------|---|-----|----|-----------|\nAdditive      |   |  \
      |   |     |    |           |\nidentity (0)  |   |        |   |     |    \
|           |\n--------------|---|--------|---|-----|----|-----------|\n\
Multiplicative|   |        |   |     |    |           |\nidentity (1)  |   |  \
      |   |     |    |           |\n\
--------------|---|--------|---|-----|----|-----------|\nAdditive      |   |  \
      |   |     |    |           |\ninverses exist|   |        |   |     |    \
|           |\n--------------|---|--------|---|-----|----|-----------|\n\
Multiplicative|   |        |   |     |    |           |\ninverses exist|   |  \
      |   |     |    |           |\nexcept for 0  |   |        |   |     |    \
|           |\n--------------|---|--------|---|-----|----|-----------|\na + b \
= b + a |   |        |   |     |    |           |\n\
--------------|---|--------|---|-----|----|-----------|\na \[CenterDot] b = b \
\[CenterDot] a |   |        |   |     |    |           |\n\
--------------|---|--------|---|-----|----|-----------|\na \[CenterDot] b = 0 \
only|   |        |   |     |    |           |\nif a or b = 0 |   |        |   \
|     |    |           |\n\
--------------|---|--------|---|-----|----|-----------|\n(a + b) \[CenterDot] \
c = |   |        |   |     |    |           |\n(a\[CenterDot]c) + (b\
\[CenterDot]c) |   |        |   |     |    |           |\n\
--------------|---|--------|---|-----|----|-----------|\na \[CenterDot] (b + \
c) = |   |        |   |     |    |           |\n(a\[CenterDot]b) + (a\
\[CenterDot]c) |   |        |   |     |    |           |\n\
--------------|---|--------|---|-----|----|-----------|"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Courier New",
 FontSize->10],

Cell[TextData[{
 "\nEXPERIMENT:\nComplete the above table by marking an \"",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "\" whenever a group possesses a given property for all elements in the \
group, and marking an \"",
 Cell[BoxData[
  FormBox["o", TraditionalForm]]],
 "\" otherwise. The first row already been done. You may have to redefine \
some of the groups to test some of this properties. For which groups do all \
of the properties hold? Which properties hold for all of these groups? "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[" ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We want to pay special attention to the properties that hold for all of the \
groups studied so far. In fact, let us define a ",
 StyleBox["ring",
  FontSlant->"Italic"],
 " as a group possessing all of these properties. In this way, we force all \
six of the above groups to be rings. \n\nEXPERIMENT:\nIf the above table was \
completed correctly, one ought to be able to come up with a definition of a \
ring. Try to find a definition that uses only one sentence."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["               ", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["If you give up, open the following cell.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["DEFINITION 9.2:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437184945859375*^9, 3.43718494996875*^9}}],

Cell[TextData[{
 "A ",
 Cell[BoxData[
  FormBox[
   StyleBox["ring",
    FontSlant->"Italic"], TraditionalForm]]],
 " is an abelian group with the operation ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "+", ")"}], TraditionalForm]]],
 " on which a second associative operation ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "\[CenterDot]", ")"}], TraditionalForm]]],
 " is defined such that the two distributive laws \n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", "+", " ", "b"}], ")"}], " ", "\[CenterDot]", " ", 
     "c"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "\[CenterDot]", "c"}], ")"}], " ", "+", " ", 
     RowBox[{"(", 
      RowBox[{"b", "\[CenterDot]", "c"}], ")"}], " "}]}], TraditionalForm]]],
 "\nand \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", " ", "\[CenterDot]", " ", 
     RowBox[{"(", 
      RowBox[{"b", " ", "+", " ", "c"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "\[CenterDot]", "b"}], ")"}], " ", "+", " ", 
     RowBox[{"(", 
      RowBox[{"a", "\[CenterDot]", "c"}], ")"}], " "}]}], TraditionalForm]]],
 "\n\nhold for all ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " in the ring."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "For any ring we will use the symbol 0 to denote the additive identity of a \
ring, and the notation ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "x"}], TraditionalForm]]],
 " for the additive inverse of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".\n\nNotice that for a group to be a ring, only 9 of the above 13 \
properties must hold. However, the other 4 properties mentioned in the table \
will sometimes play an important role as we study rings. Thus, if a ring has \
one of these additional 4 properties, we will give the ring a special name.\n\
\nDEFINITION 9.3:\nA ring for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "x"}]}], TraditionalForm]]],
 " for all elements ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is called a ",
 StyleBox["commutative ring",
  FontSlant->"Italic"],
 ".\n\nObviously, commutative rings play a similar role in ring theory that \
Abelian groups play in group theory.\n\nDEFINITION 9.4:\nA ring for which \
there is an element ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " such that\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "e"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"e", "\[CenterDot]", "x"}], " ", "=", " ", "x"}], " "}], 
   TraditionalForm]]],
 "\n\nfor all elements ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in the ring is called a ",
 StyleBox["ring with identity",
  FontSlant->"Italic"],
 ". The element ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " is called the ",
 StyleBox["multiplicative identity",
  FontSlant->"Italic"],
 " of the ring.\n\nIt should be noted before we go on that if a ring has a \
multiplicative identity, it has only one. If there were two, say ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "\[Prime]"], TraditionalForm]]],
 ", then\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"e", " ", "=", " ", 
     RowBox[{
      RowBox[{"e", "\[CenterDot]", 
       SuperscriptBox["e", "\[Prime]"]}], " ", "=", " ", 
      RowBox[{
       SuperscriptBox["e", "\[Prime]"], "."}]}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437185239984375*^9, 3.437185252234375*^9}}],

Cell[TextData[{
 "The next definition will deal with rings for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " implies that either ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " must be 0. However, it is reasonable to first prove the following \
proposition:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "LEMMA 9.1\nIf ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is any element in a ring, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"0", "\[CenterDot]", "x"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "0"}], " ", "=", " ", "0"}]}], ","}], 
   TraditionalForm]]],
 " where 0 is the additive identity."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718530584375*^9, 3.43718530596875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "This proof is just a little tricky because there are no other propositions \
to rely on. Thus, every step must directly use one of the 9 properties of \
rings. (The temptation is to rely on some property we suspect is true, but \
haven't yet proven.)\n\nNote that\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"0", "\[CenterDot]", "x"}], " ", "+", " ", 
       RowBox[{"0", "\[CenterDot]", "x"}]}], ")"}], " ", "=", "  ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0", " ", "+", " ", "0"}], ")"}], "\[CenterDot]", "x"}], " ", 
      "=", " ", 
      RowBox[{"0", "\[CenterDot]", "x"}]}]}], ","}], TraditionalForm]]],
 "             so\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"0", "\[CenterDot]", "x"}], " ", "+", " ", 
        RowBox[{"0", "\[CenterDot]", "x"}]}], ")"}], " ", "+", " ", 
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{"0", "\[CenterDot]", "x"}], ")"}]}], ")"}]}], " ", "=", " ", 
     
     RowBox[{
      RowBox[{"0", "\[CenterDot]", "x"}], " ", "+", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{"0", "\[CenterDot]", "x"}], ")"}]}], ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\nHence\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"0", "\[CenterDot]", "x"}], " ", "+", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"0", "\[CenterDot]", "x"}], " ", "+", " ", 
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{"0", "\[CenterDot]", "x"}], ")"}]}], ")"}]}], ")"}]}], 
      " ", "=", " ", "0"}], ","}], " "}], TraditionalForm]]],
 "                   so\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"0", "\[CenterDot]", "x"}], " ", "+", " ", "0"}], " ", "=", " ", 
    
    RowBox[{
     RowBox[{"0", "\[CenterDot]", "x"}], "          ", "=", " ", "0."}]}], 
   TraditionalForm]]],
 "\n\nSimilarly,\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "0"}], " ", "+", " ", 
        RowBox[{"x", "\[CenterDot]", "0"}]}], ")"}], " ", "=", "  ", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"0", " ", "+", " ", "0"}], ")"}]}], " ", "=", " ", 
       RowBox[{"x", "\[CenterDot]", "0"}]}]}], ","}]}], TraditionalForm]]],
 "              so\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "0"}], " ", "+", " ", 
        RowBox[{"x", "\[CenterDot]", "0"}]}], ")"}], " ", "+", " ", 
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{"x", "\[CenterDot]", "0"}], ")"}]}], ")"}]}], " ", "=", " ", 
     
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "0"}], " ", "+", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{"x", "\[CenterDot]", "0"}], ")"}]}], ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\nHence\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "0"}], " ", "+", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "0"}], " ", "+", " ", 
        RowBox[{"(", 
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{"x", "\[CenterDot]", "0"}], ")"}]}], ")"}]}], ")"}]}], " ",
      "=", " ", "0"}], ","}], TraditionalForm]]],
 "                     so\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "0"}], " ", "+", " ", "0"}], " ", "=", " ",
      
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "0"}], "          ", "=", " ", "0."}]}]}], 
   TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "This proof shows that we can get the equivalent of subtraction by adding \
the additive inverse.  But although we can add, subtract, and multiply \
elements in a ring, we cannot, in general, divide elements.  In fact, we can \
find some rings for which the product of two non-zero elements produces 0, \
such as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"3", "\[CenterDot]", "2"}], "=", "0"}], TraditionalForm]]],
 " in the ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718530584375*^9, 3.43718530596875*^9}, {
  3.437185439578125*^9, 3.43718555425*^9}}],

Cell[TextData[{
 "\nDEFINITION 9.5:\nIf ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is a non-zero element of a ring such that either ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " for a non-zero element ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is called a ",
 StyleBox["zero divisor",
  FontSlant->"Italic"],
 " of the ring. If a ring has no zero divisors, it is called a ",
 StyleBox["ring without zero divisors",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437185407859375*^9, 3.437185409796875*^9}, 
   3.437185598921875*^9}],

Cell[TextData[{
 "We see from this definition that 2 and 3 are zero divisors of the ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"3", "\[CenterDot]", "2"}], "=", "0"}], TraditionalForm]]],
 " in this ring.  A related definition stems from the product of two elements \
equaling the multiplicative identity."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718530584375*^9, 3.43718530596875*^9}, {
  3.437185439578125*^9, 3.43718555425*^9}, {3.437185605015625*^9, 
  3.437185680484375*^9}}],

Cell[TextData[{
 "\nDEFINITION 9.6:\nIf, for the element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in a ring with identity, there is a element ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " such that\n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"y", "\[CenterDot]", "x"}], " ", "=", " ", "e"}]}], ","}], 
   TraditionalForm]]],
 "\n\nwe say that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " has a multiplicative inverse, or is ",
 StyleBox["invertible",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437185407859375*^9, 3.437185409796875*^9}, 
   3.437185598921875*^9, {3.43718568903125*^9, 3.437185691171875*^9}, 
   3.437185730125*^9}],

Cell[TextData[{
 "\nJust because an element is not a zero divisor does not mean that it is \
invertiable.  For example, 2 is not a zero divisor of the ring ",
 "\[DoubleStruckCapitalZ], yet 2 is not invertible in this ring.\n",
 "\nThere is one special ring to introduce, the smallest possible ring. It is \
called the ",
 StyleBox["trivial ring",
  FontSlant->"Italic"],
 ", and is defined by the command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437185407859375*^9, 3.437185409796875*^9}, 
   3.437185598921875*^9, {3.43718568903125*^9, 3.437185691171875*^9}, {
   3.437185730125*^9, 3.437185798828125*^9}}],

Cell["DefMultMod[1]", "Input",
 PageWidth->Infinity],

Cell["\<\
The only element of this ring is the zero element, so here are its addition \
and multiplication tables:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["AddTable[{0}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{RasterBox[{{{1, 1, 0}, {1, 1, 0}}, {{1, 1, 1}, {1, 1, 0}}}], 
   InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}], InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], 
   LineBox[{{1, 0}, {1, 1}, {2, 1}}]}]], "Output",
 CellChangeTimes->{3.437185815875*^9, 3.442747454852*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["MultTable[{0}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{RasterBox[{{{1, 1, 0}, {1, 1, 0}}, {{1, 1, 1}, {1, 1, 0}}}], 
   InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}], InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], 
   LineBox[{{1, 0}, {1, 1}, {2, 1}}]}]], "Output",
 CellChangeTimes->{3.43718581696875*^9, 3.442747457242625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztW0Fu2zAQXEupU+QVfkKfkFN78h9iBK0lp0GLJMcA7gPygnwgd98NHXNo
C+eaNoARoD3klh+w3LUoiaRpUUUL2O0QFkXN7OzSNEUCXHg4usjeno4u8uPR
4M3Z6GOWH58PXn8401DaI+p90dfnAXFbEZlK6Y/6yhUjf779U27pYLlcvlos
FvL0Y0Ud8O07NxM1nZK0DkCBAgUKFChQoECB2llqIs0+PT4SXV5SVa6viZ6e
6KRS5psN37uOV/Yp3d0RDYck96TEbm9rbFJFyCIVJ24oI+SucWGjvRK7v19h
zOVVqHGkYrI2VEJZRv3SnMdCP8pTZgcI2uWu27ETez6nSsht0586wLtIRTDU
bFYb9Z3hYM4P1aYIhrq6Whk9P9dCPWWkMOeHalOsD5Uw/bJqOW4d9h930RhE
/o32S4yLnonyNA4Oe0iRtU0mno3u1Li52TSZQorgtzLTgB0YIa8Oxpkfqk3h
hcoaw2Fe2Rftr3uEIhjK9JH7tecMx/qlq03hLV3NBdkIkwa2eUHepPAWZLOp
cB95fzCF27pv/qYSMPQ2lS3ZNEGBAgUKFChQoEDtPkUpc+roiFRREH2TWpKC
YsnXvqNRWlMUhTo8JL60huv005QdrdE8qLKyogMFChQoUKBAge4A2pI08o7t
RBSbEPJOE0Udm+Pxjj1rdXw6RjSxORlPzV9fyRH3rBTO6bTpsXMiRTzKUbzl
p3OWxPRsXvbf7llbXuFhSyx/J1FhqTvnHix153SCNX87ZwgsdedDf+/N63SO
b73ssWf0W7NKAQUKFChQoED/Z/Rv/4Ok2abeL+orXQA=\
\>"]]
}, Open  ]],

Cell[TextData[{
 "Although this may seem like a boring example, this ring is rather \
interesting because the multiplicative identity is 0! (Zero times any element \
is that element, since that element would be 0.) In fact, 0 has itself for \
its multiplicative inverse. These two facts are true for no other ring, since \
there would be a non-zero element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", and hence\n\n                                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"0", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], ","}]}], 
   TraditionalForm]]],
 " not ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".\n\nLikewise, for a non-trival ring, 0 cannot have a multiplicative \
inverse, for otherwise\n\n                                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"0", "\[CenterDot]", 
       RowBox[{"(", 
        SuperscriptBox[
         RowBox[{"0", " "}], 
         RowBox[{"-", "1"}]], ")"}]}], " ", "=", " ", "0"}], ",", " "}]}], 
   TraditionalForm]]],
 "\n\nwhich we just proved was not the multiplicative identity.\n\nEven \
though 0 is never invertable except for the trivial ring, we have seen \
several cases where all other elements have inverses.\n\nDEFINITION 9.7\nA \
ring for which every non-zero element has a multiplicative inverse is called \
a ",
 StyleBox["division ring",
  FontSlant->"Italic"],
 ".\n\nNote that division rings automatically have some of the other \
properties we have discussed."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4371858365625*^9, 3.43718583865625*^9}}],

Cell["\<\
PROPOSITION 9.5
A division ring always has a multiplicative identity and has no zero \
divisors.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718585096875*^9, 3.437185851859375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We just saw that the trivial ring has an identity and has no zero divisors, \
so we may assume that the ring has a non-zero element ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " has an multiplicative inverse ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ", so we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"y", " ", "\[CenterDot]", " ", "z"}], " ", "=", " ", "e"}], 
    ","}], TraditionalForm]]],
 " the identity. Thus, every division ring must have an identity.\n\nNow \
suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " in a division ring, with both ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " non-zero. Then ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " has a multiplicative inverse ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ", so that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", " ", "\[CenterDot]", " ", "z"}], " ", "=", " ", "e"}], 
   TraditionalForm]]],
 ". But then\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"x", " ", "=", " ", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", "e"}], " ", "=", " ", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{"y", "\[CenterDot]", "z"}], ")"}]}], " ", "=", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "\[CenterDot]", "z"}], 
         " ", "=", " ", 
         RowBox[{
          RowBox[{"0", "\[CenterDot]", "z"}], " ", "=", " ", "0"}]}]}]}]}], 
     ","}]}], TraditionalForm]]],
 "\n\nwhich contradicts the fact that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is non-zero. Thus, a division ring has no zero divisors."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "We see from this proposition that a division ring must have all but one of \
the 13 properties we listed, the remaining property being the commutivity of \
multiplication. We will give a division ring different names depending on \
whether or not this last property holds.\n\nDEFINITION 9.8\nA non-trivial \
division ring for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "x"}]}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is called a ",
 StyleBox["field",
  FontSlant->"Italic"],
 ". A division ring for which multiplication is not commutative is called a \
",
 StyleBox["skew field",
  FontSlant->"Italic"],
 ".\n\nWe can now classify each possible type of ring. For example, the ring \
",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is a commutative ring with an identity and without zero divisors. The ring \
of even integers, however, has no identity element, so we would call this a \
commutative ring without zero divisors. Both ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 " and the reals satisfied all 13 properties, so these two rings are fields. \
The ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " has zero divisors, so we would call this a commutative ring with identity. \
The quaternions have all the properties of a field except that multiplication \
is not commutative, so this would be an example of a skew field."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437185914515625*^9, 3.4371859166875*^9}}],

Cell["\<\
EXPERIMENT:
How many different classifications of rings are possible? Note that the \
trivial ring is technically not a field, so this ring would be in a catagory \
all by itself. By the end of this chapter we will see examples of each \
different type of ring.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Entering Finite Rings into GAP and ",
 StyleBox["Mathematica",
  FontSlant->"Italic"]
}], "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718598059375*^9, 3.4371859824375*^9}}],

Cell[TextData[{
 "Now that we have started working with rings, it is time to learn how to \
program ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to work with any finite ring. \n\nIn the chapters on group theory, we \
entered finite groups into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by using the generators of the group. We will use a very similar process \
for finite rings. If we consider a finite ring simply as an abelian group \
under addition, we can find a set of generators ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " for this group. That is, we temporally ignore the multiplication \
structure, and proceed as we did for groups. For each element in ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " we determine the additive order of the element. That is, for each \
generator ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " we want to find the smallest number ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " such that"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417216997139997`*^9, 3.4417217018077497`*^9}}],

Cell[TextData[{
 "                                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     UnderscriptBox[
      UnderscriptBox[
       RowBox[{"x", "+", "x", "+", "\[Ellipsis]", "+", "x", "+", "x"}], 
       "\[UnderBrace]"], 
      RowBox[{"n", " ", "times"}]], " ", "=", " ", "0."}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["We introduce a simple notation for the above equation.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718601425*^9, 3.437186021796875*^9}}],

Cell[TextData[{
 "\nDEFINITION 9.9\nIf ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a positive integer, and ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is any element in a ring, we define ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "x", " "}], TraditionalForm]]],
 "inductively by letting ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", " ", "x"}], " ", "=", " ", "x"}], TraditionalForm]]],
 ", and \n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", " ", "x"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"n", "-", "1"}], ")"}], " ", "x"}], " ", "+", " ", 
     RowBox[{"x", "."}]}]}], TraditionalForm]]],
 "\n\nWe also define ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "n"}], ")"}], " ", "x"}], TraditionalForm]]],
 "  to be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", " ", 
    RowBox[{"(", 
     RowBox[{"n", " ", "x"}], ")"}]}], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " a positive integer. Finally, we define ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"0", " ", "x"}], " ", "=", " ", "0."}], TraditionalForm]]],
 " "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718601425*^9, 3.43718605965625*^9}}],

Cell[TextData[{
 "Because this \"multiplication by an integer\" is merely a shorthand for \
repeated addition, we immediately see that\n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"m", " ", "+", " ", "n"}], ")"}], " ", "x"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"m", " ", "x"}], " ", "+", " ", 
      RowBox[{"n", " ", "x"}]}]}], ","}], TraditionalForm]]],
 "\nand\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"m", " ", "n"}], ")"}], " ", "x"}], " ", "=", " ", 
    RowBox[{"m", " ", 
     RowBox[{"(", 
      RowBox[{"n", " ", "x"}], ")"}]}]}], TraditionalForm]]],
 "\n\nfor any element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and any integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ".\n\nHowever, we have other rules which are true for this notation."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718601425*^9, 3.437186066109375*^9}}],

Cell[TextData[{
 "LEMMA 9.2\nLet ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " be any two elements in a ring, and let ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " be an integer. Then\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"n", " ", "x"}], ")"}], "\[CenterDot]", "y"}], "  ", "=", "  ", 
    
    RowBox[{
     RowBox[{"n", " ", 
      RowBox[{"(", 
       RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], "  ", "=", "  ", 
     RowBox[{"x", "\[CenterDot]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"n", " ", "y"}], ")"}], "."}]}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.4371860799375*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We will proceed by induction. The statement is certainly true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "0"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ". Suppose that the statement is true for the previous case ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ".  But then\n           \n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"n", "-", "1"}], ")"}], " ", "x"}], ")"}], "\[CenterDot]", 
       "y"}], " ", "+", " ", 
      RowBox[{"x", "\[CenterDot]", "y"}]}], "  ", "=", "  ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"n", "-", "1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], " ", "+", " ", 
       RowBox[{"x", "\[CenterDot]", "y"}]}], "  ", "=", "  ", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"n", "-", "1"}], ")"}], " ", "y"}], ")"}]}], " ", "+", " ",
        
       RowBox[{"x", "\[CenterDot]", "y"}]}]}]}]}], TraditionalForm]]],
 ".\n\nHence, by the distributive law,\n \n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"n", "-", "1"}], ")"}], " ", "x"}], " ", "+", " ", "x"}], 
       ")"}], "\[CenterDot]", "y"}], "  ", "=", "  ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"n", "-", "1"}], ")"}], "+", "1"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], "  ", "=", "  ", 
      RowBox[{"x", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"n", "-", "1"}], ")"}], " ", "y"}], " ", "+", " ", "y"}], 
        ")"}]}]}]}], ","}], TraditionalForm]]],
 "\nand so\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"n", " ", "x"}], ")"}], "\[CenterDot]", "y"}], "  ", "=", "  ", 
    
    RowBox[{
     RowBox[{"n", " ", 
      RowBox[{"(", 
       RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], "  ", "=", "  ", 
     RowBox[{"x", "\[CenterDot]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"n", " ", "y"}], ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nHence, the statement is true for all positive integers. \n\nFor \
negative integers, we can merely show that\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"n", " ", "x"}], ")"}], "\[CenterDot]", "y"}], " ", "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"-", "n"}], ")"}], " ", "x"}], ")"}], "\[CenterDot]", 
      "y"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"n", " ", "x"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", "n"}], ")"}], " ", "x"}]}], ")"}], "\[CenterDot]", 
      "y"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"n", " ", "-", " ", "n"}], ")"}], " ", "x"}], ")"}], 
       "\[CenterDot]", "y"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"0", "\[CenterDot]", "y"}], " ", "=", " ", "0."}]}]}]}], 
   TraditionalForm]]],
 "\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"n", " ", 
       RowBox[{"(", 
        RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], " ", "+", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", "n"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"n", " ", "-", " ", "n"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"0", " ", 
        RowBox[{"(", 
         RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], " ", "=", " ", 
       "0."}]}]}]}], TraditionalForm]]],
 "\n \n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"n", " ", "y"}], ")"}]}], " ", "+", " ", 
      RowBox[{"x", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", "n"}], ")"}], " ", "y"}], ")"}]}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"n", " ", "y"}], " ", "+", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "n"}], ")"}], " ", "y"}]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"n", " ", "-", " ", "n"}], ")"}], " ", "y"}], ")"}]}], " ",
        "=", " ", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "0"}], " ", "=", " ", "0."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nThus,",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"-", "n"}], ")"}], " ", "x"}], ")"}], "\[CenterDot]", "y"}], 
     ",", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "n"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], ","}]}], 
   TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "n"}], ")"}], " ", "y"}], ")"}]}], TraditionalForm]]],
 " are the additive inverses of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"n", " ", "x"}], ")"}], "\[CenterDot]", "y"}], ",", " ", 
    RowBox[{"n", " ", 
     RowBox[{"(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{"n", " ", "y"}], ")"}]}], TraditionalForm]]],
 " respectively. But since these latter three are equal for positive ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", we have\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"-", "n"}], ")"}], " ", "x"}], ")"}], "\[CenterDot]", "y"}], 
     "  ", "=", "  ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", "n"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], "  ", "=", "  ", 
      RowBox[{"x", "\[CenterDot]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "n"}], ")"}], " ", "y"}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nHence the lemma is proven for all integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417217478858747`*^9, 3.4417219659327497`*^9}}]
}, Open  ]],

Cell[TextData[{
 "We can now use this notation within ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to generate a finite ring. For example, suppose that we wish to define a \
ring whose additive group is isomorphic to \n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubsuperscriptBox["Z", "15", "*"],
     TraditionalForm], " ", "=", " ", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "4", ",", "7", ",", "8", ",", "11", ",", "13", ",", 
       "14"}], "}"}], "."}]}], TraditionalForm]]],
 "\n \nTwo elements which generate this group are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "=", "2"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", "14"}], TraditionalForm]]],
 ". Since \n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "4"], " ", "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "15"}], ")"}]}]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["14", "2"], " ", "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "15"}], ")"}]}]}], TraditionalForm]]],
 ","
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is of order 4 and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is of order 2. But in the new notation, we would write \n\n            4 a \
= 0   and   2 b = 0.\n\nsince 0 is the additive identity for the ring.\n\nTo \
define this group in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we define both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"4", "a"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "b"}], TraditionalForm]]],
 " to be 0. The following three commands do this."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
InitRing
Define[4 a, 0]
Define[2 b, 0]\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "This is sufficient to define the group structure of the ring to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". The eight elements of the group are denoted as follows:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["R = AddGroup[{a,b}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.437186165875*^9, 3.442747475227*^9}]
}, Open  ]],

Cell["\<\
We combine two elements of this group with a plus sign rather than the dot \
that we used for groups. For example, here is the sum of two elements:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["(3 a + b) + 2a", "Input",
 PageWidth->Infinity,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 RowBox[{"a", "+", "b"}]], "Output",
 CellChangeTimes->{3.442747477602*^9}]
}, Open  ]],

Cell["\<\
We can then produce an \"addition\" table for this group as follows:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["AddTable[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 
      0, 0}, {1, 1, 0}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {1, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {
      0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 0, 
      0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {0.7, 0.8, 0.7}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {1,
       1, 0}, {1, 0, 1}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 
      0.8}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.43718618621875*^9, 3.442747480367625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "Notice that there are several differences between defining a group and \
defining the group structure of a ring. The obvious difference is that we use \
the plus sign instead of the dot for our operation. Also, when we defined a \
group, we began by telling ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " the identity element. But for a ring, the additive identity is always \
denoted 0, and the multiplicative identity may not exist. So the first \
statement merely has to tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that we are defining a ring. Finally, we do not need an extra \"Define\" \
statement to tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " the group is commutative, as we did before for defining ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "There is one drawback to this approach--we must choose our generators \
carefully! Suppose we had chosen the generators ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "=", "2"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", "7"}], TraditionalForm]]],
 ". These two elements generate the group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 ", but both are of order 4. So the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " commands for entering these two generators would be"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing
Define[4 a, 0]
Define[4 b, 0]
R = AddGroup[{a,b}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"2", " ", "b"}], ",", 
   RowBox[{"3", " ", "b"}], ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}], ",", 
   RowBox[{"a", "+", 
    RowBox[{"2", " ", "b"}]}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", 
    RowBox[{"2", " ", "b"}]}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", 
    RowBox[{"2", " ", "b"}]}], ",", 
   RowBox[{"a", "+", 
    RowBox[{"3", " ", "b"}]}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", 
    RowBox[{"3", " ", "b"}]}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", 
    RowBox[{"3", " ", "b"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.437186205046875*^9, 3.44274748525825*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives 16 elements instead of 8! The problem is that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is not using the identity ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"2", "a"}], " ", "=", " ", 
     RowBox[{"2", "b"}]}], ","}], TraditionalForm]]],
 " which is true since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "2"], " ", "\[Congruent]", " ", 
    RowBox[{
     SuperscriptBox["7", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "15"}], ")"}]}]}], TraditionalForm]]],
 ". One solution would be to add an additional ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command defining ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"2", "a"}], " ", "=", " ", 
     RowBox[{"2", "b"}]}], ","}], TraditionalForm]]],
 " but this produces some potiential problems later on. A better solution is \
simply to make the following restriction on the set of generators."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "DEFINITION 9.10:\nLet ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " be an abelian group. A",
 StyleBox[" basis",
  FontSlant->"Italic"],
 " is a set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"B", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["x", "1"], ",", " ", 
      SubscriptBox["x", "2"], ",", " ", 
      SubscriptBox["x", "3"], ",", " ", 
      RowBox[{"\[Ellipsis]", " ", 
       SubscriptBox["x", "k"]}]}], "}"}]}], TraditionalForm]]],
 " which generates the group such that the only way in which\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["n", "1"], 
      SubscriptBox["x", "1"]}], " ", "+", " ", 
     RowBox[{
      SubscriptBox["n", "2"], 
      SubscriptBox["x", "2"]}], " ", "+", " ", 
     RowBox[{
      SubscriptBox["n", "3"], 
      SubscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]", " ", "+", " ", 
     RowBox[{
      SubscriptBox["n", "k"], 
      SubscriptBox["x", 
       RowBox[{"k", " "}]]}]}], "=", " ", "0"}], TraditionalForm]]],
 "\n\nfor integers ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], ",", " ", 
    SubscriptBox["n", "2"], ",", " ", 
    SubscriptBox["n", "3"], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     SubscriptBox["n", "k"]}]}], TraditionalForm]]],
 "  is if \n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["n", "1"], 
     SubscriptBox["x", "1"]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["n", "2"], 
      SubscriptBox["x", "2"]}], "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["n", "3"], 
       SubscriptBox["x", "3"]}], "=", " ", 
      RowBox[{"\[Ellipsis]", " ", "=", " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["n", "k"], 
         SubscriptBox["x", "k"]}], " ", "=", " ", "0."}]}]}]}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718621290625*^9, 3.43718621471875*^9}, {
  3.437186256421875*^9, 3.437186256421875*^9}}],

Cell[TextData[{
 "For a finite group, it is clear that every combination of the form\n\n      \
    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["n", "1"], 
       SubscriptBox["x", "1"]}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["n", "2"], 
       SubscriptBox["x", "2"]}], "+", " ", 
      RowBox[{
       SubscriptBox["n", "3"], 
       SubscriptBox["x", "3"]}], "+", " ", "\[Ellipsis]", " ", "+", " ", 
      RowBox[{
       SubscriptBox["n", "k"], 
       SubscriptBox["x", "k"]}]}], " ", ","}], " "}], TraditionalForm]]],
 "\n\nwhere each ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "i"], TraditionalForm]]],
 " is non-negative and less then the order of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " , forms a distinct element. Also, every element of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " could be put in that form. Thus, the product of the orders of all the \
elements of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " equals the order of the group.\n\nIt should be noted that ",
 StyleBox["any",
  FontSlant->"Italic"],
 " finite abelian group has a basis, as guaranteed by the fundemental theorem \
of finite abelian groups (6.2).  For example, we found two elements of orders \
4 and 2 which generate ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 " , and since 4\[CenterDot]2 = 8, the order of the group, so these two \
generators form a basis. \n\nIf we begin with a basis it is clear that the \
commands of the form\n\n",
 StyleBox["Define[",
  FontWeight->"Bold"],
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 StyleBox[" ",
  FontWeight->"Bold"],
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 StyleBox[", 0],",
  FontWeight->"Bold"],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the order of the generator ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", will be sufficient to define the additive group. All we need to consider \
now is the multiplicative definitions.\n\nSince we have two generators ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}], TraditionalForm]]],
 ", we will need to define ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "2"], " ", "=", " ", "4"}], TraditionalForm]]],
 " multiplications:\n                                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "a"}], ",", "      ", "\n", 
    RowBox[{"a", "\[CenterDot]", "b"}], ",", "     ", "\n", 
    RowBox[{"b", "\[CenterDot]", "a"}], ",", "     ", "\n", 
    RowBox[{"b", "\[CenterDot]", 
     RowBox[{"b", "."}]}]}], TraditionalForm]]],
 "\n\nThese four products could be defined to be any of the 8 elements of the \
ring. This means that there are up to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["8", "4"],
     TraditionalForm], "  ", "=", " ", "4096"}], TraditionalForm]]],
 " ways to finish defining this ring! However, very few of these ways of \
defining the products will satisfy both the distributive laws and the \
associative laws. For example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", ".", "b"}], TraditionalForm]]],
 " cannot be defined to be ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", otherwise we have the contradiction\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", "a"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"a", " ", "+", " ", "a"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"b", "\[CenterDot]", "b"}], " ", "+", " ", 
       RowBox[{"b", "\[CenterDot]", "b"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"b", " ", "+", " ", "b"}], ")"}], "\[CenterDot]", "b"}], " ",
        "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"2", "b"}], ")"}], "\[CenterDot]", "b"}], " ", "=", " ", 
        RowBox[{
         RowBox[{"0", "\[CenterDot]", "b"}], " ", "=", " ", "0."}]}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nHere is an example of definitions which will not produce such a \
contradiction:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4371862540625*^9, 3.4371862540625*^9}, {
  3.437186480796875*^9, 3.4371864809375*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitRing
Define[4 a, 0]; Define[2 b, 0]
Define[a.a, a]; Define[b.b, b]
Define[a.b, 0]; Define[b.a, 0]
R = Ring[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437186513421875*^9, 3.437186522609375*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.43718652384375*^9, 3.442747491430125*^9}]
}, Open  ]],

Cell["\<\
Here is a table of both the addition and multiplication operation:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell[BoxData["R"], "Input",
 CellChangeTimes->{3.437186569671875*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.437186570234375*^9, 3.442747494117625*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["AddTable[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 
      0, 0}, {1, 1, 0}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {1, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {
      0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 0, 
      0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {0.7, 0.8, 0.7}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {1,
       1, 0}, {1, 0, 1}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 
      0.8}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.442747495805125*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["MultTable[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {
      0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}}, {{
      1, 0.6, 0}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0, 1, 0}, {1, 0.6, 0}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0,
       1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {
      0.7, 0.8, 0.7}}, {{0, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{1, 0, 0}, {1, 1, 0}, {
     1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 
     1}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {
     0, 1, 1}, {1, 1, 0}, {0, 1, 1}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1,
      1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.43718653528125*^9, 3.442747497742625*^9}]
}, Open  ]],

Cell[TextData[{
 "Of course we still have not ",
 StyleBox["proven",
  FontSlant->"Italic"],
 " that this is a ring, since we have not verified the distributive laws and \
the associtivity law for multiplication. This tedious task can be handled my \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckRing[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData["\<\"These elements will generate a ring.\"\>"], "Print",
 CellChangeTimes->{{3.437186544578125*^9, 3.437186583140625*^9}, 
   3.4427475000395*^9}]
}, Open  ]],

Cell[TextData[{
 "Here we used the entire ring as the argument for the ",
 StyleBox["CheckRing",
  FontWeight->"Bold"],
 " command, and found that this was indeed a ring. However, in most of the \
commands in this package it suffices to give ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " just the basis for the additive group. Thus, we could have instead typed"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckRing[{a,b}]", "Input",
 PageWidth->Infinity],

Cell[BoxData["\<\"These elements will generate a ring.\"\>"], "Print",
 CellChangeTimes->{3.44274751025825*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that this ",
 StyleBox["CheckRing",
  FontWeight->"Bold"],
 " command runs much faster when only the generators are given. \n\nSince ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is obviously commutative from the multiplication table, the next question \
is whether ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " has an identity element. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can search the ring for an identity with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["FindIdent[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"a", "+", "b"}]], "Output",
 CellChangeTimes->{3.4371871601875*^9, 3.44274751400825*^9}]
}, Open  ]],

Cell[TextData[{
 "Even though we did not use the identity element to construct the ring, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " found one!\n\nBy looking at the multiplication table"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {
      0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}}, {{
      1, 0.6, 0}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0, 1, 0}, {1, 0.6, 0}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0,
       1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {
      0.7, 0.8, 0.7}}, {{0, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{1, 0, 0}, {1, 1, 0}, {
     1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 
     1}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {
     0, 1, 1}, {1, 1, 0}, {0, 1, 1}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1,
      1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.442747515852*^9}]
}, Open  ]],

Cell[TextData[{
 "we see that there are many zero-divisors. (How many?) Hence, this is not a \
division ring. This ring has similar properties as the ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " that was studied in the last section.\n\nCan we try defining a \
non-commutative ring? It is apparent that we need at least two additive \
generators, since the multiplication is completely determined by the products \
of the generators. Let us still use the group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 " , only this time we will make ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "b"}], " ", "=", " ", "b"}], 
   TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{"2", "a"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing
Define[4 a, 0]
Define[2 b, 0]
Define[a.b, b]
Define[b.a, 2 a]
Define[a.a, a]
Define[b.b, b]
CheckRing[{a,b}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "b", "\[InvisibleSpace]", "\<\" . (\"\>", "\[InvisibleSpace]", "a", 
   "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "b", 
   "\[InvisibleSpace]", "\<\") is not equal to (\"\>", "\[InvisibleSpace]", 
   "b", "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "a", 
   "\[InvisibleSpace]", "\<\") . \"\>", "\[InvisibleSpace]", "b"}],
  SequenceForm[$CellContext`b, " . (", $CellContext`a, " . ", $CellContext`b, 
   ") is not equal to (", $CellContext`b, " . ", $CellContext`a, 
   ") . ", $CellContext`b],
  Editable->False]], "Print",
 CellChangeTimes->{3.442747518805125*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "b", "\[InvisibleSpace]", "\<\" . (\"\>", "\[InvisibleSpace]", "b", 
   "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "a", 
   "\[InvisibleSpace]", "\<\") is not equal to (\"\>", "\[InvisibleSpace]", 
   "b", "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "b", 
   "\[InvisibleSpace]", "\<\") . \"\>", "\[InvisibleSpace]", "a"}],
  SequenceForm[$CellContext`b, " . (", $CellContext`b, " . ", $CellContext`a, 
   ") is not equal to (", $CellContext`b, " . ", $CellContext`b, 
   ") . ", $CellContext`a],
  Editable->False]], "Print",
 CellChangeTimes->{3.44274751882075*^9}],

Cell[BoxData["\<\"Associtive Property does not hold.\"\>"], "Print",
 CellChangeTimes->{3.442747518836375*^9}]
}, Open  ]],

Cell[TextData[{
 "Does this form a ring? No, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " complained that sometimes the associative law doesn't hold. Can we chance \
the definition of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "a"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "b"}], TraditionalForm]]],
 " to make this a ring?\n\nEXPERIMENT:\nBy changing only the elements ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " in the two lines\n\n",
 StyleBox["Define[a.a, a]",
  FontWeight->"Bold"],
 "\n",
 StyleBox["Define[b.b, b]",
  FontWeight->"Bold"],
 "\n\nof the above ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command, see if you can form a ring. \n\nHint: Although this seems like a \
trial and error process with 8\[CenterDot]8 = 64 combinations, all but a \
handful can be eliminated using algebra! (There are actually two different \
solutions.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Assuming that you have found the right definition for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "a"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "b"}], TraditionalForm]]],
 ", you can generate the ring with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["R = Ring[{a,b}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442747550992625*^9}]
}, Open  ]],

Cell["Does this ring have an identity? We can find out:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["FindIdent[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData["\<\"No identity element found.\"\>"], "Print",
 CellChangeTimes->{3.442747548273875*^9}]
}, Open  ]],

Cell["\<\
So there is no identity element. In fact, if we look at the
multiplication table:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["MultTable[R]", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "we find that there is one element which, when multiplied by any element, \
gives 0! Hence every element is a zero-divisor. This example shows that rings \
can have rather bizarre multiplication properties.\n\nSince we have seen an \
example of a non-communitive ring without an identity, can we find a \
non-communitive ring with an identity? The following proposition shows that \
we will not be able to use ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 " for the additive group.\n \nPROPOSITION 9.6\nIf a ring with identity has \
an additive structure which can be generated with less than three elements, \
then the ring is commutative."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437187208515625*^9, 3.43718720996875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are two elements of the ring which generate the group under addition. That \
is, every element can be expressed as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"m", " ", "x"}], " ", "+", " ", 
    RowBox[{"n", " ", "y"}]}], TraditionalForm]]],
 " for integers ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".  In particular, the identity element \n \n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"e", " ", "=", " ", 
    RowBox[{
     RowBox[{"m", " ", "x"}], " ", "+", " ", 
     RowBox[{"n", " ", "y"}]}]}], TraditionalForm]]],
 " \n\nfor some integers ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " commutes with both ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", we have\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"m", " ", 
       RowBox[{"x", "\[CenterDot]", "x"}]}], " ", "+", " ", 
      RowBox[{"n", " ", 
       RowBox[{"y", "\[CenterDot]", "x"}]}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"m", " ", "x"}], " ", "+", " ", 
         RowBox[{"n", " ", "y"}]}], ")"}], "\[CenterDot]", "x"}], " ", "=", 
      " ", 
      RowBox[{
       RowBox[{"e", "\[CenterDot]", "x"}], " ", "=", " ", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "e"}], " ", "=", " ", 
        RowBox[{
         RowBox[{"m", " ", 
          RowBox[{"x", "\[CenterDot]", "x"}]}], " ", "+", " ", 
         RowBox[{"n", " ", 
          RowBox[{"x", "\[CenterDot]", "y"}]}]}]}]}]}]}], ","}], 
   TraditionalForm]]],
 "\n\nso     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", " ", 
     RowBox[{"y", "\[CenterDot]", "x"}]}], " ", "=", " ", 
    RowBox[{"n", " ", 
     RowBox[{"x", "\[CenterDot]", "y"}]}]}], TraditionalForm]]],
 ".\n\nLikewise, \n \n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"m", " ", 
        RowBox[{"x", "\[CenterDot]", "y"}]}], " ", "+", " ", 
       RowBox[{"n", " ", 
        RowBox[{"y", "\[CenterDot]", "y"}]}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"m", " ", "x"}], " ", "+", " ", 
          RowBox[{"n", " ", "y"}]}], ")"}], "\[CenterDot]", "y"}], " ", "=", 
       " ", 
       RowBox[{
        RowBox[{"e", "\[CenterDot]", "y"}], " ", "=", " ", 
        RowBox[{
         RowBox[{"y", "\[CenterDot]", "e"}], " ", "=", " ", 
         RowBox[{
          RowBox[{"m", " ", 
           RowBox[{"y", "\[CenterDot]", "x"}]}], " ", "+", " ", 
          RowBox[{"n", " ", 
           RowBox[{"y", "\[CenterDot]", "y"}]}]}]}]}]}]}], ","}]}], 
   TraditionalForm]]],
 "\n\nso   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"m", " ", 
      RowBox[{"x", "\[CenterDot]", "y"}]}], " ", "=", " ", 
     RowBox[{"m", " ", 
      RowBox[{"y", "\[CenterDot]", 
       RowBox[{"x", "."}]}]}]}]}], TraditionalForm]]],
 "\n\nBy the greatest common divisor theorem (1.2), there are integers ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " such that \n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"u", " ", "m"}], " ", "+", " ", 
     RowBox[{"v", " ", "n"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"GCD", "[", 
      RowBox[{"m", ",", "n"}], "]"}], "."}]}], TraditionalForm]]],
 "\n\nIf we let ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " denote the greatest common divisor of ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", then\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"c", "(", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "y"}], " ", "-", " ", 
        RowBox[{"y", "\[CenterDot]", "x"}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"u", " ", "m"}], " ", "+", " ", 
         RowBox[{"v", " ", "n"}]}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "y"}], " ", "-", " ", 
         RowBox[{"y", "\[CenterDot]", "x"}]}], ")"}]}]}], 
     "                     ", "\n", "\t\t\t                   ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"u", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"m", " ", 
            RowBox[{"x", "\[CenterDot]", "y"}]}], " ", "-", " ", 
           RowBox[{"m", " ", 
            RowBox[{"y", "\[CenterDot]", "x"}]}]}], ")"}]}], " ", "+", " ", 
        RowBox[{"v", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"n", " ", 
            RowBox[{"x", "\[CenterDot]", "y"}]}], " ", "-", " ", 
           RowBox[{"n", " ", 
            RowBox[{"y", "\[CenterDot]", "x"}]}]}], ")"}]}]}], " ", "=", " ", 
       "0."}]}]}]}], TraditionalForm]]],
 "\n\nWhat we need to show is that",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", "y"}], " ", "-", " ", 
       RowBox[{"y", "\[CenterDot]", "x"}]}], ")"}], " ", "=", " ", "0."}]}], 
   TraditionalForm]]],
 " The tempting thing to do is divide by ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", but this operation is not allowed in rings. Instead, we will again \
utilize the identity element. Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"c", " ", "=", " ", 
     RowBox[{"GCD", "[", 
      RowBox[{"m", ",", "n"}], "]"}]}], ","}], TraditionalForm]]],
 " there are integers ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " such that  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "=", " ", 
    RowBox[{"a", " ", "c"}]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", 
    RowBox[{"b", " ", "c"}]}], TraditionalForm]]],
 ".\n \nThen \n\n  ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "y"}], " ", "-", " ", 
      RowBox[{"y", "\[CenterDot]", "x"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"e", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "y"}], " ", "-", " ", 
         RowBox[{"y", "\[CenterDot]", "x"}]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"a", " ", "c", " ", "x"}], " ", "+", " ", 
         RowBox[{"b", " ", "c", " ", "y"}]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "y"}], " ", "-", " ", 
         RowBox[{"y", "\[CenterDot]", "x"}]}], ")"}]}]}]}], " "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t\t         ", 
    RowBox[{"=", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"a", " ", "x"}], " ", "+", " ", 
        RowBox[{"b", " ", "y"}]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", " ", 
       RowBox[{"c", "(", 
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "y"}], " ", "-", " ", 
         RowBox[{"y", "\[CenterDot]", "x"}]}], ")"}], ")"}]}]}], " "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t\t\t      ", 
    RowBox[{"=", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"a", " ", "x"}], " ", "+", " ", 
         RowBox[{"b", " ", "y"}]}], ")"}], "\[CenterDot]", " ", "0"}], " ", 
      "=", " ", "0."}]}]}], TraditionalForm]}]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "x"}]}], TraditionalForm]]],
 ", and the ring is commutative. "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4371873380625*^9, 3.437187338171875*^9}}]
}, Open  ]],

Cell[TextData[{
 "\nThe smallest group which requires more than two generators is the group \
",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ". So naturally we should begin with this group in looking for a \
non-communative ring with an identity. We may suppose that the ring is \
additively generated by the multiplicative identity ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ", along with two other elements ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ". With this information, we can make the following definitions:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
InitRing;
Define[2 e,0];
Define[2 a,0];
Define[2 b,0];
Define[e.e,e];
Define[e.a,a];
Define[e.b,b];
Define[a.e,a];
Define[b.e,b];\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "The last five definitions force ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " to be a multiplicative identity. \n\nFinally, we have to define ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "a"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "a"}], ","}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", 
    RowBox[{"b", "."}]}], TraditionalForm]]],
 " Suppose we try to finish defining the ring in the following way:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[2 e,0];
Define[2 a,0];
Define[2 b,0];
Define[e.e,e];
Define[e.a,a];
Define[e.b,b];
Define[a.e,a];
Define[b.e,b];
Define[a.b,a];
Define[b.a,b];
Define[a.a,0];
Define[b.b,0];
CheckRing[{a,b,e}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "a", "\[InvisibleSpace]", "\<\" . (\"\>", "\[InvisibleSpace]", "b", 
   "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "a", 
   "\[InvisibleSpace]", "\<\") is not equal to (\"\>", "\[InvisibleSpace]", 
   "a", "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "b", 
   "\[InvisibleSpace]", "\<\") . \"\>", "\[InvisibleSpace]", "a"}],
  SequenceForm[$CellContext`a, " . (", $CellContext`b, " . ", $CellContext`a, 
   ") is not equal to (", $CellContext`a, " . ", $CellContext`b, 
   ") . ", $CellContext`a],
  Editable->False]], "Print",
 CellChangeTimes->{3.442747565352*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "a", "\[InvisibleSpace]", "\<\" . (\"\>", "\[InvisibleSpace]", "b", 
   "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "b", 
   "\[InvisibleSpace]", "\<\") is not equal to (\"\>", "\[InvisibleSpace]", 
   "a", "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "b", 
   "\[InvisibleSpace]", "\<\") . \"\>", "\[InvisibleSpace]", "b"}],
  SequenceForm[$CellContext`a, " . (", $CellContext`b, " . ", $CellContext`b, 
   ") is not equal to (", $CellContext`a, " . ", $CellContext`b, 
   ") . ", $CellContext`b],
  Editable->False]], "Print",
 CellChangeTimes->{3.442747565352*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "b", "\[InvisibleSpace]", "\<\" . (\"\>", "\[InvisibleSpace]", "a", 
   "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "a", 
   "\[InvisibleSpace]", "\<\") is not equal to (\"\>", "\[InvisibleSpace]", 
   "b", "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "a", 
   "\[InvisibleSpace]", "\<\") . \"\>", "\[InvisibleSpace]", "a"}],
  SequenceForm[$CellContext`b, " . (", $CellContext`a, " . ", $CellContext`a, 
   ") is not equal to (", $CellContext`b, " . ", $CellContext`a, 
   ") . ", $CellContext`a],
  Editable->False]], "Print",
 CellChangeTimes->{3.442747565367625*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
  "b", "\[InvisibleSpace]", "\<\" . (\"\>", "\[InvisibleSpace]", "a", 
   "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "b", 
   "\[InvisibleSpace]", "\<\") is not equal to (\"\>", "\[InvisibleSpace]", 
   "b", "\[InvisibleSpace]", "\<\" . \"\>", "\[InvisibleSpace]", "a", 
   "\[InvisibleSpace]", "\<\") . \"\>", "\[InvisibleSpace]", "b"}],
  SequenceForm[$CellContext`b, " . (", $CellContext`a, " . ", $CellContext`b, 
   ") is not equal to (", $CellContext`b, " . ", $CellContext`a, 
   ") . ", $CellContext`b],
  Editable->False]], "Print",
 CellChangeTimes->{3.44274756538325*^9}],

Cell[BoxData["\<\"Associtive Property does not hold.\"\>"], "Print",
 CellChangeTimes->{3.44274756538325*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " complains that this is not a ring. Can we try changing the last two \
definitions so that this forms a ring?\n"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "EXPERIMENT: \nCan you form a ring by changing the zeros in the lines\n\n",
 StyleBox["Define[a.a, 0]",
  FontWeight->"Bold"],
 "\n",
 StyleBox["Define[b.b, 0]",
  FontWeight->"Bold"],
 "\n\nin the previous ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command? There is a unique answer, and it can be found without resorting \
to trial and error methods."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Once we have completed the definition of the ring, we can look at the \
addition and multiplication tables.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
R = Ring[{e,a,b}]
AddTable[R]\
\>", "Input",
 PageWidth->Infinity],

Cell["MultTable[R]", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We can verify by looking at the tables that this ring indeed has an \
identity, or we can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find it."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["FindIdent[R]", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "It is easy to see any finite ring can be quickly entered into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". In fact many infinite rings such as the Quaternions, can also be explored \
with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". This will allow us to experiment with many different rings, and find \
properties which are common to all rings. In the next\nsection we will look \
at some of the basic relationships between rings. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Some Properties of Rings", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Now that we can enter finite rings into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", let us turn our attention to using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to help us discover some truths about rings. In particular, we want to \
study in what circumstances multiplicative inverses exist. \n\nOne of the \
simpliest rings to study are the rings ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 "  for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "1."}], TraditionalForm]]],
 "\n\nWe have learned how to define the addition structure with a ",
 StyleBox["DefSumMod",
  FontWeight->"Bold"],
 " command, and the multiplication can be defined using a ",
 StyleBox["DefMultMod",
  FontWeight->"Bold"],
 " command. We actually can define both of these at once with the command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["DefMod[15]", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "This defines both the addition and multiplication operations at the same \
time. The ring is generated by the identity element, 1. Thus, the elements of \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "15"], TraditionalForm]]],
 "   are:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Z15 = Ring[{1}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", 
   ",", "8", ",", "9", ",", "10", ",", "11", ",", "12", ",", "13", ",", 
   "14"}], "}"}]], "Output",
 CellChangeTimes->{3.43718745865625*^9, 3.4427475799145*^9}]
}, Open  ]],

Cell[TextData[{
 "We can see both the addition and multiplication table for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "15"], TraditionalForm]]],
 " :"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["AddTable[Z15]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.2, 0.8, 0.7}, {0.2, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}}, {{1, 0.4, 0.4}, {1, 0.4, 0.4}, {
      0.2, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}}, {{0.6, 
      0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 1, 0}, {
      1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {0.8, 0.6, 0.2}}, {{0.8, 0.6, 0.2}, {0.8, 0.6, 0.2}, {0.6, 
      0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}}, {{0.8, 0.2, 
      0.5}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6,
       0.5}, {0.5, 0.6, 0.8}}, {{0.5, 0.6, 0.8}, {0.5, 0.6, 0.8}, {0.8, 0.2, 
      0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 
      0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}}, {{0.8, 0.6, 
      0.5}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 1, 0}, {1, 0,
       1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {
      0.7, 0.8, 0.7}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 
      0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 1, 0}, {1, 0,
       1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4,
       0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {
      0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 
      0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 0, 0}, {1, 
      0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 
      0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6,
       0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 
      0.7}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6,
       0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 
      0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 
      0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 
      0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 15.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {11.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {12.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"12\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {13.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {14.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {15.5, 15.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 14.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 13.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 12.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 11.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"12\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 14.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 13.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 12.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 11.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 10.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 15}, {16, 15}}]}]], "Output",
 CellChangeTimes->{3.44274758232075*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["MultTable[Z15]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.2, 0.8, 0.7}, {1, 1, 0}, {0.2, 0.8, 0.7}, {1, 0.4, 0.4}, {
      0.6, 0.6, 0.2}, {0.8, 0.6, 0.2}, {0.8, 0.2, 0.5}, {0.5, 0.6, 0.8}, {0.8,
       0.6, 0.5}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0, 1, 0}, {
      1, 0, 0}, {0, 1, 1}, {1, 0, 1}}, {{1, 0.4, 0.4}, {1, 1, 0}, {
      1, 0.4, 0.4}, {0.8, 0.6, 0.2}, {0.5, 0.6, 0.8}, {0.7, 0.8, 0.7}, {0.6, 
      0.4, 0.8}, {1, 0, 0}, {1, 0, 1}, {0.2, 0.8, 0.7}, {0.6, 0.6, 0.2}, {0.8,
       0.2, 0.5}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}}, {{0.6, 
      0.6, 0.2}, {1, 1, 0}, {0.6, 0.6, 0.2}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {1,
       0, 0}, {1, 1, 0}, {0.6, 0.6, 0.2}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {1, 0,
       0}, {1, 1, 0}, {0.6, 0.6, 0.2}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {1, 0, 
      0}}, {{0.8, 0.6, 0.2}, {1, 1, 0}, {0.8, 0.6, 0.2}, {0.7, 0.8, 0.7}, {1, 
      0, 0}, {0.2, 0.8, 0.7}, {0.8, 0.2, 0.5}, {1, 0.6, 0}, {0, 1, 1}, {
      1, 0.4, 0.4}, {0.5, 0.6, 0.8}, {0.6, 0.4, 0.8}, {1, 0, 1}, {0.6, 0.6, 
      0.2}, {0.8, 0.6, 0.5}, {0, 1, 0}}, {{0.8, 0.2, 0.5}, {1, 1, 0}, {0.8, 
      0.2, 0.5}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.8, 0.2, 0.5}, {0.6, 0.4, 
      0.8}, {1, 1, 0}, {0.8, 0.2, 0.5}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.8, 0.2,
       0.5}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.8, 0.2, 0.5}, {0.6, 0.4, 0.8}}, {{
      0.5, 0.6, 0.8}, {1, 1, 0}, {0.5, 0.6, 0.8}, {1, 0, 0}, {0.6, 0.6, 
      0.2}, {1, 0.6, 0}, {1, 1, 0}, {0.5, 0.6, 0.8}, {1, 0, 0}, {0.6, 0.6, 
      0.2}, {1, 0.6, 0}, {1, 1, 0}, {0.5, 0.6, 0.8}, {1, 0, 0}, {0.6, 0.6, 
      0.2}, {1, 0.6, 0}}, {{0.8, 0.6, 0.5}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0,
       1}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0.8, 0.2, 0.5}, {1, 0, 0}, {0.8, 0.6, 
      0.2}, {0, 1, 0}, {0.6, 0.6, 0.2}, {0.6, 0.4, 0.8}, {1, 0.4, 0.4}, {
      1, 0.6, 0}, {0.2, 0.8, 0.7}, {0.7, 0.8, 0.7}}, {{0.7, 0.8, 0.7}, {1, 1, 
      0}, {0.7, 0.8, 0.7}, {0.2, 0.8, 0.7}, {1, 0.6, 0}, {1, 0.4, 0.4}, {0.6, 
      0.4, 0.8}, {0.6, 0.6, 0.2}, {0, 1, 0}, {0.8, 0.6, 0.2}, {1, 0, 0}, {0.8,
       0.2, 0.5}, {0, 1, 1}, {0.5, 0.6, 0.8}, {1, 0, 1}, {0.8, 0.6, 0.5}}, {{
      1, 0.6, 0}, {1, 1, 0}, {1, 0.6, 0}, {0.6, 0.6, 0.2}, {1, 0, 0}, {0.5, 
      0.6, 0.8}, {1, 1, 0}, {1, 0.6, 0}, {0.6, 0.6, 0.2}, {1, 0, 0}, {0.5, 
      0.6, 0.8}, {1, 1, 0}, {1, 0.6, 0}, {0.6, 0.6, 0.2}, {1, 0, 0}, {0.5, 
      0.6, 0.8}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0.8, 0.2, 
      0.5}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0.8, 0.2, 0.5}, {1, 1, 0}, {0.6, 0.4,
       0.8}, {0.8, 0.2, 0.5}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0.8, 0.2, 0.5}, {1,
       1, 0}, {0.6, 0.4, 0.8}, {0.8, 0.2, 0.5}}, {{0, 1, 0}, {1, 1, 0}, {0, 1,
       0}, {0.8, 0.6, 0.5}, {0.6, 0.6, 0.2}, {1, 0, 1}, {0.6, 0.4, 0.8}, {0.5,
       0.6, 0.8}, {1, 0.4, 0.4}, {0, 1, 1}, {1, 0.6, 0}, {0.8, 0.2, 0.5}, {
      0.2, 0.8, 0.7}, {1, 0, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.2}}, {{1, 0, 
      0}, {1, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0.6, 0.6, 
      0.2}, {1, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0.6, 0.6, 
      0.2}, {1, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0.6, 0.6, 
      0.2}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 
      0.6, 0.5}, {0.8, 0.2, 0.5}, {0.6, 0.6, 0.2}, {0.2, 0.8, 0.7}, {1, 0, 
      1}, {1, 0, 0}, {0.6, 0.4, 0.8}, {0.7, 0.8, 0.7}, {0.5, 0.6, 0.8}, {0.8, 
      0.6, 0.2}, {1, 0.4, 0.4}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 
      0.2}, {0.6, 0.6, 0.2}, {1, 0.4, 0.4}, {0.2, 0.8, 0.7}}, {{1, 1, 0}, {1, 
     1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}, {0.8, 0.2, 0.5}, {0.8, 0.6, 0.2}, {0.6, 0.6, 0.2}, {
      1, 0.4, 0.4}, {0.2, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 15.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {11.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {12.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"12\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {13.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {14.5, 15.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {15.5, 15.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 14.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 13.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 12.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 11.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"12\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 14.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 14.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 13.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 13.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 12.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 12.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 11.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 11.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 10.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 10.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"13\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"11\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {11.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {12.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {13.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {14.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {15.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 15}, {16, 15}}]}]], "Output",
 CellChangeTimes->{3.442747584336375*^9}]
}, Open  ]],

Cell[TextData[{
 "We can perform simple operations in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "15"], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["7 + 9", "Input",
 PageWidth->Infinity],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.442747587086375*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["7 . 9", "Input",
 PageWidth->Infinity],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{3.442747588492625*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["1/7", "Input",
 PageWidth->Infinity],

Cell[BoxData["13"], "Output",
 CellChangeTimes->{3.44274758982075*^9}]
}, Open  ]],

Cell[TextData[{
 "This last operation shows that we can take multiplicative inverses of some \
of the elements.\n\nEXPERIMENT:\nChange the 15 in the following command to \
other positive integers. This produces both a multiplication table for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 "  along with a circle graph of the inverse function."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
DefMod[15]
R = Ring[{1}];
MultTable[R];
CircleGraph[R,Inv]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.40673664307580015`, 0.9135454576426009}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7431448254773941, 0.6691306063588582}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9510565162951535, 0.30901699437494745`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9945218953682734, -0.10452846326765333`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8660254037844387, -0.4999999999999998}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5877852522924732, -0.8090169943749473}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.20791169081775931`, -0.9781476007338057}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.20791169081775907`, -0.9781476007338057}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.587785252292473, -0.8090169943749476}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.8660254037844384, -0.5000000000000004}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9945218953682733, -0.10452846326765423`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9510565162951536, 0.30901699437494723`}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.743144825477394, 0.6691306063588585}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.40673664307580015`, 0.913545457642601}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], {}, {InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.4474103073833802, 1.0049000034068611`}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8174593080251336, 0.7360436669947441}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0461621679246689`, 0.3399186938124422}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.0939740849051007`, -0.11498130959441867`}, {-1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9526279441628827, -0.5499999999999998}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.6465637775217207, -0.8899186938124422}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.22870285989953526`, -1.0759623608071864`}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.22870285989953498`, -1.0759623608071864`}, {1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.6465637775217203, -0.8899186938124424}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9526279441628823, -0.5500000000000005}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"11\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.0939740849051007`, -0.11498130959441967`}, {1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"12\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.046162167924669, 0.339918693812442}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"13\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.8174593080251334, 0.7360436669947443}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.4474103073833802, 1.0049000034068611`}, {1, 0}]}, {{
     InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[1, 0, 0], LineBox[{{0., 1.}, {0., 0.1}}], 
      PolygonBox[{{0., 0.1}, {0.05000000000000001, 
       0.2}, {-0.05000000000000001, 0.2}}]}}, 
    {RGBColor[0, 1, 0], 
     LineBox[{{0.31337860042607996`, 0.8777086626880708}, {0.2803460665077814,
       0.8568879589791071}, {0.2578746451944051, 0.82501814215885}, {
      0.24922707150468354`, 0.7869935518352078}, {0.25585735273088983`, 
      0.7485138250838206}, {0.27667805643985355`, 0.7154812911655221}, {
      0.3085766338839582, 0.6930098698521457}, {0.34657246358375293`, 
      0.6843622961624242}, {0.38505219033514004`, 0.6909925773886305}, {
      0.41808472425343857`, 0.7118132810975942}, {0.4405561455668149, 
      0.7436830979178513}, {0.44920371925653646`, 0.7817076882414935}, {
      0.44257343803033017`, 0.8201874149928807}, {0.40673664307580015`, 
      0.9135454576426009}}], 
     PolygonBox[{{0.40673664307580015`, 0.9135454576426009}, {
      0.2954602029488149, 0.9243876840129309}, {0.331296997903345, 
      0.8310296413632108}}]}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0.7431448254773941, 
       0.6691306063588582}, {-0.20791169081775907`, -0.9781476007338057}}], 
      PolygonBox[{{-0.20791169081775907`, -0.9781476007338057}, \
{-0.11461042062853712`, -0.9165450603553618}, {-0.20121296100698102`, \
-0.8665450603553618}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.20791169081775907`, -0.9781476007338057}, {
       0.7431448254773941, 0.6691306063588582}}], 
      PolygonBox[{{0.7431448254773941, 0.6691306063588582}, {
       0.6498435552881722, 0.6075280659804143}, {0.7364460956666161, 
       0.5575280659804144}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[1, 0.6, 0], 
      LineBox[{{0.9510565162951535, 0.30901699437494745`}, {
       0.09510565162951536, 0.030901699437494747`}}], 
      PolygonBox[{{0.09510565162951536, 0.030901699437494747`}, {
       0.2056621529777781, 0.014250573060231805`}, {0.17476045354028336`, 
       0.10935622468974718`}}]}}, 
    {RGBColor[0.7, 0.8, 0.7], 
     LineBox[{{0.9315898562632896, -0.026813867121956242`}, {
      0.9015805759790079, -0.0018320117701330213`}, {0.8643265279453769, 
      0.009691264895291346}, {0.8254907463083896, 0.006165351588892878}, {
      0.7909432210126088, -0.012031310081242919`}, {
      0.7659613656607855, -0.04204059036552469}, {
      0.7545084123129866, -0.0792946383991557}, {
      0.7579640023017598, -0.11813042003614302`}, {
      0.7761606639718955, -0.15267794533192378`}, {
      0.8061699442561773, -0.17765980068374698`}, {
      0.8434239922898082, -0.18918307734917136`}, {
      0.8822597739267956, -0.18565716404277288`}, {
      0.9168072992225763, -0.1674605023726371}, {
      0.9945218953682734, -0.10452846326765333`}}], 
     PolygonBox[{{0.9945218953682734, -0.10452846326765333`}, {
      0.9704471543361382, 0.004652152430535614}, {
      0.8927325581904411, -0.058279886674448125`}}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.8, 0.6, 0.5], 
      LineBox[{{0.8660254037844387, -0.4999999999999998}, {
       0.08660254037844388, -0.04999999999999998}}], 
      PolygonBox[{{0.08660254037844388, -0.04999999999999998}, {
       0.14820508075688776`, -0.1433012701892219}, {
       0.19820508075688775`, -0.056698729810778024`}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.5, 0.6, 0.8], 
      LineBox[{{0.5877852522924732, -0.8090169943749473}, {
       0.05877852522924733, -0.08090169943749474}}], 
      PolygonBox[{{0.05877852522924733, -0.08090169943749474}, {
       0.07710620073974728, -0.19119266148961317`}, {
       0.15800790017724203`, -0.13241413626036583`}}]}}, {
     {RGBColor[0.8, 0.2, 0.5], 
      LineBox[{{
       0.20791169081775931`, -0.9781476007338057}, {-0.743144825477394, 
       0.6691306063588585}}], 
      PolygonBox[{{-0.743144825477394, 
       0.6691306063588585}, {-0.736446095666616, 
       0.5575280659804146}, {-0.6498435552881721, 0.6075280659804145}}]}, 
     {RGBColor[0.8, 0.2, 0.5], 
      LineBox[{{-0.743144825477394, 0.6691306063588585}, {
       0.20791169081775931`, -0.9781476007338057}}], 
      PolygonBox[{{0.20791169081775931`, -0.9781476007338057}, {
       0.20121296100698127`, -0.8665450603553618}, {
       0.1146104206285374, -0.9165450603553618}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.8, 0.6, 0.2], 
      LineBox[{{-0.587785252292473, -0.8090169943749476}, \
{-0.05877852522924731, -0.08090169943749476}}], 
      PolygonBox[{{-0.05877852522924731, -0.08090169943749476}, \
{-0.158007900177242, -0.13241413626036586`}, {-0.07710620073974724, \
-0.19119266148961317`}}]}}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.6, 0.6, 0.2], 
      LineBox[{{-0.8660254037844384, -0.5000000000000004}, \
{-0.08660254037844384, -0.050000000000000044`}}], 
      PolygonBox[{{-0.08660254037844384, -0.050000000000000044`}, \
{-0.1982050807568877, -0.05669872981077816}, {-0.14820508075688765`, \
-0.143301270189222}}]}}, 
    {RGBColor[1, 0.4, 0.4], 
     LineBox[{{-0.9168072992225762, -0.16746050237263793`}, \
{-0.8822597739267954, -0.18565716404277371`}, {-0.843423992289808, \
-0.1891830773491721}, {-0.806169944256177, -0.1776598006837477}, \
{-0.7761606639718953, -0.15267794533192447`}, {-0.7579640023017595, \
-0.11813042003614371`}, {-0.7545084123129865, -0.07929463839915638}, \
{-0.7659613656607855, -0.04204059036552539}, {-0.7909432210126088, \
-0.012031310081243654`}, {-0.8254907463083896, 
      0.006165351588892129}, {-0.8643265279453769, 
      0.009691264895290555}, {-0.9015805759790079, -0.001832011770133854}, \
{-0.9315898562632896, -0.02681386712195709}, {-0.9945218953682733, \
-0.10452846326765423`}}], 
     PolygonBox[{{-0.9945218953682733, -0.10452846326765423`}, \
{-0.948273318775068, -0.20631780044548648`}, {-0.8853412796700844, \
-0.12860320429978936`}}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"?\"\>",
        FontSize->18,
        FontWeight->"Bold"],
       StyleForm[#, FontSize -> 18, FontWeight -> "Bold"]& ], {0, 0}], 
     {RGBColor[0.2, 0.8, 0.7], 
      LineBox[{{-0.9510565162951536, 
       0.30901699437494723`}, {-0.09510565162951537, 0.030901699437494726`}}],
       PolygonBox[{{-0.09510565162951537, 
       0.030901699437494726`}, {-0.17476045354028336`, 
       0.10935622468974714`}, {-0.20566215297777812`, 
       0.01425057306023177}}]}}, 
    {RGBColor[1, 0.6, 1], 
     LineBox[{{-0.44257343803033017`, 
      0.8201874149928808}, {-0.44920371925653646`, 
      0.7817076882414937}, {-0.4405561455668149, 
      0.7436830979178514}, {-0.4180847242534386, 
      0.7118132810975943}, {-0.38505219033514004`, 
      0.6909925773886306}, {-0.34657246358375293`, 
      0.6843622961624243}, {-0.3085766338839582, 
      0.6930098698521459}, {-0.27667805643985355`, 
      0.7154812911655222}, {-0.25585735273088983`, 
      0.7485138250838207}, {-0.24922707150468354`, 
      0.7869935518352078}, {-0.2578746451944051, 
      0.8250181421588502}, {-0.28034606650778143`, 
      0.8568879589791072}, {-0.31337860042608, 
      0.8777086626880709}, {-0.40673664307580015`, 0.913545457642601}}], 
     PolygonBox[{{-0.40673664307580015`, 
      0.913545457642601}, {-0.4892524593551903, 
      0.8381058124701458}, {-0.3958944167054701, 0.8022690175156157}}]}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.44274759225825*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
In each case, observe from the circle graph which elements have \
multiplicative inverses. Which elements are zero divisors? Is there a \
relationship between the invertable elements and the zero divisors? Can you \
replace the 15 with some integer so that the ring has no zero divisors?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[" ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Before going on, we should double check that it is imposible for an element \
to have ",
 StyleBox["two",
  FontSlant->"Italic"],
 " inverses. The proof is similar to the corrisponding result for groups."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "LEMMA 9.3\nLet ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be an element in a ring with identity. Then if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " has an multiplicative inverse, the inverse is unique. We denote the \
multiplicative inverse of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437187485046875*^9, 3.437187485828125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " are two inverses of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". Then\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", " ", "=", " ", 
     RowBox[{
      RowBox[{"y", "\[CenterDot]", "e"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"y", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"x", "\[CenterDot]", "z"}], ")"}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"y", "\[CenterDot]", "x"}], ")"}], "\[CenterDot]", "z"}], 
        " ", "=", " ", 
        RowBox[{
         RowBox[{"e", "\[CenterDot]", "z"}], " ", "=", " ", "z"}]}]}]}]}], 
    ","}], TraditionalForm]]],
 "\n\nwhich is a contradiction."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "PROPOSITION 9.7\nIf ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " has an identity element, then the invertible elements of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " form a group under multiplication. This group is denoted ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "*"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43718751140625*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since the identity element is invertible, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "*"], TraditionalForm]]],
 " is non-empty. Also, if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is invertible, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]], ")"}], 
     RowBox[{"-", "1"}]], " ", "=", " ", "x"}], TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " is also in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "*"], TraditionalForm]]],
 ". Finally, if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are both invertible, then since\n   \n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["y", 
         RowBox[{"-", "1"}]], "\[CenterDot]", 
        SuperscriptBox["x", 
         RowBox[{"-", "1"}]]}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", 
       SuperscriptBox["x", 
        RowBox[{"-", "1"}]]}], " ", "=", " ", "e"}]}]}], TraditionalForm]]],
 ",\n\nwe see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is invertible. Thus, the set of invertible elements forms a group."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "Now we can return our attention to the ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". In trying to find values for ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " in which ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 "  has no zero divisors, you may have noticed the following:\n\nPROPOSITION \
9.8\nFor ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "1"}], TraditionalForm]]],
 ", the ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " has no zero divisors if and only if ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is prime."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4371875385625*^9, 3.437187539109375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First suppose that ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is not prime. Then we can express ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", 
    RowBox[{"a", " ", "b"}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are less then ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " represents the identity element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ", we would then have\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", " ", "e"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"b", " ", "e"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", " ", "b"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"e", "\[CenterDot]", "e"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"a", " ", "b"}], ")"}], " ", "e"}], " ", "=", " ", 
       RowBox[{
        RowBox[{"n", " ", "e"}], " ", "=", " ", "0."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nBut since ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are both less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", " ", "e"}], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"b", " ", "e"}], ")"}], TraditionalForm]]],
 " are both non-zero. Hence, these would both be zero divisors in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ".\n\nNow suppose that ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is prime, and that there are two non-zero elements",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"(", 
     RowBox[{"a", " ", "e"}], ")"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"b", " ", "e"}], ")"}], TraditionalForm]]],
 " such that  \n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", "e"}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"b", " ", "e"}], ")"}]}], " ", "=", " ", "0."}], 
   TraditionalForm]]],
 "  Then\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", " ", "e"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"b", " ", "e"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", " ", "b"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"e", "\[CenterDot]", "e"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"a", " ", "b"}], ")"}], " ", "e"}], " ", "=", " ", 
       "0."}]}]}]}], TraditionalForm]]],
 "\n\nThis would imply that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", " ", "b"}], ")"}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". But since ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is prime, we would have to conclude that either ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". But this contradicts the fact that both",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"(", 
     RowBox[{"a", " ", "e"}], ")"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"b", " ", "e"}], ")"}], TraditionalForm]]],
 " are non-zero. Thus, if ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is prime, there are no zero divisors in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "Even if ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is not prime, one of the observations that can be made while studying ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " is that the zero divisors were precisely the non-zero elements which did \
not have an inverse. Is this true in the other rings that we have studied? \
For example, here is multiplication table of the non-commutative ring with an \
identity of order 8 that we found before:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[2 e,0];
Define[2 a,0];
Define[2 b,0];
Define[e.e,e];
Define[e.a,a];
Define[e.b,b];
Define[a.e,a];
Define[b.e,b];
Define[a.b,a];
Define[b.a,b];
Define[a.a,a];
Define[b.b,b];
MultTable[Ring[{e,a,b}]]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {1, 1, 0}, {0, 1, 1}, {1, 0, 1}, {1, 0, 0}, {
      0.7, 0.8, 0.7}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0, 1, 0}}, {{
      1, 0.6, 0}, {1, 1, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.6, 0.4, 0.8}}, {{0.6, 0.4, 0.8}, {1, 1, 
      0}, {1, 1, 0}, {1, 0, 0}, {1, 0, 0}, {0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {1, 0.6, 0}}, {{0, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {
      1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{
     1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 0, 0}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 1}, {
     1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {1, 1, 0}, {0, 1, 1}}, {{1, 0, 1}, {1, 1,
      0}, {1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {1, 1, 0}, {1, 1, 0}, {
     1, 0, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1,
      0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.4427475997895*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that there are two invertable elements, determined by the 2 ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 "'s inside the table. The other 5 non-zero elements can be seen to be zero \
divisors. This suggests a pattern, but before we can prove that this pattern \
continues to other rings, we will need the following lemma.\n\nLEMMA 9.4\nLet \
",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " be elements of a ring. If ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is non-zero, and is not a zero divisor, and \n\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "\[CenterDot]", "b"}], " ", "=", " ", 
      RowBox[{"a", "\[CenterDot]", "c"}]}], ","}]}], TraditionalForm]]],
 "\n\nthen ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", 
    RowBox[{"c", "."}]}], TraditionalForm]]],
 " Likewise, if\n          \n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{"c", "\[CenterDot]", "a"}]}], TraditionalForm]]],
 "\n\nfor a non-zero and not a zero divisor, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", "c"}], TraditionalForm]]],
 ". This is called the ",
 StyleBox["cancellation law for multiplication",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437187591703125*^9, 3.437187591828125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The tempting thing to do is to multiply both sides of the equation by ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["a", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 ". But the inverse of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " may not exist, so we have to use the properties of rings instead.\n\nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"a", "\[CenterDot]", "b"}], " ", "=", " ", 
     RowBox[{"a", "\[CenterDot]", "c"}]}], ","}], TraditionalForm]]],
 " then we have\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"0", "  ", "=", "  ", 
     RowBox[{
      RowBox[{
       RowBox[{"a", "\[CenterDot]", "b"}], " ", "-", " ", 
       RowBox[{"a", "\[CenterDot]", "c"}]}], "  ", "=", "  ", 
      RowBox[{"a", "\[CenterDot]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"b", " ", "-", " ", "c"}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nBut since ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is not a zero-divisor and is non-zero, we must have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", " ", "-", " ", "c"}], " ", "=", " ", "0."}], 
   TraditionalForm]]],
 " Hence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", 
    RowBox[{"c", "."}]}], TraditionalForm]]],
 "\n\nLikewise, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", 
    RowBox[{"c", "\[CenterDot]", "a"}]}], TraditionalForm]]],
 ", then\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"0", "  ", "=", "  ", 
      RowBox[{
       RowBox[{
        RowBox[{"b", "\[CenterDot]", "a"}], " ", "-", " ", 
        RowBox[{"c", "\[CenterDot]", "a"}]}], "  ", "=", "  ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"b", " ", "-", " ", "c"}], ")"}], "\[CenterDot]", "a"}]}]}], 
     ","}]}], TraditionalForm]]],
 "\n\nand since a is non-zero and not a zero divisor,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", " ", "-", " ", "c"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", 
    RowBox[{"c", "."}]}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "We are now ready to show a relationship between zero-divisors and \
invertable elements. Notice that in the ring ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ", the element 2 is not invertible, but neither is it a zero divisor. This \
example seems to break the pattern that we have been observing, but also \
notice that ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is an ",
 StyleBox["infinite",
  FontSlant->"Italic"],
 " ring. Perhaps if we consider only ",
 StyleBox["finite",
  FontSlant->"Italic"],
 " rings, that is, rings with only a finite number of elements, we will be \
able to prove that a relationship between zero-divisors and invertable \
elements will always exist.\n\nPROPOSITION 9.9\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a finite ring. If ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is a non-zero element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which is not a zero divisor, then ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " has an identity element and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " has a multiplicative inverse in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Hence, every non-zero element in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is either a zero divisor or is invertable."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43718763553125*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "To utilize the fact that ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is finite, let us construct a sequence of powers of ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ":\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"{", " ", 
      RowBox[{
       SuperscriptBox["b", "1"], ",", " ", 
       SuperscriptBox["b", "2"], ",", " ", 
       SuperscriptBox["b", "3"], ",", " ", "\[Ellipsis]"}], " ", "}"}], 
     "."}]}], TraditionalForm]]],
 "\n\nSince ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is finite, two elements of this sequence must be equal, say\n\n            \
          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"b", " "}], "m"], " ", "=", "  ", 
    SuperscriptBox["b", "n"]}], TraditionalForm]]],
 "\n                      \nfor ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "<", " ", 
    RowBox[{"n", "."}]}], TraditionalForm]]],
 "  Using the law of cancellation, we have  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["b", 
     RowBox[{"m", "-", "1"}]], "=", " ", 
    SuperscriptBox["b", 
     RowBox[{"n", "-", "1"}]]}], TraditionalForm]]],
 ". Continuing this way, we eventually get\n\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"b", " ", "=", " ", 
     RowBox[{
      SuperscriptBox["b", 
       RowBox[{"n", "-", "m", "+", "1"}]], "."}]}]}], TraditionalForm]]],
 " \n\n(It is tempting to use lemma 9.4 one more time to get\n\n              \
         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"e", " ", "=", "  ", 
     SuperscriptBox["b", 
      RowBox[{"n", "-", "m"}]]}], ","}], TraditionalForm]]],
 "\n                       \nbut unfortunately we have yet to prove that ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " has an identity.)\n\nIf we now let ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"a", " ", "=", " ", 
      RowBox[{"n", " ", "-", " ", "m", " ", "+", " ", "1"}]}], ","}]}], 
   TraditionalForm]]],
 " we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", ">", " ", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["b", "a"], " ", "=", " ", 
    RowBox[{"b", "."}]}], TraditionalForm]]],
 "\n\nNext, let us show that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", 
    RowBox[{"a", "-", "1"}]], TraditionalForm]]],
 " is an identity element in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". For any element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", we have\n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"x", "\[CenterDot]", 
      SuperscriptBox["b", "a"]}], " ", "=", " ", 
     RowBox[{"x", "\[CenterDot]", "b"}]}], ","}], TraditionalForm]]],
 "\n\nand since ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is non-zero and not a zero divisor, we can use the law of cancellation to \
get \n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", 
     SuperscriptBox["b", 
      RowBox[{"a", "-", "1"}]]}], " ", "=", " ", 
    RowBox[{"x", "."}]}], TraditionalForm]]],
 "\n\nLikewise, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", "a"], "\[CenterDot]", "x"}], " ", "=", " ", 
    RowBox[{"b", "\[CenterDot]", "x"}]}], TraditionalForm]]],
 ", we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["b", 
      RowBox[{"a", "-", "1"}]], "\[CenterDot]", "x"}], " ", "=", " ", "x"}], 
   TraditionalForm]]],
 ". Hence, there is an identity element in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", namely ",
 Cell[BoxData[
  FormBox[
   RowBox[{"e", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["b", 
      RowBox[{"a", "-", "1"}]], "."}]}], TraditionalForm]]],
 "\n\nFinally, we need to construct an inverse for the element ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "=", "2"}], TraditionalForm]]],
 ", then we have just shown that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", "e"}], TraditionalForm]]],
 ", and hence ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is it's own inverse. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", ">", " ", "2"}], TraditionalForm]]],
 ", consider the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["b", 
     RowBox[{"a", "-", "2"}]], "."}], TraditionalForm]]],
 " We have that \n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["b", 
       RowBox[{"a", "-", "2"}]], "\[CenterDot]", "b"}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["b", 
       RowBox[{"a", "-", "1"}]], " ", "=", " ", "e"}]}], ",", "  ", 
    RowBox[{
     RowBox[{"and", "  ", 
      RowBox[{"b", "\[CenterDot]", 
       SuperscriptBox["b", 
        RowBox[{"a", "-", "2"}]]}]}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["b", 
       RowBox[{"a", "-", "1"}]], "=", " ", 
      RowBox[{"e", "."}]}]}]}], TraditionalForm]]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", 
    RowBox[{
     RowBox[{"a", "-", "2"}], " "}]], TraditionalForm]]],
 "is the multiplicative inverse of ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718766934375*^9, 3.4371876700625*^9}}]
}, Open  ]],

Cell["\<\
There are some immediate consequences to proposition 9.4.5.

COROLLARY 9.1
Every finite ring with no zero divisors is a division ring.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437187745375*^9, 3.437187746015625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
The trivial ring is already considered to be a division ring, so we may \
assume that the ring is non-trivial. Then there exists a non-zero element \
which is not a zero divisor, so by proposition 9.9, the ring has an identity. \
Also by proposition 9.9, every non-zero element will have a multiplicative \
inverse, so the ring is a division ring.\
\>", "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437187760609375*^9, 3.437187765171875*^9}}]
}, Open  ]],

Cell[TextData[{
 "This corollary allows us to find many examples of fields.\n\nCOROLLARY 9.2\n\
The ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " is a field if, and only if, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is prime."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437187774078125*^9, 3.437187774640625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "1"}], TraditionalForm]]],
 ", then the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "n"], " ", "=", " ", 
    SubscriptBox["Z", "1"]}], TraditionalForm]]],
 " is the trivial ring, which we did not consider to be a field. We may \
suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "1"}], TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is prime, then by proposition 9.8  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " has no zero divisors, and so by corollary 9.1  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 "  is a division ring. Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " is obviously commutative, this tells us that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " is a field.\n\nNow suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is not prime. By proposition 9.8, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " has zero divisors, which cannot exist is a field according to proposition \
9.5. Therefore ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " is a field if, and only if, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is prime."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4371877928125*^9, 3.4371878186875*^9}}]
}, Open  ]],

Cell["\<\

To conclude this chapter, let us find an example of each of the 11 different \
types of rings that could exist. Every ring will fall into one of the \
following catagories.
\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Type I:    The Trivial Ring", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell["\<\
   Of course, there is only the one ring in this  
   catagory, whose addition and multiplication    
   tables are both given by the following command:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
DefSumMod[1];
MultTable[{0}]
\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.442747623898875*^9}],

Cell[BoxData[
 GraphicsBox[{RasterBox[{{{1, 1, 0}, {1, 1, 0}}, {{1, 1, 1}, {1, 1, 0}}}], 
   InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}], InsetBox[
    TagBox[
     StyleBox["\<\"0\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], 
   LineBox[{{1, 0}, {1, 1}, {2, 1}}]}]], "Output",
 CellChangeTimes->{3.442747624305125*^9}]
}, Open  ]],

Cell["Type II:   Fields", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "   We have already seen several examples of fields \n   in the first \
section, such as the rational \n   numbers and the real numbers. But \
corollary 9.2 \n   has given us many more examples. Here is the \n   addition \
and multiplication table for one of \n   the finite fields, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "7"], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437187840828125*^9, 3.43718784171875*^9}}],

Cell[CellGroupData[{

Cell["\<\
DefMod[7]
R = Ring[{1}];
AddTable[R]
MultTable[R]
\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.442747627211375*^9, 3.442747642867625*^9}}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0.6, 0}, {1, 0.6, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 
      0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}}, {{0,
       1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {1, 1, 0}, {1, 0, 1}, {
      0, 1, 1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {1, 1, 0}, {1, 
      0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {
      0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 1, 
      1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}}}], InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 7}, {8, 7}}]}]], "Output",
 CellChangeTimes->{{3.44274762775825*^9, 3.4427476435395*^9}}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0.6, 0}, {1, 1, 0}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0, 1, 
      0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {0.6,
       0.4, 0.8}, {1, 0, 0}, {1, 0, 1}, {1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}}, {{
      0, 1, 0}, {1, 1, 0}, {0, 1, 0}, {1, 0, 1}, {0.6, 0.4, 0.8}, {0, 1, 1}, {
      1, 0.6, 0}, {1, 0, 0}}, {{1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {
      0, 1, 1}, {0.6, 0.4, 0.8}, {1, 0, 1}, {0, 1, 0}}, {{0, 1, 1}, {1, 1, 
      0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {1, 0, 1}, {1, 0, 0}, {0.6, 0.4, 
      0.8}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 7}, {8, 7}}]}]], "Output",
 CellChangeTimes->{{3.44274762775825*^9, 3.4427476436645*^9}}]
}, Open  ]],

Cell["Type III:  Skew Fields", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell["\<\
   The only example of a skew field that we have 
   seen is the quaternions, the first known       
   example. Unlike fields, skew fields are quite 
   rare--in fact there are no finite skew fields. \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Type IV:   Commutative rings with an identity and without  zero 
                  divisors, but which are not fields\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "   We have seen one important example of a ring of \n   this type, namely \
the ring of integers ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". All \n   rings of this type possess many of the\n   properties of \
integers, such as having \"prime\n   elements.\" Thus, we give this important \
class \n   of rings a special name--",
 StyleBox["integral domains",
  FontSlant->"Italic"],
 ". We \n   will study integral domains in depth in \n   chapters 10 and 13."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437187858921875*^9, 3.43718786340625*^9}}],

Cell["\<\
Type V:    Non-commutative rings with an identity and without zero 
                  divisors, but which are not skew fields\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "   By proposition 9.9, any ring of this type must \n   be an infinite ring. \
So the natural place to  \n   look for such a ring would be some restriction \
\n   of the quaternions. We consider the ",
 StyleBox["integer",
  FontSlant->"Italic"],
 " \n   ",
 StyleBox["quaternions",
  FontSlant->"Italic"],
 ", that is, quaternions of the form\n \n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "+", " ", 
    RowBox[{"b", " ", "I"}], " ", "+", " ", 
    RowBox[{"c", " ", "J"}], " ", "+", " ", 
    RowBox[{"d", " ", "K"}]}], TraditionalForm]]],
 "  \n\n   where ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " are all integers. \n   Certainly the product and sum of two integer \n   \
quaternions would yield integer quaternions, \n   and so these would form a \
ring. However, 2 has \n   no inverse in the integer quaternions, so this \n   \
is not a skew field."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437187876625*^9, 3.43718787753125*^9}, {
  3.43718793928125*^9, 3.437187939875*^9}}],

Cell["\<\
Type VI:   Commutative rings without an identity without zero divisors\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "   We saw one example of a ring of this type in \n   the first section, the \
even integers. In fact, \n   if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "1"}], TraditionalForm]]],
 ", then all multiples of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " will form a \n   ring of this type. We will see more examples of \n   \
rings of this type when we study ",
 StyleBox["ideals",
  FontSlant->"Italic"],
 " in \n   chapter 2."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Type VII:  Non-commutative rings without an identity and without zero
                 divisors\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "   Again, by proposition 9.9, all rings of this \n   form must be infinite \
rings. We can apply the\n   same trick used for the type VI ring to the\n   \
integer quaternions, and obtain the ",
 StyleBox["even",
  FontSlant->"Italic"],
 "\n   ",
 StyleBox["quaternions",
  FontSlant->"Italic"],
 ", that is, quaternions of the form\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"a", " ", "+", " ", 
     RowBox[{"b", " ", "I"}], " ", "+", " ", 
     RowBox[{"c", " ", "J"}], " ", "+", " ", 
     RowBox[{"d", " ", "K"}]}]}], TraditionalForm]]],
 "\n\n   where ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " are all even integers. It \n   is easy to check that the sum and product \
of \n   two even quaternions yield even quaternions. \n   However, this ring \
does not have an identity \n   element."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437187964140625*^9, 3.4371879660625*^9}}],

Cell["\<\
Type VIII: Commutative rings with an identity and with zero divisors\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "   Proposition 9.8 gives us many examples of such \n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"rings", "--"}], 
    SubscriptBox["Z", "n"]}], TraditionalForm]]],
 " for any ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "1"}], TraditionalForm]]],
 " which is not prime. \n   Here is one example:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43718807940625*^9}],

Cell[CellGroupData[{

Cell["\<\
DefMod[8];
R = Ring[{1}];
AddTable[R]
MultTable[R]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.44274763619575*^9, 3.44274763763325*^9}}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {
      0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 0, 
      0}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 
      0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}}, {{1, 1, 
      0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {
      0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8,
       0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.4427476554145*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {1, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6,
       0.4, 0.8}, {0, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}}, {{1, 0.6, 0}, {
      1, 1, 0}, {1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}, {1, 1, 0}, {1, 0.6, 0}, {0,
       1, 0}, {0, 1, 1}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0, 1,
       1}, {0.7, 0.8, 0.7}, {0, 1, 0}, {1, 0, 1}, {1, 0.6, 0}, {1, 0, 0}}, {{
     0, 1, 0}, {1, 1, 0}, {0, 1, 0}, {1, 1, 0}, {0, 1, 0}, {1, 1, 0}, {0, 1, 
     0}, {1, 1, 0}, {0, 1, 
     0}}, {{1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {1, 0, 1}, {0, 1, 
      0}, {0.7, 0.8, 0.7}, {0, 1, 1}, {0.6, 0.4, 0.8}}, {{0, 1, 1}, {1, 1, 
      0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {1, 1, 0}, {0, 1, 1}, {0, 1, 
      0}, {1, 0.6, 0}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.44274765557075*^9}]
}, Open  ]],

Cell["\<\
Type IX:   Non-commutative rings with an identity and with zero 
                 divisors.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "   We constructed one example of this type of ring \n   in the last \
section. Let us call this ring\n   ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "8"], TraditionalForm]]],
 ", the smallest non-commutative ring with\n   an identity. Here is the \
addition and \n   multiplication table of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "8"], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[2 e,0];
Define[2 a,0];
Define[2 b,0];
Define[e.e,e];
Define[e.a,a];
Define[e.b,b];
Define[a.e,a];
Define[b.e,b];
Define[a.b,a];
Define[b.a,b];
Define[a.a,a];
Define[b.b,b];
R = Ring[{e,a,b}];
AddTable[R]
MultTable[R]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.442747659273875*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 
      0.8}, {0, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}}, {{
      1, 0.6, 0}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 0.4,
       0.8}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0, 1}, {1, 1, 0}, {
      1, 0, 0}, {0, 1, 1}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}, {0.7, 0.8, 
      0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0, 1, 0}}, {{0, 1, 1}, {0, 1, 1}, {
      1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 
      0}, {0.6, 0.4, 0.8}}, {{1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 
      1, 1}, {0.6, 0.4, 0.8}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}}, {{1, 
      1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4,
       0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 
      1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7,
       0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.44274766019575*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {1, 1, 0}, {0, 1, 1}, {1, 0, 1}, {1, 0, 0}, {
      0.7, 0.8, 0.7}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0, 1, 0}}, {{
      1, 0.6, 0}, {1, 1, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.6, 0.4, 0.8}}, {{0.6, 0.4, 0.8}, {1, 1, 
      0}, {1, 1, 0}, {1, 0, 0}, {1, 0, 0}, {0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {1, 0.6, 0}}, {{0, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {
      1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{
     1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 0, 0}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 1}, {
     1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {1, 1, 0}, {0, 1, 1}}, {{1, 0, 1}, {1, 1,
      0}, {1, 0, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {1, 1, 0}, {1, 1, 0}, {
     1, 0, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1,
      0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b+e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b+e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.442747660336375*^9}]
}, Open  ]],

Cell["\<\
   Proposition 9.6 shows that there is no smaller ring of this type.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718810996875*^9, 3.43718813528125*^9}}],

Cell["\<\
Type X:    Commutative rings without an identity and with zero 
                 divisors\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "   One of the simpliest example of ring of this \n   type would be the ring \
of even elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "8"], TraditionalForm]]],
 ":\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "2", ",", "4", ",", "6"}], "}"}], TraditionalForm]]],
 ". By setting ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", "2"}], TraditionalForm]]],
 ", we can list the \n   elements of this ring as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", " ", "a", ",", " ", 
     RowBox[{"2", "a"}], ",", " ", 
     RowBox[{"3", "a"}]}], "}"}], TraditionalForm]]],
 ", and \n   hence enter this ring into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with the \n   commands"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[4 a,0];
Define[a.a,2a];
R = Ring[{a}];
AddTable[R]
MultTable[R]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.44274766394575*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1,
     1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {
    0, 1, 1}, {1, 0, 0}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}, {1, 0, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.442747664477*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}}, {{0, 1,
     1}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{1, 0, 1}, {1, 1, 0}, {
    0, 1, 1}, {1, 1, 0}, {0, 1, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
    0}, {1, 1, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.44274766450825*^9}]
}, Open  ]],

Cell["\<\
   Are there any smaller rings which fit into this 
   catagory?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Type XI:   Non-commutative rings without an identity and with zero
                 divisors\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell["\<\
   We constructed one example of a ring of this
   type in the previous section:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[4 a, 0];
Define[2 b, 0];
Define[a.b, b];
Define[b.a, 2 a];
Define[a.a, a];
Define[b.b, 0];
CheckRing[{a,b}];
R = Ring[{a,b}];
AddTable[R]
MultTable[R]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.442747669680125*^9, 3.442747670367625*^9}}],

Cell[BoxData["\<\"These elements will generate a ring.\"\>"], "Print",
 CellChangeTimes->{3.442747671711375*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 
      0, 0}, {1, 1, 0}, {1, 0, 1}}, {{0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}, {1, 1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {
      0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 0, 
      0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {0.7, 0.8, 0.7}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {1,
       1, 0}, {1, 0, 1}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {0.6, 0.4, 
      0.8}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.442747671836375*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {
      0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{
      1, 0.6, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 
      1, 0}, {1, 1, 0}, {1, 1, 0}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 0}, {
      0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4,
       0.8}}, {{0, 1, 0}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 
     0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 
     1}}, {{1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {
      0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}}, {{0, 1, 1}, {1, 1, 0}, {
     0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 
     1}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 0}, {1, 1, 0}, {
     1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.442747671977*^9}]
}, Open  ]],

Cell["\<\
   This is not the smallest ring of this type. In 
   fact, there is a non-commutative ring of      
   order 4, with the following addition and
   multiplication tables:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
            
     + | 0 | a | b | c |       \[CenterDot] | 0 | a | b | c |
    ---|---|---|---|---|      ---|---|---|---|---|
     0 | 0 | a | b | c |       0 | 0 | 0 | 0 | 0 |
    ---|---|---|---|---|      ---|---|---|---|---|
     a | a | 0 | c | b |       a | 0 | a | a | 0 |
    ---|---|---|---|---|      ---|---|---|---|---|
     b | b | c | 0 | a |       b | 0 | b | b | 0 |
    ---|---|---|---|---|      ---|---|---|---|---|
     c | c | b | a | 0 |       c | 0 | c | c | 0 |
    ---|---|---|---|---|      ---|---|---|---|---|
    \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Courier New"],

Cell[TextData[{
 "            We will refer to this ring as ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", 
    RowBox[{"4.", " "}]], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
EXPERIMENT:
See if you can change the definition of the last ring to produce this \
non-commutative ring of order 4.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
After the trivial ring, these rings are basically ordered from the most \
important to the least. As you might guess from this list, the study of rings \
can go deeper than group theory. However, by making observations on the rings \
mentioned in this list we can get a feel for how rings of different types \
behave. This quickly leads us to the proofs of many theorems.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Problems"
}], "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437188164796875*^9, 3.4371881669375*^9}}],

Cell["", "Input",
 PageWidth->Infinity,
 FontFamily->"System",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->RGBColor[1, 0, 0],
 Background->GrayLevel[1]],

Cell["\<\
9.1 ) Notice that in the plot of rational numbers between
   0.03 and 0.1, \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.437188170828125*^9}],

Cell[CellGroupData[{

Cell["Z = ShowRationals[0.03,0.1]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.442747685148875*^9}],

Cell[BoxData[
 GraphicsBox[{PointBox[NCache[{Rational[1, 10], 1}, {0.1, 1}]], 
   PointBox[NCache[{Rational[1, 11], 0.75}, {0.09090909090909091, 0.75}]], 
   PointBox[NCache[{Rational[1, 12], 0.5625}, {0.08333333333333333, 0.5625}]],
    PointBox[
    NCache[{Rational[1, 13], 0.421875}, {0.07692307692307693, 0.421875}]], 
   PointBox[NCache[{Rational[1, 14], 0.31640625}, {0.07142857142857142, 
     0.31640625}]], 
   PointBox[NCache[{Rational[1, 15], 0.2373046875}, {0.06666666666666667, 
     0.2373046875}]], 
   PointBox[NCache[{Rational[1, 16], 0.177978515625}, {0.0625, 
     0.177978515625}]], 
   PointBox[NCache[{Rational[1, 17], 0.13348388671875}, {
     0.058823529411764705`, 0.13348388671875}]], 
   PointBox[NCache[{Rational[1, 18], 0.1001129150390625}, {
     0.05555555555555555, 0.1001129150390625}]], 
   PointBox[NCache[{Rational[1, 19], 0.07508468627929688}, {
     0.05263157894736842, 0.07508468627929688}]], 
   PointBox[NCache[{Rational[1, 20], 0.056313514709472656`}, {0.05, 
     0.056313514709472656`}]], 
   PointBox[NCache[{Rational[1, 21], 0.04223513603210449}, {
     0.047619047619047616`, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[2, 21], 0.04223513603210449}, {
     0.09523809523809523, 0.04223513603210449}]], 
   PointBox[NCache[{Rational[1, 22], 0.03167635202407837}, {
     0.045454545454545456`, 0.03167635202407837}]], 
   PointBox[NCache[{Rational[1, 23], 0.023757264018058777`}, {
     0.043478260869565216`, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[2, 23], 0.023757264018058777`}, {
     0.08695652173913043, 0.023757264018058777`}]], 
   PointBox[NCache[{Rational[1, 24], 0.017817948013544083`}, {
     0.041666666666666664`, 0.017817948013544083`}]], 
   PointBox[NCache[{Rational[1, 25], 0.013363461010158062`}, {0.04, 
     0.013363461010158062`}]], 
   PointBox[NCache[{Rational[2, 25], 0.013363461010158062`}, {0.08, 
     0.013363461010158062`}]]},
  AspectRatio->NCache[
    Rational[1, 2], 0.5],
  Axes->{True, False},
  AxesOrigin->{0, 0},
  PlotRange->All,
  Prolog->{
    AbsolutePointSize[2], 
    AbsoluteThickness[1]}]], "Output",
 CellChangeTimes->{3.442747685602*^9}]
}, Open  ]],

Cell[TextData[{
 "   that most of the points shown seem to lie on a curve.\n   Try to find \
the equation of this curve, using the fact \n   that each dot is three \
fourths closer to the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-axis than \n   the previous dot. Verify your answer by plotting the \n   \
curve with the points, using the following command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Show[Z, Plot[(function goes in here),
                         {x, 0.03, 0.1}]]\
\>", "Input",
 PageWidth->Infinity],

Cell["   (Hint: Scale the function so that f(0.1) = 1.)", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["       ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "9.2 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " or GAP to define a ring of \n   order 2 which has no identity element. \
Show \n   both the addition table and the multiplication table."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718820009375*^9, 3.43718824365625*^9}}],

Cell["    ", "Input",
 PageWidth->Infinity,
 AspectRatioFixed->True,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "9.3 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " or GAP to find a \n    non-commutative ring of order 8, for which the \n   \
 additive group is isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ", formed \n    from the basis ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"a", ",", " ", "b", ",", " ", "c"}], "}"}], TraditionalForm]]],
 ", and for which \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "b"}], " ", "=", " ", "a"}], 
   TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "a"}], " ", "=", " ", "b"}], 
   TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "c"}], " ", "=", " ", "c"}], 
   TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"c", "\[CenterDot]", "a"}], " ", "=", " ", "a"}], 
   TraditionalForm]]],
 ".\n   (Hint: Using the associtivity law, determine what ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["a", "2"], TraditionalForm]]],
 ",\n   ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", "2"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["c", "2"], TraditionalForm]]],
 " must be. Then show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "\[CenterDot]", "b"}], TraditionalForm]]],
 " must commute \n   with ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ". Use trial and error to determine ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[CenterDot]", "c"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718824784375*^9, 3.43718826578125*^9}}],

Cell["", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "  9.4 ) Define in GAP or ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " the smallest \n     non-commutative ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "4"], TraditionalForm]]],
 " described in the text."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718827746875*^9, 3.4371883371875*^9}}],

Cell["         ", "Input",
 PageWidth->Infinity,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "  9.5 ) Define in GAP or ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " the smallest \n     non-commutative ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "8"], TraditionalForm]]],
 " with an identity \n     described in the text.\n    (Hint: The basis can \
be chosen to be ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", and \n    ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ".)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43718834665625*^9, 3.4371883758125*^9}}],

Cell["         ", "Input",
 PageWidth->Infinity,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{838, 677},
WindowMargins->{{Automatic, 74}, {Automatic, 0}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowSelection->True,
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.5,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[568, 21, 320, 9, 222, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[913, 34, 281, 8, 29, "Name",
 Evaluatable->False],
Cell[1197, 44, 710, 16, 70, "Name",
 Evaluatable->False]
}, Closed]],
Cell[1922, 63, 266, 12, 142, "Title",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2213, 79, 104, 3, 32, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2342, 86, 173, 4, 43, "Input",
 InitializationCell->True],
Cell[2518, 92, 183, 2, 32, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2750, 100, 150, 3, 105, "Section",
 Evaluatable->False],
Cell[2903, 105, 1734, 33, 629, "Text",
 Evaluatable->False],
Cell[4640, 140, 42, 1, 43, "Input"],
Cell[4685, 143, 632, 16, 213, "Text",
 Evaluatable->False],
Cell[5320, 161, 42, 1, 43, "Input"],
Cell[5365, 164, 2801, 81, 496, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[8191, 249, 58, 1, 43, "Input"],
Cell[8252, 252, 88633, 1805, 265, 85304, 1749, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[96900, 2060, 1021, 20, 360, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[97946, 2084, 59, 1, 43, "Input"],
Cell[98008, 2087, 3223, 63, 262, 1985, 41, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[101246, 2153, 1016, 26, 409, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[102287, 2183, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[102375, 2187, 1385, 53, 96, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[103775, 2243, 413, 10, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[104213, 2257, 58, 1, 43, "Input"],
Cell[104274, 2260, 88611, 1805, 265, 85304, 1749, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[192900, 4068, 894, 16, 360, "Text",
 Evaluatable->False],
Cell[193797, 4086, 41, 1, 43, "Input"],
Cell[193841, 4089, 1039, 22, 409, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[194905, 4115, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[194993, 4119, 359, 7, 75, "SmallText",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[195377, 4130, 170, 4, 43, "Input"],
Cell[195550, 4136, 85346, 1750, 262, "Output"]
}, Open  ]],
Cell[280911, 5889, 373, 9, 96, "SmallText",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[281309, 5902, 126, 2, 43, "Input"],
Cell[281438, 5906, 90478, 1833, 262, "Output"]
}, Open  ]],
Cell[371931, 7742, 127, 2, 34, "SmallText",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[372083, 7748, 52, 1, 43, "Input"],
Cell[372138, 7751, 181341, 3608, 262, 171028, 3437, "CachedBoxData", \
"BoxData", "Output"]
}, Open  ]],
Cell[553494, 11362, 1249, 30, 215, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[554758, 11395, 759, 21, 215, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[555542, 11420, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[555630, 11424, 2384, 83, 413, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[558029, 11510, 1210, 35, 313, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[559264, 11549, 54, 1, 43, "Input"],
Cell[559321, 11552, 1116, 15, 668, "Output"]
}, Open  ]],
Cell[560452, 11570, 826, 19, 286, "Text",
 Evaluatable->False],
Cell[561281, 11591, 48, 1, 43, "Input"],
Cell[561332, 11594, 916, 17, 384, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[562273, 11615, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[562361, 11619, 3222, 99, 603, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[565598, 11721, 745, 21, 164, "Text",
 Evaluatable->False],
Cell[566346, 11744, 51, 1, 43, "Input"],
Cell[566400, 11747, 247, 8, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[566672, 11759, 63, 1, 43, "Input"],
Cell[566738, 11762, 7089, 208, 481, "Output"]
}, Open  ]],
Cell[573842, 11973, 220, 5, 90, "Text",
 Evaluatable->False],
Cell[574065, 11980, 52, 1, 43, "Input"],
Cell[574120, 11983, 114, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[574259, 11989, 63, 1, 43, "Input"],
Cell[574325, 11992, 7063, 209, 481, "Output"]
}, Open  ]],
Cell[581403, 12204, 1271, 30, 458, "Text",
 Evaluatable->False],
Cell[582677, 12236, 42, 1, 43, "Input"],
Cell[582722, 12239, 257, 6, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[583004, 12249, 201, 6, 66, "Input"],
Cell[583208, 12257, 363, 8, 42, "Output"]
}, Open  ]],
Cell[583586, 12268, 120, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[583731, 12274, 51, 1, 43, "Input"],
Cell[583785, 12277, 11764, 343, 481, "Output"]
}, Open  ]],
Cell[595564, 12623, 1140, 42, 215, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[596729, 12669, 50, 1, 43, "Input"],
Cell[596782, 12672, 135, 3, 42, "Output"]
}, Open  ]],
Cell[596932, 12678, 620, 15, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[597577, 12697, 228, 4, 43, "Input"],
Cell[597808, 12703, 265, 7, 42, "Output"]
}, Open  ]],
Cell[598088, 12713, 464, 10, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[598577, 12727, 273, 4, 43, "Input"],
Cell[598853, 12733, 316, 8, 42, "Output"]
}, Open  ]],
Cell[599184, 12744, 426, 12, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[599635, 12760, 55, 1, 43, "Input"],
Cell[599693, 12763, 184, 5, 42, "Output"]
}, Open  ]],
Cell[599892, 12771, 1280, 35, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[601197, 12810, 175, 3, 43, "Input"],
Cell[601375, 12815, 258, 6, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[601670, 12826, 174, 3, 43, "Input"],
Cell[601847, 12831, 260, 6, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[602144, 12842, 109, 2, 43, "Input"],
Cell[602256, 12846, 834, 19, 93, "Output"]
}, Open  ]],
Cell[603105, 12868, 261, 7, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[603391, 12879, 109, 2, 43, "Input"],
Cell[603503, 12883, 939, 30, 110, "Output"]
}, Open  ]],
Cell[604457, 12916, 113, 2, 41, "Text",
 Evaluatable->False],
Cell[604573, 12920, 43, 1, 43, "Input"],
Cell[604619, 12923, 371, 9, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[605015, 12936, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[605103, 12940, 140, 3, 34, "SmallText",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[605268, 12947, 175, 3, 43, "Input"],
Cell[605446, 12952, 258, 6, 42, "Output"]
}, Open  ]],
Cell[605719, 12961, 226, 4, 34, "SmallText",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[605970, 12969, 263, 6, 45, "Input"],
Cell[606236, 12977, 917, 30, 110, "Output"]
}, Open  ]],
Cell[607168, 13010, 681, 20, 60, "SmallText",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[607874, 13034, 151, 3, 43, "Input"],
Cell[608028, 13039, 119, 2, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[608184, 13046, 144, 3, 43, "Input"],
Cell[608331, 13051, 92, 1, 42, "Output"]
}, Open  ]],
Cell[608438, 13055, 968, 27, 101, "SmallText",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[609431, 13086, 174, 3, 43, "Input"],
Cell[609608, 13091, 280, 6, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[609925, 13102, 270, 6, 43, "Input"],
Cell[610198, 13110, 210, 5, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[610445, 13120, 269, 8, 43, "Input"],
Cell[610717, 13130, 120, 2, 42, "Output"]
}, Open  ]],
Cell[610852, 13135, 1819, 59, 148, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[612686, 13197, 900, 21, 188, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[613623, 13223, 97, 2, 105, "Section",
 Evaluatable->False],
Cell[613723, 13227, 517, 9, 188, "Text",
 Evaluatable->False],
Cell[614243, 13238, 2985, 53, 1846, "Text",
 Evaluatable->False],
Cell[617231, 13293, 571, 13, 213, "Text",
 Evaluatable->False],
Cell[617805, 13308, 40, 1, 43, "Input"],
Cell[617848, 13311, 550, 11, 237, "Text",
 Evaluatable->False],
Cell[618401, 13324, 85, 2, 41, "Text",
 Evaluatable->False],
Cell[618489, 13328, 110, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[618624, 13334, 150, 3, 41, "Text",
 Evaluatable->False],
Cell[618777, 13339, 1453, 51, 237, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[620245, 13393, 2483, 74, 727, "Text",
 Evaluatable->False],
Cell[622731, 13469, 481, 17, 90, "Text",
 Evaluatable->False],
Cell[623215, 13488, 508, 17, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[623748, 13509, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[623836, 13513, 4182, 134, 403, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[628033, 13650, 713, 19, 164, "Text",
 Evaluatable->False],
Cell[628749, 13671, 906, 33, 164, "Text",
 Evaluatable->False],
Cell[629658, 13706, 630, 17, 115, "Text",
 Evaluatable->False],
Cell[630291, 13725, 863, 28, 213, "Text",
 Evaluatable->False],
Cell[631157, 13755, 657, 14, 188, "Text",
 Evaluatable->False],
Cell[631817, 13771, 52, 1, 43, "Input"],
Cell[631872, 13774, 182, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[632079, 13783, 52, 1, 43, "Input"],
Cell[632134, 13786, 686, 20, 481, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[632857, 13811, 53, 1, 43, "Input"],
Cell[632913, 13814, 1598, 36, 481, 639, 19, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[634526, 13853, 1703, 44, 635, "Text",
 Evaluatable->False],
Cell[636232, 13899, 239, 7, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[636496, 13910, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[636584, 13914, 2037, 69, 260, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[638636, 13986, 1774, 44, 507, "Text",
 Evaluatable->False],
Cell[640413, 14032, 339, 8, 139, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[640789, 14045, 235, 7, 142, "Section",
 Evaluatable->False],
Cell[641027, 14054, 1124, 29, 311, "Text",
 Evaluatable->False],
Cell[642154, 14085, 408, 14, 60, "Text",
 Evaluatable->False],
Cell[642565, 14101, 186, 3, 41, "Text",
 Evaluatable->False],
Cell[642754, 14106, 1339, 50, 237, "Text",
 Evaluatable->False],
Cell[644096, 14158, 1102, 36, 262, "Text",
 Evaluatable->False],
Cell[645201, 14196, 813, 29, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[646039, 14229, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[646127, 14233, 7360, 251, 608, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[653502, 14487, 1287, 44, 267, "Text",
 Evaluatable->False],
Cell[654792, 14533, 658, 22, 237, "Text",
 Evaluatable->False],
Cell[655453, 14557, 85, 5, 88, "Input"],
Cell[655541, 14564, 260, 7, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[655826, 14575, 58, 1, 43, "Input"],
Cell[655887, 14578, 361, 10, 42, "Output"]
}, Open  ]],
Cell[656263, 14591, 225, 5, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[656513, 14600, 166, 6, 43, "Input"],
Cell[656682, 14608, 89, 2, 42, "Output"]
}, Open  ]],
Cell[656786, 14613, 146, 4, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[656957, 14621, 50, 1, 43, "Input"],
Cell[657010, 14624, 23005, 528, 481, 11749, 342, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[680030, 15155, 921, 23, 265, "Text",
 Evaluatable->False],
Cell[680954, 15180, 637, 20, 142, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[681616, 15204, 105, 6, 111, "Input"],
Cell[681724, 15212, 888, 28, 93, "Output"]
}, Open  ]],
Cell[682627, 15243, 1040, 34, 166, "Text",
 Evaluatable->False],
Cell[683670, 15279, 2071, 70, 264, "Text",
 Evaluatable->False],
Cell[685744, 15351, 4337, 128, 1051, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[690106, 15483, 230, 8, 133, "Input"],
Cell[690339, 15493, 366, 10, 42, "Output"]
}, Open  ]],
Cell[690720, 15506, 144, 4, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[690889, 15514, 69, 1, 43, "Input"],
Cell[690961, 15517, 367, 10, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[691365, 15532, 50, 1, 43, "Input"],
Cell[691418, 15535, 11791, 343, 481, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[703246, 15883, 51, 1, 43, "Input"],
Cell[703300, 15886, 11688, 342, 481, "Output"]
}, Open  ]],
Cell[715003, 16231, 387, 12, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[715415, 16247, 51, 1, 43, "Input"],
Cell[715469, 16250, 160, 2, 32, "Print"]
}, Open  ]],
Cell[715644, 16255, 436, 11, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[716105, 16270, 55, 1, 43, "Input"],
Cell[716163, 16273, 111, 1, 32, "Print"]
}, Open  ]],
Cell[716289, 16277, 554, 17, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[716868, 16298, 51, 1, 43, "Input"],
Cell[716922, 16301, 111, 2, 42, "Output"]
}, Open  ]],
Cell[717048, 16306, 263, 7, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[717336, 16317, 51, 1, 43, "Input"],
Cell[717390, 16320, 11664, 342, 481, "Output"]
}, Open  ]],
Cell[729069, 16665, 967, 28, 240, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[730061, 16697, 164, 10, 201, "Input"],
Cell[730228, 16709, 642, 12, 32, "Print"],
Cell[730873, 16723, 641, 12, 32, "Print"],
Cell[731517, 16737, 110, 1, 32, "Print"]
}, Open  ]],
Cell[731642, 16741, 1075, 34, 384, "Text",
 Evaluatable->False],
Cell[732720, 16777, 368, 12, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[733113, 16793, 54, 1, 43, "Input"],
Cell[733170, 16796, 345, 10, 42, "Output"]
}, Open  ]],
Cell[733530, 16809, 119, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[733674, 16815, 51, 1, 43, "Input"],
Cell[733728, 16818, 102, 1, 32, "Print"]
}, Open  ]],
Cell[733845, 16822, 159, 5, 66, "Text",
 Evaluatable->False],
Cell[734007, 16829, 51, 1, 43, "Input"],
Cell[734061, 16832, 831, 16, 363, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[734917, 16852, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[735005, 16856, 8233, 273, 854, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[743253, 17132, 706, 20, 191, "Text",
 Evaluatable->False],
Cell[743962, 17154, 176, 11, 223, "Input"],
Cell[744141, 17167, 711, 25, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[744877, 17196, 255, 16, 336, "Input"],
Cell[745135, 17214, 639, 12, 32, "Print"],
Cell[745777, 17228, 639, 12, 32, "Print"],
Cell[746419, 17242, 642, 12, 32, "Print"],
Cell[747064, 17256, 641, 12, 32, "Print"],
Cell[747708, 17270, 109, 1, 32, "Print"]
}, Open  ]],
Cell[747832, 17274, 245, 7, 90, "Text",
 Evaluatable->False],
Cell[748080, 17283, 448, 14, 237, "Text",
 Evaluatable->False],
Cell[748531, 17299, 185, 5, 66, "Text",
 Evaluatable->False],
Cell[748719, 17306, 76, 4, 66, "Input"],
Cell[748798, 17312, 51, 1, 43, "Input"],
Cell[748852, 17315, 245, 8, 66, "Text",
 Evaluatable->False],
Cell[749100, 17325, 51, 1, 43, "Input"],
Cell[749154, 17328, 539, 13, 188, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[749730, 17346, 97, 2, 105, "Section",
 Evaluatable->False],
Cell[749830, 17350, 928, 26, 286, "Text",
 Evaluatable->False],
Cell[750761, 17378, 49, 1, 43, "Input"],
Cell[750813, 17381, 326, 10, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[751164, 17395, 54, 1, 43, "Input"],
Cell[751221, 17398, 280, 6, 68, "Output"]
}, Open  ]],
Cell[751516, 17407, 224, 8, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[751765, 17419, 52, 1, 43, "Input"],
Cell[751820, 17422, 37493, 1082, 481, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[789350, 18509, 53, 1, 43, "Input"],
Cell[789406, 18512, 37391, 1081, 481, "Output"]
}, Open  ]],
Cell[826812, 19596, 201, 8, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[827038, 19608, 44, 1, 43, "Input"],
Cell[827085, 19611, 70, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[827192, 19617, 44, 1, 43, "Input"],
Cell[827239, 19620, 70, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[827346, 19626, 42, 1, 43, "Input"],
Cell[827391, 19629, 70, 1, 42, "Output"]
}, Open  ]],
Cell[827476, 19633, 440, 10, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[827941, 19647, 105, 6, 111, "Input"],
Cell[828049, 19655, 42861, 812, 484, 13843, 334, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[870925, 20470, 365, 7, 139, "Text",
 Evaluatable->False],
Cell[871293, 20479, 40, 1, 43, "Input"],
Cell[871336, 20482, 288, 8, 90, "Text",
 Evaluatable->False],
Cell[871627, 20492, 581, 20, 117, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[872233, 20516, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[872321, 20520, 909, 32, 116, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[873245, 20555, 453, 15, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[873723, 20574, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[873811, 20578, 1589, 57, 201, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[875415, 20638, 811, 28, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[876251, 20670, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[876339, 20674, 4105, 157, 409, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[880459, 20834, 571, 14, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[881055, 20852, 261, 16, 336, "Input"],
Cell[881319, 20870, 11706, 342, 476, "Output"]
}, Open  ]],
Cell[893040, 21215, 1608, 49, 458, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[894673, 21268, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[894761, 21272, 2302, 77, 347, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[897078, 21352, 1506, 43, 360, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[898609, 21399, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[898697, 21403, 5581, 195, 943, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[904293, 21601, 275, 8, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[904593, 21613, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[904681, 21617, 496, 9, 116, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[905192, 21629, 410, 13, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[905627, 21646, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[905715, 21650, 1660, 58, 208, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[907390, 21711, 252, 8, 139, "Text",
 Evaluatable->False],
Cell[907645, 21721, 330, 12, 42, "Text",
 Evaluatable->False],
Cell[907978, 21735, 230, 6, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[908233, 21745, 118, 6, 88, "Input"],
Cell[908354, 21753, 681, 20, 481, "Output"]
}, Open  ]],
Cell[909050, 21776, 320, 12, 42, "Text",
 Evaluatable->False],
Cell[909373, 21790, 511, 12, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[909909, 21806, 163, 8, 133, "Input"],
Cell[910075, 21816, 9240, 271, 481, "Output"],
Cell[919318, 22089, 9241, 271, 481, "Output"]
}, Open  ]],
Cell[928574, 22363, 325, 12, 42, "Text",
 Evaluatable->False],
Cell[928902, 22377, 279, 7, 115, "Text",
 Evaluatable->False],
Cell[929184, 22386, 428, 15, 93, "Text",
 Evaluatable->False],
Cell[929615, 22403, 643, 14, 213, "Text",
 Evaluatable->False],
Cell[930261, 22419, 436, 15, 93, "Text",
 Evaluatable->False],
Cell[930700, 22436, 1261, 36, 360, "Text",
 Evaluatable->False],
Cell[931964, 22474, 381, 14, 68, "Text",
 Evaluatable->False],
Cell[932348, 22490, 537, 16, 164, "Text",
 Evaluatable->False],
Cell[932888, 22508, 406, 15, 93, "Text",
 Evaluatable->False],
Cell[933297, 22525, 1170, 35, 335, "Text",
 Evaluatable->False],
Cell[934470, 22562, 379, 14, 68, "Text",
 Evaluatable->False],
Cell[934852, 22578, 448, 15, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[935325, 22597, 161, 7, 111, "Input"],
Cell[935489, 22606, 11689, 343, 481, "Output"],
Cell[947181, 22951, 11647, 343, 481, "Output"]
}, Open  ]],
Cell[958843, 23297, 402, 15, 93, "Text",
 Evaluatable->False],
Cell[959248, 23314, 461, 14, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[959734, 23332, 322, 19, 381, "Input"],
Cell[960059, 23353, 11790, 343, 476, "Output"],
Cell[971852, 23698, 11708, 342, 476, "Output"]
}, Open  ]],
Cell[983575, 24043, 210, 5, 41, "Text",
 Evaluatable->False],
Cell[983788, 24050, 400, 15, 93, "Text",
 Evaluatable->False],
Cell[984191, 24067, 837, 28, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[985053, 24099, 168, 9, 156, "Input"],
Cell[985224, 24110, 3650, 108, 481, "Output"],
Cell[988877, 24220, 3659, 108, 481, "Output"]
}, Open  ]],
Cell[992551, 24331, 142, 5, 66, "Text",
 Evaluatable->False],
Cell[992696, 24338, 403, 15, 93, "Text",
 Evaluatable->False],
Cell[993102, 24355, 158, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[993285, 24364, 280, 14, 268, "Input"],
Cell[993568, 24380, 112, 1, 32, "Print"],
Cell[993683, 24383, 11791, 343, 561, "Output"],
Cell[1005477, 24728, 11698, 342, 561, "Output"]
}, Open  ]],
Cell[1017190, 25073, 248, 7, 115, "Text",
 Evaluatable->False],
Cell[1017441, 25082, 644, 16, 311, "Text",
 Evaluatable->False],
Cell[1018088, 25100, 221, 8, 43, "Text",
 Evaluatable->False],
Cell[1018312, 25110, 194, 6, 90, "Text",
 Evaluatable->False],
Cell[1018509, 25118, 449, 8, 164, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[1018995, 25131, 210, 7, 105, "Section",
 Evaluatable->False],
Cell[1019208, 25140, 156, 6, 43, "Input"],
Cell[1019367, 25148, 195, 6, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1019587, 25158, 108, 2, 43, "Input"],
Cell[1019698, 25162, 2158, 44, 302, "Output"]
}, Open  ]],
Cell[1021871, 25209, 437, 10, 139, "Text",
 Evaluatable->False],
Cell[1022311, 25221, 126, 4, 66, "Input"],
Cell[1022440, 25227, 119, 2, 41, "Text",
 Evaluatable->False],
Cell[1022562, 25231, 46, 1, 43, "Input"],
Cell[1022611, 25234, 350, 9, 90, "Text",
 Evaluatable->False],
Cell[1022964, 25245, 181, 7, 43, "Input"],
Cell[1023148, 25254, 1748, 65, 220, "Text",
 Evaluatable->False],
Cell[1024899, 25321, 39, 1, 43, "Input"],
Cell[1024941, 25324, 370, 12, 66, "Text",
 Evaluatable->False],
Cell[1025314, 25338, 161, 6, 43, "Input"],
Cell[1025478, 25346, 620, 22, 139, "Text",
 Evaluatable->False],
Cell[1026101, 25370, 161, 6, 43, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
