(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    610691,      18399]
NotebookOptionsPosition[    587215,      17618]
NotebookOutlinePosition[    587816,      17642]
CellTagsIndexPosition[    587773,      17639]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Abstract Algebra: An Interactive Approach", "Special2",
 Evaluatable->False,
 CellChangeTimes->{{3.437270485109375*^9, 3.437270496328125*^9}},
 TextAlignment->Center,
 FontFamily->"Arial",
 FontSize->36,
 FontWeight->"Bold",
 FontSlant->"Italic",
 FontColor->RGBColor[1, 0, 1],
 Background->RGBColor[0.500008, 1, 1]],

Cell[CellGroupData[{

Cell["\[Copyright]2009   William Paulsen", "Name",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4372705000625*^9, 3.437270500703125*^9}},
 FontFamily->"Arial",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]],

Cell["\<\
All rights reserved:  These materials can be duplicated or transmitted freely \
for educational use on condition that all copyright statements are retained \
unaltered.  No part of these materials may be sold, by any means or in any \
form, without written permission from the editor.  All software is provided \
\"as is\" without warranty of any kind, either expressed or implied, \
including, but not limited to, the implied warranties of merchantability and \
fitness for a particular purpose.\
\>", "Name",
 Editable->False,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Arial",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]]
}, Closed]],

Cell["\<\
Chapter 10
The Structure Within Rings\
\>", "Title",
 CellFrame->True,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 TextAlignment->Center,
 FontFamily->"Arial",
 FontSize->24,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell["Initialization", "Special2",
 Evaluatable->False,
 TextAlignment->Center,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["<<c:\\math\\ring.m", "Input",
 PageWidth->Infinity,
 InitializationCell->True,
 CellChangeTimes->{{3.43727050521875*^9, 3.43727050828125*^9}},
 AspectRatioFixed->True],

Cell[BoxData["\<\"Initialization Done\"\>"], "Print",
 CellChangeTimes->{3.437270516078125*^9, 3.43730312540625*^9, 
  3.437323442109375*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Subrings", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In the last chapter we introduced rings, and used ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to define several examples of rings. It is natural to ask whether we can \
have smaller rings within a larger ring, just as we saw smaller groups inside \
of a larger group. So the natural definition would be as follows:\n\n\
DEFINITION 10.1:\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a ring. A non-empty subset ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a ",
 StyleBox["subring",
  FontSlant->"Italic"],
 " if ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a ring with respect to the addition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "+", ")"}], TraditionalForm]]],
 " and multiplication ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "\[CenterDot]", ")"}], TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ".\n\nIt is clear that in testing to see if a subset of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a ring, we do not need to check all 9 of the ring properties, since \
many of these properties will be \"inherited\" from the ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". The next propostion shows us a quick way to check if a subset is indeed a \
subring.\n\nPROPOSITION 10.1\nA non-empty subset ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a subring of a ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " if and only if whenever ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "-", "y"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437270528140625*^9, 3.4372705538125*^9}, {
  3.4417224224171247`*^9, 3.4417224330264997`*^9}, {3.4417224984796247`*^9, 
  3.4417224994327497`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Certainly if ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a subring, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "-", "y"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " would be in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " whenever ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". So let us suppose that ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is non-empty, and is closed with respect to subtraction and \
multiplication. If ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is any element in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "-", "x"}], " ", "=", " ", "0"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " contains an additive identity. Also, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"0", "-", "x"}], "=", " ", 
    RowBox[{"-", "x"}]}], TraditionalForm]]],
 " would also be in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " contains additive inverses of all of its elements. Then whenever ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "-", 
     RowBox[{"(", 
      RowBox[{"-", "y"}], ")"}]}], " ", "=", " ", 
    RowBox[{"x", " ", "+", " ", "y"}]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is closed with respect to addition. The commutative and associative \
properties of addition, as well as the associative and two distributive laws \
for multiplication, come from the original ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Finally, ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is closed with respect to multiplication, so ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a subring."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "Notice that from the definition every non-trivial ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " will contain at least two subrings: the trivial ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "0", "}"}], TraditionalForm]]],
 " will be a subring, as well as the entire ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". These two subrings are called the ",
 StyleBox["trivial subrings",
  FontSlant->"Italic"],
 ".\n\nLet us look at an example where we find other subrings. Here is the \
very first ring that we defined in section 9.3 of order 8:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[4 a, 0]; Define[2 b, 0]
Define[a.a, a]; Define[b.b, b]
Define[a.b, 0]; Define[b.a, 0]
R = Ring[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437270624078125*^9, 3.4372706384375*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.43727066478125*^9, 3.44274777788325*^9}]
}, Open  ]],

Cell["Recall that this ring has an identity element, namely", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["FindIdent[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"a", "+", "b"}]], "Output",
 CellChangeTimes->{3.43727066865625*^9, 3.442747781148875*^9}]
}, Open  ]],

Cell[TextData[{
 "Can we find a subring of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "? Suppose we considered only the elements which did not involve ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["S = {0, a, 2a, 3a}", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.43727067209375*^9, 3.442747783227*^9}]
}, Open  ]],

Cell[TextData[{
 "Does this form a subring? We can look at the multiplication tables for ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["AddTable[S]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1,
     1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {
    0, 1, 1}, {1, 0, 0}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}, {1, 0, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.44274778538325*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MultTable[S]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}}, {{0, 1,
     1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}}, {{1, 0, 1}, {1, 1, 0}, {
    1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
    0}, {1, 1, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.44274778707075*^9}]
}, Open  ]],

Cell[TextData[{
 "and see that ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is closed with respect to multiplication and addition. Furthermore, \
additive inverses exist for all elements, so ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is also closed with respect to subtraction. Thus, by proposition 10.1.1, \
this is a subring.      \n\nNotice that this subring does not include the \
multiplicative identity of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Does that mean that this subring is without an identity?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["FindIdent[S]", "Input",
 PageWidth->Infinity],

Cell[BoxData["a"], "Output",
 CellChangeTimes->{3.437270675875*^9, 3.442747789805125*^9}]
}, Open  ]],

Cell[TextData[{
 "In fact, the subring ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " does have a multiplicative identity, but it is different than the identity \
of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "! In general the existence of a multiplicative identity of a subring is \
totally independent of the multiplicative identity of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". A ring without an identity may have a subring with an identity, or \
vice-versa. This is the main reason why the existance of a multiplicative \
identity is not required in the definition of a ring.\n\nEXPERIMENT:\nTry to \
find all of the subrings of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", using the ",
 StyleBox["AddTable",
  FontWeight->"Bold"],
 " and ",
 StyleBox["MultTable",
  FontWeight->"Bold"],
 " commands. For example, the set "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["S = {0, a, b, a + b}", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", "b", ",", 
   RowBox[{"a", "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442747792602*^9}]
}, Open  ]],

Cell["is not a subring by looking at the tables", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["AddTable[S]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 0, 0}, {0, 0, 0}, {1, 0, 1}, {0, 0, 0}}, {{0, 1,
     1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {
    0, 0, 0}, {1, 0, 0}, {0, 0, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}, {1, 0, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], 
     InsetBox["\<\" \"\>", {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], 
     InsetBox["\<\" \"\>", {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], 
     InsetBox["\<\" \"\>", {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], 
     InsetBox["\<\" \"\>", {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.442747794711375*^9}]
}, Open  ]],

Cell["and", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[S]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{0, 1,
     1}, {1, 1, 0}, {1, 1, 0}, {0, 1, 1}, {0, 1, 1}}, {{1, 0, 1}, {1, 1, 0}, {
    1, 0, 1}, {1, 1, 0}, {1, 0, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
    0}, {1, 1, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.44274780588325*^9}]
}, Open  ]],

Cell[TextData[{
 "and seeing the black squares in one of the tables. How many ways can you \
pick a set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " so that the black squares aren't there?\n\nHint: The additive group for \
this ring is ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 ". How many subgroups of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 " are there?\n\nIn the case of groups, the intersection of a number of \
subgroups was again a subgroup. This last experiment suggested a close tie \
between subgroups and subrings, so it is natural to ask whether an \
intersection of a number of subrings is another subring. This quickly leads \
to the next proposition.\n\nPROPOSITION 10.2\nGiven any non-empty collection \
of subrings of the group ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", denoted by ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 ", then the intersection of all of the subrings in the collection \n\n       \
                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["H", 
     RowBox[{"*", " "}]], "=", " ", 
    RowBox[{
     StyleBox[
      UnderscriptBox[
       RowBox[{"  ", 
        SubscriptBox[
         StyleBox["\[Intersection]",
          FontSize->24], " "]}], 
       RowBox[{" ", 
        RowBox[{"H", " ", "\[Element]", " ", "L"}]}]],
      FontSize->18], "H"}]}], TraditionalForm]]],
 "\nis a subring of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4372706975*^9, 3.437270720609375*^9}, {
  3.442747812602*^9, 3.44274781369575*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "                         \nFirst of all, note that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["H", "*"], TraditionalForm]]],
 " is not the empty set, since 0 is in each ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " in the collection.  We now can apply proposition 10.1. Let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " be two elements in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["H", "*"], TraditionalForm]]],
 ".  Then, for every ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Element]", " ", "L"}], TraditionalForm]]],
 ", we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", ",", 
    RowBox[{"y", " ", "\[Element]", " ", "H"}]}], TraditionalForm]]],
 ".\n\nSince each ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", we have\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "-", " ", "y"}], " ", "\[Element]", " ", "H", " "}], 
   TraditionalForm]]],
 "    and   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "\[Element]", " ", "H"}], 
   TraditionalForm]]],
 "\n           \nfor all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", "\[Element]", "L"}], TraditionalForm]]],
 ".  Therefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "-", "y"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["H", "*"], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["H", "*"], TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437270765109375*^9, 3.43727080021875*^9}, {
  3.437270847625*^9, 3.4372708698125*^9}}]
}, Open  ]],

Cell[TextData[{
 "As with subgroups, we now have a general method of producing subrings of a \
ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " be any subset of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". We can consider the collection ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " of all subrings of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which contain the set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.438517673765625*^9, 3.4385176944375*^9}, {
  3.43851774603125*^9, 3.438517756484375*^9}}],

Cell[TextData[{
 "This collection is non-empty since it contains the subring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " itself. So by proposition 10.2,"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437270884078125*^9, 3.437270884265625*^9}}],

Cell[TextData[{
 "\n                              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"[", "S", "]"}], " "], "=", " ", 
    RowBox[{
     StyleBox[
      UnderscriptBox[
       RowBox[{"  ", 
        SubscriptBox[
         StyleBox["\[Intersection]",
          FontSize->24], " "]}], 
       RowBox[{" ", 
        RowBox[{"H", " ", "\[Element]", " ", "L"}]}]],
      FontSize->18], "H"}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43727088771875*^9, 3.437270891359375*^9}, {
  3.4385177011875*^9, 3.4385177011875*^9}}],

Cell[TextData[{
 "is a subring of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". By the way that the collection was defined, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " contains ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ".  \n\nActually, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " is the ",
 StyleBox["smallest",
  FontSlant->"Italic"],
 " subring of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " containing the subset ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". For if ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which contains ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[Element]", " ", "L"}], TraditionalForm]]],
 ", so that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"[", "S", "]"}], " ", "\[SubsetEqual]", " ", "H"}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373032168125*^9, 3.43730321996875*^9}, {
  3.43851770490625*^9, 3.43851773321875*^9}, {3.43851844390625*^9, 
  3.438518446875*^9}}],

Cell[TextData[{
 "\nDEFINITION 10.2:\nWe call ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " the subring of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " ",
 StyleBox["generated",
  FontSlant->"Italic"],
 " by the set ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ".  \n\nJust as in the case for the Group command, the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command ",
 StyleBox["Ring",
  FontWeight->"Bold"],
 " finds ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", "S", "]"}], TraditionalForm]]],
 " for any set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". Thus, we can find the subring generated by ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " with the command\n\n",
 StyleBox["Ring[S]",
  FontWeight->"Bold"],
 "\n\nprovided that the ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " has already been entered.\n\nFor example, let us use this command to find \
all of the subrings of the non-commutative group of order 8."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437270899265625*^9, 3.43727092609375*^9}, {
  3.438517716578125*^9, 3.4385177244375*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[4 a, 0]; Define[2 b, 0]
Define[a.a, a]; Define[b.b, 0]
Define[a.b, b]; Define[b.a, 2 a]
R = Ring[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437270948640625*^9, 3.43727101796875*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.437303253890625*^9, 3.44274782832075*^9}]
}, Open  ]],

Cell["\<\
Let us first try to find rings generated by only one element. Of course we \
would have the trivial ring\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Ring[{0}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", "0", "}"}]], "Output",
 CellChangeTimes->{3.442747830523875*^9}]
}, Open  ]],

Cell["\<\
but let us look at the other 7 elements. The ring generated by the next \
element\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Ring[{a}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442747833086375*^9}]
}, Open  ]],

Cell[TextData[{
 "contains 4 elements, while the ring generated by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "a"}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Ring[{2a}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4427478351645*^9}]
}, Open  ]],

Cell["\<\
contains only two. 

EXPERIMENT:
By going though the remaining 5 elements, determine how many subrings can be \
generated by just one element.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[BoxData["     "], "Input"],

Cell["\<\

Can there be any subrings which require two generators? Since the additive \
structure of a subring is a subgroup of the additive structure of the \
original ring, Lagrange's theorem holds for rings as well as groups. Thus, \
any subring of a group of order 8 must be of order 1,2,4, or 8. The \
experiment has obviously found all subrings of order 2. The only other \
non-trivial subrings would be of order 4. Such a subring could not contain \
any element which generated a ring of order 4. Thus, the only elements such a \
subring could contain would be 0, 2a, b, and 2a+b. Let us check if these four \
elements form a subring.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["S = {0, 2a, b, 2a + b}", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "a"}], ",", "b", ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442752772336375*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["AddTable[S]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}}, {{0, 1,
     1}, {0, 1, 1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {
    1, 1, 0}, {1, 0, 0}, {0, 1, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
    1}, {1, 0, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.44275277432075*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MultTable[S]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{0, 1,
     1}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{1, 0, 1}, {1, 1, 0}, {
    1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
    0}, {1, 1, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.44275277550825*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "So this gives us one more non-trivial subring, which gives a total of 6 \
non-trivial subrings.\n\nLet us turn our attention to finding the subrings of \
an infinite ring. Obviously, we will not be able to have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " produce multiplication tables, but we ought to be able to derive some \
subrings for some common infinite rings. The simpliest such ring is the ring \
of integers, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". The subring generated by a positive number ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " would be the set of all multiples of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". This is indeed a subring, since the difference and the product of two \
multiplies of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " yields a multiple of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". What is interesting is that ",
 StyleBox["all",
  FontSlant->"Italic"],
 " subrings of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " are produced this way.\n\nPROPOSTION 10.3\nA subring of the ring of \
integers ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " consists of all multiples of some non-negative number ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". This subring is denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437271050625*^9, 3.43727105078125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First of all, the trivial subring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "0", "}"}], TraditionalForm]]],
 " can be considered the set of all multiples of ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 ". Also, the entire ring ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " could be considered all of the multiples of 1. Let ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " be a non-trivial subring, and let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "x"}], TraditionalForm]]],
 " is also in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " must contain some positive integers. Let ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " be the smallest positive integer contained in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". Certainly all multiples of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " would be in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", but suppose that some element ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is not a multiple of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Then by the greatest common divisor theorem (1.2), there exists two \
integers ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " such that\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"u", " ", "n"}], " ", "+", " ", 
      RowBox[{"v", " ", "m"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"GCD", "(", 
       RowBox[{"n", ",", "m"}], ")"}], "."}]}]}], TraditionalForm]]],
 "\n\nSince ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is closed under addition, this implies that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"GCD", "(", 
    RowBox[{"n", ",", "m"}], ")"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ".  But ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " is not a multiple of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"GCD", "(", 
     RowBox[{"n", ",", "m"}], ")"}], " ", "<", " ", "n"}], 
   TraditionalForm]]],
 ". But this contradicts the fact that ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the ",
 StyleBox["smallest",
  FontSlant->"Italic"],
 " positive integer in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " consists exactly of all of the multiples of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", " ", "=", " ", 
    RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}]}], TraditionalForm]]],
 ". "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437271125875*^9, 3.437271126046875*^9}}]
}, Open  ]],

Cell[TextData[{
 "Although the subrings of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " are easily classified, this is not the case with the ring of real numbers. \
Consider the set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " of all numbers of the form  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"y", " ", 
     RadicalBox["2", " "]}]}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are rational numbers. We can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " verify that the product of two such numbers"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ClearDefs
Expand[(x1 + y1 2^(1/2))(x2 + y2 2^(1/2) )]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  RowBox[{"x1", " ", "x2"}], "+", 
  RowBox[{
   SqrtBox["2"], " ", "x2", " ", "y1"}], "+", 
  RowBox[{
   SqrtBox["2"], " ", "x1", " ", "y2"}], "+", 
  RowBox[{"2", " ", "y1", " ", "y2"}]}]], "Output",
 CellChangeTimes->{3.43727122653125*^9, 3.4427527835395*^9}]
}, Open  ]],

Cell[TextData[{
 "produces a number in this form. Since ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is obviously closed with respect to subtraction, ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalR]", TraditionalForm]]],
 ".\n\nEXPERIMENT:\nDoes the set of all numbers of the form   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"y", " ", 
     RadicalBox["2", "3"]}]}], TraditionalForm]]],
 " form a subring? If not, what is the subring generated by these numbers?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["     ", "Input",
 PageWidth->Infinity],

Cell["\<\
Clearly, the subrings of the real numbers can be much more complicated than \
the subrings of the integers. A natural question to ask is whether we can \
produce quotient rings from these subrings the way we constructed quotient \
groups out of the normal subgroups. We will look into this in the next \
section.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Quotient Rings and Ideals", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
When we studied group theory, one of the most important concepts we \
discovered was being able to  form a quotient group out of the cosets of \
certain subgroups. After some investigation we learned exactly which \
subgroups yielded such a quotient group: the normal subgroups. A natural \
question is whether it is possible to form quotient rings out of the cosets \
of a subring.

Let us look at an example. Here is the non-commutative ring of order eight \
from the last section.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417226843702497`*^9, 3.4417226914483747`*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[4 a, 0]; Define[2 b, 0]
Define[a.a, a]; Define[b.b, 0]
Define[a.b, b]; Define[b.a, 2 a]
R = Ring[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437271341453125*^9, 3.43727137340625*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.43727142121875*^9, 3.44275278925825*^9}]
}, Open  ]],

Cell["We found this ring has 6 non-trivial subrings.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
S1 = {0,a,2a,3a};
S2 = {0,2a};
S3 = {0,b};
S4 = {0,a+b,2a,3a+b};
S5 = {0,2a + b};
S6 = {0,2a,b,2a+b};\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "If there is a quotient ring, what would the elements of this quotient be? \
We would expect the additive structure of the quotient ring to be the \
additive quotient group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "S"}], TraditionalForm]]],
 ". Since addition is commutative, we know that this quotient group always \
exists. We can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the cosets of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " under the operation of addition. Since left and right cosets are the same \
when working with rings, we will simply use the command ",
 StyleBox["Coset[S,R] ",
  FontWeight->"Bold"],
 "for the list of cosets of the subgroup ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". For the subgroup ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "1"], TraditionalForm]]],
 ", here are the cosets:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Q = Coset[R,S1]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "a", ",", 
     RowBox[{"2", " ", "a"}], ",", 
     RowBox[{"3", " ", "a"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"b", ",", 
     RowBox[{"a", "+", "b"}], ",", 
     RowBox[{
      RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
     RowBox[{
      RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4372714299375*^9, 3.442752793977*^9}]
}, Open  ]],

Cell[TextData[{
 "We can ",
 StyleBox["add",
  FontSlant->"Italic"],
 " two cosets together using the following definition:\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", " ", "+", " ", "Y"}], " ", "=", " ", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"x", " ", "+", " ", "y"}], "  ", "|", "  ", 
      RowBox[{
      "x", " ", "\[Epsilon]", " ", "X", "   ", "and", "   ", "y", " ", 
       "\[Epsilon]", " ", "Y"}]}], "}"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "This gives the quotient ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " the natural addition. We can see the addition table for ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " as follows: "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["AddTable[Q]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 1}, {1, 0, 1}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0,
     1}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}}}], InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"{0,a,2a,3a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{b,a+b,2a+b,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"{0,a,2a,3a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{b,a+b,2a+b,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"{0,a,2a,3a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{b,a+b,2a+b,3a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{b,a+b,2a+b,3a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,a,2a,3a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 2}, {3, 2}}]},
  ImageSize->{417.9999999999978, Automatic}]], "Output",
 CellChangeTimes->{3.4427527960395*^9}]
}, Open  ]],

Cell[TextData[{
 "Can we now multiply elements of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " to form a ring? The natural way to define the product of two sets is the \
way we defined such a product for groups:\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", "\[CenterDot]", "Y"}], " ", "=", " ", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "y"}], "  ", "|", "  ", 
      RowBox[{"x", " ", "\[Element]", " ", 
       RowBox[{"X", "   ", "and", "   ", "y"}], " ", "\[Element]", " ", 
       "Y"}]}], "}"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437271503953125*^9, 3.43727150715625*^9}}],

Cell[TextData[{
 "\nWill such a product of two cosets in ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " yield another coset? Let's try it!"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[Q]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 1}, {1, 1, 0}, {0, 0, 0}}, {{1, 1, 0}, {1, 1, 0}, {0, 0,
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"{0,a,2a,3a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{b,a+b,2a+b,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"{0,a,2a,3a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{b,a+b,2a+b,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"{0,a,2a,3a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], 
     InsetBox["\<\" \"\>", {2.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,a,2a,3a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], 
     InsetBox["\<\" \"\>", {2.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 2}, {3, 2}}]}]], "Output",
 CellChangeTimes->{3.442752808117625*^9}]
}, Open  ]],

Cell[TextData[{
 "Hang on! Why are there black squares in the table! ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is trying to tell us that the product of two cosets is not always a coset. \
For example,"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["{b,a+b,2a+b,3a+b} . {0,a,2a,3a}", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442752814773875*^9}]
}, Open  ]],

Cell["in indeed a coset, but the product", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["{0,a,2a,3a}.{b,a+b,2a+b,3a+b} ", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.437271528921875*^9, 3.442752816148875*^9}]
}, Open  ]],

Cell[TextData[{
 "produced way too many elements! What is going on? Since {0,a,2a,3a} is the \
zero element for ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", we would ",
 StyleBox["expect",
  FontSlant->"Italic"],
 " the product to be {0,a,2a,3a}. But we somehow got some extra elements. \
Where did they come from?\n\nEXPERIMENT:\nThe above product demonstrates that \
there are two elements of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "2"], TraditionalForm]]],
 ", such that\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["x", "1"], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"b", "+", " ", 
       SubscriptBox["x", "2"]}], ")"}]}], " "}], TraditionalForm]]],
 "\n\nis not in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", along with a little trial and error, to find two such elements. For \
example, to see whether ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["x", "1"],
     TraditionalForm], "=", " ", 
    RowBox[{
     FormBox[
      SubscriptBox["x", "2"],
      TraditionalForm], "=", "0"}]}], TraditionalForm]]],
 " works, enter\n"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
x1 = 0
x2 = 0
x1 . (b + x2)\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.44275282425825*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.44275282425825*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.44275282425825*^9}]
}, Open  ]],

Cell[TextData[{
 "Change ",
 StyleBox["x1",
  FontWeight->"Bold"],
 " and ",
 StyleBox["x2",
  FontWeight->"Bold"],
 " so that the result is not in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ".\n\nAlthough it seems as though our attempt to define a quotient ring has \
failed miserably, we might want to study this example to understand ",
 StyleBox["why",
  FontSlant->"Italic"],
 " we failed to have a quotient group. Since the additive identity of the \
quotient group is the subring ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " times any other coset should give ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " again. But that means that subring ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " times any element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " would have to produce only elements in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". But we have seen an example where this wasn't the case, namely"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["S1 . (b + x2)", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "b"}], "}"}]], "Output",
 CellChangeTimes->{3.442752834336375*^9}]
}, Open  ]],

Cell[TextData[{
 "Since the elements produced were not all in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " did not act like the zero element in the product.\n\nBut wait! What if we \
found a subring for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", "\[CenterDot]", "x"}], TraditionalForm]]],
 " always was a subset of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " for every ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "? By a similar logic, we would also have to require that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "S"}], TraditionalForm]]],
 " be a subset of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". A quick way to test this is to see if the products ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", "\[CenterDot]", "R"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[CenterDot]", "S"}], TraditionalForm]]],
 " are both subsets of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". Let us check the second subring of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " that we found."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["S2 . R", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.43727155290625*^9, 3.44275283794575*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["R . S2", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.437271555015625*^9, 3.442752839352*^9}]
}, Open  ]],

Cell["\<\
So far, so good. This at least ensures that the additive identity {0, 2a} \
will behave property for multiplication. Let us use the multiplication table \
to see if the product of any two cosets will again be a coset. \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Q = Coset[R,S2]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"2", " ", "a"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", 
     RowBox[{"3", " ", "a"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"b", ",", 
     RowBox[{
      RowBox[{"2", " ", "a"}], "+", "b"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "+", "b"}], ",", 
     RowBox[{
      RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.437271585109375*^9, 3.442752842430125*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["MultTable[Q]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{0, 1,
     1}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{1, 0, 1}, {1, 1, 0}, {
    1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
    0}, {1, 1, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"{0,2a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a,3a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{b,2a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a+b,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"{0,2a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a,3a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{b,2a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a+b,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"{0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{a,3a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{b,2a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{a+b,3a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{a,3a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{b,2a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{a+b,3a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.43727158725*^9, 3.442752843555125*^9}]
}, Open  ]],

Cell[TextData[{
 "This looks pretty good--no black squares, so we have at least defined a \
multiplication for these cosets. It is interesting that this multiplication \
table is not commutative, even though all of the subrings of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " were commutative. So this quotient cannot be like any of the subrings of \
",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ".\n\nBut look closely at this table. Not every product yields a coset, \
sometimes it yields only a ",
 StyleBox["subset",
  FontSlant->"Italic"],
 " of a coset. For appearances, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has colored such a subset according to the color of the coset. \n\nOne way \
to rectify this slight blemish in our multiplication table is to add the \
identity coset to each entry in the table. That is, instead of defining the \
product of the cosets ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " to be ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"X", "\[CenterDot]", "Y"}], ","}], " "}], TraditionalForm]]],
 "we define the product of two cosets to be\n\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"X", "*", "Y"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"X", "\[CenterDot]", "Y"}], " ", "+", " ", 
      RowBox[{"S", "."}]}]}]}], TraditionalForm]]],
 "\n\nWe can repair the multiplication table with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["QuotientRing = True", "Input",
 PageWidth->Infinity],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.437271647015625*^9, 3.442752857352*^9}]
}, Open  ]],

Cell["\<\
A new multiplication table will now be constructed using this new definition \
of the product of two cosets.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[Q]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{0, 1,
     1}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{1, 0, 1}, {1, 1, 0}, {
    1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
    0}, {1, 1, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"{0,2a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a,3a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{b,2a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a+b,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"{0,2a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a,3a}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{b,2a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a+b,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{a,3a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{b,2a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{a+b,3a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{a,3a}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{b,2a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{a+b,3a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.43727164965625*^9, 3.4427528585395*^9}]
}, Open  ]],

Cell[TextData[{
 "This looks much better! We still have not shown that this multiplication \
table obeys the laws of a ring, but we will get to that shortly. First, let \
us see if any of the other subrings of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " could form a multiplication table."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Q = Coset[R,S3]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", 
     RowBox[{"a", "+", "b"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"2", " ", "a"}], ",", 
     RowBox[{
      RowBox[{"2", " ", "a"}], "+", "b"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"3", " ", "a"}], ",", 
     RowBox[{
      RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.43727166515625*^9, 3.44275286319575*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["MultTable[Q]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 0}, {1, 1, 0}, {0, 0, 0}, {0, 1, 1}, {0, 0, 0}}, {{0, 1,
     1}, {1, 1, 0}, {0, 0, 0}, {1, 1, 0}, {0, 0, 0}}, {{1, 0, 1}, {1, 1, 0}, {
    0, 0, 0}, {0, 1, 1}, {0, 0, 0}}, {{1, 1, 0}, {1, 1, 0}, {0, 0, 0}, {1, 1, 
    0}, {0, 0, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"{0,b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a,a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{2a,2a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{3a,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"{0,b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a,a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{2a,2a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{3a,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"{0,b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], 
     InsetBox["\<\" \"\>", {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], 
     InsetBox["\<\" \"\>", {4.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], 
     InsetBox["\<\" \"\>", {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2a,2a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], 
     InsetBox["\<\" \"\>", {4.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], 
     InsetBox["\<\" \"\>", {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], 
     InsetBox["\<\" \"\>", {4.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], 
     InsetBox["\<\" \"\>", {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{2a,2a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], 
     InsetBox["\<\" \"\>", {4.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 4}, {5, 4}}]}]], "Output",
 CellChangeTimes->{3.4427528646645*^9}]
}, Open  ]],

Cell["This subring seemed to fail, but notice that the products", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
R.S3
S3.R\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "b"}], "}"}]], "Output",
 CellChangeTimes->{3.442752868086375*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442752868086375*^9}]
}, Open  ]],

Cell[TextData[{
 "are not both contained in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "3"], TraditionalForm]]],
 ".\n\nEXPERIMENT:\nTry forming the multiplication tables for the cosets of \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "4"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "5"], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["S", "6"], TraditionalForm]]],
 ". For which subrings do the cosets form multiplication tables? For which \
subrings are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[CenterDot]", "S"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", "\[CenterDot]", "R"}], TraditionalForm]]],
 " both contained in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "The pattern is apparent. Subrings of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[CenterDot]", "S"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", "\[CenterDot]", "R"}], TraditionalForm]]],
 " are both contained in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " have a special property, and hence we give such subrings a special name. \
Rather then calling them \"normal subrings\" we will use the more traditional \
name.\n\nDEFINITION 10.3:\nA subring ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " of a ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is called an ",
 StyleBox["ideal",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " if both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", "\[CenterDot]", "R"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[CenterDot]", "I"}], TraditionalForm]]],
 " are contained in the subring ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ".\n\nWe now are ready to prove that the cosets of an ideal ready do form a \
quotient ring.\n\nPROPOSITION 10.4\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a ring, and let ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " be an ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". The the addition quotient group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "I"}], TraditionalForm]]],
 " forms a ring, with the product of two cosets ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " being ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", "*", "Y"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"X", "\[CenterDot]", "Y"}], " ", "+", " ", 
     RowBox[{"I", "."}]}]}], TraditionalForm]]],
 " This ring is called the ",
 StyleBox["quotient ring",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"I", "."}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4372716834375*^9, 3.4372716975625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The quotient group ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "I"}], TraditionalForm]]],
 " is an abelian group, so we need only to check that the multiplication is \
closed, and that the associativity and two distributive laws hold. \n\nLet ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " be two cosets of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"I", "."}]}], TraditionalForm]]],
 " Let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be an element in ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " an element in ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ". Then the product of the cosets ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 " is\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"X", "*", "Y"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"X", "\[CenterDot]", "Y"}], " ", "+", " ", "I"}], " ", "=", " ",
      
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"x", " ", "+", " ", "I"}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"y", " ", "+", " ", "I"}], ")"}]}], " ", "+", " ", "I"}], 
      " ", "=", " ", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", "y"}], " ", "+", " ", 
       RowBox[{"I", "\[CenterDot]", "y"}], " ", "+", " ", 
       RowBox[{"x", "\[CenterDot]", "I"}], " ", "+", " ", 
       RowBox[{"I", "\[CenterDot]", "I"}], " ", "+", " ", 
       RowBox[{"I", "."}]}]}]}]}], TraditionalForm]]],
 "\n\nBecause ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " is an ideal, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", "\[CenterDot]", "y"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "I"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", "\[CenterDot]", "I"}], TraditionalForm]]],
 " are all subsets of ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ". Hence, the sum ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"I", "\[CenterDot]", "y"}], " ", "+", " ", 
    RowBox[{"x", "\[CenterDot]", "I"}], " ", "+", " ", 
    RowBox[{"I", "\[CenterDot]", "I"}], " ", "+", " ", "I"}], 
   TraditionalForm]]],
 " will be a subset of ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ". But since the last term of this expression is ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"I", "\[CenterDot]", "y"}], " ", "+", " ", 
    RowBox[{"x", "\[CenterDot]", "I"}], " ", "+", " ", 
    RowBox[{"I", "\[CenterDot]", "I"}], " ", "+", " ", "I"}], 
   TraditionalForm]]],
 " contains the ideal ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", so this sum equals ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ". Thus,\n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "+", " ", "I"}], ")"}], "*", 
       RowBox[{"(", 
        RowBox[{"y", " ", "+", " ", "I"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"X", "*", "Y"}], " ", "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"X", "\[CenterDot]", "Y"}], " ", "+", " ", "I"}], " ", "=", 
        " ", 
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "y"}], " ", "+", " ", "I"}]}]}]}], 
     ","}]}], TraditionalForm]]],
 "\n\nwhich is a coset of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "I"}], TraditionalForm]]],
 ".\n\nNow suppose that ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["Y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["Z", TraditionalForm]]],
 " are three cosets of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "/", "I"}], ","}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " being representative elements, respectively. Then\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"X", "*", "Y"}], ")"}], "*", "Z"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"x", " ", "+", " ", "I"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"y", " ", "+", " ", "I"}], ")"}]}], ")"}], "*", 
       RowBox[{"(", 
        RowBox[{"z", " ", "+", " ", "I"}], ")"}]}]}], " ", "\n", 
     "\t\t\t            ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "y"}], " ", "+", " ", "I"}], ")"}], "*",
        
       RowBox[{"(", 
        RowBox[{"z", " ", "+", " ", "I"}], ")"}]}]}], " ", "\n", 
     "\t\t\t\t         ", 
     RowBox[{"=", "  ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "\[CenterDot]", "z"}], 
        " ", "+", " ", "I"}], ")"}]}], "\n", "\t\t\t             ", 
     RowBox[{"=", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{"y", "\[CenterDot]", "z"}], ")"}]}], " ", "+", " ", "I"}], 
       ")"}]}], " ", "\n", "\t\t\t   \t\t   ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "+", " ", "I"}], ")"}], "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"y", "\[CenterDot]", "z"}], " ", "+", " ", "I"}], ")"}]}]}], 
     " ", "\n", "\t\t\t\t\t\t  ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "+", " ", "I"}], ")"}], "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"y", " ", "+", " ", "I"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"z", " ", "+", " ", "I"}], ")"}]}], ")"}]}]}], " ", "\n", 
     "\t\t\t\t\t\t   ", 
     RowBox[{"=", " ", 
      RowBox[{"X", "*", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"Y", "*", "Z"}], ")"}], "."}]}]}]}]}], TraditionalForm]]],
 "\n\nSo multiplication is associative. Also,\n\n     ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"X", "*", 
      RowBox[{"(", 
       RowBox[{"Y", " ", "+", " ", "Z"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "+", " ", "I"}], ")"}], "*", 
      RowBox[{"(", 
       RowBox[{"y", " ", "+", " ", "z", " ", "+", " ", "I"}], ")"}]}]}], 
    " "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t             ", 
    RowBox[{"=", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"x", "(", 
        RowBox[{"y", "+", "z"}], ")"}], " ", "+", " ", "I"}], ")"}]}], " "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t\t          ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "y"}], " ", "+", " ", 
      RowBox[{"x", "\[CenterDot]", "z"}], " ", "+", " ", "I"}]}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t\t\t      ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "y"}], " ", "+", " ", "I"}], ")"}], " ", 
      "+", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "z"}], " ", "+", " ", "I"}], ")"}]}]}], 
    " "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t\t\t\t   ", 
    RowBox[{
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"X", "*", "Y"}], " ", "+", " ", 
       RowBox[{"X", "*", "Z"}]}]}], ",", " "}]}], TraditionalForm]}]],
 "\nand \n \n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"X", " ", "+", " ", "Y"}], ")"}], "*", "Z"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "+", " ", "y", " ", "+", " ", "I"}], ")"}], "*", 
       RowBox[{"(", 
        RowBox[{"z", " ", "+", " ", "I"}], ")"}]}]}], " ", "\n", 
     "\t\t\t               ", 
     RowBox[{"=", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"x", "+", "y"}], ")"}], "\[CenterDot]", "z"}], " ", "+", 
        " ", "I"}], ")"}]}], " ", "\n", "\t\t\t\t            ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", "z"}], " ", "+", " ", 
       RowBox[{"y", "\[CenterDot]", "z"}], " ", "+", " ", "I"}]}], " ", "\n", 
     "\t\t\t\t\t         ", 
     RowBox[{"=", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "z"}], " ", "+", " ", "I"}], ")"}], " ",
        "+", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"y", "\[CenterDot]", "z"}], " ", "+", " ", "I"}], ")"}]}]}], 
     " ", "\n", "\t\t\t\t\t\t     ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"X", "*", "Z"}], " ", "+", " ", 
       RowBox[{"Y", "*", 
        RowBox[{"Z", "."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nThus, the two distributive laws hold, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "I"}], TraditionalForm]]],
 " is a ring.       "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "From the experiment, you should have discovered that three of the six \
non-trivial subrings were ideals. The two trivial subrings, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "0", "}"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " itself, will be ideals for any ring. These two ideals are called the ",
 StyleBox["trivial ideals",
  FontSlant->"Italic"],
 ".\n\nLet us see if we can find ideals for other rings. We have already \
discovered through proposition 10.3 that every subring of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[CenterDot]", "\[DoubleStruckCapitalZ]"}], 
   TraditionalForm]]],
 " for some non-negative ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", " ", "=", " ", 
    RowBox[{"n", "\[CenterDot]", "Z"}]}], TraditionalForm]]],
 ", then since any multiple of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " times any integer gives a multiple of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"S", "\[CenterDot]", "\[DoubleStruckCapitalZ]"}], " ", "=", " ", 
    "S"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "\[CenterDot]", "S"}], " ", "=", " ", 
    "S"}], TraditionalForm]]],
 ". Thus, any subring of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is an ideal of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". \n\nWhat is the quotient ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "/", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}], ")"}], "?"}]}], 
   TraditionalForm]]],
 " The cosets can all be expressed in the form   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", " ", "+", " ", 
     RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}]}], ","}], 
   TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", " ", "=", " ", "0"}], ",", " ", "1", ",", " ", "2", ",", " ",
     
    RowBox[{
     RowBox[{"\[Ellipsis]", " ", "n"}], "-", "1"}]}], TraditionalForm]]],
 ". Addition and multiplication is easily discribed:\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{" ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"a", " ", "+", " ", 
           RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}]}], ")"}], "+", 
         RowBox[{"(", 
          RowBox[{"b", " ", "+", " ", 
           RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}]}], ")"}]}], " ", "=",
         " ", 
        RowBox[{"(", 
         RowBox[{"a", " ", "+", " ", "b", " ", "+", " ", 
          RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}]}], ")"}]}], " ", "\n",
        "\t\t\t                                                 ", 
       RowBox[{"=", " ", 
        RowBox[{"a", "+", 
         RowBox[{"b", "  ", 
          RowBox[{"(", 
           RowBox[{"Mod", " ", "n"}], "]"}]}]}]}]}], ")"}], "+", " ", 
     RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}]}], ","}], 
   TraditionalForm]]],
 " \n\nand \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", " ", "+", " ", 
         RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}]}], ")"}], 
       "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"b", " ", "+", " ", 
         RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}]}], ")"}]}], " ", "=", 
      " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"a", "\[CenterDot]", "b"}], " ", "+", " ", 
        RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}]}], ")"}]}], " ", "\n", 
     "\t\t\t                                               ", 
     RowBox[{"=", 
      RowBox[{
       RowBox[{
        RowBox[{"a", "\[CenterDot]", "b"}], " ", 
        RowBox[{"(", 
         RowBox[{"Mod", "  ", "n"}], ")"}]}], "+", " ", 
       RowBox[{"n", " ", 
        RowBox[{"\[DoubleStruckCapitalZ]", "."}]}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nClearly the quotient ring behaves exactly like the ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". To put this another way, the quotient ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "/", 
    RowBox[{"(", 
     RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}], ")"}]}], 
   TraditionalForm]]],
 " is  ",
 StyleBox["isomorphic",
  FontSlant->"Italic"],
 " to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". We will study ring isomorphisms in the next section.\n\nIn contrast, let \
us consider a ring such as the rational numbers ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 ". We can quickly see that the only ideals of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 " are the trival ideals, even though there are a host of subrings of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 ". The proof holds for any field, so let us proof the following proposition.\
\n\nPROPOSITION 10.5\nAny field or skew field can only have trivial ideals."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437272232015625*^9, 3.4372722324375*^9}, {
  3.43727227721875*^9, 3.437272343015625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be a field or skew field, and suppose that there is a non-trivial ideal ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Then there is a non-zero element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", and hence ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " exists in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Thus\n\n                            ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "=", " ", 
    RowBox[{
     RowBox[{"x", "\[CenterDot]", 
      SuperscriptBox["x", 
       RowBox[{"-", "1"}]]}], "  ", "\[Element]", " ", 
     RowBox[{"I", "\[CenterDot]", "K"}], "  ", "\[SubsetEqual]", " ", "I"}]}],
    TraditionalForm]]],
 ".\n                            \nSo the multiplicative identity ",
 Cell[BoxData[
  FormBox["1", TraditionalForm]]],
 " is contained in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ".  But then, \n\n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", " ", "=", " ", 
    RowBox[{
     RowBox[{"1", "\[CenterDot]", "K"}], " ", "\[SubsetEqual]", " ", 
     RowBox[{"I", "\[CenterDot]", "K"}], " ", "\[SubsetEqual]", " ", "I"}]}], 
   TraditionalForm]]],
 ".\n                         \nHence, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", " ", "=", " ", "K"}], TraditionalForm]]],
 ", so the only ideals of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are the trivial ideals."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437272362*^9, 3.437272362015625*^9}}]
}, Open  ]],

Cell[TextData[{
 "We wish to develop a way to find the ideals of a general ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". We have already observed that the intersection of two subrings is again a \
subring. Using this fact, we were able to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find all of the subrings. So the natural question is whether the \
intersection of two ideals gives an ideal. If so, we will be able to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find all of the ideals of a finite ring.\n\nPROPOSITION 10.6\nIf ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 " is a non-empty collection of ideal of a ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", then the intersection of all of these ideals \n\n                         \
             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["I", 
     RowBox[{"*", " "}]], "=", " ", 
    RowBox[{
     StyleBox[
      UnderscriptBox[
       RowBox[{"  ", 
        SubscriptBox[
         StyleBox["\[Intersection]",
          FontSize->24], " "]}], 
       RowBox[{" ", 
        RowBox[{"I", " ", "\[Element]", " ", "L"}]}]],
      FontSize->18], 
     StyleBox["I",
      FontSize->12]}]}], TraditionalForm]]],
 "\nis an ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437272386859375*^9, 3.437272408734375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["I", "*"], TraditionalForm]]],
 " is an intersection of subrings of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", by proposition 10.2  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["I", "*"], TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Thus, we only need to check that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["I", "*"],
     TraditionalForm], "\[CenterDot]", "R"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[CenterDot]", 
    SuperscriptBox["I", "*"]}], TraditionalForm]]],
 " are contained in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["I", "*"], TraditionalForm]]],
 ".\n\nSuppose that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is an element of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["I", "*"], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in each ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", " ", "\[Element]", " ", "L"}], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "R"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[CenterDot]", "x"}], TraditionalForm]]],
 " are subsets of each ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " in the collection. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "R"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[CenterDot]", "x"}], TraditionalForm]]],
 " will both be subsets of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["I", "*"], TraditionalForm]]],
 ". Since this result is true for every ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["I", "*"], TraditionalForm]]],
 ", we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["I", "*"], "\[CenterDot]", "R"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "\[CenterDot]", 
    SuperscriptBox["I", "*"]}], TraditionalForm]]],
 " are both subsets of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["I", "*"], TraditionalForm]]],
 ". Therefore, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["I", "*"], TraditionalForm]]],
 " is an ideal."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43727244521875*^9, 3.43727245971875*^9}}]
}, Open  ]],

Cell[TextData[{
 "We can now define the smallest ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which contains a subset ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". We proceed as we did for subrings, and consider the collection of all \
ideals of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " containing ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". This collection will be non-empty, since ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " itself is such an ideal. If we denote this collection by ",
 Cell[BoxData[
  FormBox["L", TraditionalForm]]],
 ", then the smallest ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " containing ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " would have to be\n\n                                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", "S", ")"}], " ", "=", " ", 
    RowBox[{
     StyleBox[
      UnderscriptBox[
       RowBox[{"  ", 
        SubscriptBox[
         StyleBox["\[Intersection]",
          FontSize->24], " "]}], 
       RowBox[{" ", 
        RowBox[{"I", " ", "\[Element]", " ", "L"}]}]],
      FontSize->18], 
     StyleBox["I",
      FontSize->12]}]}], TraditionalForm]]],
 ".\nWe call ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "S", ")"}], TraditionalForm]]],
 " the ",
 StyleBox["ideal generated by",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". Notice the distintion between this notation and the notation of the \
subring generated by ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"[", "S", "]"}], "."}], TraditionalForm]]],
 "\n\nWe can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " find the ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " generated by any set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " with the command ",
 StyleBox["Ideal[R,S]",
  FontWeight->"Bold"],
 ".\n\nFor example, in the non-commutative ring of order 8 which we were \
working with earlier,"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437272480015625*^9, 3.437272483296875*^9}, {
  3.43851860771875*^9, 3.4385186373125*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[4 a, 0];
Define[2 b, 0];
Define[a.b, b];
Define[b.a, 2 a];
Define[a.a, a];
Define[b.b, 0];
R = Ring[{a,b}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442752883367625*^9}]
}, Open  ]],

Cell[TextData[{
 "we can find all of the ideals rather quickly. For example, the ideal \
generated by the element ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Ideal[R,{a}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.44275288525825*^9, 3.4427528867895*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.44275288775825*^9}]
}, Open  ]],

Cell[TextData[{
 "so ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " cannot be contained in any non-trivial ideal. Likewise, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", "a"}], TraditionalForm]]],
 " would generate the whole ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"3", "\[CenterDot]", "3"}], "a"}], " ", "=", " ", "a"}], 
   TraditionalForm]]],
 ". If we try the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "+", "b"}], ","}], TraditionalForm]]],
 " we get"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Ideal[R,{a+b}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442752888930125*^9}]
}, Open  ]],

Cell[TextData[{
 "This tells us that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "+", "b"}], TraditionalForm]]],
 " is not in a non-trivial ideal. The same thing is true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"3", "a"}], "+", "b"}], " ", "=", " ", 
    RowBox[{"3", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "+", "b"}], ")"}], "."}]}]}], TraditionalForm]]],
 " So the only three elements which could be in a non-trivial ideal form the \
ideals"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Ideal[R,{2a}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442752890398875*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Ideal[R,{b}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "a"}], ",", "b", ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4427528912895*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Ideal[R,{2a+b}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.44275289213325*^9}]
}, Open  ]],

Cell[TextData[{
 "If we require two of the three of these elements to be in the ideal, it is \
clear that we will have the same ideal as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "b", ")"}], TraditionalForm]]],
 ". Thus, there are 5 ideals in this ring, which we can express as\n\n        \
  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", "0", ")"}], ",", " ", 
    RowBox[{"(", "a", ")"}], ",", " ", 
    RowBox[{"(", 
     RowBox[{"2", "a"}], ")"}], ",", " ", 
    RowBox[{"(", "b", ")"}], ",", "   ", 
    RowBox[{"and", "    ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", "a"}], "+", "b"}], ")"}]}]}], TraditionalForm]]],
 ".\n\nNotice that all 5 ideals could be generated with only one element. We \
will give a special name for these ideals."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437272583234375*^9, 3.437272612015625*^9}}],

Cell[TextData[{
 "DEFINITION 10.4:\nAn ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which is generated by only one element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is called a ",
 StyleBox["principal ideal",
  FontSlant->"Italic"],
 ". If all of the ideals of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " are principle ideals, then the ring is called a ",
 StyleBox["principal ideal ring",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437272583234375*^9, 3.437272616921875*^9}}],

Cell["\<\

EXPERIMENT:
In chapter 1 we found a non-commutative ring of order 8 with an identity:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437272583234375*^9, 3.437272850375*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[2 e,0];
Define[2 a,0];
Define[2 b,0];
Define[e.e,e];
Define[e.a,a];
Define[e.b,b];
Define[a.e,a];
Define[b.e,b];
Define[a.b,a];
Define[b.a,b];
Define[a.a,a];
Define[b.b,b];
R = Ring[{e,a,b}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", "e", ",", 
   RowBox[{"a", "+", "e"}], ",", 
   RowBox[{"b", "+", "e"}], ",", 
   RowBox[{"a", "+", "b", "+", "e"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442752896148875*^9}]
}, Open  ]],

Cell["\<\
Find all of the ideals of this ring. Is this a principal ideal ring?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[" ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "It should be noted that the ring of integers ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is a principal ideal ring, since all ideals (in fact all subrings) are of \
the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[CenterDot]", "\[DoubleStruckCapitalZ]"}], 
   TraditionalForm]]],
 ", which is generated by the single element ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is also an integral domain, we will combine the two terms and call ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " a ",
 StyleBox["principal ideal domain",
  FontSlant->"Italic"],
 ", or PID. Principal ideal domains play an important role in ring theory, as \
we will see in section 12.3."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437272878578125*^9, 3.4372728815625*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Ring Isomorphisms", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "As we work with different rings, it is natural to ask whether we can \
classify the finite rings the way that we did for small groups. Of course, we \
will consider two rings to be \"equivalent\" if the elements of one ring can \
be renamed to form the other ring. We have already seen an example of \
equivalent rings: the quotient ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "/", 
    RowBox[{"(", 
     RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}], ")"}]}], 
   TraditionalForm]]],
 " was essentually the same ring as ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". We would like to formalize this equivalence using the same technique that \
we used for groups.\n\nDEFINITION 10.5:\nLet ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " be two rings. A ",
 StyleBox["ring isomorphism",
  FontSlant->"Italic"],
 " from ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " is a one-to-one mapping\n\n                                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", ":", " ", 
    RowBox[{"A", " ", "\[RightArrow]", " ", "B"}]}], TraditionalForm]]],
 "\nsuch that \n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"x", "+", "y"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "x", ")"}], " ", "+", " ", 
     RowBox[{"\[Florin]", "(", "y", ")"}], "    "}]}], TraditionalForm]]],
 "and                     \n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Florin]", "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{"\[Florin]", "(", "y", ")"}]}], "            ", "for", "  ", 
      "all", "    ", "x"}]}], ",", 
    RowBox[{"y", " ", "\[Element]", " ", 
     RowBox[{"A", "."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43730285334375*^9, 3.437302901078125*^9}, {
  3.437302999859375*^9, 3.4373030715*^9}}],

Cell[TextData[{
 "If there exists a ring isomorphism from ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " which is surjective, then we say that the rings ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " are ",
 StyleBox["isomorphic",
  FontSlant->"Italic"],
 ", denoted by  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", "B"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "For example, we can define a mapping for the quotient ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "/", 
    RowBox[{"(", 
     RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}], ")"}]}], 
   TraditionalForm]]],
 " as follows:\n\n       \[Florin](a + n Z) = a (Mod n),  a = 0, 1, 2, \
\[Ellipsis], ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"n", "-", "1"}], ")"}], TraditionalForm]]],
 ".\n \nThen clearly ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an injective and surjective function from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "/", 
    RowBox[{"(", 
     RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}], ")"}]}], 
   TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ". Furthermore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"a", " ", "+", " ", "b"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "a", ")"}], " ", "+", " ", 
     RowBox[{"\[Florin]", "(", "b", ")"}]}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"a", "\[CenterDot]", "b"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "a", ")"}], "\[CenterDot]", 
     RowBox[{
      RowBox[{"\[Florin]", "(", "b", ")"}], "."}]}]}], TraditionalForm]]],
 " So we have that\n\n                                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "/", 
     RowBox[{"(", 
      RowBox[{"n", " ", "\[DoubleStruckCapitalZ]"}], ")"}]}], "  ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "n"]}], TraditionalForm]]],
 ".\n"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43730309871875*^9}],

Cell["\<\
Let us look at another example of a ring isomorphism. Consider the following \
ring of order 10. \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[10 a,0]
Define[a.a,2 a]
A = Ring[{a}] \
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", 
   RowBox[{"4", " ", "a"}], ",", 
   RowBox[{"5", " ", "a"}], ",", 
   RowBox[{"6", " ", "a"}], ",", 
   RowBox[{"7", " ", "a"}], ",", 
   RowBox[{"8", " ", "a"}], ",", 
   RowBox[{"9", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.437303125703125*^9, 3.437303275546875*^9, 
  3.44275290019575*^9}]
}, Open  ]],

Cell["Here are the addition and multiplication tables:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
AddTable[A]
MultTable[A]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437303131984375*^9, 3.43730313321875*^9}}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6,
       0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 
      0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 
      1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}, {{1, 1, 
      1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.437303135203125*^9, 3.442752902398875*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 
      1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {
      0, 1, 1}}, {{0.8, 0.6, 0.5}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 
      0.6, 0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 
      0.5}, {0, 1, 0}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 
      0.5}, {0, 1, 1}, {1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0,
       1, 1}, {1, 0.6, 0}}, {{1, 0.6, 0}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 1, 0}, {
      1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}}, {{0, 1, 0}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0,
       1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {
      0, 1, 1}}, {{1, 0, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 
      0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0,
       1, 0}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}}, {{1, 0, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {
      0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 0.6,
       0.5}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.437303135203125*^9, 3.442752902648875*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that the additive structure of this ring is like ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "10"], TraditionalForm]]],
 ", but the multiplicative structure is totally different, since there is no \
identity element."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["FindIdent[A]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.43730314240625*^9}],

Cell[BoxData["\<\"No identity element found.\"\>"], "Print",
 CellChangeTimes->{3.437303142921875*^9, 3.442752906992625*^9}]
}, Open  ]],

Cell["\<\
With a little thought one realizes that any ring of order 10 must have the \
same additive structure, since there is only one abelian group of order 10. \
Let us try to find another ring of order 10.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
Define[10 b,0]
Define[b.b, 6b]
B = Ring[{b}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "b", ",", 
   RowBox[{"2", " ", "b"}], ",", 
   RowBox[{"3", " ", "b"}], ",", 
   RowBox[{"4", " ", "b"}], ",", 
   RowBox[{"5", " ", "b"}], ",", 
   RowBox[{"6", " ", "b"}], ",", 
   RowBox[{"7", " ", "b"}], ",", 
   RowBox[{"8", " ", "b"}], ",", 
   RowBox[{"9", " ", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4373031655625*^9, 3.437303280390625*^9, 
  3.442752908555125*^9}]
}, Open  ]],

Cell[TextData[{
 "Here are the addition and multiplication tables for the ring ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
B1 = {0,b,2b,3b,4b,5b,6b,7b,8b,9b};
AddTable[B1]
MultTable[B1]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437303299828125*^9, 3.437303317203125*^9}}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6,
       0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 
      0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 
      1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}, {{1, 1, 
      1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"9b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{{3.437303304296875*^9, 3.4373033180625*^9}, 
   3.442752910352*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 
      1}, {1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}}, {{0.8, 0.6, 0.5}, {1, 1, 0}, {0.8, 0.6, 0.5}, {
      1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {
      1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0, 1, 
      1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 
      1, 0}, {1, 0.6, 0}, {0.8, 0.6, 0.5}}, {{1, 0.6, 0}, {1, 1, 0}, {
      1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0, 1, 0}, {1, 1, 0}, {
      1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0, 1, 0}}, {{0.6, 0.4, 0.8}, {
      1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{0, 1, 0}, {1, 1, 0}, {0, 1, 
      0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8,
       0.6, 0.5}, {0, 1, 1}, {1, 0.6, 0}}, {{1, 0, 0}, {1, 1, 0}, {0.8, 0.6, 
      0.5}, {1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {
      1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {0,
       1, 0}, {1, 0.6, 0}, {0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}}, {{1, 0, 1}, {1, 1, 0}, {1, 0.6, 0}, {0, 1,
       1}, {0.8, 0.6, 0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {
      0.8, 0.6, 0.5}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      0.5, 0.6, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{{3.437303304296875*^9, 3.4373033180625*^9}, 
   3.442752910602*^9}]
}, Open  ]],

Cell[TextData[{
 "If we compare the multiplication table to that of the ring ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[A]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.43730332678125*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 
      1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {
      0, 1, 1}}, {{0.8, 0.6, 0.5}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 
      0.6, 0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 
      0.5}, {0, 1, 0}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 
      0.5}, {0, 1, 1}, {1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0,
       1, 1}, {1, 0.6, 0}}, {{1, 0.6, 0}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 1, 0}, {
      1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}}, {{0, 1, 0}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0,
       1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {
      0, 1, 1}}, {{1, 0, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 
      0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0,
       1, 0}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}}, {{1, 0, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {
      0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 0.6,
       0.5}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.442752912586375*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "We find many simularities, but they are not exactly the same.\n\n\
EXPERIMENT:\nCan you find an isomorphism from ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " by rearranging the elements of ",
 StyleBox["B1",
  FontWeight->"Bold"],
 " above so that the multiplication tables for ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " have the same color pattern? Careful! The ",
 StyleBox["addition",
  FontSlant->"Italic"],
 " tables for ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " must ",
 StyleBox["also",
  FontSlant->"Italic"],
 " have the same color pattern! Rather than using trial and error, see if you \
use inductive reasoning using the properties of the ring. There is, in fact, \
only one solution."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[" ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "One way to help find an isomorphism between ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " is to show that both of these are isomorphic to a subring of the familiar \
rings ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". For example, consider ",
 Cell[BoxData[
  FormBox[Cell[TextData[{
    "2 ",
    Cell[BoxData[
     FormBox[
      SubscriptBox["Z", "20"], TraditionalForm]]]
   }]], TraditionalForm]]],
 ", the even elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "20"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437303422765625*^9, 3.437303436609375*^9}}],

Cell[CellGroupData[{

Cell["\<\
Define[20e,0];
Define[e.e,e];
R = Ring[{2 e}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "e"}], ",", 
   RowBox[{"4", " ", "e"}], ",", 
   RowBox[{"6", " ", "e"}], ",", 
   RowBox[{"8", " ", "e"}], ",", 
   RowBox[{"10", " ", "e"}], ",", 
   RowBox[{"12", " ", "e"}], ",", 
   RowBox[{"14", " ", "e"}], ",", 
   RowBox[{"16", " ", "e"}], ",", 
   RowBox[{"18", " ", "e"}]}], "}"}]], "Output",
 CellChangeTimes->{3.44275291525825*^9}]
}, Open  ]],

Cell[TextData[{
 "(The reason why we did not use the ",
 StyleBox["DefMod",
  FontWeight->"Bold"],
 " command to load ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "20"], TraditionalForm]]],
 " is because we would erase the rings ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ".)\n\nIf we compare the multiplication tables for ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[A]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.437303468328125*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 
      1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {
      0, 1, 1}}, {{0.8, 0.6, 0.5}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 
      0.6, 0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 
      0.5}, {0, 1, 0}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 
      0.5}, {0, 1, 1}, {1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0,
       1, 1}, {1, 0.6, 0}}, {{1, 0.6, 0}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 1, 0}, {
      1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}}, {{0, 1, 0}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0,
       1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {
      0, 1, 1}}, {{1, 0, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 
      0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0,
       1, 0}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}}, {{1, 0, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {
      0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 0.6,
       0.5}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.442752916773875*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MultTable[R]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.43730347*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 
      1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {
      0, 1, 1}}, {{0.8, 0.6, 0.5}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 
      0.6, 0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 
      0.5}, {0, 1, 0}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 
      0.5}, {0, 1, 1}, {1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0,
       1, 1}, {1, 0.6, 0}}, {{1, 0.6, 0}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 1, 0}, {
      1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}}, {{0, 1, 0}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0,
       1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {
      0, 1, 1}}, {{1, 0, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 
      0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0,
       1, 0}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}}, {{1, 0, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {
      0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 0.6,
       0.5}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"12e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"16e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"18e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"12e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"16e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"18e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.44275291813325*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
we see that they have the same color pattern. Furthermore, the addition \
tables\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["AddTable[A]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.442752920836375*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6,
       0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 
      0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 
      1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}, {{1, 1, 
      1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"9a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.442752921336375*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["AddTable[R]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.442752923242625*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}}, {{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 0.6, 0}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}}, {{0.6,
       0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 
      0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 
      1, 0}}, {{0, 1, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}}, {{1, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {
      0.5, 0.6, 0.8}, {1, 1, 0}, {1, 0, 1}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 
      1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}, {1, 1, 0}}, {{1, 1, 0}, {1, 1, 
      0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}, {{1, 1, 
      1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 0.8}}}],
    InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"12e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"16e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"18e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"12e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"16e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"18e\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"18e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"18e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"18e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"6e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"18e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"18e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"10e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"18e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"18e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"14e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"18e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"18e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"18e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"10e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"14e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16e\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.442752923727*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "are the same, showing that \n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->14], " ", 
    RowBox[{"2", " ", 
     SubscriptBox["Z", "20"]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\nEXPERIMENT:\nUsing a different letter for the identity element, define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "60"], TraditionalForm]]],
 ", and form the subring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"6", " ", 
    SubscriptBox["Z", "60"]}], TraditionalForm]]],
 ". Does this have the same color pattern in the multiplication table as the \
ring ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[BoxData[" "], "Input"],

Cell[TextData[{
 "These last two isomorphisms suggest a pattern that can be extended to any \
finite ring. \n\nPROPOSITION 10.7\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a finite ring whose additive structure is a cyclic group of order ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be a generator of the additive group. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "=", " ", 
    RowBox[{"k", "\[CenterDot]", "x"}]}], TraditionalForm]]],
 " for some positive integer ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[LessEqual]", "n"}], TraditionalForm]]],
 ", and \n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"k", " ", 
     RowBox[{
      SubscriptBox["Z", 
       RowBox[{"k", "\[CenterDot]", "n"}]], "."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437303489265625*^9, 3.437303510234375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "=", " ", "0"}], TraditionalForm]]],
 ", we can let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "=", " ", "n"}], TraditionalForm]]],
 ", so that ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " will be positive and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", "\[CenterDot]", "x"}], " ", "=", " ", 
    RowBox[{"0", " ", "=", " ", 
     SuperscriptBox["x", "2"]}]}], TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "2"], TraditionalForm]]],
 " is not zero, then since ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " generates the additive group, there is a ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], "=", " ", 
    RowBox[{"k", "\[CenterDot]", "x"}]}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", " ", "<", " ", "k", " ", "<", " ", 
    RowBox[{"n", "."}]}], TraditionalForm]]],
 "\n\nNow the natural mapping is one which sends ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"a", "\[CenterDot]", "x"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"k", "\[CenterDot]", "a"}], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"Mod", " ", "k", " ", "n"}], ")"}], "."}]}]}], 
   TraditionalForm]]],
 " This is obviously one-to-one and onto, since the value of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " ranges from 0 to",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"n", "-", "1"}]}], TraditionalForm]]],
 ". To check that this is an isomorphism, note that \n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", 
       RowBox[{
        RowBox[{"a", "\[CenterDot]", "x"}], " ", "+", " ", 
        RowBox[{"b", "\[CenterDot]", "x"}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"\[Florin]", "(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"a", "+", "b"}], ")"}], "\[CenterDot]", "x"}], ")"}], " ", 
       "=", " ", 
       RowBox[{
        RowBox[{"k", "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{"a", "+", "b"}], ")"}]}], " ", 
        RowBox[{"(", 
         RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}]}]}], 
     "                    ", "\n", "\t\t\t\t               ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"k", "\[CenterDot]", "a"}], " ", 
         RowBox[{"(", 
          RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{"k", "\[CenterDot]", "b"}], " ", 
         RowBox[{"(", 
          RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{"\[Florin]", "(", 
         RowBox[{"a", "\[CenterDot]", "x"}], ")"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"\[Florin]", "(", 
          RowBox[{"b", "\[CenterDot]", "x"}], ")"}], "."}]}]}]}]}]}], 
   TraditionalForm]]],
 "\nAlso,\n\n  ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", "\[CenterDot]", "x"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"b", "\[CenterDot]", "x"}], ")"}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"\[Florin]", "(", 
       RowBox[{"a", "\[CenterDot]", "b", "\[CenterDot]", 
        SuperscriptBox["x", "2"]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"\[Florin]", "(", 
        RowBox[{
        "a", "\[CenterDot]", "b", "\[CenterDot]", "k", "\[CenterDot]", "x"}], 
        ")"}], " ", "=", " ", 
       RowBox[{
        RowBox[{
        "k", "\[CenterDot]", "a", "\[CenterDot]", "b", "\[CenterDot]", "k"}], 
        " ", 
        RowBox[{"(", 
         RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}]}]}]}], 
    "                   "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t\t                ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", "\[CenterDot]", "a"}], " ", 
         RowBox[{"(", 
          RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}], ")"}], "\[CenterDot]",
        
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", "\[CenterDot]", "b"}], " ", 
         RowBox[{"(", 
          RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}], ")"}]}], " ", "=", 
      " ", 
      RowBox[{
       RowBox[{"\[Florin]", "(", 
        RowBox[{"a", "\[CenterDot]", "x"}], ")"}], "\[CenterDot]", 
       RowBox[{
        RowBox[{"\[Florin]", "(", 
         RowBox[{"b", "\[CenterDot]", "x"}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]}]],
 "\n\nTherefore, ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an isomorphism, and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->14], " ", 
    RowBox[{"k", " ", 
     SubscriptBox["Z", 
      RowBox[{"k", " ", "n"}]]}]}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437303621515625*^9, 3.4373036398125*^9}}]
}, Open  ]],

Cell[TextData[{
 "This proposition suggests that rings of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", 
    SubscriptBox["Z", 
     RowBox[{"k", " ", "n"}]]}], TraditionalForm]]],
 "  are important. Hence, we will refer to such rings by the following \
terminology.\n\nDEFINITION 10.6:\nA ",
 StyleBox["cyclic ring",
  FontSlant->"Italic"],
 " is a ring whose additive group is cyclic.\n\nNote that this definition of \
cyclic rings also includes the infinite rings ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " and its subrings ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[CenterDot]", "\[DoubleStruckCapitalZ]"}], 
   TraditionalForm]]],
 ".\n\nProposition 10.7 shows that for the two cyclic rings ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " that we worked with in the beginning of this section, \n                   \
 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"A", " ", 
     StyleBox["\[TildeTilde]",
      FontSize->16], " ", 
     RowBox[{"2", " ", 
      SubscriptBox["Z", "20"]}]}], " "}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"B", " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"6", " ", 
     SubscriptBox["Z", "60."]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437303646609375*^9, 3.437303669625*^9}}],

Cell[TextData[{
 "Our experiment showed that in fact ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", " ", 
    FormBox[
     SubscriptBox["Z", "20"],
     TraditionalForm]}], TraditionalForm]]],
 " must be isomorphic to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"6", " ", 
    FormBox[
     SubscriptBox["Z", "60"],
     TraditionalForm]}], TraditionalForm]]],
 ". But this isomorphism was not nearly as obvious. How would anyone guess \
that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "a", ")"}], TraditionalForm]]],
 " had to be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"7", " ", "b"}], TraditionalForm]]],
 "?\n\nIn order to prove that such isomorphisms exist in general, we will \
need a few lemmas about number theory. Once these are proven, we will be able \
to determine ",
 StyleBox["all ",
  FontSlant->"Italic"],
 "non-isomorphic rings of order 10."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "LEMMA 10.1\nLet ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " be a positive divisor of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " be the largest divisor of ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " which is coprime to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"n", "/", "d"}], ")"}], TraditionalForm]]],
 ". Then if ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " is coprime to both ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"n", "/", "d"}], ")"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373037101875*^9, 3.4373037106875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"GCD", "(", 
    RowBox[{"q", ",", "n"}], ")"}], TraditionalForm]]],
 " is not 1. Then there is a prime number ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " which divides neither ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"n", "/", "d"}], ")"}], TraditionalForm]]],
 ", yet divides ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ".  Thus, ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " must divide ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 ".\n\nNow ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[CenterDot]", "p"}], TraditionalForm]]],
 " will be coprine to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"n", "/", "d"}], ")"}], TraditionalForm]]],
 " since both ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " are. Also, since ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " is not a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " while ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "\[CenterDot]", "p"}], TraditionalForm]]],
 " will be a divisor of ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 ". But we defined ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " to be the ",
 StyleBox["largest",
  FontSlant->"Italic"],
 "  factor of ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " coprime to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"n", "/", "d"}], ")"}], TraditionalForm]]],
 ". This contradiction shows that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"GCD", "(", 
     RowBox[{"q", ",", "n"}], ")"}], " ", "=", " ", "1"}], 
   TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437303764890625*^9, 3.437303806515625*^9}}]
}, Open  ]],

Cell[TextData[{
 "With this lemma, we can strengthen the greatest common divisor theorem \
(1.2).\n\nLEMMA 10.2\nGiven two positive numbers ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", there exist ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " such that\n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"u", " ", "x"}], " ", "+", " ", 
      RowBox[{"v", " ", "y"}]}], " ", "=", " ", 
     RowBox[{"GCD", "(", 
      RowBox[{"x", ",", "y"}], ")"}]}], ","}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437303821203125*^9, 3.437303825703125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The greatest common divisor theorem (1.2) would give us values for ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 ", but there would be no way to guarantee that ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " would be coprime to ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ".\n\nLet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "=", " ", 
    RowBox[{"GCD", "(", 
     RowBox[{"x", ",", "y"}], ")"}]}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", "/", "k"}], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"y", "/", "k"}], ")"}], TraditionalForm]]],
 " are coprime, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", "/", "k"}], ")"}], TraditionalForm]]],
 " has an multiplicative inverse in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", 
    RowBox[{"(", 
     RowBox[{"y", "/", "k"}], ")"}]], TraditionalForm]]],
 ", say ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". That is,\n\n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["x", "k"], "\[CenterDot]", "n"}], " ", "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", 
       FractionBox["y", "k"]}], ")"}]}]}], TraditionalForm]]],
 ".\n                         \nLet ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " be the largest divisor of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " which is coprime to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"y", "/", "k"}], ")"}], TraditionalForm]]],
 ". By the Chinese remainder theorem (1.3), there is a number ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " such that\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "\[Congruent]", " ", 
    RowBox[{"n", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", 
       FractionBox["y", "k"]}], ")"}]}]}], TraditionalForm]]],
 "\nand\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", "  ", "f"}], ")"}]}]}], TraditionalForm]]],
 ".\n                     \nSince ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"y", "/", "k"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"y", "/", "k"}], ")"}], TraditionalForm]]],
 ". Also, ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 ", so by lemma 10.1  ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ". Also, \n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", " ", "\[CenterDot]", 
     FractionBox["x", "k"]}], "\[Congruent]", " ", 
    RowBox[{"1", "  ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", 
       FractionBox["y", "k"]}], ")"}]}]}], TraditionalForm]]],
 "\n                          \nso there is a ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " such that  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"u", "\[CenterDot]", 
      FractionBox["x", "k"]}], " ", "+", " ", 
     RowBox[{"v", "\[CenterDot]", 
      FractionBox["y", 
       RowBox[{"k", " "}]]}]}], " ", "=", " ", "1."}], TraditionalForm]]],
 "\n\nMultiplying both sides by ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " gives us\n\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"u", "\[CenterDot]", "x"}], " ", "+", " ", 
     RowBox[{"v", "\[CenterDot]", "y"}]}], " ", "=", " ", 
    RowBox[{"k", " ", "=", " ", 
     RowBox[{
      RowBox[{"GCD", "(", 
       RowBox[{"x", ",", "y"}], ")"}], "."}]}]}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437303905375*^9, 3.43730394771875*^9}}]
}, Open  ]],

Cell[TextData[{
 "With lemma 10.2, we can now prove determine which cyclic rings of the form \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", 
    SubscriptBox["Z", 
     RowBox[{"x", "\[CenterDot]", "n"}]]}], TraditionalForm]]],
 " are isomorphic."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43730398384375*^9, 3.43730398434375*^9}}],

Cell[TextData[{
 "THEOREM 10.1:  THE CYCLIC RING THEOREM\nIf ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are positive integers, then\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", 
     SubscriptBox["Z", 
      RowBox[{"x", "\[CenterDot]", "n"}]]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"k", " ", 
     SubscriptBox["Z", 
      RowBox[{"k", "\[CenterDot]", "n"}]]}]}], TraditionalForm]]],
 "\n             \nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "=", " ", 
    RowBox[{
     RowBox[{"GCD", "(", 
      RowBox[{"x", ",", "n"}], ")"}], "."}]}], TraditionalForm]]],
 "        "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437303988875*^9, 3.437303989546875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", " ", "=", " ", 
     RowBox[{"GCD", "(", 
      RowBox[{"x", ",", "n"}], ")"}]}], ","}], TraditionalForm]]],
 " by lemma 10.2 we can find integers ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " such that  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"u", "\[CenterDot]", "x"}], " ", "+", " ", 
     RowBox[{"v", "\[CenterDot]", "n"}]}], " ", "=", " ", "k"}], 
   TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". We now define a mapping ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", 
    SubscriptBox["Z", 
     RowBox[{"k", " ", "n"}]]}], TraditionalForm]]],
 "  to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", 
    SubscriptBox["Z", 
     RowBox[{"x", " ", "n"}]]}], TraditionalForm]]],
 " as follows:\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{
       RowBox[{"k", "\[CenterDot]", "w"}], " ", 
       RowBox[{"(", 
        RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"u", "\[CenterDot]", "x", "\[CenterDot]", "w"}], " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"Mod", " ", "x", " ", "n"}], ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\n\nNote that this is well defined, since if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[CenterDot]", "w"}], TraditionalForm]]],
 " is equivalent to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"k", "\[CenterDot]", "p"}], " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", 
       RowBox[{"k", "\[CenterDot]", "n"}]}], ")"}]}], ","}], 
   TraditionalForm]]],
 " then \n\n  ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{"w", " ", "\[Congruent]", " ", 
     RowBox[{"p", "  ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"Mod", " ", "n"}], ")"}], "  ", "\[DoubleLongRightArrow]", 
        " ", "x"}], "\[CenterDot]", "w"}]}], "    ", "\[Congruent]", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "p"}], "  ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", "x", " ", "n"}], ")"}]}]}], 
    "                                  "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t                      ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[DoubleLongRightArrow]", " ", "u"}], "\[CenterDot]", "x", 
      "\[CenterDot]", "w"}], " ", "\[Congruent]", "    ", 
     RowBox[{
      RowBox[{"u", "\[CenterDot]", "x", "\[CenterDot]", "p"}], " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"Mod", " ", 
         RowBox[{"x", "\[CenterDot]", "n"}]}], ")"}], "."}], " "}]}]}], 
   TraditionalForm]}]],
 "\nNext we need to show that \[Florin] is a homomorphism from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", 
    SubscriptBox["Z", 
     RowBox[{"k", " ", "n"}]]}], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", 
    SubscriptBox["Z", 
     RowBox[{"x", " ", "n"}]]}], TraditionalForm]]],
 ".  If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", 
    RowBox[{
     RowBox[{"k", "\[CenterDot]", "w"}], " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", 
    RowBox[{
     RowBox[{"k", "\[CenterDot]", "z"}], " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 ", then \n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", 
     RowBox[{"a", "+", "b"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{
       RowBox[{"k", "\[CenterDot]", "w"}], " ", "+", " ", 
       RowBox[{
        RowBox[{"k", "\[CenterDot]", "z"}], " ", 
        RowBox[{"(", 
         RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}]}], ")"}], " ", "=", " ",
      
     RowBox[{
      RowBox[{
       RowBox[{"u", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"x", "\[CenterDot]", "w"}], " ", "+", " ", 
          RowBox[{"x", "\[CenterDot]", "z"}]}], ")"}]}], " ", 
       RowBox[{"(", 
        RowBox[{"Mod", " ", "x", " ", "n"}], ")"}]}], "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"u", "\[CenterDot]", "x", "\[CenterDot]", "w"}], " ", "+", 
        " ", 
        RowBox[{
         RowBox[{"u", "\[CenterDot]", "x", "\[CenterDot]", "z"}], " ", 
         RowBox[{"(", 
          RowBox[{"Mod", " ", "x", " ", "n"}], ")"}]}]}], " ", "=", 
       RowBox[{
        RowBox[{"f", "(", "a", ")"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"f", "(", "b", ")"}], ".", " "}]}]}]}]}]}], 
   TraditionalForm]]],
 "\n\n",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"a", "\[InvisibleComma]", 
       RowBox[{"\[CenterDot]", "b"}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"f", "(", 
       RowBox[{
        RowBox[{
        "k", "\[CenterDot]", "w", "\[CenterDot]", "k", "\[CenterDot]", "z"}], 
        "  ", 
        RowBox[{"(", 
         RowBox[{"Mod", " ", "k", " ", "n"}], ")"}]}], " ", ")"}], "  ", "=", 
      " ", 
      RowBox[{
       RowBox[{
       "u", "\[CenterDot]", "x", "\[CenterDot]", "w", "\[CenterDot]", "k", 
        "\[CenterDot]", "z"}], " ", 
       RowBox[{"(", 
        RowBox[{"Mod", " ", "x", " ", "n"}], ")"}]}]}]}], " "}], 
   TraditionalForm], "\[IndentingNewLine]", 
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"             ", 
      RowBox[{"=", " ", 
       RowBox[{
       "u", "\[CenterDot]", "x", "\[CenterDot]", "w", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"u", "\[CenterDot]", "x"}], " ", "+", " ", 
          RowBox[{"v", "\[CenterDot]", "n"}]}], ")"}]}]}]}],
     TraditionalForm], "\[InvisibleComma]", 
    RowBox[{
     RowBox[{"\[CenterDot]", "z"}], " ", 
     RowBox[{"(", 
      RowBox[{"Mod", " ", "x", " ", "n"}], ")"}]}], " "}], 
   TraditionalForm], "\[IndentingNewLine]", 
  FormBox[
   RowBox[{"             ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
        "u", "\[CenterDot]", "x", "\[CenterDot]", "w", "\[CenterDot]", "u", 
         "\[CenterDot]", "x", "\[CenterDot]", "z"}], " ", "+", " ", 
        RowBox[{
        "u", "\[CenterDot]", "x", "\[CenterDot]", "w", "\[CenterDot]", "v", 
         "\[CenterDot]", "n", "\[CenterDot]", "z"}]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", "x", " ", "n"}], ")"}]}]}], " "}], 
   TraditionalForm], "\[IndentingNewLine]", 
  FormBox[
   RowBox[{"             ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"u", "\[CenterDot]", "x", "\[CenterDot]", "w"}], ")"}], 
        "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"u", "\[CenterDot]", "x", "\[CenterDot]", "z"}], ")"}]}], 
       " ", 
       RowBox[{"(", 
        RowBox[{"Mod", " ", 
         RowBox[{"x", "\[CenterDot]", "n"}]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"f", "(", "a", ")"}], " ", "\[CenterDot]", 
       RowBox[{
        RowBox[{"f", "(", "b", ")"}], "."}]}]}]}]}], TraditionalForm]}]],
 "\nSo \[Florin] is indeed a homomorphism from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", 
    SubscriptBox["Z", 
     RowBox[{"k", " ", "n"}]]}], TraditionalForm]]],
 " to  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", 
    SubscriptBox["Z", 
     RowBox[{"x", " ", "n"}]]}], TraditionalForm]]],
 ".\n   Since ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is coprime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " has an inverse, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["u", 
     RowBox[{"-", "1"}]], "(", 
    RowBox[{"Mod", " ", "n"}], ")"}], TraditionalForm]]],
 ". Then we see that ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is onto, since any element ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "a"}], " ", 
    RowBox[{"(", 
     RowBox[{"Mod", " ", 
      RowBox[{"x", "\[CenterDot]", "n"}]}], ")"}], " ", "in", " ", "x", " ", 
    SubscriptBox["Z", 
     RowBox[{"x", "\[CenterDot]", "n"}]]}], TraditionalForm]]],
 " can be obtained by taking\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{
      RowBox[{"k", "\[CenterDot]", "a", "\[CenterDot]", 
       SuperscriptBox["u", 
        RowBox[{"-", "1"}]]}], " ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", 
        RowBox[{"k", "\[CenterDot]", "n"}]}], ")"}]}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"u", "\[CenterDot]", "x", "\[CenterDot]", "a", "\[CenterDot]", 
       SuperscriptBox["u", 
        RowBox[{"-", "1"}]]}], " ", 
      RowBox[{"(", 
       RowBox[{"Mod", " ", 
        RowBox[{"x", "\[CenterDot]", "n"}]}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "a"}], " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"Mod", " ", 
         RowBox[{"x", "\[CenterDot]", "n"}]}], ")"}], "."}]}]}], " "}], 
   TraditionalForm]]],
 "\n\nFinally, both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", 
    SubscriptBox["Z", 
     RowBox[{"x", "\[CenterDot]", "n"}]]}], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", 
    SubscriptBox["Z", 
     RowBox[{"k", "\[CenterDot]", "n"}]]}], TraditionalForm]]],
 " contain ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " elements, so by the pigeon hole principle ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " must be a one-to-one function. Thus, ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an isomorphism, and\n\n                                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", 
     SubscriptBox["Z", 
      RowBox[{"x", "\[CenterDot]", "n"}]]}], "  ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    RowBox[{"k", " ", 
     RowBox[{
      SubscriptBox["Z", 
       RowBox[{"k", "\[CenterDot]", "n"}]], " ", "."}]}]}], 
   TraditionalForm]]],
 " "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437304008109375*^9, 3.43730400859375*^9}, {
  3.4373040729375*^9, 3.43730411175*^9}, {3.4417230198858747`*^9, 
  3.4417232087452497`*^9}, {3.4417233958389997`*^9, 3.4417234271514997`*^9}}]
}, Open  ]],

Cell[TextData[{
 "We can now understand why ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "\[CenterDot]", 
    SubscriptBox["Z", "20"]}], TraditionalForm]]],
 " was isomorphic to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"6", "\[CenterDot]", 
    SubscriptBox["Z", "60"]}], TraditionalForm]]],
 ". We merely have to notice that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{"2", " ", "=", " ", 
     RowBox[{"GCD", "(", 
      RowBox[{"10", ",", "6"}], ")"}]}]}], TraditionalForm]]],
 ". Since the only rings of order 10 are cyclic rings, there are in fact four \
possibilities for a ring of order 10:\n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "10"], ",", "   ", 
    RowBox[{"2", "\[CenterDot]", 
     SubscriptBox["Z", "20"]}], ",", "   ", 
    RowBox[{"5", "\[CenterDot]", 
     SubscriptBox["Z", "50"]}], ",", "   ", 
    RowBox[{"and", "   ", 
     RowBox[{"10", "\[CenterDot]", 
      RowBox[{
       SubscriptBox["Z", "100"], "."}]}]}]}], TraditionalForm]]],
 "\n\nTo see if these four rings are distinct, we can look at the \
multiplication tables."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
DefMod[10]
MultTable[Ring[{1}]]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {1, 1, 0}, {0.5, 0.6, 0.8}, {0.8, 0.6, 0.5}, {
      0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 0}, {0, 
      1, 1}, {1, 0, 1}}, {{0.8, 0.6, 0.5}, {1, 1, 0}, {0.8, 0.6, 0.5}, {
      1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {
      1, 0.6, 0}, {0, 1, 0}, {0, 1, 1}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0.7, 
      0.8, 0.7}, {0, 1, 0}, {1, 0, 1}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {0, 
      1, 1}, {0.5, 0.6, 0.8}, {1, 0.6, 0}, {1, 0, 0}}, {{1, 0.6, 0}, {1, 1, 
      0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0, 1, 0}, {1, 1, 0}, {
      1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0, 1, 0}}, {{0.6, 0.4, 0.8}, {
      1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}}, {{0,
       1, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {1, 0.6, 0}, {
      1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {1, 0.6, 0}}, {{1, 0, 
      0}, {1, 1, 0}, {1, 0, 0}, {1, 0.6, 0}, {0.5, 0.6, 0.8}, {0, 1, 1}, {0.6,
       0.4, 0.8}, {0.8, 0.6, 0.5}, {1, 0, 1}, {0, 1, 0}, {0.7, 0.8, 0.7}}, {{
      0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 0.6, 
      0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 0.6, 0.5}}, {{
      1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 0.6, 
      0.8}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"7\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"9\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"9\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"7\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"6\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.442752934273875*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
DefMod[20]
MultTable[Ring[{2}]]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 
      1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {
      0, 1, 1}}, {{0.8, 0.6, 0.5}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 
      0.6, 0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 
      0.5}, {0, 1, 0}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 
      0.5}, {0, 1, 1}, {1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0,
       1, 1}, {1, 0.6, 0}}, {{1, 0.6, 0}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 1, 0}, {
      1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}}, {{0, 1, 0}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0,
       1, 0}, {0, 1, 1}, {1, 1, 0}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0, 1, 0}, {
      0, 1, 1}}, {{1, 0, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 
      0.5}, {0, 1, 0}, {1, 1, 0}, {1, 0.6, 0}, {0, 1, 1}, {0.8, 0.6, 0.5}, {0,
       1, 0}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}, {1, 1, 0}, {0, 1, 0}, {0.8, 0.6, 0.5}, {0, 1, 1}, {
      1, 0.6, 0}}, {{1, 0, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {
      0.8, 0.6, 0.5}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {1, 0.6, 0}, {0.8, 0.6,
       0.5}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"12\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"16\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"18\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"6\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"8\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"12\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"14\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"16\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"18\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"16\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"12\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"8\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.44275293569575*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
DefMod[50]
MultTable[Ring[{5}]]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 
      0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 
      1, 0}, {0.6, 0.4, 0.8}}, {{0.8, 0.6, 0.5}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {
      1, 1, 0}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}}, {{1, 0.6, 0}, {1, 1, 0}, {1, 1, 0}, {
      1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}}, {{0, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 
     0}}, {{1, 0, 0}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 
      0}, {0.6, 0.4, 0.8}}, {{0, 1, 1}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 
     1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 0, 1}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 
      0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {1, 1, 
      0}, {0.6, 0.4, 0.8}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 
     1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"15\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"20\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"25\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"30\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"35\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"40\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"45\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"15\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"20\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"25\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"30\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"35\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"40\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"45\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"25\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.442752937211375*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
DefMod[100]
MultTable[Ring[{10}]]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4427529386645*^9, 3.44275294007075*^9}}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.5, 0.6, 0.8}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {
      1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{0.8,
       0.6, 0.5}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 
      1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{0.7, 0.8, 0.7}, {
      1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{1, 0.6, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {
      1, 1, 0}, {1, 1, 0}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
      0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {
      1, 1, 0}}, {{0, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 
     1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{1, 0, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{0, 1, 1}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 0}}, {{1, 0, 1}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}}, {{
     1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0.5, 
      0.6, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 10.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"20\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"30\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"40\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"50\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"60\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"70\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"80\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 10.5}], InsetBox[
     TagBox[
      StyleBox["\<\"90\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {10.5, 10.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"10\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"20\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"30\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"40\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"50\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"60\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"70\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"80\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"90\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 9.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {10.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 10}, {11, 10}}]}]], "Output",
 CellChangeTimes->{3.44275294207075*^9}]
}, Open  ]],

Cell["\<\
The last ring has the property that any product of elements gives 0. Notice \
that there are exactly as many rings of order 10 as there are divisors of 10. \
We can generalize this result to give the following corollary.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "COROLLARY 10.1\nThe number of non-isomorphic cyclic rings of order ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is precisly the number of divisors of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " (including 1 and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ")."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437304192515625*^9, 3.437304193078125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "By proposition 10.7 every cyclic ring of order ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is isomorphic to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", 
    SubscriptBox["Z", 
     RowBox[{"k", "\[CenterDot]", "n"}]]}], TraditionalForm]]],
 " for some value of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ". By the cyclic ring theorem, we see that this is isomorphic to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", " ", 
    SubscriptBox["Z", 
     RowBox[{"d", "\[CenterDot]", "n"}]]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", " ", "=", " ", 
    RowBox[{
     RowBox[{"GCD", "(", 
      RowBox[{"k", ",", "n"}], ")"}], "."}]}], TraditionalForm]]],
 " Hence ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " is a divisor of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". We need to show that two different rings of this form are non-isomorphic. \
Consider the rings ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", " ", "=", " ", 
    RowBox[{"d", " ", 
     SubscriptBox["Z", 
      RowBox[{"d", "\[CenterDot]", "n"}]]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"B", " ", "=", " ", 
    RowBox[{"f", " ", 
     SubscriptBox["Z", 
      RowBox[{" ", 
       RowBox[{"f", "\[CenterDot]", "n"}]}]]}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " are different divisors of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Perhaps the easiest way to show that these are different is to count the \
number of elements in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " which can appear in the multiplication tables. The elements that can \
appear in the table for ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " are\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["d", "2"], ",", " ", 
    RowBox[{"2", " ", 
     SuperscriptBox["d", "2"]}], ",", " ", 
    RowBox[{"3", " ", 
     SuperscriptBox["d", "2"]}], ",", " ", 
    RowBox[{
     RowBox[{"\[Ellipsis]", " ", 
      RowBox[{"n", "\[CenterDot]", "d"}]}], " ", "=", " ", "0"}], ","}], 
   TraditionalForm]]],
 "\n\nwhile the elements appearing in the multiplication table of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " are\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["f", "2"], ",", " ", 
    RowBox[{"2", " ", 
     SuperscriptBox["f", "2"]}], ",", " ", 
    RowBox[{"3", " ", 
     SuperscriptBox["f", "2"]}], ",", " ", 
    RowBox[{
     RowBox[{"\[Ellipsis]", " ", 
      RowBox[{"n", "\[CenterDot]", "f"}]}], " ", "=", " ", "0"}], ","}], 
   TraditionalForm]]],
 "\n \nThus, there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "/", "d"}], TraditionalForm]]],
 " such elements of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "/", "f"}], TraditionalForm]]],
 " elements of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " are different, we see that the rings ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " are not isomorphic. Therefore, there is a one-to-one correspondance \
between the factors of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and the cyclic rings of order ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437304199265625*^9, 3.437304199859375*^9}}]
}, Open  ]],

Cell[TextData[{
 "Although this corollary seems to be a big help in finding ",
 StyleBox["all",
  FontSlant->"Italic"],
 " finite rings, there are, in fact, many non-cyclic rings. For example, \
there are 8 non-cyclic rings of order 4, which when combined with the 3 \
cyclic rings from corollary 10.1 gives a total of 11 rings of order 4. There \
are 52 rings of order 8 (4 cyclic, 20 with additive group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "15", "*"], TraditionalForm]]],
 ", and 28 with an additive group ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "24", "*"], TraditionalForm]]],
 ".)\n\nAs one can see, finding all rings of a given order is a much more \
difficult task than finding the groups of the same order. In fact, it is \
still unknown how many non-isomorphic rings there are of order 16. Here is a \
table of the number of rings of a given order:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43730429015625*^9, 3.43730429134375*^9}}],

Cell["\<\

   n | Rings of order n        n | Rings of order n
   --|------------------       --|------------------
   1 |   1                    17 |   2
   2 |   2                    18 |  22
   3 |   2                    19 |   2
   4 |  11                    20 |  22
   5 |   2                    21 |   4
   6 |   4                    22 |   4
   7 |   2                    23 |   2
   8 |  52                    24 | 104
   9 |  11                    25 |  11
  10 |   4                    26 |   4
  11 |   2                    27 |  59
  12 |  22                    28 |  22
  13 |   2                    29 |   2
  14 |   4                    30 |   8
  15 |   4                    31 |   2
  16 | 390                    32 | ???
\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Courier New",
 FontSize->10],

Cell["\<\
There are 18590 known rings of order 32, but it has not been proven this is \
all of them.  There are some patterns in this table. Can you find any?   \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437304321484375*^9, 3.43730433465625*^9}, {
  3.4417237732608747`*^9, 3.4417237867296247`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Homomorphisms and Kernels", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In the last section we determined two rings were isomorphic by finding a \
one-to-one and onto function mapping one ring to the other. If we considered \
a similar function which was not nessesarily one-to-one or onto, we would \
have a ",
 StyleBox["ring homomorphism",
  FontSlant->"Italic"],
 ".\n\nDEFINITION 10.7:\nIf ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " are two rings, then a mapping  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", ":", 
    RowBox[{"A", " ", "\[RightArrow]", " ", "B"}]}], TraditionalForm]]],
 " such that \n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", 
       RowBox[{"x", " ", "+", " ", "y"}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"\[Florin]", "(", "x", ")"}], " ", "+", " ", 
       RowBox[{"\[Florin]", "(", "y", ")"}]}]}], ","}], " "}], 
   TraditionalForm]]],
 "\nand \n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"x", "\[CenterDot]", "y"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "x", ")"}], " ", "\[CenterDot]", " ", 
     RowBox[{"\[Florin]", "(", "y", ")"}]}]}], TraditionalForm]]],
 "\n\nfor all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " is called a ",
 StyleBox["ring homomorphism",
  FontSlant->"Italic"],
 ". \n\nNotice that a ring homomorphism preserves both of the ring \
operations. In particular, a ring homomorphism will also be a group \
homomorphism from the additive group of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to the additive group of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". Thus, we can immediately apply the results of group homomorphisms to see \
two properties of ring homomorphisms:\n\nIf ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is a homomorphism from ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", then \n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "0", ")"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 "\nand \n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{"-", "x"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"\[Florin]", "(", "x", ")"}]}], "    ", "for", " ", "all", "    ",
        "x"}], " ", "\[Element]", " ", "A"}]}]}], TraditionalForm]]],
 ".\n\nWe have seen some examples of ring homomorphisms in the last section, \
since any isomorphism is certainly a homomorphism. But let us see if we can \
define a homomorphism between two non-isomorphic rings. Consider for example \
a homomorphism between ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 ".\n\nFirst we must define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " simultaneously. Because the ",
 StyleBox["DefMod",
  FontWeight->"Bold"],
 " command can only define one ring at a time, we must use the following \
commands to define these rings."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437323350640625*^9, 3.43732335275*^9}, {
  3.437323411359375*^9, 3.437323415359375*^9}}],

Cell["\<\
InitRing;
Define[3a,0]; Define[a.a,a]
Define[6b,0]; Define[b.b,b]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437323433078125*^9, 3.437323437984375*^9}}],

Cell[TextData[{
 "Now the ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " is given by"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
Z3 = Ring[{a}]
MultTable[Z3]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.437323449921875*^9, 3.442752987602*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 0, 1}}, {{1, 0, 1}, {1, 1,
     0}, {1, 0, 1}, {0, 1, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
    0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 3}, {4, 3}}]}]], "Output",
 CellChangeTimes->{3.437323449921875*^9, 3.442752987648875*^9}]
}, Open  ]],

Cell[TextData[{
 "while the ring ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " is given by"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
Z6 = Ring[{b}]
MultTable[Z6]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "b", ",", 
   RowBox[{"2", " ", "b"}], ",", 
   RowBox[{"3", " ", "b"}], ",", 
   RowBox[{"4", " ", "b"}], ",", 
   RowBox[{"5", " ", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.437323452390625*^9, 3.4427529894145*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0, 1, 0}, {1, 0, 
      0}, {0, 1, 1}, {1, 0, 1}}, {{0, 1, 0}, {1, 1, 0}, {0, 1, 0}, {0, 1, 
     1}, {1, 1, 0}, {0, 1, 0}, {0, 1, 1}}, {{1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {
     1, 1, 0}, {1, 0, 0}, {1, 1, 0}, {1, 0, 0}}, {{0, 1, 1}, {1, 1, 0}, {0, 1,
      1}, {0, 1, 0}, {1, 1, 0}, {0, 1, 1}, {0, 1, 
     0}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}}, {{1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"4b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"5b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"5b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"4b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 6}, {7, 6}}]}]], "Output",
 CellChangeTimes->{3.437323452390625*^9, 3.44275298950825*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that we used different generators for the two rings so we could define \
both rings at the same time. \n\nNow suppose we wish to define a homomorphism \
",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 ". We already see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "0", ")"}], TraditionalForm]]],
 " must be 0, and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"2", "a"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{"-", "a"}], ")"}], " ", "=", " ", 
     RowBox[{"-", 
      RowBox[{"\[Florin]", "(", "a", ")"}]}]}]}], TraditionalForm]]],
 ". Thus, the homomorphism is determined completely by the value of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "a", ")"}], "."}], TraditionalForm]]],
 " A natural choice would be to let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "a", ")"}], " ", "=", " ", 
    RowBox[{"2", "b"}]}], TraditionalForm]]],
 ". We can define a ring homomorphism the same way as we did for group \
homomorphisms, so the command ",
 StyleBox["Homomorph[F] ",
  FontWeight->"Bold"],
 "defines ",
 StyleBox["F",
  FontWeight->"Bold"],
 " to be a ring homomorphism. We can try to define a homomorphism from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " with the commands:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Homomorph[F]
Define[F[a],2b]\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We can now use the command ",
 StyleBox["CheckHomo[F,R]",
  FontWeight->"Bold"],
 " to check to see if the function ",
 StyleBox["F",
  FontWeight->"Bold"],
 ", which is defined on the ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", is a homomorphism. If we check the function we just defined,"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[F,Z3]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"f[\"\>", "\[InvisibleSpace]", 
   RowBox[{"2", " ", "a"}], "\[InvisibleSpace]", "\<\"] . f[\"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"2", " ", "a"}], 
   "\[InvisibleSpace]", "\<\"] is not equal to f[\"\>", "\[InvisibleSpace]", 
   "a", "\[InvisibleSpace]", "\<\"]\"\>"}],
  SequenceForm[
  "f[", 2 $CellContext`a, "] . f[", 2 $CellContext`a, 
   "] is not equal to f[", $CellContext`a, "]"],
  Editable->False]], "Print",
 CellChangeTimes->{3.43732347478125*^9, 3.442752992930125*^9}],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.4373234748125*^9, 3.442752992930125*^9}]
}, Open  ]],

Cell[TextData[{
 "we see that this is not a homomorphism!\n\nEXPERIMENT:\nReplace the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "b"}], TraditionalForm]]],
 " in the command defining ",
 StyleBox["F",
  FontWeight->"Bold"],
 " with other elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 ", and check to see which are homomorphisms. How many possible homomorphisms \
are there from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["     ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "\nEXPERIMENT: \nDesign another homomorphism ",
 StyleBox["G",
  FontWeight->"Bold"],
 " which maps elements from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 ". Notice that all that must be defined in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is the element ",
 StyleBox["G[b]",
  FontWeight->"Bold"],
 ". Then use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to check if this really is a homomorphism."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["     ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "In both of these experiments one may have noticed that one possible mapping \
would be to send all elements to 0. This in fact will always yield a \
homomorphism, and we will give this a special name.\n\nDEFINITION 10.8\nIf ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " are any two rings, then the mapping \n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", ":", " ", 
    RowBox[{"A", " ", "\[RightArrow]", " ", "B"}]}], TraditionalForm]]],
 "\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "x", ")"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 "  for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "\[Element]", " ", "A"}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437323519765625*^9, 3.437323528609375*^9}}],

Cell[TextData[{
 "is called the ",
 StyleBox["zero homomorphism",
  FontSlant->"Italic"],
 " from  ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ".\n\nAlthough the zero homomorphism works for both of the above \
experiments, there are also non-zero homomorphisms which map from ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 ". If you haven't found these already, go back and try to find them. \n\n\
Once we have the homomorphism ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " defined, we can define ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "S", ")"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a ",
 StyleBox["set",
  FontSlant->"Italic"],
 " of elements in the domain of ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ", to be the set of all values ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "x", ")"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". We can alse define the inverse image of an element ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " to be ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "y", ")"}], TraditionalForm]]],
 ", the set of elements such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "x", ")"}], " ", "=", " ", "y"}], 
   TraditionalForm]]],
 ". In fact, we can define the inverse image of a set of elements in the same \
way: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "T", ")"}], TraditionalForm]]],
 " is the set of elements such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "x", ")"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 ". We can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the image of a set by merely entering ",
 StyleBox["F[S]",
  FontWeight->"Bold"],
 ". (Unlike the group theory notebooks, we do not have to enclose the set in \
an extra pair of curly braces.) We can find the inverse image of an element \
or a set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " with the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command ",
 StyleBox["HomoInverse[\[Florin], S, A]",
  FontWeight->"Bold"],
 " where ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " is the domain of ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ".\n\nFor example, here are the non-zero homomorphisms from the previous two \
experiments."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417238133702497`*^9, 3.4417238353077497`*^9}}],

Cell[CellGroupData[{

Cell["\<\
Homomorph[F]
Define[F[a],4b]
CheckHomo[F,Z3]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.442752998680125*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Homomorph[G]
Define[G[b],a]
CheckHomo[G,Z6]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.442753000086375*^9}]
}, Open  ]],

Cell[TextData[{
 "To compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", 
    RowBox[{"{", 
     RowBox[{"0", ",", "a"}], "}"}], ")"}], TraditionalForm]]],
 ", we enter the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["F[{0,a}]  ", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"4", " ", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4427530019145*^9}]
}, Open  ]],

Cell[TextData[{
 "We can also find the inverse image of the element a under ",
 StyleBox["G",
  FontWeight->"Bold"],
 " by the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["HomoInverse[G,a,Z6]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"b", ",", 
   RowBox[{"4", " ", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442753003352*^9}]
}, Open  ]],

Cell[TextData[{
 "Likewise, we can take the inverse image of the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "a"}], "}"}], TraditionalForm]]],
 " with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["HomoInverse[G,{0,a},Z6]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "b", ",", 
   RowBox[{"3", " ", "b"}], ",", 
   RowBox[{"4", " ", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.44275300475825*^9}]
}, Open  ]],

Cell[TextData[{
 "\nEXPERIMENT:\nWhat is the difference between ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["G", 
     RowBox[{"-", "1"}]], "[", "x", "]"}], TraditionalForm]]],
 "? Does ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "[", 
     RowBox[{"G", "[", "x", "]"}], "]"}], " "}], TraditionalForm]]],
 " always produce ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "? What about ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "[", 
    RowBox[{"F", "[", "x", "]"}], "]"}], TraditionalForm]]],
 "?  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We can ask whether the image or inverse image of a subring will again be a \
subring. This is actually very easy to prove, as seen in the next \
proposition.\n\nPROPOSITION 10.8\nSuppose \[Florin] is a homomorphism from \
the ring ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to the ring ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". Then if ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "S", ")"}], TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". Likewise, if ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "T", ")"}], TraditionalForm]]],
 " will be a subring of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43732366171875*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". We will use proposition 10.1 to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "S", ")"}], TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". The element ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "0", ")"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "S", ")"}], TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "S", ")"}], TraditionalForm]]],
 " is non-empty. If ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " are two elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "S", ")"}], TraditionalForm]]],
 ", then there exist elements ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " such that \n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "x", ")"}], " ", "=", " ", "u"}], 
   TraditionalForm]]],
 " \nand \n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "y", ")"}], " ", "=", " ", "v"}]}], 
   TraditionalForm]]],
 ".\n\nBut ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "-", "y"}], TraditionalForm]]],
 " are will also be in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", and so \n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"x", "\[CenterDot]", "y"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{"\[Florin]", "(", "y", ")"}]}], " ", "=", " ", 
     RowBox[{"u", "\[CenterDot]", "v"}]}]}], TraditionalForm]]],
 "\nand \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"x", "-", "y"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", "x", ")"}], " ", "-", " ", 
      RowBox[{"\[Florin]", "(", "y", ")"}]}], " ", "=", " ", 
     RowBox[{"u", " ", "-", " ", "v"}]}]}], TraditionalForm]]],
 "\n\nmust be in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "S", ")"}], TraditionalForm]]],
 ". Thus, by proposition 10.1, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "S", ")"}], TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ".\n\nNow suppose that ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". Since 0 is contained in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "T", ")"}], TraditionalForm]]],
 ", we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "T", ")"}], TraditionalForm]]],
 " is non-empty. If ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are two elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "T", ")"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "y", ")"}], TraditionalForm]]],
 " will be two elements of ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 ". Thus,\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"x", "\[CenterDot]", "y"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"\[Florin]", "(", "y", ")"}]}]}], TraditionalForm]]],
 "\nand \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"x", "-", "y"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "x", ")"}], " ", "-", " ", 
     RowBox[{"\[Florin]", "(", "y", ")"}]}]}], TraditionalForm]]],
 "\n\nwould be elements of ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 ". Hence, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "-", "y"}], TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "T", ")"}], TraditionalForm]]],
 ". Thus, by proposition 10.1, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "T", ")"}], TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43732367778125*^9, 3.4373237258125*^9}}]
}, Open  ]],

Cell[TextData[{
 "We can define the kernel and the image of a homomorphism in the same way \
that we did for group homomorphisms.\n\nDEFINITION 10.9:\nGiven a \
homomorphism ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " from the ring ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to the ring ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", the ",
 StyleBox["kernel",
  FontSlant->"Italic"],
 " of \[Florin] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "0", ")"}], TraditionalForm]]],
 ", denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Ker", "(", "\[Florin]", ")"}], TraditionalForm]]],
 ". The",
 StyleBox[" image",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "A", ")"}], TraditionalForm]]],
 ", denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Im", "(", "\[Florin]", ")"}], TraditionalForm]]],
 ".\n\nWe can use the HomoInverse command to find the kernel of a \
homomorphism, or we can simply use the command ",
 StyleBox["Kernel[",
  FontWeight->"Bold"],
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 StyleBox[",A] ",
  FontWeight->"Bold"],
 "to find the kernel of the homomorphism whose domain is ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". For example, the kernels of the homomorphisms ",
 StyleBox["F",
  FontWeight->"Bold"],
 " and ",
 StyleBox["G",
  FontWeight->"Bold"],
 " that we have defined are "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43732373646875*^9, 3.43732374003125*^9}}],

Cell[CellGroupData[{

Cell["Kernel[F,Z3]  ", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", "0", "}"}]], "Output",
 CellChangeTimes->{3.442753010180125*^9}]
}, Open  ]],

Cell["and ", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Kernel[G,Z6]  ", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"3", " ", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442753011523875*^9}]
}, Open  ]],

Cell[TextData[{
 "The images are even easier to find using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["F[Z3]  ", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "b"}], ",", 
   RowBox[{"4", " ", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442753012852*^9}]
}, Open  ]],

Cell["and ", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G[Z6]  ", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442753013867625*^9}]
}, Open  ]],

Cell[TextData[{
 "When we have a homomorphism from ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " is a trivial subring of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", we have by proposition 10.8 that the image will be a subring of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". Likewise, the kernel of a homomorphism will be a subring of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "0", "}"}], TraditionalForm]]],
 " is a subring of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". However, we can say even more about the kernel.\n\nPROPOSITION 10.9\nIf \
",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is a homomorphism from the ring ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to the ring ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", then the kernel of ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an ideal of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ". Furthermore, ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is injective if and only if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Ker", "(", "\[Florin]", ")"}], " ", "=", " ", 
    RowBox[{"{", "0", "}"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373238065625*^9, 3.43732382209375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in the kernel of ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is any other element of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". Then \n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", 
       RowBox[{"x", "\[CenterDot]", "y"}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Florin]", "(", "x", ")"}], "\[CenterDot]", 
        RowBox[{"\[Florin]", "(", "y", ")"}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{"0", " ", "\[CenterDot]", " ", 
         RowBox[{"\[Florin]", "(", "y", ")"}]}], " ", "=", " ", "0"}]}]}], 
     ","}]}], TraditionalForm]]],
 "\nand \n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"y", "\[CenterDot]", "x"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", "y", ")"}], "\[CenterDot]", 
      RowBox[{"\[Florin]", "(", "x", ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Florin]", "(", "y", ")"}], " ", "\[CenterDot]", " ", "0"}], 
      " ", "=", " ", "0."}]}]}], TraditionalForm]]],
 "\n\nHence,  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "x"}], TraditionalForm]]],
 " are in the kernel of ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ", so the kernel is an ideal of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ".\n\nIf ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is injective, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "0", ")"}], TraditionalForm]]],
 " can only contain one element, which must be 0. On the other hand, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Florin]", 
      RowBox[{"-", "1"}]], "(", "0", ")"}], " ", "=", " ", 
    RowBox[{"{", "0", "}"}]}], TraditionalForm]]],
 ", then\n\n            ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"f", "(", "y", ")"}], "  ", "\[DoubleLongRightArrow]", "  ", 
       RowBox[{"\[Florin]", "(", "x", ")"}]}], " ", "-", " ", 
      RowBox[{"f", "(", "y", ")"}]}], " ", "=", " ", "0"}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                         ", 
    RowBox[{
     RowBox[{"\[DoubleLongRightArrow]", " ", 
      RowBox[{"\[Florin]", "(", 
       RowBox[{"x", " ", "-", " ", "y"}], ")"}]}], " ", "=", " ", "0"}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                         ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[DoubleLongRightArrow]", " ", "x"}], " ", "-", " ", "y"}], 
     " ", "=", " ", "0"}]}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                         ", 
    RowBox[{
     RowBox[{"\[DoubleLongRightArrow]", " ", "x"}], " ", "=", " ", 
     RowBox[{"y", "."}]}], "  "}], TraditionalForm]}]],
 "\n\nTherefore, ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is injective if and only if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Ker", "(", "\[Florin]", ")"}], " ", "=", " ", 
    RowBox[{"{", "0", "}"}]}], TraditionalForm]]],
 ".            "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell["\<\
Let us produce some more examples of homomorphisms. Let us look at the \
non-commutative ring of order 8 that we looked at earlier in this chapter.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing
Define[4 a, 0]; Define[2 b, 0]
Define[a.a, a]; Define[b.b, 0]
Define[a.b, b]; Define[b.a, 2 a]
R = Ring[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43732394090625*^9, 3.437323985875*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.437323986484375*^9, 3.442753020898875*^9}]
}, Open  ]],

Cell[TextData[{
 "If we wanted to define a homomorphism from ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " to some other ring ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", the kernel would have to be an ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". But we have already found that ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " has only three non-trivial ideals:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
I1 = {0,2a};
I2 = {0,2a+b};
I3 = {0,2a,b,2a+b};\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Thus, the kernel would have to be ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "0", "}"}], TraditionalForm]]],
 ", or one of these three ideals. If the kernel is all of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", then the homomorphism is the zero homomorphism. On the other extreme, if \
the kernel is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "0", "}"}], TraditionalForm]]],
 ", then the homomorphism is injective by proposition 10.4.2. To produce a \
more interesting homomorphism, we would have to use one of the three above \
ideals. The question is, to which ring should we map ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "? \n\nIt seems that a natural answer would be a quotient ring! Since there \
is a natural ",
 StyleBox["group",
  FontSlant->"Italic"],
 " homomorphism from ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "I"}], TraditionalForm]]],
 ", we can ask whether this group homomorphism extends to become a ring \
homomorphism.\n\nLet us try this out with the ideal ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "1"], TraditionalForm]]],
 ". The quotient ring is given by:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Q = Coset[R,I1]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"2", " ", "a"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", 
     RowBox[{"3", " ", "a"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"b", ",", 
     RowBox[{
      RowBox[{"2", " ", "a"}], "+", "b"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "+", "b"}], ",", 
     RowBox[{
      RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.44275302469575*^9}]
}, Open  ]],

Cell[TextData[{
 "We wish to define a homomorphism ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "(", "x", ")"}], TraditionalForm]]],
 " which maps an element in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " to the coset of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " containing that element. We only need to define ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "[", "a", "]"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "[", "b", "]"}], TraditionalForm]]],
 " to complete the definition."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Homomorph[i]
Define[i[a],{a,3a}]
Define[i[b],{b,2a+b}]\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We can see if this works with the ",
 StyleBox["CheckHomo",
  FontWeight->"Bold"],
 " command."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[i,R]", "Input",
 PageWidth->Infinity],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.442753028711375*^9}]
}, Open  ]],

Cell[TextData[{
 "So ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " defines a homomorphism from ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". The kernel of this homomorphism is given by:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Kernel[i,R]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "a"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442753030352*^9}]
}, Open  ]],

Cell[TextData[{
 "\nEXPERIMENT:\nTry forming similar homomorphisms using the ideals ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "2"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "3"], TraditionalForm]]],
 " instead. Use the ",
 StyleBox["CheckHomo",
  FontWeight->"Bold"],
 " command to see if they really are homomorphisms."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "In general, we can form a homomorphism from a ring ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " to a quotient ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "/", "I"}], TraditionalForm]]],
 " using the same technique. We will state this as a lemma:\n\nLEMMA 10.3\nIf \
",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " is an ideal of the ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", then the natural mapping \n\n                                ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", ":", " ", 
    RowBox[{"R", " ", "\[RightArrow]", " ", 
     RowBox[{"R", "/", "I"}]}]}], TraditionalForm]]],
 "\ndefined by  \n                              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"i", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"x", " ", "+", " ", 
     StyleBox["I",
      FontSlant->"Italic"]}]}], TraditionalForm]]],
 "\n                              \nis a surjective ring homomorphism from ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "I"}], TraditionalForm]]],
 " with the kernel being ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437324248125*^9, 3.4373242690625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "It is clear that the rule ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"i", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{"x", " ", "+", " ", "I"}]}]}], TraditionalForm]]],
 " defines a surjective mapping ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " from ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "I"}], TraditionalForm]]],
 ", and that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Ker", "(", "i", ")"}], " ", "=", " ", 
    RowBox[{"I", "."}]}], TraditionalForm]]],
 " We need only to check that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "(", "x", ")"}], TraditionalForm]]],
 " is a homomorphism.\n\nSince\n                          ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"i", "(", 
      RowBox[{"x", "+", "y"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", "+", "y"}], ")"}], " ", "+", " ", "I"}]}], 
    "                                   "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t    ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "+", " ", "I"}], ")"}], " ", "+", " ", 
      RowBox[{"(", 
       RowBox[{"y", " ", "+", " ", "I"}], ")"}]}]}], 
    "                                  "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t\t ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"i", "(", "x", ")"}], " ", "+", " ", 
      RowBox[{"i", "(", "y", ")"}]}]}]}], TraditionalForm]}]],
 "\nand \n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"i", "(", 
       RowBox[{"x", "\[CenterDot]", "y"}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", "y"}], " ", "+", " ", "I"}]}], 
     "                                  ", "\n", "\t\t\t   ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "+", " ", "I"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"y", " ", "+", " ", "I"}], ")"}]}]}], 
     "                                 ", "\n", "\t\t\t   ", 
     RowBox[{
      RowBox[{"=", " ", 
       RowBox[{
        RowBox[{"i", "(", "x", ")"}], "\[CenterDot]", 
        RowBox[{"i", "(", "y", ")"}]}]}], ","}]}]}], TraditionalForm]]],
 "\n\nwe see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "(", "x", ")"}], TraditionalForm]]],
 " is indeed a surjective homomorphism."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437324276328125*^9, 3.437324278234375*^9}}]
}, Open  ]],

Cell[TextData[{
 "In the homomorphisms produced by lemma 10.3, the image of the homomorphism \
is isomorphic to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "/", 
    RowBox[{"Ker", "(", "\[Florin]", ")"}]}], TraditionalForm]]],
 ". It wouldn't be too surprising if this were true for ",
 StyleBox["all",
  FontSlant->"Italic"],
 " ring homomorphisms, since the first isomorphism theorem studied in the \
volume on groups shows that the additive group on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Im", "(", "\[Florin]", ")"}], TraditionalForm]]],
 " would be group isomorphic to the additive structure of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "/", 
    RowBox[{"Ker", "(", "\[Florin]", ")"}]}], TraditionalForm]]],
 ". It is easy to show that the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Im", "(", "\[Florin]", ")"}], TraditionalForm]]],
 " is isomorphic to the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "/", 
    RowBox[{"Ker", "(", "\[Florin]", ")"}]}], TraditionalForm]]],
 " as well, giving us an isomorphism theorem for rings. \n\nTHEOREM 10.2:  \
THE FIRST RING ISOMORPHISM THEOREM\nLet ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " be a ring homomorphism from a ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " to a ring ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", whose image is ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". If the kernel of ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", then there is a natural surjective isomorphism \n\n                       \
     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", " ", ":", " ", 
    RowBox[{
     RowBox[{"R", "/", "I"}], " ", "\[RightArrow]", " ", "H"}]}], 
   TraditionalForm]]],
 "\n                            \nwhich causes the following diagram to \
commute:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373243014375*^9, 3.437324301546875*^9}, {
  3.43732436478125*^9, 3.437324389078125*^9}, {3.43906184015625*^9, 
  3.4390618449375*^9}}],

Cell[TextData[{
 "                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"              ", 
    RowBox[{"R", "                  ", "\n", 
     RowBox[{"\[Florin]", 
      StyleBox["\[LowerLeftArrow]",
       FontSize->24], "   ", 
      StyleBox["\[LowerRightArrow]",
       FontSize->24], 
      RowBox[{
       FormBox["",
        TraditionalForm], "i"}]}], "        ", "\n", 
     RowBox[{
      RowBox[{"H", "  ", 
       StyleBox["\[LongLeftRightArrow]",
        FontSize->24], " ", "R"}], "/", "I"}], "   ", "\n", "               ",
      "\[Phi]"}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43732441521875*^9, 3.437324416890625*^9}}],

Cell[TextData[{
 "(Here, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "(", "x", ")"}], TraditionalForm]]],
 " is the homomorphism defined in lemma 10.3.) Thus, \n\n                     \
                ",
 Cell[BoxData[
  FormBox[
   RowBox[{"H", " ", "\[TildeTilde]", " ", 
    RowBox[{"R", "/", "I"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43732455884375*^9, 3.437324578734375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The diagram actually helps us determine how ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " needs to be defined. For each coset ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", " ", "+", " ", "I"}], ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "I"}], TraditionalForm]]],
 ", we need to have\n\n                              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", 
     RowBox[{"x", " ", "+", " ", "I"}], ")"}], " ", "=", " ", 
    RowBox[{"\[Florin]", "(", "x", ")"}]}], TraditionalForm]]],
 "\n\nin order for the diagram to commute. To prove that this rule defines a \
mapping, we need to show that this is well defined. That is, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"x", " ", "+", " ", "I"}], " ", "=", " ", 
     RowBox[{"y", " ", "+", " ", "I"}]}], ","}], TraditionalForm]]],
 " it needs to be true that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"\[Florin]", "(", "y", ")"}]}], TraditionalForm]]],
 ", or else there would be a contradiction in the definition of ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 ". But\n            ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "+", " ", "I"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"y", " ", "+", " ", 
      RowBox[{
       StyleBox["I",
        FontSlant->"Italic"], 
       StyleBox["   ",
        FontSlant->"Italic"], "\[DoubleLongLeftRightArrow]", "   ", "x"}], 
      " ", "-", " ", "y"}], " ", "\[Element]", " ", "I"}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                               ", 
    RowBox[{
     RowBox[{"\[DoubleLongLeftRightArrow]", "  ", 
      RowBox[{"\[Florin]", "(", 
       RowBox[{"x", " ", "-", " ", "y"}], ")"}]}], " ", "=", " ", "0"}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                               ", 
    RowBox[{
     RowBox[{"\[DoubleLongLeftRightArrow]", "  ", 
      RowBox[{"\[Florin]", "(", "x", ")"}]}], " ", "=", "  ", 
     RowBox[{"\[Florin]", "(", "y", ")"}]}]}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t                        ", 
    RowBox[{
     RowBox[{"\[DoubleLongLeftRightArrow]", "  ", 
      RowBox[{"\[Phi]", "(", 
       RowBox[{"x", "+", "I"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"\[Phi]", "(", 
       RowBox[{"y", "+", "I"}], ")"}], "."}]}]}], TraditionalForm]}]],
 "\n\nSo we see that the definition of \[Phi] will not produce any such \
contradictions. \n\nTo show that ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is a homomorphism, we have that \n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"x", "+", "I"}], ")"}], "+", 
        RowBox[{"(", 
         RowBox[{"y", "+", "I"}], ")"}]}], ")"}], " ", "=", " ", 
      RowBox[{"\[Phi]", "(", 
       RowBox[{"x", "+", "y", "+", "I"}], ")"}]}], 
     "                                         ", "\n", 
     "\t\t\t                       ", 
     RowBox[{"=", " ", 
      RowBox[{"\[Florin]", "(", 
       RowBox[{"x", "+", "y"}], ")"}]}], 
     " \t                                      ", "\n", 
     "\t\t\t\t                   ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"\[Florin]", "(", "x", ")"}], " ", "+", " ", 
       RowBox[{"\[Florin]", "(", "y", ")"}]}]}], 
     " \t\t\t                                  ", "\n", 
     "\t\t\t\t\t                ", 
     RowBox[{
      RowBox[{"=", " ", 
       RowBox[{
        RowBox[{"\[Phi]", "(", 
         RowBox[{"x", "+", "I"}], ")"}], " ", "+", " ", 
        RowBox[{"\[Phi]", "(", 
         RowBox[{"y", "+", "I"}], ")"}]}]}], ","}]}]}], TraditionalForm]]],
 "\nand \n        ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", "+", "I"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"y", "+", "I"}], ")"}]}], ")"}], " ", "=", " ", 
    RowBox[{"\[Phi]", "(", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "y"}], "+", 
      StyleBox["I",
       FontSlant->"Italic"]}], 
     StyleBox[")",
      FontSlant->"Italic"]}]}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                               ", 
    RowBox[{"=", " ", 
     RowBox[{"\[Florin]", "(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}]}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t                            ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"\[Florin]", "(", "x", ")"}], "\[CenterDot]", " ", 
      RowBox[{"\[Florin]", "(", "y", ")"}]}]}]}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t                     ", 
    RowBox[{
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"\[Phi]", "(", 
        RowBox[{"x", "+", "I"}], ")"}], "\[CenterDot]", 
       RowBox[{"\[Phi]", "(", 
        RowBox[{"y", "+", "I"}], ")"}]}]}], ","}]}], TraditionalForm]}]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is a homomorphism from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "I"}], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ". It is apparent that this homomorphism is onto, and \n\n                   \
",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]", "(", 
       RowBox[{"x", "+", "I"}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"0", "   ", "\[DoubleLongLeftRightArrow]", "   ", 
        RowBox[{"\[Florin]", "(", "x", ")"}]}], " ", "=", " ", "0"}]}], 
     "                             ", "\n", "\t            ", 
     RowBox[{
      RowBox[{"\[DoubleLongLeftRightArrow]", "  ", "x"}], " ", "\[Element]", 
      " ", "I"}], "                             ", "\n", "\t            ", 
     RowBox[{
      RowBox[{
       RowBox[{"\[DoubleLongLeftRightArrow]", "   ", "x"}], "+", "I"}], " ", 
      "=", " ", 
      RowBox[{"I", "."}]}]}]}], TraditionalForm]]],
 "\n\nSo the kernel of ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", "I", "}"}], ","}], TraditionalForm]]],
 " the zero element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "/", "I"}], TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is an isomorphism from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "I"}], TraditionalForm]]],
 " onto ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]]],
 ", so  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "/", "I"}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", "H"}], TraditionalForm]]],
 ".\nSince the mapping ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " was defined so that the diagram commutes, the theorem is proved."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437324639953125*^9, 3.43732466115625*^9}, {
  3.437324691328125*^9, 3.437324728421875*^9}, {3.4417240897452497`*^9, 
  3.4417240922296247`*^9}, {3.4417241487452497`*^9, 3.4417241550733747`*^9}}]
}, Open  ]],

Cell[TextData[{
 "It should be noted that there are second and third ring isomorphism \
theorems, which correspond to the second and third isomorphism theorems \
studied in our work with groups. These are covered in problems 10.46 and \
10.47. However, we are mainly interested in the first ring isomorphism \
theorem, because this allows us to guarantee the existance of isomorphisms \
without having to actually write them down. This becomes particularly \
important when we start working with infinite rings, since the isomorphisms \
could become unwieldy.\n\nSo far, the rings we have defined in this chapter \
using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " have all been finite rings. This allowed us to see the multiplication \
tables and obverve patterns in the structure of the rings. However, many of \
the important applications of ring theory involve infinite rings. In fact, in \
the next chapter you will be mainly working with infinite rings. But before \
you worry about how complicated it must be to define an infinite ring in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", it should be pointed out that in the process of defining a finite ring, \
one also defines an infinite ring unawares. Consider the example of the \
non-commutative ring of order 8:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373247528125*^9, 3.43732475728125*^9}, {
  3.439061853953125*^9, 3.43906187928125*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitRing;
Define[4 a, 0]; Define[2 b, 0]
Define[a.a, a]; Define[b.b, 0]
Define[a.b, b]; Define[b.a, 2 a]
R = Ring[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43732478503125*^9, 3.437324810859375*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442753038773875*^9}]
}, Open  ]],

Cell["The multiplication table is given by", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {
      0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{
      1, 0.6, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 
      1, 0}, {1, 1, 0}, {1, 1, 0}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0, 0}, {
      0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4,
       0.8}}, {{0, 1, 0}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 
     0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 
     1}}, {{1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {
      0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}}, {{0, 1, 1}, {1, 1, 0}, {
     0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 
     1}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}, {{1, 1, 0}, {1, 1, 0}, {
     1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.44275304019575*^9}]
}, Open  ]],

Cell["\<\
Notice from the multiplication table that there is no identity element. This \
can also be verified by the command:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["FindIdent[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData["\<\"No identity element found.\"\>"], "Print",
 CellChangeTimes->{3.442753041805125*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can multiply any element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " by any integer, and simplify it to an element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["(3 a + b) . 7", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"a", "+", "b"}]], "Output",
 CellChangeTimes->{3.442753044336375*^9}]
}, Open  ]],

Cell[TextData[{
 "But what would happen if we ",
 StyleBox["added",
  FontSlant->"Italic"],
 " an integer to an element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["(3 a + b) + 7", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"7", "+", 
  RowBox[{"3", " ", "a"}], "+", "b"}]], "Output",
 CellChangeTimes->{3.442753045711375*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not simplify this, since this is no longer an element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". But could this be an element of a larger ring?\n\nEXPERIMENT:\nLet "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["A = 7 + 3 a + b", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"7", "+", 
  RowBox[{"3", " ", "a"}], "+", "b"}]], "Output",
 CellChangeTimes->{3.442753048242625*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["B = -2 + a + b", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "2"}], "+", "a", "+", "b"}]], "Output",
 CellChangeTimes->{3.442753049180125*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["F = 1 + b", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"1", "+", "b"}]], "Output",
 CellChangeTimes->{3.442753050898875*^9}]
}, Open  ]],

Cell[TextData[{
 "Does the product of two of these elements form an element of the same type? \
(An integer plus an element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ") Does ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"A", "\[CenterDot]", "B"}], ")"}], "\[CenterDot]", "F"}], " ", 
    "=", " ", 
    RowBox[{"A", "\[CenterDot]", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"B", "\[CenterDot]", "F"}], ")"}], "?"}]}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "From the experiment it appears we may have another ring defined as well as \
the ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Suppose we let ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " denote the set of all expressions of the form (integer + element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "). It seems that the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " operations cause ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " to be a ring in its own right. ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " is of course an infinite ring, since it contains all of the integers. But \
it also contains the elements of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". We say that ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " are both ",
 StyleBox["embedded",
  FontSlant->"Italic"],
 " in ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 ". \n\nDEFINITION 10.10:\nWe say that the ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is ",
 StyleBox["embedded",
  FontSlant->"Italic"],
 " in the ring ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " if there exists an injective homomorphism ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", ":", 
    RowBox[{"R", " ", "\[RightArrow]", " ", "S"}]}], TraditionalForm]]],
 ". The mapping ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is called an ",
 StyleBox["embedding",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". \n\nEXPERIMENT:\nCan you find an embedding of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 "? How about an embedding of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 "? This is actually very easy, since there is a \"copy\" of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 ". Does ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 " have an identity element? If so, what would it have to be?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437324836296875*^9, 3.43732486696875*^9}, {
  3.4417241870421247`*^9, 3.4417241997139997`*^9}, {3.4417242546358747`*^9, 
  3.4417242620264997`*^9}}],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "It is not surprizing that we can expand any ring to get an infinite ring \
which contains that ring along with the ring ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". The details are included in the following theorem.\n\nTHEOREM 10.3:  THE \
EMBEDDING THEOREM\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a ring. Then ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " can be embedded in a ring ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " which has a multiplicative identity."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437324900015625*^9, 3.43732491290625*^9}, {
  3.4417242951983747`*^9, 3.4417242957296247`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Rather than expressing elements as an integer plus an element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", we will use an order pair ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"n", ",", "x"}], ")"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is an integer and ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", to denote the elements of our new ring. Thus, we define ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " to be the set\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", "=", " ", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"n", ",", "x"}], ")"}], "  ", "|", "   ", 
        RowBox[{"n", " ", "\[Element]", " ", "\[DoubleStruckCapitalZ]"}]}], 
       ",", "  ", 
       RowBox[{"x", " ", "\[Element]", " ", "R"}]}], "}"}], "."}]}], 
   TraditionalForm]]],
 "\n             \nWe define addition and multiplication on ",
 Cell[BoxData[
  FormBox["X", TraditionalForm]]],
 " as follows:\n  \n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["n", "1"], ",", 
        SubscriptBox["x", "1"]}], ")"}], " ", "+", " ", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["n", "2"], ",", 
        SubscriptBox["x", "2"]}], ")"}]}], " ", "=", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["n", "1"], "+", " ", 
        SubscriptBox["n", "2"]}], ",", " ", 
       RowBox[{
        SubscriptBox["x", "1"], "+", " ", 
        SubscriptBox["x", "2"]}]}], ")"}]}], ","}], TraditionalForm]]],
 "\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["n", "1"], ",", 
       SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["n", "2"], ",", 
       SubscriptBox["x", "2"]}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["n", "1"], "\[CenterDot]", 
        SubscriptBox["n", "2"]}], ",", " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["x", "1"], "\[CenterDot]", 
         SubscriptBox["x", "2"]}], " ", "+", " ", 
        RowBox[{
         SubscriptBox["n", "1"], "\[CenterDot]", 
         SubscriptBox[
          RowBox[{"x", " "}], "2"]}], "+", " ", 
        RowBox[{
         SubscriptBox["n", "2"], "\[CenterDot]", 
         SubscriptBox["x", "1"]}]}]}], ")"}], "."}]}], TraditionalForm]]],
 "\n\nIt is clear that ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " forms an abelian group under addition, with the zero element being ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"0", ",", "0"}], ")"}], TraditionalForm]]],
 ". The product of two elements of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is clearly in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", so we only need to check the associativity of multiplication, and the two \
distributive laws. We have that             \n\n ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["n", "1"], ",", 
           SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["n", "2"], ",", 
           SubscriptBox["x", "2"]}], ")"}]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["n", "3"], ",", 
         SubscriptBox["x", "3"]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["n", "1"], 
          SubscriptBox["n", "2"]}], ",", " ", 
         RowBox[{
          RowBox[{
           SubscriptBox["x", "1"], "\[CenterDot]", 
           SubscriptBox["x", "2"]}], " ", "+", " ", 
          RowBox[{
           SubscriptBox["n", "1"], "\[CenterDot]", 
           SubscriptBox["x", 
            RowBox[{"2", " "}]]}], "+", " ", 
          RowBox[{
           SubscriptBox["n", "2"], "\[CenterDot]", 
           SubscriptBox["x", "1"]}]}]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["n", "3"], ",", 
         SubscriptBox["x", "3"]}], ")"}]}]}],
     TraditionalForm], "   "}], TraditionalForm], "\[IndentingNewLine]", 
  FormBox[
   RowBox[{"                                          ", 
    RowBox[{"=", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["n", "1"], 
         SubscriptBox["n", "2"], 
         SubscriptBox["n", "3"]}], ",", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"(", " ", 
           RowBox[{
            RowBox[{
             SubscriptBox["x", "1"], "\[CenterDot]", 
             SubscriptBox["x", "2"]}], " ", "+", " ", 
            RowBox[{
             SubscriptBox["n", "1"], "\[CenterDot]", 
             SubscriptBox["x", 
              RowBox[{"2", " "}]]}], "+", " ", 
            RowBox[{
             SubscriptBox["n", "2"], "\[CenterDot]", 
             SubscriptBox["x", "1"]}]}], ")"}], "\[CenterDot]", 
          SubscriptBox["x", "3"]}], " ", "+", " ", 
         RowBox[{
          SubscriptBox["n", "1"], " ", 
          SubscriptBox["n", "2"], 
          SubscriptBox["x", 
           RowBox[{"3", " "}]]}], "+", "\[IndentingNewLine]", 
         "                                      ", 
         RowBox[{
          SubscriptBox["n", "3"], "(", " ", 
          RowBox[{
           RowBox[{
            SubscriptBox["x", "1"], "\[CenterDot]", 
            SubscriptBox["x", "2"]}], " ", "+", " ", 
           RowBox[{
            SubscriptBox["n", "1"], "\[CenterDot]", 
            SubscriptBox["x", 
             RowBox[{"2", " "}]]}], "+", " ", 
           RowBox[{
            SubscriptBox["n", "2"], "\[CenterDot]", 
            SubscriptBox["x", "1"]}]}], ")"}]}]}], "   ", ")"}], " ", 
      FormBox[
       RowBox[{
       " ", "\[IndentingNewLine]", "                                   ", 
        RowBox[{"=", 
         RowBox[{
          RowBox[{"(", " ", 
           RowBox[{
            RowBox[{
             SubscriptBox["n", "1"], 
             SubscriptBox["n", "2"], 
             SubscriptBox["n", "3"]}], ",", " ", 
            RowBox[{
             RowBox[{
              SubscriptBox["x", "1"], "\[CenterDot]", 
              SubscriptBox["x", "2"], " ", "\[CenterDot]", 
              SubscriptBox["x", "3"]}], " ", "+", " ", 
             RowBox[{
              SubscriptBox["n", "1"], " ", 
              RowBox[{
               SubscriptBox["x", "2"], "\[CenterDot]", " ", 
               SubscriptBox["x", "3"]}]}], " ", "+", " ", 
             RowBox[{
              SubscriptBox["n", "2"], 
              RowBox[{
               SubscriptBox["x", "1"], "\[CenterDot]", " ", 
               SubscriptBox["x", "3"]}]}], "  ", "+", "\[IndentingNewLine]", 
             "                               ", 
             RowBox[{
              SubscriptBox["n", "1"], 
              SubscriptBox["n", "2"], 
              SubscriptBox["x", "3"]}], " ", "+", " ", 
             RowBox[{
              SubscriptBox["n", "3"], 
              RowBox[{
               SubscriptBox["x", "1"], "\[CenterDot]", 
               SubscriptBox["x", "2"]}]}], " ", "+", " ", 
             RowBox[{
              SubscriptBox["n", "1"], 
              SubscriptBox["n", "3"], 
              SubscriptBox["x", "2"]}], "  ", "+", " ", 
             RowBox[{
              SubscriptBox["n", "2"], 
              SubscriptBox["n", "3"], 
              SubscriptBox["x", "1"]}]}]}], " ", ")"}], "."}]}]}],
       TraditionalForm]}]}]}], TraditionalForm]}]],
 "\nAlso, \n\n",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{" ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["n", "1"], ",", 
          SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "2"], ",", 
            SubscriptBox["x", "2"]}], ")"}], "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "3"], ",", 
            SubscriptBox["x", "3"]}], ")"}]}], ")"}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["n", "1"], ",", 
          SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["n", "2"], 
           SubscriptBox["n", "3"]}], ",", 
          RowBox[{
           RowBox[{
            SubscriptBox["x", "2"], "\[CenterDot]", 
            SubscriptBox["x", "3"]}], " ", "+", " ", 
           RowBox[{
            SubscriptBox["n", "2"], 
            SubscriptBox["x", "3"]}], " ", "+", " ", 
           RowBox[{
            SubscriptBox["n", "3"], 
            SubscriptBox["x", "2"]}]}]}], ")"}]}]}]}],
     TraditionalForm], " "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                                                     ", 
    RowBox[{"=", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["n", "1"], 
        SubscriptBox["n", "2"], 
        SubscriptBox["n", "3"]}], ",", 
       RowBox[{
        RowBox[{
         SubscriptBox["x", "1"], "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["x", "2"], "\[CenterDot]", 
            SubscriptBox["x", "3"]}], " ", "+", " ", 
           RowBox[{
            SubscriptBox["n", "2"], 
            SubscriptBox["x", "3"]}], " ", "+", " ", 
           RowBox[{
            SubscriptBox["n", "3"], 
            SubscriptBox["x", "2"]}]}], ")"}]}], " ", "+", " ", 
        "\[IndentingNewLine]", 
        "                                                       ", 
        RowBox[{
         SubscriptBox["n", "1"], "(", 
         RowBox[{
          RowBox[{
           SubscriptBox["x", "2"], "\[CenterDot]", 
           SubscriptBox["x", "3"]}], " ", "+", " ", 
          RowBox[{
           SubscriptBox["n", "2"], 
           SubscriptBox["x", "3"]}], " ", "+", " ", 
          RowBox[{
           SubscriptBox["n", "3"], 
           SubscriptBox["x", "2"]}]}], ")"}], " ", "+", " ", 
        RowBox[{
         SubscriptBox["n", "2"], 
         SubscriptBox["n", "3"], 
         SubscriptBox["x", "1"], Cell[""]}]}]}], ")"}]}]}], 
   TraditionalForm], "\[IndentingNewLine]", 
  FormBox[
   RowBox[{"                                                   ", 
    RowBox[{"=", 
     RowBox[{
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["n", "1"], 
         SubscriptBox["n", "2"], 
         SubscriptBox[
          RowBox[{"n", " "}], "3"]}], ",", 
        RowBox[{
         RowBox[{
          SubscriptBox["x", "1"], "\[CenterDot]", 
          SubscriptBox["x", "2"], "\[CenterDot]", 
          SubscriptBox["x", "3"]}], " ", "+", " ", 
         RowBox[{
          SubscriptBox["n", "2"], 
          RowBox[{
           SubscriptBox["x", "1"], "\[CenterDot]", 
           SubscriptBox["x", "3"]}]}], " ", "+", " ", 
         RowBox[{
          SubscriptBox["n", "3"], 
          RowBox[{
           SubscriptBox["x", "1"], "\[CenterDot]", 
           SubscriptBox["x", "2"]}]}], " ", "+", "\[IndentingNewLine]", 
         "                                                    ", 
         RowBox[{
          SubscriptBox["n", "1"], 
          RowBox[{
           SubscriptBox["x", "2"], "\[CenterDot]", 
           SubscriptBox["x", "3"]}]}], " ", "+", " ", 
         RowBox[{
          SubscriptBox["n", "1"], 
          SubscriptBox["n", "2"], 
          SubscriptBox["x", "3"]}], "+", " ", 
         RowBox[{
          SubscriptBox["n", "1"], 
          SubscriptBox["n", "3"], 
          SubscriptBox["x", "2"]}], "+", " ", 
         RowBox[{
          SubscriptBox["n", "2"], 
          SubscriptBox["n", "3"], 
          SubscriptBox["x", "1"]}]}]}], " ", ")"}], ".", "   "}]}]}], 
   TraditionalForm]}]],
 "\n\nThese two are equal, so multiplication in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is associative. We also have\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{" ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "1"], ",", 
            SubscriptBox["x", "1"]}], ")"}], "+", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "2"], ",", 
            SubscriptBox["x", "2"]}], ")"}]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["n", "3"], ",", 
          SubscriptBox["x", "3"]}], ")"}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["n", "1"], "+", " ", 
           SubscriptBox["n", "2"]}], ",", "  ", 
          RowBox[{
           SubscriptBox["x", "1"], "+", " ", 
           SubscriptBox["x", "2"]}]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["n", "3"], ",", 
          SubscriptBox["x", "3"]}], ")"}]}]}]}],
     TraditionalForm], " ", 
    FormBox[
     RowBox[{" ", "\n", "\t ", 
      RowBox[{"=", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           SubscriptBox["n", "1"], 
           SubscriptBox["n", "3"]}], "+", " ", 
          RowBox[{
           SubscriptBox["n", "2"], " ", 
           SubscriptBox["n", "3"]}]}], ",", 
         RowBox[{
          RowBox[{
           SubscriptBox["x", "1"], "\[CenterDot]", 
           SubscriptBox["x", "3"]}], "+", 
          RowBox[{
           SubscriptBox["x", "2"], "\[CenterDot]", 
           SubscriptBox["x", "3"]}], "+", 
          RowBox[{
           SubscriptBox["n", "1"], 
           SubscriptBox["x", "3"]}], "+", " ", 
          RowBox[{
           SubscriptBox["n", "2"], 
           SubscriptBox["x", "3"]}], "+", 
          RowBox[{
           SubscriptBox["n", "3"], "\[CenterDot]", 
           SubscriptBox["x", "1"]}], "+", 
          RowBox[{
           SubscriptBox["n", "3"], " ", 
           SubscriptBox["x", "2"]}]}]}], ")"}], " "}]}],
     TraditionalForm], 
    FormBox[
     RowBox[{" ", "\n", "\t ", 
      RowBox[{
       RowBox[{
        RowBox[{"=", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "1"], ",", 
            SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "3"], ",", 
            SubscriptBox["x", "3"]}], ")"}]}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["n", "2"], ",", 
           SubscriptBox["x", "2"]}], ")"}], "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["n", "3"], ",", 
           SubscriptBox["x", "3"]}], ")"}]}]}], ","}]}],
     TraditionalForm]}], TraditionalForm]]],
 "\n\nand\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{" ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["n", "1"], ",", 
          SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "2"], ",", 
            SubscriptBox["x", "2"]}], ")"}], "+", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "3"], ",", 
            SubscriptBox["x", "3"]}], ")"}]}], ")"}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["n", "1"], ",", 
          SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["n", "2"], "+", " ", 
           SubscriptBox["n", "3"]}], ",", 
          RowBox[{
           SubscriptBox["x", "2"], "+", 
           SubscriptBox["x", "3"]}]}], ")"}]}]}]}],
     TraditionalForm], " ", 
    FormBox[
     RowBox[{"   ", "\n", "\t", 
      RowBox[{"=", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["n", "1"], " ", 
          SubscriptBox["n", "2"]}], "+", " ", 
         RowBox[{
          SubscriptBox["n", "1"], " ", 
          SubscriptBox["n", 
           RowBox[{
            RowBox[{"3", ","}], " "}]], 
          RowBox[{
           SubscriptBox["x", "1"], "\[CenterDot]", 
           SubscriptBox[
            RowBox[{"x", " "}], "2"]}]}], "+", " ", 
         RowBox[{
          SubscriptBox["x", "1"], "\[CenterDot]", " ", 
          SubscriptBox["x", "3"]}], " ", "+", " ", 
         RowBox[{
          SubscriptBox["n", "1"], 
          SubscriptBox["x", "2"]}], " ", "+", " ", 
         RowBox[{
          SubscriptBox["n", "1"], 
          SubscriptBox["x", "3"]}], " ", "+", " ", 
         RowBox[{
          SubscriptBox["n", "2"], 
          SubscriptBox["x", "1"]}], " ", "+", " ", 
         RowBox[{
          SubscriptBox["n", "3"], 
          SubscriptBox["x", "1"]}]}], ")"}], " "}]}],
     TraditionalForm], " ", 
    FormBox[
     RowBox[{"   ", "\n", "\t", 
      RowBox[{
       RowBox[{
        RowBox[{"=", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "1"], ",", 
            SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "2"], ",", 
            SubscriptBox["x", "2"]}], ")"}]}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["n", "1"], ",", 
           SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["n", "3"], ",", 
           SubscriptBox["x", "3"]}], ")"}]}]}], ","}]}],
     TraditionalForm]}], TraditionalForm]]],
 "\n\nso the two distributive laws are satisfied. Thus, ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is a ring.\n\nFurthermore, the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"1", ",", "0"}], ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " acts as a multiplicative identity, since\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"n", ",", " ", "x"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"1", ",", " ", "0"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"n", "\[CenterDot]", "1"}], ",", " ", 
         RowBox[{
          RowBox[{"x", "\[CenterDot]", "0"}], " ", "+", " ", 
          RowBox[{"n", "\[CenterDot]", "0"}], " ", "+", " ", 
          RowBox[{"1", "\[CenterDot]", "x"}]}]}], ")"}], " ", "=", " ", 
       RowBox[{"(", 
        RowBox[{"n", ",", " ", "x"}], ")"}]}]}], ","}]}], TraditionalForm]]],
 " \nand\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", ",", " ", "0"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"n", ",", " ", "x"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"1", "\[CenterDot]", "n"}], ",", " ", 
        RowBox[{
         RowBox[{"0", "\[CenterDot]", "x"}], " ", "+", " ", 
         RowBox[{"1", "\[CenterDot]", "x"}], " ", "+", " ", 
         RowBox[{"n", "\[CenterDot]", "0"}]}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"n", ",", " ", "x"}], ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nAll that is left is to show that the ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " can be embedded into ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". We can define a mapping from ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " simply by letting ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"0", ",", "x"}], ")"}], "."}]}], TraditionalForm]]],
 " This is certainly an injective mapping, and it is easy to check that\n\n   \
            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", "x", ")"}], " ", "+", " ", 
      RowBox[{"\[Florin]", "(", "y", ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0", ",", "x"}], ")"}], " ", "+", " ", 
       RowBox[{"(", 
        RowBox[{"0", ",", "y"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0", ",", 
         RowBox[{"x", "+", "y"}]}], ")"}], " ", "=", " ", 
       RowBox[{"\[Florin]", "(", 
        RowBox[{"x", "+", "y"}], ")"}]}]}]}], ","}], TraditionalForm]]],
 "\nand\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Florin]", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"\[Florin]", "(", "y", ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"0", ",", "x"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"0", ",", "y"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"0", ",", 
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "y"}], " ", "+", " ", 
         RowBox[{"0", "\[CenterDot]", "y"}], " ", "+", " ", 
         RowBox[{"0", "\[CenterDot]", "x"}]}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0", ",", 
         RowBox[{"x", "\[CenterDot]", "y"}]}], ")"}], " ", "=", " ", 
       RowBox[{
        RowBox[{"\[Florin]", "(", 
         RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nSo we have an embedding of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", which completes the proof."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43732494140625*^9, 3.437325058953125*^9}}]
}, Open  ]],

Cell[TextData[{
 "We call the ring ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " used in this theorem the ",
 StyleBox["extension of",
  FontSlant->"Italic"],
 "  ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " ",
 StyleBox["by the integers",
  FontSlant->"Italic"],
 ". This ring is important because it allows us to treat any ring as though \
it has a multiplicative identity by using the ring ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " in place of the ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". However, some of the properties of the ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " are lost in taking the extension.\n\nEXPERIMENT:\nConsider the extension \
of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " by the integers. What type of ring will be produced? It is helpful to \
first define ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " as the identity:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43732506425*^9, 3.437325077640625*^9}}],

Cell["\<\
InitRing
Define[e.e, e]\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Now consider the extension of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " by the integers as elements of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "+", " ", 
    RowBox[{"n", " ", "e"}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are integers. Can you find a zero divisor in this ring?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["    ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "This experiment demonstrates that we are able to work with infinite rings \
with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " even though we cannot display a multiplication table for these rings. In \
the next chapter, we will learn how to define infinite rings much more \
efficiently if there are no zero divisors in the ring. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Problems"
}], "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43732512003125*^9, 3.43732512246875*^9}}],

Cell["", "Input",
 PageWidth->Infinity,
 FontFamily->"System",
 FontSize->10,
 FontWeight->"Bold",
 FontColor->RGBColor[1, 0, 0],
 Background->GrayLevel[1]],

Cell["10.1 ) Find all or the subrings of the ring of order 8: ", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.437325125703125*^9}],

Cell["\<\
InitRing
Define[4 a, 0]; Define[2 b, 0]
Define[a.a, a]; Define[b.b, 0]
Define[a.b, b]; Define[b.a, 0]
R = Ring[{a, b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437325133703125*^9, 3.437325147765625*^9}, {
  3.442753070992625*^9, 3.442753072352*^9}}],

Cell["   Which of these subrings are ideals?", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["          ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "\n10.2 ) Load the rings ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "12"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 " into ",
 StyleBox["Mathematica \n   ",
  FontSlant->"Italic"],
 "simultaneously with the commands:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437325179109375*^9, 3.43732519515625*^9}}],

Cell["\<\
InitRing
Define[12 a, 0]; Define[a.a,a]
Z12 = Ring[{a}]
Define[6 b, 0]; Define[b.b,b]
Z6 = Ring[{b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43732520471875*^9, 3.43732523859375*^9}}],

Cell[TextData[{
 "   Show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{"0", ",", " ", 
      RowBox[{"6", "a"}]}], "}"}]}], TraditionalForm]]],
 " is an ideal of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "12"], TraditionalForm]]],
 ", and \n   display addition and multiplication tables of \n   the quotient \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "12"], "/", "I"}], TraditionalForm]]],
 ", showing that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "12"], "/", "I"}], TraditionalForm]]],
 " is \n   isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "6"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437325228*^9, 3.437325258953125*^9}}],

Cell["     ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "  \n10.3 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the 8 non-isomorphic \n   non-cyclic rings of order 4.\n   (Hint: \
The additive group must be isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["Z", "8", "*"], TraditionalForm]]],
 ",\n   so the ring is defined by:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
InitRing
Define[2 a, 0]; Define[2 b, 0]
Define[a.a, ???]
Define[b.b, ???]
Define[a.b, ???]
Define[b.a, ???]
CheckRing[{a, b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43732529115625*^9, 3.43732529375*^9}}],

Cell[TextData[{
 "    Fill in each ??? with a member of \n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", " ", "a", ",", " ", "b", ",", " ", 
     RowBox[{"a", "+", "b"}]}], "}"}], TraditionalForm]]],
 " to see whether a ring is \n    formed. Is there a faster way than\n    \
trying all ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["4", "4"], "=", " ", "256"}], TraditionalForm]]],
 " combinations?)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["        ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "\n10.4 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to display the \n   multiplication tables of all rings of order 6."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437325335640625*^9, 3.43732533925*^9}}],

Cell["       ", "Input",
 PageWidth->Infinity]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{735, 478},
WindowMargins->{{-3, Automatic}, {Automatic, 1}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.5,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[568, 21, 322, 9, 222, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[915, 34, 285, 8, 29, "Name",
 Evaluatable->False],
Cell[1203, 44, 709, 16, 70, "Name",
 Evaluatable->False]
}, Closed]],
Cell[1927, 63, 269, 12, 205, "Title",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2221, 79, 104, 3, 32, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2350, 86, 173, 4, 43, "Input",
 InitializationCell->True],
Cell[2526, 92, 141, 2, 32, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2716, 100, 81, 2, 105, "Section",
 Evaluatable->False],
Cell[2800, 104, 2156, 70, 556, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[4981, 178, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[5069, 182, 2441, 94, 260, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[7525, 279, 658, 18, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[8208, 301, 229, 8, 133, "Input"],
Cell[8440, 311, 365, 10, 68, "Output"]
}, Open  ]],
Cell[8820, 324, 123, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[8968, 330, 51, 1, 43, "Input"],
Cell[9022, 333, 113, 2, 42, "Output"]
}, Open  ]],
Cell[9150, 338, 285, 10, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[9460, 352, 57, 1, 43, "Input"],
Cell[9520, 355, 201, 5, 42, "Output"]
}, Open  ]],
Cell[9736, 363, 206, 6, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[9967, 373, 50, 1, 43, "Input"],
Cell[10020, 376, 7464, 172, 403, 3611, 107, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17521, 553, 51, 1, 43, "Input"],
Cell[17575, 556, 3661, 108, 403, "Output"]
}, Open  ]],
Cell[21251, 667, 616, 16, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[21892, 687, 51, 1, 43, "Input"],
Cell[21946, 690, 89, 1, 42, "Output"]
}, Open  ]],
Cell[22050, 694, 939, 27, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[23014, 725, 59, 1, 43, "Input"],
Cell[23076, 728, 156, 4, 42, "Output"]
}, Open  ]],
Cell[23247, 735, 111, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[23383, 741, 50, 1, 43, "Input"],
Cell[23436, 744, 3301, 96, 403, "Output"]
}, Open  ]],
Cell[26752, 843, 73, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[26850, 849, 51, 1, 43, "Input"],
Cell[26904, 852, 3657, 108, 403, "Output"]
}, Open  ]],
Cell[30576, 963, 1716, 50, 572, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[32317, 1017, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[32405, 1021, 1958, 73, 260, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[34378, 1097, 710, 25, 115, "Text",
 Evaluatable->False],
Cell[35091, 1124, 297, 8, 66, "Text",
 Evaluatable->False],
Cell[35391, 1134, 623, 21, 101, "Text",
 Evaluatable->False],
Cell[36017, 1157, 1255, 49, 164, "Text",
 Evaluatable->False],
Cell[37275, 1208, 1201, 42, 384, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[38501, 1254, 232, 8, 133, "Input"],
Cell[38736, 1264, 366, 10, 68, "Output"]
}, Open  ]],
Cell[39117, 1277, 182, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[39324, 1286, 48, 1, 43, "Input"],
Cell[39375, 1289, 92, 2, 42, "Output"]
}, Open  ]],
Cell[39482, 1294, 159, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[39666, 1303, 48, 1, 43, "Input"],
Cell[39717, 1306, 183, 5, 42, "Output"]
}, Open  ]],
Cell[39915, 1314, 203, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[40143, 1325, 49, 1, 43, "Input"],
Cell[40195, 1328, 137, 4, 42, "Output"]
}, Open  ]],
Cell[40347, 1335, 220, 8, 164, "Text",
 Evaluatable->False],
Cell[40570, 1345, 31, 0, 43, "Input"],
Cell[40604, 1347, 709, 13, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[41338, 1364, 61, 1, 43, "Input"],
Cell[41402, 1367, 208, 6, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41647, 1378, 50, 1, 43, "Input"],
Cell[41700, 1381, 7932, 180, 403, 3623, 107, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49669, 1566, 51, 1, 43, "Input"],
Cell[49723, 1569, 6881, 162, 403, 3618, 107, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[56619, 1734, 1628, 45, 482, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[58272, 1783, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[58360, 1787, 3135, 117, 362, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[61510, 1907, 758, 26, 167, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[62293, 1937, 100, 4, 66, "Input"],
Cell[62396, 1943, 287, 8, 46, "Output"]
}, Open  ]],
Cell[62698, 1954, 646, 19, 196, "Text",
 Evaluatable->False],
Cell[63347, 1975, 44, 1, 43, "Input"],
Cell[63394, 1978, 390, 8, 164, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[63821, 1991, 98, 2, 105, "Section",
 Evaluatable->False],
Cell[63922, 1995, 631, 13, 286, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[64578, 2012, 232, 8, 133, "Input"],
Cell[64813, 2022, 365, 10, 68, "Output"]
}, Open  ]],
Cell[65193, 2035, 116, 2, 41, "Text",
 Evaluatable->False],
Cell[65312, 2039, 148, 8, 156, "Input"],
Cell[65463, 2049, 1020, 31, 286, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[66508, 2084, 54, 1, 43, "Input"],
Cell[66565, 2087, 454, 14, 68, "Output"]
}, Open  ]],
Cell[67034, 2104, 548, 18, 115, "Text",
 Evaluatable->False],
Cell[67585, 2124, 289, 10, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[67899, 2138, 50, 1, 43, "Input"],
Cell[67952, 2141, 1542, 42, 648, "Output"]
}, Open  ]],
Cell[69509, 2186, 729, 20, 115, "Text",
 Evaluatable->False],
Cell[70241, 2208, 214, 7, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[70480, 2219, 51, 1, 43, "Input"],
Cell[70534, 2222, 1302, 35, 536, "Output"]
}, Open  ]],
Cell[71851, 2260, 280, 8, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[72156, 2272, 70, 1, 43, "Input"],
Cell[72229, 2275, 183, 5, 42, "Output"]
}, Open  ]],
Cell[72427, 2283, 104, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[72556, 2289, 69, 1, 43, "Input"],
Cell[72628, 2292, 323, 9, 42, "Output"]
}, Open  ]],
Cell[72966, 2304, 1423, 51, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[74414, 2359, 74, 5, 88, "Input"],
Cell[74491, 2366, 69, 1, 42, "Output"],
Cell[74563, 2369, 69, 1, 42, "Output"],
Cell[74635, 2372, 69, 1, 42, "Output"]
}, Open  ]],
Cell[74719, 2376, 1094, 36, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[75838, 2416, 52, 1, 43, "Input"],
Cell[75893, 2419, 115, 3, 42, "Output"]
}, Open  ]],
Cell[76023, 2425, 1277, 45, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[77325, 2474, 45, 1, 43, "Input"],
Cell[77373, 2477, 159, 4, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[77569, 2486, 45, 1, 43, "Input"],
Cell[77617, 2489, 158, 4, 42, "Output"]
}, Open  ]],
Cell[77790, 2496, 296, 6, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[78111, 2506, 54, 1, 43, "Input"],
Cell[78168, 2509, 537, 18, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[78742, 2532, 51, 1, 43, "Input"],
Cell[78796, 2535, 3802, 108, 561, "Output"]
}, Open  ]],
Cell[82613, 2646, 1578, 44, 409, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[84216, 2694, 58, 1, 43, "Input"],
Cell[84277, 2697, 92, 1, 42, "Output"]
}, Open  ]],
Cell[84384, 2701, 186, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[84595, 2710, 51, 1, 43, "Input"],
Cell[84649, 2713, 3815, 108, 561, "Output"]
}, Open  ]],
Cell[88479, 2824, 367, 9, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[88871, 2837, 54, 1, 43, "Input"],
Cell[88928, 2840, 535, 18, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89500, 2863, 51, 1, 43, "Input"],
Cell[89554, 2866, 3038, 84, 561, "Output"]
}, Open  ]],
Cell[92607, 2953, 127, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[92759, 2959, 56, 4, 66, "Input"],
Cell[92818, 2965, 115, 3, 42, "Output"],
Cell[92936, 2970, 139, 4, 42, "Output"]
}, Open  ]],
Cell[93090, 2977, 855, 33, 164, "Text",
 Evaluatable->False],
Cell[93948, 3012, 42, 1, 43, "Input"],
Cell[93993, 3015, 2209, 79, 384, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[96227, 3098, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[96315, 3102, 9351, 314, 891, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[105681, 3419, 5495, 166, 824, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[111201, 3589, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[111289, 3593, 1768, 59, 229, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[113072, 3655, 1465, 44, 346, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[114562, 3703, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[114650, 3707, 2477, 97, 157, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[117142, 3807, 2255, 80, 395, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[119422, 3891, 170, 10, 201, "Input"],
Cell[119595, 3903, 345, 10, 42, "Output"]
}, Open  ]],
Cell[119955, 3916, 239, 8, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[120219, 3928, 114, 2, 43, "Input"],
Cell[120336, 3932, 344, 10, 42, "Output"]
}, Open  ]],
Cell[120695, 3945, 648, 26, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[121368, 3975, 53, 1, 43, "Input"],
Cell[121424, 3978, 345, 10, 42, "Output"]
}, Open  ]],
Cell[121784, 3991, 556, 19, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[122365, 4014, 52, 1, 43, "Input"],
Cell[122420, 4017, 139, 4, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[122596, 4026, 51, 1, 43, "Input"],
Cell[122650, 4029, 206, 6, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[122893, 4040, 54, 1, 43, "Input"],
Cell[122950, 4043, 163, 5, 42, "Output"]
}, Open  ]],
Cell[123128, 4051, 906, 25, 213, "Text",
 Evaluatable->False],
Cell[124037, 4078, 599, 20, 115, "Text",
 Evaluatable->False],
Cell[124639, 4100, 227, 7, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[124891, 4111, 254, 16, 336, "Input"],
Cell[125148, 4129, 281, 7, 42, "Output"]
}, Open  ]],
Cell[125444, 4139, 146, 4, 41, "Text",
 Evaluatable->False],
Cell[125593, 4145, 40, 1, 43, "Input"],
Cell[125636, 4148, 975, 27, 139, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[126648, 4180, 90, 2, 105, "Section",
 Evaluatable->False],
Cell[126741, 4184, 2231, 64, 384, "Text",
 Evaluatable->False],
Cell[128975, 4250, 611, 25, 66, "Text",
 Evaluatable->False],
Cell[129589, 4277, 1875, 61, 237, "Text",
 Evaluatable->False],
Cell[131467, 4340, 175, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[131667, 4349, 102, 6, 111, "Input"],
Cell[131772, 4357, 433, 12, 42, "Output"]
}, Open  ]],
Cell[132220, 4372, 118, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[132363, 4378, 136, 5, 66, "Input"],
Cell[132502, 4385, 17658, 512, 561, "Output"],
Cell[150163, 4899, 17508, 510, 561, "Output"]
}, Open  ]],
Cell[167686, 5412, 314, 9, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[168025, 5425, 92, 2, 43, "Input"],
Cell[168120, 5429, 124, 1, 32, "Print"]
}, Open  ]],
Cell[168259, 5433, 277, 6, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[168561, 5443, 91, 5, 88, "Input"],
Cell[168655, 5450, 432, 12, 42, "Output"]
}, Open  ]],
Cell[169102, 5465, 202, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[169329, 5476, 175, 6, 88, "Input"],
Cell[169507, 5484, 29679, 711, 561, 17594, 511, "CachedBoxData", "BoxData", \
"Output"],
Cell[199189, 6197, 17531, 511, 561, "Output"]
}, Open  ]],
Cell[216735, 6711, 194, 6, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[216954, 6721, 92, 2, 43, "Input"],
Cell[217049, 6725, 33132, 768, 561, 17444, 509, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[250196, 7496, 997, 34, 213, "Text",
 Evaluatable->False],
Cell[251196, 7532, 40, 1, 43, "Input"],
Cell[251239, 7535, 829, 31, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[252093, 7570, 92, 5, 88, "Input"],
Cell[252188, 7577, 415, 12, 42, "Output"]
}, Open  ]],
Cell[252618, 7592, 583, 23, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[253226, 7619, 93, 2, 43, "Input"],
Cell[253322, 7623, 33132, 768, 561, 17444, 509, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[286491, 8396, 86, 2, 43, "Input"],
Cell[286580, 8400, 34224, 785, 561, 17488, 509, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[320819, 9188, 158, 5, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[321002, 9197, 92, 2, 43, "Input"],
Cell[321097, 9201, 17636, 512, 561, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[338770, 9718, 92, 2, 43, "Input"],
Cell[338865, 9722, 30374, 721, 561, 17666, 511, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[369254, 10446, 325, 12, 90, "Text",
 Evaluatable->False],
Cell[369582, 10460, 502, 17, 139, "Text",
 Evaluatable->False],
Cell[370087, 10479, 27, 0, 43, "Input"],
Cell[370117, 10481, 1086, 34, 239, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[371228, 10519, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[371316, 10523, 5198, 169, 346, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[376529, 10695, 1472, 47, 313, "Text",
 Evaluatable->False],
Cell[378004, 10744, 934, 32, 188, "Text",
 Evaluatable->False],
Cell[378941, 10778, 953, 39, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[379919, 10821, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[380007, 10825, 1997, 82, 137, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[382019, 10910, 1031, 37, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[383075, 10951, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[383163, 10955, 4178, 150, 598, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[387356, 11108, 393, 12, 66, "Text",
 Evaluatable->False],
Cell[387752, 11122, 862, 30, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[388639, 11156, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[388727, 11160, 10634, 351, 610, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[399376, 11514, 1166, 36, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[400567, 11554, 78, 4, 66, "Input"],
Cell[400648, 11560, 17483, 511, 561, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[418168, 12076, 78, 4, 66, "Input"],
Cell[418249, 12082, 17447, 510, 561, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[435733, 12597, 78, 4, 66, "Input"],
Cell[435814, 12603, 17471, 511, 561, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[453322, 13119, 143, 5, 66, "Input"],
Cell[453468, 13126, 17274, 508, 561, "Output"]
}, Open  ]],
Cell[470757, 13637, 298, 6, 90, "Text",
 Evaluatable->False],
Cell[471058, 13645, 442, 14, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[471525, 13663, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[471613, 13667, 3793, 134, 379, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[475421, 13804, 1021, 22, 265, "Text",
 Evaluatable->False],
Cell[476445, 13828, 851, 25, 447, "Text",
 Evaluatable->False],
Cell[477299, 13855, 347, 7, 66, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[477683, 13867, 98, 2, 105, "Section",
 Evaluatable->False],
Cell[477784, 13871, 3680, 115, 850, "Text",
 Evaluatable->False],
Cell[481467, 13988, 178, 6, 88, "Input"],
Cell[481648, 13996, 188, 8, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[481861, 14008, 75, 4, 66, "Input"],
Cell[481939, 14014, 168, 4, 42, "Output"],
Cell[482110, 14020, 2413, 70, 561, "Output"]
}, Open  ]],
Cell[484538, 14093, 190, 8, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[484753, 14105, 75, 4, 66, "Input"],
Cell[484831, 14111, 271, 7, 42, "Output"],
Cell[485105, 14120, 7114, 209, 561, "Output"]
}, Open  ]],
Cell[492234, 14332, 1721, 58, 262, "Text",
 Evaluatable->False],
Cell[493958, 14392, 75, 4, 66, "Input"],
Cell[494036, 14398, 392, 13, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[494453, 14415, 54, 1, 43, "Input"],
Cell[494510, 14418, 536, 12, 32, "Print"],
Cell[495049, 14432, 94, 1, 42, "Output"]
}, Open  ]],
Cell[495158, 14436, 651, 24, 164, "Text",
 Evaluatable->False],
Cell[495812, 14462, 44, 1, 43, "Input"],
Cell[495859, 14465, 624, 24, 139, "Text",
 Evaluatable->False],
Cell[496486, 14491, 44, 1, 43, "Input"],
Cell[496533, 14494, 944, 27, 237, "Text",
 Evaluatable->False],
Cell[497480, 14523, 3127, 111, 462, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[500632, 14638, 91, 5, 88, "Input"],
Cell[500726, 14645, 73, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[500836, 14651, 90, 5, 88, "Input"],
Cell[500929, 14658, 73, 1, 42, "Output"]
}, Open  ]],
Cell[501017, 14662, 316, 13, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[501358, 14679, 49, 1, 43, "Input"],
Cell[501410, 14682, 137, 4, 42, "Output"]
}, Open  ]],
Cell[501562, 14689, 201, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[501788, 14700, 58, 1, 43, "Input"],
Cell[501849, 14703, 136, 4, 42, "Output"]
}, Open  ]],
Cell[502000, 14710, 257, 9, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[502282, 14723, 62, 1, 43, "Input"],
Cell[502347, 14726, 182, 5, 42, "Output"]
}, Open  ]],
Cell[502544, 14734, 695, 28, 117, "Text",
 Evaluatable->False],
Cell[503242, 14764, 41, 1, 43, "Input"],
Cell[503286, 14767, 1154, 42, 190, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[504465, 14813, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[504553, 14817, 5205, 193, 567, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[509773, 15013, 1677, 60, 288, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[511475, 15077, 53, 1, 43, "Input"],
Cell[511531, 15080, 92, 2, 42, "Output"]
}, Open  ]],
Cell[511638, 15085, 74, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[511737, 15091, 53, 1, 43, "Input"],
Cell[511793, 15094, 139, 4, 42, "Output"]
}, Open  ]],
Cell[511947, 15101, 181, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[512153, 15112, 46, 1, 43, "Input"],
Cell[512202, 15115, 170, 5, 42, "Output"]
}, Open  ]],
Cell[512387, 15123, 74, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[512486, 15129, 46, 1, 43, "Input"],
Cell[512535, 15132, 149, 4, 42, "Output"]
}, Open  ]],
Cell[512699, 15139, 1547, 55, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[514271, 15198, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[514359, 15202, 3564, 116, 372, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[517938, 15321, 225, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[518188, 15330, 228, 8, 133, "Input"],
Cell[518419, 15340, 367, 10, 42, "Output"]
}, Open  ]],
Cell[518801, 15353, 478, 16, 90, "Text",
 Evaluatable->False],
Cell[519282, 15371, 94, 5, 88, "Input"],
Cell[519379, 15378, 1325, 40, 311, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[520729, 15422, 54, 1, 43, "Input"],
Cell[520786, 15425, 514, 18, 42, "Output"]
}, Open  ]],
Cell[521315, 15446, 605, 22, 90, "Text",
 Evaluatable->False],
Cell[521923, 15470, 101, 5, 88, "Input"],
Cell[522027, 15477, 179, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[522231, 15488, 53, 1, 43, "Input"],
Cell[522287, 15491, 73, 1, 42, "Output"]
}, Open  ]],
Cell[522375, 15495, 332, 13, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[522732, 15512, 50, 1, 43, "Input"],
Cell[522785, 15515, 136, 4, 42, "Output"]
}, Open  ]],
Cell[522936, 15522, 430, 15, 115, "Text",
 Evaluatable->False],
Cell[523369, 15539, 41, 1, 43, "Input"],
Cell[523413, 15542, 1317, 43, 286, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[524755, 15589, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[524843, 15593, 2630, 85, 280, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[527488, 15681, 2104, 63, 384, "Text",
 Evaluatable->False],
Cell[529595, 15746, 715, 22, 128, "Text",
 Evaluatable->False],
Cell[530313, 15770, 462, 15, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[530800, 15789, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[530888, 15793, 7201, 217, 750, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[538104, 16013, 1471, 26, 409, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[539600, 16043, 232, 8, 133, "Input"],
Cell[539835, 16053, 345, 10, 42, "Output"]
}, Open  ]],
Cell[540195, 16066, 106, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[540326, 16072, 51, 1, 43, "Input"],
Cell[540380, 16075, 11700, 342, 561, "Output"]
}, Open  ]],
Cell[552095, 16420, 193, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[552313, 16429, 51, 1, 43, "Input"],
Cell[552367, 16432, 102, 1, 32, "Print"]
}, Open  ]],
Cell[552484, 16436, 324, 12, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[552833, 16452, 52, 1, 43, "Input"],
Cell[552888, 16455, 92, 2, 42, "Output"]
}, Open  ]],
Cell[552995, 16460, 246, 10, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[553266, 16474, 52, 1, 43, "Input"],
Cell[553321, 16477, 125, 3, 42, "Output"]
}, Open  ]],
Cell[553461, 16483, 321, 9, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[553807, 16496, 54, 1, 43, "Input"],
Cell[553864, 16499, 125, 3, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[554026, 16507, 53, 1, 43, "Input"],
Cell[554082, 16510, 120, 3, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[554239, 16518, 48, 1, 43, "Input"],
Cell[554290, 16521, 92, 2, 42, "Output"]
}, Open  ]],
Cell[554397, 16526, 572, 20, 66, "Text",
 Evaluatable->False],
Cell[554972, 16548, 41, 1, 43, "Input"],
Cell[555016, 16551, 2628, 92, 384, "Text",
 Evaluatable->False],
Cell[557647, 16645, 41, 1, 43, "Input"],
Cell[557691, 16648, 746, 20, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[558462, 16672, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[558550, 16676, 22260, 687, 1226, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[580825, 17366, 1232, 41, 213, "Text",
 Evaluatable->False],
Cell[582060, 17409, 70, 4, 66, "Input"],
Cell[582133, 17415, 526, 18, 66, "Text",
 Evaluatable->False],
Cell[582662, 17435, 43, 1, 43, "Input"],
Cell[582708, 17438, 424, 10, 115, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[583169, 17453, 210, 7, 105, "Section",
 Evaluatable->False],
Cell[583382, 17462, 156, 6, 40, "Input"],
Cell[583541, 17470, 168, 3, 41, "Text",
 Evaluatable->False],
Cell[583712, 17475, 277, 9, 133, "Input"],
Cell[583992, 17486, 108, 2, 41, "Text",
 Evaluatable->False],
Cell[584103, 17490, 49, 1, 43, "Input"],
Cell[584155, 17493, 435, 16, 90, "Text",
 Evaluatable->False],
Cell[584593, 17511, 211, 8, 133, "Input"],
Cell[584807, 17521, 822, 31, 115, "Text",
 Evaluatable->False],
Cell[585632, 17554, 44, 1, 43, "Input"],
Cell[585679, 17557, 392, 12, 142, "Text",
 Evaluatable->False],
Cell[586074, 17571, 233, 10, 178, "Input"],
Cell[586310, 17583, 506, 16, 117, "Text",
 Evaluatable->False],
Cell[586819, 17601, 47, 1, 43, "Input"],
Cell[586869, 17604, 281, 8, 90, "Text",
 Evaluatable->False],
Cell[587153, 17614, 46, 1, 43, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
