(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[   1269132,      28200]
NotebookOptionsPosition[   1241187,      27282]
NotebookOutlinePosition[   1241790,      27306]
CellTagsIndexPosition[   1241747,      27303]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Abstract Algebra: An Interactive Approach", "Special2",
 Evaluatable->False,
 CellChangeTimes->{{3.4373889318125*^9, 3.4373889425625*^9}},
 TextAlignment->Center,
 FontFamily->"Arial",
 FontSize->36,
 FontWeight->"Bold",
 FontSlant->"Italic",
 FontColor->RGBColor[1, 0, 1],
 Background->RGBColor[0.500008, 1, 1]],

Cell[CellGroupData[{

Cell["\[Copyright]2009   William Paulsen", "Name",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437388946828125*^9, 3.4373889474375*^9}},
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]],

Cell["\<\
All rights reserved:  These materials can be duplicated or transmitted freely \
for educational use on condition that all copyright statements are retained \
unaltered.  No part of these materials may be sold, by any means or in any \
form, without written permission from the editor.  All software is provided \
\"as is\" without warranty of any kind, either expressed or implied, \
including, but not limited to, the implied warranties of merchantability and \
fitness for a particular purpose.\
\>", "Name",
 Editable->False,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]]
}, Closed]],

Cell["\<\
Chapter 11
Integral Domains and Fields\
\>", "Title",
 CellFrame->True,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 TextAlignment->Center,
 FontFamily->"New York",
 FontSize->24,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell["Initialization", "Special2",
 Evaluatable->False,
 TextAlignment->Center,
 AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["<<c:\\math\\ring.m", "Input",
 PageWidth->Infinity,
 InitializationCell->True,
 CellChangeTimes->{{3.437388950625*^9, 3.437388952609375*^9}},
 AspectRatioFixed->True],

Cell[BoxData["\<\"Initialization Done\"\>"], "Print",
 CellChangeTimes->{3.4373950179665003`*^9, 3.437483381012*^9, 
  3.4375316279375*^9, 3.4417248980733747`*^9, 3.4419704092139997`*^9, 
  3.4427540859145*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polynomial Rings", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In this chapter we will consider some useful ways of forming integral \
domains and fields from smaller rings. These provide us with useful examples \
for experimentation in hopes of finding properties of general integral \
domains and fields.\n\nOne major source of integral domains are the ",
 StyleBox["polynomial rings",
  FontSlant->"Italic"],
 ". We can construct a polynomial ring from any ring, but the polynomial \
rings with the familiar properties are formed either from fields or integral \
domains.\n\nDEFINITION 11.1:\nLet ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be a communitive ring. We define the set of polynomials in ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " over ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 ", to be the set of all expressions of the form\n                            \
             \n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubscriptBox["k", "0"], "+", " ", 
     RowBox[{
      SubscriptBox["k", "1"], " ", "x"}], "+", " ", 
     RowBox[{
      SubscriptBox["k", "2"], " ", 
      SuperscriptBox["x", "2"]}], "+", " ", 
     RowBox[{
      SubscriptBox["k", "3"], " ", 
      SuperscriptBox["x", "3"]}], "+", " ", "\[Ellipsis]", "  "}]}], 
   TraditionalForm]]],
 "\n\nwhere the coefficients ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "n"], TraditionalForm]]],
 "  are elements of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", and only a ",
 StyleBox["finite",
  FontSlant->"Italic"],
 " number of the coefficients are non-zero. If ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "d"], TraditionalForm]]],
 "  is the last non-zero coefficient, then ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " is called the ",
 StyleBox["degree",
  FontSlant->"Italic"],
 " of the polynomial.\n\nNotice that if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "=", "0"}], TraditionalForm]]],
 ", we essentually obtain elements of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Thus, the non-zero elements of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " are the polynomials of degree 0, which are refered to as ",
 StyleBox["constant polynomial",
  FontSlant->"Italic"],
 "s. The degree for the zero polynomial \n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", " ", "+", " ", 
    RowBox[{"0", " ", "x"}], " ", "+", " ", 
    RowBox[{"0", " ", 
     SuperscriptBox["x", "2"]}], "+", " ", 
    RowBox[{"0", " ", 
     SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]"}], 
   TraditionalForm]]],
 "\nis not defined.\n\nBy convention, the terms with zero coefficients are \
omited when writing polynomials. Thus, the second degree polynomial in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", "x", "]"}], TraditionalForm]]],
 "\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"1", " ", "+", " ", 
     RowBox[{"0", " ", "x"}], " ", "+", " ", 
     RowBox[{"3", " ", 
      SuperscriptBox["x", "2"]}], " ", "+", " ", 
     RowBox[{"0", " ", 
      SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]", "  "}]}], 
   TraditionalForm]]],
 "\n\nwould be writen ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "+", " ", 
    RowBox[{"3", " ", 
     RowBox[{
      SuperscriptBox["x", "2"], "."}]}]}], TraditionalForm]]],
 " The one exception to this convention is the zero polynomial, which is \
simply writen as 0.\n        \nWe can define the sum and product of two \
polynomial in the familar way. If\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", " ", "=", " ", 
    RowBox[{
     SubscriptBox["a", "0"], "+", " ", 
     RowBox[{
      SubscriptBox["a", "1"], " ", "x"}], "+", " ", 
     RowBox[{
      SubscriptBox["a", "2"], " ", 
      SuperscriptBox["x", "2"]}], "+", " ", 
     RowBox[{
      SubscriptBox["a", "3"], " ", 
      SuperscriptBox["x", "3"]}], "+", " ", "\[Ellipsis]"}]}], 
   TraditionalForm]]],
 "  and\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{"B", " ", "=", " ", 
    RowBox[{
     SubscriptBox["b", "0"], "+", " ", 
     RowBox[{
      SubscriptBox["b", "1"], " ", "x"}], "+", " ", 
     RowBox[{
      SubscriptBox["b", "2"], " ", 
      SuperscriptBox["x", "2"]}], "+", " ", 
     RowBox[{
      SubscriptBox["b", "3"], " ", 
      SuperscriptBox["x", "3"]}], "+", " ", "\[Ellipsis]"}]}], 
   TraditionalForm]]],
 " \nthen \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"A", "+", "B"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["a", "0"], "+", 
        SubscriptBox["b", "0"]}], ")"}], "+", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["a", "1"], "+", 
         SubscriptBox["b", "1"]}], ")"}], " ", "x"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["a", "2"], "+", 
         SubscriptBox["b", "2"]}], ")"}], " ", 
       SuperscriptBox["x", "2"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["a", "3"], "+", 
         SubscriptBox["b", "3"]}], ")"}], " ", 
       SuperscriptBox["x", "3"]}], "+", "\[Ellipsis]"}]}]}], 
   TraditionalForm]]],
 "\n\nand"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43738899375*^9, 3.437389013046875*^9}, {
  3.437389044078125*^9, 3.437389121984375*^9}}],

Cell[TextData[{
 "               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"A", "\[CenterDot]", "B"}], " ", "=", " ", 
    RowBox[{
     UnderoverscriptBox[
      StyleBox["\[CapitalSigma]",
       FontSize->24], 
      RowBox[{"i", "=", "0"}], "\[Infinity]"], 
     RowBox[{
      UnderoverscriptBox[
       StyleBox["\[CapitalSigma]",
        FontSize->24], 
       RowBox[{"j", "=", "0"}], "\[Infinity]"], "(", 
      RowBox[{
       SubscriptBox["a", "i"], "\[CenterDot]", 
       SubscriptBox["b", "j"]}], ")"}], 
     SuperscriptBox["x", Cell[TextData[Cell[BoxData[
       FormBox[
        RowBox[{"(", 
         RowBox[{"i", "+", "j"}], ")"}], TraditionalForm]]]]]]}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437389131359375*^9, 3.437389293875*^9}, {
  3.43738961253125*^9, 3.43738970296875*^9}, {3.43738974359375*^9, 
  3.4373898423125*^9}}],

Cell[TextData[{
 "Although this looks like a double infinite sum, bear in mind that only a \
finite number of the terms will be non-zero. Thus, we do not need to be \
concerned about such issues as \"convergence.\" If fact, this product could \
be written as\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      RowBox[{
       RowBox[{"A", " ", "\[CenterDot]", " ", "B"}], " ", "=", "  ", 
       RowBox[{
        SubscriptBox["a", "0"], "\[CenterDot]", 
        SubscriptBox["b", "0"]}]}],
      TraditionalForm], "          ", "\n", "\t                 ", 
     FormBox[
      RowBox[{
       RowBox[{"+", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["a", "0"], "\[CenterDot]", " ", 
           SubscriptBox["b", "1"]}], " ", "+", " ", 
          RowBox[{
           SubscriptBox["a", "1"], "\[CenterDot]", 
           SubscriptBox["b", "0"]}]}], ")"}]}], " ", "x", " "}],
      TraditionalForm], "      ", "\n", "\t\t ", 
     FormBox[
      RowBox[{"            ", 
       RowBox[{
        RowBox[{"+", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["a", "0"], "\[CenterDot]", " ", 
            SubscriptBox["b", "2"]}], " ", "+", " ", 
           RowBox[{
            SubscriptBox["a", "1"], "\[CenterDot]", " ", 
            SubscriptBox["b", "1"]}], " ", "+", " ", 
           RowBox[{
            SubscriptBox["a", "2"], "\[CenterDot]", " ", 
            SubscriptBox["b", "0"]}]}], ")"}]}], " ", 
        SuperscriptBox["x", 
         RowBox[{"2", " "}]]}]}],
      TraditionalForm], "        ", "\n", "\t                  ", 
     FormBox[
      RowBox[{
       RowBox[{
        RowBox[{"+", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            SubscriptBox["a", "0"], "\[CenterDot]", 
            SubscriptBox["b", "3"]}], " ", "+", " ", 
           RowBox[{
            SubscriptBox["a", "1"], "\[CenterDot]", " ", 
            SubscriptBox["b", "2"]}], "+", " ", 
           RowBox[{
            SubscriptBox["a", "2"], "\[CenterDot]", " ", 
            SubscriptBox["b", "1"]}], " ", "+", " ", 
           RowBox[{
            SubscriptBox["a", "3"], "\[CenterDot]", " ", 
            SubscriptBox["b", "0"]}]}], ")"}]}], " ", 
        SuperscriptBox["x", "3"]}], " ", "+", " ", 
       RowBox[{"\[Ellipsis]", "."}]}],
      TraditionalForm]}]}], TraditionalForm]]],
 "\n\nso each coefficient is determined by a finite sum. \n\nBefore we prove \
that the set of polynomials forms a ring, we need the following lemma:\n\n\
LEMMA 11.1\nLet ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " be two non-zero polynomials in ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " over ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " of degree ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " respectively, where ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is a field or an integral domain. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "\[CenterDot]", "B"}], TraditionalForm]]],
 " is a polynomial of degree ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"m", "+", "n"}], ","}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", " ", "+", " ", "B"}], TraditionalForm]]],
 " is a polynomial of degree no greater than the larger of ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437389858625*^9, 3.43738987546875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " be a polynomial of degree ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ",\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", " ", "=", " ", 
    RowBox[{
     SubscriptBox["a", "0"], "  ", "+", " ", 
     RowBox[{
      SubscriptBox["a", "1"], " ", "x"}], " ", "+", " ", 
     RowBox[{
      SubscriptBox["a", "2"], " ", 
      SuperscriptBox["x", "2"]}], " ", "+", " ", 
     RowBox[{
      SubscriptBox["a", "3"], " ", 
      SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]", " ", "+", " ",
      
     RowBox[{
      SubscriptBox["a", "m"], " ", 
      SuperscriptBox["x", "m"]}]}]}], TraditionalForm]]],
 "\n           \nand ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " be a polynomial of degree ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ",  \n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"B", " ", "=", " ", 
     RowBox[{
      SubscriptBox["b", "0"], "+", " ", 
      RowBox[{
       SubscriptBox["b", "1"], "x"}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["b", "2"], " ", 
       SuperscriptBox["x", "2"]}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["b", "3"], " ", 
       SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]", " ", "+", 
      " ", 
      RowBox[{
       SubscriptBox["b", "n"], " ", 
       RowBox[{
        SuperscriptBox["x", "n"], "."}]}]}]}]}], TraditionalForm]]],
 "\n          \nHere, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "m"], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["b", "n"], TraditionalForm]]],
 "  are non-zero elements of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". The product is determined by\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"A", "\[CenterDot]", "B"}], " ", "=", " ", 
    RowBox[{
     StyleBox[
      UnderoverscriptBox[
       SubscriptBox[
        StyleBox["\[Sum]",
         FontSize->24], " "], 
       RowBox[{"i", "=", "0"}], "\[Infinity]"],
      FontSize->16], 
     RowBox[{
      StyleBox[
       UnderoverscriptBox[
        SubscriptBox[
         StyleBox["\[Sum]",
          FontSize->24], " "], 
        RowBox[{"j", "=", "0"}], "\[Infinity]"],
       FontSize->16], 
      RowBox[{
       RowBox[{
        SubscriptBox["a", "i"], "\[CenterDot]", 
        SubscriptBox["b", "j"]}], 
       SuperscriptBox["x", 
        RowBox[{"i", "+", "j"}]]}]}]}]}], TraditionalForm]]],
 ".\n               \nNote that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "i"], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["b", "j"], TraditionalForm]]],
 "  are zero for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", " ", ">", " ", "m"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"j", " ", ">", " ", "n"}], TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"i", "+", "j"}], " ", ">", " ", 
    RowBox[{"m", "+", "n"}]}], TraditionalForm]]],
 ", either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", " ", ">", " ", "m"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"j", " ", ">", " ", "n"}], TraditionalForm]]],
 ", and in either case ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["a", "i"], " ", "\[CenterDot]", " ", 
     SubscriptBox["b", "j"]}], "  ", "=", " ", "0"}], TraditionalForm]]],
 ". Thus, there are no non-zero\nterms in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", " ", "\[CenterDot]", " ", "B"}], TraditionalForm]]],
 " with coefficients larger than ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "+", "n"}], TraditionalForm]]],
 ". However, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"i", "+", "j"}], " ", "=", " ", 
    RowBox[{"m", "+", "n"}]}], TraditionalForm]]],
 ", the\nonly non-zero term would be the one coming from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", " ", "=", " ", "m"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"j", " ", "=", " ", "n"}], TraditionalForm]]],
 ", giving\n\n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["a", "m"], "\[CenterDot]", 
     SubscriptBox["b", "n"]}], "  ", 
    SuperscriptBox["x", 
     RowBox[{
      RowBox[{"m", "+", "n"}], " "}]]}], TraditionalForm]]],
 ".\n\nSince there are no zero divisors in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "m"], "\[CenterDot]", 
    SubscriptBox["b", "n"]}], TraditionalForm]]],
 "  is non-zero, so\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "\[CenterDot]", "B"}], TraditionalForm]]],
 " is a polynomial of degree ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "+", "n"}], TraditionalForm]]],
 ".\n\nNext we turn our attention to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "+", "B"}], TraditionalForm]]],
 ". We may assume without loss of generality\nthat ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " is no more than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Then the sum of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " can be expressed as\n                                            \n        \
     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      RowBox[{" ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["a", "0"], "+", " ", 
          SubscriptBox["b", "0"]}], ")"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["a", "1"], "+", " ", 
           SubscriptBox["b", "1"]}], ")"}], " ", "x"}], " ", "+", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["a", "2"], " ", "+", " ", 
           SubscriptBox["b", "2"]}], ")"}], " ", 
         SuperscriptBox["x", "2"]}], " ", "+", " ", "\[Ellipsis]", "  "}]}],
      TraditionalForm], "\n", "\t\t          ", 
     FormBox[
      RowBox[{"      ", 
       RowBox[{
        RowBox[{
         RowBox[{"+", " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["a", "m"], "+", " ", 
            SubscriptBox["b", "m"]}], ")"}]}], " ", 
         SuperscriptBox["x", "m"]}], "  ", "+", " ", 
        RowBox[{
         SubscriptBox["b", 
          RowBox[{"m", "+", "1"}]], " ", 
         SuperscriptBox["x", 
          RowBox[{"m", "+", "1"}]]}], "   ", "+", " ", "\[Ellipsis]", " ", 
        "+", " ", 
        RowBox[{
         SubscriptBox["b", "n"], " ", 
         RowBox[{
          SuperscriptBox["x", "n"], "."}]}]}]}],
      TraditionalForm]}]}], TraditionalForm]]],
 "\n\nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "<", " ", "n"}], TraditionalForm]]],
 ", this clearly is a polynomial with degree ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Even if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "=", " ", "n"}], TraditionalForm]]],
 ", this\nstill gives a polynomial whose degree cannot be more than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437389888890625*^9, 3.43738989296875*^9}, {
  3.437389927484375*^9, 3.437389940703125*^9}, {3.437390164703125*^9, 
  3.437390228671875*^9}}]
}, Open  ]],

Cell[TextData[{
 "This lemma verifies that the both sum and product of two polynomials are \
polynomials. The remaining ring properties are not difficult to verify.\n\n\
PROPOSITION 11.1\nLet ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be an integral domain or a field. Then the set of polynomials in ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " over ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " forms an integral domain."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437389940890625*^9, 3.437389940890625*^9}, {
  3.4373946194977503`*^9, 3.4373946196383753`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We have seen that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 " is closed under addition and multiplication. By the commutativity of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", addition and multiplication are obviously commutative. It is also clear \
that the zero polynomial acts as the additive identity in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 ". Also, the additive inverse of \n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"A", " ", "=", " ", 
     RowBox[{
      SubscriptBox["a", "0"], " ", "+", " ", 
      RowBox[{
       SubscriptBox["a", "1"], " ", "x"}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["a", "2"], "  ", 
       SuperscriptBox["x", "2"]}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["a", "3"], " ", 
       SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]"}]}]}], 
   TraditionalForm]]],
 "\n\nis given by\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "A"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", 
         SubscriptBox["a", "0"]}], ")"}], " ", "+", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"-", 
          SubscriptBox["a", "1"]}], ")"}], " ", "x"}], " ", "+", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"-", 
          SubscriptBox["a", "2"]}], ")"}], " ", 
        SuperscriptBox["x", "2"]}], " ", "+", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"-", 
          SubscriptBox["a", "3"]}], ")"}], " ", 
        SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]"}]}], ","}]}],
    TraditionalForm]]],
 "\n\nsince the sum of these two polynomials is\n  \n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"A", " ", "+", " ", 
      RowBox[{"(", 
       RowBox[{"-", "A"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"0", " ", "+", " ", 
       RowBox[{"0", " ", "x"}], " ", "+", " ", 
       RowBox[{"0", " ", 
        SuperscriptBox["x", "2"]}], "+", " ", 
       RowBox[{"0", " ", 
        SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]"}], " ", "=", 
      " ", "0."}]}]}], TraditionalForm]]],
 "\n\nThe polynomial with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["b", "0"], "=", " ", "1"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["b", 
     RowBox[{"j", " "}]], "=", " ", "0"}], TraditionalForm]]],
 " for all positive ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ",\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"I", " ", "=", " ", 
      RowBox[{"1", " ", "+", " ", 
       RowBox[{"0", " ", "x"}], " ", "+", " ", 
       RowBox[{"0", " ", 
        SuperscriptBox["x", "2"]}], "+", " ", 
       RowBox[{"0", " ", 
        SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]"}]}], ","}]}],
    TraditionalForm]]],
 "\n      \nacts as the multiplicative identity, since  \n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"I", "\[CenterDot]", "A"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"A", "\[CenterDot]", "I"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       StyleBox[
        UnderoverscriptBox[
         SubscriptBox[
          StyleBox["\[Sum]",
           FontSize->24], " "], 
         RowBox[{"i", "=", "0"}], "\[Infinity]"],
        FontSize->16], 
       RowBox[{
        StyleBox[
         UnderoverscriptBox[
          SubscriptBox[
           StyleBox["\[Sum]",
            FontSize->24], " "], 
          RowBox[{"j", "=", "0"}], "\[Infinity]"],
         FontSize->16], 
        RowBox[{
         RowBox[{
          SubscriptBox["a", "i"], "\[CenterDot]", 
          SubscriptBox["b", "j"]}], 
         SuperscriptBox["x", 
          RowBox[{"i", "+", "j"}]]}]}]}], " ", "=", " "}]}]}], 
   TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox[
      UnderoverscriptBox[
       SubscriptBox[
        StyleBox["\[Sum]",
         FontSize->24], " "], 
       RowBox[{"i", "=", "0"}], "\[Infinity]"],
      FontSize->16], 
     RowBox[{
      RowBox[{
       SubscriptBox["a", "i"], "\[CenterDot]", "1"}], " ", 
      SuperscriptBox["x", "i"]}]}], " ", "=", " ", "A"}], TraditionalForm]]],
 ".\n\nTo check associativity of addition and multiplication, we need three \
polynomials\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"   ", 
    RowBox[{
     FormBox[
      RowBox[{" ", 
       RowBox[{
        RowBox[{"A", " ", "=", " ", 
         RowBox[{
          SubscriptBox["a", "0"], " ", "+", " ", 
          RowBox[{
           SubscriptBox["a", "1"], " ", "x"}], " ", "+", " ", 
          RowBox[{
           SubscriptBox["a", "2"], "  ", 
           SuperscriptBox["x", "2"]}], " ", "+", " ", 
          RowBox[{
           SubscriptBox["a", "3"], " ", 
           SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]"}]}], 
        ","}]}],
      TraditionalForm], "        ", "\n", "\t", 
     RowBox[{
      FormBox[
       RowBox[{
        RowBox[{"B", " ", "=", " ", 
         RowBox[{
          SubscriptBox["b", "0"], " ", "+", " ", 
          RowBox[{
           SubscriptBox["b", "1"], " ", "x"}], " ", "+", " ", 
          RowBox[{
           SubscriptBox["b", "2"], " ", 
           SuperscriptBox["x", "2"]}], " ", "+", " ", 
          RowBox[{
           SubscriptBox["b", "3"], " ", 
           SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]"}]}], 
        ","}],
       TraditionalForm], "     ", "and"}], "\n", "\t", 
     FormBox[
      RowBox[{"C", " ", "=", " ", 
       RowBox[{
        SubscriptBox["c", "0"], " ", "+", " ", 
        RowBox[{
         SubscriptBox["c", "1"], "x"}], " ", "+", " ", 
        RowBox[{
         SubscriptBox["c", "2"], " ", 
         SuperscriptBox["x", "2"]}], " ", "+", " ", 
        RowBox[{
         SubscriptBox["c", "3"], " ", 
         SuperscriptBox["x", "3"]}], " ", "+", " ", 
        RowBox[{"\[Ellipsis]", "."}]}]}],
      TraditionalForm]}]}], TraditionalForm]]],
 "\n\nThen ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"A", " ", "+", " ", "B"}], ")"}], " ", "+", " ", "C"}], " ", 
     "="}], " "}], TraditionalForm]]],
 "\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{" ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["a", "0"], "+", " ", 
          SubscriptBox["b", "0"]}], ")"}], "+", 
        SubscriptBox["c", "0"], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["a", "1"], "+", 
             SubscriptBox["b", "1"]}], ")"}], "+", 
           SubscriptBox["c", "1"]}], ")"}], " ", "x"}], " ", "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["a", "2"], "+", " ", 
             SubscriptBox["b", "2"]}], ")"}], "+", 
           SubscriptBox["c", "2"]}], ")"}], " ", 
         SuperscriptBox["x", "2"]}], "+", " ", "\[Ellipsis]"}]}],
      TraditionalForm], " ", 
     FormBox[
      RowBox[{"\n", 
       RowBox[{
        RowBox[{"=", 
         RowBox[{
          SubscriptBox["a", "0"], "+", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["b", "0"], "+", 
            SubscriptBox["c", "0"]}], ")"}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["a", "1"], "+", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["b", "1"], "+", 
               SubscriptBox["c", "1"]}], ")"}]}], ")"}], "x"}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["a", "2"], "+", 
             RowBox[{"(", 
              RowBox[{
               SubscriptBox["b", "2"], "+", 
               SubscriptBox["c", "2"]}], ")"}]}], ")"}], 
           SuperscriptBox["x", "2"]}], "+", "\[Ellipsis]"}]}], "\n", 
        "\t\t "}]}],
      TraditionalForm]}], "=", " ", 
    RowBox[{"A", " ", "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"B", " ", "+", " ", "C"}], ")"}], "."}]}]}], 
   TraditionalForm]]],
 "\n\nAlso, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"A", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"B", "\[CenterDot]", "C"}], ")"}]}], " ", "=", " ", 
    FormBox[
     RowBox[{"A", "\[CenterDot]", 
      RowBox[{"(", " ", 
       RowBox[{
        StyleBox[
         UnderoverscriptBox[
          SubscriptBox[
           StyleBox["\[Sum]",
            FontSize->24], " "], 
          RowBox[{"j", "=", "0"}], "\[Infinity]"],
         FontSize->16], 
        RowBox[{
         StyleBox[
          UnderoverscriptBox[
           SubscriptBox[
            StyleBox["\[Sum]",
             FontSize->24], " "], 
           RowBox[{"k", "=", "0"}], "\[Infinity]"],
          FontSize->16], 
         RowBox[{
          RowBox[{
           SubscriptBox["b", "j"], "\[CenterDot]", 
           SubscriptBox["c", "k"]}], " ", 
          SuperscriptBox["x", 
           RowBox[{"j", "+", "k"}]]}]}]}], " ", ")"}]}],
     TraditionalForm]}], TraditionalForm]]],
 "\n             =",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox[
     UnderoverscriptBox[
      SubscriptBox[
       StyleBox["\[Sum]",
        FontSize->24], " "], 
      RowBox[{"i", "=", "0"}], "\[Infinity]"],
     FontSize->16], 
    RowBox[{
     StyleBox[
      UnderoverscriptBox[
       SubscriptBox[
        StyleBox["\[Sum]",
         FontSize->24], " "], 
       RowBox[{"j", "=", "0"}], "\[Infinity]"],
      FontSize->16], 
     RowBox[{
      StyleBox[
       UnderoverscriptBox[
        SubscriptBox[
         StyleBox["\[Sum]",
          FontSize->24], " "], 
        RowBox[{"k", "=", "0"}], "\[Infinity]"],
       FontSize->16], 
      RowBox[{
       RowBox[{
        SubscriptBox["a", "i"], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["b", "j"], "\[CenterDot]", 
          SubscriptBox["c", "k"]}], ")"}]}], " ", 
       SuperscriptBox["x", 
        RowBox[{"i", "+", "j", "+", "k"}]]}]}]}]}], TraditionalForm]]],
 "\n  \n        = ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox[
      UnderoverscriptBox[
       SubscriptBox[
        StyleBox["\[Sum]",
         FontSize->24], " "], 
       RowBox[{"i", "=", "0"}], "\[Infinity]"],
      FontSize->16], 
     RowBox[{
      StyleBox[
       UnderoverscriptBox[
        SubscriptBox[
         StyleBox["\[Sum]",
          FontSize->24], " "], 
        RowBox[{"j", "=", "0"}], "\[Infinity]"],
       FontSize->16], 
      RowBox[{
       StyleBox[
        UnderoverscriptBox[
         SubscriptBox[
          StyleBox["\[Sum]",
           FontSize->24], " "], 
         RowBox[{"k", "=", "0"}], "\[Infinity]"],
        FontSize->16], 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["a", "i"], "\[CenterDot]", 
           SubscriptBox["b", "j"]}], ")"}], "\[CenterDot]", 
         SubscriptBox["c", "k"]}], "  ", 
        SuperscriptBox["x", 
         RowBox[{"i", "+", "j", "+", "k"}]]}]}]}]}], "  ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"A", "\[CenterDot]", "B"}], ")"}], "\[CenterDot]", "C"}]}], 
   TraditionalForm]]],
 ".\nThe two distributive laws are also easy to verify using the summation \
notation.\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"A", "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"B", " ", "+", " ", "C"}], ")"}]}], " ", "="}], " "}], 
   TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"A", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       StyleBox[
        UnderoverscriptBox[
         SubscriptBox[
          StyleBox["\[Sum]",
           FontSize->24], " "], 
         RowBox[{"j", "=", "0"}], "\[Infinity]"],
        FontSize->16], 
       StyleBox[" ",
        FontSize->16], 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["b", "j"], "+", 
          SubscriptBox["c", "j"]}], ")"}], " ", 
        SuperscriptBox["x", "j"]}]}], ")"}]}], " ", "=", " "}], 
   TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox[
     UnderoverscriptBox[
      SubscriptBox[
       StyleBox["\[Sum]",
        FontSize->24], " "], 
      RowBox[{"i", "=", "0"}], "\[Infinity]"],
     FontSize->16], 
    RowBox[{
     StyleBox[
      UnderoverscriptBox[
       SubscriptBox[
        StyleBox["\[Sum]",
         FontSize->24], " "], 
       RowBox[{"j", "=", "0"}], "\[Infinity]"],
      FontSize->16], 
     RowBox[{
      RowBox[{
       SubscriptBox["a", "i"], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["b", "j"], "+", 
         SubscriptBox["c", "j"]}], ")"}]}], " ", 
      SuperscriptBox["x", 
       RowBox[{"i", "+", "j"}]]}]}]}], TraditionalForm]]],
 "\n  \n        = ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox[
     UnderoverscriptBox[
      SubscriptBox[
       StyleBox["\[Sum]",
        FontSize->24], " "], 
      RowBox[{"i", "=", "0"}], "\[Infinity]"],
     FontSize->16], 
    RowBox[{
     StyleBox[
      UnderoverscriptBox[
       SubscriptBox[
        StyleBox["\[Sum]",
         FontSize->24], " "], 
       RowBox[{"j", "=", "0"}], "\[Infinity]"],
      FontSize->16], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["a", "i"], "\[CenterDot]", 
         SubscriptBox["b", "j"]}], " ", "+", " ", 
        RowBox[{
         SubscriptBox["a", "i"], "\[CenterDot]", 
         SubscriptBox["c", "j"]}]}], ")"}], "  ", 
      SuperscriptBox["x", 
       RowBox[{"i", "+", "j"}]], "  "}]}]}], TraditionalForm]]],
 "\n        \n     = ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox[
     UnderoverscriptBox[
      SubscriptBox[
       StyleBox["\[Sum]",
        FontSize->24], " "], 
      RowBox[{"i", "=", "0"}], "\[Infinity]"],
     FontSize->16], 
    RowBox[{
     StyleBox[
      UnderoverscriptBox[
       SubscriptBox[
        StyleBox["\[Sum]",
         FontSize->24], " "], 
       RowBox[{"j", "=", "0"}], "\[Infinity]"],
      FontSize->16], 
     RowBox[{
      RowBox[{
       SubscriptBox["a", "i"], "\[CenterDot]", 
       SubscriptBox["b", "j"]}], "   ", 
      SuperscriptBox["x", 
       RowBox[{"i", "+", "j"}]], "  "}]}]}], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox[
     UnderoverscriptBox[
      SubscriptBox[
       StyleBox["\[Sum]",
        FontSize->24], " "], 
      RowBox[{"i", "=", "0"}], "\[Infinity]"],
     FontSize->16], 
    RowBox[{
     StyleBox[
      UnderoverscriptBox[
       SubscriptBox[
        StyleBox["\[Sum]",
         FontSize->24], " "], 
       RowBox[{"j", "=", "0"}], "\[Infinity]"],
      FontSize->16], " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["a", "i"], "\[CenterDot]", 
       SubscriptBox["c", "j"]}], "  ", 
      SuperscriptBox["x", 
       RowBox[{"i", "+", "j"}]], "  "}]}]}], TraditionalForm]]],
 "\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"=", " ", 
     RowBox[{"A", "\[CenterDot]", "B"}]}], " ", "+", " ", 
    RowBox[{"A", "\[CenterDot]", "C"}]}], TraditionalForm]]],
 ". We can use the fact that multiplication is commutative to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"A", "+", "B"}], ")"}], "\[CenterDot]", "C"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"A", "\[CenterDot]", "C"}], " ", "+", " ", 
     RowBox[{"B", "\[CenterDot]", 
      RowBox[{"C", "."}]}]}]}], TraditionalForm]]],
 " Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 " is a commutative ring with identity.\n\nNext, let us show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 " has no zero divisors. Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"A", "\[CenterDot]", "B"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 ", with both ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " being non-zero polynomials. Say that ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " has degree ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " has degree ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Then by lemma 11.1 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "\[CenterDot]", "B"}], TraditionalForm]]],
 " has degree ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"m", "+", "n"}], ","}], TraditionalForm]]],
 " which is impossible if either ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " were positive. But if ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " are constant polynomials, then",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", "0"], "\[CenterDot]", 
      SubscriptBox["b", "0"]}], " ", "=", " ", "0"}]}], TraditionalForm]]],
 ", which would indicate that either ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "0"], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["b", "0"], TraditionalForm]]],
 " is 0,\nsince ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " has no zero divisors. Thus, either ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " would have to be 0, so we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 " has no zero divisors.\n\nFinally, let us show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 " is not a field, by showing that the polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"1", " ", "+", " ", "x"}], ")"}], TraditionalForm]]],
 " is not invertible. Suppose that there was a polynomial ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"A", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"1", " ", "+", " ", "x"}], ")"}]}], " ", "=", " ", "1."}], 
   TraditionalForm]]],
 " Then ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " is not 0, so suppose ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " has degree ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ". Then by lemma 11.1, we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"m", " ", "+", " ", "1"}], " ", "=", " ", "0"}], ","}], 
   TraditionalForm]]],
 " telling us ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "=", " ", 
    RowBox[{"-", "1"}]}], TraditionalForm]]],
 ", which is impossible. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"1", " ", "+", " ", "x"}], ")"}], TraditionalForm]]],
 " has no inverse in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 ", and therefore ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 " is an integral domain."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43738994228125*^9, 3.437389964765625*^9}, {
  3.4373946534352503`*^9, 3.4373946934040003`*^9}, {3.4373947287946253`*^9, 
  3.4373948677946253`*^9}, {3.4373949429665003`*^9, 3.4373949430915003`*^9}}]
}, Open  ]],

Cell[TextData[{
 "Although this proposition holds for polynomials defined over a integral \
domain, there is no reason why we cannot have ",
 StyleBox["Mathematica ",
  FontSlant->"Italic"],
 "work with polynomials defined over any commutative ring. However, we will \
discover that the familar properties of polynomials will radically change!\n\n\
Let us consider the commutative ring of order 8 that we worked with in \
Chapter 9."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing
Define[4 a, 0]; Define[2 b, 0]
Define[a.a, a]; Define[b.b, b]
Define[a.b, 0]; Define[b.a, 0]
R = Ring[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4373949699508753`*^9, 3.4373949830602503`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4373950180446253`*^9, 3.442753153055125*^9, 
  3.442753251727*^9}]
}, Open  ]],

Cell[TextData[{
 "We form a polynomial ring over ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " by defining a new symbol ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", and declaring that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " commutes with every element in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Thus, we have the two definitions:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437389965125*^9, 3.437389965625*^9}}],

Cell["\<\
x =.
Define[x.a, a.x]
Define[x.b, b.x]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4373899663125*^9, 3.437389968234375*^9}, {
   3.4427532152895*^9, 3.442753217711375*^9}, 3.442753259492625*^9}],

Cell[TextData[{
 "This forces the coefficients to appear in front of the power of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". We can now enter in a typical polynomial."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437389968796875*^9, 3.437389968796875*^9}}],

Cell[CellGroupData[{

Cell["Y = a.x + b", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.437389969359375*^9}],

Cell[BoxData[
 RowBox[{"b", "+", 
  InterpretationBox["\<\"\\!\\(a\\) \[CenterDot] \\!\\(x\\)\"\>",
   StringForm["`` \[CenterDot] ``", $CellContext`a, $CellContext`x],
   Editable->False]}]], "Output",
 CellChangeTimes->{3.4373950225758753`*^9, 3.442753276430125*^9}]
}, Open  ]],

Cell["\<\
We can experiment with powers of this first degree polynomial.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Y^2", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"b", "+", 
  InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(x\\\\)\\\"\\) \[CenterDot] \\!\\(x\\)\"\>",
   StringForm["`` \[CenterDot] ``", 
    Dot[$CellContext`a, $CellContext`x], $CellContext`x],
   Editable->False]}]], "Output",
 CellChangeTimes->{3.4373950253571253`*^9, 3.44275328132075*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Y^3", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"b", "+", 
  InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(a\\\\\\\
\\) \\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(x\\\\\\\\)\\\\\\\"\\\\) \
\\\\[CenterDot] \\\\!\\\\(x\\\\)\\\"\\) \[CenterDot] \\!\\(x\\)\"\>",
   StringForm["`` \[CenterDot] ``", 
    Dot[
     Dot[$CellContext`a, $CellContext`x], $CellContext`x], $CellContext`x],
   Editable->False]}]], "Output",
 CellChangeTimes->{3.4373950273727503`*^9, 3.4427532831645*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Y^4", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"b", "+", 
  InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(\\\\\\\"\\\\\\\\!\\\\\\\\(\\\\\\\
\\\\\\\\\"\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(a\\\\\\\\\\\\\\\\) \\\\\\\\\\\\\\\
\\[CenterDot] \
\\\\\\\\\\\\\\\\!\\\\\\\\\\\\\\\\(x\\\\\\\\\\\\\\\\)\\\\\\\\\\\\\\\"\\\\\\\\) \
\\\\\\\\[CenterDot] \\\\\\\\!\\\\\\\\(x\\\\\\\\)\\\\\\\"\\\\) \\\\[CenterDot] \
\\\\!\\\\(x\\\\)\\\"\\) \[CenterDot] \\!\\(x\\)\"\>",
   StringForm["`` \[CenterDot] ``", 
    Dot[
     Dot[
      Dot[$CellContext`a, $CellContext`x], $CellContext`x], $CellContext`x], \
$CellContext`x],
   Editable->False]}]], "Output",
 CellChangeTimes->{3.4373950288883753`*^9, 3.442753284211375*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that powers of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " are printed as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "x", "\[CenterDot]", "x", "\[CenterDot]", "\[CenterEllipsis]", 
    "\[CenterDot]", 
    RowBox[{"x", "."}]}], TraditionalForm]]],
 " This is one of the drawbacks of using the dot for multiplication in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". We will soon circumvent this problem.\n\nIt seems as though",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", ".", "x"}], " ", "+", " ", "b"}], ")"}]}], 
   TraditionalForm]]],
 " satisfies the simplistic property:\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"u", "+", " ", "v"}], ")"}], "n"], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["u", "n"], "+", "  ", 
      SuperscriptBox["v", 
       RowBox[{"n", " "}]]}]}]}], TraditionalForm]]],
 ".\n\nDoes this hold for all polynomials in this ring?\n\nEXPERIMENT:\nTry \
to find a first degree polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"u", ".", "x"}], " ", "+", " ", "v"}], ")"}], ","}], 
   TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", which does not satisfy the above property for some value of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437389969984375*^9, 3.437389974*^9}, {
  3.4373950544508753`*^9, 3.4373950549665003`*^9}}],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell["\<\
In the process of doing this experiment, you may have noticed a property that \
is even more bizarre: Sometimes the square of a first degree polynomial is \
not a second degree polynomial! In fact, consider\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["(2 a.x + a + b)^2  ", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.437389974609375*^9}],

Cell[BoxData[
 RowBox[{"a", "+", "b"}]], "Output",
 CellChangeTimes->{3.4373950769352503`*^9, 3.442753293148875*^9}]
}, Open  ]],

Cell[TextData[{
 "which yields the identity element in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", "a", " ", "x"}], " ", "+", "a", "+", "b"}], ")"}], 
   TraditionalForm]]],
 " is its own multiplicative inverse! To further complicate matters, notice \
that polynomials may be \"factored\" in more than one way. Note that "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43738997534375*^9, 3.43738997534375*^9}, {
  3.4373950957946253`*^9, 3.4373950986852503`*^9}}],

Cell[CellGroupData[{

Cell["(b + 2 a.x) . (b + a.x)    ", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43738997578125*^9, 3.437389976171875*^9}}],

Cell[BoxData[
 RowBox[{"b", "+", 
  RowBox[{"2", " ", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(x\\\\)\\\"\\) \[CenterDot] \\!\\(x\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`x], $CellContext`x],
    Editable->False]}]}]], "Output",
 CellChangeTimes->{3.4373951076071253`*^9, 3.442753295477*^9}]
}, Open  ]],

Cell["and", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["(b + 2 a.x) . (2 a + b + a.x)     ", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437389976625*^9, 3.437389977*^9}}],

Cell[BoxData[
 RowBox[{"b", "+", 
  RowBox[{"2", " ", 
   InterpretationBox["\<\"\\!\\(\\\"\\\\!\\\\(a\\\\) \\\\[CenterDot] \
\\\\!\\\\(x\\\\)\\\"\\) \[CenterDot] \\!\\(x\\)\"\>",
    StringForm["`` \[CenterDot] ``", 
     Dot[$CellContext`a, $CellContext`x], $CellContext`x],
    Editable->False]}]}]], "Output",
 CellChangeTimes->{3.4373951101852503`*^9, 3.442753297555125*^9}]
}, Open  ]],

Cell[TextData[{
 "yield the same quadratic polynomial.\n\nEXPERIMENT:\nSee if you can find a \
third polynomial whose product with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"b", " ", "+", " ", 
     RowBox[{"2", " ", 
      RowBox[{"a", ".", "x"}]}]}], ")"}], TraditionalForm]]],
 " is the same as the two above. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437389977421875*^9, 3.437389986078125*^9}}],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Such properties of polynomials are enough to drive any begining algebra \
student crazy. Because of the bizarre properties of polynomials over general \
rings, we mainly focus out attention to polynomial rings ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is an integral domain or field. We will discover in Chapter 13 the exact \
conditions for which every polynomial in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 " will have a unique factorization. \n\nAs we work with polynomials in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", it would be better if we could use the standard multiplication notation \
instead of useing the dot. This would allow polynomials to appear in the \
familiar format:\n                             \n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "+", " ", 
    RowBox[{"a", " ", "x"}], "+", " ", 
    RowBox[{"b", " ", 
     RowBox[{
      SuperscriptBox["x", "2"], "."}]}]}], TraditionalForm]]],
 "\n\nAlthough we cannot get ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to use this easier format for polynomials over general rings, there is a \
property of integral domains and fields that lets us enter these rings into \
",
 StyleBox["Mathematica ",
  FontSlant->"Italic"],
 "another way. To introduce this special property, we first need a \
definition.\n\nDEFINITION 11.2:\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a ring. We define the ",
 StyleBox["characteristic",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " to be the smallest positive number ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " for all elements ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". If no such positive number exists, we say the ring has ",
 StyleBox["characteristic 0. ",
  FontSlant->"Italic"],
 "\n\nFor integral domains or fields, the characteristic plays an extremely \
important role, as the next proposition illustrates.\n\nPROPOSITION 11.2\nLet \
",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a non-trivial ring with no zero-divisors. If the characteristic is 0, \
then for ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " an integer and ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " a non-zero element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " only if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "0"}], TraditionalForm]]],
 ". If the characteristic is positive then it is a prime number ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", and for non-zero ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " if, and only if, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373951419977503`*^9, 3.4373951953571253`*^9}, {
  3.4417246921827497`*^9, 3.4417247869171247`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " for some non-zero ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Then for any other non-zero element ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", \n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"      ", 
    RowBox[{"0", " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"n", "\[CenterDot]", "x"}], ")"}], "\[CenterDot]", "y"}], " ",
       "=", " ", 
      RowBox[{
       RowBox[{"n", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"x", "\[CenterDot]", "y"}], ")"}]}], " ", "=", " ", 
       RowBox[{"x", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"n", "\[CenterDot]", "y"}], ")"}]}]}]}]}]}], 
   TraditionalForm]]],
 ".\n\nBut ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is non-zero, and the ring has no zero-divisors, so we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "y"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 ". This argument works in both ways, so\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"(*", ")"}], TraditionalForm]]],
 "      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "x"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"0", "   ", "\[DoubleLongLeftRightArrow]", "   ", "n"}], 
      "\[CenterDot]", "y"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"0", "      ", "if", "   ", "x"}], "\[NotEqual]", 
      RowBox[{"0", "   ", "and", "  ", "y"}], "\[NotEqual]", "0"}]}]}], 
   TraditionalForm]]],
 ".\n\nIf ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " was not zero, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "n", "|"}], TraditionalForm]]],
 " would be a positive number such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in the ring. Hence, if the ring has characteristic 0, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " implies that either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", "0"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "0."}], TraditionalForm]]],
 "\n\nNow suppose that the ring has positive characteristic, and let ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " be any non-zero element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " be the smallest positive integer for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "\[CenterDot]", "x"}], " ", "=", " ", "0."}], 
   TraditionalForm]]],
 " If ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is not prime, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", " ", "=", " ", 
     RowBox[{"a", "\[CenterDot]", "b"}]}], ","}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", "<", "a", "<", "p"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", "<", "b", "<", 
    RowBox[{"p", "."}]}], TraditionalForm]]],
 " But then \n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "\[CenterDot]", "x"}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"b", "\[CenterDot]", "x"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "\[CenterDot]", "b"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       SuperscriptBox["x", "2"], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"p", "\[CenterDot]", "x"}], ")"}], "\[CenterDot]", "x"}], " ",
       "=", " ", 
      RowBox[{
       RowBox[{"0", "\[CenterDot]", "x"}], " ", "=", " ", "0."}]}]}]}], 
   TraditionalForm]]],
 "\n\nSince the ring has no zero divisors, either ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 ". But this contradicts the fact that ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " was the ",
 StyleBox["smallest",
  FontSlant->"Italic"],
 " number such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "\[CenterDot]", "x"}], " ", "=", " ", "0."}], 
   TraditionalForm]]],
 " Thus, ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is prime. By ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(*", ")"}], TraditionalForm]]],
 " we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "\[CenterDot]", "y"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " for ",
 StyleBox["every",
  FontSlant->"Italic"],
 " element in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", and since this cannot be true for any smaller integer, we have that the \
characteristic of the ring is the prime number ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ".\n\nIt is easy to see that if ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", 
    RowBox[{"c", "\[CenterDot]", "p"}]}], TraditionalForm]]],
 " for some integer ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ". Thus, for any element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ",\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "x"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"c", "\[CenterDot]", "p"}], ")"}], "\[CenterDot]", "x"}], " ", 
     "=", " ", 
     RowBox[{
      RowBox[{"c", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"p", "\[CenterDot]", "x"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"c", "\[CenterDot]", "0"}], " ", "=", " ", "0."}]}]}]}], 
   TraditionalForm]]],
 "\n\nSuppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " for some ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " which is not a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"GCD", "(", 
     RowBox[{"n", ",", "p"}], ")"}], " "}], TraditionalForm]]],
 "must be 1, and so by the greatest common divisor theorem (1.2), there are \
integers ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"u", "\[CenterDot]", "n"}], " ", "+", " ", 
     RowBox[{"v", "\[CenterDot]", "p"}]}], " ", "=", " ", "1"}], 
   TraditionalForm]]],
 ". But then\n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"x", " ", "=", " ", 
     RowBox[{
      RowBox[{"1", "\[CenterDot]", "x"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"u", "\[CenterDot]", "n"}], " ", "+", " ", 
          RowBox[{"v", "\[CenterDot]", "p"}]}], ")"}], "\[CenterDot]", "x"}], 
       " ", "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"u", "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{"n", "\[CenterDot]", "x"}], ")"}]}], " ", "+", " ", 
         RowBox[{"v", "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{"p", "\[CenterDot]", "x"}], ")"}]}]}], " ", "=", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"u", "\[CenterDot]", "0"}], " ", "+", " ", 
          RowBox[{"v", "\[CenterDot]", "0"}]}], " ", "=", " ", 
         "0."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nSo for non-zero ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "x"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 " if and only if ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373955367946253`*^9, 3.4373955396383753`*^9}}]
}, Open  ]],

Cell[TextData[{
 "Characteristics are important because they give us a new way of defining \
integral domains and fields in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". We begin by telling ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " the characteristic ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " of the ring we want to define with the command ",
 StyleBox["InitDomain[",
  FontWeight->"Bold"],
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 StyleBox["]",
  FontWeight->"Bold"],
 "\n\nThis command does three things. First, it tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that the ring to be defined is commutative, so the regular multiplication \
notation can be used instead of the dot. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " defines the identity element to be 1. Finally, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " assumes that the ring to be defined has no zero-divisors, and takes into \
account propostion 11.2, defining ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " times ",
 StyleBox["anything",
  FontSlant->"Italic"],
 " to be 0. Of course there is no guarantee that the ring formed will not \
have zero-divisors. This must be checked separately.\n\nFor example, let us \
define a ring with characteristic 3."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373955869821253`*^9, 3.4373955881227503`*^9}}],

Cell["InitDomain[3]", "Input",
 PageWidth->Infinity],

Cell["\<\
As this point, we can test some of the properties of the ring.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["2 + 2", "Input",
 PageWidth->Infinity],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.442753306852*^9, 3.442753345992625*^9, 
  3.442754087586375*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["2 * 2", "Input",
 PageWidth->Infinity],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.442753307852*^9, 3.4427533467895*^9, 3.442754089227*^9}]
}, Open  ]],

Cell[TextData[{
 "So far, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is acting very much like it did when we defined ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 " using the ",
 StyleBox["DefMod",
  FontWeight->"Bold"],
 " command. But there is an important difference. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will now assume that 3 times any variable is 0. So the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["2 i + 5 i ", "Input",
 PageWidth->Infinity],

Cell[BoxData["i"], "Output",
 CellChangeTimes->{3.442753309211375*^9, 3.442753347898875*^9, 
  3.442754092805125*^9}]
}, Open  ]],

Cell[TextData[{
 "simplifies to just ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 ". If we consider ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", "i"}], TraditionalForm]]],
 " to be an additional element in our ring, we will need to tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " what ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["i", "2"], TraditionalForm]]],
 " is. Let us try imitating the complex numbers, and let  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["i", "2"],
     TraditionalForm], " ", "=", " ", 
    RowBox[{"-", "1."}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Define[i^2, -1]", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "This is all ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " needs to form a ring containing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", "i"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["K = Ring[{i}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1", ",", "2", ",", "i", ",", 
   RowBox[{"2", " ", "i"}], ",", 
   RowBox[{"1", "+", "i"}], ",", 
   RowBox[{"2", "+", "i"}], ",", 
   RowBox[{"1", "+", 
    RowBox[{"2", " ", "i"}]}], ",", 
   RowBox[{"2", "+", 
    RowBox[{"2", " ", "i"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.4373956301383753`*^9, 3.44275333188325*^9, 
  3.442753398711375*^9, 3.44275409519575*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives us nine elements. We can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " verify that this is a ring."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckRing[K]", "Input",
 PageWidth->Infinity],

Cell[BoxData["\<\"These elements will generate a ring.\"\>"], "Print",
 CellChangeTimes->{3.4373956327477503`*^9, 3.44275412725825*^9}]
}, Open  ]],

Cell["Let us look at the addition and multiplication tables.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["AddTable[K]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {0, 1, 0}, {0.7, 0.8, 0.7}, {
      0, 1, 1}, {1, 0.6, 0}, {1, 1, 0}, {1, 0, 1}, {1, 0, 0}, {0.6, 0.4, 
      0.8}}, {{0.7, 0.8, 0.7}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {0, 1, 0}, {
      1, 0, 1}, {0.6, 0.4, 0.8}, {0, 1, 1}, {1, 1, 0}, {1, 0.6, 0}, {1, 0, 
      0}}, {{1, 0.6, 0}, {1, 0.6, 0}, {1, 0, 0}, {0.6, 0.4, 0.8}, {0.8, 0.6, 
      0.5}, {0, 1, 1}, {0, 1, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 1}}, {{
      0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {1, 0, 0}, {0.7, 0.8, 
      0.7}, {1, 0, 1}, {0.8, 0.6, 0.5}, {0, 1, 0}, {0, 1, 1}, {1, 1, 0}}, {{0,
       1, 0}, {0, 1, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}, {1, 1, 0}, {1, 0, 
      0}, {1, 0, 1}, {0, 1, 1}, {0.6, 0.4, 0.8}, {1, 0.6, 0}}, {{1, 0, 0}, {1,
       0, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0, 1, 0}, {1, 1, 0}, {0.7, 0.8, 
      0.7}, {0.8, 0.6, 0.5}, {1, 0, 1}, {0, 1, 1}}, {{0, 1, 1}, {0, 1, 1}, {1,
       1, 0}, {1, 0, 1}, {1, 0.6, 0}, {0.8, 0.6, 0.5}, {1, 0, 0}, {0.6, 0.4, 
      0.8}, {0, 1, 0}, {0.7, 0.8, 0.7}}, {{1, 0, 1}, {1, 0, 1}, {0, 1, 1}, {1,
       1, 0}, {0.6, 0.4, 0.8}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0, 0}, {0.8, 
      0.6, 0.5}, {0, 1, 0}}, {{1, 1, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 
      0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 
      0.6, 0.5}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 
      1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 
      0.5}}}], InsetBox[
    TagBox[
     StyleBox["\<\"+\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 9}, {10, 9}}]}]], "Output",
 CellChangeTimes->{3.4373956719508753`*^9, 3.44275413132075*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["MultTable[K]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.8, 0.6, 0.5}, {1, 1, 0}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {
      0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {0, 1, 
      0}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 
      0.4, 0.8}, {0.8, 0.6, 0.5}, {0, 1, 1}, {0, 1, 0}, {1, 0, 0}, {1, 0, 
      1}}, {{1, 0.6, 0}, {1, 1, 0}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 
      0.5}, {0.6, 0.4, 0.8}, {1, 0, 1}, {1, 0, 0}, {0, 1, 0}, {0, 1, 1}}, {{
      0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}}, {{0, 1, 0}, {1, 1, 0}, {0, 1, 0}, {1, 0, 0}, {1, 0, 1}, {0, 1, 
      1}, {0.7, 0.8, 0.7}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {1, 0.6, 0}}, {{
      1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {0, 1, 0}, {0, 1, 1}, {1, 0, 1}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {0.7, 0.8, 0.7}}, {{0, 1, 
      1}, {1, 1, 0}, {0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {1, 0, 0}, {0.8, 0.6, 
      0.5}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}}, {{1, 0, 1}, {1, 1,
       0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}}, {{1, 1, 0}, {1, 1, 0}, {
     1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}}}], 
   InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 9}, {10, 9}}]}]], "Output",
 CellChangeTimes->{3.4373956742321253`*^9, 3.442754133648875*^9}]
}, Open  ]],

Cell[TextData[{
 "The multiplication table shows that there are no zero divisors. By \
corollary 9.1, ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is a field. We could call ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " the field of \"complex numbers modulo 3\".\n\nEXPERIMENT:\nTry forming a \
similar ring with characteristic 5 instead of 3. That is, form a ring of \
complex numbers modulo 5. Use the multiplication table to determine if there \
are zero divisors. Is this a field?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373956899821253`*^9, 3.4373956907321253`*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can now work with polynomials over an integral domain or field using the \
familiar notation. For example, we can reload the field of complex numbers \
modulo 3. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitDomain[3];
Define[i^2, -1]
K = Ring[{i}]   \
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1", ",", "2", ",", "i", ",", 
   RowBox[{"2", " ", "i"}], ",", 
   RowBox[{"1", "+", "i"}], ",", 
   RowBox[{"2", "+", "i"}], ",", 
   RowBox[{"1", "+", 
    RowBox[{"2", " ", "i"}]}], ",", 
   RowBox[{"2", "+", 
    RowBox[{"2", " ", "i"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.4417248981983747`*^9, 3.44275413638325*^9}]
}, Open  ]],

Cell[TextData[{
 "We can now form polynomials in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Y = (1 + i) x + 2", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.4373899984375*^9}],

Cell[BoxData[
 RowBox[{"2", "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", "i"}], ")"}], " ", "x"}]}]], "Output",
 CellChangeTimes->{3.4373957134040003`*^9, 3.4417249030421247`*^9, 
  3.442754138227*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Z = (2 + i) x^2 + 2 i x + 1 + 2i", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43738999903125*^9, 3.43738999940625*^9}}],

Cell[BoxData[
 RowBox[{"1", "+", 
  RowBox[{"2", " ", "i"}], "+", 
  RowBox[{"2", " ", "i", " ", "x"}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"2", "+", "i"}], ")"}], " ", 
   SuperscriptBox["x", "2"]}]}]], "Output",
 CellChangeTimes->{3.4373957156383753`*^9, 3.4417249091983747`*^9, 
  3.442754139242625*^9}]
}, Open  ]],

Cell["We can form products as we would with standard polynomials", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Y^2", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 SuperscriptBox[
  RowBox[{"(", 
   RowBox[{"2", "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", "i"}], ")"}], " ", "x"}]}], ")"}], "2"]], "Output",
 CellChangeTimes->{3.4373957206540003`*^9, 3.4417249136514997`*^9, 
  3.442754140602*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Y Z", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"2", "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", "i"}], ")"}], " ", "x"}]}], ")"}], " ", 
  RowBox[{"(", 
   RowBox[{"1", "+", 
    RowBox[{"2", " ", "i"}], "+", 
    RowBox[{"2", " ", "i", " ", "x"}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"2", "+", "i"}], ")"}], " ", 
     SuperscriptBox["x", "2"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.4373957237477503`*^9, 3.4427541422895*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not automatically expand such expressions, but leaves them in \
factored form. We have now totally eliminated the need to use the period for \
multiplication. But notice what happens if we use the dot anyway:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Y.Y", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"1", "+", "x", "+", 
  RowBox[{"i", " ", "x"}], "+", 
  RowBox[{"2", " ", "i", " ", 
   SuperscriptBox["x", "2"]}]}]], "Output",
 CellChangeTimes->{3.4373957292477503`*^9, 3.4417249961514997`*^9, 
  3.44275414369575*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Y.Z", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"2", "+", "i", "+", 
  RowBox[{"2", " ", "x"}], "+", 
  RowBox[{"i", " ", "x"}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox["x", "2"]}], "+", 
  RowBox[{"i", " ", 
   SuperscriptBox["x", "2"]}], "+", 
  SuperscriptBox["x", "3"]}]], "Output",
 CellChangeTimes->{3.4373957313415003`*^9, 3.442754144680125*^9}]
}, Open  ]],

Cell[TextData[{
 "This time ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " expanded the expressions. This gives a streamlined method of expanding \
products.\n\n",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can factor polynomials defined over any finite field. In the next chapter \
we will prove that such factorizations are unique. (We will also see an \
example of an integral domain, not a field, for which factorizations are not \
unique.) If ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " tries to factor ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "+", " ", "1"}], TraditionalForm]]],
 " in the standard way (using the ring ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 "),"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43739000740625*^9, 3.43739000740625*^9}, {
  3.4373957829040003`*^9, 3.4373957830758753`*^9}}],

Cell[CellGroupData[{

Cell["Factor[x^2 + 1]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{
  3.43739001096875*^9, {3.4373957778415003`*^9, 3.4373957780446253`*^9}}],

Cell[BoxData[
 RowBox[{"1", "+", 
  SuperscriptBox["x", "2"]}]], "Output",
 CellChangeTimes->{{3.4373957539821253`*^9, 3.4373957783883753`*^9}, 
   3.44275414644575*^9}]
}, Open  ]],

Cell[TextData[{
 "it finds the polynomial is irreducable. But if we factor the polynomial \
over the field ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ","
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Factor[x^2 + 1, K]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{
  3.43739001153125*^9, {3.4373957751071253`*^9, 3.4373957753883753`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"i", "+", "x"}], ")"}], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"2", " ", "i"}], "+", "x"}], ")"}]}]], "Output",
 CellChangeTimes->{{3.4373957583102503`*^9, 3.4373957758883753`*^9}, 
   3.442754148148875*^9}]
}, Open  ]],

Cell[TextData[{
 "we find that it does factor.\n\nEXPERIMENT:\nTry looking at other quadratic \
polynomials which do not factor over the integers. Do they factor using the \
field ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 "? What about cubic polynomials?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell["\<\
The polynomial rings defined over integral domains give us some good examples \
of integral domains. In the next chapter we will find other ways of forming \
integral domains, some of which have some unusual properties. But even these \
are based on polynomial rings. So polynomials are the basic building blocks \
that are used for forming new integral domains and fields.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Field of Quotients", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In the last section, we found a way to form integral domains by imitating \
the familar polynomials from high school algebra. In this section we will \
show how we can form a field from an integral domain. In fact we will imitate \
mathematics from grade school\[LongDash]the fractions.\n\nWe can view a \
fraction as one integer divided by another. Thus, the properties of the \
rational numbers stem entirely on the properties of the integers, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". Although the integers have some special properties, some of which we will \
discover later in this chapter, perhaps the most important property of the \
integers is that it is a simple example of an integral domain. This raises \
the question: Can we form fractions out of any integral domain? The answer is \
\"yes\", but before we can proceed, let us review the basic definitions of \
addition, multiplication, and equality of the rational numbers ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 ".\n\nFirst of all, there is a restriction that we put on the rational \
number ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "/", "q"}], TraditionalForm]]],
 ": that ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " cannot be 0. It is reasonable to expect that we will have to make a \
similar restriction for general integal domains.\n\nWe can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " show us how two rational numbers are added:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417251694171247`*^9, 3.4417251775733747`*^9}}],

Cell[CellGroupData[{

Cell["Together[x/y + u/v]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"v", " ", "x"}], "+", 
   RowBox[{"u", " ", "y"}]}], 
  RowBox[{"v", " ", "y"}]]], "Output",
 CellChangeTimes->{3.44275415144575*^9}]
}, Open  ]],

Cell[TextData[{
 "Since nether ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " are 0, the denominator of the sum is not zero, so this is a rational \
number. Similarly, the product"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["(x/y) * (u/v)", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 FractionBox[
  RowBox[{"u", " ", "x"}], 
  RowBox[{"v", " ", "y"}]]], "Output",
 CellChangeTimes->{3.442754154992625*^9}]
}, Open  ]],

Cell[TextData[{
 "is a rational number. However, there is a twist in how we view two rational \
numbers as being ",
 StyleBox["equal",
  FontSlant->"Italic"],
 ". Since fourth grade, we view\n   \n                              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox[
     RowBox[{
      FractionBox["2", "4"], " ", "=", " ", 
      FractionBox["3", "6"]}],
     FontSize->16], ","}], TraditionalForm]]],
 "\n \neven though these are two different fractions. Both the numerators and \
denominators are different. What we really mean is that these two fractions \
are ",
 StyleBox["equivalent",
  FontSlant->"Italic"],
 ", and in fact we define\n\n                           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox[
      FractionBox["x", "y"],
      FontSize->16], " ", "\[Congruent]", " ", 
     RowBox[{
      RowBox[{
       StyleBox[
        FractionBox["u", "v"],
        FontSize->16], "    ", "\[DoubleLongLeftRightArrow]", "   ", "x"}], 
      "\[CenterDot]", "v"}]}], " ", "=", " ", 
    RowBox[{"u", "\[CenterDot]", "y"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "This forms an equivalence relation on the set of fractions ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "/", "y"}], TraditionalForm]]],
 ". We have already seen equivalence relations while working with cosets of a \
group. What we call a rational number is really a set of fractions of the \
form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "/", "y"}], TraditionalForm]]],
 " which are all equivalent. In other words, 1/2 really represents a ",
 StyleBox["set",
  FontSlant->"Italic"],
 " of fractions\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"{", 
     RowBox[{
      FractionBox["1", "2"], ",", " ", 
      FractionBox[
       RowBox[{"-", "1"}], 
       RowBox[{"-", "2"}]], ",", " ", 
      FractionBox["2", "4"], ",", " ", 
      FractionBox[
       RowBox[{"-", "2"}], 
       RowBox[{"-", "4"}]], ",", 
      FractionBox["3", "6"], ",", " ", 
      FractionBox[
       RowBox[{"-", "3"}], 
       RowBox[{"-", "6"}]], ",", " ", "\[Ellipsis]"}], " ", "}"}]}], 
   TraditionalForm]]],
 "\n\nwith each element of this set equivalent to any other. Since ",
 Cell[BoxData[
  FormBox[
   FractionBox["2", "4"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   FractionBox["3", "6"], TraditionalForm]]],
 " are both in this set, we consider them to be ",
 StyleBox["equal",
  FontSlant->"Italic"],
 " to ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", "2"], TraditionalForm]]],
 ". \n\nWe can define a similar equivalence relation for any integral domain \
",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".\n\nDEFINITION 11.3:\nLet ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be an integral domain, and let ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " denote the set of all ordered pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 " of elements of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " non-zero:\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", ",", "y"}], ")"}], "  ", "|", "  ", "x"}], ",", 
      RowBox[{"y", " ", "\[Element]", " ", 
       RowBox[{
        RowBox[{"K", "    ", "and", "    ", "y"}], "\[NotEqual]", "0"}]}]}], 
     "}"}]}], TraditionalForm]]],
 ".\n               \nWe define a relation on ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " by\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", ",", "y"}], ")"}], " ", "\[Congruent]", " ", 
    RowBox[{"(", 
     RowBox[{"u", ",", "v"}], ")"}]}], TraditionalForm]]],
 "  if  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "v"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "u"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437482962746375*^9, 3.437482979746375*^9}}],

Cell[TextData[{
 "The first step is to show that this relation is an equivalence relation.\n\n\
LEMMA 11.2\nThe above relation is an equivalence relation on ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43748299085575*^9, 3.437482990965125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We need to show that the relation is reflexive, symmetric, and transitive. \
Let",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", ",", "y"}], ")"}], ","}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"u", ",", "v"}], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"s", ",", "t"}], ")"}], TraditionalForm]]],
 " be arbitrary elements of ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ".\n\nReflexive:"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", ",", "y"}], ")"}], " ", "\[Congruent]", " ", 
    RowBox[{"(", 
     RowBox[{"x", ",", "y"}], ")"}]}], TraditionalForm]]],
 "  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "is equivalent to saying",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", 
      RowBox[{"x", "\[CenterDot]", "y"}]}], ","}]}], TraditionalForm]]],
 " which is, of course, true. So this relation is reflexive.\n\nSymmetric:"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "       ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", ",", "y"}], ")"}], " ", "\[Congruent]", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"u", ",", "v"}], ")"}], "   ", "\[DoubleLongRightArrow]", 
       "   ", "x"}], "\[CenterDot]", "v"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"y", "\[CenterDot]", 
      RowBox[{"u", "   ", "\[DoubleLongRightArrow]", "  ", "u"}], 
      "\[CenterDot]", "y"}], "=", 
     RowBox[{"v", "\[CenterDot]", "x"}]}]}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                                     ", 
    RowBox[{
     RowBox[{"\[DoubleLongRightArrow]", " ", 
      RowBox[{"(", 
       RowBox[{"u", ",", "v"}], ")"}]}], " ", "\[Congruent]", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", ",", "y"}], ")"}], "."}]}]}], TraditionalForm]}]],
 " "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "so this relation is also symmetric. \n\nTransitive:\n\n If ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", ",", "y"}], ")"}], " ", "\[Congruent]", " ", 
    RowBox[{"(", 
     RowBox[{"u", ",", "v"}], ")"}], " "}], TraditionalForm]]],
 " and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"u", ",", "v"}], ")"}], " ", "\[Congruent]", " ", 
    RowBox[{"(", 
     RowBox[{"s", ",", "t"}], ")"}]}], TraditionalForm]]],
 ", then  "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[Cell[BoxData[
 FormBox[GridBox[{
    {
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", ",", "y"}], ")"}], " ", "\[Congruent]", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"u", ",", "v"}], ")"}], "  ", "\[DoubleLongRightArrow]", 
         "  ", "x"}], "\[CenterDot]", "v"}]}], "=", 
      RowBox[{
       RowBox[{"y", "\[CenterDot]", 
        RowBox[{"u", "  ", "\[DoubleLongRightArrow]", " ", "x"}], 
        "\[CenterDot]", "v", "\[CenterDot]", "t"}], " ", "=", " ", 
       RowBox[{"y", "\[CenterDot]", "u", "\[CenterDot]", "t"}]}]}]},
    {
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"u", ",", "v"}], ")"}], " ", "\[Congruent]", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"s", ",", "t"}], ")"}], "  ", "\[DoubleLongRightArrow]", 
         "  ", "u"}], "\[CenterDot]", "t"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"v", "\[CenterDot]", 
        RowBox[{"s", "  ", "\[DoubleLongRightArrow]", "  ", "u"}], 
        "\[CenterDot]", "t", "\[CenterDot]", "y"}], " ", "=", " ", 
       RowBox[{"v", "\[CenterDot]", "s", "\[CenterDot]", "y"}]}]}]}
   }], TraditionalForm]]]], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437483049152625*^9, 3.43748304973075*^9}, {
  3.437483081027625*^9, 3.4374830856995*^9}}],

Cell[TextData[{
 "These two statements imply that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "v", "\[CenterDot]", "t"}], " ", "=", " ", 
    RowBox[{"v", "\[CenterDot]", "s", "\[CenterDot]", "y"}]}], 
   TraditionalForm]]],
 ".\nNotice that in the last step we had to use the commutativity of \
multiplication. Using commutativity again, we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "t", "\[CenterDot]", "v"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "s", "\[CenterDot]", "v"}]}], 
   TraditionalForm]]],
 ", and since ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " has no zero divisors and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " is non-zero, we can use lemma 9.4 to say that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "t"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "s"}]}], TraditionalForm]]],
 ". Then"
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437483057512*^9, 3.437483101433875*^9}}],

Cell[TextData[{
 "              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "t"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"y", "\[CenterDot]", 
      RowBox[{"s", "   ", "\[DoubleLongRightArrow]", "  ", 
       RowBox[{"(", 
        RowBox[{"x", ",", "y"}], ")"}]}]}], " ", "\[Congruent]", " ", 
     RowBox[{"(", 
      RowBox[{"s", ",", "t"}], ")"}]}]}], TraditionalForm]]],
 ","
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
so we have the transitive law holding. Therefore, this relation is an \
equivalence relation.\
\>", "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "This equivalence relation becomes the key for defining our field of \
quotients, as the next definition shows.\n\nDEFINITION 11.4\nLet ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be an integral domain, let ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " denote the set \n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", ",", "y"}], ")"}], "  ", "|", "  ", "x"}], ",", 
      RowBox[{"y", " ", "\[Element]", " ", 
       RowBox[{
        RowBox[{"K", "   ", "and", "  ", "y"}], "\[NotEqual]", "0"}]}]}], 
     "}"}]}], TraditionalForm]]],
 ","
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437483114121375*^9, 3.437483123308875*^9}}],

Cell[TextData[{
 "and let the equivalence relation on ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " be\n \n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", ",", "y"}], ")"}], " ", "\[Congruent]", " ", 
    RowBox[{"(", 
     RowBox[{"u", ",", "v"}], ")"}]}], TraditionalForm]]],
 "  if  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "v"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "u"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "For each ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    FractionBox["x", "y"], ")"}], TraditionalForm]]],
 "denote the equivalence class of ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " that contains ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " denote the set of all equivalence classes ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    FractionBox["a", "b"], ")"}], TraditionalForm]]],
 ". The set ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " is called the ",
 StyleBox["set of quotients",
  FontSlant->"Italic"],
 " for ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".\n\nThis definition allows us to replace an equivalence of two expressions \
with an equality. We now have that\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox["x", "y"], ")"}],
      TraditionalForm], " ", "=", " ", 
     FormBox[
      RowBox[{"(", 
       FractionBox["u", "v"], ")"}],
      TraditionalForm]}]}], TraditionalForm]]],
 "     if and only if   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"x", "\[CenterDot]", "v"}], " ", "=", " ", 
     RowBox[{"u", "\[CenterDot]", 
      RowBox[{"y", "."}]}]}]}], TraditionalForm]]],
 "\n\nFor example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{"x", "\[CenterDot]", "z"}], 
        RowBox[{"y", "\[CenterDot]", "z"}]], ")"}],
      TraditionalForm], " ", "=", " ", 
     FormBox[
      RowBox[{"(", 
       FractionBox["x", "y"], ")"}],
      TraditionalForm]}]}], TraditionalForm]]],
 " for any non-zero ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "z", "\[CenterDot]", "y"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "z", "\[CenterDot]", "x"}]}], 
   TraditionalForm]]],
 ".\n\nThe next step is to define addition and multiplication on our set of \
quotients ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". Once again, we will use the rational numbers to guide us in the \
definition.\n\nLEMMA 11.3\nLet ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be an integral domain, and let ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " be the set of quotients for ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". The addition and multiplication of two equivalence classed in ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", defined by\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox["x", "y"], ")"}],
       TraditionalForm], "+", " ", 
      FormBox[
       RowBox[{"(", 
        FractionBox["u", "v"], ")"}],
       TraditionalForm]}], " ", "=", " ", 
     FormBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "v"}], " ", "+", " ", 
         RowBox[{"u", "\[CenterDot]", "y"}]}], 
        RowBox[{"y", "\[CenterDot]", "v"}]], ")"}],
      TraditionalForm]}]}], TraditionalForm]]],
 "\nand\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox["x", "y"], ")"}],
        TraditionalForm], "\[CenterDot]", "  ", 
       FormBox[
        RowBox[{"(", 
         FractionBox["u", "v"], ")"}],
        TraditionalForm]}], " ", "=", " ", 
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{"x", "\[CenterDot]", "u"}], 
         RowBox[{"y", "\[CenterDot]", "v"}]], ")"}],
       TraditionalForm]}], ","}]}], TraditionalForm]]],
 "\n\nare both well defined operations on ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". That is, the sum and product do not depend on the choice of the \
representative elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"u", ",", "v"}], ")"}], TraditionalForm]]],
 " of the equivalence classes."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43748315354325*^9, 3.4374831540745*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The first observation we need to make is that the formulas for the sum and \
product both form valid elements of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "v"}], TraditionalForm]]],
 " is non-zero as long as ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " are both non-zero. \n\nNext let us work to show that addition does not \
depend on the choice of representative elements ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"u", ",", "v"}], ")"}], TraditionalForm]]],
 ". That is, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox["x", "y"], ")"}],
      TraditionalForm], " ", "=", "  ", 
     FormBox[
      RowBox[{"(", 
       FractionBox["a", "b"], ")"}],
      TraditionalForm]}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox["u", "v"], ")"}],
      TraditionalForm], " ", "=", "  ", 
     FormBox[
      RowBox[{"(", 
       FractionBox["c", "d"], ")"}],
      TraditionalForm]}]}], TraditionalForm]]],
 ", we need to show that \n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox["x", "y"], ")"}],
       TraditionalForm], " ", "+", "  ", 
      FormBox[
       RowBox[{"(", 
        FractionBox["u", "v"], ")"}],
       TraditionalForm]}], "=", "  ", 
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox["a", "b"], ")"}],
       TraditionalForm], " ", "+", "  ", 
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox["c", "d"], ")"}],
        TraditionalForm], "."}]}]}]}], TraditionalForm]]],
 "\n\nThat is, we have to prove that \n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", " ", 
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "v"}], " ", "+", " ", 
         RowBox[{"u", " ", "\[CenterDot]", "y"}]}], 
        RowBox[{"y", "\[CenterDot]", "v"}]], ")"}],
      TraditionalForm], " ", "=", " ", 
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{
          RowBox[{"a", "\[CenterDot]", "d"}], " ", "+", " ", 
          RowBox[{"c", "\[CenterDot]", "b"}]}], 
         RowBox[{"b", "\[CenterDot]", "d"}]], ")"}],
       TraditionalForm], " ", "."}]}]}], TraditionalForm]]],
 "\n           \nWe have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox["x", "y"], ")"}],
      TraditionalForm], " ", "=", "  ", 
     FormBox[
      RowBox[{"(", 
       FractionBox["a", "b"], ")"}],
      TraditionalForm]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox["u", "v"], ")"}],
      TraditionalForm], " ", "=", "  ", 
     FormBox[
      RowBox[{"(", 
       FractionBox["c", "d"], ")"}],
      TraditionalForm]}]}], TraditionalForm]]],
 ", so we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "b"}], " ", "=", " ", 
    RowBox[{"a", "\[CenterDot]", "y"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "\[CenterDot]", "d"}], " ", "=", " ", 
    RowBox[{"c", "\[CenterDot]", "v"}]}], TraditionalForm]]],
 ". Multiplying the first equation by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "\[CenterDot]", "d"}], TraditionalForm]]],
 " and the second by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "b"}], TraditionalForm]]],
 ", we get\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
     "x", "\[CenterDot]", "b", "\[CenterDot]", "v", "\[CenterDot]", "d"}], 
     "  ", "=", "  ", 
     RowBox[{
     "a", "\[CenterDot]", "y", "\[CenterDot]", "v", "\[CenterDot]", "d"}]}]}],
    TraditionalForm]]],
 "\nand\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "u", "\[CenterDot]", "d", "\[CenterDot]", "y", "\[CenterDot]", "b"}], 
    "  ", "=", "  ", 
    RowBox[{"c", "\[CenterDot]", "v", "\[CenterDot]", "y", "\[CenterDot]", 
     RowBox[{"b", "."}]}]}], TraditionalForm]]],
 "\n\nAdding this two equations together and factoring, we get\n\n            \
   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"x", "\[CenterDot]", "v"}], " ", "+", " ", 
       RowBox[{"u", "\[CenterDot]", "y"}]}], ")"}], "\[CenterDot]", "b", 
     "\[CenterDot]", "d"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", "\[CenterDot]", "d"}], " ", "+", " ", 
       RowBox[{"c", "\[CenterDot]", "b"}]}], ")"}], "\[CenterDot]", "y", 
     "\[CenterDot]", 
     RowBox[{"v", "."}]}]}], TraditionalForm]]],
 "\n\nThis gives us \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{
          RowBox[{"x", "\[CenterDot]", "v"}], " ", "+", " ", 
          RowBox[{"u", " ", "\[CenterDot]", "y"}]}], 
         RowBox[{"y", "\[CenterDot]", "v"}]], ")"}],
       TraditionalForm], " ", "=", " ", 
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{
          RowBox[{"a", "\[CenterDot]", "d"}], " ", "+", " ", 
          RowBox[{"c", "\[CenterDot]", "b"}]}], 
         RowBox[{"b", "\[CenterDot]", "d"}]], ")"}],
       TraditionalForm]}], ","}]}], TraditionalForm]]],
 "\n\nwhich is what we wanted.\n\nWe also need to show that multiplication is \
well defined, that is\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox["x", "y"], ")"}],
       TraditionalForm], "  ", "\[CenterDot]", " ", 
      FormBox[
       RowBox[{"(", 
        FractionBox["u", "v"], ")"}],
       TraditionalForm]}], "  ", "=", " ", 
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox["a", "b"], ")"}],
       TraditionalForm], "  ", "\[CenterDot]", " ", 
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox["c", "d"], ")"}],
        TraditionalForm], " ", "."}]}]}]}], TraditionalForm]]],
 "\n\nBut since ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"x", "\[CenterDot]", "b"}], " ", "=", " ", 
     RowBox[{"a", "\[CenterDot]", "y"}]}]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"u", "\[CenterDot]", "d"}], " ", "=", " ", 
    RowBox[{"c", "\[CenterDot]", "v"}]}], TraditionalForm]]],
 ", we can multiply these two equations together to get\n\n                   \
    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
     "x", "\[CenterDot]", "b", "\[CenterDot]", "u", "\[CenterDot]", "d"}], 
     " ", "=", " ", 
     RowBox[{
     "a", "\[CenterDot]", "y", "\[CenterDot]", "c", "\[CenterDot]", "v"}]}], 
    ","}], TraditionalForm]]],
 " \nor\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", "\[CenterDot]", "u"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"b", "\[CenterDot]", "d"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "\[CenterDot]", "c"}], ")"}], "\[CenterDot]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"y", "\[CenterDot]", "v"}], ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\nTherefore, \n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{"x", "\[CenterDot]", "u"}], 
        RowBox[{"y", "\[CenterDot]", "v"}]], ")"}],
      TraditionalForm], "    ", "=", " ", 
     FormBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{"a", "\[CenterDot]", "c"}], 
        RowBox[{"b", "\[CenterDot]", "d"}]], ")"}],
      TraditionalForm]}], " ", ","}], TraditionalForm]]],
 "\n\nso multiplication also is well defined."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "Now that we have defined addition and multiplication on the sets of \
equivalent classes ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", we are ready to show that ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " is in fact a ",
 StyleBox["field",
  FontSlant->"Italic"],
 ".\n\nTHEOREM 11.1:  THE FIELD OF QUOTIENTS THEOREM\nLet ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be an integral domain, and let ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " be the set of quotients for ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " forms a field using the above definitions of addition and multiplication. \
The field ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " is called ",
 StyleBox["the field of quotients for",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437483216683875*^9, 3.4374832176995*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We have already noted that addition and multiplication are closed in ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ".\n\nWe next want to look at the properties of addition. From the \
definition,\n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox["x", "y"], ")"}],
        TraditionalForm], "   ", "+", " ", 
       FormBox[
        RowBox[{"(", 
         FractionBox["u", "v"], ")"}],
        TraditionalForm]}], "    ", "=", " ", 
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox[
          RowBox[{
           RowBox[{"x", "\[CenterDot]", "v"}], " ", "+", " ", 
           RowBox[{"u", " ", "\[CenterDot]", "y"}]}], 
          RowBox[{"y", "\[CenterDot]", "v"}]], ")"}],
        TraditionalForm], "  ", "=", 
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox["u", "v"], ")"}],
         TraditionalForm], "  ", "+", " ", 
        FormBox[
         RowBox[{"(", 
          FractionBox["x", "y"], ")"}],
         TraditionalForm]}]}]}], " ", ","}]}], TraditionalForm]]],
 "\n\nwe see that addition is commutative. Let ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " be any non-zero element of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"(", 
      FractionBox["0", "z"], ")"}],
     TraditionalForm], " "}], TraditionalForm]]],
 " acts as the additive identity:\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox["u", "v"], ")"}],
       TraditionalForm], "  ", "+", " ", 
      FormBox[
       RowBox[{"(", 
        FractionBox["0", "z"], ")"}],
       TraditionalForm]}], "  ", "=", " ", 
     RowBox[{
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox["0", "z"], ")"}],
        TraditionalForm], "  ", "+", 
       FormBox[
        RowBox[{"(", 
         FractionBox["u", "v"], ")"}],
        TraditionalForm]}], "  ", "=", " ", 
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox[
          RowBox[{
           RowBox[{"0", "\[CenterDot]", "v"}], " ", "+", " ", 
           RowBox[{"u", "\[CenterDot]", "z"}]}], 
          RowBox[{"z", "\[CenterDot]", "v"}]], ")"}],
        TraditionalForm], "  ", "=", " ", 
       RowBox[{
        RowBox[{"(", 
         FractionBox[
          RowBox[{"u", "\[CenterDot]", "z"}], 
          RowBox[{"z", "\[CenterDot]", "v"}]], ")"}], " ", "=", " ", 
        RowBox[{
         FormBox[
          RowBox[{"(", 
           FractionBox["u", "v"], ")"}],
          TraditionalForm], " ", "."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nLikewise, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{"-", "u"}], "v"], ")"}],
     TraditionalForm], " "}], TraditionalForm]]],
 " is the additive inverse of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"(", 
      FractionBox["u", "v"], ")"}],
     TraditionalForm], " "}], TraditionalForm]]],
 ":\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox["u", "v"], ")"}],
      TraditionalForm], "  ", "+", " ", 
     FormBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{"-", "u"}], "v"], ")"}],
      TraditionalForm]}], "  ", "=", 
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{"-", "u"}], "v"], ")"}],
       TraditionalForm], " ", "+", " ", 
      FormBox[
       RowBox[{"(", 
        FractionBox["u", "v"], ")"}],
       TraditionalForm]}], "  ", "=", " ", 
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{
          RowBox[{
           RowBox[{"-", "u"}], "\[CenterDot]", "v"}], " ", "+", " ", 
          RowBox[{"u", "\[CenterDot]", "v"}]}], 
         RowBox[{"v", "\[CenterDot]", "v"}]], ")"}],
       TraditionalForm], " ", "=", " ", 
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox["0", 
          RowBox[{"v", "\[CenterDot]", "v"}]], ")"}],
        TraditionalForm], " ", "=", " ", 
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox["0", "z"], ")"}],
         TraditionalForm], " ", "."}]}]}]}]}], TraditionalForm]]],
 "\n\nThe associativity of addition is straightforward:\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FormBox[
          RowBox[{"(", 
           FractionBox["x", "y"], ")"}],
          TraditionalForm], "  ", "+", " ", 
         FormBox[
          RowBox[{"(", 
           FractionBox["u", "v"], ")"}],
          TraditionalForm]}], " ", ")"}], " ", "+", " ", 
       FormBox[
        RowBox[{"(", 
         FractionBox["a", "b"], ")"}],
        TraditionalForm]}], "  ", "=", " ", 
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox[
          RowBox[{
           RowBox[{"x", "\[CenterDot]", "v"}], " ", "+", " ", 
           RowBox[{"u", "\[CenterDot]", "y"}]}], 
          RowBox[{"y", "\[CenterDot]", "v"}]], ")"}],
        TraditionalForm], " ", "+", " ", 
       FormBox[
        RowBox[{"(", 
         FractionBox["a", "b"], ")"}],
        TraditionalForm]}]}], "                               ", "\n", 
     "\t\t\t                                        ", 
     RowBox[{
      RowBox[{"=", " ", 
       FormBox[
        RowBox[{"(", 
         FractionBox[
          RowBox[{
           RowBox[{"x", "\[CenterDot]", "v", "\[CenterDot]", "b"}], " ", "+", 
           " ", 
           RowBox[{"u", "\[CenterDot]", "y", "\[CenterDot]", "b"}], " ", "+", 
           " ", 
           RowBox[{"a", "\[CenterDot]", "y", "\[CenterDot]", "v"}]}], 
          RowBox[{"y", "\[CenterDot]", "v", "\[CenterDot]", "b"}]], ")"}],
        TraditionalForm]}], " ", ","}]}]}], TraditionalForm]]],
 "\n    while\n\n   ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox["x", "y"], ")"}],
       TraditionalForm], "  ", "+", " ", 
      RowBox[{"(", 
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox["u", "v"], ")"}],
         TraditionalForm], "  ", "+", " ", 
        FormBox[
         RowBox[{"(", 
          FractionBox["a", "b"], ")"}],
         TraditionalForm]}], " ", ")"}]}], " ", "=", 
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox["x", "y"], ")"}],
       TraditionalForm], "  ", "+", " ", 
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{
          RowBox[{"u", "\[CenterDot]", "b"}], " ", "+", " ", 
          RowBox[{"a", "\[CenterDot]", "v"}]}], 
         RowBox[{"v", "\[CenterDot]", "b"}]], ")"}],
       TraditionalForm]}]}], "                              "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t                                      ", 
    RowBox[{"=", " ", 
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{
          RowBox[{"x", "\[CenterDot]", "v", "\[CenterDot]", "b"}], " ", "+", 
          " ", 
          RowBox[{"u", "\[CenterDot]", "b", "\[CenterDot]", "y"}], " ", "+", 
          " ", 
          RowBox[{"a", "\[CenterDot]", "v", "\[CenterDot]", "y"}]}], 
         RowBox[{"y", "\[CenterDot]", "v", "\[CenterDot]", "b"}]], ")"}],
       TraditionalForm], " ", "."}]}]}], TraditionalForm]}]],
 "\n\nSo ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " forms a group with respect to addition.\n\nNext we look at the properties \
of multiplication. Multiplication is obviously commutative, since\n\n         \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox["x", "y"], ")"}],
      TraditionalForm], "  ", "\[CenterDot]", 
     FormBox[
      RowBox[{"(", 
       FractionBox["u", "v"], ")"}],
      TraditionalForm]}], "  ", "=", " ", 
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{"x", "\[CenterDot]", "u"}], 
        RowBox[{"y", "\[CenterDot]", "v"}]], ")"}],
      TraditionalForm], "  ", "=", " ", 
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{"u", "\[CenterDot]", "x"}], 
         RowBox[{"v", "\[CenterDot]", "y"}]], ")"}],
       TraditionalForm], "  ", "=", " ", 
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox["u", "v"], ")"}],
        TraditionalForm], " ", "\[CenterDot]", 
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox["x", "y"], ")"}],
         TraditionalForm], " ", "."}]}]}]}]}], TraditionalForm]]],
 "\n\nWe also have associativity for multiplication:\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FormBox[
          RowBox[{"(", 
           FractionBox["x", "y"], ")"}],
          TraditionalForm], "  ", "\[CenterDot]", 
         FormBox[
          RowBox[{"(", 
           FractionBox["u", "v"], ")"}],
          TraditionalForm]}], " ", ")"}], " ", "\[CenterDot]", 
       FormBox[
        RowBox[{"(", 
         FractionBox["a", "b"], ")"}],
        TraditionalForm]}], "  ", "=", " ", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox[
           RowBox[{"x", "\[CenterDot]", "u"}], 
           RowBox[{"y", "\[CenterDot]", "v"}]], ")"}],
         TraditionalForm], " ", "\[CenterDot]", " ", 
        FormBox[
         RowBox[{"(", 
          FractionBox["a", "b"], ")"}],
         TraditionalForm]}], " ", "=", " ", 
       FormBox[
        RowBox[{"(", 
         FractionBox[
          RowBox[{"x", "\[CenterDot]", "u", "\[CenterDot]", "a"}], 
          RowBox[{"y", "\[CenterDot]", "v", "\[CenterDot]", "b"}]], ")"}],
        TraditionalForm]}]}], "                              ", "\n", 
     "\t\t\t\t                                ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox["x", "y"], ")"}],
         TraditionalForm], "  ", "\[CenterDot]", " ", 
        FormBox[
         RowBox[{"(", 
          FractionBox[
           RowBox[{"u", "\[CenterDot]", "a"}], 
           RowBox[{"v", "\[CenterDot]", "b"}]], ")"}],
         TraditionalForm]}], "  ", "=", 
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox["x", "y"], ")"}],
         TraditionalForm], " ", "\[CenterDot]", " ", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           FormBox[
            RowBox[{"(", 
             FractionBox["u", "v"], ")"}],
            TraditionalForm], "  ", "\[CenterDot]", 
           FormBox[
            RowBox[{"(", 
             FractionBox["a", "b"], ")"}],
            TraditionalForm]}], " ", ")"}], "."}]}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nThe element ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"(", 
      FractionBox["z", "z"], ")"}],
     TraditionalForm], " "}], TraditionalForm]]],
 " acts as the multiplicative identity for any ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", "\[NotEqual]", "0"}], TraditionalForm]]],
 ".\n\n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{"(", 
       FractionBox["z", "z"], ")"}],
      TraditionalForm], "  ", "\[CenterDot]", " ", 
     FormBox[
      RowBox[{"(", 
       FractionBox["x", "y"], ")"}],
      TraditionalForm]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox["x", "y"], ")"}],
       TraditionalForm], "  ", "\[CenterDot]", " ", 
      FormBox[
       RowBox[{"(", 
        FractionBox["z", "z"], ")"}],
       TraditionalForm]}], "  ", "=", " ", 
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{"x", "\[CenterDot]", "z"}], 
         RowBox[{"y", "\[CenterDot]", "z"}]], ")"}],
       TraditionalForm], "  ", "=", " ", 
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox["x", "y"], ")"}],
        TraditionalForm], " ", "."}]}]}]}], TraditionalForm]]],
 "\n\nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", "0"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"(", 
      FractionBox["x", "y"], ")"}],
     TraditionalForm], "  ", "=", " ", 
    FormBox[
     RowBox[{"(", 
      FractionBox["0", "z"], ")"}],
     TraditionalForm], " "}], TraditionalForm]]],
 ". Otherwise, the multiplicative inverse of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"(", 
      FractionBox["x", "y"], ")"}],
     TraditionalForm], " "}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{"(", 
      FractionBox["y", "x"], ")"}],
     TraditionalForm], " "}], TraditionalForm]]],
 ", since\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox["x", "y"], ")"}],
       TraditionalForm], "  ", "\[CenterDot]", " ", 
      FormBox[
       RowBox[{"(", 
        FractionBox["y", "x"], ")"}],
       TraditionalForm]}], "  ", "=", " ", 
     RowBox[{
      FormBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{"x", "\[CenterDot]", "y"}], 
         RowBox[{"y", "\[CenterDot]", "x"}]], ")"}],
       TraditionalForm], "  ", "=", " ", 
      RowBox[{
       FormBox[
        RowBox[{"(", 
         FractionBox["z", "z"], ")"}],
        TraditionalForm], " ", "."}]}]}]}], TraditionalForm]]],
 "\n\nThus, every non-zero element of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " has a multiplicative inverse. Finally, we have the two distribution laws. \
Because of the commutativity of multiplication, we only need to check one. \
Since\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FormBox[
          RowBox[{"(", 
           FractionBox["u", "v"], ")"}],
          TraditionalForm], "  ", "+", " ", 
         FormBox[
          RowBox[{"(", 
           FractionBox["a", "b"], ")"}],
          TraditionalForm]}], " ", ")"}], " ", "\[CenterDot]", 
       FormBox[
        RowBox[{"(", 
         FractionBox["x", "y"], ")"}],
        TraditionalForm]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox[
           RowBox[{
            RowBox[{"u", "\[CenterDot]", "b"}], " ", "+", " ", 
            RowBox[{"a", "\[CenterDot]", "v"}]}], 
           RowBox[{"v", "\[CenterDot]", "b"}]], ")"}],
         TraditionalForm], "  ", 
        FormBox[
         RowBox[{"\[CenterDot]", 
          RowBox[{"(", 
           FractionBox["x", "y"], ")"}]}],
         TraditionalForm]}], " ", "=", " ", 
       FormBox[
        RowBox[{"(", 
         FractionBox[
          RowBox[{
           RowBox[{"u", "\[CenterDot]", "b", "\[CenterDot]", "x"}], " ", "+", 
           " ", 
           RowBox[{"a", "\[CenterDot]", "v", "\[CenterDot]", "x"}]}], 
          RowBox[{"v", "\[CenterDot]", "b", "\[CenterDot]", "y"}]], ")"}],
        TraditionalForm]}]}], " ", ","}]}], TraditionalForm]]],
 "\nwhile\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox["u", "v"], ")"}],
         TraditionalForm], "\[CenterDot]", 
        RowBox[{"(", 
         FractionBox["x", "y"], ")"}]}], "  ", "+", " ", 
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox["a", "b"], ")"}],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         RowBox[{"(", 
          FractionBox["x", "y"], ")"}],
         TraditionalForm]}]}], " ", " ", "=", " ", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox[
           RowBox[{"u", "\[CenterDot]", "x"}], 
           RowBox[{"v", "\[CenterDot]", "y"}]], ")"}],
         TraditionalForm], " ", "+", 
        FormBox[
         RowBox[{"(", 
          FractionBox[
           RowBox[{"a", "\[CenterDot]", "x"}], 
           RowBox[{"b", "\[CenterDot]", "y"}]], ")"}],
         TraditionalForm]}], "  ", "=", " ", 
       RowBox[{
        FormBox[
         RowBox[{"(", 
          FractionBox[
           RowBox[{
            RowBox[{
            "u", "\[CenterDot]", "x", "\[CenterDot]", "b", "\[CenterDot]", 
             "y"}], " ", "+", " ", 
            RowBox[{
            "a", "\[CenterDot]", "x", "\[CenterDot]", "v", "\[CenterDot]", 
             "y"}]}], 
           RowBox[{
           "v", "\[CenterDot]", "y", "\[CenterDot]", "b", "\[CenterDot]", 
            "y"}]], ")"}],
         TraditionalForm], "  ", "=", 
        RowBox[{"(", 
         FractionBox[
          RowBox[{
           RowBox[{"u", "\[CenterDot]", "x", "\[CenterDot]", "b"}], " ", "+", 
           " ", 
           RowBox[{"a", "\[CenterDot]", "x", "\[CenterDot]", "v"}]}], 
          RowBox[{"v", "\[CenterDot]", "b", "\[CenterDot]", "y"}]], 
         ")"}]}]}]}], " ", ","}]}], TraditionalForm]]],
 "\n\nwe have the distributive laws holding, and therefore ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " is a field."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417252247608747`*^9, 3.4417252310264997`*^9}, {
  3.4417252634171247`*^9, 3.4417253563858747`*^9}, {3.4417253865733747`*^9, 
  3.4417255329483747`*^9}}]
}, Open  ]],

Cell[TextData[{
 "Notice that in the construction of the field ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", we never used the identity element of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". Hence, if we started with a commutative ring without zero divisors \
instead of an integral domain, the construction would still produce a field. \
We can mention this as a corollary.\n\nCOROLLARY 11.1\nLet ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be any commutative ring with no zero divisors. Then the set of quotients \
",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " defined above forms a field.\n\nAlthough the field of quotients was \
designed from the way we formed rational numbers from the set of integers, we \
can apply the field of quotients to any other integral domain. The only other \
integral domains that we have worked with so far are the polynomial rings \
from the last section. What happens if we form a field of quotients for the \
polynomial ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 "? \n\nLet us first consider the most familar polynomial ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", "x", "]"}], TraditionalForm]]],
 "\[LongDash]the polynomials with integer coefficients. An element in the \
field of quotients would be of the form\n\n                               ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    FractionBox[
     RowBox[{"p", "(", "x", ")"}], 
     RowBox[{"q", "(", "x", ")"}]],
    FontSize->16], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " are polynomials with integer coefficients. But we consider two such \
fractions \n\n                             ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    FractionBox[
     RowBox[{"p", "(", "x", ")"}], 
     RowBox[{"q", "(", "x", ")"}]],
    FontSize->16], TraditionalForm]]],
 "   and   ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{"r", "(", "x", ")"}], 
    RowBox[{"s", "(", "x", ")"}]], TraditionalForm]],
  FontSize->16],
 "\n\nto be equivalent if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"p", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"s", "(", "x", ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"r", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"q", "(", "x", ")"}]}]}], TraditionalForm]]],
 ". Thus, if there are any common factors between ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 ", we can cancel them out to produce a simpler, equivalent fraction. \n\nFor \
example, the fractions "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43739002296875*^9, 3.437390029546875*^9}, {
  3.437483327277625*^9, 3.437483327762*^9}}],

Cell[CellGroupData[{

Cell["\<\
ClearDefs;
A = (3 x^2 + 5 x - 2) / (2 x^2 + 7 x + 6)\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437390030421875*^9, 3.43739003175*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"-", "2"}], "+", 
   RowBox[{"5", " ", "x"}], "+", 
   RowBox[{"3", " ", 
    SuperscriptBox["x", "2"]}]}], 
  RowBox[{"6", "+", 
   RowBox[{"7", " ", "x"}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["x", "2"]}]}]]], "Output",
 CellChangeTimes->{3.437483381090125*^9, 3.442754170961375*^9}]
}, Open  ]],

Cell["and ", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["B = (3 x^2 - 4 x + 1) / (2 x^2 + x - 3)", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43739003228125*^9, 3.4373900333125*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"1", "-", 
   RowBox[{"4", " ", "x"}], "+", 
   RowBox[{"3", " ", 
    SuperscriptBox["x", "2"]}]}], 
  RowBox[{
   RowBox[{"-", "3"}], "+", "x", "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["x", "2"]}]}]]], "Output",
 CellChangeTimes->{3.437483383027625*^9, 3.442754173352*^9}]
}, Open  ]],

Cell["can be seen to be equivalent, since", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Expand[(3 x^2 + 5 x - 2) * (2 x^2 + x - 3)]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437390033828125*^9, 3.437390035046875*^9}}],

Cell[BoxData[
 RowBox[{"6", "-", 
  RowBox[{"17", " ", "x"}], "-", 
  RowBox[{"8", " ", 
   SuperscriptBox["x", "2"]}], "+", 
  RowBox[{"13", " ", 
   SuperscriptBox["x", "3"]}], "+", 
  RowBox[{"6", " ", 
   SuperscriptBox["x", "4"]}]}]], "Output",
 CellChangeTimes->{3.437483386058875*^9, 3.442754175242625*^9}]
}, Open  ]],

Cell["and ", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Expand[(3 x^2 - 4 x + 1) * (2 x^2 + 7 x + 6)]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437390035453125*^9, 3.4373900365625*^9}}],

Cell[BoxData[
 RowBox[{"6", "-", 
  RowBox[{"17", " ", "x"}], "-", 
  RowBox[{"8", " ", 
   SuperscriptBox["x", "2"]}], "+", 
  RowBox[{"13", " ", 
   SuperscriptBox["x", "3"]}], "+", 
  RowBox[{"6", " ", 
   SuperscriptBox["x", "4"]}]}]], "Output",
 CellChangeTimes->{3.43748338848075*^9, 3.442754176742625*^9}]
}, Open  ]],

Cell[TextData[{
 "are the same.\n\nEXPERIMENT:\nCan you find a simpler polynomial that is \
equivalent to ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43739003715625*^9, 3.4373900409375*^9}}],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We call the field of quotients for the polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", "x", "]"}], TraditionalForm]]],
 " the ",
 StyleBox["field of rational functions in",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "(", "x", ")"}], "."}], 
   TraditionalForm]]],
 " \n\nHere two other ways to verify that the two polynomials are \
equivalent:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437390044953125*^9, 3.43739004615625*^9}}],

Cell[CellGroupData[{

Cell["Simplify[A - B]", "Input",
 PageWidth->Infinity],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.437483395637*^9, 3.442754178961375*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simplify[A/B]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437483373027625*^9, 3.437483374183875*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.437483397012*^9, 3.44275418019575*^9}]
}, Open  ]],

Cell[TextData[{
 "The first command obviously shows that ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " are equal. The second command shows that the quotient of the two \
polynomials is the identity element, hence ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " must represent the same element in the field. \n\nIt should be mentioned \
that a rational function, in this context, is not a function! The rational \
functions ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " are merely ",
 StyleBox["elements",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "(", "x", ")"}], TraditionalForm]]],
 ", which may in turn be arguments for some homomorphism. To say that \"",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " is undefined when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{"-", "2"}]}], TraditionalForm]]],
 "\" or \"",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " is undefined at ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "1"}], TraditionalForm]]],
 "\" is meaningless, since ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is not a variable for which numbers can be plugged in. Rather, ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is merely a symbol that is used as a place holder. This is why we can say \
that ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " are truely equal, even though the \"graphs\" would disagree at two points.\
\n\nWe can form rational functions from any integral domain ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". This produces the field ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "(", "x", ")"}], TraditionalForm]]],
 ", the ",
 StyleBox["rational functions in",
  FontSlant->"Italic"],
 "  ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " ",
 StyleBox["over",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ".\n\nFor example, let us use the field of order 9 that was discovered in \
the last section."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437390052609375*^9, 3.437390057234375*^9}, 
   3.437483414996375*^9, 3.4417295510577497`*^9}],

Cell[CellGroupData[{

Cell["\<\
InitDomain[3];
Define[i^2,-1];
K = Ring[{i}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1", ",", "2", ",", "i", ",", 
   RowBox[{"2", " ", "i"}], ",", 
   RowBox[{"1", "+", "i"}], ",", 
   RowBox[{"2", "+", "i"}], ",", 
   RowBox[{"1", "+", 
    RowBox[{"2", " ", "i"}]}], ",", 
   RowBox[{"2", "+", 
    RowBox[{"2", " ", "i"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.43748352129325*^9, 3.44275418275825*^9}]
}, Open  ]],

Cell[TextData[{
 "Here is a typical rational function in ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " over ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437390057953125*^9, 3.437390057953125*^9}}],

Cell[CellGroupData[{

Cell["A = (i x^2 + x^2 + 2 x i + 2 x + 2)/(x^2 + i x + 1)", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43739005859375*^9, 3.437390060609375*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"2", "+", 
   RowBox[{"2", " ", "x"}], "+", 
   RowBox[{"2", " ", "i", " ", "x"}], "+", 
   SuperscriptBox["x", "2"], "+", 
   RowBox[{"i", " ", 
    SuperscriptBox["x", "2"]}]}], 
  RowBox[{"1", "+", 
   RowBox[{"i", " ", "x"}], "+", 
   SuperscriptBox["x", "2"]}]]], "Output",
 CellChangeTimes->{3.437483523652625*^9, 3.43748358629325*^9, 
  3.44275418413325*^9}]
}, Open  ]],

Cell[TextData[{
 "We can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " factor this over a finite field ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Factor[A,K]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"i", "+", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"i", "+", "x", "+", 
     RowBox[{"i", " ", "x"}]}], ")"}]}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{"2", " ", "i"}], "+", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "+", 
     RowBox[{"2", " ", "i"}], "+", "x"}], ")"}]}]]], "Output",
 CellChangeTimes->{3.43748353366825*^9, 3.43748358860575*^9, 
  3.44275418550825*^9}]
}, Open  ]],

Cell[TextData[{
 "According to this factorization, the rational function ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " does not simplify. Or does it? Consider a simplier rational function."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["B = (2 x - i) / (x - i x + i)", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437390072015625*^9, 3.437390072765625*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"2", " ", "i"}], "+", 
   RowBox[{"2", " ", "x"}]}], 
  RowBox[{"i", "+", "x", "+", 
   RowBox[{"2", " ", "i", " ", "x"}]}]]], "Output",
 CellChangeTimes->{3.437483544433875*^9, 3.437483590746375*^9, 
  3.44275418663325*^9}]
}, Open  ]],

Cell[TextData[{
 "Looking at the quotient of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Factor[A/B,K]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4374835514495*^9, 3.437483597887*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"i", "+", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"i", "+", "x", "+", 
     RowBox[{"i", " ", "x"}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"i", "+", "x", "+", 
     RowBox[{"2", " ", "i", " ", "x"}]}], ")"}]}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{"2", " ", "i"}], "+", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "+", 
     RowBox[{"2", " ", "i"}], "+", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", "i"}], "+", 
     RowBox[{"2", " ", "x"}]}], ")"}]}]]], "Output",
 CellChangeTimes->{{3.43748355448075*^9, 3.43748359835575*^9}, 
   3.442754187805125*^9}]
}, Open  ]],

Cell[TextData[{
 "tells us nothing, for this looks hopelessly complex. But if we simplify the \
difference between ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ","
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Simplify[A-B]", "Input",
 PageWidth->Infinity],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.437483608933875*^9, 3.44275418913325*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " shows us that these two expressions are the same rational function in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "(", "x", ")"}], TraditionalForm]]],
 "! \n\nEXPERIMENT:\nIs there another way to factor the numerator and \
denominator of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " so that cancelation occurs, leaving the fraction ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 "? This factorization has to be done by hand, but can be checked using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".\n\nHint: You already have one of the factors for both the numerator and \
denominator. It also might help to look at the multiplication table for ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437390083421875*^9, 3.437390083421875*^9}}],

Cell[CellGroupData[{

Cell["MultTable[K]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.8, 0.6, 0.5}, {1, 1, 0}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {
      0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {0, 1, 
      0}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 
      0.4, 0.8}, {0.8, 0.6, 0.5}, {0, 1, 1}, {0, 1, 0}, {1, 0, 0}, {1, 0, 
      1}}, {{1, 0.6, 0}, {1, 1, 0}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 
      0.5}, {0.6, 0.4, 0.8}, {1, 0, 1}, {1, 0, 0}, {0, 1, 0}, {0, 1, 1}}, {{
      0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 
      0}}, {{0, 1, 0}, {1, 1, 0}, {0, 1, 0}, {1, 0, 0}, {1, 0, 1}, {0, 1, 
      1}, {0.7, 0.8, 0.7}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {1, 0.6, 0}}, {{
      1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {0, 1, 0}, {0, 1, 1}, {1, 0, 1}, {
      1, 0.6, 0}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {0.7, 0.8, 0.7}}, {{0, 1, 
      1}, {1, 1, 0}, {0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {1, 0, 0}, {0.8, 0.6, 
      0.5}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}}, {{1, 0, 1}, {1, 1,
       0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {
      1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}}, {{1, 1, 0}, {1, 1, 0}, {
     1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}}}], 
   InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+2i\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2i\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 9}, {10, 9}}]}]], "Output",
 CellChangeTimes->{3.442754197867625*^9}]
}, Open  ]],

Cell[" ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "As you can see from this experiment, the definition of the quotient field \
does not depend whether elements in the integral domain can be factored \
uniquely. However, unique factorization is an important property that we will \
study in depth in chapter 12. We will learn that the polynomial ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 " used in the above example really ",
 StyleBox["does",
  FontSlant->"Italic"],
 " have a type of unique factorization, after we have studied the true \
definition of what a unique factorization is. But before we go into this, let \
us look closely at some of the more familiar fields: the rational numbers, \
the real numbers, and the complex numbers. These fields will be the basis for \
defining many other fields, so it is natural to learn the properties of these \
fields before going on to study more complicated fields."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437390091203125*^9, 3.437390091203125*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Complex Numbers", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In this section we will explore the field of complex numbers. Although we \
will develop this field algebraically, the results of this section have \
applications in many different fields, including differential equations.\n\n\
We are somewhat familar with complex numbers from algebra. These are numbers \
of the form  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "+", " ", 
    RowBox[{"b", " ", "i"}]}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " represents the \"square root of negative one.\"  In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", a blackboard style  \[ImaginaryI] is used, but one can enter in a capital \
I. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can perform all algebraic opperations in complex numbers as easily as it \
can with real numbers."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4417296429327497`*^9, 3.4417297092139997`*^9}}],

Cell[CellGroupData[{

Cell["\<\
ClearDefs;
(2 + 3 I) + (4 - I)  \
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.442754613680125*^9, 3.442754617836375*^9}}],

Cell[BoxData[
 RowBox[{"6", "+", 
  RowBox[{"2", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{3.43748366473075*^9, 3.4419704190264997`*^9, 
  3.44275420250825*^9, 3.442754619367625*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["(2 + 3 I) * (4 - I)  ", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"11", "+", 
  RowBox[{"10", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{3.437483673183875*^9, 3.4419704213233747`*^9, 
  3.44275420388325*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["(2 + 3 I) / (4 - I)", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.442754205461375*^9, 3.44275420682075*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["5", "17"], "+", 
  FractionBox[
   RowBox[{"14", " ", "\[ImaginaryI]"}], "17"]}]], "Output",
 CellChangeTimes->{3.437483674808875*^9, 3.4419704231202497`*^9, 
  3.442754207211375*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that even though we used a capital ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " to input the complex number, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " displays this with a special \"\[ImaginaryI]\".  However, in this \
definition it is not at all clear where the \"",
 Cell[BoxData[
  FormBox["\[ImaginaryI]", TraditionalForm]]],
 "\" came from. This gives the complex numbers a rather mysterious quality \
which is compounded by their common misnomer, \"imaginary numbers.\" \n\nWe \
would like to define the complex numbers in a way that is more realistic. \
Instead of considering quantities of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", " ", "+", " ", 
      RowBox[{"b", " ", "i"}]}], ")"}], ","}], TraditionalForm]]],
 " we will consider ordered pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", ",", " ", "b"}], ")"}], TraditionalForm]]],
 ". This identifies the complex numbers with points in the plane, which makes \
them easier to visualize. This representation of complex numbers was first \
discovered by Gauss in his Ph.D. dissertation.\n\nLet us declare the \
following properties for ordered pairs of real numbers:\n\n1)    ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", ",", "b"}], ")"}], " ", "=", " ", 
     RowBox[{"(", 
      RowBox[{"c", ",", "d"}], ")"}]}]}], TraditionalForm]]],
 "  if and only if  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "=", "c"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", "d"}], TraditionalForm]]],
 ".\n\n2)     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", ",", "b"}], ")"}], " ", "+", " ", 
     RowBox[{"(", 
      RowBox[{"c", ",", "d"}], ")"}]}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", "+", "c"}], ",", " ", 
      RowBox[{"b", "+", "d"}]}], ")"}]}], TraditionalForm]]],
 ".\n\n3)     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"c", ",", "d"}], ")"}]}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"a", "\[CenterDot]", "c"}], " ", "-", " ", 
       RowBox[{"b", "\[CenterDot]", "d"}]}], ",", " ", 
      RowBox[{
       RowBox[{"a", "\[CenterDot]", "d"}], " ", "+", " ", 
       RowBox[{"b", "\[CenterDot]", "c"}]}]}], ")"}]}], TraditionalForm]]],
 ".\n\nWe define ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 " to be the set of all ordered pairs of real numbers. Using these three \
properties, it is not hard to prove the following:\n\nPROPOSITION 11.3\nThe \
set ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 " forms a field, called the ",
 StyleBox["field of complex numbers",
  FontSlant->"Italic"],
 ". This field contains a subfield isomorphic to the real numbers."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43748367979325*^9, 3.437483767027625*^9}, {
  3.437483839465125*^9, 3.437483839965125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Because the real numbers are closed with respect to both addition and \
multiplication, it is clear that both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{"a", "+", "c"}], ",", " ", 
     RowBox[{"b", "+", "d"}]}], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "\[CenterDot]", "c"}], " ", "-", " ", 
      RowBox[{"b", "\[CenterDot]", "d"}]}], ",", " ", 
     RowBox[{
      RowBox[{"a", "\[CenterDot]", "d"}], " ", "+", " ", 
      RowBox[{"b", "\[CenterDot]", "c"}]}]}], ")"}], TraditionalForm]]],
 " would be defined for all real numbers ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 " is closed with respect to both addition and multiplication. Furthermore, \
since\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"c", ",", "d"}], ")"}], " ", "+", " ", 
     RowBox[{"(", 
      RowBox[{"a", ",", "b"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"c", "+", "a"}], ",", " ", 
       RowBox[{"d", "+", "b"}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"a", "+", "c"}], ",", " ", 
        RowBox[{"b", "+", "d"}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", ",", "b"}], ")"}], " ", "+", " ", 
       RowBox[{"(", 
        RowBox[{"c", ",", "d"}], ")"}]}]}]}]}], TraditionalForm]]],
 "\nand\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"c", ",", "d"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"a", ",", "b"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"c", "\[CenterDot]", "a"}], " ", "-", " ", 
          RowBox[{"d", "\[CenterDot]", "b"}]}], ",", " ", 
         RowBox[{
          RowBox[{"c", "\[CenterDot]", "b"}], " ", "+", " ", 
          RowBox[{"d", "\[CenterDot]", "a"}]}]}], ")"}], " ", "=", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"a", "\[CenterDot]", "c"}], " ", "-", " ", 
          RowBox[{"b", "\[CenterDot]", "d"}]}], ",", " ", 
         RowBox[{
          RowBox[{"a", "\[CenterDot]", "d"}], " ", "+", " ", 
          RowBox[{"b", "\[CenterDot]", "c"}]}]}], ")"}]}]}], "      ", "\n", 
     "\t\t\t\t            ", 
     RowBox[{
      RowBox[{"=", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"c", ",", "d"}], ")"}]}]}], ","}]}]}], TraditionalForm]]],
 "\n\nwe see that both addition and multiplication are commutative. The \
element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"0", ",", "0"}], ")"}], TraditionalForm]]],
 " acts as the zero element, since\n\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"0", ",", "0"}], ")"}], " ", "+", " ", 
     RowBox[{"(", 
      RowBox[{"a", ",", "b"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", ",", "b"}], ")"}], "."}]}], TraditionalForm]]],
 " \n\nThe addition inverse of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "a"}], ",", 
      RowBox[{"-", "b"}]}], ")"}], ","}], TraditionalForm]]],
 " since\n\n                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", ",", "b"}], ")"}], " ", "+", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "a"}], ",", 
       RowBox[{"-", "b"}]}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"0", ",", "0"}], ")"}], "."}]}], TraditionalForm]]],
 "\n\nNote that the order on the last two sums is irrelevant, since addition \
has already been shown to be commutative.\n\nTo show that addition is \
associative, we note that\n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", ",", "b"}], ")"}], " ", "+", " ", 
      RowBox[{"[", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"c", ",", "d"}], ")"}], " ", "+", " ", 
        RowBox[{"(", 
         RowBox[{"e", ",", "f"}], ")"}]}], " ", "]"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", ",", "b"}], ")"}], " ", "+", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"c", "+", "e"}], ",", " ", 
         RowBox[{"d", "+", "f"}]}], ")"}]}], " ", "=", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"a", "+", "c", "+", "e"}], ",", " ", 
        RowBox[{"b", "+", "d", "+", "f"}]}], ")"}]}]}], ","}], 
   TraditionalForm]]],
 "\nwhile\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"[", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", ",", "b"}], ")"}], " ", "+", " ", 
       RowBox[{"(", 
        RowBox[{"c", ",", "d"}], ")"}]}], " ", "]"}], " ", "+", " ", 
     RowBox[{"(", 
      RowBox[{"e", ",", "f"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"a", "+", "c"}], ",", " ", 
        RowBox[{"b", "+", "d"}]}], ")"}], " ", "+", " ", 
      RowBox[{"(", 
       RowBox[{"e", ",", "f"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"a", "+", "c", "+", "e"}], ",", " ", 
        RowBox[{"b", "+", "d", "+", "f"}]}], ")"}], "."}]}]}], 
   TraditionalForm]]],
 "\n\nTo show that multiplication is associative is a little more \
complicated: we have\n\n ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
      RowBox[{"[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"c", ",", "d"}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"e", ",", "f"}], ")"}]}], "]"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"c", "\[CenterDot]", "e"}], " ", "-", " ", 
         RowBox[{"d", "\[CenterDot]", "f"}]}], ",", " ", 
        RowBox[{
         RowBox[{"c", "\[CenterDot]", "f"}], " ", "+", " ", 
         RowBox[{"d", "\[CenterDot]", "e"}]}]}], ")"}]}]}], " "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{"=", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "\[CenterDot]", "c", "\[CenterDot]", "e"}], " ", "-", 
         " ", 
         RowBox[{"a", "\[CenterDot]", "d", "\[CenterDot]", "f"}], " ", "-", 
         RowBox[{"b", "\[CenterDot]", "c", "\[CenterDot]", "f"}], " ", "-", 
         " ", 
         RowBox[{"b", "\[CenterDot]", "d", "\[CenterDot]", "e"}]}], ",", " ", 
        
        RowBox[{
         RowBox[{"a", "\[CenterDot]", "c", "\[CenterDot]", "f"}], " ", "+", 
         " ", 
         RowBox[{"a", "\[CenterDot]", "d", "\[CenterDot]", "e"}], " ", "+", 
         " ", 
         RowBox[{"b", "\[CenterDot]", "c", "\[CenterDot]", "e"}], " ", "-", 
         " ", 
         RowBox[{"b", "\[CenterDot]", "d", "\[CenterDot]", "f"}]}]}], ")"}]}],
      ","}]}], TraditionalForm]}]],
 "\n\nand \n\n ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"[", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"c", ",", "d"}], ")"}]}], "]"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"e", ",", "f"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "\[CenterDot]", "c"}], " ", "-", " ", 
         RowBox[{"b", "\[CenterDot]", "d"}]}], ",", " ", 
        RowBox[{
         RowBox[{"a", "\[CenterDot]", "d"}], " ", "+", " ", 
         RowBox[{"b", "\[CenterDot]", "c"}]}]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"e", ",", "f"}], ")"}]}]}], " "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "\[CenterDot]", "c", "\[CenterDot]", "e"}], " ", "-", 
         " ", 
         RowBox[{"b", "\[CenterDot]", "d", "\[CenterDot]", "e"}], " ", "-", 
         " ", 
         RowBox[{"a", "\[CenterDot]", "d", "\[CenterDot]", "f"}], " ", "-", 
         " ", 
         RowBox[{"b", "\[CenterDot]", "c", "\[CenterDot]", "f"}]}], ",", " ", 
        
        RowBox[{
         RowBox[{"a", "\[CenterDot]", "c", "\[CenterDot]", "f"}], " ", "-", 
         " ", 
         RowBox[{"b", "\[CenterDot]", "d", "\[CenterDot]", "f"}], " ", "+", 
         " ", 
         RowBox[{"a", "\[CenterDot]", "d", "\[CenterDot]", "e"}], " ", "+", 
         " ", 
         RowBox[{"b", "\[CenterDot]", "c", "\[CenterDot]", "e"}]}]}], ")"}], 
      "."}]}]}], TraditionalForm]}]],
 "\n\nBy comparing these two, we see that they are equal, so multiplication \
is associative. \n\nWe need to test the distributive laws next. The left \
distributive law we can get by expanding:\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"c", ",", "d"}], ")"}], " ", "+", " ", 
         RowBox[{"(", 
          RowBox[{"e", ",", "f"}], ")"}]}], "]"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"c", "+", "e"}], ",", " ", 
         RowBox[{"d", "+", "f"}]}], ")"}]}]}], "                         ", 
     "\n", "\t\t                                   \t", 
     RowBox[{"=", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "\[CenterDot]", "c"}], " ", "+", " ", 
         RowBox[{"a", "\[CenterDot]", "e"}], " ", "-", " ", 
         RowBox[{"b", "\[CenterDot]", "d"}], " ", "-", " ", 
         RowBox[{"b", "\[CenterDot]", "f"}]}], ",", " ", 
        RowBox[{
         RowBox[{"a", "\[CenterDot]", "d"}], " ", "+", " ", 
         RowBox[{"a", "\[CenterDot]", "f"}], " ", "+", " ", 
         RowBox[{"b", "\[CenterDot]", "c"}], " ", "+", " ", 
         RowBox[{"b", "\[CenterDot]", "e"}]}]}], ")"}]}], 
     "                        ", "\n", 
     "\t\t\t \t                              ", 
     RowBox[{"=", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"a", "\[CenterDot]", "c"}], " ", "-", " ", 
          RowBox[{"b", "\[CenterDot]", "d"}]}], ",", " ", 
         RowBox[{
          RowBox[{"a", "\[CenterDot]", "d"}], " ", "+", " ", 
          RowBox[{"b", "\[CenterDot]", "c"}]}]}], ")"}], " ", "+", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"a", "\[CenterDot]", "e"}], " ", "-", " ", 
          RowBox[{"b", "\[CenterDot]", "f"}]}], ",", " ", 
         RowBox[{
          RowBox[{"a", "\[CenterDot]", "f"}], " ", "+", " ", 
          RowBox[{"b", "\[CenterDot]", "e"}]}]}], ")"}]}]}], "          ", 
     "\n", "\t\t\t\t                            \t", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"c", ",", "d"}], ")"}]}], " ", "+", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"e", ",", "f"}], ")"}], "."}]}]}]}], "   "}]}], 
   TraditionalForm]]],
 "\n\nThus, the left distributive law is satisfied. However, the right \
distributive law follows from the left distributive law, and using the \
commutative multiplication:\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"a", ",", "b"}], ")"}], " ", "+", " ", 
         RowBox[{"(", 
          RowBox[{"c", ",", "d"}], ")"}]}], "]"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"e", ",", "f"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"e", ",", "f"}], ")"}], "\[CenterDot]", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"a", ",", "b"}], ")"}], " ", "+", " ", 
         RowBox[{"(", 
          RowBox[{"c", ",", "d"}], ")"}]}], "]"}]}]}], " ", "\n", 
     "\t                                 \t\t  ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"e", ",", "f"}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"a", ",", "b"}], ")"}]}], " ", "+", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"e", ",", "f"}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"c", ",", "d"}], ")"}]}]}]}], "     ", "\n", 
     "\t\t\t\t                                ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"e", ",", "f"}], ")"}]}], " ", "+", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"c", ",", "d"}], ")"}], "\[CenterDot]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"e", ",", "f"}], ")"}], "."}]}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nWe have now shown that the set ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 " forms a commutative ring. To show that this ring has a multiplicative \
identity, we consider the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", ",", "0"}], ")"}], "."}], TraditionalForm]]],
 " Since the ring is commutative, we only need to check \n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"a", ",", "b"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"1", "\[CenterDot]", "a"}], " ", "-", " ", 
         RowBox[{"0", "\[CenterDot]", "b"}]}], ",", " ", 
        RowBox[{
         RowBox[{"1", "\[CenterDot]", "b"}], " ", "+", " ", 
         RowBox[{"0", "\[CenterDot]", "a"}]}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", ",", "b"}], ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nFinally, we need to show that every non-zero element has an inverse. If \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 " is non-zero, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", "2"], " ", "+", " ", 
    SuperscriptBox["b", "2"]}], TraditionalForm]]],
 " will be a positive number. Hence\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     FractionBox["a", 
      RowBox[{
       SuperscriptBox["a", "2"], "+", " ", 
       SuperscriptBox["b", "2"]}]], " ", ",", " ", 
     FractionBox[
      RowBox[{"-", "b"}], 
      RowBox[{
       SuperscriptBox["a", "2"], "+", " ", 
       SuperscriptBox["b", "2"]}]]}], " ", ")"}], TraditionalForm]]],
 "\n\nis an element of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ". The product\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
      FormBox[
       RowBox[{"(", 
        RowBox[{
         FractionBox["a", 
          RowBox[{
           SuperscriptBox["a", "2"], "+", " ", 
           SuperscriptBox["b", "2"]}]], " ", ",", " ", 
         FractionBox[
          RowBox[{"-", "b"}], 
          RowBox[{
           SuperscriptBox["a", "2"], "+", " ", 
           SuperscriptBox["b", "2"]}]]}], " ", ")"}],
       TraditionalForm]}], "=", " ", 
     RowBox[{
      FormBox[
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           SuperscriptBox["a", "2"], "+", " ", 
           SuperscriptBox["b", "2"]}], 
          RowBox[{
           SuperscriptBox["a", "2"], "+", " ", 
           SuperscriptBox["b", "2"]}]], " ", ",", " ", 
         FractionBox[
          RowBox[{
           RowBox[{
            RowBox[{"-", "a"}], "\[CenterDot]", "b"}], " ", "+", " ", 
           RowBox[{"a", "\[CenterDot]", "b"}]}], 
          RowBox[{
           SuperscriptBox["a", "2"], "+", " ", 
           SuperscriptBox["b", "2"]}]]}], " ", ")"}],
       TraditionalForm], " ", "=", " ", 
      RowBox[{"(", 
       RowBox[{"1", ",", "0"}], ")"}]}]}]}], TraditionalForm]]],
 "\n\nverifies that  \n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"a", ",", "b"}], ")"}], 
      RowBox[{"-", "1"}]],
     TraditionalForm], " ", "=", "  ", 
    FormBox[
     RowBox[{"(", 
      RowBox[{
       FractionBox["a", 
        RowBox[{
         SuperscriptBox["a", "2"], "+", " ", 
         SuperscriptBox["b", "2"]}]], " ", ",", " ", 
       FractionBox[
        RowBox[{"-", "b"}], 
        RowBox[{
         SuperscriptBox["a", "2"], "+", " ", 
         SuperscriptBox["b", "2"]}]]}], " ", ")"}],
     TraditionalForm]}], TraditionalForm]]],
 " \n                         \nsince multiplication is commutative. \
Therefore, the set \[DoubleStruckCapitalC] forms a field.\n\nThe second part \
of this proposition is to show that ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 " contains a copy of the real numbers as a subfield. Consider the mapping ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " which maps real numbers to ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ", given by\n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", ",", "0"}], ")"}], "."}]}]}], TraditionalForm]]],
 "\n\nTo check that ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is a homomorphism, we check that\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Florin]", "(", "x", ")"}], " ", "+", " ", 
     RowBox[{"\[Florin]", "(", "y", ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", ",", "0"}], ")"}], " ", "+", " ", 
      RowBox[{"(", 
       RowBox[{"y", ",", "0"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"x", " ", "+", " ", "y"}], ",", "0"}], ")"}], " ", "=", " ", 
      RowBox[{"\[Florin]", "(", 
       RowBox[{"x", " ", "+", " ", "y"}], ")"}]}]}]}], TraditionalForm]]],
 "\nand\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Florin]", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"\[Florin]", "(", "y", ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", ",", "0"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"y", ",", "0"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "y"}], " ", "+", " ", "0"}], ",", " ", 
        RowBox[{"0", " ", "+", " ", "0"}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"x", "\[CenterDot]", "y"}], ",", "0"}], ")"}], " ", "=", " ",
        
       RowBox[{
        RowBox[{"\[Florin]", "(", 
         RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is a homomorphism from the reals to ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ". It is clear that ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is one-to-one, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", ",", "0"}], ")"}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"y", ",", "0"}], ")"}]}], TraditionalForm]]],
 " if and only if",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"x", " ", "=", " ", "y"}]}], TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an embedding of the reals into ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ", and thus the image of ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ":\n\n                                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", ",", "0"}], ")"}], "  ", "|", "  ", 
     RowBox[{"x", " ", "\[Element]", " ", "\[DoubleStruckCapitalR]"}]}], " ", 
    "}"}], TraditionalForm]]],
 "\n                      \nis isomorphic to the real numbers."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4374844259495*^9, 3.4374844259495*^9}}]
}, Open  ]],

Cell[TextData[{
 "The purpose of constructing the complex numbers was to produce a field for \
which we can take the square root of negative one. We can now show that we \
have succeeded in doing this.\n\nLEMMA 11.4\nThere are exactly two solutions \
to the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "0"}], ")"}]}], TraditionalForm]]],
 " in the field ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ", given by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"0", ",", 
     RowBox[{"\[PlusMinus]", "1"}]}], ")"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4374844374495*^9, 3.4374844375745*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 " solves the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "0"}], ")"}]}], TraditionalForm]]],
 ", we have that \n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"a", ",", "b"}], ")"}], "2"], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["a", "2"], " ", "-", " ", 
         SuperscriptBox["b", "2"]}], ",", " ", 
        RowBox[{"2", " ", 
         RowBox[{"a", "\[CenterDot]", "b"}]}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "0"}], ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " must satisfy the two equations\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"a", " "}], "2"], " ", "-", " ", 
     SuperscriptBox["b", "2"]}], " ", "=", " ", 
    RowBox[{"-", "1"}]}], TraditionalForm]]],
 ",\nand\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"a", "\[CenterDot]", "b"}]}], " ", "=", " ", "0"}]}], 
   TraditionalForm]]],
 ".\n\nThe second equation implies that either ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " must be 0. But if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", "0"}], TraditionalForm]]],
 ", then the first equation becomes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", "2"], " ", "=", " ", 
    RowBox[{"-", "1"}]}], TraditionalForm]]],
 ", which has no real solutions. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", "0"}], TraditionalForm]]],
 ", and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", " ", 
     SuperscriptBox["b", "2"]}], " ", "=", " ", 
    RowBox[{"-", "1"}]}], TraditionalForm]]],
 ". There are two real solutions for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", ":"}], TraditionalForm]]],
 "  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PlusMinus]", "1"}], TraditionalForm]]],
 ". Thus,\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"0", ",", "1"}], ")"}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"0", ",", 
     RowBox[{"-", "1"}]}], ")"}], TraditionalForm]]],
 "\n\nboth solve the equations for ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and so \n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}], "2"], " ", "=", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"0", ",", 
         RowBox[{"-", "1"}]}], ")"}], "2"], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "0"}], ")"}], "."}]}]}]}], 
   TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43748462279325*^9, 3.43748463066825*^9}}]
}, Open  ]],

Cell[TextData[{
 "By defining the complex numbers as ordered pairs, we have in a sence taken \
some of the mystery out of the complex numbers. Lemma 11.4 shows that the \
square root of negative one comes as a natural consequence of the way we \
defined the product.\nWe can now convert ordered pairs to the customary \
notation by defining ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"0", ",", "1"}], ")"}]}], TraditionalForm]]],
 ", and identifying the identity element (1,0) with 1.\nThen any complex \
number ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 " can be written\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", ",", "b"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", ",", "0"}], ")"}], " ", "+", " ", 
      RowBox[{"(", 
       RowBox[{"0", ",", "b"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"a", "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"1", ",", "0"}], ")"}]}], " ", "+", " ", 
       RowBox[{"b", "\[CenterDot]", " ", 
        RowBox[{"(", 
         RowBox[{"0", ",", "1"}], ")"}]}]}], " ", "=", " ", 
      RowBox[{"a", " ", "+", " ", 
       RowBox[{"b", " ", 
        RowBox[{"i", "."}]}]}]}]}]}], TraditionalForm]]],
 "\nWe can rewrite the rules for addition and multiplication in \
\[DoubleStruckCapitalC] as follows:\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", "+", " ", 
       RowBox[{"b", " ", "i"}]}], ")"}], " ", "+", " ", 
     RowBox[{"(", 
      RowBox[{"c", " ", "+", " ", 
       RowBox[{"d", " ", "i"}]}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", "+", " ", "c"}], ")"}], " ", "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"b", " ", "+", " ", "d"}], ")"}], " ", "i"}]}]}], 
   TraditionalForm]]],
 ".\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", " ", "+", " ", 
       RowBox[{"b", " ", "i"}]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"c", " ", "+", " ", 
       RowBox[{"d", " ", "i"}]}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", "\[CenterDot]", "c"}], " ", "-", " ", 
       RowBox[{"b", "\[CenterDot]", "d"}]}], ")"}], " ", "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"b", "\[CenterDot]", "c"}], " ", "+", " ", 
        RowBox[{"a", "\[CenterDot]", " ", "d"}]}], ")"}], " ", "i"}]}]}], 
   TraditionalForm]]],
 ".\n\nIn working with groups, we found that the group automorphisms revealed \
many of the important properties of the group.  This will also be true for \
rings.  Let us extend the group automorphisms to apply to rings.\n\n\
DEFINITION 11.5:\nA ",
 StyleBox["ring automorphism",
  FontSlant->"Italic"],
 " is a one-to-one and onto ring homomorphism which maps a ring to itself.\n\n\
LEMMA 11.5\nThe set of all ring automorphisms forms a group."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4375329585*^9, 3.437532966875*^9}, 3.4375332076875*^9, {
   3.43753334178125*^9, 3.437533462515625*^9}, {3.43753456909375*^9, 
   3.437534686859375*^9}, {3.437534808984375*^9, 3.437534813171875*^9}, {
   3.43753495371875*^9, 3.437534976046875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We first note that if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "x", ")"}], TraditionalForm]]],
 " is an automorphism of a ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], TraditionalForm]]],
 " is well defined, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "x", ")"}], TraditionalForm]]],
 " is both one-to-one and onto. We see that\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[Florin]", 
          RowBox[{"-", "1"}]], "(", "x", ")"}], " ", "+", " ", 
        RowBox[{
         SuperscriptBox["\[Florin]", 
          RowBox[{"-", "1"}]], "(", "y", ")"}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Florin]", "(", 
         RowBox[{
          SuperscriptBox["\[Florin]", 
           RowBox[{"-", "1"}]], "(", "x", ")"}], ")"}], " ", "+", " ", 
        RowBox[{"\[Florin]", "(", 
         RowBox[{
          SuperscriptBox["\[Florin]", 
           RowBox[{"-", "1"}]], "(", "y", ")"}], ")"}]}], " ", "=", " ", 
       RowBox[{"x", " ", "+", " ", "y"}]}]}], ","}]}], TraditionalForm]]],
 "\n\nso ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Florin]", 
      RowBox[{"-", "1"}]], "(", 
     RowBox[{"x", " ", "+", " ", "y"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Florin]", 
       RowBox[{"-", "1"}]], "(", "x", ")"}], " ", "+", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Florin]", 
        RowBox[{"-", "1"}]], "(", "y", ")"}], "."}]}]}], TraditionalForm]]],
 " Also, \n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Florin]", "(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[Florin]", 
          RowBox[{"-", "1"}]], "(", "x", ")"}], "\[CenterDot]", 
        RowBox[{
         SuperscriptBox["\[Florin]", 
          RowBox[{"-", "1"}]], "(", "y", ")"}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Florin]", "(", 
         RowBox[{
          SuperscriptBox["\[Florin]", 
           RowBox[{"-", "1"}]], "(", "x", ")"}], ")"}], " ", "\[CenterDot]", 
        " ", 
        RowBox[{"\[Florin]", "(", 
         RowBox[{
          SuperscriptBox["\[Florin]", 
           RowBox[{"-", "1"}]], "(", "y", ")"}], ")"}]}], " ", "=", " ", 
       RowBox[{"x", "\[CenterDot]", "y"}]}]}], ","}]}], TraditionalForm]]],
 "\n\nso ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Florin]", 
      RowBox[{"-", "1"}]], "(", 
     RowBox[{"x", "\[CenterDot]", "y"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Florin]", 
       RowBox[{"-", "1"}]], "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Florin]", 
        RowBox[{"-", "1"}]], "(", "y", ")"}], "."}]}]}], TraditionalForm]]],
 " Thus, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Florin]", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " is a ring homomorphism. Since ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " was both one-to-one and onto, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Florin]", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " is both one-to-one and onto. Therefore, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Florin]", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 " is a ring automorphism. \n\nIf ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " are two ring automorphisms, then \n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{"\[Phi]", "(", 
       RowBox[{"x", " ", "+", " ", "y"}], ")"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"\[Florin]", "(", 
       RowBox[{
        RowBox[{"\[Phi]", "(", "x", ")"}], " ", "+", " ", 
        RowBox[{"\[Phi]", "(", "y", ")"}]}], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"\[Florin]", "(", 
        RowBox[{"\[Phi]", "(", "x", ")"}], ")"}], " ", "+", " ", 
       RowBox[{"\[Florin]", "(", 
        RowBox[{"\[Phi]", "(", "y", ")"}], ")"}]}]}]}]}], TraditionalForm]]],
 "\nand\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"\[Phi]", "(", 
      RowBox[{"x", "\[CenterDot]", "y"}], ")"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{
       RowBox[{"\[Phi]", "(", "x", ")"}], " ", "\[CenterDot]", " ", 
       RowBox[{"\[Phi]", "(", "y", ")"}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"\[Florin]", "(", 
       RowBox[{"\[Phi]", "(", "x", ")"}], ")"}], " ", "\[CenterDot]", " ", 
      RowBox[{
       RowBox[{"\[Florin]", "(", 
        RowBox[{"\[Phi]", "(", "y", ")"}], ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\n\nThe combination ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", 
    RowBox[{"\[Phi]", "(", "x", ")"}], ")"}], TraditionalForm]]],
 " is also one-to-one and onto, so this product, which we can denote ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "\[CenterDot]", "\[Phi]"}], TraditionalForm]]],
 ", is a ring automorphism. Since the set of all ring automorphisms is closed \
with respect to multiplication and inverses, and the set of all ring \
automorphisms is a subgroup of the set of all ",
 StyleBox["group",
  FontSlant->"Italic"],
 " automorphisms with respect to addition, we see that this set is a group."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 " Thus, it is a natural question to ask whether there are any non-trivial \
ring automorphisms of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ".\n\nPROPOSITION 11.4\nBesides the identity automorphism, there is another \
automorphism on ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ", given by\n \n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Phi]", "[", 
      RowBox[{"(", 
       RowBox[{"a", ",", "b"}], ")"}], "]"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", ",", 
        RowBox[{"-", "b"}]}], ")"}], "."}]}]}], TraditionalForm]]],
 "\nIn fact, these are the only automorphisms for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "x", ")"}], " ", "=", " ", "x"}], 
   TraditionalForm]]],
 " for all real numbers ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43748464891825*^9, 3.43748472410575*^9}, {
   3.43753332053125*^9, 3.437533327203125*^9}, 3.43753473446875*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We check that\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"(", 
         RowBox[{"a", ",", "b"}], ")"}], "]"}], " ", "+", " ", 
       RowBox[{"\[Phi]", "[", 
        RowBox[{"(", 
         RowBox[{"c", ",", "d"}], ")"}], "]"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"a", ",", 
          RowBox[{"-", "b"}]}], ")"}], " ", "+", " ", 
        RowBox[{"(", 
         RowBox[{"c", ",", 
          RowBox[{"-", "d"}]}], ")"}]}], " ", "=", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"a", "+", "c"}], ",", 
         RowBox[{
          RowBox[{"-", "b"}], " ", "-", " ", "d"}]}], ")"}]}]}], 
     "                                ", "\n", 
     "\t\t\t                              ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"a", "+", "c"}], ",", 
          RowBox[{"b", "+", "d"}]}], ")"}], "]"}], " ", "=", " ", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"a", ",", "b"}], ")"}], " ", "+", " ", 
          RowBox[{"(", 
           RowBox[{"c", ",", "d"}], ")"}]}], "]"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"(", 
         RowBox[{"a", ",", "b"}], ")"}], "]"}], " ", "\[CenterDot]", " ", 
       RowBox[{"\[Phi]", "[", 
        RowBox[{"(", 
         RowBox[{"c", ",", "d"}], ")"}], "]"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"a", ",", 
          RowBox[{"-", "b"}]}], ")"}], "\[CenterDot]", 
        RowBox[{"(", 
         RowBox[{"c", ",", 
          RowBox[{"-", "d"}]}], ")"}]}], " ", "=", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"a", "\[CenterDot]", "c"}], " ", "-", " ", 
          RowBox[{"b", "\[CenterDot]", "d"}]}], ",", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "a"}], "\[CenterDot]", "d"}], " ", "-", " ", 
          RowBox[{"b", "\[CenterDot]", "c"}]}]}], ")"}]}]}], "        ", "\n",
      "\t\t\t                            ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"a", "\[CenterDot]", "c"}], " ", "-", " ", 
           RowBox[{"b", "\[CenterDot]", "d"}]}], ",", " ", 
          RowBox[{
           RowBox[{"a", "\[CenterDot]", "d"}], " ", "+", " ", 
           RowBox[{"b", "\[CenterDot]", "c"}]}]}], ")"}], "]"}], " ", "=", 
       " ", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"a", ",", "b"}], ")"}], "\[CenterDot]", 
          RowBox[{"(", 
           RowBox[{"c", ",", "d"}], ")"}]}], "]"}], "."}]}]}], " "}]}], 
   TraditionalForm]]],
 "\n\nThus, \[Phi] is a homomorphism. Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", ",", 
      RowBox[{"-", "b"}]}], ")"}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"0", ",", "0"}], ")"}]}], TraditionalForm]]],
 " if, and only if, ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are both 0, the kernel of \[Phi] is just ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"(", 
     RowBox[{"0", ",", "0"}], ")"}], "}"}], TraditionalForm]]],
 ", and so \[Phi] is one-to-one. Also, \[Phi] is onto, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "[", 
     RowBox[{"(", 
      RowBox[{"a", ",", 
       RowBox[{"-", "b"}]}], ")"}], "]"}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"a", ",", "b"}], ")"}]}], TraditionalForm]]],
 ". Therefore, \[Phi] is an automorphism.\n   To show that there are exactly \
two such automorphisms, suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", 
    RowBox[{"(", "x", ")"}]}], TraditionalForm]]],
 " is an automorphism of \[DoubleStruckCapitalC] for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", "x"}], TraditionalForm]]],
 " for all real numbers ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".  Then  \n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"f", "[", 
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}], "]"}], "2"], "=", " ", 
    RowBox[{
     RowBox[{"f", "[", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"0", ",", "1"}], ")"}], "2"], "]"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"f", "[", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "0"}], ")"}], "]"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "0"}], ")"}], Cell[""]}]}]}]}], 
   TraditionalForm]]],
 ", \nso by lemma 11.4  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"(", 
      RowBox[{"0", ",", "1"}], ")"}], "]"}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"0", ",", 
      RowBox[{"\[PlusMinus]", "1"}]}], ")"}]}], TraditionalForm]]],
 ".  If ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"(", 
      RowBox[{"0", ",", "1"}], ")"}], "]"}], " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"0", ",", "1"}], ")"}]}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", "x"}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[Element]", "\[DoubleStruckCapitalC]"}], 
   TraditionalForm]]],
 ", and if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"(", 
      RowBox[{"0", ",", "1"}], ")"}], "]"}], "=", 
    RowBox[{"(", 
     RowBox[{"0", ",", 
      RowBox[{"-", "1"}]}], ")"}]}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"\[Phi]", "(", "x", ")"}]}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.4374847825745*^9, {3.437484819433875*^9, 3.43748516160575*^9}, {
   3.4374852359495*^9, 3.437485279371375*^9}}]
}, Open  ]],

Cell[TextData[{
 "The automorphism found in proposition 11.4 is called the ",
 StyleBox["conjugate",
  FontSlant->"Italic"],
 ". The conjugate of ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " is generally denoted by ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["z", "_"], TraditionalForm]]],
 ". That is, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"z", " ", "=", " ", 
     RowBox[{"a", " ", "+", " ", 
      RowBox[{"b", " ", "i"}]}]}], ","}], TraditionalForm]]],
 " then\n\n                                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["z", "_"], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Phi]", "(", "z", ")"}], " ", "=", " ", 
     RowBox[{"a", " ", "-", " ", 
      RowBox[{"b", " ", 
       RowBox[{"i", "."}]}]}]}]}], TraditionalForm]]],
 "\n                                 \nWe call an element of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 " to be ",
 StyleBox["real",
  FontSlant->"Italic"],
 " if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    OverscriptBox["z", "_"], " ", "=", " ", "z"}], TraditionalForm]]],
 ". Clearly the real elements of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 " are \n\n                                ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", " ", "+", " ", 
      RowBox[{"0", " ", "i"}]}], "  ", "|", "  ", 
     RowBox[{"x", " ", "\[Element]", " ", "\[DoubleStruckCapitalR]"}]}], " ", 
    "}"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437485310371375*^9, 3.43748531179325*^9}, {
  3.437485345183875*^9, 3.437485366340125*^9}, {3.437534993203125*^9, 
  3.43753499675*^9}}],

Cell[TextData[{
 "These elements are, of course, the image of the embedding found in \
proposition 11.3 of the reals into ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ".\n\nThe conjugate automorphism is defined in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " as"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437485395840125*^9, 3.43748539673075*^9}}],

Cell["Conjugate[z]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.442754588211375*^9, 3.44275459307075*^9}, {
  3.442754631836375*^9, 3.442754634352*^9}}],

Cell[TextData[{
 "so the conjugate of ",
 StyleBox["3 + 4 I",
  FontWeight->"Bold"],
 " is given by:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Conjugate[3 + 4 I]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"3", "-", 
  RowBox[{"4", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{3.442754241398875*^9, 3.442754595727*^9, 
  3.442754691555125*^9}]
}, Open  ]],

Cell["\<\

EXPERIMENT:
Try multiplying an element by its conjugate.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["(3 + 4 I) * Conjugate[3 + 4 I]", "Input",
 PageWidth->Infinity],

Cell[BoxData["25"], "Output",
 CellChangeTimes->{3.44275424838325*^9, 3.442754597992625*^9, 
  3.44275469332075*^9}]
}, Open  ]],

Cell[TextData[{
 "Try this with several different elements of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ". What do you observe?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "It is an easy computation to prove that the observation found by this \
experiment holds for all complex numbers:\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"z", "\[CenterDot]", 
       OverscriptBox[
        RowBox[{"z", " "}], "_"]}], "  ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"a", " ", "+", " ", 
         RowBox[{"b", " ", "i"}]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"a", " ", "-", " ", 
         RowBox[{"b", " ", "i"}]}], ")"}]}]}], "\n", "\t         ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{
        SuperscriptBox["a", "2"], "+", " ", 
        RowBox[{"a", "\[CenterDot]", "b", "\[CenterDot]", "i"}], " ", "-", 
        " ", 
        RowBox[{"a", "\[CenterDot]", "b", "\[CenterDot]", "i"}], " ", "-", 
        " ", 
        RowBox[{
         SuperscriptBox["b", "2"], " ", 
         SuperscriptBox["i", "2"]}]}], " ", "=", " ", 
       RowBox[{
        SuperscriptBox["a", "2"], " ", "+", " ", 
        RowBox[{
         SuperscriptBox["b", "2"], "."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nNot only is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", "\[CenterDot]", 
    OverscriptBox[
     RowBox[{"z", " "}], "_"]}], TraditionalForm]]],
 " always a real number, but it is also non-negative. Moreover, there is a \
geometric significance to this formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", "2"], "+", 
    SuperscriptBox["b", "2"]}], TraditionalForm]]],
 ". By the Pythagorian theorem, this is the square of the distance from the \
point ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 " to the origin. Thus, it makes sense to give a special name to the square \
root of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", "\[CenterDot]", 
    OverscriptBox[
     RowBox[{"z", " "}], "_"]}], TraditionalForm]]],
 ".\n\nDEFINITION 11.6\nWe define the ",
 StyleBox["absolute value",
  FontSlant->"Italic"],
 " of a complex number ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", " ", "=", " ", 
    RowBox[{"a", " ", "+", " ", 
     RowBox[{"b", " ", "i"}]}]}], TraditionalForm]]],
 " to be \n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", "z", "|"}], " ", "=", " ", 
    RadicalBox[
     RowBox[{"z", "\[CenterDot]", 
      OverscriptBox[
       RowBox[{"z", " "}], "_"]}], " "]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437485430746375*^9, 3.437485456465125*^9}, {
  3.43753501846875*^9, 3.4375350188125*^9}}],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", "\[CenterDot]", 
    OverscriptBox[
     RowBox[{"z", " "}], "_"]}], TraditionalForm]]],
 " is non-negative real number for all elements of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "z", "|"}], TraditionalForm]]],
 " is well defined and is non-negative. The geometric interpretation of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", "z", "|"}], TraditionalForm]]],
 " is the distance from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 " to the origin. In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", the function ",
 StyleBox["Abs[z]",
  FontWeight->"Bold"],
 " gives the absolute value for both real and complex numbers."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43748545785575*^9, 3.437485468277625*^9}}],

Cell[CellGroupData[{

Cell["Abs[3 + 4 I]", "Input",
 PageWidth->Infinity],

Cell[BoxData["5"], "Output",
 CellChangeTimes->{3.442754698367625*^9}]
}, Open  ]],

Cell[TextData[{
 "Here is a property of the absolute value familar for real numbers, which \
can easily be shown to hold for all complex numbers as well.\n\nPROPOSITION \
11.5\nFor any two elements ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ", \n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"|", 
      RowBox[{"x", "\[CenterDot]", "y"}], "|"}], " ", "=", " ", 
     RowBox[{"|", "x", "|", 
      RowBox[{"\[CenterDot]", 
       RowBox[{"|", "y", "|", "."}]}]}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437485525387*^9, 3.43748552660575*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We have",
 "\n",
 "   ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"|", 
       RowBox[{"x", "\[CenterDot]", "y"}]}], "|"}], " ", "=", " ", 
     RowBox[{
      RadicalBox[
       RowBox[{"x", "\[CenterDot]", "y", "\[CenterDot]", 
        OverscriptBox[
         RowBox[{"x", "\[CenterDot]", "y"}], "_"]}], " "], " ", "=", " ", 
      RadicalBox[
       RowBox[{"x", "\[CenterDot]", "y", "\[CenterDot]", 
        OverscriptBox[
         RowBox[{"x", " "}], "_"], "\[CenterDot]", 
        OverscriptBox[
         RowBox[{"y", " "}], "_"]}], " "]}]}], " "}], TraditionalForm], "\n", 
 
  FormBox[
   RowBox[{"               ", 
    RowBox[{"=", " ", 
     RowBox[{
      RadicalBox[
       RowBox[{"x", "\[CenterDot]", 
        OverscriptBox[
         RowBox[{"x", " "}], "_"], "\[CenterDot]", "y", "\[CenterDot]", 
        OverscriptBox[
         RowBox[{"y", " "}], "_"]}], " "], " ", "=", " ", 
      RowBox[{
       RadicalBox[
        RowBox[{"x", "\[CenterDot]", 
         OverscriptBox[
          RowBox[{"x", " "}], "_"]}], " "], "\[CenterDot]", 
       RadicalBox[
        RowBox[{"y", "\[CenterDot]", 
         OverscriptBox[
          RowBox[{"y", " "}], "_"]}], " "]}]}]}], " "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"               ", 
    RowBox[{"=", " ", 
     RowBox[{"|", "x", "|", 
      RowBox[{"\[CenterDot]", 
       RowBox[{"|", "y", "|", "."}]}]}]}]}], TraditionalForm]}]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "Since there is a geometric interpretation of the absolute value, this \
proposition suggests that there is also a geometric interpretation to the \
product of two complex numbers.\n\nFrom polar coordinates it is known that \
any point in the plane can be located by knowing its distance ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " from the origin, and its angle ",
 Cell[BoxData[
  FormBox["\[Theta]", TraditionalForm]]],
 " from the positive ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-axis. Execute the following command to see a picture."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
Show[Graphics[{Line[{{0,0},{3,4}}], 
               Circle[{0,0},1,{0,0.92729}], 
               Text[\"\[Theta]\",{0.7,0.4}], 
               Text[\"r\",{1.5,2.5}],
               Text[\"(x,y) = x + y i\",{3.2,4},{-1,0}]}], 
               Axes -> True, AspectRatio -> 1, 
               PlotRange -> All]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.437487338715125*^9}],

Cell[BoxData[
 GraphicsBox[{LineBox[{{0, 0}, {3, 4}}], CircleBox[{0, 0}, 1, {0, 0.92729}], 
   InsetBox["\<\"\[Theta]\"\>", {0.7, 0.4}], 
   InsetBox["\<\"r\"\>", {1.5, 2.5}], 
   InsetBox["\<\"(x,y) = x + y i\"\>", {3.2, 4}, {-1, 0}]},
  AspectRatio->1,
  Axes->True,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.437487339215125*^9, 3.442754703336375*^9}]
}, Open  ]],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is the absolute value of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", " ", "+", " ", 
     RowBox[{"y", " ", "i"}]}], ")"}], TraditionalForm]]],
 ", perhaps the angle ",
 Cell[BoxData[
  FormBox["\[Theta]", TraditionalForm]]],
 " also is significant to the complex number. By using trigonometry in the \
above figure, we have that\n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{"r", "  ", "cos", " ", "\[Theta]"}]}], TraditionalForm]]],
 ",\nand\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "=", " ", 
    RowBox[{"r", "  ", "sin", " ", "\[Theta]"}]}], TraditionalForm]]],
 ".\nThus, \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "+", " ", 
     RowBox[{"y", " ", "i"}]}], " ", "=", " ", 
    RowBox[{"r", "(", " ", 
     RowBox[{
      RowBox[{"cos", " ", "\[Theta]"}], " ", "+", " ", 
      RowBox[{"i", " ", "sin", " ", "\[Theta]"}]}], ")"}]}], 
   TraditionalForm]]],
 ".\n\nThis form is called the ",
 StyleBox["polar form",
  FontSlant->"Italic"],
 " of the complex number ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"y", " ", 
     RowBox[{"i", "."}]}]}], TraditionalForm]]],
 " We have already seen that ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is the absolute value of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"y", " ", 
     RowBox[{"i", "."}]}]}], TraditionalForm]]],
 " The angle ",
 Cell[BoxData[
  FormBox["\[Theta]", TraditionalForm]]],
 " is called the ",
 StyleBox["argument",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"y", " ", "i"}]}], TraditionalForm]]],
 ". We can find the argument of a complex number in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with the command ",
 StyleBox["Arg[z]",
  FontWeight->"Bold"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Arg[3 + 4 I]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"ArcTan", "[", 
  FractionBox["4", "3"], "]"}]], "Output",
 CellChangeTimes->{3.4374876233245*^9, 3.44275471194575*^9}]
}, Open  ]],

Cell["This angle is approximately", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["N[Arg[3 + 4 I]]", "Input",
 PageWidth->Infinity],

Cell[BoxData["0.9272952180016122`"], "Output",
 CellChangeTimes->{3.437487627965125*^9, 3.44275471425825*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " puts the answer in radians, and we will soon understand why radians should \
always be used in expressing the angle ",
 Cell[BoxData[
  FormBox["\[Theta]", TraditionalForm]]],
 ". \n\nAlthough the absolute value of a complex number is unique, the \
argument is not uniquely determined. This is because for every angle ",
 Cell[BoxData[
  FormBox["\[Theta]", TraditionalForm]]],
 ", we consider the angles\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Ellipsis]\[Ellipsis]", ",", " ", 
    RowBox[{"\[Theta]", "-", 
     RowBox[{"6", "\[Pi]"}]}], ",", " ", 
    RowBox[{"\[Theta]", "-", 
     RowBox[{"4", "\[Pi]"}]}], ",", " ", 
    RowBox[{"\[Theta]", "-", 
     RowBox[{"2", "\[Pi]"}]}], ",", " ", "\[Theta]", ",", " ", 
    RowBox[{"\[Theta]", "+", 
     RowBox[{"2", "\[Pi]"}]}], ",", " ", 
    RowBox[{"\[Theta]", "+", 
     RowBox[{"4", "\[Pi]"}]}], ",", " ", 
    RowBox[{"\[Theta]", "+", 
     RowBox[{"6", "\[Pi]"}]}], ",", " ", "\[Ellipsis]\[Ellipsis]"}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "All of these angles have the same sine and cosine, and hence are \
interchangable in the polar coordinate system. We call these angles ",
 StyleBox["coterminal",
  FontSlant->"Italic"],
 ". The set of angles coterminal to ",
 Cell[BoxData[
  FormBox["\[Theta]", TraditionalForm]]],
 " can be writen\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Theta]", " ", "+", " ", 
      RowBox[{"2", " ", "\[Pi]", " ", "n"}]}], "  ", "|", "   ", 
     RowBox[{"n", " ", "\[Element]", " ", "\[DoubleStruckCapitalZ]"}]}], 
    "}"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437487663887*^9, 3.437487663887*^9}}],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " always picks ",
 Cell[BoxData[
  FormBox["\[Theta]", TraditionalForm]]],
 " to be between ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "\[Pi]"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["\[Pi]", TraditionalForm]]],
 ", but there are times when we need to consider the angles coterminal to ",
 Cell[BoxData[
  FormBox["\[Theta]", TraditionalForm]]],
 " as well.\n\nFor example, let us use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to help us find the polar form of  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", 
     RadicalBox["3", " "]}], "-", " ", "i"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["The absolute value is given by", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Abs[- Sqrt[3] - I]", "Input",
 PageWidth->Infinity],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.44275471763325*^9}]
}, Open  ]],

Cell["while the argument is given by", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Arg[- Sqrt[3] - I]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"5", " ", "\[Pi]"}], "6"]}]], "Output",
 CellChangeTimes->{3.442754719648875*^9}]
}, Open  ]],

Cell[TextData[{
 "So ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has converted the point",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RadicalBox["3", " "]}], ",", " ", 
      RowBox[{"-", "1"}]}], ")"}]}], TraditionalForm]]],
 " into polar coordinates, and found that\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RadicalBox["3", " "]}], " ", "-", " ", "i"}], "  ", "=", "  ", 
    RowBox[{
     RowBox[{"2", 
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RadicalBox["3", " "], "2"]}], " ", "-", " ", 
        FractionBox["i", "2"]}], ")"}]}], " ", "=", " ", 
     RowBox[{"2", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"cos", "(", 
         FractionBox[
          RowBox[{
           RowBox[{"-", "5"}], "\[Pi]"}], "6"], ")"}], " ", "+", " ", 
        RowBox[{"i", " ", 
         RowBox[{"sin", "(", 
          FractionBox[
           RowBox[{
            RowBox[{"-", "5"}], "\[Pi]"}], "6"], ")"}]}]}], ")"}]}]}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "However, we could have used any coterminal angle instead of the one ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " gave us. Thus, the following are also polar forms of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", 
     RadicalBox["3", " "]}], "-", "i"}], TraditionalForm]]],
 ":\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"cos", "(", 
       FractionBox[
        RowBox[{"7", "\[Pi]"}], "6"], ")"}], " ", "+", " ", 
      RowBox[{"i", " ", 
       RowBox[{"sin", "(", 
        FractionBox[
         RowBox[{"7", "\[Pi]"}], "6"], ")"}]}]}], ")"}]}], 
   TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"cos", "(", 
       FractionBox[
        RowBox[{"19", "\[Pi]"}], "6"], ")"}], " ", "+", " ", 
      RowBox[{"i", " ", 
       RowBox[{"sin", "(", 
        FractionBox[
         RowBox[{"19", "\[Pi]"}], "6"], ")"}]}]}], ")"}]}], 
   TraditionalForm]]],
 ", \[Ellipsis]\[Ellipsis]"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "There actually will be an infinite number of such expressions.\n\nThe \
usefulness of the polar form of a complex number is hinted at by the next \
lemma. This lemma makes use of the trigonometric identities\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"cos", "(", 
     RowBox[{"A", " ", "+", " ", "B"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"cos", "(", "A", ")"}], " ", 
      RowBox[{"cos", "(", "B", ")"}]}], " ", "-", " ", 
     RowBox[{
      RowBox[{"sin", "(", "A", ")"}], " ", 
      RowBox[{"sin", "(", "B", ")"}]}]}]}], TraditionalForm]]],
 "\nand\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"sin", "(", 
     RowBox[{"A", " ", "+", " ", "B"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"sin", "(", "A", ")"}], " ", 
      RowBox[{"cos", "(", "B", ")"}]}], " ", "+", " ", 
     RowBox[{
      RowBox[{"cos", "(", "A", ")"}], " ", 
      RowBox[{
       RowBox[{"sin", "(", "B", ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nLEMMA 11.6\nIf \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["z", "1"], " ", "=", " ", 
    RowBox[{
     SubscriptBox["r", "1"], "(", 
     RowBox[{
      RowBox[{"cos", " ", 
       SubscriptBox["\[Theta]", "1"]}], " ", "+", " ", 
      RowBox[{"i", " ", "sin", " ", 
       SubscriptBox["\[Theta]", "1"]}]}], ")"}]}], TraditionalForm]]],
 "\nand\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["z", "2"], " ", "=", " ", 
    RowBox[{
     SubscriptBox["r", "2"], "(", 
     RowBox[{
      RowBox[{"cos", " ", 
       SubscriptBox["\[Theta]", "2"]}], " ", "+", " ", 
      RowBox[{"i", " ", "sin", " ", 
       SubscriptBox["\[Theta]", "2"]}]}], ")"}]}], TraditionalForm]]],
 " \nthen \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["z", "1"], "\[CenterDot]", 
     SubscriptBox["z", "2"]}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["r", "1"], "\[CenterDot]", 
     RowBox[{
      SubscriptBox["r", "2"], "(", 
      RowBox[{
       RowBox[{"cos", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Theta]", "1"], "+", 
          SubscriptBox["\[Theta]", "2"]}], ")"}]}], " ", "+", " ", 
       RowBox[{"i", " ", "sin", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Theta]", "1"], "+", 
          SubscriptBox["\[Theta]", "2"]}], ")"}]}]}], ")"}]}]}], 
   TraditionalForm]]],
 ".\n\nSo the argument of the product is the sum of the arguments."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43748770879325*^9, 3.437487708902625*^9}, {
  3.437535065859375*^9, 3.437535066421875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We note that \n\n ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["z", "1"], "\[CenterDot]", 
      SubscriptBox["z", "2"]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["r", "1"], "(", 
       RowBox[{
        RowBox[{"cos", " ", 
         SubscriptBox["\[Theta]", "1"]}], " ", "+", " ", 
        RowBox[{"i", " ", "sin", " ", 
         SubscriptBox["\[Theta]", "1"]}]}], ")"}], "\[CenterDot]", 
      RowBox[{
       SubscriptBox["r", "2"], "(", 
       RowBox[{
        RowBox[{"cos", " ", 
         SubscriptBox["\[Theta]", "2"]}], " ", "+", " ", 
        RowBox[{"i", " ", "sin", " ", 
         SubscriptBox["\[Theta]", "2"]}]}], ")"}]}]}], " "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"            ", 
    RowBox[{"=", "  ", 
     RowBox[{
      SubscriptBox["r", "1"], "\[CenterDot]", 
      RowBox[{
       RowBox[{
        SubscriptBox["r", "2"], "(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"cos", " ", 
            RowBox[{
             SubscriptBox["\[Theta]", "1"], "\[CenterDot]", "cos"}], " ", 
            SubscriptBox["\[Theta]", "2"]}], " ", "-", " ", 
           RowBox[{"sin", " ", 
            RowBox[{
             SubscriptBox["\[Theta]", "1"], "\[CenterDot]", "sin"}], " ", 
            SubscriptBox["\[Theta]", "2"]}]}], ")"}], " ", "+", "\n", 
         "\t\t\t\t\t                     ", 
         RowBox[{"i", "\[CenterDot]", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"cos", " ", 
             RowBox[{
              SubscriptBox["\[Theta]", "1"], "\[CenterDot]", "sin"}], " ", 
             SubscriptBox["\[Theta]", "2"]}], " ", "+", " ", 
            RowBox[{"sin", " ", 
             RowBox[{
              SubscriptBox["\[Theta]", "1"], "\[CenterDot]", "cos"}], " ", 
             SubscriptBox["\[Theta]", "2"]}]}], ")"}]}]}], ")"}], "."}]}]}]}],
    TraditionalForm]}]],
 "\n \nUsing the trigonometric identities, this simplifies to\n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["z", "1"], "\[CenterDot]", 
     SubscriptBox["z", "2"]}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["r", "1"], "\[CenterDot]", 
     RowBox[{
      SubscriptBox["r", "2"], "(", 
      RowBox[{
       RowBox[{"cos", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Theta]", "1"], "+", 
          SubscriptBox["\[Theta]", "2"]}], ")"}]}], " ", "+", " ", 
       RowBox[{"i", " ", "sin", " ", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[Theta]", "1"], "+", 
          SubscriptBox["\[Theta]", "2"]}], ")"}]}]}], ")"}]}]}], 
   TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell["\<\
We can now use induction to prove the following important theorem:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "THEOREM 11.2: DE MOIVRE'S THEOREM\nIf ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is an integer, and \n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", " ", "=", " ", 
    RowBox[{"r", " ", 
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{"cos", " ", "\[Theta]"}], "  ", "+", " ", 
       RowBox[{"i", " ", "sin", " ", "\[Theta]"}]}], " ", ")"}]}]}], 
   TraditionalForm]]],
 "\n\nis a non-zero complex number in polar form, then \n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox["z", "n"], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["r", "n"], "  ", 
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{"cos", " ", 
         RowBox[{"(", 
          RowBox[{"n", " ", "\[Theta]"}], ")"}]}], " ", "+", " ", 
        RowBox[{"i", " ", "sin", " ", 
         RowBox[{"(", 
          RowBox[{"n", " ", "\[Theta]"}], ")"}]}]}], " ", ")"}]}]}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437487747777625*^9, 3.437487748965125*^9}, 
   3.4419766513546247`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let us first prove the theorem for positive values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". For ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "1"}], TraditionalForm]]],
 ", the statement is obvious. Let us assume that the statement is true for \
the previous case. That is,\n\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox["z", 
      RowBox[{"n", "-", "1"}]], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["r", 
       RowBox[{"n", "-", "1"}]], "  ", 
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{"cos", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"n", "-", "1"}], ")"}], " ", "\[Theta]"}], ")"}]}], " ", 
        "+", " ", 
        RowBox[{"i", " ", "sin", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"n", "-", "1"}], ")"}], " ", "\[Theta]"}], ")"}]}]}], " ",
        ")"}]}]}]}], TraditionalForm]]],
 ".\n                       \nWe want to prove that the theorem holds for",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", "n"}], TraditionalForm]]],
 " as well. Using lemma 11.6, we have\n\n                   ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["z", "n"], " ", "=", 
     RowBox[{
      SuperscriptBox["z", 
       RowBox[{"n", "-", "1"}]], "\[CenterDot]", "z"}]}], " "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"           ", 
    RowBox[{"=", 
     RowBox[{
      RowBox[{"[", " ", 
       RowBox[{
        SuperscriptBox["r", 
         RowBox[{"n", "-", "1"}]], " ", 
        RowBox[{"(", " ", 
         RowBox[{
          RowBox[{"cos", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"n", "-", "1"}], ")"}], " ", "\[Theta]"}], ")"}]}], " ",
           "+", " ", 
          RowBox[{"i", " ", "sin", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"n", "-", "1"}], ")"}], " ", "\[Theta]"}], ")"}]}]}], 
         " ", ")"}]}], "]"}], "\[CenterDot]", 
      RowBox[{"[", 
       RowBox[{"r", "(", 
        RowBox[{
         RowBox[{"cos", " ", "\[Theta]"}], " ", "+", " ", 
         RowBox[{"i", " ", "sin", " ", "\[Theta]"}]}], ")"}], "]"}]}]}], 
    " "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"           ", 
    RowBox[{"=", "  ", 
     RowBox[{
      SuperscriptBox["r", "n"], "(", " ", 
      RowBox[{
       RowBox[{"cos", "(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"n", "-", "1"}], ")"}], " ", "\[Theta]"}], " ", "+", " ", 
         "\[Theta]"}], ")"}], " ", "+", " ", 
       RowBox[{"i", " ", 
        RowBox[{"sin", "(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"n", "-", "1"}], ")"}], " ", "\[Theta]"}], " ", "+", " ", 
          "\[Theta]"}], ")"}]}]}], ")"}]}]}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t    ", 
    RowBox[{"=", "  ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["r", "n"], "(", " ", 
       RowBox[{
        RowBox[{"cos", "(", 
         RowBox[{"n", " ", "\[Theta]"}], ")"}], " ", "+", " ", 
        RowBox[{"i", " ", 
         RowBox[{"sin", "(", 
          RowBox[{"n", " ", "\[Theta]"}], ")"}]}]}], ")"}], "."}]}]}], 
   TraditionalForm]}]],
 "\n\nThus, the theorem is true for ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " if we assume it is true for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ", and hence by induction it is true whenever ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is positive.\n\nIf ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " is non-zero, then letting ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "0"}], TraditionalForm]]],
 " gives \n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{"   ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["r", "0"], "(", " ", 
      RowBox[{
       RowBox[{"cos", "(", 
        RowBox[{"0", " ", "\[Theta]"}], ")"}], " ", "+", " ", 
       RowBox[{"i", " ", 
        RowBox[{"sin", "(", 
         RowBox[{"0", " ", "\[Theta]"}], ")"}]}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"1", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"i", "\[CenterDot]", "0"}]}], ")"}]}], " ", "=", " ", 
      RowBox[{"1", " ", "=", " ", 
       SuperscriptBox["z", "0."]}]}]}]}], TraditionalForm]]],
 "\n          \nSo the theorem holds for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "0"}], TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " is non-zero, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", " ", ">", " ", "0"}], TraditionalForm]]],
 ", and so\n\n               ",
 Cell[BoxData[{
  RowBox[{
   FormBox[
    RowBox[{
     RowBox[{"[", 
      RowBox[{
       SuperscriptBox["r", 
        RowBox[{"-", "n"}]], "(", " ", 
       RowBox[{
        RowBox[{"cos", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "n"}], " ", "\[Theta]"}], ")"}]}], "+", 
        RowBox[{"i", " ", "sin", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "n"}], " ", "\[Theta]"}], ")"}]}]}], ")"}], "]"}], 
     "\[CenterDot]", 
     RowBox[{"[", 
      RowBox[{
       SuperscriptBox["r", "n"], "(", " ", 
       RowBox[{
        RowBox[{"cos", " ", 
         RowBox[{"(", 
          RowBox[{"n", " ", "\[Theta]"}], ")"}]}], "+", 
        RowBox[{"i", " ", "sin", " ", 
         RowBox[{"(", 
          RowBox[{"n", " ", "\[Theta]"}], ")"}]}]}], " ", ")"}], "]"}]}],
    TraditionalForm], "\n", " "}], "\n", 
  RowBox[{"                ", 
   RowBox[{"=", "  ", 
    RowBox[{
     SuperscriptBox["r", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "n"}], "+", "n"}], ")"}]], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"cos", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "n"}], " ", "\[Theta]"}], " ", "+", " ", 
          RowBox[{"n", " ", "\[Theta]"}]}], ")"}]}], " ", "+", " ", 
       RowBox[{"i", " ", 
        RowBox[{"sin", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "n"}], " ", "\[Theta]"}], " ", "+", " ", 
           RowBox[{"n", " ", "\[Theta]"}]}], ")"}]}]}]}], ")"}]}]}], 
   " "}], "\n", 
  RowBox[{"                ", 
   RowBox[{"=", "   ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["r", "0"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"cos", " ", "0"}], " ", "+", " ", 
        RowBox[{"i", " ", "sin", " ", "0"}]}], ")"}]}], " ", "=", " ", 
     "1."}]}]}]}]],
 "\n\nNow, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "<", " ", "0"}], TraditionalForm]]],
 ", then the theorem holds for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "n"}], TraditionalForm]]],
 ", and so\n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["z", 
       RowBox[{"-", "n"}]], "\[CenterDot]", 
      RowBox[{"[", 
       RowBox[{
        SuperscriptBox["r", "n"], "(", " ", 
        RowBox[{
         RowBox[{"cos", "(", 
          RowBox[{"n", " ", "\[Theta]"}], ")"}], " ", "+", " ", 
         RowBox[{"i", " ", 
          RowBox[{"sin", "(", 
           RowBox[{"n", " ", "\[Theta]"}], ")"}]}]}], ")"}], "]"}]}], " ", 
     "=", " ", "1"}]}], TraditionalForm]]],
 ", \nhence  \n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["r", "n"], "(", " ", 
      RowBox[{
       RowBox[{"cos", "(", 
        RowBox[{"n", " ", "\[Theta]"}], ")"}], " ", "+", " ", 
       RowBox[{"i", " ", 
        RowBox[{"sin", "(", 
         RowBox[{"n", " ", "\[Theta]"}], ")"}]}]}], ")"}], " ", "=", " ", 
     SuperscriptBox["z", "n"]}]}], TraditionalForm]]],
 " \neven when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "<", " ", "0"}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437487788871375*^9, 3.437487789152625*^9}, {
  3.437535079*^9, 3.437535079890625*^9}, {3.4419766650577497`*^9, 
  3.4419767928233747`*^9}}]
}, Open  ]],

Cell[TextData[{
 "De Moivre's Theorem (11.2) allows us to quickly raise a complex number to a \
integer power. For example, we can compute\n\n  ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RadicalBox["3", " "]}], " ", "-", " ", "i"}], ")"}], "5"], " ", "=", 
    "   ", 
    RowBox[{
     SuperscriptBox["2", "5"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"cos", "(", 
        FractionBox[
         RowBox[{
          RowBox[{"-", "25"}], "\[Pi]"}], "6"], ")"}], " ", "+", " ", 
       RowBox[{"i", " ", 
        RowBox[{"sin", "(", 
         FractionBox[
          RowBox[{
           RowBox[{"-", "25"}], "\[Pi]"}], "6"], ")"}]}]}], ")"}]}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t                        ", 
    RowBox[{"=", "  ", 
     RowBox[{
      RowBox[{"32", " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RadicalBox["3", " "], "2"], " ", "-", " ", 
         FractionBox["i", "2"]}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"16", 
        RadicalBox["3", " "]}], " ", "-", " ", 
       RowBox[{"16", " ", 
        RowBox[{"i", "."}]}]}]}]}]}], TraditionalForm]}]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43748792491825*^9, 3.43748792604325*^9}}],

Cell[TextData[{
 "We can check this result with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Expand[(- Sqrt[3] - I)^5]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "16"}], " ", "\[ImaginaryI]"}], "+", 
  RowBox[{"16", " ", 
   SqrtBox["3"]}]}]], "Output",
 CellChangeTimes->{3.442754730742625*^9}]
}, Open  ]],

Cell[TextData[{
 "Here is another example:\n\n ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RadicalBox["2", " "], "2"], " ", "-", " ", 
       RowBox[{
        FractionBox[
         RadicalBox["2", " "], "2"], "i"}]}], ")"}], "27"], " ", "=", "  ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"cos", "(", 
          FractionBox[
           RowBox[{"-", "\[Pi]"}], "4"], ")"}], " ", "+", " ", 
         RowBox[{"i", " ", 
          RowBox[{"sin", "(", 
           FractionBox[
            RowBox[{"-", "\[Pi]"}], "4"], ")"}]}]}], ")"}]}], ")"}], "27"]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"                                    ", 
    RowBox[{"=", "  ", 
     RowBox[{
      SuperscriptBox["1", "27"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"cos", "(", 
         FractionBox[
          RowBox[{
           RowBox[{"-", "27"}], "\[Pi]"}], "4"], ")"}], "+", " ", 
        RowBox[{"i", " ", 
         RowBox[{"sin", "(", 
          FractionBox[
           RowBox[{
            RowBox[{"-", "27"}], "\[Pi]"}], "4"], ")"}]}]}], ")"}]}]}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t                               ", 
    RowBox[{"=", "  ", 
     RowBox[{
      RowBox[{"-", " ", 
       FractionBox[
        RadicalBox["2", " "], "2"]}], " ", "-", " ", 
      RowBox[{
       FractionBox[
        RadicalBox["2", " "], "2"], " ", 
       RowBox[{"i", " ", "."}]}]}]}]}], TraditionalForm]}]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "This can also be verified using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Expand[(Sqrt[2]/2  - I Sqrt[2]/2 )^27]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"1", "+", "\[ImaginaryI]"}], 
   SqrtBox["2"]]}]], "Output",
 CellChangeTimes->{3.442754734930125*^9}]
}, Open  ]],

Cell[TextData[{
 "We can also use De Moivre's theorem to find the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-th root of 1.  We first define\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["e", "n"], " ", "=", " ", 
    RowBox[{
     RowBox[{"cos", "(", 
      FractionBox[
       RowBox[{"2", "\[Pi]"}], "n"], ")"}], " ", "+", " ", 
     RowBox[{"i", " ", 
      RowBox[{"sin", "(", 
       FractionBox[
        RowBox[{"2", "\[Pi]"}], "n"], ")"}]}]}]}], TraditionalForm]]],
 ".\n            \nFor example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["e", "1"], " ", "=", " ", "1"}], ","}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["e", "2"], " ", "=", " ", 
    RowBox[{"-", "1"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["e", "3"], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{"-", "1"}], " ", "+", " ", 
       RowBox[{"i", " ", "/", " ", "3"}]}], " ", ")"}], "/", "2"}]}], 
   TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["e", "4"], "=", " ", "i"}], TraditionalForm]]],
 ". From DeMoivre's theorem (11.2), \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SubscriptBox["e", "n"], ")"}], "n"], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"cos", "(", 
       RowBox[{"2", " ", "\[Pi]"}], ")"}], " ", "+", " ", 
      RowBox[{"i", " ", 
       RowBox[{"sin", "(", 
        RowBox[{"2", " ", "\[Pi]"}], ")"}]}]}], " ", "=", "1"}]}], 
   TraditionalForm]]],
 ". \n\nSo ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "n"], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-th root of 1.  In fact, all ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-th roots of 1 are given by the numbers ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "n"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "n"], "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "n"], "3"], TraditionalForm]]],
 ", ... up to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     SubscriptBox["e", "n"], "n"], "=", " ", "1"}], TraditionalForm]]],
 ".\n\nLet us look at an example. The eighth root of unity, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "8"], TraditionalForm]]],
 ", is given by \n\n                                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["e", "8"], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FractionBox["1", "2"], "+", " ", 
       FractionBox["i", "2"]}], ")"}], 
     RadicalBox["2", " "]}]}], TraditionalForm]]],
 ".\n                                       \nWe can enter this into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with the command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437531138609375*^9, 3.437531176296875*^9}, {
  3.437531206953125*^9, 3.437531347828125*^9}, {3.43753138*^9, 
  3.4375313823125*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitDomain[0]
e8 = (1/2 + I/2) Sqrt[2]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43753162234375*^9, 3.437531622921875*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"1", "+", "\[ImaginaryI]"}], 
  SqrtBox["2"]]], "Output",
 CellChangeTimes->{3.437531628125*^9, 3.442754747336375*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["InitDomain",
  FontWeight->"Bold"],
 " command clears out the previous fields that were defined, and allows the \
dot to be used for the product. Thus,"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["e8 . e8", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43753164603125*^9, 3.4375316475625*^9}}],

Cell[BoxData["\[ImaginaryI]"], "Output",
 CellChangeTimes->{3.437531649234375*^9, 3.442754752711375*^9}]
}, Open  ]],

Cell[TextData[{
 "simplifies to just",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", "I"}], TraditionalForm]]],
 ".\n\nWe can now consider the group generated by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["w", "8"], TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["G = Group[{e8}]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437531651984375*^9, 3.437531652265625*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", 
   RowBox[{"-", "1"}], ",", 
   RowBox[{"-", "\[ImaginaryI]"}], ",", "\[ImaginaryI]", ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"1", "+", "\[ImaginaryI]"}], 
     SqrtBox["2"]]}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"1", "-", "\[ImaginaryI]"}], 
     SqrtBox["2"]]}], ",", 
   FractionBox[
    RowBox[{"1", "-", "\[ImaginaryI]"}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{"1", "+", "\[ImaginaryI]"}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{3.4375316528125*^9, 3.442754754992625*^9}]
}, Open  ]],

Cell[TextData[{
 "This gives the 8 roots of unity, and shows that these elements form a \
group. In fact, the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-th roots of unity will form a cyclic group isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "n"], TraditionalForm]]],
 ".\n\nLet us rearrange the elements of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
G = { I, (1/2 + 1/2 I)Sqrt[2], 1 ,(1/2 - 1/2 I)Sqrt[2],- I,
    (-1/2-1/2 I)Sqrt[2],-1,(-1/2+1/2 I)Sqrt[2]}\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\[ImaginaryI]", ",", 
   FractionBox[
    RowBox[{"1", "+", "\[ImaginaryI]"}], 
    SqrtBox["2"]], ",", "1", ",", 
   FractionBox[
    RowBox[{"1", "-", "\[ImaginaryI]"}], 
    SqrtBox["2"]], ",", 
   RowBox[{"-", "\[ImaginaryI]"}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"1", "+", "\[ImaginaryI]"}], 
     SqrtBox["2"]]}], ",", 
   RowBox[{"-", "1"}], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"1", "-", "\[ImaginaryI]"}], 
     SqrtBox["2"]]}]}], "}"}]], "Output",
 CellChangeTimes->{3.43753167475*^9, 3.44275475775825*^9}]
}, Open  ]],

Cell[TextData[{
 "so that the elements appear in the circle graph as they do in the complex \
plane. For example, the circle graph which multiplies every element by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Omega]", "8"], TraditionalForm]]],
 " is given by"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CircleGraph[G, Mult[e8]]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437531678609375*^9, 3.437531678875*^9}}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"I\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"-I\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"(1+I)/Sqrt[2]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"(1-I)/Sqrt[2]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"(-1-I)/Sqrt[2]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"-1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"(-1+I)/Sqrt[2]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {
    {RGBColor[1, 0, 0], 
     LineBox[{{0., 1.}, {-0.7071067811865477, 0.7071067811865474}}], 
     PolygonBox[{{-0.7071067811865477, 
      0.7071067811865474}, {-0.5955846563171645, 
      0.699181147797492}, {-0.6338529995536735, 0.7915691010486207}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.7071067811865477, 
      0.7071067811865474}, {-1., -1.8369701987210297`*^-16}}], 
     PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.9155376801379267, 
      0.07325378163287403}, {-1.0079256333890554`, 0.111522124869383}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-1., -1.8369701987210297`*^-16}, {-0.7071067811865475, \
-0.7071067811865477}}], 
     PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6991811477974921, -0.5955846563171645}, {-0.7915691010486208, \
-0.6338529995536735}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{-0.7071067811865475, -0.7071067811865477}, {
      1.2246467991473532`*^-16, -1.}}], 
     PolygonBox[{{
      1.2246467991473532`*^-16, -1.}, {-0.07325378163287409, \
-0.9155376801379267}, {-0.11152212486938305`, -1.0079256333890554`}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{1.2246467991473532`*^-16, -1.}, {
      0.7071067811865476, -0.7071067811865475}}], 
     PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
      0.5955846563171644, -0.6991811477974921}, {
      0.6338529995536734, -0.7915691010486208}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{0.7071067811865476, -0.7071067811865475}, {1., 
      6.123233995736766*^-17}}], 
     PolygonBox[{{1., 6.123233995736766*^-17}, {
      0.9155376801379267, -0.07325378163287415}, {
      1.0079256333890554`, -0.11152212486938312`}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{1., 6.123233995736766*^-17}, {0.7071067811865475, 
      0.7071067811865476}}], 
     PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
      0.6991811477974921, 0.5955846563171644}, {0.7915691010486208, 
      0.6338529995536734}}]}, 
    {RGBColor[1, 0, 0], 
     LineBox[{{0.7071067811865475, 0.7071067811865476}, {0., 1.}}], 
     PolygonBox[{{0., 1.}, {0.07325378163287419, 0.9155376801379267}, {
      0.11152212486938318`, 1.0079256333890554`}}]}}},
  AspectRatio->Automatic,
  ImageSize->{240.6249999999991, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.4375316804375*^9, 3.442754760555125*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "We are mainly interested in those elements of this subgroup which are \
generators.\n\nDEFINITION 11.7:\nA complex number ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " is called a ",
 StyleBox["primitive",
  FontSlant->"Italic"],
 "  ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 StyleBox["-th root of unity",
  FontSlant->"Italic"],
 " if the powers of ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " produce all ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " solutions to the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "n"], "=", " ", "1"}], TraditionalForm]]],
 ".\n                                                           \nIt is clear \
that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "n"], TraditionalForm]]],
 "  is a primitive ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-th root of unity, but also ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{
     FormBox[
      RowBox[{"(", 
       SubscriptBox["e", "n"]}],
      TraditionalForm], ")"}], "k"], TraditionalForm]]],
 "  is an ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-th root of unity if ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " are coprime. \n\nEXPERIMENT:\nBy trial and error, determine which elements \
of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " are primitive eighth roots of unity. Note that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "8"], "2"], TraditionalForm]]],
 " is ",
 StyleBox["not",
  FontSlant->"Italic"],
 " is primitive eighth root, since the circle graph"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43753171246875*^9, 3.43753174784375*^9}, 
   3.437531784578125*^9, {3.437535116203125*^9, 3.437535116671875*^9}}],

Cell[CellGroupData[{

Cell["CircleGraph[G,Mult[e8^2]]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437531790484375*^9, 3.43753179075*^9}}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865475, 0.7071067811865476}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7071067811865476, -0.7071067811865475}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865475, -0.7071067811865477}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7071067811865477, 0.7071067811865474}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"I\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"-I\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"(1+I)/Sqrt[2]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052023, 0.7778174593052024}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"(1-I)/Sqrt[2]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.7778174593052024, -0.7778174593052023}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"(-1-I)/Sqrt[2]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052023, -0.7778174593052025}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"-1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"(-1+I)/Sqrt[2]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.7778174593052025, 0.7778174593052022}, {1, 0}]}, {{
     {RGBColor[1, 0, 0], 
      LineBox[{{0., 1.}, {-1., -1.8369701987210297`*^-16}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.8939339828220179, 
       0.035355339059327216`}, {-0.9646446609406727, 
       0.10606601717798196`}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {
       1.2246467991473532`*^-16, -1.}}], 
      PolygonBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.03535533905932728, \
-0.8939339828220179}, {-0.10606601717798202`, -0.9646446609406727}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {1., 
       6.123233995736766*^-17}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {
       0.8939339828220179, -0.03535533905932732}, {
       0.9646446609406726, -0.10606601717798207`}}]}, 
     {RGBColor[1, 0, 0], LineBox[{{1., 6.123233995736766*^-17}, {0., 1.}}], 
      PolygonBox[{{0., 1.}, {0.03535533905932738, 0.8939339828220179}, {
       0.10606601717798213`, 0.9646446609406726}}]}}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865475, 0.7071067811865476}, {-0.7071067811865477,
        0.7071067811865474}}], 
      PolygonBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.6071067811865477, 
       0.6571067811865474}, {-0.6071067811865477, 0.7571067811865474}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.7071067811865477, 
       0.7071067811865474}, {-0.7071067811865475, -0.7071067811865477}}], 
      PolygonBox[{{-0.7071067811865475, -0.7071067811865477}, \
{-0.6571067811865475, -0.6071067811865477}, {-0.7571067811865475, \
-0.6071067811865477}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.7071067811865475, -0.7071067811865477}, {
       0.7071067811865476, -0.7071067811865475}}], 
      PolygonBox[{{0.7071067811865476, -0.7071067811865475}, {
       0.6071067811865476, -0.6571067811865475}, {
       0.6071067811865476, -0.7571067811865475}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.7071067811865476, -0.7071067811865475}, {0.7071067811865475,
        0.7071067811865476}}], 
      PolygonBox[{{0.7071067811865475, 0.7071067811865476}, {
       0.6571067811865474, 0.6071067811865476}, {0.7571067811865475, 
       0.6071067811865476}}]}}}},
  AspectRatio->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{3.43753179221875*^9, 3.44275476525825*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "shows that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "8"], "2"], TraditionalForm]]],
 " does not generate the group. Replace the 2 in the above example until all \
primitive eigth roots are found. (You may have to enlarge the graph each \
time.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43753179678125*^9}],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Here is another example--one twelvth root of unity is given by\n\n          \
         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["e", "12"], " ", "=", " ", 
    RowBox[{
     FractionBox[
      RadicalBox["3", " "], "2"], " ", "+", " ", 
     FractionBox["i", "2."]}]}], TraditionalForm]]],
 "\n                   \nLet us form a circle graphs of the twelvth roots of \
unity."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.437531833125*^9}],

Cell[CellGroupData[{

Cell["\<\
e12 = Sqrt[3]/2 + I/2
H = {I, 1/2 + I Sqrt[3]/2, I/2 + Sqrt[3]/2, 1,
    -I/2 + Sqrt[3]/2,1/2 - I Sqrt[3]/2 , -I, 
    -1/2 - I Sqrt[3]/2, -I/2 - Sqrt[3]/2, -1, 
    I/2 - Sqrt[3]/2,-1/2 + I Sqrt[3]/2}\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437531836859375*^9, 3.43753183709375*^9}},
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 RowBox[{
  FractionBox["\[ImaginaryI]", "2"], "+", 
  FractionBox[
   SqrtBox["3"], "2"]}]], "Output",
 CellChangeTimes->{3.437531838546875*^9, 3.4427547692895*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\[ImaginaryI]", ",", 
   RowBox[{
    FractionBox["1", "2"], "+", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      SqrtBox["3"]}], "2"]}], ",", 
   RowBox[{
    FractionBox["\[ImaginaryI]", "2"], "+", 
    FractionBox[
     SqrtBox["3"], "2"]}], ",", "1", ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", "2"]}], "+", 
    FractionBox[
     SqrtBox["3"], "2"]}], ",", 
   RowBox[{
    FractionBox["1", "2"], "-", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      SqrtBox["3"]}], "2"]}], ",", 
   RowBox[{"-", "\[ImaginaryI]"}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], "-", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      SqrtBox["3"]}], "2"]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", "2"]}], "-", 
    FractionBox[
     SqrtBox["3"], "2"]}], ",", 
   RowBox[{"-", "1"}], ",", 
   RowBox[{
    FractionBox["\[ImaginaryI]", "2"], "-", 
    FractionBox[
     SqrtBox["3"], "2"]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], "+", 
    FractionBox[
     RowBox[{"\[ImaginaryI]", " ", 
      SqrtBox["3"]}], "2"]}]}], "}"}]], "Output",
 CellChangeTimes->{3.437531838546875*^9, 3.44275476932075*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["CircleGraph[H,Mult[e12^3]]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43753184359375*^9, 3.437531843859375*^9}},
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 1, 1], CircleBox[{0, 0}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.49999999999999994`, 0.8660254037844387}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8660254037844386, 0.5000000000000001}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1., 6.123233995736766*^-17}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.8660254037844387, -0.4999999999999998}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5000000000000003, -0.8660254037844385}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.2246467991473532`*^-16, -1.}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.4999999999999994, -0.866025403784439}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.8660254037844384, -0.5000000000000004}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1., -1.8369701987210297`*^-16}, {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {-0.866025403784439, 0.49999999999999933`},
      {0.1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5000000000000004, 0.8660254037844384}, {0.1, 0}], 
    InsetBox[
     TagBox[
      StyleBox["\<\"\[FilledSmallCircle]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-2.4492935982947064`*^-16, 1.}, {0.1, 0}]}, InsetBox[
    TagBox[
     StyleBox["\<\"I\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, 1.1}, {0, -1}], InsetBox[
    TagBox[
     StyleBox["\<\"-I\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0, -1.1}, {0, 1}], {InsetBox[
     TagBox[
      StyleBox["\<\"1/2+(I/2)*Sqrt[3]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5499999999999999, 0.9526279441628827}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"I/2+Sqrt[3]/2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9526279441628825, 0.5500000000000002}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {1.1, 6.735557395310444*^-17}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"-I/2+Sqrt[3]/2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.9526279441628827, -0.5499999999999998}, {-1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"1/2-(I/2)*Sqrt[3]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {0.5500000000000004, -0.9526279441628824}, {-1, 0}]}, {
    InsetBox[
     TagBox[
      StyleBox["\<\"-1/2-(I/2)*Sqrt[3]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5499999999999994, -0.952627944162883}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"-I/2-Sqrt[3]/2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.9526279441628823, -0.5500000000000005}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"-1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-1.1, -2.0206672185931328`*^-16}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"I/2-Sqrt[3]/2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.952627944162883, 0.5499999999999993}, {1, 0}], InsetBox[
     TagBox[
      StyleBox["\<\"-1/2+(I/2)*Sqrt[3]\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 
       18]& ], {-0.5500000000000005, 0.9526279441628823}, {1, 0}]}, {{
     {RGBColor[1, 0, 0], 
      LineBox[{{0., 1.}, {-1., -1.8369701987210297`*^-16}}], 
      PolygonBox[{{-1., -1.8369701987210297`*^-16}, {-0.8939339828220179, 
       0.035355339059327216`}, {-0.9646446609406727, 
       0.10606601717798196`}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{-1., -1.8369701987210297`*^-16}, {
       1.2246467991473532`*^-16, -1.}}], 
      PolygonBox[{{
       1.2246467991473532`*^-16, -1.}, {-0.03535533905932728, \
-0.8939339828220179}, {-0.10606601717798202`, -0.9646446609406727}}]}, 
     {RGBColor[1, 0, 0], 
      LineBox[{{1.2246467991473532`*^-16, -1.}, {1., 
       6.123233995736766*^-17}}], 
      PolygonBox[{{1., 6.123233995736766*^-17}, {
       0.8939339828220179, -0.03535533905932732}, {
       0.9646446609406726, -0.10606601717798207`}}]}, 
     {RGBColor[1, 0, 0], LineBox[{{1., 6.123233995736766*^-17}, {0., 1.}}], 
      PolygonBox[{{0., 1.}, {0.03535533905932738, 0.8939339828220179}, {
       0.10606601717798213`, 0.9646446609406726}}]}}, {
     {RGBColor[0, 1, 0], 
      LineBox[{{0.49999999999999994`, 
       0.8660254037844387}, {-0.866025403784439, 0.49999999999999933`}}], 
      PolygonBox[{{-0.866025403784439, 
       0.49999999999999933`}, {-0.7564918689004061, 
       0.47758561319579806`}, {-0.7823737734106583, 0.5741781958247049}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.866025403784439, 
       0.49999999999999933`}, {-0.4999999999999994, -0.866025403784439}}], 
      PolygonBox[{{-0.4999999999999994, -0.866025403784439}, \
{-0.4775856131957981, -0.7564918689004062}, {-0.574178195824705, \
-0.7823737734106583}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{-0.4999999999999994, -0.866025403784439}, {
       0.8660254037844387, -0.4999999999999998}}], 
      PolygonBox[{{0.8660254037844387, -0.4999999999999998}, {
       0.7564918689004059, -0.4775856131957985}, {
       0.782373773410658, -0.5741781958247053}}]}, 
     {RGBColor[0, 1, 0], 
      LineBox[{{0.8660254037844387, -0.4999999999999998}, {
       0.49999999999999994`, 0.8660254037844387}}], 
      PolygonBox[{{0.49999999999999994`, 0.8660254037844387}, {
       0.4775856131957986, 0.7564918689004059}, {0.5741781958247054, 
       0.782373773410658}}]}}, {
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0.8660254037844386, 0.5000000000000001}, {-0.5000000000000004,
        0.8660254037844384}}], 
      PolygonBox[{{-0.5000000000000004, 
       0.8660254037844384}, {-0.4163483696262196, 
       0.791847207959733}, {-0.3904664651159676, 0.8884397905886398}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.5000000000000004, 
       0.8660254037844384}, {-0.8660254037844384, -0.5000000000000004}}], 
      PolygonBox[{{-0.8660254037844384, -0.5000000000000004}, \
{-0.791847207959733, -0.4163483696262196}, {-0.8884397905886398, \
-0.3904664651159676}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{-0.8660254037844384, -0.5000000000000004}, {
       0.5000000000000003, -0.8660254037844385}}], 
      PolygonBox[{{0.5000000000000003, -0.8660254037844385}, {
       0.4163483696262195, -0.7918472079597331}, {
       0.3904664651159675, -0.8884397905886399}}]}, 
     {RGBColor[0.6, 0.4, 0.8], 
      LineBox[{{0.5000000000000003, -0.8660254037844385}, {0.8660254037844386,
        0.5000000000000001}}], 
      PolygonBox[{{0.8660254037844386, 0.5000000000000001}, {
       0.7918472079597332, 0.4163483696262193}, {0.88843979058864, 
       0.3904664651159673}}]}}}},
  AspectRatio->Automatic,
  ImageSize->{332.6249999999998, Automatic},
  PlotRange->All]], "Output",
 CellChangeTimes->{3.437531844484375*^9, 3.44275477532075*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "This graph shows that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox[
      SubscriptBox["e", "12"], "3"],
     TraditionalForm], "=", " ", "I"}], TraditionalForm]]],
 ", and hence this is not a primitive twelve root of unity. But it is not \
hard to find primitive twelve roots from this circle graph.\n\nEXPERIMENT:\n\
Try changing the power in the last command to find all primitive twelvth \
roots of unity."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43753185546875*^9}],

Cell[BoxData["       "], "Input"],

Cell[TextData[{
 "Thus, De Moivre's Theorem (11.2) gives us a very fast means of raising \
complex numbers to integer powers. It is not difficult to imagine that this \
formula could be used to raise a complex number to any real number as well. \
But what if we want to consider raising a number to a ",
 StyleBox["complex",
  FontSlant->"Italic"],
 " power? Could we somehow use De Moivre's Theorem (11.2) to help us find, \
for example, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", "i"], TraditionalForm]]],
 " ?\n\nFirst, we need to clarify exactly what we mean by raising an element \
to the power of an element. Since this is not a standard field operation, it \
is not surprising to learn that in most fields, raising an element to the \
power of an element is absurd. However, since the complex numbers contain a \
copy of the real numbers, such a concept seems plausible. If we examine how \
we take powers in the real number system we will discover that we utilize the \
exponiential function, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "x"], TraditionalForm]]],
 "  to compute quantities such as ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    SqrtBox["2"]], TraditionalForm]]],
 ". We use the fact that \n \n          2 = ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", 
    RowBox[{"ln", " ", "2"}]], TraditionalForm]]],
 ",   so    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["2", 
      SqrtBox["2"]],
     TraditionalForm], "=", " ", 
    FormBox[
     SuperscriptBox[
      RowBox[{
       FormBox[
        RowBox[{"(", 
         SuperscriptBox["e", 
          RowBox[{"ln", " ", "2"}]]}],
        TraditionalForm], ")"}], 
      SqrtBox["2"]],
     TraditionalForm]}], TraditionalForm]]],
 "= ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", 
    RowBox[{"(", 
     RowBox[{"ln", " ", "2", " ", 
      SqrtBox["2"]}], ")"}]], TraditionalForm]]],
 " ,\n\nwhich is approximately "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43748795735575*^9, 3.4374879583245*^9}, {
  3.437531870859375*^9, 3.437531899796875*^9}}],

Cell[CellGroupData[{

Cell["N[E^(Sqrt[2] Log[2])]", "Input",
 PageWidth->Infinity],

Cell[BoxData["2.665144142690225`"], "Output",
 CellChangeTimes->{3.43753192946875*^9, 3.442754782648875*^9}]
}, Open  ]],

Cell[TextData[{
 "It is clear that if we know how to take the exponential function of a \
complex number, we will be able to raise any positive number to a complex \
power.\n\nThe key algebraic property of the exponiential function is that\n\n \
       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["e", 
      RowBox[{"x", "+", "y"}]],
     TraditionalForm], "=", "  ", 
    RowBox[{
     SuperscriptBox["e", "x"], "\[CenterDot]", " ", 
     SuperscriptBox["e", "y"]}]}], TraditionalForm]]],
 "   for all  ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[Element]", "\[DoubleStruckCapitalR]"}], 
   TraditionalForm]]],
 ". \n\nThis indicates that the exponiential function is a ",
 StyleBox["group homomorphism",
  FontSlant->"Italic"],
 " mapping the additive group of real numbers to the multiplicative group of \
real numbers. The reason we can consider raising an element of the real \
numbers to the power of an element stems from the existance of a group \
homomorphism between the additive structure and the multiplicative structure. \
\n\nUsing this obversation about the real numbers, let us rephase the \
question. Can we use De Moivre's theorem (11.2) to extend the exponiential \
function into a group homomorphism from the additive structure of ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ", denoted ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "+"], TraditionalForm]]],
 " , to the multiplicative structure ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "*"], TraditionalForm]]],
 ", thereby allowing us to  compute expressions such as ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", "i"], TraditionalForm]]],
 " ? \n\nIf such a group homomorphism exists, then\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["e", 
      RowBox[{"a", "+", 
       RowBox[{"b", " ", "i"}]}]],
     TraditionalForm], "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["e", "a"], "\[CenterDot]", " ", 
      SuperscriptBox["e", 
       RowBox[{"b", " ", "i"}]]}], "=", "  ", 
     RowBox[{
      SuperscriptBox["e", "a"], "\[CenterDot]", 
      SuperscriptBox[
       RowBox[{"(", 
        SuperscriptBox["e", "i"], ")"}], "b"]}]}]}], TraditionalForm]]],
 "\n\nso using De Moivre's theorem we can compute the exponential of any \
complex number ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", " ", "+", " ", 
     RowBox[{"b", " ", "i"}]}], ")"}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is an integer, provided we know the value of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "i"], TraditionalForm]]],
 ".  Can ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " give us a hint as to what e  should be? Let's try it."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4375319506875*^9, 3.43753201090625*^9}}],

Cell[CellGroupData[{

Cell["E^I", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 SuperscriptBox["\[ExponentialE]", "\[ImaginaryI]"]], "Output",
 CellChangeTimes->{3.437532056984375*^9, 3.44275478719575*^9}]
}, Open  ]],

Cell["\<\
That didn't help much. Let us try looking at the approximation.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["N[E^I]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"0.5403023058681398`", "\[InvisibleSpace]", "+", 
  RowBox[{"0.8414709848078965`", " ", "\[ImaginaryI]"}]}]], "Output",
 CellChangeTimes->{3.437532060796875*^9, 3.442754789867625*^9}]
}, Open  ]],

Cell[TextData[{
 "This at least tells us that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "i"], TraditionalForm]]],
 "  is a complex number, but the decimals don't look familar. What we are \
really interested in is the absolute value and the argument.\n\nEXPERIMENT:\n\
Using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", try to determine the ",
 StyleBox["exact",
  FontSlant->"Italic"],
 " value for the absolute value and the argument of the number ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "i"], TraditionalForm]]],
 ". Note that in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", both the ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " must be capitalized."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Although the absolute value and argument of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "i"], TraditionalForm]]],
 "  seem simple enough, there is no way to algebraically ",
 StyleBox["prove",
  FontSlant->"Italic"],
 " that these values are correct. The reason is simple enough: there are ",
 StyleBox["many",
  FontSlant->"Italic"],
 " ways to extend the exponiental function to become a group homomorphism \
from ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "+"], TraditionalForm]]],
 "  to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "*"], TraditionalForm]]],
 ". So typically an additional property of the homomorphism is imposed, such \
as the new function must have the derivative equal to itself. However, such \
additional properties are not algebraic: they all involve calculus in some \
way. In fact we cannot determine the value of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "i"], TraditionalForm]]],
 " without calculus. Problems 11.21 through 11.23 show three ways of using \
calculus to determining the value of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "i"], TraditionalForm]]],
 ".\n\nFor now, let us just use our observations from the experiment and \
assume that\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox["e", "i"], "=", " ", 
     RowBox[{
      RowBox[{"cos", "(", " ", 
       RowBox[{"1", " ", "radian"}], " ", ")"}], " ", "+", " ", 
      RowBox[{"i", " ", 
       RowBox[{
        RowBox[{"sin", "(", " ", 
         RowBox[{"1", " ", "radian"}], " ", ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nFrom this, we have by De Moivre's theorem (11.2) that\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     FormBox[
      RowBox[{" ", 
       RowBox[{
        SuperscriptBox["e", 
         RowBox[{"a", " ", "+", " ", 
          RowBox[{"b", " ", "i"}]}]], "=", " ", 
        RowBox[{
         RowBox[{
          SuperscriptBox["e", "a"], "\[CenterDot]", 
          SuperscriptBox[
           RowBox[{"(", 
            SuperscriptBox["e", "i"], ")"}], "b"]}], "=", " ", 
         RowBox[{
          SuperscriptBox["e", "a"], "\[CenterDot]", 
          SuperscriptBox["1", "b"], "\[CenterDot]", 
          RowBox[{"(", " ", 
           RowBox[{
            RowBox[{"cos", " ", 
             RowBox[{"(", 
              RowBox[{"b", "\[CenterDot]", "1"}], ")"}]}], " ", "+", " ", 
            RowBox[{"i", " ", "sin", " ", 
             RowBox[{"(", 
              RowBox[{"b", "\[CenterDot]", "1"}], ")"}]}]}], " ", 
           ")"}]}]}]}]}],
      TraditionalForm], "  ", "\n", "\t\t        ", 
     FormBox[
      RowBox[{"=", "  ", 
       RowBox[{
        SuperscriptBox["e", "a"], " ", 
        RowBox[{"(", " ", 
         RowBox[{
          RowBox[{"cos", "(", "b", ")"}], " ", "+", " ", 
          RowBox[{"i", " ", 
           RowBox[{"sin", "(", "b", ")"}]}]}], " ", ")"}]}]}],
      TraditionalForm], " "}]}], TraditionalForm]]],
 "\n   \nwhenever ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is an integer. It makes sence to define this as the exponiential function \
for all complex numbers. Notice that radian measure must be used in this \
formula!\n\nPROPOSITION 11.6\nFor ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"z", " ", "=", " ", 
     RowBox[{"a", " ", "+", " ", 
      RowBox[{"b", " ", "i"}]}]}], ","}], TraditionalForm]]],
 " the function\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "z", ")"}], "  ", "=", " ", 
     RowBox[{
      SuperscriptBox["e", "a"], "\[CenterDot]", 
      RowBox[{
       RowBox[{"(", " ", 
        RowBox[{
         RowBox[{"cos", "(", "b", ")"}], " ", "+", " ", 
         RowBox[{"i", " ", 
          RowBox[{"sin", "(", "b", ")"}]}]}], " ", ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\n              \ndefines a group homomorphism from ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "+"], TraditionalForm]]],
 "  to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "*"], TraditionalForm]]],
 ", which is an extension of the standard exponiential function. This \
function is called the ",
 StyleBox["complex exponiential",
  FontSlant->"Italic"],
 " function, and is also denoted  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "z"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43753212225*^9, 3.43753217196875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["z", "1"], " ", "=", " ", 
     RowBox[{
      SubscriptBox["a", "1"], " ", "+", " ", 
      RowBox[{
       SubscriptBox["b", "1"], " ", "i"}]}]}], ","}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["z", "2"], " ", "=", " ", 
     RowBox[{
      SubscriptBox["a", "2"], " ", "+", " ", 
      RowBox[{
       SubscriptBox["b", "2"], " ", "i"}]}]}], ","}], TraditionalForm]]],
 " we observe that\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{
       SubscriptBox["z", "1"], " ", "+", " ", 
       SubscriptBox["z", "2"]}], ")"}], " ", "=", "  ", 
     RowBox[{
      SuperscriptBox["e", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["a", "1"], "+", " ", 
         SubscriptBox["a", "2"]}], ")"}]], " ", 
      RowBox[{
       RowBox[{"(", " ", 
        RowBox[{
         RowBox[{"cos", "(", 
          RowBox[{
           SubscriptBox["b", "1"], "+", " ", 
           SubscriptBox["b", "2"]}], ")"}], " ", "+", " ", 
         RowBox[{"i", " ", "sin", " ", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["b", "1"], "+", " ", 
            SubscriptBox["b", "2"]}], ")"}]}]}], " ", ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\n\nBy lemma 11.5, this equals \n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      RowBox[{" ", 
       RowBox[{
        RowBox[{
         RowBox[{
          SuperscriptBox["e", 
           SubscriptBox["a", "1"]], "(", " ", 
          RowBox[{
           RowBox[{"cos", "(", 
            SubscriptBox["b", "1"], ")"}], " ", "+", " ", 
           RowBox[{"i", " ", "sin", " ", 
            RowBox[{"(", 
             SubscriptBox["b", "1"], ")"}]}]}], ")"}], " ", "\[CenterDot]", 
         " ", 
         SuperscriptBox["e", 
          SubscriptBox["a", "2"]]}], " ", 
        RowBox[{"(", " ", 
         RowBox[{
          RowBox[{"cos", "(", 
           SubscriptBox["b", "2"], ")"}], " ", "+", " ", 
          RowBox[{"i", " ", "sin", " ", 
           RowBox[{"(", 
            SubscriptBox["b", "2"], ")"}]}]}], ")"}]}]}],
      TraditionalForm], " ", "\n", "                 ", 
     RowBox[{"=", " ", 
      RowBox[{
       RowBox[{"\[Florin]", "(", 
        SubscriptBox["z", "1"], ")"}], " ", "\[CenterDot]", " ", 
       RowBox[{
        RowBox[{"\[Florin]", "(", 
         SubscriptBox["z", "2"], ")"}], "."}]}]}]}]}], TraditionalForm]]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is a group homomorphism from ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "+"], TraditionalForm]]],
 "  to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "*"], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4375322049375*^9, 3.437532205046875*^9}}]
}, Open  ]],

Cell[TextData[{
 "The real exponential function is one-to-one, but is not onto since there is \
no number for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["e", "x"],
     TraditionalForm], "  ", "=", " ", 
    RowBox[{"-", "1"}]}], TraditionalForm]]],
 ". However, the complex exponential function ",
 StyleBox["is",
  FontSlant->"Italic"],
 " onto, since for every non-zero complex number in polar form, \n\n          \
         ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"z", " ", "=", " ", 
     RowBox[{"r", " ", 
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{"cos", " ", "\[Theta]"}], " ", "+", " ", 
        RowBox[{"i", " ", "sin", " ", "\[Theta]"}]}], ")"}]}]}]}], 
   TraditionalForm]]],
 ",\n\nthere is a complex number whose exponential is ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ", namely  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"ln", "(", "r", ")"}], " ", "+", " ", 
    RowBox[{"i", " ", "\[Theta]"}]}], TraditionalForm]]],
 ". This can be seen by observing that\n\n       ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    SuperscriptBox["e", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"ln", " ", "r"}], " ", "+", " ", 
        RowBox[{"i", " ", "\[Theta]"}]}], ")"}], " "}]], " ", "=", " ", 
    RowBox[{
     SuperscriptBox["e", 
      RowBox[{"ln", " ", "r"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"cos", " ", "\[Theta]"}], " ", "+", " ", 
       RowBox[{"i", " ", "sin", " ", "\[Theta]"}]}], ")"}]}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t                   ", 
    RowBox[{"=", "  ", 
     RowBox[{
      RowBox[{"r", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"cos", " ", "\[Theta]"}], " ", "+", " ", 
         RowBox[{"i", " ", "sin", " ", "\[Theta]"}]}], ")"}]}], " ", "=", " ",
       
      RowBox[{"z", "."}]}]}]}], TraditionalForm]}]],
 "\n\nThe drawback of the complex exponential function is that it is ",
 StyleBox["not",
  FontSlant->"Italic"],
 " one-to-one! Note that"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["E^( 2 Pi I)   ", "Input",
 PageWidth->Infinity],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.442754812648875*^9}]
}, Open  ]],

Cell[TextData[{
 "gives the same result as ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "0"], TraditionalForm]]],
 ". This suggests that we should look at the kernal of the group \
homomorphism, that is, the set of all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", " ", "=", " ", 
    RowBox[{"x", " ", "+", " ", 
     RowBox[{"i", " ", "y"}]}]}], TraditionalForm]]],
 " such that\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["e", "z"], "=", "  ", 
    RowBox[{
     SuperscriptBox["e", 
      RowBox[{"x", " ", "+", " ", 
       RowBox[{"y", " ", "i"}]}]], "=", " ", "1."}]}], TraditionalForm]]],
 "\n\nIt is apparent that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " must be ",
 Cell[BoxData[
  FormBox["1", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"cos", "(", "y", ")"}], " ", "=", " ", "1"}], ","}], 
   TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"sin", "(", "y", ")"}], " ", "=", " ", "0."}], 
   TraditionalForm]]],
 "\n\nThus, the kernal of the complex exponiential function is the set\n\n    \
                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"2", " ", "k", " ", "\[Pi]", " ", "i"}], "  ", "|", "  ", 
      RowBox[{"k", " ", "\[Element]", " ", "\[DoubleStruckCapitalZ]"}]}], " ",
      "}"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.442754823555125*^9, 3.442754823555125*^9}}],

Cell[TextData[{
 "This is, of course, a normal subgroup of the additive group ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalC]", "+"], TraditionalForm]]],
 ". We can denote this set by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", " ", "=", " ", 
    RowBox[{
     SuperscriptBox["\[Florin]", 
      RowBox[{"-", "1"}]], "(", "1", ")"}]}], TraditionalForm]]],
 ". We can similarly define the inverse exponiential for any non-zero number.\
\n\nDEFINITION 11.8:\nFor any non-zero complex number ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ", we define the ",
 StyleBox["complex logarithm",
  FontSlant->"Italic"],
 " of ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ", denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"log", "(", "z", ")"}], TraditionalForm]]],
 ", to be the set of elements ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["e", 
     RowBox[{"x", " "}]], "=", " ", 
    RowBox[{"z", "."}]}], TraditionalForm]]],
 "\n\nNotice that we use the function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"ln", "(", "x", ")"}], TraditionalForm]]],
 " to denote the ",
 StyleBox["real",
  FontSlant->"Italic"],
 " logarithm, while we use ",
 Cell[BoxData[
  FormBox[
   RowBox[{"log", "(", "z", ")"}], TraditionalForm]]],
 " to denote the complex logarithm.\n\nWe have already observed that when ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " is written in polar form,\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", " ", "=", " ", 
    RowBox[{"r", " ", 
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{"cos", " ", "\[Theta]"}], " ", "+", " ", 
       RowBox[{"i", " ", "sin", " ", "\[Theta]"}]}], ")"}]}]}], 
   TraditionalForm]]],
 ",\n\nthat one value of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " which satisfies the equation is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{
     RowBox[{"ln", "(", "r", ")"}], " ", "+", " ", 
     RowBox[{"\[Theta]", " ", "i"}]}]}], TraditionalForm]]],
 ". We also know that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Florin]", 
     RowBox[{"-", "1"}]], "(", "z", ")"}], TraditionalForm]]],
 " will be a coset of the kernal of ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 ". Thus, we have\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"log", "(", "z", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"ln", "(", "r", ")"}], " ", "+", 
     RowBox[{"\[Theta]", " ", "i"}], " ", "+", " ", "N"}]}], 
   TraditionalForm]]],
 ".\n\nFor example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"log", "(", 
    RowBox[{"-", "1"}], ")"}], TraditionalForm]]],
 " is the set\n\n           ",
 Cell[BoxData[{
  RowBox[{
   FormBox[
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Pi]", " ", "i"}], " ", "+", " ", 
       RowBox[{"2", "k", " ", "\[Pi]", " ", "i"}]}], "  ", "|", "   ", 
      RowBox[{"k", " ", "\[Element]", " ", "\[DoubleStruckCapitalZ]"}]}], " ",
      "}"}],
    TraditionalForm], " ", "\n"}], "\n", 
  RowBox[{"             ", 
   RowBox[{"=", " ", 
    RowBox[{"{", 
     RowBox[{"\[Ellipsis]", ",", " ", 
      RowBox[{
       RowBox[{"-", "5"}], "\[Pi]", " ", "i"}], ",", " ", 
      RowBox[{
       RowBox[{"-", "3"}], " ", "\[Pi]", " ", "i"}], ",", " ", 
      RowBox[{
       RowBox[{"-", "\[Pi]"}], " ", "i"}], ",", " ", 
      RowBox[{"\[Pi]", " ", "i"}], ",", " ", 
      RowBox[{"3", "\[Pi]", " ", "i"}], ",", " ", 
      RowBox[{"5", "\[Pi]", " ", "i"}], ",", "\[Ellipsis]"}], "}"}]}]}]}]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437532278984375*^9, 3.437532280609375*^9}, {
  3.437532325671875*^9, 3.437532325671875*^9}, {3.437535155734375*^9, 
  3.43753515621875*^9}}],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " ",
 StyleBox["Log",
  FontWeight->"Bold"],
 " function works for complex numbers,"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Log[I]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 FractionBox[
  RowBox[{"\[ImaginaryI]", " ", "\[Pi]"}], "2"]], "Output",
 CellChangeTimes->{3.442754829680125*^9}]
}, Open  ]],

Cell[TextData[{
 "but only gives one element of the set. Thus, we must add the kernal ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " to this result to obtain the set given by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"log", "(", "z", ")"}], TraditionalForm]]],
 ".\n\nIt is informative to see the graphs of the complex exponiential and \
complex logarithm. Even though these graphs are really surfaces in four \
dimensions, we can look at the real part and the complex part of the \
functions separately, giving us two three dimensional graphs. For example, \
the real part of the exponiential function is given by:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Plot3D[E^x Cos[y], {x,-2,2},{y,-6,6}, PlotRange -> All]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx0nXVcVU/zx0kBAQUBUbADu7u4xwa7sRu7O7C7Fbu7QQUVC7wXFFEUUEFQ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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFnXf8V+P7x8/7vM/7zDchI2SLkJ3skIQySpTIKIVQ9spsWNl7p1Q2yR6R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         "]], Polygon3DBox[CompressedData["
1:eJxFnXnYV9PXxs93OPOTqZLMKiJSIkOIDEUDSRMhlQYVmaVIkjFCIqEQKhky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         "]], Polygon3DBox[CompressedData["
1:eJwt1wnYzlUax/FH3tWbeEWSLUKv7NlL2aKyb4Op11qWpEy0U9YQY8u+CzGW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         "]]}]}, {}, {}, {}}, {
     {GrayLevel[0], 
      Line3DBox[{1069, 1, 971, 333, 2014, 16, 2019, 31, 2023, 46, 2026, 61, 
       2040, 76, 2055, 91, 2071, 2072, 106, 2088, 121, 2092, 136, 2096, 151, 
       2100, 166, 2114, 181, 2128, 196, 2447, 525, 1080, 211, 1068, 577, 1278,
        212, 1279, 213, 1280, 214, 1281, 215, 1282, 216, 1283, 217, 1284, 218,
        1285, 219, 1286, 220, 1287, 221, 1288, 222, 1289, 223, 1290, 224, 
       1642, 538, 1081, 225, 1029, 537, 2155, 210, 886, 2141, 195, 512, 2671, 
       845, 180, 1025, 497, 2669, 842, 165, 2420, 483, 150, 2753, 939, 135, 
       2383, 452, 828, 120, 2803, 1008, 441, 2086, 105, 870, 2068, 90, 403, 
       2053, 75, 979, 392, 2644, 807, 60, 973, 2344, 379, 45, 2680, 857, 30, 
       2305, 347, 1070, 15, 972, 346, 1185, 14, 1184, 13, 1183, 12, 1181, 11, 
       1179, 10, 1867, 891, 9, 1866, 890, 8, 889, 1176, 7, 888, 1175, 6, 1173,
        5, 1171, 4, 1169, 3, 1167, 2, 1643, 539, 1069}]}, {
      Line3DBox[{2014, 2292, 2880, 3097, 2013, 2998, 2156, 2015, 2999, 2157, 
       2016, 3000, 2158, 2017, 2995, 3001, 2159, 2865, 2996, 3204, 2866, 2293,
        2160, 2506, 3140, 2538, 2539, 2294, 2161, 2507, 2540, 2541, 2295, 
       2881, 3082, 2508, 2542, 2543, 2296, 3098, 2162, 2509, 2544, 2545, 2298,
        3099, 2163, 2297, 2546, 2547, 2300, 3100, 2164, 2299, 2548, 2549, 
       3002, 2302, 2165, 2301, 2550, 3148, 2551, 2304, 2166, 2303, 3101, 2552,
        2553, 2306, 2167, 2305}], 
      Line3DBox[{2019, 2882, 3024, 2018, 2883, 3025, 2020, 3003, 2168, 2021, 
       2915, 3004, 2169, 2307, 2916, 3005, 2170, 2308, 3102, 2211, 2171, 2309,
        3103, 2708, 2212, 2673, 2837, 2838, 2310, 2709, 2213, 2674, 2839, 
       2840, 3198, 2311, 2710, 2214, 2675, 2841, 3083, 2993, 2312, 2711, 2215,
        2676, 3199, 2842, 2843, 2313, 2712, 2216, 2677, 3200, 2844, 2845, 
       2314, 2713, 2917, 3006, 2678, 2846, 2847, 2315, 2714, 3172, 2217, 2679,
        2848, 2849, 2316, 3104, 2715, 2218, 2681, 2850, 2851, 2680}], 
      Line3DBox[{2023, 2884, 3026, 2022, 2885, 3027, 2024, 2630, 2886, 3160, 
       2631, 2940, 3007, 2172, 2450, 3135, 2317, 2451, 2173, 2318, 3105, 2219,
        2453, 2174, 2321, 2988, 3190, 2220, 2455, 2175, 2324, 2817, 2221, 
       2457, 2632, 3084, 2970, 2327, 2818, 2222, 2459, 2633, 3085, 2971, 2330,
        2820, 2223, 2461, 2972, 3086, 2634, 2333, 2822, 2224, 2463, 3161, 
       2635, 2636, 2336, 2824, 3191, 2225, 2465, 2637, 2638, 2338, 2826, 3192,
        2226, 2467, 2639, 2640, 2341, 2989, 3193, 2227, 2469, 2641, 2642, 
       2344}], Line3DBox[{2026, 2887, 3028, 2025, 2888, 3029, 2027, 2889, 
       3030, 2028, 2942, 3031, 2347, 2029, 3106, 2319, 2452, 2320, 2589, 2030,
        3107, 2322, 2454, 2323, 2590, 2031, 2961, 3108, 2325, 2456, 2326, 
       2591, 2032, 2592, 2328, 2458, 2329, 3152, 2962, 2033, 2819, 2331, 2460,
        2332, 3153, 2963, 2034, 2821, 2334, 2462, 2335, 3154, 2593, 2035, 
       2823, 2337, 2464, 3032, 2941, 2594, 2036, 2825, 3109, 2339, 2466, 2340,
        2595, 2037, 2827, 3110, 2342, 2468, 2343, 2596, 2038, 2990, 3111, 
       2345, 2470, 2346, 2770, 2643, 2644}], 
      Line3DBox[{2040, 2890, 3033, 2039, 2891, 3034, 2041, 2892, 3035, 2042, 
       2943, 3136, 2471, 2348, 2043, 2918, 3112, 2349, 2228, 2044, 3087, 2229,
        2350, 2230, 2045, 3088, 2231, 2351, 2232, 2046, 2233, 2352, 2234, 
       3113, 2047, 2765, 2235, 2353, 2236, 3114, 2048, 2766, 2237, 2354, 3036,
        2919, 2554, 2049, 2767, 2238, 2355, 3037, 2920, 2555, 2050, 2768, 
       2921, 3179, 2356, 2239, 2556, 2051, 2769, 3089, 2240, 2357, 2241, 2557,
        2052, 3180, 2771, 2242, 2358, 2243, 2558, 2053}], 
      Line3DBox[{2055, 2893, 3038, 2054, 2894, 3039, 2056, 2359, 3040, 2944, 
       2057, 2922, 3041, 2244, 2058, 2923, 3173, 2716, 2245, 2059, 2924, 3042,
        2246, 2060, 3174, 2717, 2718, 2682, 2720, 2719, 2061, 2721, 2722, 
       2683, 2724, 2723, 3115, 2062, 2725, 2726, 2684, 2727, 3175, 2977, 2063,
        2728, 2729, 2685, 3043, 2731, 2730, 2064, 2732, 2733, 2686, 3044, 
       2735, 2734, 2065, 2736, 2737, 3176, 2687, 2739, 2738, 2066, 2740, 3167,
        2741, 2688, 2743, 2742, 2067, 3177, 2744, 2745, 2689, 2247, 2068}], 
      Line3DBox[{2072, 3008, 2176, 2070, 3009, 2177, 2074, 2646, 3010, 2178, 
       2647, 2946, 3011, 2472, 2521, 2179, 2510, 2948, 3012, 2474, 2180, 2473,
        2950, 3013, 2476, 2181, 2475, 2991, 3194, 2871, 2828, 2478, 2648, 
       2649, 2182, 2477, 2829, 2872, 2830, 2479, 2650, 2651, 3162, 2183, 2368,
        2775, 2873, 2776, 2480, 2652, 2653, 3163, 2184, 2369, 2780, 2874, 
       2781, 2481, 2974, 3137, 2654, 2185, 2370, 2785, 2875, 2786, 2482, 3164,
        2655, 2656, 2186, 2371, 2791, 2876, 3206, 2984, 2483, 2657, 2658, 
       2187, 2372, 2795, 2877, 2796, 2484, 2659, 2660, 2188, 2373, 2986, 3185,
        2878, 2801, 2485, 2661, 2836, 2189, 2803}], 
      Line3DBox[{2086, 2804, 2805, 2363, 2802, 3118, 2987, 2085, 2799, 2800, 
       2362, 2798, 3117, 2797, 2084, 2793, 2794, 2361, 2985, 3116, 2792, 2083,
        2789, 2790, 3050, 2360, 2788, 2787, 2082, 2784, 3184, 2983, 2692, 
       2783, 2782, 2081, 2982, 3183, 2779, 2691, 2778, 2777, 2080, 2981, 3182,
        2774, 2690, 2367, 2079, 2869, 2870, 2868, 2879, 3205, 2997, 2078, 
       2366, 3049, 2949, 2077, 2365, 2772, 2980, 3181, 2773, 2076, 2364, 3048,
        2945, 2075, 2973, 3047, 2645, 2073, 3046, 2896, 2069, 3045, 2895, 
       2071}], Line3DBox[{2088, 2897, 3051, 2087, 3014, 2190, 2089, 3015, 
       2191, 2090, 2947, 3016, 2192, 2511, 2597, 2598, 3017, 2374, 2193, 2512,
        2964, 3155, 2599, 2375, 2194, 2513, 3141, 2600, 2601, 2376, 2522, 
       2195, 2514, 2602, 2603, 2377, 2523, 2898, 3144, 2515, 2604, 2605, 2378,
        2524, 3119, 2196, 2516, 2606, 2607, 2379, 3145, 2525, 2197, 2517, 
       2608, 2609, 2380, 3146, 2526, 2198, 2518, 2610, 2611, 3018, 2381, 2527,
        2199, 2519, 2612, 3156, 2613, 2382, 2528, 2200, 2520, 3142, 2614, 
       2615, 2384, 2806, 2201, 2383}], 
      Line3DBox[{2092, 2899, 3052, 2091, 2900, 3053, 2093, 3019, 2202, 2094, 
       2925, 3020, 2203, 2385, 2926, 3021, 2204, 2386, 3120, 2248, 2205, 2387,
        3121, 2746, 2249, 2693, 2852, 2853, 2388, 2747, 2250, 2694, 2854, 
       2855, 3201, 2389, 2748, 2251, 2695, 2856, 3090, 2994, 2390, 2749, 2252,
        2696, 3202, 2857, 2858, 2391, 2750, 2253, 2697, 3203, 2859, 2860, 
       2392, 2751, 2927, 3022, 2698, 2861, 2862, 2393, 2752, 3178, 2254, 2699,
        2863, 2864, 2394, 3122, 2867, 2255, 2754, 2206, 2753}], 
      Line3DBox[{2096, 2901, 3054, 2095, 2902, 3055, 2097, 2903, 3056, 2098, 
       2951, 3023, 2207, 2395, 3123, 2256, 2208, 2396, 3124, 2257, 2486, 2209,
        2397, 3125, 2258, 2488, 2210, 2400, 2259, 2490, 2904, 3091, 2403, 
       2831, 2260, 2492, 2905, 3092, 2406, 2832, 2261, 2494, 2959, 3093, 2529,
        2409, 2833, 2262, 2496, 3147, 2530, 2531, 2412, 2834, 3195, 2263, 
       2498, 2532, 2533, 2414, 2835, 3196, 2264, 2500, 2534, 2535, 2417, 2992,
        3197, 2265, 2502, 2536, 2537, 2420}], 
      Line3DBox[{2100, 2906, 3057, 2099, 2907, 3058, 2101, 2663, 2664, 3165, 
       2102, 2953, 3059, 2423, 2103, 3131, 2425, 2426, 2104, 2965, 3126, 2398,
        2487, 2399, 2616, 2105, 2966, 3127, 2401, 2489, 2402, 2617, 2106, 
       2618, 2404, 2491, 2405, 3157, 2967, 2107, 2619, 2407, 2493, 2408, 3158,
        2968, 2108, 2620, 2410, 2495, 2411, 3159, 2621, 2109, 2622, 2413, 
       2497, 3060, 2952, 2623, 2110, 2624, 3128, 2415, 2499, 2416, 2625, 2111,
        2626, 3129, 2418, 2501, 2419, 2627, 2112, 2969, 3130, 2421, 2503, 
       2422, 2815, 2662, 2669}], 
      Line3DBox[{2114, 2908, 3061, 2113, 2909, 3062, 2115, 2910, 3063, 2116, 
       2954, 3138, 2504, 2424, 2628, 2117, 2955, 3139, 2505, 2427, 2629, 2118,
        3094, 2266, 2428, 2267, 2559, 2119, 3187, 2808, 2268, 2429, 2269, 
       2560, 2120, 2809, 2270, 2430, 2271, 2561, 3143, 2121, 2810, 2272, 2431,
        2273, 3166, 2975, 2122, 2811, 2274, 2432, 3064, 2928, 2665, 2123, 
       2812, 2275, 2433, 3065, 2929, 2666, 2124, 2813, 2930, 3188, 2434, 2276,
        2667, 2125, 2814, 3095, 2277, 2435, 2278, 2668, 2126, 3189, 2816, 
       2279, 2436, 2280, 2670, 2671}], 
      Line3DBox[{2128, 2911, 3066, 2127, 2912, 3067, 2129, 2913, 3068, 2130, 
       2931, 3069, 2281, 2131, 2932, 3186, 2807, 2282, 2132, 2933, 3132, 2437,
        2283, 2133, 3096, 2284, 2438, 2563, 2562, 2134, 2564, 2565, 2439, 
       2567, 2566, 3134, 2135, 2568, 2569, 2440, 2570, 3149, 2960, 2136, 2571,
        2572, 2441, 3070, 2574, 2573, 2137, 2575, 2576, 2442, 3071, 2578, 
       2577, 2138, 2579, 2580, 3150, 2443, 2582, 2581, 2139, 2583, 3133, 2584,
        2444, 2586, 2585, 2140, 3151, 2587, 2588, 2445, 2756, 2755, 2141}], 
      Line3DBox[{2155, 2291, 2707, 2764, 3171, 2979, 2154, 2290, 2706, 2763, 
       3170, 2762, 2153, 2289, 2705, 2978, 3169, 2761, 2152, 2939, 3081, 2704,
        2760, 2759, 2151, 2938, 3080, 2703, 2758, 2757, 2150, 2937, 3079, 
       2702, 2288, 2149, 2936, 3078, 2701, 2287, 2148, 2286, 2700, 3168, 2935,
        2147, 2285, 3077, 2934, 2146, 2449, 3076, 2958, 2145, 2448, 3075, 
       2957, 2144, 2976, 3074, 2672, 2143, 3073, 2914, 2142, 2956, 3072, 2446,
        2447}]}, {
      Line3DBox[{1167, 1292, 2998, 1168, 3025, 1300, 1187, 3027, 1305, 1197, 
       3029, 1310, 1203, 3034, 1315, 1208, 3039, 1320, 1217, 3046, 1325, 3009,
        1226, 1330, 3014, 1231, 3053, 1335, 1239, 3055, 1340, 1249, 3058, 
       1345, 1256, 3062, 1350, 1261, 3067, 1357, 1269, 3073, 1362, 1279}], 
      Line3DBox[{1169, 1293, 2999, 1170, 1301, 3003, 1188, 1815, 1816, 3160, 
       1306, 1198, 3030, 1311, 1204, 3035, 1316, 1209, 3040, 1530, 1531, 1321,
        1218, 3047, 1828, 1829, 1326, 3010, 1227, 1331, 3015, 1232, 1336, 
       3019, 1240, 3056, 1341, 1250, 1841, 3165, 1842, 1346, 1257, 3063, 1351,
        1262, 3068, 1358, 1270, 3074, 1848, 1849, 1363, 1280}], 
      Line3DBox[{1171, 1294, 3000, 1172, 1376, 1377, 3004, 1189, 1493, 1817, 
       1494, 3007, 1199, 3031, 1513, 1514, 1648, 3136, 1515, 1516, 1210, 3041,
        1408, 1532, 1409, 1219, 3048, 1546, 1830, 1547, 3011, 1649, 1548, 
       1549, 3016, 1233, 1423, 1424, 3020, 1241, 1580, 1581, 3023, 1251, 3059,
        1597, 1843, 1598, 1658, 3138, 1599, 1600, 1263, 3069, 1449, 1450, 
       1271, 3075, 1629, 1850, 1630, 1281}], 
      Line3DBox[{1173, 1990, 1991, 3001, 1174, 1378, 1483, 1379, 3005, 1190, 
       1495, 1644, 3135, 1496, 1645, 1498, 3106, 1664, 1499, 1770, 1517, 3112,
        1396, 1674, 1397, 1999, 1876, 3173, 1410, 1533, 1411, 1915, 3181, 
       1916, 1550, 1675, 1551, 3012, 1651, 1554, 1676, 1555, 3017, 1786, 1564,
        1425, 1570, 1426, 3021, 1242, 1434, 1582, 3123, 1435, 1252, 1601, 
       3131, 1686, 1602, 1659, 3139, 1603, 1696, 1604, 2004, 1931, 3186, 1451,
        1618, 1452, 1272, 3076, 1631, 1660, 1632, 1282}], 
      Line3DBox[{1175, 1368, 1992, 1994, 1993, 3204, 1473, 1380, 1484, 3102, 
       1381, 1191, 1387, 1497, 3105, 1388, 1646, 1501, 3107, 1665, 1772, 1771,
        1518, 1398, 3087, 1521, 1399, 1211, 3042, 1412, 1534, 1413, 1220, 
       3049, 1552, 1650, 1553, 3013, 1653, 1556, 1677, 1788, 1787, 3155, 1565,
        1427, 1571, 3120, 1428, 1243, 1436, 1583, 3124, 1437, 1655, 1800, 
       1801, 3126, 1687, 1585, 1605, 1444, 3094, 1697, 1753, 1752, 1607, 3132,
        1453, 1619, 1454, 1273, 3077, 1460, 1633, 1461, 1283}], 
      Line3DBox[{1176, 1369, 1661, 1725, 3140, 1724, 1474, 1382, 1485, 1869, 
       3103, 1868, 1851, 1389, 1500, 3190, 1941, 1940, 1942, 1773, 1774, 3108,
        1666, 1776, 1775, 1519, 1400, 3088, 1522, 1401, 1855, 1880, 3174, 
       1881, 1535, 1414, 2008, 3205, 2011, 2012, 1652, 1950, 1949, 3194, 2009,
        1951, 1952, 1678, 1790, 3141, 1789, 1566, 1429, 1572, 1892, 3121, 
       1891, 1859, 1438, 1584, 3125, 1439, 1656, 1802, 1803, 3127, 1688, 1805,
        1804, 2005, 1932, 3187, 1933, 1698, 1755, 1754, 1608, 1455, 3096, 
       1620, 1905, 1904, 1862, 3168, 1462, 1634, 1463, 1284}], 
      Line3DBox[{8, 580, 2507, 23, 352, 2310, 38, 365, 2324, 53, 2032, 586, 
       68, 2046, 396, 83, 2061, 409, 98, 2079, 557, 2477, 113, 598, 2514, 128,
        457, 2388, 143, 469, 2400, 158, 2106, 608, 173, 2120, 618, 188, 2134, 
       517, 203, 2148, 530, 218}], 
      Line3DBox[{1179, 1370, 1475, 1297, 3099, 1180, 1733, 1383, 1488, 1964, 
       3199, 1965, 1194, 1392, 1504, 3086, 1820, 1821, 1822, 1781, 1505, 3154,
        1669, 1702, 1703, 1826, 1738, 3036, 1404, 1525, 1970, 1971, 1214, 
       1886, 3043, 1415, 1538, 1976, 1977, 1223, 3184, 1921, 1419, 1922, 1833,
        1834, 3137, 1835, 1795, 1559, 1681, 1710, 3145, 1711, 1236, 1747, 
       1430, 1575, 1980, 3202, 1981, 1246, 1899, 1440, 1588, 3093, 1714, 1715,
        1839, 1810, 1589, 3159, 1691, 1720, 1721, 1960, 1846, 3064, 1445, 
       1610, 1354, 1266, 1765, 3070, 1456, 1623, 1986, 1987, 1276, 3080, 1466,
        1637, 1366, 1287}], 
      Line3DBox[{1181, 1371, 1476, 1298, 3100, 1182, 1734, 1384, 1489, 1966, 
       3200, 1967, 1195, 1393, 1506, 1823, 3161, 1824, 1825, 1782, 3032, 1508,
        1670, 1704, 1705, 1827, 1739, 3037, 1405, 1526, 1972, 1973, 1215, 
       1887, 3044, 1416, 1539, 1706, 1707, 1224, 1923, 3050, 1420, 1924, 1836,
        3164, 1837, 1838, 1796, 1560, 1682, 1712, 3146, 1713, 1237, 1748, 
       1431, 1576, 1982, 3203, 1983, 1247, 1900, 1441, 1590, 1716, 3147, 1717,
        1840, 1811, 3060, 1592, 1692, 1722, 1723, 1961, 1847, 3065, 1446, 
       1611, 1355, 1267, 1766, 3071, 1457, 1624, 1988, 1989, 1277, 3081, 1467,
        1638, 1367, 1288}], 
      Line3DBox[{1183, 1372, 1477, 1373, 3002, 1726, 1479, 1735, 1385, 1490, 
       1386, 3006, 1870, 1852, 1873, 1394, 1507, 3191, 1395, 1943, 1647, 1944,
        1510, 3109, 1671, 1511, 1783, 1520, 1912, 3179, 1406, 1527, 1407, 
       1877, 1856, 1888, 3176, 1417, 1540, 1418, 1740, 1542, 3116, 1926, 1421,
        2003, 1422, 1925, 3206, 1654, 1953, 1561, 1683, 1562, 3018, 1797, 
       1567, 1749, 1432, 1577, 1433, 3022, 1893, 1860, 1901, 1442, 1591, 3195,
        1443, 1956, 1657, 1812, 1594, 3128, 1693, 1595, 1934, 1606, 1935, 
       3188, 1447, 1612, 1448, 1758, 1614, 1767, 3150, 1458, 1625, 1459, 1906,
        1863, 3169, 1909, 1468, 1639, 1469, 1289}], 
      Line3DBox[{1184, 1374, 1478, 659, 3148, 1481, 672, 1491, 3172, 898, 
       1853, 904, 1509, 3192, 1042, 1151, 1043, 3110, 1672, 742, 1129, 978, 
       3089, 1528, 912, 1857, 3167, 929, 1541, 682, 1544, 3117, 1003, 1135, 
       1002, 2877, 1162, 1054, 1684, 762, 3156, 1568, 694, 1578, 3178, 938, 
       1861, 953, 1593, 3196, 1064, 1065, 780, 3129, 1694, 1023, 1145, 1024, 
       3095, 1613, 707, 1616, 3133, 720, 1626, 963, 1864, 3170, 969, 1640, 
       1470, 1290}], 
      Line3DBox[{1185, 1375, 1480, 1728, 3101, 1727, 1482, 1736, 1737, 1492, 
       1872, 3104, 1871, 1854, 1874, 1875, 1512, 3193, 1946, 1945, 2007, 1947,
        1948, 3111, 1673, 1785, 1784, 2002, 1913, 3180, 1914, 1529, 1879, 
       1878, 1858, 1889, 3177, 1890, 1543, 1742, 1741, 1545, 3118, 1929, 1930,
        1563, 1928, 1927, 3185, 2010, 1954, 1955, 1685, 1799, 3142, 1798, 
       1569, 1750, 1751, 1579, 2001, 3122, 2000, 1894, 1902, 1903, 1596, 3197,
        1958, 1957, 1959, 1813, 1814, 3130, 1695, 1937, 1936, 2006, 1938, 
       3189, 1939, 1615, 1760, 1759, 1617, 1768, 3151, 1769, 1627, 1908, 1907,
        1865, 3171, 1910, 1911, 1641, 1471, 1642}], 
      Line3DBox[{1278, 1361, 1628, 3072, 1268, 1356, 3066, 1260, 1349, 3061, 
       1255, 1344, 3057, 1248, 1339, 3054, 1238, 1334, 3052, 1230, 1329, 3051,
        1225, 3008, 1324, 3045, 1216, 1319, 3038, 1207, 1314, 3033, 1202, 
       1309, 3028, 1196, 1304, 3026, 1186, 1299, 3024, 1166, 1291, 3097, 1472,
        1643}], 
      Line3DBox[{1285, 1364, 1635, 1464, 3078, 1274, 1359, 1984, 1621, 1762, 
       3134, 1761, 1264, 1352, 1699, 1757, 3143, 1756, 1258, 1347, 1718, 1689,
        3157, 1807, 1806, 1253, 1342, 3091, 1586, 1896, 1895, 1244, 1337, 
       3201, 1978, 1573, 1744, 1743, 1234, 1332, 3144, 1708, 1679, 1792, 1791,
        1228, 3162, 1327, 1831, 1557, 1918, 1917, 3182, 1221, 1322, 1974, 
       1536, 1883, 3115, 1882, 1212, 1317, 1968, 1523, 3113, 1402, 1205, 1312,
        1700, 1667, 3152, 1778, 1777, 1200, 1307, 1818, 3084, 1502, 1390, 
       1192, 1302, 3198, 1962, 1486, 1730, 1729, 1177, 1295, 3082, 1662, 1996,
        1995, 1866}], 
      Line3DBox[{1286, 1365, 1636, 1465, 3079, 1275, 1360, 1985, 1622, 1764, 
       1763, 3149, 1265, 1353, 1609, 3166, 1845, 1844, 1259, 1348, 1719, 1690,
        3158, 1809, 1808, 1254, 1343, 3092, 1587, 1898, 1897, 1245, 1338, 
       1979, 3090, 1574, 1746, 1745, 1235, 3119, 1333, 1709, 1680, 1794, 1793,
        1229, 3163, 1328, 1832, 1558, 1920, 1919, 3183, 1222, 1323, 1975, 
       1537, 1885, 1884, 3175, 1213, 1318, 1969, 1524, 3114, 1403, 1206, 1313,
        1701, 1668, 3153, 1780, 1779, 1201, 1308, 1819, 3085, 1503, 1391, 
       1193, 1303, 1963, 3083, 1487, 1732, 1731, 1178, 3098, 1296, 1663, 1998,
        1997, 1867}]}, {}, {}}},
   VertexNormals->CompressedData["
1:eJx0vXlczV33Ph6ilChjMlMkhCgi7SJNiEKUypCizBGSNCBCiQohjSKKJqlU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    "]],
  Axes->True,
  BoxRatios->{1, 1, 0.4},
  Method->{"RotationControl" -> "Globe"},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.44275483550825*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["The graph of the imaginary part is very similar.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Plot3D[E^x Sin[y], {x,-2,2},{y,-6,6}, PlotRange -> All]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJyEnXVYVdu39wklFBXE7u46dqB72d3d3Z3HOMaxO1Cxuwsb270BG5UQUEBF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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFnXXcFdXXxe+duJOPAaIoYICdSKgodgIqdidgByp2gqAodmMHdnc3Kip2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         "]], Polygon3DBox[CompressedData["
1:eJwtnXPc7EgThcczmcld275r2969a9u2bdu8a35r27Zt2za/59mTP/J7u9JI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         "]], Polygon3DBox[CompressedData["
1:eJwt13m8z1Uex/FvuJfr3rIlyZQty1SyFioalKVsWSaufV/KVmPGkpAW1Uz2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         "]], 
        Polygon3DBox[{{2197, 2196, 3108, 1476, 1477, 
         3109}}]}]}, {}, {}, {}}, {
     {GrayLevel[0], 
      Line3DBox[{1157, 1, 1036, 321, 2134, 16, 2138, 31, 2142, 46, 2146, 61, 
       2161, 76, 2176, 91, 2192, 2193, 106, 2210, 121, 2213, 136, 2216, 151, 
       2220, 166, 2235, 181, 2250, 196, 2565, 519, 1177, 211, 1156, 582, 1361,
        212, 1362, 213, 1363, 214, 1364, 215, 2018, 1028, 216, 2019, 1029, 
       217, 1365, 218, 926, 1366, 219, 927, 1367, 220, 928, 1368, 221, 929, 
       1369, 222, 1957, 930, 223, 1958, 931, 224, 1698, 529, 894, 225, 810, 
       528, 2277, 210, 1101, 516, 890, 2263, 195, 503, 2248, 180, 924, 2233, 
       165, 2549, 471, 869, 150, 1091, 2529, 460, 1167, 135, 2850, 915, 120, 
       2895, 987, 2208, 105, 1230, 414, 854, 2189, 90, 1072, 402, 2174, 75, 
       906, 2159, 60, 2471, 368, 45, 1044, 2457, 356, 833, 30, 2446, 343, 824,
        15, 737, 342, 1275, 14, 947, 1274, 13, 944, 1273, 12, 1967, 941, 11, 
       1965, 938, 10, 1962, 935, 9, 1959, 932, 8, 1270, 7, 1269, 6, 1268, 5, 
       1266, 4, 1264, 3, 1262, 2, 1699, 530, 1157}]}, {
      Line3DBox[{2134, 2428, 3062, 3287, 2133, 3186, 2278, 2135, 3187, 2279, 
       2136, 3131, 3188, 2280, 2429, 3101, 3189, 2566, 2281, 2430, 3102, 3190,
        2567, 2282, 2431, 3288, 2330, 2568, 2283, 2433, 2957, 2959, 2958, 
       2569, 2775, 2776, 3063, 3353, 2435, 2960, 2962, 2961, 2570, 2777, 2778,
        3354, 2284, 2437, 2963, 2965, 2964, 2571, 1902, 2779, 2285, 2439, 
       2966, 2968, 2967, 2572, 3355, 2780, 2781, 2286, 2441, 2969, 2971, 2970,
        3191, 2573, 2782, 2783, 2287, 2443, 2972, 3050, 2973, 2574, 2784, 
       2785, 2288, 2444, 3290, 2974, 3051, 2975, 2575, 2786, 3007, 2289, 
       2446}], Line3DBox[{2138, 3064, 3214, 2137, 3065, 3215, 2139, 3192, 
       2290, 2140, 3134, 3193, 2576, 2291, 2616, 3132, 3194, 2448, 2646, 2647,
        2617, 3161, 3342, 2432, 2449, 2648, 2649, 2618, 3319, 2713, 2434, 
       2450, 2650, 2651, 2619, 2714, 2436, 2451, 2652, 2653, 3326, 2620, 2715,
        2438, 2452, 2654, 3289, 3150, 2621, 2716, 2440, 2453, 3327, 2655, 
       2656, 2622, 2717, 2442, 2454, 3328, 2657, 2658, 2623, 2718, 3133, 3195,
        2455, 2659, 2660, 2624, 2913, 3372, 2445, 2456, 2661, 2662, 2625, 
       3320, 2915, 2447, 2458, 2787, 2914, 2788, 2457}], 
      Line3DBox[{2142, 3066, 3216, 2141, 3067, 3217, 2143, 3068, 3218, 2144, 
       3135, 3196, 2292, 2626, 3032, 3385, 2331, 2860, 2293, 2627, 3183, 3386,
        2332, 2862, 2294, 2459, 3291, 2333, 2868, 2295, 2460, 2685, 2334, 
       2462, 3069, 3266, 2461, 2686, 2335, 2465, 2663, 3267, 3151, 2464, 2864,
        2336, 2826, 3181, 3268, 3017, 2467, 2865, 2337, 2828, 3383, 3018, 
       3019, 2468, 2866, 3362, 2338, 2830, 3020, 3021, 2469, 2867, 3363, 2339,
        2832, 3022, 3023, 2470, 3172, 3364, 2340, 2834, 3024, 3025, 2471}], 
      Line3DBox[{2146, 3070, 3219, 2145, 3071, 3220, 2147, 2472, 2473, 3292, 
       2148, 3103, 3221, 2341, 2149, 3269, 2342, 2861, 2343, 2150, 3270, 2344,
        2863, 2345, 2151, 3173, 3271, 2346, 2869, 2347, 2870, 2152, 2687, 
       2348, 2463, 2349, 3334, 3153, 2153, 2688, 2350, 2466, 2351, 3335, 3154,
        2154, 2872, 2352, 2827, 2353, 3365, 2873, 2155, 2874, 2354, 2829, 
       3222, 3104, 2875, 2156, 2876, 3272, 2355, 2831, 2356, 2877, 2157, 2878,
        3273, 2357, 2833, 2358, 2879, 2158, 3174, 3274, 2359, 2835, 2360, 
       2880, 2159}], 
      Line3DBox[{2161, 3072, 3223, 2160, 3073, 3224, 2162, 2789, 3225, 3168, 
       2163, 3136, 3226, 2474, 2164, 2917, 3138, 3373, 2916, 2475, 2165, 3293,
        2476, 2477, 2166, 3275, 2361, 2871, 2362, 2167, 2363, 2836, 2364, 
       2919, 2918, 3294, 2168, 2365, 2837, 2366, 2920, 3374, 3178, 2169, 2921,
        3035, 2367, 2838, 3227, 3105, 2923, 2922, 2170, 2924, 3036, 2368, 
       2839, 3228, 3106, 2926, 2925, 2171, 2927, 3037, 3107, 3375, 2840, 2369,
        2929, 2928, 2172, 2930, 3038, 2370, 2841, 2371, 2932, 2931, 2173, 
       3376, 2933, 3039, 2372, 2842, 2373, 2935, 2934, 2174}], 
      Line3DBox[{2176, 3074, 3229, 2175, 3075, 3230, 2177, 3076, 3231, 2178, 
       3137, 3312, 2577, 2720, 2719, 2179, 3139, 3313, 2578, 2722, 2721, 2180,
        3140, 3314, 2579, 2724, 2723, 2181, 3377, 2976, 3052, 2977, 2580, 
       2726, 2790, 2725, 2182, 2978, 3053, 2979, 2581, 2728, 2791, 2727, 3321,
        2183, 2980, 3054, 2981, 2582, 2729, 2792, 3343, 3162, 2184, 2982, 
       3055, 2983, 2583, 3232, 2731, 2793, 2730, 2185, 2984, 3056, 2985, 2584,
        3233, 2733, 2794, 2732, 2186, 2986, 3057, 2987, 3378, 2585, 2735, 
       2795, 2734, 2187, 2988, 3058, 2989, 2586, 2737, 2796, 2736, 2188, 3379,
        2990, 3059, 2991, 2587, 2798, 3008, 2797, 2189}], 
      Line3DBox[{2193, 3197, 2296, 2191, 3198, 2297, 2195, 3199, 2298, 2197, 
       3109, 3200, 2299, 2881, 3033, 3387, 3184, 2882, 2300, 2489, 3112, 3201,
        2301, 2490, 3175, 3366, 2883, 2843, 3026, 2302, 2491, 2884, 2885, 
       2844, 3027, 3359, 2303, 2492, 2886, 2887, 2845, 3028, 3360, 2304, 2493,
        2888, 2889, 2846, 3182, 3361, 2305, 2494, 2890, 2891, 2847, 3384, 
       3029, 2306, 2495, 2892, 3367, 3176, 2848, 3030, 2307, 2496, 2893, 3368,
        2894, 2849, 3031, 2308, 2497, 3185, 3388, 3034, 2898, 2309, 2895}], 
      Line3DBox[{2208, 2896, 2897, 2488, 3301, 3116, 2207, 2696, 2697, 2487, 
       3300, 3115, 2206, 2694, 2695, 2486, 3299, 3114, 2205, 2692, 2693, 3237,
        2485, 2381, 2204, 2691, 3338, 3157, 2484, 2380, 2203, 3156, 3337, 
       2690, 2483, 2379, 2202, 3155, 3336, 2689, 2482, 2378, 2201, 2377, 2481,
        3298, 3113, 2200, 2376, 2480, 3297, 3111, 2199, 2375, 2479, 3296, 
       3110, 2198, 2374, 2478, 3295, 3108, 2196, 3236, 3079, 2194, 3235, 3078,
        2190, 3234, 3077, 2192}], 
      Line3DBox[{2210, 3080, 3238, 2209, 3202, 2310, 2211, 2498, 3203, 2311, 
       2499, 3117, 3204, 2312, 2500, 3118, 3205, 2936, 2313, 2501, 3119, 3206,
        2937, 2314, 2502, 3302, 2382, 2588, 2315, 2503, 2383, 2589, 2664, 
       3081, 3329, 2505, 2384, 2590, 2665, 3315, 2316, 2507, 2385, 2591, 3330,
        2666, 2317, 2509, 2386, 2592, 3331, 2667, 2318, 2511, 3120, 3207, 
       2593, 2668, 2319, 2513, 3121, 3208, 2594, 2669, 2320, 2514, 3304, 2387,
        2595, 2851, 2321, 2850}], 
      Line3DBox[{2213, 3082, 3239, 2212, 3083, 3240, 2214, 2799, 3209, 2322, 
       2800, 3142, 3210, 2596, 2670, 2671, 2628, 3144, 3211, 2597, 2672, 2673,
        2629, 3322, 2518, 2598, 2674, 2675, 2630, 3323, 2992, 2504, 2599, 
       2801, 2938, 2802, 2520, 2939, 3040, 2506, 2523, 2803, 2940, 2804, 3356,
        2522, 2941, 3041, 2508, 2524, 2676, 3303, 3152, 2631, 2993, 2510, 
       2600, 3332, 2677, 2678, 2632, 2994, 2512, 2601, 3333, 2679, 2680, 2633,
        2995, 3141, 3212, 2602, 2681, 2682, 2634, 2996, 3380, 2515, 2603, 
       2683, 2684, 2635, 3324, 2997, 2516, 2604, 2805, 2942, 2806, 2529}], 
      Line3DBox[{2216, 3084, 3241, 2215, 3085, 3242, 2217, 3086, 3243, 2218, 
       3143, 3213, 2323, 2636, 2738, 3344, 2517, 2531, 2324, 2637, 3163, 3345,
        2519, 2533, 2325, 2638, 3164, 3346, 2521, 2535, 2326, 2639, 2698, 
       2388, 2537, 3087, 3276, 2640, 2699, 2389, 2539, 3088, 3277, 2641, 2739,
        2525, 2541, 3089, 3305, 2642, 2740, 2526, 2543, 3306, 2327, 2643, 
       2741, 3347, 2527, 2545, 2328, 2644, 2742, 3348, 2528, 2547, 2329, 2645,
        3165, 3349, 2530, 2550, 2943, 3042, 2944, 2549}], 
      Line3DBox[{2220, 3090, 3244, 2219, 3091, 3245, 2221, 3092, 3246, 2222, 
       3122, 3247, 2390, 2223, 3278, 2391, 2532, 2392, 2224, 3279, 2393, 2534,
        2394, 2225, 3280, 2395, 2536, 2396, 2700, 2226, 2701, 2397, 2538, 
       2398, 3339, 3158, 2227, 2702, 2399, 2540, 2400, 3340, 3159, 2228, 2703,
        2401, 2542, 2402, 3341, 2704, 2229, 2705, 2403, 2544, 3248, 3123, 
       2706, 2230, 2707, 3281, 2404, 2546, 2405, 2708, 2231, 2709, 3282, 2406,
        2548, 2407, 2710, 2232, 3160, 3283, 2408, 2551, 2409, 2899, 2233}], 
      Line3DBox[{2235, 3093, 3249, 2234, 3094, 3250, 2236, 3095, 3251, 2237, 
       3145, 3252, 2552, 2238, 3124, 3253, 2410, 2239, 3284, 2411, 2412, 2240,
        3285, 2413, 2852, 2414, 2241, 2900, 2415, 2853, 2416, 2901, 3307, 
       2242, 2902, 2417, 2854, 2418, 3369, 3177, 2243, 2903, 2419, 2855, 3254,
        3125, 2904, 2244, 2905, 2420, 2856, 3255, 3126, 2906, 2245, 2907, 
       3127, 3370, 2857, 2421, 2908, 2246, 2909, 3286, 2422, 2858, 2423, 2910,
        2247, 3371, 2911, 2424, 2859, 2425, 2912, 2248}], 
      Line3DBox[{2250, 3096, 3256, 2249, 3097, 3257, 2251, 3098, 3258, 2252, 
       3147, 3259, 2553, 2253, 3146, 3316, 2605, 2744, 2743, 2254, 3166, 3317,
        2745, 2606, 2747, 2746, 2255, 3350, 2748, 2749, 2607, 2751, 2750, 
       2256, 2752, 2998, 2753, 2608, 2755, 2807, 2754, 3325, 2257, 2756, 2999,
        2757, 2609, 2758, 2808, 3351, 3167, 2258, 2759, 3000, 2760, 2610, 
       3260, 2762, 2809, 2761, 2259, 2763, 3001, 2764, 2611, 3261, 2766, 2810,
        2765, 2260, 2767, 3002, 2768, 3352, 2612, 2770, 2811, 2769, 2261, 
       3003, 3060, 3004, 2613, 2772, 2812, 2771, 2262, 3381, 3005, 3061, 3006,
        2614, 2814, 3009, 2813, 2263}], 
      Line3DBox[{2277, 2824, 3016, 2825, 2563, 2956, 3049, 3389, 3180, 2276, 
       2822, 3015, 2823, 2562, 2955, 3048, 2954, 2275, 2820, 3014, 2821, 2561,
        3179, 3311, 3047, 2953, 2274, 2818, 3013, 2819, 3265, 2560, 2952, 
       3046, 2951, 2273, 2817, 3012, 3382, 3171, 2559, 2950, 3045, 2949, 2272,
        3170, 3358, 3011, 2816, 2558, 2948, 3044, 2947, 2271, 3169, 3357, 
       3010, 2815, 2557, 2946, 3043, 2945, 2270, 2427, 2556, 3310, 3130, 2269,
        2426, 2555, 3309, 3129, 2268, 2711, 2712, 2554, 3308, 3128, 2267, 
       2773, 2774, 2615, 3318, 3148, 2266, 3264, 3100, 2265, 3263, 3099, 2264,
        3149, 3262, 2564, 2565}]}, {
      Line3DBox[{1262, 1371, 3186, 1263, 3215, 1376, 1277, 3217, 1381, 1282, 
       3220, 1386, 1290, 3224, 1391, 1299, 3230, 1396, 1304, 3235, 1401, 3198,
        1311, 1406, 3202, 1320, 3240, 1411, 1328, 3242, 1416, 1333, 3245, 
       1423, 1341, 3250, 1428, 1351, 3257, 1433, 1357, 3263, 1438, 1362}], 
      Line3DBox[{1264, 1372, 3187, 1265, 1377, 3192, 1278, 3218, 1382, 1283, 
       1567, 3292, 1568, 1387, 1291, 3225, 1913, 1914, 1392, 1300, 3231, 1397,
        1305, 3236, 1402, 3199, 1312, 1615, 1616, 1407, 3203, 1321, 1924, 
       1925, 1412, 3209, 1329, 3243, 1417, 1334, 3246, 1424, 1342, 3251, 1429,
        1352, 3258, 1434, 1358, 3264, 1439, 1363}], 
      Line3DBox[{1266, 1533, 1534, 3188, 1267, 1551, 1552, 3193, 1704, 1553, 
       1554, 3196, 1284, 3221, 1461, 1569, 1462, 1292, 3226, 1580, 1915, 1581,
        1705, 3312, 1582, 1583, 1599, 3295, 1476, 1477, 3200, 1313, 1489, 
       1617, 1490, 3204, 1322, 1632, 1926, 1633, 3210, 1714, 1634, 1635, 3213,
        1335, 3247, 1502, 1503, 1343, 3252, 1663, 1664, 1353, 3259, 1681, 
       1682, 1722, 3318, 1683, 1684, 1364}], 
      Line3DBox[{1268, 1442, 1535, 1443, 3189, 1700, 1537, 1723, 1538, 3194, 
       1555, 1452, 1733, 3385, 1453, 2113, 1970, 1463, 3269, 1570, 1464, 2026,
        2027, 3373, 1584, 1735, 1585, 1707, 3313, 1588, 1736, 1589, 1850, 
       1600, 3296, 1478, 1987, 1479, 2118, 3387, 1988, 1491, 1618, 1492, 3205,
        2042, 2043, 1636, 1746, 1637, 3211, 1715, 1638, 1755, 3344, 1639, 
       1873, 1647, 1504, 3278, 1653, 1505, 1344, 3253, 1513, 1665, 1514, 1718,
        3316, 1667, 1765, 1668, 1685, 3308, 1523, 1775, 1524, 2121, 2018}], 
      Line3DBox[{1269, 1444, 1536, 1445, 3190, 1701, 1834, 1835, 1724, 3342, 
       1837, 1836, 1556, 1454, 1734, 3386, 2115, 2114, 1971, 1465, 3270, 1571,
        1466, 1293, 1586, 3293, 1706, 1587, 1708, 3314, 1591, 1737, 1852, 
       1851, 1601, 3297, 1480, 1606, 1481, 3201, 1314, 1493, 1619, 1494, 3206,
        2044, 2045, 2046, 1747, 3322, 1640, 1716, 1641, 1756, 3345, 1875, 
       1874, 1648, 1506, 3279, 1654, 1507, 1345, 1515, 3284, 1666, 1516, 1719,
        3317, 1883, 1884, 1766, 1886, 1885, 1686, 3309, 1525, 1776, 2123, 
       2122, 2019}], 
      Line3DBox[{1270, 1446, 1539, 3288, 1447, 1702, 1838, 1839, 1725, 1841, 
       3319, 1840, 1557, 1455, 1559, 3291, 1456, 1975, 1976, 1977, 3271, 1572,
        2117, 2116, 1978, 1471, 3275, 1590, 2060, 2059, 2128, 2061, 3377, 
       2062, 1738, 1854, 1853, 1602, 3298, 1482, 1607, 1990, 1989, 3366, 1950,
        1993, 1994, 1620, 3302, 1495, 1710, 1864, 1865, 1748, 2067, 3323, 
       2066, 2130, 2068, 2069, 1757, 3346, 1877, 1876, 1649, 1508, 3280, 1655,
        2005, 2004, 1953, 1517, 3285, 1669, 1518, 1720, 1887, 3350, 1888, 
       1767, 1890, 1889, 1687, 3310, 1526, 1690, 1527, 1365}], 
      Line3DBox[{8, 328, 2433, 23, 586, 2619, 38, 359, 2460, 53, 2152, 374, 
       68, 2167, 388, 83, 2182, 599, 98, 2201, 418, 2491, 113, 430, 2503, 128,
        451, 2520, 143, 617, 2639, 158, 2226, 476, 173, 2241, 489, 188, 2256, 
       627, 203, 2270, 521, 218}], 
      Line3DBox[{1273, 1969, 1450, 1545, 1451, 3191, 2054, 1703, 1848, 1548, 
       1730, 1549, 3195, 1849, 1558, 2021, 1459, 1564, 3362, 1460, 1972, 1944,
        1981, 1469, 3272, 1577, 1470, 1982, 1947, 2037, 3375, 1474, 2125, 
       1475, 2036, 1709, 2063, 3378, 1596, 1743, 1597, 1861, 1603, 3299, 1485,
        1612, 1486, 1991, 3367, 1951, 2001, 1498, 1626, 1499, 3207, 1711, 
       1870, 1629, 1752, 1630, 3212, 2070, 1717, 2071, 1645, 1762, 3347, 1646,
        1880, 1650, 1829, 1511, 3281, 1660, 1512, 2006, 1954, 2015, 3370, 
       1521, 1675, 1522, 2076, 1721, 1897, 3352, 1678, 1772, 1679, 1898, 1688,
        3311, 2049, 1530, 1695, 1531, 2020, 1957}], 
      Line3DBox[{1274, 948, 1547, 1115, 3050, 1243, 1116, 1731, 3372, 1042, 
       1223, 1043, 1565, 3363, 955, 1945, 972, 3273, 1578, 974, 1948, 1067, 
       3038, 1228, 1066, 1251, 3058, 1128, 1744, 757, 1604, 3300, 1487, 1613, 
       986, 3368, 1952, 1002, 1628, 1500, 3208, 1712, 767, 1753, 3380, 1138, 
       1257, 1139, 1763, 3348, 784, 1651, 709, 3282, 1661, 1011, 1955, 1024, 
       3286, 1677, 1152, 1259, 3060, 1153, 1773, 1099, 1241, 3048, 1100, 1696,
        1035, 1958}], 
      Line3DBox[{1275, 1807, 1808, 1550, 2056, 3290, 2055, 2127, 2057, 2058, 
       1732, 2023, 3320, 2022, 2124, 2024, 2025, 1566, 3364, 1974, 1973, 1946,
        1983, 1984, 3274, 1579, 1986, 1985, 1949, 2040, 3376, 2041, 1598, 
       2039, 2038, 2129, 2064, 3379, 2065, 1745, 1863, 1862, 1605, 3301, 1488,
        1614, 2120, 2119, 3388, 1992, 2002, 2003, 1631, 3304, 1501, 1713, 
       1871, 1872, 1754, 2073, 3324, 2072, 2131, 2074, 2075, 1764, 3349, 1882,
        1881, 1652, 1830, 1831, 3283, 1662, 2008, 2007, 1956, 2016, 3371, 
       2017, 1680, 2078, 2077, 2132, 2079, 3381, 2080, 1774, 2051, 2050, 2126,
        3389, 2052, 2053, 1697, 1833, 1832, 1698}], 
      Line3DBox[{1361, 1437, 1689, 3262, 1356, 1432, 3256, 1350, 1427, 3249, 
       1340, 1422, 3244, 1332, 1415, 3241, 1327, 1410, 3239, 1319, 1405, 3238,
        1310, 3197, 1400, 3234, 1303, 1395, 3229, 1298, 1390, 3223, 1289, 
       1385, 3219, 1281, 1380, 3216, 1276, 1375, 3214, 1261, 1370, 3287, 1532,
        1699}], 
      Line3DBox[{1366, 1440, 2107, 1691, 1939, 1938, 3357, 1359, 1435, 1801, 
       1768, 1892, 3325, 1891, 1354, 1430, 1930, 1670, 2010, 3307, 2009, 1346,
        1425, 2101, 1656, 3339, 1824, 1823, 1336, 1418, 3276, 1758, 1820, 
       1819, 1330, 1413, 3356, 1927, 1642, 2048, 2047, 1323, 1408, 3329, 1790,
        1621, 1996, 1995, 1315, 3359, 1403, 2095, 1608, 1814, 1813, 3336, 
       1306, 1398, 1784, 1739, 1856, 3321, 1855, 1301, 1393, 1916, 1592, 2029,
        3294, 2028, 1294, 1388, 2089, 1573, 3334, 1810, 1809, 1285, 1383, 
       3266, 1560, 1908, 1907, 1279, 1378, 3326, 1777, 1726, 1843, 1842, 1271,
        1373, 3353, 1899, 1540, 1961, 1960, 1959}], 
      Line3DBox[{1367, 1441, 2108, 1692, 1941, 1940, 3358, 1360, 1436, 1802, 
       1769, 1894, 1893, 3351, 1355, 1431, 1931, 1671, 3369, 2012, 2011, 1347,
        1426, 2102, 1657, 3340, 1826, 1825, 1337, 1419, 3277, 1759, 1822, 
       1821, 1331, 1414, 1796, 3303, 1749, 1867, 1866, 1324, 3315, 1409, 1791,
        1622, 1998, 1997, 1316, 3360, 1404, 2096, 1609, 1816, 1815, 3337, 
       1307, 1399, 1785, 1740, 1858, 1857, 3343, 1302, 1394, 1917, 1593, 3374,
        2031, 2030, 1295, 1389, 2090, 1574, 3335, 1812, 1811, 1286, 1384, 
       1783, 3267, 1561, 1910, 1909, 1280, 1379, 1778, 3289, 1727, 1845, 1844,
        1272, 3354, 1374, 1900, 1541, 1964, 1963, 1962}], 
      Line3DBox[{1368, 2110, 2109, 1693, 1528, 1942, 3382, 2083, 1804, 1803, 
       1770, 1673, 3260, 1895, 1934, 1933, 1932, 1672, 1519, 3254, 2013, 1348,
        2104, 2103, 1658, 3341, 1509, 1827, 1338, 1420, 3305, 1760, 1643, 
       1878, 1928, 1798, 3332, 1797, 1750, 1624, 1868, 1325, 1793, 3330, 1792,
        1623, 1496, 1999, 1317, 3361, 2098, 2097, 1610, 1483, 1817, 3338, 
       1308, 1787, 1786, 1741, 1594, 3232, 1859, 1920, 1919, 1918, 2033, 1472,
        3227, 2032, 1296, 2092, 2091, 1575, 3365, 1467, 1979, 1287, 2086, 
       2085, 3268, 1562, 1457, 1911, 2081, 1780, 3327, 1779, 1728, 1543, 1846,
        1903, 1902, 1901, 1542, 1448, 1966, 1965}], 
      Line3DBox[{1369, 2112, 2111, 1694, 1529, 3265, 1943, 2084, 1806, 1805, 
       1771, 1676, 3261, 1896, 1937, 1936, 1935, 1674, 1520, 3255, 2014, 1349,
        2106, 2105, 1659, 1510, 3248, 1828, 1339, 3306, 1421, 1761, 1644, 
       1879, 1929, 1800, 3333, 1799, 1751, 1627, 1869, 1326, 1795, 3331, 1794,
        1625, 1497, 2000, 1318, 2100, 3384, 2099, 1611, 1484, 3237, 1818, 
       1309, 1789, 1788, 1742, 1595, 3233, 1860, 1923, 1922, 1921, 2035, 1473,
        3228, 2034, 1297, 2094, 2093, 1576, 1468, 3222, 1980, 1288, 2088, 
       3383, 2087, 1563, 1458, 1912, 2082, 1782, 3328, 1781, 1729, 1546, 1847,
        1906, 1905, 3355, 1904, 1544, 1449, 1968, 1967}]}, {}, {}}},
   VertexNormals->CompressedData["
1:eJx0fXlYjd33PooSIUSSOWQKkZlVRAkhpFIZEkLGREKDMcpUIXOmVIYmqVSr
eZ7neTx1Gk9CMvv52s+z36v9+/jrvd7nOm/P3vda6173uvc+5x21bb+uWbcu
XboslO7SRezPP99fnOntP/Uxxtgvv3DNJhLXHDOy//WmFV67eHaZvckLL/12
neGQGIcL5l1JXHmqFeZWrpx118cX3TZ+em/zJBVnyDpstBjeCuVWGO337g2a
yDnu9/XJxk1XtY3enRCBo5PVk0tt71Bm9OFi+66FeGrgA0PpIy1g0e51e/aN
KATlXeA6uxzl0y5/m1zWBGeDL+85eiIOj7sq/zA5Uo2F+kN/f3RtgFkXb1mr
aifhqNVK4+pCajF1vu2O+w/roG+ow13Hm6n4Y8bm9KFX6/GXzdgE07k10NLg
HWqnmIGfd/ZL7BvViA6nLWOOHquA5xmhT6ZaZeL7r0J1P61mjNm5x39/lyI4
fLWww7FnFpZ3v+BVvrQFJ4691efzwRwIaX61rM40Cx9t028eUdWCu8+cy3yW
mwafXmZ/tHbPwoL531OnLBThowJl97zX8WDwpcsj9+AsHDxg62PPQyKc8U37
x+2JUaBQHJ+8ZQZizrLbPXd4vqA4r8nNtft5LxYbJRZ1TGz1pzivjb2i1TE0
BQ9MiXa1qQ2mOPs6tGzN8clClffH5K/Oi6Q4L5g03uV8cD5+8/zghGWxFOeH
R09dsXEvxTPPEq6WLEmmONfGWKzefawKJZ6VrjXolUlxrjJb4DXhqgDHzFgp
c6w6m+L8zEBijS0IMVmjpt3fII/ibG3ar2OPbgO+vfkNlqsXUJxbqvacGRTX
iBPPP1rk71ZIcW7sf9EkKLAJP9XuzI0+WERxPurg3+25ZjP2ve1U/eZLEe7i
cP7+wH1z4v1mHHLN9mGIVjF6cDh7/W7YbJbVjIuc1H/PPFpMcX72feCDeeqI
zqeWO8fufgE8zjPsrS8resaiktFzO/kEf+Bxnjgwb3QXpRT8JJSzSfMNBh7n
O3vrH/sEZOEC/7ZJc+UigcdZ/ZpzwL3ofOzpIbh2yy8WeJyz1jonTHlWiqfG
D5TV6p0MPM55l3IiJtlV4a7Y2W6p2RnA4zz23QEXbXcBxmw9Y+0Qmg08ztNk
PhpmLRdiprtC4NnJecDjHGapF+Bl3IASaW1LB/YrAB5n1TsmaV8yG/G91NYv
L7YWAo9zs026hFhUE/YpviNsml8EkzicjXafF6qub8Zd9qrmUVFFwOO8+8GP
thM+zZjWb1LhUJli4HH+vUp797TyZuyvNat0JhQDj7P5bq8Q7/2PUeOYWXCV
VSTFOWbs17VRr73wp0yH05roOIqzmWVt9rQaX2yzb/6hdzeV4nx9xO7H48WD
8KGByqatz7IpzlleNvODdMLwp5ii+atvBRTnybsFu4szo/Cc2LSXP5TLKc5G
aodPysfG4StBosGIA9UUZ93HhYPP30/CkXO6egYH1FKcByvseFT2ORUPuiVl
jnCspzjHvvNL3eCUgW9kDy23DGmkOF+JNx7d/2Um6i66vfOZWjPF+ZPUIty4
OAudH1RmXVrYAnw+B+RtWld1LwvTP2xofVjYQnHWCCr0GJ+bha9Pj3nsOlNE
cf7cIi77+mMW+i4Wavw2F1Gce7rLyCtd8IfVeWetyurCKM5xojKXlJPBYAq+
I4YOiaE4Zyd8kJSSj4QP7goWDxWSKM553b9hw89YmFNfOdXBJJPiPN5Z8/yF
n8lQtqNtUkevPIrz1e8t0U9+ZIKidMFRyVPFFGfrOwsdCyJzocvq/Jxq5UqK
s9LJRc4mFoVwJWv2kJW9ayCNwzljX5cuZfdKIO1k+YifF2opzou/B6leHlgO
n8xGmKZYCinO95IMOypSK8BeepbJuNR6irPCtp4uMZaV8GFi5tbh3g00n60m
fsiWSq2E4mPmGxeoNALPzx+btZ4l/64Ey6d2q9scGinOnyL6OBUPq4LQsW6P
cvwaKc57DO0WH/wZCf0D5AdteOxCcR62ZE/2WMt4ULSO7d3zkzvFWWpa9P1J
59KgMdk4dmA/D4pzaoP0jbRFOeAaWhj0ctEzivOOW2eKnkcUQle7Jfbt7i8o
zjU/Rgx8MbYClhuKD5/VxZ/irPO2dkxmjxpI+Zq/6vPPNxTnm1evew4yqwPv
MXoSHtUhNJ+fubTvfra2AWbfV6otWRJOce4Vl7+53KUJnH7qblOpRIqz790i
H+fxLdAhqTx7R1EkRHM4D9k2quPsZBEcPZX5YM6BKJrPeZvXXLqcKoJmqWXn
36RG0Xx+V6YedXBSK+w8EDrUSSya4hz71vnj1W2t8KPE1mTnmGiKc4z9SzmL
TyFwY0n2rD3WQbQPhpnOXO46LgriZeqN7geE0z6YNCDcd7hzPDybntb307kY
2gdhdLft+3akwdock69jjyTRPvhrUZH+9bpsmCRzSTzUOJP2walaO9xOJhfA
hDGSFTHuubQP6vy4UFfbrwxuTO4VJzupCIs4nE89y7g7YGgVXB0QGuv/oIz2
wbqOH5GFen9w7flU4UNEJe2Dbrs7lOz71YLLdZWoLQur8QyHc9nhpk1FGnVw
v1L83WatGtoHJ56bqDq1txB0hOKXv32qQT6fF47Nk9x7TgjSXSL75pgIaB/8
Zdx3aV26EGbtT57hdk9A+6C4wpqoLl+FoB88dbVUtID2wchg43fqR56Gcf9O
cXZ0Pv47edOLMO6/pzhH9V+/uP6Jf5g5eR/F2Up50fXDOsFh3PoozgHqPR5c
UsMwbj8UZ5VspWJtn5iw02T/FOefxaLgMInEMA4vqjdQakSXeK3UMA5finOf
qdW/pn3LCOM+R3GepuausGB3VtgQ8nep3sjY+1hfyTw7zJasg+I87/arIe3S
OWFGZN0UZ1jlpyd+OCdMheyT4mx2xuBn66ucMA4XivOMlKJ+mJUTtpbgSHHO
IvmM55h8jiP5jL5MPueSfMZbTD7rkHxGYPJZEv7mM45k8nkByWeUZ/LZmOQz
nubymcf5PMlnPM7kcxvJZ/wt2TmfH5J8xrNcPvM4N5B8xqtMPs8i+YzA5LM2
yWf89Rs75XMvk7/5jOOZfO5H8hm1mXzeTfgZdzL8PJTwM7ox/CxB+BlnpnTm
52TCz/iR4efthJ/xJMPPVYSf8SbDzysJP+PEb5352ZXwM8ordubnJ4SfMYjh
Z0nCzzjoV2d+fkX4GdV7En7mcR5E+BljOH7mcc4i/Ixze3Xm52DCz/iG4eco
ws+oXdqZn7/c+qs3cBOjN14RvYF7GL0RTfQG/mD0RjTRGwiM3uhD9AbWMXrD
nugNnMrojR1Eb2APRm/IEL2BNzi9weMcRPQG5jF6YxzRG/iL0Rt2RG/gRU5v
8H2wN9Eb+IPRG2ZEb6CA0xs8zjVEb+ApRm8IiN7AWEZvOBL9DIsZ/dxC9DN8
Y/SzPdHP0Mro5yCin+Eeo5+7eP/Vz/CN0c/aRD/DaUY/2xH9DC8Y/XyC6GcY
xuhnTaKfwYLRz1+JfgZ/Rj97E/0MOpx+5nHu1+uvfoaLjH7OIPoZkhn9vI3o
Z/Bh9PNA0V/9DC8Y/fySzIOwl5kH55B5EAYw8+A0Mg9CITMPPiDzIExk5sFl
ZB6Etoed58FCMg/CFmYeLCLzICxh5sGJZB6EV8w8OIvMgxDCzYO/OZyjyTwI
zamd58H5ZB6EXGYe/EDmQWgrIvMgj/M2Mg/CCmYePEjmQfDn5kHe3+iu83ce
hHbNzvPgQOJvwAfG31hB/A34zfgb2sTfgIuMv+FD/A3QYPyN2cTfgP7PO/sb
7sTfgIeMv1FF/A2QY/yNUuJvwBzG3/Ag/gZUMv7GYeJvQBbjbwiJvwHLGH9D
QPwN6FdH/A2eNw4SfwMmc/4GP6e0E38DZnP+Bo/zI+JvgBHjb3B+HfzDr4N/
+HXwD78O/uHXwT/8OviHXwf/8OvgH34d/MOvg3/4dfAPvw7+4dfBP/w6+Idf
B6xf57tCb6/61myUXjenIk0nD7dmOiv4hDfDhtg94ouy8vH7vo7fkYeLcein
NavF/tRbTrexI/22luDxA+YOehEVWDXhlnL7lHpobDxybfSNChzX03PEqNXV
mGPs0ut9uwAsJrit1TtdhWmzUsIvrBLg4Iqy0yeOVsGiKbuexsdV465Rfjv1
YmpxjO0S87gxpaBgM9Ui8F4Nrm3K/+gVXoc+3Y9MD5TLh6EOXcq0xgrwEY6z
LDAU4qOeGWqe0lmwZ/WQFxVWAnxg9tjRIkSIvW/bGSr3TIYLGS6Zfs/+fL5p
b0nZeyEefHCk+HBEDOh9nKtcfyQbe08527d5QR7w+13bd/z4qsZ8nPLi+Bdn
42Lg92tcrz3siF0J3r/39qX/4wrg92sx9ajkw5AKHOXofDUGqoHfr7vmhndi
N6twRvWB/FMLBMDvN9gkXr+8uBrXLYpN6e9XC/x+jfpWhEcE1OBobaMlrt51
wO9Xb9eN003zBfglfn3JQk0h8Pvta5j2q8xZgA1K4VfOPBVCL26/FydtKa8N
F+CAig7D+FIh8Pu1uzRSY9aH5+A74f0Auco82HDW7rW6pAhGdjlolBL7Gg7+
Kk5/WVJCcRg8yV56bHAAfKnrUhymXUVxOLpnlssHr7fg0BRavCBSQHGolpl2
cMHKd/Dgxglht/dCioPD8LVL5iyNAKUrGmN1LzVSHF7a7/msMTcShryNmN/T
spnikL1u8Jaqj5Ew84xT6Zp3LRQHc/8CdWnzKMjtqlyRcUFEcRhYrzHu6uso
MGh/5W0t3gp83FdGi4YKCqNg3tnkqk8rWykOquLW/ncfVEG3Uc83l/vl0/Vr
DirpdWC9AKwHSd3pq1xM17/DY3nI5Et10H4q17dKooyuf9Hs5zXzJ9ZDxpma
kC3DK+j6lb+UqUxTa4DYipxDSosr6frb7o15GCRqAKftKl/O+1fS9ctvWzcx
fGMj/H7z+fn4z5V0/WoGC0pXuDUCrhtllDysiq6/i5qE/QXjaDD2NVIYNiQX
+TiapIUuMzFNgMldDiQtuVVE6/fhqJRutZJpoHuuxmpyfAWt337Xl28p25MF
2U883j/pV0Prd1jW7R0RcrngWXSzccyvWszlcOj3rPX7N618WPYt+uN803pa
vzr75uzVrCmAFQrRmxcpNuJYDoeAoDFuvT4XgrDM66doTROtX8lm08vXHhXB
nKFj3r+Tbqb16165JchUqhg6dMX9DGybkc/nK27Hni/ULIZjsunb2qKaaf3K
ERxQn8HhJMEBxzI4JBIcUJvBYTzBAdMYHOYQHNCDw4HPB2WCAwKDw16CAy7h
cODzIZ/ggJUMDpMIDjiNwSGM4IBtHA58PvgRHHA/g4MayVu8zuStMclblLHt
nLf9CQ+gP8MD138f+D8eQEuGB3DiXx7ArwwP9CE8gGcZHjAnPIAeHA/w+fNz
2F8ewEkMD2wnPIAKDA+cJzyAsxkeUCA8gAUMD8QJ//IAGjM88CPqLw/gAoYH
thD+BzGG/zcT/ofRDP/vJfwP1xj+tyf8D0M4/uf3+5jwP0xk+D+G8D8s5/if
r5edhP9BjuF/U8L/IGL4X4HwP1Rw/M/v143wP/Rk+P8p6e8wmOnvmqS/g8T+
zv09lvR3uMD09wLS32Eq099NSX+HPKa/zyT9HQ4x/V2G9HcwZPp7P9Lf4QXX
3z24/W4m/R28uP7O88MJ0t/hJdvfp/ftkZwcgkFHL12+VxeO0W8bZdvvtcIA
x7d7MvYGoWLPG/66v0OonpwS9cL0fFkqGh/NtNj4LpPyybSTW09eWp+JM0ZX
xduLsqk+vDDDabLq5VzcbxJ6/3huAdWH1lueyEWPKUUpvzewYu0KivO9vj/6
dCgUodeIJ4Ihw8qobvxiNuzI8aNV+P2AnEzBpzUU//StP8YOvFKGWt+ej1w8
+o9u4vRkqbOlS0u7AGONPxgqGayncYmRCJZyflOJc3qp63aY1FCdOarnzg0f
ptRj0br1feStN9J4WS+IGjV2RjXWndMabzCklvq30/oFj+uS2YjJ1osPWkkY
0jji+mlrLy+swTzPinQVnTqqS2ud0kI8w5sxI8O0wK9uE41vQvpuncSGGpTY
eEG/VFZI5wKr6+ljF0mKMOnlyzcPxhvTuM+I3b3PbJ0Ad5vMPLXyqpD6Zicd
VYszC0SIYjuaRrUZU54859XFWPy6ABOT1ZVqC4VU34YXjrgbur4VgwTSe8x2
m1D+vDPRu26iTxaWlg9c7bJJRPNkhLGuuFmQAI92jV4rI1ZP9XBalYVR271W
fOK6zO2Ynwke4D5ffFj2pp9cPk50cfW0OLyU5smzhg1Kn3+m4u2TquEr/8z9
/PPZKspVLfeKsH6xv7+DdgLlGcnNomcz0yqwzK7HwJ6NqZRnJAK3y5YNrEEn
jVRNA4MsyjNGqU9EXcXqcFGS9fZtYrmUZ6wfjPL02FWPAZkXHVdMz6c8UzRl
2CGJiY1Y9PRCbU1SAeXVmxKFd77pNWGa+KUeZ0oKKc+I3f2s1zagGbXu+T/S
PV9Eeebr1NIFz882o3ztmUNBoiKqM0egVNy3N614xqRmVnqiCcXt22U7H53E
ZnS+U/B29aRiyj8nAgY6K88tQk+9tX19xMv+8w+XB+3S9i1Dlz7zZaoGVFEf
4LbHpQXrEitxvLmmxxudGuq3qC0uXL5zSwoelrpvmzHtv/7iOvH0wFua1ail
M27mE4la6g+M0Zm51m1TOqoemXtv5dsGik9Oaab75HU1aGaX5XV8fh31DdZn
NA1R6iLAIbK+j090E1IfbNK3g4L5uwWoc9lJ8twJIfUNbvuOnzrKS4BVQyvj
qmKF1DewfdQzyDtNgEGN7RcbRcL/zjftukV6qT0G9bNKoZ+rMmGh8aYTuQUi
8Hra9VXrx6fQ94BRvqJaDvVbBt0Yk1cy9wXUtoRN3RZZSP2WqIg7t0Jj/KAq
uc+dgvEV1G/ZmwhvZQLfwJkJN0+OkayhOFc+cPX0vBQChlL7Bz7bUUfPK3NH
OhqdrwqDTwfzf5mva6A4312atEjuCII4XF+Q7NYEPD8ctJi3OOlhJNz5eu65
24QWipt132P1D1WjQHO54eiHyiLqt6x+Wy0tvBoFTz5klwaki+h55asJzVu6
J0SBdOmY1/emtFLccpsDvWRds2F8xuFg3+YEWkdHpBLlts/MAyP3cH+bkmBa
dx+C69LynxeAju/m23g0ndZX19yKRWM2l0HEvv799zRk0/pqvpM8VbxrCYyQ
lTlS5ImUn89udzpTMKESAvfeFF3MzKW4lW25HdwtphK8VevuTXkWQ/k5aaXt
NKMef+Z4LyPbIttCmrcLswesMhomAM21osGmrgmUn88eXhuzzKEW3Ix9n0f5
l1Ccc584zRkxUwjHulw2Mp6RQvX22NJX72wshDD5Y6rYmknlNJ8V42Y++xLa
AAt/VeU/npBO+Xn0ADW/PbH1sMLRQCASVPwXl/zmsADXJlC0UC09MTGT8vPM
g9PODn/UAMvvPQmaZV9J8/zxjz22H7KaQdY5GvYLMvEFxw9m+x2l+09shE8T
p//Cwkqa/3GDgxwV3Vsgr2vB5mkGWZSfg8Qiyl4eb4RH6oO+POtdReMr/3X3
KzEZEXxtL+h++EEW5WdT7QQXe69GUJFUvJs2sYrWi8m4om9LN4ogv/jTOo/E
LMrbij+XSq8PzITv5XuUo1tSaT70n1l/rGLpH107IvnLjT3ZNL7Gmk9vee8q
hrwB5pr9y/NpHEX3U9dJ+lVA+X5tw0EHS/6L12VDW0vNamiXeSBR61lB+6bs
HbMpPZYKwKRn7OCpzlUU/9bQL0qzQ2shzPho0PrMaoqzr41W4qjAOlh11qzu
3fMa2u9sP9XvkF4rhE97J127O01AcZPQ/J5v81oIsnPaYauDgOIzxre058Z9
IrBeuPvIuPwsyrdiTt2ac+uEMGyGh6aOv4DiY7AnQjhQKRkWh0ZLFT4opfqk
debULTGfMmDG+kODVp2oojrEZui1E7bp2bBFqtulMFcB1Rsrts7KH74kDxJb
VAb4aAiprohcsuWkn1IBRBk5ZEnrN1D/KmVDyUl/m0LYPL8+UyqlkeoE5VuT
DYv1iuDu7/0dFaFNVA98eV3pt6isCBS+B2h/WNVM/au+4sFZLZOLIUbkKTnu
aTPt7xNf2cz7oh+G8WoPnJ51z0CeP3s/vH1xx4RIDN++8lbU6yzq720+Ndjh
VUIsjl42bq1beD7VbycIPjiXwUde9S8+OJXBx5Xgg4YcPny9q5N6R1mm3ncR
3DCKwa2I1Dsu5Oqdz6sKgieGMngqkXrHnz8713szwRkNGJxPkXrHbky9ryX4
oxuD/3NS79jmROqdz095379xwQFcXPh6Tyb1jn5Mvc8g8cJQJl4jSb1jLlvv
pK7Rj6lra8LzOIHh+VTSF3Ah0xfaCJ9j2cDOfF5P+gKaM32hkvA52jN83p7z
t19gJtMvSsz3tmcF1WJ4rLHWVsskGndl4iPhYMZHGnOuOeChUz3Odl0vNjsh
lcbdqsJ3oL5IiPLW+wdN8XpH5/TQQ+UFxxwbcY9XeLrmiwiqT1SJv4QtjL+k
SPwlzOH8JZ6fhcRfwruMv9SP+EvYJ6izv6RE+ATXM3yiSnwnzGB8p0DVKYqx
W9JxQc2a73N8Cyiev6Y+HD6+JBsz7uyJXbWrjOI2IGN4PT7Mx9tPLz1TeFpF
8RntOW3sGvlitHiqNSvRUEBx0Flt9PZ7WynG5kdHql6vozg82ZF5XdijAsvV
zh2Kn15PcdjefqVCoFqJPRqCbGW0GqiOnSKa4tH6tBJvXW8xTf/cQHGY/fjk
aqOGSgxUeh3YvrmR4qBP+g4uZvqOzrIShap+VThhoNXdm/caKQ633P7qMaxn
9NghosdQwOgxaaLH8Byjx/YRPYabOD3G59V6osewg9FjfYgew+6cHuP5YTzR
Y3if0WNTiB7DFZwe4+u9C9Fj+IzTY3xdbyZ6DPsxeiyIzDXgwsw13j/dfAU7
MiE8JMfqfnk2xUEu+2fQqqe5ILt8RNvxyAJg5mJQTus8F18kcwHcZuYCbi6G
ww6d5+JnZF4AJ25eYObiP/XUeS5+QuYIGM7NEb87z8UgsbfzXPyQzBGgxs0R
zFwMt107z8XlZI6ATcwcwc3F4NzceS7eTOYL6MPNFxM7z8WQldJ5Lp5N5g7Q
YOYObi6Gm6M7z8XPyDwCBcw8ws3F0Eu8T6e52InMKfCCmVO4+RcWB/7P+RcW
e3Wef4cR/wTMGP/Eoe9HL5frRdCxQEF/5okEGvdbxmbJCvhHp37bIB86Po3G
V9KgY9Wu3jXgUN3NxPhUFo1jznvpYLWvtXDA9s4LWeVcqpN3vbW0zzSuh0fO
qspPt+TTuPSyfam4f3gjNEwfrvv0RwHFv0qusbVUuwma61UXO/crovr24tnF
LbYSzVAu1rA/ObyI4hn3yHCfuHUzTAm8XSA1rpjixs2/MO9U5/n30xL3dc7v
muHExFG7xpoW076mSfwoiGH8qB7Ej4JpjB91hfhFEMT4RWHEF4LVnC/E80YI
8X9AjfF/9hGfB0Scz8PzRgDxc6CC8XPeEd8GZDjfhs9PJeLPgCXjz1gTHway
OR+Gv5/Qn/gq4MD4KpwPA6wP83LCXc+pMs/wWtdF2qPboyk+F0j+YBmTPw9J
/uBTJn96k/zB/Uz+FJD8QQMuf/h630fyBy8w+SND8gczmfwRkvzBTC5/+Dq9
RvIHo5n8SSb5g/24/OH9k28kT9CEyZOJskWJV4bl4pzBfS6LaUdTvj21WavH
9fI88Ip9MXusphd9/mdido4/Ugw2uVIBp3pmUb59NOjTgPCiEsgwf7/3uJwv
7deqw0aeu7C8Cq47Llmn9T2A9utRv6OKkiMEoDKsQnD621var62JboHHxzrr
ljCiW0D4vLNuKe6hKpA+1Ax2Gq7H7Q9E0n69efbWkEUhLbDR7vZC9ylR4M2f
g2ws3/P6nAhGOh+VXv8kivbrVWtNbq/p1grp/ovEEpqiaL++qSobF6/dCt7v
p73ylI0Gnpd+kzkOU5g57jKZ47CcmeNGkjkOdZk57huZ4/A1M8eFkzkOgZnj
nMgch3XMHDeQzHHYg5nj+pF5DaW5eY1f/0kSXxwU1zm+D0gcUXZ35zjOIHHE
eiaOI0kc0Z2JYwGJCyYycTEicUEPJi5KJC5owcRFm8QFFQI6x8WNxAW/cHHh
83kh8T8hnfE/+xH/E94x/mdv4n/CMcb/3Er8TxjP+J+2xP+E24z/WUH8T4hg
/M/7xP8EP8b/lCL+J0xi/M+u0/76n/BLQPxPfr9drvz1OWE/43MaengFPB2Y
C4bzVo7seTCa9kGO34DltwdJ+yL3dinC6S8Wqe1bqkXn0x9LtvWxPFaBdzP7
vBjgrEPn07uDzE5vnVuDShOHK7t76dK+cD9bIezOwzrUEuaqb+7YQPvCu8IH
39+7NuAOh5rXd/L0aV/Qfq7mP6GsCaWcvYuie2+ifcFnv0Z1zyMtWLe5dm/w
LSPaF452r4x9e0KESgNClf1vGdO+kDGv10vz4a14v2dwqLeqCe0LskPS+i8/
1Yom0qky4ddNaF94SHQjxjO6UZroRhzJ6MbpL/Yl7OyeByNHSwk+yqTQz7++
ZdAu/6sAamMnqimODPvvnrBkv+4rZ5TD4+zbcpcCoihuM3M3bIw6VA1GG/zs
VjyMo7gVkvkOPnHzHa8nR5A5Do5wcxyvG18KFacuDW8Ec9tvn/bpZ9Bzk4dp
ahNea/zp+wPmFjy6lklx+/G5m2+wegt8yvCt/DA2i87dBassPtSUtsDIV1Wl
tXZZ1PfYP/hEgmCOCB4v7l2VHZRF5+gAsl/0YfarTvaLp5j9ziH7xanMfv3I
+nEst34+7k/I+tGAWz9/X7Frx9/1Ywa3fj7upWT9+O1l5/VbkvXjSWb9E0gc
cS4TxzDSv/AS079siR+LCYwfG018V+zuTXxXPl4nib+Kvoy/qkB8VNRkfFQ5
4pfiLsYvnUh8UdRnfFFD4n+i2KTO/ucL4nOiH+NzcnUNKnH/s65h4Pv/Wdew
deH/rGuo//k/6xq+3fmfdf2nL//PugbDk/+zruHJ9P9Z1xCm9D/rGsomd67r
ceScFBYw56Q25JwUrJlz0hUC7zNH/+hamfIfDsOHhYCDtbhbjm4zmMx7rZEm
qkKJMbUSUa0R0PfZlTHKcxph14tMq/w1tbjTatRCwbZoWJh+fNHNPUJ46He+
5ODGemwr+lwh9TsWumyZ3Th8iAAunyz8JopoxJSAG4faZBJgVu3c+g23KmH8
E7vZb62aMatFukPLORFmxvrmTrpUDJ/H944outmClSPzP7w6lwQet4vTeobn
gtAy2WfsChFGa3mH641KBmm13K2D6jPAXn5ktkqQCEdVjHF8eTIZHllqlJTq
JEKB206dRyuK8USlQsb9X0L63ld9B58/IJEPJ6f6V8Qdi4SQR4ldC6JboIvd
fPd8+UooefB8bfm1LOT3W791s8YtxRrYeWni4c3Vucjvt9hEqeui0lr4NVzv
5sqiAuT3+/iAWfBP8Xp41+9Z3/Uzi5F/b75K6su2Kw2wftfQWao1JTiD26+a
7/URd1waITuoyODkr1Lk9zt1Z/DChzOb4FeXHgpKr8qQ3+/OJU6FMa5NcPK7
xQmpweXI71fTc3ph75Iy1GlweKoqqqHrsZru6fR9RyX6Vhy0n1BXS9ej8z31
xGvLKhzv0TsYdIR0PU/mvNzfZVw1Pti5UPRjXj1dj8e4nwvbXapRR3/UgJS0
erqeR1Jbiu/kVePx8Y++HxzbQNfzqOhKzK0jBRgRljlF5f5t5HEeHRc42v9N
GR7ftudjWfFD5PNNc+bbLVdXVqPXmG7rVsY8ofjPdvysG/9H5z+W2NfPpuMV
Xf96v/Qvjr1a8PHxuC0bjvvS9ecqlS5yVfmjlxyC4e0oP7p+vawdHo4NIozy
P9LntKMfXb9WySS7eL1W9O24NeREgh9d/7Lnd23WDi4Dtandd466HELXOWrf
kXixpipwN3oQ0XEc6ToTHp0dqLeyFnqsaFL0eBNN82Tj/V6CVbr1MLcs/OA5
4zj8vZnEJbnZVnJ5SCNYPSnrE22dQPelF7zPLPBAM0R1+Xw79nsi3ddV7wel
7tdaYJz83Eqnr0l0X4HSY1tQQwTXjs2513wjme7rW4LYsaDXIvj4PExU/SmZ
7mtni2n7nLnlMG/Z2LrpuU341GhcsI15NOz3W+dTPbwcP25MSm9790ePiB+V
TJsYx38e2c+njjEYcaCiAK2kVLaYZGRR3sjIMiqY7JCPVu0T+v10FEDlvR12
S9NrYdnKtuYylSIMFiZe3fOrDub3Luy/Xr8aEk50hM+Z7YP5o2eFRW0S0s8H
Vz/5gK0vMELMZFVRXj3lkyl7FG69P/oKF4+YrWQ5rpH+nadZe87sTXmNVaY9
72Uvb6L1PpXUBegzdWFO6gJCmbpQI3UBM7i64HnJjdQFeDF1cZ3UBRgydeFC
6gLOMXWhtDp/o8Q6IW5XT50VdCCL7uuD7A0ru0uVmOR3UfuiSyXF4ZhpytuU
hw3YPWKPh/+EHLqv43MVvoac/VMvk4wiQy5XUxzqytatiDNNRzErC+uTBwSw
iMvDd/ruVbVrM7HG7Bs0fKylf1/n8pCbfQ2zsOLiiglqG4V0PTZjD9QsWJqN
k/TPnHB3qad/X3q3r0ZGvxzc8kBDe//TBrqepMyKT65hOeha/vj+KadG+PK8
f3ZKYhk8yYm21dHORRv7c/mjVzQBn8/jqrTH9wjPxTP14tmHapvgc/fX+PJM
AejM2rFmt1we3smePH/blub/+H//oOPLt+ThhyX3GvtGNUOof7uf4+xskP2W
Vzv3Vh72nSPbf3XvFtoXysbIjjWPyUP3uCkyO5a1wDSf3j97VabAhkf9nR0F
eTi+eZND+6EW2i+Klr2oyv6Zh2FbgkZ+dW2BXlz+iwX0CJVPL4CCHIMih7os
ygNcvQBbL9qEN7AHwxu1wdqDD6tWotIZufszj4fh9L3jh+oaNcFowie4muGT
RS+Gy1fOqURB8f3LS3+8A/7zBS5xJkYeNbit0KS386IoHLGycejrunpIIvyD
pdqd+edt1lMHh6c1aOmSkKv9IRL4z+cde/xRsrIOHybW+yyWi0U+HwwIX6Ek
x1d8PrzwaHheLqjDBcpH7MwwBiq4z3ueP1fadqcBn3c5dGCIbTzyeZJK+A3V
GX6Lamyw3eTRgP0qt6+6NzSe5tX0R42Ppb2b8JtztNiSVYnI548B4UM8z/Dh
3N3nreteN+HpyHWyKVUJNN+y7+pOsJNpQanuzUO6OyYhn1cuhD+xcUhn/lwq
cu94MrgFbStPZ8jJJdE8/Lh6556Nn1uwINY0t5duMvL59pbwLa5n+Dai6vbN
Fz9asEeGtvOzsiSanz8JD+MbhodTtwT2M/DPwnzl6hsNO0WUVxWuqvUNLMrG
yphiC92RqVSfcP0UPMP/Zz+FPqad++n5IQFhW1f9qaeVNjvF9yah47cDS2YN
b+H7LGxW7Nxnj7pdviY/pgzSG58vupeUQfOzYKiZ1/LIWlCTulvR9dVzmlcB
9yRv5TypgowRF6T8IYfmYZxpnfzb/HrQm5eeX6L8gva7AF+FIQevCODL+Jic
ufvykc+fWaS/Q6Bk5/7uNXGS5TlxIdgZzTh3QaGI5pUu6fsw06Zz3z93X2GI
WVk99H0fmrx7ZAnNnxyiB0D8dGc9EKS/tfXoyEYY6RH3rdKulObJBqITwDCA
0Qk/jraNEzXCqW6PApZuLaP5wOkHWPGls37YH27zVWJPE2DThFiHhjLk+Ufm
iKPig8Am2LF/5oOUP32V55lQgieGMHj6ED2J9xg9eYXsF9+3dt5vJdGZOIPR
mUj2i78ekv22c/vVJvoT/Tj9+ZDDR5fsF42Z/c4huhRLGV16jOwX7zH7PUD0
KhoxelWe4IBLOBx4/vQm+hxvMfr8+iFY+722BHNhmaVGaSzVG+UF5v6Gkdk4
1/h1u1VSNvB5/igtV+yjchUOvh6acdA/ic4vTtcGfTQ8VIBeG6RU8WQB5VX1
ysndUvYKcKF+sOjApXTaN9MzHJrNzErx+MDW+OmHSyl/9vFO/KGyrAmHrxHK
pWzPpfPL1fZlVYu6CfBQWtqFjl4Cykt79ObNV9vUjK0lTZ8/zsijfe3yTLP5
c3vUov1eO5UD/Wop/9QUf/8iLdGCEZembd8dmAd9OZxT/fyzwnxqUWLF6Bka
QbWUZ2pMPhZ4WLRgdcVAr2Fd8+Exh/MTm9VG8/rV4bzUy1c/yNfRPrjb8cay
Qyvq8JvuGjVNvTra73TIPAgFZZ3nQXXSj+AR04+2kDkRcHTnOVFA+h3sYfpd
COlHsJLpR3vIXAkLmLkyj/Q7cGT63WvSj6Av048ekTkUIpk5NIf0O0jn+h3/
+VjSj6C0onM/uk7mVnDl5lZVru4ek34HJUy/m0/6ESzn+lEHF/dJZM6FW8yc
O4X0O5h+mfQ7/vNapB/BEq4f8XX6hczF8ISbi/k6TSP9DjS4fsd/Por0I0hK
79yPmsgcDWeZObqN9DsYENe5350j8zVUl3eer9cfaVAVT82GG5XdXco1Umk/
Wn1I8nTdknLwmJtY3qO8iZ43KRtUrHK/7odO9/o/n6QciXW6NVvSvVthjbWq
8ae6Gry/d7vVNlEZ1Rtm4bv2jvSoww/FU/arJ+bSPqJxtCh+WnktjjSxT719
sJLyoUK0+RHdP3xope/nF5GXT59nbnvbELNUiMrn15j9tK2ifDjEZYrzJKtG
9LwrMJmzqpA+Nz366sNElXrs7eo4OWp6Ne0XJyt2PepwaMLZ4svdBmgU0eea
Z+U8Y2LrsbD/+6k/H1RTPrx0xunrswnN+Dw6NHt8WRF9Pnz4KU9thQb067Fw
TXpVNeXDQYLplpOdm/H0158+znOKka9fDk9k8bwxu8PdYHUDJlhPHz5Dsgaf
cPohtiqyZKhIhB1T5ub6T0qhz9sPvJl/qL0Ge99Wn+6bUEbrQvPKjAHDRLVo
PNhuv75RJdXzPV0DJYP0hHhu78GjxuZVVLcvn6Q/M2lJPa7so7J3vnw15bEh
ViM8U/Pr8dfe003156tpXj2JHWjqotyAGo5nPm1Pqqb5k7/BdW3PryIc2qq5
dKVqyn/ntk6tZk8/ifCk3ySpSwHJwOui7feGnk7Y2oDxdZtG3GqtBn5fTZZt
wkkvU+GH1WCDXKtoeo93b9eHkrsMMuHjXG2DFOk4eo93KeF56MnxPM8z9mGS
SkLXWrAavEzmd1swfT6FzGugxcxrHuLbxSY/rYdnhs6htifCgM/PU2Reg7rw
zvPaiRnL7fUGNsH2rTgieSVSPpElfQR+ru7cRyK1FvS+k9UMcqv3Lwv1jKT4
HyR9BFKZPpLj/iBP/0/9nr57dXHDmij6vJ70EXDn+ggfF8Hs9dpVPiKoLEzo
lhMSRZ83kD4C4Vwf4eN1d9KWZRpyrSDeNzPB6ksUfb5ncu36Y14tUHR1RXjJ
lHwal7L5Sjdsb7TCnPldih41+dE8LGmvc3zu0Qorvwd6drT50bh3THj2/ZBx
Jn6/2DJ5xNQAep/wY5dhcr28sjHKIHZZa8Ubep+wkOhS3NWzsy61JXHEGiaO
8USX4ua5RJfycbxP4oijNnWO43ESL2zi4sXHMYLEBTczccki+GM6g38VwRmn
FHXG2Z3giboMnufHvs0tTG3CN90qDsxdX05xqyR44p15BE++LswI/vibw59/
3mtrg9zHgBaUMPMO9lPNp/dJfhIegC/unXkgPmDChMk+deBlekCkeTeXPl9N
+AF0GX5Iq0xVNaqvhzQ126UWT/Pp8/6EN+A4wxsJcDCt0b4RhHtlLVOHFNLn
uoRPABg+abu/aZbylSbw2aS17l6fIvp8DOEZ+MDwzFrPS07jZzVDhryLp8Wz
Ivr8FeEfmM3wT5PLNtUi92ZoHvF6aP+uxfS5BeEZCGF45gvhJVjA8RL/XIv0
L/Bj+td80qdgItOnEkk/gnlcP+Lr3ZD0HRjM9J1FpL+AgOkvsqSPQDjTR1Zc
7T7J7bcIDnXNkfgWlkzjHkn6Aqgqd+4LTqSPQA7XR/j84fo1sP2am1uBnVs1
9LYYqS/NxNM19+VirzbRc97rbZWPd3bJQue2Yte8tGZ6zqsdvnZQ45EsbDUI
nt7k2kLPefXM3fp7hGXhhaM95of0FtFz3rY9D68OqMvCn0/v2LXpiug57zxy
jxF2cPcY+fNoc3KPEd4x9xgjyT1GyGXuMR4m9xjhe33ne4wLyT1GuMLcY1xG
7jGCF3ePkb9P8jlmzO0jdVGwt/BqcOT21v/uz5N7jDCcuce4XPVYn70zksO4
+xD0voTBzEXHD81ND6sk/07vS1y7dV2mW3RmGHdOT+9LLHxU2Hbxc1YYd65P
70tsWOcrPSEuO4w7/6b3JZbJh6QJluWEcTxL70t8aR/2/rNrThiG/VgZsb6V
3pdY6H5dcs6mYpTwFGmnFzRT3tav1ioRfq/GyqH1F130G2hebV7VPGrfuwTo
qSywOCn/3/dSLcj3hmA/970hfl965PtBoMJ8P8iE5BXYMnn1iuQVODJ5tZPk
FTQxeWVJ8gpOc3nFx8uTxAv3M/Hqu/dvvsFXJt96uXr1WNUjBYxS2q3NZwtp
XKw0dNR9OtLgcneNB/cjGmhczqz75RTYNxMs3vlV3b3VROMidT+x8lBiJqiY
vnkgmd9M46IZ9Ov6oQVZMD7yZL3evRYal4KZU7PVHLJg06ZNvTVlRfT+Erce
ZNdzfqjGeXXDfAzK6lhe6Sqg/t4nvd29ZWIL0FXRVal7dS3V1dz6kV3/iHNl
M6f0KMIPMYua7kgJ6bwzu3jLk98KxZj95MMXvV71lN+4/SK7350p9coakiWo
pLnNqaOmnvoVQjN15a3JJfjB0kw34EYD1eEcPsjic8dxsIvdzlIcVN83YfP4
RjrvOJhsHfZSUIonHteZedxu/E+fEzyRxbNZtePMMM0yPKVrN25peyOdd5bJ
jzz+zqUM27O1BmTNb6J8y+GPLP4JCdbaVellmHFpzTTNQ03U97CU8LZW+1GG
Z5TVB/e520T5uW7Cz5QlpYgZPaQfTPhZTO/heKzwcwmKj8HWG4f070dW0ns4
x96Zd7s0pwQzFo5I/r++x+NWGrjUZ97tUtz0vrTVcUgj9Q0uX1T7vMOmDCdK
KG7vJmykc9+HG4HTh4mX4z711pXRpk3UB3Ax7G758o/O0Fkx/+PSl03UB5hN
6hd7M/X7Stt3aouhJyrqem+/0EtA53cZw/ehTXpeaKCktO881lL/RIncPwcX
5v55CTmPgFLuPIKfxz3IuQMoMucOnw9pbXpv6YvxWof2vYRmisOjx0W3FJp8
UfPXxTvnpFqofm4j5xHgzJxHvP34WRij7Ydicu69g1+1UNwq+zfIfLnhh277
vtw4N11E9cYack4BJ5hzirk/Nj2pzfbDBft3dzO7JaI4Bxb31rrY1R9/HIm4
eaReRH2b/gf+nl9AI3N+oZ+nt8lprD+u/aWTpDGulcYlV2GIzSR1f8wdIu35
akMr1SeSin/PNeAqc66xKVJRRl3PHwc8/JFdYd1K47hukMzz5Tv88fbcbmfM
3FqpbhHX/HveAYHMeccS8j0dmMB8T8dqavLWTTeiwkaTeqb3Pw+IjfpkIRcF
yyw+RR58XEP9GXvhp335TTEgeLbbzqq6jvotu1RsfxuviodXlwr9ve43UD45
O0WuxnFwImz+bbD254smyg/zpo99f21NEkhGHP7pNrCF+rSL70UsKR2cDMdf
Wjme/dpC67cobq17ydVk2GYX5n7mjIjWoxzpX9CP6V/LSf+CZq5/8Xqm2N9/
Skh0TlgMuR9G74V6kP3iEma/1+q2Vmo4ReNJ//mJt3Rq6TwSSnDAKgaH6rd3
x69QjsM5kVfDq9fXU93oRfBBbwafURUup0qXJqBgXsv4EWGNlG+TCG5oyOHG
+0hJM94HOMYm4pKP9kohh5sprzoQPFGMw5Pnz99ThnwYGZmE4hP2Lg93baE+
sDPBGY8yOO/5aJqoaJSMuUJv48+aIsqTY+P/4o+bGfxv9yjduiIkGUuUKhd/
8RdRPtTbJJg6oTAZldV1bS3yRdQHXr3iL8/AGIZnTp/8e14Mecx58W9yLgzq
zLnwrsN/eQPiGN5Q/vSXB4DlgUBS1zCfqes8UqewmqtTZW5fX7sMOSstk485
9UNz8//8Hb6+ikk9Qn+mHmvI+Tgc5M7H+XwYlZl12kFUAMXTFwQczK+l85cx
OTcHf+bcvJXcb4EjzP2Wy6RfQCLXL/h8EK1s8en2owRW9N0tNtGxgc5fn0gf
gfVcH+HzQa/oUtQyuTIQM9O6cfkPf/J8+Jz0FxjD9JeZJtHo/Oc9jyPUf++a
0kTnL+mbf/sO7GT6zhBd3V1p08sh4pS2/GHnJspvL0k/gmVcP+LzYfOsfgkq
+8uh/62zs6zjmijOTr75z3ceRpBLSDYtFC+hvLQr/kuPh2dj4LNb/FjTuEp6
L53TRcDqIk7nAKtzON0CrG7hdAiwOoTTFcDqCk4nAKsTun3fO27xNl/0WJ3w
qXpSBP3eQavdW61bcdU4bsxHp7XdSmmeT9/l2PBJrBbznxg4r95UQfOhq9zG
Y34BdThTU2CErZU0zw8HuH/0Khbip7kZN9JTqmiee5ZLBljeqEe/A20z6zdW
0zhqp2gvCz0vwjsVye03NyTTeA1LOlt2TLIBT1m4Bna8qKbPNcm8ia+ZeXNz
l5qnyd6tGDF7aVxbuAmdCwJDFif5FInQPeqUzbK7ybQuWiT6Lvda0YDyc39U
Da6ops+zyPfdQIX5vpuQnPvANObcZ5rbLFedm1VwsofW2eIHVbSOdCWfZUa2
CsCpNrhG5nMx9XmM7F8L5NfXwKtlLe9PGdfQOvox2ChZsasQhpsdkkt9UEaf
75ILVfM73wjWEw446efnUJxvk3Mf2MWc+3h7lXs1XxPAHKP6EU13BLTuZitL
D9hlUw/XxWfeqj9XQZ8XffDvs7OyCezf+NrJFebSeN0i50RgxZwTHfk4/GvF
tlowCpP4tMSiltapvnnYITnLBhgrfHPc1bSSPhdrrLIofdcMlw8EfJh2Io/G
PZ+cK8FP7c7nSv5u4a8PCWphW0vQfdnWWlrXw5dMeLC6TyPcNczTqYyvpM/j
taZYyau3wIvHRd2yK/Jonrwm51AwlTmHkpRd6ZcxvQ5mprrMurWgjvJA/4sy
U0x3N0LQ1jFOE7pV0efNV2H1gyt/dF6i8Pd5+XyaJ0fIeRaIuPMsnjdkiA8G
1ds7+2CLj2kmvTepg8gMz/kZ5nWUT/Q3BFgOedgIxQoiGVSsos9PEV8OCjlf
jtcJ8aNlT982FcGl5eOPlmRk0TwfRr7/jlrM99+vkHN5nM6cy+8h34vHOOZ7
8fbkXB69mXN5EbmXiKuZe4lJA8VsPjcLcGt4qGrk+UT6vK+BhMF1IyGqpZiq
pNl7U957R87lsZI7l+ef15D7jVjN3G/cq//bcnutEC/Ma/swSSyVPp8U3xZg
/IcHW94KFfNfvKT86UfO8dGUOcev3lYQ+L61FieVbfSZsTOT9sH07fMPjb9S
j7ftLzsMMs6m/FZKfGN89a2zb5xK8EczBv9ZJL6YzMTXds+Anh1vq2FP5YU3
QeNK6X5drx3z8P8kgMgHodp9zlTQdTb0cQ5SfVwHPfrVL47qXkX5f/P8F931
k4SQ06a8X1RdRXXpZK8vu2+fqYcjhzYMidhfTfk/zP50mKa9CO7LXvgml/pH
73D5P8jGLnXF53rQrJw+csEfvufrIoecj8Am5nyE41Uw39iZV63f7G4bnyOC
5neTr3UfmEJ11PEh4/avmNsAWx9m7r/XUU3vG5wjOKA+hwOPv3mZndJ9rVp0
ldQYXX4ph+Jzl+CDLxl85n94qJeXI0RzT4PVLVV59HkbwQ1/9O2M21C5ZRV6
4Q24U07xq/XjAvp8F8ETEzg8+X66cbCnVUKvJizJ2NX1fn4hxVmV4IzmDM6O
EUmf/Cub8PAiobrS7SL6fATBGRcwOCOJC27j4sI/P01ww/UcbjyeJwjO+IrB
WTR9dKJuTjWscpPylRufCiM5vbrXu/fu+b0FUHjU+sFCfEbnBSHJf5Dh8p/H
uQ+pU1BKJXXKxyWX1AWcYOqCqzt4H9y57qYIZYO91JrB3S6/coGEL8W599vF
ilGvW0A1TLevbYEvxfO7Ynd1K3cRHNcet1HWyI/itnLx92+G41tB4uTvIW4v
/Cg+Uj30Wr8cb4UTFivHdC/1o3llQfaL/Zj9NhJ88ASDjzJZJ0oz6zQlfRbd
mT7bh6wfPd51Xn8W6Zv4mOmbP8i+sGp55339aPjbBzGA6YPcfnHPCbJfvk6R
9DXMYPpaL4IDVuztjEMt6VOoz/SpdqLroD+n6/j46mxY3N5PtxYOqEwZ4rAt
h+bDXKL3IJLRex4JuvPNy4VQanki9UFAHn0uRXQgTGB0YG3L8v6xiQ3Qp/7S
vCMHC+hzG6IPQcjowwW2RqOSBjeB7uv5Rn0fFdLnfkQ3wlNGN/pJtCsOaGmC
nNiPxd9Ni+jz8UQ3wkFGN64iOhNMGJ3ZQXQgSHM6kK+vYKIbwZDRjYlbM8Pk
XJ9A/cmb+REvo+g5Bfcc2eeaL1UuzktoxhVDpz8otyimfTySnH9hJHP+lUDO
uVDInHNFkfMsFLPofJ7VRM6tMJE5t9Im51PYzJxP1ZJzKOw1svM5VNnDITJd
c5tx8DaFRR9Ui+m53pZCrcWnNraCy4zA4V2HRdO+tvNd3Eyf/CaMbXx94b1J
Oe0XxuTzWM18XkD+Pnzd2vnvbyb3LkCqhNy74HEYTO5XwBXmfsVAco8CYrh7
FLz/dpTclwBd5r7EGXIvAmKZexF9yf0HuM/df+BxUCPxgl1MvLi5Cdi5KY/M
uajLzLkLyDyLUsw8K0/mVvRm5tYuZD7FOG4+5e9jaJA5FOW5OZRfz0FyjxqK
mHvUM8h9aTBk7kunknvRYMnciz5G7j+DC3P/WY3ccwYJ5p6zPbnPDMOZ+8zj
uv71JSCF8SWMiE8FdoxP9Zj4UTCf86P4+ai8/K/vBELGd3Ig/hIsY/ylN8RH
AknGRxpB/CIoZPwiHeILQQXjC0URPxlYP/lUzd/vBQD7vQC77L/3/4G9/y/7
5K8/DEs5f5hfp+GAv34vuDJ+r3LJX/8WvnL+Lb9O3WF//VjIYfzYbOKvgjvj
r3LnLMCes3DnJsCem3DnIMCeg3DnGsCea3DnFMCeU3DnDsCeO8jtV7F96JCE
pge3n5uhWUDn5fe+972razNQ1Vxn8pahZfReX5BE+sYL3VJhTZTCqHf51dR/
m63hfrHdJQhsTh07FtVaROeXnsTHAxXGx+uYcUh324Yk6Bpz4cjFdQX082Zr
DmxY8zIDuk/eNSR9Yhmda5aTPohnuT7I98c7pN9hO9PvKkhfwwlMX1Ml/Qst
uP7F60kf0qfwPdOnZoo7a13f14zWr00HhhQV0X700cm9b3VoMyqt9h01dEQx
7TvLlq7svzNTCDqTzpv4e5f/dy6z5F5+dvcGcJl3UsxVoZL2TfuSHiGZyQ3Q
OtYnYfedSvreS28VkncvbgSFmrARFvWV9L0vOxwLai80grtz8It02Sr6XmPC
81DP8PwYm+E33K3r8MR3W7Mg/WSq2zfWX16hMaEB92g/VMw/lUb1obLjEruv
U4RoGXlbLic5lOqNt++ejpzxsQHF+0esu/ErnOK8iszF2MTMxdvJvABezLww
i8wFYM/MBQOJ/gd7Rv+vJTof2hmd70D0PFxm9LzXWfk3X7Y2wxL/HN/KfsVU
l3ac8PYe96oZnKZuTMjXK6Z6zJd8HpWYz/8in8f9zOfjBhp6LxLUwOqG0+nC
n0E0/6cR3OA3Etz4/AwhuIG9TGfcImucDq+WbIahYQMCt8+IpH6s9MDNLy5a
toBizo47gu5RND8vP/7tYWEkAhVx7bH1l6JonsSe9NGb3SCClG/nfZaWRVH/
WeN8kIPyglYQlxaYzpGOpnophqwf/Zn1I1kPbuPWw+MvRdaDFsx6LpL14Clm
PdFkPTjkO1kPj6c6WQ+u49bD4zmP1BcYcvVF/XNSX9CLqa+g4LAjP37G4KRX
73fPNfLHokClbfLn/vCrt4G7t+FrvBOXgzdbk/HN0b2D8+a0wt1H4RpWp5Jw
zT3NZe/M39Ln+bfcmioLA3BsyLuI+t5Z2G3+yjUGESIwKPUrhjGZ+GJxSFvf
J+H0ufyp53KHyoLR7fUGHSujPNSU3dU8sqsIPvgINUNn5+LLb5c81a5F0+dJ
R4efXx8Qgad8hcr5m4tx8hRVw2HCZvCr31G46WY0hj3ZtXd7WwXW6PSNX6LX
BKf9tjdp/4jD+cIajak11Tjs4N7Texc2wNH2nOHT2+pQMUKlX5JMHhpd6DDv
5l3zz3utTme+/NKfloV7ps0Id7rQgoEvROWbPbPAetMi25uHs1Bs/wiVQT9a
0PR+Unp4awoUhFj3GHU/C4/62GpMXirC+YEq1vEf42CAkptNaY9YtN/+pWqz
qj/wOE/f2fVygmMS7krf/3banLfA45mxWizffGImfvhqnrvRLBx43Pat8Nqx
5Xsd7sqd/XVGbS4Yc+v/l372PWTyclf9c7S/cKzk0JA4+l7r7zbqM2Neo+ik
fI9WYTJ9702pE72khwaiZd3EO30ls+h7V3scH2QwLQRNcr22mOrlAR+XmG3v
rt1wTUX1CH/3CtV6mBf5oCL7z3owomivqCMdTz5enx8+v/GPjlRx+fmHj0cu
WFZvsCUTh5+a9s0noglspn6xKjQq+f/uL/G/y7Fz2O+dIfgWZuFPecGZaLp+
eNFnaIVWBByaW//rxr0Eun4PIynxa5Ni4N7SzC5va9Po+i+nFresEU+EbVic
siU0m67fMSTlslh7FnRNWnhtW9dS4PPnc7e1TkpqJdDz/tsFTVBL9xUVsvBM
n/NlELHz2j3LHkLoyu2r52PFyS7zKsC3NVtJxrAeTnD7suTuO/1g7jsNCQkU
E8TFQcF0tLs/0Z3ua5Sg5VtuTAosN4vRz+r1kO7rSNJ3H3WXLFh1t8f3U2ue
0H2JBdhcMPlRDwvyJxkvrwoDPp//dQ924KPLKsc9IiF/rW2kQloYrffk4uqy
MapxMHWQc/IQ42ha10KHexO0i5Ihb/41l5uQQOt3+E0VmCdbAw+7ai0eea4S
eXzevDd+sPulABxmBee6CKuQx2eFVp9hO8prQS8i4IxXazXycffjfi9Uh/md
GXl545A3tj5hy220XWN/x9F1xszpaD57wS9sblTEwMfSqXSd14YO63VMJSjM
8/XJR1+j/uOf2sgpNaMy3oWdTdu4WgfyKZ/A0qcbq5Jjw9x3v61Y9aiS8kbE
wqUJvs4ZYdN/ZXYfv6iB7uvIvGDzNXJZYeeMIxfLGzYhn88Sv437HhmYHVa4
/YbchZZmui/2Phifz8MI/hjJ4J9L8MfRDP7tBH+MY/C/PkPO1yErEyeoSxWv
GJlG96VE4oKuTFwiSVzwCBMXPRIX1GLiwv7+D7/+wSRvcalK57wdTvIWrzB5
e5jkLT5h8rYryVsMZfL2X/eB1xIewEUMD4wiPIAnGB64QngAvRgeOEZ4AM05
HljG4VZG6h1lmHp/SuodE5h6b370t94xhKt3Hjf2dxp53IoID8MphodfER6G
ZoaHgwkPw0GGh60JD8Mmhoclbg9b6jsEwVbj9AY9/WLg+6Pi7LG/NuZFg0/J
3bmr6ysov30ivA2LGN7+QHgbrDne5vc7j/D2n77dmbfZ+4H8fuVIvwMLpt/N
Jv0OdJh+l0P6HWQx/W7gzdzWALVcuLwvoeHzHx3M79eS9EFYy/TBf92X9iM6
B9QYneNG9AyYM3pmPdEtkMroloNEP8B8Tj/w7/2XbyNcaSYv4fgc2mZa3Hs/
IA7tzu1SfPifvgJWX3E6ClgdxeklYPUSp3OA1Tkd0+S6nJuVCmnK5wUxC+op
DzhajE3qej0d3gwsu6S6pJHygOzJkU2H/+C/+ODV3GvxTZQH2PuTfHz7R66T
1PGPxr2LBGNqXr/BO2lPIk0u/ukjiTsMjn2Pw2th33d19wxHpfvxc9fotIKR
W1bcgS1JOP3llqrH4nFonmXo3dAkgvWZt00zPWvx4PErVg5hRWirvm+jeFgN
LOW+fzSCude9ZVP0wDtSsTjTrvClxOqXoKJjcsLIuRU2BB2T7eOehGMjTb9F
PPMB/r0rA1YVD1HLxC4b5OSbA54C/96BR9zXbxIKUfmwzWKN5Azg38v/jkom
8zsq6oVRqyvXvkK5HVXWNh9i6HuN5lqsdOv7FpcH37XZVplA36s4PaXUZHIU
9lwZJy03IpW+N1dGvWPF72xMLv8+PmiWEDLcx+5Tq6iBPO53VE4y90xWzu+y
ePmCQJAYv/z2s6Ro+l5BwZKBY7q+Aclnw5c+00+k70XpBQ2DI/ygWb+b1r2E
TPj84/QusTQR9Jil27r7Wj6sXbD9+/mzdXS/MznfTJ/zzXidYBIzZdaJnFg4
k2l+Wd39EX3v62uKN7w1kyBapPXszs/X9L3vRZM2bpuSAYcrMgtazkbQ957v
17HFybwevJx7iVYWJdL38t/fETDf39ELsEnJHxsFuxR/KbrtCEH+vaYTT90f
cyseFjz4ZbA18h3Nq8+JS+viH6VBZcwyRYf0sP/yKj/EZ8YfHaJhe3P+p/hS
mlePuN+NCWe+55u5L9pS7rsjmNVW5AuXxdH3ls1RSAie4QG3wytURBOS6Xtr
klQne3f3g7q+r7IG52XQ96p8+Gpqv/gN2OXrdwvr3YCnuPcW/cPfk7I/XTa3
VwSmX/swz311BH2v6pZBHamfI/H0hulyEVti6XuLN5w0/9w1GqfKJmT0epmG
PM4BJx8Pc1Spwo7u0b4DdlWjHffeo9zv0rxmfn+jx48sqc3acXi/d12TY8c5
Gl8LvwOx5xySMevrjRqX4wfoe8OiVqvtU8jCVxtWDJFS86TvPTxHZcGwyQ0Y
1iS1M+vIfRpf/vdkLjO/J2M1tOGKRGAI9hf/cEYgGUXf2/WK1awREVHoNvSg
6bjAGJpXy6cmxqTrJ+LoFZdSWmfE0zoaW/I94PqsCvwVsGTZ9Uc1tI5yj7YF
Ph5eg8k+CS5j19aA6obbdj9rKuFnnlFx4546VNG9gYtGVcPavMjHc6NKYDT3
+9J5zP+/zLxAvGx9qQcInNEtoyyWrnN+vlZTLtyGMPQc/mh1Ml3nCKsvV3YM
swG5PKuOZccy6Tqv3nNf9GJiNJT672+TSqin+Lj9w0c9+nxyr4v+0bB/5eg5
s68F0vd6vzB0z+4ZDy6LFYyWnwin711QenhBqmUSmKzw2NTRFEvfG779m7JG
YC0EbDn+avf1Ivpe/ntGa7jvGfH1Pr5ijq+lWCwYpN55nHr3Jc3Dkz3Me628
ngT2oTZL5KRe0HxITDX/vX9hJlw81GfhIJVnNP/N1rV8uFUhhGq3pojTAzJp
3SVyv8NznPudCv69H0rXLBcTvYaeemcKi2VjaR9pHSwsVa76f3Rdd1zP3/e3
szI+CCE70jAzyyk72Xtkj2zZKSTJjsyIRJIZKiTitLRLe6937+rdzoyP+evX
Pff1/Xa/+vzzeTzOI+/Xveee8zzz3vMcOippBh8rCZG+69/niFLT1v5wYU3Y
8+f9I6XvLrPv+7WNajx4WQafGzFdIX1Xl/LA8UIe+M+bwotHNN7h85n3TfYr
J2HukmP6uvXLoOG/q60bHo3HRyFH+im7xUn0NQUjfzWNe4etow8ZRthmYO+S
4CNKtiVQ+NFhn5IsFS+WZDntKo2X6E0bg5pOw2js+fpcnvt7GVr4LGvtt74I
/JwKR9W7nYO9A9tMquicINHXT9a1tF+fgy2Gux3oFq5Ah+cBDVT2poMqzYWc
TXOjuP1V+frq2tx9Mrx0peJf18lFeDMhPCD5ZSxs6BjQ1m1HAfay/eLdsF6+
RDeeaJyhciUeX8P7TS3PxwHfl97gQcpzDN7hAINlc09PSoLEoYdOrKgshU6j
NV/M+pyKdbSnhEyJjwe+r3bhRi6vH+VgiPLuiAs/44Gv33zb8cKwydF4P/ro
JJMimUQ/ZR3d+eWgfJysdtnH8LYc+L6W0fzH7sL8r0bGbivqHCvA+A3KXQaU
5gFff0xUo8S0CzLs1rOrrfOIIom+jeY8Phbmu9nVQl9bP9xozrrzsMRGf8uu
f9MkPvQ+u/rKiv0+MOCfjHRt/f/Q1xwxzL3U4AZqKoeNuj4lR+JDy8i6PxQt
gyFs7ZNk59dZEt3A97l3E11n6OAx85Fzbqm031Ca86grzHfruWLNofMqN2FI
zw8hN16WS/uyNinV1xodDSVO03U9o8okuprJhMHr7FJgcZrmgmb9smAv8XnE
pPdqF/YWgIV2T8swzVy4RN/VovmMwYRvXH76/wg58qlMAZYPFsdHTZNLv996
bU+TLIsS6KyHmYsupUv0ejSHcbEwf/An/H0+44BVd76WtomAPaVGjs9MkiU9
mnky7eqrxf7wcPrtfkcjUpHLW9v+R4LrTIqHc0Xj1HaYpEl6lJR3vfXQka+h
Q0WTOY0TsyX6xtZFz+Xz08DNx16zmVOGpEc/ek+YY34iE9Ia997umlyInA8v
aD5jAc0l5Hy4cnLVcO/R2XDq8OHM8O3FmKPTQm1XRZW+2D8+mLwtHm6e/jzG
VrNM0qOu75XTdj3KhRWWBS8b/SjEdXdNz+SfjoQjV81Wy6zyoYGW5gelJgps
dbR/aPSsYGhby9xGVaLPE+gHS8E7/7I/+kQZxvU5mirxbSrjJ24V+Nk7v5o/
2FbgT2/GTzwi8HM/4xteEvjm+dGz2z8j4vDp1NlNT24ukfgWS/McZQLftjL+
4FXizw3ijw/jJx4S+KnD+IbziG+mxLeLjG9Yqcn41pr41pXmNnoI+lu/Frrc
vlp/cYigvyvqVes7LhX0vTiiWn/xnaC/35m+g6jvDRqXK3n2i0Vz2bZZ/XfI
JL2zjGvSI+N4JN6MVw1Jq1sq6d35WuY5DmB6je8FvW6+vBoHkOMA5+dp7/5f
J815B/szAvxn/U6U1j+H4Tl4CXgeGNzA7FGDGDjdUjfaeHeGtH51hufwSasm
nl9huA1uhNt8X6oM5+G5gPPe3bH0xsEcyFiYsE/9pULCt7U0z7GTgOcZDLeh
g4DbrRjOQwjhPN/vz2/V9hd8BPub2sr7Qt1278A32qLL4/lJEm5kMvsLLoL9
fcXsLAwV7Gw9ZpehF9llvt/CIqdYp675YJj2z81IH7lkf9vS3MaFwtzG5czO
Qn/BzrZidhkcBbv8nfk5qCT4ORq1zGHUIfpiga5aJ/fPrOxY7Dz98yutFwlI
eUfQfYBNDnkmopLrrTfz+6Qi5Smh4szF0flZVfrf0OLfFoa3JX5+PX06eZMs
Dc/ZP30718xVkp9Xf2b79vZIxbZNV5+Xh2Yi5UHh69r8Vun7quLwmMCx+l3k
Uj5B8bHNgmOZMtTF4Podn+dJ+YTXbyOym+Tn4t2y16cuKPKR58NPVLa1nfOy
FJ1UHELtZvlIOCPOv+Y409pHxbRX+zxUKXE32D5Zgc+o/sLnKm70YvMj+N9P
slAO9l2ThyMCGkSfua3A1VSXWTf04wqZfTm+PR0fZ+b5UjoXtQ12s1pdKMfb
6gum1XvrI8nn1xELD+x0yMN/F+UrD85WoB7VcVJpHqLnpZrzIDj9xpma9Ds0
J/GyME/EpZb5ifdCim2TClOwwesHFhm3g4Gf74NLBdrn9LOwd9q5VY8PRAA/
39e9U1ZtVciw0/Rvb4wmxgA/rzaLcVpCnVJcdLx3xoddqTDccm7fSJdYWPpn
ok/96FR8G5J1Lu9xpvT3oyYonVlxJge9PW6/GqQsl/KBj4O3d7z6QYZu16IW
e7vmSfnA3LC9XuO+5+LMPiZHBiXmgwWdrzjXm5+Lfl6whrd2Hk76abihYKgC
+Dl+6+OVfGdfHqYZDzrofVYBvI7W39N9k8OjPBw6pP2KfhEK4Pw3XnZ3duMD
ltCxeGBC1MJYKc/QbVraBt3yWxBhvMK2t0UsbL907YR2QDlY1eIHtjjV4cvQ
vEfQb2Njqzcq6RKfG6zMPjbfzxNiX34cbbYtR+IzdNGK6D7kOfTL8Pt3q7lc
4ludqw89jln5QrjduLWWv/6TRzWKzhioJnsDG9vFzlRqWyLxbfkv01NGc/zA
at/6yxtvlUp5VHEOOOeb/HrkkgQTf9DNU127fEc58PqjTauFuX43/MG3k/rh
g8VVcRfxLT8ie/+LCH9oN+/i83TdqniV+BZHcwY1hHlSfP5gz1rmD84R5lLt
JPpKgb7hsc6D0JGJsPnsnc83lWKlOGK8TGZRUDcFbJbsdbruHSrhjEmPZh/H
zE0G+5+OukZpSZI9Wjjm9cioXVnQZVDya72zIRI96lCWo8qRNLi75GOrfqOe
SjgW1bUjHtdJhZ1GV9R+3Q6S7MInL4Phw47nQKOEoeYmxonSebXbNP7gygk5
8NVtR8RMs9uSvXgwNXpAwoh8sAwZ3Hv33HSpHlHb/RSt7xs+lW0vhTr7lPzq
rQyScMyV5gm2tKs5X6zlScunj8eUQ/Sgm25K0W8lf8nY0HPqoQ2lMMlwcPqi
/ckSLrXpNnRVkXUxvLzSr9v01jKwz7+0dmPZW2kO4HdhDuCbcQ/N9z9OgH1B
pgeWFcVKdqF/o64PIzunQnj3lu2evUmU7EJcuvy8qW8m1DWdHuiAqTiI+LPS
5tk/jw0UkDLvVh+zHnlSvXvxmlZb3a4qYPkkE+8h2/KkevdR65FHBiZWxRFt
slY1vpUn1bub2MYfVboWAZ3mlCzOHZolffeUlduCewtiYKRBnYK9cplkXxJK
xjT4/j0BWhl98/f+UiDZl6L+y738XiVB6teg4MScQsm+5A49u75lsxSYqtJk
SMj0Ysm+PCU/v1CYP97MPu7cOftUeP3Izbjp9xLJjtyakb79YpVfpHhr47fP
tFTa18aGMZNSB6fBc+zm8uJhqbQv1TClDb8nP0O9uTuiG9rGSHmwS11eXbBa
8BqPbRyd7toiBjn+8Lnt8wU/v5me4dzGb4KxtFh77xx5inReAxjf8B+BbzcZ
31BX4Nt7xjdsIvCt8YBqvmGcwLe6w6r5hhOIbxx/Bpbf9vnlWYxXP9m4lHVP
kuRZnOfOcUmH8ROfCfzk8/W+CvP1ghmfMVvg8ywm/6guyP8/TF/QRtCXM+xc
8BGdC7cL5jRHr7+AS3y+Xvda5uuNEHAvnOgg0OMZzuBcAWdiGS6hqYBL+gz3
0FnAvRUMJ9FcwEkVhks4T8ClWQwPcayAh8a9oszejyjETbcCZ0xaGynVoR6s
d5uWlFyEg+tcnmJQ5Rfzc+xNec5YIc/ZmMX7CEK834vlB/CYkB+Y+7Ht+QYn
onDQrUmvAxsnS3bzbYKp56DyWJx5w3u2f2W6ZDf1K1pumTIqETcuUjJefjpH
wmHThSOfg2MOxru5jb+QVyT5A++s39UfnJ6D9zveDss2Kpb8gejUDh2MGshw
0WMPm0/HiyV/wOZktR1HLbLjybSeJyuq7TjGCXY8s3O1HUdtwY5bO1bbcYwS
7Lh/VLUdxy1kx3ndvz2z43hIsONHhbiP68slZsdxONlxvt/BzI4jkh3n+73E
7Dh2EOy4Zy3z6byJflag/znmEGT6KRa8XXBumFuCdF5xF+LryMMT4cyOj49G
/ZMq8ecbiwtAXYgLylkcAeeEOGIT8yfhheBPjmP+JLiTPzmK+PmS+ZPgJPiT
ZcyfhEnkT3I8X36jxY2Fr0thmZflrLU7XkjyvI7izY5CvGnE/EwYTX7m05rx
AvQPrhkvNOxb7X9CjOB/dmVxAWgIcYEpiyMAztSMI0YxfxW0BH+V4gJ4cfuv
8QKEP6xJV6O5cquEuK9jLfPmBi2ZfSYyMwVU4nom6CQFS3ZkUuKZjZoDsqDV
gNh+C30iJDsS3XDowIPZMuh/3LRDww0xkh1pbrptycbPJXBYY/PFcQ9TkccL
yqzOC1ZCndedxYnQXYgTy1icCKOFODGTxYmgL8SJXixOBE+KE7ketaa4e5EQ
DzZi8SB0o3iQn68ei/tgDMV9/ByLWRwHDRazOI7rEfXpgdinR/V6FOv1wxmf
8WdsTT5PZ3zGj/0Zn225f8X4jGoCn/9hfMZ1xOdhxGeNWuag0ZwsuCvMQas7
/PWvJuXpsDPZcUXgj1hJf/3mXbdMb5sODx9Hx6+a+ljCw5EHHygbm+XA4Bmo
1r+Ol/T3c1bboWJblb9/+/q9mZf9ga8nl/mHWLGW+Yd8/VuZf4hhgn+4jvmH
OFPwDy8y/xA3Cf7hPpr/1U6Y//WGrR91n7D1cz6PYOvHw7R+zueZbP0YSOvn
cvuYxdFQ4lszjvZkcTQ0oDia/04Ai6OhsRBHq7I4GvSFOHphLXOvLvcO1U80
Scdp9na6OvaTJTnZUDH1+o+rOXgpyivPv+sMSU7eWhw6HXNfjk0Hrovzbj9b
4vNw7TPNe40uwrtfNL/Mt14g6dHk5vIm7ReVYJ9mzraH7RZJfWj1vQY9OVxW
inEjLsT1bWci6RHPk4S518S9s6/ql2gElOOuvVPLOmcskc5xhSLE8MDICty4
uevkshlLpXNsM/PB5mtHKnDw78d94hyXSuf4hdkX9BPsSzCzL+go2Je1ZqX+
F+smQ7Bp9tklN6Ik+bwWNe+22pg0OOGZ8aB16BvkdINRHrswJxsSvrfLMXwc
IOFY29/Tl1/LzoWuKy4UFHQNlvjWudXtK3NGFELoIKObZ9dEIrc77fQehO6D
YpA3K3v5Iyhawh/ffqe1XAJKIHnBmoR142Mkvolz4aW+RINbM5YcLgP0PXe9
3tRYye+906j12qSvZdDIaZCBh32slHcqqPu7c65BOTjYxj+zfhWLI4lvN9l+
0Zj2y/k2nu0X3YX9dmD7xffLa+5Xle0LXYR9+bN94TNhX+L8ay4PXdm+8Crt
i8uDO9sXyq7V3Fcp2xdup31xeTBh54u5dL58X5XDqvEKbQW8KmNxOqoKcbrq
mvtuxiPyUW+D54zgOemSPxbWZNeiiX8KcJnu0fFrnDIl/8GzZ3lTi1mFeN/D
fMPgWdmSPybOj+bn+OvgmAGHMorwJL7UfeeWI/ljq9TOd39kWIz1f1xy+6TI
Ab7ftEkeue7Wxbj48ZiYvlXxOLfvpO/w4MFf9R0GZ/1V3+HZuL/qO+S30Pqb
vsMh9Zr6Xo/pexVeMn23qKnv0lxdQd9h2OW/6ju4nfqrvsOrLk9q6HtLln8G
dSH/3Jfln+EfIf9cf7zPhPotirC10dF1I2RPpT6EM9RnFUJ9VsMor7KM+XsY
T/4ez8Pv+Hdhnc/bSrBU08Bhq1qcROf41uduzf1eYXMPMUyYe5hKfVaWQp/V
9ayyRhcmK8DXzNbBc0C81D/gSv1CPkK/UD8WL8NrIV7eNmjUZPUABVw7MdlV
dVeOVEdwITxRFfDEi/qfpwj9z2p1+l/eeycVA952TG7+KF9azy7q5/EQ+nk6
qlfXPTFWqHsuLqhjdfyxHLubLt2lo5EvrSed4vpsIa73pb7fMcJ6eD/PC6Gf
pz/Lg6GPZc082LCG2KFnbiniGMOLyRY34CLR7xH+lAj4s4TmcjoIcznfru+Y
vKVZVXwW02tJ0K1nUj8Vn+u0TZh7cpzVF6B1+5r1haxtv5w6rS+BAUGNlz2Z
HSfRub7siK0pP7k0N1khzL09/U1zucXvZCxw6zlrpHoZDlj+b/Ok5sGg/iHe
fl6xDCe6JnzfX1Ek0Zex/h/MEvp/eo74eSCm/Bq07F3Hzl6rSKK/rGW+kn0t
9Dus7wvqNmJ9X/x3GjD5gXBBfjbQPCYfQX7WEf2VOL+MyQ9ok/xwvj0muZ0h
yIkN6/fDPKHfz80oXqvZwQIcsUVr0fg7ScD70/jcpfrC3CVOVxLohbrVfYy4
QOhjXJTq2DHvXiguswnZYrfpP3xuWcscpWa10Hv1XPtcPTQVlQc46XTKKpJw
ZiX1dSsJfd2td46efN8yHXvrKHWzsy6R4kSreROeflDPwBS5S+74IaXA5WFe
P5eiF8rv8Gn2YK/KU1XxKNFJrkCUq25MrmAKyRX/ez7nSFuY+8DpPQQ6n3NU
JMyV4PQsga44OFwp1yEXx91razfaJxSnKj84GDKlCDocO/wo11yOirVWeTpb
PCT6nR31F6Q65eJqiyUef8xCgdMddUb8qLSQY/7mXUmlSU8k+jrWJ4YR1CfG
5SGY6Tt2Jn3n58jnHE0Q9J3T9QS6nOk7thX0vZDmWUeQXvNzfDn8ik9YFd/9
Y66dUj8dL8X1MW5lpSMGlGGe8+dsm7nx0vne6DJnUfcZVf+ueea+R/3eSfnA
3TTfJ0Z4R4jP/YkQ6A38Vq/T6pYDjTyd79mNSZL4OcLr4q19KrkwMsOqz73u
0RJ9J+u3hOCimv2WWyuPLNMLK4TOZzXdT3/3l/jG5/44C3N/OD1MoA9nuA3X
DRluc74tp3nWUwV8BkfDEb3HFMGppm5GjsdkEt/W9L/jH1pQBJETQoaUVMok
eT5+cm+P1d3KwPjMKQxUTZHkuRnjA37wqMmH0YwP2ELgw32a1+Ms2OWHRL8q
0C2ZXcZTZJf5vpDsnb5wz8WI7Qv30b54/tmM7Qt9aV98/WfZvrAv7Yvv1+FG
q88zf8VglxBb46ZjE6V870maj3NfmI9jR/S7An2A73HrxzdTUNntmJO1RZak
R6UJv6cc35KF06YZZu95kiLRXx5+VLb8dwkazvttPcIvUsIxbmfdhXPUH3io
tc5YOfbVvVsw/atCkvMhu5drZfjLUXde3ifvM4USjl21CG6gdqQQu71YmPo2
Si7hmAHNr7lNeNKf1s/pjgJdTvNrthBeifRVAv0+wxnQF3DmKsMZcCCcmUb0
PIZjsE7AsfYMx0DXtCaOdWD3AuD39pr3AhowPxbcJ9X0Y/n8mivC/BpOPyPQ
9zA/FmLIj+Vy6Exzuh+Sv8rPJZHhDwQL+POM4RVkEV5x+WwSVXn2xbJcNOm5
ePLuUTIJB8bSfYruwn2Kj6zvBYcJfS+TIzR67Y4pxMNqR9zPuWRK9NOFZ0d9
mFyMf7acWmTbOwXvzT/esueCdFCj/o3pQr6ugPA2UcBbBdFjBPpz1reMYULf
8oKuS/ZuMMvFIrd7cgcNmUTnc1UWC/3ME4i+VJin0Mg2MD8yqxAHtRnfcP72
TIn/lyheiBP4/2zS8OMBEeFQJ//I9g+tgqV+g4Pru467/zgK6kY73x3XJVDq
N+D1dHWhfuTO/B/oIvg/++gehILuQfD1e7E6DrQR6jjI9Bp+zGV6zemKzB6f
j7iUgtJIDSXbU37Az6U/9X8GCP2fywjPVwk4wHF+gUDXzK7fof/Hdzh6HB7Z
1feVVPc8P2rr+K/vY3Dv4Q4+zTO8pLonr8f1Fepu25idwmx7Zqc4H/LZ+nEG
rf8+rZ/jlYuwnhVMHiBbkIdp1N8+Wzj3i6wPGUDoQ1Zm8gAdBXkoNNcydppX
DC9vrTDb/DFZWs8Gqju0F+oOTqS/zwX5uU50D4E+jd17Alvh3lMdpr+wmvSX
80eP5lxoCPrL519oEZ3H0WOY/oIj6S/fVw7FNR+EuOb5J/d+d9fH4tLdARXq
B8ukusmBG88yg91jsXOT/SfOfCqT6iO/fW8Py0qJxYYN/RPs9cqlfMhuVteD
MKrr8d/JZnU9cKa63kr6nSxW14PpQl3P6MlRh6MXQ30pDyLlSZSybu42GRHv
S3Im5d/2H3ihe/ZwvK+2g12F9cgKKY/xp+Kz9vFxwVC6LC61OCNXyi/Fsf3C
ImG/19h+oT3tl+d/Or+u3i/UEfZL7wNALe8DgPg+AN3LQ/FeHt1fQPH+wujf
L4yzPDMwuZOdhvbTPIlO9/hQvMdH9x1QvO8wguixAp3u/aF47+/Ypq6v/ygl
YbjH74cLXxQjj9/FeQFcnlvVMkcgTP9t14AVyXjSs3DpILNSyT/UV3m7t8mP
NzhmQuKsZaPTpDyh2dJfPwZGB+D5PsMTBmVkS3nC8aFpyuWXk/Gb0vkTEatL
pX7RSbattikNS8Fue5qrHFUtk/yTBo6DnSzr5+L6jqO2KFfFcdw/SWPygOWC
PIzs1GPuoAevMGfuuy2jjOSSXxHXYZpPh44XYGv2FbsNF/9Dn8nq2rBGqGtP
3Y839K4E4cgJoabbzP7TD3yK3jc4LdyTrb99a/DbxsHYwvnj20YG/+n/H8ri
SugsxJXjX2/aUXE8GO//2t7sTP0Kab95m1faH59xE9r+vmEXfLBC2u8tFodC
NMWhnH6PxaHwSIhDy0ZV96XAJ6Ev5VGXxZOVVP19ST4kunL4iNwzPTxA982I
bhV75ZIfNdfjbIMeF0MhoNIjJ9sxF7k/tqSr9x3fUS/Bed3InfvOlUvy0HKO
v0qvp/G+VG+T+jq2st/HIfT7/Hds2O/jG/p9/l0Pdv8OD9D9O64vphfnut77
9g67P9P5vE/+H/3qXMt7772Jnkt0bpePVCj//Jn7ArVmddqV9qpUwtVG9dRn
DDBN+Z/7/pVsv+gk7Hf20KwN+PsldpzyZafG/Aopjkh6++tdZWE8dhr5RelE
4X/yAEeYfEKWIJ/b21fLJ5qRfHL+6DO5gmYkV1xfbJj8wF1BfoI3VcsPthPk
52fYZs8pE+Nh7eLxvwvHKyQ767T+7ar66Rng8itimq5bnkQvo/fSdwv51Q9E
3ynQJ//7c0bSqCTITPhgedW9WNIXL7pXfkzQlxUMB+AT4QDn52qGA6BKOMDX
35rhACwTcGDwhGVO/1x8A11j24Ymj0+T5HnHqBFLptoHgIX2zgvj8rOlPDbh
GIg4Zu0xAU33FqNFxafyeROypfUMePv68OKyMvx1b6xLSUS8xGf5mK6twxZU
4MVxCWOa2L6U6LadOxuY9CjBSwvrqvo3zZHoBz2ycFMnBU5redTIxSxfom9j
fTigIfTh3GLxJrQV4s16hi0uDT6cDd3VIytU78il8yo0GmC93VkOSh+n/Bjr
kC3RF9N73Y+F97qXEN1doFd8d3PM6pYLxYqsdasOFUjnWBK87er62YUwMPRd
sensbMmP2k3vonwX3kXRIX81SPBXj6u4Dq4fXgZxa5aNbGQeLfHZiMWtoCbE
reMK92zUOV4ODVNMNBoZvZP49m2Jmvlm7VJY9+ac1/QxGZI8jGFxLvSjOJfT
3VicC20pzuW/E8b6LdFM6Le8z/JCeF3IC+mxPkx8KfRhHqR3px8KeSH+HvUd
gb6E3WtGDeFesyurX+B1oX5xeOXlf7Q3FMMB5SsfHu3NluLEzBhDKw95Gagu
Sp3yq2GChD+/H1rtPjS3Alqc9xq61fyVRF9iW++Z7z8lMPqGbkWr3jkSvaRz
C8ezjRQwepVrYKOj+RJenWLfxdXCd9ew38EBwu9Ust+p8h9q/k4uWyf+WMjW
qUP0Ou7V60T7czXXufNTSB+jb7lQOWvRz1vXAyWcvLt/2Y1Px2QQcbfZ45X7
YiX69zflg1WrcDNzrPbc8UYXJfqaZ63LCibLIfq+iYbx8leSfUl16LWjw8ty
yD3cN2hYQxdJ3lTvLlimV4WbDXpF3h0w3kWSk3y7NWcmNqqAxveLbjybGiyt
81/2XVw9nn2X47Yp+y6a0Xc5fRvbF6rPrrmvW2xfmCPsK4WtE1WO1FznYaZH
2GVtTT3qyNaP0Lvm+gvZ+tHnXs31A9MvnCHoVwXTLzwh6NdxhpOwXMBJO4Z7
cFzAvZMM92CMgHu6DFchm3BVyoczXAUVAVdzWRyHSUIc50DxfpQQlx1j+Q3o
sLVmfkNGcVOpEDc9YPehMI/uQ3HcG8DsGsoFu7aI7kf0Eu5HOJB/eEqwd9dZ
XQMWC3WNgbW8dxrK7qPBNbqPxuPc8ayuAQ2oriHVxYR7B3xf2wmffwn4rMX8
KOgj+FHyWt43OMfuFYIP3Svk66ksr/ajQEfwox5T/3aR0L89yWjXpbETwv/H
v1o0rLqOhq2EOtrGWu7XD35ZfT8X+f1c/l1b5rfDcMFv30/9nMMF/twl/+SE
cF5DWLwG6UK8RvEXiPHXKnbvG/6he988zqL4C8T4qyX1xZkIebZWtcy5CzNf
mr3pTRA6143Ps9FIlfyBslreI/1I9CECPd1SK2HRxlhoq/X8S387mYQznfxv
ySL3BsKUzCHr7n7PlXBscmfD7ycd3SCxbE6jO63TJTv4XLfa74VOgt/7hPm9
0J38Xk4fdHHTwWN2QaC66vj944P/c194ZS3vmpoSvYFA91cMuz3TqxR9NUKO
juyQJt2nyPdtb94yswh2Dtmp3e9OjtT/5rB7s+MVt1I4YVzHWjE9Dc/Q3zsx
Om4hOv+djfvM903YLYfYkc4lEQeew+5npRvCquLBbqWHDb4MroDZXx289jQO
AP47G9jf4+BR7O/30N93YX+PLvT3/PdD2frhhrD+xjetMuPSAvDj+k8zOjZ9
jhVbP1/OPVoBipl9D9dp8BotWinCPoEHnlun5Gb+ogJ6TfbTGx0TisY4doVm
3ddYYbsgPk6nApLO+f7jNSgGr3j9Ue9YEYCnF6cUtb9cDsGW75/eehyEfw5/
7J7s6wJOE68+8TtdAZPyQqDjRMQuSZszFXOugYn1UOs9WAErtl44YK8fgXdx
y/4XD+/Cj1a6Hy+PrIDkS0t+NI6Kxc56X3Wjuz6BHi45h2IelEPHLHs9t/Pe
WN+s87mvGv4AzW96yc5VwOe+Js3qxnlgR6cjXlM++kC9pY3bJftXwCn9nEql
logTx32+svRaMJw375v42LAC3ky+mNR8ehDecMuwSD4bBVdvn2jywrsczi9o
adDWxAGWN+q69LNlECzI0F075kIFdC7qLHtw5yxoD5tz0KXyDbRYUuZzM6AC
1I37TYkJvg7xX4LnVf4Mh77nTLKUx1WAze26jyo+3YaWZiZJvQzipTnXBRd7
JqaPeAgFZb79V/qlgBXNcXN5c/Xyy0APyAlvcTW5TzaoRp3+rpVZAhNDwbv1
02dgq+Gwv2djOaQu6PTn04Ui8HK+cOfOSR9Y1HRrW7e1BRA1ymrt9RsFcKvb
cZOjMl+o3Jb0e/3sIvhj2Ttk1Qg5rB0fNrrDLoQGcE4v/GIJ2NrsDNxjng1z
N48cE3bDD679e+TuRY0yeGu60XNrnVQwaWleeEPXvwpHF/W4oVMOmr0vt6jc
Fg8DvXOVFfb+cPtjXIZXdDlsOnwkxi0hCuw1Spc3DPEH5Yyej520K+BWss6V
xMfBkEpzV7fQHE9dehfXRM/529IufrBt14XAHcteSud4/JMdjDN5AckHbxc3
qfCSzrFv6Ob+x0e+hfaf+/j9XuovnaOfT9bnk7FRMC44+vvI+qGwObLrmu/H
yuHUApNLewfEg2ZCrPyjIhoCo+94uR4rg/j43d8rBqTAipjBz9Qs48FnfUmQ
4/cSUC4K+2LjlAnrghVWc1KTIQ7dXUxfF8Hm08+nHzGXwbx/racv65kB3rfn
BS+JK4BVQWEqDyPl8HLtafPD8mzYOWLpmatb5aD+1WHGhGv5YKi6pFEHUxmo
fF8U2D43G7aWFHR4LisAT59M436bc8HZ7N1ys6WpsEmxfE/pUwU0+/ngqUF7
OVjnuyWu8YuHKYO7OocMLYSiwq894q3kMOrLC8des6IhWnNSer5NIUy62mZ1
2Ws5HIsfd861dQj0Cui97fWTQjjc0DeqSa4cJhs9un5/lz9YPd/WNkLzLYzw
am0gszmMXB8Paxt2X1/XD2xtU9+XN7BCro9H1s02ayGLgAerJhx/X/8kcn1M
6qu0NntYHHwqHPN9+Z2zyPXx30cP2zTcmQIBT35+zX5zFbePyRige6YMjt14
9D4oOQtUD4Ts32J1E2GYVcKBBlV4dPWR+fq8XDiY0GyOw9bbWM83cnWLiCLY
s7b9/txpBaD3PeBN1PR7mImtluvJC2CLf/KwTVOLwLfp2HOnnj3EziM7q7Y7
JAfDis1mRS4lcHibQcd9ix/j3Y/TOqvVz4FMrWVPu04sg19PGvXoee0J+uuA
+9MDqdDm6jKfYQvKYaT/8Duth3lgTnTTBivS4yHz+u+e4+tUQLCF2cV1lzxw
SKcti5fuj4bCXxsedauS14cLwqe2fOeBiqLvE00Gh/B5N/Cb5t0MIXk+6jBW
ya35Gwg74xUT4IkSrtatU/XfCQtfZ6YPqD/yTOiUAxXQelVBVLzPc8i73XRn
q3+9cZPK9RaHX9Hf993pO4j9Ls6k9/NHR8DunLeBcHNySkuX9CAsJxz+/z+v
M8/adw3TQxzc7tD8zWoVMPzZQiXvmDBYut3AauGoCLQjfK7++0VHfDWYPktz
25832zGrWWwMuK/v6upSNw65vixVPztSe3AinAl9PELDKwm5vsS9DlrW4nMq
vF59sG/P8HTk+jJh9rvKn9ezwK2xqXX06hx8Tvqy8KLVnrKFMrj59kn7IRdy
cQfpyzQT+ekh73PB3mVf4YiJecj1Rc8yMXStch4oVrsGXjHPx2ukLwc/Dp1p
FZwHl/Vc0sJHF+BB0pelrkYVxsPzofXlm/ddXxTgSNKXLjNPK8ks8+Fqo+jU
unUVaEv6cvKHWl/tW/nQcIfCr52mAo1IXw6xc0R/4RyV2XlhlnBeI9i5oBOd
C7ePlkzvcKigd4eY3uERQe8OMb1DUe8WM5zELX/HSUwUcLIPw0lsJ+BkZxXH
Zp0DI1C54b0gF+9gyd45MXuHKwR714vZO9QR7F1/Zu8wQbB3bVYazSzq5oqh
+2bdvuoWCyYZ3pWer8ohm9k7FO3ddWbvULR345i9Q9HePWb2DhcK9s6J2TsU
7d0yZu9QtHczmb1D0d4tZPYOub3TInunzewdcnu3kezdKWbvULR3iTS3WrR3
qsxvgYZ/91ugk+C32DG/BSb/3W8BF8Fv2a0b1iV/aCQUZbW903hwgmTvjv1o
NzM8NBa25xW3z9udKtm7OlvM9zu9TYTOJV9kdlrZkr27l/VjttKTVFiTn7aj
bFUuvCD9rTdQ/aNd50zQHPksaEdynmTv0lcoNg8Kzoau3p+u7sQCaE/6qxa3
+fyCuBxQ+eW8sKFKoWTvTq9cm3l9swxy7qy/tvBzoWTvjO98T20XJYPjDj93
Nt1fBHqkv02PoW1Iw1wo7XoibFh0kWTvVjsOTnqjngsQPfzCxl9Fkr0LY/4n
1LGt6X9OYf4ndBX8z9XM/4QHgv+ZyvxPUBP8z1/2w475VCZDvTyLjU4PngK3
dzZjfjT0HZEFvovl3jP/qdJTsneb2mZNv2aTC0uXdulh/h6B27uVKZtcL2Tk
w4xWX3I3pwdAFtm7Ny5HXgZHFMKWL4nylaveArd3V6eE7p/WrgQ26Xfout0k
BO6RvTOSy1w7eZRCt6Fnf7thKASSvTNfc3mJr28ZjG96KNXKLQy4vWu+7rPX
phVVdvBLwi+7IeGgS/buz/3mlyYFlkPLejMqKo+FQyHZOyM2RxLshTmSzVgc
Ad+EOKIvixdgBsUL3E4ls3gBnChe4PZIviHm8wXtREj0zHIfpxMm2Z2AOw8V
bzzToO1atdu2b95JdsfT6sV6p/k5MLenZuGsLfEYS3LbwnrVgSaT5BCy7uT2
z6VJkt2x7ppj8TU1H9JnPi0f9SNVsjtd71qsdUpVQBN7B6eLizOwLcntwPM6
o0MMikCtZNzCm42yJLtjWdBqhEyjGBzP/QgP65Qt2Z3l90JPDfQpBtsRTmMH
BGdLdsenskfo1BYlYFg2zMCjXQ4eIbnF8UtbRkAJvLuj37vL4BzJ7vTtrJo0
wS4UB+qbZ/+Y+Ap1TZZpLxpeAc/aeOl1bhSD33tfbtkz2R/TWpls2X23HBJ/
1JGv80rA4OcDnu5tE4rh5s31Pj0sg/tpDfYPVE/DHddtT+3fG41P1E7N0RhU
Cj4fozt1Vc7Bjj/HHxsbEIee386Y+zQrhv17Fi5sVFeO3y+utu1jlITlySNm
uekqYPu3HO0yTwU2vVnYqWRgBi5VfRlpq5cDJ4NfF8l7FeGqD/IuU99nYuW3
acuGFqTCuG+7ns5WKcbbVrl2g9pmo/u2dycPDUqAonnp82Y8KMaMrBfd7UOz
caCDTdqpJu/g6RQ9qz/1S7D+5lXZdVVzULmPjfznjRDQu5fU49awEixxDdo+
bUQOpihpz0uT+0OTkUt/X8EYXNPt9UWv7x4wK+FphV1UlfzU84r8djAJvQIS
TfLdn0J588Ep41PKYEHigMrrC8LQacXIbxdOxcGUU8tzfSvLYE/ECY0WPWOw
/7Dnm+3PJkPzWW0C0l1KwfDtlYqtSgmopB55TEkzE2z7/rAbcr8Y4ooMD19T
SsHtAR2eu22VwXj5jSe/fymq/bRWCed8rwt+3f+7VwdNL/uuFvyx///7g0rO
kt9lQn5Xtb+Xc9bXj9kh5Pax2k8rueBrxewWdiL7WP332Zd8fzE7h2lkH6vp
QTa+EcwuYjTZx2q6upVvMvs7rLOP2cd61Qvd59uR/S4eJftY/V3z3b40Xxy5
faymR27zJX8Rtck+Vv++wWbfQWyfyONBRl/vq8f4gq5kH6vpzW19ya/FoYQn
eidK6g0LDETt0z83d/kSiENJ/i0nbRhWfCMMmx4xj2ybF44JWs1VNzuXw/zx
U3uDTQz+erE77dX8WEn+jzrkRt5MTcDsAwPNUyoSJfl33XptV9uLqVh0Ymfb
qT/S0IvkP3T63LOrxmVhnXZfL86ulyPJf+hs5YaymFwcvmCO+/q+ebiM5N/e
Vmnjmy9ytHYBV5W1+fiV5D/g3tM7oU/zsN7nc9Mf6xbgI5L/PfZ/Zt3ql4/L
cjY+UDwrQF2S/8nOV7VDzPLx3/pexuPrKbAlyf+cXeu3Ol3Jx8GFx2z8dBSS
/OufGDRNOzkW1fCIfkR/e0zws/AZ+rIc9A9ahpYsjMVOeamOmccjJTmPV3s7
a3XbJBzQ5v3VSrU4Sc5HLz6p5rwiHetN075bti5JkvM+7Qp32qzLwdJus9Wz
gtJgAsm5s4+ybb0NCqyYP9L2a0EuyGaXbxtQ5Tdc1G/UYV6TQnS43tX82Eo5
2KZaL9IOj4Fks++7e6wvxKuvZ7RxfyiHXbtPVCx2DoM39I6lDr2vxd/RGkvv
SfYX3pP8cWDquJt74nGd7ItvYWg69IjfGTizRwk0DV1mWH4gGcdOctb0vlyF
R69NA48/K4QDjTssa6mbjlu3b39utksOGY//bb90Zz6oua6OaTIjC3/OOrG7
zdN8WJD5s8L1uwxudyl/3Wh6DnofeGEdt0wB8vu7jf50yASVRxZxN1Rk6NQr
s32+dSE4+sZMNS9Ngnh6T1KN3iXj9anb9UZ37yeTocWXUOfou0VwXX1+aaxq
BLxJ7DCsQYtc7JY5d+Wg4iI4MEhh3ONcEMyg94XE9+jK/piZRAQ9hq2/06Ld
09NhJc2lzSf6DqLzebVf+1kr937hBV8L6qT5TpZBZ5pL+4no34nO59VO2zj0
/Md73mBd8jJNzy8PcjUu63zRLpToh4kuI7pX6wHb9Ka8AudL+xT13isgccn5
Zu+/5IEn0W8SPZ7o09Rmjh0+/g1onBnXe9bJYuiYnWmzb48MphK9H9HbE/2s
9cbKcSP8QNX7zagmO0tB3Wrs+rc9MyR6J6L3IPoz4T3MR5SvNvZMNlRe7w+J
dXWy3x0rh9tN3hncUY79H7oL0SsV49TtH/vD4i+P7u9tUAGtHA8u0mkS/j/0
5kTXCijvlJfiDyNtw2Wfp1TADuddaTveBP4PfSvRI1ZdM3FPzIaVG46rTJ9W
5e+SfPZl+gXvSb+4HTnWLrT/LItcyA/O+aIwy4TzEL3n1MV8UP7m+VFJSQ7Z
JhubPorLgsutixw7OebBOGuL0OtV//c4Z725z54cmF3Z1rxCLRfeuHeVbWmU
D1sCPKdq1JfBSHmr7B2qMnDqW3//8wEF8HDFrKvFuTJQkJwPnm24ZoOhAjS+
vE+6558L10nObVaWjg5sWgjaDy+aXdtYFZ9n7Y5FhxgI7zsm3GheIRRP8wvU
c5XDLZJzZU3F5sWmhdC5e9bRqc/kUDw3vWve9DBwONyp0Pl8ISw+/8Bqf5Ic
rEn+Fdmrvh9zqqKfGPhrTY4cGi4LO3OufyAsYngOdX0YnocRnp9geA55hOeP
Cc/vMjyHMsLzJ4TnkQzPoT7heSnheXq3JzPDvmdB8ZW2jqeCcrD7hS/fP20q
gG9WKkXzrGSgazlwnrEsF9ee2fHiYj05dLveMsKymRwSJ2v5Zx3IQ9W8plva
tMqGp2NSZQ5d82BynnPaVqd8rOu37n7z2ymwg97JvEHvZHJ8y2DfRf7drjW/
i0Ppu6vpu93ZdzGJvqtS87vIv/sb2XdP0nuYv+k9zJb0Hua6kDxLb6tc/P76
y67CwEwJD9PeD8UKpzz886m8wy3HHAkPTeidTE8BlxIZ/uAWAZfKib5TwKUS
hj/IcakL4Q/hFf4r4NI4hj94iPBHTvgznegcl3KI/pDhD3JcSiL8eUp0EZcm
MfzBvgIuTSe6JtFViH6a4Q9yXOpD+GNPdBGXvKjeJ+LSZIY/KOKSSL9J9E8M
f1DEpc8CvSnRNRn+IMefnYQ/It2M6ObM74VB5Pc2I38gSD94fv2D8WAzRffW
4V3p8KlH2aJ6Q0tgLPOHoRn5w4fJT2jI7C8Ykv21I3yLZ34y7BT85P3MLsMW
sstZJIf2V5euMChLB7WIjD6ju+RJ+NaZ2Wv4QfZ6EclnszpaL42zssDZTOWy
i26BhG+3mB2H52THCwnf2jE7DtyOOxG+XQndM8D8rgwMZ0VOmbukSMI3suPA
7TjHt7BvLg3DSmXwcvrrOZVVcSbHt9fMvgO37xzf3H0crALb5oL6pKg3FpVF
0Ijw7RWLv6AzxV8cr6xY/AW/KP4qIbx6n1XhqZvxGtNWbekHIz0x0abTq95e
FZDL4mWME+JlfxYvYxshXvZg8TLOEeJloHt8gXS/ZgDVPSmORh5H83rHy9jb
hw7dluOK8yEJkz/6SXPrDrL4GjOE+PrRzaK7WXkFqK6z6+AaDJTm8VHcjTzu
bkdxd2BxkdXim0X4NXv1VKdOwdKcvgEsHkcej1+neHzUhqN7Cx6X4Dq/2e0i
ZCHS/D6K0/GqEKdPLL/y1bV9Ga7MsXnXoUOYNEdvGYvfsZb4HQ0ofj9K8bsx
m4+M+2g+Mp+X58fieoymuJ7no56z+choSfOR+RzA0187TlX7k4ZK9UPOTTPy
htzY5QebZpWC163bLTtF5uDjH+3PjP3uC1GmmgY+v4vhYOqFjmnD8jDMPe+Z
rZk/DNJs9C5vQyH4/qucNX2tAntoHrvYvH8QvNwmyzmvlw8VpZ2VKhoV47Zu
dqqjdgRDblrh1sF+MljxqPThra8laH76/PSHXUNh+ajOXbPuZ4D3auvhyyeU
odzvV6mpQRgYf2rWJ+lUEvg1mm68p2M5Zp6RTdxXGQaWs264LdgUC28Wub5I
uVyOwybkdjFZHQ6d5mh6Hr4ezvNFeJbyRbw+otie8OamvBzHq7r+znQNhyka
D5ekDgyCuiwPiTwPyeXzPstD4lrKQ3I5rM/ykKhNeUgub90XFex9YZmOF32/
NWvsXqVPNq2b3u2XL/mxN8guJJBdoLwl8rwll0P1ETHlziez0PPa9bqQlQ+j
+2Y9fuIu+x//tgPZC8pzIs9zOpF8nmF5TsymPOfBmnlOPEZ5zpE185xYS54T
DYQ8Z4lVPb97BrfAwLbvy0pZDIxesnhfQnI55BF9DNH1iW7A4kToLsSJiSxO
hMEUJ3K7sKbzh4r5a9Kg1K9p1KPiBMkuGLD4ERpR/MjtgtFNH6+xz6Lg/DvF
hkXv3qFx6gFIaV8O2ZA6Cd/HQfeEHjuiJyagu30T08LEUkgzrHu9z+lkiHA3
aBc1PBUbWo7N+/yxGMpzc7y95mTAFvfRN1PmVsXlj0fi/sWFcH/koceWK3Jg
vqbpQp0TMvxJ53sH1/R7PqQKh+vGHdlzQI78vPILHgbUs5NDnbEJht0q8lBr
wO64FccywNGq1Z17O/Ngc+Cjz/0U+Xj5y4qpPbYmwR0b72kXq/znlLgNjutt
CrCv6zuPDnNiwVnhNvz80nwwcWw64aasAHeqqN0fbR4OX95+thx6Lh9i9XPi
Szoo0PRo+YMxbYNYnqTtDl9rytuo/nfeZs4e39+Ut+F1jeo8jJmlbyTlbXhd
o5re2Mo3hfI2vK5RTS886Eu/i7yuUf37GdZS3iaI8jbV9IOHpLyN1n/nbXIO
SXmbDf+dt+lgI+VtXChvc+lCbM9/LINwfo9L62duDkZV28+9lFUqILJZzwV3
uodjmVXHBpXfI5DX35+xeiI+FOqJJqyeiHZCPTGW1RNRrCeOY/VE5PVEjgNb
7jffMKp5Hu7ds9dZH92Q2yOqM+INqjPyOkXLhUoLz5ko8EXEqkFR1veR2yOq
P6JYf9QM/uC1pE8x7nih6JX00F2ak6vP6pLI65LcHmkr2r24Z1CKcw4m5egp
PcFKskdUr0Rer+T1DmXvMb38H5dhyatZLa2Sn+BnskdUx0Rex9SrWcdER6pj
cns0ZcyP74v6VKDPvj8dLz70QG2yR00bzav4ZlGBOpun9GyY4SHNfV5H70nu
Et6TbE/vSb6k9yR70buRf3akLLi8Mw2X/L5p2EE/UcrDUL0eger1vH6xLfl0
nXcF2XikWez7OrdSIV77Xlw3x0Jex8c3Qh0/YXjkgrs5CtT5VP7hQfsH6EP2
K8JQc3aDM8X4WeXF+m09HiG3X9dZPIvuFM/yvM0QFs8ij2d53qYPvUt5RHiX
MoLFs1hE8SzP21B9H38J9X2KZ5HHs/vJ31O/b+mUdy0Rdzo+P+hgGgsc33Ip
PtomxE2LVo20/p2ThnNeaoUPuZ0IHPc+UHz0TcjnRDY4065hvxyMfG4id26c
BhwPjSg+shHyOdbjlozR8sjFXslD/ilRzQKOk+5C3MTzOU5O78xnV8nnH5+b
R6Ne5oA+4acRxUeivVtketdiyPQCvAJq8/t/lYEm4aqdEDf1pviof+jJ9BPz
FdgnKuqFcmQucLz1FOKmhxQ3Fbbwm6L8W4Hd13Tf2muWHDgOTxLiplsUN5k0
+X5m18pCPF4Rm/7HRQ67CJ8/CnFTC4qbqE8GbahPhtdf0g1OKa10KsShRTYG
O+LksI7wXEOIp7ZTPLWQ1RfghlBfULB6BDyjekQZ1SNOh/fvNCUtGRo/nDN/
jUmkZAdbWciUO9/PBIurD1o+dYnFBiQP7ZtOdc5MkkFjv+W76g1NlOygpeec
YSo382BDv59aw4+mIPdzyiaPWnSgrAB29O/+O3FMOvJzVFhmPlNxLgSdc53O
rZiQif3ovFrlp/SKCygCozGXJxjczJLsoMHTnoMvWRRDlEYjH7TMluyg6x2j
eWuLikE5p6OF9Z9s3EF8/nRFr/2WfiXwJqgwab16jmQHS1j8AslC/FLI+gzB
UugztGP8wUZ/5w9aEn+4vqgw/iDnT13ijwXjD9bCH9xO/Bldkz/Yn/jD/YSW
jD9YC3+Q80eD+HOL8QdbEH921eQPcv6sJ/6MWfE6FDUS8cH3ykgtQ0/opp2S
1LJLOTT01Ly69nIi7DTI+IUHLyOnn92GeuNPpUP3wZf3Rxg6I48XJqhvHlPe
UwajLkC8z/ZbyOOF4MyBfe2r5KTMR93Wfd0d5PEC4S1oEt7yeCGS4S1UCHjr
eGR+vazjpaC1rfhG0IAnyOOF69OnvjLLLoM1bb4+PPXxCU6meEFnqeOB5Ofl
sGnHTGO/TR5oQfHCp27KFXH/P7fzXWb3A889kMcL045t+reHbQX8k/ZndpjM
A40pLihlfiByP5Cf7z3mByL3A3/X9AOxK/mBUNMPRO4H6tT0A3EL+YFXavqB
mEp+oEZNPxCXkB+4u6YfiNwP5LhB54iWwjnas3PEPn8/RzT4+zniezrHgXSO
dC6o8/dzwXXCuWizc0Gzv58LJv39XFCFzoXHaxS3QmMhbn3K4lZwp7g1smbc
CrXErVBL3Aq1xK1QS9wKuRS3Tq4Zt0ItcSsMp7hVlfZL8SmI8enJ4AsxmU/j
YO+/ZvW/6QdLcQ31gWD9v/eBIO8DMajZB4LL/t4HgrX0geDmv/eBYC19IFhL
HwiKfSDKrA8EeR/I0Jp9IKhMfSC873EP6y/CEqG/6ATrL8IdQn8R9YVi2N/7
QlFN6Au9yfpC8YjQF0p9nnj0732e+If6PANq9nniqL/3eWII9Xnq1uzzxEfU
58n7XuxY/zBqCf3Diax/GFcK/cOtWP8w8v5hnsfYwvqHUewfXsP6h9GH+od5
HqMP6x/GMdQ/3K5m/zDy/mGeZ9jM+oeR9w8frNk/jLx/mOcZYlj/ME6k/mEp
r5V3//CeLpn4OfPnIbUuPmCzt8HF+FmlMF2gHyL6kpGPx0WVy1DeI1/Jv+IN
tHI701NneDEsF+gtiL7+YczupBn5OGt3d/28lQEA0RajHTYqYKNA1yf6DY+j
6dvmF2JKamV20z9BUHf5sGK1jnngItD/LGN0/j6kB72fM5zuy/R1PTjMe3cp
Pi1T/jrJLhSGBj1J0DyZBhpEf0b0wUT/2qf5m1SHMgzolvTx0ZEwuOWYFtXk
dcL/0J2JXrQz/EFv43J0nXT/9bzu4dDaIGGFSuG7/6E3J7qNare4QVU4XD+7
53H3/eFwe+e49IxpoXCY6A2I7kL0JizvXRVnsry3HuW9K2IuND98LwdPL901
YswHGfL8+UztAwZf1+fi1FLjwt7v5Mjz5HPpXce+9C4fvx+0rpHLsj7H8lDb
dv2YyhYFWF7QL3H80cQq/Flw8VzLfJzkFnZ64/UCnEp58n0d3honL89HZf/y
P2afC5Dnw51TzwRe3pWMfV/HaA+67og+LqF1kwPK4BbRRwl0tbdPe3g+y8Q3
Kzd+yky7gVzeehE9nOhc3sYP8V5uPyUXf/WsN3tKoCtyeZtI9Ca9GJ3Lmy69
P6lO75Xxd05me0R/O96sKu62eLt8rsUTHEbnPofoZUQfRPT4vhmjLwwqxxeH
XoB3dw90pXNPILoP0W8QfW7s2pvHi8qxhdeuFjbHPZDLwzyityQ6l4dJ6ZoH
g+dV4O+vlzvuC/HAO3TuIp3Lw4/U0r79dNLw/DLP4AZZuXAl2tNsrFEBaD0c
YtzfKhO3a9yahaPywcVEv/HD9FzwpHsii6lvlvJM4CLcE6G8FJQzuYJTJFdX
Sa5mMLkCY5IrfZKrs6wfBng/zAfqhwlk/TDA+2HuUz8MyRtokbx9JHkzZ30y
sJz6ZLSpT4bkECaSHM4kOTRm/TPA+2eaUv8MySdw+XxP8ml4bp5iYLIMm5Yf
On+xOAMuE99Mb+9r37xDHt6zqn954rgcuEV8cxf6jXk/1TOi18JP/Er8pP4s
WEV9yA2pD9la6EO+Sn3IwZSvm010J6L719KfzPuvFhHdiOjUXwY6Qt/yFsrv
iXRTotsJ/cy3Ke9Xlrzu1AL9NGi4Q57l1+Ydjt++R3d0j1JYlzqxz7oXKTCn
4agFUx5FoWfApsCzZ0vBm+pEy6lO1I3ycu5CPUhG+bcAoe6jT3m2EVTfMaX6
znfKp00Q6jj/Ut4MZY4OD3+WYWH0ZDu3zDB45fnF4/iwOHBl+APaAv7cIPpQ
gd6b4Q8ECPjTlejBAl5NYvgDdQX8IbyChgI9tdOae0Z++RDX5Fp23Ud3kdu7
ZKInE53bx5BVBareSYWQMiI6KV3nIXJ795bomUTn9pFwD7QF3CN8g1LCN27v
CA+hTMBDwjcQ8Y3wEF4KeEj4Bhz3mhG+zRXorWviHvwifLtRE/ckOreP3laP
9zYrjMcN0cF/NmjHgLJDfC/9mWXwNaXVW43YfDTZplU2/rYM3j68qzQ9KAtm
fyq57FGFC8+erhg68EoupA2ozHJfkQKdsh8EdgpRYJOFbpW+unLo6REWsbQ8
Do4/8F6nBoWY3KRx8aSLctDannXnd70omMb8H/gq+D/kL8EHwV8ypHcL3wp1
1WXML4J8wS9aSvQswY8iPYIlgh75EH2lUIclPwpmC34U+V0wQ/C7SB+ht6CP
j4neV6jb3mR+F6QKfpcz0ZMEP430Gr4Iek11Xqgk+kii29P7h4+E971HMhyA
1YQDPwgHqP4La4X6L/l1wP09bsf7EN2D6EOITnhSZXcYnnwjPKF6MawgOs/P
kx8Ioh8o0rnfSLgECsIlX8IlP6JzvHpJdPIb4Rb5jc1q+pMSnesR+Y1Qh/zG
GzX9Sagr+Jm+XXeXtwhJgLyr5pnfLoXi6WA/q7cuZbDfON9kys54sNHxS/Je
FoL/nmrZ2K2kDHakRU5Vrl/ln3t3jijslYpXyB9YY1Tn9895+diw64D7RdtS
sdnG89pKmlX+56FIR81hCrz25khMPb90HEV+Qqf2xT1KdiswzftOgsX3dFyh
l/92yP2cKr90zcZZPwtxSecG3SPCM9EqRmXLuQdp4Fji6HwnpwitHN5+CM7N
wgryEw4dNTTw+lmEEz5ez9j/Mwv3RlTmb0hJAOit/CTwRDHOzH+ddOl+NvK+
jgPXsj4lXi3GFurPj+u/zMbzbXYt7to0BnYMWVqQ8L4YT9zZc06ndQ6Wkp9w
dsh6lcLvxbgwQCsusl0ODu29Y+/2baGgX+eZ3SmtEgwbarVg8JAcVKK+jspT
FWtufy7HZR6aTU96hcNdE/UXlusD/od+m+ibF6Y717cPg9ctTTUnqwdBoZGt
1vOP5ZBC+J9A+D+a9DSYcD6dcL4O6Rd/j1dLeI93toDzQwQ857jtUgtut6gF
n7n/mT2q7yWrSxVQPLJOqkuJB96ifWUJ9DtEX7NsU1aF1jvcPTkxym3eRXy5
JardXqUKOH31/Ne2v+Pw7rdoJ+9TF9Hha9CTf+zKuR3Ed4IdJLuJqYLdDGL8
wUTBDhLfMFuwm7ROLBLWSftCvn5XolMcCjze5PwX8Yq/9yXiz5CacaWEDzdr
xo/gQnqtXItec/6TXMEikqtbNeUNFhOd72sb01+YTvp7jfR3OtNT4HrK/flT
7H4HrKb7HR/Jnx/P7ncAv9/xgPz5K0xP4QDpKffn6d4HpNO9Dy3y50lPYQbp
6QzS02fsPgjw+yDcnyc9heOkp9yfp3siwO+JJPE+eaanwPW0KenpmB1ldVzX
5mCbjpEeUZYK4Hn+c0IfMq9b9YxfsPNbXxmuSjwbrHyuUKpbiX3IvG610ife
dcUtGaq29g2ZbVgEPG8s9iHzutWpKacXDiqT4a6eWu8bPCiS6lZfhT5kXrc6
FPE2zqZDLq4xqb+wWVGRVJ/SEfqQeX2K6oDQm+qAPE9O9T7g9T6eJ1/I6nrA
63raxB+q30Ffqt/xPLmC1emA1+n4fhexehycoHocz5NTfQ10qb7G6yDV/QZm
e32pXxN5HbO6z2HBfl/q+0Tet1P99+MO+lL/KPK6ZDV9k7Uv9aEiP8dq+pBD
0jsw/Lyq6U8P+VJ/DPJzqabXsfF94/tzyps5FahM/D/B7gsjvy/M+3A0Jjv2
XGOfjwanXGf7qSuQ9zM/Dbi+YHBQMrT8WK9dbP2q9VH9gvqU4Dz1KfFzob4j
8KC+I34uJLfwD8ktr1+QfMJKkk9+LiSH0FGQQ5I34PLGz8WayRWsJbnaQOdi
YzNXp1PIO5yRF67fyS9JqmsX0X0Esa59rOG3H9cD4nG08c1TGsYZUl37M91H
+Cb0/S652vGhdYMc1D2w5XZFkyocJ7/dfszmA8/zc9DgWPMrh4wKJb99gJZS
J/vdMlx3/3G4l0qR5LefUgw80+GdDAN2Wxf9e74INMlvf8b4j5z/vH5KfcUg
1s35expivB/B6u/A6+/8HFfTexo8rj9Ecf0cek/jqnC/eI7wzoZ/Le9s8Lie
03lc36+W9zc2U/wu0tcT/YzwLge/h5Vcy7sc1fI/Y4tvgHAfrZreaptvd6ZX
yM+9Wl1mHfaN/Zv+2pvrUT8TRv73fbSgQ77ZpNeJ/63X+7bpUV8U8nxUNd3X
yleV9L3jf+u72So93gfF8yrV3zXZK+FAn//GgeWT9Gg/GPDf/VGfd0j4wPu3
q/dVR1/qm9L4776pVnOkvqm1xGf29xsl3GhJuFH993o20n033k8yl93zgkF0
zyuR7BfhCXA8aUJ4Uv07Wja+gd7F7b44VSDH+SThvjm/T/eT5QnhEuUJL1G+
y47e4bxD73Ba03tTOix/CDsof3iT8mCkpzCM9NSP9HT6tu5Xj7/OAaM4ve+L
6hQCvwdH+guGpL/JpL+tlzy8tnSJDDwv3d6TWFgI/H7cQKbXsJ70uivp9YxD
F/p+QhnkNhvV2NS8CIaQ/2DH9B0CSd/7kL4njjbdc+27DFyeP/1UEFUELch/
aEzveZrSe57a9E6U37Cv3QM758KLkatsvf4UAb9nJ/b3DqJ+KtGfdyP/Suzv
bUF9Vr1ZXxCIfUEvWR4DNlMeoznlMXLpXc129K4mP5e36Y3ueMyRw72HR+Y5
Ds2GphRnySb2/rjBMB9a6mtu7aghAx5PUT4EeD6En5fxPFc795ACeOrUbaL2
0lzg93bVhnw9pOFSAEmX7H1yx+cCj7/msPwJ8PxJEp3jh21qfVVdFFAwPXWz
byu5dI7KTTzsTU8o4P3cay4r68qBx2WqLN8CzSjf0o3Od3OH7m2z+1bhxMOu
V5payaXz7aH+quUZ1UJIurV1b9AuOfB4jfIzkEL5GX7uP8vqt1fZWwjTbq5z
feMnl849/MXsr+5mheCS7Vhq/EoOPI67Tu9zdqH3Obk89FgY4hF+pxA+f3jt
0EAhl+Th5M4+lhUuhRAc3jEhSy6H50M7v+h4OAAyWV8rdhP6WlNYXyuGC32t
FN+hGN8pU1/iI+pL5HkYyu9hqhD39aO+xI3Ul8jzKmJcw+PNAnbfCvl9K44n
Pdl+8RPtl/vPd6/G/Y7xkkOPp19a73RIRqm+oL1p28/1eXClWD+rCaRgOt3j
GF3++PKoAQXQ3+qR18Ljacjz59+11rX45lcA6rsVo061SMf5dI9jwatWXqhS
CPUne295G5KBwSSfT39qOjY9UAiT1szvlq6eibnU13e//Gukq2kRXGketP/A
nCzM4HK4eEvdHf/vxyzN2196Lgt5v9+s5rLx/Q2KISbw0oAGs7OxF8lbu/WV
hhZBxRBk2Xne7PJs1Cb5WZMTtudIZjHstW6t5VkvB69RH2D73rkl87qXwOJl
buPDNXOQ9/vdY/zBXsQfB+KPJuMPOhB/+H2rVXLvhGat8jB1Q7/jbUqT8drT
Fy7bs6v8O8Y3HEh8u0F8+5fxDXsT3/j9l326Z1+usShAW5eVe1dEpWG0c0PF
rg25nJ/YkPgZQPz0YvxEI+JnPvGzdOmnxYOnFGLn8ACH8G8ZOCco0k+nJBMe
MD6jI/E5hfj8nvEZOZ/5PZq4rUarf5wowlbJWk/zd2bhsS0LnNxdkjn/MZb4
z/X9GL3vuo3ed+V9mHqN9FtrzSvGlC3xxVqm2WjW5UyW3gzpvPAtnVdfOq/V
7LzQgs7rJp3XCnondhi9E8v1WoWdI/JzPEjnuIn1D4M59Q93pzwq9RXDdqGv
mPQUnpOe5pCeUl8xeBA9i+ikp7CV9FSP9FSL6Fx/ef5Ti/UPw2zqH+b1Duor
hllCX3Fz1j8MpdQ/zOsgnF4s9BX/6NXQcPeVcug/WX1+OxMPfE35xp9E1yE6
z0NSXzG8pL7iIWQHOb2WfmPQpn5jbgdFehOiE/9xi8B/4gPOFPhA+8Ii2te3
mvtCbWFfvC/6Ba1zsNAXLa6T7isBr0/l1KxPSXls3gfO89JrKP/8jdY5Ucgn
V9I6/YQ8MK9Pkb8B3N/g6xTp/J4R+RtgTv6GMq1fpHM+6ww8cWfloBKcYR01
IGdoDnL7Jebxbgt5JJ5f4nmw1SzfC2K+l/K6kCrkdeexvC6IeV0blr8Fnr+1
ID+B8rSgLORpKR8LYj6W9gXTaV/etC/yS5H7pZ/IzznQ9ryj21AZ1jGRTRzs
Wwg8T/UP80uR+6UPyZ+5zO4JIr8nyPNUM5lfitwv7U9+C90TRH5PsILyVEnM
L0XulzYj/+QRuyeI/J4gt8vkl6IP+aXJZJerw4telnqUB0F+H5a9s7FLj+7t
Y/x/x1PdNupRHznye/csTumox+OdexTvVMdNflN86R4R3vzvfMjBBb5IcQ2/
f9pN+9j/FXbd8Vi3379oadCQpyQNDQ1PiTTIKQ1StI0yGkppUPQkRENKaJoN
KelpioyKm+u2520VKsq4kZH71vC0+Xn1Odf9PF2vb6/fv5/X5/W5zrnO+Vxn
XOe8T1DWNAG5YbRhxuZhpRI/9ufnc110sP+EuP83fmy0lcQvtF/VUyOox1Wp
BvB5XZ0in1ED69AeuR7xN7VReQNR7WafZ0XWArUvmrwBCk4Vb+BN7KnyYj0h
UHthd3OUpuGyRvCNDtZSuiKEu8YD1XV654OffPa1OX6NkPDoUoJpiRBOupZ+
nb40A2TR74phznPa98Ge2+gHkgz0A6m+oX0H1r6jvQZPxl63cvYXRqD9pfyi
PQU5xp6ifYRyxj7e8iQXrhU3w/DPvew3f60i95DfuXImmjsVW8AicN/qQNVq
Qvn9m3ufKOL7d359n1gy7+9GO7gfz2d6/4j7BjFMHw3uG+xm+mVY+/WZsV+0
/4Wek98ZO5XI2CNqd6b/xu70w3OPPT+n/+b8pO/L8tf0MX6YShx161SED+LI
ZcENvtUpMazk5i4RV5y7ROcrdXfr5XRgVxoJNwi/J+wXAS0GQYqjL4phbbzz
UNmQHDKAv+Vr8s27MCk0c85KYzEYRbQ92Z0TQ+JvCzWHG/EhhGfoE3hDDMZO
8i8txyYQmy/NP4YdyoDrM9/lrNwohgnqeZVWU1NIz+UZA4aNyocdxevvNLWI
QGpzVd+4yArSd2fQZ7cu+ejka7VsJ3Uw7WifyoobV2BPZ47lxZIUkPtsvz/x
nhgWzgrcfco1HIp6dljUnMgCu0FDgs33iME1d7Gm2SEerHunfn+edywIdOV6
+T4Ug/iYoa98QSpEK58brqiaCGvWudzd6CyG7dkuG0EuBdxXRYRM7X+aTF7o
8bFXvBiuvzoXfdMnE/Zdkd4yJ+kCKXYwD9voLga/SKXsfhECUCHCXPPIEDKo
34E+V5XFsOZLzo7OhHjIURsZaWb5hOgGFx6b+qiLnssKGuqyySA2T35RlsIn
/LK+cWaHxaDPsylqyEwDi7MVphd7ZZL6lkHeb8eI4dvhY6/m9Esmxefezw1Z
kUw0jK3cLPzEcI+bb0u+9eTm29I4Wl8w5VyWcjrp/X3nDa0e2wmV145oh3Sv
o7kk+mug8IKLA6HyqjDjefNG8MizSJMI28wnEnnFBuaf7euXSvonnFlr+I0v
kdeyadlpBWbZRHGZT55YI1Mir7LTKvdX19WSkyEaat/SXsM8lNejVNddTS11
RH4Q333Oj2podFbwiJtfA93LLF4272wgU1cHEt0xtbC6lB8+J6UCmswtXG7z
vcjnoaMsReNTJfKNVJHJULM+TbTDglzMP/4r308DG6Rrrf2J4YsYa4/dhdC8
xGbRWWkxOM2Rtqq7EgeBQvtTT+4nSuQ+R964Q8WLB5s+uvolGqZK5G59y+nY
l4epcFtXR19BLhvmr6iUdZcXgyZf79u1N3zopjdlollcEFB9GF1vlaxnkgk+
npe6Be+9ClQfxh/NjN1qIYAzTlOGpgtuANWHMsuQQ55KaXDg861Fg7IfEYJy
X+G5NCxtXA4c87Pb/1iXSOR+dYys1uuYUtLi6ai2eFsuUb15fvwY61a4bD5i
c6ZBBYkMVu7483QREWZ0VyzitYAV4u08QFwLE8S1eMLc41BcdA2DNmX/g13n
tpqKa86UWvDHOknB9VR9J4c3kGh+5O3DtlqgdZKqXH8ZeGJ/2Q2Ma+y+fMu/
GN3lt/kuqRmsISDLTistF+5B/NWiYzpTGPzVLGVld5sJxWBwdHz+O14JmYR8
fZjUHLb+fCl81SpwkTleTuqQr5/f0T+l44l9pojjBfs4HBK4gjgkLohDUhia
1mqTVwfWSYOEUhYNZJ32Z9m945/C5H03gufNroero2WE7/IbyKF5jY861wso
rgh0Iq7IIMQV2cXxRTQZvrI5+slypJ/KpZ2jn/xA+qlckB6yEelZ8ys9JAzp
OYj06CE+CSvHaOwj02Lk6PJo2ieDtYXgUZmasrqjFISWJ+fNlG6FB9z8BRDi
/AVaB3uI8w+hO/qHVL4lHO4TUNyncJRvKKeH0Ih6SOV1kdNDuId6SOWVxOFf
gRriX1G5jJyvpde3voiUjXtR2nIhXbKfIo0teelnXpDST7lrlP6OBbW3B6KH
z2mlcQRpYupGXpe55nl/eErcngcVeg6Igu9B3s0Cky47N79j/gqfbNL7dO0u
Tc1ieDh+4FRvTRG8Uq5UHdPlz24UZdk1qeVCDD63LdaSmqr/DAwn7499cuoS
od/ZxPWNgiP2jZZh3+jB1Affnb6/AJ9J507Zu1wjlE55rp8UYrGfdDz2k6J9
J9OYeNOOW5eYMOvWcHSS7Ugnpd+DW5ecx3Wn4rqOHL8gzfDbzO0nlOB+/onv
V3L7Bi7MvilycoGnjFwGcX4CHEQ/4RL6CVgPQO5hPQCtx9it9Ub39dkq8mmI
fciJyw1giThyWkx9Mq0foHW57VhHSvH6tjL47V543xGJ89x/9Px1nnsuh18H
cxC/jq67FuezT8A68GDU/8tYv/qRWdeKwdGlOIG0XlQW75XC8F5pHuKKJGH9
2wysf6P1KtMZfuULDLzUOp6TBLcjuqXLnsG+nlGT2v5ugTacU7wY5xTTORFX
cI7wSGa+EuKZQCozl/wc1mnEMXUakTintT8zp3U1ztmk/p477qcjh4cJ/RAP
0wL3cynOzTyCczMv4n72xnmC9N6B0rkE8fy9EN8Y+/sl93R0P6/9Fy/xjrsO
yoM8/y/eY7ctkvtrii+E+gbtqG8UtzBN1rO8ybnrPK+vcJIyqpE8x7wc0Lxc
KJ5vmE8Dmk+j9Efi/JpaPD/pfBYuTrTVwXtBQvGRTNefzvBeU0Ls5t/cH+78
UiLfn68/OMIb9b/uE7vVaVP8AcRNhwlJo0I6krLI4cyAMC/vBDI0xW5n6yIx
NKf3v7KqQ0Cs+rs7ONTzScaJ6oLJJV1+snZb3PLCEqJpZm+20CSTGF9faxUy
WASVGn/5aKqUkx33t9gkPMojE96pHpTr2Qq1Ku+r0+UriY5vwZ6GliKSkJtU
e9C15bd5y2u8p44rtIvIaJfX+771jYYXvlYzLWtEsAD7Oh8wfZ2TJkV4d+i/
JBEfbY5HtcdBVEXIAqP+rXA4U9t+SD8RyR1SHzO4OQfGKvY8M/ppMRSOm93p
9S6blPTPUWyoLgaXhKeXpyhw/kPbiHM8fVdD//TODPIiVnWzoheHL8pvv8Cb
lZIsHz4gn8Qf2PVH6WzEHQ0K5N18cOj6l5RiIq29fKV5Mvohuxx4xwSmK4yh
jOgP3f52dHd8ftlBJ9juUZXR9WoiNJbLXGjSApnmiYr5Cmkk6hbJTbVOk+x/
f/CaV9SSReSdFdTX78ghK3sGiMNyRND9yITbOjUCsqDjerBje6Fk/6OW2Xf/
aFpC/lQ+VxiU8VSy/8/GTj4io1NGNjSXqfYc9lyy/zefqge0iupI7UmH3kUu
DSSB9/2CX1sxdBok1f2jV0+m2LZnzCtrILxOWcsltvlQWR2xM0y/mPg++RQV
6XyO9HnyfeXxAklfLTnK9GN2+rjYP0p/SU53G7+UdyGMULkcj928VfVW13+l
D5E7baIJlcuTqNACp89FpI/rddNvFnkSucwqULrWRzadWK/uzLOvD4eyRPW7
Z/zFYILxoAwTDy7WG2FvvyKebJgSkM6r5kNSrtbI3iFi2Cf/+ZVVRiLp9cE8
VOdVBqRUDX150kQMqhgP9mLiQesrAo1pH4Mh68CJ+VeL06DR8U1OcldcI1X8
NCJ8/zUwcNjLM9LLgQlu75cMWC+mOG8Q9f/4w3QeUPBfNtm2bgS+xEUZB2g8
gR1KYYLYLjpDLuQ+yFuYDv7+Xn6HHhLQfm3vXmrWFX/huSrDnKuPuX0Ddt9m
oL99DP1tep6wdUdBWN9S1bnhjodbOgwf9biPeJwXiRvzaXdTgBis9EZ+WqeY
C1ucG/yjl54hh9eI3lStFsMNhSONN44XweQm1du2WQEkXe1xxtZGEbwX97Lx
y38LCnD3fIhJFBmxqHxbVEiFpK/ZDvtnDbCf9N7GwU2vuiXBpoxNQxXzk8kt
8abh1mfEcHDE04FxdqmQr+y7bnV0Ghmx4ayFuoEYXgaf/7BmbhYc7CVzq006
h2QMKZ5iVC6S4M/QvmPKF41ny5h49nFlHs80Ip0slF0t8nJzIVSvdmLc2sjE
rZeNlP/2fJlMDCdpETu1BIlexQ85lC23N42k7R1QIzcyRaJXxr+JW+Wmp0a6
mPoT6duNfW83/KtXw8YHLh0uDiQzfXzhi/O/elXd4+qX4uUhRMbmmG38jCIo
v9T6Ivq9CJai387qW9Rv8B9Cjh4Ij5z+CDZ5lcXs8OBL9G1Bt+I8KblkCIvd
O1ejJl2ib/3rNjXark+Ds296fHu1IBcsDf+I8n4nggLu/AT2/HTDuOAoxgW1
TFxA7Rr1i8ZhnRVbB/im5+xpl2XT4fW7eqVC/2tA9XCY424r2cs54Ga9OOz9
7QigehinrjLHR7cIFlztL/P94B2gejg1++MrZ+1SqDd7edxo00O4O7Ojd/hw
EdyQ6WX9Y/Zb+JjcIehrlg1KqJ+035nidC1F/RyIfukBxi9NgsT1vtOywHu1
oHyVV6JEP48lt717YSgAqQ+xhVYnUiT6ibgHsAxxDwKR37GrHBYuuFpJFgsm
TvsnsA6y1t6yPFsjhJmtctl1s6uIgdmtzugdDZD6cfNEmXvVdD+JkPETXvkM
a9eqi4TJO/t4JCtUSOaQ1m+qOmnKfwjFCe91HfZWS+Y4Dxs5NW+MZjxMqeR/
sXcWwgycm5wemm/5zCIFtOoUt210FEnmcW8daF7LD0uBpBETPA83i2AbztHO
zKs69DgvBRRMAuIrZopBG+cFD2k5H7Btw2sYfalZodH8OUzVdWrfa9sETf+k
HvFwFxK7O981HpyqkvArFf80z2FCPUnd2xr6fmwNDJ3m8vZOSTXkcnwRlq9K
ji9SxPA1hOOLUL7oPOg0ji9C+aLzyjdxfBHKF50zns7xRVi+1rRVN9qLSkCg
6T3xgeAluI5YEjj7YAvcddwukDIrg56iRf0HvaqS8Jt/yrzE6HkNqAW1t/iF
NsEERWP5UXL5IId5UScmL4rxLGHjWTYOon0c7P0s9Sfvc/SQXgw9X41e2s0U
hUPesk3Hx7sUA85vh916my/dvfMChNkBqTP3PZPwRetnFLB+huZpDRCX7DyD
SxbN+SEwjfFDyjk/BCwYP8QJ8yqhTF4lgvNPgPonj3/1T4D6J4non7B5FYrX
inkbwn6fxokqTJ0/i/s6GL/D5g9XYv5w+m9weGg92C2sB6P7hn4y2DF+spDz
k0Gb8ZOTf5PfoPGFAzPXtQfinLj8b38MTjD+2FDMJzxk8gkXB6S5q4bWkj5l
Y/Ys+y4k9PxBfxv+Zvzt+RgnZjL9WacQ58GZwXmIMd9wao+CkHi2tydU/8WH
2h6pY4atb4IG55bWigIRcQzJ7/HhfC6kzTasMZ+aCYvqPuwcMrgJBKOzNbT2
3SP+Pc6rHCkQUvpJDEN//faLG7RdhMT7W/usgOE3Cf2+EV/dt81eTKqWKBek
lkaTVPz+9IVBPULPNYL02Cyr2eoZQL+Pz4kU8zyCsyPkN3aEsHbkPubPv2M+
geoD7R+fyOQNKI4Ki6eH/hWh/pXSr/4V2Y3+FV238OK96JMePMjzW7TN9Ucj
aPOvVpXcEYJ8QaW6ck0y7B5avKq3fAtIfZtx4celakqnJO9B6Qzh9AFYfaB5
D0o/ndeZy61L6Lpzcd1B3LpkF64rjevq/bD1XbqWTw677QjeGf4WcL496DJ5
D3Um7zGD+X+3Ig6bI4PD5vbJMcQ04QUR+pOqd+nP4I/nbz5Ib2uB95gPWYL5
EDpPk8U3u47/l4X6lb3Nho0k+JX09teXu/jG8xzr34gykz95yOk5HEU9F6Ie
0j6XGCZ/gnEr5GDcqoLx0Sq8L3Nj7sto/9dzpk6A4ixRf4P6w/gfkVL8jwJR
n9n8DP2+EtYhsN//mZcI3MtD+0qoXv2Mc/ts5B3n7DGhevXz+cBVOuWc/SZU
viz+vwRHjrObhNpNus9s/ofOJ13I6Q+w+vOTzrodOqh/hPo5P587m/HQ/pF4
9Ad+Pi/azFML8hMfmSsm1B+YxfljwPpjbB6b0s/Wx1J9YOtg6f2pDeaB9zN5
4N/Vo7L3I1Q/aR2mEqOHDdx5CD7MeYjxOIRjPD4G9Q3PSahmzkkWF5HSj+c2
7GPO7WWYxzuKebygX/1/SR8Q1U+unvygpG6cxssLcJ7Fn8w8C5N/OvtYNZSQ
AUlukYfHVEj+65/6VurIc+H0QDIX/ufz+Zo8D6/t48K8xEQH/To2L0f16v8A
Kf/7zA==
    "]],
  Axes->True,
  BoxRatios->{1, 1, 0.4},
  Method->{"RotationControl" -> "Globe"},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.442754838492625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
The real part of the logarithm function does not depend on which number of \
the set we pick. Therefore we can draw a graph of the real part of the \
logarithm function.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ParametricPlot3D[{r Cos[t], r Sin[t], Log[r]},
                 {t, -Pi, Pi},{r, .01, 3}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx9nXlcTd8X95uUMlPGjJlnChF3XZllJnxJmVJEmZIQkiGEzJooJYqIDJna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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFmwn4VtMWxr99hm3ImClDGikZ4hoyDxkyXhQypzSJpFKkUaXSSKNULkmG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         "]], Polygon3DBox[CompressedData["
1:eJwtm3f8T9Ufx88d58pIRrRlRFOSlrathVC2jNCSSovSIu3QokWppGUko0Ib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         "]], Polygon3DBox[CompressedData["
1:eJwt1neYVdUZxeE7c+eCdIbeGUAhVlAiGBGIBSSxoCAmCIJgKAIDSJkQqdKD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         "]]}]}, {}, {}, {}}, {{}, {}, {}, {
      Line3DBox[{233, 232, 1600, 504, 258, 1609, 524, 272, 1615, 538, 1669, 
       286, 552, 1672, 300, 464, 1498, 314, 478, 1512, 328, 578, 724, 1621, 
       592, 738, 1633, 606, 752, 1643, 620, 766, 1651, 634, 1701, 780, 648, 
       1709, 794, 662, 1718, 343, 450}], 
      Line3DBox[{505, 816, 965, 817, 1601, 506, 837, 977, 838, 1610, 525, 855,
        989, 856, 539, 873, 1734, 1001, 874, 553, 1673, 891, 1013, 892, 1051, 
       1770, 909, 1025, 910, 1057, 1773, 927, 1037, 928, 579, 725, 1622, 593, 
       739, 1634, 607, 753, 1644, 621, 767, 1652, 635, 781, 1659, 649, 1710, 
       795, 663, 1719, 945, 946, 1046}], 
      Line3DBox[{507, 818, 966, 819, 1602, 508, 839, 978, 1750, 840, 526, 857,
        990, 1754, 858, 540, 875, 1735, 1002, 876, 554, 893, 1740, 1014, 894, 
       1052, 1771, 911, 1026, 912, 1058, 1774, 929, 1038, 930, 580, 726, 1623,
        594, 740, 1635, 608, 754, 1645, 622, 768, 1653, 636, 782, 1660, 650, 
       796, 1665, 664, 1720, 947, 948, 1047}], 
      Line3DBox[{509, 820, 967, 821, 510, 841, 979, 1751, 842, 527, 859, 991, 
       1755, 860, 541, 877, 1736, 1003, 878, 555, 895, 1741, 1015, 896, 1053, 
       913, 1744, 1027, 914, 1059, 1775, 931, 1039, 932, 581, 727, 1624, 595, 
       741, 1636, 609, 755, 1646, 623, 769, 1654, 637, 783, 1661, 651, 797, 
       1666, 665, 949, 1747, 950, 1048}], 
      Line3DBox[{8, 362, 1462, 23, 375, 1475, 38, 388, 1488, 53, 1075, 401, 
       68, 1090, 414, 83, 1105, 427, 98, 1120, 440, 113, 1135, 128, 1150, 143,
        1165, 158, 1180, 173, 1195, 188, 1210, 203, 1224, 218}], 
      Line3DBox[{247, 246, 1605, 514, 265, 1612, 531, 279, 1616, 545, 1670, 
       293, 559, 1675, 307, 471, 1505, 321, 485, 1519, 335, 585, 731, 1628, 
       599, 745, 1639, 613, 759, 1647, 627, 773, 1655, 641, 1705, 787, 655, 
       1714, 801, 669, 1723, 350, 457}], 
      Line3DBox[{515, 828, 971, 829, 1606, 516, 846, 983, 847, 1613, 532, 864,
        995, 865, 1617, 546, 882, 1737, 1007, 883, 560, 1676, 900, 1019, 901, 
       1054, 1772, 918, 1031, 919, 1060, 1776, 936, 1043, 937, 586, 732, 1629,
        600, 746, 1640, 614, 760, 1648, 628, 774, 1656, 642, 788, 1662, 656, 
       1715, 802, 670, 1724, 954, 955, 1049}], 
      Line3DBox[{517, 830, 972, 831, 1607, 518, 848, 984, 1752, 849, 533, 866,
        996, 1756, 867, 547, 884, 1738, 1008, 885, 561, 902, 1742, 1020, 903, 
       1055, 920, 1032, 921, 1061, 1777, 938, 1044, 939, 587, 733, 1630, 601, 
       747, 1641, 615, 761, 1649, 629, 775, 1657, 643, 789, 1663, 657, 803, 
       1667, 671, 1725, 956, 957, 1050}], 
      Line3DBox[{519, 832, 973, 833, 520, 850, 985, 1753, 851, 534, 868, 997, 
       1757, 869, 548, 886, 1739, 1009, 887, 562, 904, 1743, 1021, 905, 1056, 
       922, 1745, 1033, 923, 1062, 1778, 940, 1045, 941, 588, 734, 1631, 602, 
       748, 1642, 616, 762, 1650, 630, 776, 1658, 644, 790, 1664, 658, 804, 
       1668, 672, 958, 1749, 959, 960}], 
      Line3DBox[{673, 805, 1746, 942, 659, 791, 1706, 645, 777, 1698, 631, 
       763, 1692, 617, 749, 1688, 603, 735, 1686, 589, 1618, 721, 575, 715, 
       1034, 924, 1680, 569, 709, 1022, 1768, 906, 563, 703, 1010, 1764, 888, 
       549, 697, 998, 1758, 870, 535, 691, 1730, 986, 852, 521, 685, 1726, 
       974, 834, 501, 679, 962, 811, 961}], 
      Line3DBox[{674, 806, 943, 1716, 660, 792, 1707, 646, 778, 1699, 632, 
       764, 1693, 618, 750, 1689, 604, 736, 590, 1619, 722, 576, 716, 1035, 
       925, 1681, 570, 710, 1023, 907, 564, 704, 1011, 1765, 889, 550, 698, 
       999, 1759, 871, 536, 692, 1731, 987, 853, 522, 686, 1727, 975, 835, 
       502, 1598, 680, 963, 812, 813}], 
      Line3DBox[{675, 807, 944, 1717, 661, 793, 1708, 647, 779, 1700, 633, 
       765, 1694, 619, 751, 605, 1632, 737, 591, 1620, 723, 577, 717, 1036, 
       926, 1682, 571, 711, 1024, 908, 1677, 565, 705, 1012, 890, 1671, 551, 
       699, 1000, 1760, 872, 537, 1614, 693, 988, 854, 523, 1608, 687, 976, 
       836, 503, 1599, 681, 964, 814, 815}], 
      Line3DBox[{676, 808, 1748, 951, 666, 798, 1711, 652, 784, 1702, 638, 
       770, 1695, 624, 756, 1690, 610, 742, 1687, 596, 1625, 728, 582, 718, 
       1040, 933, 1683, 572, 712, 1028, 1769, 915, 566, 706, 1016, 1766, 897, 
       556, 700, 1004, 1761, 879, 542, 694, 1732, 992, 861, 528, 688, 1728, 
       980, 843, 511, 682, 968, 822, 823}], 
      Line3DBox[{677, 809, 952, 1721, 667, 799, 1712, 653, 785, 1703, 639, 
       771, 1696, 625, 757, 1691, 611, 1637, 743, 597, 1626, 729, 583, 719, 
       1041, 934, 1684, 573, 713, 1029, 916, 1678, 567, 707, 1017, 1767, 898, 
       557, 701, 1005, 1762, 880, 543, 695, 1733, 993, 862, 529, 689, 1729, 
       981, 844, 512, 1603, 683, 969, 824, 825}], 
      Line3DBox[{678, 810, 953, 1722, 668, 800, 1713, 654, 786, 1704, 640, 
       772, 1697, 626, 758, 612, 1638, 744, 598, 1627, 730, 584, 720, 1042, 
       935, 1685, 574, 714, 1030, 917, 1679, 568, 708, 1018, 899, 1674, 558, 
       702, 1006, 1763, 881, 544, 696, 994, 863, 530, 1611, 690, 982, 845, 
       513, 1604, 684, 970, 826, 827}]}, {
      Line3DBox[{1063, 1314, 679, 1456, 1315, 1598, 1232, 1457, 1316, 1599, 
       1233, 1458, 1317, 1600, 1234, 1459, 1561, 1601, 1235, 1460, 1562, 1602,
        1236, 1461, 821, 1237, 1462, 1318, 682, 1463, 1319, 1603, 1238, 1464, 
       1320, 1604, 1239, 1465, 1321, 1605, 1240, 1466, 1563, 1606, 1241, 1467,
        1564, 1607, 1242, 1468, 833, 1243, 1322}], 
      Line3DBox[{1064, 1323, 1523, 1726, 1469, 1324, 1524, 1727, 1470, 1325, 
       1608, 1244, 1471, 1326, 1609, 1245, 1472, 1565, 1610, 1246, 1473, 1750,
        1327, 1247, 1474, 1751, 1328, 1248, 1475, 1329, 1525, 1728, 1476, 
       1330, 1526, 1729, 1477, 1331, 1611, 1249, 1478, 1332, 1612, 1250, 1479,
        1566, 1613, 1251, 1480, 1752, 1333, 1252, 1481, 1753, 1334, 1253, 
       1065}], Line3DBox[{1066, 1335, 1527, 1730, 1482, 1336, 1528, 1731, 
       1483, 1337, 1614, 1254, 1484, 1338, 1615, 1255, 1485, 856, 1256, 1486, 
       1754, 1339, 1257, 1487, 1755, 1340, 1258, 1488, 1341, 1529, 1732, 1489,
        1342, 1530, 1733, 1490, 1343, 696, 1491, 1344, 1616, 1259, 1492, 1567,
        1617, 1260, 1493, 1756, 1345, 1261, 1494, 1757, 1346, 1262, 1067}], 
      Line3DBox[{1069, 1347, 1348, 1758, 1068, 1349, 1350, 1759, 1070, 1351, 
       1352, 1760, 1071, 1353, 1669, 1354, 1072, 1734, 1355, 1356, 1073, 1735,
        1357, 1358, 1074, 1736, 1359, 1360, 1075, 1361, 1362, 1761, 1076, 
       1363, 1364, 1762, 1077, 1365, 1366, 1763, 1078, 1367, 1670, 1368, 1079,
        1737, 1369, 1370, 1080, 1738, 1371, 1372, 1081, 1739, 1373, 1374, 
       1082}], Line3DBox[{1084, 1375, 1376, 1764, 1083, 1377, 1378, 1765, 
       1085, 1379, 1671, 1568, 1086, 1380, 1672, 1381, 1087, 1569, 1673, 1382,
        1088, 1740, 1383, 1384, 1089, 1741, 1385, 1386, 1090, 1387, 1388, 
       1766, 1091, 1389, 1390, 1767, 1092, 1391, 1674, 1570, 1093, 1392, 1675,
        1393, 1094, 1571, 1676, 1394, 1095, 1742, 1395, 1396, 1096, 1743, 
       1397, 1398, 1097}], 
      Line3DBox[{1099, 1399, 1495, 1400, 1768, 1098, 1401, 1496, 907, 1100, 
       1402, 1497, 1677, 1572, 1101, 1403, 1498, 1404, 1102, 1573, 1770, 1499,
        1405, 1103, 1574, 1771, 1500, 1406, 1104, 1744, 1407, 1501, 1408, 
       1105, 1409, 1502, 1410, 1769, 1106, 1411, 1503, 1678, 1575, 1107, 1412,
        1504, 1679, 1576, 1108, 1413, 1505, 1414, 1109, 1577, 1772, 1506, 
       1415, 1110, 920, 1507, 1416, 1111, 1745, 1417, 1508, 1418, 1112}], 
      Line3DBox[{106, 575, 107, 576, 108, 577, 109, 578, 110, 579, 111, 580, 
       112, 581, 113, 582, 114, 583, 115, 584, 116, 585, 117, 586, 118, 587, 
       119, 588, 120}], 
      Line3DBox[{1114, 1419, 1509, 1680, 1578, 1113, 1420, 1510, 1681, 1579, 
       1115, 1421, 1511, 1682, 1580, 1116, 1422, 1512, 1423, 1117, 1581, 1773,
        1513, 1424, 1118, 1582, 1774, 1514, 1425, 1119, 1583, 1775, 1515, 
       1426, 1120, 1427, 1516, 1683, 1584, 1121, 1428, 1517, 1684, 1585, 1122,
        1429, 1518, 1685, 1586, 1123, 1430, 1519, 1431, 1124, 1587, 1776, 
       1520, 1432, 1125, 1588, 1777, 1521, 1433, 1126, 1589, 1778, 1522, 1434,
        1127}], 
      Line3DBox[{1129, 1618, 1263, 1128, 1619, 1264, 1130, 1620, 1265, 1131, 
       1621, 1266, 1132, 1622, 1267, 1133, 1623, 1268, 1134, 1624, 1269, 1135,
        1625, 1270, 1136, 1626, 1271, 1137, 1627, 1272, 1138, 1628, 1273, 
       1139, 1629, 1274, 1140, 1630, 1275, 1141, 1631, 1276, 1142}], 
      Line3DBox[{1144, 1531, 1686, 1143, 736, 1145, 1632, 1277, 1146, 1633, 
       1278, 1147, 1634, 1279, 1148, 1635, 1280, 1149, 1636, 1281, 1150, 1532,
        1687, 1151, 1637, 1282, 1152, 1638, 1283, 1153, 1639, 1284, 1154, 
       1640, 1285, 1155, 1641, 1286, 1156, 1642, 1287, 1157}], 
      Line3DBox[{1159, 1533, 1688, 1158, 1534, 1689, 1160, 751, 1161, 1643, 
       1288, 1162, 1644, 1289, 1163, 1645, 1290, 1164, 1646, 1291, 1165, 1535,
        1690, 1166, 1536, 1691, 1167, 758, 1168, 1647, 1292, 1169, 1648, 1293,
        1170, 1649, 1294, 1171, 1650, 1295, 1172}], 
      Line3DBox[{1174, 1537, 1692, 1173, 1538, 1693, 1175, 1539, 1694, 1176, 
       1651, 1296, 1177, 1652, 1297, 1178, 1653, 1298, 1179, 1654, 1299, 1180,
        1540, 1695, 1181, 1541, 1696, 1182, 1542, 1697, 1183, 1655, 1300, 
       1184, 1656, 1301, 1185, 1657, 1302, 1186, 1658, 1303, 1187}], 
      Line3DBox[{1189, 1543, 1698, 1188, 1544, 1699, 1190, 1545, 1700, 1191, 
       1546, 1701, 1192, 1659, 1304, 1193, 1660, 1305, 1194, 1661, 1306, 1195,
        1547, 1702, 1196, 1548, 1703, 1197, 1549, 1704, 1198, 1550, 1705, 
       1199, 1662, 1307, 1200, 1663, 1308, 1201, 1664, 1309, 1202}], 
      Line3DBox[{1204, 1551, 1706, 1203, 1552, 1707, 1205, 1553, 1708, 1206, 
       1554, 1709, 1207, 1555, 1710, 1208, 1665, 1310, 1209, 1666, 1311, 1210,
        1556, 1711, 1211, 1557, 1712, 1212, 1558, 1713, 1213, 1559, 1714, 
       1214, 1560, 1715, 1215, 1667, 1312, 1216, 1668, 1313, 1217}], 
      Line3DBox[{1231, 1455, 1454, 1749, 1230, 1453, 1725, 1597, 1229, 1452, 
       1724, 1596, 1228, 1451, 1723, 1450, 1227, 1595, 1722, 1449, 1226, 1594,
        1721, 1448, 1225, 1748, 1447, 1446, 1224, 1445, 1444, 1747, 1223, 
       1443, 1720, 1593, 1222, 1442, 1719, 1592, 1221, 1441, 1718, 1440, 1220,
        1591, 1717, 1439, 1219, 1590, 1716, 1438, 1218, 1746, 1436, 1435, 
       1437}]}}},
   VertexNormals->CompressedData["
1:eJyE/Xlcjd/3/4/LkClD5pBZmUMpKmfJPKeBShoMkaGEjCFJmTJECA1EKoQy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    "]],
  Axes->True,
  PlotRange->{{-2.9999997864282695`, 
   2.9999997864285715`}, {-2.999999786428496, 
   2.999999786428496}, {-2.939868153392189, 1.098612217477631}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.442754841367625*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell["\<\
This surface looks like a tornado funnel. Even though there are many values \
for the imaginary part of the logarithm function, we could consider graphing \
all possible values at once. The result looks like the following graph.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ParametricPlot3D[{r Cos[t], r Sin[t], t},
                 {t, -3 Pi, 3 Pi},{r, 0, 10}, 
                 PlotPoints -> {30,5}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJyUvXlcT8/3B96+72nTXkQIFULpXhRJ9ixRdomEsqedKClbZYsoZMmS7PK6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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxFm3UcFtXTxe/uPndREQUsDBRB7AJBwELKIlTsJkRF7G4RBAu7FRVUsP3Z
jSJ2i2IrIrai2IoY7/lyxs/7xz47O3v37n127505c2Z25SGHDjykTCldklOq
tO+nn7+kaFqkdIC2TtK/I/2BdUoX69wqkr+S7qDK+s8lj5D8tuRBat9Rx29J
/lb7g6V/V/JPjZQuk9xe8r5qs0F2+9naH1i5/Z7Sd9DxDMnfq/0F0reTvKv0
60n/huSDJHfO7vN77Q9Rm/ckPyP9KB3/KPk67fev3H6O+jlXclvJf0geL3k1
yTvy3yS/Kfl97dfRNbtFP+vqf54guVtp+WnpltT5KWrXUfoVpXtXfd0m/dHa
fyN9D+nXlH6mjh+V/ijtx+r4VvVxnOQzJN8m+Tft39bxTWpzgvY/6NpeunYt
/pPkjSWfq3a3aNtL8t/SHyn9TMl/6toXeB863lTn19fxSzr+Q/u1ta0v/TJq
P1X7DdR+Helm6R5PqM3h2n8mfXfpN9a16+nc89IfIv0n9Jfc92hd/7Pklwv3
f1LIp0v/i+Qk3VFq/7HEM6XfJFuupD9a+lnJY2Ncv0p+lXeoNjvFfbnnL/Fs
ea7cq7vkVyX/Jnmu5G665nfJ+2tsL0neUfLz2q+ofnaQfJHa3KltP8kj1eY3
nesjuZd0M3gekjtr/1PyfDhQbWaozS6St1ebr3XuX8nbSv5C8j/Mq4bnAXOA
tqtlt99Y8p7aPuL/qH1PFojkjWjL/Jc8S/99TbX/UOeX0/4p6Tup2e867iW5
VJuD1eZO5pn0/+he10teQ/JROtdF17wv+RftD5P+A24heZLkNfkPkg+t3OZ0
td84ez6MZBySP5I8TvKm2c+/1r2OUftPJH/IHNH+XY3ld50/XPoPdbyY5Acl
ry95uMYzXfLOXCv9bZLXltyQfLPktSSfpP67ZV+7iPZ3S7+u5JN17Vm6x+2l
31Wb7Pd1sPQf6Xh3yYfq/H2S15O8gs4/K7mz5JUkvyB5Q8mrqc2HuuZB3edj
7dfS8T3S7yT5c233Sf5U+xuz5Q+031D33EHtP5J+dbW/W/q3pN9A+u2kf03y
+pL7S35fbdqqzf/U5j3JK/E+JL+pNh3VZlu1+UT6fjq+V/p3tO8k/faF7djD
Ore/5CclryH9FoVt1/3S7yH5PcmdpR8o+Uf1/YOOl6hsox5VmxHSv6X9sjwr
+pe8guQ7JL+iNuvp2n6F7epDOje08HufIvlQyc9q/4+Ob2BdqO0ibNL/Xdpe
Y6tHSG6ubVHJ/aUbpm0J7Ii2+dJlbS11z9fV19WS95H+H+1rbYtL/6r0lzPP
JbfQtoquf066QvKNuu99kmdpDFdKXph5pePzmb/qZ2lti0m+QeeXk9xFcm9d
P1jbH5InSb+89F0l36Xr1tXx5epnUfXzso4vLjz/n5B8pOSHJbdnvUm+h7mk
4yvUvr90n/K/Jb8ofRNdP1nyFJ1fTed669xMbWfr+M9o37X2NdPVfglssvR9
GSP2EVug/Wesn8r2YaPaNuI1tW8u+Ra1/1LH52bf9znt11a7rQvru6nNGOk/
KdznaMk3ZfszntUq0n2gvu7V8crSv6z7dAm716G27ePac3RuvvRHSPea2nRl
/qvNXJ0rtLXHB0rfTfrdpP9VuopxFu7nZOmfUJvVda5PYfvG2L6IZ7Kh5FHY
AslnZtvJLdWms/TvSze/dl9dsEGF9adF+06SR2L/m8jf6XhztXlF/6mZ9DdL
/ZT6W1N9balzraSboXM3FH6u43Tur+Q+z5I8T/JL2q+r9n2le1Ntl8Y/hg3f
oLYdf0Ft1tHxNpKfkbyW5K0kv6FtbLa/6CFdR7WfHnrkU7ADks/I9iPI9Hlq
2MYXdL9zsBuSX8H/l54PzAXGOSfZhtyF/VebHSSfkby/Tsd767+fl/188R08
Y+Qx4V8Gahsr+dra48OnMQ8HaDs9eX5tH20mSR6u7TkdT9b+IN5BaezCOmKN
bKhxLSt5ce2X0b6VtrUk36i2+2t7SsffJtsx7Nv52f0v8HGF9cylm9T28Nq+
gPFuF2O+Qbp9tT1eeo4zVub59dIN0faojjevbUfGJePEPSQ31PeF2XOU+blJ
bflMtbk0uz02p0dtW3Nusm7v6GeQnuMQbedIviy7DbaI/8f/vDSe+Y7x/FnH
/eMZTuS5lbZ/18U7wj9PkLxLad+7i97zRYHN9tB+r8rzn/nI++Zds16RR4dN
6BfypWo7Qud6Jts2xtOssF3jvbSR/lTwiO5xrORNpF+pNL4tmvj58P/+Uh99
JO8q+W/JW5ceC/91UDwTbHaL0jacfUtta0jeSu2XLG3DW0abi9U+q/+DJe+r
fv7hHWkcP4U9wi6BUY/XfmJpHMgaY62BSW4qjaXAUZtK30bHq+Jbk9fvTdp/
lrzWWNesZ/TghM+T+8cesg5p0yH5v/KfBwfW5zmMTx73kuGPZjLndP4q1ouO
VyiNyQ/l/WlbWfLWtZ8zz5v3ydqbUzjGWFHHV2u/j573Hepnm+SYgX6wsRsX
bkNsMk/XDdQzmRP3X0rbmjrfDB9V2m+yX1zb6pK3rP1seQ8nsnZ0j6OTr+P6
S5Lbcs1FkrfR/fuCEXXtorw3bRdKf4L62U7XHiV5i9rtuSdrYudYF9jt3bWd
zfOPuXpW+ILNS9vwiRrDbPWzftg5zmHrsPHdo03HsC3Yur3Ubm9tuiQ9AM7P
Pr5WcpuG3/ViGu/ildteJd1y2rYP23Jatu+mP2zWLmq3PP5E+6/U5nad65Mt
E/ucIv3XyTh/s9IxAjb/JOm/TPYt2PEFOLy2j8GPjMm+5tew2z1j/Hep/y2y
+7wGDAJeZB1KHgdGknwrmEXyg8xRyWs3jM/BD8dVnr/nS9cUDCf5Du07Nozn
L1H/3Xmm4e/wdWCDC7TfLBsng2eOrdxmoq67Qsf3sz6JQyXfxXsC74AhJU+S
fKXkByRfL3nVhmMcYs+7wTdgEO1zw3P1crW7V9uhjEe6FtoGoNf+ErCuzo2V
/K/GsJX0F2G7wLTSj5D8jvS7Sr8yfkzyN5Jv1D166X5fJONg8PAmye8TTMY7
Jc4j7joxOebaqHRsi26T0P+QfC2492Rd+62u+UfHT+r6vjr+Ptnn47fwWcSV
oyvriRfplziOe3YNu71Z7bnOPGeO7xrzfJew0ftUjq27xXguzm4PnsFGYiuJ
u8BEHM8P+9Al+uc9bhXz6mawZvbcA/fju8FIj9FXtj8H955Q2YZNkL5ntnwf
6z/7eT6Ezc32peAU7DZY5QrwTvYcAycfX9lGgutGVm6Pfd+9MkaCBxgl+bvk
PvABxIpgENYFOIS4gvhio+TY57TK4+Qa9MQytO0d7XnWxBfERMTivD/iaOIN
9PAGYF6eC3Ob2H3TWJtfY4MqxzuDsdXamui6Yey1LSz5ILia7OeP7/5X+3ba
joB/yPbL+OgSGyv5KPB19MO7mwe2LfxOGEMrycdJf3y2vb0Me6Dt4GT88Jd0
bbUdRkyajROwf9i3JcHkpe3QUoWfAc9iacnbEz/V5lWwPfzHJaS/OtuG4zeZ
48yrFti6bDuMncc+P1Wb2zpa+mO0LST95dxfuvaSr0AnedXCfBI8AhzC2oVt
5WKBxZAvYy0w37RNl/xc4fbHJ9vujviksOfXZ6+zp6KfteM6ri/wF6XnyjKh
6xD6fuELWJvgR/DP8jqxTel1sSzPWf/pSW2/lY6trs3uo1PhcTCGudrWk9w6
Oc6dkH18nZ7xPOLn5JgM2wXmxDYwx1qqzbT4X80L+8eTsn36rIh/8b8dIt7B
jxMP7xb+CN4O/q6N5J2wo7W5G87vXphvJK78JPsYm4MtYz5PimuJ+3bBRobt
hbeDv1spOd5EvjzWNbEC+GdUab+xheQl1b5RGecQR97bcJ9vl8ZJYCRizJk6
t7Ou/xifoHs10zWLEltXHudOhc/rkgRhCgYDh+Mf4Rg6hy0iju8Udph3DW5n
jF1izNsyf2tzfPy/rsX/48mR4YuvLm2Dj8EnZuMi7oV9xQbS5/hsfAK2eQc7
WnoMH8DfZPsCbCC2Bns4EF9am4fEd4AL+e+nleZveuMHscuV7Sh+eUBhfgY7
SSxB3ApPuUb2O3q2NIbjvXeQfpHKNq2obOPbFMY+YKCVmNCV46cVCz8rbPbt
gQHw/3fGvbDHxDLP6LpvSsdicF/wXnAg8FFwUROTeTA4sOuT8fgp2Zh8x4af
53H894bn7THxTLC12FjeHbgGHmaa7vN5aQyMzcfeg4G/wB6Ezf9Wcsew7eCH
T5L5B+Ye/gRuYarafAZGCB+Nfya2hS+FK4VLmY//qmx7wZsnZGNO5gmcNPP5
S+xZ5es/Kj0f4cbBZvDIcMjEfth3nhtxGjgbjA13TlzZLpmzn9swNj6d/9Hw
fIMX/6ZhHA5uxVaDXXfVdfcG5oGrh6cHe8Ptg7npn7XOWFj7zGHiVdYU/Dwx
wDXYGexT8Jm7gekCz8B5wmVep22Kjj8uHS/M0n6pynz+W6zX7Lk0oGFbfYTk
caUxIFjoFLBXaU77a7W5WW0ekLxfad8GPh9S2udh279Qm8nYLLXpIXmQ5GH8
B93r54btQ3ft95R+qOTNJe8teV/Jm0neXfIQyfuqv6HhI4gXiRuXK4zrwTD4
QWJrYuwVCvMBxOOtJR9Qmr/DH+1ZOnbCDxIL7hO+lfiYOBn/W2psQyv7tYNK
c3ysd3zvlpX9by+NbV+NbX+N7Wf8YmVeYjb3r7w2ziiNMbfl3YGja+NRYv5P
S6+p77AflfH5YJ1/LPDtIDiAwMPkcH5sOM7tqv222H3Jm0reVfJgyXur7cOB
4TcBp0s/SPpt4DIlH5Js58E0+LUtpD9A+gMlbyR5R+IXyVs3jEnADFtKPlDy
CMlbwadKPkhyv4bxyWGS+8DRSh6e/H4apf0786174bwP85O4l/E/ovZzsufq
BfjnwnbpWI19qrZfSnMp3xV+d/tl9w+vMjT7mcOTwJnAA+KX/5cdU3MPMCSc
Fz4dzgEujHmyV/a8AqeBr8FvPIs7smNnuARyKbk2n3+BdD+Unnvg7gmBycHG
8IZggMHZPACcBjgQvg8MdqGu+7H0HCYug8sDR+2WPZ+J0QZov522PSTvnP2+
iDdZL9iLNrG+zq5s54lJ4drAaeQE2pW2xV0lty293uHqz6js78h9fKft9dK5
kVbZ/pEcxUK1bSh5iXnZ/cDbwdmB/fpnxyY/xDiZb+AxsNkzgUmOhrvS9n1p
zAXmAe8Qm5DTAgdun80JEKeAn8lpgW12yJ5vYGliRLj92cmxYNPaMtgfDgR/
8KD+1xfZ9vBu7Vdmvamfe+O/M8+eVpv52f7o9cKcKjE7MTQ8P/4mx/pljhK7
wM/gC4mhl6stnyh5mdr2/0G1WUVteqiv++OZby75J3wB+KByfudJ+KjC/DPc
CFzLNOn+yPaV8AOfZtvwqZJ/lTxB8uOSf8q2yeTynuf9Fc4/PiP5VGyU9gtx
D3yo5LmSr02Oq5as7f+OgPPKtjnkqeDw8dnkMP/NlskZrlraF96WzdvA35C/
gsNnzpNjLGu3Ib5frPbzJ16H32bOwE+0rv08e2TzUfjobbJjKHw0cdUatf09
cV6H0tgCvoHcEJiDWBBei/ETLxI/0p5YEO6U+I44b9XaMvxHm9rXEou3q90n
ubPH4/nAU50m/aTS+bJp2p5K5vDf1fZqcly/WsjkH8khvob/krys5OeT8dQH
pf0gXOvPhXHUzmrzbcP+DlxB3oOc3UWs3cK4izwDfC+Yjfzyh9peT865tc7G
deRYyau+mJyPeyM7X3gAa1/X3lw6l7dOXEv+7vVoTy5jBmtJ8lmluUW4JvIX
c7W9Vzq32C7awIkQJ5XJWK5X4dwi+I08D7lOMMZGhTEJuccyuw04hDkCnmmF
bawcs5CrhDPnv5NfbZKdYwWfMPexV5X0LSrbVfIqbbP58EfgxLI5TPAka4uc
LLnZvxqWa9515XzKq2CX0uMkd9osmxdtjx2r3A+YkNwguVEwJDwpbch/Ns3O
gTZhDJW5VfKoC2e3J7ezSjZWJ26CZ15Q8wBOy+ZBqYu4XfJDkneV/EPDMrUZ
c1i3umY0+L40jwQHfqr0N5SuwbhL1z6SXBdxq+QHk2OjF0vjn92l/6XhNuSp
H2JOJ+ejH9H2RHI+93ttb5SuiWiZHXMNUftFY56TO35M21TJe0tfxbXUUbyo
7bnkZzaj9HMbpjZLSP9scm76WewO71365iFT00GMxjPZS/q/tU1Jngsvl54P
ncOHgkE7hW3hua2fHReDralV+TIb15JX/zkbj1Ib8lvDOKFL2Hb01LB8l93n
htn+HYx6pvaTS/Oo5LSJwYi/OobtXVDTov7+1PaY5D20/13bo8m1KPdl64dX
jn/B08OzcQ75FGoJXsWHJdd1vBw2gXoZOF76p46mden/QgyKTWNdl03Mq8wr
HReuF2PDhyykcxNKX9ekiXl57PHCkgdWztPWTczLb0tcoHd9ZDLOJyYnXiV/
R2xDXAP/3VD7rXVt/4ax6eHJtrZpE3NP5PEWaeKcBNxSJfmR0vwl/RCHMr/K
mCfwjeQau1dey/gz1jN5QnKL3Lc3cYGODwifcmbYQGzILdl2hFiPXD4xMPnI
ifSRnFskZiNeG1k71/OCxnONdNeCbZLHxf8kTqYt18AvELeSs+8X/V0PVknO
Y+KvOZ6kbXLca0qMjbiP2JaaA2JpYt7p2TE2eJiaBrgPYlJqF3ifUwvXFsDz
rVk55qNveAW4JDgoYkZ4EPiFInw394JfBSNRZ4P/aRprk3k3L+bkWPX5leTN
Yt4OCtt7S8hXhZ0hH9ZT8rjKfoeYa7DOD9GWk3ERa2pI+Kbz1W7zmPPXVq7P
OVznjtC2cHLM91jpuA9/0jp8Cvk6bFqP5BoI7lEn24PFww6Av6iRAINto5PP
1o4Dv86u06DP52rbNvKq1F4M17ZQMj4nBgDzozuwWDDNFsQCrHGOH8i2/9gW
ai8OjjF3jmvbJ8cN1HXQz1GB/8B+4CJqPw4pjA8PK4yRsEXU0hGD82y+a/j5
oDuxtn86QmNtWXmcU7P9wrTkegie3SLJ/c2uHSeDv6gPoX/wLfHYntGGe4NL
qXkAmzGnqZGhPg3egzoWfBXcxWYh44PO1f48bc0Zm9r/ru1pjWsd3e8k7e8v
XKPDfvmUFrwccsejdfwn80v9n1i4LoP6EPpfL2wQ62W8ts9r8ynUjnC/FmrT
AtsW9p+6DuICxkFd00m1fTDYjPWMf9xBx0eXxhDUY9AveUnWFmsM7ogc/1nx
X4apj5+1PaFrVtK1x5Q+B9/zZGnOh1oPamPoZz/iv9rnqBuifoM+V9a1x0p3
YeGaFfYt8Xe16womJa9R1uoyyZiT8YDBiAOosdhQ5zZW26PIgyRzRYyLa4lX
qPN7qDQe5v+DZ8j7kOv/H/9N13ZpmNea3/B8Za7+KvmebP+yuu6xWmW7xLvl
fYNLiEGpd3m4tE3CNoEdqU0i7sBmDdX+h9rrszkYUvthhbEt+OrJZMyMTcTv
U1PzRsO2dCnildq8FZgNfA9uO65w3ELMAtYingFvgf2o9QH/gQkfzX6n1ElR
Jwk3CI6i/mOrmD+8b3Ld1J1Sn7BP2EvsJjzeHdrfqa1Vsp3GXtPPzhrTiaXt
7XaSDyvtI6jjvCXas7+18LtbHVyQjcnP1FgaDXMs1G4Rd9AOe4BNId6nnou6
UMbQJ8aDz4VPhnPeJWw8th7egzo04gjG+k7MbdYmcQV+nH56x/iJl9fSmNeu
/P+IX/Af+HqeMc8aewhGaBU44YrKWK5Xct6mKs0vkT8njiMveTLYQMcrJOfs
iB2oKyLWu0v6voy/MJ8wSvIvhWMQcnn0R7/nSb4VTM57ScZ7HCP3qh3rEuf2
rN2e/NNNlTEhGJVcEjwOHE4RfZJvoq4GXoX6pTdqY0lqY6gPpN718cJYgoAC
ruya0vP69OR5/nlhvqWv2k6vjYnhS2YX5kz6xruAx+b/8T/hTAar/3sr55Hh
VeAKhyXXinA9dULwPz0brkF9tTYWfaV0re2UwnmWYdncHHMGHwgOxA/iz/Hr
5PJ+iudZxP/k/8Ip3aj2ONYttfsq9GOScS/n0JPDAm+BterSz/n85NrjOZVt
C3m2vwtzjNzzy9r8NffkXRJLkrv7szBv+Ucckwckzpgb8cWfoadW5yR4g8p8
FH1zD+p5yOXB8cHvUacJz0yt5kcxf+C7+tTGi9Qvzo9rqSMijiQuI06k9pO6
ZZ4PdUe0oS1xM5zwgng5mweEA2S+Mm+ps/09ZPLOR+r815W5xGPhKyrzkL1r
5/jIfb6dHXNRB3ZsNtcMz8yz5JnCGy9cuv0FydcgLxTzgPlAbhouh2vgY7EZ
a4TdGAUXVJl/Ox6OpTJH+luMlTi9e+1ny7Mn39q34TqBa0tjZGqEiN3J6RCD
H6bzsyr7+n669v3accjsGM/o5PnCvIGThOeBIzom7C72dyLPR9csXRlXwO+c
rK2Z5KezY9unkzkc7DIxOOdP0bZoMsd0tLamydgHDAR31CVkYqhucS/iaGw/
PmBC8v7T2rmBsyrX91J/DhcFf0WbR+O+8NLUAINTwaj4t98ihoKfGqNt8eT9
WG2LJdeswh1xDKdFTSljg+Mi1gOD7tawfcQ2Tg9bSg6F+Pfqyrmw5/GTyXHB
BmFj4R+oA8Yfghv/CMzD82yt61bUdprOTasdN5HnAde0DGxDHS882KjCPCFc
Gc+Ua0b99/wL43Z4TnwjPvL65FpvZPgH+EV4OfTUYFK7S66K+P7niN+PV9sT
4h3BKT2jsQ1MzhfDp8OlU4OELcAOPaHzS+v67ZLjUHh5OHn47+8Lc+DE6bSj
DXz5j4U582mVOStiYXSco96JvDH5Y3Liz8b/ot7jndrx/lul60qI96i5hWMn
riOmwwbDJ2CH50afJMWpM0RGd2d2PoL3RN6DeJHcB+Nl3NQNUvsNR3dPYT7h
scp5kLsL65ZNrs3leDnJL9WO01/S2GbUXgdvSl4ebrby/IdTha/m+YJ7P6ud
a5kT9yVHQB6BnAT5CLiRB4iP6as2rp8Ot1j4v49NrqntUTlnNJT/VLlGCA5h
YuVcNjr4EPTUx3Ev7knui1wefCBYCczEfFgqsNl1hXE0eJL85Mm166LIeRJ/
EIeQizskO0eDTYb/II4gvoBfJp8Htw+nQD0TNuqU2nUz5Gnh6cE3YBv4VzAN
eOZK3X/Z8AsTtG8fNpY2cLVwm68U5uSpVyEfgo+8J2T8ND6a/skHkNcGz5Mf
OTvGid3B5sDNgMeIy8gBgT/AHjtWzqd+EbYdPb4SrEFdC3hj+8o5M7hr7DF1
LfjBMrALOGdwyEXwGNTKULtCrTx54s9iDPBE8Gnk9Kh/pc6emIKcwkXh9y6J
54yMn8OHvhTPgXwEHAn32KJyPpA2+MHb9fw6NZzjpsaR74X4VojaubaBB8hX
kLeAeySnQW4D7hE+G14b/hMeCD6I/00+AXuNrSaPgU3HZqLHdmOTyVHAh8C9
ENOBrcHVYFXwKFgUngee5NYYA3XhN8W1YFlwLN8eXFmYY4EfgicCm3Nfckir
R59gejAw54mjusR7wV+CQxg7/gzbS54BX4Wf2ic7t4tfHq/nsVJgS2KrFwvn
g/bIzvPOi//yVek8wvjSsRd1cdTU8q0RGI+cAN+W3BUysWi3eJ7EnMSGcGjw
UHBQ5CioxZ8UMjE28TX5DWKuG0Mmxlsn3hEx2uT4L9h9bDucHP0S7/I9CTny
98L2YgexsdRWIGMP34v5x/wnruMe9D8r7sd9P4kxMbaZcT/uBbcNL4h/4RuA
rpVrq8gPwwvii6mxol6WOmRiNq7lmVOLTc3rfzW2yNTKIlODRQ0Y62ZwxAvk
58Fh4Cm+PTqy9vdHI6L//2qiqeuiLpoarcGBr8hRE0OTp4afZf1jo2bHc+QZ
ksckxgYDY3cPq217iYFpw1r4MuYl85n3zzxYUIui9ofUjn3g/XlWvDtyInzP
Rp3nh7E2WBdgGTANeYnNwh5hi8DaYG7qYj6IdcWagv8gjiVvQDxMP6w7eB24
G+qV4JJoz1ogr0V+C54c7LmgRq1wHQF4GixNbTD8At+S4M/JFRFTgDOJ68DD
1CYgg5cHVK4pmRF98i7At9R/Uj9DrTt2kufeJPoht1SGfcJekz+F672itM3B
F5Ev/zdk/NMZMVfJr+O74RbIZexXmGOGax4Q8QhY5L8aUWQwCe98SMwZ8iHw
nNTGUA91TYyT/C91BoOS66Goi6I+DQ4e3pgaFeqlyVnznQhcNTEIddxN4xle
GNdSc0+9N7w4/Dg8ATkE+Du4O2Ti7atjDNSkUo/K/4AbpwamS+ipY2cM8Kpt
k/sDR7aL/0KOp01yvgJusX3I1AGMj/eOL785bBScAnwCuUo4EHiqKbV5JGo3
+P6Q+hhqY6hz+bLhWhdy6W8XzqcfHM+NWrIR8ayoJ2HM1KtRV4Y9Yf4QG/Ld
DzUDfOcIlqPGke8qqXmhpoH4Gn8CBsCnkE/v1rA/4jvYhwtzC+QxyI+QyyCn
9G/DeaVTtf+jcr0NfYAnqD9jvIyb2lfqaPjeBlyE3bgyu7aH5wAPBQdFLST1
EMT1+Fx8OlwHdolvRaihwa7wn7FdQ2MdUcNDbQ7ztnWsKewV9Tn4XPw73AJ1
l9R5LB99HljavmHnhoc8I8b8e6w18AMxNTU++HpicOouqJUZnFwnS60GnAM5
saWy82JP1M5rUMtzqc5dVphjZJ0SizJmvtPj3BLJdbbkacjR3Kj9Wg3XcVED
Sw0E/m4v9de6YfxPLS21INSBgAHBguQjF6n9LR7++ji1aVe5f76D43vAqwrj
CK7Bft4ifYeGa+qo0YMnxj/yXRj5esa9eWDRBXnS2t940oY6X+pIqCGhhpdv
A/A128H5RO6AesXvIu/QX/LMiAH53pOc/iOF4wniiuYx19DBS1NLS/0NtTc7
wCEEZ7IzHFTD3BR1bgs1bOepdaUupGXMc+pL6HN3tVmm4XhnH8ntGv6+lfqo
NRr2Wawh5hU2hzq3xRr2NZO1X7fhuj7OHxRzb3DYVbgF5tqC7zmLyFfUztUS
zxB/ks99uvZaptYSjveF2vlavq2gvojvGd4rvE5HJnPCL9fOib5We62Ru56a
zKNRkzAtOQ6hJgfu/a3auVjGQo0a9pA6AL4fpgaAXAD1Dw8U5kr4do78Pjb1
zcLvkTiJeIm8MNzpzNq1C6z5D2rXB4DNvq5dk8S31NRY4ReoV8evgH/5jp06
ywW18JI3qPzd+GX6H0s3XA9GLLt89nwmrwjuBONzH+6HrRmntotE/LVTdm0Z
XNB/3wpS48T32dRN49eIfb6pXacO5u5TGbfzXegmlb8hBTd+Vbse683oh29Y
+Gad+iPWPt+JNIkYlhpP8qLUeb5be41TdzxG5/+pzCu+E++OWiy+PVk91vK7
8V5Zp8TJ1PARt8L5LB22As6RWmdiQ+rKqF/Elo6ujXmIW/lObL/Kfg3bjw9o
Fr6Ab8n4TphvE7BdM8LGUkMJHsA37h/zYVRtXETcSvyEDT0/bOm+IWNTOQaf
YKfpi/8OvhgWfRJXU3MMxwI3C8+JL+D7VHhC+ELyel+F/xpTGx/ynSZziPfO
O2beM//B8B+GfJrkC3Vd8+A3LtZ+ieAJz9O+WXAatOUa7MP/Ab/6MkY=
         "]], Polygon3DBox[CompressedData["
1:eJxNmwn8V8P3xu+9M/d+UdmSNUIoe/b9R3ahiMpOKksokkRk52fLUpYsLVoU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         "]], Polygon3DBox[CompressedData["
1:eJwt1HtolmUYBvC3b9/3ZR6KFRZBFIaVikEEQVGMTmIeYDoIC1fqbGVlm9VK
DRRKyIKkOSsVKbJS0tTKTSscHShymYc2zcoOW5blNotptQ6a9bt5vj8u7uu6
7vu5n8P7PO+wmvqqulyWZZVQgOZ8lnUiQ8qy7FuJFbyLi1k2Cz6ie6CXN13N
NDhKL4VaXpuaZ4xbBvfgbfw2NeWwDx/AH3dKlt0lNwx6jOnmfwiD6DvhA/wQ
HJRbqn47bzt9t9hIV+s1Ffp4jTBT3Xy5SuseSN+k5rg4hp7L/x2/nnfEvPXG
1cFfvOegztgmuSlqX86luWLOc+lqvJm3vFR3kn8zbxPdVNrvw/SNarv1rpD/
jT9K/9m8W+Wfor+Bs3i1vEm8xfTncA69HDbnkhe5reqO83bSBf2u1bfGHIf4
++W/42+DC9U8Dy34O3GW+L9wG4xV9xh/F/xJT4UxvEfpT2EwvQTewJuNHYQ3
iWeIp8e68Tfju4sX0cNhOr6F97SYp8tgCv4671T8yXzim3lF/MzomU99Wnjl
+GEYj7/EWwgV1jSCtzqfem+TuwR/UGy398/i/uEHoq/9/yG3A8/zK5zLYrlf
8V+gCn9N7ig+L5/4Rl4ffYeYQafwA//dOGs1j/BO4BPVDBFrzDGjkL5rfN9j
sEGuQ90efHbMQXdBJb6OtwSq1ffzFui5IZfOOc67Sq/JhZSPuiMwVs08Y/vx
G4w5Zh89ambyWnmd8IX68phT7Tr8J95wtaNjD3FfeL28y+JOQwNvfaxVvpf+
mV6r55Vx3/kfw+Ny3XKHYXIx1TeIV6vdre6KOMd4h/Q/xp5HH6RXm+sEvoi/
MZfuVdyvhWoWwEl6FczlnU1P1HMt/UTpPscPZQLvFXpRaexKepR+I2EGvpW/
2xyz6FfjvfB20dfQD8U35l1Ft1tnga6F93g/Qpd+B8wxAr6mB8ZdUHd5nAm8
lUtvMN5i3KW4U3vgAfwr/v3GzSmk9x3v/G/oMG6o/Mg4S3q9XCN9Gn8A3FJM
b+dSfE0+rb+VNxr/D97W7zpraIh9wVB6B4znzaf3w5zYt/5bYB++l9ev5n3j
XzTf9/F/kLtX7tmy9E+Nf+snuXQ2cUZfxvrFnaUe0asF6vEOub3xPulNcF/s
Pd4EPi3WnUv3Ou53n3lbeS+Ytyv+Vby62GNZGh992uM/KF6g7ny4vZj+Pf8D
VzPizg==
         "]]}]}, {}, {}, {}}, {{}, {}, {}, {
      Line3DBox[{625, 784, 1776, 700, 735, 701, 1659, 785, 771, 1772, 712, 
       1736, 713, 1664, 640, 1709, 683, 1671, 1672, 655, 1726, 750, 825, 751, 
       1694, 855, 767}], 
      Line3DBox[{626, 786, 1777, 787, 736, 789, 1762, 788, 772, 714, 1737, 
       1738, 1739, 715, 641, 684, 1673, 1674, 1675, 656, 752, 1768, 826, 857, 
       1788, 856, 858}], 
      Line3DBox[{627, 790, 1778, 791, 737, 793, 1763, 792, 773, 716, 1740, 
       1741, 1742, 717, 642, 685, 1676, 1677, 1678, 657, 753, 1769, 827, 860, 
       1789, 859, 861}], 
      Line3DBox[{630, 799, 1779, 703, 740, 704, 1660, 800, 774, 1773, 720, 
       1747, 721, 1666, 645, 1715, 688, 1680, 1681, 660, 1728, 756, 834, 757, 
       1695, 862, 768}], 
      Line3DBox[{631, 801, 1780, 705, 741, 706, 1661, 802, 775, 1774, 722, 
       1748, 723, 1667, 646, 1716, 689, 1682, 1683, 661, 1729, 758, 835, 759, 
       1696, 863, 769}], 
      Line3DBox[{632, 803, 1781, 804, 742, 806, 1765, 805, 776, 724, 1749, 
       1750, 1751, 725, 647, 690, 1684, 1685, 1686, 662, 760, 1770, 836, 865, 
       1793, 864, 866}], 
      Line3DBox[{636, 813, 1783, 708, 746, 709, 1662, 814, 777, 1775, 729, 
       1757, 730, 1669, 651, 1724, 694, 1689, 1690, 666, 1731, 764, 844, 765, 
       1697, 867, 770}], 
      Line3DBox[{637, 815, 1784, 816, 747, 818, 1767, 817, 778, 731, 1758, 
       1759, 1760, 732, 652, 695, 1691, 1692, 1693, 667, 766, 1771, 845, 869, 
       1797, 868, 870}], 
      Line3DBox[{668, 696, 1786, 820, 819, 1785, 748, 653, 681, 1706, 1705, 
       1704, 638, 674, 1734, 1733, 1732, 710, 623, 671, 1798, 846, 733, 781, 
       1761, 780, 779}], 
      Line3DBox[{669, 697, 1791, 829, 828, 1790, 754, 658, 686, 1712, 1711, 
       1710, 643, 676, 1745, 1744, 1743, 718, 628, 672, 1800, 849, 738, 796, 
       1764, 795, 794}], 
      Line3DBox[{670, 698, 1795, 838, 837, 1794, 761, 663, 691, 1719, 1718, 
       1717, 648, 678, 1754, 1753, 1752, 726, 633, 673, 1802, 852, 743, 809, 
       1766, 808, 807}], 
      Line3DBox[{518, 517, 839, 762, 664, 1687, 692, 1721, 1720, 649, 679, 
       1755, 727, 634, 559, 744, 1782, 423, 810}], 
      Line3DBox[{782, 783, 1698, 699, 734, 847, 1799, 848, 624, 1701, 711, 
       1735, 675, 1663, 639, 1707, 1708, 682, 1670, 654, 1725, 749, 821, 822, 
       1787, 823, 824}], 
      Line3DBox[{797, 798, 1699, 702, 739, 850, 1801, 851, 629, 1702, 719, 
       1746, 677, 1665, 644, 1713, 1714, 687, 1679, 659, 1727, 755, 830, 831, 
       1792, 832, 833}], 
      Line3DBox[{811, 812, 1700, 707, 745, 853, 1803, 854, 635, 1703, 728, 
       1756, 680, 1668, 650, 1722, 1723, 693, 1688, 665, 1730, 763, 840, 841, 
       1796, 842, 843}]}, {
      Line3DBox[{31, 311, 32, 312, 623, 33, 313, 34, 314, 624, 35, 315, 36, 
       771, 316, 37, 317, 38, 772, 318, 39, 319, 40, 773, 320, 41, 321, 628, 
       42, 322, 43, 323, 629, 44, 324, 45, 774, 325, 46, 326, 47, 775, 327, 
       48, 328, 49, 776, 329, 50, 330, 633, 51, 331, 52, 332, 634, 53, 333, 
       54, 334, 635, 55, 335, 56, 777, 336, 57, 337, 58, 778, 338, 59, 339, 
       60}], Line3DBox[{872, 340, 1443, 538, 1327, 344, 1445, 539, 1798, 1329,
        348, 1447, 540, 1330, 352, 1449, 1799, 541, 1331, 356, 1451, 542, 
       1332, 360, 1659, 1453, 543, 1333, 364, 1455, 544, 1334, 1762, 368, 
       1457, 545, 1335, 372, 1459, 546, 1336, 1763, 376, 1461, 547, 1337, 380,
        1463, 548, 1800, 1338, 384, 1465, 549, 1339, 388, 1467, 1801, 550, 
       1340, 392, 1469, 551, 1341, 396, 1660, 1471, 552, 1342, 400, 1473, 553,
        1343, 404, 1661, 1475, 554, 1344, 408, 1477, 555, 1345, 1765, 412, 
       1479, 556, 1346, 416, 1481, 557, 1802, 1347, 420, 1483, 558, 1348, 424,
        1485, 559, 1349, 428, 1487, 560, 1350, 432, 1489, 1803, 561, 1351, 
       436, 1491, 562, 1352, 440, 1662, 1493, 563, 1353, 444, 1495, 564, 1354,
        1767, 448, 1497, 565, 1355, 449, 1499, 566, 1357}], 
      Line3DBox[{61, 62, 638, 63, 64, 639, 65, 66, 640, 67, 68, 641, 69, 70, 
       642, 71, 643, 72, 73, 644, 74, 75, 645, 76, 77, 646, 78, 79, 647, 80, 
       648, 81, 82, 649, 83, 84, 650, 85, 86, 651, 87, 88, 652, 89, 90}], 
      Line3DBox[{905, 1253, 1109, 902, 1256, 1610, 1734, 908, 1259, 1110, 911,
        1261, 1663, 1111, 914, 1264, 1112, 917, 1643, 1664, 1113, 920, 1268, 
       1114, 923, 1739, 1271, 1115, 926, 1274, 1116, 929, 1742, 1277, 1117, 
       932, 1280, 1611, 1745, 935, 1283, 1118, 938, 1285, 1665, 1119, 941, 
       1288, 1120, 944, 1646, 1666, 1121, 947, 1292, 1122, 950, 1648, 1667, 
       1123, 953, 1296, 1124, 956, 1751, 1299, 1125, 959, 1302, 1612, 1754, 
       962, 1305, 1126, 965, 1307, 679, 968, 1310, 1127, 971, 1312, 1668, 
       1128, 974, 1315, 1129, 977, 1651, 1669, 1130, 980, 1319, 1131, 983, 
       1760, 1322, 1132, 986, 1325, 1133, 989}], 
      Line3DBox[{987, 1324, 1500, 1323, 984, 1321, 1498, 1320, 1758, 981, 
       1318, 1496, 1317, 978, 1316, 1494, 1775, 1650, 975, 1314, 1492, 1313, 
       972, 1649, 1703, 1490, 1311, 969, 1309, 1488, 1308, 966, 727, 1486, 
       1306, 963, 1304, 1484, 1303, 960, 1752, 1301, 1482, 1300, 957, 1298, 
       1480, 1297, 1749, 954, 1295, 1478, 1294, 951, 1293, 1476, 1774, 1647, 
       948, 1291, 1474, 1290, 945, 1289, 1472, 1773, 1645, 942, 1287, 1470, 
       1286, 939, 1644, 1702, 1468, 1284, 936, 1282, 1466, 1281, 933, 1743, 
       1279, 1464, 1278, 930, 1276, 1462, 1275, 1740, 927, 1273, 1460, 1272, 
       924, 1270, 1458, 1269, 1737, 921, 1267, 1456, 1266, 918, 1265, 1454, 
       1772, 1642, 915, 1263, 1452, 1262, 912, 1641, 1701, 1450, 1260, 909, 
       1258, 1448, 1257, 906, 1732, 1255, 1446, 1254, 900, 1252, 1444, 1251, 
       903}], Line3DBox[{91, 92, 653, 93, 94, 654, 95, 96, 655, 97, 98, 656, 
       99, 100, 657, 101, 658, 102, 103, 659, 104, 105, 660, 106, 107, 661, 
       108, 109, 662, 110, 663, 111, 112, 664, 113, 114, 665, 115, 116, 666, 
       117, 118, 667, 119, 120}], 
      Line3DBox[{995, 1136, 992, 1615, 1706, 998, 1139, 1001, 1670, 1140, 
       1004, 1143, 1007, 1672, 1145, 1010, 1148, 1013, 1675, 1151, 1016, 1154,
        1019, 1678, 1157, 1022, 1621, 1712, 1025, 1160, 1028, 1679, 1161, 
       1031, 1164, 1034, 1681, 1166, 1037, 1169, 1040, 1683, 1171, 1043, 1174,
        1046, 1686, 1177, 1049, 1628, 1719, 1052, 1180, 1055, 1687, 1181, 
       1058, 1184, 1061, 1688, 1185, 1064, 1188, 1067, 1690, 1190, 1070, 1193,
        1073, 1693, 1196, 1076, 1199, 1079}], 
      Line3DBox[{1077, 1197, 1074, 1194, 1691, 1071, 1191, 1068, 1724, 1633, 
       1065, 1186, 1062, 1722, 1631, 1059, 1182, 1056, 1720, 1629, 1053, 1178,
        1050, 1717, 1626, 1047, 1175, 1684, 1044, 1172, 1041, 1716, 1625, 
       1038, 1167, 1035, 1715, 1624, 1032, 1162, 1029, 1713, 1622, 1026, 1158,
        1023, 1710, 1619, 1020, 1155, 1676, 1017, 1152, 1014, 1149, 1673, 
       1011, 1146, 1008, 1709, 1618, 1005, 1141, 1002, 1707, 1616, 999, 1137, 
       996, 1704, 1613, 990, 1134, 993}], 
      Line3DBox[{455, 309, 1410, 453, 1582, 569, 1411, 457, 1786, 1583, 571, 
       1412, 460, 1584, 573, 1413, 1787, 463, 1585, 575, 1414, 466, 1586, 577,
        1694, 1415, 469, 1587, 579, 1416, 472, 1588, 1788, 581, 1417, 475, 
       1589, 583, 1418, 478, 1590, 1789, 585, 1419, 481, 1591, 587, 1420, 484,
        1791, 1592, 589, 1421, 487, 1593, 591, 1422, 1792, 490, 1594, 593, 
       1423, 493, 1595, 595, 1695, 1424, 496, 1596, 597, 1425, 499, 1597, 599,
        1696, 1426, 502, 1598, 601, 1427, 505, 1599, 1793, 603, 1428, 508, 
       1600, 605, 1429, 511, 1795, 1601, 607, 1430, 514, 1602, 609, 1431, 517,
        1603, 611, 1432, 520, 1604, 613, 1433, 1796, 523, 1605, 615, 1434, 
       526, 1606, 617, 1697, 1435, 529, 1607, 619, 1436, 532, 1608, 1797, 621,
        1437, 535, 1609, 567, 1409, 310, 451}], 
      Line3DBox[{1108, 1408, 1407, 1107, 1406, 1405, 1771, 1106, 1404, 1403, 
       1105, 1402, 1731, 1658, 1104, 1401, 1400, 1103, 1657, 1730, 1399, 1102,
        1398, 1397, 1101, 762, 1396, 1100, 1395, 1394, 1099, 1794, 1393, 1392,
        1098, 1391, 1390, 1770, 1097, 1389, 1388, 1096, 1387, 1729, 1656, 
       1095, 1386, 1385, 1094, 1384, 1728, 1655, 1093, 1383, 1382, 1092, 1654,
        1727, 1381, 1091, 1380, 1379, 1090, 1790, 1378, 1377, 1089, 1376, 
       1375, 1769, 1088, 1374, 1373, 1087, 1372, 1371, 1768, 1086, 1370, 1369,
        1085, 1368, 1726, 1653, 1084, 1367, 1366, 1083, 1652, 1725, 1365, 
       1082, 1364, 1363, 1081, 1785, 1362, 1361, 1080, 1359, 1358, 1360}], 
      Line3DBox[{211, 151, 210, 152, 733, 212, 153, 213, 154, 734, 214, 155, 
       215, 735, 156, 216, 157, 217, 736, 158, 218, 159, 219, 737, 160, 220, 
       161, 738, 221, 162, 222, 163, 739, 223, 164, 224, 740, 165, 225, 166, 
       226, 741, 167, 227, 168, 228, 742, 169, 229, 170, 743, 230, 171, 231, 
       172, 744, 232, 173, 233, 174, 745, 234, 175, 235, 746, 176, 236, 177, 
       237, 747, 178, 238, 179, 240}], 
      Line3DBox[{1328, 1438, 1200, 1326, 1201, 1439, 871, 1502, 1202, 1501, 
       1203, 1503, 1761, 873, 1505, 1204, 1504, 1205, 1506, 874, 1508, 1206, 
       1507, 1698, 1634, 1509, 875, 1511, 1207, 1510, 1208, 1512, 876, 1514, 
       1635, 1776, 1513, 1209, 1515, 877, 1517, 1210, 1516, 1211, 1518, 878, 
       1777, 1520, 1212, 1519, 1213, 1521, 879, 1523, 1214, 1522, 1215, 1524, 
       880, 1778, 1526, 1216, 1525, 1217, 1527, 881, 1529, 1218, 1528, 1219, 
       1530, 1764, 882, 1532, 1220, 1531, 1221, 1533, 883, 1535, 1222, 1534, 
       1699, 1636, 1536, 884, 1538, 1223, 1537, 1224, 1539, 885, 1541, 1637, 
       1779, 1540, 1225, 1542, 886, 1544, 1226, 1543, 1227, 1545, 887, 1547, 
       1638, 1780, 1546, 1228, 1548, 888, 1550, 1229, 1549, 1230, 1551, 889, 
       1781, 1553, 1231, 1552, 1232, 1554, 890, 1556, 1233, 1555, 1234, 1557, 
       1766, 891, 1559, 1235, 1558, 1236, 1560, 892, 1562, 1237, 1561, 1238, 
       1782, 1563, 893, 1565, 1239, 1564, 1240, 1566, 894, 1568, 1241, 1567, 
       1700, 1639, 1569, 895, 1571, 1242, 1570, 1243, 1572, 896, 1574, 1640, 
       1783, 1573, 1244, 1575, 897, 1577, 1245, 1576, 1246, 1578, 898, 1784, 
       1580, 1247, 1579, 1248, 1581, 899, 1440, 1249, 1356, 1250, 1441, 
       1442}], Line3DBox[{242, 241, 452, 243, 819, 456, 244, 459, 245, 821, 
       462, 246, 465, 825, 247, 468, 248, 471, 826, 249, 474, 250, 477, 827, 
       251, 480, 252, 828, 483, 253, 486, 254, 830, 489, 255, 492, 834, 256, 
       495, 257, 498, 835, 258, 501, 259, 504, 836, 260, 507, 261, 837, 510, 
       262, 513, 263, 839, 516, 264, 519, 265, 840, 522, 266, 525, 844, 267, 
       528, 268, 531, 845, 269, 534, 270, 300}], 
      Line3DBox[{904, 180, 901, 181, 1733, 907, 182, 910, 183, 1735, 913, 184,
        916, 1736, 185, 919, 186, 922, 1738, 187, 925, 188, 928, 1741, 189, 
       931, 190, 1744, 934, 191, 937, 192, 1746, 940, 193, 943, 1747, 194, 
       946, 195, 949, 1748, 196, 952, 197, 955, 1750, 198, 958, 199, 1753, 
       961, 200, 964, 201, 1755, 967, 202, 970, 203, 1756, 973, 204, 976, 
       1757, 205, 979, 206, 982, 1759, 207, 985, 208, 988}], 
      Line3DBox[{994, 1135, 991, 1614, 1705, 997, 1138, 1000, 1617, 1708, 
       1003, 1142, 1006, 1671, 1144, 1009, 1147, 1012, 1674, 1150, 1015, 1153,
        1018, 1677, 1156, 1021, 1620, 1711, 1024, 1159, 1027, 1623, 1714, 
       1030, 1163, 1033, 1680, 1165, 1036, 1168, 1039, 1682, 1170, 1042, 1173,
        1045, 1685, 1176, 1048, 1627, 1718, 1051, 1179, 1054, 1630, 1721, 
       1057, 1183, 1060, 1632, 1723, 1063, 1187, 1066, 1689, 1189, 1069, 1192,
        1072, 1692, 1195, 1075, 1198, 1078}]}}},
   VertexNormals->CompressedData["
1:eJysvXk4ll3UPpzmQZOSJJUGpEEzSrY0F5UmUWmgaKLZWFGUNIhUIpmHkHke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    "]],
  Axes->True,
  PlotRange->{{-9.999997499997887, 9.9999975}, {-9.99999749999947, 
   9.99999749999947}, {-9.424777310784693, 9.424777310784693}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.442754844227*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]
}, Open  ]],

Cell[TextData[{
 "This graph looks like a multi-level parking garage. Notice that for any \
ordered pair ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 ", there will be an infinite number of points for ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ". Yet one can travel on this surface from any point to any other point. The \
different \"levels\" of the parking garage correspond to different values of \
the logarithm. \n\nOnce the logarithm is defined, we can actually define \
raising any complex number to any complex power! We simply use the laws of \
exponients and say that\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"z", " "}]], "=", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["e", 
        RowBox[{"log", "(", "x", ")"}]], ")"}], "z"], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["e", 
       RowBox[{"z", "\[CenterDot]", 
        RowBox[{"log", "(", "x", ")"}]}]], "."}]}]}], TraditionalForm]]],
 "\n\nNotice that this gives a ",
 StyleBox["set",
  FontSlant->"Italic"],
 " of numbers, not just a single number. In other words, we are to take the \
set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"log", "(", "x", ")"}], TraditionalForm]]],
 ", multiply each element by ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ", and take the exponiential of this collection to get a collection of \
answers. Although there will at times be an infinite number of elements in \
the set ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "z"], TraditionalForm]]],
 " , this will not always be so.\n\nEXPERIMENT:    \n                         \
                   \nUse ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find some of the elements of the set \n\n                              \
",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"8", "i"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"1", "/", "3"}], ")"}]], TraditionalForm]]],
 ". \n\nConsider 5 different values for the logarithm. You may have to use \
N[%] to see an approximation of your answers. What do you observe about the \
solutions?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "The duplications observed in this experiment are not a coincidence. \
Whenever ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " is a real, rational number, there will only be a finite number of possible \
values for ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "z"], TraditionalForm]]],
 ". This is seen by the next proposition.\n\nPROPOSITION 11.7\nFor each \
integer ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "0"}], TraditionalForm]]],
 ", and any non-zero complex number ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 ", then there are exactly ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " values for  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"(", 
      RowBox[{"1", "/", "n"}], ")"}]], "."}], TraditionalForm]]],
 " Thus, there are exactly ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " solutions for ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " to the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["x", "n"],
     TraditionalForm], " ", "=", " ", "z"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437532355671875*^9, 3.437532390921875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " have the polar form\n                            ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"z", " ", "=", " ", 
     RowBox[{"r", " ", 
      RowBox[{
       RowBox[{"(", " ", 
        RowBox[{
         RowBox[{"cos", " ", "\[Theta]"}], " ", "+", " ", 
         RowBox[{"i", " ", "sin", " ", "\[Theta]"}]}], ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\nThen ",
 Cell[BoxData[
  FormBox[
   RowBox[{"log", "(", "z", ")"}], TraditionalForm]]],
 " is the set\n                               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"ln", "(", "r", ")"}], " ", "+", " ", 
      RowBox[{"\[Theta]", " ", "i"}], " ", "+", " ", 
      RowBox[{"2", " ", "k", " ", "\[Pi]", " ", "i"}]}], "  ", "|", " ", 
     RowBox[{"k", " ", "\[Epsilon]", " ", "\[DoubleStruckCapitalZ]"}]}], " ", 
    "}"}], TraditionalForm]]],
 ".\nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"log", "(", "z", ")"}], "/", "n"}], TraditionalForm]]],
 " is given by the set\n\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", " ", 
    RowBox[{
     RowBox[{
      StyleBox[
       FractionBox[
        RowBox[{"ln", "(", "r", ")"}], "n"],
       FontSize->16], " ", "+", 
      StyleBox[" ",
       FontSize->16], 
      StyleBox[
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"\[Theta]", "  ", "+", " ", 
           RowBox[{"2", " ", "k", " ", "\[Pi]"}]}], ")"}], " ", "i"}], "n"],
       FontSize->16]}], "  ", "|", " ", 
     RowBox[{"k", " ", "\[Epsilon]", " ", "\[DoubleStruckCapitalZ]"}]}], " ", 
    "}"}], TraditionalForm]]],
 ".\n                       \nThus, the exponential function of the elements \
of this set are given by\n\n ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["e", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"ln", "(", "r", ")"}], "/", "n"}], ")"}]], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"cos", "(", 
          StyleBox[
           FractionBox[
            RowBox[{"(", 
             RowBox[{"\[Theta]", "  ", "+", " ", 
              RowBox[{"2", " ", "k", " ", "\[Pi]"}]}], ")"}], "n"],
           FontSize->14], ")"}], "+", 
         RowBox[{"i", " ", 
          RowBox[{"sin", "(", 
           StyleBox[
            FractionBox[
             RowBox[{"(", 
              RowBox[{"\[Theta]", "  ", "+", " ", 
               RowBox[{"2", " ", "k", " ", "\[Pi]"}]}], ")"}], "n"],
            FontSize->14], ")"}]}]}], ")"}]}], "|", " ", 
      RowBox[{"k", " ", "\[Element]", " ", "\[DoubleStruckCapitalZ]"}]}], " ",
      "}"}], "\n"}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"   ", 
    RowBox[{"=", 
     RowBox[{
      RowBox[{"{", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["r", 
          RowBox[{"(", 
           RowBox[{"1", "/", "n"}], ")"}]], "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"cos", "(", 
            StyleBox[
             FractionBox[
              RowBox[{"(", 
               RowBox[{"\[Theta]", "  ", "+", " ", 
                RowBox[{"2", " ", "k", " ", "\[Pi]"}]}], ")"}], "n"],
             FontSize->14], ")"}], "+", 
           RowBox[{"i", " ", 
            RowBox[{"sin", "(", 
             StyleBox[
              FractionBox[
               RowBox[{"(", 
                RowBox[{"\[Theta]", "  ", "+", " ", 
                 RowBox[{"2", " ", "k", " ", "\[Pi]"}]}], ")"}], "n"],
              FontSize->14], ")"}]}]}], ")"}]}], "|", " ", 
        RowBox[{"k", " ", "\[Element]", " ", "\[DoubleStruckCapitalZ]"}]}], 
       " ", "}"}], "."}]}]}], TraditionalForm]}]],
 "\n\nNotice that for two different values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " which differ by ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", the arguments of the cosine and sine will differ by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", " ", "\[Pi]"}], TraditionalForm]]],
 ". Hence, we only have to consider the values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " from 0 to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"n", "-", "1"}], ")"}], TraditionalForm]]],
 ".  This gives us the set \n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["r", 
          RowBox[{"(", 
           RowBox[{"1", "/", "n"}], ")"}]], "\[CenterDot]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"cos", "(", 
            StyleBox[
             FractionBox[
              RowBox[{"(", 
               RowBox[{"\[Theta]", "  ", "+", " ", 
                RowBox[{"2", " ", "k", " ", "\[Pi]"}]}], ")"}], "n"],
             FontSize->14], ")"}], "+", 
           RowBox[{"i", " ", 
            RowBox[{"sin", "(", 
             StyleBox[
              FractionBox[
               RowBox[{"(", 
                RowBox[{"\[Theta]", "  ", "+", " ", 
                 RowBox[{"2", " ", "k", " ", "\[Pi]"}]}], ")"}], "n"],
              FontSize->14], ")"}]}]}], ")"}]}], "  ", "|", "\n", 
        "\t\t                                                                 \
", "k"}], "=", "0"}], ",", "1", ",", "2", ",", "\[Ellipsis]", ",", 
      RowBox[{"n", "-", "1"}]}], " ", "}"}], "."}], TraditionalForm]]],
 "\n\nHowever, these ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " solutions will have arguments that differ by less than ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", " ", "\[Pi]"}], ","}], TraditionalForm]]],
 " so these ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " solutions are distinct.\n\nFinally, we must show that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is an element of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["z", 
    RowBox[{"(", 
     RowBox[{"1", "/", "n"}], ")"}]], TraditionalForm]]],
 " if, and only if, ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " solves the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "n"], " ", "=", " ", "z"}], TraditionalForm]]],
 ". But for any element in the above expression, we have that\n\n ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "n"], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["r", 
       RowBox[{"n", "(", 
        RowBox[{"1", "/", "n"}], ")"}]], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"cos", "(", 
         StyleBox[
          FractionBox[
           RowBox[{"n", "(", 
            RowBox[{"\[Theta]", "  ", "+", " ", 
             RowBox[{"2", " ", "k", " ", "\[Pi]"}]}], ")"}], "n"],
          FontSize->14], ")"}], "+", 
        RowBox[{"i", " ", 
         RowBox[{"sin", "(", 
          StyleBox[
           FractionBox[
            RowBox[{"n", "(", 
             RowBox[{"\[Theta]", "  ", "+", " ", 
              RowBox[{"2", " ", "k", " ", "\[Pi]"}]}], ")"}], "n"],
           FontSize->14], ")"}]}]}], ")"}]}]}], "  "}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t  ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"r", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"cos", " ", "\[Theta]"}], " ", "+", " ", 
         RowBox[{"i", " ", "sin", " ", "\[Theta]"}]}], ")"}]}], " ", "=", " ",
       
      RowBox[{"z", "."}]}]}]}], TraditionalForm]}]],
 "\n\nLikewise, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["x", "n"],
     TraditionalForm], " ", "=", " ", "z"}], TraditionalForm]]],
 ", we can raise both sides to the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", "n"}], ")"}], "-", "th"}], TraditionalForm]]],
 " power to get that the two sets ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{
     FormBox[
      RowBox[{"(", 
       SuperscriptBox["x", "n"]}],
      TraditionalForm], ")"}], 
    RowBox[{"(", 
     RowBox[{"1", "/", "n"}], ")"}]], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["z", 
    RowBox[{"(", 
     RowBox[{"1", "/", "n"}], ")"}]], TraditionalForm]]],
 " are equal. Since the element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is certainly in the first set, it must also be in the set ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["z", 
    RowBox[{"(", 
     RowBox[{"1", "/", "n"}], ")"}]], TraditionalForm]]],
 "  that we have just computed."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4375324286875*^9, 3.43753243303125*^9}}]
}, Open  ]],

Cell["\<\
This last proposition is very useful for finding square roots and cube roots \
of complex numbers. This turns out to have some important applications in \
finding the roots of real polynomials! In fact, complex numbers and the \
functions we have defined in this section also have many applications in the \
real world. The complex exponiential function was fundamental to the \
invention of the short wave radio. The complex logarithm can be used in \
solving real valued differential equations. So even though these numbers are \
labled \"imaginary,\" they are by no means just a figment of someone's \
imagination.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Ordered Commutative Rings", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The integers, the rational numbers, and the real numbers all have one \
property that most rings do not have\[LongDash]these three rings have an ",
 StyleBox["ordering",
  FontSlant->"Italic"],
 ". That is, given two different elements in the ring, we say that one of \
them is greater than the other. Most rings do not have such an ordering, but \
we will find that some rings can be ordered in more than one way! The \
ordering of a ring can give us new insight into the structure of the ring.\n\n\
We begin by making a formal definition of an ordered ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". If there is a way to tell whether one element is greater than another, we \
should be able to distinquish those elements which are greater than zero, \
called the ",
 StyleBox["positive elements",
  FontSlant->"Italic"],
 "  ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". The main properties of positive numbers that we want to mimic is that the \
sum and product of two positive numbers yields positive numbers. Thus we have \
the following definition.\n\nDEFINITION 11.9:\nA ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is ",
 StyleBox["ordered",
  FontSlant->"Italic"],
 " if there exists a set ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " such that the three properties hold:\n\n1) ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is closed under addition.\n\n2) ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is closed under multiplication.\n\n3) For each ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", one and only one of the following statements is true:\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "\[Element]", "P"}], ","}], TraditionalForm]]],
 "   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "0"}], TraditionalForm]]],
 ",   or  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", "x"}], " ", "\[Element]", " ", "P"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373964371852503`*^9, 3.4373964961696253`*^9}, {
  3.437532525765625*^9, 3.437532527203125*^9}, {3.437535184828125*^9, 
  3.437535185296875*^9}}],

Cell[TextData[{
 "The third property is sometimes called the ",
 StyleBox["law of trichotomy",
  FontSlant->"Italic"],
 ". With this law, we can define what it means for one element to be greater \
than another.\n\nDEFINITION 11.10:\nWe say that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is greater than ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", denoted  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", ">", " ", "y"}], TraditionalForm]]],
 ",  if  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "-", "y"}], " ", "\[Element]", " ", "P"}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373965014508753`*^9, 3.4373965014508753`*^9}, {
  3.4375325375625*^9, 3.437532539078125*^9}, {3.437535194890625*^9, 
  3.437535195375*^9}}],

Cell[TextData[{
 "Likewise, we say that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is smaller than ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", denoted  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "<", " ", "y"}], TraditionalForm]]],
 ",  if  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", "-", "x"}], " ", "\[Element]", " ", "P"}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373965045446253`*^9, 3.4373965045446253`*^9}}],

Cell[TextData[{
 "By the law of trichotomy, either \n  \n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"x", " ", ">", " ", "y"}], ",", "     ", 
     RowBox[{"x", " ", "<", " ", "y"}], ",", "     ", 
     RowBox[{
      RowBox[{"or", "    ", "x"}], " ", "=", " ", 
      RowBox[{"y", "."}]}]}]}], TraditionalForm]]],
 "\n\nThis notation keeps us from having to constantly refer to the set ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Instead of writing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "\[Element]", " ", "P"}], TraditionalForm]]],
 ", we can merely write ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", ">", " ", "0"}], TraditionalForm]]],
 ". \n\nWe begin by proving some simple properties of the \"greater than\" \
sign.\n\nLEMMA 11.7\nIf ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " are elements in an ordered ring, then we have the following three \
properties:\n\n1) If  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"x", " ", ">", " ", "y"}]}], TraditionalForm]]],
 ",   then   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "+", "z"}], " ", ">", " ", 
    RowBox[{"y", "+", "z"}]}], TraditionalForm]]],
 ".\n\n2) If   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", ">", " ", "y"}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", " ", ">", " ", "0"}], TraditionalForm]]],
 ",  then  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "z"}], " ", ">", " ", 
    RowBox[{"y", "\[CenterDot]", "z"}]}], TraditionalForm]]],
 ".\n\n3) If   ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", ">", " ", "y"}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", ">", " ", "z"}], TraditionalForm]]],
 ",  then  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", ">", " ", "z"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373965111696253`*^9, 3.4373965111696253`*^9}, {
  3.437532563890625*^9, 3.437532566078125*^9}, {3.437532693203125*^9, 
  3.4375327028125*^9}, {3.437535209953125*^9, 3.4375352104375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "To prove the first statement, note that since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", ">", " ", "y"}], TraditionalForm]]],
 ", we have that\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "-", " ", "y"}], " ", "\[Element]", " ", 
    RowBox[{"P", "."}]}], TraditionalForm]]],
 "\nBut then \n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "+", " ", "z"}], ")"}], " ", "-", " ", 
      RowBox[{"(", 
       RowBox[{"y", "+", "z"}], ")"}]}], "  ", "\[Element]", " ", "P"}], 
    ","}], TraditionalForm]]],
 "\nand so \n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"x", " ", "+", " ", "z"}], " ", ">", " ", 
     RowBox[{"y", " ", "+", " ", 
      RowBox[{"z", "."}]}]}]}], TraditionalForm]]],
 " \n\nFor the second statement, we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", ">", " ", "y"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", " ", ">", " ", "0"}], TraditionalForm]]],
 ", and so\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", " ", "-", " ", "y"}], ")"}], " ", "\[Element]", " ", "P", 
    "  "}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", " ", "\[Element]", " ", "P"}], TraditionalForm]]],
 ".\n            \nSince ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is closed under multiplication, we have that\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "-", " ", "y"}], ")"}], "\[CenterDot]", "z"}], "  ", 
    "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "z"}], " ", "-", " ", 
      RowBox[{"y", "\[CenterDot]", "z"}]}], "  ", "\[Element]", " ", "P"}]}], 
   TraditionalForm]]],
 ",\n            \nand so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "z"}], " ", ">", " ", 
    RowBox[{"y", "\[CenterDot]", "z"}]}], TraditionalForm]]],
 ".\n\nFinally, if  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"x", " ", ">", " ", "y"}]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", ">", " ", "z"}], TraditionalForm]]],
 ", then both \n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "-", " ", "y"}], "  ", "\[Element]", " ", "P"}], 
   TraditionalForm]]],
 "   and   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"y", " ", "-", " ", "z"}], " ", "\[Element]", " ", "P"}], 
   TraditionalForm]]],
 ".\n            \nSince ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is closed under addition, we have that \n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "-", " ", "y"}], " ", ")"}], " ", "+", " ", 
     RowBox[{"(", 
      RowBox[{"y", " ", "-", " ", "z"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"x", " ", "-", " ", "z"}], "  ", "\[Element]", " ", "P"}]}], 
   TraditionalForm]]],
 ",\n               \nand so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", ">", " ", "z"}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4373965185446253`*^9, 3.4373965369352503`*^9}, {
  3.4375327265*^9, 3.437532763546875*^9}, {3.437535230859375*^9, 
  3.437535230859375*^9}}]
}, Open  ]],

Cell[TextData[{
 "Given a ring that has an ordering, one of the great challenges is \
determining the set of positive elements ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Are there any elements of an ordered ring that we know ",
 StyleBox["must",
  FontSlant->"Italic"],
 " be in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 "?  Yes, there are at least some elements given by the next proposition.\n\n\
PROPOSITION 11.8\nFor any non-zero element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in an ordered ring, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "2"], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.437532782328125*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is non-zero, by the law of trichotomy either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", ">", " ", "0"}], TraditionalForm]]],
 ", or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", "x"}], " ", ">", " ", "0"}], TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", ">", "0"}], TraditionalForm]]],
 " then \n                                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "=", " ", 
    RowBox[{
     RowBox[{"x", "\[CenterDot]", "x"}], " ", ">", " ", "0."}]}], 
   TraditionalForm]]],
 " \n\nOn the other hand, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", "x"}], " ", ">", " ", "0"}], TraditionalForm]]],
 ", then \n\n                             ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox["x", "2"], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", "x"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"-", "x"}], ")"}]}], " ", ">", " ", "0."}]}]}], 
   TraditionalForm]]],
 " \n\nThus, in either case ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "2"], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "\nAn immediate consequence of this is that if the ring has an identity ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"e", " ", ">", " ", "0"}], TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"e", " ", "=", " ", 
    FormBox[
     SuperscriptBox["e", "2"],
     TraditionalForm]}], TraditionalForm]]],
 ". An additional statement can be proved if the ring is an integral domain.\n\
\nCOROLLARY 11.2\nIf ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is an ordered integral domain with multiplicative identity 1, and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is any positive integer, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[CenterDot]", "1"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". In particular, the characteristic of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " must be ",
 Cell[BoxData[
  FormBox["0.", TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43753282403125*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      SuperscriptBox["1", "2"],
      TraditionalForm], " ", "=", " ", "1"}], ","}], TraditionalForm]]],
 " we have from proposition 11.8 that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", ">", " ", "0"}], TraditionalForm]]],
 ". Proceeding by induction, let us assume that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"n", "-", "1"}], ")"}], "\[CenterDot]", "1"}], " ", ">", "0"}], 
   TraditionalForm]]],
 ", and show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "1"}], " ", ">", " ", "0"}], 
   TraditionalForm]]],
 ". But this is easy, since\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "1"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"n", "-", "1"}], ")"}], "\[CenterDot]", "1"}], " ", "+", " ", 
      
      RowBox[{"1", "\[CenterDot]", "1"}]}], " ", "=", "  ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"n", "-", "1"}], ")"}], "\[CenterDot]", "1"}], " ", "+", " ",
        "1"}], " ", ">", " ", "0."}]}]}], TraditionalForm]]],
 "\n\nThus, we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[CenterDot]", "1"}], " ", ">", " ", "0"}], 
   TraditionalForm]]],
 " for every positive number ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". This immediately implies that the characteristic is zero, for if ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " had a positive characteristic ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "\[CenterDot]", "1"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 ", and we would have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", " ", ">", " ", "0"}], TraditionalForm]]],
 ", a contradiction."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4375328365*^9, 3.437532837015625*^9}, {
  3.4419768651202497`*^9, 3.4419768901983747`*^9}}]
}, Open  ]],

Cell[TextData[{
 "The standard examples of ordered rings are the integers, the rationals, and \
the real numbers. It should be noted that the complex numbers do ",
 StyleBox["not",
  FontSlant->"Italic"],
 " form an ordered ring, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["i", "2"], " ", "=", " ", 
     RowBox[{
      RowBox[{"-", "1"}], " ", "<", " ", "0"}]}], ","}], TraditionalForm]]],
 " and by proposition 11.8, any square must be positive.\n\nHere is an very \
different example of an ordered integral domain. Consider all numbers of the \
form \n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"y", 
     RadicalBox["2", " "]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.437532868484375*^9}],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are integers. This forms a ring, since the product of any two such \
numbers"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ClearDefs;
Expand[(x1 + y1 Sqrt[2])*(x2 + y2 Sqrt[2])]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  RowBox[{"x1", " ", "x2"}], "+", 
  RowBox[{
   SqrtBox["2"], " ", "x2", " ", "y1"}], "+", 
  RowBox[{
   SqrtBox["2"], " ", "x1", " ", "y2"}], "+", 
  RowBox[{"2", " ", "y1", " ", "y2"}]}]], "Output",
 CellChangeTimes->{3.44275486319575*^9}]
}, Open  ]],

Cell[TextData[{
 "yields a number of the same form. We will call this ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    RadicalBox["2", " "], "]"}], TraditionalForm]]],
 ", the ring formed by ajoining the square root of 2 to ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". By propostion 9.3, this ring has no zero divisors, so this is an integral \
domain.\n\nSince all elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    RadicalBox["2", " "], "]"}], TraditionalForm]]],
 " are real, we could use the standard ordering, letting ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " consist of all numbers which, when viewed as a real number, are positive. \
But we could order these numbers in a non-standard way! By corollary 11.2, \
the positive integers must be in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", but there is no way of proving that the square root of 2 is in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ".  (There is no number in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Z", "[", 
    SqrtBox["2"], "]"}], TraditionalForm]]],
 " whose square is ",
 Cell[BoxData[
  FormBox[
   SqrtBox["2"], TraditionalForm]]],
 ".)\nThus, there is nothing preventing us from stating that ",
 Cell[BoxData[
  FormBox[
   SqrtBox["2"], TraditionalForm]]],
 " is negative, and therefore ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    SqrtBox["2"]}], TraditionalForm]]],
 "  would be in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 "! The rest of the elements would be identified as either positive or \
negative accordingly.\n\nFor example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "+", " ", 
    SqrtBox[
     RowBox[{"2", " "}]]}], TraditionalForm]]],
 "would be negative, since\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", " ", "+", " ", 
       RadicalBox["2", " "]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RadicalBox["2", " "]}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"-", "1"}], " ", "<", " ", "0"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437532884046875*^9, 3.43753290434375*^9}}],

Cell[TextData[{
 "and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "-", " ", 
    SqrtBox["2"]}], TraditionalForm]]],
 "  is the sum of two positive numbers, so this term is positive.\n\n\
EXPERIMENT:\nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RadicalBox["2", "  "], " ", "<", "0"}], TraditionalForm]]],
 ", is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", " ", "+", " ", 
    RowBox[{"2", " ", 
     SqrtBox["2"]}]}], TraditionalForm]]],
 "  positive or negative? \n(Hint: multiply by the \"conjugate\" as we did in \
the last example.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
To see what is really going on in this example, it is helpful to look at the \
ring automorphisms, which were introduced in the last section.  The \
automorphism of particular interest is as follows:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4375329585*^9, 3.437532966875*^9}, 3.4375332076875*^9, {
   3.437535309*^9, 3.437535372796875*^9}}],

Cell[TextData[{
 "      \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", ":", " ", 
    RowBox[{
     RowBox[{"\[DoubleStruckCapitalZ]", "[", 
      FormBox[
       RadicalBox["2", " "],
       TraditionalForm], "]"}], " ", "\[RightArrow]", " ", 
     RowBox[{"\[DoubleStruckCapitalZ]", "[", 
      RadicalBox["2", " "], "]"}]}]}], TraditionalForm]]],
 ",\n                 \n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"x", "+", 
      RowBox[{"y", 
       RadicalBox["2", " "]}]}], ")"}], " ", "=", " ", 
    RowBox[{"x", "-", 
     RowBox[{"y", 
      RadicalBox["2", " "]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43753536803125*^9, 3.4375353760625*^9}}],

Cell[TextData[{
 "This automorphism can be defined in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". We define the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    RadicalBox["2", " "], "]"}], TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["InitDomain[0]  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Since ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " already knows that Sqrt[2] \[CenterDot] Sqrt[2] = 2, we do not need to \
tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " anything else to define the ring!\n\nWe now can define the homomorphism:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Homomorph[F];
Define[ F[Sqrt[2]], - Sqrt[2]]\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "1", "]"}], TraditionalForm]]],
 " must be 1, this is all we need to define the homomorphism. We can check \
that this is a homomophism with the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[F,{1,Sqrt[2]}]", "Input",
 PageWidth->Infinity],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.442754873930125*^9}]
}, Open  ]],

Cell[TextData[{
 "Since we are working with an infinite ring, we included only a basis for \
the ring instead of the whole ring as the second argument of the CheckHomo \
command. \n\nWhat is the relationship between this homomorphism and the \
unusual ordering that we gave the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["2"], "]"}], TraditionalForm]]],
 "? If we let ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " denote the set of positive elements using the \"standard\" ordering, and \
let ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"P", " "}], "\[Prime]"], TraditionalForm]]],
 " be the set of positive elements under the unusual ordering we saw above, \
then\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SuperscriptBox["P", 
      RowBox[{" ", "\[Prime]"}]], "=", " ", 
     RowBox[{"\[Florin]", "(", "P", ")"}]}]}], TraditionalForm]]],
 "\n\nIn fact, if ",
 Cell[BoxData[
  FormBox["\[Phi]", TraditionalForm]]],
 " is any automorphism on an ordered ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " being the set of positive elements, then there is an alternative way the \
order the ring by using ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "(", "P", ")"}], TraditionalForm]]],
 " as the set of positive elements.\n\nWhile we are working with the integral \
domain ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["2"], "]"}], TraditionalForm]]],
 " we might mention what happens if we consider the field of quotients of \
this ring. Certainly this must include all numbers of the form\n\n            \
          ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "+", 
    RowBox[{"y", 
     RadicalBox["2", " "]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are ",
 StyleBox["rational",
  FontSlant->"Italic"],
 " numbers. But are there other elements? We need to check that all elements \
of this form have a multiplicative inverse. For example, is"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["1/(1 + Sqrt[2])", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{"1", "+", 
   SqrtBox["2"]}]]], "Output",
 CellChangeTimes->{3.442754878648875*^9}]
}, Open  ]],

Cell[TextData[{
 "of this form? There is way to check all such expressions with the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command ",
 StyleBox["CheckField",
  FontWeight->"Bold"],
 ". By including enough elements to additively generate the field, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will try to find a formula for the inverse of any element."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitDomain[0]
CheckField[{1,Sqrt[2]}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Identity element = \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Identity element = ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.442754882992625*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"An element is invertable as long as \"\>", "\[InvisibleSpace]",
    
   RowBox[{
    SuperscriptBox[
     RowBox[{"C", "[", "1", "]"}], "2"], "-", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"C", "[", "2", "]"}], "2"]}]}], 
   "\[InvisibleSpace]", "\<\" is not zero.\"\>", 
   "\[InvisibleSpace]", "\<\" \"\>"}],
  SequenceForm[
  "An element is invertable as long as ", C[1]^2 - 2 C[2]^2, " is not zero.", 
   " "],
  Editable->False]], "Print",
 CellChangeTimes->{3.442754883023875*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"If the only element which is not invertable is 0,\"\>", 
   "\[InvisibleSpace]", "\<\" then this is a field.\"\>", 
   "\[InvisibleSpace]", "\<\" \"\>"}],
  SequenceForm[
  "If the only element which is not invertable is 0,", 
   " then this is a field.", " "],
  Editable->False]], "Print",
 CellChangeTimes->{3.4427548830395*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The inverse of \"\>", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{"C", "[", "1", "]"}], "+", 
    RowBox[{
     SqrtBox["2"], " ", 
     RowBox[{"C", "[", "2", "]"}]}]}], 
   "\[InvisibleSpace]", "\<\" is given by \"\>", "\[InvisibleSpace]", 
   FractionBox[
    RowBox[{
     RowBox[{"C", "[", "1", "]"}], "-", 
     RowBox[{
      SqrtBox["2"], " ", 
      RowBox[{"C", "[", "2", "]"}]}]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"C", "[", "1", "]"}], "2"], "-", 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"C", "[", "2", "]"}], "2"]}]}]]}],
  SequenceForm[
  "The inverse of ", C[1] + 2^Rational[1, 2] C[2], 
   " is given by ", (C[1] - 2^Rational[1, 2] C[2])/(C[1]^2 - 2 C[2]^2)],
  Editable->False]], "Print",
 CellChangeTimes->{3.442754883055125*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " comes up with a formula for the inverse of \n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"C", "[", "1", "]"}], " ", "+", " ", 
     RowBox[{
      RowBox[{"C", "[", "2", "]"}], " ", 
      RowBox[{"Sqrt", "[", "2", "]"}]}]}]}], TraditionalForm]]],
 "\nto be\n                  ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    FractionBox[
     RowBox[{
      RowBox[{"C", "[", "1", "]"}], " ", "-", " ", 
      RowBox[{
       RowBox[{"C", "[", "2", "]"}], " ", 
       RowBox[{"Sqrt", "[", "2", "]"}]}]}], 
     RowBox[{
      SuperscriptBox[
       RowBox[{"C", "[", "1", "]"}], "2"], " ", "-", " ", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "2", "]"}], "2"]}]}]],
    FontSize->16], TraditionalForm]]],
 "\n\nwhich is in the correct form. By proposition 1.3, the donominator will \
not be zero for rational ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "[", "1", "]"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "[", "2", "]"}], TraditionalForm]]],
 ", so this is a field. We will call this field ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["2"], "]"}], TraditionalForm]]],
 ". \n\nThe command ",
 StyleBox["CheckField",
  FontWeight->"Bold"],
 " actually does more than just verify that a field is possible, it actually \
defines the field in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "! Note that the expression"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["1/(1 + Sqrt[2])", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "1"}], "+", 
  SqrtBox["2"]}]], "Output",
 CellChangeTimes->{3.442754889773875*^9}]
}, Open  ]],

Cell[TextData[{
 "now simplifies into the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"y", 
     FormBox[
      SqrtBox["2"],
      TraditionalForm]}]}], TraditionalForm]]],
 ". Once the CheckField command sucessfully finds a field, it updates the \
definition of division to simplify any operations performed in the field. \
Thus, even"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Expand[(2 - 5 Sqrt[2])/(1 + Sqrt[2])]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "12"}], "+", 
  RowBox[{"7", " ", 
   SqrtBox["2"]}]}]], "Output",
 CellChangeTimes->{3.442754892727*^9}]
}, Open  ]],

Cell[TextData[{
 "simplifies. \n\nNow that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["2"], "]"}], TraditionalForm]]],
 " is defined in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", the automorphism ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " extends to an automorphism on the field ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["2"], "]"}], TraditionalForm]]],
 ". Thus, the unusual ordering that we gave to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["2"], "]"}], TraditionalForm]]],
 " should also extend to the field of quotients. In fact this generally \
happens, as seen in the next proposition.\n\nPROPOSITION 11.9\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be an ordered integral domain, with ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " the set of positive elements. Then if ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " is the field of quotients on ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", then the ordering on ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " can be extended in a unique way to an ordering on ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ". That is, there is a unique set ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["P", 
    RowBox[{" ", "\[Prime]"}]], TraditionalForm]]],
 " which forms an ordering on ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", with\n\n                               ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", " ", "\[Element]", " ", 
    RowBox[{"P", "  ", "\[DoubleLongRightArrow]", " ", 
     RowBox[{"(", 
      FractionBox["p", "1"], ")"}]}], " ", "\[Element]", " ", 
    SuperscriptBox["P", 
     RowBox[{"\[Prime]", " "}]]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{
  3.43753548478125*^9, {3.437535518375*^9, 3.437535526578125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We will begin by showing that the ordering is uniquely determined. Since \
for any ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", we have \n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      FractionBox["1", "p"], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      FractionBox["p", "1"], ")"}]}], "=", 
    RowBox[{
     RowBox[{"(", 
      FractionBox["p", "p"], ")"}], "=", 
     RowBox[{
      RowBox[{"(", 
       FractionBox["1", "1"], ")"}], "=", 
      RowBox[{"1", " ", "\[Element]", " ", "P"}]}]}]}], TraditionalForm]]],
 ",\n         \n",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    FractionBox["1", "p"], ")"}], TraditionalForm]]],
 " must be considered to be positive in the new ordering. But then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    FractionBox["n", "p"], ")"}], TraditionalForm]]],
 " must be positive whenever ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Thus ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["P", 
    RowBox[{" ", "\[Prime]"}]], TraditionalForm]]],
 " contains at least those elements of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    FractionBox["n", "p"], ")"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Note that every non-zero element in the field of quotients ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 " must be of one of the 4 forms\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     FractionBox["n", "p"], ")"}], ",", "  ", 
    RowBox[{"(", 
     FractionBox[
      RowBox[{"-", "n"}], "p"], ")"}], ",", "  ", 
    RowBox[{"(", 
     FractionBox["n", 
      RowBox[{"-", "p"}]], ")"}], ",", "  ", 
    RowBox[{"(", 
     FractionBox[
      RowBox[{"-", "n"}], 
      RowBox[{"-", "p"}]], ")"}], ","}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". But the first and the last expressions are equivalent, and the middle two \
are also equivalent. Thus, for every non-zero element of ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", either that element or its negative is of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    FractionBox["n", "p"], ")"}], TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["P", 
    RowBox[{" ", "\[Prime]"}]], TraditionalForm]]],
 " cannot contain any more elements besides those of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    FractionBox["n", "p"], ")"}], TraditionalForm]]],
 ", and hence ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["P", 
    RowBox[{" ", "\[Prime]"}]], TraditionalForm]]],
 " is uniquely determined.\n\nNow, suppose we consider the set of elements ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["P", 
    RowBox[{" ", "\[Prime]"}]], TraditionalForm]]],
 " which can be expressed in the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    FractionBox["n", "p"], ")"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Does this form an ordering on ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 "? We have already seen that the law of trichotomy has already been \
demonstrated. All we need to show is that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["P", 
    RowBox[{" ", "\[Prime]"}]], TraditionalForm]]],
 " is closed under addition and multiplication. But this is clear by looking \
at the formulas\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      FractionBox["x", "y"], ")"}], " ", "+", 
     RowBox[{"(", 
      FractionBox["u", "v"], ")"}]}], " ", "=", " ", 
    RowBox[{"(", 
     FractionBox[
      RowBox[{
       RowBox[{"x", "\[CenterDot]", "v"}], " ", "+", " ", 
       RowBox[{"u", "\[CenterDot]", "y"}]}], 
      RowBox[{"y", "\[CenterDot]", "v"}]], ")"}]}], TraditionalForm]]],
 "\nand\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      FractionBox["x", "y"], ")"}], " ", "\[CenterDot]", " ", 
     RowBox[{"(", 
      FractionBox["u", "v"], ")"}]}], " ", "=", 
    RowBox[{
     RowBox[{"(", 
      FractionBox[
       RowBox[{"x", "\[CenterDot]", "u"}], 
       RowBox[{"y", "\[CenterDot]", "v"}]], ")"}], "."}]}], 
   TraditionalForm]]],
 "\n\nThus, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["P", 
    RowBox[{" ", "\[Prime]"}]], TraditionalForm]]],
 " forms an ordering on ",
 Cell[BoxData[
  FormBox["Q", TraditionalForm]]],
 ", and is an extension of the ordering ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437535529703125*^9, 3.437535529703125*^9}}]
}, Open  ]],

Cell[TextData[{
 "What if we had considered ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    RadicalBox["2", "3"], "]"}], TraditionalForm]]],
 " instead of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["2"], "]"}], TraditionalForm]]],
 " ? We can write ",
 Cell[BoxData[
  FormBox[
   RadicalBox["2", "3"], TraditionalForm]]],
 "  as  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    RowBox[{"(", 
     RowBox[{"1", "/", "3"}], ")"}]], TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", and use the checkfield to see what happens."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitDomain[0]
CheckField[{1,2^(1/3)}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData["\<\"Additive group generated is not closed under \
multiplication.\"\>"], "Print",
 CellChangeTimes->{3.442754899680125*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " complains about this not being closed under multiplication. No wonder, \
since \n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["2", 
       RowBox[{"(", 
        RowBox[{"1", "/", "3"}], ")"}]], " ", "\[CenterDot]", " ", 
      SuperscriptBox["2", 
       RowBox[{"(", 
        RowBox[{"1", "/", "3"}], ")"}]]}], " ", "=", " ", 
     SuperscriptBox["2", 
      RowBox[{"(", 
       RowBox[{"2", "/", "3"}], ")"}]]}], ","}], TraditionalForm]]],
 " \n\nwhich yields a term we didn't consider. Thus, we have to include all \
numbers of the form\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "+", 
     RowBox[{"y", 
      RadicalBox["2", "3"]}], "+", " ", 
     RowBox[{"z", 
      RadicalBox["4", "3"]}]}], "  ", "\[Element]", "  ", 
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     RadicalBox["2", "3"], "]"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437535685875*^9, 3.437535692296875*^9}}],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we would enter this as:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["x + y 2^(1/3) + z 2^(2/3)", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437535695984375*^9, 3.4375356963125*^9}}],

Cell[BoxData[
 RowBox[{"x", "+", 
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"1", "/", "3"}]], " ", "y"}], "+", 
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"2", "/", "3"}]], " ", "z"}]}]], "Output",
 CellChangeTimes->{3.442754908336375*^9}]
}, Open  ]],

Cell["\<\
We can check if such expressions are now closed under multiplication:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
Expand[(x1 + y1 2^(1/3) + z1 2^(2/3))*
       (x2 + y2 2^(1/3) + z2 2^(2/3))]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4375356996875*^9, 3.43753570075*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"x1", " ", "x2"}], "+", 
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"1", "/", "3"}]], " ", "x2", " ", "y1"}], "+", 
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"1", "/", "3"}]], " ", "x1", " ", "y2"}], "+", 
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"2", "/", "3"}]], " ", "y1", " ", "y2"}], "+", 
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"2", "/", "3"}]], " ", "x2", " ", "z1"}], "+", 
  RowBox[{"2", " ", "y2", " ", "z1"}], "+", 
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"2", "/", "3"}]], " ", "x1", " ", "z2"}], "+", 
  RowBox[{"2", " ", "y1", " ", "z2"}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox["2", 
    RowBox[{"1", "/", "3"}]], " ", "z1", " ", "z2"}]}]], "Output",
 CellChangeTimes->{3.44275491075825*^9}]
}, Open  ]],

Cell[TextData[{
 "This is a mess, but it is clear that the product is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    RadicalBox["2", "3"], "]"}], TraditionalForm]]],
 ". Let us see if we have a field."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitDomain[0]
CheckField[{1,2^(1/3),2^(2/3)}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Identity element = \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Identity element = ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.442754914492625*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"An element is invertable as long as \"\>", "\[InvisibleSpace]",
    
   RowBox[{
    SuperscriptBox[
     RowBox[{"C", "[", "1", "]"}], "3"], "+", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"C", "[", "2", "]"}], "3"]}], "-", 
    RowBox[{"6", " ", 
     RowBox[{"C", "[", "1", "]"}], " ", 
     RowBox[{"C", "[", "2", "]"}], " ", 
     RowBox[{"C", "[", "3", "]"}]}], "+", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"C", "[", "3", "]"}], "3"]}]}], 
   "\[InvisibleSpace]", "\<\" is not zero.\"\>", 
   "\[InvisibleSpace]", "\<\" \"\>"}],
  SequenceForm[
  "An element is invertable as long as ", C[1]^3 + 2 C[2]^3 - 6 C[1] C[2] 
   C[3] + 4 C[3]^3, " is not zero.", " "],
  Editable->False]], "Print",
 CellChangeTimes->{3.44275491450825*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"If the only element which is not invertable is 0,\"\>", 
   "\[InvisibleSpace]", "\<\" then this is a field.\"\>", 
   "\[InvisibleSpace]", "\<\" \"\>"}],
  SequenceForm[
  "If the only element which is not invertable is 0,", 
   " then this is a field.", " "],
  Editable->False]], "Print",
 CellChangeTimes->{3.442754914523875*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The inverse of \"\>", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{"C", "[", "1", "]"}], "+", 
    RowBox[{
     SuperscriptBox["2", 
      RowBox[{"1", "/", "3"}]], " ", 
     RowBox[{"C", "[", "2", "]"}]}], "+", 
    RowBox[{
     SuperscriptBox["2", 
      RowBox[{"2", "/", "3"}]], " ", 
     RowBox[{"C", "[", "3", "]"}]}]}], 
   "\[InvisibleSpace]", "\<\" is given by \"\>", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"C", "[", "1", "]"}], "2"], "-", 
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{"1", "/", "3"}]], " ", 
       RowBox[{"C", "[", "1", "]"}], " ", 
       RowBox[{"C", "[", "2", "]"}]}], "+", 
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{"2", "/", "3"}]], " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "2", "]"}], "2"]}], "-", 
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{"2", "/", "3"}]], " ", 
       RowBox[{"C", "[", "1", "]"}], " ", 
       RowBox[{"C", "[", "3", "]"}]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{"C", "[", "2", "]"}], " ", 
       RowBox[{"C", "[", "3", "]"}]}], "+", 
      RowBox[{"2", " ", 
       SuperscriptBox["2", 
        RowBox[{"1", "/", "3"}]], " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "3", "]"}], "2"]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"C", "[", "1", "]"}], "3"], "+", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "2", "]"}], "3"]}], "-", 
      RowBox[{"6", " ", 
       RowBox[{"C", "[", "1", "]"}], " ", 
       RowBox[{"C", "[", "2", "]"}], " ", 
       RowBox[{"C", "[", "3", "]"}]}], "+", 
      RowBox[{"4", " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "3", "]"}], "3"]}]}], ")"}]}]}],
  SequenceForm[
  "The inverse of ", C[1] + 2^Rational[1, 3] C[2] + 2^Rational[2, 3] C[3], 
   " is given by ", (C[1]^2 - 2^Rational[1, 3] C[1] C[2] + 
    2^Rational[2, 3] C[2]^2 - 2^Rational[2, 3] C[1] C[3] - 2 C[2] C[3] + 
    2 2^Rational[1, 3] C[3]^2)/(C[1]^3 + 2 C[2]^3 - 6 C[1] C[2] C[3] + 
   4 C[3]^3)],
  Editable->False]], "Print",
 CellChangeTimes->{3.44275491457075*^9}]
}, Open  ]],

Cell["\<\
We have successfully defined a field! We may now perform operations in this \
field.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["1/(1 + 2^(1/3) - 3 2^(2/3))", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "87"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "7"}], "-", 
    RowBox[{"17", " ", 
     SuperscriptBox["2", 
      RowBox[{"1", "/", "3"}]]}], "-", 
    RowBox[{"4", " ", 
     SuperscriptBox["2", 
      RowBox[{"2", "/", "3"}]]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.442754920336375*^9}]
}, Open  ]],

Cell[TextData[{
 "Does this field have an unusual ordering, as the field ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["2"], "]"}], TraditionalForm]]],
 " did? Note that in this field, \n\n            2",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RadicalBox["2", "3"], " ", "=", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RadicalBox["4", "3"], ")"}], "2"], " ", ">", " ", "0."}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "So by proposition 11.8, the cube root of 2 must be positive. Likewise, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    RowBox[{"(", 
     RowBox[{"2", "/", "3"}], ")"}]], TraditionalForm]]],
 " must be positive, since it is a perfect square. It seems that there is \
little chance for finding an unusual ordering for this field. To eliminate \
this chance altogether, note that\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", 
        RadicalBox["2", "3"]}], " ", "-", " ", "5"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"16", 
        RadicalBox["4", "3"]}], " ", "+", " ", 
       RowBox[{"20", " ", 
        RadicalBox["2", "3"]}], " ", "+", " ", "25"}], ")"}]}], " ", "=", " ",
     
    RowBox[{
     RowBox[{"128", "-", "125"}], " ", "=", " ", 
     RowBox[{"3", " ", ">", " ", "0."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437535796328125*^9, 3.43753579846875*^9}}],

Cell[TextData[{
 "Hence, we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"4", " ", 
      RadicalBox["2", "3"]}], "  ", "-", " ", "5", " "}]}], 
   TraditionalForm]]],
 " must be positive, even though numerically it is very close to 0.\n\nThe \
reason there is only one way to order this field stems from the fact that \
there is only one automorphism on this field\[LongDash]the trivial one. For \
if ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an automorphism of  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    FormBox[
     RadicalBox["2", "3"],
     TraditionalForm], "]"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", " ", 
    RadicalBox["2", "3"], " ", ")"}], TraditionalForm]]],
 "  must be a number whose cube is 2. But there is only one such ",
 StyleBox["real",
  FontSlant->"Italic"],
 " number in this field. Thus,\n\n                                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RadicalBox["2", "3"], ")"}], " ", "=", " ", 
     RadicalBox["2", "3"]}], ","}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "and therefore ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", "x", ")"}], " ", "=", " ", "x"}], 
   TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in the field.\n\nLet us look at one last example of a field with several \
automorphisms, and therefore several posible ways of defining an ordering on \
the field. Consider the set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " of numbers of the form\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"y", "  ", 
     RowBox[{"cos", "(", 
      FractionBox["\[Pi]", "9"], ")"}]}], " ", "+", " ", 
    RowBox[{"z", "  ", 
     RowBox[{"cos", "(", 
      FractionBox[
       RowBox[{"2", "\[Pi]"}], "9"], ")"}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " are rational numbers. To multiply two such numbers together, we need to \
use some trigometric identities:\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["cos", "2"], "(", "x", ")"}], " ", "=", " ", 
    StyleBox[
     FractionBox[
      RowBox[{"1", " ", "+", " ", 
       RowBox[{"cos", "(", 
        RowBox[{"2", "x"}], ")"}]}], "2"],
     FontSize->16]}], TraditionalForm]]],
 ",\n                  \n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"cos", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"cos", "(", "y", ")"}]}], " ", "=", " ", 
    StyleBox[
     FractionBox[
      RowBox[{
       RowBox[{"cos", "(", 
        RowBox[{"x", "+", "y"}], ")"}], " ", "+", " ", 
       RowBox[{"cos", "(", 
        RowBox[{"x", "-", "y"}], ")"}]}], "2"],
     FontSize->16]}], TraditionalForm]]],
 "\nand\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"cos", "(", 
     FractionBox[
      RowBox[{"4", "\[Pi]"}], "9"], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"cos", "(", 
      FractionBox["\[Pi]", "9"], ")"}], " ", "-", " ", 
     RowBox[{"cos", "(", 
      FractionBox[
       RowBox[{"2", "\[Pi]"}], "9"], ")"}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
EXPERIMENT:
Can you derive the last trig identity from the other two?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is already familiar with these trignometric identities. Thus, we can see \
that the product of two elements of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " as follows:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
Expand[(x1 + y1 Cos[Pi/9] + z1 Cos[2 Pi/9])* 
       (x2 + y2 Cos[Pi/9] + z2 Cos[2 Pi/9])]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  RowBox[{"x1", " ", "x2"}], "+", 
  FractionBox[
   RowBox[{"y1", " ", "y2"}], "2"], "+", 
  FractionBox[
   RowBox[{"y2", " ", "z1"}], "4"], "+", 
  FractionBox[
   RowBox[{"y1", " ", "z2"}], "4"], "+", 
  FractionBox[
   RowBox[{"z1", " ", "z2"}], "2"], "+", 
  RowBox[{"x2", " ", "y1", " ", 
   RowBox[{"Cos", "[", 
    FractionBox["\[Pi]", "9"], "]"}]}], "+", 
  RowBox[{"x1", " ", "y2", " ", 
   RowBox[{"Cos", "[", 
    FractionBox["\[Pi]", "9"], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "y2", " ", "z1", " ", 
   RowBox[{"Cos", "[", 
    FractionBox["\[Pi]", "9"], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "y1", " ", "z2", " ", 
   RowBox[{"Cos", "[", 
    FractionBox["\[Pi]", "9"], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "z1", " ", "z2", " ", 
   RowBox[{"Cos", "[", 
    FractionBox["\[Pi]", "9"], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "y1", " ", "y2", " ", 
   RowBox[{"Cos", "[", 
    FractionBox[
     RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}], "+", 
  RowBox[{"x2", " ", "z1", " ", 
   RowBox[{"Cos", "[", 
    FractionBox[
     RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}], "+", 
  RowBox[{"x1", " ", "z2", " ", 
   RowBox[{"Cos", "[", 
    FractionBox[
     RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", "z1", " ", "z2", " ", 
   RowBox[{"Cos", "[", 
    FractionBox[
     RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}]}]], "Output",
 CellChangeTimes->{3.4419769329952497`*^9, 3.442754927773875*^9}]
}, Open  ]],

Cell[TextData[{
 "As messy as this is, one can see that it is an element of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " is closed under multiplication. We can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " check that this is a field."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitDomain[0]
CheckField[{1, Cos[Pi/9], Cos[2 Pi/9]}]\
\>", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Identity element = \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Identity element = ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.442754930852*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"An element is invertable as long as \"\>", "\[InvisibleSpace]",
    
   RowBox[{
    FractionBox["1", "8"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"8", " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "1", "]"}], "3"]}], "-", 
      RowBox[{"6", " ", 
       RowBox[{"C", "[", "1", "]"}], " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "2", "]"}], "2"]}], "+", 
      SuperscriptBox[
       RowBox[{"C", "[", "2", "]"}], "3"], "-", 
      RowBox[{"6", " ", 
       RowBox[{"C", "[", "1", "]"}], " ", 
       RowBox[{"C", "[", "2", "]"}], " ", 
       RowBox[{"C", "[", "3", "]"}]}], "+", 
      RowBox[{"6", " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "2", "]"}], "2"], " ", 
       RowBox[{"C", "[", "3", "]"}]}], "-", 
      RowBox[{"6", " ", 
       RowBox[{"C", "[", "1", "]"}], " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "3", "]"}], "2"]}], "+", 
      RowBox[{"3", " ", 
       RowBox[{"C", "[", "2", "]"}], " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "3", "]"}], "2"]}], "-", 
      SuperscriptBox[
       RowBox[{"C", "[", "3", "]"}], "3"]}], ")"}]}], 
   "\[InvisibleSpace]", "\<\" is not zero.\"\>", 
   "\[InvisibleSpace]", "\<\" \"\>"}],
  SequenceForm[
  "An element is invertable as long as ", 
   Rational[1, 8] (8 C[1]^3 - 6 C[1] C[2]^2 + C[2]^3 - 6 C[1] C[2] C[3] + 
    6 C[2]^2 C[3] - 6 C[1] C[3]^2 + 3 C[2] C[3]^2 - C[3]^3), " is not zero.", 
   " "],
  Editable->False]], "Print",
 CellChangeTimes->{3.442754930867625*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"If the only element which is not invertable is 0,\"\>", 
   "\[InvisibleSpace]", "\<\" then this is a field.\"\>", 
   "\[InvisibleSpace]", "\<\" \"\>"}],
  SequenceForm[
  "If the only element which is not invertable is 0,", 
   " then this is a field.", " "],
  Editable->False]], "Print",
 CellChangeTimes->{3.44275493088325*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The inverse of \"\>", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{"C", "[", "1", "]"}], "+", 
    RowBox[{
     RowBox[{"C", "[", "2", "]"}], " ", 
     RowBox[{"Cos", "[", 
      FractionBox["\[Pi]", "9"], "]"}]}], "+", 
    RowBox[{
     RowBox[{"C", "[", "3", "]"}], " ", 
     RowBox[{"Cos", "[", 
      FractionBox[
       RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}]}], 
   "\[InvisibleSpace]", "\<\" is given by \"\>", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox[
          RowBox[{"C", "[", "1", "]"}], "2"]}], "-", 
        SuperscriptBox[
         RowBox[{"C", "[", "2", "]"}], "2"], "-", 
        RowBox[{
         RowBox[{"C", "[", "2", "]"}], " ", 
         RowBox[{"C", "[", "3", "]"}]}], "-", 
        SuperscriptBox[
         RowBox[{"C", "[", "3", "]"}], "2"], "-", 
        RowBox[{"4", " ", 
         RowBox[{"C", "[", "1", "]"}], " ", 
         RowBox[{"C", "[", "2", "]"}], " ", 
         RowBox[{"Cos", "[", 
          FractionBox["\[Pi]", "9"], "]"}]}], "+", 
        RowBox[{"4", " ", 
         RowBox[{"C", "[", "2", "]"}], " ", 
         RowBox[{"C", "[", "3", "]"}], " ", 
         RowBox[{"Cos", "[", 
          FractionBox["\[Pi]", "9"], "]"}]}], "+", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          RowBox[{"C", "[", "3", "]"}], "2"], " ", 
         RowBox[{"Cos", "[", 
          FractionBox["\[Pi]", "9"], "]"}]}], "+", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          RowBox[{"C", "[", "2", "]"}], "2"], " ", 
         RowBox[{"Cos", "[", 
          FractionBox[
           RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}], "-", 
        RowBox[{"4", " ", 
         RowBox[{"C", "[", "1", "]"}], " ", 
         RowBox[{"C", "[", "3", "]"}], " ", 
         RowBox[{"Cos", "[", 
          FractionBox[
           RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          RowBox[{"C", "[", "3", "]"}], "2"], " ", 
         RowBox[{"Cos", "[", 
          FractionBox[
           RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}]}], ")"}]}], ")"}], "/", 
    
    RowBox[{"(", 
     RowBox[{
      RowBox[{"8", " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "1", "]"}], "3"]}], "-", 
      RowBox[{"6", " ", 
       RowBox[{"C", "[", "1", "]"}], " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "2", "]"}], "2"]}], "+", 
      SuperscriptBox[
       RowBox[{"C", "[", "2", "]"}], "3"], "-", 
      RowBox[{"6", " ", 
       RowBox[{"C", "[", "1", "]"}], " ", 
       RowBox[{"C", "[", "2", "]"}], " ", 
       RowBox[{"C", "[", "3", "]"}]}], "+", 
      RowBox[{"6", " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "2", "]"}], "2"], " ", 
       RowBox[{"C", "[", "3", "]"}]}], "-", 
      RowBox[{"6", " ", 
       RowBox[{"C", "[", "1", "]"}], " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "3", "]"}], "2"]}], "+", 
      RowBox[{"3", " ", 
       RowBox[{"C", "[", "2", "]"}], " ", 
       SuperscriptBox[
        RowBox[{"C", "[", "3", "]"}], "2"]}], "-", 
      SuperscriptBox[
       RowBox[{"C", "[", "3", "]"}], "3"]}], ")"}]}]}],
  SequenceForm[
  "The inverse of ", C[1] + C[2] Cos[Rational[1, 9] Pi] + 
   C[3] Cos[Rational[2, 9] Pi], " is given by ", 
   2 (8 C[1]^3 - 6 C[1] C[2]^2 + C[2]^3 - 6 C[1] C[2] C[3] + 6 C[2]^2 C[3] - 
     6 C[1] C[3]^2 + 3 C[2] C[3]^2 - C[3]^3)^(-1) (4 C[1]^2 - C[2]^2 - C[2] 
    C[3] - C[3]^2 - 4 C[1] C[2] Cos[Rational[1, 9] Pi] + 
    4 C[2] C[3] Cos[Rational[1, 9] Pi] + 2 C[3]^2 Cos[Rational[1, 9] Pi] + 
    2 C[2]^2 Cos[Rational[2, 9] Pi] - 4 C[1] C[3] Cos[Rational[2, 9] Pi] - 2 
    C[3]^2 Cos[Rational[2, 9] Pi])],
  Editable->False]], "Print",
 CellChangeTimes->{3.442754930898875*^9}]
}, Open  ]],

Cell[TextData[{
 "Even though the formula for the inverse is a complete mess, but the main \
point is that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " came up with a formula! Now we have defined the set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " to be a field. We can now perform rather complex divisions."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["1/(4 + 3 Cos[Pi/9] - 5 Cos[2 Pi/9])", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  FractionBox["2", "163"], " ", 
  RowBox[{"(", 
   RowBox[{"45", "-", 
    RowBox[{"58", " ", 
     RowBox[{"Cos", "[", 
      FractionBox["\[Pi]", "9"], "]"}]}], "+", 
    RowBox[{"48", " ", 
     RowBox[{"Cos", "[", 
      FractionBox[
       RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.44275493532075*^9}]
}, Open  ]],

Cell[TextData[{
 "Since the elements of this field are all real there is a natural ordering \
of the elements of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". Are there other ways to order this field? We want to look for \
automorphisms on the field ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". But consider the following:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
Homomorph[F]
Define[ F[Cos[Pi/9]], - Cos[2 Pi/9]]
Define[ F[Cos[2 Pi/9]], Cos[Pi/9] - Cos[2 Pi/9] ] \
\>", "Input",
 PageWidth->Infinity],

Cell["Is this an automorphism? Let's check.", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckHomo[F,{1, Cos[Pi/9], Cos[2 Pi/9]}]", "Input",
 PageWidth->Infinity],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.442754942523875*^9}]
}, Open  ]],

Cell[TextData[{
 "Apparently it is! Furthermore, we could consider the homomorphism ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Florin]", "2"], "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{"\[Florin]", "(", "x", ")"}], ")"}], ":"}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["F[F[Cos[Pi/9]]]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"Cos", "[", 
    FractionBox["\[Pi]", "9"], "]"}]}], "+", 
  RowBox[{"Cos", "[", 
   FractionBox[
    RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}]], "Output",
 CellChangeTimes->{3.442754944742625*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["F[F[Cos[2 Pi/9]]]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"Cos", "[", 
   FractionBox["\[Pi]", "9"], "]"}]}]], "Output",
 CellChangeTimes->{3.44275494650825*^9}]
}, Open  ]],

Cell[TextData[{
 "This shows that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Florin]", "2"], TraditionalForm]]],
 " is even another automorphism on ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ".\n\nEXPERIMENT:\nIf we considered the automorphism ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Florin]", "3"], "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"\[Florin]", "(", 
     RowBox[{"\[Florin]", "(", 
      RowBox[{"\[Florin]", "(", "x", ")"}], ")"}], ")"}]}], 
   TraditionalForm]]],
 ", would we get yet another automorphism?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Are there any automorphisms on the field ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " besides these? We can show that there are not. We will take advantage of \
the trig identity\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"cos", "(", 
      RowBox[{"3", "x"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"4", " ", 
       SuperscriptBox["cos", "3"], "x"}], " ", "-", " ", 
      RowBox[{"3", " ", "cos", " ", "x"}]}]}]}], TraditionalForm]]],
 "\n\nfrom which we get\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox["1", "2"], " ", "=", " ", 
    RowBox[{
     RowBox[{"cos", "(", 
      FractionBox[
       RowBox[{"3", "\[Pi]"}], "9"], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"4", " ", 
       RowBox[{
        SuperscriptBox["cos", "3"], "(", 
        FractionBox["\[Pi]", "9"], ")"}]}], " ", "-", " ", 
      RowBox[{"3", " ", 
       RowBox[{"cos", "(", 
        FractionBox["\[Pi]", "9"], ")"}]}]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"cos", "(", 
    FractionBox["\[Pi]", "9"], ")"}], TraditionalForm]]],
 " satisfies the polynomial equation\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"4", " ", 
       SuperscriptBox["x", "3"]}], " ", "-", " ", 
      RowBox[{"3", " ", "x"}]}], " ", "=", " ", 
     FractionBox["1", 
      RowBox[{"2", "."}]]}]}], TraditionalForm]]],
 "\n\nBecause ",
 Cell[BoxData[
  FormBox["\[Florin]", TraditionalForm]]],
 " is an automorphism, we have to have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", 
    RowBox[{"cos", "(", 
     FractionBox["\[Pi]", "9"], ")"}], ")"}], TraditionalForm]]],
 " satisfying the same polynomial equation. But there are only three roots to \
a cubic equation, and so there are only three possible values for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", 
    RowBox[{"cos", "(", 
     FractionBox["\[Pi]", "9"], ")"}], ")"}], TraditionalForm]]],
 ". Each of these three solutions produces a unique automorphism on ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", since\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"cos", "(", 
      FractionBox[
       RowBox[{"2", "\[Pi]"}], "9"], ")"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{
       RowBox[{"2", " ", 
        RowBox[{
         SuperscriptBox["cos", "2"], "(", 
         FractionBox["\[Pi]", "9"], ")"}]}], " ", "-", " ", "1"}], ")"}], " ",
      "=", " ", 
     RowBox[{
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"\[Florin]", "(", 
         RowBox[{"cos", "(", 
          FractionBox["\[Pi]", "9"], ")"}], ")"}], "2"]}], " ", "-", " ", 
      "1."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43739009646875*^9, 3.437390097*^9}}],

Cell[TextData[{
 "By lemma 11.5, we see that the group of automorphisms of this ring must be \
isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "3"], TraditionalForm]]],
 ". The three automorphisms give us three ways to define an ordering on the \
field ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ":\n\n1)      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", 
    FormBox[
     SubscriptBox[">", "1"],
     TraditionalForm], " ", "b"}], TraditionalForm]]],
 "    if ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is larger than ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " as real numbers.\n\n2)     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"a", " ", 
     SubscriptBox[">", "2"], "  ", 
     RowBox[{"b", "    ", "if", "  ", 
      RowBox[{"\[Florin]", "(", "a", ")"}]}], " ", 
     SubscriptBox[">", "1"], "  ", 
     RowBox[{
      RowBox[{"\[Florin]", "(", "b", ")"}], "."}]}]}], TraditionalForm]]],
 "\n\n3)      ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", 
    FormBox[
     SubscriptBox[">", "3"],
     TraditionalForm], "  ", "b"}], TraditionalForm]]],
 "    if  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"\[Florin]", "(", "a", ")"}], ")"}], " ", 
    SubscriptBox[">", "1"], "  ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      RowBox[{"\[Florin]", "(", "b", ")"}], ")"}], "."}]}], 
   TraditionalForm]]],
 "\n\nEXPERIMENT:\nComsider the numbers"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437535962765625*^9, 3.437536017859375*^9}}],

Cell[CellGroupData[{

Cell["A = 1 - 2 Cos[Pi/9]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"1", "-", 
  RowBox[{"2", " ", 
   RowBox[{"Cos", "[", 
    FractionBox["\[Pi]", "9"], "]"}]}]}]], "Output",
 CellChangeTimes->{3.442754953227*^9}]
}, Open  ]],

Cell["and", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["B = 1 + 3 Cos[2 Pi/9]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"1", "+", 
  RowBox[{"3", " ", 
   RowBox[{"Cos", "[", 
    FractionBox[
     RowBox[{"2", " ", "\[Pi]"}], "9"], "]"}]}]}]], "Output",
 CellChangeTimes->{3.44275495513325*^9}]
}, Open  ]],

Cell[TextData[{
 "Are these numbers positive or negative in the three ordering systems? Can \
you find a irrational number in ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " which is positive regardless of which ordering is used?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell["\<\
We have seen some rather unusual examples of ordered fields in this section. \
However, in all cases the field that was used was a subfield of the real \
numbers. This is not just a coincidence, since if we stray into the complex \
numbers, we will no longer have an ordering. In the next section, we will \
look in depth into the field of complex numbers. We need to have a basic \
understanding of this field since many other fields will be described in \
terms of these complex numbers. \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Problems"
}], "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437536031640625*^9, 3.437536038*^9}}],

Cell["", "Input",
 PageWidth->Infinity,
 FontFamily->"System",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->RGBColor[1, 0, 0],
 Background->GrayLevel[1]],

Cell["11.1 ) In the field of \"complex numbers modulo 3\":", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.437536041171875*^9}],

Cell["\<\
InitDomain[3]
Define[i^2, -1]
CheckField[{ 1, i}]
F = Ring[{1, i}]\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "    factor the polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "3"], " ", "+", " ", "1"}], ",", " ", 
    RowBox[{
     SuperscriptBox["x", "3"], " ", "+", " ", "2"}], ","}], 
   TraditionalForm]]],
 " \n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "3"], " ", "+", " ", "i"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "3"], " ", "+", " ", 
    RowBox[{"2", "i"}]}], TraditionalForm]]],
 ". What do you notice \n    about the factorizations? Knowing how \n    ",
 StyleBox["real",
  FontSlant->"Italic"],
 " polynomials factor, explain what is \n    happening."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43739010478125*^9, 3.437390106*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "\n11.2 ) Consider a rational function ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " in the field of \n   \"complex numbers modulo 3\":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
InitDomain[3]
Define[i^2, -1]
CheckField[{ 1, i}]
F = Ring[{1, i}]
A = (x^2 + x + i x + 2 + 2 i)/
       (x^2 + i x^2 + x + 2 i x + 1)
Factor[A, F]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437390107046875*^9, 3.43739010890625*^9}},
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "    Although ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " does not seem to simplify, \n    there is a quotient of first degree \n    \
polynomials which is equivalent to ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". \n    Find such a simplification.\n    (Hint: Multiply the denominator by \
a \n    constant so that so the coeffient for \n    the highest power of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is 1.)"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437390114796875*^9, 3.437390114796875*^9}}],

Cell["          ", "Input",
 PageWidth->Infinity],

Cell["    Reduced form:", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "   11.3 ) Follow th example of ",
 Cell[BoxData[
  FormBox[Cell[TextData[Cell[BoxData[
    FormBox[
     RowBox[{"\[DoubleStruckCapitalZ]", "[", 
      RadicalBox["2", "3"], "]"}], TraditionalForm]]]]], TraditionalForm]]],
 " to define\n    the integral domain ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["5"], "]"}], TraditionalForm]]],
 " in ",
 StyleBox["Mathematica.\n    ",
  FontSlant->"Italic"],
 "Then define ",
 StyleBox["F",
  FontWeight->"Bold"],
 " to be a non-trivial ring \n    automorphism for this domain."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437536070921875*^9, 3.4375360929375*^9}, {
  3.4419814153389997`*^9, 3.4419815216671247`*^9}}],

Cell["       ", "Input",
 PageWidth->Infinity],

Cell["11.4 ) Using the commands ", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.43753609515625*^9}],

Cell["\<\
InitDomain[0]
CheckField[ {1, Cos[Pi/5] } ]\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "    verify that all numbers of the form \n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "+", " ", 
     RowBox[{"y", " ", 
      RowBox[{"cos", "(", 
       RowBox[{"\[Pi]", "/", "5"}], ")"}]}]}], ","}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 ", \n    forms a field. Find a non-trivial ring \n    automorphism on this \
field.\n    (Hint: Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to compute \n    ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["cos", "2"], TraditionalForm]]],
 "(\[Pi]/5). How is this field related to \n    the integral domain in \
problem 11.3?)  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["     ", "Input",
 PageWidth->Infinity],

Cell["\<\
11.5 ) Expain why the ring \"complex numbers 
     modulo 5\"\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43753620128125*^9, 3.4375362016875*^9}, 
   3.43753623575*^9}],

Cell["\<\
InitDomain[5]
Define[i^2, -1]\
\>", "Input",
 PageWidth->Infinity,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "   does not form a field. Can you determine \n   a pattern as to which \
integers \"complex \n   numbers modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "\" forms a field?  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["       ", "Input",
 PageWidth->Infinity,
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{516, 681},
WindowMargins->{{172, Automatic}, {14, Automatic}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.5,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[568, 21, 318, 9, 285, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[911, 34, 286, 8, 29, "Name",
 Evaluatable->False],
Cell[1200, 44, 710, 16, 70, "Name",
 Evaluatable->False]
}, Closed]],
Cell[1925, 63, 273, 12, 199, "Title",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2223, 79, 104, 3, 32, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2352, 86, 172, 4, 43, "Input",
 InitializationCell->True],
Cell[2527, 92, 210, 3, 32, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2786, 101, 89, 2, 105, "Section",
 Evaluatable->False],
Cell[2878, 105, 5499, 170, 1325, "Text",
 Evaluatable->False],
Cell[8380, 277, 933, 29, 77, "Text",
 Evaluatable->False],
Cell[9316, 308, 3715, 113, 630, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[13056, 425, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[13144, 429, 7359, 257, 933, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[20518, 689, 641, 17, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[21184, 710, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[21272, 714, 19560, 679, 1993, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[40847, 1396, 499, 11, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[41371, 1411, 234, 8, 133, "Input"],
Cell[41608, 1421, 391, 11, 68, "Output"]
}, Open  ]],
Cell[42014, 1435, 511, 17, 115, "Text",
 Evaluatable->False],
Cell[42528, 1454, 219, 7, 88, "Input"],
Cell[42750, 1463, 313, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[43088, 1475, 92, 2, 43, "Input"],
Cell[43183, 1479, 268, 5, 42, "Output"]
}, Open  ]],
Cell[43466, 1487, 140, 4, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[43631, 1495, 42, 1, 43, "Input"],
Cell[43676, 1498, 351, 7, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44064, 1510, 42, 1, 43, "Input"],
Cell[44109, 1513, 471, 9, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44617, 1527, 42, 1, 43, "Input"],
Cell[44662, 1530, 673, 14, 42, "Output"]
}, Open  ]],
Cell[45350, 1547, 1762, 60, 409, "Text",
 Evaluatable->False],
Cell[47115, 1609, 41, 1, 43, "Input"],
Cell[47159, 1612, 284, 6, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[47468, 1622, 100, 2, 43, "Input"],
Cell[47571, 1626, 116, 2, 42, "Output"]
}, Open  ]],
Cell[47702, 1631, 606, 17, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[48333, 1652, 131, 2, 43, "Input"],
Cell[48467, 1656, 376, 8, 42, "Output"]
}, Open  ]],
Cell[48858, 1667, 73, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[48956, 1673, 130, 2, 43, "Input"],
Cell[49089, 1677, 379, 8, 42, "Output"]
}, Open  ]],
Cell[49483, 1688, 462, 13, 164, "Text",
 Evaluatable->False],
Cell[49948, 1703, 41, 1, 43, "Input"],
Cell[49992, 1706, 3579, 112, 1048, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[53596, 1822, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[53684, 1826, 8588, 312, 923, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[62287, 2141, 1446, 41, 507, "Text",
 Evaluatable->False],
Cell[63736, 2184, 52, 1, 43, "Input"],
Cell[63791, 2187, 140, 4, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[63956, 2195, 44, 1, 43, "Input"],
Cell[64003, 2198, 114, 2, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[64154, 2205, 44, 1, 43, "Input"],
Cell[64201, 2208, 106, 1, 42, "Output"]
}, Open  ]],
Cell[64322, 2212, 498, 17, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[64845, 2233, 49, 1, 43, "Input"],
Cell[64897, 2236, 117, 2, 42, "Output"]
}, Open  ]],
Cell[65029, 2241, 677, 25, 119, "Text",
 Evaluatable->False],
Cell[65709, 2268, 54, 1, 43, "Input"],
Cell[65766, 2271, 260, 11, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[66051, 2286, 52, 1, 43, "Input"],
Cell[66106, 2289, 424, 11, 68, "Output"]
}, Open  ]],
Cell[66545, 2303, 208, 7, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[66778, 2314, 51, 1, 43, "Input"],
Cell[66832, 2317, 135, 1, 58, "Print"]
}, Open  ]],
Cell[66982, 2321, 124, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[67131, 2327, 50, 1, 43, "Input"],
Cell[67184, 2330, 14600, 423, 374, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[81821, 2758, 51, 1, 43, "Input"],
Cell[81875, 2761, 14582, 424, 374, "Output"]
}, Open  ]],
Cell[96472, 3188, 641, 15, 262, "Text",
 Evaluatable->False],
Cell[97116, 3205, 42, 1, 43, "Input"],
Cell[97161, 3208, 297, 8, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[97483, 3220, 94, 5, 88, "Input"],
Cell[97580, 3227, 378, 10, 68, "Output"]
}, Open  ]],
Cell[97973, 3240, 172, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[98170, 3251, 96, 2, 43, "Input"],
Cell[98269, 3255, 212, 6, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[98518, 3266, 135, 2, 43, "Input"],
Cell[98656, 3270, 316, 9, 45, "Output"]
}, Open  ]],
Cell[98987, 3282, 128, 2, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[99140, 3288, 42, 1, 43, "Input"],
Cell[99185, 3291, 266, 8, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[99488, 3304, 42, 1, 43, "Input"],
Cell[99533, 3307, 471, 15, 72, "Output"]
}, Open  ]],
Cell[100019, 3325, 365, 9, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[100409, 3338, 42, 1, 43, "Input"],
Cell[100454, 3341, 243, 6, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[100734, 3352, 42, 1, 43, "Input"],
Cell[100779, 3355, 332, 9, 45, "Output"]
}, Open  ]],
Cell[101126, 3367, 951, 27, 288, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[102102, 3398, 148, 3, 43, "Input"],
Cell[102253, 3403, 169, 4, 45, "Output"]
}, Open  ]],
Cell[102437, 3410, 230, 8, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[102692, 3422, 151, 3, 43, "Input"],
Cell[102846, 3427, 265, 8, 42, "Output"]
}, Open  ]],
Cell[103126, 3438, 333, 9, 164, "Text",
 Evaluatable->False],
Cell[103462, 3449, 42, 1, 43, "Input"],
Cell[103507, 3452, 451, 8, 213, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[103995, 3465, 95, 2, 105, "Section",
 Evaluatable->False],
Cell[104093, 3469, 1660, 33, 703, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[105778, 3506, 58, 1, 43, "Input"],
Cell[105839, 3509, 183, 6, 60, "Output"]
}, Open  ]],
Cell[106037, 3518, 315, 11, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[106377, 3533, 52, 1, 43, "Input"],
Cell[106432, 3536, 136, 4, 60, "Output"]
}, Open  ]],
Cell[106583, 3543, 1167, 37, 338, "Text",
 Evaluatable->False],
Cell[107753, 3582, 3044, 103, 716, "Text",
 Evaluatable->False],
Cell[110800, 3687, 345, 9, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[111170, 3700, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[111258, 3704, 627, 25, 116, "SmallText",
 Evaluatable->False],
Cell[111888, 3731, 312, 12, 41, "Text",
 Evaluatable->False],
Cell[112203, 3745, 391, 12, 96, "SmallText",
 Evaluatable->False],
Cell[112597, 3759, 957, 30, 92, "Text",
 Evaluatable->False],
Cell[113557, 3791, 572, 20, 116, "SmallText",
 Evaluatable->False],
Cell[114132, 3813, 1386, 36, 62, "Text",
 Evaluatable->False],
Cell[115521, 3851, 1081, 32, 137, "SmallText",
 Evaluatable->False],
Cell[116605, 3885, 480, 16, 41, "Text",
 Evaluatable->False],
Cell[117088, 3903, 176, 5, 55, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[117279, 3911, 826, 25, 213, "Text",
 Evaluatable->False],
Cell[118108, 3938, 565, 21, 90, "Text",
 Evaluatable->False],
Cell[118676, 3961, 4549, 164, 900, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[123250, 4129, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[123338, 4133, 8396, 291, 1159, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[131749, 4427, 1061, 37, 262, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[132835, 4468, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[132923, 4472, 17601, 591, 1793, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[150539, 5066, 3020, 86, 1012, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[153584, 5156, 161, 5, 66, "Input"],
Cell[153748, 5163, 345, 11, 68, "Output"]
}, Open  ]],
Cell[154108, 5177, 74, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[154207, 5183, 141, 2, 43, "Input"],
Cell[154351, 5187, 318, 10, 68, "Output"]
}, Open  ]],
Cell[154684, 5200, 105, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[154814, 5206, 148, 2, 43, "Input"],
Cell[154965, 5210, 313, 9, 45, "Output"]
}, Open  ]],
Cell[155293, 5222, 74, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[155392, 5228, 148, 2, 43, "Input"],
Cell[155543, 5232, 312, 9, 45, "Output"]
}, Open  ]],
Cell[155870, 5244, 352, 12, 139, "Text",
 Evaluatable->False],
Cell[156225, 5258, 41, 1, 43, "Input"],
Cell[156269, 5261, 658, 22, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[156952, 5287, 54, 1, 43, "Input"],
Cell[157009, 5290, 89, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[157135, 5296, 118, 2, 43, "Input"],
Cell[157256, 5300, 88, 1, 42, "Output"]
}, Open  ]],
Cell[157359, 5304, 2475, 83, 605, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[159859, 5391, 91, 5, 88, "Input"],
Cell[159953, 5398, 375, 10, 68, "Output"]
}, Open  ]],
Cell[160343, 5411, 307, 11, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[160675, 5426, 155, 2, 43, "Input"],
Cell[160833, 5430, 403, 12, 68, "Output"]
}, Open  ]],
Cell[161251, 5445, 256, 10, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[161532, 5459, 50, 1, 43, "Input"],
Cell[161585, 5462, 484, 16, 64, "Output"]
}, Open  ]],
Cell[162084, 5481, 265, 7, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[162374, 5492, 134, 2, 43, "Input"],
Cell[162511, 5496, 274, 8, 62, "Output"]
}, Open  ]],
Cell[162800, 5507, 222, 9, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[163047, 5520, 113, 2, 43, "Input"],
Cell[163163, 5524, 704, 23, 64, "Output"]
}, Open  ]],
Cell[163882, 5550, 298, 11, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[164205, 5565, 52, 1, 43, "Input"],
Cell[164260, 5568, 91, 1, 42, "Output"]
}, Open  ]],
Cell[164366, 5572, 928, 25, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[165319, 5601, 51, 1, 43, "Input"],
Cell[165373, 5604, 14558, 424, 374, "Output"]
}, Open  ]],
Cell[179946, 6031, 40, 1, 43, "Input"],
Cell[179989, 6034, 1059, 20, 409, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[181085, 6059, 88, 2, 105, "Section",
 Evaluatable->False],
Cell[181176, 6063, 991, 25, 360, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[182192, 6092, 145, 5, 66, "Input"],
Cell[182340, 6099, 196, 4, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[182573, 6108, 60, 1, 43, "Input"],
Cell[182636, 6111, 177, 4, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[182850, 6120, 123, 2, 43, "Input"],
Cell[182976, 6124, 222, 6, 61, "Output"]
}, Open  ]],
Cell[183213, 6133, 3190, 92, 850, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[186428, 6229, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[186516, 6233, 21574, 664, 2560, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[208105, 6900, 823, 25, 215, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[208953, 6929, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[209041, 6933, 3490, 131, 465, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[212546, 7067, 3415, 95, 750, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[215986, 7166, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[216074, 7170, 5767, 179, 630, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[221856, 7352, 1156, 35, 262, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[223037, 7391, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[223125, 7395, 6436, 217, 491, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[229576, 7615, 1744, 57, 262, "Text",
 Evaluatable->False],
Cell[231323, 7674, 434, 12, 164, "Text",
 Evaluatable->False],
Cell[231760, 7688, 162, 3, 43, "Input"],
Cell[231925, 7693, 167, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[232117, 7704, 57, 1, 43, "Input"],
Cell[232177, 7707, 171, 4, 42, "Output"]
}, Open  ]],
Cell[232363, 7714, 135, 6, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[232523, 7724, 69, 1, 43, "Input"],
Cell[232595, 7727, 116, 2, 42, "Output"]
}, Open  ]],
Cell[232726, 7732, 228, 7, 66, "Text",
 Evaluatable->False],
Cell[232957, 7741, 41, 1, 43, "Input"],
Cell[233001, 7744, 2628, 82, 539, "Text",
 Evaluatable->False],
Cell[235632, 7828, 970, 33, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[236627, 7865, 51, 1, 43, "Input"],
Cell[236681, 7868, 70, 1, 42, "Output"]
}, Open  ]],
Cell[236766, 7872, 816, 25, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[237607, 7901, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[237695, 7905, 1536, 51, 117, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[239246, 7959, 649, 16, 262, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[239920, 7979, 395, 10, 178, "Input"],
Cell[240318, 7991, 362, 8, 315, "Output"]
}, Open  ]],
Cell[240695, 8002, 2067, 73, 433, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[242787, 8079, 51, 1, 43, "Input"],
Cell[242841, 8082, 142, 3, 60, "Output"]
}, Open  ]],
Cell[242998, 8088, 97, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[243120, 8094, 54, 1, 43, "Input"],
Cell[243177, 8097, 109, 1, 42, "Output"]
}, Open  ]],
Cell[243301, 8101, 1122, 30, 283, "Text",
 Evaluatable->False],
Cell[244426, 8133, 742, 21, 188, "Text",
 Evaluatable->False],
Cell[245171, 8156, 756, 28, 167, "Text",
 Evaluatable->False],
Cell[245930, 8186, 100, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[246055, 8192, 57, 1, 43, "Input"],
Cell[246115, 8195, 69, 1, 42, "Output"]
}, Open  ]],
Cell[246199, 8199, 100, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[246324, 8205, 57, 1, 43, "Input"],
Cell[246384, 8208, 136, 4, 60, "Output"]
}, Open  ]],
Cell[246535, 8215, 1179, 44, 191, "Text",
 Evaluatable->False],
Cell[247717, 8261, 1106, 41, 195, "Text",
 Evaluatable->False],
Cell[248826, 8304, 2695, 82, 556, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[251546, 8390, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[251634, 8394, 2779, 83, 226, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[254428, 8480, 144, 4, 66, "Text",
 Evaluatable->False],
Cell[254575, 8486, 1129, 36, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[255729, 8526, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[255817, 8530, 8208, 268, 915, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[264040, 8801, 1358, 44, 220, "Text",
 Evaluatable->False],
Cell[265401, 8847, 170, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[265596, 8858, 64, 1, 43, "Input"],
Cell[265663, 8861, 184, 6, 46, "Output"]
}, Open  ]],
Cell[265862, 8870, 1645, 56, 190, "Text",
 Evaluatable->False],
Cell[267510, 8928, 172, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[267707, 8939, 77, 1, 43, "Input"],
Cell[267787, 8942, 157, 5, 68, "Output"]
}, Open  ]],
Cell[267959, 8950, 3170, 114, 586, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[271154, 9068, 150, 5, 66, "Input"],
Cell[271307, 9075, 156, 4, 68, "Output"]
}, Open  ]],
Cell[271478, 9082, 254, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[271757, 9094, 109, 2, 43, "Input"],
Cell[271869, 9098, 104, 1, 42, "Output"]
}, Open  ]],
Cell[271988, 9102, 304, 12, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[272317, 9118, 120, 2, 43, "Input"],
Cell[272440, 9122, 593, 19, 123, "Output"]
}, Open  ]],
Cell[273048, 9144, 456, 14, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[273529, 9162, 154, 4, 66, "Input"],
Cell[273686, 9168, 590, 19, 123, "Output"]
}, Open  ]],
Cell[274291, 9190, 329, 9, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[274645, 9203, 126, 2, 43, "Input"],
Cell[274774, 9207, 10936, 227, 163, 5187, 131, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[285725, 9437, 1887, 67, 413, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[287637, 9508, 126, 2, 43, "Input"],
Cell[287766, 9512, 10911, 225, 157, 5195, 130, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[298692, 9740, 399, 12, 117, "Text",
 Evaluatable->False],
Cell[299094, 9754, 42, 1, 43, "Input"],
Cell[299139, 9757, 516, 16, 204, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[299680, 9777, 426, 13, 133, "Input"],
Cell[300109, 9792, 179, 5, 67, "Output"],
Cell[300291, 9799, 1266, 45, 226, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[301594, 9849, 243, 7, 43, "Input"],
Cell[301840, 9858, 24400, 468, 262, 8016, 198, "CachedBoxData", "BoxData", \
"Output"]
}, Open  ]],
Cell[326255, 10329, 564, 16, 215, "Text",
 Evaluatable->False],
Cell[326822, 10347, 33, 0, 43, "Input"],
Cell[326858, 10349, 2135, 62, 647, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[329018, 10415, 60, 1, 43, "Input"],
Cell[329081, 10418, 108, 1, 42, "Output"]
}, Open  ]],
Cell[329204, 10422, 3030, 87, 901, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[332259, 10513, 42, 1, 43, "Input"],
Cell[332304, 10516, 140, 2, 45, "Output"]
}, Open  ]],
Cell[332459, 10521, 141, 4, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[332625, 10529, 45, 1, 43, "Input"],
Cell[332673, 10532, 206, 3, 42, "Output"]
}, Open  ]],
Cell[332894, 10538, 834, 29, 262, "Text",
 Evaluatable->False],
Cell[333731, 10569, 42, 1, 43, "Input"],
Cell[333776, 10572, 4582, 134, 1027, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[338383, 10710, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[338471, 10714, 3013, 100, 252, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[341499, 10817, 2130, 69, 437, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[343654, 10890, 53, 1, 43, "Input"],
Cell[343710, 10893, 70, 1, 42, "Output"]
}, Open  ]],
Cell[343795, 10897, 1576, 54, 357, "Text",
 Evaluatable->False],
Cell[345374, 10953, 3841, 123, 784, "Text",
 Evaluatable->False],
Cell[349218, 11078, 225, 10, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[349468, 11092, 45, 1, 43, "Input"],
Cell[349516, 11095, 129, 3, 61, "Output"]
}, Open  ]],
Cell[349660, 11101, 689, 15, 286, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[350374, 11120, 94, 1, 43, "Input"],
Cell[350471, 11123, 241131, 3915, 322, 157026, 2534, "CachedBoxData", \
"BoxData", "Output"]
}, Open  ]],
Cell[591617, 15041, 118, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[591760, 15047, 94, 1, 43, "Input"],
Cell[591857, 15050, 245149, 3978, 322, 165236, 2666, "CachedBoxData", \
"BoxData", "Output"]
}, Open  ]],
Cell[837021, 19031, 247, 6, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[837293, 19041, 136, 4, 66, "Input"],
Cell[837432, 19047, 146450, 2387, 366, 85829, 1391, "CachedBoxData", \
"BoxData", "Output"]
}, Open  ]],
Cell[983897, 21437, 306, 6, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[984228, 21447, 174, 5, 88, "Input"],
Cell[984405, 21454, 168030, 2738, 437, 94818, 1536, "CachedBoxData", \
"BoxData", "Output"]
}, Open  ]],
Cell[1152450, 24195, 2278, 64, 833, "Text",
 Evaluatable->False],
Cell[1154731, 24261, 42, 1, 43, "Input"],
Cell[1154776, 24264, 1278, 45, 290, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1156079, 24313, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[1156167, 24317, 8653, 277, 1037, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[1164835, 24597, 695, 12, 335, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[1165567, 24614, 98, 2, 105, "Section",
 Evaluatable->False],
Cell[1165668, 24618, 2301, 63, 801, "Text",
 Evaluatable->False],
Cell[1167972, 24683, 838, 27, 188, "Text",
 Evaluatable->False],
Cell[1168813, 24712, 539, 21, 66, "Text",
 Evaluatable->False],
Cell[1169355, 24735, 2276, 76, 507, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1171656, 24815, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[1171744, 24819, 3462, 117, 588, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[1175221, 24939, 796, 26, 239, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1176042, 24969, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[1176130, 24973, 1381, 53, 213, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[1177526, 25029, 1121, 38, 288, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1178672, 25071, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[1178760, 25075, 2148, 75, 224, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[1180923, 25153, 851, 24, 267, "Text",
 Evaluatable->False],
Cell[1181777, 25179, 283, 11, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1182085, 25194, 101, 4, 66, "Input"],
Cell[1182189, 25200, 267, 8, 46, "Output"]
}, Open  ]],
Cell[1182471, 25211, 2320, 71, 565, "Text",
 Evaluatable->False],
Cell[1184794, 25284, 614, 22, 194, "Text",
 Evaluatable->False],
Cell[1185411, 25308, 399, 8, 115, "Text",
 Evaluatable->False],
Cell[1185813, 25318, 813, 27, 129, "Text",
 Evaluatable->False],
Cell[1186629, 25347, 399, 15, 97, "Text",
 Evaluatable->False],
Cell[1187031, 25364, 54, 1, 43, "Input"],
Cell[1187088, 25367, 351, 11, 139, "Text",
 Evaluatable->False],
Cell[1187442, 25380, 91, 4, 66, "Input"],
Cell[1187536, 25386, 293, 9, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1187854, 25399, 63, 1, 43, "Input"],
Cell[1187920, 25402, 73, 1, 42, "Output"]
}, Open  ]],
Cell[1188008, 25406, 1934, 57, 646, "Text",
 Evaluatable->False],
Cell[1189945, 25465, 398, 14, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1190368, 25483, 54, 1, 43, "Input"],
Cell[1190425, 25486, 126, 4, 67, "Output"]
}, Open  ]],
Cell[1190566, 25493, 444, 13, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1191035, 25510, 84, 4, 66, "Input"],
Cell[1191122, 25516, 216, 5, 32, "Print"],
Cell[1191341, 25523, 552, 16, 85, "Print"],
Cell[1191896, 25541, 380, 9, 83, "Print"],
Cell[1192279, 25552, 835, 25, 94, "Print"]
}, Open  ]],
Cell[1193129, 25580, 1605, 53, 437, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1194759, 25637, 54, 1, 43, "Input"],
Cell[1194816, 25640, 122, 4, 46, "Output"]
}, Open  ]],
Cell[1194953, 25647, 437, 14, 118, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1195415, 25665, 76, 1, 43, "Input"],
Cell[1195494, 25668, 144, 5, 46, "Output"]
}, Open  ]],
Cell[1195653, 25676, 1998, 63, 493, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1197676, 25743, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[1197764, 25747, 5443, 197, 992, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[1203222, 25947, 703, 27, 143, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1203950, 25978, 84, 4, 66, "Input"],
Cell[1204037, 25984, 138, 2, 58, "Print"]
}, Open  ]],
Cell[1204190, 25989, 1121, 35, 255, "Text",
 Evaluatable->False],
Cell[1205314, 26026, 167, 7, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1205506, 26037, 128, 2, 43, "Input"],
Cell[1205637, 26041, 248, 8, 45, "Output"]
}, Open  ]],
Cell[1205900, 26052, 147, 4, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1206072, 26060, 184, 5, 66, "Input"],
Cell[1206259, 26067, 776, 23, 100, "Output"]
}, Open  ]],
Cell[1207050, 26093, 300, 9, 82, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1207375, 26106, 92, 4, 66, "Input"],
Cell[1207470, 26112, 216, 5, 32, "Print"],
Cell[1207689, 26119, 823, 23, 112, "Print"],
Cell[1208515, 26144, 382, 9, 83, "Print"],
Cell[1208900, 26155, 2207, 63, 200, "Print"]
}, Open  ]],
Cell[1211122, 26221, 162, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1211309, 26230, 66, 1, 43, "Input"],
Cell[1211378, 26233, 350, 12, 60, "Output"]
}, Open  ]],
Cell[1211743, 26248, 533, 18, 139, "Text",
 Evaluatable->False],
Cell[1212279, 26268, 1052, 31, 229, "Text",
 Evaluatable->False],
Cell[1213334, 26301, 1259, 40, 326, "Text",
 Evaluatable->False],
Cell[1214596, 26343, 873, 28, 227, "Text",
 Evaluatable->False],
Cell[1215472, 26373, 1523, 53, 282, "Text",
 Evaluatable->False],
Cell[1216998, 26428, 147, 5, 90, "Text",
 Evaluatable->False],
Cell[1217148, 26435, 42, 1, 43, "Input"],
Cell[1217193, 26438, 313, 10, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1217531, 26452, 137, 4, 66, "Input"],
Cell[1217671, 26458, 1550, 47, 285, "Output"]
}, Open  ]],
Cell[1219236, 26508, 385, 13, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1219646, 26525, 100, 4, 66, "Input"],
Cell[1219749, 26531, 213, 5, 32, "Print"],
Cell[1219965, 26538, 1553, 43, 185, "Print"],
Cell[1221521, 26583, 381, 9, 83, "Print"],
Cell[1221905, 26594, 3835, 103, 475, "Print"]
}, Open  ]],
Cell[1225755, 26700, 396, 11, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1226176, 26715, 74, 1, 43, "Input"],
Cell[1226253, 26718, 376, 12, 60, "Output"]
}, Open  ]],
Cell[1226644, 26733, 412, 12, 139, "Text",
 Evaluatable->False],
Cell[1227059, 26747, 147, 5, 88, "Input"],
Cell[1227209, 26754, 107, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1227341, 26760, 79, 1, 43, "Input"],
Cell[1227423, 26763, 73, 1, 42, "Output"]
}, Open  ]],
Cell[1227511, 26767, 403, 13, 68, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1227939, 26784, 54, 1, 43, "Input"],
Cell[1227996, 26787, 249, 8, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1228282, 26800, 56, 1, 43, "Input"],
Cell[1228341, 26803, 142, 4, 58, "Output"]
}, Open  ]],
Cell[1228498, 26810, 644, 21, 192, "Text",
 Evaluatable->False],
Cell[1229145, 26833, 41, 1, 43, "Input"],
Cell[1229189, 26836, 1120, 36, 253, "Text",
 Evaluatable->False],
Cell[1230312, 26874, 1950, 62, 420, "Text",
 Evaluatable->False],
Cell[1232265, 26938, 1606, 56, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1233896, 26998, 58, 1, 43, "Input"],
Cell[1233957, 27001, 170, 5, 58, "Output"]
}, Open  ]],
Cell[1234142, 27009, 73, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[1234240, 27015, 60, 1, 43, "Input"],
Cell[1234303, 27018, 198, 6, 60, "Output"]
}, Open  ]],
Cell[1234516, 27027, 304, 8, 115, "Text",
 Evaluatable->False],
Cell[1234823, 27037, 41, 1, 43, "Input"],
Cell[1234867, 27040, 568, 10, 262, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[1235472, 27055, 206, 7, 105, "Section",
 Evaluatable->False],
Cell[1235681, 27064, 156, 6, 43, "Input"],
Cell[1235840, 27072, 164, 3, 41, "Text",
 Evaluatable->False],
Cell[1236007, 27077, 113, 6, 111, "Input"],
Cell[1236123, 27085, 818, 28, 143, "Text",
 Evaluatable->False],
Cell[1236944, 27115, 42, 1, 43, "Input"],
Cell[1236989, 27118, 229, 7, 90, "Text",
 Evaluatable->False],
Cell[1237221, 27127, 372, 15, 178, "Input"],
Cell[1237596, 27144, 600, 16, 188, "Text",
 Evaluatable->False],
Cell[1238199, 27162, 49, 1, 43, "Input"],
Cell[1238251, 27165, 87, 2, 41, "Text",
 Evaluatable->False],
Cell[1238341, 27169, 764, 23, 140, "Text",
 Evaluatable->False],
Cell[1239108, 27194, 46, 1, 43, "Input"],
Cell[1239157, 27197, 137, 3, 41, "Text",
 Evaluatable->False],
Cell[1239297, 27202, 90, 4, 66, "Input"],
Cell[1239390, 27208, 879, 30, 190, "Text",
 Evaluatable->False],
Cell[1240272, 27240, 44, 1, 43, "Input"],
Cell[1240319, 27243, 224, 7, 66, "Text",
 Evaluatable->False],
Cell[1240546, 27252, 189, 9, 66, "Input"],
Cell[1240738, 27263, 271, 8, 90, "Text",
 Evaluatable->False],
Cell[1241012, 27273, 159, 6, 43, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
