(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    365058,      12640]
NotebookOptionsPosition[    346892,      12034]
NotebookOutlinePosition[    347604,      12061]
CellTagsIndexPosition[    347561,      12058]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Abstract Algebra: An Interactive Approach", "Special2",
 Evaluatable->False,
 CellChangeTimes->{{3.437744861328125*^9, 3.43774487925*^9}},
 TextAlignment->Center,
 FontFamily->"Arial",
 FontSize->36,
 FontWeight->"Bold",
 FontSlant->"Italic",
 FontColor->RGBColor[1, 0, 1],
 Background->RGBColor[0.500008, 1, 1]],

Cell[CellGroupData[{

Cell["\[Copyright]2009   William Paulsen", "Name",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437744883875*^9, 3.437744884515625*^9}},
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]],

Cell["\<\
All rights reserved:  These materials can be duplicated or transmitted freely \
for educational use on condition that all copyright statements are retained \
unaltered.  No part of these materials may be sold, by any means or in any \
form, without written permission from the editor.  All software is provided \
\"as is\" without warranty of any kind, either expressed or implied, \
including, but not limited to, the implied warranties of merchantability and \
fitness for a particular purpose.\
\>", "Name",
 Editable->False,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 FontFamily->"Geneva",
 FontSize->10,
 FontSlant->"Italic",
 FontColor->RGBColor[0, 0, 1],
 Background->GrayLevel[1]]
}, Closed]],

Cell["\<\
Chapter 12
Unique Factorization\
\>", "Title",
 CellFrame->True,
 Evaluatable->False,
 CellHorizontalScrolling->False,
 TextAlignment->Center,
 FontFamily->"New York",
 FontSize->24,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell["Initialization", "Special2",
 Evaluatable->False,
 TextAlignment->Center,
 AspectRatioFixed->True],

Cell["<<c:\\math\\ring.m", "Input",
 PageWidth->Infinity,
 InitializationCell->True,
 CellChangeTimes->{{3.437744887125*^9, 3.4377448905625*^9}},
 AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Factorization of Polynomials", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In the last chapter, we saw how we could create an integral domain from a \
field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " by considering all polynomials using the variable ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " with coefficients in ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 ". We called this ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ". In this section we want to investigate how such polynomials would factor. \
Most of the statements in this section are very familiar from the properties \
of polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", "x", "]"}], TraditionalForm]]],
 " studied in a standard algebra course. \n\nWe say that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " factors if there are two non-constant polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"g", "(", "x", ")"}], " ", "\[CenterDot]", " ", 
     RowBox[{"h", "(", "x", ")"}]}]}], TraditionalForm]]],
 ". We also say that both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " divide the polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " may also factor into non-constant polynomials. We want to show that we can \
factor ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " into polynomials which cannot be factored further. We also want to lay \
down the groundwork for showing that the polynomials produced by this \
factorization are in some sense uniquely determined.\n\nOne the the standard \
techniques learned in algebra is that we can do \"long division\" on \
polynomials. A sample problem would be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["x", "3"], " ", "-", " ", 
     RowBox[{"3", " ", 
      SuperscriptBox["x", "2"]}], " ", "+", " ", 
     RowBox[{"4", " ", "x"}], " ", "-", " ", "5"}], ")"}], 
   TraditionalForm]]],
 " divided by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["x", "2"]}], " ", "-", " ", "5"}], ")"}], 
   TraditionalForm]]],
 ". Using long division, we have:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774498075*^9, 3.437744996890625*^9}, {
  3.4377463334375*^9, 3.437746333796875*^9}}],

Cell[TextData[{
 "\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{
      FractionBox["1", "2"], "x"}],
     TraditionalForm], " ", "-", " ", 
    FormBox[
     FractionBox["3", "2"],
     TraditionalForm]}], TraditionalForm]]],
 "\n              ____________________\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", " ", 
     SuperscriptBox["x", "2"]}], " ", "-", " ", "5"}], TraditionalForm]]],
 " ) ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["x", "3"],
     TraditionalForm], " ", "-", " ", 
    FormBox[
     RowBox[{"3", 
      SuperscriptBox["x", "2"]}],
     TraditionalForm], " ", "+", " ", 
    RowBox[{"4", "   ", "x"}], " ", "-", " ", "5"}], TraditionalForm]]],
 "\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox["x", 
      RowBox[{"3", "         "}]],
     TraditionalForm], "       ", "-", " ", 
    RowBox[{
     FormBox[
      FractionBox["5", "2"],
      TraditionalForm], "x"}]}], TraditionalForm]]],
 "\n              \[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\
\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\
\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\
\[LongDash]\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", " ", 
     FormBox[
      RowBox[{"3", " ", 
       FormBox[
        SuperscriptBox["x", "2"],
        TraditionalForm]}],
      TraditionalForm]}], "+", " ", 
    RowBox[{
     FormBox[
      FractionBox["13", "2"],
      TraditionalForm], "x"}], "  ", "-", " ", "5"}], TraditionalForm]]],
 "\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", " ", 
     FormBox[
      RowBox[{"3", " ", 
       FormBox[
        SuperscriptBox["x", "2"],
        TraditionalForm]}],
      TraditionalForm]}], "                ", "+", 
    FormBox[
     FractionBox["15", "2"],
     TraditionalForm]}], TraditionalForm]]],
 "\n              \[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\
\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\
\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\[LongDash]\
\[LongDash]\n                         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      FractionBox["13", "2"],
      TraditionalForm], " ", "x"}], " ", "-", 
    FormBox[
     FractionBox["25", "2"],
     TraditionalForm]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4377449975*^9, 3.43774500175*^9}},
 FontFamily->"Courier New",
 FontSize->12,
 FontWeight->"Plain",
 FontSlant->"Plain",
 FontTracking->"Plain",
 FontVariations->{"Outline"->False,
 "Shadow"->False,
 "Underline"->False},
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "\nThus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "3"], " ", "-", " ", 
    RowBox[{"3", " ", 
     SuperscriptBox["x", "2"]}], " ", "+", " ", 
    RowBox[{"4", " ", "x"}], " ", "-", " ", "5"}], TraditionalForm]]],
 " divided by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", " ", 
     SuperscriptBox["x", "2"]}], " ", "-", " ", "5"}], TraditionalForm]]],
 " yields ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["1", "2"], "x"}], "-", " ", 
    FractionBox["3", "2"]}], TraditionalForm]]],
 ", with a remainder of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FractionBox["13", "2"], " ", "x"}], " ", "-", " ", 
    FractionBox["25", "2"]}], TraditionalForm]]],
 ". We can write this as\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["x", "3"], " ", "-", " ", 
      RowBox[{"3", " ", 
       SuperscriptBox["x", "2"]}], " ", "+", " ", 
      RowBox[{"4", " ", "x"}], " ", "-", " ", "5"}], ")"}], " ", "=", "   ", 
    "\n", "\t       ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         SuperscriptBox["x", "2"]}], " ", "-", " ", "5"}], ")"}], " ", 
      "\[CenterDot]", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FractionBox["1", "2"], "x"}], " ", "-", " ", 
        FractionBox["3", "2"]}], ")"}]}], " ", "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FractionBox["13", "2"], " ", "x"}], " ", "-", 
        FractionBox[
         RowBox[{" ", "25"}], "2"]}], ")"}], "."}]}]}], TraditionalForm]]],
 "\n\nFortunately, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has two commands which performs this tedious long division for you:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4377450023125*^9, 3.437745006234375*^9}}],

Cell[CellGroupData[{

Cell["\<\
PolynomialQuotient[x^3 - 3 x^2 + 4 x - 5, 
                         2 x^2 - 5, x]\
\>", "Input",
 PageWidth->Infinity,
 PageBreakAbove->True,
 CellChangeTimes->{{3.43774500728125*^9, 3.437745008390625*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["3", "2"]}], "+", 
  FractionBox["x", "2"]}]], "Output",
 CellChangeTimes->{3.4419815687764997`*^9, 3.442755104305125*^9}]
}, Open  ]],

Cell["and ", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
PolynomialRemainder[x^3 - 3 x^2 + 4 x - 5, 
                          2 x^2 - 5, x]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437745008875*^9, 3.437745009859375*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["25", "2"]}], "+", 
  FractionBox[
   RowBox[{"13", " ", "x"}], "2"]}]], "Output",
 CellChangeTimes->{3.4419815708389997`*^9, 3.442755106961375*^9}]
}, Open  ]],

Cell[TextData[{
 "\nEXPERIMENT:\nTry replacing the polynomials in the last two statements \
with different polynomials in ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". What do you observe about the degree of the remainder in comparison to \
the degree of the divisor?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745013796875*^9, 3.437745013796875*^9}}],

Cell["    ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "This \"long division\" algorithm works for any field, not just the rational \
numbers ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 ". We can prove this by induction on the degree of the dividend.\n \nTHEOREM \
12.1:  THE DIVISION ALGORITHM THEOREM\nLet ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " be a field, and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " be the set of polynomials in ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " over ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " be two elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " non-zero. Then there exist unique polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " such that \n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{"q", "(", "x", ")"}]}], " ", "+", " ", 
     RowBox[{"r", "(", "x", ")"}], " "}]}], TraditionalForm]]],
 "\n\nand either ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"r", "(", "x", ")"}], " ", "=", " ", "0"}], TraditionalForm]]],
 " or the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " is less than the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745031671875*^9, 3.4377450380625*^9}, {
  3.437746384515625*^9, 3.437746384640625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We begin by showing that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " exist, and then prove that they are unique. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", "0"}], TraditionalForm]]],
 ", or if the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is less than the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 ", we can simply let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"q", "(", "x", ")"}], " ", "=", " ", "0"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"r", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], "."}]}], TraditionalForm]]],
 " So we may suppose that the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is at least as large as the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], "."}], TraditionalForm]]],
 " Let ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " be the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], ","}], TraditionalForm]]],
 " and let ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " be the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], "."}], TraditionalForm]]],
 "\n\nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", 
    RowBox[{"m", " ", "=", " ", "0"}]}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " are both non-zero constants in the field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 ", so we may pick ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " to be the constant polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", " ", "\[CenterDot]", " ", 
    SuperscriptBox["g", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 ", and pick ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"r", "(", "x", ")"}], " ", "=", " ", "0"}], TraditionalForm]]],
 ".  Thus, we can find a suitable ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "0."}], TraditionalForm]]],
 "\n\nNow let us proceed by induction on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". That is, we will assume that we can find a suitable ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " whenever the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is less than ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "."}], TraditionalForm]]],
 " Let \n\n                ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{" ", 
       RowBox[{
        RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            SubscriptBox["a", "n"], 
            SuperscriptBox["x", "n"]}], " ", "+", " ", 
           RowBox[{
            SubscriptBox["a", 
             RowBox[{"n", "-", "1"}]], 
            SuperscriptBox["x", 
             RowBox[{"n", "-", "1"}]]}], " ", "+"}], " ", "..."}], " ", "+", 
         " ", 
         SubscriptBox["a", "0"]}]}]}],
      TraditionalForm], ",", "    ", "and"}], "                "}], 
   TraditionalForm], "\n", 
  FormBox[
   FormBox[
    RowBox[{" ", 
     RowBox[{
      RowBox[{"g", "(", "x", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          SubscriptBox["b", "m"], 
          SuperscriptBox["x", "m"]}], " ", "+", " ", 
         RowBox[{
          SubscriptBox["b", 
           RowBox[{"m", "-", "1"}]], 
          SuperscriptBox["x", 
           RowBox[{"m", "-", "1"}]]}], " ", "+"}], " ", "..."}], " ", "+", 
       " ", 
       RowBox[{
        SubscriptBox["b", "0"], "."}]}]}]}],
    TraditionalForm], TraditionalForm]}]],
 "\n\nSince ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is at least as large as ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", we can consider the polynomial\n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "(", "x", ")"}], " ", "=", "  ", 
    RowBox[{
     SubscriptBox["a", "n"], 
     SubsuperscriptBox["b", "m", 
      RowBox[{"-", "1"}]], 
     SuperscriptBox["x", 
      RowBox[{
       RowBox[{"n", "-", "m"}], " "}]]}]}], TraditionalForm]]],
 "  .\n\nof degree ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "m"}], TraditionalForm]]],
 ". By lemma 11.1, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "(", "x", ")"}], " ", "\[CenterDot]", " ", 
    RowBox[{"g", "(", "x", ")"}]}], TraditionalForm]]],
 " has degree ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and in fact, since\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"p", "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      RowBox[{"g", "(", "x", ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SubscriptBox["a", "n"], 
         SuperscriptBox["x", "n"]}], " ", "+", " ", 
        RowBox[{
         SubscriptBox["a", "n"], 
         SubsuperscriptBox["b", "m", 
          RowBox[{"-", "1"}]], 
         SubscriptBox["b", 
          RowBox[{"m", "-", "1"}]], " ", 
         SuperscriptBox["x", 
          RowBox[{"n", "-", "1"}]]}], " ", "+"}], " ", "..."}], " ", "+", " ",
       
      RowBox[{
       SubscriptBox["a", "n"], 
       SubsuperscriptBox["b", "m", 
        RowBox[{"-", "1"}]], 
       SubscriptBox["b", "0"], " ", 
       SuperscriptBox["x", 
        RowBox[{"n", "-", "m"}]]}]}]}], "   ", ","}], TraditionalForm]]],
 "\n\nthe coefficient of the ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "n"], TraditionalForm]]],
 "  term would be ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "n"], TraditionalForm]]],
 ". Thus,",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "-", " ", 
     RowBox[{
      RowBox[{"p", "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      RowBox[{"g", "(", "x", ")"}]}]}]}], TraditionalForm]]],
 " is of degree less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". So by the induction hypothesis, there exist polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " such that\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "-", " ", 
     RowBox[{
      RowBox[{"p", "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      RowBox[{"g", "(", "x", ")"}]}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"z", "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      RowBox[{"g", "(", "x", ")"}]}], " ", "+", " ", 
     RowBox[{"r", "(", "x", ")"}]}]}], TraditionalForm]]],
 "\n\nwith the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " less than the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], "."}], TraditionalForm]]],
 " Thus, \n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"p", "(", "x", ")"}], " ", "+", " ", 
         RowBox[{"z", "(", "x", ")"}]}], ")"}], " ", "\[CenterDot]", " ", 
       RowBox[{"g", "(", "x", ")"}]}], " ", "+", " ", 
      RowBox[{
       RowBox[{"r", "(", "x", ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nBy letting ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"q", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"p", "(", "x", ")"}], " ", "+", " ", 
      RowBox[{"z", "(", "x", ")"}]}]}], ","}], TraditionalForm]]],
 " we have proved that suitable ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " exist.\n\n\nNext, let us prove that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"r", "(", "x", ")"}], " "}], TraditionalForm]]],
 "are unique. Suppose that there is a second pair ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     OverscriptBox["q", "_"],
     TraditionalForm], "(", "x", ")"}], TraditionalForm]]],
 " and",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      OverscriptBox[
       RowBox[{" ", "r"}], "_"],
      TraditionalForm], 
     RowBox[{"(", "x", ")"}]}]}], TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       OverscriptBox["q", "_"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      RowBox[{"g", "(", "x", ")"}]}], " ", "+", "  ", 
     RowBox[{
      OverscriptBox[
       RowBox[{" ", "r"}], "_"], 
      RowBox[{
       RowBox[{"(", "x", ")"}], "."}]}]}]}], TraditionalForm]]],
 " Then\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        OverscriptBox["q", "_"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
       RowBox[{"g", "(", "x", ")"}]}], " ", "+", " ", 
      RowBox[{
       OverscriptBox[
        RowBox[{" ", "r"}], "_"], 
       RowBox[{"(", "x", ")"}]}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"q", "(", "x", ")"}], " ", "\[CenterDot]", " ", 
       RowBox[{"g", "(", "x", ")"}]}], " ", "+", " ", 
      RowBox[{"r", "(", "x", ")"}]}]}], ",", " "}], TraditionalForm]]],
 "\nor\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{
         OverscriptBox["q", "_"], "(", "x", ")"}], " ", "-", " ", 
        RowBox[{"q", "(", "x", ")"}]}], " ", ")"}], " ", "\[CenterDot]", " ", 
      
      RowBox[{"g", "(", "x", ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       OverscriptBox[
        RowBox[{" ", "r"}], "_"], 
       RowBox[{"(", "x", ")"}]}], " ", "-", " ", 
      RowBox[{
       RowBox[{"r", "(", "x", ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n       \nThe left hand side is either 0 (when ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      OverscriptBox["q", "_"],
      TraditionalForm], "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"q", "(", "x", ")"}]}], TraditionalForm]]],
 " ), or has degree at least ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is of degree ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ". The right hand side is either 0, or has a degree less than ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ". This is a contradiction unless both sides of the equation are 0. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      OverscriptBox["q", "_"],
      TraditionalForm], "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"q", "(", "x", ")"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     OverscriptBox[
      RowBox[{" ", "r"}], "_"], 
     RowBox[{"(", "x", ")"}]}], " ", "=", " ", 
    RowBox[{"r", "(", "x", ")"}]}], TraditionalForm]]],
 ", and the uniqueness has been proven."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774503865625*^9, 3.437745074171875*^9}, {
  3.437746458734375*^9, 3.437746458890625*^9}}]
}, Open  ]],

Cell[TextData[{
 "\nCOROLLARY 12.1\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be an integral domain, and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " be two polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ". If there is a field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " containing ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " as polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", and if the leading coefficient of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is 1, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745075125*^9, 3.43774507871875*^9}, {
  3.437746543390625*^9, 3.437746543515625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "The only time that we needed to use a division in the proof of the division \
algorithm theorem (12.1) is when we divided by the leading coefficient of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 ". Thus, if the leading coefficient of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is 1, we can do all of the operations in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " instead of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ". The result is that there are polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " such that\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{"q", "(", "x", ")"}]}], " ", "+", " ", 
     RowBox[{"r", "(", "x", ")"}]}]}], TraditionalForm]]],
 "\n                  \nin ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " in the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ". So there is an ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " such that \n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"h", "(", "x", ")"}]}]}], TraditionalForm]]],
 ".\n\nBut ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " can also be viewed as polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", and the division algorithm shows that these are uniquely defined, even in \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"q", "(", "x", ")"}], " ", "=", "  ", 
     RowBox[{"h", "(", "x", ")"}]}], ","}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"r", "(", "x", ")"}], " ", "=", " ", "0"}], TraditionalForm]]],
 ". Therefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745079171875*^9, 3.437745088265625*^9}, {
  3.43774656634375*^9, 3.437746566453125*^9}}]
}, Open  ]],

Cell[TextData[{
 "We are used to thinking of polynomials as functions, rather then as \
elements in a domain. If we want to \"evaluate\" a polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " at a particular value ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", we run into a technical problem, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is not a function. The division algorithm comes to our rescue on the \
occasions when we do need to evaluate polynomials at a particular value.   \n\
\nDEFINITION 12.1:\nLet ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " be a field or integral domain, and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 " be the set of polynomials in ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " over ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". For a fixed element ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", define the mapping\n\n                                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Phi]", "y"], ":", " ", 
    RowBox[{
     RowBox[{"K", "[", "x", "]"}], " ", "\[RightArrow]", " ", "K"}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774508865625*^9, 3.437745090625*^9}, {
  3.437746612296875*^9, 3.4377466140625*^9}}],

Cell[TextData[{
 "by  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "y"], "(", " ", 
     RowBox[{"f", "(", "x", ")"}], " ", ")"}], "=", " "}], 
   TraditionalForm]]],
 "the remainder ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is divided by the first degree polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", " ", "-", " ", "y"}], ")"}], TraditionalForm]]],
 ". Since either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " is 0 or is of degree 0, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " is in fact in ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745091109375*^9, 3.4377450931875*^9}, {
  3.437746641296875*^9, 3.437746650203125*^9}}],

Cell[TextData[{
 "PROPOSITION 12.1\nThe mapping ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Phi]", 
     RowBox[{"y", " "}]], ":", " ", 
    RowBox[{
     RowBox[{"K", "[", "x", "]"}], " ", "\[RightArrow]", " ", "K"}]}], 
   TraditionalForm]]],
 " is a homomorphism, called the ",
 StyleBox["evaluation homomorphism at ",
  FontSlant->"Italic"],
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "."}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774509371875*^9, 3.43774509371875*^9}, {
  3.437746653*^9, 3.437746653109375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "1"], "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "2"], "(", "x", ")"}], TraditionalForm]]],
 " be two polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"K", "[", "x", "]"}], TraditionalForm]]],
 ". By the division algorithm theorem (12.1) there exists ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["q", "1"], "(", "x", ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["q", "2"], "(", "x", ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "y"], "(", 
     RowBox[{
      SubscriptBox["f", "1"], "(", "x", ")"}], ")"}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["r", "1"], "(", "x", ")"}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "y"], "(", 
     RowBox[{
      SubscriptBox["f", "2"], "(", "x", ")"}], ")"}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["r", "2"], "(", "x", ")"}]}], TraditionalForm]]],
 " such that\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", "1"], "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "-", " ", "y"}], ")"}], " ", 
       RowBox[{
        SubscriptBox["q", "1"], "(", "x", ")"}]}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["r", "1"], "(", "x", ")"}]}]}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["f", "2"], "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", "y"}], ")"}], " ", 
      RowBox[{
       SubscriptBox["q", "2"], "(", "x", ")"}]}], " ", "+", " ", 
     RowBox[{
      SubscriptBox["r", "2"], "(", "x", ")"}]}]}], TraditionalForm]]],
 ".\n      \nThen \n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["f", "1"], "(", "x", ")"}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["f", "2"], "(", "x", ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "-", " ", "y"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["q", "1"], "(", "x", ")"}], " ", "+", " ", 
         RowBox[{
          SubscriptBox["q", "2"], "(", "x", ")"}]}], ")"}]}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["r", "1"], "(", "x", ")"}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["r", "2"], "(", "x", ")"}]}]}], ","}], 
   TraditionalForm]]],
 " \nand \n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["f", "1"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
     RowBox[{
      SubscriptBox["f", "2"], "(", "x", ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"x", "-", "y"}], ")"}], "\[CenterDot]", " ", 
         RowBox[{
          SubscriptBox["q", "1"], "(", "x", ")"}]}], " ", "+", " ", 
        RowBox[{
         SubscriptBox["r", "1"], "(", "x", ")"}]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"x", "-", "y"}], ")"}], "\[CenterDot]", " ", 
         RowBox[{
          SubscriptBox["q", "2"], "(", "x", ")"}]}], " ", "+", " ", 
        RowBox[{
         SubscriptBox["r", "2"], "(", "x", ")"}]}], ")"}]}], " ", "="}]}], 
   TraditionalForm]]],
 "\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", "-", "y"}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"x", "-", "y"}], ")"}], " ", 
        RowBox[{
         SubscriptBox["q", "1"], "(", "x", ")"}], " ", 
        RowBox[{
         SubscriptBox["q", "2"], "(", "x", ")"}]}], " ", "+", " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["q", "1"], "(", "x", ")"}], " ", 
        RowBox[{
         SubscriptBox["r", "2"], "(", "x", ")"}]}], " ", "+", " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["q", "2"], "(", "x", ")"}], " ", 
        RowBox[{
         SubscriptBox["r", "1"], "(", "x", ")"}]}]}], " ", ")"}]}], " ", "+", 
    " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["r", "1"], "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{
      RowBox[{
       SubscriptBox["r", "2"], "(", "x", ")"}], "."}]}]}], 
   TraditionalForm]]],
 "\n \nBy the uniqueness of the division algorithm, we have that \n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["\[Phi]", "y"], "(", 
      RowBox[{
       RowBox[{
        SubscriptBox["f", "1"], "(", "x", ")"}], " ", "+", " ", 
       RowBox[{
        SubscriptBox["f", "2"], "(", "x", ")"}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["r", "1"], "(", "x", ")"}], " ", "+", " ", 
       RowBox[{
        SubscriptBox["r", "2"], "(", "x", ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Phi]", "y"], "(", 
        RowBox[{
         SubscriptBox["f", "1"], "(", "x", ")"}], ")"}], " ", "+", " ", 
       RowBox[{
        SubscriptBox["\[Phi]", "y"], "(", 
        RowBox[{
         SubscriptBox["f", "2"], "(", "x", ")"}], ")"}]}]}]}], ","}], 
   TraditionalForm]]],
 "   and \n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Phi]", "y"], "(", 
      RowBox[{
       RowBox[{
        SubscriptBox["f", "1"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
       RowBox[{
        SubscriptBox["f", "2"], "(", "x", ")"}]}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["r", "1"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
       RowBox[{
        SubscriptBox["r", "2"], "(", "x", ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[Phi]", "y"], "(", 
        RowBox[{
         SubscriptBox["f", "1"], "(", "x", ")"}], ")"}], " ", "\[CenterDot]", 
       " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["\[Phi]", "y"], "(", 
         RowBox[{
          SubscriptBox["f", "2"], "(", "x", ")"}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 " \n\nThus, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Phi]", "y"], TraditionalForm]]],
 " is a homomorphism."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745094203125*^9, 3.437745110125*^9}, {
  3.437746688421875*^9, 3.43774668853125*^9}}]
}, Open  ]],

Cell[TextData[{
 "We will often denote ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Phi]", "y"], "(", " ", 
    RowBox[{"f", "(", "x", ")"}], " ", ")"}], TraditionalForm]]],
 " by the conventional notation, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "y", ")"}], TraditionalForm]]],
 ". However, whenever we want to emphasize the homomorphism property, we will \
use the notation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "y"], "(", 
     RowBox[{"f", "(", "x", ")"}], ")"}], " "}], TraditionalForm]]],
 "for the evaluation homomorphism. This homomophism can be used in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with the command ",
 StyleBox["ReplaceAll",
  FontWeight->"Bold"],
 ". To evaluate the polynomial  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "3"], " ", "+", " ", 
    RowBox[{"5", " ", 
     SuperscriptBox["x", "2"]}], " ", "+", " ", 
    RowBox[{"4", " ", "x"}], " ", "-", " ", "4"}], TraditionalForm]]],
 " at ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "3"}], TraditionalForm]]],
 ", enter "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745110765625*^9, 3.437745112828125*^9}}],

Cell[CellGroupData[{

Cell["ReplaceAll[ x^3 + 5 x^2 + 4 x - 4, x -> 3]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437745113640625*^9, 3.43774511459375*^9}},
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData["80"], "Output",
 CellChangeTimes->{3.4427551162895*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice how a minus sign and a greater than sign make up the arrow in this \
command. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " also provides an abbreviation for this command:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["x^3 + 5 x^2 + 4 x - 4 /. x -> 3", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437745115109375*^9, 3.437745116140625*^9}},
 FontFamily->"Courier",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->GrayLevel[0],
 Background->GrayLevel[1]],

Cell[BoxData["80"], "Output",
 CellChangeTimes->{3.442755119273875*^9}]
}, Open  ]],

Cell[TextData[{
 "Here, the /. is an abbreviation for ",
 StyleBox["ReplaceAll",
  FontWeight->"Bold"],
 ", but it appears ",
 StyleBox["after",
  FontSlant->"Italic"],
 " the polynomial.\n\nEXPERIMENT:\nTry changing the 3 in the above command \
with different integers. Can you find an integer for which the polynomial \
evaluates to 0?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "This experiment shows how we can use the evaluation homomorphism to define \
what it means for a polynomial to have a root.\n\nDEFINITION 12.2:\nLet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " be a polynomial over the field or integral domain ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is an element of ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "r"], "(", 
     RowBox[{"f", "(", "x", ")"}], ")"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is called a ",
 StyleBox["zero",
  FontSlant->"Italic"],
 ", or a ",
 StyleBox["root",
  FontSlant->"Italic"],
 ", of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ". Of course this is equivalent to saying that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", "-", "r"}], ")"}], TraditionalForm]]],
 " is a factor of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ".\n\nExample 12.1:  \n   Consider the polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "+", " ", "1"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "5"], "[", "x", "]"}], TraditionalForm]]],
 ". We can visually evaluate this polynomial at ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "2"}], TraditionalForm]]],
 " to see that \n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Phi]", "2"], "(", 
     RowBox[{
      SuperscriptBox["x", "2"], " ", "+", " ", "1"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["2", "2"], " ", "+", " ", "1"}], " ", "\[Congruent]", 
     " ", "0."}]}], TraditionalForm]]],
 "\n\nin the field ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 ". Thus, 2 is a ",
 StyleBox["root",
  FontSlant->"Italic"],
 ", or a ",
 StyleBox["zero",
  FontSlant->"Italic"],
 ", of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "+", " ", "1."}], TraditionalForm]]],
 " \n\nEXPERIMENT: \nSince 2 is a root, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", "-", "2"}], ")"}], TraditionalForm]]],
 " must be a factor of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "+", " ", "1."}], TraditionalForm]]],
 " What is the other factor?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745129875*^9, 3.437745145296875*^9}, {
  3.43774677321875*^9, 3.43774682015625*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "As one can imagine, the factorization of a polynomial over an arbitrary \
field can be more cumbersome then the customary factorization. For a finite \
field, (such as ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 "), almost the only way to find roots is by trial and error. Fortunately, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can do this very quickly. However, the good news is that if we have found \
enough roots to a polynomial, we already have the factorization.\n\n\
PROPOSITION 12.2\nLet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " be a polynomial over the field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " that has positive degree ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and leading coefficient ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "n"], TraditionalForm]]],
 ". \nIf ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "1"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "3"], TraditionalForm]]],
 ", \[Ellipsis] ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "n"], TraditionalForm]]],
 " are ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " distinct zeros of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ", then\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", "n"], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        SubscriptBox["r", "1"]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        SubscriptBox["r", "2"]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        SubscriptBox["r", "3"]}], ")"}]}], " ", "\[CenterEllipsis]", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        SubscriptBox["r", "n"]}], ")"}], "."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745151125*^9, 3.437745153078125*^9}, {
  3.437746852484375*^9, 3.43774687346875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Again, we will proceed by induction on the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ", which we will call ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "1"}], TraditionalForm]]],
 ", then",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["a", "1"], "x"}], " ", "+", " ", 
      SubscriptBox["a", "0"]}]}]}], TraditionalForm]]],
 ", and since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "1"], TraditionalForm]]],
 " is a root, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["a", "1"], 
      SubscriptBox["r", "1"]}], " ", "+", " ", 
     SubscriptBox["a", "0"]}], " ", "=", " ", "0."}], TraditionalForm]]],
 " Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "0"], " ", "=", " ", 
    RowBox[{
     RowBox[{"-", " ", 
      SubscriptBox["a", "1"]}], " ", 
     SubscriptBox["r", "1"]}]}], TraditionalForm]]],
 " , and hence \n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["a", "1"], "x"}], " ", "-", " ", 
       RowBox[{
        SubscriptBox["a", "1"], " ", 
        SubscriptBox["r", "1"]}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["a", "1"], "(", " ", 
        RowBox[{"x", " ", "-", " ", 
         SubscriptBox["r", "1"]}], ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nSo the proposition is true when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "1"}], TraditionalForm]]],
 ". Now we will apply the induction hypothesis on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "n"], TraditionalForm]]],
 " is a root of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ", we have that \n\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        SubscriptBox["r", "n"]}], ")"}], " ", 
      RowBox[{"g", "(", "x", ")"}]}]}]}], TraditionalForm]]],
 "\n\nfor some ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], ","}], TraditionalForm]]],
 " which by lemma 11.1 is of degree ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ". Furthermore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " have the same leading coefficient, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "n"], TraditionalForm]]],
 ".  For",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"i", " ", "=", " ", "1"}], ",", " ", "2", ",", " ", "...", ",", 
     " ", 
     RowBox[{"n", "-", "1"}]}]}], TraditionalForm]]],
 ", we have\n\n         ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"0", " ", "=", "  ", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[Phi]", 
        SubscriptBox["r", "i"]], "(", 
       RowBox[{"f", "(", "x", ")"}], ")"}], " ", "=", "  ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["r", "i"], "-", " ", 
          SubscriptBox["r", "n"]}], ")"}], " ", "\[CenterDot]", " ", 
        SubscriptBox[
         RowBox[{"\[Phi]", " "}], 
         SubscriptBox["r", "i"]]}], 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"g", "(", "x", ")"}], ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{"(", 
       SubscriptBox["r", "i"]}],
      TraditionalForm], " ", "-", " ", 
     FormBox[
      SubscriptBox["r", "n"],
      TraditionalForm]}], ")"}], TraditionalForm]]],
 " is not 0, we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " has",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"n", "-", "1"}]}], TraditionalForm]]],
 " distinct roots, namely ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["r", 
      RowBox[{"1", ","}]], 
     SubscriptBox["r", "2"]}], ",", " ", 
    SubscriptBox["r", "3"], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     SubscriptBox["r", 
      RowBox[{"n", "-", "1"}]]}]}], TraditionalForm]]],
 ". Thus, by induction,\n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", "n"], "(", 
      RowBox[{"x", " ", "-", " ", 
       SubscriptBox["r", "1"]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"x", " ", "-", " ", 
       SubscriptBox["r", "2"]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"x", " ", "-", " ", 
       SubscriptBox["r", "3"]}], ")"}], " ", "\[Ellipsis]", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        SubscriptBox["r", 
         RowBox[{"n", "-", "1"}]]}], ")"}], "."}]}]}], TraditionalForm]]],
 "\n\nThus,   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["a", "n"], "(", 
       RowBox[{"x", " ", "-", " ", 
        SubscriptBox["r", "1"]}], ")"}], 
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        SubscriptBox["r", "2"]}], ")"}], 
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        SubscriptBox["r", "3"]}], ")"}], " ", "\[Ellipsis]", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "-", " ", 
         SubscriptBox["r", "n"]}], ")"}], "."}]}]}]}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745153703125*^9, 3.43774516171875*^9}, {
  3.43774733096875*^9, 3.43774734565625*^9}, {3.43774741525*^9, 
  3.437747478890625*^9}}]
}, Open  ]],

Cell[TextData[{
 "\nCOROLLARY 12.2\nA polynomial of positive degree ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " over the field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " has at \nmost ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " distinct zeros in ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437747482109375*^9, 3.437747482640625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " has at least",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"n", "+", "1"}]}], TraditionalForm]]],
 " roots, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["r", "1"], ",", 
    SubscriptBox["r", "2"], ",", 
    RowBox[{"...", " ", 
     SubscriptBox["r", "n"]}], ",", " ", 
    SubscriptBox["r", 
     RowBox[{"n", "+", "1"}]]}], TraditionalForm]]],
 ". From proposition 12.2,\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["a", "n"], "(", 
        RowBox[{"x", " ", "-", " ", 
         SubscriptBox["r", "1"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", " ", "-", " ", 
         SubscriptBox["r", "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", " ", "-", " ", 
         SubscriptBox["r", "3"]}], ")"}]}], " ", "..."}], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        SubscriptBox["r", "n"]}], ")"}], "."}]}]}], TraditionalForm]]],
 "\n             \nSince ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", 
    RowBox[{"n", "+", "1"}]], TraditionalForm]]],
 "  is also is root, we have\n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Phi]", 
       SubscriptBox["r", 
        RowBox[{"n", "+", "1"}]]], "(", 
      RowBox[{"f", "(", "x", ")"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SubscriptBox["a", "n"], "(", 
         RowBox[{
          SubscriptBox["r", 
           RowBox[{"n", "+", "1"}]], "-", " ", 
          SubscriptBox["r", "1"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["r", 
           RowBox[{"n", "+", "1"}]], "-", " ", 
          SubscriptBox["r", "2"]}], ")"}], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["r", 
           RowBox[{"n", "+", "1"}]], "-", " ", 
          SubscriptBox["r", "3"]}], ")"}]}], " ", "..."}], " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["r", 
          RowBox[{"n", "+", "1"}]], "-", " ", 
         SubscriptBox["r", "n"]}], ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nBut all of the terms on the right hand side are non-zero, which is a \
contradiction. Thus, there can be at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " distinct zeros of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], "."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745162390625*^9, 3.437745164546875*^9}, {
  3.4377474919375*^9, 3.437747528140625*^9}}]
}, Open  ]],

Cell[TextData[{
 "We can use proposition 12.2 to do some factorizations in different fields. \
For example, the polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "+", " ", "1"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "5"], "[", "x", "]"}], TraditionalForm]]],
 " was found to have one root, 2. With a bit of experimenting another root \
could be found: 3. Thus\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "2"], " ", "+", " ", "1"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "-", " ", "2"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"x", " ", "-", " ", "3"}], ")"}], "  ", "in", "  ", 
     RowBox[{
      SubscriptBox["Z", "5"], "."}]}]}], TraditionalForm]]],
 "\n\nEXPERIMENT:\nUse direct evaluation to find three roots of the \
polynomial \n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", "  ", 
    RowBox[{
     RowBox[{"3", " ", 
      SuperscriptBox["x", "3"]}], " ", "+", " ", 
     RowBox[{"2", " ", 
      SuperscriptBox["x", "2"]}], " ", "+", " ", 
     RowBox[{"2", " ", "x"}], " ", "+", " ", 
     RowBox[{"6", "  ", "in", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["Z", "7"], "[", "x", "]"}], "."}]}]}]}], 
   TraditionalForm]]],
 " \n\nFor example, 0 is not a root since "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774516525*^9, 3.43774517090625*^9}, {
  3.437747539265625*^9, 3.43774753971875*^9}}],

Cell[CellGroupData[{

Cell["3 x^3 + 2 x^2 + 2 x + 6 /. x -> 0", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437745171390625*^9, 3.437745172140625*^9}}],

Cell[BoxData["6"], "Output",
 CellChangeTimes->{3.4427551286645*^9}]
}, Open  ]],

Cell[TextData[{
 "is not divisible by 7. Once you have found 3 roots, write the factorization \
of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774517275*^9, 3.43774517275*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Here is an application of corollary 12.2 that has many applications even \
using the real number field.\n\nCOROLLARY 12.3\nLet ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " be a field, and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "0"], ",", " ", 
    SubscriptBox["x", "1"], ",", " ", 
    SubscriptBox["x", "2"], ",", " ", 
    SubscriptBox["x", "3"], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     SubscriptBox["x", "n"]}]}], TraditionalForm]]],
 "  be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " distinct elements of ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["y", "0"], ",", " ", 
    SubscriptBox["y", "1"], ",", " ", 
    SubscriptBox["y", "2"], ",", " ", 
    SubscriptBox["y", "3"], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     SubscriptBox["y", "n"]}]}], TraditionalForm]]],
 "  be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " values in ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " (not necessarily distinct.) Then there is a unique polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "x", ")"}], TraditionalForm]]],
 " with degree at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " such that\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      SubscriptBox["x", "0"], ")"}], " ", "=", " ", 
     SubscriptBox["y", "0"]}], ",", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      SubscriptBox["x", "1"], ")"}], " ", "=", " ", 
     SubscriptBox["y", "1"]}], ",", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      SubscriptBox["x", "2"], ")"}], " ", "=", " ", 
     SubscriptBox["y", "2"]}], ",", "\[Ellipsis]", " ", ",", 
    RowBox[{
     RowBox[{"\[Florin]", "(", 
      SubscriptBox["x", "n"], ")"}], " ", "=", " ", 
     RowBox[{
      SubscriptBox["y", "n"], "."}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745173171875*^9, 3.437745173171875*^9}, {
  3.43774755484375*^9, 3.437747563875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "To prove uniqueness, suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " are two such polynomials. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"h", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"\[Florin]", "(", "x", ")"}], " ", "-", " ", 
     RowBox[{"g", "(", "x", ")"}]}]}], TraditionalForm]]],
 " will have roots at ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "0"], ",", " ", 
    SubscriptBox["x", "1"], ",", " ", 
    SubscriptBox["x", "2"], ",", " ", 
    SubscriptBox["x", "3"], ",", " ", "\[Ellipsis]", ",", " ", 
    RowBox[{
     SubscriptBox["x", "n"], "."}]}], TraditionalForm]]],
 " But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " would have degree at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", which contradicts corollary 12.2. Thus,\nthe polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Florin]", "(", "x", ")"}], TraditionalForm]]],
 " is unique.\n\nTo show that this polynomial exists, we will first construct \
the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-th degree polynomial\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["f", "0"], "(", "x", ")"}], " ", "=", " ", 
    StyleBox[
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "-", " ", 
         SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"x", "-", 
         SubscriptBox["x", "2"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"x", "-", 
         SubscriptBox["x", "3"]}], ")"}], "\[CenterDot]", "\[CenterEllipsis]",
        "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"x", "-", 
         SubscriptBox["x", "n"]}], ")"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["x", "0"], "-", 
         SubscriptBox["x", "1"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["x", "0"], "-", 
         SubscriptBox["x", "2"]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["x", "0"], "-", 
         SubscriptBox["x", "3"]}], ")"}], "\[CenterDot]", "\[CenterEllipsis]",
        "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["x", "0"], "-", 
         SubscriptBox["x", "n"]}], ")"}]}]],
     FontSize->16]}], TraditionalForm]]],
 "\n\nfor which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{
       SubscriptBox["f", "0"], "(", 
       SubscriptBox["x", "0"], ")"}],
      TraditionalForm], " ", "=", " ", "1"}], ","}], TraditionalForm]]],
 " but ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "1"], ",", " ", 
    SubscriptBox["x", "2"], ",", " ", 
    SubscriptBox["x", "3"], ",", " ", "\[Ellipsis]", ",", " ", 
    SubscriptBox["x", "n"]}], TraditionalForm]]],
 "  are roots of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["f", "0"], "(", "x", ")"}], "."}], TraditionalForm]]],
 " (Note that since all of the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "i"], TraditionalForm]]],
 "  are distinct, the denominator is not 0.)\n\nWe can likewise define ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "1"], TraditionalForm]]],
 "(x), ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "2"], TraditionalForm]]],
 "(x), \[Ellipsis], ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "n"], TraditionalForm]]],
 "(x) such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", "1"], "(", 
      SubscriptBox["x", "1"], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["f", "2"], "(", 
       SubscriptBox["x", "2"], ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["f", "3"], "(", 
        SubscriptBox["x", "3"], ")"}], " ", "=", " ", 
       RowBox[{
        RowBox[{"\[CenterDot]", "\[CenterDot]", "\[CenterDot]"}], " ", "=", 
        " ", 
        RowBox[{
         RowBox[{
          SubscriptBox["f", "n"], "(", 
          SubscriptBox["x", "n"], ")"}], " ", "=", " ", "1"}]}]}]}]}]}], 
   TraditionalForm]]],
 ", yet the remaining ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "i"], TraditionalForm]]],
 "'s are roots for each polynomial. Finally, we construct the polynomial\n\n\
",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"g", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["y", "0"], " ", 
       RowBox[{
        SubscriptBox["f", "0"], "(", "x", ")"}]}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["y", "1"], " ", 
       RowBox[{
        SubscriptBox["f", "1"], "(", "x", ")"}]}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["y", "2"], " ", 
       RowBox[{
        SubscriptBox["f", "2"], "(", "x", ")"}]}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["y", "3"], " ", 
       RowBox[{
        SubscriptBox["f", "3"], "(", "x", ")"}]}], " ", "+", " ", 
      "\[Ellipsis]", " ", "+", " ", 
      RowBox[{
       SubscriptBox["y", "n"], " ", 
       RowBox[{
        RowBox[{
         SubscriptBox["f", "n"], "(", "x", ")"}], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nClearly ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " will be a polynomial of degree at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and also",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"g", "(", 
      SubscriptBox["x", "0"], ")"}], " ", "=", " ", 
     SubscriptBox["y", "0"]}]}], TraditionalForm]]],
 " , ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", 
     SubscriptBox["x", "1"], ")"}], " ", "=", " ", 
    SubscriptBox["y", "1"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", 
     SubscriptBox["x", "2"], ")"}], " ", "=", " ", 
    SubscriptBox["y", "2"]}], TraditionalForm]]],
 " , ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", 
     SubscriptBox["x", "3"], " ", ")"}], " ", "=", " ", 
    SubscriptBox["y", "3"]}], TraditionalForm]]],
 " , \[Ellipsis] ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", 
     SubscriptBox["x", "n"], ")"}], " ", "=", " ", 
    SubscriptBox["y", "n"]}], TraditionalForm]]],
 ".\n\nThus, we have constructed the required polynomial."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745174765625*^9, 3.43774518496875*^9}, {
  3.43774758753125*^9, 3.437747598671875*^9}}]
}, Open  ]],

Cell[TextData[{
 "This corollary shows, for example, that knowing just three points of a \
quadratic function is sufficient to determine the quadratic function. \
Although this has the obvious applications to graphing polynomials, we will \
find in the next section some surprising real world applications when we \
apply this corollary to different fields. \n\nThe ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command\n\n",
 StyleBox["InterpolatingPolynomial",
  FontWeight->"Bold"],
 "[",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["x", "0"], ",", 
       SubscriptBox["y", "0"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["x", "1"], ",", 
       SubscriptBox["y", "1"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["x", "2"], ",", 
       SubscriptBox["y", "2"]}], "}"}], ",", 
     RowBox[{"\[Ellipsis]", " ", 
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["x", "n"], ",", 
        SubscriptBox["y", "n"]}], "}"}]}]}], "}"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "]\n\nfinds the polynomial in ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " derived in corollary 4.3. For example, "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745194953125*^9, 3.43774519559375*^9}}],

Cell[CellGroupData[{

Cell["InterpolatingPolynomial[{{1,2}, {2,4}, {3,8}}, x]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.437745196015625*^9}],

Cell[BoxData[
 RowBox[{"2", "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "x"}], ")"}], " ", "x"}]}]], "Output",
 CellChangeTimes->{3.442755133555125*^9}]
}, Open  ]],

Cell[TextData[{
 "finds the quadratic polynomial such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "1", ")"}], " ", "=", " ", "2"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "2", ")"}], "=", " ", "4"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "3", ")"}], " ", "=", " ", "8"}], TraditionalForm]]],
 ".\n\nEXPERIMENT:\nUse ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find a third degree polynomial such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "0", ")"}], " ", "=", " ", "1"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "1", ")"}], " ", "=", " ", "5"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "3", ")"}], " ", "=", " ", "2"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "5", ")"}], " ", "=", " ", 
    RowBox[{"-", "1"}]}], TraditionalForm]]],
 ". Have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " verify that this is the correct polynomial by evaluating at each of the \
four points.  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745196484375*^9, 3.437745208578125*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We are now ready to define the polynomials which in many ways acts as the \
prime numbers of number theory.\n\nDEFINITION 12.3:\nA polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is said to be ",
 StyleBox["irreducible",
  FontSlant->"Italic"],
 " over ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " has positive degree, and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " cannot be expressed as a product \n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"h", "(", "x", ")"}]}]}], TraditionalForm]]],
 "\n\nwhere both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " have positive degree. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is not irreducible, it is called ",
 StyleBox["reducible",
  FontSlant->"Italic"],
 ".\n\nWe saw above that  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "+", " ", "1"}], TraditionalForm]]],
 " was reducible over ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "5"], "."}], TraditionalForm]]],
 " However, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will claim that this polynomial is irreducible:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774521609375*^9, 3.437745220125*^9}, {
  3.437747669875*^9, 3.437747671734375*^9}}],

Cell[CellGroupData[{

Cell["Factor[x^2 + 1]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{3.437745220671875*^9}],

Cell[BoxData[
 RowBox[{"1", "+", 
  SuperscriptBox["x", "2"]}]], "Output",
 CellChangeTimes->{3.442755142352*^9}]
}, Open  ]],

Cell[TextData[{
 "The reason of course is that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is viewing this polynomial as an element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", "x", "]"}], TraditionalForm]]],
 ", not ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Z", "5"], "[", "x", "]"}], TraditionalForm]]],
 ". Yet this polynomial ",
 StyleBox["does",
  FontSlant->"Italic"],
 " have a factorization if we were allowed to work with complex numbers:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745221140625*^9, 3.4377452215*^9}}],

Cell[CellGroupData[{

Cell["Expand[(x + I)(x - I)]", "Input",
 PageWidth->Infinity,
 PageBreakAbove->True,
 CellChangeTimes->{{3.437745222109375*^9, 3.4377452225625*^9}}],

Cell[BoxData[
 RowBox[{"1", "+", 
  SuperscriptBox["x", "2"]}]], "Output",
 CellChangeTimes->{3.442755144398875*^9}]
}, Open  ]],

Cell[TextData[{
 "Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "+", " ", "1"}], TraditionalForm]]],
 " is reducable over ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalC]", TraditionalForm]]],
 ", the field of complex numbers. Thus, whether a polynomial is reducible or \
irreducible over ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " greatly depends on the field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 ".\n\nIt should be noted that if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " both have positive degree, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
    RowBox[{"h", "(", "x", ")"}]}], TraditionalForm]]],
 " has degree at least 2. Thus, all polynomials of degree 1 must be \
irreducible. Constant polynomials, however, are not considered to be \
irreducible. \n\nAlthough it can be tricky to decide whether a polynomial is \
reducible or irreducible, there is a way to test polynomials of low degree.\n\
\nPROPOSITION 12.3\nIf ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is a polynomial of degree 2 or 3 over the field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is reducible over ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " if, and only if, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " has a zero in ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745223546875*^9, 3.437745226984375*^9}, {
  3.437747715796875*^9, 3.4377477163125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " has a zero in ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 ", say ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ". Then \n\n                   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "-", " ", "r"}], ")"}], " ", 
     RowBox[{"q", "(", "x", ")"}]}]}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " has degree one less than ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ". This shows that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is reducible.\n\nNow suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is reducible. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"h", "(", "x", ")"}]}]}], TraditionalForm]]],
 ", where the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], " ", "plus", "  ", "the", " ", "degree", 
    " ", "of", "  ", 
    RowBox[{"h", "(", "x", ")"}], " ", "is", " ", "2", "  ", "or", "  ", 
    "3"}], TraditionalForm]]],
 ". Thus, either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " has degree 1. We may suppose ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " has degree one, and so\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["a", "1"], "x"}], " ", "+", " ", 
       SubscriptBox["a", "0"]}], ")"}], " ", 
     RowBox[{"h", "(", "x", ")"}], " "}]}], TraditionalForm]]],
 "\n                 \nThen  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", 
     SubscriptBox["a", "0"]}], 
    SubsuperscriptBox["a", "1", 
     RowBox[{"-", "1"}]]}], TraditionalForm]]],
 " is a root of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ", and the proof is complete."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745227515625*^9, 3.437745234890625*^9}, {
  3.437747737*^9, 3.43774773846875*^9}}]
}, Open  ]],

Cell[TextData[{
 "We can use this proposition to determine whether polynomials of degree less \
than 4 are irreducible over a finite field: simply plug in all elements of \
the field, and see if any of them produce 0 in that field. For example, \
consider\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "3"], " ", "+", " ", 
    RowBox[{"2", " ", 
     SuperscriptBox["x", "2"]}], " ", "-", " ", 
    RowBox[{"3", " ", "x"}], " ", "+", " ", "4", " "}], TraditionalForm]]],
 "  over ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 ".\n            \nWe have:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745243046875*^9, 3.43774524378125*^9}}],

Cell[CellGroupData[{

Cell["x^3 + 2 x^2 - 3 x + 4 /. x -> 0", "Input",
 PageWidth->Infinity,
 PageBreakAbove->True,
 CellChangeTimes->{{3.437745244640625*^9, 3.437745245234375*^9}}],

Cell[BoxData["4"], "Output",
 CellChangeTimes->{3.442755149977*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["x^3 + 2 x^2 - 3 x + 4 /. x -> 1", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43774524540625*^9, 3.437745245953125*^9}}],

Cell[BoxData["4"], "Output",
 CellChangeTimes->{3.442755150930125*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["x^3 + 2 x^2 - 3 x + 4 /. x -> 2", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437745246125*^9, 3.437745246671875*^9}}],

Cell[BoxData["14"], "Output",
 CellChangeTimes->{3.4427551521645*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["x^3 + 2 x^2 - 3 x + 4 /. x -> 3", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437745246875*^9, 3.4377452474375*^9}}],

Cell[BoxData["40"], "Output",
 CellChangeTimes->{3.442755153102*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["x^3 + 2 x^2 - 3 x + 4 /. x -> 4", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.4377452476875*^9, 3.43774524828125*^9}}],

Cell[BoxData["88"], "Output",
 CellChangeTimes->{3.44275515419575*^9}]
}, Open  ]],

Cell[TextData[{
 "One of these, namely when ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " was replaced by 3, produced a multiple of 5, which is equivalent to 0 in \
the field ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 ". Thus, this polynomial is reducible."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745248703125*^9, 3.437745248703125*^9}}],

Cell["\<\
EXPERIMENT:
Can you determine the other factor through the division algorithm?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["    ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "EXPERIMENT:\nDetermine whether the polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], " ", "+", " ", 
    RowBox[{"3", " ", "x"}], " ", "+", " ", "4"}], TraditionalForm]]],
 " is reducible over ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774525284375*^9, 3.437745253078125*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "PROPOSITION 12.4\nIf ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " is a field, then all polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " of positive degree are either irreducible, or can be expressed as a \
product of irreducible polynomials."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745253734375*^9, 3.437745253734375*^9}, {
  3.437747768578125*^9, 3.4377477691875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " has degree 1, then we have seen that it is irreducible. Let us proceed by \
induction on the degree ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is not irreducible, then we can express ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{"h", "(", "x", ")"}]}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " are polynomials of degree at least one. But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " must have degree less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Thus, by induction, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " are either irreducible, or can be written as a product of irreducible \
polynomials. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " can be written as a product of irreducible polynomials."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745254328125*^9, 3.437745258203125*^9}, {
  3.437747798140625*^9, 3.437747823203125*^9}}]
}, Open  ]],

Cell["\<\
One last tool we have to help us find irreducible polynomials is the Greatest \
Common Divisor (GCD) of two polynomials. The proof of the next theorem \
mimicks the proof of the greatest common divisor theorem for integers (1.2).\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437747866015625*^9, 3.43774787896875*^9}}],

Cell[TextData[{
 "THEOREM 12.2:  THE GREATEST COMMON DIVISOR THEOREM FOR POLYNOMIALS\n\nLet \
",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " be a field, and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " be the polynomials in ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " over the field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 ". Given two non-zero polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", there exists a non-zero polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " such that \n\n(1) ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " divides both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 ".\n\n(2) There exist polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"t", "(", "x", ")"}], TraditionalForm]]],
 " such\n    that\n \n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"f", "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{"s", "(", "x", ")"}]}], " ", "+", " ", 
      RowBox[{
       RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{"t", "(", "x", ")"}]}]}], " ", "=", " ", 
     RowBox[{"h", "(", "x", ")"}]}]}], TraditionalForm]]],
 ".\n\nFurthermore, the polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " is unique except for multiplication by a constant."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745262484375*^9, 3.437745267140625*^9}, {
  3.43774785428125*^9, 3.43774785478125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let us consider the set of",
 StyleBox[" all",
  FontSlant->"Italic"],
 " polynomials that can be produced by\n\n         ",
 StyleBox["f(",
  FontSlant->"Italic"],
 "x",
 StyleBox[")\[CenterDot]s(",
  FontSlant->"Italic"],
 "x",
 StyleBox[") + g(",
  FontSlant->"Italic"],
 "x",
 StyleBox[")\[CenterDot]t(",
  FontSlant->"Italic"],
 "x",
 StyleBox[")",
  FontSlant->"Italic"],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"t", "(", "x", ")"}], TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ". Call this set ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". Both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " contains non-zero polynomials. Consider a non-zero polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " of the lowest degree. By the division algorithm theorem (12.1) , we can \
find polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " such that \n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"q", "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{"h", "(", "x", ")"}]}], " ", "+", " ", 
      RowBox[{
       RowBox[{"r", "(", "x", ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " is either 0, or has lower degree than ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 ". But then\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"r", "(", "x", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"f", "(", "x", ")"}], " ", "-", " ", 
        RowBox[{
         RowBox[{"q", "(", "x", ")"}], "\[CenterDot]", 
         RowBox[{"h", "(", "x", ")"}]}]}], " ", "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", " ", "-", " ", 
           RowBox[{
            RowBox[{"q", "(", "x", ")"}], "\[CenterDot]", 
            RowBox[{"s", "(", "x", ")"}]}]}], ")"}], "\[CenterDot]", 
         RowBox[{"f", "(", "x", ")"}]}], " ", "-", " ", 
        RowBox[{
         RowBox[{"q", "(", "x", ")"}], "\[CenterDot]", 
         RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
         RowBox[{"t", "(", "x", ")"}]}]}]}]}], ","}]}], TraditionalForm]]],
 "\n\nwhich is in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". But if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " is not zero, the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " would be less than the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 ", and we picked ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " to be of the lowest degree. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"r", "(", "x", ")"}], " ", "=", " ", "0"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ". By a similar argument, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], "."}], TraditionalForm]]],
 "\n\nTo prove that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " is unique, note that since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], ","}], TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " divides all polynomials in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". So if there is another polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " that divides both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], ","}], TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " would divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "(", "x", ")"}], TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "(", "x", ")"}], TraditionalForm]]],
 " would\nalso divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "(", "x", ")"}], TraditionalForm]]],
 " would have to have the same degree, and \n\n                            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"d", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{"u", "\[CenterDot]", 
      RowBox[{"h", "(", "x", ")"}]}]}], ","}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is a constant polynomial. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " is unique up to multiplication by a constant."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774526784375*^9, 3.4377452850625*^9}, {
  3.437747916078125*^9, 3.43774791621875*^9}}]
}, Open  ]],

Cell[TextData[{
 "DEFINITION 12.4:\nGiven 2 polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", the ",
 StyleBox["greatest common divisor ",
  FontSlant->"Italic"],
 "is the polynomial given in the above theorem whose leading coefficient is \
1.\n\nThe ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command \n\n",
 StyleBox["PolynomialGCD",
  FontWeight->"Bold"],
 "[ poly1, poly2]\n\nfinds the greatest common divisors of two polynomials. \
Thus, the command"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774528584375*^9, 3.43774528584375*^9}, {
  3.437747956484375*^9, 3.437747957984375*^9}}],

Cell[CellGroupData[{

Cell["PolynomialGCD[x^3 - 1, x^4 - 1]", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437745286359375*^9, 3.437745286671875*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "1"}], "+", "x"}]], "Output",
 CellChangeTimes->{3.442755164273875*^9}]
}, Open  ]],

Cell[TextData[{
 "shows that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "-", "1"}], TraditionalForm]]],
 " divides both polynomials. Thus, there are two polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"t", "(", "x", ")"}], TraditionalForm]]],
 " such that\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "3"], "-", " ", "1"}], ")"}], "\[CenterDot]", 
      RowBox[{"s", "(", "x", ")"}]}], " ", "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["x", "4"], "-", " ", "1"}], ")"}], "\[CenterDot]", 
      RowBox[{"t", "(", "x", ")"}]}]}], " ", "=", " ", 
    RowBox[{"x", " ", "-", " ", "1."}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745287171875*^9, 3.437745289640625*^9}}],

Cell[TextData[{
 "EXPERIMENT:\nCan you find polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"t", "(", "x", ")"}], TraditionalForm]]],
 " that solve the above equation?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4377452900625*^9, 3.43774529371875*^9}, {
  3.437745440078125*^9, 3.43774544065625*^9}}],

Cell["     ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "COROLLARY 12.4\nLet ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " be a field, and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " be polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is an irreducible divisor of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
    RowBox[{"h", "(", "x", ")"}]}], TraditionalForm]]],
 ", then either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4377454411875*^9, 3.4377454439375*^9}, {
  3.437747980484375*^9, 3.43774798184375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " divides neither ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"h", "(", "x", ")"}], "."}], TraditionalForm]]],
 " Then the greatest common divisor of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " must have degree less than the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], "."}], TraditionalForm]]],
 " But the ",
 Cell[BoxData[
  FormBox["GCD", TraditionalForm]]],
 " must divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], ","}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is irreducible. Thus, the greatest common divisor of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " must be 1. Likewise the ",
 Cell[BoxData[
  FormBox["GCD", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " must be also be 1. By the greatest common divisor\ntheorem (12.2) , there \
exist polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "(", "x", ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"t", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "(", "x", ")"}], TraditionalForm]]],
 " such that\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"f", "(", "x", ")"}], "\[CenterDot]", 
        RowBox[{"r", "(", "x", ")"}]}], " ", "+", " ", 
       RowBox[{
        RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
        RowBox[{"s", "(", "x", ")"}]}]}], " ", "=", " ", "1"}], ","}]}], 
   TraditionalForm]]],
 "\nand \n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"f", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{"t", "(", "x", ")"}]}], " ", "+", " ", 
     RowBox[{
      RowBox[{"h", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{"u", "(", "x", ")"}]}]}], " ", "=", " ", "1."}], 
   TraditionalForm]]],
 "\n\nBy multiplying these two together, we have\n\n         ",
 Cell[BoxData[{
  RowBox[{
   RowBox[{"1", "=", 
    FormBox[
     RowBox[{
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"f", "(", "x", ")"}], "\[CenterDot]", 
         RowBox[{"r", "(", "x", ")"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
         RowBox[{"s", "(", "x", ")"}]}]}], " ", ")"}], "\[CenterDot]", 
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"f", "(", "x", ")"}], "\[CenterDot]", 
         RowBox[{"t", "(", "x", ")"}]}], " ", "+", " ", 
        RowBox[{
         RowBox[{"h", "(", "x", ")"}], "\[CenterDot]", 
         RowBox[{"u", "(", "x", ")"}]}]}], " ", ")"}]}],
     TraditionalForm]}], " "}], "\n", 
  RowBox[{"\t ", 
   RowBox[{"=", "  ", 
    RowBox[{
     RowBox[{"f", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", "x", ")"}], "2"], "\[CenterDot]", 
       RowBox[{"r", 
        RowBox[{"(", "x", ")"}]}], "\[CenterDot]", "t"}], 
      RowBox[{"(", "x", ")"}]}], " ", "+", " ", 
     RowBox[{"f", 
      RowBox[{
       RowBox[{"(", "x", ")"}], "\[CenterDot]", "r"}], 
      RowBox[{
       RowBox[{"(", "x", ")"}], "\[CenterDot]", "h"}], 
      RowBox[{
       RowBox[{"(", "x", ")"}], "\[CenterDot]", "u"}], 
      RowBox[{"(", "x", ")"}]}], " ", "+", " ", 
     RowBox[{"f", 
      RowBox[{
       RowBox[{"(", "x", ")"}], "\[CenterDot]", "g"}], 
      RowBox[{
       RowBox[{"(", "x", ")"}], "\[CenterDot]", "s"}], 
      RowBox[{
       RowBox[{"(", "x", ")"}], "\[CenterDot]", "t"}], 
      RowBox[{"(", "x", ")"}]}], 
     "                                        \t\t                            \
    ", "+", " ", 
     RowBox[{"g", 
      RowBox[{
       RowBox[{"(", "x", ")"}], "\[CenterDot]", "h"}], 
      RowBox[{
       RowBox[{"(", "x", ")"}], "\[CenterDot]", "s"}], 
      RowBox[{
       RowBox[{"(", "x", ")"}], "\[CenterDot]", "u"}], 
      RowBox[{
       RowBox[{"(", "x", ")"}], "."}]}]}]}]}]}]],
 "\n\nNote that all of the terms on the right hand side are multiples of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " (including the last term, since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
    RowBox[{"h", "(", "x", ")"}]}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ".) But the left hand side is 1, which cannot be a multiple of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ". Thus, we have a contradiction, and so either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], "."}], TraditionalForm]]],
 " "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4377454446875*^9, 3.4377454586875*^9}, {
  3.437748016140625*^9, 3.43774807759375*^9}}]
}, Open  ]],

Cell[TextData[{
 "The irreducible polynomials will play the same role in the domain ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " as prime numbers play in the domain ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". The main property of integer factorizations is that every positive number \
greater than one can be factored ",
 StyleBox["uniquely",
  FontSlant->"Italic"],
 " into a product of primes. We would like to prove something similar \
polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", but we immediately see that an apparent complication:\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"2", " ", 
        SuperscriptBox["x", "3"]}], " ", "-", " ", 
       RowBox[{"2", " ", 
        SuperscriptBox["x", "2"]}], " ", "-", " ", 
       RowBox[{"2", " ", "x"}], " ", "-", " ", "4"}], "  ", "=", "  ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "x"}], " ", "-", " ", "4"}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["x", "2"], " ", "+", " ", "x", " ", "+", " ", "1"}], 
        ")"}]}]}], "                           ", "\n", 
     "\t\t                                                   \t", 
     RowBox[{"=", "  ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "-", " ", "2"}], ")"}], 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox["x", "2"]}], " ", "+", " ", 
          RowBox[{"2", " ", "x"}], " ", "+", " ", "2"}], ")"}], ".", 
        " "}]}]}]}]}], TraditionalForm]]],
 "\n\nYet by the way we have defined irreducible elements, \n",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", "x"}], " ", "-", " ", "4"}], ")"}], 
   TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["x", "2"], " ", "+", " ", "x", " ", "+", " ", "1"}], 
    ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", " ", "-", " ", "2"}], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["x", "2"]}], " ", "+", " ", 
     RowBox[{"2", " ", "x"}], " ", "+", " ", "2"}], ")"}], 
   TraditionalForm]]],
 " are all irreducible polynomials.\n\nIn the next section we will modify our \
definition of unique factorization so that the two factorizations above would \
be considered as equivalent. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4377454593125*^9, 3.437745463578125*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unique Factorization Domains", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "In this section we wish to determine a general definition of unique \
factorization which would apply not only to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", but for ",
 StyleBox["any ",
  FontSlant->"Italic"],
 "integral domain.\n\nDEFINITION 12.5:\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a commutative ring. We say that an element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a ",
 StyleBox["unit",
  FontSlant->"Italic"],
 " if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " has a multiplicative inverse. \n\nIn chapter 9 we defined the set of \
invertible elements of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " as ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["R", "*"], TraditionalForm]]],
 ", and in proposition 9.7 we showed that they formed a group under \
multiplication. The units of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " will play the same role as the constant polynomials do in the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ". In fact, we can model the definition of reducible and irreducible \
elements of a ring on the definition of irreducible polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "[", "x", "]"}], "."}], TraditionalForm]]],
 "\n\nDEFINITION 12.6:\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a commutative ring. If a non-zero element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is not a unit, and can be expressed as a product  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "=", " ", 
     RowBox[{"y", "\[CenterDot]", "z"}]}], ","}], TraditionalForm]]],
 " where neither ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " are units, then we say that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is ",
 StyleBox["reducible",
  FontSlant->"Italic"],
 ". If a non-zero element is neither a unit, nor reducible, we say it is ",
 StyleBox["irreducible",
  FontSlant->"Italic"],
 ".\n\nAlthough this definition is mainly applied to integral domains, we can \
apply the definition to any ring with an identity. Consider the following \
ring:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745464375*^9, 3.437745465390625*^9}, {
  3.437748170328125*^9, 3.43774817146875*^9}, {3.43783301084375*^9, 
  3.437833023671875*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitRing
Define[4a, 0]; Define[2b, 0]
Define[a.a, a]; Define[b.b, b]
Define[a.b, 0]; Define[b.a, 0]
R = Ring[{a,b}]
MultTable[R]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.43783309140625*^9, 3.437833104890625*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.442755177742625*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {
      0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}}, {{
      1, 0.6, 0}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0, 1, 0}, {1, 0.6, 0}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0,
       1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {
      0.7, 0.8, 0.7}}, {{0, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{1, 0, 0}, {1, 1, 0}, {
     1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 
     1}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {
     0, 1, 1}, {1, 1, 0}, {0, 1, 1}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1,
      1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.442755177852*^9}]
}, Open  ]],

Cell["\<\

EXPERIMENT:
From the multiplication table, can you find an irreducible element in this \
ring? How many are there?\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745465890625*^9, 3.4377454704375*^9}}],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Let us consider the more familiar ring, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". The only two elements with multiplicative inverses are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PlusMinus]", "1"}], TraditionalForm]]],
 ". The irreducible elements are of course the prime numbers, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "2", ",", " ", "3", ",", " ", "5", ",", " ", "7", ",", " ", "11", ",", " ",
     "13", ",", " ", 
    RowBox[{"\[Ellipsis]", "."}]}], TraditionalForm]]],
 " But by this definition, the ",
 StyleBox["negative ",
  FontSlant->"Italic"],
 "of a prime number is also irreducible. For example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "3"}], TraditionalForm]]],
 " can be expressed as a product in two different ways: ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"-", "1"}], " ", "\[CenterDot]", " ", "3"}]}], 
   TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", "3"}], " ", "\[CenterDot]", " ", "1"}], TraditionalForm]]],
 ", but both of these products involve units. The dilemma of defining \
negative numbers as primes is that numbers can now be written as a product of \
primes in more than one way:\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{"12", " ", "=", " ", 
    RowBox[{
     RowBox[{"2", "\[CenterDot]", "2", "\[CenterDot]", "3"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"2", "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"-", "2"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"-", "3"}], ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"-", "2"}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{"-", "2"}], ")"}], "\[CenterDot]", "3."}]}]}]}], 
   TraditionalForm]]],
 "\n\nEXPERIMENT:\nBy allowing negative numbers to be prime, how many ways \
can 30 be expressed as a product of prime numbers?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[" ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Because we now are including negative primes, we also have to redefine what \
is meant by unique factorization.  The first step is to understand the \
relationship between these different factorizations.\n\nDEFINITION 12.7:\nLet \
",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a communative ring with identity. We say that the element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is an ",
 StyleBox["associate",
  FontSlant->"Italic"],
 " of an element ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " if there is a unit ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "=", " ", 
    RowBox[{"x", "\[CenterDot]", 
     RowBox[{"z", "."}]}]}], TraditionalForm]]],
 "\n\nNote that if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is an associate of ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "=", " ", 
     RowBox[{"y", "\[CenterDot]", 
      SuperscriptBox["z", 
       RowBox[{"-", "1"}]]}]}], ","}], TraditionalForm]]],
 " so that ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is an associate of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".  Even though we saw 3 different factorizations of 12, these are related \
via associates."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437833124296875*^9, 3.4378331465*^9}, {
   3.43783319115625*^9, 3.437833218609375*^9}, 3.437833295671875*^9, {
   3.4419817877921247`*^9, 3.4419819015577497`*^9}}],

Cell[TextData[{
 "\nDEFINITION 12.8:\nA ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " has ",
 StyleBox["unique factorization",
  FontSlant->"Italic"],
 " if the following two conditions are satisfied:\n\n 1. If ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is non-zero, and is not a unit of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", \n     then ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " can be written as a product of \n     irreducible elements of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ".",
 "\n",
 " 2. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["y", "1"], "\[CenterDot]", 
       SubscriptBox["y", "2"], "\[CenterDot]", 
       SubscriptBox["y", "3"]}], "\[CenterEllipsis]", " ", 
      SubscriptBox["y", "m"]}], " ", "=", "  ", 
     RowBox[{
      RowBox[{
       SubscriptBox["z", "1"], "\[CenterDot]", 
       SubscriptBox["z", "2"], "\[CenterDot]", 
       SubscriptBox["z", "3"]}], " ", "\[CenterEllipsis]", " ", 
      SubscriptBox["z", "n"]}]}]}], TraditionalForm]]],
 "\n     are two expressions of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " as a product of \n     irreducible elements, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "=", " ", "n"}], TraditionalForm]]],
 " and it is \n     possible to reorder ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["z", "1"], ",", 
    SubscriptBox["z", "2"], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     SubscriptBox["z", "n"]}]}], TraditionalForm]]],
 " so that \n     each pair ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["y", "i"], ",", 
     SubscriptBox["z", "i"]}], ")"}], TraditionalForm]]],
 " are associates.\n\nFurthermore, if ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is an integral domain, then ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a ",
 StyleBox["unique factorization domain",
  FontSlant->"Italic"],
 ", abbreviated as ",
 StyleBox["UFD",
  FontSlant->"Italic"],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437833124296875*^9, 3.4378331465*^9}, {
  3.43783319115625*^9, 3.437833218609375*^9}, {3.437833295671875*^9, 
  3.437833298578125*^9}, {3.4419819141514997`*^9, 3.4419819349639997`*^9}}],

Cell[TextData[{
 "This definition sheds light on the polynomials which seemed to have two \
different factorizations. In is sense the factorizations ",
 StyleBox["are",
  FontSlant->"Italic"],
 " unique, as long as we take associates into account. \n\nWe would like to \
find a quick way to determine whether an integral domain is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ". The needed tool will be the definition of the ",
 StyleBox["prime ",
  FontSlant->"Italic"],
 "elements. Although we have already defined a prime element in in integers \
",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ", for a general ring we wish to define a prime element as one which \
satisfies a different property.\n\nDEFINITION 12.9:\nA non-zero element ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " of a commutative ring is ",
 StyleBox["prime",
  FontSlant->"Italic"],
 " if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is not a unit, and whenever ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "\[CenterDot]", "z"}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", then either ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " must be a multiple of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ".\n\nAlthough primes and irreducible elements are the same in ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ", for many other rings they are totally different. Consider once again the \
commutative ring of order 8:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43783315615625*^9, 3.437833172390625*^9}, {
  3.4378332275*^9, 3.437833317375*^9}}],

Cell[CellGroupData[{

Cell["\<\
InitRing
Define[4a, 0]; Define[2b, 0]
Define[a.a, a]; Define[b.b, b]
Define[a.b, 0]; Define[b.a, 0]
R = Ring[{a,b}]
MultTable[R]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437833333765625*^9, 3.43783334396875*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4427551840395*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.7, 0.8, 0.7}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {
      0, 1, 0}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 0.8}}, {{
      1, 0.6, 0}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {0, 1, 0}, {
      1, 0.6, 0}, {0, 1, 0}, {1, 0.6, 0}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {1, 0,
       1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {
      0.7, 0.8, 0.7}}, {{0, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{1, 0, 0}, {1, 1, 0}, {
     1, 0, 0}, {0, 1, 1}, {1, 0, 1}, {1, 1, 0}, {1, 0, 0}, {0, 1, 1}, {1, 0, 
     1}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 1, 0}, {
     0, 1, 1}, {1, 1, 0}, {0, 1, 1}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1,
      1}, {1, 0, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}}, {{1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1,
      1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"3a+b\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"3a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a+b\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 8}, {9, 8}}]}]], "Output",
 CellChangeTimes->{3.442755184148875*^9}]
}, Open  ]],

Cell["\<\
The irreducible element is also a prime element, but there are prime elements \
in this ring which are not irreducible.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
EXPERIMENT:
Find all of the prime elements of this ring. Show that any non-zero element \
is either a unit or can be expressed as a product of primes.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["    ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Although in this ring we found prime elements which were not irreducible, \
we can show that this can only happen when the ring has zero divisors.\n\n\
LEMMA 12.1\nIf ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is an integral domain, and ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is a prime element of ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is irreducible."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43783335071875*^9, 3.43783335084375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is prime, it is neither 0 nor a unit. Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "z"}]}], TraditionalForm]]],
 ", where neither ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " are units. Since ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is prime, we have that either ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". Suppose that ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "=", " ", 
    RowBox[{"x", "\[CenterDot]", "w"}]}], TraditionalForm]]],
 " for some number ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 ". Then\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"x", " ", "=", " ", 
     RowBox[{
      RowBox[{"y", "\[CenterDot]", "z"}], " ", "=", " ", 
      RowBox[{"x", "\[CenterDot]", "w", "\[CenterDot]", 
       RowBox[{"z", "."}]}]}]}]}], TraditionalForm]]],
 "\n\nSince ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is an integral domain, we know that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is not a zero-divisor, so we can use Lemma 9.3 and say that\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "=", " ", 
    RowBox[{"w", " ", "\[CenterDot]", " ", 
     RowBox[{"z", "."}]}]}], TraditionalForm]]],
 "\n\nBut this indicates that ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " is a unit, which contradicts the original assumption that neither ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " were units. Thus, ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is irreducible."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4378333835625*^9, 3.43783338409375*^9}}]
}, Open  ]],

Cell[TextData[{
 "Even though a prime element is irreducible, it is ",
 StyleBox["not",
  FontSlant->"Italic"],
 " true that an irreducible element is prime! Consider for example the \
numbers of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"y", 
     RadicalBox[
      RowBox[{"-", "5"}], " "]}]}], TraditionalForm]]],
 "where ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are integers. This ring is refered to as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "5"}]], "]"}], TraditionalForm]]],
 ". To determine the irreducible elements of this ring, let us define the \
following function on ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox[
      RowBox[{"-", "5"}]], "]"}], "."}], TraditionalForm]]],
 "\n\n    ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", 
     RowBox[{"x", " ", "+", " ", 
      RowBox[{"y", 
       RadicalBox[
        RowBox[{"-", "5"}], " "]}]}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "+", " ", 
        RowBox[{"y", " ", 
         RadicalBox[
          RowBox[{"-", "5"}], " "]}]}], ")"}], 
      RowBox[{"(", 
       RowBox[{"x", "-", 
        RowBox[{"y", 
         RadicalBox[
          RowBox[{"-", "5"}], " "]}]}], ")"}]}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      RowBox[{"5", 
       SuperscriptBox["y", "2"]}]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "\nNotice that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "z", ")"}], TraditionalForm]]],
 " is the product of the number ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " with its complex conjugate. The full signifigence of this function will be \
revealed later in this chapter. For now we can observe that if ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "5"}]], "]"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "[", 
     RowBox[{"a", "\[CenterDot]", "b"}], "]"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"N", "[", "a", "]"}], "\[CenterDot]", 
     RowBox[{"N", "[", "b", "]"}]}]}], TraditionalForm]]],
 ". Hence, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "[", "a", "]"}], TraditionalForm]]],
 " is a group homomorphism from the multiplicative group of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "5"}]], "]"}], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[DoubleStruckCapitalZ]", "+"], TraditionalForm]]],
 ". This homomorphism will help us to determine the irreducible elements of \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "5"}]], "]"}], TraditionalForm]]],
 ".\n\nLet us begin by finding the units of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "5"}]], "]"}], TraditionalForm]]],
 ". If  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", 
    RowBox[{"x", " ", "+", " ", 
     RowBox[{"y", " ", 
      SqrtBox[
       RowBox[{
        RowBox[{"-", "5"}], " "}]]}]}]}], TraditionalForm]]],
 "is invertible, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "[", "a", "]"}], TraditionalForm]]],
 " must be invertible. Hence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "2"], " ", "+", " ", 
     RowBox[{"5", " ", 
      SuperscriptBox["y", "2"]}]}], " ", "=", " ", "1."}], 
   TraditionalForm]]],
 " Then only integer solutions to this equation is when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "=", "0"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", 
    RowBox[{"\[PlusMinus]", "1."}]}], TraditionalForm]]],
 " Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PlusMinus]", "1"}], TraditionalForm]]],
 " are the two units of this ring.\n                                          \
           \nNext let us find an irreducible element. Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "[", "2", "]"}], " ", "=", " ", "4"}], TraditionalForm]]],
 ", the only way a product of non-units ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " could be equal 4 is if both ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "[", "a", "]"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "[", "b", "]"}], TraditionalForm]]],
 " were equal to 2. But the equation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "2"], " ", "+", " ", 
     RowBox[{"5", " ", 
      SuperscriptBox["y", "2"]}]}], " ", "=", " ", "2"}], TraditionalForm]]],
 " clearly has no integer solutions. Thus, 2 is an irreducible element in \
this ring. By the same reasoning, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "[", "3", "]"}], " ", "=", " ", "9"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["x", "2"], " ", "+", " ", 
     RowBox[{"5", " ", 
      SuperscriptBox["y", "2"]}]}], " ", "=", " ", "3"}], TraditionalForm]]],
 " has no solutions, so 3 is irreducible.\n\nHowever, neither 2 nor 3 are \
prime elements of this ring! Consider the product\n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", " ", "+", " ", 
       RadicalBox[
        RowBox[{"-", "5"}], " "]}], ")"}], 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RadicalBox[
        RowBox[{"-", "5"}], " "]}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"1", " ", "+", " ", "5"}], " ", "=", " ", "6"}]}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 PageBreakAbove->True],

Cell[TextData[{
 "This product is a multiple of 2 and 3, but neither factor is a multiple of \
2 or 3. Thus, 2 and 3 are not prime in this ring.\n\nThis example shows a \
ring which is ",
 StyleBox["not",
  FontSlant->"Italic"],
 " a unique factorization domain. We have seen two ways of factoring the \
number 6, and the two factorizations are not equivalent in terms of \
associates. But the fact that neither 2 nor 3 are ",
 StyleBox["prime",
  FontSlant->"Italic"],
 " is a clue as to why this ring is not a UFD."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "PROPOSITION 12.5\nAn integral domain is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 " if, and only if, all non-zero, non-units can be written as a product of \
primes."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437833414265625*^9, 3.437833414921875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We begin be showing that if ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ", then all irreducible elements are prime. Suppose ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 " is irreducible, and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "w"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "z"}]}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " have factorizations into irreducible elements:\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{
     SubscriptBox["x", "1"], "\[CenterDot]", " ", 
     SubscriptBox["x", "2"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
     " ", 
     SubscriptBox["x", "n"]}]}], TraditionalForm]]],
 "\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"y", " ", "=", " ", 
     RowBox[{
      SubscriptBox["y", "1"], "\[CenterDot]", " ", 
      SubscriptBox["y", "2"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
      " ", 
      SubscriptBox["y", "m"]}]}]}], TraditionalForm]]],
 "\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"z", " ", "=", " ", 
     RowBox[{
      SubscriptBox["z", "1"], "\[CenterDot]", " ", 
      SubscriptBox["z", "2"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
      " ", 
      RowBox[{
       SubscriptBox["z", "k"], "."}]}]}]}], TraditionalForm]]],
 "\n\nThus,      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["x", "1"], "\[CenterDot]", " ", 
     SubscriptBox["x", "2"], " ", "\[CenterDot]", "\[CenterDot]", 
     "\[CenterDot]", " ", 
     SubscriptBox["x", "n"], " ", "\[CenterDot]", " ", "w"}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["y", "1"], "\[CenterDot]", " ", 
     SubscriptBox["y", "2"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
     " ", 
     SubscriptBox["y", "m"], "\[CenterDot]", " ", 
     SubscriptBox["z", "1"], "\[CenterDot]", " ", 
     SubscriptBox["z", "2"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
     " ", 
     RowBox[{
      SubscriptBox["z", "k"], "."}]}]}], TraditionalForm]]],
 "\n\nSince a factorization is unique, and all terms in this product are \
irreducible, we have that ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 " is an associate to one of the terms on the right hand side. Thus, either \
",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 ", and hence ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 " is prime.\n\nSince a non-zero element which is not a unit in a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 " can be expressed as a product of irreducible elements, we have shown that \
all such elements can be expressed as a product of primes.\n\nNow let us \
suppose that all non-zero, non-unit elements in an integral domain can be \
expressed as a product of primes. The first part of the definition of a UFD \
is obviously fulfilled since the prime elements are irreducible. Suppose  we \
have another factorization in terms of irreducible elements.\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["p", "1"], "\[CenterDot]", " ", 
     SubscriptBox["p", "2"], "\[CenterDot]", " ", 
     SubscriptBox["p", "3"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
     " ", 
     SubscriptBox["p", "n"]}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["z", "1"], "\[CenterDot]", " ", 
     SubscriptBox["z", "2"], "\[CenterDot]", " ", 
     SubscriptBox["z", "3"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
     " ", 
     RowBox[{
      SubscriptBox["z", "m"], "."}]}]}], TraditionalForm]]],
 "\n\nHere, the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "i"], TraditionalForm]]],
 "  are prime elements, while the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["z", "j"], TraditionalForm]]],
 " are merely irreducible elements. We need to prove that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "m"}], TraditionalForm]]],
 ", and that, after a rearrangement of the ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 "'s, we have that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "i"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["z", "i"], TraditionalForm]]],
 "  are associates. We will proceed by induction on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", the number of primes in the factorization. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "1"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "=", " ", "1"}], TraditionalForm]]],
 ", otherwise we would have a prime number (which is irreducible) expressed \
as a product of two or more irreducible elements. Also, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "1"], TraditionalForm]]],
 " = ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["z", "1"], TraditionalForm]]],
 ", and so trivially the ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 "'s are associates of the ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 "'s.\n\nNext, we will consider the general case. Since the right hand side \
of\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["p", "1"], "\[CenterDot]", " ", 
     SubscriptBox["p", "2"], "\[CenterDot]", " ", 
     SubscriptBox["p", "3"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
     " ", 
     SubscriptBox["p", "n"]}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["z", "1"], "\[CenterDot]", " ", 
     SubscriptBox["z", "2"], "\[CenterDot]", " ", 
     SubscriptBox["z", "3"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
     " ", 
     RowBox[{
      SubscriptBox["z", "m"], "."}]}]}], TraditionalForm]]],
 "\n\nis a multiple of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "n"], TraditionalForm]]],
 " , one of the ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 "'s must be a multiple of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "n"], TraditionalForm]]],
 ". Suppose that\n\n                            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["z", "k"], "=", " ", 
    RowBox[{
     SubscriptBox["p", "n"], "\[CenterDot]", " ", "u"}]}], 
   TraditionalForm]]],
 ".\n                            \nSince ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["z", "k"], TraditionalForm]]],
 " is irreducible, we find that ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " is a unit, hence ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["z", "k"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "n"], TraditionalForm]]],
 " are associates. We now can write\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["p", "1"], "\[CenterDot]", " ", 
      SubscriptBox["p", "2"], "\[CenterDot]", " ", 
      SubscriptBox["p", "3"]}], "\[CenterEllipsis]", " ", 
     RowBox[{
      SubscriptBox["p", 
       RowBox[{"n", "-", "1"}]], "\[CenterDot]", " ", 
      SubscriptBox["p", "n"]}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["z", "1"], "\[CenterDot]", " ", 
      SubscriptBox["z", "2"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
      " ", 
      SubscriptBox["z", 
       RowBox[{"k", "-", "1"}]], "\[CenterDot]", " ", 
      SubscriptBox["p", "n"], "\[CenterDot]", " ", "u", " ", "\[CenterDot]", 
      " ", 
      SubscriptBox[
       RowBox[{"z", " "}], 
       RowBox[{"k", "+", "1", " "}]]}], "\[CenterEllipsis]", " ", 
     RowBox[{
      SubscriptBox["z", "m"], "."}]}]}], TraditionalForm]]],
 "\n\nSince the ring is an integral domain, we can use lemma 9.3 and cancel \
out the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "n"], TraditionalForm]]],
 ".\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["p", "1"], "\[CenterDot]", " ", 
      SubscriptBox["p", "2"], "\[CenterDot]", " ", 
      SubscriptBox["p", "3"]}], "\[CenterEllipsis]", " ", 
     SubscriptBox["p", 
      RowBox[{"n", "-", "1"}]]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["z", "1"], "\[CenterDot]", " ", 
      SubscriptBox["z", "2"]}], 
     RowBox[{
      SubscriptBox["\[CenterEllipsis]z", 
       RowBox[{"k", "-", "1"}]], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"u", "\[CenterDot]", " ", 
        SubscriptBox["z", 
         RowBox[{"k", "+", "1"}]]}], ")"}]}], "\[CenterEllipsis]", " ", 
     RowBox[{
      SubscriptBox["z", "m"], "."}]}]}], TraditionalForm]]],
 "\n\nThe unit ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " may be multiplied by any of the irreducible elements ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " to produce another irreducible element. We now can apply the induction \
hypothesis, which would says that there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 "'s left, and that a rearrangement of the ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 "'s would make ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "i"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["z", "i"], TraditionalForm]]],
 "  associates. Therefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", " ", "=", " ", "n"}], TraditionalForm]]],
 ", and some rearrangement of the ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 "'s in \n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["p", "1"], "\[CenterDot]", " ", 
      SubscriptBox["p", "2"], "\[CenterDot]", " ", 
      SubscriptBox["p", "3"]}], "\[CenterEllipsis]", " ", 
     SubscriptBox["p", "n"]}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["z", "1"], "\[CenterDot]", " ", 
     SubscriptBox["z", "2"], "\[CenterDot]", " ", 
     SubscriptBox["z", "3"], "\[CenterDot]", "\[CenterDot]", "\[CenterDot]", 
     " ", 
     SubscriptBox["z", "m"]}]}], TraditionalForm]]],
 "\n\nwill allow ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["p", "i"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["z", "i"], TraditionalForm]]],
 " to be associates, proving that the ring is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43783347853125*^9, 3.437833480734375*^9}, {
  3.437833589984375*^9, 3.437833597203125*^9}}]
}, Open  ]],

Cell[TextData[{
 "This proposition will help us greatly in determining whether an integral \
domain is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ". We usually will proceed in two steps: proving that any element can be \
written as a product of irreducible elements, and then proving that any \
irreducible element is prime.\n\nCOROLLARY 12.5\nIf ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " is a field, then the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"UFD", "."}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774547715625*^9, 3.43774547715625*^9}, {
  3.437833633859375*^9, 3.437833634421875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "From proposition 12.4, every polynomial of positive degree is either \
irreducible, or can be expressed as a product of irreducible polynomials. By \
corollary 12.4, all irreducible polynomials are prime. Thus, by proposition \
12.5, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"UFD", "."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745477796875*^9, 3.437745477796875*^9}, {
  3.43783399015625*^9, 3.437834004234375*^9}}]
}, Open  ]],

Cell[TextData[{
 "Although this corollary proves that polynomials over the rational numbers \
have a unique factorization, we still have not proven that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", "x", "]"}], TraditionalForm]]],
 ", the polynomials over the integers, is a unique factorization domain. \
Corollary 12.5 will not help us, since ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is not a field. Yet is seems plausible that we could prove that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", "x", "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ", merely by using the fact that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", "Z", "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ". In the process, let us prove that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 " whenever ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ". First, we will need to prove a few lemmas. This next lemma, commonly \
refered to as Gauss' lemma, uses the formula for the product of two \
polynomials."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745478203125*^9, 3.437745479140625*^9}, {
  3.437834025953125*^9, 3.437834026515625*^9}}],

Cell[TextData[{
 "LEMMA 12.2: Gauss' Lemma\nIf ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is an integral domain, then a prime element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is also a prime element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], "."}], TraditionalForm]]],
 " "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745479921875*^9, 3.437745479921875*^9}, {
  3.437834047390625*^9, 3.437834061859375*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We need to show that if ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"h", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"f", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{"g", "(", "x", ")"}]}]}], ","}], TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " must divide either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 ". Suppose that  ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " does not divide all of the coefficients of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], ","}], TraditionalForm]]],
 " nor does ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " divide all of the\ncoefficients of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], "."}], TraditionalForm]]],
 " Let\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["a", "0"], " ", "+", " ", 
     RowBox[{
      SubscriptBox["a", "1"], "x"}], " ", "+", " ", 
     RowBox[{
      SubscriptBox["a", "2"], " ", 
      SuperscriptBox["x", "2"]}], " ", "+", " ", 
     RowBox[{
      SubscriptBox["a", "3"], " ", 
      SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]"}]}], 
   TraditionalForm]]],
 ",\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"g", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      SubscriptBox["b", "0"], " ", "+", " ", 
      RowBox[{
       SubscriptBox["b", "1"], " ", "x"}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["b", "2"], " ", 
       SuperscriptBox["x", "2"]}], " ", "+", " ", 
      RowBox[{
       SubscriptBox["b", "3"], " ", 
       SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]"}]}]}], 
   TraditionalForm]]],
 ",\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{"  ", 
    RowBox[{
     RowBox[{"h", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{"g", "(", "x", ")"}]}], " ", "=", " ", 
      RowBox[{
       SubscriptBox["c", "0"], " ", "+", " ", 
       RowBox[{
        SubscriptBox["c", "1"], "x"}], " ", "+", " ", 
       RowBox[{
        SubscriptBox["c", "2"], " ", 
        SuperscriptBox["x", "2"]}], " ", "+", " ", 
       RowBox[{
        SubscriptBox["c", "3"], 
        SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]"}]}]}]}], 
   TraditionalForm]]],
 ".\n\nLet ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "i"], TraditionalForm]]],
 "  be the first coefficient of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " which is not divisible by ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["b", "j"], TraditionalForm]]],
 "  be the first coefficient of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " which is not divisible by ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " is divisible by ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", we know that the coefficient ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["c", 
    RowBox[{"i", "+", "j"}]], TraditionalForm]]],
 "  must be divisible by ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". But\n\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["c", 
     RowBox[{"i", "+", "j"}]], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", "0"], " ", 
      SubscriptBox["b", 
       RowBox[{"i", "+", "j"}]]}], "  ", "+", " ", 
     RowBox[{
      SubscriptBox["a", "1"], " ", 
      SubscriptBox["b", 
       RowBox[{"i", "+", "j", "-", "1"}]]}], " ", "+", " ", "\[Ellipsis]", 
     " ", "+", " ", 
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"i", "-", "1"}]], " ", 
      SubscriptBox["b", 
       RowBox[{"j", "+", "1"}]]}], " ", "+", " ", 
     RowBox[{
      SubscriptBox["a", "i"], "  ", 
      SubscriptBox["b", "j"]}], " ", "+", " ", 
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"i", "+", "1"}]], " ", 
      SubscriptBox["b", 
       RowBox[{"j", "-", "1"}]]}], " ", "+", " ", "\[Ellipsis]", " ", "+", 
     " ", 
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"i", "+", "j"}]], " ", 
      RowBox[{
       SubscriptBox["b", "0"], "."}]}]}]}], TraditionalForm]]],
 "\n\nNote that all terms on the right hand side except ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", 
     RowBox[{"i", " "}]], 
    SubscriptBox["b", "j"]}], TraditionalForm]]],
 "  are divisible by ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". (Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "0"], ",", " ", 
    SubscriptBox["a", "1"], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     SubscriptBox["a", 
      RowBox[{"i", "-", "1"}]]}]}], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["b", "0"], ",", 
    SubscriptBox["b", "1"], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     SubscriptBox["b", 
      RowBox[{"j", "-", "1"}]]}]}], TraditionalForm]]],
 " are all multiples of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ".) So ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", 
     RowBox[{"i", " "}]], " ", 
    SubscriptBox["b", "j"]}], TraditionalForm]]],
 "  is also a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". But this contradicts the fact that ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", and neither ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "i"], TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["b", "j"], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is prime in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], "."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4377454805*^9, 3.437745488875*^9}, {3.43783409140625*^9,
   3.437834092015625*^9}, {3.4378341338125*^9, 3.4378341595625*^9}, {
  3.4378342826875*^9, 3.43783428634375*^9}}]
}, Open  ]],

Cell[TextData[{
 "With Gauss' lemma (12.2), we can see that whenever a product of several \
polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " is divisible by a ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", a prime number of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", then one of those polynomials must have been divisible by ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". We can use induction to extend this argument to when ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " any element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". \n\nLEMMA 12.3\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a unique factorization domain, and let\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["g", "1"], "(", "x", ")"}], ",", " ", 
    RowBox[{
     SubscriptBox["g", "2"], "(", "x", ")"}], ",", " ", 
    RowBox[{
     SubscriptBox["g", "3"], "(", "x", ")"}], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     RowBox[{
      SubscriptBox["g", "n"], "(", "x", ")"}], " "}]}], TraditionalForm]]],
 "\n\nbe polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " which are not divisible by any prime element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " be a polynomial in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ", and let ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " be two elements in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " such that\n\n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"c", "\[CenterDot]", 
     RowBox[{"f", "(", "x", ")"}]}], " ", "=", " ", 
    RowBox[{"d", "\[CenterDot]", 
     RowBox[{
      SubscriptBox["g", "1"], "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{
      SubscriptBox["g", "2"], "(", "x", ")"}], "\[CenterDot]", 
     RowBox[{
      SubscriptBox["g", "3"], "(", "x", ")"}], "\[CenterDot]", " ", 
     "\[Ellipsis]", " ", "\[CenterDot]", 
     RowBox[{
      RowBox[{
       SubscriptBox["g", "n"], "(", "x", ")"}], "."}]}]}], 
   TraditionalForm]]],
 "\n\nThen ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " is divisible by ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774548940625*^9, 3.437745493171875*^9}, {
  3.437834338109375*^9, 3.43783434675*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " is a unit in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", then obviously ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ". We will now use induction on the number of prime factors of ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " in the ring ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " contains a prime ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", then by lemma 12.2, one of the terms on the right hand side must be a \
multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". But none of the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "i"], "(", "x", ")"}], TraditionalForm]]],
 " are divisible by a prime, so we find that ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Then we have\n  \n            ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      FractionBox["c", "p"], "\[CenterDot]", " ", 
      RowBox[{"f", "(", "x", ")"}]}], " ", "=", " ", 
     RowBox[{
      FractionBox["d", "p"], " ", "\[CenterDot]", 
      RowBox[{
       SubscriptBox["g", "1"], "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{
       SubscriptBox["g", "2"], "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{
       SubscriptBox["g", "3"], "(", "x", ")"}], "\[CenterDot]", 
      "\[CenterEllipsis]", "\[CenterDot]", 
      RowBox[{
       SubscriptBox["g", "n"], "(", "x", ")"}]}]}], ","}], 
   TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "/", "p"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "/", "p"}], TraditionalForm]]],
 " are both in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "/", "p"}], TraditionalForm]]],
 " contains one less prime factor than ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", we can use induction to say that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "/", "p"}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "/", "p"}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " would be divisible by ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4377454938125*^9, 3.43774549675*^9}, {
  3.437834379578125*^9, 3.437834379671875*^9}}]
}, Open  ]],

Cell[TextData[{
 "The next step in proving that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 " is to find the irreducible elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ". If there is a field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " which contains ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", we can use the irreducible elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " to find the irreducible elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ".\n\nLEMMA 12.4\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a unique factorization domain, and let ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " be a field containing ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Then if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is a polynomial in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " which is irreducible in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "[", "x", "]"}], ","}], TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " can be writen \n\n                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"c", "\[CenterDot]", 
     RowBox[{"g", "(", "x", ")"}]}]}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " is an element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is irreducible in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745497640625*^9, 3.43774550190625*^9}, {
  3.437834707734375*^9, 3.437834708375*^9}, {3.4419820284171247`*^9, 
  3.4419820346827497`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We want to first show that we can express \n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{"c", "\[CenterDot]", 
      RowBox[{"g", "(", "x", ")"}]}]}], ","}], TraditionalForm]]],
 "\n\nwhere the only elements of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " are units. Let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "0"], TraditionalForm]]],
 " be the constant coefficient of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], "."}], TraditionalForm]]],
 " Notice that if an element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], ","}], TraditionalForm]]],
 " then that element must divide ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "0"], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"UFD", ","}], TraditionalForm]]],
 " there are only a finite \nnumber of primes in the factorization of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "0"], TraditionalForm]]],
 ". Let us proceed by induction on the number of primes in this \
factorization.\n\nIf there are no prime elements of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], ","}], TraditionalForm]]],
 " we can let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", " ", "=", " ", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"f", "(", "x", ")"}]}], TraditionalForm]]],
 ". If there ",
 StyleBox["is",
  FontSlant->"Italic"],
 " a prime element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which divides ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], ","}], TraditionalForm]]],
 " we can write\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{"p", "\[CenterDot]", 
      RowBox[{"h", "(", "x", ")"}]}]}], ","}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], "."}], TraditionalForm]]],
 " But then the constant coefficient of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "(", "x", ")"}], TraditionalForm]]],
 " will contain one less prime in its prime factorization, so by induction we \
have\n\n                       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"h", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{"d", "\[CenterDot]", 
      RowBox[{"g", "(", "x", ")"}]}]}], ","}], TraditionalForm]]],
 "\n\nwhere the only elements of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " are units. Then we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"c", " ", "=", " ", 
     RowBox[{"b", "\[CenterDot]", "d"}]}], ","}], TraditionalForm]]],
 " and\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"c", "\[CenterDot]", 
     RowBox[{
      RowBox[{"g", "(", "x", ")"}], "."}]}]}], TraditionalForm]]],
 "\n\nAll that is left to show is that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is irreducible in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], "."}], TraditionalForm]]],
 " Suppose that\n \n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"g", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"r", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{"s", "(", "x", ")"}]}]}]}], TraditionalForm]]],
 "                      \n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "(", "x", ")"}], TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ". We then have \n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{"c", "\[CenterDot]", 
      RowBox[{"r", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{"s", "(", "x", ")"}]}]}], ","}], TraditionalForm]]],
 "\n\nBut there is a field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " containing ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is irreducible in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "[", "x", "]"}], "."}], TraditionalForm]]],
 " Thus, either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "(", "x", ")"}], TraditionalForm]]],
 " are units in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", which are constant polynomials. But we designed ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " so that the only constants in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " which divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " are units of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is irreducible in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], "."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774550265625*^9, 3.43774551703125*^9}, {
  3.43783479284375*^9, 3.437834793921875*^9}}]
}, Open  ]],

Cell[TextData[{
 "Although this lemma refers to some field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " which contains ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", there is a natural field to use--the field of quotients in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". We can use this field to show that, in fact, the irreducible elements of \
",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which we found in lemma 12.4 are in fact prime elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ".\n\nLEMMA 12.5\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a unique factorization domain, and let ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " be the field of quotients for ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Then if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is irreducible over ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is prime in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], "."}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745517453125*^9, 3.437745520359375*^9}, {
  3.43783490115625*^9, 3.4378349063125*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"r", "(", "x", ")"}], "\[CenterDot]", 
    RowBox[{"s", "(", "x", ")"}]}], TraditionalForm]]],
 " is divisible by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], "."}], TraditionalForm]]],
 " We need to show that either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "(", "x", ")"}], TraditionalForm]]],
 " is divisible by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], "."}], TraditionalForm]]],
 " Yet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is irreducible in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", which is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 " since ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " is a field. Thus, either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"s", "(", "x", ")"}], TraditionalForm]]],
 " ",
 StyleBox["is",
  FontSlant->"Italic"],
 "\ndivisible by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ". Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " is divisible. Then we have\n\n                ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"r", "(", "x", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{"k", "(", "x", ")"}]}]}], ","}]}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "(", "x", ")"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ". The coefficients of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "(", "x", ")"}], TraditionalForm]]],
 " are in the quotient field of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", so we may write\n\n              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"k", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{
     StyleBox[
      FractionBox[
       SubscriptBox["a", "0"], 
       SubscriptBox["b", "0"]],
      FontSize->16], " ", "+", " ", 
     RowBox[{
      StyleBox[
       FractionBox[
        SubscriptBox["a", "1"], 
        SubscriptBox["b", "1"]],
       FontSize->16], "x"}], " ", "+", " ", 
     RowBox[{
      StyleBox[
       FractionBox[
        SubscriptBox["a", "2"], 
        SubscriptBox["b", "2"]],
       FontSize->16], " ", 
      SuperscriptBox["x", "2"]}], " ", "+", " ", 
     RowBox[{
      StyleBox[
       FractionBox[
        SubscriptBox["a", "3"], 
        SubscriptBox["b", "3"]],
       FontSize->16], " ", 
      SuperscriptBox["x", "3"]}], " ", "+", " ", "\[CenterEllipsis]", " ", 
     "+", " ", 
     RowBox[{
      StyleBox[
       FractionBox[
        SubscriptBox["a", "n"], 
        SubscriptBox["b", "n"]],
       FontSize->16], " ", 
      RowBox[{
       SuperscriptBox["x", "n"], "."}]}]}]}], TraditionalForm]]],
 "\n              \nLet ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " be the product of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["b", "0"], "\[CenterDot]", 
    SubscriptBox["b", "1"], "\[CenterDot]", 
    SubscriptBox["b", "2"], "\[CenterDot]", 
    SubscriptBox["b", "3"], "\[CenterDot]", "\[Ellipsis]", "\[CenterDot]", 
    RowBox[{
     SubscriptBox["b", "n"], "."}]}], TraditionalForm]]],
 " Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"j", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"c", "\[CenterDot]", 
     RowBox[{"k", "(", "x", ")"}]}]}], TraditionalForm]]],
 " is an polynomial in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], "."}], TraditionalForm]]],
 " Thus we have\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"c", "\[CenterDot]", 
     RowBox[{"r", "(", "x", ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"c", "\[CenterDot]", 
        RowBox[{"k", "(", "x", ")"}]}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{
       RowBox[{"j", "(", "x", ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"j", "(", "x", ")"}]}], TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ". We now can apply lemma 12.4 to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"j", "(", "x", ")"}], TraditionalForm]]],
 ", and write\n\n                           ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"j", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{"d", "\[CenterDot]", 
      RowBox[{
       RowBox[{"q", "(", "x", ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " is not divisible by any prime in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Then \n\n                          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"c", "\[CenterDot]", 
      RowBox[{"r", "(", "x", ")"}]}], " ", "=", " ", 
     RowBox[{"d", "\[CenterDot]", 
      RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{"q", "(", "x", ")"}]}]}], ","}], TraditionalForm]]],
 "\n\nso we can apply lemma 12.3, since neither ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "(", "x", ")"}], TraditionalForm]]],
 " is divisible by a prime of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Hence, ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " is divisible by ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", and\n\n                 ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"r", "(", "x", ")"}], " ", "=", 
     StyleBox[" ",
      FontSize->16], 
     RowBox[{
      StyleBox[
       FractionBox["d", "c"],
       FontSize->16], " ", "\[CenterDot]", 
      RowBox[{"g", "(", "x", ")"}], "\[CenterDot]", 
      RowBox[{
       RowBox[{"q", "(", "x", ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nTherefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " is divisible by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 ", and hence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is prime in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], "."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745520953125*^9, 3.437745538109375*^9}, {
  3.437835013140625*^9, 3.437835053609375*^9}, {3.437835108375*^9, 
  3.437835110234375*^9}}]
}, Open  ]],

Cell[TextData[{
 "At this point all of the major battles have been fought. All that is left \
to do is put the pieces together to show that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"UFD", "."}], TraditionalForm]]],
 "\n\nTHEOREM 12.3:  THE UNIQUE FACTORIZATION DOMAIN THEOREM\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " is a unique factorization domain if, and only if, ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a unique factorization domain."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745538625*^9, 3.437745539109375*^9}, {
  3.43783512290625*^9, 3.43783512371875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "First of all, if ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is not a UFD, then there is some element ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which is not expressible as a product of primes. But then ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " cannot be expressed as a product of primes in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], ","}], TraditionalForm]]],
 " since such a product must consist of constant polynomials, and this would \
contradict the fact that ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " cannot be expressed as a product of primes in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " would not be a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"UFD", "."}], TraditionalForm]]],
 "\n\nNow suppose that ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ". We need to show that any non-zero polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " is either a unit, or is expressible as a product of prime polynomials. If \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " has degree 0, and is not a unit of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", then since ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ", the constant ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " can be expressed as a product of primes in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". By Lemma 12.2, any prime in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is also a prime in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ". Thus, if the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is zero, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " is either a unit, or can be expressed as a product of primes in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ".\n\nNow suppose ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " has positive degree. Let ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " be the field of quotients over ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is a unique factorization domain by corollary 12.5. Thus, we can write\n\n \
          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["g", "1"], "(", "x", ")"}], "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["g", "2"], "(", "x", ")"}], "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["g", "3"], "(", "x", ")"}], "\[CenterDot]", " ", 
      "\[Ellipsis]", " ", "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["g", "n"], "(", "x", ")"}]}]}], ","}], 
   TraditionalForm]]],
 "\n\nwhere each ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "i"], "(", "x", ")"}], TraditionalForm]]],
 " is irreducible in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ". For each ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "i"], "(", "x", ")"}], TraditionalForm]]],
 ", let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["c", "i"], TraditionalForm]]],
 " be the product of the denominators of all of the coefficients. Then ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["h", "i"], TraditionalForm]]],
 "(x) = ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["c", "i"], TraditionalForm]]],
 "\[CenterDot] ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "i"], "(", "x", ")"}], TraditionalForm]]],
 " will be in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ", and we have\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{" ", 
      RowBox[{
       RowBox[{
        RowBox[{
         SubscriptBox["c", "1"], "\[CenterDot]", 
         SubscriptBox["c", "2"], "\[CenterDot]", 
         SubscriptBox["c", "3"], "\[CenterDot]", " ", "\[Ellipsis]", " ", 
         "\[CenterDot]", " ", 
         SubscriptBox["c", "n"], "\[CenterDot]", " ", 
         RowBox[{"f", "(", "x", ")"}]}], " ", "=", " ", 
        RowBox[{
         SubscriptBox["c", "1"], "\[CenterDot]", " ", 
         RowBox[{
          SubscriptBox["g", "1"], "(", "x", ")"}], "\[CenterDot]", " ", 
         SubscriptBox["c", "2"], "\[CenterDot]", " ", 
         RowBox[{
          SubscriptBox["g", "2"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
         SubscriptBox["c", "3"], "\[CenterDot]", " ", 
         RowBox[{
          SubscriptBox["g", "3"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
         "\[Ellipsis]", " ", "\[CenterDot]", " ", 
         SubscriptBox["c", "n"], "\[CenterDot]", " ", 
         RowBox[{
          SubscriptBox["g", "n"], "(", "x", ")"}]}]}], "\n", 
       "\t                     "}]}],
     TraditionalForm], "                           ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["h", "1"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
     RowBox[{
      SubscriptBox["h", "2"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
     RowBox[{
      SubscriptBox["h", "3"], "(", "x", ")"}], "\[CenterDot]", " ", 
     "\[Ellipsis]", " ", "\[CenterDot]", 
     RowBox[{
      RowBox[{
       SubscriptBox["h", "n"], "(", "x", ")"}], "."}]}]}], 
   TraditionalForm]]],
 "\n\nSince ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["c", "i"], TraditionalForm]]],
 "  is a unit in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["h", "i"],
     TraditionalForm], "(", "x", ")"}], TraditionalForm]]],
 " will still all be irreducible in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "[", "x", "]"}], "."}], TraditionalForm]]],
 " We can now apply lemma 12.4 on each of the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["h", "i"], "(", "x", ")"}], TraditionalForm]]],
 " and find an element ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["d", "i"], TraditionalForm]]],
 "  in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", " "}], TraditionalForm]]],
 " such that \n\n                                ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["h", "i"], "(", "x", ")"}], " ", "=", " ", 
      RowBox[{
       SubscriptBox["d", "i"], "\[CenterDot]", " ", 
       RowBox[{
        SubscriptBox["j", "i"], "(", "x", ")"}]}]}], ","}]}], 
   TraditionalForm]]],
 "\n\nwhere the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["j", "i"], "(", "x", ")"}], TraditionalForm]]],
 " are irreducible in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ". By lemma 12.5, the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["j", "i"], "(", "x", ")"}], TraditionalForm]]],
 " are prime in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ". We now can express\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SubscriptBox["c", "1"], "\[CenterDot]", " ", 
      SubscriptBox["c", "2"], "\[CenterDot]", " ", 
      SubscriptBox["c", "3"], "\[CenterDot]", " ", "\[Ellipsis]", " ", 
      "\[CenterDot]", " ", 
      SubscriptBox["c", "n"], "\[CenterDot]", " ", 
      RowBox[{"f", "(", "x", ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["d", "1"], "\[CenterDot]", " ", 
       RowBox[{
        SubscriptBox["j", "1"], "(", "x", ")"}], "\[CenterDot]", " ", 
       SubscriptBox["d", "2"], "\[CenterDot]", " ", 
       RowBox[{
        SubscriptBox["j", "2"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
       SubscriptBox["d", "3"], "\[CenterDot]", " ", 
       RowBox[{
        SubscriptBox["j", "3"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
       "\[Ellipsis]", " ", "\[CenterDot]", " ", 
       SubscriptBox["d", "n"], "\[CenterDot]", " ", "j"}], " ", 
      RowBox[{
       RowBox[{"(", "x", ")"}], "."}]}]}]}], TraditionalForm]]],
 "\n\nLet  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", " ", "=", " ", 
    RowBox[{
     SubscriptBox["c", "1"], "\[CenterDot]", " ", 
     SubscriptBox["c", "2"], "\[CenterDot]", " ", 
     SubscriptBox["c", "3"], "\[CenterDot]", " ", "\[Ellipsis]", " ", 
     "\[CenterDot]", " ", 
     SubscriptBox["c", "n"]}]}], TraditionalForm]]],
 "  and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"D", " ", "=", " ", 
    RowBox[{
     SubscriptBox["d", "1"], "\[CenterDot]", " ", 
     SubscriptBox["d", "2"], "\[CenterDot]", " ", 
     SubscriptBox["d", "3"], "\[CenterDot]", " ", "\[Ellipsis]", " ", 
     "\[CenterDot]", " ", 
     RowBox[{
      SubscriptBox["d", "n"], "."}]}]}], TraditionalForm]]],
 " We can then write\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"C", " ", "\[CenterDot]", " ", 
      RowBox[{"f", "(", "x", ")"}]}], " ", "=", " ", 
     RowBox[{"D", " ", "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["j", "1"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["j", "2"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["j", "3"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      "\[Ellipsis]", " ", "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["j", "n"], "(", "x", ")"}]}]}], ","}], 
   TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox["C", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["D", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", and the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["j", "i"], "(", "x", ")"}], TraditionalForm]]],
 " are prime polynomials in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ". We can now apply lemma 12.3, which states that ",
 Cell[BoxData[
  FormBox["D", TraditionalForm]]],
 " must be a multiple of ",
 Cell[BoxData[
  FormBox["C", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Thus\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      FormBox[
       StyleBox[
        FractionBox["D", "C"],
        FontSize->16],
       TraditionalForm], " ", "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["j", "1"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["j", "2"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["j", "3"], "(", "x", ")"}], " ", "\[CenterDot]", " ", 
      "\[Ellipsis]", " ", "\[CenterDot]", " ", 
      RowBox[{
       SubscriptBox["j", "n"], "(", "x", ")"}]}]}], ","}], 
   TraditionalForm]]],
 "\n\nwhere ",
 Cell[BoxData[
  FormBox[
   RowBox[{"D", "/", "C"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"D", "/", "C"}], TraditionalForm]]],
 " can be expressed as a product of  primes in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", which by lemma 12.2 are primes in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " can be expressed as a product of primes in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", "[", "x", "]"}], ","}], TraditionalForm]]],
 " and so by proposition 12.5, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"UFD", "."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745539609375*^9, 3.43774556034375*^9}, {
  3.437835163421875*^9, 3.437835178765625*^9}, {3.437835211671875*^9, 
  3.437835262625*^9}}]
}, Open  ]],

Cell[TextData[{
 "Not only does this theorem determine when we can consider polynomial \
factorization to be unique, but this theorem also applies to factoring \
polynomials in more than one variable. \n\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", " ", "[", "x", "]"}], TraditionalForm]]],
 " is an integral domain, we can consider another variable ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", and consider the polynomial ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", " ", "[", "x", "]"}], " ", "[", "y", "]"}], 
   TraditionalForm]]],
 ". A typical element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", " ", "[", "x", "]"}], " ", "[", "y", "]"}], 
   TraditionalForm]]],
 " would be\n\n  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      SubscriptBox["c", "0"], "(", "x", ")"}], " ", "+", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["c", "1"], "(", "x", ")"}], " ", "y"}], " ", "+", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["c", "2"], "(", "x", ")"}], " ", 
      SuperscriptBox["y", "2"]}], " ", "+", " ", 
     RowBox[{
      RowBox[{
       SubscriptBox["c", "3"], "(", "x", ")"}], " ", 
      SuperscriptBox["y", "3"]}], " ", "+", " ", "\[Ellipsis]", " ", "+", " ",
      
     RowBox[{
      RowBox[{
       SubscriptBox["c", "n"], "(", "x", ")"}], " ", 
      RowBox[{
       SuperscriptBox["y", "n"], "."}]}]}]}], TraditionalForm]]],
 "\n\nwhere each ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["c", "i"], "(", "x", ")"}], TraditionalForm]]],
 " is a polynomial in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", " ", "[", "x", "]"}], TraditionalForm]]],
 ". If each ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["c", "i"], "(", "x", ")"}], TraditionalForm]]],
 " is written\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["c", "i"], "(", "x", ")"}], "  ", "=", " ", 
    RowBox[{
     SubscriptBox["d", 
      RowBox[{"0", "a"}]], "+", " ", 
     RowBox[{
      SubscriptBox["d", 
       RowBox[{"1", "a"}]], " ", "x"}], " ", "+", " ", 
     RowBox[{
      SubscriptBox["d", 
       RowBox[{"2", "a"}]], " ", 
      SuperscriptBox["x", "2"]}], "+", " ", 
     RowBox[{
      SubscriptBox["d", 
       RowBox[{"3", "a"}]], " ", 
      SuperscriptBox["x", "3"]}], " ", "+", " ", "\[Ellipsis]", " "}]}], 
   TraditionalForm]]],
 "\n\nWe find that the polynomial in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", " ", "[", "x", "]"}], " ", "[", "y", "]"}], 
   TraditionalForm]]],
 " could be written \n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["d", 
     RowBox[{"0", "  ", "0"}]], "   ", "+", " ", 
    RowBox[{
     SubscriptBox["d", 
      RowBox[{"1", "  ", "0"}]], " ", "x"}], " ", "+", " ", 
    RowBox[{
     SubscriptBox["d", 
      RowBox[{"0", "  ", "1"}]], " ", "y"}], " ", "+", " ", 
    RowBox[{
     SubscriptBox["d", 
      RowBox[{"2", "  ", "0"}]], "   ", 
     SuperscriptBox["x", "2"]}], " ", "+", " ", 
    RowBox[{
     SubscriptBox["d", 
      RowBox[{"1", "  ", "1"}]], 
     RowBox[{"x", "\[CenterDot]", "y"}]}], " ", "+", " ", 
    RowBox[{
     SubscriptBox["d", 
      RowBox[{"0", "  ", "2"}]], 
     SuperscriptBox["y", "2"]}], " ", "+", " ", 
    RowBox[{"\[Ellipsis]", " ", "."}]}], TraditionalForm]]],
 " \n\nIf we make the convention that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", 
    RowBox[{"y", "\[CenterDot]", "x"}]}], TraditionalForm]]],
 ", then it is easy to see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"R", " ", "[", "x", "]"}], " ", "[", "y", "]"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"R", " ", "[", "y", "]"}], " ", "[", "x", "]"}], ".", " "}]}], 
   TraditionalForm]]],
 "\n\nDEFINITION 12.10:\nWe will denote the polynomial ring of two variables \
by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", " ", "[", 
     RowBox[{"x", ",", "y"}], "]"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"R", " ", "[", "x", "]"}], " ", "[", "y", "]"}], "."}]}], 
   TraditionalForm]]],
 " The variables ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are called ",
 StyleBox["indeterminates",
  FontSlant->"Italic"],
 ". Likewise, we denote the polynomial ring \nof n indeterminates by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", " ", "[", 
     RowBox[{
      SubscriptBox["x", "1"], ",", 
      SubscriptBox["x", "2"], ",", 
      SubscriptBox["x", "3"], ",", " ", "\[Ellipsis]", " ", ",", " ", 
      SubscriptBox["x", "n"]}], "]"}], "."}], TraditionalForm]]],
 "\n\nCOROLLARY 12.6\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a unique factorization domain and let  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "1"], ",", " ", 
    SubscriptBox["x", "2"], ",", " ", 
    SubscriptBox["x", "3"], ",", " ", 
    RowBox[{"\[Ellipsis]", " ", 
     SubscriptBox["x", "n"]}]}], TraditionalForm]]],
 "  be indeterminates over ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Then  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"R", " ", "[", 
     RowBox[{
      SubscriptBox["x", "1"], ",", 
      SubscriptBox["x", "2"], ",", 
      SubscriptBox["x", "3"], ",", 
      RowBox[{"\[Ellipsis]", " ", 
       SubscriptBox["x", "n"]}]}], "]"}], " "}], TraditionalForm]]],
 " is a unique factorization domain."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745560546875*^9, 3.43774568346875*^9}, {
  3.4377459123125*^9, 3.437745936921875*^9}, {3.437835354421875*^9, 
  3.43783536796875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We will use induction on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ", the unique factorization domain theorem (12.3) shows that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "[", "x", "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"UFD", "."}], TraditionalForm]]],
 " Otherwise, we write \n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"R", " ", "[", 
     RowBox[{
      SubscriptBox["x", "1"], ",", " ", 
      SubscriptBox["x", "2"], ",", " ", 
      SubscriptBox["x", "3"], ",", " ", "\[Ellipsis]", ",", " ", 
      SubscriptBox["x", "n"]}], "]"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"R", " ", "[", 
      RowBox[{
       SubscriptBox["x", "1"], ",", " ", 
       SubscriptBox["x", "2"], ",", " ", 
       SubscriptBox["x", "3"], ",", " ", 
       RowBox[{"\[Ellipsis]", " ", 
        SubscriptBox["x", 
         RowBox[{"n", "-", "1"}]]}]}], "]"}], " ", "[", 
     SubscriptBox["x", "n"], "]"}]}], TraditionalForm]]],
 ".\n\nBy the induction hypothesis, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", " ", "[", 
    RowBox[{
     SubscriptBox["x", "1"], ",", " ", 
     SubscriptBox["x", "2"], ",", " ", 
     SubscriptBox["x", "3"], ",", " ", "\[Ellipsis]", ",", 
     SubscriptBox["x", 
      RowBox[{"n", "-", "1"}]]}], "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"UFD", "."}], TraditionalForm]]],
 " So by the unique factorization domain theorem (12.3), ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", " ", "[", 
    RowBox[{
     SubscriptBox["x", "1"], ",", " ", 
     SubscriptBox["x", "2"], ",", " ", 
     SubscriptBox["x", "3"], ",", " ", "\[Ellipsis]", ",", 
     SubscriptBox["x", "n"]}], "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"UFD", "."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437745937703125*^9, 3.43774594253125*^9}, {
  3.43783538246875*^9, 3.437835391875*^9}}]
}, Open  ]],

Cell[TextData[{
 "Polynomials in several varibles are of considerable importance in geometry, \
since curves and surfaces are described by equations in several variables. \
Although ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s Factor command will be able to factor polynomials in many variables, its \
ability is limited to when ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is either ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalQ]", TraditionalForm]]],
 ". For example, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can factor"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
ClearDefs
Factor[x^3 y^2 + x^2 y - x y^2 - 2 x + y]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437745943171875*^9, 3.437745954703125*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"x", " ", "y"}]}], ")"}], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"2", " ", "x"}], "-", "y", "+", 
    RowBox[{
     SuperscriptBox["x", "2"], " ", "y"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.4427552096645*^9}]
}, Open  ]],

Cell[TextData[{
 "over the integers, but cannot factor this over any other ring, even a \
finite field. The reason is simply a lack of efficient algorithms for \
factoring polynomials in more than one variable.\n\nEXPERIMENT:\nJust to \
demonstrate the difficulty of factoring a polynomial with more than one \
variable, try to factor the polynomial\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "5"], " ", "-", " ", 
    RowBox[{
     SuperscriptBox["x", "4"], "y"}], " ", "+", " ", 
    RowBox[{"2", " ", "x", " ", 
     SuperscriptBox["y", "2"]}], " ", "-", " ", 
    RowBox[{"4", " ", "x"}], " ", "-", " ", 
    RowBox[{"4", " ", "y"}]}], TraditionalForm]]],
 "\n\nby hand. One would think that having 2 variables would give more clues \
to the factorization, but in fact the factorization becomes tricky. If you \
totally give up, have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " give you the factorization."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4377459671875*^9, 3.437745975234375*^9}}],

Cell["   ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ability to factor such polynomials will later come in very handy, since \
the problem of factoring a polynomial over an unusual ring is many times \
equivalant to the problem of factoring a polynomial of several variables over \
",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Principal Ideal Domains", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Although we have found that polynomial rings created from unique \
factorization domains produce more unique factorization domains, there still \
is the question of how to tell whether a given ring is a unique factorization \
domain. The answer lies in the ideals of the ring. In fact, the ideals of a \
ring were discovered by Kummer in 1835 in an attempt to prove that certain \
rings were unique factorization domains. In this section we will explore the \
interesting interconnection between the ideals of a ring, and the prime and \
irreducible elements of the ring.\n\nWe begin by recalling that many ideals \
can be generated with only one element. In fact, many rings, such as the \
integers ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ", are such that every ideal is generated by only one element. We called \
such rings ",
 StyleBox["principle ideal rings",
  FontSlant->"Italic"],
 ", or ",
 Cell[BoxData[
  FormBox["PIR", TraditionalForm]]],
 "'s. (When the ring is also a domain, we call it a principle ideal domain, \
or ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 ".) In fact, ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 "'s are so common that it is somewhat tricky to find an example of an \
integral domain that is not a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 ". \n\nConsider the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", " ", "=", " ", 
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     RowBox[{"x", ",", "y"}]}]}], TraditionalForm]]],
 "]. We saw by corollary 12.6 that this is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ". We would now like to show that this is ",
 StyleBox["not",
  FontSlant->"Italic"],
 " a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"PID", "."}], TraditionalForm]]],
 " Consider the ideal of elements without a constant term. This ideal can be \
expressed as  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", ",", " ", "y"}], ")"}], TraditionalForm]]],
 " but it requires at least two elements to generate this ideal in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     RowBox[{"x", ",", "y"}], "]"}], "."}], TraditionalForm]]],
 " Thus, this ideal is not a principle ideal, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    RowBox[{"x", ",", "y"}], "]"}], TraditionalForm]]],
 " is not a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 ", even though it is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ".\n\nThe following definition sheds light on the connection between ideals \
and unique factorization domains.\n\nDEFINITION 12.11:\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a commutative ring, and let ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " be a non-trivial ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". (Thus, ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is neither ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "0", "}"}], TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ".) We say that ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a ",
 StyleBox["prime ideal",
  FontSlant->"Italic"],
 " if, whenever ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", then either ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". \n\nWhen we first defined a prime element of a ring, we were careful to \
mention that the ring did not have to be an integral domain. By defining \
prime elements for all commutative rings, we open the door to showing a \
connection between prime ideals and prime elements.\n\nPROPOSITION 12.6\nLet \
",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a commutative ring with an identity. Then ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " if, and only if, the principal ideal ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "p", ")"}], TraditionalForm]]],
 " is a prime ideal."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4377459975*^9, 3.437746012453125*^9}, {3.437835824*^9, 
  3.43783585040625*^9}, {3.4419820625421247`*^9, 3.4419820649483747`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is prime. Then ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is neither 0 nor a unit, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "p", ")"}], TraditionalForm]]],
 " cannot be the zero ring. If",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"(", "p", ")"}], " ", "=", " ", "R"}]}], TraditionalForm]]],
 ", then there must be some element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " which makes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"p", "\[CenterDot]", "x"}], " ", "=", " ", "1."}], 
   TraditionalForm]]],
 " But this is impossible, since ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is not a unit. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "p", ")"}], TraditionalForm]]],
 " would be a non-trivial ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Now suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "p", ")"}], TraditionalForm]]],
 ". Then there must be some ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", "y"}], " ", "=", " ", 
    RowBox[{"p", "\[CenterDot]", 
     RowBox[{"z", "."}]}]}], TraditionalForm]]],
 " Since ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is prime, either ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". So either ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", "p", ")"}], ","}], TraditionalForm]]],
 " making ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "p", ")"}], TraditionalForm]]],
 " a prime ideal.\n\nNow suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "p", ")"}], TraditionalForm]]],
 " is a prime ideal. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "p", ")"}], TraditionalForm]]],
 " is neither ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "0", "}"}], TraditionalForm]]],
 " nor ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is neither 0 nor a unit. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "y"}], TraditionalForm]]],
 " would be in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "p", ")"}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "p", ")"}], TraditionalForm]]],
 " is a prime ideal, either ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " would then be in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", "p", ")"}], "."}], TraditionalForm]]],
 " But this would indicate that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is a prime element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437908465875*^9, 3.43790846784375*^9}, {
  3.43790887340625*^9, 3.437908873875*^9}}]
}, Open  ]],

Cell[TextData[{
 "\nAlthough this proposition refers to principle ideals, it is certainly \
possible for an ideal to be a prime ideal without being even a principle \
ideal. For example, the ideal ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 " of the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    RowBox[{"x", ",", "y"}], "]"}], TraditionalForm]]],
 " is not a principle ideal, yet it is a prime ideal. To see this, note that \
we can characterize the ideal as\n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", ",", "y"}], ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Florin]", "(", 
           RowBox[{"x", ",", "y"}], ")"}], " ", "\[Element]", " ", 
          RowBox[{"\[DoubleStruckCapitalZ]", "[", 
           RowBox[{"x", ",", "y"}], "]"}]}], "  ", "|", "  ", 
         RowBox[{"\[Florin]", "(", 
          RowBox[{"0", ",", "0"}], ")"}]}], "=", "0"}], "}"}], "."}]}]}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774605265625*^9, 3.43774608596875*^9}}],

Cell[TextData[{
 "Thus, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", 
     RowBox[{"x", ",", "y"}], ")"}], "\[CenterDot]", 
    RowBox[{"g", "(", 
     RowBox[{"x", ",", "y"}], ")"}]}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 ", we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"f", "(", 
       RowBox[{"0", ",", "0"}], ")"}], "\[CenterDot]", 
      RowBox[{"g", "(", 
       RowBox[{"0", ",", "0"}], ")"}]}], " ", "=", " ", "0"}], ","}], 
   TraditionalForm]]],
 " so either",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"0", ",", "0"}], ")"}], " ", "=", " ", "0"}]}], 
   TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"g", "(", 
     RowBox[{"0", ",", "0"}], ")"}], " ", "=", " ", "0."}], 
   TraditionalForm]]],
 " Therefore, either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", ",", "y"}], ")"}], ","}], TraditionalForm]]],
 " so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"x", ",", "y"}], ")"}], TraditionalForm]]],
 " is a prime ideal.\n\nAlthough proposition 12.6 gives us a test for \
determining whether an element is prime, to impliment this we need a test to \
see whether an ideal is a prime ideal. The next proposition does the trick.\n\
\nPROPOSITION 12.7\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a commutative ring with identity, and let ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " be a non-trival ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a prime ideal if, and only if, the quotient ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "P"}], TraditionalForm]]],
 " has no zero divisors."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437746069046875*^9, 3.437746095125*^9}, {
   3.437843038390625*^9, 3.437843039125*^9}, 3.437908885671875*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Assume that ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a prime ideal. Let us suppose that the product of two elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "P"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "+", "P"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "+", "P"}], TraditionalForm]]],
 ", is the zero element. That is,  \n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", " ", "+", " ", "P"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"b", " ", "+", " ", "P"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"a", "\[CenterDot]", "b"}], " ", "+", " ", "P"}], " ", "=", 
      " ", 
      RowBox[{"0", " ", "+", " ", 
       RowBox[{"P", "."}]}]}], " "}]}], TraditionalForm]]],
 "\n\nThis implies that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a prime ideal, either ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Thus, either \n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", " ", "+", " ", "P"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"0", " ", "+", " ", 
      RowBox[{"P", "    ", "or", "  ", "b"}], " ", "+", " ", "P"}], " ", "=", 
     " ", 
     RowBox[{"0", " ", "+", " ", 
      RowBox[{"P", "."}]}]}]}], TraditionalForm]]],
 "\n\nThus, we have shown that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "P"}], TraditionalForm]]],
 " has no zero divisors.\n\nNow suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "P"}], TraditionalForm]]],
 " has no zero divisors. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[CenterDot]", "b"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", then we have the following holding in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "P"}], TraditionalForm]]],
 ":\n\n          ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", " ", "+", " ", "P"}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"b", " ", "+", " ", "P"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"a", "\[CenterDot]", "b"}], " ", "+", " ", "P"}], " ", "=", 
      " ", 
      RowBox[{"0", " ", "+", " ", 
       RowBox[{"P", "."}]}]}], " "}]}], TraditionalForm]]],
 "\n\nSince ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", "P"}], TraditionalForm]]],
 " has no zero divisors, either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "+", "P"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "+", "P"}], TraditionalForm]]],
 " must be equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", "+", "P"}], TraditionalForm]]],
 ". Thus, either ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ", and since ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a non-trivial ideal, ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " is a prime ideal."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437908905640625*^9, 3.437908916765625*^9}}]
}, Open  ]],

Cell["\<\
Let us try to use this proposition to find the prime elements of the \
following familar commutitive ring. \
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
InitRing
Define[4a, 0]; Define[2b, 0]
Define[a.a, a]; Define[b.b, b]
Define[a.b, 0]; Define[b.a, 0]
R = Ring[{a,b}]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437908996671875*^9, 3.437909007125*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"3", " ", "a"}], ",", "b", ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4419821453077497`*^9, 3.4427552224145*^9}]
}, Open  ]],

Cell[TextData[{
 "We determined that the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", "a"}], "+", "b"}], TraditionalForm]]],
 " was irreducible in this ring. Let us determine whether ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", "a"}], "+", "b"}], TraditionalForm]]],
 " is prime by computing the quotient ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", "a"}], "+", "b"}], ")"}]}], TraditionalForm]]],
 ".\n\nFirst, we find the principle ideal generated by 2a+b:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["S = Ideal[R,{2a + b}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"2", " ", "a"}], ",", "b", ",", 
   RowBox[{
    RowBox[{"2", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4419821493702497`*^9, 3.442755224352*^9}]
}, Open  ]],

Cell["\<\

This forms a non-trivial ideal, so we can now consider the quotent ring.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Q = Coset[R,S]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", 
     RowBox[{"2", " ", "a"}], ",", "b", ",", 
     RowBox[{
      RowBox[{"2", " ", "a"}], "+", "b"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", 
     RowBox[{"3", " ", "a"}], ",", 
     RowBox[{"a", "+", "b"}], ",", 
     RowBox[{
      RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4419821540264997`*^9, 3.442755225727*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Q", "[", 
   RowBox[{"[", "2", "]"}], "]"}], ".", 
  RowBox[{"Q", "[", 
   RowBox[{"[", "2", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4419821670108747`*^9, 3.4419821800733747`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"a", ",", 
   RowBox[{"3", " ", "a"}], ",", 
   RowBox[{"a", "+", "b"}], ",", 
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", "b"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.4419821712608747`*^9, 3.4419821805733747`*^9}, 
   3.442755227367625*^9}]
}, Open  ]],

Cell["\<\
The quotient group is only two elements, so there is a good chance that this \
has no zero divisors. We can double check by looking at the multiplication \
table.\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["\<\
QuotientRing = True
MultTable[Q]\
\>", "Input",
 PageWidth->Infinity,
 CellChangeTimes->{{3.437909022015625*^9, 3.437909023203125*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.442755229336375*^9}],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{1, 0, 1}, {1, 1, 0}, {1, 0, 1}}, {{1, 1, 0}, {1, 1, 0}, {1, 1,
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}}}], InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"{0,2a,b,2a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a,3a,a+b,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"{0,2a,b,2a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"{a,3a,a+b,3a+b}\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a,b,2a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a,b,2a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"{0,2a,b,2a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"{a,3a,a+b,3a+b}\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 2}, {3, 2}}]}]], "Output",
 CellChangeTimes->{3.442755229367625*^9}]
}, Open  ]],

Cell[TextData[{
 "Thus, the quotient is isomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "2"], TraditionalForm]]],
 ". By proposition 12.7, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", "a"}], "+", "b"}], TraditionalForm]]],
 " is prime.\n\nEXPERIMENT:\nTry the above procedure on the other 6 non-zero \
elements of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Which elements are prime? "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43790907103125*^9, 3.437909071515625*^9}}],

Cell["     ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Although we can clearly use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " along with proposition 12.7 to find the prime elements of any finite ring, \
we are mainly interested in finding the prime elements of an infinite ring. \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can still help us out, since the quotient ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"(", "p", ")"}]}], TraditionalForm]]],
 " will usually be finite.\n\nLet us consider the example of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "5"}]], "]"}], TraditionalForm]]],
 ". We saw in the last section that 3 was an irreducible element. Let us use \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to see if 3 is a prime number in this ring. We need to determine the ring \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox[
      RowBox[{"-", "5"}]], "]"}], "/", 
    RowBox[{
     RowBox[{"(", "3", ")"}], "."}]}], TraditionalForm]]],
 "\n\nWe begin by noticing that since 3 is in the ideal (3), every element \
multiplied by 3 in the quotient ring must be 0. Thus, the characteristic of \
the quotient ring is 3. We can start by defining the quotient ring as a \
domain:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43790907465625*^9, 3.437909075421875*^9}}],

Cell["InitDomain[3]", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "If we denote the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SqrtBox[
      RowBox[{"-", "5"}]],
     TraditionalForm], " ", "+", 
    RowBox[{"(", "3", ")"}]}], TraditionalForm]]],
 " by ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", "2"], " ", "=", " ", 
    RowBox[{
     RowBox[{"-", "5"}], " ", "+", 
     RowBox[{
      RowBox[{"(", "3", ")"}], "."}]}]}], TraditionalForm]]],
 " Thus, we can define"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["Define[a^2,-5]", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We now can see the quotient ring as the ring generated by 1 and ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ":"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["R = Ring[{1,a}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1", ",", "2", ",", "a", ",", 
   RowBox[{"2", " ", "a"}], ",", 
   RowBox[{"1", "+", "a"}], ",", 
   RowBox[{"2", "+", "a"}], ",", 
   RowBox[{"1", "+", 
    RowBox[{"2", " ", "a"}]}], ",", 
   RowBox[{"2", "+", 
    RowBox[{"2", " ", "a"}]}]}], "}"}]], "Output",
 CellChangeTimes->{3.44275523532075*^9}]
}, Open  ]],

Cell["\<\
This ring has 9 elements. We have already seen a field of 9 elements, so it \
is certainly possible that this ring has no zero divisors. Let's check:\
\>", "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["MultTable[R]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 GraphicsBox[{
   RasterBox[{{{0.8, 0.6, 0.5}, {1, 1, 0}, {0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {
      0.8, 0.6, 0.5}, {0.6, 0.4, 0.8}, {0.6, 0.4, 0.8}, {1, 1, 0}, {1, 1, 
      0}, {0.8, 0.6, 0.5}}, {{0.7, 0.8, 0.7}, {1, 1, 0}, {0.7, 0.8, 0.7}, {
      1, 0.6, 0}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {1, 1, 0}, {0.7, 0.8, 0.7}, {
      1, 0.6, 0}, {1, 1, 0}}, {{1, 0.6, 0}, {1, 1, 0}, {1, 0.6, 0}, {0.7, 0.8,
       0.7}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {1, 1, 0}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {1, 1, 0}}, {{0.6, 0.4, 0.8}, {1, 1, 0}, {0.6, 0.4, 0.8}, {0.8, 
      0.6, 0.5}, {0.6, 0.4, 0.8}, {0.8, 0.6, 0.5}, {0.8, 0.6, 0.5}, {1, 1, 
      0}, {1, 1, 0}, {0.6, 0.4, 0.8}}, {{0, 1, 0}, {1, 1, 0}, {0, 1, 0}, {1, 
      0, 0}, {0, 1, 1}, {1, 0, 1}, {0.8, 0.6, 0.5}, {1, 0.6, 0}, {0.7, 0.8, 
      0.7}, {0.6, 0.4, 0.8}}, {{1, 0, 0}, {1, 1, 0}, {1, 0, 0}, {0, 1, 0}, {1,
       0, 1}, {0, 1, 1}, {0.6, 0.4, 0.8}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.8, 
      0.6, 0.5}}, {{0, 1, 1}, {1, 1, 0}, {0, 1, 1}, {1, 0, 1}, {0, 1, 0}, {1, 
      0, 0}, {0.8, 0.6, 0.5}, {0.7, 0.8, 0.7}, {1, 0.6, 0}, {0.6, 0.4, 
      0.8}}, {{1, 0, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 
      0}, {0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}}, {{
     1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}, {1, 1, 0}, {1, 1, 0}, {1, 1, 
     0}}, {{1, 1, 1}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {1, 0, 0}, {0, 1, 0}, {
      0.6, 0.4, 0.8}, {1, 0.6, 0}, {0.7, 0.8, 0.7}, {0.8, 0.6, 0.5}}}], 
   InsetBox[
    TagBox[
     StyleBox["\<\"\[CenterDot]\"\>",
      FontSize->18],
     StyleForm[#, FontSize -> 18]& ], {0.5, 9.5}], {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {1.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {2.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {3.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {4.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {5.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {6.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {7.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {8.5, 9.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {9.5, 9.5}]}, {InsetBox[
     TagBox[
      StyleBox["\<\"0\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 8.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 7.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 6.5}], InsetBox[
     TagBox[
      StyleBox["\<\"a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 5.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 4.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 3.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 2.5}], InsetBox[
     TagBox[
      StyleBox["\<\"1+2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 1.5}], InsetBox[
     TagBox[
      StyleBox["\<\"2+2a\"\>",
       FontSize->18],
      StyleForm[#, FontSize -> 18]& ], {0.5, 0.5}]}, {{InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 8.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 8.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 7.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 7.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 6.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 6.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 5.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 5.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 4.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 4.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 3.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 3.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 2.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 2.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 1.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 1.5}]}, {InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {1.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {2.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {3.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {4.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {5.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"1+a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {6.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {7.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"0\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {8.5, 0.5}], InsetBox[
      TagBox[
       StyleBox["\<\"2+2a\"\>",
        FontSize->18],
       StyleForm[#, FontSize -> 18]& ], {9.5, 0.5}]}}, 
   LineBox[{{1, 0}, {1, 9}, {10, 9}}]}]], "Output",
 CellChangeTimes->{3.44275523707075*^9}]
}, Open  ]],

Cell[TextData[{
 "The zero divisors in this ring are evident, so 3 is ",
 StyleBox["not",
  FontSlant->"Italic"],
 " a prime element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "5"}]], " ", "]"}], TraditionalForm]]],
 ". Another way of checking whether there are zero divisors is with the ",
 StyleBox["CheckField",
  FontWeight->"Bold"],
 " command. Recall that we only have to enter in a basis for the ring, rather \
than the whole ring. Thus, "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["CheckField[{1,a}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Identity element = \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Identity element = ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.442755239492625*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"An element is invertable as long as \"\>", "\[InvisibleSpace]",
    
   RowBox[{
    SuperscriptBox[
     RowBox[{"C", "[", "1", "]"}], "2"], "+", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"C", "[", "2", "]"}], "2"]}]}], 
   "\[InvisibleSpace]", "\<\" is not zero.\"\>", 
   "\[InvisibleSpace]", "\<\" \"\>"}],
  SequenceForm[
  "An element is invertable as long as ", C[1]^2 + 2 C[2]^2, " is not zero.", 
   " "],
  Editable->False]], "Print",
 CellChangeTimes->{3.442755239523875*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"1", "+", "a"}], 
   "\[InvisibleSpace]", "\<\" is not invertable.\"\>"}],
  SequenceForm[1 + $CellContext`a, " is not invertable."],
  Editable->False]], "Print",
 CellChangeTimes->{3.4427552395395*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"2", "+", "a"}], 
   "\[InvisibleSpace]", "\<\" is not invertable.\"\>"}],
  SequenceForm[2 + $CellContext`a, " is not invertable."],
  Editable->False]], "Print",
 CellChangeTimes->{3.442755239555125*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"1", "+", 
    RowBox[{"2", " ", "a"}]}], 
   "\[InvisibleSpace]", "\<\" is not invertable.\"\>"}],
  SequenceForm[1 + 2 $CellContext`a, " is not invertable."],
  Editable->False]], "Print",
 CellChangeTimes->{3.44275523957075*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"2", "+", 
    RowBox[{"2", " ", "a"}]}], 
   "\[InvisibleSpace]", "\<\" is not invertable.\"\>"}],
  SequenceForm[2 + 2 $CellContext`a, " is not invertable."],
  Editable->False]], "Print",
 CellChangeTimes->{3.442755239586375*^9}],

Cell[BoxData["\<\"This is only a communitive ring with identity.\"\>"], \
"Print",
 CellChangeTimes->{3.442755239617625*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The inverse of \"\>", "\[InvisibleSpace]", 
   RowBox[{
    RowBox[{"C", "[", "1", "]"}], "+", 
    RowBox[{"a", " ", 
     RowBox[{"C", "[", "2", "]"}]}]}], 
   "\[InvisibleSpace]", "\<\" is given by \"\>", "\[InvisibleSpace]", 
   FractionBox[
    RowBox[{
     RowBox[{"C", "[", "1", "]"}], "+", 
     RowBox[{"2", " ", "a", " ", 
      RowBox[{"C", "[", "2", "]"}]}]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"C", "[", "1", "]"}], "2"], "+", 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"C", "[", "2", "]"}], "2"]}]}]]}],
  SequenceForm[
  "The inverse of ", C[1] + $CellContext`a C[2], 
   " is given by ", (C[1] + 2 $CellContext`a C[2])/(C[1]^2 + 2 C[2]^2)],
  Editable->False]], "Print",
 CellChangeTimes->{3.442755239617625*^9}]
}, Open  ]],

Cell[TextData[{
 "finds all of the zero divisors of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ".\n\nEXPERIMENT:                                      \nUse the same \
procedure to show that 2 is not a prime element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "5"}]], "]"}], TraditionalForm]]],
 ". Is 7 a prime element in this ring? How about 11? Note that in the rings \
produced in the last two examples are too big to display multiplication \
tables, so we must rely on the ",
 StyleBox["CheckField",
  FontWeight->"Bold"],
 " command.\n\nAt this point you may be wondering whether there are ",
 StyleBox["any",
  FontSlant->"Italic"],
 " prime elements in the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "5"}]], "]"}], TraditionalForm]]],
 ". Let us try one more example. Consider the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", " ", "+", " ", 
    RowBox[{"2", " ", 
     RadicalBox[
      RowBox[{"-", "5"}], " "]}]}], TraditionalForm]]],
 ". Is this element prime?\n\nDefining the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox[
      RowBox[{"-", "5"}]], " ", "]"}], " ", "/", 
    RowBox[{"(", 
     RowBox[{"3", "+", 
      RowBox[{"2", 
       SqrtBox[
        RowBox[{"-", "5"}]]}]}], ")"}]}], TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is a \nlittle bit trickier since the characteristic must be an integer, \
not ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", "+", 
    RowBox[{"2", 
     RowBox[{
      SqrtBox[
       RowBox[{"-", "5"}]], "."}]}]}], TraditionalForm]]],
 " To find an ",
 StyleBox["integer",
  FontSlant->"Italic"],
 " in this ideal that we can use for the characteristic, note that\n\n      \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"3", " ", "+", " ", 
       RowBox[{"2", " ", 
        RadicalBox[
         RowBox[{"-", "5"}], " "]}]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"3", "-", 
       RowBox[{"2", 
        RadicalBox[
         RowBox[{"-", "5"}], " "]}]}], ")"}]}], " ", "=", " ", "29"}], 
   TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Thus, 29 is in the ideal ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"3", "+", 
     RowBox[{"2", 
      SqrtBox[
       RowBox[{"-", "5"}]]}]}], ")"}], TraditionalForm]]],
 ". Let us use this as the characteristic."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["\<\
InitDomain[29]
Define[a^2,-5]\
\>", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "Of course this would give us the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[DoubleStruckCapitalZ]", "[", 
      SqrtBox[
       RowBox[{"-", "5"}]], "]"}], "/", 
     RowBox[{"(", "29", ")"}]}], ","}], TraditionalForm]]],
 " rather than the ring we are looking for. To reduce the elements further, \
we would like to find an integer within the coset \n\n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"a", " ", "+", " ", 
      RowBox[{"(", 
       RowBox[{"3", " ", "+", " ", 
        RowBox[{"2", " ", "a"}]}], ")"}]}], " ", "=", "  ", 
     RowBox[{
      SqrtBox[
       RowBox[{"-", "5"}]], "  ", "+", " ", 
      RowBox[{
       RowBox[{"(", " ", 
        RowBox[{"3", " ", "+", " ", 
         RowBox[{"2", " ", 
          SqrtBox[
           RowBox[{"-", "5"}]]}]}], " ", ")"}], "."}]}]}]}], 
   TraditionalForm]]],
 "\n\nThat is, we would like to find numbers ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " such that "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["a + (3 + 2 a).(x + y a)", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "is an integer.\n\nEXPERIMENT:\nReplace the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in the above command so that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " produces an integer. There will be many solutions for ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", but the integer produced will always be the same.\n\nThis experiment \
shows that the element ",
 Cell[BoxData[
  FormBox[
   SqrtBox[
    RowBox[{"-", "5"}]], TraditionalForm]]],
 " is equivalent to 13 in the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox[
      RowBox[{"-", "5"}]], "]"}], "/", 
    RowBox[{"(", " ", 
     RowBox[{"3", " ", "+", " ", 
      RowBox[{"2", 
       SqrtBox[
        RowBox[{"-", "5"}]]}]}], " ", ")"}]}], TraditionalForm]]],
 ". Thus, every element of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"y", "\[CenterDot]", 
     SqrtBox[
      RowBox[{"-", "5"}]]}]}], TraditionalForm]]],
 " will be equivalent to an integer. We can find the quotient ring in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " simply by finding the ring generated by 1."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Q = Ring[{1}]", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", 
   ",", "8", ",", "9", ",", "10", ",", "11", ",", "12", ",", "13", ",", "14", 
   ",", "15", ",", "16", ",", "17", ",", "18", ",", "19", ",", "20", ",", 
   "21", ",", "22", ",", "23", ",", "24", ",", "25", ",", "26", ",", "27", 
   ",", "28"}], "}"}]], "Output",
 CellChangeTimes->{3.44275527063325*^9}]
}, Open  ]],

Cell[TextData[{
 "We could use the ",
 StyleBox["CheckField",
  FontWeight->"Bold"],
 " command to see if this is a field, but in this case the answer is obvious. \
The order of the quotient ring is "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[CellGroupData[{

Cell["Print[Length[Q]];", "Input",
 PageWidth->Infinity],

Cell[BoxData["29"], "Print",
 CellChangeTimes->{3.442755272492625*^9}]
}, Open  ]],

Cell[TextData[{
 "which is a prime number. There are only two rings of this order, and one of \
them has no identity. Since 1 is clearly an identity of the quotient ring, we \
have\n\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[DoubleStruckCapitalZ]", "[", 
      RadicalBox[
       RowBox[{"-", "5"}], " "], "]"}], "/", 
     RowBox[{"(", 
      RowBox[{"3", "+", 
       RowBox[{"2", 
        RadicalBox[
         RowBox[{"-", "5"}], " "]}]}], ")"}]}], " ", 
    StyleBox["\[TildeTilde]",
     FontSize->16], " ", 
    SubscriptBox["Z", "29"]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "This ring has no zero divisors, so by proposition 12.7, the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", " ", "+", " ", 
    RowBox[{"2", " ", 
     SqrtBox[
      RowBox[{"-", "5"}]]}]}], TraditionalForm]]],
 " is prime.\n\nEXPERIMENT:                   \nIf you tried the above \
procedure to find ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox[
      RowBox[{"-", "5"}]], "]"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", " ", "+", " ", 
      SqrtBox[
       RowBox[{"-", "5"}]]}], ")"}]}], TraditionalForm]]],
 ", what ring would you get?"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437909187359375*^9, 3.437909187984375*^9}}],

Cell[" ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "We have seen that proposition 12.7 is a useful way of determining whether \
an element is prime. Let us use this proposition to show that in a principle \
ideal domain, irreducible elements are also prime elements. This amounts to \
showing that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"(", "p", ")"}]}], TraditionalForm]]],
 " has no zero divisors whenever ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is irreducible. However, we can actually prove more, which will be very \
useful later on. \n\nLEMMA 12.6             \nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a principle ideal domain, and let  ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 "  be an irreducible element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Then the quotient ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"(", "p", ")"}]}], TraditionalForm]]],
 " is a field."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43790919403125*^9, 3.437909207796875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is an integral domain, it is clear that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"(", "p", ")"}]}], TraditionalForm]]],
 " is a commutative ring, and contains the identity element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", " ", "+", " ", 
    RowBox[{
     RowBox[{"(", "p", ")"}], "."}]}], TraditionalForm]]],
 " Thus, we have to show that all non-zero elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"(", "p", ")"}]}], TraditionalForm]]],
 " have an inverse. Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "+", " ", 
    RowBox[{"(", "p", ")"}]}], TraditionalForm]]],
 " be a non-zero element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{
     RowBox[{"(", "p", ")"}], "."}]}], TraditionalForm]]],
 " We immediately have that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is not a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Thus, we can consider the ideal generated by both ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ":\n\n                             ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"(", 
     RowBox[{"x", ",", " ", "p"}], ")"}]}], TraditionalForm]]],
 ".\n\nSince ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 ", there is some element ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", ",", "p"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", "d", ")"}], "."}]}], TraditionalForm]]],
 " Then both ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " would be multiples of ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 ". But we already observed that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is not a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " cannot be a multiple of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is irreducible, so ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " must be a unit. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", "d", ")"}], " ", "=", " ", "R"}], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", ",", "p"}], ")"}], " ", "=", " ", "R"}], 
   TraditionalForm]]],
 ". This means that there are elements ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " such that\n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "u"}], " ", "+", " ", 
      RowBox[{"p", "\[CenterDot]", "v"}]}], " ", "=", " ", "1."}]}], 
   TraditionalForm]]],
 "\n\nWe now claim that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "+", " ", 
    RowBox[{"(", "p", ")"}]}], TraditionalForm]]],
 " is our sought after inverse. Note that\n\n   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"[", 
       RowBox[{"x", " ", "+", " ", 
        RowBox[{"(", "p", ")"}]}], "]"}], "\[CenterDot]", 
      RowBox[{"[", 
       RowBox[{"u", " ", "+", " ", 
        RowBox[{"(", "p", ")"}]}], "]"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"x", "\[CenterDot]", "u"}], " ", "+", " ", 
       RowBox[{"(", "p", ")"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"x", "\[CenterDot]", "u"}], " ", "+", " ", 
        RowBox[{"p", "\[CenterDot]", "v"}], " ", "+", " ", 
        RowBox[{"(", "p", ")"}]}], " ", "=", " ", 
       RowBox[{"1", " ", "+", " ", 
        RowBox[{
         RowBox[{"(", "p", ")"}], "."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nSince every non-zero element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"(", "p", ")"}]}], TraditionalForm]]],
 " is invertible, we have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"(", "p", ")"}]}], TraditionalForm]]],
 " is a field."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "From this lemma, it is easy to see that an irreducible element of a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 " must also be a prime element. Thus, we are on our way to showing that a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 " is a unique factorization domain. By proposition 12.5, we only need to \
show that every non-invertible element can be expressed as a product of \
irreducible factors. In order to eliminate the posibility of an \"infinite \
chain\" of irreducible elements, each one dividing the previous, we will use \
the following lemma.\n\nLEMMA 12.7\nLet ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a principle ideal ring. If there is an infinite sequence of larger and \
larger ideals of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " satisfying\n\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["I", "1"], " ", "\[SubsetEqual]", " ", 
    SubscriptBox["I", "2"], " ", "\[SubsetEqual]", " ", 
    SubscriptBox["I", "3"], " ", "\[SubsetEqual]", " ", "\[CenterEllipsis]", 
    " ", "\[SubsetEqual]", " ", 
    SubscriptBox[
     StyleBox["I",
      FontSlant->"Italic"], "n"], " ", "\[SubsetEqual]", " ", 
    SubscriptBox["I", 
     RowBox[{"n", "+", "1"}]], " ", "\[SubsetEqual]", " ", 
    "\[CenterEllipsis]"}], TraditionalForm]]],
 ",\n           \nthen there exists an integer m such that",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubscriptBox["I", "n"], "=", " ", 
     SubscriptBox["I", "m"]}]}], TraditionalForm]]],
 "  for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", 
    RowBox[{"m", "."}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43790928759375*^9, 3.4379093020625*^9}, {
  3.4419822690264997`*^9, 3.4419822726358747`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since we have an infinite sequence of ideals, we can consider taking the \
union of all of them:\n                                    ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", "  ", "=", 
    StyleBox[" ", "Subsubtitle"], 
    OverscriptBox[
     UnderscriptBox[
      RowBox[{" ", 
       SubscriptBox[
        StyleBox["\[Union]",
         FontSize->24], " "]}], 
      StyleBox[
       RowBox[{"n", "=", "1"}],
       FontSize->10]], 
     StyleBox["\[Infinity]",
      FontSize->12]]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "n"], TraditionalForm]]],
 ".\n\nLet us show that ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " is an ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Note that any element of ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "k"], TraditionalForm]]],
 " for some integer ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ". In fact, if ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are two elements of ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", we can pick the larger of the two values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " to show that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are both in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "k"], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "\[PlusMinus]", " ", "y"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "k"], TraditionalForm]]],
 ", since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "k"], TraditionalForm]]],
 " is an ideal. Thus ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "\[PlusMinus]", " ", "y"}], TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ". This shows that ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " is a subgroup of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " under addition. Now let ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]]],
 " be in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "z"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", "\[CenterDot]", "x"}], TraditionalForm]]],
 " are both in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "k"], TraditionalForm]]],
 " , so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[CenterDot]", "z"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"z", "\[CenterDot]", "x"}], TraditionalForm]]],
 " are in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ". Therefore, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"I", "\[CenterDot]", "R"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"R", "\[CenterDot]", "I"}], " ", "=", " ", 
     RowBox[{"I", "."}]}]}], TraditionalForm]]],
 " This shows that ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " is an ideal. \n\nSince ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a principle ideal ring, there is some element ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", " ", "=", " ", 
    RowBox[{
     RowBox[{"(", "a", ")"}], "."}]}], TraditionalForm]]],
 " Then ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "m"], TraditionalForm]]],
 "  for some ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "m"], TraditionalForm]]],
 "  is contained in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", so we must have that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", " ", "=", " ", 
    RowBox[{
     SubscriptBox["I", "m"], "."}], " "}], TraditionalForm]]],
 " Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["I", "n"], "=", " ", 
    SubscriptBox["I", "m"]}], TraditionalForm]]],
 "  for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "m"}], TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "We now have all we need to show that a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 " is in fact a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ".\n\nTHEOREM 12.4:  THE PRINCIPLE IDEAL DOMAIN THEOREM\nEvery principle \
ideal domain is an unique factorization domain."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437909370265625*^9, 3.437909370875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Our strategy is to first show that an irreducible element is a prime \
element, and then show that every element is a finite product of irreduciable \
elements. Let ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " be an irreducible element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", which is a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"PID", "."}], TraditionalForm]]],
 " By lemma 12.6 ",
 Cell[BoxData[
  FormBox[
   RowBox[{"R", "/", 
    RowBox[{"(", "p", ")"}]}], TraditionalForm]]],
 " is a field, so it certainly has no zero divisors. Thus, by proposition \
12.7, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", "p", ")"}], TraditionalForm]]],
 " is a prime ideal, so by proposition 12.6, ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is prime. Let us now show that every non-invertible element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " can be written as a product of irreducible elements. Suppose this is not \
true for some element ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "0"], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "0"], TraditionalForm]]],
 " is not irreducible, so we can find elements ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["y", "1"], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["x", "1"], "\[CenterDot]", 
     SubscriptBox["y", "1"]}], " ", "=", " ", 
    SubscriptBox["x", 
     RowBox[{"0", " "}]]}], TraditionalForm]]],
 ". But ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["y", "1"], TraditionalForm]]],
 "cannot both be irreducible, so we can assume ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "1"], TraditionalForm]]],
 "is reducible. By induction we can continue this process to form a sequence\n\
\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"{", " ", 
     RowBox[{
      SubscriptBox["x", "0"], ",", " ", 
      SubscriptBox["x", "1"], ",", " ", 
      SubscriptBox["x", "2"], ",", " ", 
      SubscriptBox["x", "3"], ",", " ", "\[Ellipsis]"}], " ", "}"}]}], 
   TraditionalForm]]],
 "\n\nfor which each term in the sequence divides the previous term. Then we \
have an infinite chain of ideals   \n               \n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     SubscriptBox["x", "0"], ")"}], " ", "\[SubsetEqual]", " ", 
    RowBox[{"(", 
     SubscriptBox["x", "1"], ")"}], " ", "\[SubsetEqual]", " ", 
    RowBox[{"(", 
     SubscriptBox["x", "2"], ")"}], " ", "\[SubsetEqual]", " ", 
    RowBox[{"(", 
     SubscriptBox["x", "3"], ")"}], " ", "\[SubsetEqual]", " ", 
    "\[CenterEllipsis]"}], TraditionalForm]]],
 ".\n           \nBy lemma 12.7, there is a number ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     SubscriptBox["x", "n"], ")"}], " ", "=", " ", 
    RowBox[{"(", 
     SubscriptBox["x", "m"], ")"}], " "}], TraditionalForm]]],
 "for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", ">", " ", "m"}], TraditionalForm]]],
 ". But this contradicts the fact that all of the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "n"], TraditionalForm]]],
 " are reducible. Therefore every element of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " can be expressed as a product of irreducible elements. By proposition \
12.5, ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a unique factorization domain."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4379093820625*^9, 3.437909412046875*^9}, {
  3.43790953471875*^9, 3.4379095486875*^9}, {3.4419822976046247`*^9, 
  3.4419824037452497`*^9}}]
}, Open  ]],

Cell[TextData[{
 "This theorem reveals the most important use of principle ideal domains--it \
enables us to find unique factorization domains. For example, the integers \
\[DoubleStruckCapitalZ] was proven to be a PID from proposition 10.3, so we \
now can see that \[DoubleStruckCapitalZ] is a UFD, as result that was \
promised in section 1.3.\n\nIt should be noted that not all unique \
factorization domains are ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 "'s--in fact we discovered that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    RowBox[{"x", ",", "y"}], "]"}], TraditionalForm]]],
 " is not a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 ", even though it is a ",
 Cell[BoxData[
  FormBox["UFD", TraditionalForm]]],
 ". However, many of the important unique factorization domains are also \
principle ideal domains. In the next section we will find many more examples \
of principle ideal domains."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437746101921875*^9, 3.437746104484375*^9}, {
  3.4379095895625*^9, 3.437909660390625*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Euclidean Domains", "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We have already seen the importance of principle ideal domains to determine \
whether a ring is a unique factorization domain. However, we still have the \
problem of determining whether a given integral domain is a principle ideal \
domain. This can usually be done quite easily. For example, the integers ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " was proven to be a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 " from proposition 10.3. \n\nWe could use a very simular arguement to show \
that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 " for any field ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 ". If I is a non-trivial ideal of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", we can find a non-zero element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " with the lowest degree. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "x", ")"}], TraditionalForm]]],
 " is also in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", then by the division algorithm\n\n                      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"g", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "(", "x", ")"}], "\[CenterDot]", 
       RowBox[{"q", "(", "x", ")"}]}], " ", "+", " ", 
      RowBox[{"r", "(", "x", ")"}]}]}], ","}], TraditionalForm]]],
 "\n\nwith the degree of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " less then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "x", ")"}], "."}], TraditionalForm]]],
 " But ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "(", "x", ")"}], TraditionalForm]]],
 " would also be in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", and since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 " has least degree of all the non-zero elements in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", ","}], TraditionalForm]]],
 " we must have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"r", "(", "x", ")"}], " ", "=", " ", "0."}], TraditionalForm]]],
 " Therefore all elements of ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " are multiples of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", " ", "=", " ", 
    RowBox[{"(", 
     RowBox[{"f", "(", "x", ")"}], ")"}]}], TraditionalForm]]],
 ".\n\nRather than making this a formal proposition, we want to study this \
example, since we can prove that ",
 StyleBox["many",
  FontSlant->"Italic"],
 " different domains are ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 "'s the same way. There were two keys to the proof that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " was a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 ": the fact that every polynomial had a degree, and the division algorithm. \
Whenever we have an integral domain which has a property like a division \
algorithm, there is a good chance that we can use this division algorithm to \
prove that the ring is a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"PID", "."}], TraditionalForm]]],
 " Let us formulate what we mean by a \"division algorithm.\"\n\nDEFINITION \
12.12:\nAn integral domain ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is called an ",
 StyleBox["Euclidean domain",
  FontSlant->"Italic"],
 " it there is a function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", "(", "x", ")"}], TraditionalForm]]],
 " defined on the non-zero elements of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " such that the following three properties hold.\n\n   1. ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", "(", "x", ")"}], TraditionalForm]]],
 " is a non-negative integer for every \n      non-zero ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ".\n\n   2. Whenever both ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " are non-zero, \n       ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Mu]", "(", 
     RowBox[{"x", "\[CenterDot]", "y"}], ")"}], " ", "\[GreaterEqual]", " ", 
    RowBox[{"\[Mu]", "(", "x", ")"}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774611259375*^9, 3.437746118234375*^9}, {
  3.437910063*^9, 3.437910063125*^9}, {3.437910123078125*^9, 
  3.4379101399375*^9}}],

Cell[TextData[{
 "\n   3. For and ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " non-zero, there \n      exist elements ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " such that\n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", " ", "=", " ", 
     RowBox[{
      RowBox[{"q", "\[CenterDot]", "y"}], " ", "+", " ", "r"}]}], ","}], 
   TraditionalForm]]],
 "\n\n      where either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", " ", "=", " ", "0"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Mu]", "(", "r", ")"}], " ", "<", " ", 
    RowBox[{
     RowBox[{"\[Mu]", "(", "y", ")"}], "."}]}], TraditionalForm]]],
 "\n\nThe function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", "(", "x", ")"}], TraditionalForm]]],
 " is called the ",
 StyleBox["Euclidean valuation",
  FontSlant->"Italic"],
 " on ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ".\n\nLet us first look at some examples of Euclidean domains. Since this \
definition was modeled after the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 ", it is expected that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is a Euclidean domain. The function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", "(", 
    RowBox[{"f", "(", "x", ")"}], ")"}], TraditionalForm]]],
 " would be the degree of the polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "x", ")"}], TraditionalForm]]],
 ". Properties 1 and 2 come from the definition of the degree, and lemma \
11.1. Property 3 we observed in the division algorithm theorem (12.1). Thus, \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is a Euclidean domain whenever ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " is a field.\n\nHowever, there are many other examples of Euclidean \
domains. Consider the set of integers, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ". We can use the absolute value for the valuation: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Mu]", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"|", "x", "|", "."}]}], TraditionalForm]]],
 " Clearly properties 1 and 2 hold, and the third property comes from modular \
arithmetic. Thus, ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 " is also a Euclidean domain. \n\nWhenever we have a Euclidean domain, we \
can prove that the domain is a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 ", using the exact same argument as we did for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "[", "x", "]"}], "."}], TraditionalForm]]],
 "\n \nTHEOREM 12.5:  THE EUCLIDIAN DOMAIN THEOREM\nEvery Euclidean domain is \
a principle ideal domain."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774612303125*^9, 3.43774612559375*^9}, {
  3.43791014228125*^9, 3.437910142984375*^9}, {3.437910181890625*^9, 
  3.437910187671875*^9}, {3.437910241484375*^9, 3.437910242265625*^9}, {
  3.4419824505733747`*^9, 3.4419824537921247`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " be a Euclidean domain, and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", "(", "x", ")"}], TraditionalForm]]],
 " be the valuation. If ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " is an ideal, we consider the set\n\n                             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", " ", "=", " ", 
    RowBox[{"{", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Mu]", "(", "x", ")"}], " ", "|", " ", 
       RowBox[{"x", " ", "\[Epsilon]", " ", 
        StyleBox["I",
         FontSlant->"Italic"]}]}], 
      StyleBox[",",
       FontSlant->"Italic"], 
      StyleBox[" ",
       FontSlant->"Italic"], 
      RowBox[{
       StyleBox["x",
        FontSlant->"Italic"], "\[NotEqual]", "0"}]}], "}"}]}], 
   TraditionalForm]]],
 ".\n                             \nThe set ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 " consists of non-negative integers, so there is a smallest number in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Pick an element ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 " so that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", "(", "y", ")"}], TraditionalForm]]],
 " is the minimal number in ",
 Cell[BoxData[
  FormBox["P", TraditionalForm]]],
 ". Then for any other ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", we have \n\n                                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", 
    RowBox[{
     RowBox[{"y", "\[CenterDot]", "q"}], " ", "+", " ", "r"}]}], 
   TraditionalForm]]],
 "    \n\nfor some ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Mu]", "(", "r", ")"}], " ", "<", " ", 
    RowBox[{"\[Mu]", "(", "y", ")"}]}], TraditionalForm]]],
 ". Then ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", but if ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " was non-zero, then this would contradict the minimality of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", "(", "y", ")"}], TraditionalForm]]],
 ". Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", " ", "=", " ", "0"}], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is a multiple of ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ". Since this is true for all ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox["I", TraditionalForm]]],
 ", we see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"I", " ", "=", " ", 
    RowBox[{
     RowBox[{"(", "y", ")"}], "."}]}], TraditionalForm]]],
 " Thus, every ideal of ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a principal ideal, so ",
 Cell[BoxData[
  FormBox["R", TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"PID", "."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "We started this section by showing that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is a principle ideal ring whenever ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " is a field, but let us formally make this a corollary of the Euclidean \
domain theorem.\n\nCOROLLARY 12.7\nLet ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " be a field. Then the ring of polynomials ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is a principle ideal domain."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774612609375*^9, 3.437746127640625*^9}, {
  3.437910315546875*^9, 3.437910316140625*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "We have already seen that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is a Euclidean domain whenever ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " is a field. By the Euclidean domain theorem (12.5), ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "[", "x", "]"}], TraditionalForm]]],
 " is a ",
 Cell[BoxData[
  FormBox["PID", TraditionalForm]]],
 "."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43774612809375*^9, 3.4377461285625*^9}, {
  3.43791033278125*^9, 3.437910333484375*^9}}]
}, Open  ]],

Cell[TextData[{
 "The only problem with this definition of the Euclidean domain is that it \
gives no help in determining what the valuation function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", "(", "x", ")"}], TraditionalForm]]],
 " should be. In fact, there may many possible valuation functions for a \
given integral domain. (See problem 12.29 for an alternative definition of an \
Euclidean domain that does not involve a valuation function.) \n\nFor the \
remainder of this chapter, we will consider an interesting class of integral \
domains, some of which are Euclidean domains, and some are not. This class of \
domains will help us to see some general techniques for finding a valuation \
function for a domain.\n\nDEFINITION 12.13:\nLet ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " be an integer which is not divisible by the square of any ",
 Cell[BoxData[
  FormBox[
   RowBox[{"integer", " ", ">", " ", "1."}], TraditionalForm]]],
 " Then the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " is called a ",
 StyleBox["quadratic domain.",
  FontSlant->"Italic"],
 "\n\nWe have already worked with some examples of quadratic domains. For \
example, we found two possible ways to order the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox["2"], "]"}], ","}], TraditionalForm]]],
 " using ring homomorphisms. The quadratic domain ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " will always have two automorphisms, the identity mapping, and the \
automorphism\n\n               ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Florin]", "(", 
     RowBox[{"x", " ", "+", " ", 
      RowBox[{"y", " ", 
       RadicalBox["n", " "]}]}], ")"}], " ", "=", " ", 
    RowBox[{"x", " ", "-", " ", 
     RowBox[{"y", 
      RadicalBox["n", " "]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437910347578125*^9, 3.43791038534375*^9}}],

Cell[TextData[{
 "\nWe define the function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "a", ")"}], TraditionalForm]]],
 " as the product of the two automorphisms:\n \n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", 
     RowBox[{"x", "+", 
      RowBox[{"y", 
       RadicalBox["n", " "]}]}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", "+", 
        RowBox[{"y", 
         RadicalBox["n", " "]}]}], ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"x", "-", 
        RowBox[{"y", 
         RadicalBox["n", " "]}]}], ")"}]}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox["x", "2"], "-", " ", 
      RowBox[{"n", "\[CenterDot]", 
       SuperscriptBox["y", "2."]}]}]}]}], TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "It is interesting that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "a", ")"}], TraditionalForm]]],
 " is always an integer. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "At first glance it may be difficult to see what ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "a", ")"}], TraditionalForm]]],
 " has to do with the Euclidean domains. Our goal is to construct a valuation \
function from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "a", ")"}], TraditionalForm]]],
 ". We first need to verify some elementary properties of this function.  In \
the process, we will notice these properties are still valid if we extend ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "a", ")"}], TraditionalForm]]],
 " to be defined on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 ".\n\nLEMMA 12.8\nLet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], " ", "]"}], TraditionalForm]]],
 " be a quadratic domain, and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", 
     RowBox[{"x", " ", "+", " ", 
      RowBox[{"y", "  ", 
       SqrtBox["n"]}]}], ")"}], " ", "=", "  ", 
    RowBox[{
     SuperscriptBox["x", "2"], " ", "-", " ", 
     RowBox[{
      SuperscriptBox["y", "2"], " ", 
      RowBox[{"n", "."}]}]}]}], TraditionalForm]]],
 "  Then for the rings ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 ",\n\n    1. ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", "a", ")"}], " ", "=", " ", "0"}], TraditionalForm]]],
 " if, and only if, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", "0."}], TraditionalForm]]],
 "\n\n    2. ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", " ", 
     RowBox[{"(", 
      RowBox[{"a", "\[CenterDot]", "b"}], ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"N", "(", "a", ")"}], "\[CenterDot]", 
     RowBox[{"N", "(", "b", ")"}]}]}], TraditionalForm]]],
 "\n     \n    3. ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", 
     RowBox[{"\[PlusMinus]", "1"}], ")"}], " ", "=", " ", "1."}], 
   TraditionalForm]]]
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4379104320625*^9, 3.4379104654375*^9}, {
  3.4419824979171247`*^9, 3.4419826006046247`*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "1.  It is easy to see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", "0", ")"}], " ", "=", " ", "0"}], TraditionalForm]]],
 " by definition. If     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", 
     RowBox[{"x", " ", "+", " ", 
      RowBox[{"y", "  ", 
       SqrtBox["n"]}]}], " ", ")"}], " ", "=", " ", "0"}], 
   TraditionalForm]]],
 ", then\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "+", " ", 
        RowBox[{"y", " ", 
         FormBox[
          SqrtBox["n"],
          TraditionalForm]}]}], " ", ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        RowBox[{"y", "  ", 
         FormBox[
          SqrtBox["n"],
          TraditionalForm]}]}], " ", ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       FormBox[
        SuperscriptBox["x", "2"],
        TraditionalForm], " ", "-", " ", 
       RowBox[{
        FormBox[
         SuperscriptBox["y", "2"],
         TraditionalForm], " ", "n"}]}], " ", "=", " ", "0."}]}], " "}], 
   TraditionalForm]]],
 "\n \n   If ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " is non-zero, then we find that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SqrtBox["n"], "=", " ", 
     RowBox[{"|", 
      FractionBox["x", "y"], "|"}]}], " ", ","}], TraditionalForm]]],
 "  which is ridiculous since ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is not a perfect square, and so ",
 Cell[BoxData[
  FormBox[
   SqrtBox["n"], TraditionalForm]]],
 " is irrational. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "=", " ", "0"}], TraditionalForm]]],
 ", and hence ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is also 0. So ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", "a", ")"}], " ", "=", " ", "0"}], TraditionalForm]]],
 " if, and only if, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", "0"}], TraditionalForm]]],
 ".\n\n2) A quick computation shows that if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", 
    RowBox[{
     SubscriptBox["x", "1"], " ", "+", " ", 
     RowBox[{
      SubscriptBox["y", "1"], 
      FormBox[
       SqrtBox["n"],
       TraditionalForm]}]}]}], TraditionalForm]]],
 " , and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"b", " ", "=", " ", 
     RowBox[{
      SubscriptBox["x", "2"], "+", " ", 
      RowBox[{
       SubscriptBox["y", "2"], " ", 
       SqrtBox["n"]}]}]}], ","}], TraditionalForm]]],
 " then\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "b"}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["x", "1"], "+", 
        RowBox[{
         SubscriptBox["y", "1"], " ", 
         SqrtBox["n"]}]}], " ", ")"}], "\[CenterDot]", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["x", "2"], "+", 
        RowBox[{
         SubscriptBox["y", "2"], " ", 
         SqrtBox["n"]}]}], " ", ")"}]}], "=", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["x", "1"], "\[CenterDot]", " ", 
         SubscriptBox["x", "2"]}], "+", " ", 
        RowBox[{
         SubscriptBox["y", "1"], "\[CenterDot]", " ", 
         SubscriptBox["y", "2"], "\[CenterDot]", " ", "n"}]}], ")"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["x", "1"], "\[CenterDot]", 
          SubscriptBox["y", "2"]}], "+", " ", 
         RowBox[{
          SubscriptBox["y", "1"], "\[CenterDot]", 
          SubscriptBox["x", "2"]}]}], ")"}], 
       RowBox[{
        SqrtBox["n"], ".", " "}]}]}]}]}], TraditionalForm]]],
 "\n                                                          \n   So\n   \n\
",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{
      FormBox[
       RowBox[{
        RowBox[{"N", "(", 
         RowBox[{"a", "\[CenterDot]", "b"}], ")"}], " ", "=", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SubscriptBox["x", "1"], "\[CenterDot]", " ", 
           SubscriptBox["x", "2"]}], " ", "+", " ", 
          RowBox[{
           SubscriptBox["y", "1"], "\[CenterDot]", 
           SubscriptBox["y", "2"], "\[CenterDot]", "n"}]}]}]}],
       TraditionalForm], ")"}], "2"], " ", "-", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["x", "1"], "\[CenterDot]", 
         SubscriptBox["y", "2"]}], " ", "+", " ", 
        RowBox[{
         SubscriptBox["y", "1"], "\[CenterDot]", 
         SubscriptBox["x", "2"]}]}], ")"}], "2"], "\[CenterDot]", "n"}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"               ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox["x", "1", "2"], 
       SubsuperscriptBox["x", "2", "2"]}], "+", " ", 
      RowBox[{"2", " ", 
       SubscriptBox["x", "1"], 
       SubscriptBox["x", "2"], 
       SubscriptBox["y", "1"], 
       SubscriptBox["y", "2"], "n"}], " ", "+", " ", 
      RowBox[{
       SubsuperscriptBox["y", "1", "2"], " ", 
       SubsuperscriptBox["y", "2", "2"], 
       SuperscriptBox["n", "2"]}], "-", " ", 
      RowBox[{
       SubsuperscriptBox["x", "1", 
        RowBox[{"2", " "}]], 
       SubsuperscriptBox["y", "2", "2"], "n"}], " ", "-", " ", 
      RowBox[{"2", " ", 
       SubscriptBox["x", "1"], 
       SubscriptBox["x", "2"], 
       SubscriptBox["y", "1"], 
       SubscriptBox["y", "2"], "n"}], "-", 
      RowBox[{
       SubsuperscriptBox["y", "1", "2"], 
       SubsuperscriptBox["x", "2", "2"], "n"}]}]}]}], TraditionalForm], "\n", 
 
  FormBox[
   RowBox[{"               ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubsuperscriptBox["x", "1", "2"], 
        SubsuperscriptBox["x", "2", "2"]}], "+", " ", 
       RowBox[{
        SubsuperscriptBox["y", "1", "2"], 
        SubsuperscriptBox["y", "2", "2"], 
        SuperscriptBox["n", "2"]}], "-", " ", 
       RowBox[{
        SubsuperscriptBox["x", "1", "2"], 
        SubsuperscriptBox["y", "2", "2"], "n"}], " ", "-", " ", 
       RowBox[{
        SubsuperscriptBox["y", "1", "2"], 
        SubsuperscriptBox["x", "2", 
         RowBox[{"2", " "}]], "n"}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["x", "1", 
          RowBox[{"2", " "}]], "-", " ", 
         RowBox[{
          SubsuperscriptBox["y", "1", "2"], "n"}]}], ")"}], "\[CenterDot]", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["x", "2", "2"], "-", 
         RowBox[{
          SubsuperscriptBox["y", "2", "2"], "n"}]}], ")"}]}]}]}]}], 
   TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t\t     ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"N", 
       RowBox[{"(", "a", ")"}]}], "\[CenterDot]", 
      RowBox[{
       RowBox[{"N", 
        RowBox[{"(", "b", ")"}]}], "."}]}]}]}], TraditionalForm]}]],
 "\n\n3) This is easy, since",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[PlusMinus]", "1"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"\[PlusMinus]", "1"}], " ", "+", " ", 
      RowBox[{"0", "  ", 
       SqrtBox["n"]}]}]}]}], TraditionalForm]]],
 " . So ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", 
     RowBox[{"\[PlusMinus]", "1"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[PlusMinus]", "1"}], ")"}], "2"], " ", "-", " ", 
      RowBox[{"0", "\[CenterDot]", "n"}]}], " ", "=", " ", "1."}]}], 
   TraditionalForm]]],
 " \n  "
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437910474484375*^9, 3.437910528140625*^9}, {
  3.4419826209796247`*^9, 3.4419826241671247`*^9}}]
}, Open  ]],

Cell[TextData[{
 "We can use the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "a", ")"}], TraditionalForm]]],
 " function to prove that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " is a field.  "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4379105618125*^9, 3.437910578421875*^9}, {
  3.4419826734952497`*^9, 3.4419827194952497`*^9}}],

Cell[TextData[{
 "COROLLARY 12.8\nLet ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " be an integer which is not divisible by the square of any ",
 Cell[BoxData[
  FormBox[
   RowBox[{"integer", " ", ">", " ", "1."}], TraditionalForm]]],
 " Then the ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " is a field."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437910655703125*^9, 3.43791068078125*^9}, 
   3.4419827391671247`*^9, 3.4419827748389997`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " is obviously a commutative ring with an identity, all we need to show is \
that every non-zero element has an inverse. Let ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", 
    RowBox[{"x", " ", "+", " ", 
     RowBox[{"y", " ", 
      SqrtBox["n"]}]}]}], TraditionalForm]]],
 "  be a non-zero element. Then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "b", ")"}], TraditionalForm]]],
 " is non-zero by lemma 12.8. Consider the element\n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"c", " ", "=", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "-", " ", 
        RowBox[{"y", " ", 
         SqrtBox["n"]}]}], " ", ")"}], " ", "/", " ", 
      RowBox[{
       RowBox[{"N", "(", "b", ")"}], "."}]}]}]}], TraditionalForm]]],
 "\nThen\n           ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"b", "\[CenterDot]", "c"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "+", " ", 
         RowBox[{"y", " ", 
          SqrtBox["n"]}]}], " ", ")"}], "\[CenterDot]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"x", " ", "-", " ", 
          RowBox[{"y", " ", 
           SqrtBox["n"]}]}], " ", ")"}], " ", "/", " ", 
        RowBox[{"N", "(", "b", ")"}]}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"N", "(", "b", ")"}], "/", 
        RowBox[{"N", "(", "b", ")"}]}], " ", "=", " ", "1."}]}]}]}], 
   TraditionalForm]]],
 "\n\nSo every non-zero element has an inverse. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " is a field."
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.43791071015625*^9, 3.437910711265625*^9}}]
}, Open  ]],

Cell[TextData[{
 "Using these three properties of the norm function, we are able to determine \
at least some of the irreducible elements of the ring ",
 Cell[BoxData[
  FormBox["K", TraditionalForm]]],
 ". This reveals the connection between the norm, and the unique \
factorization properties of the ring.\n\nPROPOSITION 12.8\nLet ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " be a quadratic domain, and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", 
     RowBox[{"x", " ", "+", " ", 
      RowBox[{"y", "  ", 
       SqrtBox["n"]}]}], ")"}], " ", "=", "  ", 
    RowBox[{
     SuperscriptBox["x", "2"], " ", "-", " ", 
     RowBox[{
      SuperscriptBox["y", "2"], " ", 
      RowBox[{"n", "."}]}]}]}], TraditionalForm]]],
 "  Then \n     \n  1. ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", "a", ")"}], " ", "=", " ", 
    RowBox[{"\[PlusMinus]", "1"}]}], TraditionalForm]]],
 "  if, and only if, ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is a unit in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 ", \n\nand \n \n  2. If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "a", ")"}], TraditionalForm]]],
 " is a prime number in ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is an \n     irreducible element of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.4379105618125*^9, 3.437910578421875*^9}, {
   3.4419826734952497`*^9, 3.4419827194952497`*^9}, 3.4419827883233747`*^9}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", "a", ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"N", "(", 
      RowBox[{"x", " ", "+", " ", 
       RowBox[{"y", " ", 
        SqrtBox["n"]}]}], ")"}], " ", "=", " ", 
     RowBox[{"\[PlusMinus]", "1."}]}]}], TraditionalForm]]],
 " Consider the element \n\n             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "-", " ", 
       RowBox[{"y", " ", 
        SqrtBox["n"]}]}], " ", ")"}], "/", " ", 
     RowBox[{
      RowBox[{"N", "(", "a", ")"}], "."}]}]}], TraditionalForm]]],
 "\n\nThen ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"a", "\[CenterDot]", "b"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", " ", "+", " ", 
        RowBox[{"y", "  ", 
         SqrtBox["n"]}]}], ")"}], "\[CenterDot]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"x", " ", "-", " ", 
         RowBox[{"y", "  ", 
          SqrtBox["n"]}]}], " ", ")"}], "/", " ", 
       RowBox[{"N", "(", "a", ")"}]}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"N", "(", "a", ")"}], "/", 
       RowBox[{"N", "(", "a", ")"}]}], " ", "=", " ", "1."}]}]}], 
   TraditionalForm]]],
 " So ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " has an inverse, and therefore is a unit in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", " ", 
     SqrtBox["n"], " ", "]"}], "."}], TraditionalForm]]],
 "\n\nNow suppose that ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is a unit in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], " ", "]"}], TraditionalForm]]],
 ". Then a has an inverse, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", 
     RowBox[{"-", "1"}]], "."}], TraditionalForm]]],
 " Then \n\n        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", " ", "=", " ", 
     RowBox[{
      RowBox[{"N", "(", "1", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"N", "(", 
        RowBox[{"a", "\[CenterDot]", 
         SuperscriptBox["a", 
          RowBox[{"-", "1"}]]}], ")"}], " ", "=", " ", 
       RowBox[{
        RowBox[{"N", "(", "a", ")"}], "\[CenterDot]", 
        RowBox[{"N", "(", 
         SuperscriptBox["a", 
          RowBox[{"-", "1"}]], ")"}]}]}]}]}], ","}], TraditionalForm]]],
 "                           \n\nwhich shows that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "a", ")"}], TraditionalForm]]],
 " must be ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PlusMinus]", "1."}], TraditionalForm]]],
 "              \n\nNow suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", "a", ")"}], " ", "=", " ", "p"}], TraditionalForm]]],
 ", an prime number in ",
 Cell[BoxData[
  FormBox["\[DoubleStruckCapitalZ]", TraditionalForm]]],
 ", and that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "=", " ", 
    RowBox[{"b", "\[CenterDot]", "c"}]}], TraditionalForm]]],
 ". Then       \n\n                  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"p", " ", "=", " ", 
     RowBox[{
      RowBox[{"N", "(", "a", ")"}], " ", "=", " ", 
      RowBox[{
       RowBox[{"N", "(", 
        RowBox[{"b", "\[CenterDot]", "c"}], ")"}], " ", "=", " ", 
       RowBox[{
        RowBox[{"N", "(", "b", ")"}], "\[CenterDot]", 
        RowBox[{
         RowBox[{"N", "(", "c", ")"}], "."}]}]}]}]}]}], TraditionalForm]]],
 "\n                       \nSince ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is prime, either ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", "b", ")"}], " "}], TraditionalForm]]],
 "or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "c", ")"}], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PlusMinus]", "1"}], TraditionalForm]]],
 ". So either ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " must be a unit in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 ", so ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is irreducible in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox["n"], "]"}], "."}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False]
}, Open  ]],

Cell[TextData[{
 "We can now use the Euclidian function ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Mu]", "(", "x", ")"}], " ", "=", " ", 
    RowBox[{"|", 
     RowBox[{"N", "(", "x", ")"}], "|"}]}], TraditionalForm]]],
 " to prove the following:\n\nPROPOSITION 12.9       \nThe integral domains \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "2"}]], "]"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox[
     RowBox[{"-", "1"}]], "]"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["2"]}], TraditionalForm]]],
 "], and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["3"], "]"}], TraditionalForm]]],
 " are Euclidian domains."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437910725796875*^9, 3.437910734921875*^9}}],

Cell[CellGroupData[{

Cell["Proof:", "Subsubsection",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell[TextData[{
 "Let us work with all four domains at the same time by considering ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox["n"], "]"}], ","}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", " ", "=", " ", 
     RowBox[{"-", "2"}]}], ",", " ", 
    RowBox[{"-", "1"}], ",", " ", "2", ",", " ", 
    RowBox[{"or", " ", "3"}]}], TraditionalForm]]],
 ". \n\nIf we let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Mu]", "(", "x", ")"}], " ", "=", " ", 
     RowBox[{"|", 
      RowBox[{"N", "(", "x", ")"}], "|"}]}], ","}], TraditionalForm]]],
 " then clearly ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", "(", "x", ")"}], TraditionalForm]]],
 " is a non-negative integer. Furthermore,",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"\[Mu]", "(", "x", ")"}], " ", "=", " ", "0"}]}], 
   TraditionalForm]]],
 " only when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", "=", " ", "0"}], TraditionalForm]]],
 ". Thus, if ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " are two elements of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 ", then\n \n",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Mu]", "(", 
       RowBox[{"u", "\[CenterDot]", "v"}], ")"}], " ", "=", " ", 
      RowBox[{"|", 
       RowBox[{"N", "(", 
        RowBox[{"u", "\[CenterDot]", "v"}], ")"}], "|"}]}], "\n", 
     "\t\t\t    ", 
     RowBox[{"=", 
      RowBox[{"|", 
       RowBox[{"N", "(", "u", ")"}], "|", 
       RowBox[{"\[CenterDot]", 
        RowBox[{"|", 
         RowBox[{"N", "(", "v", ")"}], "|"}]}]}]}], " \t              ", "\n",
      "\t\t\t\t ", 
     RowBox[{"=", "  ", 
      RowBox[{
       RowBox[{"\[Mu]", "(", "u", ")"}], "\[CenterDot]", 
       RowBox[{"\[Mu]", "(", "v", ")"}]}]}], "\n", "\t\t\t\t", 
     RowBox[{
      RowBox[{"\[LessEqual]", " ", 
       RowBox[{
        RowBox[{"\[Mu]", "(", "u", ")"}], "\[CenterDot]", "1"}]}], "  ", "=", 
      " ", 
      RowBox[{
       RowBox[{"\[Mu]", "(", "u", ")"}], "."}]}], " "}]}], 
   TraditionalForm]]],
 "\n\nSo the first two conditions for the valuation function are easily \
satisfied. The last condition is harder to prove. We need to show that for \
any ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox["n"], "]"}]}], TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " non-zero, there are elements ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " such that \n\n                              ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"x", " ", "=", " ", 
      RowBox[{
       RowBox[{"q", "\[CenterDot]", "y"}], " ", "+", " ", "r"}]}], ","}]}], 
   TraditionalForm]]],
 "\n                              \nwith either ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", " ", "=", " ", "0"}], TraditionalForm]]],
 ", or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Mu]", "(", "r", ")"}], " ", "<", " ", 
    RowBox[{"\[Mu]", "(", "y", ")"}]}], TraditionalForm]]],
 ". We can consider ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " to be in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalQ]", "[", 
     SqrtBox["n"], "]"}], ","}], TraditionalForm]]],
 " which is a field from corollary 12.8, so we can compute\n\n                \
   ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"t", " ", "=", " ", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", 
       SuperscriptBox["y", 
        RowBox[{"-", "1"}]]}], " ", "=", " ", 
      RowBox[{"u", " ", "+", " ", 
       RowBox[{"v", " ", 
        RowBox[{
         SqrtBox["n"], "."}]}]}]}]}]}], TraditionalForm]]],
 "\n\nOf course, ",
 Cell[BoxData[
  FormBox["t", TraditionalForm]]],
 " will be in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " instead of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 ", so we cannot use this for our ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ". However, we can find an element \"closest\" to ",
 Cell[BoxData[
  FormBox["t", TraditionalForm]]],
 " in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " by finding the integers ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " nearest to ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 ". That is, we will select integers ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " \nand ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " such that\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{"(*", ")"}], TraditionalForm]]],
 "                     ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", 
    RowBox[{"p", "-", "u"}], "|", 
    RowBox[{"\[LessEqual]", " ", 
     FractionBox[
      RowBox[{"  ", "1"}], "2"]}]}], TraditionalForm]]],
 "  and  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"|", 
    RowBox[{"k", "-", "v"}], "|", 
    RowBox[{"\[LessEqual]", " ", 
     FractionBox[
      RowBox[{" ", "1"}], 
      RowBox[{" ", "2"}]]}]}], TraditionalForm]]],
 ". \n\nWe now let",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"q", " ", "=", " ", 
     RowBox[{"p", " ", "+", " ", 
      RowBox[{"k", " ", 
       SqrtBox["n"]}]}]}]}], TraditionalForm]]],
 " , which is in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 ". The remainder ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " would be given by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"q", "\[CenterDot]", "y"}], " ", "-", " ", "x"}], 
   TraditionalForm]]],
 ". All we need to do is show that",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"r", " ", "=", " ", "0"}]}], TraditionalForm]]],
 ", or ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Mu]", "(", "r", ")"}], " ", "<", " ", 
    RowBox[{
     RowBox[{"\[Mu]", "(", "y", ")"}], "."}]}], TraditionalForm]]],
 "\n\nNow, the norm ",
 Cell[BoxData[
  FormBox[
   RowBox[{"N", "(", "x", ")"}], TraditionalForm]]],
 " is valid on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalQ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 ", so we can compute\n\n      ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"N", "(", 
     RowBox[{"q", " ", "-", " ", "t"}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"N", "(", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"p", " ", "-", " ", "u"}], ")"}], " ", "+", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"k", " ", "-", " ", "v"}], ")"}], "  ", 
        SqrtBox["n"]}]}], " ", ")"}], " ", "=", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"p", "-", "u"}], ")"}], "2"], " ", "-", " ", 
      RowBox[{"n", " ", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"k", "-", "v"}], ")"}], "2"], "."}]}]}]}]}], 
   TraditionalForm]]],
 "\n\nBy ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(*", 
    RowBox[{")", ","}]}], TraditionalForm]]],
 " we see that if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"n", " ", ">", " ", "0"}], ","}], " "}], TraditionalForm]]],
 "\n\n                              ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "n"}], "/", "4"}], " ", "\[LessEqual]", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"p", "-", "u"}], ")"}], "2"], "-", " ", 
     RowBox[{"n", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", "v"}], ")"}], "2"]}]}], " ", "\[LessEqual]", " ", 
    RowBox[{"1", "/", "4"}]}], TraditionalForm]]],
 ".\n                              \nOn the other hand, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "<", " ", "0"}], TraditionalForm]]],
 ", then \n\n                             ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", " ", "\[LessEqual]", " ", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"p", "-", "u"}], ")"}], "2"], "-", 
     RowBox[{"n", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", "v"}], ")"}], "2"]}]}], " ", "\[LessEqual]", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "n"}], ")"}], "/", "4"}]}], TraditionalForm]]],
 ".\n                             \nThus, as long as  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", "\[LessEqual]", " ", "n", " ", "\[LessEqual]", 
     " ", "3"}], ","}], TraditionalForm]]],
 " we have that \n\n                        ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"|", 
     RowBox[{"N", "(", 
      RowBox[{"q", " ", "-", " ", "t"}], ")"}], "|"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"|", 
        RowBox[{"(", 
         RowBox[{"p", "-", "u"}], ")"}]}], "2"], " ", "-", " ", 
      RowBox[{"n", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"k", "-", "v"}], ")"}], "2"]}]}], "|", " ", 
     RowBox[{"\[LessEqual]", 
      FractionBox["3", "4"], "<", " ", "1."}]}]}], TraditionalForm]]],
 "\n\nThus, \n             ",
 Cell[BoxData[{
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Mu]", "(", "r", ")"}], " ", "=", " ", 
     RowBox[{
      RowBox[{"|", 
       RowBox[{"N", "(", "r", ")"}], "|"}], " ", "=", " ", 
      RowBox[{"|", 
       RowBox[{"N", "(", 
        RowBox[{
         RowBox[{"q", "\[CenterDot]", "y"}], " ", "-", " ", "x"}], ")"}], 
       "|"}]}]}], " "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t  ", 
    RowBox[{"=", " ", 
     RowBox[{"|", 
      RowBox[{"N", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"q", " ", "-", " ", 
          RowBox[{"x", "\[CenterDot]", 
           SuperscriptBox["y", 
            RowBox[{"-", "1"}]]}]}], ")"}], "\[CenterDot]", "y"}], ")"}], 
      "|"}]}], " "}], TraditionalForm], "\n", 
  FormBox[
   RowBox[{"\t\t  ", 
    RowBox[{"=", " ", 
     RowBox[{
      RowBox[{"|", 
       RowBox[{"N", "(", 
        RowBox[{"q", "-", "t"}], ")"}], "|", 
       RowBox[{"\[CenterDot]", 
        RowBox[{"|", 
         RowBox[{"N", "(", "y", ")"}], "|", " ", "\n", "\t\t  ", 
         RowBox[{"<", " ", 
          RowBox[{"|", 
           RowBox[{"N", "(", "y", ")"}], "|"}]}]}]}]}], " ", "=", " ", 
      RowBox[{
       RowBox[{"\[Micro]", "(", "y", ")"}], "."}]}]}]}], TraditionalForm]}]],
 "\n\nTherefore, the function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", "(", "x", ")"}], TraditionalForm]]],
 " serves as an valuation function on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 ", and so ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["n"], "]"}], TraditionalForm]]],
 " is a Euclidean domain for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", " ", "=", " ", 
     RowBox[{"-", "2"}]}], ",", " ", 
    RowBox[{"-", "1"}], ",", " ", "2", ",", " ", 
    RowBox[{"and", " ", "3."}]}], TraditionalForm]]]
}], "SmallText",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437910778953125*^9, 3.437910781890625*^9}, {
  3.4419829600421247`*^9, 3.4419829869171247`*^9}}]
}, Open  ]],

Cell[TextData[{
 "One of these four domains has special applications. The ring ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox[
      RowBox[{"-", "1"}]], "]"}], " ", "=", " ", 
    RowBox[{"\[DoubleStruckCapitalZ]", "[", "i", "]"}]}], TraditionalForm]]],
 " is called the domain of ",
 StyleBox["Gaussian integers",
  FontSlant->"Italic"],
 ".  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s FactorInteger command can find the prime factorization over the Gaussian \
integers by setting a condition ",
 StyleBox["GaussianIntegers",
  FontWeight->"Bold"],
 " to true. For example, we can factor the number 5 as follows:"
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437910910265625*^9, 3.437910917734375*^9}}],

Cell[CellGroupData[{

Cell["FactorInteger[5, GaussianIntegers -> True] ", "Input",
 PageWidth->Infinity],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "\[ImaginaryI]"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "+", 
      RowBox[{"2", " ", "\[ImaginaryI]"}]}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"2", "+", "\[ImaginaryI]"}], ",", "1"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.442755299523875*^9}]
}, Open  ]],

Cell[TextData[{
 "This reveals that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"5", " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "I"}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"1", " ", "+", " ", 
       RowBox[{"2", " ", "I"}]}], ")"}], "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"2", " ", "+", " ", "I"}], ")"}]}]}], TraditionalForm]]],
 ". The first of these is a unit, but the other two factors are apparently \
both prime.\n\nEXPERIMENT:\nReplace the 5 with different numbers to see the \
prime factorization over this new domain. Try this with complex numbers as \
well. See if you can find a pattern on which Gaussian integers are prime. "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False],

Cell["  ", "Input",
 PageWidth->Infinity],

Cell[TextData[{
 "By investigating further the divisibility properties of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", "i", "]"}], TraditionalForm]]],
 ", one can prove the classic \"two squares theorem\" of Fermat: every prime \
number of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", " ", "n"}], " ", "+", " ", "1"}], TraditionalForm]]],
 " is the sum of two squares. (See problem 13.18.) It is interesting that the \
study of domains other than the familiar integers yields new information \
about the integers."
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437911148875*^9, 3.437911149296875*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Problems"
}], "Section",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{{3.437911155125*^9, 3.437911157578125*^9}}],

Cell["", "Input",
 PageWidth->Infinity,
 FontFamily->"System",
 FontSize->12,
 FontWeight->"Bold",
 FontColor->RGBColor[1, 0, 0],
 Background->GrayLevel[1]],

Cell[TextData[{
 "12.1 ) Use the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command \n     ",
 StyleBox["InterpolatingPolynomial",
  FontWeight->"Bold"],
 " to find a third \n     degree polynomial such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "n", ")"}], " ", "=", " ", 
    RowBox[{"n", "!"}]}], TraditionalForm]]],
 " \n     for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", " ", "=", " ", "1"}], ",", " ", "2", ",", " ", "3", ",", " ",
     
    RowBox[{"and", " ", "4"}]}], TraditionalForm]]],
 ".  How close is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "5", ")"}], TraditionalForm]]],
 " \n     to 120? "
}], "Text",
 Evaluatable->False,
 CellHorizontalScrolling->False,
 CellChangeTimes->{3.437911178734375*^9}],

Cell[BoxData["   "], "Input"],

Cell[TextData[{
 "\n12.2 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to determine whether\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "3"], "+", 
    RowBox[{"2", " ", 
     SuperscriptBox["x", "2"]}], "+", 
    RowBox[{"3", " ", "x"}], "+", "2"}], TraditionalForm]]],
 "  is irreducible over ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "5"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellChangeTimes->{{3.437746132671875*^9, 3.437746133453125*^9}}],

Cell["    ", "Input"],

Cell[TextData[{
 "12.3 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to determine whether \n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "3"], "+", 
    RowBox[{"2", " ", 
     SuperscriptBox["x", "2"]}], "+", " ", 
    RowBox[{"3", " ", "x"}], "+", " ", "5"}], TraditionalForm]]],
 "  is irreducible over ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Z", "7"], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False,
 CellChangeTimes->{{3.437746134078125*^9, 3.437746134828125*^9}, {
  3.43791118803125*^9, 3.437911190796875*^9}}],

Cell["   ", "Input"],

Cell[TextData[{
 "12.4 ) Define the domain ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["6"], "]"}], TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " \n     as follows:"
}], "Text",
 Evaluatable->False,
 CellChangeTimes->{3.43791120053125*^9}],

Cell["\<\
InitDomain[0];
Define[a^6, 6] \
\>", "Input"],

Cell[TextData[{
 "     Show that the element ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", " ", "=", " ", 
    RowBox[{"5", " ", "+", " ", 
     RowBox[{"2", " ", "a"}]}]}], TraditionalForm]]],
 " is a unit \n     by finding its inverse. Use the element ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]]],
 " \n     to find yet another unit of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["6"], "]"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False],

Cell["          ", "Input"],

Cell[TextData[{
 "                                            \n12.5 ) Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to show that the ring\n     ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[DoubleStruckCapitalZ]", "[", 
     SqrtBox["6"], "]"}], "/", 
    RowBox[{"(", "11", ")"}]}], TraditionalForm]]],
 " has no zero divisors. Use \n     this to prove that 11 is a prime element \
\n     of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", 
    SqrtBox["6"], "]"}], TraditionalForm]]],
 "."
}], "Text",
 Evaluatable->False],

Cell["     ", "Input"],

Cell[TextData[{
 "12.6 ) Use the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command"
}], "Text",
 Evaluatable->False,
 CellChangeTimes->{3.4379112331875*^9}],

Cell["FactorInteger[2, GaussianIntegers -> True]", "Input"],

Cell[TextData[{
 "     to determine whether 2 is prime in the \n     domain ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", "i", "]"}], TraditionalForm]]],
 ". Try this using the numbers 3, \n     5, 7, 11, 13, 19, 23, 29, and 31 in \
place \n     of 2. Which of these numbers are prime in\n     the domain ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[DoubleStruckCapitalZ]", "[", "i", "]"}], TraditionalForm]]],
 "?"
}], "Text",
 Evaluatable->False],

Cell["   ", "Input"]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{520, 644},
WindowMargins->{{75, Automatic}, {6, Automatic}},
DockedCells->(FrontEndExecute[{
   FrontEnd`NotebookApply[
    FrontEnd`InputNotebook[], #, Placeholder]}]& ),
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.5,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (June 19, 2007)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[568, 21, 318, 9, 285, "Special2",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[911, 34, 285, 8, 29, "Name",
 Evaluatable->False],
Cell[1199, 44, 710, 16, 70, "Name",
 Evaluatable->False]
}, Closed]],
Cell[1924, 63, 266, 12, 142, "Title",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[2215, 79, 104, 3, 32, "Special2",
 Evaluatable->False],
Cell[2322, 84, 170, 4, 43, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[2529, 93, 101, 2, 105, "Section",
 Evaluatable->False],
Cell[2633, 97, 2966, 93, 617, "Text",
 Evaluatable->False],
Cell[5602, 192, 2872, 101, 309, "Text",
 Evaluatable->False],
Cell[8477, 295, 1930, 66, 296, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[10432, 365, 216, 6, 66, "Input",
 PageBreakAbove->True],
Cell[10651, 373, 177, 5, 60, "Output"]
}, Open  ]],
Cell[10843, 381, 74, 2, 41, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[10942, 387, 193, 5, 66, "Input"],
Cell[11138, 394, 203, 6, 60, "Output"]
}, Open  ]],
Cell[11356, 403, 411, 10, 164, "Text",
 Evaluatable->False],
Cell[11770, 415, 43, 1, 43, "Input"],
Cell[11816, 418, 2104, 74, 384, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[13945, 496, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[14033, 500, 12205, 434, 1295, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[26253, 937, 1418, 55, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[27696, 996, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[27784, 1000, 3193, 114, 362, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[30992, 1117, 1499, 46, 364, "Text",
 Evaluatable->False],
Cell[32494, 1165, 1011, 38, 119, "Text",
 Evaluatable->False],
Cell[33508, 1205, 603, 20, 94, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[34136, 1229, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[34224, 1233, 6749, 230, 581, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[40988, 1466, 1252, 41, 223, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[42265, 1511, 259, 7, 43, "Input"],
Cell[42527, 1520, 69, 1, 42, "Output"]
}, Open  ]],
Cell[42611, 1524, 272, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[42908, 1536, 249, 7, 43, "Input"],
Cell[43160, 1545, 71, 1, 42, "Output"]
}, Open  ]],
Cell[43246, 1549, 404, 12, 188, "Text",
 Evaluatable->False],
Cell[43653, 1563, 42, 1, 43, "Input"],
Cell[43698, 1566, 2783, 101, 619, "Text",
 Evaluatable->False],
Cell[46484, 1669, 42, 1, 43, "Input"],
Cell[46529, 1672, 2271, 74, 435, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[48825, 1750, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[48913, 1754, 6140, 220, 554, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[55068, 1977, 472, 17, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[55565, 1998, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[55653, 2002, 2844, 95, 310, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[58512, 2100, 1583, 48, 390, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[60120, 2152, 138, 2, 43, "Input"],
Cell[60261, 2156, 68, 1, 42, "Output"]
}, Open  ]],
Cell[60344, 2160, 309, 10, 66, "Text",
 Evaluatable->False],
Cell[60656, 2172, 42, 1, 43, "Input"],
Cell[60701, 2175, 2186, 71, 308, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[62912, 2250, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[63000, 2254, 6767, 233, 627, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[69782, 2490, 1403, 43, 384, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[71210, 2537, 130, 2, 43, "Input"],
Cell[71343, 2541, 185, 6, 42, "Output"]
}, Open  ]],
Cell[71543, 2550, 1329, 48, 237, "Text",
 Evaluatable->False],
Cell[72875, 2600, 42, 1, 43, "Input"],
Cell[72920, 2603, 1856, 65, 435, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[74801, 2672, 96, 2, 43, "Input"],
Cell[74900, 2676, 113, 3, 45, "Output"]
}, Open  ]],
Cell[75028, 2682, 633, 20, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[75686, 2706, 148, 3, 43, "Input",
 PageBreakAbove->True],
Cell[75837, 2711, 116, 3, 45, "Output"]
}, Open  ]],
Cell[75968, 2717, 1883, 60, 484, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[77876, 2781, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[77964, 2785, 2617, 93, 332, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[80596, 2881, 750, 20, 239, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[81371, 2905, 159, 3, 43, "Input",
 PageBreakAbove->True],
Cell[81533, 2910, 67, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[81637, 2916, 135, 2, 43, "Input"],
Cell[81775, 2920, 70, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[81882, 2926, 133, 2, 43, "Input"],
Cell[82018, 2930, 69, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82124, 2936, 131, 2, 43, "Input"],
Cell[82258, 2940, 68, 1, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82363, 2946, 133, 2, 43, "Input"],
Cell[82499, 2950, 70, 1, 42, "Output"]
}, Open  ]],
Cell[82584, 2954, 433, 13, 90, "Text",
 Evaluatable->False],
Cell[83020, 2969, 156, 5, 90, "Text",
 Evaluatable->False],
Cell[83179, 2976, 43, 1, 43, "Input"],
Cell[83225, 2979, 454, 15, 92, "Text",
 Evaluatable->False],
Cell[83682, 2996, 42, 1, 43, "Input"],
Cell[83727, 2999, 501, 14, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[84253, 3017, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[84341, 3021, 1792, 62, 198, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[86148, 3086, 372, 7, 139, "Text",
 Evaluatable->False],
Cell[86523, 3095, 2074, 73, 433, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[88622, 3172, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[88710, 3176, 6257, 234, 732, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[94982, 3413, 697, 21, 286, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[95704, 3438, 136, 2, 43, "Input"],
Cell[95843, 3442, 110, 3, 42, "Output"]
}, Open  ]],
Cell[95968, 3448, 965, 32, 121, "Text",
 Evaluatable->False],
Cell[96936, 3482, 438, 14, 90, "Text",
 Evaluatable->False],
Cell[97377, 3498, 44, 1, 43, "Input"],
Cell[97424, 3501, 1211, 47, 139, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[98660, 3552, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[98748, 3556, 5782, 197, 616, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[104545, 3756, 2740, 83, 577, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[107322, 3844, 101, 2, 105, "Section",
 Evaluatable->False],
Cell[107426, 3848, 2640, 84, 727, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[110091, 3936, 240, 9, 156, "Input"],
Cell[110334, 3947, 345, 10, 68, "Output"],
Cell[110682, 3959, 11664, 342, 378, "Output"]
}, Open  ]],
Cell[122361, 4304, 257, 8, 139, "Text",
 Evaluatable->False],
Cell[122621, 4314, 41, 1, 43, "Input"],
Cell[122665, 4317, 2000, 58, 458, "Text",
 Evaluatable->False],
Cell[124668, 4377, 40, 1, 43, "Input"],
Cell[124711, 4380, 1616, 52, 362, "Text",
 Evaluatable->False],
Cell[126330, 4434, 2369, 78, 409, "Text",
 Evaluatable->False],
Cell[128702, 4514, 1785, 51, 580, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[130512, 4569, 240, 9, 156, "Input"],
Cell[130755, 4580, 343, 10, 68, "Output"],
Cell[131101, 4592, 11667, 342, 378, "Output"]
}, Open  ]],
Cell[142783, 4937, 197, 5, 90, "Text",
 Evaluatable->False],
Cell[142983, 4944, 228, 6, 115, "Text",
 Evaluatable->False],
Cell[143214, 4952, 43, 1, 43, "Input"],
Cell[143260, 4955, 610, 19, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[143895, 4978, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[143983, 4982, 2170, 78, 321, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[146168, 5063, 1664, 60, 295, "Text",
 Evaluatable->False],
Cell[147835, 5125, 4474, 155, 796, "Text",
 Evaluatable->False,
 PageBreakAbove->True],
Cell[152312, 5282, 583, 14, 262, "Text",
 Evaluatable->False],
Cell[152898, 5298, 332, 9, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[153255, 5311, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[153343, 5315, 10799, 340, 1551, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[164157, 5658, 785, 22, 237, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[164967, 5684, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[165055, 5688, 599, 16, 112, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[165669, 5707, 1531, 44, 311, "Text",
 Evaluatable->False],
Cell[167203, 5753, 520, 17, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[167748, 5774, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[167836, 5778, 6616, 240, 655, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[174467, 6021, 2708, 93, 507, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[177200, 6118, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[177288, 6122, 2712, 100, 301, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[180015, 6225, 2171, 79, 409, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[182211, 6308, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[182299, 6312, 6296, 229, 881, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[188610, 6544, 1507, 52, 306, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[190142, 6600, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[190230, 6604, 7392, 273, 790, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[197637, 6880, 765, 22, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[198427, 6906, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[198515, 6910, 12552, 422, 1309, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[211082, 7335, 5735, 190, 981, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[216842, 7529, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[216930, 7533, 2085, 67, 204, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[219030, 7603, 712, 22, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[219767, 7629, 164, 5, 66, "Input"],
Cell[219934, 7636, 316, 11, 48, "Output"]
}, Open  ]],
Cell[220265, 7650, 1091, 25, 435, "Text",
 Evaluatable->False],
Cell[221359, 7677, 42, 1, 43, "Input"],
Cell[221404, 7680, 444, 12, 139, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[221885, 7697, 96, 2, 105, "Section",
 Evaluatable->False],
Cell[221984, 7701, 4701, 143, 1315, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[226710, 7848, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[226798, 7852, 3685, 149, 321, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[230498, 8004, 1264, 36, 237, "Text",
 Evaluatable->False],
Cell[231765, 8042, 2340, 85, 409, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[234130, 8131, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[234218, 8135, 3571, 132, 444, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[237804, 8270, 185, 5, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[238014, 8279, 225, 8, 133, "Input"],
Cell[238242, 8289, 367, 10, 68, "Output"]
}, Open  ]],
Cell[238624, 8302, 621, 21, 164, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[239270, 8327, 60, 1, 43, "Input"],
Cell[239333, 8330, 229, 6, 42, "Output"]
}, Open  ]],
Cell[239577, 8339, 151, 5, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[239753, 8348, 53, 1, 43, "Input"],
Cell[239809, 8351, 458, 14, 68, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[240304, 8370, 223, 6, 43, "Input"],
Cell[240530, 8378, 286, 8, 42, "Output"]
}, Open  ]],
Cell[240831, 8389, 240, 6, 115, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[241096, 8399, 145, 5, 66, "Input"],
Cell[241244, 8406, 73, 1, 42, "Output"],
Cell[241320, 8409, 1508, 41, 309, "Output"]
}, Open  ]],
Cell[242843, 8453, 558, 18, 164, "Text",
 Evaluatable->False],
Cell[243404, 8473, 44, 1, 43, "Input"],
Cell[243451, 8476, 1450, 41, 467, "Text",
 Evaluatable->False],
Cell[244904, 8519, 52, 1, 43, "Input"],
Cell[244959, 8522, 592, 25, 71, "Text",
 Evaluatable->False],
Cell[245554, 8549, 53, 1, 43, "Input"],
Cell[245610, 8552, 205, 7, 66, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[245840, 8563, 54, 1, 43, "Input"],
Cell[245897, 8566, 354, 10, 68, "Output"]
}, Open  ]],
Cell[246266, 8579, 227, 5, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[246518, 8588, 51, 1, 43, "Input"],
Cell[246572, 8591, 14613, 424, 373, "Output"]
}, Open  ]],
Cell[261200, 9018, 576, 17, 171, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[261801, 9039, 56, 1, 43, "Input"],
Cell[261860, 9042, 216, 5, 32, "Print"],
Cell[262079, 9049, 552, 16, 85, "Print"],
Cell[262634, 9067, 259, 7, 32, "Print"],
Cell[262896, 9076, 261, 7, 32, "Print"],
Cell[263160, 9085, 287, 8, 32, "Print"],
Cell[263450, 9095, 288, 8, 32, "Print"],
Cell[263741, 9105, 124, 2, 58, "Print"],
Cell[263868, 9109, 812, 23, 82, "Print"]
}, Open  ]],
Cell[264695, 9135, 2334, 79, 590, "Text",
 Evaluatable->False],
Cell[267032, 9216, 314, 12, 73, "Text",
 Evaluatable->False],
Cell[267349, 9230, 76, 4, 66, "Input"],
Cell[267428, 9236, 1143, 39, 251, "Text",
 Evaluatable->False],
Cell[268574, 9277, 62, 1, 43, "Input"],
Cell[268639, 9280, 1377, 47, 397, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[270041, 9331, 52, 1, 43, "Input"],
Cell[270096, 9334, 422, 8, 93, "Output"]
}, Open  ]],
Cell[270533, 9345, 266, 8, 90, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[270824, 9357, 56, 1, 43, "Input"],
Cell[270883, 9360, 70, 1, 32, "Print"]
}, Open  ]],
Cell[270968, 9364, 669, 22, 146, "Text",
 Evaluatable->False],
Cell[271640, 9388, 753, 24, 174, "Text",
 Evaluatable->False],
Cell[272396, 9414, 40, 1, 43, "Input"],
Cell[272439, 9417, 1087, 31, 335, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[273551, 9452, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[273639, 9456, 4524, 168, 568, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[278178, 9627, 1852, 48, 482, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[280055, 9679, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[280143, 9683, 4345, 176, 422, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[284503, 9862, 436, 12, 188, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[284964, 9878, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[285052, 9882, 3939, 128, 566, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[289006, 10013, 1141, 27, 360, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[290184, 10045, 90, 2, 105, "Section",
 Evaluatable->False],
Cell[290277, 10049, 4801, 156, 1119, "Text",
 Evaluatable->False],
Cell[295081, 10207, 3454, 109, 874, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[298560, 10320, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[298648, 10324, 3244, 124, 362, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[301907, 10451, 737, 21, 213, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[302669, 10476, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[302757, 10480, 599, 20, 75, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[303371, 10503, 2136, 55, 731, "Text",
 Evaluatable->False],
Cell[305510, 10560, 843, 29, 177, "Text",
 Evaluatable->False],
Cell[306356, 10591, 215, 8, 41, "Text",
 Evaluatable->False],
Cell[306574, 10601, 2357, 78, 486, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[308956, 10683, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[309044, 10687, 7840, 266, 691, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[316899, 10956, 459, 15, 73, "Text",
 Evaluatable->False],
Cell[317361, 10973, 579, 18, 122, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[317965, 10995, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[318053, 10999, 1949, 62, 303, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[320017, 11064, 1838, 58, 461, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[321880, 11126, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[321968, 11130, 4447, 158, 537, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[326430, 11291, 995, 35, 178, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[327450, 11330, 85, 2, 42, "Subsubsection",
 Evaluatable->False],
Cell[327538, 11334, 11881, 417, 1438, "SmallText",
 Evaluatable->False]
}, Open  ]],
Cell[339434, 11754, 813, 23, 195, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[340272, 11781, 82, 1, 43, "Input"],
Cell[340357, 11784, 411, 14, 42, "Output"]
}, Open  ]],
Cell[340783, 11801, 755, 19, 237, "Text",
 Evaluatable->False],
Cell[341541, 11822, 41, 1, 43, "Input"],
Cell[341585, 11825, 692, 17, 188, "Text",
 Evaluatable->False]
}, Open  ]],
Cell[CellGroupData[{
Cell[342314, 11847, 209, 7, 105, "Section",
 Evaluatable->False],
Cell[342526, 11856, 156, 6, 43, "Input"],
Cell[342685, 11864, 777, 28, 139, "Text",
 Evaluatable->False],
Cell[343465, 11894, 29, 0, 43, "Input"],
Cell[343497, 11896, 516, 19, 92, "Text",
 Evaluatable->False],
Cell[344016, 11917, 21, 0, 43, "Input"],
Cell[344040, 11919, 573, 20, 68, "Text",
 Evaluatable->False],
Cell[344616, 11941, 20, 0, 43, "Input"],
Cell[344639, 11943, 315, 12, 73, "Text",
 Evaluatable->False],
Cell[344957, 11957, 55, 3, 66, "Input"],
Cell[345015, 11962, 499, 17, 97, "Text",
 Evaluatable->False],
Cell[345517, 11981, 27, 0, 43, "Input"],
Cell[345547, 11983, 572, 19, 153, "Text",
 Evaluatable->False],
Cell[346122, 12004, 22, 0, 43, "Input"],
Cell[346147, 12006, 169, 7, 41, "Text",
 Evaluatable->False],
Cell[346319, 12015, 59, 0, 43, "Input"],
Cell[346381, 12017, 472, 12, 139, "Text",
 Evaluatable->False],
Cell[346856, 12031, 20, 0, 43, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
